/******************************************************************************
 * FILE PURPOSE: Implementation of ATK
 ******************************************************************************
 * FILE NAME:   atk.c  
 *
 * DESCRIPTION:
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2011, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* Standard C header files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include "atk.h"
#include "atkloc.h"
#include "emureg.h"
#include "vctypes.h"
#include "ti/xdais/ialg.h"
/*#include "atk_aac_he_enc.h"*/
/*#include "atk_mp3_dec.h"*/
#include "types.h"
#include "codecs/atkCodecAPI.h"

/*********************************************************************************
 * FUNCTION PURPOSE: Exception function
 *********************************************************************************
  DESCRIPTION:      Relays a fatal exception through the global exception
                    pointer.
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
void atk_exception(tuint ID, tuint code)
{
   atkContext.debugInfo (ID, DEBUG_EXCEPTION_FATAL, code, 0, NULL);
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
tint atkOpen (void *atkInst, atkCfg_t *cfg)
{
  atkInst_t *inst = (atkInst_t *)atkInst;
  audDecoderAPI_t            *decAPI;
  audEncoderAPI_t            *encAPI;

  atk_exc_assert((inst->state == ATK_STATE_CLOSED), ATK_EXC_OPEN, inst);

  /* Store configuration data */
  inst->cfg     = *cfg;
  inst->sysInst = cfg->sysInst;
  
  strcpy(inst->codecName, cfg->codecName);
  decAPI = getAudDecoderAPI(inst->codecName);
  if (decAPI != NULL){
      ATK_SET_TYPE(inst, ATK_DECODE);
      decAPI->decInitInstance(inst->codecInst);
      decAPI->decXdmVer = atkGetXdmVer(inst->codecName);
  }

  encAPI = getAudEncoderAPI(inst->codecName);
  if (encAPI != NULL){
     ATK_SET_TYPE(inst, ATK_ENCODE);
     encAPI->encInitInstance(inst->codecInst);
     encAPI->encXdmVer = atkGetXdmVer(inst->codecName);
  }

  /* reset tx and rx stats */
  memset(&inst->stats,0,sizeof(atkStats_t));
  inst->state = ATK_STATE_OPENED_NOT_CONFIGURED;
  
  return(ATK_NOERR);
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
tint atkCommitDecoder(atkInst_t *inst)
{
    audDecoderAPI_t            *decAPI;
    void                       *decFxns;
    IAUDDEC1_Fxns              *iaudDecFxns;
    IAUDDEC1_Status            *status;
    IAUDDEC1_Params            *staticParams;
    IAUDDEC1_DynamicParams     *dynamicParams;
    tint retVal = ATK_NOERR;
    tulong size;
    tulong resDepBitMap = 0;   /* Set this to 0 to assume all the audio buffers are resolution independent */

    decAPI = getAudDecoderAPI(inst->codecName);
    if (decAPI == NULL){
        atk_exception(inst->ID, ATK_EXC_INVD_CODEC);
    }
    
    decFxns        = decAPI->decGetFxns(inst->codecInst);
    staticParams   = (IAUDDEC1_Params *)decAPI->decGetStaticParams(inst->codecInst, &size);
    dynamicParams  = (IAUDDEC1_DynamicParams *)decAPI->decGetDynamicParams(inst->codecInst, &size);
    status         = (IAUDDEC1_Status *)decAPI->decGetStatus(inst->codecInst);

    if ((inst->handle =  (IALG_Handle)atkContext.ALG_Create (inst->sysInst,
        (IALG_Fxns *) decFxns, 
        (IALG_Handle) NULL, 
        (IALG_Params *) staticParams, &resDepBitMap)) == NULL)
    {
        atk_exception(inst->ID, ATK_EXC_COMMIT_FAIL);
        return(retVal); /* not necessary - just to avoid warning */
    }

    /* Assigning Algorithm handle fxns field to iaudDecfxns                  */
    iaudDecFxns = (IAUDDEC1_Fxns *)inst->handle->fxns;
    iaudDecFxns->control((IAUDDEC1_Handle)inst->handle, XDM_SETPARAMS, dynamicParams, status);

    return(retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
tint atkCommitEncoder(atkInst_t *inst)
{
    audEncoderAPI_t            *encAPI;
    void *encFxns;
    tulong size;
    tint retVal;
    tulong resDepBitMap = 0; /* Set this to 0 to assume all the audio buffers are resolution independent */

    encAPI = getAudEncoderAPI(inst->codecName);
    if (encAPI == NULL){
       atk_exception(inst->ID, ATK_EXC_INVD_CODEC);
    }

  switch (encAPI->encXdmVer)
    {
        case ATK_XDM_VER_0P9:
          {
              IAUDENC_Fxns               *iaudENCFxns;
              IAUDENC_Status             *status;
              IAUDENC_Params             *staticParams;
              IAUDENC_DynamicParams      *dynamicParams;

              /* Get XDM0.9 pointers from the codec */
              encFxns        = encAPI->encGetFxns(inst->codecInst);
              staticParams   = (IAUDENC_Params *)encAPI->encGetStaticParams(inst->codecInst, &size);
              dynamicParams  = (IAUDENC_DynamicParams *)encAPI->encGetDynamicParams(inst->codecInst, &size);
              status         = (IAUDENC_Status *)encAPI->encGetStatus(inst->codecInst);

              if ((inst->handle =  (IALG_Handle)atkContext.ALG_Create (inst->sysInst,
                  (IALG_Fxns *) encFxns,
                  (IALG_Handle) NULL,
                  (IALG_Params *) staticParams, &resDepBitMap)) == NULL)
              {
                  atk_exception(inst->ID, ATK_EXC_COMMIT_FAIL);
              }

              /* Assigning Algorithm handle fxns field to iaudEncFxns */
              iaudENCFxns = (IAUDENC_Fxns *)inst->handle->fxns;
              retVal = iaudENCFxns->control((IAUDENC_Handle)inst->handle, XDM_SETPARAMS,dynamicParams,status);
           }
           break;

        case ATK_XDM_VER_1P0:
          {
              IAUDENC1_Fxns               *iaudENCFxns;
              IAUDENC1_Status             *status;
              IAUDENC1_Params             *staticParams;
              IAUDENC1_DynamicParams      *dynamicParams;

              /* Get XDM1.0 pointers from the codec */
              encFxns        = encAPI->encGetFxns(inst->codecInst);
              staticParams   = (IAUDENC1_Params *)encAPI->encGetStaticParams(inst->codecInst, &size);
              dynamicParams  = (IAUDENC1_DynamicParams *)encAPI->encGetDynamicParams(inst->codecInst, &size);
              status         = (IAUDENC1_Status *)encAPI->encGetStatus(inst->codecInst);

              if ((inst->handle =  (IALG_Handle)atkContext.ALG_Create (inst->sysInst,
                  (IALG_Fxns *) encFxns,
                  (IALG_Handle) NULL,
                  (IALG_Params *) staticParams, &resDepBitMap)) == NULL)
              {
                  atk_exception(inst->ID, ATK_EXC_COMMIT_FAIL);
              }

              /* Assigning Algorithm handle fxns field to iaudEncFxns */
              iaudENCFxns = (IAUDENC1_Fxns *)inst->handle->fxns;
              retVal = iaudENCFxns->control((IAUDENC1_Handle)inst->handle, XDM_SETPARAMS,dynamicParams,status);
           }
           break;
    }
    return(retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
tint atkCommitCodec (void *atkInst) 
{
  atkInst_t *inst = (atkInst_t *)atkInst;
  tint retVal = ATK_NOERR;
  tuint type  = ATK_GET_TYPE(inst);

  switch(type) {
  case ATK_ENCODE:
      retVal = atkCommitEncoder(inst);
      break;

  case ATK_DECODE:
      retVal = atkCommitDecoder(inst);
      break;
  }

  /* set ATK state to OPEN as memory is allocated */
  inst->state = ATK_STATE_OPEN;
  return(retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
void atkClose (void *atkInst)
{
  atkInst_t *inst = (atkInst_t *)atkInst;

  if(inst->state == ATK_STATE_OPEN){
     atkContext.ALG_Free(inst->sysInst);
  }
  inst->state = ATK_STATE_CLOSED;  
  return;
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
tint atkControl (void *atkInst, atkCtrl_t *ctrl) 
{
    atkInst_t *inst = (atkInst_t *)atkInst;
    tint retVal = ATK_NOERR;
  
    switch(ctrl->code) {
      case ATK_GET_STATIC_CFG_PTR:
          atkGetStaticCfgPtr (inst, &(ctrl->u.codecCfgPtr.cfgPtr), &(ctrl->u.codecCfgPtr.size));
          break;

      case ATK_GET_DYNAMIC_CFG_PTR:
          atkGetDynamicCfgPtr (inst, &(ctrl->u.codecCfgPtr.cfgPtr), &(ctrl->u.codecCfgPtr.size));
          break;

      case ATK_COMMIT_CODEC:
          retVal = atkCommitCodec(inst);
          break;

      case ATK_GET_CFG:
          retVal = atkGetConfig(inst, &(ctrl->u.atkCfg));
          break;

      case ATK_GET_STATS:
          break;

      case ATK_RESET_CODEC:
          retVal = atkReset(inst);
          break;

      default:
          break;
    }
    return (retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
void atkDelete (void **atkInst)
{
   atkInst_t *inst = (atkInst_t *) *atkInst;

   /* Ensure it is already closed */
   atk_exc_assert (inst->state == ATK_STATE_CLOSED, ATK_EXC_DELETE, inst);

   *atkInst = NULL;   /* Mark the instance as free */
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
void atkStatsUpdateFrameDetails (atkStats_t *stats, tulong frameSize) {

  stats->totalNumBytes += frameSize;
  stats->bitRateByteCount += frameSize;
  stats->bitRateFrameCounter++;

  /* TODO: This must not be hardcoded!!! */
  /* use ... videncDynamicParams.intraFrameInterval instead!!! */
  if (stats->bitRateFrameCounter >= 30) {
    tulong curTime = *(atkContext.curTime);
    tulong deltaTime = curTime - stats->bitRateTime;
    tulong deltaTimeInSec = deltaTime/1000;
    /* we accumulated bitRateByteCount in deltaTime */
    /* bytes per sec */
    stats->bitrate = (tulong)((stats->bitRateByteCount * 8)/(tulong)deltaTimeInSec);

    /* Reset */
    stats->bitRateTime         = curTime;
    stats->bitRateFrameCounter = 0;
    stats->bitRateByteCount    = 0;
  }
  return;
}

/*********************************************************************************
 * FUNCTION PURPOSE: 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs:
                    Output: 
 *********************************************************************************/
void atkGetStats (void *atkInst, bool reset, atkStats_t *stats) {
    atkInst_t *inst = (atkInst_t *)atkInst;
    memcpy(stats,&inst->stats,sizeof(atkStats_t ));
    if (reset) {
      memset(&inst->stats,0,sizeof(atkStats_t));
    }
    return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Current ATK & Codec Configuration
 ******************************************************************************
 * DESCRIPTION: Get Current ATK Configuration from instance and report to application
 *
 * void atkCodecConfig (
 *    void         *atkInst,
 *    atkCfg_t     *cfg)  
 *
 *****************************************************************************/
tint atkGetConfig (void *atkInst, atkCfg_t *cfg) {
  atkInst_t *inst = (atkInst_t *)atkInst;

  memcpy(cfg, &inst->cfg,sizeof(atkCfg_t));
  return(ATK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Static Config Pointer from ATK to configure
 ******************************************************************************
 * DESCRIPTION: Get Static Config Pointer from ATK to configure
 *
 * void atkGetStaticCfgPtr (
 *    void         *atkInst,     - Instance ptr
 *    tulong       **cfgPtr,     - Return the static Cfg ptr to get initialized
 *    tulong        *size        - Return the size of the static Cfg structure )  
 *
 *****************************************************************************/
tint atkGetStaticCfgPtr (void *atkInst,  tulong **cfgPtr, tulong *size) {
  atkInst_t *inst = (atkInst_t *)atkInst;
  tuint type      = ATK_GET_TYPE(inst);
  audDecoderAPI_t *decAPI;
  audEncoderAPI_t *encAPI;

  *cfgPtr = NULL;
  *size   = 0;

  switch(type) {
  case ATK_ENCODE:
      encAPI = getAudEncoderAPI(inst->codecName);
      *cfgPtr = encAPI->encGetStaticParams(inst->codecInst, size);
      **cfgPtr = *size;
      break;

  case ATK_DECODE:
      decAPI = getAudDecoderAPI(inst->codecName);
      *cfgPtr = decAPI->decGetStaticParams(inst->codecInst, size);
      **cfgPtr = *size;
      break;
  }
  return (ATK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Dynamic Config Pointer from ATK to configure
 ******************************************************************************
 * DESCRIPTION: Get Dynamic Config Pointer from ATK to configure
 *
 * void atkGetDynamicCfgPtr (
 *    void         *atkInst,     - Instance ptr
 *    tulong       **cfgPtr,     - Return the Dynamic Cfg ptr to get initialized
 *    tulong        *size        - Return the size of the Dynamic Cfg structure )  
 *
 *****************************************************************************/
tint atkGetDynamicCfgPtr (void *atkInst, tulong **cfgPtr, tulong *size)
{
  atkInst_t *inst = (atkInst_t *)atkInst;
  tuint type      = ATK_GET_TYPE(inst);
  audDecoderAPI_t *decAPI;
  audEncoderAPI_t *encAPI;

  *cfgPtr = NULL;
  *size   = 0;

  switch(type) {
  case ATK_ENCODE:
      encAPI = getAudEncoderAPI(inst->codecName);
      *cfgPtr = encAPI->encGetDynamicParams(inst->codecInst, size);
      **cfgPtr = *size;
      break;

  case ATK_DECODE:
      decAPI = getAudDecoderAPI(inst->codecName);
      *cfgPtr = decAPI->decGetDynamicParams(inst->codecInst, size);
      **cfgPtr = *size;
      break;
  }
  return (ATK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Reset Codec on the fly
 ******************************************************************************
 * DESCRIPTION: Reset the Codec on the fly when there is an error
 *
 * tint atkReset (
 *    atkInst_t         *inst,     - Instance ptr )  
 *
 *****************************************************************************/
tint atkReset (atkInst_t *inst)
{
    return(ATK_NOERR);
}

/*nothing past this point**/
