/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   atk_decode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "codecs/atkCodecAPI.h"
#include "xdm.h"
#include "atkloc.h"

XDAS_Int32 atkDecodeWriteAudioInterleaved(XDAS_UInt8* outBufPtr, XDAS_UInt16 *out, XDAS_Int16 nch,XDAS_Int32 nSamples, XDAS_Int32 outputBitsPerSample)
{
    XDAS_Int16 i;
    XDAS_UInt16 c,c1,c2;
    XDAS_Int32 outbufSize = 0;

    if(outputBitsPerSample == 16)
    {
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i=i+1)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;
                memcpy(&outBufPtr[outbufSize++],&c2, 1);
                memcpy(&outBufPtr[outbufSize++],&c1, 1);
            }
        }
        else
        {
            for(i=0;i<nSamples*2;i=i+2)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outBufPtr[outbufSize++],&c2, 1);
                memcpy(&outBufPtr[outbufSize++],&c1, 1);
                /* Right Channel  */
                c    = out[i+1];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outBufPtr[outbufSize++],&c2, 1);
                memcpy(&outBufPtr[outbufSize++],&c1, 1);
            }
        }
    } else {/*24-bit samples */
        XDAS_Int32* out24 = (XDAS_Int32 *)out;
        XDAS_Int32 c24;
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i=i+1)
            {
                /* Left Channel */
                c24 = out24[i];
                memcpy(&outBufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
        else
        {
            for(i=0;i<nSamples*2;i=i+2)
            {

                /* Left Channel */
                c24 = out24[i];
                memcpy(&outBufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
                /* Left Channel */
                c24 = out24[i + 1];
                memcpy(&outBufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
    }
    return(outbufSize);
}


XDAS_Int32 atkDecodeWriteAudioBlock(XDAS_UInt8* outBufPtr, XDAS_UInt16 *out, XDAS_Int16 nch,XDAS_Int32 nSamples, XDAS_Int32 outputBitsPerSample)
{
    XDAS_Int16 i;
    XDAS_UInt16 c,c1,c2;
    XDAS_Int32 outbufSize = 0;

    if(outputBitsPerSample == 16)
    {
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i++)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outBufPtr[outbufSize++],&c2, 1);
                memcpy(&outBufPtr[outbufSize++],&c1, 1);
            }
        }
        else
        {
            for(i=0;i<nSamples;i++)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outBufPtr[outbufSize++],&c2, 1);
                memcpy(&outBufPtr[outbufSize++],&c1, 1);
                /* Right Channel  */
                c    = out[i+nSamples];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outBufPtr[outbufSize++],&c2, 1);
                memcpy(&outBufPtr[outbufSize++],&c1, 1);
            }
        }
    }
    else /* 24-bit */
    {
        XDAS_Int32* out24 = (XDAS_Int32 *)out;
        XDAS_Int32 c24;
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i=i+1)
            {
                /* Left Channel */
                c24 = out24[i];
                memcpy(&outBufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
        else
        {
            for(i=0;i<nSamples;i=i+1)
            {

                /* Left Channel */
                c24 = out24[i];
                memcpy(&outBufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
                /* Left Channel */
                c24 = out24[i + nSamples];
                memcpy(&outBufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
    }
    return(outbufSize);
}


/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Audio Decode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Audio Decode Call
*
*****************************************************************************/
tint atkDecode(void *atkInst, ifvisys_audDecInput_t *audDecInput, ifvisys_audDecOutput_t *audDecOutput)
{
    atkInst_t *inst  = (atkInst_t *)atkInst;

    /* Input/Output  buffers  and their sizes                                  */
    IAUDDEC1_Fxns              *iaudDecFxns;
    IAUDDEC1_Status            *status;
    IAUDDEC1_InArgs            *inArgs;
    IAUDDEC1_OutArgs           *outArgs;
    IAUDDEC1_Params            *staticParams;
    IAUDDEC1_DynamicParams     *dynamicParams;
    void *codecInst;

    /* Input/Output Buffer Descriptors                                         */
    XDM1_BufDesc inputBufDesc;
    XDM1_BufDesc outBufDesc;

    /* Other variables                                                         */
    XDAS_Int32  nDecodedSamples = 0;
    XDAS_Int16  nch, outputBitsPerSample;
    tulong size, bytesGenerated;
    tint retVal;
    audDecoderAPI_t *decAPI;
    tword *decTempOutBuf;

    decAPI = getAudDecoderAPI(inst->codecName);

    codecInst = inst->codecInst;
    /* Get XDM1.0 pointers from the codec */
    inArgs                = (IAUDDEC1_InArgs *)decAPI->decGetInArgs(codecInst);
    outArgs               = (IAUDDEC1_OutArgs *)decAPI->decGetOutArgs(codecInst);
    staticParams          = (IAUDDEC1_Params *)decAPI->decGetStaticParams(codecInst, &size);
    dynamicParams         = (IAUDDEC1_DynamicParams *)decAPI->decGetDynamicParams(codecInst, &size);
    status                = (IAUDDEC1_Status *)decAPI->decGetStatus(codecInst);

    /* Assigning Algorithm handle fxns field to iaudDecfxns                  */
    iaudDecFxns = (IAUDDEC1_Fxns *)inst->handle->fxns;

    iaudDecFxns->control((IAUDDEC1_Handle)inst->handle, XDM_SETPARAMS,dynamicParams,status);
    /*-----------------------------------------------------------------------*/
    /*  The GETBUFINFO call gives information for number and size of input   */
    /*  and output buffers. For this first call, the actual width and height */
    /*  of the picture is not known hence, this control call returns the out */
    /*  buffer sizes based on the maxHeight and maxWidth provided as part    */
    /*  of the input params from testparams file.                            */
    /*-----------------------------------------------------------------------*/
    iaudDecFxns->control((IAUDDEC1_Handle)inst->handle, XDM_GETBUFINFO, dynamicParams, status);

    /*-----------------------------------------------------------------------*/
    /*  Initialize the input buffer properties as required by algorithm      */
    /*  based on info received by preceding GETBUFINFO call. First init the  */
    /*  number of input bufs.                                                */
    /*-----------------------------------------------------------------------*/

    inputBufDesc.numBufs          = status->bufInfo.minNumInBufs;
    inputBufDesc.descs[0].bufSize = status->bufInfo.minInBufSize[0];
    inputBufDesc.descs[0].buf     = (XDAS_Int8 *)audDecInput->bitStream.ptr;

    /*-----------------------------------------------------------------------*/
    /*  Initialize the output buffer properties as required by algorithm     */
    /*  based on info received by preceding GETBUFINFO call.                 */
    /*-----------------------------------------------------------------------*/
    outBufDesc.numBufs          = status->bufInfo.minNumOutBufs;
    outBufDesc.descs[0].bufSize = status->bufInfo.minOutBufSize[0];
    decTempOutBuf = audDecOutput->pcm.pcmFrame.ptr + outBufDesc.descs[0].bufSize;
    outBufDesc.descs[0].buf     = (XDAS_Int8 *)decTempOutBuf;

    /* Resetting bytesGenerated variable                                     */
    outArgs->bytesConsumed     = 0;
    inArgs->numBytes           = audDecInput->bitStream.size;
    inArgs->lfeFlag            = 0;
    inArgs->desiredChannelMode = IAUDIO_2_0;
    
    /* Basic Algorithm process() call                                      */
    retVal = iaudDecFxns->process((IAUDDEC1_Handle)inst->handle, 
                                  (XDM1_BufDesc *)&inputBufDesc,
                                  (XDM1_BufDesc *)&outBufDesc,
                                  inArgs,
                                  outArgs);

    if((retVal == XDM_EFAIL) || (outArgs->extendedError)) {
        inst->stats.errorFrames++;
        inst->stats.lastErrorCode = outArgs->extendedError;
    } else {
        inst->stats.goodFrames++;
    }

    /* Optional: Read status via control()                                 */
    iaudDecFxns->control((IAUDDEC1_Handle)inst->handle, XDM_GETSTATUS, dynamicParams, status);

    nDecodedSamples = status->numSamples;
    nch = (short)(status->channelMode);
    outputBitsPerSample = status->outputBitsPerSample;
    
    audDecOutput->pcm.pcmFrame.size = 0;

    /* Check for frame ready                    */
    if (outArgs->extendedError == XDM_EOK)
    {
        if (status->pcmFormat == IAUDIO_INTERLEAVED) {
            bytesGenerated = atkDecodeWriteAudioInterleaved(audDecOutput->pcm.pcmFrame.ptr, (XDAS_UInt16 *)outBufDesc.descs[0].buf, nch,nDecodedSamples, outputBitsPerSample);
        } else if (status->pcmFormat == IAUDIO_BLOCK) {
            bytesGenerated = atkDecodeWriteAudioBlock(audDecOutput->pcm.pcmFrame.ptr, (XDAS_UInt16 *)outBufDesc.descs[0].buf, nch,nDecodedSamples, outputBitsPerSample);
        }
        audDecOutput->pcm.pcmFrame.size = bytesGenerated;
        inst->stats.totalNumBytes += bytesGenerated;
    }
    return(outArgs->bytesConsumed);
}

/* nothing past this point */
