#ifndef _ATKLOC_H
#define _ATKLOC_H
/******************************************************************************
 * FILE PURPOSE: Macros and definitions private to ATK
 ******************************************************************************
 * FILE NAME:   atkloc.h  
 *
 * DESCRIPTION: Defines macros and definitions seen only by the ATK unit.
 *
 * (C) Copyright 2011, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
/* Standard C header files */
#include <stdio.h>
#include <stdlib.h>
#include "xdc/std.h"
#include "atk.h"
#include "vctypes.h"
#include <ti/mas/types/types.h>
#include "memmgm.h"

/* Possible channel states */
typedef enum {
  ATK_STATE_OPEN,
  ATK_STATE_CLOSED,
  ATK_STATE_OPENED_NOT_CONFIGURED
} atkState_t;

/************************************************************/
/* ATK common instance (for decoding and encoding channels) */
/************************************************************/
typedef struct atkInst_s {
  tuint        ID;
  tint         state;
  char         codecName[10];
  /* Algorithm specific handle */
  IALG_Handle handle;
#define ATK_ENCODE 1
#define ATK_DECODE 2
#define ATK_GET_TYPE(a)       UTL_GET_BITFIELD(((atkInst_t *)a)->audioBitField0, 0, 2)
#define ATK_SET_TYPE(a,b)     UTL_SET_BITFIELD(((atkInst_t *)a)->audioBitField0, b, 0, 2)
  tuint        audioBitField0;
  atkCfg_t     cfg;
  void         *sysInst;
  void         *codecInst;
  atkStats_t   stats;
} atkInst_t;

/*----------------------------*/
/* Exception and Debug Macros */
/*----------------------------*/
extern debugInfo_t atk_debug_info;

/* Assert macro for the exceptions */
#define atk_exc_assert(expr,code,inst) { \
  (expr) ? ((void)0) : atk_exception((inst)->ID, code); }

void atk_exception(tuint id, tuint code);
void atkStatsUpdateFrameDetails (atkStats_t *stats, tulong frameSize);
tint atkCommitCodec (void *atkInst);
tint atkGetConfig (void *atkInst, atkCfg_t *cfg);
tint atkGetStaticCfgPtr (void *atkInst,  tulong **cfgPtr, tulong *size);
tint atkGetDynamicCfgPtr (void *atkInst, tulong **cfgPtr, tulong *size);

/* ATK video codec memory management */
extern void atkAllocHeap(tuint chan, tuint coreID);
extern void atkFreeHeap(tuint chan);
extern void atkResetScratch(void);
tint atkReset (atkInst_t *inst);


#endif
/* Nothing past this point */

