/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM Compliant Encoder
******************************************************************************
* FILE NAME:   atk_aac_he_enc.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_AAC_HE_ENC_CLIENT_H
#define _VCT_AAC_HE_ENC_CLIENT_H

#include <ti/mas/types/types.h>
#include "aacenc_tii.h"
#include "iaacenc.h"

typedef struct aacHEEncInst_s {
    AACENC_Params             extn_params;
    IAACENC_DynamicParams     extn_dynamicParams;
    IAACENC_InArgs            inArgs;
    IAACENC_OutArgs           outArgs;
    IAACENC_Status            status;
    IAACENC_Fxns              *IALG_Fxns;
} aacHEEncInst_t;

void aacHEInitInstance(void *aacHEEncInst);
void aacHESetStaticParams(void *aacHEEncInst, void *codecParams);
void aacHESetDynamicParams(void *aacHEEncInst, void *dynParams);
void *aacHEGetStaticParams(void *aacHEEncInst, tulong *size);
void *aacHEGetDynamicParams(void *aacHEEncInst, tulong *size);
void *aacHEGetInArgs(void *aacHEEncInst);
void *aacHEGetOutArgs(void *aacHEEncInst);
void *aacHEGetStatus(void *aacHEEncInst);
void *aacHEGetFxns(void *aacHEEncInst);

#define aacHEEncAPI {        \
    (const char *)"AACHEENC",\
    0,                       \
    sizeof(aacHEEncInst_t),  \
    aacHEInitInstance,       \
    aacHESetStaticParams,    \
    aacHESetDynamicParams,   \
    aacHEGetStaticParams,    \
    aacHEGetDynamicParams,   \
    aacHEGetInArgs,          \
    aacHEGetOutArgs,         \
    aacHEGetStatus,          \
    aacHEGetFxns             \
}

#endif

/* nothing past this point */
