/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Encoder
******************************************************************************
* FILE NAME:   atk_aac_le_enc.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "atk_aac_le_enc.h"
#include <string.h>

void *aacLEGetStaticParams(void *aacLEEncInst, tulong *size) {
   aacLEEncInst_t *inst = (aacLEEncInst_t *)aacLEEncInst;
   *size = sizeof(IAUDENC1_Params);
   return((void *)&inst->extn_params);
}

void *aacLEGetDynamicParams(void *aacLEEncInst, tulong *size){
   aacLEEncInst_t *inst = (aacLEEncInst_t *)aacLEEncInst;
   *size = sizeof(IAUDENC1_DynamicParams);
   return((void *)&inst->extn_dynamicParams);
}

void *aacLEGetInArgs(void *aacLEEncInst) {
   aacLEEncInst_t *inst          = (aacLEEncInst_t *)aacLEEncInst;
   IMP4AACENC_InArgs *inArgs     = &(inst->inArgs);

   /* Currently no Ancillary Data is given */
   inArgs->audenc_inArgs.ancData.bufSize = 0L;
   inArgs->audenc_inArgs.ancData.buf     = (XDAS_Int8 *) NULL;
   return((void *)inArgs);
}

void *aacLEGetOutArgs(void *aacLEEncInst) {
   aacLEEncInst_t *inst = (aacLEEncInst_t *)aacLEEncInst;
   return((void *)&inst->outArgs);
}

void *aacLEGetStatus(void *aacLEEncInst) {
   aacLEEncInst_t *inst = (aacLEEncInst_t *)aacLEEncInst;
   return((void *)&inst->status);
}

void *aacLEGetFxns(void *aacLEEncInst) {
   aacLEEncInst_t *inst = (aacLEEncInst_t *)aacLEEncInst;
   return((void *)(inst->IALG_Fxns));
}

void aacLEInitInstance(void *aacLEEncInst)
{
    aacLEEncInst_t *inst = (aacLEEncInst_t *)aacLEEncInst;
    memset(inst, 0, sizeof(aacLEEncInst_t));

    inst->IALG_Fxns                    = (IMP4AACENC_Fxns *)&MP4AACENC_TIJ_IMP4AACENC;
    inst->status.audenc_status.size    = sizeof(IAUDENC1_Status);
    inst->inArgs.audenc_inArgs.size    = sizeof(IMP4AACENC_InArgs);
    inst->outArgs.audenc_outArgs.size  = sizeof(IMP4AACENC_OutArgs);
    return;
}

void aacLESetStaticParams(void *aacLEEncInst, void *codecParams)
{
    aacLEEncInst_t *inst           = (aacLEEncInst_t *)aacLEEncInst;
    IMP4AACENC_Params *ext_params  = &(inst->extn_params);
    IAUDENC1_Params *params        = &ext_params->audenc_params;

    params->size                   = sizeof(IAUDENC1_Params);
    params->sampleRate             = 48000;
    params->bitRate                = 88000;
    params->channelMode            = IAUDIO_2_0;//1
    params->dataEndianness         = XDM_LE_16; //2
    params->encMode                = IAUDIO_CBR; //0
    params->inputFormat            = IAUDIO_INTERLEAVED; //1
    params->inputBitsPerSample     = 16;
    params->maxBitRate             = 192000;
    params->dualMonoMode           = 0;
    params->crcFlag                = 0;
    params->ancFlag                = 0;
    params->lfeFlag                = 0;

    ext_params->aot                = 0;
    ext_params->quality            = 0;
    ext_params->transMode          = 0;

    return;
}

void aacLESetDynamicParams(void *aacLEEncInst, void *dynParams)
{
    aacLEEncInst_t *inst                   = (aacLEEncInst_t *)aacLEEncInst;
    IMP4AACENC_DynamicParams *extn_dynpar  = &inst->extn_dynamicParams;
    IAUDENC1_DynamicParams *dynamicParams  = &extn_dynpar->audenc_dynamicparams;
    IMP4AACENC_Params *params              = (IMP4AACENC_Params *)dynParams;

    memset((void *)dynamicParams, 0, sizeof(IMP4AACENC_DynamicParams));

    dynamicParams->size                   = sizeof(IAUDENC1_DynamicParams);
    dynamicParams->bitRate                = 88000;
    dynamicParams->sampleRate             = 48000;
    dynamicParams->channelMode            = 1;
    dynamicParams->lfeFlag                = 0;
    dynamicParams->dualMonoMode           = 0;
    dynamicParams->inputBitsPerSample     = 16;

    return;
}

/* nothing past this point */
