/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM Compliant Encoder
******************************************************************************
* FILE NAME:   atk_aac_le_enc.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_AACENC_LE_CLIENT_H
#define _VCT_AACENC_LE_CLIENT_H

#include <ti/mas/types/types.h>
#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include "imp4aacenc.h"

typedef struct aacLEEncInst_s {
    IMP4AACENC_Params         extn_params;
    IMP4AACENC_DynamicParams  extn_dynamicParams;
    IMP4AACENC_InArgs         inArgs;
    IMP4AACENC_OutArgs        outArgs;
    IMP4AACENC_Status         status;
    IMP4AACENC_Fxns           *IALG_Fxns;
} aacLEEncInst_t;

void aacLEInitInstance(void *aacLEEncInst);
void aacLESetStaticParams(void *aacLEEncInst, void *codecParams);
void aacLESetDynamicParams(void *aacLEEncInst, void *dynParams);
void *aacLEGetStaticParams(void *aacLEEncInst, tulong *size);
void *aacLEGetDynamicParams(void *aacLEEncInst, tulong *size);
void *aacLEGetInArgs(void *aacLEEncInst);
void *aacLEGetOutArgs(void *aacLEEncInst);
void *aacLEGetStatus(void *aacLEEncInst);
void *aacLEGetFxns(void *aacLEEncInst);

#define aacLEEncAPI {          \
    (const char *)"AACLEENC",  \
    0,                         \
    sizeof(aacLEEncInst_t),    \
    aacLEInitInstance,         \
    aacLESetStaticParams,      \
    aacLESetDynamicParams,     \
    aacLEGetStaticParams,      \
    aacLEGetDynamicParams,     \
    aacLEGetInArgs,            \
    aacLEGetOutArgs,           \
    aacLEGetStatus,            \
    aacLEGetFxns               \
}

#endif

/* nothing past this point */
