/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctMp3DecClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "atk_mp3_dec.h"
#include <string.h>

void *mp3DecGetStaticParams(void *mp3DecInst, tulong *size) {
   mp3DecInst_t *inst = (mp3DecInst_t *)mp3DecInst;
   *size = sizeof(IMP3DEC_Params);
   return((void *)&inst->extn_params);
}

void *mp3DecGetDynamicParams(void *mp3DecInst, tulong *size){
   mp3DecInst_t *inst = (mp3DecInst_t *)mp3DecInst;
   *size =  sizeof(IMP3DEC_DynamicParams);
   return((void *)&inst->extn_dynamicParams);
}

void *mp3DecGetInArgs(void *mp3DecInst) {
   mp3DecInst_t *inst = (mp3DecInst_t *)mp3DecInst;
   return((void *)&inst->inArgs);
}

void *mp3DecGetOutArgs(void *mp3DecInst) {
   mp3DecInst_t *inst = (mp3DecInst_t *)mp3DecInst;
   return((void *)&inst->outArgs);
}

void *mp3DecGetStatus(void *mp3DecInst) {
   mp3DecInst_t *inst = (mp3DecInst_t *)mp3DecInst;
   return((void *)&inst->status);
}

void *mp3DecGetFxns(void *mp3DecInst) {
   mp3DecInst_t *inst = (mp3DecInst_t *)mp3DecInst;
   return((void *)&inst->IALG_Fxns);
}

void mp3DecInitInstance(void *mp3DecInst)
{
    mp3DecInst_t *inst = (mp3DecInst_t *)mp3DecInst;
    extern const IMP3DEC_Fxns MP3DEC_TII_IMP3DEC;

    memset(inst, 0, sizeof(mp3DecInst_t));
    inst->IALG_Fxns                    = MP3DEC_TII_IMP3DEC;
    inst->status.auddec_status.size    = sizeof(IMP3DEC_Status);
    inst->inArgs.auddec_inArgs.size    = sizeof(IMP3DEC_InArgs);
    inst->outArgs.auddec_outArgs.size  = sizeof(IMP3DEC_OutArgs);
    return;
}

void mp3DecSetStaticParams(void *mp3DecInst, void *codecParams)
{
    mp3DecInst_t    *inst         = (mp3DecInst_t *)mp3DecInst;
    IMP3DEC_Params  *ext_params   = &(inst->extn_params);
    IAUDDEC1_Params *params       = &ext_params->auddec_params;

    params->size                   = sizeof(IMP3DEC_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(IMP3DEC_Params));
    } else {
      params->outputPCMWidth         = 16;
      params->pcmFormat              = IAUDIO_INTERLEAVED;
      params->dataEndianness         = XDM_LE_16;
      /* No Codec specific Static Param */
    }
    return;
}

void mp3DecSetDynamicParams(void *mp3DecInst, IAUDDEC1_Params *params)
{
    mp3DecInst_t *inst                    = (mp3DecInst_t *)mp3DecInst;
    IMP3DEC_DynamicParams *extn_dynpar    = &inst->extn_dynamicParams;
    IAUDDEC1_DynamicParams *dynamicParams = &extn_dynpar->auddec_dynamicparams;

    dynamicParams->size               = sizeof(IMP3DEC_DynamicParams);
    dynamicParams->downSampleSbrFlag  = 0;

    return;
}
/* nothing past this point */
