/******************************************************************************
* FILE PURPOSE: Install XDM compliant Codecs
******************************************************************************
* FILE NAME:   atkCodecAPI.h
*
* DESCRIPTION: Install XDM compliant Codecs
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#ifndef _ATK_CODEC_API_H
#define _ATK_CODEC_API_H
#include <ti/mas/types/types.h>
#include <string.h>
#include <stdio.h>
#include "xdc/std.h"
#include <ti/xdais/dm/xdm.h>
#include <ti/bios/include/std.h>
#include <ti/xdais/ialg.h>
#include <ti/xdais/dm/iaudenc.h>
#include <ti/xdais/dm/iaudenc1.h>
#include <ti/xdais/dm/iauddec1.h>

typedef enum {
    ATK_XDM_VER_NOT_DEFINED   = 0,
    ATK_XDM_VER_0P9           = 1,
    ATK_XDM_VER_1P0           = 2
} atkXdmVer_t;

atkXdmVer_t atkGetXdmVer (const char *codecName);

typedef struct audEncoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  atkXdmVer_t  encXdmVer;                     /**< Codec XDM version info */
  tuint        encInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*encInitInstance)(void *inst);
  void         (*encSetStaticParams)(void *inst, void *codecParams);
  void         (*encSetDynamicParams)(void *inst, void *dynParams);
  void*        (*encGetStaticParams)(void *inst, tulong *size);
  void*        (*encGetDynamicParams)(void *inst, tulong *size);
  void*        (*encGetInArgs)(void *inst);
  void*        (*encGetOutArgs)(void *inst);
  void*        (*encGetStatus)(void *inst);
  void*        (*encGetFxns)(void *inst);
} audEncoderAPI_t;

typedef struct audDecoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  atkXdmVer_t  decXdmVer;                     /**< Codec XDM version info */
  tuint        decInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*decInitInstance)(void *inst);
  void         (*decSetStaticParams)(void *inst, void *codecParams);
  void         (*decSetDynamicParams)(void *inst, IAUDDEC1_Params *params);
  void*        (*decGetStaticParams)(void *inst,  tulong *size);
  void*        (*decGetDynamicParams)(void *inst, tulong *size);
  void*        (*decGetInArgs)(void *inst);
  void*        (*decGetOutArgs)(void *inst);
  void*        (*decGetStatus)(void *inst);
  void*        (*decGetFxns)(void *inst);
} audDecoderAPI_t;

#define audEncNullAPI {    \
  (const char *)"NULL",    \
  0,                       \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

#define audDecNullAPI {    \
  (const char *)"NULL",    \
  0,                       \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* audEncoderAPI_t getAudEncoderAPI(
*   const char *encName        - Codec Name string
* )
*****************************************************************************/
static inline audEncoderAPI_t *getAudEncoderAPI(const char *encName)
{
  extern audEncoderAPI_t supportedAudEncoders[];
  audEncoderAPI_t *suppEnc = supportedAudEncoders;

  while(strcmp(suppEnc->name, encName) != 0) {
     if(strcmp(suppEnc->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppEnc++;
     }
  }
  return(suppEnc);
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* audDecoderAPI_t getAudDecoderAPI(
*   const char *decName        - Codec Name string
* )
*****************************************************************************/
static inline audDecoderAPI_t *getAudDecoderAPI(const char *decName)
{
  extern audDecoderAPI_t supportedAudDecoders[];
  audDecoderAPI_t *suppDec = supportedAudDecoders;

  while(strcmp(suppDec->name, decName) != 0) {
     if(strcmp(suppDec->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppDec++;
     }
  }
  return(suppDec);
}

static inline tuint getMaxAudInstSize(void)
{
  extern audDecoderAPI_t supportedAudDecoders[];
  audDecoderAPI_t *suppDec = supportedAudDecoders;
  extern audEncoderAPI_t supportedAudEncoders[];
  audEncoderAPI_t *suppEnc = supportedAudEncoders;
  tuint  maxInstSize = 0;

  while(strcmp(suppDec->name, "NULL") != 0) {
     if(suppDec->decInstanceSize >  maxInstSize) {
        maxInstSize = suppDec->decInstanceSize;
     }
     suppDec++;
  }

  while(strcmp(suppEnc->name, "NULL") != 0) {
     if(suppEnc->encInstanceSize >  maxInstSize) {
        maxInstSize = suppEnc->encInstanceSize;
     }
     suppEnc++;
  }
  return(maxInstSize);
}

#endif

/* nothing past this point */
