/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _BERT_H
#define _BERT_H

#include "memmgm.h"
#include "types.h"

#define BERT_NBUF			1
#define BERT_INSTANCE_BUFN	0

/* BERT Error codes */
enum {
  BERT_NOMEMORY  = -1,
  BERT_NOERR     = 0,
  BERT_ERROR     = 1
};

typedef struct{
	tuint	dummy;
}bertSizeConfig_t;

typedef struct{
	tuint			ID;
	bertSizeConfig_t	sizeCfg;
}bertNewConfig_t;

typedef struct{
	tuint	moduleConfig;
	tuint	measureInterval;
	tuint	telUpperThreshold;
	tuint	telLowerThreshold;
	tuint	netUpperThreshold;
	tuint	netLowerThreshold;
	tuint	statsReportMode;
	tuint	netTestPattern;
	tuint	telTestPattern;
	tuint	bitsPerSample;
	tuint   reportInterval;
	tuint   netChannelID;
	tuint   telChannelID;
	void    (*statsReportFn)(void *inst, bool reset, tuint trans_id, tuint reason, 
	                         tuint priority, tuint priorityValidFlag);
	void    *statsReportTargetInst;
}bertOpenConfig_t;

typedef struct {
	tuint	reason;
	tuint	telSyncLostNum;
	tuint	telBerCurrent;
	tulong	telRcvBits;
	tulong	telErrBits;
	tulong	telErrSeconds;
	tuint   telRcvMode;
	tuint	netSyncLostNum;
	tuint	netBerCurrent;
	tulong	netRcvBits;
	tulong	netErrBits;
	tulong	netErrSeconds;
	tuint   netRcvMode;
}bertStats_t;

typedef struct{
  void (*bertReceiveInFcn)            (void**, tint, tint, void *, tint, tint, tint, void **);
  void (*bertSendInFcn)               (void**, void *, void *, tuint);
  void (*bertPatternGenTelFcn)        (void **, tuint, void*);
  void (*bertPatternDetectTelFcn)     (void **, void *, tuint);
}gg_bert_ctable_t;

//void genx14x15(bertInterface *inst, int nBlockSize, unsigned char *outPtr);
//void decode(bertInterface *inst, int nBlockSize, unsigned char *inPtr);
tint bertGetSizes(tint *nbuf, const memBuffer_t **bufs, bertSizeConfig_t *cfg);

tint bertNew(void **bertInst, tint bufs, memBuffer_t *buf, bertNewConfig_t *cfg);

tint bertOpen(void **bertInst, bertOpenConfig_t *cfg);

void bertClose(void **bertInst);

void bertDelete(void **bertInst);

void bertGetStats(void **berInst,bool reset, bertStats_t *stats);

void bertReceiveIn(void **bertInst, tint frame_type, tint cwseg_size, void *cw_in,
                   tint noise_level, tint fifo_size, tint codingType, void **decode);

void bertSendIn(void **bertInst, void *cw, void *output, tuint block_size);

void bertPatternGenTel(void **bertInst, tuint frame_size, void *frame_ptr);

tuint bertGetModuleConfig(void **bertInst);

void bertPatternDetectTelFcn (void **bertInst, void *cw, tuint block_size);

#endif /* _BERT_H */
