/******************************************************************************
 * FILE PURPOSE: Bit Error Rate Test unit(BERT) init source file.
 ******************************************************************************
 * FILE NAME: bertinit.c   
 *
 * DESCRIPTION: The BERT is a software ECO which performs Bit Error Rate test
 *				of the telephony network and packet network. The module can be
 *				configured to send specified pattern to packet network or 
 *				telephony network, to find error bits from input data received 
 *				from telephony network or packet network.
 * GLOBAL FUNCTIONS:
 *
 * bertNew()          : creates and instance of BERT
 * bertGetSizes()     : calculate buffer size for BERT
 * (C) Copyright 2002, Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* System level header files */
#include <string.h>
#include "types.h"

/* System utilities' API's */
#include "memmgm.h"

/* BERT high level header files */
#include "bert.h"
#include "bertloc.h"
#include "bertport.h"
#include "stdlib.h"

memBuffer_t	bertBufs[BERT_NBUF] =
{ /* class				log2align  size  volat  base */
	MEM_CLASS_EXTERNAL,    0,      0,    0,     NULL
};

/******************************************************************************
 * FUNCTION PURPOSE: Open and configure an instance of BERT.
 ******************************************************************************
 * DESCRIPTION: This function configures an instance of BERT.
 *  void bertGetSizes (
 *    void        *bertInst,   - a pointer to BERT instance
 *    bertNewConfig_t *cfg)    - a pointer to BERT configuration structure
 *
 *****************************************************************************/
tint bertGetSizes(tint *nBufs, const memBuffer_t **bufs, bertSizeConfig_t *cfg)
{
	//no cfg info needed.
	*nBufs = BERT_NBUF;	/* report the number of buffers */
	bertBufs[BERT_INSTANCE_BUFN].size = sizeof(bertInst_t);

	*bufs = &bertBufs[0];
	return (BERT_NOERR);
}


/******************************************************************************
 * FUNCTION PURPOSE: Creates an instance for BERT
 ******************************************************************************
 * DESCRIPTION: This function configures an instance of BERT.
 *  void bertNew (
 *    void        *bertInst,   - a pointer to BERT instance
 *    bertNewConfig_t *cfg)    - a pointer to BERT configuration structure
 *
 *****************************************************************************/
tint	bertNew(void **bertInst, tint nBufs, memBuffer_t *buf, bertNewConfig_t *cfg)
{
	memBuffer_t	*bufp;
	bertInst_t	*inst;
	/* Insure the instance pointer is NULL */
	if (*bertInst != NULL || nBufs != BERT_NBUF)
		return(BERT_ERROR);
	bufp = &buf[BERT_INSTANCE_BUFN];

	/*size check */
	if (bufp->size < sizeof(bertInst_t))
		return(BERT_ERROR);
	
	inst = (bertInst_t *)bufp->base;
    
    memset(inst,0,sizeof(bertInst_t));

	inst->ID = cfg->ID;
	inst->state = BERT_STATE_CLOSED;
	*bertInst = inst;
	return(BERT_NOERR);
}
