/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"

#ifndef NULL
#define NULL ((void *)0)
#endif

/* Module ID of module who allocates GMC's freed by BERT */
#define BERT_GMC_ALLOCATOR 0x0800

typedef struct {
    tulong  rxErrBits;          /* total error bits */
                                /* (options=-v5) */
    tulong  rxErrBitsOld;       /* Old value of rxErrBits */
    tulong  rxBits;             /* total bits received */
                                /* (options=-v5) */
    tulong  rxErrSecond;        /* number of error seconds */
                                /* (options=-v5) */
    tulong  rxErrLongCnt;       /* total error bits in one measurement interval */
                                /* (options=-v5) */

    tuint   rxScratch;          /* the status of the rx descrambler */
                                /* (private) */
    tint    rxShortCnt;         /* counter to get the one-second timer */
                                /* (private) */
    tint    rxIntervalCounter;  /* how many seconds left in current measurement interval */
                                /* (private) */
    tuint   upperThreshold;     /* Number of error bits per interval to
                                 * transition from OK state to error state */
                                /* (options=-v3) */
    tuint   lowerThreshold;     /* Number of error bits per interval to
                                 * transition from error state to OK state */
                                /* (options=-v3) */
    tuint   testPattern;        /* 0: the use 32767 pseudo random sequence;
                                 * 1: 0,1,...255 repeatedly;
                                 * 2: pseudo random sequence with chanID; */
                                /* (options=-v3) */
    tuint   syncLostNum;        /* number of sync lost events */
                                /* (options=-v5) */
    tuint   rxErr2OK;           /* number of error-to-ok events */
                                /* (options=-v5) */
    tuint   rxOK2Err;           /* number of ok-to-error events */
                                /* (options=-v5) */
    bool    rxRunning;          /* TRUE: the receiver has detected sync pattern after bert mode startup */
    bool    rxSync;             /* TRUE: rx is working in sync mode. FALSE: rx is working in no sync mode */
    bool    rxOKFlag;           /* TRUE: rx is in ok state, FALSE: rx is in error state */
    bool    rxErrBitsSaturation;/* TRUE: saturated */
    bool    rxStateTransition;  /* TRUE: state transition to report */
    tuint   txScratch;          /* the status of tx srambler */
                                /* (private) */
    tuint   instantBER;         /* the bit error rate of last valid measurement */
    tuint   reportFlag;         /* status report flag */
    tuint   measureInterval;    /* Measurement interval in seconds */
                                /* (options=-v3) */
    tuint   channelID;          /* Channel ID used for testPattern=2 test mode */
                                /* (options=-v3) */
    tuint   sync_id_counter;    /* (private) */
    tuint   id_sync_offset;     /* (private) */
    tuint   last_byte;          /* Last processed byte */
}bertInterface_t;


typedef struct bertInst_s{
    tuint           ID;             /* instance ID */ /* (options=-h) */
    tuint           state;          /* state of instance -- 0=closed; 1=open */
    tuint           moduleConfig;   /* Bitfield: 0x0001 = tele tx enable;
                                     *           0x0002 = tele rx enable;
                                     *           0x0004 = net tx enable;
                                     *           0x0008 = net rx enable.
                                     */ 
                                     /* (options=-h-v3) */
    bertInterface_t telephony;       /* Telephony configuration and control */
    bertInterface_t packetNet;       /* Packet network configuration and control */
    tuint           bitsPerSample;   /* bits in one sample of telephony interface (7 or 8 bit) */
                                     /* (options=-v3) */
    tuint           packetPrevSeq;   /* packet sequece of last packet received for network interface */
    tint            report_timer;    /* use an independent timer for peroidic stats report, unit is second */
    tint            sys_timer;       /* a second click generator based on PCM sample rates */
    tint            net_packet_click;/* used to sync with pcm side if there
                                        is connection lost or network congestion*/
    tuint           statsReportMode; /* Bitfield where:
                                      * 0x0001 = periodic reporting;
                                      * 0x0002 = reporting on tele state change;
                                      * 0x0004 = reporting on net state change.
                                      */
                                     /* (options=-h-v3) */
    tuint           prdRptInterval;  /* Periodic report interval in sec */
                                     /* (options=-v3) */
    tuint           sys_blk_size;
    void            (*statsReportFn)(void *inst, bool reset, tuint trans_id, 
                                     tuint reason, tuint priority, tuint priorityValidFlag); /*stats report fcn */
    void            *statsReportTargetInst; /* status report target instance */
}bertInst_t;

enum{
    BERT_STATE_CLOSED,
    BERT_STATE_OPEN
};

tint ber2samples(tint ber, tint measureInterval);
