#ifndef _DBERT_H
#define _DBERT_H
/*****************************************************************************
 * FILE PURPOSE: Defines an API for Daignostic Bit Error Rate Test (DBERT)
 *****************************************************************************
 * FILE NAME: dbert.h
 *
 * DESCRIPTION: Defines the API used to interact with the DBERT module
 *
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dglob.h"

/****************************************************************************
 * DEFINITION: Return values from DBERT 
 ****************************************************************************/
typedef enum {
  DBERT_SUCCESS = 0,
  DBERT_ERR_STATE_CLOSED
} dbertReturn_t;

/****************************************************************************
 * DEFINITION: DBERT pattern lock status
 ****************************************************************************/
typedef enum {
  DBERT_PATTERN_LOCKED,        /* DBERT has just locked to pattern       */
  DBERT_PATTERN_MAINTAINED,    /* DBERT has maintained pattern lock      */
  DBERT_PATERN_LOST,           /* DBERT has just lost pattern lock       */
  DBERT_PATTERN_NONE           /* DBERT has not established pattern lock */
} dbertPatternLock_t;
  

/****************************************************************************
 * DBERT Pattern Generation. Replaces input data with the generated pattern.
 *****************************************************************************/
dbertReturn_t dbertGenerate (void *dbertInst, globData_t *data);

/*****************************************************************************
 * DBERT Pattern Detection. Input data is unchanged. If the pattern is locked
 * on exit from this call, the bitErrCount returns the number of bit errors
 * found in the current block. 
 ****************************************************************************/
dbertReturn_t dbertLock (void *dbertInst, globData_t *data, 
                         dbertPatternLock_t *pl, tuint *bitErrCount);


/****************************************************************************
 * DBERT API call table. The dbert API is referenced by the system through a
 * pointer to this call table.
 ****************************************************************************/
typedef struct {
  dbertReturn_t (*dbertGenerate) (void *, globData_t *);
  dbertReturn_t (*dbertLock) (void *, globData_t *, dbertPatternLock_t *, 
                  tuint *);
} dbertCallTable_t;

#endif /* _DBERT_H */

