/*****************************************************************************
 * FILE PURPOSE: Diag configuration file for the C6416 processor
 *****************************************************************************
 * FILE NAME: diagcfg.h
 *
 * DESCRIPTION: Defines build time parameters for the diag program.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/***************************************************************************
 * DEFINITION: Bootloader boot function call. The function call made when
 *             the bootloader is ready to boot a core on the CPU. This
 *             is processor dependent. All boot functions are stored in
 *             dsiuboot.h which is included in diagcfg.c.
 *
 *             The default entry point must also be defined. This can be
 *             over-wridden by the boot core control message. The entry
 *             point is a byte address
 *****************************************************************************/
#define DIAG_SYSTEM_BOOT_FCN     nyshCoreBoot

/*****************************************************************************
 * DEFINITION: The function to get information left from the ROM
 *             boot loader.
 *****************************************************************************/
#define BOOT_PREV_INFO_FCN       nyshRomBootInfo

/***************************************************************************
 * DEFINITION: The function to configure the memory (cache, ddr)
 ***************************************************************************/
#define MEM_CONFIG_FCN           shannonMemConfig


/***************************************************************************
 * DEFINITION: Diag for the 5441 uses a single TDM port, with 4 sample
 *             subframe rate, BDX delay enabled.
 ***************************************************************************/
#define DIAG_TDM_PORTS_USED             0
#define DIAG_TDM_MAX_CHANNELS_PER_CORE  0 /* Required for device info */

/*****************************************************************************
 * The number of channels per core. Parameter will configure resources required
 * for QMSS/CPPI.
 *****************************************************************************/
#define DIAG_MAX_CHANNELS       1


//#define DIAG_INCLUDE_TDMSIM /* Generate spoofed TDM interrupts for simulation */
//#define DIAG_INCLUDE_OTBRUN /* Spoofed configuration to run after download */

/* Include NEU for messaging and voice */
#define DIAG_INCLUDE_NEU
#define DIAG_NEU_UDP_TABLE_SIZE          3
#define DIAG_AAL5_RX_ASSEMBLY_BUF_SIZE   2100  /* AAL5 Rx Buffer */
#define DIAG_AAL5_TX_SEGMENT_BUF_SIZE    2200  /* AAL5 Tx Buffer */

/* Include AAL1 packetization and network encapsulation */
/* #define DIAG_INCLUDE_DPVP */
#define DIAG_NEU_AAL5_RX_ASSEMBLY_BUF_SIZE 100  /* AAL5 Rx Buffer */
#define DIAG_NEU_AAL5_TX_SEGMENT_BUF_SIZE  110  /* Tx scratch buffer size */

/* System message task queue and buffer size definitions */
#define DIAG_MSG_TASK_SIZE    32     /* Support for 32 messages */
#define DIAG_MSG_RX_BUFFER_SIZE   (DIAG_AAL5_RX_ASSEMBLY_BUF_SIZE+2)
#define DIAG_MSG_TX_BUFFER_SIZE   DIAG_MSG_RX_BUFFER_SIZE
#define DIAG_MSG_RX_FIFO_SIZE     DIAG_MSG_RX_BUFFER_SIZE


/****************************************************************************
 * Tomahawk currently uses only the gmac for boot
 ****************************************************************************/
#define DIAG_BOOT_CONFIG_GMAC
#define DIAG_UTOPIA_PORTS_USED			GG_UTOPIA_PORTS_USED

/*****************************************************************************
 * DEFINITION: CPGMAC software configuration
 *****************************************************************************/
#define DIAG_GMAC_PORTS_USED          1
#define DIAG_GMAC_TX_BUFSIZE_WORDS    64  /* Comma seperated list of tx buffer sizes */
#define DIAG_GMAC_RX_BUFSIZE_WORDS    64  /* Comma seperated list of rx buffer sizes */
#define DIAG_GMAC_RX_ASSEMSIZE_WORDS  0x600 /* Comma seperated list of rx assembly sizes */
#define DIAG_GMAC_DMASIZE_TX_0        0x1000  /* Size of the entire buffer, words */
#define DIAG_GMAC_DMASIZE_RX_0        0x1000

/*****************************************************************************
 * DEFINITION: Fifo size used to store cells/packets received from the net
 *****************************************************************************/
#define DIAG_RX_NET_FIFO_SIZE           (4*928)  /* 32 58 byte cells        */

/****************************************************************************
 * DEFINITION: NEU build time configurations
 ****************************************************************************/
#define DIAG_INCLUDE_NEU
#define GG_INCLUDE_NEU_UDPIP       /* Only Required for RTP */
#define GG_INCLUDE_NEU_ETHERNET    /* Only Required for RTP */
#define GG_INCLUDE_NEU_PPP         /* Only Required for RTP */
#define GG_INCLUDE_NEU_LLCSNAP     /* Only Required for RTP */
#define DIAG_NEU_UDP_CORE_ID_MASK     0x001C  /* UDP core ID mask */


/*****************************************************************************
 * DEFINITION: Heap sizes
 *****************************************************************************/
#define DIAG_HEAP_SIZE        1  /* dummy as 0 is giving compile error */
#define DIAG_BOOT_HEAP_SIZE   0x02300
#define DIAG_HAL_HEAP_SIZE    0x00200

/*****************************************************************************
 * DEFINITION: System checksum block definitions. The attributes must
 *             agree with the placement in the linker command file.
 *****************************************************************************/
#define DIAG_INCLUDE_SYS_CHKSUM
#define DIAG_SYS_CHKSUM_NBLOCKS     12  /* 7 local blocks plus the shared memory, plus external */

/* Block 0 is the shared memory */
#define DIAG_SYS_CHKSUM_BLOCK_00_SIZE_WORDS  0x003FFD00
#define DIAG_SYS_CHKSUM_BLOCK_00_FLAGS       DCFG_SYS_CHKSUM_FLAG_SHARED

/* Block 1 is the local L2 from core 0 */
#define DIAG_SYS_CHKSUM_BLOCK_01_SIZE_WORDS  0x00030000
#define DIAG_SYS_CHKSUM_BLOCK_01_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(0)

/* Block 2 is the local L2 from core 1 */
#define DIAG_SYS_CHKSUM_BLOCK_02_SIZE_WORDS  0x00070000
#define DIAG_SYS_CHKSUM_BLOCK_02_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(1)

/* Block 3 is the local L2 from core 2 */
#define DIAG_SYS_CHKSUM_BLOCK_03_SIZE_WORDS  0x00070000
#define DIAG_SYS_CHKSUM_BLOCK_03_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(2)

/* Block 4 is the local L2 from core 3 */
#define DIAG_SYS_CHKSUM_BLOCK_04_SIZE_WORDS  0x00070000
#define DIAG_SYS_CHKSUM_BLOCK_04_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(3)

/* Block 5 is the local L2 from core 4 */
#define DIAG_SYS_CHKSUM_BLOCK_05_SIZE_WORDS  0x00070000
#define DIAG_SYS_CHKSUM_BLOCK_05_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(4)

/* Block 6 is the local L2 from core 5 */
#define DIAG_SYS_CHKSUM_BLOCK_06_SIZE_WORDS  0x00070000
#define DIAG_SYS_CHKSUM_BLOCK_06_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(5)

/* Block 7 is the local L2 from core 6 */
#define DIAG_SYS_CHKSUM_BLOCK_07_SIZE_WORDS  0x00070000
#define DIAG_SYS_CHKSUM_BLOCK_07_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(6)

/* Block 8 is the local L2 from core 7 */
#define DIAG_SYS_CHKSUM_BLOCK_08_SIZE_WORDS  0x00070000
#define DIAG_SYS_CHKSUM_BLOCK_08_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(7)

/* Block 9 is the external memory */
#define DIAG_SYS_CHKSUM_BLOCK_09_SIZE_WORDS  0x007F0000
#define DIAG_SYS_CHKSUM_BLOCK_09_FLAGS       DCFG_SYS_CHKSUM_FLAG_SHARED

/* Block 10 is the boot redirection block in core 0 */
#define DIAG_SYS_CHKSUM_BLOCK_10_SIZE_WORDS  0x00000020
#define DIAG_SYS_CHKSUM_BLOCK_10_FLAGS       DCFG_SYS_CHKSUM_FLAG_CORE(0)

/* Block 11 is the hole in shared for the interrupt vectors */
#define DIAG_SYS_CHKSUM_BLOCK_11_SIZE_WORDS  0x00000200
#define DIAG_SYS_CHKSUM_BLOCK_11_FLAGS       DCFG_SYS_CHKSUM_FLAG_SHARED
/**************************************************************************
 * CPPI and QM buffers
 **************************************************************************/
/* The following is configuration for Tx/Rx control and packet linked buffers.
   The total number of buffers per core must be 2^x (e.g.,128, 256,...).
   The size needs to be multiple of Cache line size (CACHE_L1D_LINESIZE = 64).  */
/* Rx control */
#define DIAG_QM_RX_LINKED_BUF_1_SIZE            320
#define DIAG_QM_RX_LINKED_BUF_1_NBUFS           5 /*(DIAG_MAX_CHANNELS + 4)*/

/* Tx control */
#define DIAG_QM_TX_LINKED_BUF_1_SIZE            320
#define DIAG_QM_TX_LINKED_BUF_1_NBUFS           5 /*(DIAG_MAX_CHANNELS + 4)*/

/* Rx packet */
#define DIAG_QM_RX_LINKED_BUF_2_SIZE            (((352+63)/64) * 64)
#define DIAG_QM_RX_LINKED_BUF_2_NBUFS           44

/* Tx packet */
#define DIAG_QM_TX_LINKED_BUF_2_SIZE            (((300+63)/64) * 64)
#define DIAG_QM_TX_LINKED_BUF_2_NBUFS           10

#define DIAG_QM_NUM_DESCS              (DIAG_QM_RX_LINKED_BUF_1_NBUFS + \
                                        DIAG_QM_TX_LINKED_BUF_1_NBUFS + \
                                        DIAG_QM_RX_LINKED_BUF_2_NBUFS + \
                                        DIAG_QM_TX_LINKED_BUF_2_NBUFS) /* Would need to be  2^x */
#define DIAG_QM_DESC_SIZE              80



