#ifndef _GGHWCFG_H
#define _GGHWCFG_H
/******************************************************************************
 * FILE PURPOSE: HAL build time configuration file
 ******************************************************************************
 * FILE NAME: gghwcfg.h
 *
 * DESCRIPTION: Build time configuration for HAL in the boot mode. Only one
 *              network port is enabled.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#define HAL_C64X 64

/* CPU CORE TYPE */
#define HAL_CONFIG_CPU_CORE             HAL_C64X

/* This particular build uses only one McBSP port does not use the Utopia Interface, but uses HPI */
#define GG_TDM_PORTS_USED               0

/* This particular build does not use the Utopia Interface, but uses HPI */
#define GG_UTOPIA_PORTS_USED            0

/* Include hal direct message support */
#define GG_HAL_INCLUDE_MSG

/* Include the DDR check */
#define GG_HWCFG_INCLUDE_DDR_CHECK

/* Include HAL gmac support */
//#define GG_HAL_INCLUDE_GMAC
//#define GG_GMAC_PORTS_USED              1

                   /*****************************************
                    *      Peripheral Support Config        *
                    *****************************************/

/*****************************************************************************
 * Cache build time configuration
 *****************************************************************************/
#define HAL_L1P_CACHE_0K               0x0
#define HAL_L1P_CACHE_4K               0x1 
#define HAL_L1P_CACHE_8K               0x2
#define HAL_L1P_CACHE_16K              0x3
#define HAL_L1P_CACHE_32K              0x4

#define HAL_L1P_CACHE_MODE             HAL_L1P_CACHE_32K 

#define HAL_L1D_CACHE_0K               0x0
#define HAL_L1D_CACHE_4K               0x1 
#define HAL_L1D_CACHE_8K               0x2
#define HAL_L1D_CACHE_16K              0x3
#define HAL_L1D_CACHE_32K              0x4

#define HAL_L1D_CACHE_MODE             HAL_L1D_CACHE_16K 

#define HAL_L2_CACHE_0K                0x0
#define HAL_L2_CACHE_32K               0x1 
#define HAL_L2_CACHE_64K               0x2
#define HAL_L2_CACHE_128K              0x3
#define HAL_L2_CACHE_256K              0x7

#define HAL_L2_CACHE_MODE              HAL_L2_CACHE_32K 


/* External memory cache configuration */
/* There are  256 MAR registers. Instead of specifying them all, a -1 terminated list of
 * mar registers and a corresponding list of values is specified. Any values not 
 * specified are left unchanged.
 *
 *  A value of 1 indicates cacheable, 0 non-cachable
 *
 *  MAR 224: 0xe0000000 - 0xe0ffffff  cacheable
 *  MAR 225: 0xe1000000 - 0xe1ffffff  cacheable
 */
#define HAL_MAR_REGS    { 224, 225, -1 }
#define HAL_MAR_VALS    {   1,   1,  0 }

/* MPAX Table setup to enable DDR MMRs */
#define HAL_MPAX_XMC_TBL                                     \
        {                                                    \
          {                                                  \
           2,                                                \
           0,                                                \
           0,                                                \
           0,                                                \
           CSL_DDR3_EMIF_CONFIG_REGS,                        \
           0x1,                                              \
           0x00000000,                                       \
           12,                                               \
           0x77                                              \
          }                                                  \
        }

/*****************************************************************************
 * EMIF Configuration Values for SRAM
 *****************************************************************************/
#define EMIF_CTLCE_WRITE_SETUP          2
#define EMIF_CTLCE_WRITE_STROBE         2
#define EMIF_CTLCE_WRITE_HOLD           1

#define EMIF_CTLCE_READ_SETUP           2
#define EMIF_CTLCE_READ_STROBE          2
#define EMIF_CTLCE_READ_HOLD            1

/*****************************************************************************
 * EMIF Configuration Values for DDR2 SDRAM
 *****************************************************************************/     
// Defines the CAS latency: 
#define HAL_EMIF_DDR2_CAS_LATENCY         5  // 2 to 5                                       
// No of banks of conected DDR SDRAM  
#define HAL_EMIF_DDR2_I_BANK              2  // 0->1bank, 1->2, 2->4, 3->8                               PS_256,
// Chip select signal used  
#define HAL_EMIF_DDR2_E_BANK              0  // 0->CS_1, 1->CS_2
// Page size of the connected DDR SDRAM
#define HAL_EMIF_DDR2_PAGE_SIZE           1  // 0->256, 1->512, 2->1024, 3->2048 words
// Data width of connected DDR SDRAM
#define HAL_EMIF_DDR2_DATA_WIDTH          0  // 0->32-bit, 1->16-bit

#define HAL_EMIF_DDR2_SDRAM_REFRESH_CTR   0x00000D06  // Default EMIF setting
#define HAL_EMIF_DDR2_SDRAM_TIMING_REG1   0xFFFFFFFB  // 0x26DB5BC9  
#define HAL_EMIF_DDR2_SDRAM_TIMING_REG2   0x01FFFFFF  // 0x0016C722  
                                                
#define HAL_EMIF_DDR2_PHY_RST             0x50006420
#define HAL_EMIF_DDR2_PHY_RLAT            2   // 1 to 7
/*******************************************************************************
 * MAC configuration
 *******************************************************************************/
#define GG_GMAC_HEAP_BUF_ALIGN   3   /* 32 bit alignments */
#define GG_GMAC_UNICAST_INDEX        /* Arrayed mac indexing used */
#define GG_GMAC_SOFT_RESET           /* Soft reset enabled */
#define GG_GMAC_MAX_ISR_COUNT  0     /* Number of interrupts before processing forced */
#define GG_GMAC_EMIC_WRAPPER         /* EMIC (ethernet multicore interrupt combiner */
#define GG_GMAC_PULSE_INTERRUPTS     /* Only pulse interrupts on Tomahawk */

/*******************************************************************************
 * MAC-EMIC Configuration. Disable the transmit and receive pacing units
 *******************************************************************************/
#define GG_GMAC_EMIC_RPCFG_VAL     0x0000000f
#define GG_GMAC_EMIC_TPCFG_VAL     0x0000000f
#define GG_GMAC_EMIC_PRESCALE_VAL  0x00000000


/*********************************************************************************
 * Interrupt configuration
 *********************************************************************************/
/* Vector 8 assigned to the edma memory transfers on channels 33-38, which are
 * routed through the edma region 0 register. Although this build is intended
 * to run only on core 0, there is no reason it couldn't run on all cores. On
 * tomahawk the tpcc local event corresponds to marx, where x is the core 0-5. */
#if 0
/* TPCC doesn't go through GEM directly.  Need to deal with INTC. */
#define GG_HWCFG_VEC08_ENABLED
#define CSL_DEVICE_ISR_DMA_XFER   CSL_DEVICE_ISR_TPCC_LOCAL
#define GG_HWCFG_VEC08_SOURCE     CSL_DEVICE_ISR_DMA_XFER
#endif

//#define GG_HWCFG_VEC15_ENABLED  
/* Temporay event mapping */
#define CSL_DEVICE_ISR_TIMER_8  66
#define GG_HWCFG_VEC15_SOURCE   CSL_DEVICE_ISR_TIMER_8

#define GG_DMA_CHANNELS_PER_CORE    2   /* 1 channel for memory moves */
#define GG_DMA_CHANNELS_N           12   /* 6 elements in the channels and reload sets array */
#define GG_DMA_CHANNELS         { 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44 }  /* Channels and event numbers are identical */
#define GG_DMA_RELOAD_SETS      { 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81 }  /* Parameter reloads on mem to mem transfer */
#define GG_DMA_QUEUTC_MAP       0x00003210  /* One queue to one transfer controller */
#define GG_DMA_QUEPRI           0x1110      /* Queue 0 gets priority 0, the others 1 */
#define GG_DMA_QNUM4            0x03232320  /* Maps channels 39:32 to queues. Other channel use the defaults */
#define GG_DMA_QNUM5            0x03232323  /* Maps channels 47:47 to queues. Other channel use the defaults */

/* There is only 1 edma interrupt source for this build per core. Core 0 uses
 * mar0, core 1 mar1, etc. */
#define GG_DMA_NUM_MAR_REGS     6
#define GG_DMA_NUM_ELEMS_MAR0       GG_DMA_CHANNELS_PER_CORE
#define GG_DMA_MAR_SOURCE_0         GG_DMA_CHANNELS

#define GG_DMA_SOURCE_ARRAY         { GG_DMA_MAR_SOURCE_0 }
#define GG_DMA_NSOURCE_ARRAY        { GG_DMA_NUM_ELEMS_MAR0, GG_DMA_NUM_ELEMS_MAR0,   \
                                      GG_DMA_NUM_ELEMS_MAR0, GG_DMA_NUM_ELEMS_MAR0,   \
                                      GG_DMA_NUM_ELEMS_MAR0, GG_DMA_NUM_ELEMS_MAR0 }



#endif

