/********************************************************************************** 
 * FILE PURPOSE: Allocate build time configurable buffers
 **********************************************************************************
 * FILE NAME: diagbuf.c
 *
 * DESCRIPTION: Instantiates the build time buffers
 *
 * Copyright (C) 2005, Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "gghwcfg.h"
#include "diagbld.h"


/**********************************************************************************
 * TDM Memory buffers
 **********************************************************************************/
#if ((DIAG_TDM_PORTS_USED >= 1) || (DIAG_ITDM_PORTS_USED >= 1))

#pragma DATA_SECTION(tdmDmaTx0, ".tdmDmaTx0")
word tdmDmaTx0[DIAG_TDM_DMA_BUFSIZE_0];

#pragma DATA_SECTION(tdmDmaRx0, ".tdmDmaRx0")
word tdmDmaRx0[DIAG_TDM_DMA_BUFSIZE_0];

#endif

/******************************************************************************
 * GMAC memory buffers
 ******************************************************************************/
#if (DIAG_GMAC_PORTS_USED >= 1)

#pragma DATA_SECTION(gmacDmaTx0, ".gmacDmaTx0")
word gmacDmaTx0[DIAG_GMAC_DMASIZE_TX_0];

#pragma DATA_SECTION(gmacDmaRx0, ".gmacDmaRx0")
word gmacDmaRx0[DIAG_GMAC_DMASIZE_RX_0];

#pragma DATA_SECTION(gmacRxAssem0, ".gmacRxAssem0")
word gmacRxAssem0[DIAG_GMAC_RX_ASSEMSIZE_WORDS];

#endif

#if (DIAG_GMAC_PORTS_USED >= 2)

#pragma DATA_SECTION(gmacDmaTx1, ".gmacDmaTx1")
word gmacDmaTx1[DIAG_GMAC_DMASIZE_TX_1];

#pragma DATA_SECTION(gmacDmaRx1, ".gmacDmaRx1")
word gmacDmaRx1[DIAG_GMAC_DMASIZE_RX_1];

#pragma DATA_SECTION(gmacRxAssem1, ".gmacRxAssem1")
word gmacRxAssem1[DIAG_GMAC_RX_ASSEMSIZE_WORDS];

#endif


/********************************************************************************
 * Utopia memory buffers
 ********************************************************************************/
#if (DIAG_UTOPIA_PORTS_USED >= 1)

#pragma DATA_SECTION(utopiaDmaTx0, ".utopiaDmaTx0")
word utopiaDmaTx0[DIAG_UTOPIA_DMASIZE_TX_0];

#pragma DATA_SECTION(utopiaDmaRx0, ".utopiaDmaRx0")
word utopiaDmaRx0[DIAG_UTOPIA_DMASIZE_RX_0];

#ifdef DIAG_UTOPIA_DMASIZE_CA_0
#pragma DATA_SECTION(utopiaDmaCa0, ".utopiaDmaCa0")
word utopiaDmaCa0[DIAG_UTOPIA_DMASIZE_CA_0];
#endif

#endif


/*********************************************************************************
 * Diag Heap Size
 *********************************************************************************/
#pragma DATA_SECTION(diagHeap, ".diagHeap")
word diagHeap[DIAG_HEAP_SIZE];

/*********************************************************************************
 * Boot Heap Size
 *********************************************************************************/
#pragma DATA_SECTION(bootHeap, ".bootHeap")
word bootHeap[DIAG_BOOT_HEAP_SIZE];

/*********************************************************************************
 * Hal Heap Size
 *********************************************************************************/
#pragma DATA_SECTION(halHeap, ".halHeap")
word halHeap[DIAG_HAL_HEAP_SIZE];
/**********************************************************************************
 * Memory test
 **********************************************************************************/
#ifdef DIAG_INCLUDE_MEMTEST

#if (DIAG_MEMTEST_NBLOCKS > 0)
  #pragma DATA_SECTION(mtblock0, ".mtblock0")
  word mtblock0[DIAG_MEMTEST_BLOCK_0_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 1)
  #pragma DATA_SECTION(mtblock1, ".mtblock1")
  word mtblock1[DIAG_MEMTEST_BLOCK_1_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 2)
  #pragma DATA_SECTION(mtblock2, ".mtblock2")
  word mtblock2[DIAG_MEMTEST_BLOCK_2_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 3)
  #pragma DATA_SECTION(mtblock3, ".mtblock3")
  word mtblock3[DIAG_MEMTEST_BLOCK_3_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 4)
  #pragma DATA_SECTION(mtblock4, ".mtblock4")
  word mtblock4[DIAG_MEMTEST_BLOCK_4_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 5)
  #pragma DATA_SECTION(mtblock5, ".mtblock5")
  word mtblock5[DIAG_MEMTEST_BLOCK_5_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 6)
  #pragma DATA_SECTION(mtblock6, ".mtblock6")
  word mtblock6[DIAG_MEMTEST_BLOCK_6_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 7)
  #pragma DATA_SECTION(mtblock7, ".mtblock7")
  word mtblock7[DIAG_MEMTEST_BLOCK_7_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 8)
  #pragma DATA_SECTION(mtblock8, ".mtblock8")
  word mtblock8[DIAG_MEMTEST_BLOCK_8_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 9)
  #pragma DATA_SECTION(mtblock9, ".mtblock9")
  word mtblock9[DIAG_MEMTEST_BLOCK_9_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 10)
  #pragma DATA_SECTION(mtblock10, ".mtblock10")
  word mtblock10[DIAG_MEMTEST_BLOCK_10_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 11)
  #pragma DATA_SECTION(mtblock11, ".mtblock11")
  word mtblock11[DIAG_MEMTEST_BLOCK_11_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 12)
  #pragma DATA_SECTION(mtblock12, ".mtblock12")
  word mtblock12[DIAG_MEMTEST_BLOCK_12_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 13)
  #pragma DATA_SECTION(mtblock13, ".mtblock13")
  word mtblock13[DIAG_MEMTEST_BLOCK_13_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 14)
  #pragma DATA_SECTION(mtblock14, ".mtblock14")
  word mtblock14[DIAG_MEMTEST_BLOCK_14_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 15)
  #pragma DATA_SECTION(mtblock15, ".mtblock15")
  word mtblock15[DIAG_MEMTEST_BLOCK_15_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 16)
  #pragma DATA_SECTION(mtblock16, ".mtblock16")
  word mtblock16[DIAG_MEMTEST_BLOCK_16_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 17)
  #pragma DATA_SECTION(mtblock17, ".mtblock17")
  word mtblock17[DIAG_MEMTEST_BLOCK_17_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 18)
  #pragma DATA_SECTION(mtblock18, ".mtblock18")
  word mtblock18[DIAG_MEMTEST_BLOCK_18_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 19)
  #pragma DATA_SECTION(mtblock19, ".mtblock19")
  word mtblock19[DIAG_MEMTEST_BLOCK_19_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 20)
  #pragma DATA_SECTION(mtblock20, ".mtblock20")
  word mtblock20[DIAG_MEMTEST_BLOCK_20_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 21)
  #pragma DATA_SECTION(mtblock21, ".mtblock21")
  word mtblock21[DIAG_MEMTEST_BLOCK_21_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 22)
  #pragma DATA_SECTION(mtblock22, ".mtblock22")
  word mtblock22[DIAG_MEMTEST_BLOCK_22_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 23)
  #pragma DATA_SECTION(mtblock23, ".mtblock23")
  word mtblock23[DIAG_MEMTEST_BLOCK_23_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 24)
  #pragma DATA_SECTION(mtblock24, ".mtblock24")
  word mtblock24[DIAG_MEMTEST_BLOCK_24_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 25)
  #pragma DATA_SECTION(mtblock25, ".mtblock25")
  word mtblock25[DIAG_MEMTEST_BLOCK_25_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 26)
  #pragma DATA_SECTION(mtblock26, ".mtblock26")
  word mtblock26[DIAG_MEMTEST_BLOCK_26_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 27)
  #pragma DATA_SECTION(mtblock27, ".mtblock27")
  word mtblock27[DIAG_MEMTEST_BLOCK_27_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 28)
  #pragma DATA_SECTION(mtblock28, ".mtblock28")
  word mtblock28[DIAG_MEMTEST_BLOCK_28_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 29)
  #pragma DATA_SECTION(mtblock29, ".mtblock29")
  word mtblock29[DIAG_MEMTEST_BLOCK_29_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 30)
  #pragma DATA_SECTION(mtblock30, ".mtblock30")
  word mtblock30[DIAG_MEMTEST_BLOCK_30_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 31)
  #pragma DATA_SECTION(mtblock31, ".mtblock31")
  word mtblock31[DIAG_MEMTEST_BLOCK_31_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 32)
  #pragma DATA_SECTION(mtblock32, ".mtblock32")
  word mtblock32[DIAG_MEMTEST_BLOCK_32_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 33)
  #pragma DATA_SECTION(mtblock33, ".mtblock33")
  word mtblock33[DIAG_MEMTEST_BLOCK_33_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 34)
  #pragma DATA_SECTION(mtblock34, ".mtblock34")
  word mtblock34[DIAG_MEMTEST_BLOCK_34_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 35)
  #pragma DATA_SECTION(mtblock35, ".mtblock35")
  word mtblock35[DIAG_MEMTEST_BLOCK_35_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 36)
  #pragma DATA_SECTION(mtblock36, ".mtblock36")
  word mtblock36[DIAG_MEMTEST_BLOCK_36_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 37)
  #pragma DATA_SECTION(mtblock37, ".mtblock37")
  word mtblock37[DIAG_MEMTEST_BLOCK_37_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 38)
  #pragma DATA_SECTION(mtblock38, ".mtblock38")
  word mtblock38[DIAG_MEMTEST_BLOCK_38_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 39)
  #pragma DATA_SECTION(mtblock39, ".mtblock39")
  word mtblock39[DIAG_MEMTEST_BLOCK_39_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 40)
  #pragma DATA_SECTION(mtblock40, ".mtblock40")
  word mtblock40[DIAG_MEMTEST_BLOCK_40_SIZE];
#endif

#if (DIAG_MEMTEST_NBLOCKS > 41)
  #error diagbuf.c must be updated for more memory blocks
#endif
#endif /* DIAG_INCLUDE_MEMTEST */

/* QM buffers */
#if (DIAG_QM_RX_LINKED_BUF_1_SIZE) 
  #pragma DATA_SECTION(rxlinkramQ1, ".cppiMemRXLinkRamQ1")
  #pragma DATA_ALIGN(rxlinkramQ1, 64)
  word rxlinkramQ1[DIAG_QM_RX_LINKED_BUF_1_SIZE * DIAG_QM_RX_LINKED_BUF_1_NBUFS];
#endif

#if (DIAG_QM_RX_LINKED_BUF_2_SIZE) 
  #pragma DATA_SECTION(rxlinkramQ2, ".cppiMemRXLinkRamQ2")
  #pragma DATA_ALIGN(rxlinkramQ2, 64)
  word rxlinkramQ2[DIAG_QM_RX_LINKED_BUF_2_SIZE * DIAG_QM_RX_LINKED_BUF_2_NBUFS];
#endif

#if (DIAG_QM_RX_LINKED_BUF_3_SIZE) 
  #pragma DATA_SECTION(rxlinkramQ3, ".cppiMemRXLinkRamQ3")
  #pragma DATA_ALIGN(rxlinkramQ3, 64)
  word rxlinkramQ3[DIAG_QM_RX_LINKED_BUF_3_SIZE * DIAG_QM_RX_LINKED_BUF_3_NBUFS];
#endif

#if (DIAG_QM_RX_LINKED_BUF_4_SIZE) 
  #pragma DATA_SECTION(rxlinkramQ4, ".cppiMemRXLinkRamQ4")
  #pragma DATA_ALIGN(rxlinkramQ4, 64)
  word rxlinkramQ4[DIAG_QM_RX_LINKED_BUF_4_SIZE * DIAG_QM_RX_LINKED_BUF_4_NBUFS];
#endif

#if (DIAG_QM_TX_LINKED_BUF_1_SIZE) 
  #pragma DATA_SECTION(txlinkramQ1, ".cppiMemTXLinkRamQ1")
  #pragma DATA_ALIGN(txlinkramQ1, 64)
  word txlinkramQ1[DIAG_QM_TX_LINKED_BUF_1_SIZE * DIAG_QM_TX_LINKED_BUF_1_NBUFS];
#endif

#if (DIAG_QM_TX_LINKED_BUF_2_SIZE) 
  #pragma DATA_SECTION(txlinkramQ2, ".cppiMemTXLinkRamQ2")
  #pragma DATA_ALIGN(txlinkramQ2, 64)
  word txlinkramQ2[DIAG_QM_TX_LINKED_BUF_2_SIZE * DIAG_QM_TX_LINKED_BUF_2_NBUFS];
#endif

#if (DIAG_QM_TX_LINKED_BUF_3_SIZE) 
  #pragma DATA_SECTION(txlinkramQ3, ".cppiMemTXLinkRamQ3")
  #pragma DATA_ALIGN(txlinkramQ3, 64)
  word txlinkramQ3[DIAG_QM_TX_LINKED_BUF_3_SIZE * DIAG_QM_TX_LINKED_BUF_3_NBUFS];
#endif

#if (DIAG_QM_TX_LINKED_BUF_4_SIZE) 
  #pragma DATA_SECTION(txlinkramQ4, ".cppiMemTXLinkRamQ4")
  #pragma DATA_ALIGN(txlinkramQ4, 64)
  word txlinkramQ4[DIAG_QM_TX_LINKED_BUF_4_SIZE * DIAG_QM_TX_LINKED_BUF_4_NBUFS];
#endif

#if (DIAG_QM_DESC_SIZE)
  #pragma DATA_SECTION(QMemDescRam, ".QMemDescRam")
  #pragma DATA_ALIGN(QMemDescRam, 64)
  word QMemDescRam[DIAG_QM_DESC_SIZE * DIAG_QM_NUM_DESCS];
#endif

