/*****************************************************************************
 * FILE PURPOSE: Initialize build time configuration structure
 *****************************************************************************
 * FILE NAME: diagcfg.c
 *
 * DESCRIPTION: Initializes the diagCfg structure with values defined in
 *              diagbld.h. Only this file and diagbuf.c should include 
 *              diagbld.h. 
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "diagbld.h"
#include "diagcfg.h"
#include "gghwcfg.h"
#include "dsiuboot.h"
#include "diagapp.h"
#include "dsiuctbl.h"
#include <stdlib.h>
#include "dsiuotbrun.h"

/*****************************************************************************
 * Force a define value of DIAG_ITDM_PORTS_USED to 0 in this file only
 * if not used in the build
 *****************************************************************************/
#ifndef DIAG_ITDM_PORTS_USED
#define DIAG_ITDM_PORTS_USED 0
#endif

/*****************************************************************************
 * Allocation of the status/control registers
 *****************************************************************************/
#pragma DATA_SECTION(dregStatus, ".dregStatus")
dregStatus_t dregStatus;

#pragma DATA_SECTION(dregBootTrack, ".dregBootTrack")
dregBootTrack_t dregBootTrack;

#pragma DATA_SECTION(dregMasterCntrl, ".dregMasterCntrl")
dregMasterCntrl_t dregMasterCntrl;

#ifdef DIAG_INCLUDE_BLOCK_MSG
 #pragma DATA_SECTION(dregBlockMsgCntlMem, ".dregBlockMsgCntrl")
 dregBlockMsgCntlMem_t dregBlockMsgCntlMem;
#endif

#pragma DATA_SECTION(dregMsgCfg, ".dregMsgCfg")
dregNetCfg_t dregMsgCfg;

#pragma DATA_SECTION(dregMsgCfgShadow, ".dregMsgCfgShadow")
dregNetCfg_t dregMsgCfgShadow;

#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))

#pragma DATA_SECTION(dregTdmCfg, ".dregTdmCfg")
dregTdmCfg_t dregTdmCfg[DIAG_MAX_CHANNELS];

#pragma DATA_SECTION(dregVchCfgBlk, ".dregVchCfgBlk")
dregVchCfgBlk_t dregVchCfgBlk[DIAG_MAX_CHANNELS];

#ifdef DIAG_MAX_ITDM_STREAMS

#pragma DATA_SECTION(itdmStreamCfg, ".itdmStreamCfg")
itdmStreamCfg_t itdmStreamCfg[DIAG_MAX_ITDM_STREAMS];

#pragma DATA_SECTION(itdmChanUpdateMsg, ".itdmChanUpdateMsg")
itdmChanUpdateMsg_t itdmChanUpdateMsg;

#endif

#pragma DATA_SECTION(dregRouterCfg, ".dregRouterCfg")
dregRouterCfg_t dregRouterCfg[2*DIAG_MAX_CHANNELS];

#pragma DATA_SECTION(dregBertStats, ".dregBertStats")
dregBertStats_t dregBertStats[DIAG_MAX_CHANNELS];

#pragma DATA_SECTION(dregOpStats, ".dregOpStat")
dregOpStats_t dregOpStats;

#pragma DATA_SECTION(dregBertTestCntl, ".dregBertTestCntl")
/*initialize BERT TEST control structure*/
dregBertTestCntl_t dregBertTestCntl=
{
  DREG_ROUTE_OUT_SRC_BERT_255,/*dregBertTestCntl.bertMode*/
  0,                          /*dregBertTestCntl.firstCh*/
  DIAG_MAX_CHANNELS,          /*dregBertTestCntl.nCh*/
  TRUE                        /*dregBertTestCntl.periodicReport*/
};  


/* Simulation options*/
#pragma DATA_SECTION(dsiuSimConfig, ".dsiuSimConfig")
  /*initialize simulation config structure*/
dsiuSimConfig_t dsiuSimConfig=
{
#ifdef DIAG_INCLUDE_OTBRUN
  TRUE, /*dsiuSimConfig.opt_diag_otb_BERT*/
#else
  FALSE,
#endif
#ifdef DIAG_INCLUDE_TDMSIM
  TRUE, /*dsiuSimConfig.opt_simulate_tsip_isr*/
#else
  FALSE,
#endif
  DIAG_MAX_CHANNELS, /*dsiuSimConfig.tsipIsrNumCh*/
  DSIU_SIM_TDM_PATTERN_NO_PATTERN, /*dsiuSimConfig.opt_tdm_input_pattern*/
  /*dsiuSimConfig.inputPattern[SIM_TDM_PATTERN_SIZE]*/
  {0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,
   0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,
   0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,
   0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234,0xABCD,0x1234} 
};


#ifndef DIAG_MSG_BLOCK_N_MSGS
#define DIAG_MSG_BLOCK_N_MSGS  0
#endif

#if (DIAG_MSG_BLOCK_N_MSGS > 0)
 #pragma DATA_SECTION(dregMsgBlock, ".dregMsgBlock")
 dregMsgBlock_t dregMsgBlock[DIAG_MSG_BLOCK_N_MSGS];
#endif

/* Memory Tests predefined messages*/
#if (DIAG_PREDEF_MSG_N_MSGS > 0)
 #if (DIAG_PREDEF_MSG_N_MSGS != 6)
  #error diagcfg.c must be expanded to handle a different number of pre-defined messages
 #endif
 #pragma DATA_SECTION(dregMsgInit, ".dregMsgInit")
 dregMsgBlock_t dregMsgInit[DIAG_PREDEF_MSG_N_MSGS] = {
               { DIAG_PREDEF_PARAMS_0  },
               { DIAG_PREDEF_PARAMS_1  },
               { DIAG_PREDEF_PARAMS_2  },
               { DIAG_PREDEF_PARAMS_3  },
               { DIAG_PREDEF_PARAMS_4  },
               { DIAG_PREDEF_PARAMS_5  }
               };
#endif
                   
/* Out of the Box BERT Tests predefined messages*/
#if (DIAG_PREDEF_OTB_BERT_N_MSGS > 0)
 #if (DIAG_PREDEF_OTB_BERT_N_MSGS > 3)
  #error diagcfg.c must be expanded to handle a different number of pre-defined messages for OTB BERT test
 #endif
 #pragma DATA_SECTION(dregMsgOTBBertInit, ".dregMsgOTBBertInit")
 dregMsgBlock_t dregMsgOTBBertInit[DIAG_PREDEF_OTB_BERT_N_MSGS] = {
               { DIAG_PREDEF_OTB_BERT_PARAMS_0  },
               { DIAG_PREDEF_OTB_BERT_PARAMS_1  }
#ifdef DIAG_PREDEF_OTB_BERT_PARAMS_2
             , { DIAG_PREDEF_OTB_BERT_PARAMS_2  }
#endif
               };
#endif



#ifdef DIAG_INCLUDE_AUTOCTL
 #pragma DATA_SECTION(dregReboot, ".dregReboot")
 dregReboot_t dregReboot;

 #pragma DATA_SECTION(dregAutoCtl, ".dregAutoCtl")
 dregAutoCtl_t dregAutoCtl;
#endif


#ifdef DIAG_MEM_CLEAR_NBLOCKS
 #if (DIAG_MEM_CLEAR_NBLOCKS > 0)
  #if (DIAG_MEM_CLEAR_NBLOCKS != 4)
   #error diagcfg must be expanded to handle a different number of pre-defined memory clear blocks
  #endif
  #pragma DATA_SECTION(dregMemClearLoad, ".dregMemClearLoad")
  dregMemClear_t dregMemClearLoad[DIAG_MEM_CLEAR_NBLOCKS + 1] = {
                 { DIAG_MEM_CLEAR_BLOCK_0 },
                 { DIAG_MEM_CLEAR_BLOCK_1 },
                 { DIAG_MEM_CLEAR_BLOCK_2 },
                 { DIAG_MEM_CLEAR_BLOCK_3 },
                 {     0, NULL, 0         }
                 };

  #pragma DATA_SECTION(dregMemClearRun, ".dregMemClearRun")
  dregMemClear_t dregMemClearRun[DIAG_MEM_CLEAR_NBLOCKS + 1];
   
 #endif
#endif

#ifdef DIAG_PREDEF_DIAGAPP_MEM
 #pragma DATA_SECTION(diagAppPredef, ".diagAppPredef")
 const UINT16  diagAppPredef[] = { DIAG_PREDEF_DIAGAPP_MEM };
#endif



/*****************************************************************************
 * Allocation of the shadow config registers. These are not visible to
 * the host. These reflect the current operational status of the system
 *****************************************************************************/
#pragma DATA_SECTION(dregTdmCfgShadow, ".dregTdmCfgShadow")
dregTdmCfg_t dregTdmCfgShadow[DIAG_MAX_CHANNELS];

/* The shadow for the network configuration registers does not
 * require the blocking */
#pragma DATA_SECTION(dregVchCfgShadow, ".dregVchCfgShadow")
dregNetCfg_t dregVchCfgShadow[DIAG_MAX_CHANNELS];

#pragma DATA_SECTION(dregRouterCfgShadow, ".dregRouterCfgShadow")
dregRouterCfg_t dregRouterCfgShadow[2*DIAG_MAX_CHANNELS];



#endif /* (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST)) */


/*****************************************************************************
 * The diag to app exchange structure. Declared here and will be placed on a 
 * non-real data page to verify size requirements.
 *****************************************************************************/
#pragma DATA_SECTION(diagAppMem, ".diagApp")
diagApp_t diagAppMem;

/*****************************************************************************
 * The DSIU Call table.
 *****************************************************************************/
dsiuCallTable_t dsiuCallTable = 
{
#ifdef DIAG_INCLUDE_NEU_VOICE
  dsiuNeuCreateVoiceNeu,  /* createVoiceNeu */
  dsiuNeuOpenVoiceNeu,    /* openVoiceNeu   */
#else
  NULL,
  NULL,
#endif

#ifdef DIAG_INCLUDE_BERT
  dsiuCreateBert,         /* createBert */
  dsiuOpenBert,           /* openBert   */
  dsiuCloseBert,          /* closeBert  */
#else
  NULL,
  NULL,
  NULL,
#endif

#ifdef BOOT_PREV_INFO_FCN
  BOOT_PREV_INFO_FCN,    /* prevBootInfo */
#else
  NULL,
#endif

#ifdef MEM_CONFIG_FCN
  MEM_CONFIG_FCN,       /* memConfig */
#else
  NULL,
#endif

#ifdef DIAG_INCLUDE_BLOCK_MSG
  dsiuBlockProc,     /* blockMsgProc */
  dsiuBlockInit      /* blockMsgInit */
#else
  NULL,
  NULL
#endif


};

/*****************************************************************************
 * DCFG_C_MAX_MEM_BLOCKS is the number of memory blocks of each type defined
 * in this file. This defined value is not actually used, but care must be
 * taken when changing the number of memory blocks assigned in the linker
 * command file.
 ****************************************************************************/
#define DCFG_C_MAX_MEM_BLOCKS  5

#if (DCFG_C_MAX_MEM_BLOCKS != DCFG_MAX_MEM_BLOCKS)
# error The max number of memory blocks defined in diagcfg.c and diagcfg.h must agree
#endif

/******************************************************************************
 * Define the number of TDM ports used to 0 if it has not been defined
 *****************************************************************************/
#ifndef DIAG_TDM_PORTS_USED     
#define DIAG_TDM_PORTS_USED 0  
#endif                         

/*****************************************************************************
 * Define the number of utopia port used to 0 if it has not been defined
 *****************************************************************************/
#ifndef DIAG_UTOPIA_PORTS_USED
#define DIAG_UTOPIA_PORTS_USED  0
#endif

#ifdef DIAG_SYS_CHKSUM_BLOCK_0_SIZE_WORDS
#error diagcfg.h needs to be updated defining BLOCK_00 instead of BLOCK_0
#endif

/***************************************************************************
 * System memory check blocks
 ***************************************************************************/
#ifdef DIAG_SYS_CHKSUM_NBLOCKS
  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 1)
    #pragma DATA_SECTION(sys_blk00, ".sys_blk00")
    word sys_blk00[DIAG_SYS_CHKSUM_BLOCK_00_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 2)
    #pragma DATA_SECTION(sys_blk01, ".sys_blk01")
    word sys_blk01[DIAG_SYS_CHKSUM_BLOCK_01_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 3)
    #pragma DATA_SECTION(sys_blk02, ".sys_blk02")
    word sys_blk02[DIAG_SYS_CHKSUM_BLOCK_02_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 4)
    #pragma DATA_SECTION(sys_blk03, ".sys_blk03")
    word sys_blk03[DIAG_SYS_CHKSUM_BLOCK_03_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 5)
    #pragma DATA_SECTION(sys_blk04, ".sys_blk04")
    word sys_blk04[DIAG_SYS_CHKSUM_BLOCK_04_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 6)
    #pragma DATA_SECTION(sys_blk05, ".sys_blk05")
    word sys_blk05[DIAG_SYS_CHKSUM_BLOCK_05_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 7)
    #pragma DATA_SECTION(sys_blk06, ".sys_blk06")
    word sys_blk06[DIAG_SYS_CHKSUM_BLOCK_06_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 8)
    #pragma DATA_SECTION(sys_blk07, ".sys_blk07")
    word sys_blk07[DIAG_SYS_CHKSUM_BLOCK_07_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 9)
    #pragma DATA_SECTION(sys_blk08, ".sys_blk08")
    word sys_blk08[DIAG_SYS_CHKSUM_BLOCK_08_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 10)
    #pragma DATA_SECTION(sys_blk09, ".sys_blk09")
    word sys_blk09[DIAG_SYS_CHKSUM_BLOCK_09_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 11)
    #pragma DATA_SECTION(sys_blk10, ".sys_blk10")
    word sys_blk10[DIAG_SYS_CHKSUM_BLOCK_10_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 12)
    #pragma DATA_SECTION(sys_blk11, ".sys_blk11")
    word sys_blk11[DIAG_SYS_CHKSUM_BLOCK_11_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 13)
    #pragma DATA_SECTION(sys_blk12, ".sys_blk12")
    word sys_blk12[DIAG_SYS_CHKSUM_BLOCK_12_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 14)
    #pragma DATA_SECTION(sys_blk13, ".sys_blk13")
    word sys_blk13[DIAG_SYS_CHKSUM_BLOCK_13_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 15)
    #pragma DATA_SECTION(sys_blk14, ".sys_blk14")
    word sys_blk14[DIAG_SYS_CHKSUM_BLOCK_14_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 16)
    #pragma DATA_SECTION(sys_blk15, ".sys_blk15")
    word sys_blk15[DIAG_SYS_CHKSUM_BLOCK_15_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 17)
    #pragma DATA_SECTION(sys_blk16, ".sys_blk16")
    word sys_blk16[DIAG_SYS_CHKSUM_BLOCK_16_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 18)
    #pragma DATA_SECTION(sys_blk17, ".sys_blk17")
    word sys_blk17[DIAG_SYS_CHKSUM_BLOCK_17_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 19)
    #pragma DATA_SECTION(sys_blk18, ".sys_blk18")
    word sys_blk18[DIAG_SYS_CHKSUM_BLOCK_18_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 20)
    #pragma DATA_SECTION(sys_blk19, ".sys_blk19")
    word sys_blk19[DIAG_SYS_CHKSUM_BLOCK_19_SIZE_WORDS];
  #endif

  /* -------------------------------------------------------- */

  #if (DIAG_SYS_CHKSUM_NBLOCKS >= 21)
    #error diagcfg.c must be updated to increase the number of blocks
  #endif


/*****************************************************************************
 * Begin SIZES table
 ****************************************************************************/
tulong dcfgSysChkBlockSizeWords[] = 
{
#if (DIAG_SYS_CHKSUM_NBLOCKS > 0)
    DIAG_SYS_CHKSUM_BLOCK_00_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 1)
  , DIAG_SYS_CHKSUM_BLOCK_01_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 2)
  , DIAG_SYS_CHKSUM_BLOCK_02_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 3)
  , DIAG_SYS_CHKSUM_BLOCK_03_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 4)
  , DIAG_SYS_CHKSUM_BLOCK_04_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 5)
  , DIAG_SYS_CHKSUM_BLOCK_05_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 6)
  , DIAG_SYS_CHKSUM_BLOCK_06_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 7)
  , DIAG_SYS_CHKSUM_BLOCK_07_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 8)
  , DIAG_SYS_CHKSUM_BLOCK_08_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 9)
  , DIAG_SYS_CHKSUM_BLOCK_09_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 10)
  , DIAG_SYS_CHKSUM_BLOCK_10_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 11)
  , DIAG_SYS_CHKSUM_BLOCK_11_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 12)
  , DIAG_SYS_CHKSUM_BLOCK_12_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 13)
  , DIAG_SYS_CHKSUM_BLOCK_13_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 14)
  , DIAG_SYS_CHKSUM_BLOCK_14_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 15)
  , DIAG_SYS_CHKSUM_BLOCK_15_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 16)
  , DIAG_SYS_CHKSUM_BLOCK_16_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 17)
  , DIAG_SYS_CHKSUM_BLOCK_17_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 18)
  , DIAG_SYS_CHKSUM_BLOCK_18_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 19)
  , DIAG_SYS_CHKSUM_BLOCK_19_SIZE_WORDS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 20)
  , DIAG_SYS_CHKSUM_BLOCK_20_SIZE_WORDS
#endif
};

/*****************************************************************************
 * Begin FLAGS table
 ****************************************************************************/
tint   dcfgSysChkBlockFlags[] = {
#if (DIAG_SYS_CHKSUM_NBLOCKS > 0)
  DIAG_SYS_CHKSUM_BLOCK_00_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 1)
  , DIAG_SYS_CHKSUM_BLOCK_01_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 2)
  , DIAG_SYS_CHKSUM_BLOCK_02_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 3)
  , DIAG_SYS_CHKSUM_BLOCK_03_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 4)
  , DIAG_SYS_CHKSUM_BLOCK_04_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 5)
  , DIAG_SYS_CHKSUM_BLOCK_05_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 6)
  , DIAG_SYS_CHKSUM_BLOCK_06_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 7)
  , DIAG_SYS_CHKSUM_BLOCK_07_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 8)
  , DIAG_SYS_CHKSUM_BLOCK_08_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 9)
  , DIAG_SYS_CHKSUM_BLOCK_09_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 10)
  , DIAG_SYS_CHKSUM_BLOCK_10_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 11)
  , DIAG_SYS_CHKSUM_BLOCK_11_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 12)
  , DIAG_SYS_CHKSUM_BLOCK_12_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 13)
  , DIAG_SYS_CHKSUM_BLOCK_13_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 14)
  , DIAG_SYS_CHKSUM_BLOCK_14_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 15)
  , DIAG_SYS_CHKSUM_BLOCK_15_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 16)
  , DIAG_SYS_CHKSUM_BLOCK_16_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 17)
  , DIAG_SYS_CHKSUM_BLOCK_17_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 18)
  , DIAG_SYS_CHKSUM_BLOCK_18_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 19)
  , DIAG_SYS_CHKSUM_BLOCK_19_FLAGS
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 20)
  , DIAG_SYS_CHKSUM_BLOCK_20_FLAGS
#endif
};

/*****************************************************************************
 * Begin CHK BLOCKS table
 ****************************************************************************/
word  *dcfgSysChkBlocks[] = {
#if (DIAG_SYS_CHKSUM_NBLOCKS > 0)
  sys_blk00
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 1)
  , sys_blk01
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 2)
  , sys_blk02
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 3)
  , sys_blk03
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 4)
  , sys_blk04
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 5)
  , sys_blk05
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 6)
  , sys_blk06
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 7)
  , sys_blk07
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 8)
  , sys_blk08
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 9)
  , sys_blk09
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 10)
  , sys_blk10
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 11)
  , sys_blk11
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 12)
  , sys_blk12
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 13)
  , sys_blk13
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 14)
  , sys_blk14
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 15)
  , sys_blk15
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 16)
  , sys_blk16
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 17)
  , sys_blk17
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 18)
  , sys_blk18
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 19)
  , sys_blk19
#endif
#if (DIAG_SYS_CHKSUM_NBLOCKS > 20)
  , sys_blk20
#endif
};

#endif /* DIAG_SYS_CHKSUM_NBLOCKS */

/*****************************************************************************
 * The application provides buffer for the HAL (Hardware Abstraction Layer)
 * tdm port in the linker command file. The number of buffers is dependent on 
 * the number of TDM ports to be used.
 *****************************************************************************/
extern tuint *_lcf_hal_heap_start;  /* The hal heap buffer start addr */
extern tuint *_lcf_hal_heap_end;    /* The hal heap buffer end addr   */

#if ( (DIAG_TDM_PORTS_USED >= 1) || (DIAG_ITDM_PORTS_USED >= 1) )
extern tuint *_lcf_hal_tdm_tx_a_start;
extern tuint *_lcf_hal_tdm_tx_a_end;
extern tuint *_lcf_hal_tdm_rx_a_start;
extern tuint *_lcf_hal_tdm_rx_a_end;
#endif

#if ( (DIAG_TDM_PORTS_USED >= 2) || (DIAG_ITDM_PORTS_USED >= 2) )
extern tuint *_lcf_hal_tdm_tx_b_start;
extern tuint *_lcf_hal_tdm_tx_b_end;
extern tuint *_lcf_hal_tdm_rx_b_start;
extern tuint *_lcf_hal_tdm_rx_b_end;
#endif

#if ( (DIAG_TDM_PORTS_USED >= 3) || (DIAG_ITDM_PORTS_USED >= 3) )
extern tuint *_lcf_hal_tdm_tx_c_start;
extern tuint *_lcf_hal_tdm_tx_c_end;
extern tuint *_lcf_hal_tdm_rx_c_start;
extern tuint *_lcf_hal_tdm_rx_c_end;
#endif


#if (DIAG_UTOPIA_PORTS_USED >= 1)
extern tuint *_lcf_hal_pkt_tx_a_start;
extern tuint *_lcf_hal_pkt_tx_a_end;
extern tuint *_lcf_hal_pkt_rx_a_start;
extern tuint *_lcf_hal_pkt_rx_a_end;
 #ifdef DIAG_UTOPIA_DMASIZE_CA_0
 extern tuint *_lcf_hal_pkt_ca_a_start;
 extern tuint *_lcf_hal_pkt_ca_a_end;
 #endif
#endif

#if (DIAG_UTOPIA_PORTS_USED >= 2)
extern tuint *_lcf_hal_pkt_tx_b_start;
extern tuint *_lcf_hal_pkt_tx_b_end;
extern tuint *_lcf_hal_pkt_rx_b_start;
extern tuint *_lcf_hal_pkt_rx_b_end;
 #ifdef DIAG_UTOPIA_DMASIZE_CA_1
 extern tuint *_lcf_hal_pkt_ca_b_start;
 extern tuint *_lcf_hal_pkt_ca_b_end;
 #endif
#endif

#if (DIAG_GMAC_PORTS_USED >= 1)
extern tuint *_lcf_hal_gmac_tx_a_start;
extern tuint *_lcf_hal_gmac_tx_a_end;
extern tuint *_lcf_hal_gmac_rx_a_start;
extern tuint *_lcf_hal_gmac_rx_a_end;
extern tuint *_lcf_hal_gmac_rx_assem_a_start;
extern tuint *_lcf_hal_gmac_rx_assem_a_end;
#endif

#if (DIAG_GMAC_PORTS_USED >= 2)
extern tuint *_lcf_hal_gmac_tx_b_start;
extern tuint *_lcf_hal_gmac_tx_b_end;
extern tuint *_lcf_hal_gmac_rx_b_start;
extern tuint *_lcf_hal_gmac_rx_b_end;
extern tuint *_lcf_hal_gmac_rx_assem_b_start;
extern tuint *_lcf_hal_gmac_rx_assem_b_end;
#endif

/****************************************************************************
 * The application heap buffers
 ****************************************************************************/
extern tulong _lcf_boot_heap_buf_start;
extern tulong _lcf_boot_heap_buf_end;

extern tulong _lcf_diag_heap_buf_start;
extern tulong _lcf_diag_heap_buf_end;


/*****************************************************************************
 * Declaration: The array of tdm buffer addresses and sizes
 *****************************************************************************/
#if ( (DIAG_TDM_PORTS_USED == 0) && (DIAG_ITDM_PORTS_USED == 0) )

dcfgDma_t dcfgTdmDma[] =  {  {  NULL, NULL, NULL, NULL  }  };

#else

dcfgDma_t dcfgTdmDma[] = {

#if ( (DIAG_TDM_PORTS_USED >= 1) || (DIAG_ITDM_PORTS_USED >= 1) )
    { 
  (tuint *) &_lcf_hal_tdm_tx_a_start,      /* TDM A, DMA tx buffer start  */
  (tuint *) &_lcf_hal_tdm_tx_a_end,        /* TDM A, DMA tx buffer end    */
  (tuint *) &_lcf_hal_tdm_rx_a_start,      /* TDM A, DMA rx buffer start  */
  (tuint *) &_lcf_hal_tdm_rx_a_end,        /* TDM A, DMA rx buffer end    */

#if ( (DIAG_TDM_PORTS_USED == 1) || (DIAG_ITDM_PORTS_USED == 1) )
    }
#else
    },
#endif

#endif

#if ( (DIAG_TDM_PORTS_USED >= 2) || (DIAG_ITDM_PORTS_USED >= 2) )
    {
  (tuint *) &_lcf_hal_tdm_tx_b_start,      /* TDM B, DMA tx buffer start  */
  (tuint *) &_lcf_hal_tdm_tx_b_end,        /* TDM B, DMA tx buffer end    */
  (tuint *) &_lcf_hal_tdm_rx_b_start,      /* TDM B, DMA rx buffer start  */
  (tuint *) &_lcf_hal_tdm_rx_b_end,        /* TDM B, DMA rx buffer end    */
#if ( (DIAG_TDM_PORTS_USED == 2) || (DIAG_ITDM_PORTS_USED == 2) )
    }
#else
    },
#endif

#endif

#if ( (DIAG_TDM_PORTS_USED >= 3) || (DIAG_ITDM_PORTS_USED >= 3) )
    {
  (tuint *) &_lcf_hal_tdm_tx_c_start,      /* TDM C, DMA tx buffer start  */
  (tuint *) &_lcf_hal_tdm_tx_c_end,        /* TDM C, DMA tx buffer end    */
  (tuint *) &_lcf_hal_tdm_rx_c_start,      /* TDM C, DMA rx buffer start  */
  (tuint *) &_lcf_hal_tdm_rx_c_end,        /* TDM C, DMA rx buffer end    */
    }
#endif

   };

#endif

/****************************************************************************
 * Array of tdm configurations
 ****************************************************************************/
#if (DIAG_TDM_PORTS_USED >= 1)

  tint dcfgTdmSubFrameRate[] = { DIAG_TDM_SUB_FRAME_RATES };
  tint dcfgTdmWordSizes[]    = { DIAG_TDM_WORD_SIZES      };
  tint dcfgTdmBufInc[]       = { DIAG_TDM_BUF_INC         };
  tint dcfgTdmMaxPhase[]     = { DIAG_TDM_MAX_PHASES      };
  tint dcfgTdmMaxMegaPhase[] = { DIAG_TDM_MAX_MEGA_PHASES };

#endif

/*****************************************************************************
 * Declaration of the build time tdm configuration structure.
 *****************************************************************************/
#if (DIAG_TDM_PORTS_USED >= 1)
  dcfgTdmConfig_t dcfgTdmConfig =   {
                                     dcfgTdmDma,
                                     dcfgTdmSubFrameRate,
                                     dcfgTdmWordSizes,
                                     dcfgTdmBufInc,
                                     dcfgTdmMaxPhase,
                                     dcfgTdmMaxMegaPhase };
#endif


/****************************************************************************
 * Declaration of the build time itdm configuration structure.
 ****************************************************************************/
#if (DIAG_ITDM_PORTS_USED >= 1)
  dcfgiTdmConfig_t dcfgiTdmConfig = { dcfgTdmDma };
#endif


/****************************************************************************
 * Array of unit buffer sizes (CPPI block sizes) for mac
 ****************************************************************************/
#if (DIAG_GMAC_PORTS_USED == 0)
  tint dcfgGmacBufSizeWordsTx[]   = { 0 };
  tint dcfgGmacBufSizeWordsRx[]   = { 0 };
  tint dcfgGmacRxAssemSizeWords[] = { 0 };
#else
  tint dcfgGmacBufSizeWordsTx[]   = { DIAG_GMAC_TX_BUFSIZE_WORDS };
  tint dcfgGmacBufSizeWordsRx[]   = { DIAG_GMAC_RX_BUFSIZE_WORDS };
  tint dcfgGmacRxAssemSizeWords[] = { DIAG_GMAC_RX_ASSEMSIZE_WORDS };
#endif



/*****************************************************************************
 * Array of utopia DMA buffers
 *****************************************************************************/
#if (DIAG_UTOPIA_PORTS_USED >= 1)          

dcfgDma_t dcfgUtopiaDma[] = {

  {
  (tuint *) &_lcf_hal_pkt_tx_a_start,     /* PKT A, DMA tx buffer start  */
  (tuint *) &_lcf_hal_pkt_tx_a_end,       /* PKT A, DMA tx buffer end    */
  (tuint *) &_lcf_hal_pkt_rx_a_start,     /* PKT A, DMA rx buffer start  */
  (tuint *) &_lcf_hal_pkt_rx_a_end,       /* PKT A, DMA rx buffer end    */
#if (DIAG_UTOPIA_PORTS_USED == 1)
  }
#else
  },
#endif

#if (DIAG_UTOPIA_PORTS_USED >= 2)
  {
  (tuint *) &_lcf_hal_pkt_tx_b_start,     /* PKT B, DMA tx buffer start  */
  (tuint *) &_lcf_hal_pkt_tx_b_end,       /* PKT B, DMA tx buffer end    */
  (tuint *) &_lcf_hal_pkt_rx_b_start,     /* PKT B, DMA rx buffer start  */
  (tuint *) &_lcf_hal_pkt_rx_b_end,       /* PKT B, DMA rx buffer end    */
  }
#endif

  };

#endif

/* Catch all Buffers. On Janus these are not used */
#ifndef DIAG_UTOPIA_DMASIZE_CA_0
  tuint *dcfgUtopiaCaStart[] = { NULL };
  tuint *dcfgUtopiaCaEnd[]   = { NULL };

#else

  #if (DIAG_UTOPIA_PORTS_USED == 0)
  tuint *dcfgUtopiaCaStart[] = NULL;
  tuint *dcfgUtopiaCaEnd[]   = NULL;

  #elif (DIAG_UTOPIA_PORTS_USED == 1)

  tuint *dcfgUtopiaCaStart[] = { (tuint *)&_lcf_hal_pkt_ca_a_start };
  tuint *dcfgUtopiaCaEnd[]   = { (tuint *)&_lcf_hal_pkt_ca_a_end   };

  #elif (DIAG_UTOPIA_PORTS_USED == 2)

  tuint *dcfgUtopiaCaStart[] = { (tuint *)&_lcf_hal_pkt_ca_a_start, &_lcf_hal_pkt_ca_b_start };
  tuint *dcfgUtopiaCaEnd[]   = { (tuint *)&_lcf_hal_pkt_ca_a_end  , &_lcf_hal_pkt_ca_b_end   };

  #else
   #error diagcfg.c must be updated for more utopia ports
  #endif
#endif


/****************************************************************************
 * Array of utopia configurations
 ****************************************************************************/
#if (DIAG_UTOPIA_PORTS_USED >= 1)

  halUtopiaMode_t dcfgUtRouteMode[] = { DIAG_UTOPIA_ROUTE_MODE  };

#endif

/****************************************************************************
 * The Utopia configuration structure
 ****************************************************************************/
#if (DIAG_UTOPIA_PORTS_USED >= 1)

  dcfgUtConfig_t dcfgUtConfig = {
                                     dcfgUtopiaDma,
                                     dcfgUtopiaCaStart,
                                     dcfgUtopiaCaEnd,
                                     dcfgUtRouteMode  };
#endif


/****************************************************************************
 * Array of GMAC DMA buffers
 ****************************************************************************/
#if (DIAG_GMAC_PORTS_USED == 0)

dcfgDma_t  dcfgGmacDma[] = { NULL };

#else

dcfgDma_t  dcfgGmacDma[] = {

#if (DIAG_GMAC_PORTS_USED >= 1)
  {
  (tuint *) &_lcf_hal_gmac_tx_a_start,  /* gmacTxBufStart            */
  (tuint *) &_lcf_hal_gmac_tx_a_end,    /* gmacTxBufEnd              */
  (tuint *) &_lcf_hal_gmac_rx_a_start,  /* gmacRxBufStart            */
  (tuint *) &_lcf_hal_gmac_rx_a_end,    /* gmacRxBufEnd              */
#if (DIAG_GMAC_PORTS_USED == 1)
  }
#else
  },
#endif

#endif

#if (DIAG_GMAC_PORTS_USED >= 2)
  {
  (tuint *) &_lcf_hal_gmac_tx_b_start,  /* gmacTxBufStart            */
  (tuint *) &_lcf_hal_gmac_tx_b_end,    /* gmacTxBufEnd              */
  (tuint *) &_lcf_hal_gmac_rx_b_start,  /* gmacRxBufStart            */
  (tuint *) &_lcf_hal_gmac_rx_b_end,    /* gmacRxBufEnd              */
  }
#endif

  };

#endif

/***********************************************************************************
 * Declaration: Array of gmac rx assembly buffers
 ***********************************************************************************/
#if (DIAG_GMAC_PORTS_USED == 0)

dcfgMemBlk_t dcfgGmacAssemMem[] = { NULL };

#else

dcfgMemBlk_t dcfgGmacAssemMem[] = {

#if (DIAG_GMAC_PORTS_USED >= 1)
  {
  (tuint *) &(_lcf_hal_gmac_rx_assem_a_start),  
  (tuint *) &(_lcf_hal_gmac_rx_assem_a_end)
#if (DIAG_GMAC_PORTS_USED == 1)
  }
#else
  },
#endif

#endif

#if (DIAG_GMAC_PORTS_USED >= 2)
  {
  (tuint *) &(_lcf_hal_gmac_rx_assem_b_start),  
  (tuint *) &(_lcf_hal_gmac_rx_assem_b_end)
  }
#endif

  };

#endif


/************************************************************************************
 * Declaration: DSIU port structures - Arrays of port information 
 ************************************************************************************/
#if (DIAG_TDM_PORTS_USED > 0)
  dsiuPort_t dcfgTdmPorts[DIAG_TDM_PORTS_USED];
#endif

#if (DIAG_UTOPIA_PORTS_USED > 0)
  dsiuPort_t dcfgUtopiaPorts[DIAG_UTOPIA_PORTS_USED];
#endif
   
#if (DIAG_GMAC_PORTS_USED > 0)
  dsiuPort_t dcfgMacPorts[DIAG_GMAC_PORTS_USED];
#endif


/*****************************************************************************
 * Memory configuration
 *****************************************************************************/
#ifdef HAL_C64X
  dcfgMemCfg_t dcfgMemCfg = { HAL_L1P_CACHE_MODE,
                              HAL_L1D_CACHE_MODE,
                              HAL_L2_CACHE_MODE  };
#endif
/*****************************************************************************
 * Declaration of the Configuration structure. Must be loaded in this
 * manner since linker constants are not available at compile time
 *****************************************************************************/
diagCfg_t diagCfg = {

#if (!defined(DIAG_SYS_CHKSUM_NBLOCKS) || (DIAG_SYS_CHKSUM_NBLOCKS == 0))
  0,                            /* sysChkNblocks         */
  NULL,                         /* sysChkBlockSizesWords */
  NULL,                         /* sysChkBlockFlags      */
  NULL,                         /* sysChkBlocks          */
#else
  DIAG_SYS_CHKSUM_NBLOCKS,      /* sysChkNblocks         */
  dcfgSysChkBlockSizeWords,     /* sysChkBlockSizesWords */
  dcfgSysChkBlockFlags,         /* sysChkBlockFlags      */
  dcfgSysChkBlocks,             /* sysChkBlocks          */
#endif


#if (defined (c55mc))

  (tulong) &_lcf_external_prog_block1_start, /*External DDR block 1 start */
  (tulong) &_lcf_external_prog_block1_end,   /*External DDR block 1 end   */

  (tulong) &_lcf_external_prog_block2_start, /*External DDR block 2 start */
  (tulong) &_lcf_external_prog_block2_end,   /*External DDR block 2 end   */

  (tulong) &_lcf_external_prog_block3_start, /*External DDR block 3 start */
  (tulong) &_lcf_external_prog_block3_end,   /*External DDR block 3 end   */

  (tulong) &_lcf_external_prog_block4_start, /*External DDR block 4 start */
  (tulong) &_lcf_external_prog_block4_end,   /*External DDR block 4 end   */

  (tulong) &_lcf_external_prog_block5_start, /*External DDR block 5 start */
  (tulong) &_lcf_external_prog_block5_end,   /*External DDR block 5 end   */

  (tulong) &_lcf_burst_length,               /*External DDR burst length */

  (tulong) &_lcf_cycle_test_block,           /*External DDR Cycle test block size*/
#endif
  (word *) &_lcf_boot_heap_buf_start,      /* Boot heap buffer start addr  */
  (word *) &_lcf_boot_heap_buf_end,        /* Boot heap buffer end addr    */

  (word *) &_lcf_diag_heap_buf_start,      /* Diag heap buffer start addr  */
  (word *) &_lcf_diag_heap_buf_end,        /* Diag heap buffer end addr    */

  (word *) &_lcf_hal_heap_start,          /* The hal heap buffer start addr */
  (word *) &_lcf_hal_heap_end,            /* The hal heap buffer end addr   */


#ifdef DIAG_MAX_CHANNELS
  DIAG_MAX_CHANNELS,
#else
  0,
#endif

#ifdef DIAG_MAX_ITDM_STREAMS
  DIAG_MAX_ITDM_STREAMS,
#else
  0,
#endif

#ifdef DIAG_MAX_ITDM_1MILLISEC_STREAMS
  DIAG_MAX_ITDM_1MILLISEC_STREAMS,
#else
  0,
#endif

#ifdef DIAG_MAX_ITDM_125MICROSEC_STREAMS
  DIAG_MAX_ITDM_125MICROSEC_STREAMS,
#else
  0,
#endif

#ifdef DIAG_MAX_CHAN_IDS_PER_1MSEC_STREAM
  DIAG_MAX_CHAN_IDS_PER_1MSEC_STREAM,
#else
  0,
#endif

#ifdef DIAG_MAX_CHAN_IDS_PER_125MICROSEC_STREAM
  DIAG_MAX_CHAN_IDS_PER_125MICROSEC_STREAM,
#else
  0,
#endif


  DIAG_TDM_PORTS_USED,

#ifdef DIAG_ITDM_PORTS_USED
  DIAG_ITDM_PORTS_USED,
#else
  0,
#endif

#if (DIAG_TDM_PORTS_USED >= 1)
  &dcfgTdmConfig,
#else
  NULL,
#endif

#if (DIAG_ITDM_PORTS_USED >= 1)
  &dcfgiTdmConfig,
#else
  NULL,
#endif


#ifdef DIAG_MCBSP_OP_BLOCK
  DIAG_MCBSP_OP_BLOCK,
#else
  0,
#endif

#ifdef DIAG_MCBSP_TYPE
  DIAG_MCBSP_TYPE,
#else
  0,
#endif  

#ifdef DIAG_TDM_BDX_DELAY
  DIAG_TDM_BDX_DELAY,
#else
  0,
#endif


#if (DIAG_TDM_PORTS_USED >= 1)
#ifdef DIAG_TDM_BUF_SIZE
  DIAG_TDM_BUF_SIZE,
#else
#error Diagcfg.c - TDM ports declared but DIAG_TDM_BUF_SIZE not defined
#endif
#else
  0,
#endif

  DIAG_UTOPIA_PORTS_USED,             /* Number of Utopia ports    */

#if (DIAG_UTOPIA_PORTS_USED >= 1)
  &dcfgUtConfig,                      /* utConfig */
#else
  NULL,
#endif


#ifndef DIAG_GMAC_PORTS_USED          /* If GMAC not defined set */
#define DIAG_GMAC_PORTS_USED 0        /* 0 GMAC port usage       */
#endif
                                      
  DIAG_GMAC_PORTS_USED,               /* nGmacPorts                */

  dcfgGmacBufSizeWordsTx,             /* gmacBufSizeWordsTx        */
  dcfgGmacBufSizeWordsRx,             /* gmacBufSizeWordsRx        */
  dcfgGmacRxAssemSizeWords,           /* gmacRxAssemSizeWords      */

  dcfgGmacDma,                        /* gmacDmaMem                */
  dcfgGmacAssemMem,                   /* gmacRxAssemMem            */

#ifdef DIAG_INCLUDE_AIC
  DIAG_AIC_PORTS_USED,                /* nAicPorts */
#else
  0,
#endif


#ifdef HAL_C64X                      /* memCfg */
  &dcfgMemCfg,
#else
  NULL,
#endif

#ifdef DIAG_INCLUDE_CPU_FSYNC_RATIO   /*****************************/
  DIAG_REPORT_DIVISOR,                /*   CPU clocks per          */
  DIAG_TIMER_PERIOD,                  /*      frame sync           */
  DIAG_TIMER_TIMEOUT,                 /*       reporting           */
#else                                 /*                           */
  0,                                  /*                           */
  0,                                  /*                           */
  0,                                  /*                           */
#endif                                /*****************************/


#ifdef DIAG_APP_FRAME_RATE_0          /*******************************/
  DIAG_APP_FRAME_RATE_0,              /*                             */
#else                                 /* Application frame rate for  */
  0,                                  /* TDM port A                  */
#endif                                /*******************************/

#ifdef DIAG_APP_FRAME_RATE_1          /*******************************/
  DIAG_APP_FRAME_RATE_1,              /*                             */
#else                                 /* Application frame rate for  */
  0,                                  /* TDM port B                  */
#endif                                /*******************************/


#if (DIAG_TDM_PORTS_USED > 0)        /********************************/
#ifdef DIAG_APP_VPU_BUF_SIZE         /*                              */
  DIAG_APP_VPU_BUF_SIZE,             /*  VPU voice data buffer       */
#else                                /********************************/
#error TDM ports declared in diagbld.h but DIAG_APP_VPU_BUF_SIZE not
#endif
#else
  0,
#endif



#ifdef DIAG_INCLUDE_NEU

#ifdef DIAG_AAL5_RX_ASSEMBLY_BUF_SIZE
  DIAG_AAL5_RX_ASSEMBLY_BUF_SIZE,
#else
#error NEU included but DIAG_AAL5_RX_ASSEMBLY_BUF_SIZE not defined in diagbld.h
#endif

#ifdef DIAG_AAL5_TX_SEGMENT_BUF_SIZE
  DIAG_AAL5_TX_SEGMENT_BUF_SIZE,
#else
#error NEU included but DIAG_AAL5_TX_SEGMENT_BUF_SIZE not defined in diagbld.h
#endif

#ifdef DIAG_NEU_AAL5_RX_ASSEMBLY_BUF_SIZE
  DIAG_NEU_AAL5_RX_ASSEMBLY_BUF_SIZE, /* voiceAssemblySize */
#else
  0,
#endif

#ifdef DIAG_NEU_AAL5_TX_SEGMENT_BUF_SIZE
  DIAG_NEU_AAL5_TX_SEGMENT_BUF_SIZE,   /* voiceSegSize */
#else
  0,
#endif

#ifdef DIAG_NEU_UDP_CORE_ID_MASK
  DIAG_NEU_UDP_CORE_ID_MASK,
#else
  0,
#endif

#ifdef DIAG_NEU_UDP_VOICE_BASE
  DIAG_NEU_UDP_VOICE_BASE,
#else
  0,
#endif

#else  /* else case for DIAG_INCLUDE_NEU */

  0,
  0,
  0,
  0,
  0,


#endif /* DIAG_INCLUDE_NEU */


#if ( (DIAG_UTOPIA_PORTS_USED > 0) || (DIAG_GMAC_PORTS_USED > 0) )

#ifdef DIAG_RX_NET_FIFO_SIZE
  DIAG_RX_NET_FIFO_SIZE,
#else
#error Utopia or Gmac ports included but DIAG_RX_NET_FIFO_SIZE not defined in diagbld.h
#endif

#else
  0,
#endif /* DIAG_UTOPIA_PORTS_USED > 0  || DIAG_GMAC_PORTS_USED > 0*/


#ifdef DIAG_MSG_TASK_SIZE
  DIAG_MSG_TASK_SIZE,
#else
#error DIAG_MSG_TASK_SIZE not defined in diagbld.h
#endif

#ifdef DIAG_MSG_RX_BUFFER_SIZE
  DIAG_MSG_RX_BUFFER_SIZE,
#else
#error DIAG_MSG_RX_BUFFER_SIZE not defined in diagbld.h
#endif

#ifdef DIAG_MSG_TX_BUFFER_SIZE
  DIAG_MSG_TX_BUFFER_SIZE,
#else
#error DIAG_MSG_TX_BUFFER_SIZE not defined in diagbld.h
#endif

#ifdef DIAG_MSG_RX_FIFO_SIZE
  DIAG_MSG_RX_FIFO_SIZE,
#else
#error DIAG_MSG_RX_FIFO_SIZE not defined in diagbld.h
#endif


#ifdef DIAG_PREDEF_MSG_N_MSGS
  DIAG_PREDEF_MSG_N_MSGS,    /* nBlockMessages */
#else
  0,
#endif

#ifdef DIAG_CACHE_FLUSH_BLK_CTL
  1,                        /* blkMsgCacheFlush */
#else
  0,
#endif


/*****************************************************************************
 * Boot Config function call and ack, and simulated app startup
 *****************************************************************************/
#if defined(DIAG_BOOT_CONFIG_UTOPIA)
  dsiuBootUtopia,
  dsiuBootAck,
  dsiuUtSimuAppConfig,
#elif defined(DIAG_BOOT_CONFIG_GMAC)
  dsiuBootGmac,
  dsiuBootAck,
  dsiuGmacSimuAppConfig,
#elif defined(DIAG_BOOT_CONFIG_UT_GMAC)
  dsiuBootUtGmac,
  dsiuBootAck,
  dsiuGmacSimuAppConfig,
#else
  NULL,
  NULL,
  NULL,
#endif

/*****************************************************************************
 * System Boot (program bootloader starts) function pointer
 *****************************************************************************/
#ifdef DIAG_SYSTEM_BOOT_FCN
  DIAG_SYSTEM_BOOT_FCN,
#else
  NULL,
#endif

/*****************************************************************************
 * System default entry point
 *****************************************************************************/
#ifdef DIAG_SYSTEM_ENTRY_POINT
  (tulong)DIAG_SYSTEM_ENTRY_POINT,
#else
  (tulong)0,
#endif

/*****************************************************************************
 * System Reset (core reset) function pointer
 *****************************************************************************/
#ifdef DIAG_SYSTEM_RESET_FCN
  DIAG_SYSTEM_RESET_FCN,
#else
  NULL,
#endif

/*****************************************************************************
 * TDM sub rate function - for systems with TDM capability 
 *****************************************************************************/
#if (DIAG_TDM_PORTS_USED > 0)
  dsiuSubRateFcn,
#else
  NULL,
#endif

/*****************************************************************************
 * System Memory test and system checksum test functions
 *****************************************************************************/
#ifdef DIAG_INCLUDE_MEMTEST
  DIAG_MEMTEST_NBLOCKS,  /* numMemTestBlocks */
  DIAG_MEMTEST_FCN,      /* dsiuTestMemory   */
#else
  0,                /* numMemTestBlocks */
  NULL,             /* dsiuTestMemory */
#endif

#ifdef DIAG_INCLUDE_SYS_CHKSUM
  dsiuMemChecksum,  /* dsiuChecksum   */
#else 
  NULL,             /* dsiuCheckSum   */
#endif

#ifdef DIAG_INCLUDE_RAMSET_TEST
  dsiuTestRamset,   /* dsiuTestRamset     */
#else
  NULL,             /* dsiuTestRamset     */
#endif


/*****************************************************************************
 * Processor specific inits - function defined in diagbld.h
 *****************************************************************************/
#ifdef DIAG_PROC_STARTUP    
  DIAG_PROC_STARTUP,    /* dsiuProcStartup */
#else
  NULL,                 /* dsiuProcStartup */
#endif


/*****************************************************************************
 * Pointers to system port configuration memory.
 *****************************************************************************/
#if (DIAG_TDM_PORTS_USED > 0)
  dcfgTdmPorts,
#else
  NULL,
#endif

#if (DIAG_UTOPIA_PORTS_USED > 0)
  dcfgUtopiaPorts,
#else
  NULL,
#endif

#if (DIAG_GMAC_PORTS_USED > 0)
  dcfgMacPorts,
#else
  NULL,
#endif

#ifdef GG_IPC_MEMTEST_SYNC_MSG
  GG_IPC_MEMTEST_SYNC_MSG,
#else
  0,
#endif

#ifdef DIAG_SEC_CORE_START_DELAY
  DIAG_SEC_CORE_START_DELAY,
#else
 #ifdef DIAG_INCLUDE_AUTOCTL
  #error DIAG_INCLUDE_AUTOCTL defined, DIAG_SEC_CORE_START_DELAY should be defined as well
 #endif
  0,
#endif


/*****************************************************************************
 * Module Call tables
 *****************************************************************************/
#if (DIAG_TDM_PORTS_USED > 0)        /********************************/
  &dpiuCallTable,                    /*                              */
#else                                /*      dpiu call table         */
  NULL,                              /*                              */
#endif                               /********************************/

#ifdef ITDM
#if (DIAG_ITDM_PORTS_USED > 0)       /********************************/
  &ipiuCallTable,                    /*                              */
#else                                /*      ipiu call table         */
  NULL,                              /*                              */
#endif                               /********************************/

#if (DIAG_ITDM_PORTS_USED > 0)       /********************************/
  &ippuCallTable,                    /*                              */
#else                                /*      ippu call table         */
  NULL,                              /*                              */
#endif                               /********************************/
#endif

#if (DIAG_TDM_PORTS_USED > 0)        /********************************/
  &dvpuCallTable, 
#else                                /*      dvpu call table         */
  NULL,                              /*                              */
#endif                               /********************************/

#ifdef DIAG_INCLUDE_TGU              /********************************/
  &dtguCallTable,                    /*                              */
#else                                /*      dtgu call table         */
  NULL,                              /*                              */
#endif                               /********************************/

#ifdef DIAG_INCLUDE_DPVP             /********************************/
  &dpvpCallTable,                    /*                              */
#else                                /*      dpvp call table         */
  NULL,                              /*                              */
#endif                               /********************************/

#ifdef DIAG_INCLUDE_NEU              /********************************/
 &neuCallTable,                      /*                              */
#else                                /*      neu call table          */
  NULL,                              /*                              */
#endif                               /********************************/

#if (DIAG_UTOPIA_PORTS_USED > 0)     /********************************/
  &siuUtopiaNduCallTable,            /*                              */
#else                                /*   System utopia call table   */
  NULL,                              /*                              */
#endif                               /********************************/

#ifdef DIAG_INCLUDE_HPI              /********************************/
  &siuHpiNduCallTable,               /*                              */
#else                                /*   System HPI call table      */
  NULL,                              /*                              */
#endif                               /********************************/

#if (DIAG_GMAC_PORTS_USED > 0)       /********************************/
  &siuGmacNduCallTable,              /*                              */
#else                                /*   System GMAC call table     */
  NULL,                              /*                              */
#endif                               /********************************/

#ifdef DIAG_INCLUDE_BERT             /********************************/
  &bertCallTable,                    /*                              */
#else                                /*        Bert call table       */
  NULL,                              /*                              */
#endif                               /********************************/


 &dsiuCallTable,                     /*    DSIU call table           */

#ifdef DIAG_INCLUDE_AUTOCTL          /********************************/
  &dsiuAutoRun                       /*                              */
#else                                /*   Automated run control      */
  NULL                               /*                              */
#endif                               /********************************/



};

/***********************************************************************
 * Definition: The NEU port map array
 ***********************************************************************/
#ifndef DIAG_NEU_UDP_TABLE_SIZE
#define DIAG_NEU_UDP_TABLE_SIZE  0
#endif

#if (DIAG_NEU_UDP_TABLE_SIZE > 0)
tuint diagNeuDestPort[DIAG_NEU_UDP_TABLE_SIZE];
void *diagNeudInst[DIAG_NEU_UDP_TABLE_SIZE];

neuPortMap_t neuPortMap = {
  DIAG_NEU_UDP_TABLE_SIZE,
  0,
  diagNeuDestPort,
  diagNeudInst
};

#else

neuPortMap_t neuPortMap = {
  0,
  0,
  NULL,
  NULL,
};



/*****************************************************************************
 * Verify the number of channels divides into the number of tdm ports 
 *****************************************************************************/
#if (DIAG_TDM_PORTS_USED > 0)
#if (DIAG_MAX_CHANNELS % DIAG_TDM_PORTS_USED)
#error The number of channels must be divisible by the number of tdm ports
#endif
#endif

#endif


/*****************************************************************************
 * Optional memory test
 *****************************************************************************/
#ifdef DIAG_INCLUDE_MEMTEST

dcfgMemBlock_t dcfgMemBlock[] = {

#if (DIAG_MEMTEST_NBLOCKS == 0) 
   { NULL,
     0,
     0
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 0) 
   { mtblock0,
     DIAG_MEMTEST_BLOCK_0_SIZE,
     DIAG_MEMTEST_BLOCK_0_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 1)
  ,{ mtblock1,
     DIAG_MEMTEST_BLOCK_1_SIZE,
     DIAG_MEMTEST_BLOCK_1_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 2)
  ,{ mtblock2,
     DIAG_MEMTEST_BLOCK_2_SIZE,
     DIAG_MEMTEST_BLOCK_2_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 3)
  ,{ mtblock3,
     DIAG_MEMTEST_BLOCK_3_SIZE,
     DIAG_MEMTEST_BLOCK_3_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 4)
  ,{ mtblock4,
     DIAG_MEMTEST_BLOCK_4_SIZE,
     DIAG_MEMTEST_BLOCK_4_ATTR
   }
#endif


#if (DIAG_MEMTEST_NBLOCKS > 5)
  ,{ mtblock5,
     DIAG_MEMTEST_BLOCK_5_SIZE,
     DIAG_MEMTEST_BLOCK_5_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 6)
  ,{ mtblock6,
     DIAG_MEMTEST_BLOCK_6_SIZE,
     DIAG_MEMTEST_BLOCK_6_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 7)
  ,{ mtblock7,
     DIAG_MEMTEST_BLOCK_7_SIZE,
     DIAG_MEMTEST_BLOCK_7_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 8)
  ,{ mtblock8,
     DIAG_MEMTEST_BLOCK_8_SIZE,
     DIAG_MEMTEST_BLOCK_8_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 9)
  ,{ mtblock9,
     DIAG_MEMTEST_BLOCK_9_SIZE,
     DIAG_MEMTEST_BLOCK_9_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 10)
  ,{ mtblock10,
     DIAG_MEMTEST_BLOCK_10_SIZE,
     DIAG_MEMTEST_BLOCK_10_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 11)
  ,{ mtblock11,
     DIAG_MEMTEST_BLOCK_11_SIZE,
     DIAG_MEMTEST_BLOCK_11_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 12)
  ,{ mtblock12,
     DIAG_MEMTEST_BLOCK_12_SIZE,
     DIAG_MEMTEST_BLOCK_12_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 13)
  ,{ mtblock13,
     DIAG_MEMTEST_BLOCK_13_SIZE,
     DIAG_MEMTEST_BLOCK_13_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 14)
  ,{ mtblock14,
     DIAG_MEMTEST_BLOCK_14_SIZE,
     DIAG_MEMTEST_BLOCK_14_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 15)
  ,{ mtblock15,
     DIAG_MEMTEST_BLOCK_15_SIZE,
     DIAG_MEMTEST_BLOCK_15_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 16)
  ,{ mtblock16,
     DIAG_MEMTEST_BLOCK_16_SIZE,
     DIAG_MEMTEST_BLOCK_16_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 17)
  ,{ mtblock17,
     DIAG_MEMTEST_BLOCK_17_SIZE,
     DIAG_MEMTEST_BLOCK_17_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 18)
  ,{ mtblock18,
     DIAG_MEMTEST_BLOCK_18_SIZE,
     DIAG_MEMTEST_BLOCK_18_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 19)
  ,{ mtblock19,
     DIAG_MEMTEST_BLOCK_19_SIZE,
     DIAG_MEMTEST_BLOCK_19_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 20)
  ,{ mtblock20,
     DIAG_MEMTEST_BLOCK_20_SIZE,
     DIAG_MEMTEST_BLOCK_20_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 21)
  ,{ mtblock21,
     DIAG_MEMTEST_BLOCK_21_SIZE,
     DIAG_MEMTEST_BLOCK_21_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 22)
  ,{ mtblock22,
     DIAG_MEMTEST_BLOCK_22_SIZE,
     DIAG_MEMTEST_BLOCK_22_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 23)
  ,{ mtblock23,
     DIAG_MEMTEST_BLOCK_23_SIZE,
     DIAG_MEMTEST_BLOCK_23_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 24)
  ,{ mtblock24,
     DIAG_MEMTEST_BLOCK_24_SIZE,
     DIAG_MEMTEST_BLOCK_24_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 25)
  ,{ mtblock25,
     DIAG_MEMTEST_BLOCK_25_SIZE,
     DIAG_MEMTEST_BLOCK_25_ATTR
   }
#endif


#if (DIAG_MEMTEST_NBLOCKS > 26)
  ,{ mtblock26,
     DIAG_MEMTEST_BLOCK_26_SIZE,
     DIAG_MEMTEST_BLOCK_26_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 27)
  ,{ mtblock27,
     DIAG_MEMTEST_BLOCK_27_SIZE,
     DIAG_MEMTEST_BLOCK_27_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 28)
  ,{ mtblock28,
     DIAG_MEMTEST_BLOCK_28_SIZE,
     DIAG_MEMTEST_BLOCK_28_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 29)
  ,{ mtblock29,
     DIAG_MEMTEST_BLOCK_29_SIZE,
     DIAG_MEMTEST_BLOCK_29_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 30)
  ,{ mtblock30,
     DIAG_MEMTEST_BLOCK_30_SIZE,
     DIAG_MEMTEST_BLOCK_30_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 31)
  ,{ mtblock31,
     DIAG_MEMTEST_BLOCK_31_SIZE,
     DIAG_MEMTEST_BLOCK_31_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 32)
  ,{ mtblock32,
     DIAG_MEMTEST_BLOCK_32_SIZE,
     DIAG_MEMTEST_BLOCK_32_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 33)
  ,{ mtblock33,
     DIAG_MEMTEST_BLOCK_33_SIZE,
     DIAG_MEMTEST_BLOCK_33_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 34)
  ,{ mtblock34,
     DIAG_MEMTEST_BLOCK_34_SIZE,
     DIAG_MEMTEST_BLOCK_34_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 35)
  ,{ mtblock35,
     DIAG_MEMTEST_BLOCK_35_SIZE,
     DIAG_MEMTEST_BLOCK_35_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 36)
  ,{ mtblock36,
     DIAG_MEMTEST_BLOCK_36_SIZE,
     DIAG_MEMTEST_BLOCK_36_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 37)
  ,{ mtblock37,
     DIAG_MEMTEST_BLOCK_37_SIZE,
     DIAG_MEMTEST_BLOCK_37_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 38)
  ,{ mtblock38,
     DIAG_MEMTEST_BLOCK_38_SIZE,
     DIAG_MEMTEST_BLOCK_38_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 39)
  ,{ mtblock39,
     DIAG_MEMTEST_BLOCK_39_SIZE,
     DIAG_MEMTEST_BLOCK_39_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 40)
  ,{ mtblock40,
     DIAG_MEMTEST_BLOCK_40_SIZE,
     DIAG_MEMTEST_BLOCK_40_ATTR
   }
#endif

#if (DIAG_MEMTEST_NBLOCKS > 41)
#error diagcfg.c must be updated with more memory test blocks
#endif

};
#endif /* DIAG_INCLUDE_MEMTEST */

#ifdef PA_SS
#include "dsiunwcfg.h"
extern word QMemDescRam[];
extern word rxlinkramQ1[];
extern word rxlinkramQ2[];
extern word rxlinkramQ3[];
extern word rxlinkramQ4[];
extern word txlinkramQ1[];
extern word txlinkramQ2[];
extern word txlinkramQ3[];
extern word txlinkramQ4[];
const dsiuNwalLinkedBufsDesc_t dsiuNwalLinkedBufsDesc[] = {
#if (DIAG_QM_RX_LINKED_BUF_1_SIZE) 
  { 
    rxlinkramQ1,
    DIAG_QM_RX_LINKED_BUF_1_SIZE, 
    DIAG_QM_RX_LINKED_BUF_1_NBUFS,
    DSIU_NWAL_RX_CTL_BUFS
  },
#endif   
#if (DIAG_QM_RX_LINKED_BUF_2_SIZE) 
  { 
    rxlinkramQ2,
    DIAG_QM_RX_LINKED_BUF_2_SIZE, 
    DIAG_QM_RX_LINKED_BUF_2_NBUFS,
    DSIU_NWAL_RX_PKT_BUFS
  },
#endif   
#if (DIAG_QM_RX_LINKED_BUF_3_SIZE) 
  { 
    rxlinkramQ3,
    DIAG_QM_RX_LINKED_BUF_3_SIZE, 
    DIAG_QM_RX_LINKED_BUF_3_NBUFS,
    DSIU_NWAL_RX_PKT_BUFS
  },
#endif   
#if (DIAG_QM_RX_LINKED_BUF_4_SIZE) 
  { 
    rxlinkramQ4,
    DIAG_QM_RX_LINKED_BUF_4_SIZE, 
    DIAG_QM_RX_LINKED_BUF_4_NBUFS,
    DSIU_NWAL_RX_PKT_BUFS
  },
#endif   
#if (DIAG_QM_TX_LINKED_BUF_1_SIZE) 
  { 
    txlinkramQ1,
    DIAG_QM_TX_LINKED_BUF_1_SIZE, 
    DIAG_QM_TX_LINKED_BUF_1_NBUFS,
    DSIU_NWAL_TX_CTL_BUFS
  },
#endif   
#if (DIAG_QM_TX_LINKED_BUF_2_SIZE) 
  { 
    txlinkramQ2,
    DIAG_QM_TX_LINKED_BUF_2_SIZE, 
    DIAG_QM_TX_LINKED_BUF_2_NBUFS,
    DSIU_NWAL_TX_PKT_BUFS
  },
#endif   
#if (DIAG_QM_TX_LINKED_BUF_3_SIZE) 
  { 
    txlinkramQ3,
    DIAG_QM_TX_LINKED_BUF_3_SIZE, 
    DIAG_QM_TX_LINKED_BUF_3_NBUFS,
    DSIU_NWAL_TX_PKT_BUFS
  },
#endif   
#if (DIAG_QM_TX_LINKED_BUF_4_SIZE) 
  { 
    txlinkramQ4,
    DIAG_QM_TX_LINKED_BUF_4_SIZE, 
    DIAG_QM_TX_LINKED_BUF_4_NBUFS,
    DSIU_NWAL_TX_PKT_BUFS
  },
#endif   
  {NULL, 0, 0, DSIU_NWAL_RX_PKT_BUFS}
};

const dsiuNwalCfg_t dsiuNwalCfg = {
  DIAG_QM_NUM_DESCS,
  DIAG_QM_DESC_SIZE,
  QMemDescRam,
  dsiuNwalLinkedBufsDesc
};
#endif
