#ifndef _DIAGAPP_H
#define _DIAGAPP_H
/*****************************************************************************
 * FILE PURPOSE: Define structure to pass information from bootloader to app
 *****************************************************************************
 * FILE NAME: diagapp.h
 *
 * DESCRIPTION: Defines a data structure and a hard coded location that is
 *              used to pass configuration information from the boot config
 *              cell to the application.
 *
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* Definition: Sizes of headers */
#define DIAGAPP_RT_HEADER_SIZE_WORDS   5  /* route tag plus atm header size */
#define DIAGAPP_VC_HEADER_SIZE_WORDS   3  /* VC demux atm header size       */

#define DIAGAPP_MAX_HDR_LEN_WORDS     40

/* Definition - Mac addresses - associates mac addresses with cores */
#ifdef IP_BOOT_SUPP
#define DIAGAPP_MAX_MAC_ADDR    6
#else
#define DIAGAPP_MAX_MAC_ADDR    CPU_NCORES
#endif

typedef struct diagAppMacAddr_s {
#ifdef IP_BOOT_SUPP
  tint  core;        /* Core number of the associated mac address */
#endif /* IP_BOOT_SUPP */
  tuint macAddr[3];    /* MSW, SMSW, LSW */
} diagAppMacAddr_t;

#define DIAG_APP_MAC_ADDR_READ_PORT(x) UTL_GET_BITFIELD(x,14,2)
#define DIAG_APP_MAC_ADDR_READ_CORE(x) UTL_GET_BITFIELD(x,0,14)
#define DIAG_APP_INVALID_MAC_ADDR 0x3fff

/* Definition - IP addresses - associates IP addresses with cores */
#ifdef IP_BOOT_SUPP
/* More MAC and IP address per core are needed */
#define DIAGAPP_MAX_IP_ADDR    6 
#else
#define DIAGAPP_MAX_IP_ADDR    CPU_NCORES 
#endif

#define DIAG_APP_IP_MSG_BLOCK_FLAG   (1<<8)
#ifdef IP_BOOT_SUPP
#define DIAGAPP_IPV4_ADDR_LEN_IN_TUINT    2
#define DIAGAPP_IPV6_ADDR_LEN_IN_TUINT    8

#define DIAG_APP_IP_CORE_MASK        0x001F
#define DIAG_APP_IP_CORE_SHIFT            0

#define DIAG_APP_IP_VER_MASK         0x0020
#define DIAG_APP_IP_VER_SHIFT             5
#define DIAG_APP_IP_VER_IPV4_FLAG         0
#define DIAG_APP_IP_VER_IPV6_FLAG         1

#define DIAG_APP_IP_TYPE_MASK        0x01C0
#define DIAG_APP_IP_TYPE_SHIFT            6
#define DIAG_APP_IP_TYPE_MSG              0
#define DIAG_APP_IP_TYPE_BLOCK            1
#define DIAG_APP_IP_TYPE_NV               2
#define DIAG_APP_IP_TYPE_CORE             3

#define DIAG_APP_GET_IP_TYPE(x)  ((x & DIAG_APP_IP_TYPE_MASK) >>    \
                                  DIAG_APP_IP_TYPE_SHIFT)

#define DIAG_APP_GET_IP_VER_FLAG(x)  ((x & DIAG_APP_IP_VER_MASK) >>      \
                                  DIAG_APP_IP_VER_SHIFT)

#define DIAG_APP_GET_IP_CORE(x)  ((x & DIAG_APP_IP_CORE_MASK) >>    \
                                  DIAG_APP_IP_CORE_SHIFT)

#define DIAG_APP_SET_IP_TYPE(x, val)  x |= ((val << DIAG_APP_IP_TYPE_SHIFT)   \
                                              & DIAG_APP_IP_TYPE_MASK) 

#define DIAG_APP_SET_IP_VER_FLAG(x, val)   x |= ((val << DIAG_APP_IP_VER_SHIFT)   \
                                              & DIAG_APP_IP_VER_MASK) 

#define DIAG_APP_SET_IP_CORE(x, val)  x |= ((val << DIAG_APP_IP_CORE_SHIFT)   \
                                              & DIAG_APP_IP_CORE_MASK) 

#endif /* IP_BOOT_SUPP */

#ifdef IP_BOOT_SUPP
typedef struct diagAppIpAddr_s {
  /* Bits 6-8 ->type flag - 0 = message, 1 = block, 2 = Non Voice 3 = Processor 
     Bit 5: Protocol 0 = IPv4, 1 = IPv6
     Bits 4:0 core */
  tint  typeIPverCore;       
  tuint IpAddr[DIAGAPP_IPV6_ADDR_LEN_IN_TUINT];       /* Source IP address `*/
} diagAppIpAddr_t;
#else
typedef struct diagAppIpAddr_s {
  tuint srcIpHigh;      /* Source IP address MSW */
  tuint srcIpLow;       /* Dest IP address LSW   */
} diagAppIpAddr_t;

#endif /* IP_BOOT_SUPP */

/* Mii mode header field definitions */
typedef struct {
  tuint flags1;          /* see flags map */
  tuint flags2;
  
  tint  rxPayloadType;        /* Specifies network protocol type */
  
  tint  ipOffsetBytes;       /* >= 0 is the offset in hdrImage to the ip header.
                              * if < 0 there is no idp header */
                         
  tint  udpOffsetBytes;      /* >= 0 is the offset in hdrImage to the udp header.
                              * if < 0 there is no udp header */
                              
  tint  ethOffsetBytes;      /* >= 0 is the offset in hdrImage to the 802.3 header
                              * if < 0 there is not 802.3 header */
                              
  tuint totalHeaderLenBytes; /* Length of hdrImage */                       
 
  tuint priDiffTag;          /* Message priority differentiation tag  */
  
  tuint txMinCellBytes;      /* Minimum transmit cell length, bytes*/
  
  tuint rxOffsetBytes;       /* Offset to payload, receive side */
  
  tuint hdrImage[DIAGAPP_MAX_HDR_LEN_WORDS];   /* The image header */
  
} diagAppHeader_t;  

/* Mii flags mapping */
#define DIAGAPP_F1_UDP_GEN_CHKSUM_BIT  (1 << 0)
#define DIAGAPP_F1_UDP_CHK_CHKSUM_BIT  (1 << 1)
#define DIAGAPP_F1_UDP_CHK_REMOTE_BIT  (1 << 2)
#define DIAGAPP_F1_UDP_CHK_LOCAL_BIT   (1 << 3)

#define DIAGAPP_F1_IP_CHK_LOCAL_BIT    (1 << 7)
#define DIAGAPP_F1_IP_CHK_REMOTE_BIT   (1 << 8)
#define DIAGAPP_F1_IP_ANY_MULTI_BIT    (1 << 9)
#define DIAGAPP_F1_IP_ANY_BCAST_BIT    (1 << 10)
#define DIAGAPP_F1_IP_CHK_CHKSUM_BIT   (1 << 11)

#define DIAGAPP_F2_ETH_CHK_REMOTE_BIT  (1 << 0)
#define DIAGAPP_F2_ETH_ANY_UNICAST_BIT (1 << 1)
#define DIAGAPP_F2_ETH_ANY_MULTI_BIT   (1 << 2)
#define DIAGAPP_F2_ETH_ANY_BCAST_BIT   (1 << 3)
#define DIAGAPP_F2_ETH_CHK_PROTO_BIT   (1 << 4)

#define DIAGAPP_F2_AAL5_GEN_CRC_BIT    (1 << 13)
#define DIAGAPP_F2_AAL5_CHK_CRC_BIT    (1 << 14)



/* Utopia configuration fields */
typedef struct diagAppUtopiaCfg_s {

  tuint cellSize;   /* 0 = no utopia support, 54 and 58 are the only valid cell sizes */
  tint  slaveId;    /* The phy Id. A value of -1 reads from the pins */
 
  /* Network encap configuration - can be aal5 */
  diagAppHeader_t  msg;
  struct {
    tuint kpAlive_gfc_vp_vcih;
    tuint kpAlive_vcil_pt_clp;
    tuint kpAliveAddr_hi;
    tuint kpAliveAddr_lo;
  } kpalive;
  
} diagAppUtopiaCfg_t;

/* Mac configuration fields */
typedef struct diagAppMacCfg_s {

  tuint configFlags;    /* Bit field of mac configuration */
/* 
 * Ethernet specific options 
 * Bit 0: GE:
 *        0 - 10/100 Mb
 *        1 - 1 Gb
 * Bit 2: HD:
 *        0 - Full Duplex
 *        1 - Half Duplex
 * Bit 5: FC:
 *        0 - Disable flow control
 *        1 - Enable flow control
 * Bit 6: RGMII:
 *        0 - RGMII disabled
 *        1 - RGMII enabled
 * Bit 7: SMII
 *        0 - SMII disabled
 *        1 - SMII enabled
 * Bit 8: S3MII
 *        0 - S3MII disabled
 *        1 - S3Mii enabled
 *
 * Bit 9: RMII 10 Mb
 *		  0 - RMII 10 Mb disabled
 *		  1 - RMII 10 Mb enabled
 *
 * Bit 10: RMII 100 Mb
 *		  0 - RMII 100 Mb disabled
 *		  1 - RMII 100 Mb enabled

 * Other bits: reserved
 *
 */
#define DIAGAPP_MAC_CFG_GE  (1<<0)
#define DIAGAPP_MAC_CFG_HD  (1<<2)
#define DIAGAPP_MAC_CFG_FC  (1<<5)
#define DIAGAPP_MAC_CFG_RGMII (1<<6)
#define DIAGAPP_MAC_CFG_SMII  (1<<7)
#define DIAGAPP_MAC_CFG_S3MII (1<<8)
#define DIAGAPP_MAC_CFG_RMII_10 (1<<9)
#define DIAGAPP_MAC_CFG_RMII_100 (1<<10)

  /* Mac buffer assignment */
  tint macBufferAssign;

  /* Network encap configuration */
  diagAppHeader_t msg;
  
#ifdef IP_BOOT_SUPP
  diagAppHeader_t block;
  diagAppHeader_t nonVoice;
#endif /* IP_BOOT_SUPP */
  diagAppMacAddr_t    macAddr[DIAGAPP_MAX_MAC_ADDR];
  diagAppIpAddr_t     ipAddr[DIAGAPP_MAX_IP_ADDR];
  
  
} diagAppMacCfg_t;


/* The data structure used to communicate between bootloader and application */
typedef struct diagApp_s {

  tuint dspId;
#ifdef IP_BOOT_SUPP
  tuint nvmsgOnOff;       /* 1: Forward the messages,0: Drop the packets*/
#define DIAGAPP_NV_MSG_ONOFF_MASK  0xff
#define DIAGAPP_NV_MSG_OFF0x0
#define DIAGAPP_NV_MSG_ON 0x1
#define DIAGAPP_MULTICAST_ONOFF_MASK 0xff00
#define DIAGAPP_MULTICAST_OFF 0x100
#define DIAGAPP_MULTICAST_ON 0x0

  tuint max_nvmsg;        /* Maximum number of packets to forward */

  tuint msgRoute;   /* Initial port to route messages */
  
#endif /* IP_BOOT_SUPP */

  tuint route;      /* Identifies active config, utopia or mac using DIAGAPP_ROUTE_xxxx */
  tint  portNum;    /* Physical port number, 0 indexed         */
  
  union {
    diagAppUtopiaCfg_t  utopia;
    diagAppMacCfg_t     mac;
  } u;
  
  tuint tag_desc; /* describes the Lookup Tag used in Ethernet PDT packet */
  
} diagApp_t;

#ifdef _TMS320C6X
#pragma STRUCT_ALIGN(diagApp_s, 4)
#endif


/* Initial message routing values */
#define DIAGAPP_ROUTE_UTOPIA  0
#define DIAGAPP_ROUTE_DPRAM   1
#define DIAGAPP_ROUTE_GMAC    2

/* rxPayloadType type definitions. These must match neuproto.h */
#define DIAGAPP_PROTO_NULL                   0
#define DIAGAPP_PROTO_IPV4                   1
#define DIAGAPP_PROTO_UDP                    2
#define DIAGAPP_PROTO_ETHERNET               3
#define DIAGAPP_PROTO_PPP                    4
#define DIAGAPP_PROTO_LLC_SNAP               5
#define DIAGAPP_PROTO_AAL1                   6
#define DIAGAPP_PROTO_AAL2                   7
#define DIAGAPP_PROTO_AAL5                   8
#define DIAGAPP_PROTO_LOGICAL_CONNECTION_ID  9
/* 10 - 15 are reserved */
#define DIAGAPP_PROTO_APP_PRIMARY            16
#define DIAGAPP_PROTO_APP_SECONDARY          17
#define DIAGAPP_PROTO_APP_ALT_PRIMARY        18
#define DIAGAPP_PROTO_APP_ALT_SECONDARY      19
#define DIAGAPP_PROTO_HDLC_RELAY             20
#define DIAGAPP_PROTO_ETHERNET_PDT           21
#define DIAGAPP_PROTO_PROTOCOL_IPV6          22
/* 23 reserved for future use */
/* 24 - 27 reserved for use by HW  */
#define DIAGAPP_PROTO_H_PRIORITY_CTRL_MSG    28
#define DIAGAPP_PROTO_N_PRIORITY_CTRL_MSG    29
#define DIAGAPP_PROTO_L_PRIORITY_CTRL_MSG    30
#define DIAGAPP_PROTO_READ_WRITE_MSG         31
#define DIAGAPP_PROTO_BY_RT_TYPE             0xFF

/* The minimum packet size (bytes) for messages if UDP is enabled */
#define DIAG_APP_MIN_ETHERNET_PACKET_SIZE_BYTES 64
    

#endif /* _DIAGAPP_H */


