*******************************************************************************	
* FILE PURPOSE: Contains utility functions that are in assembly but not 
*               necessarily numerical.
*******************************************************************************
* FILE NAME:   util_s.s
*
* DESCRIPTION: 
*        
*
* TABS: NONE
*
* $Id: UTIL_S.S 1.4 1999/01/08 17:31:02 qsu Exp $
*
* REVISION HISTORY:
*
* $Log: UTIL_S.S $
* Revision 1.4  1999/01/08 17:31:02  qsu
* - utl_copy changed to utlCopy.
* Revision 1.3  1998/12/15 14:59:12  BONAL
*    - utl_prog_to_data() reverted back to revision 1.1 version and
*      utlCopy() added.
* Revision 1.2  1998/05/20 21:22:02  Gokhan
*   cosine is calculated for 1/4 period from program memory.
* Revision 1.1  1997/12/29 15:45:23  BONAL
* Initial revision
*
* (C) Copyright 1997 Texas Instruments Incorporated.            *
; * 
; *  Redistribution and use in source and binary forms, with or without 
; *  modification, are permitted provided that the following conditions 
; *  are met:
; *
; *    Redistributions of source code must retain the above copyright 
; *    notice, this list of conditions and the following disclaimer.
; *
; *    Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the 
; *    documentation and/or other materials provided with the   
; *    distribution.
; *
; *    Neither the name of Texas Instruments Incorporated nor the names of
; *    its contributors may be used to endorse or promote products derived
; *    from this software without specific prior written permission.
; *
; *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; */
*****************************************************************************


           .def  _utl_prog_to_data
           .def  _misc_utlCopy,_utlCmpPrChksum

           .text 

******************************************************************************
* FUNCTION PURPOSE: Reads the data which is program memory and writes into 
*                   specified data memory. 
******************************************************************************
* DESCRIPTION: 
*  void  utl_prog_to_data (
*    void (*tbl_in_prog)(void),  - Pointer to table in program memory. 
*                                  Input syntax is like a function call to
*                                  make sure correct address is send in.
*    tint *tbl_in_data,          - Pointer to table in data memory.
*    tint table_size)            - Size of the table.  
*
******************************************************************************

_utl_prog_to_data:
    
        NOP
        NOP
        BNOP    .S2    B3, 5

        .text
******************************************************************************
* FUNCTION PURPOSE: Reads the data which is program memory and writes into 
*                   specified data memory. 
******************************************************************************
* DESCRIPTION: 
*  void  utlCmpChksum (
*    tulong progStartAddr,       - Starting address of the program block. 
*    tuint  size,                - Size of the program block.
*    tuint  *checksum)           - Pointer to checksum.  
*
******************************************************************************

_utlCmpPrChksum:

; register usage
;   -- Starting address of the program block [Passed as 
;                                            ggChkSum_ProgBlck[i].blck_start]
;   -- Size of the program block             [Passed as 
;                                            ggChkSum_ProgBlck[i].blck_size]
;  -- Pointer to checksum   [Passed as &shared_prog_checksum == dspImageMap[30]]

     
        NOP
        NOP
        BNOP    .S2    B3, 5

;****************************************************************************
;FUNCTION PURPOSE: Copy a linear array to a circbuf.
;****************************************************************************
;*
;* DESCRIPTION: Copies elements from one buffer to another buffer. 
;*
;*  void  misc_utlCopy ( 
;*    word *inBuf,   - address of BSP buffer    
;*    word *outBuf, - address of queue buffer    
;*    tint length,  - length of data to be copied    
;*
;****************************************************************************

_misc_utlCopy:
; Arguments passed through registers
;      A4 = linSample *inbuf
;      B4 = linSample *outbuf
;      A6 = tint length
; 
; Registers Used 
;      B8 =  inbuf +  1
;      A8 =  outbuf + 1
;      A6 = A7 = B6 = B7 = 2


; Assume the number of elements to be copied to be MULTIPLE OF 2         
         SHRU   .S1     A6,3,A0         ; A0 = count/8
     ||  AND    .L1     A6,7,A1         ; A1 = 0X7 & length
     ||  MVK    .D1     2, A7           ; A6 = 2
     ||  MVK    .D2     2, B7           ; B7 = 2
     ||  MVK    .L2     2, B6   
                
    [!A0]B      .S1     LOREQ8
     ||  ADD    .L2     A4,B7, B8       ; B8 = inbuf + 1
     ||  MV     .L1     B4, A8          ; A8 = outbuf
     ||  MVK    .D1     2,A6            ; A7 = 2    
     
         LDH    .D1T1  *A4++[A6],  A16
     ||  LDH    .D2T2  *B8++[B7],  B16
     ||  ADD    .L2     B4,B6, B4       ; A8 = outbuf + 1
     ||  SHRU   .S2     A1,1,B1
     ||  AND    .L1     A1,1,A2
     
         LDH    .D1T1  *A4++[A6],  A17
     ||  LDH    .D2T2  *B8++[B7],  B17
     ||  ADD    .L1    A2,B1,A1

     
         LDH    .D1T1  *A4++[A6],  A18
     ||  LDH    .D2T2  *B8++[B7],  B18
              
         LDH    .D1T1  *A4++[A6],  A19
     ||  LDH    .D2T2  *B8++[B7],  B19
     || [!A0]B  .S2    B3
     
     [A0]SUB    .S1     A0, 1, A0       


_LOOP1:
         STH    .D1T1   A16, *A8++[A7]
     ||  STH    .D2T2   B16, *B4++[B6]
     
         LDH    .D1T1  *A4++[A6],  A16
     ||  LDH    .D2T2  *B8++[B7],  B16

     [A0]B       .S1    _LOOP1
     ||  STH    .D1T1   A17, *A8++[A7]
     ||  STH    .D2T2   B17, *B4++[B6]
     
         LDH    .D1T1  *A4++[A6],  A17
     ||  LDH    .D2T2  *B8++[B7],  B17

         STH    .D1T1   A18, *A8++[A7]
     ||  STH    .D2T2   B18, *B4++[B6]

         LDH    .D1T1  *A4++[A6],  A18
     ||  LDH    .D2T2  *B8++[B7],  B18
       
         STH    .D1T1   A19, *A8++[A7]
     ||  STH    .D2T2   B19, *B4++[B6]
     ||[!A0] B   .S2    B3 
     
         LDH    .D1T1  *A4++[A6],  A19
     ||  LDH    .D2T2  *B8++[B7],  B19
     ||[A0]SUB  .L1    A0,1,A0

; END OF THE LOOP


LOREQ8:
        [A1] STH    .D1T1   A16, *A8++[A7]
     || [B1] STH    .D2T2   B16, *B4++[B6]
     || [A1] SUB    .L1     A1,1,A1
     || [B1] SUB    .L2     B1,1,B1
     

        [A1] STH    .D1T1   A17, *A8++[A7]
     || [B1] STH    .D2T2   B17, *B4++[B6]
     || [A1] SUB    .L1     A1,1,A1
     || [B1] SUB    .L2     B1,1,B1

        [A1] STH    .D1T1   A18, *A8++[A7]
     || [B1] STH    .D2T2   B18, *B4++[B6]
     || [A1] SUB    .L1     A1,2, A1
     || [B1] SUB    .L2     B1,1,B1

        [A1] STH    .D1T1   A19, *A8++[A7]
     || [B1] STH    .D2T2   B19, *B4++[B6]
     || [B1] SUB    .L2     B1,1,B1    
       

