/*******************************************************************************
 * FILE PURPOSE: Miscellaneous operations
 *******************************************************************************
 * FILE NAME: cmisc.c
 *
 * DESCRIPTION: common routines
 *
 * TABS: None
 *
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "cmisc.h"


/*******************************************************************************
 * FUNCTION PURPOSE: Extract a word from a byte indexed array
 *******************************************************************************
 * DESCRIPTION: A single word is extracted from a byte offset in an array.
 *******************************************************************************/
tuint cmiscExtractWordByteOffset (tuint *array, tint byteOffset)
{
  tuint wordOffset;
  tuint msb;
  tuint lsb;

  wordOffset = (byteOffset + 1) >> 1;
  /* If the byte offset is even return the word */
  if (!(byteOffset & 1))
    return (array[wordOffset]);

  /* Otherwise form the word from the two bytes */
  msb = (array[wordOffset-1] & 0x00ff) << 8;
  lsb = (array[wordOffset]   & 0xff00) >> 8;
  return (msb | lsb);

} /* cmiscExtractWordByteOffset */

/**********************************************************************************
 * FUNCTION PURPOSE: Write a word into an array with a byte offset
 **********************************************************************************
 * DESCRIPTION: A single word is written to a byte offset in an array
 **********************************************************************************/
void cmiscInsertWordByteOffset (tuint *array, tint byteOffset, tuint value)
{
  tuint wordOffset;
  tuint msw;
  tuint lsw;

  wordOffset = (byteOffset + 1) >> 1;
  
  /* If the byte offset is even, write the word directly */
  if (!(byteOffset & 1))  {
    array[wordOffset] = value;
    return;
  }

  /* Otherwise place the word in as two bytes */
  msw = (array[wordOffset-1] & 0xff00) | (value >> 8);
  array[wordOffset-1] = msw;

  lsw = (array[wordOffset] & 0x00ff) | (value & 0xff00);
  array[wordOffset] = lsw;

} /* cmiscInsertWordByteOffset */



    





