/******************************************************************************
 * FILE PURPOSE: DSP utility functions
 ******************************************************************************
 * FILE NAME: util.c   
 *
 * DESCRIPTION: Contains the utility functions.
 *
 * TABS: NONE   
 *
 * $Id: util.c 1.6 1998/04/20 16:59:26 BOGDANK Exp $
 *
 * REVISION HISTORY:   
 *
 * $Log: util.c $
 * Revision 1.6  1998/04/20 16:59:26  BOGDANK
 *     - utlLFractVecMax() and utlLsVecMax() moved to utilnum.c/.s
 * Revision 1.5  1997/05/21 16:37:53  BOGDANK
 *     - new types added
 *    New types added.
 * Revision 1.4  1996/11/15 14:48:16  BOGDANK
 *     - a few functions added for different types
 *    Adding utlLFractVecMax().
 * Revision 1.3  1996/11/11 16:18:37  BOGDANK
 *     - utlFractMemSet(), utlLFractMemSet() added.
 *    Adding utlFractMemSet() and utlLFractMemSet().
 * Revision 1.2  1996/09/24 14:46:30  BONAL
 * Moved utlMoveMemory to util.c from util.h
 *
 * (C) Copyright 1995-1998, Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header types */
#include "types.h"

/* Util.c specific header files */
#include "util.h"

/*****************************************************************************
 * FUNCTION PURPOSE:  Write a constant of type Fract to an array of Fract's.
 *****************************************************************************  
 * DESCRIPTION: Sets the array a[] of length n to the constant c. Constant is
 *              of type Fract.
 *
 *  void utlFractMemSet (
 *     Fract *a,      - Pointer to the array of Fract's
 *     Fract c,       - Constant to be written to the array
 *     tint  n)       - Length of the array
 * 
 *****************************************************************************/

void utlFractMemSet (Fract *a, Fract c, tint n)
{
  int i;
  for (i = 0; i < n; i++) a[i] = c;
} /* utlFractMemSet */

/*****************************************************************************
 * FUNCTION PURPOSE:  Write a constant of type LFract to an array of LFract's.
 *****************************************************************************  
 * DESCRIPTION: Sets the array a[] of length n to the constant c. Constant is
 *              of type LFract.
 *
 *  void utlLFractMemSet (
 *     LFract *a,     - Pointer to the array of LFract's
 *     LFract c,      - Constant to be written to the array
 *     tint   n)      - Length of the array
 * 
 *****************************************************************************/

void utlLFractMemSet (LFract *a, LFract c, tint n)
{
  int i;
  for (i = 0; i < n; i++) a[i] = c;
} /* utlLFractMemSet */

/*****************************************************************************
 * FUNCTION PURPOSE: Write a constant of type linSample to an array of 
 * linSamples. 
 *****************************************************************************  
 * DESCRIPTION: Sets the array a[] of length n to the constant c. Constant is
 * type linSample.
 *
 *   void utlLsMemSet ( 
 *     linSample *a, - Pointer to the array of linSamples 
 *     linSample c,  - Constant to be written to the array             
 *     tint      n)  - Length of the array 
 * 
 *****************************************************************************/

void utlLsMemSet (linSample *a, linSample c, tint n)
{
  int i;
  for (i = 0; i < n; i++) a[i] = c;
} /* utlLsMemSet */

/*****************************************************************************
 * FUNCTION PURPOSE: Perform memory move operation.
 *****************************************************************************
 * DESCRIPTION: Used in memory move operations.
 * 
 *****************************************************************************/

void utlMoveMemory (const linSample *in, linSample *out, tint number)
{
    int counter;
    for (counter = 0; counter < number; counter++)
      *out++ = *in++;
} /* utlMoveMemory */

/* nothing past this point */
