#ifndef _UTLFIFO_H
#define _UTLFIFO_H
/*****************************************************************************
 * FILE PURPOSE: Simple FIFO definition
 *****************************************************************************
 * FILE NAME: utlfifo.h
 *
 * DESCRIPTION: Protypes and structure definitions for implementing FIFOs
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <stdlib.h>
#include "types.h"
#include "dglob.h"

/*****************************************************************************
 * DEFINITION: Fifo structure. All elements in the fifo are 16 bits.
 *****************************************************************************/
typedef struct {
  word *base;      /* Base address of FIFO         */
  tuint  size16;   /* Size of Fifo in 16 bit words */
  tuint  inIndex;  /* FIFO input index             */
  tuint  outIndex; /* FIFO output index            */
} utlFifo_t;


/*****************************************************************************
 * FIFO API
 *****************************************************************************/
word *utlFifoIn    (utlFifo_t *fifo, globData_t *data);
word *utlFifoOut   (utlFifo_t *fifo, globData_t *data);
void utlFifoBackout (utlFifo_t *fifo, tuint backoutLen);
void utlFifoReset   (utlFifo_t *fifo);
tuint utlFifoSpaceAvail (utlFifo_t *fifo);
tuint utlFifoNelems(utlFifo_t *fifo);





#endif /* _DSIUFIFO_H */





