#ifndef _DPVP_H
#define _DPVP_H
/****************************************************************************
 * FILE PURPOSE: Defines an API for Diagnostic Voice Packetization module (DPVP)
 ****************************************************************************
 * FILE NAME dpvp.h
 *
 * DESCRIPTION: Defines the API used to interact with the DPVP module
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dglob.h"

/*****************************************************************************
 * DEFINITION: Return values from DPVP
 *****************************************************************************/
typedef enum {
  DPVP_SUCCESS = 0,
  DPVP_FAIL
} dpvpReturn_t;

/*****************************************************************************
 * DEFINITION: Supported encapsulation types
 *****************************************************************************/
typedef enum {
  DPVP_ENCAP_AAL1,
  DPVP_ENCAP_AAL1_16BIT,
  DPVP_ENCAP_RAW,
  DPVP_ENCAP_RAW_16BIT,
  DPVP_ENCAP_NO_VOICE
} pvpEncap_t;

/****************************************************************************
 * DEFINITION: Channel size configuration structure
 ****************************************************************************/
typedef struct {
  tuint dummy;
} dpvpSizeConfig_t;

/*****************************************************************************
 * DEFINITION: Channel new configuration structure
 *****************************************************************************/
typedef struct {
  tuint Id;                  /* Instance ID        */
  dpvpSizeConfig_t sizeCfg;  /* Size configuration */
} dpvpNewConfig_t;

/*****************************************************************************
 * DEFINITION: Context size configuration structure
 *****************************************************************************/
typedef struct {
  tuint dummy;
} dpvpContextConfig_t;

/*****************************************************************************
 * DEFINITION: Channel open configuration structure
 *****************************************************************************/
typedef struct {
  pvpEncap_t encap;               /* Aal1 or raw encapsulation */
  
  tuint txValidDataSizeBytes;     /* Valid bytes per packet tx */
  tuint rxValidDataSizeBytes;     /* Valid bytes per packet rx */
  tuint txFill;                   /* Fill value for tx packets */
} dpvpOpenConfig_t;

/*****************************************************************************
 * DEFINITION: Run time configuration and control
 *****************************************************************************/
typedef enum {
  DPVP_CTL_SET_ENCAP = 100
} dpvpCtlCode_t;

typedef struct dpvpSetEncap_s {
  pvpEncap_t encap;
  tuint      txValidDataSizeBytes;
  tuint      rxValidDataSizeBytes;
} dpvpSetEncap_t;
 
typedef struct dpvpControl_s {
  dpvpCtlCode_t   code;
  union {
    dpvpSetEncap_t setEncap;
  } u;
} dpvpControl_t;

/*****************************************************************************
 * DEFINITION: The DPVP call table API
 *****************************************************************************/
typedef struct {
  dpvpReturn_t (*dpvpGetSizes) (tint *, dmemBuffer_t **, dpvpSizeConfig_t *);
  tuint (*dpvpGetContextSizes) (dpvpContextConfig_t *);
  dpvpReturn_t (*dpvpContextInit) (dpvpContextConfig_t *, void *);
  void *(*dpvpNew) (dpvpNewConfig_t *, dmemBuffer_t *);
  dpvpReturn_t (*dpvpOpen) (void *, dpvpOpenConfig_t *);
  dpvpReturn_t (*dpvpReceiveIn) (void *, globData_t *);
  dpvpReturn_t (*dpvpSendIn) (void *, globData_t *);
  dpvpReturn_t (*dpvpControl) (void *, dpvpControl_t *);
} dpvpCallTable_t;
extern dpvpCallTable_t dpvpCallTable;
  
#endif /* _DPVP_H */




