/******************************************************************************
 * FILE PURPOSE: Perform Core Boot
 ******************************************************************************
 * FILE NAME: dsiu5561.c
 *
 * DESCRIPTION: Contains the CPU core boot code specific to the c5561
 *              processor.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "dsiuloc.h"
#include "diagcfg.h"
#include "types.h"
#include "hal.h"


/*****************************************************************************
 * FUNCTION PURPOSE: Boot 6416 core
 *****************************************************************************
 * DESCRIPTION: Core 0 is booted by call the function at the entry symbol. 
 *              This is a one way call, with no return. The remaining cores
 *              are booted by copying the entry address into the local memory
 *              of that core at word address 0x0100.
 *
 *****************************************************************************/
void  dummy_code(void);

dsiuCoreBootReturn_t c6416CoreBoot (tuint core, tulong entry)
{
  tint loc, i;
  
  void (*goodbye)(void);

  goodbye = (void (*)())(entry);
  (*goodbye)(); /* And don't come back! */

  return (DSIU_CORE_BOOT_SUCCESS);

} /* c6416CoreBoot */
  

#if 0
/*****************************************************************************
 * FUNCTION PURPOSE: Boot 6416 core
 *****************************************************************************
 * DESCRIPTION: Core 0 is booted by call the function at the entry symbol. 
 *              This is a one way call, with no return. The remaining cores
 *              are booted by copying the entry address into the local memory
 *              of that core at word address 0x0100.
 *
 *****************************************************************************/
dsiuCoreBootReturn_t c6416CoreBoot1 (tuint core, tulong entry)
{
  void (*goodbye)(void);
/*#define CHIP_6416
#include <csl.h>
#include <csl_cache.h>
  CACHE_clean (CACHE_L1PALL, 0,0);
  CACHE_clean (CACHE_L1DALL, 0,0);
*/  
  goodbye = (void (*)())(entry);
  (*goodbye)(); /* And don't come back! */

  return (DSIU_CORE_BOOT_SUCCESS);

} /* c6416CoreBoot */
#endif  

void  dummy_code(void)
{
   asm(" NOP"); 
 
}
 
