#ifndef _DSIUPORT_H
#define _DSIUPORT_H
/*****************************************************************************
 * FILE PURPOSE: Define processor specific macros
 *****************************************************************************
 * FILE NAME: dsiuport.h
 *
 * DESCRIPTION: Macros which depend on processor
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#define DIAG_BASE_ADDRESS_MOD(a,b) (a+b)

/*****************************************************************************
 * Macro to construct an unsigned long from two unsigned ints
 *****************************************************************************/
#define DSIU_DWORD_CONSTRUCT(msw,lsw)  (((int)(msw)<<16)|((int)lsw))

#define DSIU_TDM_WORD_SIZE  8
#define DSIU_TDM_PORTA      0x10
#define DSIU_MEM_ALIGN_BITS  3   


/*****************************************************************************
 * Convert bytes to/from words. Use SIU name for system compatibility (used in
 * ndu files
 *****************************************************************************/
#define SIU_BYTES_TO_WORDS(x) (x)
#define SIU_WORDS_TO_BYTES(x) (x)

/*****************************************************************************
 * Convert words to/from dwords.
 *****************************************************************************/
#define SIU_WORDS_TO_DWORDS(x)  (((x)+3)>>2)
#define SIU_DWORDS_TO_WORDS(x)  ((x)<<2)

/*****************************************************************************
 * Convert bytes to/from tuint.
 *****************************************************************************/
#define SIU_BYTES_TO_TUINT(x)  (((x)+1)>>1)
#define SIU_TUINT_TO_BYTES(x)  ((x)<<1)

/*****************************************************************************
 * Convert from native compiler size to size in TUINT (as from sizeof macro)
 *****************************************************************************/
#define SIU_SIZE_WORDS_TO_TUINT(x)  SIU_BYTES_TO_TUINT(x)
#define SIU_SIZE_TUINT_TO_WORDS(x)  SIU_TUINT_TO_BYTES(x)

/*****************************************************************************
 * HAL heap alignment - require double word alignment 
 *****************************************************************************/
#define DSIU_PORT_ALIGN_HEAPBASE(x)  (x)=(((x)+3)&0xfffc)

/* Critical sections for the ISIU */
/* For control path */
#define ISIU_STATE_BEGIN() (thwCriticalBegin())
#define ISIU_STATE_END()   (thwCriticalEnd())

/* The maximum number of tdm ports is needed for compile time arrays */
#define DSIU_MAX_TDM_PORTS  3


/* Random generator seed value */
extern volatile unsigned int cregister TSCL;
#define DSIU_RANDOM_SEED()   (*((volatile unsigned int *)TSCL))



#endif /* _DSIUPORT_H */

