#ifndef _LLMT_H
#define _LLMT_H
/*******************************************************************************************
 * FILE PURPOSE: Define the low level memory test API
 *******************************************************************************************
 * FILE NAME: llmt.h
 *
 * DESCRIPTION: Defines the llmt interface
 *
 * Copyright (C) 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "diagreg.h"
#include "diagcfg.h"

#define LLMT_STACKSIZE  0x400
 
 
/* Error reporting */
typedef struct llmtErr_s  {
  tint errCount;
  tulong errAddr[DREG_MAX_MEM_FAILURES];
  tulong errExptd[DREG_MAX_MEM_FAILURES];
  tulong errActual[DREG_MAX_MEM_FAILURES];
} llmtErr_t;

typedef struct llmtb_s {
  word stack[LLMT_STACKSIZE];
  llmtErr_t err;
} llmtb_t;

typedef struct llmtScratchBlocks_s  {

  dcfgMemBlock_t *dscratchBlk;
  dcfgMemBlock_t *sscratchBlk;
  dcfgMemBlock_t *cscratchBlk;

} llmtScratchBlock_t;



/* Prototypes */
void llmtMarchingContinue (word *addr, tulong lenWords, llmtErr_t *err);
void llmtAA55Continue(word *addr, tulong lenWords, llmtErr_t *err);
void llmtPseudoContinue(word *addr, tulong lenWords, llmtErr_t *err);
void llmtAddrContinue(word *addr, tulong lenWords, llmtErr_t *err);
void llmtProgContinue(word *addr, tulong lenWords, llmtErr_t *err);

void llmtMarching (word *addr, tulong lenWords, llmtScratchBlock_t *scratch);
void llmtAA55     (word *addr, tulong lenWords, llmtScratchBlock_t *scratch);
void llmtPseudo   (word *addr, tulong lenWords, llmtScratchBlock_t *scratch);
void llmtAddr     (word *addr, tulong lenWords, llmtScratchBlock_t *scratch);
void llmtProg     (word *addr, tulong lenWords, llmtScratchBlock_t *scratch);

void llmt_context_switch (void (*)(word *, tulong, llmtErr_t *), word *, tulong, llmtScratchBlock_t *);
void llmt_context_restore (void);

near void llmt_block_copy (word *src, word *dst, tulong lenWords);

#endif /* _LLMT_H */
