/*****************************************************************************
 * FILE PURPOSE: SIU gain control interface with AICs
 *****************************************************************************
 * FILE NAME: siuaic.c
 *
 * DESCRIPTION: Handles gain control in AICs. This is necessary due to 
 *              hardware limitations in the AIC22 in IP phone. The analog
 *              output gain on the aic22 is handled in a digital stage and
 *              an analog stage (PGA). But an unclipped digital signal can
 *              be clipped by the digital section of the AIC22 gain, before
 *              being sent to the AIC22. Thus the analog output signal is
 *              not an accurate reflection of the input digital signal.
 * 
 *              To prevent this, this file handles gain change messages
 *              for the aics. If the aic handsfree device has a gain change
 *              request, and if the aic is an aic22, the speaker gain is
 *              limited to 0db max. If the request is for greater then 0db,
 *              the value over 0db is compensated by increasing the rx gain
 *              instance in the siu instance.
 *
 * TABS: NONE
 *
 * (C) Copyright 2000, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "dsiuloc.h"
#include "hal.h"
 

void dsiuAnalogCodecConfig (void)
{
  aicCtrl_t aicCtrl;

  /* initialize the connectivity */
  aicCtrl.config.valid_bitfield = 0x3;
  aicCtrl.config.connectivity = 0x1111;
  aicCtrl.config.gain_paths = 0x0;
  (*dsiuContext.hal->halAicControl)(0, HAL_AIC_CFG, &aicCtrl);

  return;
}


/* Nothing past this point */