#ifndef _DSIUCTBL_H
#define _DSIUCTBL_H
/***********************************************************************************
 * FILE PURPOSE: Defines the call table for DSIU
 ***********************************************************************************
 * FILE NAME: dsiuctbl.h
 *
 * DESCRIPTION: Defines a call table used to interface with optional modules. 
 *				The DSIU call table is different from all other call tables in
 *				that there will always be one. It is declared in diagcfg.c.
 *
 * TABS: None
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dglob.h"
#include "dsiuloc.h"

typedef struct {
  
  /* Functions required to support neu */
  void *(*createVoiceNeu) (tint modId);
  dsiuReturn_t (*openVoiceNeu) (void *inst);
  
  /* Bert */
  void *(*createBert) (tint modId);
  dsiuReturn_t (*openBert) (void *inst, tint mode);
  dsiuReturn_t (*closeBert) (void *inst);
  
  /* Boot information from prior boot loaders */
  dsiuReturn_t (*prevBootInfo)(dsiuPrevBootInfo_t *info);
  
  /* Memory configuration */
  dsiuReturn_t (*memConfig) (void);
  
  /* Block message support */
  void (*blockMsgProc) (void);
  void (*blockMsgInit) (void);

} dsiuCallTable_t;

extern dsiuCallTable_t dsiuCallTable;  /* Declared in diagcfg.c */

#endif /* _DSIUCTBL_H */





