/*********************************************************************************
 * FILE PURPOSE: Perform memory clear and checksum - generic version
 *********************************************************************************
 * FILE NAME: dsiumchkgen.c
 *
 * DESCRIPTION: Performs the memory clear and memory checksum functions on
 *				memory blocks defined in the linker command file.
 *
 *              Generic routines -- there are c55x versions in dsiumemchkc55x.c.
 *
 * TABS: None
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "diagcfg.h"
#include "diagreg.h"
#include "dsiuloc.h"
#include "dsiumsg.h"
#include "dsiuport.h"
#include "hal.h"

/************************************************************************************
 * Definitions: Size of local stack data used for global memory transfer
 ************************************************************************************/
#define DSIU_MCHK_BLK_COUNT_DWORDS  400
#define DSIU_MCHK_BLK_COUNT_WORDS   SIU_DWORDS_TO_WORDS(DSIU_MCHK_BLK_COUNT_DWORDS)

#define DSIU_SHARED_ADDRESS 0x000F

/************************************************************************************
 * FUNCTION PURPOSE: Initialize memory
 ************************************************************************************
 * DESCRIPTION: Initializes a block of memory.
 ************************************************************************************/
tint dsiu_clear_block (tuint *base, tulong size, tuint val)
{
  tulong i;

  for (i = 0; i < size; i++)
    base[i] = val;

  return (0);

} /* dsiu_clear_block */

tuint dsiu_chksum (tuint *base, tulong size)
{
  tulong i;
  tuint  v = 0;

  for (i = 0; i < size; i++)
    v = v ^ base[i];

  return (v);

} /* dsiu_chksum */

/* Nothing past this point */
