#ifndef _DSIUMEM_H
#define _DSIUMEM_H
/*****************************************************************************
 * FILE PURPOSE: System heap memory header file
 *****************************************************************************
 * FILE NAME: dsiumem.h
 *
 * DESCRIPTION: Definitions used to access the system heap memory functions.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiutask.h"
#include "dsiufifo.h"
#include "dglob.h"

/*****************************************************************************
 * DEFINITION: Heap memory allocation control
 *****************************************************************************/
typedef struct {
  word  *base;
  tulong size;
  tulong used;
} dsiuHeap_t;

/*****************************************************************************
 * DEFINITION: Return codes.
 *****************************************************************************/
typedef enum {
  DSIU_MEM_SUCCESS,
  DSIU_MEM_FAIL
} dsiuMemReturn_t;

/* Align mask is used to provide alignment of memory blocks to power of 2
 *  address. It is derived from the DSIU_MEM_ALIGN_BITS(dsiuport.h) which 
 *  specifies how many low order bits must be zero for each block's base address.*/
#define DSIU_MEM_ALIGN_MASK    (~((~0U)<<DSIU_MEM_ALIGN_BITS)) 


/******************************************************************************
 * API 
 ******************************************************************************/
void *dsiuBufferAlign (void *bstart, void *bend, tint log2a, tulong *rsize, tuint *alignUsed);
void *dsiuHeapAlloc (dsiuHeap_t *heap, tulong size);
dsiuMemReturn_t dsiuAllocBufs (dsiuHeap_t *heap, tint nbufs, dmemBuffer_t *memb);
dsiuFifo_t *dsiuFifoAlloc (dsiuHeap_t *heap, tuint size);
dsiuTask_t *dsiuTaskAlloc (dsiuHeap_t *heap, tint depth);
tulong dsiuCacheTest1 (void);




#endif /* _DSIUMEM_H */
