/*************************************************************************************
 * FILE PURPOSE: System level interface to full application NEU unit
 *************************************************************************************
 * FILE NAME: dsiuneu.c
 *
 * DESCRIPTION: Provides the layer interfacing the diagnostic code to the full
 *				application NEU.
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include <ti/mas/neu/neu.h>
#include <ti/mas/neu/neuproto.h>
#include "diagcfg.h"
#include "memmgm.h"
#include "diagbld.h"

#include "targethw.h" /* for thwCritical* used in neuContext */
/*****************************************************************************
 * Full application call table declaration
 *****************************************************************************/
neuCallTable_t neuCallTable =
{
  neuClose,
  neuControl,
  neuDelete,
  neuGetSizes,
  neuNew,
  neuOpen,
#if (DIAG_UTOPIA_PORTS_USED >= 1)
  neuReceiveInUt,
#else
  NULL,
#endif
  neuSendIn,
  neuPktViolThrottle,
#if (DIAG_GMAC_PORTS_USED >= 1)
  neuReceiveInGmac,
  neuGetInstance,
#else
  NULL,
  NULL,
#endif
  neuGetTxScratchBuf,
#ifdef NEU_USE_GMPISRSPLIT
  neuGetNumPendingPkts,
#else
  NULL,
#endif
  NULL,                   /* neuSendInNvPktFrmHost  */
  NULL                    /* neuNvIpReceiveIn       */
};


/**
 *  These functions are supplied to neuContext to fill the GMP function 
 *  pointers.
 *
 *  NEU only requires GMP for Utopia AAL5 SAR. DIAG currently does not support 
 *  GMP, therefore these functions will halt the DSP and aid in debugging 
 *  should they be called within NEU.
 */
void* dsiuNeuGmpAllocGmc (tuint input_MID, neuGmpHandle_t handle)
{
  dsiuPanic(DSIU_FAIL_NEU_NO_GMP);
  return NULL;
}

tbool dsiuNeuGmpFreeGmc  (tuint input_MID, neuGmpHandle_t handle, void *gmp_gmc)
{
  dsiuPanic(DSIU_FAIL_NEU_NO_GMP);
  return FALSE;
}

void* dsiuNeuGmpCreate   (tuint input_MID, neuGmpHandle_t handle)
{
  dsiuPanic(DSIU_FAIL_NEU_NO_GMP);
  return NULL;
}

void  dsiuNeuGmpDelete   (tuint input_MID, neuGmpHandle_t handle, void *headIn)
{
  dsiuPanic(DSIU_FAIL_NEU_NO_GMP);
}

tint  dsiuNeuGmpRead     (tuint input_MID, neuGmpHandle_t handle, void *head, void *dstOut, tint records_to_copy)
{
  dsiuPanic(DSIU_FAIL_NEU_NO_GMP);
  return 0;
}

void  dsiuNeuGmpWrite    (tuint input_MID, neuGmpHandle_t handle, void *head, void *srcIn, tint records_to_copy)
{
  dsiuPanic(DSIU_FAIL_NEU_NO_GMP);
}

tint  dsiuNeuGmpReadFromStart (tuint input_MID, neuGmpHandle_t handle, void *head, void *dstOut, tint records_to_copy)
{
  dsiuPanic(DSIU_FAIL_NEU_NO_GMP);
  return 0;
}


/*****************************************************************************
 * Full application context declaration.
 *****************************************************************************/
neuContext_t neuContext = {
  dsiuNeuDebugInfo,              /* debugInfo_t debugInfo                       */
  0,                             /* tuint cur_ip_id                             */
  0,                             /* tuint min_ip_id                             */
  0,                             /* tuint max_ip_id                             */
  0,                             /* cellSize                                    */
  &neuPortMap,                   /* neuPortMap_t *                              */
  neuproto_PROTOCOL_ETHERNET,   /* tint msgRxPayloadType                       */
  dsiuNeuGetTime,                /* neuGetTimeFcn_t getTime                     */
  dsiuNeuPktViol,                /* neuPktViolationFcn_t pktViolation           */
  NULL,                          /* getNeuInst                                  */
  NULL,                          /* rcuGetInst                                  */
  NULL,                          /* neuRtcpReceiveOut                           */
  NULL,                          /* neuRtcpSendOut                              */
  NULL,                          /* updateTxStats                               */
  {0L,0L,0L,0L,0L,0L,0L,0L,0,0,0,0,0}, /* neuGlblPktStats_t pktStats            */
#ifdef NEU_MAX_PAYLOAD_B         /* tuint neuMaxPayloadBytes                    */
  NEU_MAX_PAYLOAD_B,
#else 
#ifdef NEU_MAX_PKT_SEGMENTS
  80*NEU_MAX_PKT_SEGMENTS + 22,
#else
  252,
#endif
#endif
  {0,0,NULL,NULL,0},             /* neuGmcPktTrackerRoot_t neuGmcPktTrackerRoot */
#ifdef ITDM
  isiuNeuGetInst,                /* return itdm neuInstance                     */
#endif
  0,                             /* tuint tag_desc                              */
  {{0,0},                        /* nonVoiceIP.srcIPaddr                        */
   0,                            /* nonVoiceIP.nIPv4addr                        */
   {0,0,0,0,0,0,0,0,             /* nonVoiceIP.srcIPv6addr[0]                   */
    0,0,0,0,0,0,0,0},            /* nonVoiceIP.srcIPv6addr[1]                   */
   0,                            /* nonVoiceIP.nIPv6addr                        */
   0,                            /* nonVoiceIP.firstNvpTimeInMs                 */
   0,                            /* nonVoiceIP.numNvPacketInWin                 */
   100,                          /* nonVoiceIP.nvpTimeoutInIsrTicks             */
   32                            /* nonVoiceIP.nvpMaxNumPackets                 */
  },

  NULL,                          /* gmpHandle                                   */
  dsiuNeuGmpAllocGmc,
  dsiuNeuGmpFreeGmc,
  dsiuNeuGmpCreate,
  dsiuNeuGmpDelete,
  dsiuNeuGmpRead,
  dsiuNeuGmpWrite,
  dsiuNeuGmpReadFromStart,

  thwCriticalBegin,
  thwCriticalEnd,
  thwCriticalBeginFastFcn,
  thwCriticalEndFastFcn
#ifdef NONVOICE_IP
  ,0
#endif
};

/**************************************************************************************
 * FUNCTION PURPOSE: Dummy function required to support full application NEU
 **************************************************************************************
 * DESCRIPTION: Provides the required hook for the application call table
 **************************************************************************************/
void dsiuNeuDebugInfo (void *ID, tuint msgType, tuint msgCode, tuint msgLength, 
                       tuint *supportingData)
{

  DSIU_DUMMY_ARG(ID);
  DSIU_DUMMY_ARG(msgType);
  DSIU_DUMMY_ARG(msgCode);
  DSIU_DUMMY_ARG(msgLength);
  DSIU_DUMMY_ARG(supportingData);
  /* empty */

} /* dsiuNeuDebugInfo */

/**************************************************************************************
 * FUNCTION PURPOSE: Dummy function required to support full application NEU
 **************************************************************************************
 * DESCRIPTION: Provides the required hook for the application call table
 **************************************************************************************/
tint dsiuNeuSendOut (void *v, xferPktInfo_t *p)
{
  DSIU_DUMMY_ARG(v);
  DSIU_DUMMY_ARG(p);
  /* empty */
  return (0);

} /* dsiuNeuSendOut */

/**************************************************************************************
 * FUNCTION PURPOSE: Dummy function required to support full application NEU
 **************************************************************************************
 * DESCRIPTION: Provides the required hook for the application call table
 **************************************************************************************/
tulong dsiuNeuGetTime (void)
{
  /* empty */
  return (0);

} /* dsiuNeuGetTime */

/**************************************************************************************
 * FUNCTION PURPOSE: Dummy function required to support full application NEU
 **************************************************************************************
 * DESCRIPTION: Provides the required hook for the application call table
 **************************************************************************************/
void dsiuNeuPktViol (tuint ID, void *d)
{
  DSIU_DUMMY_ARG(ID);
  DSIU_DUMMY_ARG(d);
  /* empty */

} /* dsiuNeuPktViol */



/**************************************************************************************
 * FUNCTION PURPOSE: Allocates memory for an neu instance
 **************************************************************************************
 * DESCRIPTION: Takes memory from the heap to to create an neu instance.
 **************************************************************************************/
void *siuNeuCreate (void *vneuCfgSize, tint ecoId)
{
  void          *inst = NULL;
  neuNewCfg_t    neuCfgNew;
  neuGlobParms_t neuGlobCfg;
  neuSizeCfg_t  *neuCfgSize = (neuSizeCfg_t *)vneuCfgSize;

  neuCfgNew.ID      = ecoId;
  neuCfgNew.sizeCfg = *neuCfgSize;

  /* Configure the Neu global parameters */
  neuGlobCfg.cellSize = dsiuContext.siuNduContext.cellSize;
  neuGlobCfg.tag_desc = dsiuContext.diagApp.tag_desc;
  neuGlobCfg.valid_params = neu_GLOB_PARM_VALID_CELL_SIZE |
                         neu_GLOB_PARM_VALID_TAG_DESC;
  
  neu_glob_config(&neuGlobCfg);

  /* Call the full application interface function to setup the ECO. On error
   * inst will point to NULL. */
  siuFullAppEcoNewEco ((dsiuEcoGetSizes_t)neuGetSizes, vneuCfgSize, (dsiuEcoNew_t)neuNew,
                        &neuCfgNew, &inst, ecoId);

  return (inst);
} /* siuNeuCreate */

#ifdef ITDM
void *isiuNeuGetInst(void *neuInst, tint pktSize, word *pktIn)
{
    dsiuInst_t *inst = (dsiuInst_t *)&dsiuInst[0];
    tuint i;

    for (i=0; i<diagCfg.maxItdmStreams;i++,inst++)
    {
       if((*diagCfg.ctable.ippu->ippuCompareUID)((void*)inst->ippuInst,pktIn))
           break;
    }

    if (i < diagCfg.maxItdmStreams)
        return (inst);
    else
        return NULL;

}
#endif

  
