/*****************************************************************************
 * FILE PURPOSE: DSIU file defining compile time config parameters
 *****************************************************************************
 * FILE NAME: dsiunwcfg.h
 *
 * DESCRIPTION: Structures which specify buffer sizes and other build
 *              specific parameters
 *               
 * TABS: None
 *
 * Copyright (C) 2011 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"

typedef enum {
  DSIU_NWAL_TX_CTL_BUFS,
  DSIU_NWAL_RX_CTL_BUFS,
  DSIU_NWAL_TX_PKT_BUFS,
  DSIU_NWAL_RX_PKT_BUFS
} dsiuNwalLinkedBufsType_e;

typedef struct {
  void     *bufsBase;    /* Base address of the buffers (size = bufSize * nBufs) */
  uint32_t  bufSize;     /* Size of each buffer */
  uint32_t  nBufsPerCore;       /* Number of buffers per core */
  dsiuNwalLinkedBufsType_e type;
} dsiuNwalLinkedBufsDesc_t;

typedef struct {
  uint32_t  numDescsPerCore;
  uint32_t  descSize;
  void     *descMemBase;
  const dsiuNwalLinkedBufsDesc_t *linkedBufDescs;
} dsiuNwalCfg_t;

extern const dsiuNwalCfg_t dsiuNwalCfg;

/* Nothing past this point */

