/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/**************************************************************************************************
 * FILE PURPOSE: Header file for Out of the Box run
 **************************************************************************************************
 * FILE NAME: dsiuotbrun.h
 *
 * DESCRIPTION: 
 *
 **************************************************************************************************/
#include "types.h"



/**************************************************************************************************
 *                         TDM ISR SIMULATION
 *
 * TDM ISR is simulated to trigger TDM TX and RX tasks
 * Channels are staggered and a group of channels is processed during each ISR 
 * In the TX direction the simulation can input different signals as defined below.
 **************************************************************************************************/

/*Number of ISR slots that are simulated. 
  Each ISR slot will process N channels. N=(Total # of Channels)/(SIM_NUM_TDM_ISR_SLOTS)*/
#define DSIU_SIM_NUM_TDM_ISR_SLOTS         10

/*size (number of samples) of the TDM pattern array*/
#define DSIU_SIM_TDM_PATTERN_SIZE 40

/*Options for input TDM TX pattern when TDM ISR is simulated*/
typedef enum {
  
  /*This option should be used for the BERT OTB mode 
    because it does not replace the input TDM TX signal */
  DSIU_SIM_TDM_PATTERN_NO_PATTERN=0,
  
  /*This option replaces input TDM signal by pattern defined in inputPattern[] 
  (if BERT TX is on for the channel, the replacement won't work) */ 
  DSIU_SIM_TDM_PATTERN_ARRAY_PATTERN,
  
  /*This option replaces replaces input TDM signal by channel number  
  (if BERT TX is on for the channel, the replacement won't work)*/
  DSIU_SIM_TDM_PATTERN_CH_NUM_PATTERN
}dsiuSimTdmPattern_t;

/**************************************************************************************************
 *                         CONFIGURATION OF THE SIMULATION
 *
 **************************************************************************************************/
typedef struct dsiuSimConfig_s
{
    /* Out of the Box (OTB) BERT option */
    /* TRUE  --> OTB test run:  [bert or TDM(pattern)]  <->  [PKT] */
    /* FALSE --> no OTB run */
    bool opt_diag_otb_BERT;

    /* TRUE  --> simulates TSIP ISR (must be used when running on simulator) */
    /* FALSE --> uses HW TSIP ISR*/
    bool opt_simulate_tsip_isr;
    
    /*in the simulated TSIP ISR, loop through (or generate ISR for) tsipIsrNumCh PCM channels*/
    tuint tsipIsrNumCh; 
 
    /* If TSIP ISR is simulated, this variable defines which pattern is transmited in the TX direction  */
    dsiuSimTdmPattern_t opt_tdm_input_pattern;
    
    /*TDM input pattern that is used in case opt_tdm_input_pattern=SIM_TDM_PATTERN_ARRAY_PATTERN*/
    linSample inputPattern[DSIU_SIM_TDM_PATTERN_SIZE]; 
} dsiuSimConfig_t;

extern dsiuSimConfig_t dsiuSimConfig;

/**************************************************************************************************
 *                         CONFIGURATION OF THE OUT OF THE BOX (OTB) BERT TEST
 *
 **************************************************************************************************/
typedef struct dregBertTestCntl_s
{
    /*BERT mode for all channels in the test*/
    /*Options are: DREG_ROUTE_OUT_SRC_BERT_XXX*/
    tuint  bertMode;            
    
    /*First channel (0 based count) that will run BERT test*/
    tuint  firstCh;             
    
    /*Number of channels that will run BERT test.*/ 
    tuint  nCh;                 
    
    /* TRUE  --> will generate a BERT stats periodic report (every second) for all channels*/
    /* FALSE --> will report BERT stats only when there is a BERT 
                 state change IN SYNC-->NOT IN SYNC  or  NOT IN SYNC-->IN SYNC */ 
    bool  periodicReport;      
} dregBertTestCntl_t;

extern dregBertTestCntl_t dregBertTestCntl;

/**************************************************************************************************
 *                         FUNCTION PROTOTYPES
 *
 **************************************************************************************************/
void dsiuBertTestSetup (void);
void dsiuInitDiagAppMem(void);
void dsiuCfgOtbBert (void);
void dsiuTDMIsr(void);

