#ifndef _DSIUTASK_H
#define _DSIUTASK_H
/*****************************************************************************
 * FILE PURPOSE: Define system task handler
 *****************************************************************************
 * FILE NAME: dsiutask.h
 *
 * DESCRIPTION: Defines structures used to handle task posting. Tasks are
 *              generally posted from ISRs.
 *
 * TABS: None
 *
 * Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"

/*****************************************************************************
 * Definition: Task queue. Task queues are arrays of dsiuInst pointers.
 *****************************************************************************/
typedef struct {
  void **tasks;    /* Array of dsiuInst or other pointers       */
  tuint  inIndex;   /* Location to add task                      */
  tuint  outIndex;  /* Location to remove task                   */
  tuint  size;      /* Number of elements in task array          */
} dsiuTask_t;

/****************************************************************************
 * Task Return Codes
 ****************************************************************************/
typedef enum {
  DSIU_TASK_SUCCESS = 0,
  DSIU_TASK_FAIL
} dsiuTaskReturn_t;


/*****************************************************************************
 * Task API
 *****************************************************************************/
void *dsiuGetTask (dsiuTask_t *task);
dsiuTaskReturn_t dsiuPostTask (dsiuTask_t *task, void *inst);
tuint dsiuTaskSizeAvail (dsiuTask_t *task);
void dsiuTaskClear (dsiuTask_t *task);

#endif /* _DSIUTASK_H */


