#ifndef _DSIUUTOP_H
#define _DSIUUTOP_H
#include "dglob.h"
#include "dsiuloc.h"
#include "utlrepck_m.h"
/******************************************************************************
 * FILE PURPOSE: System level utopia definitions and prototypes.
 ******************************************************************************
 * DESCRIPTION: Utopia function prototypes and definitions
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/*****************************************************************************
 * DEFINITION: The maximum size of an atm cell
 ****************************************************************************/
#define DSIU_UTOPIA_MAX_CELL_SIZE  58

/*****************************************************************************
 * DEFINITION: Route Tag mode structure
 *****************************************************************************/
typedef struct {
  tuint msg_id;              /* Message ID                    */
  tuint trans_id;            /* Transaction ID                */
  tuint msg_first_word;      /* First word of message body    */
} dsiuRouteTagMsgPayload_t;
#define tsiuRouteTagMsgPayload_t dsiuRouteTagMsgPayload_t
#define SIU_ROUTE_TAG_MSG_HEADER_LENGTH   2

typedef struct {
  tuint type_dest_src;       /* Type, dest phy, source phy    */
  tuint janus_core_channel;  /* Janus ID, Core ID, Channel ID */
  tuint gfc_vpi_vci;         /* GFC, VPI, VCI                 */
  tuint vci_pt;              /* VCI, PT                       */
  tuint hec_dummy;           /* HEC, padding                  */
  tuint payload_first_word;  /* 1st word of payload           */
} dsiuRouteTagHeader_t;
#define tsiuRouteTagHeader_t dsiuRouteTagHeader_t
#define DSIU_UTOPIA_RT_HEADER_SIZE_WORDS   5  /* Size less 1st byte */ 
/* Macros for extraction of the type field */
#define DSIU_UTOPIA_RT_TYPE_MASK   0xF800
#define DSIU_UTOPIA_RT_TYPE_SHIFT      11
#define SIU_UTOPIA_RT_EXTRACT_TYPE(x) \
   ((((x)->type_dest_src)&DSIU_UTOPIA_RT_TYPE_MASK)>>DSIU_UTOPIA_RT_TYPE_SHIFT)
#define SIU_UTOPIA_RT_SET_TYPE(x,y) \
   ((x)->type_dest_src)=(((x)->type_dest_src)&~DSIU_UTOPIA_RT_TYPE_MASK)|\
                        ((y)<<DSIU_UTOPIA_RT_TYPE_SHIFT)
#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_RT_CORE_ID_MASK    0x1C0
#define DSIU_UTOPIA_RT_CORE_ID_SHIFT       6
#else
#define DSIU_UTOPIA_RT_CORE_ID_MASK    0x700
#define DSIU_UTOPIA_RT_CORE_ID_SHIFT       8
#endif
#define DSIU_UTOPIA_RT_EXTRACT_CORE_ID(x) \
    ((((x)->janus_core_channel)&DSIU_UTOPIA_RT_CORE_ID_MASK)>>\
        DSIU_UTOPIA_RT_CORE_ID_SHIFT)
#define DSIU_UTOPIA_RT_SET_CORE_ID(x,y) \
   ((x)->janus_core_channel)= \
     (((x)->janus_core_channel)&~DSIU_UTOPIA_RT_CORE_ID_MASK)|\
     ((y)<<DSIU_UTOPIA_RT_CORE_ID_SHIFT)

/* Macro to extract the janus ID */
#define DSIU_UTOPIA_RT_JANUS_ID_MASK  0xF800
#define DSIU_UTOPIA_RT_JANUS_ID_SHIFT  11
#define DSIU_UTOPIA_RT_EXTRACT_JANUS_ID(x) \
   ((((x)->janus_core_channel)&DSIU_UTOPIA_RT_JANUS_ID_MASK)>>\
    DSIU_UTOPIA_RT_JANUS_ID_SHIFT)


#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_RT_CHANNEL_MASK    0x003F
#define DSIU_UTOPIA_RT_CHANNEL_SHIFT        0
#else
#define DSIU_UTOPIA_RT_CHANNEL_MASK    0x00FF
#define DSIU_UTOPIA_RT_CHANNEL_SHIFT        0
#endif

#define SIU_UTOPIA_RT_SET_CHANNEL(x,y) \
   ((x)->janus_core_channel)= \
     (((x)->janus_core_channel)&~DSIU_UTOPIA_RT_CHANNEL_MASK)|\
     ((y)<<DSIU_UTOPIA_RT_CHANNEL_SHIFT)
/* Values for the type field, names must match application code */
#define SIU_UTOPIA_RT_TYPE_NORMAL_MSG   29
#define SIU_UTOPIA_RT_TYPE_HIGH_MSG     30
#define SIU_UTOPIA_RT_TYPE_LOW_MSG      31
#define SIU_UTOPIA_RT_TYPE_AAL1          6
#define SIU_UTOPIA_RT_TYPE_ANNOUNCE      4

/* Values for the destination phy not provided by the host at boot */
#define SIU_UTOPIA_RT_DEST_PHY_PSOP   29        /* packet switch */
#define SIU_UTOPIA_RT_DEST_PHY_MISC   29        /* keep alives   */

/* Macros for extraction of the channel field */
#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_RT_CHANNEL_MASK    0x003F
#define DSIU_UTOPIA_RT_CHANNEL_SHIFT        0
#else

#define DSIU_UTOPIA_RT_CHANNEL_MASK    0x00FF
#define DSIU_UTOPIA_RT_CHANNEL_SHIFT        0
#endif

#define SIU_UTOPIA_RT_EXTRACT_CHANNEL(x)  \
  ((((x)->janus_core_channel)&DSIU_UTOPIA_RT_CHANNEL_MASK)>>\
                              DSIU_UTOPIA_RT_CHANNEL_SHIFT)

/* Macros for extraction and setting of the destination phy */
#define DSIU_UTOPIA_RT_DEST_PHY_MASK  0x07C0
#define DSIU_UTOPIA_RT_DEST_PHY_SHIFT      6
#define DSIU_UTOPIA_RT_EXTRACT_DEST_PHY(x) \
  ((((x)->type_dest_src)&DSIU_UTOPIA_RT_DEST_PHY_MASK)>>\
                         DSIU_UTOPIA_RT_DEST_PHY_SHIFT)
#define DSIU_UTOPIA_RT_SET_DEST_PHY(x,y) \
  ((x)->type_dest_src) = \
    (((x)->type_dest_src)&~DSIU_UTOPIA_RT_DEST_PHY_MASK)|\
    ((y)<<DSIU_UTOPIA_RT_DEST_PHY_SHIFT)

/* Macros for extraction and setting of the atm header vci channel (6 bits) */
#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_RT_VCI_CH_MASK  0x03F0
#define DSIU_UTOPIA_RT_VCI_CH_SHIFT 4
#else
#define DSIU_UTOPIA_RT_VCI_CH_MASK  0x0FF0
#define DSIU_UTOPIA_RT_VCI_CH_SHIFT 4
#endif
#define DSIU_UTOPIA_RT_VCI_SET_VCI_CH(x,y) \
  ((x)->vci_pt) = \
   (((x)->vci_pt)&~DSIU_UTOPIA_RT_VCI_CH_MASK)|\
   ((y)<<DSIU_UTOPIA_RT_VCI_CH_SHIFT)

/* Macros for setting the atm header vci core */
#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_RT_VCI_CORE_MASK 0x1C00
#define DSIU_UTOPIA_RT_VCI_CORE_SHIFT 10
#else
#define DSIU_UTOPIA_RT_VCI_CORE_MASK 0x7000
#define DSIU_UTOPIA_RT_VCI_CORE_SHIFT 12
#endif
#define DSIU_UTOPIA_RT_VCI_SET_VCI_CORE(x,y) \
  ((x)->vci_pt) = \
  (((x)->vci_pt)&~DSIU_UTOPIA_RT_VCI_CORE_MASK)|\
  ((y)<<DSIU_UTOPIA_RT_VCI_CORE_SHIFT)

/* Macros for setting the atm header vpi */
#define DSIU_UTOPIA_RT_VCI_VPI_MASK 0x0FF0
#define DSIU_UTOPIA_RT_VCI_VPI_SHIFT 4
#define DSIU_UTOPIA_RT_VCI_SET_VCI_VPI(x,y) \
  ((x)->gfc_vpi_vci) = \
  (((x)->gfc_vpi_vci)&~DSIU_UTOPIA_RT_VCI_VPI_MASK)|\
  ((y)<<DSIU_UTOPIA_RT_VCI_VPI_SHIFT)


/*****************************************************************************
 * DEFINITION: VC demux mode structure
 *****************************************************************************/
typedef struct {
  tuint dsp_id;              /* DSP ID                        */
  tuint prior_core_ch;       /* Priority, core id, channel id */
  tuint msg_id;              /* Message ID                    */
  tuint trans_id;            /* Transaction ID                */
  tuint msg_first_word;      /* First word of message         */
} dsiuVcDemuxMsgPayload_t;
#define tsiuVcDemuxMsgPayload_t dsiuVcDemuxMsgPayload_t
#define SIU_VC_DEMUX_MSG_HEADER_LENGTH   4
#define DSIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_MASK  0xF800
#define DSIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_SHIFT 11
#define DSIU_UTOPIA_VC_DEMUX_MSG_EXTRACT_PRIORITY(x) \
  ((((x)->prior_core_ch) & DSIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_MASK) >> \
  DSIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_SHIFT)
#define SIU_UTOPIA_VC_DEMUX_MSG_SET_PRIORITY(x,y) \
  ((x)->prior_core_ch) = \
  ((((x)->prior_core_ch) & ~DSIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_MASK) | \
  ((y) << DSIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_SHIFT))
/* Values for the type field, names must match application code */
#define SIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_NORMAL SIU_UTOPIA_RT_TYPE_NORMAL_MSG
#define SIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_HIGH   SIU_UTOPIA_RT_TYPE_HIGH_MSG
#define SIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_LOW    SIU_UTOPIA_RT_TYPE_LOW_MSG

#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_VC_DEMUX_MSG_COREID_MASK  0x01E0
#define DSIU_UTOPIA_VC_DEMUX_MSG_COREID_SHIFT 6
#elif defined( _TMS320C6X )
#define DSIU_UTOPIA_VC_DEMUX_MSG_COREID_MASK  0x0700
#define DSIU_UTOPIA_VC_DEMUX_MSG_COREID_SHIFT 8
#endif

#define SIU_UTOPIA_VC_DEMUX_MSG_SET_COREID(x,y) \
  ((x)->prior_core_ch) = \
  ((((x)->prior_core_ch) & ~DSIU_UTOPIA_VC_DEMUX_MSG_COREID_MASK) | \
  ((y) << DSIU_UTOPIA_VC_DEMUX_MSG_COREID_SHIFT))


#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_VC_DEMUX_MSG_CHANNEL_MASK 0x003F
#define DSIU_UTOPIA_VC_DEMUX_MSG_CHANNEL_SHIFT 0
#elif defined( _TMS320C6X )
#define DSIU_UTOPIA_VC_DEMUX_MSG_CHANNEL_MASK 0x00FF
#define DSIU_UTOPIA_VC_DEMUX_MSG_CHANNEL_SHIFT 0
#endif

#define DSIU_UTOPIA_VC_DEMUX_MSG_EXTRACT_CHANNEL(x) \
  ((((x)->prior_core_ch) & DSIU_UTOPIA_VC_DEMUX_MSG_CHANNEL_MASK) >> \
  DSIU_UTOPIA_VC_DEMUX_MSG_CHANNEL_SHIFT)
  
  
/* ATM header format for 54 byte cells (16 bit utopia or 8 bit with 1 byte pad) */ 
  
typedef struct {
  tuint gfc_vpi_vci;          /* GFC, VPI and VCI fields                   */
  tuint vci_core_channel_pt;  /* VCI, VCI core, VCI channel and PT fields  */
  tuint hec_dummy;            /* HEC and dummy fields                      */
  tuint payload_first_word;   /* 1st word of payload                       */



} dsiuVcDemuxHeader_t;

#define tsiuVcDemuxHeader_t dsiuVcDemuxHeader_t
#define DSIU_UTOPIA_VC_HEADER_SIZE_WORDS   3 /* Size less first word  */

/* Macros for extraction of the vci channel field */
#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_VC_VCI_CHANNEL_MASK    0x03F0
#define DSIU_UTOPIA_VC_VCI_CHANNEL_SHIFT        4
#else
#define DSIU_UTOPIA_VC_VCI_CHANNEL_MASK    0x0FF0
#define DSIU_UTOPIA_VC_VCI_CHANNEL_SHIFT        4
#endif
/* NOTE THAT THE VCI MACROS CAN ONLY HANDLE 12 OF THE 16 BITS */
#define DSIU_UTOPIA_VC_EXTRACT_VCI_CHANNEL(x) \
  ((((x)->vci_core_channel_pt)&DSIU_UTOPIA_VC_VCI_CHANNEL_MASK)>>\
                               DSIU_UTOPIA_VC_VCI_CHANNEL_SHIFT)
#define DSIU_UTOPIA_VC_SET_VCI_CHANNEL(x,y) \
   ((x)->vci_core_channel_pt)= \
      (((x)->vci_core_channel_pt)&~DSIU_UTOPIA_VC_VCI_CHANNEL_MASK)|\
      ((y)<<DSIU_UTOPIA_VC_VCI_CHANNEL_SHIFT)


/* Macros for setting the vci core */
#if defined( __TMS320C55X__ )
#define DSIU_UTOPIA_VC_VCI_CORE_MASK   0x1C00
#define DSIU_UTOPIA_VC_VCI_CORE_SHIFT      10
#else
#define DSIU_UTOPIA_VC_VCI_CORE_MASK   0x7000
#define DSIU_UTOPIA_VC_VCI_CORE_SHIFT      12
#endif

#define DSIU_UTOPIA_VC_SET_VCI_CORE(x,y) \
   ((x)->vci_core_channel_pt)= \
      (((x)->vci_core_channel_pt)&~DSIU_UTOPIA_VC_VCI_CORE_MASK)|\
      ((y)<<DSIU_UTOPIA_VC_VCI_CORE_SHIFT)


/* Macros for setting the VPI field */
#define DSIU_UTOPIA_VC_VCI_VPI_MASK   0x0FF0
#define DSIU_UTOPIA_VC_VCI_VPI_SHIFT       4
#define DSIU_UTOPIA_VC_SET_VPI(x,y) \
   ((x)->gfc_vpi_vci)= \
      (((x)->gfc_vpi_vci)&~DSIU_UTOPIA_VC_VCI_VPI_MASK)|\
        ((y)<<DSIU_UTOPIA_VC_VCI_VPI_SHIFT)
 
  
  

/****************************************************************************/
/*  structure for 53-byte Cell for 8bit utopia                              */
/****************************************************************************/
typedef struct {
  tuint dummy_gfc_vpi;           /* dummy-8 GFC-4, VPI-4                      */
  tuint vpi_vci_core_ch_msb2; /* VPI-4 VCI-6, VCI_core-4, VCI_channel-2    */
  tuint vci_lsb4_pt_clp_hec;     /* VCI channel-4 and PT-3, CLP-1 fields HEC-8*/
  tuint payload_first_word;      /* 1st word of payload                       */
} dsiuVc53DemuxHeader_t;
#define tsiuVc53DemuxHeader_t dsiuVc53DemuxHeader_t

#define tsiuVc53DemuxHeader_t dsiuVc53DemuxHeader_t
#define DSIU_UTOPIA_VC53_HEADER_SIZE_WORDS   3 /* Size less first word  */

/* Macros for extraction of the vci channel field */
/*#define DSIU_UTOPIA_VC_VCI_CHANNEL_MASK    0x03F0
#define DSIU_UTOPIA_VC_VCI_CHANNEL_SHIFT        4
*/
/* NOTE THAT THE VCI MACROS CAN ONLY HANDLE 12(??9) OF THE 16 BITS */

#define DSIU_UTOPIA_VC53_SET_VCI_CHANNEL(x,y) dsiu_utopia_vc53_set_vci_ch(x,y)
static inline void dsiu_utopia_vc53_set_vci_ch(dsiuVc53DemuxHeader_t *x, tint y)
{
  unsigned char j;
  unsigned char inp;
  
  inp = y & 0xff;

  j = (utlRead8bits_m((word *)(&((x)->vpi_vci_core_ch_msb2)), 1) &~(0x3))| ((inp&0x30)>>4);  
  utlWrite8bits_m((word *)(&((x)->vpi_vci_core_ch_msb2)), 1, j);

  j = (utlRead8bits_m((word *)(&((x)->vci_lsb4_pt_clp_hec)), 0) &~(0xF0))| ((inp&0x0F)<<4);  
  utlWrite8bits_m((word *)(&((x)->vci_lsb4_pt_clp_hec)), 0, j);
}
 
#define DSIU_UTOPIA_VC53_EXTRACT_VCI_CHANNEL(x) dsiu_utopia_vc53_extract_vci_ch(x)
static inline tuint dsiu_utopia_vc53_extract_vci_ch(dsiuVc53DemuxHeader_t *x)
{
  unsigned char j, k;
  j = utlRead8bits_m((word *)(&((x)->vpi_vci_core_ch_msb2)), 1) &(0x3);  
  k = utlRead8bits_m((word *)(&((x)->vci_lsb4_pt_clp_hec)), 0) &(0xF0);  
  k =  (j<<0x4) | (k>>0x4);
  return (k);
}
      
/* Macros for setting the vci core */
#define DSIU_UTOPIA_VC53_VCI_CORE_MASK   0x001C
#define DSIU_UTOPIA_VC53_VCI_CORE_SHIFT      10
#define DSIU_UTOPIA_VC53_SET_VCI_CORE(x,y) dsiu_utopia_vc53_set_vci_core(x, y)

static inline void dsiu_utopia_vc53_set_vci_core(dsiuVc53DemuxHeader_t *x, tuint y)
{
  unsigned char j;
  unsigned char inp;
  inp = y & 0xff;

  j = (utlRead8bits_m((word *)(&((x)->vpi_vci_core_ch_msb2)), 1) &~(0x1C))| ((inp&0x07)<<2);  
  utlWrite8bits_m((word *)(&((x)->vpi_vci_core_ch_msb2)), 1, j);
}


/* Macros for setting the VPI field */
#define DSIU_UTOPIA_VC53_VCI_VPI_MASK   0x0FF0
#define DSIU_UTOPIA_VC53_VCI_VPI_SHIFT       4
#define DSIU_UTOPIA_VC53_SET_VPI(x,y) dsiu_utopia_vc53_set_vpi(x, y)
static inline void dsiu_utopia_vc53_set_vpi(dsiuVc53DemuxHeader_t *x, tint y)
{
  unsigned char j;
  unsigned char inp;
    
  inp = y & 0xff;

  j = (utlRead8bits_m((word *)(&((x)->dummy_gfc_vpi)), 1) &~(0xF))| ((inp&0xF0)>>4);  
  utlWrite8bits_m((word *)(&((x)->dummy_gfc_vpi)), 1, j);

  j = (utlRead8bits_m((word *)(&((x)->vpi_vci_core_ch_msb2)), 0) &~(0xF0))| ((inp&0x0F)<<(0x4)); 
  utlWrite8bits_m((word *)(&((x)->vpi_vci_core_ch_msb2)), 0, j);
}



/* Valid values for the vci channel field */
#define DSIU_UTOPIA_VC_MSG_HIGH_CHANNEL   2   /* High priority msg channel   */
#define DSIU_UTOPIA_VC_MSG_CHANNEL        3   /* Normal priority msg channel */
#define DSIU_UTOPIA_VC_MSG_LOW_CHANNEL    4   /* Low priority msg channel    */
#define SIU_UTOPIA_VC_ANU_CHANNEL         5   /* Announcement channel */


/*****************************************************************************
 * DEFINITION: The payload size for cells is always 48 bytes
 *****************************************************************************/
#define DSIU_UTOPIA_PAYLOAD_SIZE_WORDS  48

/*****************************************************************************
 * Function prototypes 
 *****************************************************************************/
dsiuReturn_t dsiuUtopiaTxCell (dsiuInst_t *inst, globData_t *data);
dsiuReturn_t dsiuUtopiaRxCell (dsiuCell_t* cell);
dsiuReturn_t dsiuUtopiaSendMessageToHost (dmsg_t *msg);
void dsiuUtopiaCellRcvd (tuint utopiaChannelId, word *cell, tuint size);
void dsiuUtopiaInit (void);

/*****************************************************************************
 * Utopia System Call table definition
 *****************************************************************************/
/* Utopia */
typedef struct {
  dsiuReturn_t (*sendPacket) (dsiuInst_t *, globData_t *);
  dsiuReturn_t (*rxCell)     (dsiuCell_t *);  /* Foreground cell processing */
  dsiuReturn_t (*sendMsg)    (dmsg_t *);
  void         (*cellRcvd)   (tuint, word *, tuint); /* ISR cell received fcn */
  void         (*utopiaInit) (void);          /* Message route initialization */
} dsiuUtopiaCallTable_t;

extern dsiuUtopiaCallTable_t dsiuUtopiaCallTable ;

#endif /* _DSIUUTOP_H */


