#ifndef _SIUNDU_H
#define _SIUNDU_H
/******************************************************************************
 * FILE PURPOSE: SIU NDU (Network Driver Unit) interface description
 ******************************************************************************
 * FILE NAME: siundu.h
 *
 * DESCRIPTION: Defines the call table used for the ndu
 *
 * TABS: NONE
 *
 * (C) Copyright 2001, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "utlxferpkt.h"

/* siu ndu Call Table */
typedef struct {
  void (*nduOpen)         (tint, tsiuNduConfig_t *);
  void (*nduClose)        (tsiuInst_t *);
  tint (*nduSendIn)       (void *, xferPktInfo_t *);
  void (*nduReceiveIn)    (void);
  void (*nduReceiveInInt) (void *, word *, tuint);
  tint (*nduControl)      (tint);

  void (*nduGetNormalMessage)       (void);
  void (*nduGetInterruptMessage)    (void);
  tint (*nduWriteMessageToDPR)      (mgb_t *, bool, siuDPRMsgType_t);
  void (*nduMSG_test)               (void);
  void (*nduHeartBeat)              (tint);
  tuint (*nduGetClockMul)           (void);
  void (*nduMsgInit)                (void);
  tint (*nduPushFifo)               (void *);
  tint (*nduFifo2DPR)               (tint);
} siuNduCallTable_t;

/* The diag code creates the ndu call table in ndu */
#ifdef DSIU
extern siuNduCallTable_t siuUtopiaNduCallTable; /* For utopia */
extern siuNduCallTable_t siuHpiNduCallTable;    /* For HPI    */
extern siuNduCallTable_t siuGmacNduCallTable;   /* For GMAC   */
#endif

/**************************************************************************
 * Control codes used for call to NDU control 
 **************************************************************************/
#define SIU_NDU_CTRL_INIT       0       /* Initialize NDU        */
#define SIU_NDU_CTRL_VOID       1       /* Exit with no changes  */
#define SIU_NDU_CTRL_MSG_ROUTE  2       /* Reset message routing */

#endif /* _SIUNDU_H */

