#ifndef _DTGU_H
#define _DTGU_H
/****************************************************************************
 * FILE PURPOSE: Defines an API for Diagnostic Tone Generation (DTGU)
 ****************************************************************************
 * FILE NAME: dtgu.h
 *
 * DESCRIPTION: Defines the API used to interact with the DTGU module
 *
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dglob.h"

/*****************************************************************************
 * DEFINITION: Return values from DTGU
 ****************************************************************************/
typedef enum {
  DTGU_SUCCESS = 0,
  DTGU_ERR_STATE_CLOSED,
  DTGU_ERR_INVALID_OPEN,
  DTGU_ERR_OPEN_INVALID_COMPAND
} dtguReturn_t;

/****************************************************************************
 * Definition: DTGU size configuration structure. Currently no information
 *             is required from the system to configure DTGU
 ****************************************************************************/
typedef struct {
  tint dummy;
} dtguSizeConfig_t;

/****************************************************************************
 * Definition: DTGU new configuration structure.
 ****************************************************************************/
typedef struct {
  tint Id;
  dtguSizeConfig_t sizeCfg;
} dtguNewConfig_t;

/****************************************************************************
 * Definition: Companding types used for tone generation
 ****************************************************************************/
typedef enum {
  DTGU_COMPAND_ALAW,
  DTGU_COMPAND_ULAW,
  DTGU_COMPAND_LINEAR
} dtguCompand_t;

/*****************************************************************************
 * Definition: DTGU open configuration structure
 ****************************************************************************/
typedef struct {
  dtguCompand_t compandLaw;
} dtguOpenConfig_t;
 
 
/***************************************************************************
 * Prototypes: Initialization and setup of DTGU
 ***************************************************************************/
dtguReturn_t dtguGetSizes (tint *nbufs, dmemBuffer_t **bufs,
						   dtguSizeConfig_t *cfg);
void *dtguNew (dtguNewConfig_t *cfg, dmemBuffer_t *bufs);
dtguReturn_t dtguOpen (void *dtguInst, dtguOpenConfig_t *cfg);

/****************************************************************************
 * Prototype: DTGU Tone generation. Replaces input data with a tone
 ****************************************************************************/
dtguReturn_t dtguGenTone (void *dtguInst, globData_t *data);

/****************************************************************************
 * Prototype: DTGU Pattern generation. Replaces input data with a pattern.
 ****************************************************************************/
dtguReturn_t dtguGenPattern (void *dtguInst, globData_t *data);

/*****************************************************************************
 * DTGU API call table. The dtgu API is referenced by the system through a
 * pointer to this call table.
 *****************************************************************************/
typedef struct {
  dtguReturn_t (*dtguGetSizes) (tint *nbufs, dmemBuffer_t **bufs,
						   dtguSizeConfig_t *cfg);
  void *(*dtguNew) (dtguNewConfig_t *cfg, dmemBuffer_t *bufs);
  dtguReturn_t (*dtguOpen) (void *dtguInst, dtguOpenConfig_t *cfg);
  dtguReturn_t (*dtguGenTone)(void *, globData_t *);
  dtguReturn_t (*dtguGenPattern)(void *, globData_t *);
} dtguCallTable_t;
extern dtguCallTable_t dtguCallTable;



#endif /* _DTGU_H */



