;*****************************************************************************
;* FILE PURPOSE: Initialises the C environment for c6x
;*****************************************************************************
;* FILE NAME:   boot.s
;*
;* DESCRIPTION: Called at reset and is used to initialize the C environment
;*              
;*
;* TABS: NONE
;*
;* REVISION HISTORY:
;*   Derived from rts.src of codegen tools version 4.20.Modified to initialize      
;*   bss and far to zero before cinit happens
;* 
;*    Initial revision
;*
;* (C) Copyright 2002, Texas Instruments Incorporated.            *
; * 
; *  Redistribution and use in source and binary forms, with or without 
; *  modification, are permitted provided that the following conditions 
; *  are met:
; *
; *    Redistributions of source code must retain the above copyright 
; *    notice, this list of conditions and the following disclaimer.
; *
; *    Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the 
; *    documentation and/or other materials provided with the   
; *    distribution.
; *
; *    Neither the name of Texas Instruments Incorporated nor the names of
; *    its contributors may be used to endorse or promote products derived
; *    from this software without specific prior written permission.
; *
; *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; */

;*****************************************************************************
            .asg	A15, FP
	        .asg	B14, DP
	        .asg	B15, SP
	        
; ************************************************************************************************
; * One conditional to allow the rest of the code to be ABI agnostic
; ************************************************************************************************
            .if __TI_EABI__
            .asg      __TI_CINIT_Base, cinit
            .asg      __TI_STACK_SIZE, __STACK_SIZE
            .asg      main,           _main
            .asg      exit,           _exit
            .asg      _auto_init_elf, __auto_init
            .asg      dregBootTrack,  _dregBootTrack
            .endif

            .ref      cinit 
            .ref	  $bss
            .ref	  __bss_core_begin
            .ref	  __bss_core_end
            .ref      __STACK_SIZE
            .ref      _main
            .ref      _exit
            .ref      __auto_init
            .ref      _dregBootTrack
             
            .def      __stack
            .def	  _c_int00

__stack:    .usect   .stack, 0, 8
	        .sect	  ".text"

_c_int00:

; Disable interrupts

			DINT
			MVKL.S2		0, B0
			MVC.S2		B0, IER
			MVKL.S2     0xffffffff, B0
			MVKH.S2     0xffffffff, B0
			MVC.S2		B0, ICR


;*************************************************************************;
;*********************Initialize Stack pointer****************************;
;*************************************************************************;
            MVKL.S2      __stack,SP
            MVKH.S2      __stack,SP
            MVKL.S2      __STACK_SIZE - 4,B0
            MVKH.S2      __STACK_SIZE - 4,B0
            ADD.D2       B0,SP,SP
            AND.D2       ~7,SP,SP
;------------------------------------------------------------------------
; Boot progress tracker - uses data memory word address 0x100 
;   First track value must be done after register initialization
;   The track value is a double word
;------------------------------------------------------------------------
            MVKL.S1      _dregBootTrack,A6
            MVKH.S1      _dregBootTrack,A6
            MVK.L2       0,B6
            STW.D1T2     B6,*A6[0]


;*************************************************************************;
;*********************Initialize BSS to 0*********************************;
;*************************************************************************;
            MVKL.S1      __bss_core_begin,A6
            MVKH.S1      __bss_core_begin,A6
            MVKL.S2      __bss_core_end,B0
            MVKH.S2      __bss_core_end,B0
            SUB.D2X      B0,A6,B0
            MVK.L2       0,B6
zero_loop$1:
    [ B0]  B.S2          zero_loop$1
    [ B0]  STB.D1T2      B6,*A6++[0x1]
    [ B0]  ADDK.S2       -1,B0
    [ B0]  STB.D1T2      B6,*A6++[0x1]
    [ B0]  ADDK.S2       -1,B0
           NOP           

;------------------------------------------------------------------------
; Boot progress tracker - uses data memory word address 0x100 
;   Second track value must be done after bss and far initialization
;   The track value is a double word
;------------------------------------------------------------------------
            MVKL.S1      _dregBootTrack,A6
            MVKH.S1      _dregBootTrack,A6
			MVK.L2       1,B6
			STW.D1T2     B6,*A6[0]


;*************************************************************************;
;*********************Call Autoinit to initialize*************************;
;*************************************************************************;
           MVKL.S2       $bss,DP
           MVKH.S2       $bss,DP
           MVKL.S1       cinit,A4
           MVKH.S1       cinit,A4
           OR.D2         0,DP,B4
           MVKL.S2       $aiRL,B3
           MVKH.S2       $aiRL,B3
           MVKL.S2       __auto_init,B0
           MVKH.S2       __auto_init,B0
           B.S2          B0
           NOP           5
;*************************************************************************;
;**************************Go Main****************************************;
;*************************************************************************;
$aiRL:

;------------------------------------------------------------------------
; Boot progress tracker - uses data memory word address 0x100 
;   Third track value must be done after autoinit is done
;   The track value is a 32-bit word 
;------------------------------------------------------------------------
           MVKL.S1      _dregBootTrack,A6
           MVKH.S1      _dregBootTrack,A6
           MVK.L2       2,B6
           STW.D1T2     B6,*A6[0]

           MVKL.S2       _main,B4
           MVKH.S2       _main,B4
           B.S2          B4
           ADDKPC.S2     RL0,B3,4
;*************************************************************************;
;**************************Call Exit**************************************;
;*************************************************************************;
RL0:       ; CALL OCCURS
;------------------------------------------------------------------------
; Boot progress tracker - uses data memory word address 0x100 
; Fourth track value of 0x00000003 indicates main has returned - this
; should never happen
;------------------------------------------------------------------------
           MVKL.S1      _dregBootTrack,A6
           MVKH.S1      _dregBootTrack,A6
           MVK.L2       3,B6
           STW.D1T2     B6,*A6[0]
                     
           MVKL    .S1   _exit,A3          
           MVKH    .S1   _exit,A3 
           MVK     .D1   0x1,A4   
           B       .S2X  A3       
           ADDKPC  .S2   RL2,B3,4 
RL2: ; CALL OCCURS              
           B       .S2   IRP  
           LDW     .D2T1 *++SP(0),A0
           NOP           4
     ;BRANCH OCCURS            


  	        .sect ".boot_redir"
			.def   _boot_redir

_boot_redir:
		mvkl	_c_int00, b0
		mvkh	_c_int00, b0
		bnop	b0, 5
		nop






