; File Header Begins
;===============================================================================
;   FILE PURPOSE : Assembly functions for target hardware Critical functions.
;-------------------------------------------------------------------------------
;   File Name    : thwcritsec.s
;   DESCRIPTION  : Assembly functions for the C64x
;===============================================================================
;   Version Number          Revision Date          Author's Name
;   ==============          =============          =============
;
;   Initial Revision        June 2002              Ravi sankar K
;
;===============================================================================
; List of functions in the file :   _thwCriticalBegin
;                                   _thwCriticalEnd
;-------------------------------------------------------------------------------
; (C) Copyright 2002 Texas Instruments Incorporated.            *
; * 
; *  Redistribution and use in source and binary forms, with or without 
; *  modification, are permitted provided that the following conditions 
; *  are met:
; *
; *    Redistributions of source code must retain the above copyright 
; *    notice, this list of conditions and the following disclaimer.
; *
; *    Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the 
; *    documentation and/or other materials provided with the   
; *    distribution.
; *
; *    Neither the name of Texas Instruments Incorporated nor the names of
; *    its contributors may be used to endorse or promote products derived
; *    from this software without specific prior written permission.
; *
; *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; */
;===============================================================================
; File Header Ends 

            .text

          .if __TI_EABI__
            ; Alias so COFF and EABI can share same code
            .asg      thwCriticalStatusSaved, _thwCriticalStatusSaved
          .endif

            .ref      _thwCriticalStatusSaved

; Function Header Begins
;-------------------------------------------------------------------------------
;   Function Name            : _thwCriticalBegin
;
;   Functional Description
;   ======================   
;       Start of critical section (up to 15 sections can be nested).
;       Save current interrupt enable/disable status, and disable all
;           interrupts. Bit INTM of CSR is pushed in global variable.
;
;   Function Call
;   =============
;       void thwCriticalBegin (void)
;
;-------------------------------------------------------------------------------
; Function Header Ends

          .if __TI_EABI__
            ; Alias so COFF and EABI can share same code
            .asg      thwCriticalBegin, _thwCriticalBegin
          .endif
            .global      _thwCriticalBegin

_thwCriticalBegin:

        MVC     .S2   CSR, B2
	||  MVKL    .S1   _thwCriticalStatusSaved, A2

        AND     .S2   B2,-2, B1        ; Clear GIE
	||  MVKH    .S1   _thwCriticalStatusSaved, A2

        MVC     .S2   B1, CSR          ; Always Disable interrupt on exit
	||  LDH     .D1   *A2, A16         ; A16 = Saved Status

        AND     .S2   B2,1, B1         ;B1=0, if Interrupts are already disabled.
		B       .S2   B3           
		NOP     2
	    SHL     .S1   A16, 1, A16      ;Create Room for the present status

 [!B1]  OR      .S1   A16, 1, A16	   ;If B1=0, Interrupts were already disabled.
        STH     .D1   A16, *A2         ; Store the Status.

;Return to calling routine



; Function Header Begins
;-------------------------------------------------------------------------------
;   Function Name            : _thwCriticalEnd
;
;   Functional Description
;   ======================   
;       End of critical section (up to 15 sections can be nested).
;       Restore interrupt enable/disable status encountered on entry to
;           critical section. Bit INTM of CSR is popped from the global variable.
;
;   Function Call
;   =============
;       void thwCriticalEnd (void)
;
;-------------------------------------------------------------------------------
; Function Header Ends

          .if __TI_EABI__
            ; Alias so COFF and EABI can share same code
            .asg      thwCriticalEnd, _thwCriticalEnd
          .endif
            .def      _thwCriticalEnd

_thwCriticalEnd:

	    MVKL    .S1   _thwCriticalStatusSaved, A2
	    MVKH    .S1   _thwCriticalStatusSaved, A2
	    LDH     .D1   *A2, A16        ; A16 = Saved Status
        MVC     .S2   CSR, B1          
	    OR      .S2   B1, 1, B1       ; Enable interrupt if needed
		NOP      
		B       .S2   B3      
		NOP
        AND     .S1   A16, 1, A1       ;If A1=0, Need to enable interrupts on return
        SHR     .S1   A16, 1, A16

  [!A1] MVC     .S2   B1, CSR          ; If A1=0, Need to enable interrupts on return

        STH     .D1   A16, *A2         ; Store the Status.
;Return to calling routine


          .if __TI_EABI__
            ; Alias so COFF and EABI can share same code
            .asg      boot_redir, _boot_redir
          .endif

  	        .sect ".boot_redir"
			.def   _boot_redir
			.ref   _c_int00

_boot_redir:
		mvkl	_c_int00, b0
		mvkh	_c_int00, b0
		bnop	b0, 5
		nop

; nothing past
