#ifndef _DGLOBAL_H
#define _DGLOBAL_H
/******************************************************************************
 * FILE PURPOSE: Definitions global to the application
 ******************************************************************************
 * FILE NAME: dglobal.h
 *
 * DESCRIPTION: Global definitions are provided. These should be used 
 *              sparingly, and essentially only for generic cross-module
 *              communication.
 *
 * TABS: None
 *
 * Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"

/*****************************************************************************
 * DEFINITION: Data passing structure.
 *****************************************************************************/
typedef struct {
  tuint dataSize;    /* Number of elements in data array */
  void *data;        /* The data                         */
  tuint ts;          /* Timestamp of 1st sample          */
} globData_t;

/*****************************************************************************
 * DEFINITION: Memory Allocation Structure
 *****************************************************************************/
typedef struct {
  tint  log2align;
  tuint size; 
  void *base;
} dmemBuffer_t;

typedef struct itdmFifoEntry_s {
  tuint chIdMapId;
  tuint tslotMapId;
  tuint numSamples;      /* Length of buffer in bytes = num of timeslots */
  bool  dataValid;       /* set this to true when data is filled into the buffer */
                         /* false when data is read from the buffer */
  word  *buf;
} itdmFifoEntry_t;

typedef struct ITDM_DATA_EXCHANGE_S {
  itdmFifoEntry_t *bufPtrs[8]; /* group of 8 buffers (1 msec) to do data exchange */ 
}ITDM_DATA_EXCHANGE_T;

/*****************************************************************************
 * Definition: ITDM Modes
 *****************************************************************************/
typedef enum {
  ITDM_MODE_1MILLISEC = 8,
  ITDM_MODE_125MICROSEC = 1
} itdmMode_t;

typedef enum {
  ITDM_ADD_TIMESLOT = 0,
  ITDM_REMOVE_TIMESLOT = 1,
  ITDM_RELOCATE_TIMESLOT = 2
} itdmChanUpdate_t;

typedef enum {
  ITDM_DIR_TO_TDM = 0,
  ITDM_DIR_FROM_TDM = 1
} itdmDirection_t;

typedef enum {
  ITDM_CHAN_UPDATE_DURING_SETUP = 0,
  ITDM_CHAN_UPDATE_ON_THE_FLY = 1
} itdmChanUpdateTime_t;

typedef enum {
  ITDM_MAP_ACTIVE = 0,
  ITDM_MAP_PENDING = 1
} itdmMapStatus_t;

/* Definition of multi-timeslot channel enables / disables */
typedef struct itdmMultiTsMap_s  {
  tint   mapId;         /* A handle provided to HAL and returned with data */
  tint   nTs;           /* The number of timeslots in the tsMap array      */
  tuint *tsMap;         /* The array of timeslots                          */
} itdmMultiTsMap_t;

#endif


