/*****************************************************************************
 * FILE PURPOSE: Linker Command file
 ****************************************************************************
 * FILE NAME: diaglnk.cmd
 *
 * DESCRIPTION: Linker memory placement
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* object Files */
dsiumain.obj
dsiuini.obj
dsiutask.obj
diagcfg.obj
boot.obj
bootv.obj
vectab.obj
rstvec.obj
dpiu.obj
dpiutxrx.obj
dvpu.obj
dvputxrx.obj
dtgu.obj
daal5.obj
aal5txrx.obj
crc32tbl.obj
utlrepck.obj
aal5crc.obj
util_s.obj
dsiumsg.obj
dsiuhdm.obj
dsiumtbl.obj
dsiuutop.obj

/* HAL object files */
tdmainit.obj
tdm_dma.obj
tdmaisr.obj
cslmcbsp.obj
csldma.obj
halinit.obj
hal.obj
cslisra.obj
cslisrb.obj
cslisrc.obj
cslisrd.obj
cslisrin.obj
cslisrsr.obj
cslintc.obj

/* Linker options */
-c
-a
-stack 0x100
-e start
-o diag.out
-m diag.map

MEMORY
{
  PAGE 0:
    VECTORS:  origin = 0x07F80, length = 0x0080
    PRG:      origin = 0x04400, length = 0x3B80
    RSTVEC:   origin = 0x0FF80, length = 0x0080

  PAGE 1:
    STACK:   origin = 0x080,  length = 0x0980
    REGIST:  origin = 0x1000, length = 0x0100
    BSS:     origin = 0x1100, length = 0x0200
    DATA:    origin = 0x1300, length = 0x0D00
    CINIT:   origin = 0x2000, length = 0x1000
    HEAP:    origin = 0x3000, length = 0x1000
	DMARX:	 origin = 0x4000, length = 0x0200
	DMATX:	 origin = 0x4200, length = 0x0200

	HALHEAP: origin = 0x8000, length = 0x0C00
}

/**************************************************************************
 * Unused memory check sections
 **************************************************************************/
__lcf_local_data_block1_start = 0;
__lcf_local_data_block1_end   = 0;

__lcf_local_data_block2_start = 0;
__lcf_local_data_block2_end   = 0;

__lcf_local_data_block3_start = 0;
__lcf_local_data_block3_end   = 0;

__lcf_local_data_block4_start = 0;
__lcf_local_data_block4_end   = 0;

__lcf_local_data_block5_start = 0;
__lcf_local_data_block5_end   = 0;

__lcf_local_prog_block1_start = 0;
__lcf_local_prog_block1_end   = 0;

__lcf_local_prog_block2_start = 0;
__lcf_local_prog_block2_end   = 0;

__lcf_local_prog_block3_start = 0;
__lcf_local_prog_block3_end   = 0;

__lcf_local_prog_block4_start = 0;
__lcf_local_prog_block4_end   = 0;

__lcf_local_prog_block5_start = 0;
__lcf_local_prog_block5_end   = 0;

__lcf_shared_prog_block1_start = 0;
__lcf_shared_prog_block1_end   = 0;
     
__lcf_shared_prog_block2_start = 0;
__lcf_shared_prog_block2_end   = 0;
     
__lcf_shared_prog_block3_start = 0;
__lcf_shared_prog_block3_end   = 0;
     
__lcf_shared_prog_block4_start = 0;
__lcf_shared_prog_block4_end   = 0;
     
__lcf_shared_prog_block5_start = 0;
__lcf_shared_prog_block5_end   = 0;

__lcf_shared_data_block1_start = 0;
__lcf_shared_data_block1_end   = 0;

__lcf_shared_data_block2_start = 0;
__lcf_shared_data_block2_end   = 0;

__lcf_shared_data_block3_start = 0;
__lcf_shared_data_block3_end   = 0;

__lcf_shared_data_block4_start = 0;
__lcf_shared_data_block4_end   = 0;

__lcf_shared_data_block5_start = 0;
__lcf_shared_data_block5_end   = 0;

     
SECTIONS
{
   /* Dummy sections are used to pass reloactable linker sections to the
    * diag program */
   .dummy1 type=DSECT  > HEAP
   {
     __lcf_heap_buf_start = .;
     . += 0x1000;
     __lcf_heap_buf_end = .;
   } PAGE 1

   .dummy2 type=DSECT  > REGIST
   {
     __lcf_config_addr = .;
   } PAGE 1
   
   .dummy3 type=DSECT > DMARX
   {
     __lcf_hal_tdm_rx_a_start = .;
	 . += 0x0200;
	 __lcf_hal_tdm_rx_a_end = .;
   } PAGE 1
   
   .dummy4 type=DSECT > DMATX
   {
     __lcf_hal_tdm_tx_a_start = .;
	 . += 0x0200;
	 __lcf_hal_tdm_tx_a_end = .;
   } PAGE 1
     
   .dummy5 type=DSECT > HALHEAP
   {
     __lcf_hal_heap_start = .;
	 . += 0x0C00;
    __lcf_hal_heap_end = .;
   } PAGE 1
   
   .switch  > PRG     PAGE 0
   .vectors > VECTORS PAGE 0
   .rstvec  > RSTVEC  PAGE 0
   .stack   > STACK   PAGE 1

   .text > PRG
   {
       *(.text)
   } PAGE 0

   .bss > BSS
   {
       __bss_core_begin = .;
       *(.bss)
       __bss_core_end = .;
   } PAGE 1

   .cinit > CINIT
   {
       __core_cinit = .;
       *(.cinit)
   } PAGE 1

   .data > DATA
   {
       *(.data)
   } PAGE 1
}