/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _GGPORT_H
#define _GGPORT_H
/******************************************************************************
 * FILE PURPOSE: GG Portability Definitions (C64x version)
 ******************************************************************************
 * FILE NAME:   ggport.h
 *
 * DESCRIPTION: Contains GG portability definitions.  Presently this file
 *              is used to make gg files common to both 54x and 55x and
 *              keep a port file to manage the changes.All Buffer lengths 
 *              should be in terms of 8-bit words for c64x
 * TABS: NONE
 ****************************************************************************/
#include "types.h"
#include <stdlib.h>

#define relocQualifier const

/*-------------------------------------------------------------------------*/
/* Alignment macro                                                         */
#ifndef align
#define align(Addr, Algnmt) (((Addr)+(Algnmt)-1)&(0xFFFFFFFFU-(Algnmt)+1))
#endif

#ifndef max
#define max(x,y) ((x) > (y) ? (x):(y))
#endif

/*-------------------------------------------------------------------------*/

#define GG_ECU_Y2X_DELAY_ALIGNMENT                          8
/******************************************************************************/
/* FUNCTION PURPOSE: Rounds the y2x_delay to next highest multiple of eight
 ******************************************************************************
 * DESCRIPTION: 64x implementation of ECU requires that the y2x delay be aligned
 *              to eight.This function transforms the measured y2x delay into
 *              the next highest multiple of eight           
 *****************************************************************************/
#define GG_ECU_Y2X_DELAY_ALIGN(y2x_delay_actual)           (((y2x_delay_actual \
                                                              + GG_ECU_Y2X_DELAY_ALIGNMENT - 1) \
                                                              / GG_ECU_Y2X_DELAY_ALIGNMENT) \
                                                              * GG_ECU_Y2X_DELAY_ALIGNMENT)


#ifdef XDAIS
  #define CPU_NUM_DEC_ADPCM_BUFF   2
  #define CPU_NUM_ENC_ADPCM_BUFF   2
#else
  #define CPU_NUM_DEC_ADPCM_BUFF   1
  #define CPU_NUM_ENC_ADPCM_BUFF   1
#endif

#define CPU_NUM_DEC_G722_BUFF    4
#define CPU_NUM_ENC_G722_BUFF    4
#define CPU_NUM_G722_BUFF        4 

#ifdef XDAIS
  #define CPU_NUM_DEC_G729AB_BUFF  11
  #define CPU_NUM_ENC_G729AB_BUFF  8
  #define CPU_NUM_G729AB_BUFF      11 
#else /* ECO Case */
  #define CPU_NUM_DEC_G729AB_BUFF  5
  #define CPU_NUM_ENC_G729AB_BUFF  5
  #define CPU_NUM_G729AB_BUFF      5 
#endif

#define CPU_G29AB_COM_RUN_START  0
#define CPU_G29AB_COM_LOAD_START 0 
#define CPU_G29AB_COM_LOAD_END   0

#ifdef XDAIS
  #define CPU_NUM_DEC_G723_BUFF    3
  #define CPU_NUM_ENC_G723_BUFF    5
  #define CPU_NUM_G723_BUFF        5
#else /* ECO Case */
  #define CPU_NUM_DEC_G723_BUFF    5
  #define CPU_NUM_ENC_G723_BUFF    7
  #define CPU_NUM_G723_BUFF        7
#endif


#define CPU_G723_COM_RUN_START   0
#define CPU_G723_COM_LOAD_START  0 
#define CPU_G723_COM_LOAD_END    0

#define CPU_NUM_DEC_G729E_BUFF   7
#define CPU_NUM_ENC_G729E_BUFF   11
#define CPU_NUM_G729E_BUFF       11 

#define CPU_G729E_COM_RUN_START  0
#define CPU_G729E_COM_LOAD_START 0 
#define CPU_G729E_COM_LOAD_END   0
#define CPU_G729E_ENC_RUN_START  0
#define CPU_G729E_ENC_LOAD_START 0 
#define CPU_G729E_ENC_LOAD_END   0
#define CPU_G729E_DEC_RUN_START  0
#define CPU_G729E_DEC_LOAD_START 0 
#define CPU_G729E_DEC_LOAD_END   0


#define CPU_NUM_AER_BUFF         10

#define CPU_NUM_DEC_G728_BUFF    6
#define CPU_NUM_ENC_G728_BUFF    6
#define CPU_NUM_G728_BUFF        6

#define CPU_G728_COM_RUN_START   0
#define CPU_G728_COM_LOAD_START  0 
#define CPU_G728_COM_LOAD_END    0
#define CPU_G728_ENC_RUN_START   0
#define CPU_G728_ENC_LOAD_START  0 
#define CPU_G728_ENC_LOAD_END    0
#define CPU_G728_DEC_RUN_START   0
#define CPU_G728_DEC_LOAD_START  0 
#define CPU_G728_DEC_LOAD_END    0

#define CPU_NUM_DEC_GSMEFR_BUFF  6
#define CPU_NUM_ENC_GSMEFR_BUFF  6
#define CPU_NUM_GSMEFR_BUFF      6

#define CPU_NUM_DEC_GSMFR_BUFF   3
#define CPU_NUM_ENC_GSMFR_BUFF   2
#define CPU_NUM_GSMFR_BUFF       3

#define CPU_NUM_DEC_QCELP13_BUFF  4	
#define CPU_NUM_ENC_QCELP13_BUFF  8	
#define CPU_NUM_QCELP13_BUFF      8	

#ifdef XDAIS
  #define CPU_NUM_DEC_ILBC_BUFF   3
  #define CPU_NUM_ENC_ILBC_BUFF   2
  #define CPU_NUM_ILBC_BUFF       3 
#else
  #define CPU_NUM_DEC_ILBC_BUFF   1
  #define CPU_NUM_ENC_ILBC_BUFF   1
  #define CPU_NUM_ILBC_BUFF       1 
#endif



#define CPU_NUM_DEC_MP3_BUFF    8 /* 6 + i/p buffer + o/p buffer*/

#ifdef XDAIS
  #define CPU_NUM_ENC_GSMAMR_BUFF  9
  #define CPU_NUM_DEC_GSMAMR_BUFF  4
  #define CPU_NUM_GSMAMR_BUFF      9
#else
  #define CPU_NUM_DEC_GSMAMR_BUFF  UNSPECIFIED
  #define CPU_NUM_ENC_GSMAMR_BUFF  UNSPECIFIED
  #define CPU_NUM_GSMAMR_BUFF      UNSPECIFIED
#endif

#define CPU_NUM_DEC_GSM_WBAMR_BUFF  5
#define CPU_NUM_ENC_GSM_WBAMR_BUFF  5
#define CPU_NUM_GSM_WBAMR_BUFF      5

#ifdef XDAIS
  #define CPU_NUM_DEC_EVRC_BUFF  3
  #define CPU_NUM_ENC_EVRC_BUFF  6
  #define CPU_NUM_EVRC_BUFF      6
#else
  #define CPU_NUM_DEC_EVRC_BUFF  UNSPECIFIED
  #define CPU_NUM_ENC_EVRC_BUFF  UNSPECIFIED
  #define CPU_NUM_EVRC_BUFF      UNSPECIFIED
#endif

#ifdef XDAIS
  #define CPU_NUM_DEC_SMV_BUFF  3
  #define CPU_NUM_ENC_SMV_BUFF  2
  #define CPU_NUM_SMV_BUFF      3
#else
  #define CPU_NUM_DEC_SMV_BUFF  UNSPECIFIED
  #define CPU_NUM_ENC_SMV_BUFF  UNSPECIFIED
  #define CPU_NUM_SMV_BUFF      UNSPECIFIED
#endif

/* NMU Buffers */
#ifdef G711A2
   #define CPU_NUM_TX_NMU_BUFF     2
   #define CPU_NUM_RX_NMU_BUFF     2
#else
   #define CPU_NUM_TX_NMU_BUFF     0 /* Only instance present and already accounted in ggmbuf.h */
   #define CPU_NUM_RX_NMU_BUFF     0 /* Only instance present and already accounted in ggmbuf.h */
#endif


/* PLC Buffers */
#ifdef G711A1
 #define CPU_NUM_PLC_BUFF        4
#else
 #define CPU_NUM_PLC_BUFF        0  /* Only instance present and already accounted in ggmbuf.h */
#endif

#define GG_ECU_MAX_TAIL_L        (GG_MAX_ECHO_TAIL_LENGTH * (GG_SAMPLING_RATE/1000))
#define GG_ECU_MAX_FILTER_L      (GG_MAX_ECHO_FILTER_SEG_LENGTH * (GG_SAMPLING_RATE/1000))     
#define GG_ECU_BLOCK_L           44

#define GG_ECU_FG_FILTER_L       GG_ECU_MAX_FILTER_L
#define GG_ECU_FG_FILTER_A       8

#define GG_ECU_BG_FILTER_L       GG_ECU_MAX_FILTER_L
#define GG_ECU_BG_FILTER_A       8

#define GG_ECU_SEARCH_L          GG_ECU_MAX_TAIL_L
#define GG_ECU_SEARCH_A          8 

#define GG_ECU_BG_ERR_L          GG_ECU_BLOCK_L    /* linSample */
#define GG_ECU_BG_ERR_A          8

/*
#define GG_ECU_RX_FIFO_L       ((IRAM_ECU_MAX_TAIL_L + GG_MAX_ECU_Y2X_DELAY +  \
                                 GG_MAX_VOICE_FRAME_LENGTH)/2)
#define GG_ECU_RX_FIFO_A       2
*/


#define GG_ECU_SRCH_WORK_L       256   /* max(tail/8*LFract, fltseg*Fract) */
#define GG_ECU_SRCH_WORK_A       8

#define GG_ECU_SCRATCH_FIFO_L    (1024 + 44) /* correspond to 128 ms tail at 5.5 ms */   
#define GG_ECU_SCRATCH_FIFO_A    8


#define GG_ECU_B0_L              656
#define GG_ECU_B0_A              8
#define GG_ECU_B1_L              512
#define GG_ECU_B1_A              8
#define GG_ECU_B2_L              512
#define GG_ECU_B2_A              8
#define GG_ECU_B3_L              (GG_ECU_BG_ERR_L * 2) /* Multiply by 2 to convert size in linSamples to bytes */
#define GG_ECU_B3_A              8
#define GG_ECU_B4_L              2224 /* (1024+8+80) Build Specific:1076 for 128ms ECU tail with DLCOMP ENabled2224 */
#define GG_ECU_B4_A              8
#define GG_ECU_B5_L              UNSPECIFIED /* Build Specific:2136 for 128ms ECU tail with DLCOMP ENabled */
#define GG_ECU_B5_A              8
#define GG_ECU_B6_L              16
#define GG_ECU_B6_A              8
#define GG_ECU_B7_L              28
#define GG_ECU_B7_A              8
#define GG_ECU_B8_L              54
#define GG_ECU_B8_A              8
#define GG_ECU_B9_L              54
#define GG_ECU_B9_A              8
#define GG_ECU_B10_L             12
#define GG_ECU_B10_A             8
#define GG_ECU_B11_L             12
#define GG_ECU_B11_A             8
#define GG_ECU_B12_L             ((GG_ECU_MAX_TAIL_L)*2) /* Multiply by 2 to convert size in linSamples to bytes*/
#define GG_ECU_B12_A             8
#define GG_ECU_B13_L             (GG_ECU_SRCH_WORK_L * 2) /* Multiply by 2 to convert size in linSamples to bytes */
#define GG_ECU_B13_A             8
#define GG_ECU_B14_L             256
#define GG_ECU_B14_A             8
                                              
                                              
/*TEC Engine channel buffer lengths */
#define GG_TEC_CH_B0_L               108
#define GG_TEC_CH_B0_A               8
#define GG_TEC_CH_B1_L               1608
#define GG_TEC_CH_B1_A               8
#define GG_TEC_CH_B2_L               384 /* Build Specific:2136 for 128ms ECU tail with DLCOMP ENabled */
#define GG_TEC_CH_B2_A               8
#define GG_TEC_CH_B3_L               384
#define GG_TEC_CH_B3_A               8
#define GG_TEC_CH_B4_L               720
#define GG_TEC_CH_B4_A               8
#define GG_TEC_CH_B5_L               1456
#define GG_TEC_CH_B5_A               8

/*---------------------------------------------------------------------------*/
/* VEU buffer (for 10ms framesize; for 5ms it should be about 80 bytes less) */
/*     (In some builds, overlaid either with ECU buffer number 14 or 2       */
/*                               or     with TEC buffer number 3)            */
/*---------------------------------------------------------------------------*/
#ifdef SIU_TDM_10MS
  #define GG_VEU_B0_L            324
#else
  #define GG_VEU_B0_L            244
#endif
#define GG_VEU_B0_A            8


/*-------------------------------------------------------------------------*/
/* PCM buffers                                                             */
/*-------------------------------------------------------------------------*/
#ifdef XDAIS
  #define GG_PCM_ENC_B0_L        16 /* Includes Sizeof Generic object */
  #define GG_PCM_ENC_B0_A        4
#else
  /* Eco build case */
  #define GG_PCM_ENC_B0_L        (0x0014*2)
  #define GG_PCM_ENC_B0_A        4
  #define GG_PCM_ENC_B1_L        (0x0200*2)
  #define GG_PCM_ENC_B1_A        4
  #define GG_PCM_ENC_B2_L        (0x0030*2)
  #define GG_PCM_ENC_B2_A        4
#endif

#ifdef XDAIS
  #define GG_PCM_DEC_B0_L        16 /* Includes Sizeof Generic object */
  #define GG_PCM_DEC_B0_A        4
#else
  #define GG_PCM_DEC_B0_L        (0x0014*2)
  #define GG_PCM_DEC_B0_A        4
  #define GG_PCM_DEC_B1_L        (0x0200*2)
  #define GG_PCM_DEC_B1_A        4
#endif


/* G711 A2 only */
/* If the positions of the static vs scratch buffers changes, or more
 * static buffers are added, must change CPU_RELOC_TX_NMU_INFO below!
 */
/* these defines can be used for dynamic allocation based on algorithm selected */
#define GG_NMU_TX_G711A2_B0_L     64   /* Nmu Tx Inst */
#define GG_NMU_TX_G711A2_B0_A     8  
#define GG_NMU_TX_G711A2_B1_L     520  /* Nmu Tx buf */
#define GG_NMU_TX_G711A2_B1_A     8  
#define GG_NMU_TX_G711A2_B2_L     988  /* Nmu Scratch 1 */
#define GG_NMU_TX_G711A2_B2_A     8

/* NMU TX - LEGACY i.e. LEVEL MATCHING */
/* these defines can be used for dynamic allocation based on algorithm selected */
#define GG_NMU_TX_LEGACY_B0_L     24            
#define GG_NMU_TX_LEGACY_B0_A     8  
#define GG_NMU_TX_LEGACY_B1_L     0  /* Bufs 1 through end are don't care */
#define GG_NMU_TX_LEGACY_B1_A     8  
#define GG_NMU_TX_LEGACY_B2_L     0  
#define GG_NMU_TX_LEGACY_B2_A     8  

/* defines to be used for static build that supports all algorithms */
#define GG_NMU_TX_ALL_B0_L       max(GG_NMU_TX_G711A2_B0_L, GG_NMU_TX_LEGACY_B0_L)
#define GG_NMU_TX_ALL_B0_A       max(GG_NMU_TX_G711A2_B0_A, GG_NMU_TX_LEGACY_B0_A)
#define GG_NMU_TX_ALL_B1_L       max(GG_NMU_TX_G711A2_B1_L, GG_NMU_TX_LEGACY_B1_L)
#define GG_NMU_TX_ALL_B1_A       max(GG_NMU_TX_G711A2_B1_A, GG_NMU_TX_LEGACY_B1_A)
#define GG_NMU_TX_ALL_B2_L       max(GG_NMU_TX_G711A2_B2_L, GG_NMU_TX_LEGACY_B2_L)
#define GG_NMU_TX_ALL_B2_A       max(GG_NMU_TX_G711A2_B2_A, GG_NMU_TX_LEGACY_B2_A)

/* NMU RX Buffers */
/* G711 A2 only */
/* If the positions of the static vs scratch buffers changes, or more
 * static buffers are added, must change CPU_RELOC_RX_NMU_INFO below!
 */
/* these defines can be used for dynamic allocation based on algorithm selected */
#define GG_NMU_RX_G711A2_B0_L     24  /* NMU instance  */
#define GG_NMU_RX_G711A2_B0_A     8  
#define GG_NMU_RX_G711A2_B1_L     62  /* Nmu Rx buf    */
#define GG_NMU_RX_G711A2_B1_A     8  
#define GG_NMU_RX_G711A2_B2_L     988 /* Nmu Scratch 1 */
#define GG_NMU_RX_G711A2_B2_A     8  

/* these defines can be used for dynamic allocation based on algorithm selected */
#define GG_NMU_RX_LEGACY_B0_L     16  /* NMU instance */
#define GG_NMU_RX_LEGACY_B0_A     8  /* Bufs 1 through end are don't care */
#define GG_NMU_RX_LEGACY_B1_L     0
#define GG_NMU_RX_LEGACY_B1_A     8
#define GG_NMU_RX_LEGACY_B2_L     0
#define GG_NMU_RX_LEGACY_B2_A     8

/* defines to be used for static build that supports all algorithms */
#define GG_NMU_RX_ALL_B0_L        max(GG_NMU_RX_G711A2_B0_L, GG_NMU_RX_LEGACY_B0_L)
#define GG_NMU_RX_ALL_B0_A        max(GG_NMU_RX_G711A2_B0_A, GG_NMU_RX_LEGACY_B0_A)
#define GG_NMU_RX_ALL_B1_L        max(GG_NMU_RX_G711A2_B1_L, GG_NMU_RX_LEGACY_B1_L)
#define GG_NMU_RX_ALL_B1_A        max(GG_NMU_RX_G711A2_B1_A, GG_NMU_RX_LEGACY_B1_A)
#define GG_NMU_RX_ALL_B2_L        max(GG_NMU_RX_G711A2_B2_L, GG_NMU_RX_LEGACY_B2_L)
#define GG_NMU_RX_ALL_B2_A        max(GG_NMU_RX_G711A2_B2_A, GG_NMU_RX_LEGACY_B2_A)

/* PLC Buffers */
/* The legacy sizes need to be published for dynamic memory allocation where
   PLC getsizes and new are based on selected algorithm */

/* Define sizes for legacy - currently No PLC (which mutes) is the only supported 
   legacy algorithm */
/* If the positions of the static vs scratch buffers changes, or more
 * static buffers are added, must change CPU_RELOC_PLC_INFO below!
 */
/* these defines can be used for dynamic allocation based on algorithm selected */
#define GG_PLC_LEGACY_B0_L        8      /* PLC instance */
#define GG_PLC_LEGACY_B0_A        8       
#define GG_PLC_LEGACY_B1_L        0      /* b1 to b3 are don't care */
#define GG_PLC_LEGACY_B1_A        2     
#define GG_PLC_LEGACY_B2_L        0     
#define GG_PLC_LEGACY_B2_A        2     
#define GG_PLC_LEGACY_B3_L        0  /* PLC scratch 2 */ 
#define GG_PLC_LEGACY_B3_A        2    /* PLC scratch 2 */ 
#define GG_PLC_LEGACY_B4_L        0  /* PLC Pitch Scratch buffer */
#define GG_PLC_LEGACY_B4_A        2

/* Define sizes for G711 Appendix I */
#define GG_PLC_G711A1_B0_L        112 /* PLC instance */ 
#define GG_PLC_G711A1_B0_A        8    /* PLC instance */ 
#define GG_PLC_G711A1_B1_L        0x24C  /* PLC scratch 1 */ 
#define GG_PLC_G711A1_B1_A        128    /* PLC scratch 1 */ 
#ifdef G711A1_USE_GMP
#define GG_PLC_G711A1_B2_L        (0xD2+10) /* PLC history */ 
#else
#define GG_PLC_G711A1_B2_L        0x690 /* PLC history */ 
#endif
#define GG_PLC_G711A1_B2_A        8    /* PLC history */ 
#define GG_PLC_G711A1_B3_L        0  /* PLC scratch 2 */ 
#define GG_PLC_G711A1_B3_A        2    /* PLC scratch 2 */
#ifdef SIU_TDM_10MS 
#define GG_PLC_G711A1_B4_L        0x370  /* PLC Pitch Scratch buffer */
#else
#define GG_PLC_G711A1_B4_L        0x320  /* PLC Pitch Scratch buffer */
#endif
#define GG_PLC_G711A1_B4_A        2

#define GG_PLC_ALL_B0_L           max(GG_PLC_LEGACY_B0_L, GG_PLC_G711A1_B0_L)
#define GG_PLC_ALL_B0_A           max(GG_PLC_LEGACY_B0_A, GG_PLC_G711A1_B0_A)
#define GG_PLC_ALL_B1_L           max(GG_PLC_LEGACY_B1_L, GG_PLC_G711A1_B1_L)
#define GG_PLC_ALL_B1_A           max(GG_PLC_LEGACY_B1_A, GG_PLC_G711A1_B1_A)
#define GG_PLC_ALL_B2_L           max(GG_PLC_LEGACY_B2_L, GG_PLC_G711A1_B2_L)
#define GG_PLC_ALL_B2_A           max(GG_PLC_LEGACY_B2_A, GG_PLC_G711A1_B2_A)
#define GG_PLC_ALL_B3_L           max(GG_PLC_G711A1_B3_L, GG_PLC_LEGACY_B3_L)
#define GG_PLC_ALL_B3_A           max(GG_PLC_G711A1_B3_A, GG_PLC_LEGACY_B3_A)
#define GG_PLC_ALL_B4_L           max(GG_PLC_G711A1_B4_L, GG_PLC_LEGACY_B4_L)
#define GG_PLC_ALL_B4_A           max(GG_PLC_G711A1_B4_A, GG_PLC_LEGACY_B4_A)



/*-------------------------------------------------------------------------*/
/* ADPCM buffers                                                             */
/*-------------------------------------------------------------------------*/
#ifdef XDAIS
  #define GG_ADPCM_ENC_B0_L        100
  #define GG_ADPCM_ENC_B0_A        8
  #define GG_ADPCM_ENC_B1_L        64
  #define GG_ADPCM_ENC_B1_A        8
  #define GG_ADPCM_ENC_B2_L        160
  #define GG_ADPCM_ENC_B2_A        8


  #define GG_ADPCM_DEC_B0_L        100
  #define GG_ADPCM_DEC_B0_A        8
  #define GG_ADPCM_DEC_B1_L        64
  #define GG_ADPCM_DEC_B1_A        8
  #define GG_ADPCM_DEC_B2_L        160
  #define GG_ADPCM_DEC_B2_A        8
#else
  #define GG_ADPCM_ENC_B0_L        UNSPECIFIED    // ECO Case undefined
  #define GG_ADPCM_ENC_B0_A        UNSPECIFIED
  #define GG_ADPCM_ENC_B1_L        UNSPECIFIED
  #define GG_ADPCM_ENC_B1_A        UNSPECIFIED
  #define GG_ADPCM_ENC_B2_L        UNSPECIFIED
  #define GG_ADPCM_ENC_B2_A        UNSPECIFIED
  #define GG_ADPCM_ENC_B3_L        UNSPECIFIED
  #define GG_ADPCM_ENC_B3_A        UNSPECIFIED
  #define GG_ADPCM_ENC_B4_L        UNSPECIFIED
  #define GG_ADPCM_ENC_B4_A        UNSPECIFIED

  #define GG_ADPCM_DEC_B0_L        UNSPECIFIED
  #define GG_ADPCM_DEC_B0_A        UNSPECIFIED
  #define GG_ADPCM_DEC_B1_L        UNSPECIFIED
  #define GG_ADPCM_DEC_B1_A        UNSPECIFIED
  #define GG_ADPCM_DEC_B2_L        UNSPECIFIED
  #define GG_ADPCM_DEC_B2_A        UNSPECIFIED
  #define GG_ADPCM_DEC_B3_L        UNSPECIFIED
  #define GG_ADPCM_DEC_B3_A        UNSPECIFIED
#endif

/*-------------------------------------------------------------------------*/
/* G729AB buffers                                                          */
/*-------------------------------------------------------------------------*/
#ifdef XDAIS
  #define GG_G729AB_ENC_B0_L       64  /*Sizeof Generic object + Enc LBRC Inst*/
  #define GG_G729AB_ENC_B0_A       8
  #define GG_G729AB_ENC_B1_L       32
  #define GG_G729AB_ENC_B1_A       8
  #define GG_G729AB_ENC_B2_L       272
  #define GG_G729AB_ENC_B2_A       8
  #define GG_G729AB_ENC_B3_L       48
  #define GG_G729AB_ENC_B3_A       8
  #define GG_G729AB_ENC_B4_L       308
  #define GG_G729AB_ENC_B4_A       8
  #define GG_G729AB_ENC_B5_L       608
  #define GG_G729AB_ENC_B5_A       8
  #define GG_G729AB_ENC_B6_L       12
  #define GG_G729AB_ENC_B6_A       8
  #define GG_G729AB_ENC_B7_L       172
  #define GG_G729AB_ENC_B7_A       8
  #define GG_G729AB_ENC_B8_L       2612
  #define GG_G729AB_ENC_B8_A       8


  #define GG_G729AB_DEC_B0_L       64  /*Sizeof Generic object + Dec LBRC Inst*/
  #define GG_G729AB_DEC_B0_A       8
  #define GG_G729AB_DEC_B1_L       40
  #define GG_G729AB_DEC_B1_A       8
  #define GG_G729AB_DEC_B2_L       44
  #define GG_G729AB_DEC_B2_A       8
  #define GG_G729AB_DEC_B3_L       32
  #define GG_G729AB_DEC_B3_A       8
  #define GG_G729AB_DEC_B4_L       12
  #define GG_G729AB_DEC_B4_A       8
  #define GG_G729AB_DEC_B5_L       120
  #define GG_G729AB_DEC_B5_A       8
  #define GG_G729AB_DEC_B6_L       308
  #define GG_G729AB_DEC_B6_A       8
  #define GG_G729AB_DEC_B7_L       600
  #define GG_G729AB_DEC_B7_A       8
  #define GG_G729AB_DEC_B8_L       20
  #define GG_G729AB_DEC_B8_A       8
  #define GG_G729AB_DEC_B9_L       12
  #define GG_G729AB_DEC_B9_A       8
  #define GG_G729AB_DEC_B10_L      2048
  #define GG_G729AB_DEC_B10_A      8
  #define GG_G729AB_DEC_B11_L      160
  #define GG_G729AB_DEC_B11_A      8
#else
/* ECO build having g29ab not present currently */
/* Enter correct values when integrating g729ab ECO 
   for c64x */
  #define GG_G729AB_ENC_B0_L       UNSPECIFIED       // LBRC instance length
  #define GG_G729AB_ENC_B0_A       UNSPECIFIED
  #define GG_G729AB_ENC_B1_L       UNSPECIFIED
  #define GG_G729AB_ENC_B1_A       UNSPECIFIED
  #define GG_G729AB_ENC_B2_L       UNSPECIFIED
  #define GG_G729AB_ENC_B2_A       UNSPECIFIED
  #define GG_G729AB_ENC_B3_L       UNSPECIFIED
  #define GG_G729AB_ENC_B3_A       UNSPECIFIED
  #define GG_G729AB_ENC_B4_L       UNSPECIFIED
  #define GG_G729AB_ENC_B4_A       UNSPECIFIED
  #define GG_G729AB_ENC_B5_L       UNSPECIFIED
  #define GG_G729AB_ENC_B5_A       UNSPECIFIED

  #define GG_G729AB_DEC_B0_L       UNSPECIFIED
  #define GG_G729AB_DEC_B0_A       UNSPECIFIED
  #define GG_G729AB_DEC_B1_L       UNSPECIFIED
  #define GG_G729AB_DEC_B1_A       UNSPECIFIED
  #define GG_G729AB_DEC_B2_L       UNSPECIFIED
  #define GG_G729AB_DEC_B2_A       UNSPECIFIED
  #define GG_G729AB_DEC_B3_L       UNSPECIFIED
  #define GG_G729AB_DEC_B3_A       UNSPECIFIED
  #define GG_G729AB_DEC_B4_L       UNSPECIFIED
  #define GG_G729AB_DEC_B4_A       UNSPECIFIED
  #define GG_G729AB_DEC_B5_L       UNSPECIFIED
  #define GG_G729AB_DEC_B5_A       UNSPECIFIED

#endif

/*-------------------------------------------------------------------------*/
/* AGC buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_AGC_B0_L              UNSPECIFIED
#define GG_AGC_B0_A              UNSPECIFIED


/*-------------------------------------------------------------------------*/
/* AER buffers                                             */
/*-------------------------------------------------------------------------*/
#define GG_AER_B0_L              UNSPECIFIED
#define GG_AER_B0_A              UNSPECIFIED
#define GG_AER_B1_L              UNSPECIFIED
#define GG_AER_B1_A              UNSPECIFIED
#define GG_AER_B2_L              UNSPECIFIED
#define GG_AER_B2_A              UNSPECIFIED
#define GG_AER_B3_L              UNSPECIFIED
#define GG_AER_B3_A              UNSPECIFIED
#define GG_AER_B4_L              UNSPECIFIED
#define GG_AER_B4_A              UNSPECIFIED
#define GG_AER_B5_L              UNSPECIFIED
#define GG_AER_B5_A              UNSPECIFIED
#define GG_AER_B6_L              UNSPECIFIED
#define GG_AER_B6_A              UNSPECIFIED
#define GG_AER_B7_L              UNSPECIFIED
#define GG_AER_B7_A              UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* PIU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_PIU_B0_L              116
#define GG_PIU_B0_A              8
#define GG_PIU_B1_L              UNSPECIFIED
#define GG_PIU_B1_A              8
#define GG_PIU_B2_L              UNSPECIFIED
#define GG_PIU_B2_A              8
#define GG_PIU_B3_L              UNSPECIFIED
#define GG_PIU_B3_A              8

/*-------------------------------------------------------------------------*/
/* TDU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_TDU_B0_L              84
#define GG_TDU_B0_A              8
#define GG_TDU_B1_L              1024
#define GG_TDU_B1_A              1024
#define GG_TDU_B2_L              340 /* Build Specific:340 when all detectors but CID CAS are enabled */
#define GG_TDU_B2_A              8
#define GG_TDU_B3_L              362
#define GG_TDU_B3_A              8
#define GG_TDU_B4_L              1 /* Not used when building with TDU_USE_GMP */
#define GG_TDU_B4_A              8

#define GG_TDULITE_B0_L        (14*2)
#define GG_TDULITE_B0_A        4
#define GG_TDULITE_B1_L        GG_TDU_B1_L
#define GG_TDULITE_B1_A        GG_TDU_B1_A
#define GG_TDULITE_B2_L        (14*2)
#define GG_TDULITE_B2_A        8
#define GG_TDULITE_B3_L        (24*2)
#define GG_TDULITE_B3_A        8
#define GG_TDULITE_B4_L        0     /* Rx delay line buffer - can be allocated from GMP */
#define GG_TDULITE_B4_A        0

/*-------------------------------------------------------------------------*/
/* VPU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_VPU_B0_L              108
#define GG_VPU_B0_A              8
#define GG_VPU_B1_L              270 /* from max voice playout size * 
                                      * vpu playout length          */
#define GG_VPU_B1_A              8

/*-------------------------------------------------------------------------*/
/* VPPU buffers                                                            */
/*-------------------------------------------------------------------------*/
#define GG_VPPU_B0_L              200
#define GG_VPPU_B0_A              8
#define GG_VPPU_B1_L              1600  /* scratch */
#define GG_VPPU_B1_A              8
#define GG_VPPU_B2_L              UNSPECIFIED  /* RFC Instance */
#define GG_VPPU_B2_A              8

/*-------------------------------------------------------------------------*/
/* TSU buffers                                                            */
/*-------------------------------------------------------------------------*/
#define GG_TSU_B0_L              UNSPECIFIED
#define GG_TSU_B0_A              8
#define GG_TSU_B1_L              32768
#define GG_TSU_B1_A              128

/*-------------------------------------------------------------------------*/
/* MMCU buffers                                                            */
/*-------------------------------------------------------------------------*/
#define GG_MMCU_B0_L              UNSPECIFIED
#define GG_MMCU_B0_A              8
#define GG_MMCU_B1_L              UNSPECIFIED
#define GG_MMCU_B1_A              8

/*-------------------------------------------------------------------------*/
/* TFTP buffers                                                            */
/*-------------------------------------------------------------------------*/
#define GG_TFTP_B0_L              UNSPECIFIED
#define GG_TFTP_B0_A              8


/*-------------------------------------------------------------------------*/
/* VPPU, VTK System Scratch buffers                                        */
/*-------------------------------------------------------------------------*/
#define GG_MAX_IMG_WIDTH   352
#define GG_MAX_IMG_HEIGTH  288

#define GG_VIDEO_ENC_OUTPUT_BUFFER_SIZE  (0x50000)  //!< Encoder output buffer
#define GG_VIDEO_DEC_INPUT_BUFFER_SIZE   (0x50000)  //!< Decoder input  buffer is up to 146KBytes 
//#define GG_VIDEO_MV_DATA_SIZE            ((GG_MAX_IMG_WIDTH*GG_MAX_IMG_WIDTH)>>7)  // Size of MV data array
#define GG_VIDEO_NUM_YUV_BUF_PER_CHAN      12
#define GG_VIDEO_YUV_PAD_LINES            (40)
#define GG_VIDEO_YUV_PAD_PELS             (20)


#define GG_SIU_RX_VIDEO_FRAME_SCRATCH_L    GG_VIDEO_DEC_INPUT_BUFFER_SIZE  /* rx scratch where vppu pkts assembled to frame for vtk */
#define GG_SIU_RX_VIDEO_FRAME_SCRATCH_A    8
#define GG_SIU_TX_VIDEO_FRAME_SCRATCH_L    GG_VIDEO_ENC_OUTPUT_BUFFER_SIZE /* + GG_VIDEO_MV_DATA_SIZE *//* encoder writes encoded frame for vppu to ship out */
#define GG_SIU_TX_VIDEO_FRAME_SCRATCH_A    8
#define GG_SIU_TXRX_VIDEO_YUV_BUFFER_L     GG_MAX_IMG_WIDTH*GG_MAX_IMG_HEIGTH + 2 * (GG_MAX_IMG_WIDTH/2*GG_MAX_IMG_HEIGTH/2)  /*YUV Data */
#define GG_SIU_TXRX_VIDEO_YUV_BUFFER_A     8

#define GG_MAX_INTERCORE_IMG_WIDTH   720
#define GG_MAX_INTERCORE_IMG_HEIGTH  576
#define GG_SIU_TXRX_VIDEO_YUV_INTERCORE_BUFFER_L  ((GG_MAX_INTERCORE_IMG_WIDTH*GG_MAX_INTERCORE_IMG_HEIGTH * 3)/2)  /*YUV Data, intercore */
#define GG_SIU_TSIP_ISR_BUFFER_L           100
#define GG_SIU_TSIP_ISR_BUFFER_A             8


/*-------------------------------------------------------------------------*/
/* VTK instance buffers                                                    */
/*-------------------------------------------------------------------------*/
#define GG_VTK_B0_L              1300
#define GG_VTK_B0_A              8

/*-------------------------------------------------------------------------*/
/* MMCU buffers                                                            */
/*-------------------------------------------------------------------------*/
#define GG_ATK_B0_L              UNSPECIFIED
#define GG_ATK_B0_A              8
#define GG_ATK_B1_L              UNSPECIFIED
#define GG_ATK_B1_A              8

/*-------------------------------------------------------------------------*/
/* PVP buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_PVP_B0_L              164
#define GG_PVP_B0_A              8
#define GG_PVP_B1_L              508
#define GG_PVP_B1_A              8
#ifdef SIU_TDM_10MS
#define GG_PVP_B2_L              80
#else
#define GG_PVP_B2_L              44
#endif

#define GG_PVP_B3_L              UNSPECIFIED /* Build specific */

#define GG_PVP_B2_A              8
#define GG_PVP_B3_A              8

/*-------------------------------------------------------------------------*/
/* MSU buffers                                                             */
/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
/* MSU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_MSU_RTP_B0_L        GG_MSU_RTP_MAX_BUFFER_SIZE
#define GG_MSU_RTP_B0_A        2

#define GG_MSU_RTCP_B0_L       GG_MSU_RTCP_MAX_BUFFER_SIZE
#define GG_MSU_RTCP_B0_A       2


/*-------------------------------------------------------------------------*/
/* PLR buffers                                                             */
/*-------------------------------------------------------------------------*/
/* Assumption here is that the worst case system vif is 30ms g711 which is
   240 bytes == 120 16bit words. This is the reference in the dsp build sheet. */
#if (defined(PLRRED) || defined(PLRFEC))
#if (GG_MAX_PLR_VOICE_RED_LEVEL > GG_MAX_PLR_TEXT_RED_LEVEL)
 #define GG_MAX_PLR_RED_LEVEL GG_MAX_PLR_VOICE_RED_LEVEL
#else
 #define GG_MAX_PLR_RED_LEVEL GG_MAX_PLR_TEXT_RED_LEVEL
#endif
#endif

#ifdef PLRRED
/* RED RTP PACKET SIZE =  (
   (12 bytes RTP header) +
   (((redLevel*4 )+ 2) bytes RED header) +
   (160 bytes for 20ms g711 payload) +
   (redLevel*(160 bytes for 20ms g711 redundant data)) */

#define GG_PLR_RED_SCRATCH_SIZE (12 + \
                                 ((GG_MAX_PLR_RED_LEVEL*4) + 2) + \
                                 160 + \
                                 (GG_MAX_PLR_RED_LEVEL*160))
#else
#define GG_PLR_RED_SCRATCH_SIZE 0
#endif /* PLRRED */

#ifdef PLRFEC
/* FEC PACKET SIZE (RTP HEADER NOT INCLUDED) =
   ((12 bytes FEC header) + (160 bytes for 20ms g711) */
#define GG_PLR_FEC_MAX_PACKET_SIZE (12 + 160)

 /* 2198 packing of FEC stream. */
 #if (GG_MAX_PLR_FEC_LEVEL_2198 > 0)
  /* 2198 ENCAPSULATION PACKET SIZE =
     (12 bytes RTP header) +
     (((redLevel*4 )+ 2) bytes RED header) +
     (160 bytes for 20ms g711 payload) +
     (redLevel*(GG_PLR_FEC_MAX_PACKET_SIZE redundant data))  */
  #define GG_PLR_FEC_SCRATCH_SIZE  (12 + \
                                   ((GG_MAX_PLR_RED_LEVEL*4) + 2) + \
                                   160 + \
                                   (GG_MAX_PLR_FEC_LEVEL_2198* \
                                    GG_PLR_FEC_MAX_PACKET_SIZE))
 #else
  /* FEC RTP PACKET SIZE =
     (12 bytes RTP header) + GG_PLR_FEC_MAX_PACKET_SIZE */
  #define GG_PLR_FEC_SCRATCH_SIZE (12 + GG_PLR_FEC_MAX_PACKET_SIZE)
 #endif /* (GG_MAX_PLR_FEC_LEVEL_2198 > 0) */
#else
#define GG_PLR_FEC_SCRATCH_SIZE 0
#endif /* PLRFEC */

#define GG_PLR_B1_RED_L        GG_PLR_RED_SCRATCH_SIZE
#define GG_PLR_B2_RED_L        GG_PLR_RED_SCRATCH_SIZE
#define GG_PLR_B1_FEC_L        GG_PLR_FEC_SCRATCH_SIZE
#define GG_PLR_B2_FEC_L        GG_PLR_FEC_SCRATCH_SIZE

#define GG_PLR_B0_L            UNSPECIFIED
#define GG_PLR_B0_A            2
#define GG_PLR_B1_L            max(GG_PLR_B1_RED_L, GG_PLR_B1_FEC_L)
#define GG_PLR_B1_A            2
#define GG_PLR_B2_L            max(GG_PLR_B2_RED_L, GG_PLR_B2_FEC_L)
#define GG_PLR_B2_A            2 
#define GG_PLR_B3_L            600
#define GG_PLR_B3_A            2

/*-------------------------------------------------------------------------*/
/* TGU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_TGU_B0_L              28
#define GG_TGU_B0_A              8

/*-------------------------------------------------------------------------*/
/* CID buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_CID_B0_L              56
#define GG_CID_B0_A              8
#define GG_CID_B1_L              106
#define GG_CID_B1_A              8
#define GG_CID_B2_L              320
#define GG_CID_B2_A              8
#define GG_CID_B3_L              44
#define GG_CID_B3_A              8

/*-------------------------------------------------------------------------*/
/* RSU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_RSU_B0_L              UNSPECIFIED
#define GG_RSU_B0_A              UNSPECIFIED
#define GG_RSU_B1_L              UNSPECIFIED
#define GG_RSU_B1_A              UNSPECIFIED
#define GG_RSU_B2_L              UNSPECIFIED
#define GG_RSU_B2_A              UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* ACU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_ACU_B0_L              220
#define GG_ACU_B0_A              8

/*-------------------------------------------------------------------------*/
/* RCU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_RCU_B0_L              304
#define GG_RCU_B0_A              8
#define GG_RCU_B1_L              64
#define GG_RCU_B1_A              4
#define GG_RCU_B2_L              288
#define GG_RCU_B2_A              4

/*-------------------------------------------------------------------------*/
/* SIU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_SIUHEAP_INST_L         72
#define GG_SIUHEAP_INST_A         8

#define GG_SIU_INST_L         0x0178
#define GG_SIU_INST_A         2

/* Part of siu heap instance which resides in external memory */
/* Keep this number tight for internal memory builds, and divisible by the alignement */
#define GG_SIUHEAP_INST_EXT_L         260
#define GG_SIUHEAP_INST_EXT_A         8

#define GG_SIU_FAX_SCRATCH_L  160  /* Scratch used by SIU in FAX mode to accumulate   */
#define GG_SIU_FAX_SCRATCH_A  2   /* 10ms data at the PCM->net split in Tx direction */

/*-------------------------------------------------------------------------*/
/* NEU buffers                                                             */
/*-------------------------------------------------------------------------*/
/* Max VoP packet: 30 ms PCM over RTP/UDP/IP/802.3 Eth W/ VLAN */
/* 240 bytes PCM                                               */
/* 12 bytes RTP                                                */
/* 8 bytes UDP                                                 */
/* 20 bytes IP                                                 */
/* 8 bytes SNAP/LLC                                            */
/* 4 bytes VLAN                                                */
/* 14 bytes Eth                                                */
/* 4 bytes route tag                                           */
/* 6 bytes ATM header                                          */
/* Max size of a packet = 240 + 12 + 8 + 20 + 8 + 4 +14 + 4 + 6 = 316 bytes */
/* Max size of 7 ATM cells   = ( 48 cell size + 10 header size )* (7+1)  */
/* Total TX scratch buffer size = 316 +  464 = 780 bytes or 390 words    */


/* Now assume the max ATM payload is 24 words (48 bytes). Up to 7 ATM cells plus
 * trailing cell may be assembled. Trailing payload is up to 4 words (8 bytes) */

#define GG_NEU_B0_L              UNSPECIFIED
#define GG_NEU_B0_A              UNSPECIFIED
#define GG_NEU_B1_L              UNSPECIFIED
#define GG_NEU_B1_A              UNSPECIFIED
#define GG_NEU_B2_L              UNSPECIFIED /* =464 for Voice, 1500 for Video */
                                             /* GG_NEU_B2_L is defined in build specific ggmemmap.beh */
#define GG_NEU_B2_A              4
#define GG_NEU_B3_L              UNSPECIFIED
#define GG_NEU_B3_A              UNSPECIFIED
#define GG_NEU_B4_L              UNSPECIFIED /* = 0 for GMAC, = 464 for AAL5 SAR, used to be 384 */
#define GG_NEU_B4_A              4

/* Utopia Related NEU buffers */
#define GG_UTNDU_NEU_INST_L              (160)
#define GG_UTNDU_NEU_SSSAR_L             (0) /* NOTUSED */
#define GG_UTNDU_NEU_TX_SCRATCH_L        (232*2)
#define GG_UTNDU_NEU_TX_MSG_SCRATCH_L    (232*2)
#define GG_UTNDU_NEU_TX_HEADER_L         (43*2)
#define GG_UTNDU_NEU_RX_PKTASSEM_L       (192*2)
#define GG_UTNDU_NEU_HLT_RX_PKTASSEM_L   (0)    /* no rx keep alives */
#define GG_UTNDU_NEU_HLT_HEADER_L        (5*2)
#define GG_UTNDU_AAL5_L		             (15*2)

#define GG_GMNDU_NEU_INST_L              GG_UTNDU_NEU_INST_L
#define GG_GMNDU_NEU_SSSAR_L             0
#define GG_GMNDU_NEU_TX_MSG_SCRATCH_L    352
#define GG_GMNDU_NEU_TX_HEADER_L         GG_UTNDU_NEU_TX_HEADER_L
#define GG_GMNDU_NEU_RX_PKTASSEM_L       0
#define GG_GMNDU_NEU_HLT_HEADER_L        GG_UTNDU_NEU_HLT_HEADER_L 
#define GG_GMNDU_NEU_HLT_RX_PKTASSEM_L   0
#define GG_GMNDU_NEU_TX_SCRATCH_L        GG_GMNDU_NEU_TX_MSG_SCRATCH_L

/*-------------------------------------------------------------------------*/
/* ANU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_ANU_B0_L              UNSPECIFIED
#define GG_ANU_B0_A              UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* PRU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_PRU_B0_L            UNSPECIFIED
#define GG_PRU_B0_A            UNSPECIFIED

/* TTU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_TTU_B0_L            UNSPECIFIED
#define GG_TTU_B0_A            UNSPECIFIED
/* Scratch buffer for TTU to be used by CTM module */
#define GG_TTU_SCRATCH_L       3200
#define GG_TTU_SCRATCH_A       8


/*-------------------------------------------------------------------------*/
/* FM buffers                                                              */
/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
/* FM buffers                                                              */
/*-------------------------------------------------------------------------*/
#define GG_FM_B0_L              384
#define GG_FM_B0_A              8                      
#define GG_FM_B1_L              200
#define GG_FM_B1_A              8                      
#define GG_FM_B2_L              80
#define GG_FM_B2_A              8                      
#define GG_FM_B3_L              0x0001
#define GG_FM_B3_A              8                      
#define GG_FM_B4_L              0x0001
#define GG_FM_B4_A              8                      
#define GG_FM_B5_L              0x0001
#define GG_FM_B5_A              8                                           
#define GG_FM_B6_L              0x0001
#define GG_FM_B6_A              8                      
#define GG_FM_B7_L              0x0001
#define GG_FM_B7_A              8                                            
#define GG_FM_B8_L              0x0001
#define GG_FM_B8_A              8                      
#define GG_FM_B9_L              1216
#define GG_FM_B9_A              8                      
#define GG_FM_B10_L             500
#define GG_FM_B10_A             8
#define GG_FM_B11_L             128
#define GG_FM_B11_A             8                 
#define GG_FM_B12_L             102
#define GG_FM_B12_A             8
#define GG_FM_B13_L             102
#define GG_FM_B13_A             8                     
#define GG_FM_B14_L             192
#define GG_FM_B14_A             8                                 
#define GG_FM_B15_L             192
#define GG_FM_B15_A             8                                 
#define GG_FM_B16_L             40
#define GG_FM_B16_A             8                                 
#define GG_FM_B17_L             72
#define GG_FM_B17_A             8                                 
#define GG_FM_B18_L             16
#define GG_FM_B18_A             8                                 
#define GG_FM_B19_L             16
#define GG_FM_B19_A             8                                 
#define GG_FM_B20_L             640
#define GG_FM_B20_A             8                      
                     
                             
/*-------------------------------------------------------------------------*/
/* FIU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_FIU_B0_L             120
#define GG_FIU_B0_A             8
#define GG_FIU_B1_L             468 
#define GG_FIU_B1_A             8
#define GG_FIU_B2_L             1508
#define GG_FIU_B2_A             8
#define GG_FIU_B3_L             UNSPECIFIED
#define GG_FIU_B3_A             8
#define GG_FIU_B4_L             (GG_NUM_FIU_OVERLAY_BUFS * 48)
#define GG_FIU_B4_A             8


/*-------------------------------------------------------------------------*/
/* NPIU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_NPIU_B0_L             1456
#define GG_NPIU_B0_A             8
#define GG_NPIU_B1_L             1984
#define GG_NPIU_B1_A             8
#define GG_NPIU_B2_L             1472 
#define GG_NPIU_B2_A             8

/*-------------------------------------------------------------------------*/
/* NXU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_NXU_B0_L             64 
#define GG_NXU_B0_A             8
#define GG_NXU_B1_L             788 
#define GG_NXU_B1_A             8

/*-------------------------------------------------------------------------*/
/* MRU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_MRU_B0_L              UNSPECIFIED
#define GG_MRU_B0_A              UNSPECIFIED
#define GG_MRU_B1_L              UNSPECIFIED
#define GG_MRU_B1_A              UNSPECIFIED
#define GG_MRU_B2_L              UNSPECIFIED
#define GG_MRU_B2_A              UNSPECIFIED
#define GG_MRU_B3_L              UNSPECIFIED
#define GG_MRU_B3_A              UNSPECIFIED
#define GG_MRU_B4_L              UNSPECIFIED
#define GG_MRU_B4_A              UNSPECIFIED
#define GG_MRU_B5_L              UNSPECIFIED
#define GG_MRU_B5_A              UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* MDMCP buffers                                                           */
/*-------------------------------------------------------------------------*/
#define GG_MDMCP_B0_L            UNSPECIFIED
#define GG_MDMCP_B0_A            UNSPECIFIED
#define GG_MDMCP_B1_L            UNSPECIFIED
#define GG_MDMCP_B1_A            UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* DMP buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_DMP_B0_L              UNSPECIFIED
#define GG_DMP_B0_A              UNSPECIFIED
#define GG_DMP_B1_L              UNSPECIFIED
#define GG_DMP_B1_A              UNSPECIFIED
#define GG_DMP_B2_L              UNSPECIFIED
#define GG_DMP_B2_A              UNSPECIFIED
#define GG_DMP_B3_L              UNSPECIFIED
#define GG_DMP_B3_A              UNSPECIFIED
#define GG_DMP_B4_L              UNSPECIFIED
#define GG_DMP_B4_A              UNSPECIFIED
#define GG_DMP_B5_L              UNSPECIFIED
#define GG_DMP_B5_A              UNSPECIFIED
/*-------------------------------------------------------------------------*/
/* MPP buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_MPP_B0_L              344
#define GG_MPP_B0_A              4
#define GG_MPP_B1_L              1994
#define GG_MPP_B1_A              4
#define GG_MPP_B2_L              284
#define GG_MPP_B2_A              4

/*-------------------------------------------------------------------------*/
/* BERT buffers                                                            */
/*-------------------------------------------------------------------------*/
#define GG_BERT_B0_L             UNSPECIFIED
#define GG_BERT_B0_A             UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* LLMRU buffers                                                           */
/*-------------------------------------------------------------------------*/
#define GG_LLMRU_B0_L            UNSPECIFIED 
#define GG_LLMRU_B0_A            UNSPECIFIED                     
#define GG_LLMRU_B1_L            UNSPECIFIED
#define GG_LLMRU_B1_A            UNSPECIFIED                      
#define GG_LLMRU_B2_L            UNSPECIFIED
#define GG_LLMRU_B2_A            UNSPECIFIED                      
#define GG_LLMRU_B3_L            UNSPECIFIED
#define GG_LLMRU_B3_A            UNSPECIFIED                 
#define GG_LLMRU_B4_L            UNSPECIFIED 
#define GG_LLMRU_B4_A            UNSPECIFIED                
#define GG_LLMRU_B5_L            UNSPECIFIED
#define GG_LLMRU_B5_A            UNSPECIFIED                 
#define GG_LLMRU_B6_L            UNSPECIFIED                 
#define GG_LLMRU_B6_A            UNSPECIFIED
#define GG_LLMRU_B7_L            UNSPECIFIED
#define GG_LLMRU_B7_A            UNSPECIFIED                      
#define GG_LLMRU_B8_L            UNSPECIFIED
#define GG_LLMRU_B8_A            UNSPECIFIED                     
#define GG_LLMRU_B9_L            UNSPECIFIED
#define GG_LLMRU_B9_A            UNSPECIFIED                 
#define GG_LLMRU_B10_L           UNSPECIFIED
#define GG_LLMRU_B10_A           UNSPECIFIED
#define GG_LLMRU_B11_L           UNSPECIFIED
#define GG_LLMRU_B11_A           UNSPECIFIED             
#define GG_LLMRU_B12_L           UNSPECIFIED
#define GG_LLMRU_B12_A           UNSPECIFIED
#define GG_LLMRU_B13_L           UNSPECIFIED
#define GG_LLMRU_B13_A           UNSPECIFIED                
#define GG_LLMRU_B14_L           UNSPECIFIED
#define GG_LLMRU_B14_A           UNSPECIFIED                            
#define GG_LLMRU_B15_L           UNSPECIFIED
#define GG_LLMRU_B15_A           UNSPECIFIED                            
#define GG_LLMRU_B16_L           UNSPECIFIED
#define GG_LLMRU_B16_A           UNSPECIFIED                            
#define GG_LLMRU_B17_L           UNSPECIFIED
#define GG_LLMRU_B17_A           UNSPECIFIED                            
#define GG_LLMRU_B18_L           UNSPECIFIED
#define GG_LLMRU_B18_A           UNSPECIFIED                            

/*-------------------------------------------------------------------------*/
/* SPRTP buffers                                                           */
/*-------------------------------------------------------------------------*/
#define GG_SPRTP_B0_L            UNSPECIFIED
#define GG_SPRTP_B0_A            UNSPECIFIED
#define GG_SPRTP_B1_L            UNSPECIFIED
#define GG_SPRTP_B1_A            UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* ADU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_ADU_B0_L              UNSPECIFIED
#define GG_ADU_B0_A              UNSPECIFIED
#define GG_ADU_B1_L              UNSPECIFIED
#define GG_ADU_B1_A              UNSPECIFIED
#define GG_ADU_B2_L              UNSPECIFIED
#define GG_ADU_B2_A              UNSPECIFIED
#define GG_ADU_B3_L              UNSPECIFIED
#define GG_ADU_B3_A              UNSPECIFIED
#define GG_ADU_B4_L              UNSPECIFIED
#define GG_ADU_B4_A              UNSPECIFIED
#define GG_ADU_B5_L              UNSPECIFIED
#define GG_ADU_B5_A              UNSPECIFIED
#define GG_ADU_B6_L              UNSPECIFIED
#define GG_ADU_B6_A              UNSPECIFIED
#define GG_ADU_B7_L              UNSPECIFIED
#define GG_ADU_B7_A              UNSPECIFIED


/*-------------------------------------------------------------------------*/
/* VCU  buffer                                                     */
/*-------------------------------------------------------------------------*/
#ifdef GG_TFO_IPE
 #ifndef GG_TFO_AE
  #define GG_VCU_B0_L              (64+16)
  #define GG_VCU_B1_L              (180)
 #else
  #define GG_VCU_B0_L              (64+16)
  #define GG_VCU_B1_L              508  
 #endif
#else
 #define GG_VCU_B0_L              64
#endif
#define GG_VCU_B0_A              8

#define GG_VCU_B1_A              8
/*-------------------------------------------------------------------------*/
/* Tx Network framing buffer (used by frame based codecs, w/ frame greater */
/* than 5 ms)                                                              */
/*-------------------------------------------------------------------------*/
#if SIU_TFO /* GG_TFO_ENABLE */
  #define GG_TX_NETWORK_L     (47*2)      /* 40 samples to be buffered + 2 words
                                         + 5 words to store LSBs for TFO */
#else
  #define GG_TX_NETWORK_L     (42*2)      /* 40 samples to be buffered + 2 words */
#endif
#define GG_TX_NETWORK_A     4

/*-------------------------------------------------------------------------*/
/* VAU buffers                                                             */
/*-------------------------------------------------------------------------*/
#define GG_VAU_B0_L              140
#define GG_VAU_B0_A              8
#define GG_VAU_B1_L              160
#define GG_VAU_B1_A              4
#define GG_VAU_B2_L              80
#define GG_VAU_B2_A              4

/*-------------------------------------------------------------------------*/
/* G729 buffer definitions (NEEDS UPDATING)                                */
/*-------------------------------------------------------------------------*/
#define GG_G729_B0_L             UNSPECIFIED
#define GG_G729_B1_L             UNSPECIFIED
#define GG_G729_B2_L             UNSPECIFIED
#define GG_G729_B3_L             UNSPECIFIED
#define GG_G729_B4_L             UNSPECIFIED
#define GG_G729_B5_L             UNSPECIFIED

#define GG_G729_B0_A             UNSPECIFIED
#define GG_G729_B1_A             UNSPECIFIED
#define GG_G729_B2_A             UNSPECIFIED
#define GG_G729_B3_A             UNSPECIFIED
#define GG_G729_B4_A             UNSPECIFIED
#define GG_G729_B5_A             UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* G729E buffer definitions (NEEDS UPDATING)                                */
/*-------------------------------------------------------------------------*/
#define GG_G729E_B0_L            UNSPECIFIED
#define GG_G729E_B1_L            UNSPECIFIED
#define GG_G729E_B2_L            UNSPECIFIED
#define GG_G729E_B3_L            UNSPECIFIED
#define GG_G729E_B4_L            UNSPECIFIED
#define GG_G729E_B5_L            UNSPECIFIED
#define GG_G729E_B6_L            UNSPECIFIED
#define GG_G729E_B7_L            UNSPECIFIED
#define GG_G729E_B8_L            UNSPECIFIED
#define GG_G729E_B9_L            UNSPECIFIED
#define GG_G729E_B10_L           UNSPECIFIED
#define GG_G729E_B11_L           UNSPECIFIED

#define GG_G729E_B0_A            UNSPECIFIED
#define GG_G729E_B1_A            UNSPECIFIED
#define GG_G729E_B2_A            UNSPECIFIED
#define GG_G729E_B3_A            UNSPECIFIED
#define GG_G729E_B4_A            UNSPECIFIED
#define GG_G729E_B5_A            UNSPECIFIED
#define GG_G729E_B6_A            UNSPECIFIED
#define GG_G729E_B7_A            UNSPECIFIED
#define GG_G729E_B8_A            UNSPECIFIED
#define GG_G729E_B9_A            UNSPECIFIED
#define GG_G729E_B10_A           UNSPECIFIED
#define GG_G729E_B11_A           UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* G723 buffer definitions                                */
/*-------------------------------------------------------------------------*/
#ifdef XDAIS
  #define GG_G723_ENC_B0_L         276  /*Sizeof Generic object + Enc LBRC Inst*/
  #define GG_G723_ENC_B1_L         936    
  #define GG_G723_ENC_B2_L         896
  #define GG_G723_ENC_B3_L         960
  #define GG_G723_ENC_B4_L         1488
  #define GG_G723_ENC_B5_L         866

  #define GG_G723_ENC_B0_A         8   
  #define GG_G723_ENC_B1_A         8 
  #define GG_G723_ENC_B2_A         8
  #define GG_G723_ENC_B3_A         8
  #define GG_G723_ENC_B4_A         8
  #define GG_G723_ENC_B5_A         8    

  #define GG_G723_DEC_B0_L         468  /*Sizeof Generic object + Dec LBRC Inst*/
  #define GG_G723_DEC_B1_L         1024
  #define GG_G723_DEC_B2_L         648
  #define GG_G723_DEC_B3_L         480
//  #define GG_G723_DEC_B4_L         480

  #define GG_G723_DEC_B0_A         8
  #define GG_G723_DEC_B1_A         8
  #define GG_G723_DEC_B2_A         8
  #define GG_G723_DEC_B3_A         8
//  #define GG_G723_DEC_B4_A         8    
#else
  #define GG_G723_ENC_B0_L         UNSPECIFIED    
  #define GG_G723_ENC_B1_L         UNSPECIFIED    
  #define GG_G723_ENC_B2_L         UNSPECIFIED    
  #define GG_G723_ENC_B3_L         UNSPECIFIED    
  #define GG_G723_ENC_B4_L         UNSPECIFIED    
  #define GG_G723_ENC_B5_L         UNSPECIFIED    
  #define GG_G723_ENC_B6_L         UNSPECIFIED    
  #define GG_G723_ENC_B7_L         UNSPECIFIED    

  #define GG_G723_ENC_B0_A         UNSPECIFIED    
  #define GG_G723_ENC_B1_A         UNSPECIFIED 
  #define GG_G723_ENC_B2_A         UNSPECIFIED    
  #define GG_G723_ENC_B3_A         UNSPECIFIED    
  #define GG_G723_ENC_B4_A         UNSPECIFIED    
  #define GG_G723_ENC_B5_A         UNSPECIFIED    
  #define GG_G723_ENC_B6_A         UNSPECIFIED    
  #define GG_G723_ENC_B7_A         UNSPECIFIED    

  #define GG_G723_DEC_B0_L         UNSPECIFIED    
  #define GG_G723_DEC_B1_L         UNSPECIFIED    
  #define GG_G723_DEC_B2_L         UNSPECIFIED    
  #define GG_G723_DEC_B3_L         UNSPECIFIED    
  #define GG_G723_DEC_B4_L         UNSPECIFIED    
  #define GG_G723_DEC_B5_L         UNSPECIFIED       

  #define GG_G723_DEC_B0_A         UNSPECIFIED    
  #define GG_G723_DEC_B1_A         UNSPECIFIED    
  #define GG_G723_DEC_B2_A         UNSPECIFIED    
  #define GG_G723_DEC_B3_A         UNSPECIFIED    
  #define GG_G723_DEC_B4_A         UNSPECIFIED    
  #define GG_G723_DEC_B5_A         UNSPECIFIED       
#endif

/*-------------------------------------------------------------------------*/
/* G728 buffer definitions (NEEDS UPDATING)                                */
/*-------------------------------------------------------------------------*/
#define GG_G728_B0_L             UNSPECIFIED
#define GG_G728_B1_L             UNSPECIFIED
#define GG_G728_B2_L             UNSPECIFIED
#define GG_G728_B3_L             UNSPECIFIED
#define GG_G728_B4_L             UNSPECIFIED
#define GG_G728_B5_L             UNSPECIFIED
#define GG_G728_B6_L             UNSPECIFIED

#define GG_G728_B0_A             UNSPECIFIED
#define GG_G728_B1_A             UNSPECIFIED
#define GG_G728_B2_A             UNSPECIFIED
#define GG_G728_B3_A             UNSPECIFIED
#define GG_G728_B4_A             UNSPECIFIED
#define GG_G728_B5_A             UNSPECIFIED
#define GG_G728_B6_A             UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* GSMEFR buffer definitions                                               */
/*-------------------------------------------------------------------------*/
#define GG_GSMEFR_ENC_B0_L       UNSPECIFIED
#define GG_GSMEFR_ENC_B0_A       UNSPECIFIED
#define GG_GSMEFR_ENC_B1_L       UNSPECIFIED
#define GG_GSMEFR_ENC_B1_A       UNSPECIFIED
#define GG_GSMEFR_ENC_B2_L       UNSPECIFIED
#define GG_GSMEFR_ENC_B2_A       UNSPECIFIED
#define GG_GSMEFR_ENC_B3_L       UNSPECIFIED
#define GG_GSMEFR_ENC_B3_A       UNSPECIFIED
#define GG_GSMEFR_ENC_B4_L       UNSPECIFIED
#define GG_GSMEFR_ENC_B4_A       UNSPECIFIED
#define GG_GSMEFR_ENC_B5_L       UNSPECIFIED
#define GG_GSMEFR_ENC_B5_A       UNSPECIFIED
#define GG_GSMEFR_ENC_B6_L       UNSPECIFIED
#define GG_GSMEFR_ENC_B6_A       UNSPECIFIED

#define GG_GSMEFR_DEC_B0_L       UNSPECIFIED
#define GG_GSMEFR_DEC_B0_A       UNSPECIFIED
#define GG_GSMEFR_DEC_B1_L       UNSPECIFIED
#define GG_GSMEFR_DEC_B1_A       UNSPECIFIED
#define GG_GSMEFR_DEC_B2_L       UNSPECIFIED
#define GG_GSMEFR_DEC_B2_A       UNSPECIFIED
#define GG_GSMEFR_DEC_B3_L       UNSPECIFIED
#define GG_GSMEFR_DEC_B3_A       UNSPECIFIED
#define GG_GSMEFR_DEC_B4_L       UNSPECIFIED
#define GG_GSMEFR_DEC_B4_A       UNSPECIFIED
#define GG_GSMEFR_DEC_B5_L       UNSPECIFIED
#define GG_GSMEFR_DEC_B5_A       UNSPECIFIED
#define GG_GSMEFR_DEC_B6_L       UNSPECIFIED
#define GG_GSMEFR_DEC_B6_A       UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* GSMFR buffer definitions                                                */
/*-------------------------------------------------------------------------*/
#ifdef XDAIS
#define GG_GSMFR_ENC_B0_L        240        /* Instance */
#define GG_GSMFR_ENC_B0_A        8
#define GG_GSMFR_ENC_B1_L        978        /* Static */
#define GG_GSMFR_ENC_B1_A        8
#define GG_GSMFR_ENC_B2_L        1008       /* Scratch */
#define GG_GSMFR_ENC_B2_A        8

#define GG_GSMFR_DEC_B0_L        110        /* Instance */
#define GG_GSMFR_DEC_B0_A        8
#define GG_GSMFR_DEC_B1_L        472        /* Static */
#define GG_GSMFR_DEC_B1_A        8
#define GG_GSMFR_DEC_B2_L        40         /* Scratch */
#define GG_GSMFR_DEC_B2_A        8
#define GG_GSMFR_DEC_B3_L        320        /* Static - Decoder output buffer */
#define GG_GSMFR_DEC_B3_A        8
#else
#define GG_GSMFR_ENC_B0_L        UNSPECIFIED
#define GG_GSMFR_ENC_B0_A        UNSPECIFIED
#define GG_GSMFR_ENC_B1_L        UNSPECIFIED
#define GG_GSMFR_ENC_B1_A        UNSPECIFIED
#define GG_GSMFR_ENC_B2_L        UNSPECIFIED
#define GG_GSMFR_ENC_B2_A        UNSPECIFIED
#define GG_GSMFR_ENC_B3_L        UNSPECIFIED
#define GG_GSMFR_ENC_B3_A        UNSPECIFIED
#define GG_GSMFR_ENC_B4_L        UNSPECIFIED
#define GG_GSMFR_ENC_B4_A        UNSPECIFIED
#define GG_GSMFR_ENC_B5_L        UNSPECIFIED
#define GG_GSMFR_ENC_B5_A        UNSPECIFIED
#define GG_GSMFR_ENC_B6_L        UNSPECIFIED
#define GG_GSMFR_ENC_B6_A        UNSPECIFIED

#define GG_GSMFR_DEC_B0_L        UNSPECIFIED
#define GG_GSMFR_DEC_B0_A        UNSPECIFIED
#define GG_GSMFR_DEC_B1_L        UNSPECIFIED
#define GG_GSMFR_DEC_B1_A        UNSPECIFIED
#define GG_GSMFR_DEC_B2_L        UNSPECIFIED
#define GG_GSMFR_DEC_B2_A        UNSPECIFIED
#define GG_GSMFR_DEC_B3_L        UNSPECIFIED
#define GG_GSMFR_DEC_B3_A        UNSPECIFIED
#define GG_GSMFR_DEC_B4_L        UNSPECIFIED
#define GG_GSMFR_DEC_B4_A        UNSPECIFIED
#define GG_GSMFR_DEC_B5_L        UNSPECIFIED
#define GG_GSMFR_DEC_B5_A        UNSPECIFIED
#define GG_GSMFR_DEC_B6_L        UNSPECIFIED
#define GG_GSMFR_DEC_B6_A        UNSPECIFIED
#endif /* XDAIS */

 /*-------------------------------------------------------------------------*/
/* QCELP13 buffer definitions (NEEDS UPDATING)                  */
/*-------------------------------------------------------------------------*/

#define GG_QCELP13_ENC_B0_L          28*2 	
#define GG_QCELP13_ENC_B0_A          4  	
#define GG_QCELP13_ENC_B1_L          1880*2
#define GG_QCELP13_ENC_B1_A          4  	
#define GG_QCELP13_ENC_B2_L          160*2	
#define GG_QCELP13_ENC_B2_A          4  	
#define GG_QCELP13_ENC_B3_L          144*2	
#define GG_QCELP13_ENC_B3_A          4  
#define GG_QCELP13_ENC_B4_L          358*2	
#define GG_QCELP13_ENC_B4_A          4  	
#define GG_QCELP13_ENC_B5_L          356*2 	
#define GG_QCELP13_ENC_B5_A          4  	
#define GG_QCELP13_ENC_B6_L          378*2	
#define GG_QCELP13_ENC_B6_A          4  	
#define GG_QCELP13_ENC_B7_L          380*2	
#define GG_QCELP13_ENC_B7_A          4  	
#define GG_QCELP13_ENC_B8_L          178*2	
#define GG_QCELP13_ENC_B8_A          4  	

#define GG_QCELP13_DEC_B0_L          28*2 	
#define GG_QCELP13_DEC_B0_A          4  	
#define GG_QCELP13_DEC_B1_L          1185*2	
#define GG_QCELP13_DEC_B1_A          4  	
#define GG_QCELP13_DEC_B2_L          10*2 	
#define GG_QCELP13_DEC_B2_A          4  	
#define GG_QCELP13_DEC_B3_L          576*2	
#define GG_QCELP13_DEC_B3_A          4  	
#define GG_QCELP13_DEC_B4_L          160*2	
#define GG_QCELP13_DEC_B4_A          4  







/*-------------------------------------------------------------------------*/
/* G722 buffer definitions (NEEDS UPDATING)                                */
/*-------------------------------------------------------------------------*/
#define GG_G722_ENC_B0_L         UNSPECIFIED
#define GG_G722_ENC_B1_L         UNSPECIFIED
#define GG_G722_ENC_B2_L         UNSPECIFIED
#define GG_G722_ENC_B3_L         UNSPECIFIED
#define GG_G722_ENC_B4_L         UNSPECIFIED

#define GG_G722_DEC_B0_L         UNSPECIFIED
#define GG_G722_DEC_B1_L         UNSPECIFIED
#define GG_G722_DEC_B2_L         UNSPECIFIED
#define GG_G722_DEC_B3_L         UNSPECIFIED
#define GG_G722_DEC_B4_L         UNSPECIFIED

#define GG_G722_ENC_B0_A         UNSPECIFIED
#define GG_G722_ENC_B1_A         UNSPECIFIED
#define GG_G722_ENC_B2_A         UNSPECIFIED
#define GG_G722_ENC_B3_A         UNSPECIFIED
#define GG_G722_ENC_B4_A         UNSPECIFIED

#define GG_G722_DEC_B0_A         UNSPECIFIED
#define GG_G722_DEC_B1_A         UNSPECIFIED
#define GG_G722_DEC_B2_A         UNSPECIFIED
#define GG_G722_DEC_B3_A         UNSPECIFIED
#define GG_G722_DEC_B4_A         UNSPECIFIED

/*-------------------------------------------------------------------------*/
/* MP3 buffer definitions                                                  */
/*-------------------------------------------------------------------------*/
#define GG_MP3_DEC_B0_L         UNSPECIFIED    /*Generic Instance*/
#define GG_MP3_DEC_B1_L         UNSPECIFIED   /*MP3 Buffers*/
#define GG_MP3_DEC_B2_L         UNSPECIFIED
#define GG_MP3_DEC_B3_L         UNSPECIFIED
#define GG_MP3_DEC_B4_L         UNSPECIFIED
#define GG_MP3_DEC_B5_L         UNSPECIFIED 
#define GG_MP3_DEC_B6_L         UNSPECIFIED
#define GG_MP3_DEC_B7_L         UNSPECIFIED   /* Input buffer */
#define GG_MP3_DEC_B8_L         UNSPECIFIED    /* Output Buffer */

#define GG_MP3_DEC_B0_A         UNSPECIFIED
#define GG_MP3_DEC_B1_A         UNSPECIFIED
#define GG_MP3_DEC_B2_A         UNSPECIFIED
#define GG_MP3_DEC_B3_A         UNSPECIFIED
#define GG_MP3_DEC_B4_A         UNSPECIFIED
#define GG_MP3_DEC_B5_A         UNSPECIFIED
#define GG_MP3_DEC_B6_A         UNSPECIFIED
#define GG_MP3_DEC_B7_A         UNSPECIFIED
#define GG_MP3_DEC_B8_A         UNSPECIFIED

/*--------------------------------------------------------------------------*/
/* EVRC/EVRC buffer definitions                                             */
/*--------------------------------------------------------------------------*/
#ifdef XDAIS
  #define GG_EVRC_ENC_B0_L          (32+100)
  #define GG_EVRC_ENC_B1_L          1228       /*Persistent - or is it just 1212?verify*/
  #define GG_EVRC_ENC_B2_L          252        /*Scratch*/
  #define GG_EVRC_ENC_B3_L          3608       /*Scratch*/

  #define GG_EVRC_ENC_B4_L          920       /*Persistent*/
  #define GG_EVRC_ENC_B5_L          864       /*Persistent*/
  #define GG_EVRC_ENC_B6_L          880       /*Persistent*/


  #define GG_EVRC_ENC_B0_A          8          /*To do : 0 */
  #define GG_EVRC_ENC_B1_A          8
  #define GG_EVRC_ENC_B2_A          8
  #define GG_EVRC_ENC_B3_A          8
  #define GG_EVRC_ENC_B4_A          8
  #define GG_EVRC_ENC_B5_A          8
  #define GG_EVRC_ENC_B6_A          8


  #define GG_EVRC_DEC_B0_L          (20+100)
  #define GG_EVRC_DEC_B1_L          1600      /*Persistent*/
  #define GG_EVRC_DEC_B2_L          384       /*Scratch*/
  #define GG_EVRC_DEC_B3_L          320       /*ouput buffer*/

  #define GG_EVRC_DEC_B0_A          8          /*To do : 0*/
  #define GG_EVRC_DEC_B1_A          8
  #define GG_EVRC_DEC_B2_A          8
  #define GG_EVRC_DEC_B3_A          8
#else 
  #define GG_EVRC_ENC_B0_L          UNSPECIFIED
  #define GG_EVRC_ENC_B0_A          UNSPECIFIED
  #define GG_EVRC_ENC_B1_L          UNSPECIFIED
  #define GG_EVRC_ENC_B1_A          UNSPECIFIED
  #define GG_EVRC_ENC_B2_L          UNSPECIFIED
  #define GG_EVRC_ENC_B2_A          UNSPECIFIED
  #define GG_EVRC_ENC_B3_L          UNSPECIFIED
  #define GG_EVRC_ENC_B3_A          UNSPECIFIED
  #define GG_EVRC_ENC_B4_L          UNSPECIFIED
  #define GG_EVRC_ENC_B4_A          UNSPECIFIED
  #define GG_EVRC_ENC_B5_L          UNSPECIFIED
  #define GG_EVRC_ENC_B5_A          UNSPECIFIED
  #define GG_EVRC_ENC_B6_L          UNSPECIFIED
  #define GG_EVRC_ENC_B6_A          UNSPECIFIED

  #define GG_EVRC_DEC_B0_L          UNSPECIFIED
  #define GG_EVRC_DEC_B0_A          UNSPECIFIED
  #define GG_EVRC_DEC_B1_L          UNSPECIFIED
  #define GG_EVRC_DEC_B1_A          UNSPECIFIED
  #define GG_EVRC_DEC_B2_L          UNSPECIFIED
  #define GG_EVRC_DEC_B2_A          UNSPECIFIED
  #define GG_EVRC_DEC_B3_L          UNSPECIFIED
  #define GG_EVRC_DEC_B3_A          UNSPECIFIED
  #define GG_EVRC_DEC_B4_L          UNSPECIFIED
  #define GG_EVRC_DEC_B4_A          UNSPECIFIED
  #define GG_EVRC_DEC_B5_L          UNSPECIFIED
  #define GG_EVRC_DEC_B5_A          UNSPECIFIED
#endif

/*-------------------------------------------------------------------------*/
/* SMV buffer definitions                                                  */
/*-------------------------------------------------------------------------*/

#ifdef XDAIS
  #define GG_SMV_ENC_B0_L          (176+100)
  #define GG_SMV_ENC_B1_L          9072
  #define GG_SMV_ENC_B2_L          10800

  #define GG_SMV_ENC_B0_A          8
  #define GG_SMV_ENC_B1_A          8
  #define GG_SMV_ENC_B2_A          8

  #define GG_SMV_DEC_B0_L          (176+100)
  #define GG_SMV_DEC_B1_L          3536
  #define GG_SMV_DEC_B2_L          3868
  #define GG_SMV_DEC_B3_L          320

  #define GG_SMV_DEC_B0_A          8
  #define GG_SMV_DEC_B1_A          8
  #define GG_SMV_DEC_B2_A          8
  #define GG_SMV_DEC_B3_A          8
#else
  #define GG_SMV_ENC_B0_L          UNSPECIFIED
  #define GG_SMV_ENC_B0_A          UNSPECIFIED
  #define GG_SMV_ENC_B1_L          UNSPECIFIED
  #define GG_SMV_ENC_B1_A          UNSPECIFIED
  #define GG_SMV_ENC_B2_L          UNSPECIFIED
  #define GG_SMV_ENC_B2_A          UNSPECIFIED
  #define GG_SMV_ENC_B3_L          UNSPECIFIED
  #define GG_SMV_ENC_B3_A          UNSPECIFIED
  #define GG_SMV_ENC_B4_L          UNSPECIFIED
  #define GG_SMV_ENC_B4_A          UNSPECIFIED
  #define GG_SMV_ENC_B5_L          UNSPECIFIED
  #define GG_SMV_ENC_B5_A          UNSPECIFIED
  #define GG_SMV_ENC_B6_L          UNSPECIFIED
  #define GG_SMV_ENC_B6_A          UNSPECIFIED
  #define GG_SMV_ENC_B7_L          UNSPECIFIED
  #define GG_SMV_ENC_B7_A          UNSPECIFIED
  #define GG_SMV_ENC_B8_L          UNSPECIFIED
  #define GG_SMV_ENC_B8_A          UNSPECIFIED
  #define GG_SMV_ENC_B9_L          UNSPECIFIED
  #define GG_SMV_ENC_B9_A          UNSPECIFIED
  #define GG_SMV_ENC_B10_L         UNSPECIFIED
  #define GG_SMV_ENC_B10_A         UNSPECIFIED
  #define GG_SMV_ENC_B11_L         UNSPECIFIED
  #define GG_SMV_ENC_B11_A         UNSPECIFIED
  #define GG_SMV_ENC_B12_L         UNSPECIFIED
  #define GG_SMV_ENC_B12_A         UNSPECIFIED
  #define GG_SMV_ENC_B13_L         UNSPECIFIED
  #define GG_SMV_ENC_B13_A         UNSPECIFIED
  #define GG_SMV_ENC_B14_L         UNSPECIFIED
  #define GG_SMV_ENC_B14_A         UNSPECIFIED
  #define GG_SMV_ENC_B15_L         UNSPECIFIED
  #define GG_SMV_ENC_B15_A         UNSPECIFIED

  #define GG_SMV_DEC_B0_L          UNSPECIFIED
  #define GG_SMV_DEC_B0_A          UNSPECIFIED
  #define GG_SMV_DEC_B1_L          UNSPECIFIED
  #define GG_SMV_DEC_B1_A          UNSPECIFIED
  #define GG_SMV_DEC_B2_L          UNSPECIFIED
  #define GG_SMV_DEC_B2_A          UNSPECIFIED
  #define GG_SMV_DEC_B3_L          UNSPECIFIED
  #define GG_SMV_DEC_B3_A          UNSPECIFIED
  #define GG_SMV_DEC_B4_L          UNSPECIFIED
  #define GG_SMV_DEC_B4_A          UNSPECIFIED
  #define GG_SMV_DEC_B5_L          UNSPECIFIED
  #define GG_SMV_DEC_B5_A          UNSPECIFIED
  #define GG_SMV_DEC_B6_L          UNSPECIFIED
  #define GG_SMV_DEC_B6_A          UNSPECIFIED
  #define GG_SMV_DEC_B7_L          UNSPECIFIED
  #define GG_SMV_DEC_B7_A          UNSPECIFIED
  #define GG_SMV_DEC_B8_L          UNSPECIFIED
  #define GG_SMV_DEC_B8_A          UNSPECIFIED
  #define GG_SMV_DEC_B9_L          UNSPECIFIED
  #define GG_SMV_DEC_B9_A          UNSPECIFIED
#endif
/*-------------------------------------------------------------------------*/
/* GSMAMR buffer definitions                                  */
/*-------------------------------------------------------------------------*/
#ifdef XDAIS
  #define GG_GSMAMR_ENC_B0_L         628  /*Sizeof Generic object + Enc LBRC Inst*/
  #define GG_GSMAMR_ENC_B1_L         504
  #define GG_GSMAMR_ENC_B2_L         640
  #define GG_GSMAMR_ENC_B3_L         482
  #define GG_GSMAMR_ENC_B4_L         676
  #define GG_GSMAMR_ENC_B5_L         376
  #define GG_GSMAMR_ENC_B6_L         268
  #define GG_GSMAMR_ENC_B7_L         1768
  #define GG_GSMAMR_ENC_B8_L         3200
  #define GG_GSMAMR_ENC_B9_L         2464

  #define GG_GSMAMR_ENC_B0_A         8  
  #define GG_GSMAMR_ENC_B1_A         8
  #define GG_GSMAMR_ENC_B2_A         8
  #define GG_GSMAMR_ENC_B3_A         8
  #define GG_GSMAMR_ENC_B4_A         8
  #define GG_GSMAMR_ENC_B5_A         8    
  #define GG_GSMAMR_ENC_B6_A         8
  #define GG_GSMAMR_ENC_B7_A         8
  #define GG_GSMAMR_ENC_B8_A         8    
  #define GG_GSMAMR_ENC_B9_A         8

  #define GG_GSMAMR_DEC_B0_L         468  /*Sizeof Generic object + Dec LBRC Inst*/
  #define GG_GSMAMR_DEC_B1_L         700    
  #define GG_GSMAMR_DEC_B2_L         628
  #define GG_GSMAMR_DEC_B3_L         688
  #define GG_GSMAMR_DEC_B4_L         320  /*Decoder o/p buffer*/

  #define GG_GSMAMR_DEC_B0_A         8
  #define GG_GSMAMR_DEC_B1_A         8
  #define GG_GSMAMR_DEC_B2_A         8
  #define GG_GSMAMR_DEC_B3_A         8
  #define GG_GSMAMR_DEC_B4_A         8
#else
  #define GG_GSMAMR_ENC_B0_L          UNSPECIFIED
  #define GG_GSMAMR_ENC_B0_A          UNSPECIFIED
  #define GG_GSMAMR_ENC_B1_L          UNSPECIFIED
  #define GG_GSMAMR_ENC_B1_A          UNSPECIFIED
  #define GG_GSMAMR_ENC_B2_L          UNSPECIFIED
  #define GG_GSMAMR_ENC_B2_A          UNSPECIFIED
  #define GG_GSMAMR_ENC_B3_L          UNSPECIFIED
  #define GG_GSMAMR_ENC_B3_A          UNSPECIFIED
  #define GG_GSMAMR_ENC_B4_L          UNSPECIFIED
  #define GG_GSMAMR_ENC_B4_A          UNSPECIFIED
  #define GG_GSMAMR_ENC_B5_L          UNSPECIFIED
  #define GG_GSMAMR_ENC_B5_A          UNSPECIFIED
  #define GG_GSMAMR_ENC_B6_L          UNSPECIFIED
  #define GG_GSMAMR_ENC_B6_A          UNSPECIFIED


  #define GG_GSMAMR_DEC_B0_L          UNSPECIFIED
  #define GG_GSMAMR_DEC_B0_A          UNSPECIFIED
  #define GG_GSMAMR_DEC_B1_L          UNSPECIFIED
  #define GG_GSMAMR_DEC_B1_A          UNSPECIFIED
  #define GG_GSMAMR_DEC_B2_L          UNSPECIFIED
  #define GG_GSMAMR_DEC_B2_A          UNSPECIFIED
  #define GG_GSMAMR_DEC_B3_L          UNSPECIFIED
  #define GG_GSMAMR_DEC_B3_A          UNSPECIFIED
  #define GG_GSMAMR_DEC_B4_L          UNSPECIFIED
  #define GG_GSMAMR_DEC_B4_A          UNSPECIFIED
  #define GG_GSMAMR_DEC_B5_L          UNSPECIFIED
  #define GG_GSMAMR_DEC_B5_A          UNSPECIFIED
#endif

/*-------------------------------------------------------------------------*/
/* ILBC buffer definitions                                                */
/*-------------------------------------------------------------------------*/
#ifdef XDAIS
#define GG_ILBC_ENC_B0_L        64        /* Instance */
#define GG_ILBC_ENC_B0_A        8
#define GG_ILBC_ENC_B1_L        1076        /* Static */
#define GG_ILBC_ENC_B1_A        8
#define GG_ILBC_ENC_B2_L        4308       /* Scratch 456*/
#define GG_ILBC_ENC_B2_A        8

#define GG_ILBC_DEC_B0_L        64        /* Instance */
#define GG_ILBC_DEC_B0_A        8
#define GG_ILBC_DEC_B1_L        2048        /* Static 1024*/
#define GG_ILBC_DEC_B1_A        8
#define GG_ILBC_DEC_B2_L        1708         /* Scratch 648*/
#define GG_ILBC_DEC_B2_A        8
#define GG_ILBC_DEC_B3_L        480        /* Static - Decoder output buffer to support 20/30msec */
#define GG_ILBC_DEC_B3_A        8
#else
#define GG_ILBC_ENC_B0_L        UNSPECIFIED
#define GG_ILBC_ENC_B0_A        UNSPECIFIED
#define GG_ILBC_ENC_B1_L        UNSPECIFIED
#define GG_ILBC_ENC_B1_A        UNSPECIFIED
#define GG_ILBC_ENC_B2_L        UNSPECIFIED
#define GG_ILBC_ENC_B2_A        UNSPECIFIED
#define GG_ILBC_ENC_B3_L        UNSPECIFIED
#define GG_ILBC_ENC_B3_A        UNSPECIFIED
#define GG_ILBC_ENC_B4_L        UNSPECIFIED
#define GG_ILBC_ENC_B4_A        UNSPECIFIED
#define GG_ILBC_ENC_B5_L        UNSPECIFIED
#define GG_ILBC_ENC_B5_A        UNSPECIFIED
#define GG_ILBC_ENC_B6_L        UNSPECIFIED
#define GG_ILBC_ENC_B6_A        UNSPECIFIED

#define GG_ILBC_DEC_B0_L        UNSPECIFIED
#define GG_ILBC_DEC_B0_A        UNSPECIFIED
#define GG_ILBC_DEC_B1_L        UNSPECIFIED
#define GG_ILBC_DEC_B1_A        UNSPECIFIED
#define GG_ILBC_DEC_B2_L        UNSPECIFIED
#define GG_ILBC_DEC_B2_A        UNSPECIFIED
#define GG_ILBC_DEC_B3_L        UNSPECIFIED
#define GG_ILBC_DEC_B3_A        UNSPECIFIED
#define GG_ILBC_DEC_B4_L        UNSPECIFIED
#define GG_ILBC_DEC_B4_A        UNSPECIFIED
#define GG_ILBC_DEC_B5_L        UNSPECIFIED
#define GG_ILBC_DEC_B5_A        UNSPECIFIED
#define GG_ILBC_DEC_B6_L        UNSPECIFIED
#define GG_ILBC_DEC_B6_A        UNSPECIFIED
#endif /* XDAIS */



/*-------------------------------------------------------------------------*/
/* GSM WBAMR buffer definitions (NEEDS UPDATING)                             */
/*-------------------------------------------------------------------------*/
#define GG_GSM_WBAMR_ENC_B0_L       UNSPECIFIED
#define GG_GSM_WBAMR_ENC_B0_A       UNSPECIFIED
#define GG_GSM_WBAMR_ENC_B1_L       UNSPECIFIED
#define GG_GSM_WBAMR_ENC_B1_A       UNSPECIFIED
#define GG_GSM_WBAMR_ENC_B2_L       UNSPECIFIED
#define GG_GSM_WBAMR_ENC_B2_A       UNSPECIFIED
#define GG_GSM_WBAMR_ENC_B3_L       UNSPECIFIED    
#define GG_GSM_WBAMR_ENC_B3_A       UNSPECIFIED 
#define GG_GSM_WBAMR_ENC_B4_L       UNSPECIFIED    
#define GG_GSM_WBAMR_ENC_B4_A       UNSPECIFIED 
#define GG_GSM_WBAMR_ENC_B5_L       UNSPECIFIED   
#define GG_GSM_WBAMR_ENC_B5_A       UNSPECIFIED 


#define GG_GSM_WBAMR_DEC_B0_L       UNSPECIFIED  
#define GG_GSM_WBAMR_DEC_B0_A       UNSPECIFIED
#define GG_GSM_WBAMR_DEC_B1_L       UNSPECIFIED 
#define GG_GSM_WBAMR_DEC_B1_A       UNSPECIFIED 
#define GG_GSM_WBAMR_DEC_B2_L       UNSPECIFIED   
#define GG_GSM_WBAMR_DEC_B2_A       UNSPECIFIED 
#define GG_GSM_WBAMR_DEC_B3_L       UNSPECIFIED   
#define GG_GSM_WBAMR_DEC_B3_A       UNSPECIFIED 
#define GG_GSM_WBAMR_DEC_B4_L       UNSPECIFIED    
#define GG_GSM_WBAMR_DEC_B4_A       UNSPECIFIED 
#define GG_GSM_WBAMR_DEC_B5_L       UNSPECIFIED   
#define GG_GSM_WBAMR_DEC_B5_A       UNSPECIFIED 

/*-------------------------------------------------------------------------*/
/* Scratch ProgMem length and alignment                                    */
/*-------------------------------------------------------------------------*/
#define PRGGMBUF_SCRATCH_L       0
#define PRGGMBUF_SCRATCH_A       0

/*-------------------------------------------------------------------------*/
/* MPP sub-instance buffers     (Unsure what is this)                      */
/*-------------------------------------------------------------------------*/
#define GG_MPP_SUBINST_L         UNSPECIFIED   /* This can be overlaid with ECU FIFO */
#define GG_MPP_SUBINST_A         UNSPECIFIED


/*-------------------------------------------------------------------------*/
/* TDU buffers    (Unsure what is this)                                    */
/*-------------------------------------------------------------------------*/
#define GG_TDU_FFT_B_L           512
#define GG_TDU_FFT_B_A           512                     

/* What is this ???   */
#define IRAM_VAU_DECISION_B_L    20
#define IRAM_VAU_DECISION_B_A    2

/*-------------------------------------------------------------------------*/
/* SAMP16K buffers (for IP Phone)                                          */
/*-------------------------------------------------------------------------*/
#define GG_UPDOWN_FIRBUFF_L      96
#define GG_UPDOWN_SAMPLES_L      160
#define GG_UPDOWN_SCRATCH_L      200

#define GG_UPDOWN_FIRBUFF_A      4

/* Relocation Info for a module. 
 * Bit 0 (LSB) contains whether the first buffer should be relocated or not, 
 * Bit 1 contains whether the second buffer should be relocated or not.
 * 0 - do NOT relocate
 * 1 - Relocate */
#define CPU_RELOC_PLC_INFO    0x5 /* 00101 - First and Third Buffers
                                   * could be relocated. */

#define CPU_RELOC_RX_NMU_INFO 0x3 /* 0011 - First and Second Buffers could be
                                   * relocated. */

#define CPU_RELOC_TX_NMU_INFO 0x3 /* 0011 - First and Second Buffers could be
                                   * relocated. */

#define CPU_RELOC_RTP_MSU_INFO 0x1 /* 0001 - First Buffer could be
                                   * relocated. */

#define CPU_RELOC_RTCP_MSU_INFO 0x1 /* 0001 - First Buffer could be
                                   * relocated. */

#define CPU_RELOC_PLR_INFO  0x1 /* 0001 - First Buffer could be
                                   * relocated. */

#define CPU_RELOC_PLR_GMPSUPP_INFO 0x8 /* 4th buffer */


/*-------------------------------------------------------------------------*/
/* MAC router buffers                                                      */
/*  The lengths are PER instance, not total length                         */
/*-------------------------------------------------------------------------*/
#define GG_MACR_CONTEXT_BUF_L   100
#define GG_MACR_IPMAC_BUF_L     100
#define GG_MACR_PORT_CXT_BUF_L  100
#define GG_MACR_STATS_BUF_L     1400
#define GG_MACR_DESTMAC_BUF_L   1500
#define GG_MACR_UDP_BUF_L       0x10000
                              
/*-------------------------------------------------------------------------*/
/* Redirection of initialization code for overlay                          */
/* Pragmas to overlay code                                                 */
/*-------------------------------------------------------------------------*/
#ifndef DEFAULT_CODE_SECTION
#pragma CODE_SECTION (platformInitPCM,      ".progOverlay");
#pragma CODE_SECTION (platformInitG726,     ".progOverlay");
#pragma CODE_SECTION (platformInitG727,     ".progOverlay");
#pragma CODE_SECTION (platformInitG729AB,   ".progOverlay");
#pragma CODE_SECTION (platformInitG729,     ".progOverlay");
#pragma CODE_SECTION (platformInitG729E,    ".progOverlay");
#pragma CODE_SECTION (platformInitG723,     ".progOverlay");
#pragma CODE_SECTION (platformInitG728,     ".progOverlay");
#pragma CODE_SECTION (platformInitILBC,     ".progOverlay");
#pragma CODE_SECTION (platformInitGSMAMR,   ".progOverlay");
#pragma CODE_SECTION (platformInitGSMEFR,   ".progOverlay");
#pragma CODE_SECTION (platformInitGSMFR,    ".progOverlay");
#pragma CODE_SECTION (platformInitG722,     ".progOverlay");
#pragma CODE_SECTION (platformInitFax,      ".progOverlay");
#pragma CODE_SECTION (platformInit,         ".progOverlay");
#pragma CODE_SECTION (platinit_xdais_membuf,".progOverlay");
#pragma CODE_SECTION (platformInitMp3,      ".progOverlay");
#pragma CODE_SECTION (ggEcumaTblInit,       ".progOverlay");
#pragma CODE_SECTION (ggConfigInit,         ".progOverlay");
#pragma CODE_SECTION (platformInitMsu,       ".progOverlay");
#pragma CODE_SECTION (platformInitEVRC,     ".progOverlay");
#pragma CODE_SECTION (platformInitUPCM,     ".progOverlay");
#pragma CODE_SECTION (platformInitAPCM,     ".progOverlay");
#endif

#endif /* _GGPORT_H INCLUDED */


