#ifndef _GGCHBUFDESC_H
#define _GGCHBUFDESC_H
/******************************************************************************
 * FILE PURPOSE: Data structure for memory allocation
 ******************************************************************************
 * FILE NAME:  ggChBufDesc.h
 *
 * DESCRIPTION: Defines data structures required for allocation of ECO
 *              memory buffers.
 *
 *    NOTE:  Presently, the number of buffers associated with each ECO is
 *           defined in this file.  This number MUST be the same as that
 *           defined in the ECO.  Therefore, when an ECO is created, SIU
 *           MUST check that the number of buffers defined in chBuffDesc
 *           for that ECO is correct.
 *
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2004, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "ggport.h"
#include "ggswcfg.h"
#include "ggmbuf.h"

/* Definition for number of buffers. It should be checked in New (Create)
   whether it matches original requests */

#define GG_NUM_DEC_VCA_BUFF    1    /* Total decoder buffers, minus inst */
#define GG_NUM_ENC_VCA_BUFF    1    /* Total encoder buffers, minus inst */
#define GG_NUM_VCA_BUFF        1    /* MAX(DEC, ENC) */

#define GG_NUM_PLC_BUFF        (1 + CPU_NUM_PLC_BUFF) /* The common increment needs to be 1
                                                         This is needed for builds that do not have G711A1 */


#define GG_NUM_TX_NMU_BUFF     (1 + CPU_NUM_TX_NMU_BUFF)
#define GG_NUM_RX_NMU_BUFF     (1 + CPU_NUM_RX_NMU_BUFF)

#ifndef XDAIS
#define GG_NUM_DEC_PCM_BUFF    (1 + 1)
#define GG_NUM_ENC_PCM_BUFF    (1 + 2)
#define GG_NUM_PCM_BUFF        (max(GG_NUM_DEC_PCM_BUFF,GG_NUM_ENC_PCM_BUFF))
#else
#define GG_NUM_DEC_PCM_BUFF    1
#define GG_NUM_ENC_PCM_BUFF    1
#define GG_NUM_PCM_BUFF        1
#endif

#define GG_NUM_DEC_ADPCM_BUFF  (1 + CPU_NUM_DEC_ADPCM_BUFF)
#define GG_NUM_ENC_ADPCM_BUFF  (1 + CPU_NUM_ENC_ADPCM_BUFF)
#define GG_NUM_ADPCM_BUFF      max(GG_NUM_DEC_ADPCM_BUFF, GG_NUM_ENC_ADPCM_BUFF)

#define GG_NUM_DEC_G729AB_BUFF (1 + CPU_NUM_DEC_G729AB_BUFF)
#define GG_NUM_ENC_G729AB_BUFF (1 + CPU_NUM_ENC_G729AB_BUFF)
#define GG_NUM_G729AB_BUFF     (1 + CPU_NUM_G729AB_BUFF)

#define GG_NUM_DEC_G729_BUFF   (1 + 4)
#define GG_NUM_ENC_G729_BUFF   (1 + 5)
#define GG_NUM_G729_BUFF       (1 + 5)

#define GG_NUM_DEC_G729E_BUFF  (1 + CPU_NUM_DEC_G729E_BUFF)
#define GG_NUM_ENC_G729E_BUFF  (1 + CPU_NUM_ENC_G729E_BUFF)
#define GG_NUM_G729E_BUFF      (1 + CPU_NUM_G729E_BUFF)

#define GG_NUM_DEC_ILBC_BUFF  (1 + CPU_NUM_DEC_ILBC_BUFF)
#define GG_NUM_ENC_ILBC_BUFF  (1 + CPU_NUM_ENC_ILBC_BUFF)
#define GG_NUM_ILBC_BUFF      (1 + CPU_NUM_ILBC_BUFF)

#define GG_NUM_DEC_G723_BUFF   (1 + CPU_NUM_DEC_G723_BUFF)
#define GG_NUM_ENC_G723_BUFF   (1 + CPU_NUM_ENC_G723_BUFF)
#define GG_NUM_G723_BUFF       (1 + CPU_NUM_G723_BUFF)

#define GG_NUM_DEC_G728_BUFF   (1 + CPU_NUM_DEC_G728_BUFF)
#define GG_NUM_ENC_G728_BUFF   (1 + CPU_NUM_ENC_G728_BUFF)
#define GG_NUM_G728_BUFF       (1 + CPU_NUM_G728_BUFF)

#define GG_NUM_DEC_GSMEFR_BUFF (1 + CPU_NUM_DEC_GSMEFR_BUFF)
#define GG_NUM_ENC_GSMEFR_BUFF (1 + CPU_NUM_ENC_GSMEFR_BUFF)
#define GG_NUM_GSMEFR_BUFF     (1 + CPU_NUM_GSMEFR_BUFF)

#define GG_NUM_DEC_G722_BUFF   (1 + CPU_NUM_DEC_G722_BUFF)
#define GG_NUM_ENC_G722_BUFF   (1 + CPU_NUM_ENC_G722_BUFF)
#define GG_NUM_G722_BUFF       (1 + CPU_NUM_G722_BUFF)
#define GG_NUM_UTOPIA_NDU_BUFF  3

/* The buffers for the GSM-FR */
#define GG_NUM_DEC_GSMFR_BUFF  (1 + CPU_NUM_DEC_GSMFR_BUFF)
#define GG_NUM_ENC_GSMFR_BUFF  (1 + CPU_NUM_ENC_GSMFR_BUFF)
#define GG_NUM_GSMFR_BUFF      (1 + CPU_NUM_GSMFR_BUFF)

/* The buffers for the Audio Codecs*/
#if (defined(AUDIO))
#define GG_NUM_DEC_MP3_BUFF  (1 + CPU_NUM_DEC_MP3_BUFF)
#endif

/* The buffers for the GSMAMR */
#define GG_NUM_DEC_GSMAMR_BUFF  (1 + CPU_NUM_DEC_GSMAMR_BUFF)
#define GG_NUM_ENC_GSMAMR_BUFF  (1 + CPU_NUM_ENC_GSMAMR_BUFF)
#define GG_NUM_GSMAMR_BUFF      (1 + CPU_NUM_GSMAMR_BUFF)

/* The buffers for the GSM_WBAMR */
#define GG_NUM_DEC_GSM_WBAMR_BUFF  (1 + CPU_NUM_DEC_GSM_WBAMR_BUFF)
#define GG_NUM_ENC_GSM_WBAMR_BUFF  (1 + CPU_NUM_ENC_GSM_WBAMR_BUFF)
#define GG_NUM_GSM_WBAMR_BUFF      (1 + CPU_NUM_GSM_WBAMR_BUFF)


#define GG_NUM_DEC_EVRC_BUFF (1 + CPU_NUM_DEC_EVRC_BUFF)
#define GG_NUM_ENC_EVRC_BUFF (1 + CPU_NUM_ENC_EVRC_BUFF)
#define GG_NUM_EVRC_BUFF     (1 + CPU_NUM_EVRC_BUFF)


#define GG_NUM_DEC_SMV_BUFF (1 + CPU_NUM_DEC_SMV_BUFF)
#define GG_NUM_ENC_SMV_BUFF (1 + CPU_NUM_ENC_SMV_BUFF)
#define GG_NUM_SMV_BUFF     (1 + CPU_NUM_SMV_BUFF)

#define GG_NUM_DEC_QCELP13_BUFF (1 + CPU_NUM_DEC_QCELP13_BUFF)
#define GG_NUM_ENC_QCELP13_BUFF (1 + CPU_NUM_ENC_QCELP13_BUFF)
#define GG_NUM_QCELP13_BUFF     (1 + CPU_NUM_QCELP13_BUFF)




#define GG_NUM_HDNDU_BUFF      8

#if (defined(GG_TFO_IPE) || defined(GG_TFO_AE))
  #define GG_NUM_VCU_BUFF        2
#else
  #define GG_NUM_VCU_BUFF        1
#endif

#define GG_NUM_VAU_BUFF        3
#define GG_NUM_ECU_BUFF        15
#define GG_NUM_AER_BUFF        CPU_NUM_AER_BUFF
#define GG_NUM_AGC_BUFF        1
#define GG_NUM_TEC_BUFF        8
#define GG_NUM_SIU_TEC_BUFF    1
#define GG_NUM_TEC_CH_BUFF     5
#define GG_NUM_TECENG_CH_BUFF  (GG_NUM_TEC_CH_BUFF + GG_NUM_SIU_TEC_BUFF)

#define GG_NUM_PIU_BUFF        4
#define GG_NUM_TDU_BUFF        5
#define GG_NUM_VPU_BUFF        2
#define GG_NUM_MSURTP_BUFF     1
#define GG_NUM_MSURTCP_BUFF    1
#define GG_NUM_PVP_BUFF        4
#define GG_NUM_CID_BUFF        4
#define GG_NUM_RSU_BUFF        3
#define GG_NUM_ACU_BUFF        1
#define GG_NUM_TGU_BUFF        1
#define GG_NUM_RCU_BUFF        3
#define GG_NUM_NEU_BUFF        5
#define GG_NUM_ANU_BUFF        1

/* PLR Buffers */
#ifdef PLR_USE_GMP_SUPPLEMENT
#define GG_NUM_PLR_BUFF        4
#else
#define GG_NUM_PLR_BUFF        3
#endif

#if defined(AUDIO)
#define GG_NUM_ADU_BUFF        8
#endif

#define GG_NUM_PRU_BUFF        1
#define GG_NUM_TTU_BUFF        1

#define GG_NUM_FM_BUFF         21
#define GG_NUM_FIU_BUFF        5

/*Number of buffers needed for 8 / 16KHz sampling rate voice channel support */
#define GG_NUM_MULTSAMPVC_BUFF 5

#define GG_NUM_MRU_BUFF        6

#define GG_NUM_MDMCP_BUFF      2
#define GG_NUM_DMP_BUFF        6
#define GG_NUM_MPP_BUFF        3
#define GG_NUM_LLMRU_BUFF     19
#define GG_NUM_SPRTP_BUFF      2
#define GG_NUM_BERT_BUFF       1
#define GG_NUM_VQM_BUFF        1
#define GG_NUM_NXU_BUFF        2
#define GG_NUM_NPIU_BUFF       3
#define GG_NUM_EQI_BUFF        1
#define GG_NUM_VEU_BUFF        1
#define GG_NUM_VPPU_BUFF       3
#define GG_NUM_VTK_BUFF        1
#define GG_NUM_ATK_BUFF        2
#define GG_NUM_TSU_BUFF        2
#define GG_NUM_CUSTOM_ECO_BUFF 1
#define GG_NUM_OVLY_BUFF       1
#define GG_NUM_MMCU_BUFF       2
#define GG_NUM_TFTP_BUFF       1

extern  const tuint ggCodecTablesInfoEntries;
/*-------------------------------------------------------------------------*/
/*  Data Structures for ECO memory buffer allocation table                 */
/*-------------------------------------------------------------------------*/
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_VCA_BUFF];
} ggbuffDescVCA_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_PCM_BUFF];
} ggbuffDescPCM_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_ADPCM_BUFF];
} ggbuffDescADPCM_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_G729AB_BUFF];
} ggbuffDescG729AB_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_G729_BUFF];
} ggbuffDescG729_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_G729E_BUFF];
} ggbuffDescG729E_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_G723_BUFF];
} ggbuffDescG723_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_G728_BUFF];
} ggbuffDescG728_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_GSMEFR_BUFF];
} ggbuffDescGSMEFR_t;

#if (defined (AUDIO))
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_DEC_MP3_BUFF];
} ggbuffDescMP3_t;
#endif

/* This is the structure defnition for GSM-FR */
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_GSMFR_BUFF];
} ggbuffDescGSMFR_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_G722_BUFF];
} ggbuffDescG722_t;

typedef struct {
   int  numBuffs;
   ggbufDesc_t bufDesc[GG_NUM_ILBC_BUFF];
} ggbuffDescILBC_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_GSMAMR_BUFF];
} ggbuffDescGSMAMR_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_GSM_WBAMR_BUFF];
} ggbuffDescGSM_WBAMR_t;

/* This structure definition is for EVRC */
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_EVRC_BUFF];
} ggbuffDescEVRC_t;

/* This structure definition is for SMV */
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_SMV_BUFF];
} ggbuffDescSMV_t;

/* This structure definition is for QCELP13 */
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_QCELP13_BUFF];
} ggbuffDescQCELP13_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_VCU_BUFF];
} ggbuffDescVCU_t;

#if defined(AUDIO)
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_ADU_BUFF];
} ggbuffDescADU_t;
#endif

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_VAU_BUFF];
} ggbuffDescVAU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_ECU_BUFF];
} ggbuffDescECU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_TEC_BUFF];
} ggbuffDescTEC_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_TECENG_CH_BUFF];
} ggbuffDescTECENG_CHAN_t;


typedef struct {
  int numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_AER_BUFF];
} ggbuffDescAER_t;

typedef struct {
  int numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_AGC_BUFF];
} ggbuffDescAGC_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_PIU_BUFF];
} ggbuffDescPIU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_TDU_BUFF];
} ggbuffDescTDU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_VPU_BUFF];
} ggbuffDescVPU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_PVP_BUFF];
} ggbuffDescPVP_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_PLR_BUFF];
} ggbuffDescPLR_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_MSURTP_BUFF];
} ggbuffDescMSURTP_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_MSURTCP_BUFF];
} ggbuffDescMSURTCP_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_TGU_BUFF];
} ggbuffDescTGU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_CID_BUFF];
} ggbuffDescCID_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_RSU_BUFF];
} ggbuffDescRSU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_ACU_BUFF];
} ggbuffDescACU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_RCU_BUFF];
} ggbuffDescRCU_t;

/*
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_NEU_BUFF];
} ggbuffDescNEU_t; */

typedef struct {
  int numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_ANU_BUFF];
} ggbuffDescANU_t;

typedef struct {
  int numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_PRU_BUFF];
} ggbuffDescPRU_t;

typedef struct {
  int numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_TTU_BUFF];
} ggbuffDescTTU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_FM_BUFF];
} ggbuffDescFM_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_FIU_BUFF];
} ggbuffDescFIU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_MRU_BUFF];
} ggbuffDescMRU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_MDMCP_BUFF];
} ggbuffDescMDMCP_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_DMP_BUFF];
} ggbuffDescDMP_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_MULTSAMPVC_BUFF];
} ggbuffDescMultsampvc_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_BERT_BUFF];
}ggbuffDescBERT_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_NXU_BUFF];
}ggbuffDescNXU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_NPIU_BUFF];
}ggbuffDescNPIU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_EQI_BUFF];
}ggbuffDescEQI_t;


typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_VQM_BUFF];
}ggbuffDescVQM_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_MPP_BUFF];
} ggbuffDescMPP_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_LLMRU_BUFF];
} ggbuffDescLLMRU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_SPRTP_BUFF];
} ggbuffDescSPRTP_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_TX_NMU_BUFF];
} ggbuffDescTxNMU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_RX_NMU_BUFF];
} ggbuffDescRxNMU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_PLC_BUFF];
} ggbuffDescPLC_t;

#if defined (VEU)
typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_VEU_BUFF];
} ggbuffDescVEU_t;
#endif

#if defined (HDVIDEO)
typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_VTK_BUFF];
}ggbuffDescVTK_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_ATK_BUFF];
}ggbuffDescATK_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_TSU_BUFF];
}ggbuffDescTSU_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_CUSTOM_ECO_BUFF];
}ggbuffDescCUSTOM_ECO_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_OVLY_BUFF];
}ggbuffDescOVLY_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_MMCU_BUFF];
}ggbuffDescMMCU_t;
#endif

#if defined(HDCT) || defined(HDVIDEO)
typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_VPPU_BUFF];
}ggbuffDescVPPU_t;
typedef struct {
  int  numBuffs;
  ggbufDesc_t buffDesc[GG_NUM_TFTP_BUFF];
}ggbuffDescTFTP_t;

#endif

#if defined(VOICE) || defined(VOICE_FAX)
/* This structure defines all of the encoder OR decoder buffers
 * required by each VCA
 */
typedef struct {
#ifdef GG_INCLUDE_PCM
  ggbuffDescPCM_t    buffDescPCM;
#endif
#ifdef GG_INCLUDE_G726
  ggbuffDescADPCM_t  buffDescADPCM;
#endif
#ifdef GG_INCLUDE_G729AB
  ggbuffDescG729AB_t buffDescG729AB;
#endif
#ifdef GG_INCLUDE_G729
  ggbuffDescG729_t   buffDescG729;
#endif
#ifdef GG_INCLUDE_G729E
  ggbuffDescG729E_t  buffDescG729E;
#endif
#ifdef GG_INCLUDE_G723
  ggbuffDescG723_t   buffDescG723;
#endif
#ifdef GG_INCLUDE_G728
  ggbuffDescG728_t   buffDescG728;
#endif
#ifdef GG_INCLUDE_GSMEFR
  ggbuffDescGSMEFR_t buffDescGSMEFR;
#endif
#ifdef GG_INCLUDE_GSMFR
  ggbuffDescGSMFR_t  buffDescGSMFR;
#endif
#ifdef GG_INCLUDE_G722
  ggbuffDescG722_t   buffDescG722;
#endif
#ifdef GG_INCLUDE_ILBC
  ggbuffDescILBC_t   buffDescILBC;
#endif
#ifdef GG_INCLUDE_GSMAMR
  ggbuffDescGSMAMR_t buffDescGSMAMR;
#endif
#ifdef GG_INCLUDE_G722_2
  ggbuffDescGSM_WBAMR_t buffDescGSM_WBAMR;
#endif
#ifdef GG_INCLUDE_EVRC
  ggbuffDescEVRC_t   buffDescEVRC;
#endif
#ifdef GG_INCLUDE_SMV
  ggbuffDescSMV_t    buffDescSMV;
#endif
#ifdef GG_INCLUDE_QCELP13
  ggbuffDescQCELP13_t    buffDescQCELP13;
#endif
#if (defined (AUDIO))
  ggbuffDescMP3_t    buffDescMP3;
#endif
} ggbuffDescCodec_t;
#endif

/*  Memory buffer allocation table structure: */
#ifdef FAX  /* Fax-only builds */
typedef struct {
  ggbuffDescPIU_t buffDescPIU;
  ggbuffDescFM_t  buffDescFM;
  ggbuffDescFIU_t buffDescFIU;
  ggbuffDescACU_t buffDescACU;
  ggbuffDescRCU_t buffDescRCU;
  ggbuffDescMSU_t buffDescMSU;
} ggchBuffDesc_t;

#else      /* Voice-only and Voice-Fax builds */
typedef struct {
#if defined(VOICE) || defined(VOICE_FAX)
  ggbuffDescCodec_t  buffDescEncoder;
  ggbuffDescCodec_t  buffDescDecoder;

  ggbuffDescVCU_t    buffDescVCU;
  ggbuffDescVAU_t    buffDescVAU;
#endif

#ifdef LEC_ECU
  ggbuffDescECU_t    buffDescECU;
#endif
#ifdef LEC_TEC
  ggbuffDescTEC_t    buffDescTEC;
#endif

#ifdef LEC_TECENG
  ggbuffDescTECENG_CHAN_t  buffDescTECENG_CHAN;
#endif

#ifdef GG_INCLUDE_AER
  ggbuffDescAER_t    buffDescAER;
  ggbuffDescAGC_t    buffDescAGC;
#endif
  ggbuffDescPIU_t    buffDescPIU;
#ifdef GG_INCLUDE_TDU
  ggbuffDescTDU_t    buffDescTDU;
#endif

#ifdef GG_TDU_SAVEBUF_EXT
  ggbuffDescTDU_t    buffDescTDUFax;
#endif
#ifdef PLR_VOICE_OVERLAY
  ggbuffDescTDU_t    buffDescTDULITE;
#endif
#if defined(VOICE) || defined(VOICE_FAX)
  ggbuffDescVPU_t    buffDescVPU;
  ggbuffDescPVP_t    buffDescPVP;
  ggbuffDescTGU_t    buffDescTGU;
  ggbuffDescCID_t    buffDescCID;
#endif
#ifdef GG_INCLUDE_RSU
  ggbuffDescRSU_t    buffDescRSU;
#endif
#if (defined(GG_INCLUDE_AAL1) || defined(GG_INCLUDE_AAL2))
  ggbuffDescACU_t    buffDescACU;
#endif
#ifdef GG_INCLUDE_RTP
  ggbuffDescRCU_t    buffDescRCU;
#endif
#if (defined(PLRRED) || defined(PLRFEC))
#ifdef PLR_VOICE_OVERLAY
#if (MAX_RED_LEVEL_TEXT > 0)
  ggbuffDescPLR_t    buffDescPLR; /* For TEXT */
#endif
#if ((MAX_RED_LEVEL_VOICE > 0) || (MAX_FEC_LEVEL_2198 > 0))
  ggbuffDescPLR_t    buffDescPLRVBD; /* For VBD */
#endif
#if ((MAX_RED_LEVEL_FAX > 0) && defined(GG_INCLUDE_FAX_RELAY))
  ggbuffDescPLR_t    buffDescPLRFAX; /* For FAX */
#endif
#else /* OVERLAY */
  ggbuffDescPLR_t    buffDescPLR; /* For everything */
#endif
#endif
#ifdef GG_INCLUDE_MSU
  ggbuffDescMSURTP_t  buffDescMSURTP;
  ggbuffDescMSURTCP_t buffDescMSURTCP;
#endif
#ifdef GG_INCLUDE_NET_ENCAP
  ggbuffDescNEU_t    buffDescNEU;
#endif
#ifdef GG_INCLUDE_ANNOUNCEMENTS
  ggbuffDescANU_t    buffDescANU;
#endif
#ifdef GG_INCLUDE_PRU
  ggbuffDescPRU_t    buffDescPRU;
#endif
#ifdef TTU
  ggbuffDescTTU_t    buffDescTTU;
#endif
#if defined(FAX) || defined(VOICE_FAX)  ||  defined(FAX_RELAY)
  ggbuffDescFM_t     buffDescFM;
  ggbuffDescFIU_t    buffDescFIU;
#endif
#ifdef GG_INCLUDE_MODEM_RELAY
  ggbuffDescMRU_t    buffDescMRU;
#endif
#ifdef GG_INCLUDE_MDMCP
  ggbuffDescMDMCP_t  buffDescMDMCP;
#endif
#if defined(GG_INCLUDE_DATA_MODEM) || defined(GG_INCLUDE_LINK_LAYER_MODEM_RELAY)
  ggbuffDescDMP_t    buffDescDMP;
#endif
#if (defined (GG_INCLUDE_CLEAR_DATA) || defined (GG_INCLUDE_FAX_MODEM) || defined (GG_INCLUDE_DATA_MODEM) || defined (AUDIO))
  ggbuffDescMPP_t    buffDescMPP;
#endif
#ifdef GG_INCLUDE_BERT
  ggbuffDescBERT_t   buffDescBERT;
#endif
#ifdef NX64
  ggbuffDescNXU_t    buffDescNXU;
#endif

#ifdef EQI
  ggbuffDescEQI_t    buffDescEQI;
#endif

#ifdef GG_INCLUDE_VQM
  ggbuffDescVQM_t    buffDescVQM;
#endif
#ifdef GG_INCLUDE_LINK_LAYER_MODEM_RELAY
  ggbuffDescLLMRU_t  buffDescLLMRU;
  ggbuffDescSPRTP_t  buffDescSPRTP;
#endif
#if defined(VOICE) || defined(VOICE_FAX)
  ggbuffDescTxNMU_t  buffDescTxNMU;
  ggbuffDescRxNMU_t  buffDescRxNMU;
  ggbuffDescPLC_t    buffDescPLC;
#endif
#if (defined(GG_MULTSAMPVC) || defined(GG_WBPACKET))
  ggbuffDescMultsampvc_t buffDescMultsampvc;
#endif
#ifdef VEU
  ggbuffDescVEU_t    buffDescVEU;
#endif
#if defined (AUDIO)
  ggbuffDescADU_t    buffDescADU;
#endif
#ifdef NX64
  ggbuffDescNPIU_t   buffDescNPIU;
#endif
#ifdef HDVIDEO
  ggbuffDescVTK_t        buffDescVTK;
  ggbuffDescATK_t        buffDescATK;
  ggbuffDescTSU_t        buffDescTSU;
  ggbuffDescCUSTOM_ECO_t buffDescCUSTOM_ECO;
  ggbuffDescOVLY_t       buffDescOVLY;
  ggbuffDescMMCU_t       buffDescMMCU;
#endif
#if (defined(HDCT) || defined(HDVIDEO))
  ggbuffDescVPPU_t       buffDescVPPU;
  ggbuffDescTFTP_t       buffDescTFTP;
#endif
} ggchBuffDesc_t;
#endif

/* External definition of memory buffer allocation table */
extern relocQualifier ggchBuffDesc_t ggchBuffDesc[];
#endif /* GGCHBUFDESC_H */

/* nothing past this point */
