/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _GGCODECSWAP_H
#define _GGCODECSWAP_H

#include "types.h"

#define GG_MAX_SUPPORTED_CHANNELS 48

#define GG_NUM_G723_ENC_SWAP_BUFS 1
#define GG_NUM_G723_DEC_SWAP_BUFS 1
#define GG_NUM_G729AB_ENC_SWAP_BUFS 1
#define GG_NUM_G729AB_DEC_SWAP_BUFS 1

typedef struct {
  void *srcAddr[GG_MAX_SUPPORTED_CHANNELS];
  void *destAddr;
  tint size;
 }ggSwapBuffDesc_t;


typedef struct{
 tint num_enc_swap_bufs;
 ggSwapBuffDesc_t *encSwapBuffDesc;
 tint num_dec_swap_bufs;
 ggSwapBuffDesc_t *decSwapBuffDesc;
}ggCodecSwapBuffDesc_t;


typedef struct{
   ggCodecSwapBuffDesc_t *g729abSwapBuffDesc;
   ggCodecSwapBuffDesc_t *g723SwapBuffDesc;
}ggCodecSwapDescInfo_t;


extern ggCodecSwapDescInfo_t ggCodecSwapDescInfo;

/* G723 */
extern ggSwapBuffDesc_t g723EncSwapBuffDesc[];
extern ggSwapBuffDesc_t g723DecSwapBuffDesc[];

/* G729AB */
extern ggSwapBuffDesc_t g729abEncSwapBuffDesc[];
extern ggSwapBuffDesc_t g729abEncSwapBuffDesc[];

/* G723 */
extern ggCodecSwapBuffDesc_t g723SwapBuffDesc;
extern ggCodecSwapBuffDesc_t g729abSwapBuffDesc;

# endif

/* Nothing Beyond this point */
