/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include "types.h"
#include "utlxferpkt.h"
#include "util.h"
#include "ggport.h"
#include "targethw.h"     /* fixed hardware definitions */
#include "gghwcfg.h"
#include "ggcfg.h"
#include "ggswcfg.h"
#include "ggmemmap.h"
#include "dpriram.h"
#include "hal.h"
#include "mgmicdsp.h"     /* ICD message definitions */
#include "common/siu.h"
#include "common/siuloc.h"
#include <ti/mas/util/debug.h>
#include <ti/mas/vppu/vppu.h>
#include <ti/mas/tftp/tftp.h>
#include "vct/src/siuVctTftp.h"

const tuint gg_rttrace_bufsize = GG_RTTRACE_BUFSIZE;
/* Variables defined in the linker command file */
#ifdef GG_BOOT_OVERLAY
extern tulong boverlay_load_start_loc;
extern tulong boverlay_run_start_loc;
extern tulong boverlay_load_size_loc;
#endif
/* RCU Call Table definitions and ACU API */
#ifdef GG_INCLUDE_RTP
#include <ti/mas/rcu/rcu.h>
#endif
/* NEU Call Table definitions and NEU API */
#ifdef GG_INCLUDE_NET_ENCAP
#include <ti/mas/neu/neu.h>
#include <ti/mas/neu/neuproto.h>
#endif

/* rcu Call Table */
rcuCallTable_t gg_rcu_ctable = {
#if defined (GG_INCLUDE_RTP) || defined (GG_INCLUDE_FAX_RELAY_T38)
    rcuClose,
    rcuControl,
    rcuDelete,
    rcuGetSizes,
    rcuNew,
    rcuOpen,
    rcuReceiveIn,
    rcuSendIn,
    rcuIsr,
    rcuPktViolThrottle,
    rcuUpdateTxStats
#else
    NULL,   /* rcuClose */
    NULL,   /* rcuControl */
    NULL,   /* rcuDelete */
    NULL,   /* rcuGetSizes */
    NULL,   /* rcuNew */
    NULL,   /* rcuOpen */
    NULL,   /* rcuReceiveIn */
    NULL,   /* rcuSendIn */
    NULL,   /* rcuPktViolThrottle */
    NULL    /* rcuUpdateTxStats */
# endif
};  /* gg_rcu_ctable */

/* RTCP Call Table */
rcuRtcpCallTable_t gg_rcuRtcp_ctable = {
#if defined (GG_INCLUDE_RTCP) && defined (GG_INCLUDE_RTP)
    rcuRtcpOpen,
    rcuRtcpGetInstSize,
    rcuRtcpSendIn,
    rcuRtcpReceiveIn,
    rcuRtcpConfig,
    rcuRtcp_sdes_update,
    rcuRtcp_send_hpkt,
    rcuRtcp_statsReq,
    rcuRtcp_update_tx,
    rcuRtcp_update_rx,
    rcuRtcp_init_stats,
    rcuRtcp_getXRVoip
#else
    NULL, /* rcuRtcpOpen         */
    NULL, /* rcuRtcpGetInstSize  */
    NULL, /* rcuRtcpSendIn       */
    NULL, /* rcuRtcpReceiveIn    */
    NULL, /* rcuRtcpConfig       */
    NULL, /* rcuRtcp_sdes_update */
    NULL, /* rcuRtcp_send_hpkt   */
    NULL, /* rcuRtcp_statsReq    */
    NULL, /* rcuRtcp_update_tx   */
    NULL, /* rcuRtcp_update_rx   */
    NULL, /* rcuRtcp_init_stats  */
    NULL  /* rcuRtcp_getXRVoip   */
#endif
}; /* RTCP call table */

vppuCallTable_t gg_vppu_ctable = {
    vppuClose,
    vppuControl,
    vppuDelete,
    vppuGetSizes,
    vppuNew,
    vppuOpen,
    vppuReceiveIn,
    vppuSendIn,
    vppuRxGetFrame,
    NULL
};

/* neu Call Table */
neuCallTable_t gg_neu_ctable = {
#if defined GG_INCLUDE_NET_ENCAP
    neuClose,
    neuControl,
    neuDelete,
    neuGetSizes,
    neuNew,
    neuOpen,
#if (GG_UTOPIA_PORTS_USED >= 1)
    neuReceiveInUt,
#else
    NULL,
#endif
    neuSendIn,
    neuPktViolThrottle,
#if ((GG_GMAC_PORTS_USED >= 1) || defined(PA_SS))
    neuReceiveInGmac,
    neuGetInstance,
#else
    NULL,
    NULL,
#endif
#else
    NULL,   /* neuClose */
    NULL,   /* neuControl */
    NULL,   /* neuDelete */
    NULL,   /* neuGetSizes */
    NULL,   /* neuNew */
    NULL,   /* neuOpen */
    NULL,   /* neuReceiveIn */
    NULL,   /* neuSendIn */
    NULL,    /* neuPktViolThrottle */
    NULL,   /* neuReceiveInGmac */
    NULL    /* neuGetInstance */
# endif
};  /* gg_neu_ctable */

cfgDma_t cfgTdmDma[] = {

#if (GG_TDM_PORTS_USED >= 1)
    {
  (tuint *) GG_BSP_TX_ADDR,      /* TDM A, DMA tx buffer start  */
  (tuint *) GG_BSP_TX_END_ADDR,        /* TDM A, DMA tx buffer end    */
  (tuint *) GG_BSP_RX_ADDR,      /* TDM A, DMA rx buffer start  */
  (tuint *) GG_BSP_RX_END_ADDR,        /* TDM A, DMA rx buffer end    */

#if (GG_TDM_PORTS_USED == 1)
    }
#else
    },
#endif

#endif

#if (GG_TDM_PORTS_USED >= 2)
    {
  (tuint *) GG_DSIGT_TX_ADDR,      /* TDM B, DMA tx buffer start  */
  (tuint *) GG_DSIGT_TX_END_ADDR,        /* TDM B, DMA tx buffer end    */
  (tuint *) GG_DSIGT_RX_ADDR,      /* TDM B, DMA rx buffer start  */
  (tuint *) GG_DSIGT_RX_END_ADDR,        /* TDM B, DMA rx buffer end    */
#if (GG_TDM_PORTS_USED == 2)
    }
#else
    },
#endif

#endif

#if (GG_TDM_PORTS_USED >= 3)
    {
  (tuint *) GG_DSIGT_TX_ADDR,      /* TDM C, DMA tx buffer start  */
  (tuint *) GG_DSIGT_TX_END_ADDR,       /* TDM C, DMA tx buffer end    */
  (tuint *) GG_DSIGT_RX_ADDR,           /* TDM C, DMA rx buffer start  */
  (tuint *) GG_DSIGT_RX_END_ADDR,       /* TDM C, DMA rx buffer end    */
    }
#endif

   };


/****************************************************************************
 * Array of tdm configurations
 ****************************************************************************/
#if (GG_TDM_PORTS_USED >= 1)

  tint cfgTdmSubFrameRate[] = { GG_TDM_SUB_FRAME_RATES };
  tint cfgTdmWordSizes[]    = { GG_TDM_WORD_SIZES      };
  tint cfgTdmBufInc[]       = { GG_TDM_BUF_INC         };
  tint cfgTdmMaxPhase[]     = { GG_TDM_MAX_PHASES      };
  tint cfgTdmMaxMegaPhase[] = { GG_TDM_MAX_MEGA_PHASES };

#endif

/*****************************************************************************
 * Declaration of the build time tdm configuration structure.
 *****************************************************************************/
#if (GG_TDM_PORTS_USED >= 1)
  cfgTdmConfig_t cfgTdmConfig =   {
                                     cfgTdmDma,
                                     cfgTdmSubFrameRate,
                                     cfgTdmWordSizes,
                                     cfgTdmBufInc,
                                     cfgTdmMaxPhase,
                                     cfgTdmMaxMegaPhase };
#endif


/*****************************************************************************
 * Array of utopia DMA buffers
 *****************************************************************************/
#if (GG_UTOPIA_PORTS_USED >= 1)

cfgDma_t cfgUtopiaDma[] = {

  {
#if (GG_UTOPIA_PORTS_USED >= 1)
  (tuint *) GG_UTOPIA_TX_A_START,     /* PKT A, DMA tx buffer start  */
  (tuint *) GG_UTOPIA_TX_A_END,       /* PKT A, DMA tx buffer end    */
  (tuint *) GG_UTOPIA_RX_A_START,     /* PKT A, DMA rx buffer start  */
  (tuint *) GG_UTOPIA_RX_A_END        /* PKT A, DMA rx buffer end    */
#else
  (tuint *) 0,     /* PKT A, DMA tx buffer start  */
  (tuint *) 0,       /* PKT A, DMA tx buffer end    */
  (tuint *) 0,     /* PKT A, DMA rx buffer start  */
  (tuint *) 0        /* PKT A, DMA rx buffer end    */
#endif

#if (GG_UTOPIA_PORTS_USED == 1)
  }
#else
  },
#endif

#if (GG_UTOPIA_PORTS_USED >= 2)
  {
  (tuint *) &_lcf_hal_pkt_tx_b_start,     /* PKT B, DMA tx buffer start  */
  (tuint *) &_lcf_hal_pkt_tx_b_end,       /* PKT B, DMA tx buffer end    */
  (tuint *) &_lcf_hal_pkt_rx_b_start,     /* PKT B, DMA rx buffer start  */
  (tuint *) &_lcf_hal_pkt_rx_b_end        /* PKT B, DMA rx buffer end    */
  }
#endif

  };

#endif

/****************************************************************************
 * Array of GMAC DMA buffers
 ****************************************************************************/
#if (GG_GMAC_PORTS_USED == 0)

cfgDma_t  cfgGmacDma[] = { NULL };

#else

cfgDma_t  cfgGmacDma[] = {

#if (GG_GMAC_PORTS_USED >= 1)
  {
  (tuint *) GG_GMAC_TX_A_START,  /* gmacTxBufStart            */
  (tuint *) GG_GMAC_TX_A_END,    /* gmacTxBufEnd              */
  (tuint *) GG_GMAC_RX_A_START,  /* gmacRxBufStart            */
  (tuint *) GG_GMAC_RX_A_END,    /* gmacRxBufEnd              */
#if (GG_GMAC_PORTS_USED == 1)
  }
#else
  },
#endif

#endif

#if (GG_GMAC_PORTS_USED >= 2)
  {
  (tuint *) GG_GMAC_TX_B_START,  /* gmacTxBufStart            */
  (tuint *) GG_GMAC_TX_B_END,    /* gmacTxBufEnd              */
  (tuint *) GG_GMAC_RX_B_START,  /* gmacRxBufStart            */
  (tuint *) GG_GMAC_RX_B_END,    /* gmacRxBufEnd              */
  }
#endif

  };

#endif

/***********************************************************************************
 * Declaration: Array of gmac rx assembly buffers
 ***********************************************************************************/
#if (GG_GMAC_PORTS_USED == 0)

cfgMemBlk_t cfgGmacAssemMem[] = { NULL };

#else

cfgMemBlk_t cfgGmacAssemMem[] = {

#if (GG_GMAC_PORTS_USED >= 1)
  {
  (tuint *) GG_GMAC_RX_A_ASSEMB_START,
  (tuint *) GG_GMAC_RX_A_ASSEMB_END
#if (GG_GMAC_PORTS_USED == 1)
  }
#else
  },
#endif

#endif

#if (GG_GMAC_PORTS_USED >= 2)
  {
  (tuint *) GG_GMAC_RX_B_ASSEMB_START,
  (tuint *) GG_GMAC_RX_B_ASSEMB_END
  }
#endif

  };

#endif
/****************************************************************************
 * Array of unit buffer sizes (CPPI block sizes) for mac
 ****************************************************************************/
#if (GG_GMAC_PORTS_USED == 0)
  tint cfgGmacBufSizeWordsTx[]   = { 0 };
  tint cfgGmacBufSizeWordsRx[]   = { 0 };
  tint cfgGmacRxAssemSizeWords[] = { 0 };
#else
  tint cfgGmacBufSizeWordsTx[]   = { GG_GMAC_TX_BUFSIZE_WORDS };
  tint cfgGmacBufSizeWordsRx[]   = { GG_GMAC_RX_BUFSIZE_WORDS };
  tint cfgGmacRxAssemSizeWords[] = { GG_GMAC_RX_ASSEMSIZE_WORDS };
#endif


/****************************************************************************
 * Array of utopia configurations
 ****************************************************************************/
#if (GG_UTOPIA_PORTS_USED >= 1)

  tint cfgUtPortNum[]   = { GG_UTOPIA_PHYS_PORTS  };
  tint cfgUtSlaveId[]   = { GG_UTOPIA_PORT_SLAVE_ID  };
  halUtopiaMode_t cfgUtRouteMode[] = { GG_UTOPIA_ROUTE_MODE  };

#else
  tint cfgUtPortNum[]   = { 0  };
  tint cfgUtSlaveId[]   = { 0  };
  halUtopiaMode_t cfgUtRouteMode[] = { HAL_UTOPIA_PASS_THRU };

#endif

/****************************************************************************
 * The Utopia configuration structure
 ****************************************************************************/
 #if (GG_UTOPIA_PORTS_USED >= 1)

  cfgUtConfig_t cfgUtConfig = {
                                     cfgUtopiaDma,
                                     cfgUtPortNum,
                                     cfgUtSlaveId,
                                     cfgUtRouteMode  };

 #else

   cfgUtConfig_t cfgUtConfig = { NULL };

 #endif
/************************************************************************************
 * Declaration: DSIU port structures - Arrays of port information
 ************************************************************************************/
/* The values in ggConfig are default to 4 channel VPBX with 8 bit PCM samples
   in MULaw */
ggConfig_t ggConfig = {
  0,                            /* MGB_SERIAL_PORT_CONF_8BIT  = 0
                                 * MGB_SERIAL_PORT_CONF_16BIT = 1  */
  GG_BSP_SEGMENT_LENGTH,
  GG_SOFTWARE_SEGMENT_LENGTH,
  GG_BSP_MAX_PHASE,
  THW_MULAW_SILENCE,            /* GG_PCM_SILENCE_PATTERN */
  8,                            /* PCM samples are 8 bit long */
  GG_MAX_NUM_CHANNELS,          /* maximum number of full channels */
  GG_MAX_AUX_PCM_CHANNELS,  /* maximum number of aux PCM channels */
#if 0
  ggVcaInfo,                /* NULL terminated table array */
  gg_vca_code,                  /* NULL Terminated codec run and load addresses
                                 * table array */
#endif
#ifdef AUDIO
  gg_ada_tables,                /* NULL terminated table array */
  gg_ada_code,                  /* NULL Terminated codec run and load addresses
                                 * table array */
#endif
#if 0
#ifdef GG_INCLUDE_AER
#if (GG_NUM_AER_PORTS > SIU_MAX_AER_PORTS)
#error Number of AER ports exceeds the allowed maximum (see siuloc.h - SIU_MAX_AER_PORTS)
#endif
  GG_NUM_AER_PORTS,
  (GG_AER_INST0_INFO & AER_INST_TAIL_MASK) * (GG_SAMPLING_RATE/1000),
  (GG_AER_INST1_INFO & AER_INST_TAIL_MASK) * (GG_SAMPLING_RATE/1000),
  (GG_AER_INST2_INFO & AER_INST_TAIL_MASK) * (GG_SAMPLING_RATE/1000),
  (GG_AER_INST3_INFO & AER_INST_TAIL_MASK) * (GG_SAMPLING_RATE/1000),
#else
  0,
  0,
  0,
  0,
  0,
#endif
#if (defined(LEC_ECU))
  GG_MAX_ECHO_TAIL_LENGTH*(GG_SAMPLING_RATE/1000),       /* Maximum ECU Tail Length */
  GG_MAX_ECHO_FILTER_SEG_LENGTH*(GG_SAMPLING_RATE/1000), /* Maximum ECU Filter segment Length */
  GG_MAX_ECHO_FILTER_SEGS,      /* Maximum ECU segments (separate hybrids) */
  0,
#elif (defined(LEC_TEC))
  GG_MAX_ECHO_TAIL_LENGTH,
  GG_MAX_ECHO_FILTER_SEG_LENGTH,
  0,
  GG_LEC_MIPS_OFFSET,
#elif (defined(LEC_TECENG))
  GG_MAX_ECHO_TAIL_LENGTH,
  GG_MAX_ECHO_FILTER_SEG_LENGTH,
  0,
  0,
#else
  0,
  0,
  0,
  0,
#endif
#if defined(GG_INCLUDE_AER) || (defined(LEC_ECU)) || defined(VEU)
  TRUE,
#else
  FALSE,                        /* neither ECU or AER is included */
#endif
#endif
  GG_SAMPLING_RATE,             /* Sampling Rate in HZ (8000 or 16000) */
  GG_MAX_FAX_FRAME_LENGTH,      /* maxFaxFrameLength */
#ifdef GG_INCLUDE_DATA_MODEM
  GG_MAX_MODEM_FRAME_LENGTH,    /* maxModemFrameLength */
  GG_MAX_MODEM_CTRL_FRAME_LENGTH, /* maxModemCtrlFrameLength */
#else
  0,                            /* maxModemFrameLength */
  0,                            /* maxModemCtrlFrameLength */
#endif
  GG_MAX_VOICE_FRAME_LENGTH,    /* maxVoiceFrameLength */
  GG_MAX_CODEC_FRAME_LENGTH,    /* maxCodecFrameLength */
#if (defined(AUDIO))
  GG_MAX_AUDIO_FRAME_LENGTH,  /*Audio Codec Frame Length*/
  GG_AUDIO_OVER_SAMPLING_FACTOR,
  GG_MAX_AUDIO_CODE_WORD_SEG_SIZE,
  GG_MAX_AUDIO_SAMPLES_PER_CODE_WORD,
#endif
  GG_VOICE_FRAME_SIZE,          /* voiceFrameSize */
  GG_MAX_VCA_INST_SIZE,         /* maxVcaInstSize */
  GG_STAGGER_RESOLUTION,        /* Staggering resolution number */
  GG_MAX_CWORD_SEG_SIZE,        /* maxCodeWordSegSize */
  GG_MAX_CWORD_SEG_SIZE_RTP,    /* maxCodeWordSegSizeRTP */
  GG_MAX_SEGMENTS_PER_PACKET,   /* maxSegmentsPerPacket */
  GG_MAX_CORE_SEGMENTS_PER_PACKET,       /* maxCoreSegmentsPerPacket */
  GG_MAX_VOICE_PLAYOUT_FIFO_SIZE,        /* maxVoicePlayoutFIFOSize */
  GG_PIU_RX_DELAY,                       /* rx delay depends on G723 build or not */
  GG_FAX_PIU_RX_DELAY,                   /* PIU rx delay for FAX */
#ifdef GG_INCLUDE_DATA_MODEM
  GG_MODEM_PIU_RX_DELAY,                 /* PIU rx delay for data modem */
#else
  0,                                     /* PIU rx delay for data modem */
#endif
  0,                                     /* Whether platform supports PCM Sync
                                          * interrupt or not. Default is 0 for
                                          * VPBX which supports PCM sync. */
#ifdef GG_REP
  GG_MAX_REPLICATE_CHANNELS,
#else
  0,
#endif

#ifdef GG_MAX_SIU_MSG_FIFO_LEN
  GG_MAX_SIU_MSG_FIFO_LEN,              /* Outgoing message fifo length, words */
#else
  0,
#endif
#ifdef GG_MAX_SIU_SIG_FIFO_LEN
  GG_MAX_SIU_SIG_FIFO_LEN,              /* Outgoing signaling message fifo length, words */
#else
  0,
#endif
#ifdef GG_MAX_SIU_LOW_FIFO_LEN          /* Low priority messages to micro */
  GG_MAX_SIU_LOW_FIFO_LEN,              /* are only supported in high density */
#else
  0,
#endif
#if (!defined(HDVIDEO) && !defined(HDCT))
  (void *) GG_SIU_FAX_SCRATCH_START,    /* siuFaxScratch. Scratch used by SIU in FAX mode */
                                        /* to accumulate 10ms data at the PCM->net split  */
#endif
#ifdef RESIZABLEHPI
#ifndef GG_DSP_MIC_XCHG_LENGTH
#error GG_DSP_MIC_XCHG_LENGTH must be defined in ggswcfg.h. The old value was 896
#else
   GG_DSP_MIC_XCHG_LENGTH,              /* DSP-micro packet exchange buffer size */
   (void *)&dprNormal                   /* Base address of HPI memory */
#endif
#endif
#ifdef GG_HD_HPI
  GG_HD_HPI_RX_START_ADDR,              /* High density HPI Rx FIFO start addr */
  GG_HD_HPI_RX_LENGTH,                  /* High density HPI Rx FIFO length     */
  GG_HD_HPI_TX_START_ADDR,              /* High density HPI Tx FIFO start addr */
  GG_HD_HPI_TX_LENGTH,                  /* High density HPI Tx FIFO length     */
  GG_HD_HPI_MAGIC,                      /* High density HPI magic value        */
#else
  0,
  0,
  0,
  0,
  0,
#endif

#ifdef GG_GCI
  GG_GCI_TX_ADDR,                        /* SecTxAddr for GCI serial port    */
  GG_GCI_RX_ADDR,                        /* SecRxAddr for GCI serial port    */
  GG_GCI_TX_END_ADDR,                    /* SecTxAddr end address for GCI    */
  GG_GCI_RX_END_ADDR,                    /* SecRxAddr end address for GCI    */
  GG_GCI_SIZE,                           /* secBufSize for GCI serial port   */
  GG_GCI_FRAME_SIZE,                     /* secFrameSize for GCI serial port */
  GG_MAX_NUM_CHANNELS,                   /* same number of GCI and voice     */
  0,                                     /* secMaxPhase = 0, no phases       */
#elif defined(GG_INCLUDE_SIOU)
  GG_BSP_TX_ADDR_PKT,                    /* secTxAddr for SIOU serial port    */
  GG_BSP_RX_ADDR_PKT,                    /* secRxAddr for SIOU serial port    */
  GG_BSP_TX_END_ADDR_PKT,                /* Tx SIOU buffer end address        */
  GG_BSP_RX_END_ADDR_PKT,                /* Rx SIOU buffer end address        */
  GG_SIOU_BSP_SIZE,                      /* secBufSize for SIOU serial port   */
  GG_SIOU_SEGMENT_LENGTH,                /* secFrameSize for SOIU serial port */
  GG_MAX_NUM_CHANNELS,                   /* same number for SIOU and voice    */
  0,                                     /* secMaxPhase = 0, no phases        */
#elif defined (GG_INCLUDE_DSIGT)
  GG_DSIGT_TX_ADDR,                      /* secTxAddr for CAS serial port    */
  GG_DSIGT_RX_ADDR,                      /* secRxAddr for CAS serial port    */
  GG_DSIGT_TX_END_ADDR,                  /* CAS tx serial buffer end address */
  GG_DSIGT_RX_END_ADDR,                  /* CAS rx serial buffer end address */
  GG_DSIGT_SIZE,                         /* secBufSize for CAS serial port   */
  GG_DSIGT_FRAME_SIZE,                   /* secFrameSize for CAS serial port */
  GG_DSIGT_NUM_CHANNELS,                 /* can be different than voice      */
#ifdef GG_DSIGT_ISR_SKIP
  GG_DSIGT_ISR_SKIP + 1,                 /* maxPhase for dsigt               */
#else
  0,                                     /* maxPhase for dsigt, no skip      */
#endif
#else
  NULL,                                  /* secTxAddr                        */
  NULL,                                  /* secRxAddr                        */
  NULL,                                  /* secTxEndAddr                     */
  NULL,                                  /* secRxEndAddr                     */
  0,                                     /* secBufSize                       */
  0,                                     /* secFrameSize                     */
  0,                                     /* secChannels                      */
  0,                                     /* secMaxPhase                      */
#endif

  /* Tdm build time configuration */
  &cfgTdmConfig,

#ifdef GG_AIC_HEAP_PAGE
  GG_AIC_HEAP_PAGE,
#else
  0,                                      /* aicHeapPage  */
#endif

#ifdef GG_DSIGT_HEAP_PAGE
  GG_DSIGT_HEAP_PAGE,
#else
  0,                                      /* disgtHeapPage */
#endif

#ifdef GG_DSIGR_HEAP_PAGE
  GG_DSIGR_HEAP_PAGE,
#else
  0,                                      /* dsigrHeapPage */
#endif

#ifdef GG_CONF_HEAP_PAGE
  GG_CONF_HEAP_PAGE,
#else
  0,                                     /* confHeapPage */
#endif

#ifdef GG_DSIGR_NUM_CHANNELS
  GG_DSIGR_NUM_CHANNELS,
#else
  0,
#endif

  5 * (GG_SAMPLING_RATE/1000) / GG_BSP_SEGMENT_LENGTH,   /* pcmIntPer5ms */
  0 /* neu_sar */
//#ifdef GG_ENABLE_CHKSUM_CHECK
//  GG_CHKSUM_PROG_BLCK_COUNT,
//  GG_CHKSUM_LOCAL_BLCK_COUNT,
//#else
//  0,
//  0,
//#endif

#ifdef GG_INCLUDE_NEU_AAL5
  | GGCFG_NEU_SAR_AAL5
#endif
#ifdef GG_INCLUDE_NEU_SSSAR
  | GGCFG_NEU_SAR_SSSAR
#endif
  ,
/* Number of internally allocated FIU buffers (per ch) */
#ifdef GG_EMPLOY_GMP_FOR_FAX
  GG_NUM_INTERNAL_FIU_BUFS,
#else
  100, /* DON'T CHANGE THIS VALUE */
#endif

/* HAL related initialisations */
 0,/* GG_FIRST_TS,   */       /* Timeslot 1st channel opens                   */
 0, /*GG_TS_OFFSET,  */       /* Number of timeslots between channels         */
  GG_TDM_PORTS_USED,      /* Number of TDM ports used at once             */
  GG_UTOPIA_PORTS_USED,   /* Number of Utopia interfaces used at one time */

#ifdef GG_GMAC_PORTS_USED
  GG_GMAC_PORTS_USED,     /* Number of GMAC interfaces used at one time */
#else
  0,
#endif

  0,
#ifdef GG_NEU_UDP_CORE_ID_MASK
  GG_NEU_UDP_CORE_ID_MASK,
#else
  0,
#endif

/*  FIXME - A common name must be used accross builds! */
#if defined(GG_DATA_HEAP_HAL_START) && defined (GG_HAL_HEAP_START)
#error both GG_DATA_HEAP_HAL_START and GG_HAL_HEAP_START are defined.
#endif

#if defined (GG_DATA_HEAP_HAL_START)
  (tuint *) GG_DATA_HEAP_HAL_START,
  (tuint *) GG_DATA_HEAP_HAL_END,                                      /*******************************/
                                         /*******************************/
#elif defined (GG_HAL_HEAP_START)
  (tuint *) GG_HAL_HEAP_START,
  (tuint *) GG_HAL_HEAP_END,                                      /*******************************/
#else
#error GG_HEAP_HAL_START or GG_DATA_HEAP_HAL_START not defined
#endif

#if (defined(GG_UTOPIA_PORTS_USED) && defined(GG_GMAC_PORTS_USED))
 #if ((GG_UTOPIA_PORTS_USED >= 1) && (GG_GMAC_PORTS_USED >= 1))
  #error No software support for simultaneous mac and utopia
 #endif
#endif

#if (GG_UTOPIA_PORTS_USED >= 1)          /*********************************/
  GG_UTOPIA_XMT_BUF_SIZE_IN,             /* Transmit buffer size          */
  GG_UTOPIA_RCV_BUF_SIZE_IN,             /* Receive buffer size           */
  GG_UTOPIA_PORT_SLAVE_ID,                /* Port slave ID                 */
#else                                    /*********************************/
  0,
  0,
  0,
#endif
/*****************************************************************************
 * Pointers to system port configuration memory.
 *****************************************************************************/
#if 0
  &cfgUtConfig,                      /* utConfig */
#endif
  cfgGmacDma,                        /* gmacDmaMem                */
  cfgGmacAssemMem,
  cfgGmacBufSizeWordsTx,             /* gmacBufSizeWordsTx        */
  cfgGmacBufSizeWordsRx,             /* gmacBufSizeWordsRx        */
  cfgGmacRxAssemSizeWords,           /* gmacRxAssemSizeWords      */

#ifdef GG_DSIGT_ISR_SKIP
  GG_DSIGT_ISR_SKIP,
#else
  0,
#endif

#if defined(GG_C5561)
  siuC5561CoreBoot,
  siuInitCoreMhz5561,
#elif defined(GG_C6486)
  siuC6486CoreBoot,
  siuInitCoreMhz6486,
#elif defined(GG_C6608)
  siuC6608CoreBoot,
  siuInitCoreMhz6608,
#else
  NULL,
  NULL,
#endif
#if 0
#ifdef GG_INCLUDE_WIRELESS_CODECS
 &ggWirelessCodecAPI,
#else
 NULL,
#endif

#ifdef GG_INCLUDE_MULTIRATEMODE_CODECS
 &ggMultiRateModeCodecAPI,
#else
 NULL,
#endif
#endif
#ifdef GG_INCLUDE_BLOCK_ASYMETRIC
  ggBlockAsymetricTxRx,
#else
  NULL,
#endif

  GG_RTTRACE_INTERVAL,
  GG_RTTRACE_PKT_FREE,
  GG_GMC_SIZE,
  GG_NV_ECO_BUFF_NUM,
#ifdef GG_BOOT_OVERLAY
  (tulong)&boverlay_load_start_loc,
  (tulong)&boverlay_run_start_loc,
  (tulong)&boverlay_load_size_loc,
#else
  0l,
  0l,
  0l,
#endif
  GG_MAX_PKT2PKT_CHANNELS,

#if (!defined(HDVIDEO) && !defined(HDCT))
#ifdef GG_INCLUDE_MACR
  &macrConfig,
#else
  NULL,
#endif
#endif
};
#ifdef GG_SHARED_MEM_DATA_ALLOCATION
#pragma DATA_SECTION (ggCh2ID, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggID2Ch, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggMaxChannelsPerWord, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggBitmapBufIndexPerWord, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggBitmapLBRCBufIndexPerWord, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggDynBufAlloc, ".gg_nonshared_bufs");
#pragma DATA_SECTION (neuContext, ".gg_nonshared_bufs");
#pragma DATA_SECTION (rcuContext, ".gg_nonshared_bufs");
#pragma DATA_SECTION (rcuRtcpContext, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggConfig, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggCh2Slot, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggCh2SSlotBasic,             ".gg_nonshared_bufs")
#pragma DATA_SECTION (ggCh2SSlotExtA,              ".gg_nonshared_bufs")
#pragma DATA_SECTION (ggCh2SSlotExtB,              ".gg_nonshared_bufs")
#pragma DATA_SECTION (ggMaxSubSlotsPerWord,        ".gg_nonshared_bufs")
#pragma DATA_SECTION (ggBitmapSubSlotIndexPerWord, ".gg_nonshared_bufs")
#pragma DATA_SECTION (rfcContext, ".gg_nonshared_bufs");
#pragma DATA_SECTION (vppuContext, ".gg_nonshared_bufs");
#pragma DATA_SECTION (tsuContext,  ".gg_nonshared_bufs");
#pragma DATA_SECTION (vtkContext, ".gg_nonshared_bufs");
#pragma DATA_SECTION (customEcoContext, ".gg_nonshared_bufs");
#endif
/******************************************************************************/
/* buffers used for dynamic buffer allocation */
tint ggCh2Slot[GG_MAX_NUM_CHANNELS + GG_MAX_PKT2PKT_CHANNELS];
tint ggCh2SlotLBRC[GG_MAX_NUM_CHANNELS];
tuint ggLBRCDP[GG_MAX_NUM_CHANNELS + GG_MAX_PKT2PKT_CHANNELS];
tuint *ggPrimaryDP;
tuint ggMaxChannelsPerWord[GG_MAX_NUM_BITMAP_WORDS];
tuint ggBitmapBufIndexPerWord[GG_MAX_NUM_BITMAP_WORDS];
tuint ggBitmapLBRCBufIndexPerWord[3];  /* assume max. LBRC channel <= 42 (TMHK 84 ch pcm build) */

ggDynBufAlloc_t ggDynBufAlloc = {
#ifdef GG_INCLUDE_DYN_BUF_ALLOCATION
  TRUE,
#else
  FALSE,
#endif
  GG_MAX_NUM_BITMAP_WORDS,
  ggMaxChannelsPerWord,
  ggBitmapBufIndexPerWord,
  ggCh2Slot,
  ggCh2SlotLBRC,
#ifdef GG_INCLUDE_DYN_CH_M_ADD_N_CONFIG
  GG_DYN_CH_MAX_LBRC_CHANNELS,
#else
  0,
#endif
  ggBitmapLBRCBufIndexPerWord
};

/******************************************************************************/

#ifdef GG_INCLUDE_DYN_HEAP_ALLOCATION

#ifdef GG_DYNHEAP_NONCH_1
/* This should go in to the build specific directory - ggmemdef.c*/
word *gg_dynheap_nonch_1_areas[GG_DYNHEAP_TYPE_NONCH_1_NHEAPS]={
 (word *)GG_DYNHEAP_NONCH_1_START  /* Defined in ggmemmap.beh */
};
#endif

siuDynHeapTypesInfo_t ggDynHeapTypesInfo[GG_NUM_DYNHEAP_TYPES]={
/* Dedicated memory chunks */
#ifdef GG_DYNHEAP_NONCH_1
{
   GG_DYNHEAP_TYPE_NONCH,/* Non channels area heap */
   GG_DYNHEAP_TYPE_NONCH_1_SIZE,/* size */
   GG_DYNHEAP_TYPE_NONCH_1_NHEAPS,/* num_heaps */
   GG_DYNHEAP_TYPE_NONCH_1_NCHUNKS,/* Num of chunks supported in this heap */
   gg_dynheap_nonch_1_areas
},
#endif

/* Voice heaps used for dynamic memory allocations */
#ifdef GG_DYNHEAP_CH
{
   GG_DYNHEAP_TYPE_CH,/* Voice heap */
   GG_DYNHEAP_TYPE_CH_SIZE,/* size = GG_CH_LENGTH */
   GG_DYNHEAP_TYPE_CH_NHEAPS,/* num_heaps */
   GG_DYNHEAP_TYPE_CH_NCHUNKS,/* Num of chunks supported in this heap */
   (word **)ggchVoiceHeapStartAddr /* Link to addresses of voice heaps */
},
#endif
};
#endif
/* Buffer used to handle channel mapping for 5ms task processing */
tint ggPrdCh[(5 * (GG_SAMPLING_RATE/1000) / GG_BSP_SEGMENT_LENGTH) + 1];
#ifdef GG_INCLUDE_RTP
rcuRtcpCallTable_t *rcuRtcpCallTable = &gg_rcuRtcp_ctable;
#endif

/******************************************************************************
 * DATA DEFINITION: VPPU global parameters (VPPU is present if Video is supported)
 *****************************************************************************/
vppuContext_t vppuContext = {
  siuRtscDebugInfo,
  &siu_isr_clock,                 /* Wallclock in ISR */
  NULL,                           /* Handle to the Headers Memory Pool */
  NULL,                           /* Handle to the Packets Memory Pool*/
  NULL,
  NULL,
  NULL,            /* For VPPU RX, set it to NULL for VPPU Tx only */
  0,
  0,
  thwCriticalBegin, /* Critical section Begin macro */
  thwCriticalEnd    /* Critical section End macro */
};

#if 0
/******************************************************************************
 * DATA DEFINITION: TSU global parameters (TSU is present if Video is supported)
 *****************************************************************************/
tsuContext_t tsuContext = {
  siuRtscDebugInfo,
  gmpAllocGmc,
  gmpFreeGmc,
  siu_tsu_coeff_avail,
  NULL
};

/******************************************************************************
 * DATA DEFINITION: VTK global parameters (VTK is present if Video is supported)
 *****************************************************************************/
vtkContext_t vtkContext = {
  siuDebugInfo,
  &siu_isr_clock,         /* Wallclock in ISR */
  siuVigdkCodecAlgCreate,
  siuVigdkCodecAlgFree
};
/******************************************************************************
 * DATA DEFINITION: CUSTOM ECO global parameters
 *****************************************************************************/
customEcoContext_t customEcoContext = {
  siuDebugInfo,
  /* Abstract rendering interface defined here */
  ovly_bmp,
  ovly_logo,
  ovly_render_simplefont,
  ovly_fixed_logo
};

ovlyContext_t ovlyContext = {
  siuRtscDebugInfo
};
#endif
#if 1
/******************************************************************************
 * DATA DEFINITION: TFTP global parameters (TFTP is present if Video is supported)
 *****************************************************************************/
tftpContext_t tftpContext = {
  siuRtscDebugInfo,
  &siu_isr_clock,
  NULL,
  siuVctTftpGmpSize,
  siuVctTftpGmpAlloc,
  siuVctTftpGmpFree,
  thwCriticalBegin, /* Critical section Begin macro */
  thwCriticalEnd    /* Critical section End macro */
};


#endif

/******************************************************************************
 * DATA DEFINITION: NEU global parameters
 *****************************************************************************/
#ifdef GG_INCLUDE_NEU_PORTMAP
#ifdef GG_SHARED_MEM_DATA_ALLOCATION
#pragma DATA_SECTION (ggNeuDestPort, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggNeuInst, ".gg_nonshared_bufs");
#pragma DATA_SECTION (ggNeuPortMap, ".gg_nonshared_bufs");
#endif

#ifndef GG_NEU_MAX_PORTMAP_CHANNELS
#error GG_NEU_MAX_PORTMAP_CHANNELS must be defined in ggswcfg.h if GG_INCLUDE_NEU_PORTMAP is defined
#endif
tuint ggNeuDestPort[GG_NEU_MAX_PORTMAP_CHANNELS*2];
void *ggNeuInst[GG_NEU_MAX_PORTMAP_CHANNELS*2];


neuPortMap_t ggNeuPortMap = {
  GG_NEU_MAX_PORTMAP_CHANNELS*2,
  0,
  ggNeuDestPort,
  ggNeuInst
};
#endif

#ifdef GG_INCLUDE_NEU
neuContext_t neuContext = {
  siuRtscDebugInfo,
  0,
  0,
  0xffff,
  0,                 /* ATM cell size set by first neuNew() call. */
#ifdef GG_INCLUDE_NEU_PORTMAP
  &ggNeuPortMap,
  neuproto_PROTOCOL_ETHERNET,
#else
  NULL,
  0,
#endif
  siuTmrGetTime,     /* Wallclock in MS */
  siuNeuViolReport,  /* Packet violation report function */
  siuGetNetEncapInst, /* Get NEU inst for packet-to-packet routing */
  NULL,      /* Get RCU inst */
  NULL,
  NULL,
  NULL,   /* Update Tx Transparent Stats on packet-to-packet routing */
  { 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0, 0, 0, 0, 0},   /* PktStats */
  1472,                   /* maxPayloadBytes */
  { 0, 0, NULL, NULL, 0}, /* neuGmcPktTrackerRoot */
  0,                      /* tag_desc */

  { 0,0, 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0, 0L, 0, 0, 0}, /* nonVoiceIP */

  NULL,          /* gmpHandle */
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,         /* gmpReadFromStart [only required for GMPISRSPLIT] */

  thwCriticalBegin,
  thwCriticalEnd,
  thwCriticalBegin,
  thwCriticalEnd
};
#endif
/* memory allocation for RCU context */
#ifdef GG_INCLUDE_RTP
rcuContext_t rcuContext = {
  siuDebugInfo,                   /* Exception/debug pointer */
#if (!defined(HDVIDEO) && !defined(HDCT))
  siu_voice_notification,         /* Voice notification function */
  siu_receive_pkt,                /* To host function */
#else
  NULL,
  NULL,
#endif
#ifdef GG_INCLUDE_DSIGR_RTP
  dsigrRtpReceiveIn,              /* DSP CAS fcn */
#else
  NULL,                           /* DSP CAS fcn */
#endif
#if (!defined(HDVIDEO) && !defined(HDCT))
  siu_stm_search,                 /* Search profile for state machine */
  siu_send_packet_ack,            /* Send pkt ack fcn */
  siuPvpReport,                   /* Packet violation */
  siu_p2p_insertTxCfg,            /* Protocol independent insertion of
                                   * Tx STM configuration */
#else
  NULL,
  NULL,
  NULL,
  NULL,
#endif
#if (!defined(HDVIDEO) && !defined(HDCT))
  siu_rtp_validate_udptl_pkt,     /* Validate UDPTL utility fcn */
#else
  NULL,
#endif
#ifdef GG_INCLUDE_MSU
  msuEncrypt,
  msuDecrypt,
#else
  NULL,
  NULL,
#endif

#ifdef GG_INCLUDE_PRU
  siu_p2prxin_pru,                /* a wrapper for the p2p msgs from RCU */
#else
  NULL,
#endif

  &siu_isr_clock,                 /* Wallclock in ISR */
  GG_INTR_PCM_SAMPLES,            /* ISR->samp */

#ifdef PLRFEC
  siuPlrReceiveOut,
  siuFecIsEnabled,
#endif

#ifdef PLRRED
  siuPlrRxOutputPkt,
  siuRedIsEnabled,
#endif

#if (defined(PLRRED) || defined(PLRFEC))
  siuPlrSendIn,
  siuPlrSendOut,
  siuPlrReceiveIn,
  siuGetPlrPayloadType,
#endif
  NULL,           /* GMP Handle */
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,

  thwCriticalBegin,
  thwCriticalEnd,
  thwCriticalBegin,   /* should be thwCriticalBeginFast */
  thwCriticalEnd      /* should be thwCriticalEndFast   */

};
#endif
/* memory allocation for RCU RTCP context */
//#ifdef GG_INCLUDE_RTCP
#if 1
rcuRtcpContext_t rcuRtcpContext = {
  {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
  0,
  0,
  NULL,
#ifdef GG_INCLUDE_MSU
  msuEncrypt,
  msuDecrypt,
#else
  NULL,
  NULL,
#endif
  NULL,
#ifdef GG_INCLUDE_VQM
  siu_rtcpxr_ready,
#else
  NULL,
#endif
  NULL, //siu_rtcp_send_xr_voip,
#if (!defined(HDVIDEO) && !defined(HDCT))
  siu_rtcp_qa_rpt,
#else
  NULL,
#endif
#ifdef GG_INCLUDE_VQM
  siu_rtcpxr_ready_piqua,
#else
  NULL,
#endif
  0,
  0,                   /* qaInitTime */
  -1                   /* PIQUA Block type to be used with XR Packets */
};
#endif
/* This table maps a priority to a hold time -- hold time is used to stay
 * in a process more than one iteration to improve cache performance.  Hold
 * time is number of iterations to stay in a task of the priority.
 */

#if (defined(GG_BIOS) || defined(GG_MULTITASKING))
  #define GG_NUM_PRIORITY_ID SIU_NUM_PRIORITY_ID_MRATE
#else
  //#define GG_NUM_PRIORITY_ID 0
#endif
#if 0
const tuint ggSchedHoldTimes[GG_NUM_PRIORITY_ID] = {
#if (defined(GG_BIOS) || defined(GG_MULTITASKING))
  0,   /* 5ms  */
  0,   /* 10ms */
  0,   /* 6ms  */
#else
  25  /* 25 ms */
#endif
};
#endif

#if defined(GG_INCLUDE_MSU)
 /*****************************************************************************
 * FUNCTION PURPOSE: Platform specific initialization for MSU
 ******************************************************************************
 * DESCRIPTION: These routines are used to circumvent the C compiler bug
 *              that fails to initialize the far function pointers when
 *              the initialized values are outside page 0.
 *
 *****************************************************************************/
int platformInitMsu(void)
{
	tint i = 0;
#ifndef GG_MSU_USE_AUXSLOT
    extern const tulong msuTe0[], msuTd0[];
    /* MSU Table relocation is not supported - MSU tables are already in local memory */
    msuContext.aesEncTables = (tulong *)msuTe0;
  #if ((MSU_ON_MICRO==1) || defined(GG_INCLUDE_SECURITY_PKTCBL_RTP) || defined(GG_INCLUDE_SECURITY_PKTCBL_RTCP))
    msuContext.aesDecTables = (tulong *)msuTd0;
  #endif
#else

  #ifdef GG_MSU_NO_TABLERELOC   /* No table relocation need, for example,Tomahawk & Himalaya */
    extern const tulong msuTe0[], msuTd0[];
    msuContext.aesEncTables = (tulong *)msuTe0;
    #if ((MSU_ON_MICRO==1) || defined(GG_INCLUDE_SECURITY_PKTCBL_RTP) || defined(GG_INCLUDE_SECURITY_PKTCBL_RTCP))
    msuContext.aesDecTables = (tulong *)msuTd0;
    #endif
  #else
    #ifndef GG_INCLUDE_DYN_AUXSLOT
    #error If MSU table relocation is required, option GG_INCLUDE_DYN_AUXSLOT is required.
    #endif
  #endif
#endif

	//Init the msuProfCallTblPtr[] structure
#ifdef GG_INCLUDE_SECURITY_SRTP
	msuProfCallTablePtr [i]->msuProfile = MSU_PT_SRTP;
	msuProfCallTablePtr [i]->msuGetSize = msuSrtpGetSize;
	msuProfCallTablePtr [i]->msuOpen = msuSrtpOpen;
	msuProfCallTablePtr [i]->msuControl = msuSrtpControl;
	msuProfCallTablePtr [i]->msuGetStats = msuSrtpGetStats;
	msuProfCallTablePtr [i]->msuEncrypt = msuSrtpEncrypt;
	msuProfCallTablePtr [i]->msuDecrypt = msuSrtpDecrypt;
	msuProfCallTablePtr [i++]->msuClose = msuSrtpClose;
#endif
#ifdef GG_INCLUDE_SECURITY_SRTCP
	msuProfCallTablePtr [i]->msuProfile = MSU_PT_SRTCP;
	msuProfCallTablePtr [i]->msuGetSize = msuSrtcpGetSize;
	msuProfCallTablePtr [i]->msuOpen = msuSrtcpOpen;
	msuProfCallTablePtr [i]->msuControl = msuSrtcpControl;
	msuProfCallTablePtr [i]->msuGetStats = msuSrtcpGetStats;
	msuProfCallTablePtr [i]->msuEncrypt = msuSrtcpEncrypt;
	msuProfCallTablePtr [i]->msuDecrypt = msuSrtcpDecrypt;
	msuProfCallTablePtr [i++]->msuClose = msuSrtcpClose;
#endif
#ifdef GG_INCLUDE_SECURITY_PKTCBL_RTP
	msuProfCallTablePtr [i]->msuProfile = MSU_PT_PKTCABLE_RTP;
	msuProfCallTablePtr [i]->msuGetSize = msuPcRtpGetSize;
	msuProfCallTablePtr [i]->msuOpen = msuPcRtpOpen;
	msuProfCallTablePtr [i]->msuControl = msuPcRtpControl;
	msuProfCallTablePtr [i]->msuGetStats = msuPcRtpGetStats;
	msuProfCallTablePtr [i]->msuEncrypt = msuPcRtpEncrypt;
	msuProfCallTablePtr [i]->msuDecrypt = msuPcRtpDecrypt;
	msuProfCallTablePtr [i++]->msuClose = msuPcRtpClose;
#endif
#ifdef GG_INCLUDE_SECURITY_PKTCBL_RTCP
	msuProfCallTablePtr [i]->msuProfile = MSU_PT_PKTCABLE_RTCP;
	msuProfCallTablePtr [i]->msuGetSize = msuPcRtcpGetSize;
	msuProfCallTablePtr [i]->msuOpen = msuPcRtcpOpen;
	msuProfCallTablePtr [i]->msuControl = msuPcRtcpControl;
	msuProfCallTablePtr [i]->msuGetStats = msuPcRtcpGetStats;
	msuProfCallTablePtr [i]->msuEncrypt = msuPcRtcpEncrypt;
	msuProfCallTablePtr [i]->msuDecrypt = msuPcRtcpDecrypt;
	msuProfCallTablePtr [i++]->msuClose = msuPcRtcpClose;
#endif
  return GGCFG_NOERR;

} /*platformInitMsu*/
#endif

#ifdef PA_SS
#include "common/dsiunwcfg.h"
extern word QMemDescRam[];
extern word rxlinkramQ1[];
extern word rxlinkramQ2[];
extern word rxlinkramQ3[];
extern word rxlinkramQ4[];
extern word txlinkramQ1[];
extern word txlinkramQ2[];
extern word txlinkramQ3[];
extern word txlinkramQ4[];
const dsiuNwalLinkedBufsDesc_t dsiuNwalLinkedBufsDesc[] = {
#if (GG_QM_RX_LINKED_BUF_1_SIZE)
  {
    rxlinkramQ1,
    GG_QM_RX_LINKED_BUF_1_SIZE,
    GG_QM_RX_LINKED_BUF_1_NBUFS_PER_CORE,
    DSIU_NWAL_RX_CTL_BUFS
  },
#endif
#if (GG_QM_RX_LINKED_BUF_2_SIZE)
  {
    rxlinkramQ2,
    GG_QM_RX_LINKED_BUF_2_SIZE,
    GG_QM_RX_LINKED_BUF_2_NBUFS_PER_CORE,
    DSIU_NWAL_RX_PKT_BUFS
  },
#endif
#if (GG_QM_RX_LINKED_BUF_3_SIZE)
  {
    rxlinkramQ3,
    GG_QM_RX_LINKED_BUF_3_SIZE,
    GG_QM_RX_LINKED_BUF_3_NBUFS_PER_CORE,
    DSIU_NWAL_RX_PKT_BUFS
  },
#endif
#if (GG_QM_RX_LINKED_BUF_4_SIZE)
  {
    rxlinkramQ4,
    GG_QM_RX_LINKED_BUF_4_SIZE,
    GG_QM_RX_LINKED_BUF_4_NBUFS_PER_CORE,
    DSIU_NWAL_RX_PKT_BUFS
  },
#endif
#if (GG_QM_TX_LINKED_BUF_1_SIZE)
  {
    txlinkramQ1,
    GG_QM_TX_LINKED_BUF_1_SIZE,
    GG_QM_TX_LINKED_BUF_1_NBUFS_PER_CORE,
    DSIU_NWAL_TX_CTL_BUFS
  },
#endif
#if (GG_QM_TX_LINKED_BUF_2_SIZE)
  {
    txlinkramQ2,
    GG_QM_TX_LINKED_BUF_2_SIZE,
    GG_QM_TX_LINKED_BUF_2_NBUFS_PER_CORE,
    DSIU_NWAL_TX_PKT_BUFS
  },
#endif
#if (GG_QM_TX_LINKED_BUF_3_SIZE)
  {
    txlinkramQ3,
    GG_QM_TX_LINKED_BUF_3_SIZE,
    GG_QM_TX_LINKED_BUF_3_NBUFS_PER_CORE,
    DSIU_NWAL_TX_PKT_BUFS
  },
#endif
#if (GG_QM_TX_LINKED_BUF_4_SIZE)
  {
    txlinkramQ4,
    GG_QM_TX_LINKED_BUF_4_SIZE,
    GG_QM_TX_LINKED_BUF_4_NBUFS_PER_CORE,
    DSIU_NWAL_TX_PKT_BUFS
  },
#endif
  {NULL, 0, 0, DSIU_NWAL_RX_PKT_BUFS}
};

const dsiuNwalCfg_t dsiuNwalCfg = {
  GG_QM_NUM_DESCS_PER_CORE,
  GG_QM_DESC_SIZE,
  QMemDescRam,
  dsiuNwalLinkedBufsDesc
};
#endif

/******************************************************************************
 * FUNCTION PURPOSE:  Platform specific initializations
 ******************************************************************************
 * DESCRIPTION: Performs any required platform-specific initializations during
 *              DSP booting.
 *
 *****************************************************************************/

int platformInit (void)
{
 int ret_val = GGCFG_NOERR;
 return ret_val;
} /* platformInit */

/* nothing past this point */
