/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _GGVIDEOCFG_H
#define _GGVIDEOCFG_H
#include "common/siuloc.h"       /* NDU call tables */
#include "types.h"        /* DSP types */
#include <ti/mas/rcu/rcu.h>          /* RCU Call Table type */
#include <ti/mas/neu/neu.h>          /* NEU Call Table type */
//#include "siucnf.h" 
#include "targethw.h"
#include "ggport.h"
#include <ti/mas/vppu/vppu.h>
#if 0
#include <ti/mas/tsu/tsu.h>
#include "vtk.h"
#include "customEcoIface.h"
#include <ti/mas/ovly/ovly.h>
#endif

/* Return Codes of platfrmInit functions.Only XDAISed codec return 
   GGCFG_ERR  currently.However API for ECO and XDAISed codecs need 
   to match */
#define GGCFG_NOERR        0
#define GGCFG_ERR          1
/******************************************************************************
 * DATA DEFINITION: buffer dynamic allocation structure
 ******************************************************************************
 * DESCRIPTION: buffer dynamic allocation structure
 *****************************************************************************/

typedef struct {
  bool  dynBufAllocEnable;
  tint  numBitmapWord;
  tuint *ggMaxChannelsPerWord;
  tuint *ggBitmapBufIndexPerWord;
  tint  *ggCh2Slot;
  tint  *ggCh2SlotLBRC;
  tuint maxLBRCBuffers;
  tuint *bitmapLBRCBufIndexPerWord;
} ggDynBufAlloc_t;

extern ggDynBufAlloc_t ggDynBufAlloc;

/******************************************************************************
 * DATA DECLARATION: 5ms task sub rate channel array
 ******************************************************************************/
extern tint ggPrdCh[];
/*****************************************************************************
 * DEFINITION: DMA buffer memory for ports. An array of these is used
 *             for a variable number of ports.
 *****************************************************************************/
typedef struct cfgDma_s
{
  tuint *txStart;  /* DMA start buffer location tx */
  tuint *txEnd;    /* DMA end buffer location tx   */
  tuint *rxStart;  /* DMA start buffer location rx */
  tuint *rxEnd;    /* DMA end buffer location rx   */
} cfgDma_t;

/*****************************************************************************
 * DEFINITION: A generic data memory block descriptor
 *****************************************************************************/
typedef struct cfgMemBlk_s
{
  tuint *start;
  tuint *end;
} cfgMemBlk_t;

/*****************************************************************************
 * DEFINTIION: Utopia build time configuration. All elements are in arrays
 *             to allow different configurations for multiple ports. An
 *             array of structures wasn't used to make declaration of
 *             the structure easier in diagcfg.c
 *****************************************************************************/
typedef struct cfgUtConfig_s
{
  /* Array of utopia dma memory sizes */ 
  cfgDma_t *utopiaDmaMem; 
  
  /* Array of ports to use */
  tint *utPortNum;
  
  /* Port configuration */
  tint *slaveId;
  halUtopiaMode_t *routeMode;
 
} cfgUtConfig_t;

/*****************************************************************************
 * DEFINITION: TDM build time configuration. All elements are in arrays
 *             to allow different configurations for multiple ports. 
 *****************************************************************************/ 
typedef struct cfgTdmConfig_s
{
  /* Array of tdm dma memory sizes */
  cfgDma_t *tdmDmaMem;
  
  /* Array of sub-frame sizes */
  tint *subFrameSize;
  
  /* Array of single sample word sizes (bits) */
  tint *wordSize;
  
  /* Array of buffer increments */
  tint *buffIncr;
  
  /* Max sub and mega phase values */
  tint *maxPhase;
  tint *maxMegaPhase;
  
} cfgTdmConfig_t;
/* Data structure for allocating non-buffers from primary as well as secondary slot.
This structure will be used to indicate external memory allocatable buffers as well */
typedef struct {
  tint mid;
  #define GG_PRIMARY_SLOT_OR_HEAP 0
  #define GG_SECOND_SLOT          1
  #define GG_EXT_SHARED_MEM       2
  word *bufType; /* Pointer to an array of descriptors for each buffer of the module */
} ggAllocType_t;

typedef struct {
  uint  spc_init;
  uint  BSP_segment_length;
  uint  software_segment_length;
  uint  bsp_max_phase;        /* Max number of TDM phases  */
  word  pcm_silence_pattern;
  uint  pcm_bits;
  uint  max_channels;          /* Total number of channels - both full and aux PCM channels */
  uint  aux_pcm_channels;      /* Auxilliary PCM channels. Sum of full & aux channels gives 
                                * total number of channels */
  uint  samplingRate;          /* Sampling rate in Hz */
  uint  maxFaxFrameLength;     /* max allowable FAX frame in # of samples */ 
                               /*     in the largest frame */ 
  uint  maxModemFrameLength;   /* max allowable Modem frame in # of samples */ 
                               /* in the largest frame */
  uint  maxModemCtrlFrameLength; /* max allowable Modem Controller frame in # of samples */ 
                                 /* in the largest frame */
  uint  maxVoiceFrameLength;   /* max allowable voice frame in # of samples */ 
                               /*     in the largest frame */ 
  uint  maxCodecFrameLength;   /* max allowable voice frame in # of samples 
                                * starting from VAU->, if G729xx, GSMxx or
                                * G723 is present in the build, it is set to
                                * 10 ms, otherwise it is 5 or 5.5 ms */
  uint  voiceFrameSize;        /* frame size for voice in ms */

  uint  maxVcaInstSize;        /* Maximum size of VCA instance */
 
  uint  stagger_resolution;    /* Resolution number used in staggering. */

  uint  maxCodeWordSegSize;    /* max codeword seg length (in bits) as the */ 
                               /*  ceiling of: max coding rate(Kbps) * max */ 
                               /*  frame duration (msec) */
  uint  maxCodeWordSegSizeRTP; /* Same as above for RTP */
  uint  maxSegmentsPerPacket;  /* Max number of codeword segs packed into */ 
                               /*   a voice packet for the max codeword   */
                               /*   segment size */
  uint  maxCoreSegmentsPerPacket; /* same as above for 40k-EADPCM */
  uint  maxVoicePlayoutFIFOSize; /* maximum play out size for voice */
  uint  piu_rx_delay;            /* 60 for G273, others = BSP_segmnet_length*/
  uint  fax_piu_rx_delay;        /* Rx delay in PIU for FAX */
  uint  modem_piu_rx_delay;      /* Rx delay in PIU for data modem */
  uint  pcm_sync_intr_enabled;   /* 1: PCM sync interrupt supported by
                                  * by this platform. (Ex. VPBX)
                                  * 0: No pcm sync interrupt (Ex. @small) */
  tint  maxReplicateChannels;      /* maximum number of replicated channels */
  uint  siuMsgFifoLen;             /* words in outgoing message FIFO */
  uint  siuSigFifoLen;             /* words in outgoing message FIFO */
  uint  siuLowFifoLen;             /* low priority outgoing message FIFO */
  uint  hdHpiRxSa;                /* High density HPI rx FIFO start address */
  uint  hdHpiRxLength;            /* High density HPI rx FIFO length        */
  uint  hdHpiTxSa;                /* High density HPI tx FIFO start address */
  uint  hdHpiTxLength;            /* High density HPI tx FIFO length        */
  uint  hdHpiMagic;               /* High density HPI magic ID value        */

  void *secTxAddr;             /* Transmit buffer for secondary serial port  */
  void *secRxAddr;             /* Receive buffer for secondary serial port   */
  void *secTxEndAddr;          /* Transmit buffer for secondary serial port  */
  void *secRxEndAddr;          /* Receive buffer for secondary serial port   */
  uint  secBufSize;            /* Size of serial port buffer                 */
  uint  secFrameSize;          /* Samples/channel serial DMA buffers         */
  uint  secChannels;           /* Number of secondary serial port channels   */
  uint  secMaxPhase;           /* Max TDM phases for secondary channel       */

  /* Tdm build time configuration */
  cfgTdmConfig_t *tdmConfig;

  uint  aicHeapPage;           /* Data page AIC heap is located on           */
  uint  dsigtHeapPage;         /* dsigt heap page                            */
  uint  dsigrHeapPage;         /* dsigr heap page                            */ 
  uint  confHeapPage;          /* Conferencing heap page                     */
  
  uint  dsigrChannels;         /* Number of dsigr channels                   */

  uint  pcmIntPer5ms;          /* Number of pcm interrupts in a 5ms period   */
  uint  neu_sar;               /* Use SSSAR and/or AAL5                      */
#define GGCFG_NEU_SAR_AAL5  0x0001
#define GGCFG_NEU_SAR_SSSAR 0x0002
  uint  fiu_num_internal_bufs; /* Number of internally allocated FIU buffers (per ch)  */

  /* HAL related structure elements */

  tint tdmFirstTimeslot;       /* Timeslot 1st channel opens                  */
  tint tdmTimeslotOffset;      /* Number of timeslots between channels        */      
  tint nTdmPorts;              /* Number of TDM ports used at one time        */
  tint nUtopiaPorts;           /* Number of Utopia interfaces used at one time*/
  tint nGmacPorts;
  tint nAicPorts;
  tuint neuUdpSrcMask;         /* DSP ID mask value for UDP source port */
  tuint *halHeapBufStart;
  tuint *halHeapBufEnd;
  
  tint  xmitBufSizeIn;          /* Packet (utopia) tx input size  */
  tint  rcvBufSizeIn;           /* Packet rx input size           */
  tint  portId;                 /* Packet port ID                 */

  /* GMAC buffer address */
  cfgDma_t *gmacDmaMem;
  cfgMemBlk_t *gmacRxAssemMem;
  tint *gmacBufsizeWordsTx;     /* Size array (by port number) of each transmit unit */
  tint *gmacBufsizeWordsRx;     /* Size array (by port number) of each receive unit  */
  tint *gmacRxAssemSizeWords;   /* Size of the rx assembly buffer                    */
   
  tuint dsigtIsrSkip;           /* Dsigt ISR skip value           */

  tint (*coreBootFcn)(tuint, tulong, tint *); /* Secondary core boot function  */
  void (*initCoreMhz)(tuint mhz); /* Initialize core mhz (mips) */
  const tuint *listOfBlockedAsymetricTxRx; /* Pointer to an array of blocked 
                                            * asymetric Tx/Rx coding types. */
  tuint rtTraceInterval;                   /* Maximum time trace data can 
                                            * sit in trace buffer, in pcm
                                            * interrupt units
                                            */
  tuint rtTracePktFree;                    /* Minimum packet free space req
                                            * for pcm trace, in HAL-defined
                                            * units (cells for utopia)
                                            */
  tuint gmcSize;                           /* GMC size which is set depending upon 
                                            * whether the build has WBAMR whose 
                                            *  codeword's size is greater than that 
                                            *  of  build PCM 
                                            */

  tint nvEcoBufNum;                         /* Number of non-voice echo buffers */                                           
  
  
  /* Init time relocation */
  tulong  loadStart;
  tulong  runStart;
  tulong  loadSize;
  
  tuint   pkt_channels;    /* Number of packet only channels for subslot builds */
} ggConfig_t;

/*-------------------------------------------------------------------------*/
/* Structure for defining segments used for global memory pool             */
/*-------------------------------------------------------------------------*/
typedef struct {
  tint  data_page;  
  word *area_start;
  word *area_end;
  tuint userMID;     /* Used to carefully tune GMP usage accross modules
                      * and channels */
} ggGmpSegment_t;
/*-------------------------------------------------------------------------*/

typedef struct {
  tuint id;/* Coding Type or MID */
  tuint ecotype;/* VMM_IDT_NONCODEC_ECO,VMM_IDT_ENCODER,VMM_IDT_DECODER */
  tuint slots_used;/* Bit-map indicating the slots used */
  #define GG_SLOT_0  0
  #define GG_SLOT_1  1
  #define GG_SLOT_2  2
  #define GG_SLOT_3  3
  #define GG_SLOT_4  4
  #define GG_SLOT_5  5
  word *slotdistr_list;/* List of slots distribution for all the buffers.
                          This buffer need be populated with the above
						  defined values
					    */
} ggBufSlotDistr_t;
extern const tuint ggSchedHoldTimes[];

extern ggConfig_t ggConfig;
extern rcuRtcpCallTable_t gg_rcuRtcp_ctable;  /* RTCP Call Table */
extern rcuRtcpCallTable_t  *rcuRtcpCallTable; /* RTCP Call Table */
extern neuCallTable_t    gg_neu_ctable;    /* NEU Call Table */
extern rcuCallTable_t    gg_rcu_ctable;    /* RCU Call Table */
extern vppuCallTable_t   gg_vppu_ctable;
#if 0
extern vtkCallTable_t gg_vtk_ctable;
extern customEcoCallTable_t gg_custom_eco_ctable;
extern ovlyCallTable_t gg_ovly_ctable;

extern word gg_vtk_scratch[];
extern tint gg_vtk_chan2slots[];
extern tuint gg_vtk_slot_bitmap[];
extern void *gg_vtk_slot_array[];

/* List of intercore YUV buffers - one pair per encoding core */
extern word gg_yuv_intercore_scratch1_bufA[], gg_yuv_intercore_scratch1_bufB[];
extern word gg_yuv_intercore_scratch2_bufA[], gg_yuv_intercore_scratch2_bufB[];
extern word gg_yuv_intercore_scratch3_bufA[], gg_yuv_intercore_scratch3_bufB[];
extern word gg_yuv_intercore_scratch4_bufA[], gg_yuv_intercore_scratch4_bufB[];
extern word gg_yuv_intercore_scratch5_bufA[], gg_yuv_intercore_scratch5_bufB[];
extern word gg_yuv_intercore_scratch6_bufA[], gg_yuv_intercore_scratch6_bufB[];

#endif
/* Platform Init Functions Prototypes */
int platformInit (void);
#endif  /* _GGVIDEOCFG_H */
