#ifndef _GGBITMAP_H
#define _GGBITMAP_H
/******************************************************************************
 * FILE PURPOSE: Golden Gateway micro bitmap specifiaction
 ******************************************************************************
 * FILE NAME: ggbitmap.h
 *
 * This file defines the dspimage array which is passed to the micro at load
 * time.  It should be included by ggswcfg.h, at the bottom of the file.
 *
 * (C) Copyright 2000, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef max
#define max(x,y) ((x) > (y) ? (x):(y))
#endif

/* data section for dspimage bitmap, not used by DSP */ 
#define DSPIMAGE_WORD1   32

/*****    DSP IMAGE BIT MAP    *****/

/* Bit map is used by micro to check which codecs are in the dsp image */
/* Bit map consists of 16 bit integers defined below                   */
/*          <----------------16 bits---------------->                  */
/*          <bit 15> <bit 14> <bit 13>  ....  <bit 0>                  */
/*          < MSB  >   ..        ..     ....  < LSB >                  */
/* NOTE: BIT 0 is reserved                                             */
/*       BIT 0 is set if it is not the last word of the bitmap         */
/*       BIT 0 is cleared on the last word of the bit map              */    

#define GGBIT_15  0x8000    /* BIT 15-0 DEFINED */
#define GGBIT_14  0x4000
#define GGBIT_13  0x2000
#define GGBIT_12  0x1000
#define GGBIT_11  0x0800
#define GGBIT_10  0x0400
#define GGBIT_9   0x0200
#define GGBIT_8   0x0100
#define GGBIT_7   0x0080
#define GGBIT_6   0x0040
#define GGBIT_5   0x0020
#define GGBIT_4   0x0010
#define GGBIT_3   0x0008
#define GGBIT_2   0x0004
#define GGBIT_1   0x0002
#define GGBIT_0   0x0001

/* Some of the bit or words locations for features are set in MKGGDSP based upon */
/* the 2nd stage command line inputs. Bits configured by the build are set here  */

/*******************************************************************************
 * WORD 1
 *******************************************************************************/
/* Define bit locations for each codec type */
#define GGBIT_PCM_MU    GGBIT_15    /* PCM Codec     */
#define GGBIT_PCM_A     GGBIT_14
#define GGBIT_ADPCM16   GGBIT_13    /* ADPCM Codec   */
#define GGBIT_ADPCM24   GGBIT_12
#define GGBIT_ADPCM32   GGBIT_11
#define GGBIT_ADPCM40   GGBIT_10
#define GGBIT_EADPCM16  GGBIT_9     /* EADPCM Codec  */
#define GGBIT_EADPCM24  GGBIT_8
#define GGBIT_EADPCM32  GGBIT_7
#define GGBIT_EADPCM40  GGBIT_6
#define GGBIT_G729      GGBIT_5     /* G729 Codec     */
#define GGBIT_G729AB    GGBIT_4     /* G729AB Codec   */
#define GGBIT_G723_5_3  GGBIT_3     /* G723 Codec     */
#define GGBIT_G723_6_3  GGBIT_2

#if defined(GG_INCLUDE_PCM)  /* Bitmap for PCM */
  #define GGBITMAP_PCM     (GGBIT_PCM_MU | GGBIT_PCM_A) 
#else
  #define GGBITMAP_PCM     0
#endif  

#if defined(GG_INCLUDE_G726)  /* Bitmap for G726 */
  #define GGBITMAP_ADPCM   (GGBIT_ADPCM16 | GGBIT_ADPCM24 | GGBIT_ADPCM32 | GGBIT_ADPCM40)
#else
  #define GGBITMAP_ADPCM   0
#endif 

#if defined(GG_INCLUDE_G727)  /* Bitmap for G727 */
  #define GGBITMAP_EADPCM  (GGBIT_EADPCM16 | GGBIT_EADPCM24 | GGBIT_EADPCM32 | GGBIT_EADPCM40)  
#else
  #define GGBITMAP_EADPCM  0
#endif 

#if defined(GG_INCLUDE_G729) || defined(GG_INCLUDE_G729_VIA_G729E)  /* Bitmap for G729 */
  #define GGBITMAP_G729    GGBIT_G729 
#else
  #define GGBITMAP_G729    0
#endif 

#if defined(GG_INCLUDE_G729AB) || defined(GG_INCLUDE_G729AB_VIA_G729E) /* Bitmap for G729AB */ 
  #define GGBITMAP_G729AB  GGBIT_G729AB
#else
  #define GGBITMAP_G729AB  0
#endif

#if defined(GG_INCLUDE_G723)   /* Bitmap for G723 */
  #define GGBITMAP_G723    (GGBIT_G723_5_3 | GGBIT_G723_6_3)
#else
  #define GGBITMAP_G723    0 
#endif

/* Bitmap of the DSPImage */
#define GGBITMAP_WORD1 (GGBITMAP_PCM | GGBITMAP_ADPCM | GGBITMAP_EADPCM | \
                        GGBITMAP_G729 | GGBITMAP_G729AB | GGBITMAP_G723 )

/*******************************************************************************
 * WORD 10
 *******************************************************************************/
/* BITS 9-15 ARE RESERVED */
#define GGBIT_BV16        GGBIT_8  /* defined by (YES/NO) in makegg */

/* PLR RED LEVEL */
/* BITS 0-3 */
#ifdef PLRRED                           /* PLR RED LEVEL */
  #define GGBITMAP_PLR_RED_LEVEL       max(GG_MAX_PLR_VOICE_RED_LEVEL, \
                                           GG_MAX_PLR_TEXT_RED_LEVEL)
#else
  #define GGBITMAP_PLR_RED_LEVEL       0
#endif

/* PLR RED LEVEL */
/* BITS 4-7 */
#ifdef PLRRED                           /* PLR FEC 2198 LEVEL */
  #define GGBITMAP_PLR_FEC_2198_LEVEL  GG_MAX_PLR_FEC_LEVEL_2198
#else
  #define GGBITMAP_PLR_FEC_2198_LEVEL  0
#endif

#if defined(GG_INCLUDE_BV16)                    /* Bitmap for Tone Detect */
  #define GGBITMAP_BV16        GGBIT_BV16
#else
  #define GGBITMAP_BV16        0
#endif

/* Bitmap of word10 */
#define GGBITMAP_WORD10 ((GGBITMAP_PLR_RED_LEVEL & 0x000F) | GGBITMAP_BV16 | \
                         ((GGBITMAP_PLR_FEC_2198_LEVEL << 4) & 0x00F0))

/*******************************************************************************
 * WORD 11
 *******************************************************************************/


/* AER tail length */
#if defined(GG_INCLUDE_AER)              /* AER tail length */
  #define GGBITMAP_AER_TAIL_LENGTH       GG_MAX_AER_TAIL_LENGTH
#else
  #define GGBITMAP_AER_TAIL_LENGTH       0
#endif

/* Bitmap of word2 */
#define GGBITMAP_WORD2 (GGBITMAP_AER_TAIL_LENGTH)

/*******************************************************************************
 * WORD 11
 *******************************************************************************/
#define GGBIT_FRF11       GGBIT_0  /* defined by (YES/NO) in makegg */
#define GGBIT_RTP         GGBIT_1  /* defined by (YES/NO) in makegg */
#define GGBIT_AAL1        GGBIT_2  /* defined by (YES/NO) in makegg */
#define GGBIT_VQMON       GGBIT_3
#define GGBIT_ADPCM       GGBIT_4  /* defined by (YES/NO) in makegg */
#define GGBIT_PLRFEC      GGBIT_5
#define GGBIT_PLRRED      GGBIT_6
#define GGBIT_CONF        GGBIT_7  /* defined by (YES/NO) in makegg */
#define GGBIT_LTEC        GGBIT_8  /* defined by (YES/NO) in makegg */
#define GGBIT_TONE_DET    GGBIT_9
#define GGBIT_DTMF_RELAY  GGBIT_10
#define GGBIT_AAL2        GGBIT_11 /* defined by (YES/NO) in makegg */
#define GGBIT_S3          GGBIT_12
#define GGBIT_ECU         GGBIT_13
#define GGBIT_AER         GGBIT_14
#define GGBIT_XPT         GGBIT_15

#if defined(GG_INCLUDE_VQM)                    /* Bitmap for VQMon */
  #define GGBITMAP_VQMON       GGBIT_VQMON
#else
  #define GGBITMAP_VQMON       0
#endif

#if defined(GG_INCLUDE_TDU)                    /* Bitmap for Tone Detect */
  #define GGBITMAP_TONE_DET    GGBIT_TONE_DET
#else
  #define GGBITMAP_TONE_DET    0
#endif

#if defined(GG_INCLUDE_DTMF_RELAY)             /* Bitmap for DTMF Relay */
  #define GGBITMAP_DTMF_RELAY  GGBIT_DTMF_RELAY
#else
  #define GGBITMAP_DTMF_RELAY  0
#endif

#if defined(GG_INCLUDE_S3)                     /* Bitmap for S3 Protocol */
  #define GGBITMAP_S3          GGBIT_S3
#else
  #define GGBITMAP_S3          0
#endif

#if defined(GG_INCLUDE_ECU)                    /* Bitmap for line echo canc.  */
  #define GGBITMAP_ECU         GGBIT_ECU
#else
  #define GGBITMAP_ECU         0
#endif

#if defined(GG_INCLUDE_AER)                    /* Bitmap for acoustic echo canc. */
  #define GGBITMAP_AER         GGBIT_AER
#else
  #define GGBITMAP_AER         0
#endif

#if defined(GG_INCLUDE_XPT)                    /* Bitmap for TRANSPARENT protocol */
  #define GGBITMAP_XPT         GGBIT_XPT
#else
  #define GGBITMAP_XPT         0
#endif

#if defined(PLRRED)                            /* Bitmap for RFC2198, PLR RED */
  #define GGBITMAP_PLRRED      GGBIT_PLRRED
#else
  #define GGBITMAP_PLRRED      0
#endif

#if defined(PLRFEC)                            /* Bitmap for RFC2733, PLR FEC */
  #define GGBITMAP_PLRFEC      GGBIT_PLRFEC
#else
  #define GGBITMAP_PLRFEC      0
#endif

/* Bitmap of the FEATURES for the DSP image */
#define GGBITMAP_WORD11  (GGBITMAP_VQMON | GGBITMAP_TONE_DET | GGBITMAP_DTMF_RELAY | \
                          GGBITMAP_S3 | GGBITMAP_ECU | GGBITMAP_AER | GGBITMAP_XPT | \
                          GGBITMAP_PLRRED | GGBITMAP_PLRFEC)

/*******************************************************************************
 * WORD 12
 *******************************************************************************/
/* The bit locations for second Codec bitmap word */
#define GGBIT_G728      GGBIT_0
#define GGBIT_8KHZ      GGBIT_1
#define GGBIT_16KHZ     GGBIT_2
#define GGBIT_G729E     GGBIT_3
#define GGBIT_MODEMHI   GGBIT_4
#define GGBIT_MODEMLO   GGBIT_5
#define GGBIT_GSMEFR    GGBIT_6
#define GGBIT_GSMFR     GGBIT_7
#define GGBIT_L16PCM    GGBIT_8
#define GGBIT_FAX_MODEM GGBIT_9
#define GGBIT_CLEAR_DATA GGBIT_10
#define GGBIT_HDLC      GGBIT_11
//#define GGBIT_V22MODEMRELAY  GGBIT_12
#define GGBIT_AIC22     GGBIT_13
#define GGBIT_AIC10     GGBIT_14
#define GGBIT_ILBC      GGBIT_15

#if defined(GG_INCLUDE_G728)                  /* Bitmap for G728 Codec */
  #define GGBITMAP_G728     GGBIT_G728
#else
  #define GGBITMAP_G728     0
#endif

#if (GG_SAMPLING_RATE == 8000)
  #define GGBITMAP_8KHZ     GGBIT_8KHZ
  #define GGBITMAP_16KHZ    0
#else
  #ifndef GG_AUDIO_SAMPLING_RATE
    #define GGBITMAP_8KHZ       0
    #define GGBITMAP_16KHZ      GGBIT_16KHZ
  #else
    #if  (GG_AUDIO_SAMPLING_RATE == 16000)
      #define GGBITMAP_8KHZ     0
      #define GGBITMAP_16KHZ    GGBIT_16KHZ
    #else
      #define GGBITMAP_8KHZ      0
      #define GGBITMAP_16KHZ     0
    #endif
  #endif
#endif

#if defined(GG_INCLUDE_G729E)                 /* Bitmap for G729E Codec */
  #define GGBITMAP_G729E    GGBIT_G729E
#else
  #define GGBITMAP_G729E    0
#endif

#if defined(GG_INCLUDE_DATA_MODEM)            /* Bitmap for data modem */ 
  #define GGBITMAP_MODEMHI     GGBIT_MODEMHI
  #define GGBITMAP_MODEMLO     GGBIT_MODEMLO
#else
  #define GGBITMAP_MODEMHI     0
  #define GGBITMAP_MODEMLO     0
#endif

#if defined(GG_INCLUDE_GSMEFR) || defined(GG_INCLUDE_GSMAMR)   
  #define GGBITMAP_GSMEFR     GGBIT_GSMEFR
#else
  #define GGBITMAP_GSMEFR     0
#endif

#if defined(GG_INCLUDE_GSMFR)               
  #define GGBITMAP_GSMFR     GGBIT_GSMFR
#else
  #define GGBITMAP_GSMFR     0
#endif

#if defined(GG_INCLUDE_L16PCM)               
  #define GGBITMAP_L16PCM     GGBIT_L16PCM
#else
  #define GGBITMAP_L16PCM     0
#endif

#if defined(GG_INCLUDE_FAX_MODEM)
  #define GGBITMAP_FAX_MODEM  GGBIT_FAX_MODEM
#else
  #define GGBITMAP_FAX_MODEM  0
#endif

#if defined(GG_INCLUDE_CLEAR_DATA)
  #define GGBITMAP_CLEAR_DATA  GGBIT_CLEAR_DATA
#else
  #define GGBITMAP_CLEAR_DATA  0
#endif

#if (defined(GG_INCLUDE_DATA_MODEM) || defined(GG_INCLUDE_CLEAR_DATA))
  #define GGBITMAP_HDLC        GGBIT_HDLC
#else
  #define GGBITMAP_HDLC        0
#endif

#if defined(GG_AIC22)
  #define GGBITMAP_AIC22  GGBIT_AIC22
#else
  #define GGBITMAP_AIC22  0
#endif
/*
#if defined(GG_V22_MODEM_RELAY)
  #define GGBITMAP_V22MODELRELAY  GGBIT_V22MODEMRELAY
#else
  #define GGBITMAP_V22MODELRELAY  0
#endif
*/
#if defined(GG_AIC10)
  #define GGBITMAP_AIC10  GGBIT_AIC10
#else
  #define GGBITMAP_AIC10  0
#endif

#if defined(GG_INCLUDE_ILBC)               
  #define GGBITMAP_ILBC     GGBIT_ILBC
#else
  #define GGBITMAP_ILBC     0
#endif



/* Bitmap of the second CODEC word for the DSP image */
#define GGBITMAP_WORD12  (GGBITMAP_G728  | GGBITMAP_8KHZ   | GGBITMAP_16KHZ |  \
                          GGBITMAP_G729E | GGBITMAP_MODEMHI | GGBITMAP_MODEMLO | \
                          GGBITMAP_GSMEFR | GGBITMAP_GSMFR  | GGBITMAP_L16PCM | \
                          GGBITMAP_FAX_MODEM | GGBITMAP_CLEAR_DATA | GGBITMAP_HDLC |\
                          GGBITMAP_AIC22 | GGBITMAP_AIC10 | GGBITMAP_ILBC )

/*******************************************************************************
 * WORD 16
 *******************************************************************************/
/* The bit locations */
#define GGBIT_G722_64      GGBIT_0
#define GGBIT_G722_56      GGBIT_1
#define GGBIT_G722_48      GGBIT_2
#define GGBIT_LBRC_ONLY    GGBIT_3
#define GGBIT_DSIGT        GGBIT_4
#define GGBIT_DLCOMP       GGBIT_5    /* defined in makegg */
#define GGBIT_CODECSWICH   GGBIT_8
#define GGBIT_VEU          GGBIT_9
/* bit 10 are reserved */
#define GGBIT_AIC20        GGBIT_11
#define GGBIT_LLMR         GGBIT_12
#define GGBIT_K56FLEX      GGBIT_13
#define GGBIT_FAX_SLF      GGBIT_14
#define GGBIT_V92          GGBIT_15

/* The bit locations for third Codec bitmap word */
#if defined(GG_INCLUDE_G722)
  #define GGBITMAP_G722_64     GGBIT_G722_64
  #define GGBITMAP_G722_56     GGBIT_G722_56
  #define GGBITMAP_G722_48     GGBIT_G722_48
#else
  #define GGBITMAP_G722_64     0
  #define GGBITMAP_G722_56     0
  #define GGBITMAP_G722_48     0
#endif
  
/* The bit is used for channel configuration */
#ifdef GG_INCLUDE_DYN_CH_M_DIV_N_CONFIG
  /* if the number of opened LBRC channel equal to the MAX LBRC channel,
     no PCM channel can be opened. */
  #define GGBITMAP_LBRC_ONLY   GGBIT_LBRC_ONLY
#else
  #define GGBITMAP_LBRC_ONLY   0
#endif
  
#if defined (GG_INCLUDE_DSIGT)
  #define GGBITMAP_DSIGT       GGBIT_DSIGT
#else
  #define GGBITMAP_DSIGT       0
#endif

#if defined(GG_INCLUDE_CSU)
#define GGBITMAP_CSU             GGBIT_CODECSWICH
#else
#define GGBITMAP_CSU             0
#endif 

#if defined(GG_AIC20)
  #define GGBITMAP_AIC20  GGBIT_AIC20
#else
  #define GGBITMAP_AIC20  0
#endif

#if defined(VEU)
  #define GGBITMAP_VEU      GGBIT_VEU
#else
  #define GGBITMAP_VEU      0
#endif

#if defined(GG_INCLUDE_LINK_LAYER_MODEM_RELAY)
  #define GGBITMAP_LLMR          GGBIT_LLMR
#else
  #define GGBITMAP_LLMR          0
#endif

#if defined(GG_INCLUDE_DATA_MODEM)
/* If data modem is included, this includes K56Flex and V.92 support */ 
  #define GGBITMAP_K56FLEX     GGBIT_K56FLEX
  #define GGBITMAP_V92         GGBIT_V92
#else
  #define GGBITMAP_K56FLEX     0
  #define GGBITMAP_V92         0
#endif

#define GGBITMAP_WORD16  (GGBITMAP_G722_64 | GGBITMAP_G722_56 | GGBITMAP_G722_48 | \
                          GGBITMAP_LBRC_ONLY | GGBITMAP_DSIGT | GGBITMAP_LLMR | \
                          GGBIT_CODECSWICH | GGBITMAP_AIC20 | GGBITMAP_VEU)

/*******************************************************************************
 * WORD 17
 *******************************************************************************/
#if defined (GG_INCLUDE_V21_CM)
  #define GGBIT_TDU_V21_CM_FAX  GGBIT_15
#else
  #define GGBIT_TDU_V21_CM_FAX  0
#endif

#if defined (GG_INCLUDE_V25_RX)
  #define GGBIT_TDU_V25RX  GGBIT_14
#else
  #define GGBIT_TDU_V25RX  0
#endif

#if defined (GG_INCLUDE_MF_R2BW)
  #define GGBIT_TDU_MFR2BW  GGBIT_13
#else
  #define GGBIT_TDU_MFR2BW  0
#endif

#if defined (GG_INCLUDE_MF_R2FW)
  #define GGBIT_TDU_MFR2FW  GGBIT_12
#else
  #define GGBIT_TDU_MFR2FW  0
#endif

#if defined (GG_INCLUDE_MF_R1)
  #define GGBIT_TDU_MFR1 GGBIT_11
#else
  #define GGBIT_TDU_MFR1 0
#endif

#if defined (GG_INCLUDE_MF_DTMF)
  #define GGBIT_TDU_DTMF GGBIT_10
#else
  #define GGBIT_TDU_DTMF 0
#endif

#if defined (GG_INCLUDE_CIDCAS)
  #define GGBIT_TDU_CIDCAS GGBIT_9
#else
  #define GGBIT_TDU_CIDCAS 0
#endif

#if defined (GG_INCLUDE_TT1KHZ)
  #define GGBIT_TDU_TT1K GGBIT_8
#else
  #define GGBIT_TDU_TT1K 0
#endif

#if defined (GG_INCLUDE_SS7COR)
  #define GGBIT_TDU_SS7COR GGBIT_7
#else
  #define GGBIT_TDU_SS7COR 0
#endif

#if defined (GG_INCLUDE_CNG)
  #define GGBIT_TDU_CNG GGBIT_6
#else
  #define GGBIT_TDU_CNG 0
#endif

#if defined (GG_INCLUDE_SF)
  #define GGBIT_TDU_SF GGBIT_5
#else
  #define GGBIT_TDU_SF 0
#endif

#if defined (GG_INCLUDE_SS7COT)
  #define GGBIT_TDU_SS7COT GGBIT_4
#else
  #define GGBIT_TDU_SS7COT 0
#endif

#if defined (GG_INCLUDE_V18A)
  #define GGBIT_TDU_V18A GGBIT_3
#else
  #define GGBIT_TDU_V18A 0
#endif

#if defined (GG_INCLUDE_CPT)
  #define GGBIT_TDU_CPT GGBIT_2
#else
  #define GGBIT_TDU_CPT 0
#endif

#if defined (GG_INCLUDE_V25)
  #define GGBIT_TDU_V25 GGBIT_1
#else
  #define GGBIT_TDU_V25 0
#endif

#if defined (GG_INCLUDE_V21)
  #define GGBIT_TDU_V21 GGBIT_0
#else
  #define GGBIT_TDU_V21 0
#endif

#define GGBITMAP_WORD17  (GGBIT_TDU_MFR2BW | GGBIT_TDU_MFR2FW | \
                          GGBIT_TDU_MFR1   | GGBIT_TDU_DTMF   | \
                          GGBIT_TDU_CIDCAS | GGBIT_TDU_TT1K   | \
                          GGBIT_TDU_SS7COR | GGBIT_TDU_CNG    | \
                          GGBIT_TDU_SF     | GGBIT_TDU_SS7COT | \
                          GGBIT_TDU_V18A   | GGBIT_TDU_CPT    | \
                          GGBIT_TDU_V25    | GGBIT_TDU_V21    | \
                          GGBIT_TDU_V25RX  | GGBIT_TDU_V21_CM_FAX) 

/*******************************************************************************
 * WORD 18
 *******************************************************************************/
#if defined (GG_INCLUDE_DSIGR_AAL2)
  #define GGBIT_DSIGR GGBIT_13
#else
  #define GGBIT_DSIGR 0
#endif

#if defined (GG_HD_HPI)
  #define GGBIT_HDHPI GGBIT_12
#else
  #define GGBIT_HDHPI 0
#endif

#if defined (GG_INCLUDE_RTCP)
  #define GGBIT_RTCP GGBIT_11
#else
  #define GGBIT_RTCP 0
#endif

#if defined (GG_INCLUDE_NEU_UDPIP)
  #define GGBIT_UDPIP GGBIT_10
#else
  #define GGBIT_UDPIP 0
#endif

#if defined (GG_INCLUDE_NEU_AAL5)
  #define GGBIT_AAL5 GGBIT_9
#else
  #define GGBIT_AAL5 0
#endif

#if defined (GG_INCLUDE_NET_ENCAP)
  #define GGBIT_ATM GGBIT_8
#else
  #define GGBIT_ATM 0
#endif

#if defined (GG_GCI)
  #define GGBIT_GCI GGBIT_7
#else
  #define GGBIT_GCI 0
#endif

#if defined (GG_INCLUDE_ANNOUNCEMENTS)
  #define GGBIT_ANNOUNCEMENTS GGBIT_6
#else
  #define GGBIT_ANNOUNCEMENTS 0
#endif

#if defined (GG_INCLUDE_CID_ETSI_DETECT)
  #define GGBIT_CID_ETSIDET GGBIT_5
#else
  #define GGBIT_CID_ETSIDET 0
#endif

#if defined (GG_INCLUDE_CID_ETSI_GENERATE)
  #define GGBIT_CID_ETSIGEN GGBIT_4
#else
  #define GGBIT_CID_ETSIGEN 0
#endif

#if defined (GG_INCLUDE_CID_NTT_DETECT)
  #define GGBIT_CID_NTTDET GGBIT_3
#else
  #define GGBIT_CID_NTTDET 0
#endif

#if defined (GG_INCLUDE_CID_NTT_GENERATE)
  #define GGBIT_CID_NTTGEN GGBIT_2
#else
  #define GGBIT_CID_NTTGEN 0
#endif

#if defined (GG_INCLUDE_CID_US_DETECT)
  #define GGBIT_CID_USDET GGBIT_1
#else
  #define GGBIT_CID_USDET 0
#endif

#if defined (GG_INCLUDE_CID_US_GENERATE)
  #define GGBIT_CID_USGEN GGBIT_0
#else
  #define GGBIT_CID_USGEN 0
#endif


#define GGBITMAP_WORD18  (GGBIT_CID_ETSIDET   | GGBIT_CID_ETSIGEN | \
                          GGBIT_CID_NTTDET    | GGBIT_CID_NTTGEN  | \
                          GGBIT_CID_USDET     | GGBIT_CID_USGEN   | \
                          GGBIT_HDHPI         | GGBIT_RTCP        | \
                          GGBIT_UDPIP         | GGBIT_AAL5        | \
                          GGBIT_ATM           | GGBIT_GCI         | \
                          GGBIT_ANNOUNCEMENTS | GGBIT_DSIGR  )

/*******************************************************************************
 * WORD 19
 *******************************************************************************/
#define GGBIT_MSU_PKTCBL    GGBIT_6
#define GGBIT_MSU_SRTP      GGBIT_7

#if defined(GG_INCLUDE_SECURITY_SRTP)    
  #define GGBITMAP_MSU_SRTP     GGBIT_MSU_SRTP 
#else
  #define GGBITMAP_MSU_SRTP     0
#endif

#if defined(GG_INCLUDE_SECURITY_PKTCBL_RTP)    
  #define GGBITMAP_MSU_PKTCBL   GGBIT_MSU_PKTCBL 
#else
  #define GGBITMAP_MSU_PKTCBL   0
#endif

#if defined(GG_INCLUDE_MSU)
  #define GGBITMAP_MSU_NUM_CHANNELS  GG_NUM_MSU_CHANNELS
#else
  #define GGBITMAP_MSU_NUM_CHANNELS  0
#endif

#define GGBITMAP_WORD19  (GGBITMAP_MSU_SRTP | GGBITMAP_MSU_PKTCBL | \
                          ( GGBITMAP_MSU_NUM_CHANNELS & 0x003F))

/*******************************************************************************
 * WORD 20
 *******************************************************************************/
#define GGBIT_FAX_TFOP      GGBIT_0
#define GGBIT_FAX_FRF11     GGBIT_1
#define GGBIT_FAX_AAL2      GGBIT_2
#define GGBIT_FAX_T38       GGBIT_3

#define GGBIT_GSM_WBAMR     GGBIT_4
#define GGBIT_GSMAMR        GGBIT_5

#define GGBIT_PLC           GGBIT_6  /* G711 Appendix I */
#define GGBIT_NMU           GGBIT_7  /* G711 Appendix II */
#define GGBIT_MIPS_AGENT    GGBIT_8
#define GGBIT_TDU_BELL      GGBIT_9
#define GGBIT_TDU_CALLDISC  GGBIT_10
#define GGBIT_CID_YDNGEN    GGBIT_11
#define GGBIT_CID_YDNDET    GGBIT_12
#define GGBIT_CID_BTGEN     GGBIT_13
#define GGBIT_CID_BTDET     GGBIT_14

#if defined(GG_INCLUDE_FAX_RELAY_S3)    
  #define GGBITMAP_FAX_TFOP     GGBIT_FAX_TFOP 
#else
  #define GGBITMAP_FAX_TFOP     0
#endif

#if defined(GG_INCLUDE_FAX_RELAY_FRF11)    
  #define GGBITMAP_FAX_FRF11    GGBIT_FAX_FRF11 
#else
  #define GGBITMAP_FAX_FRF11    0
#endif

#if defined(GG_INCLUDE_FAX_RELAY_AAL2)    
  #define GGBITMAP_FAX_AAL2     GGBIT_FAX_AAL2 
#else
  #define GGBITMAP_FAX_AAL2     0
#endif

#if defined(GG_INCLUDE_FAX_RELAY_T38)    
  #define GGBITMAP_FAX_T38      GGBIT_FAX_T38 
#else
  #define GGBITMAP_FAX_T38      0
#endif

#if defined(GG_INCLUDE_G722_2)
  #define GGBITMAP_GSM_WBAMR (GGBIT_GSM_WBAMR)
#else
  #define GGBITMAP_GSM_WBAMR 0
#endif

#if defined(GG_INCLUDE_GSMAMR)   /*Bitmap for GSMAMR */
  #define GGBITMAP_GSMAMR    (GGBIT_GSMAMR)
#else
  #define GGBITMAP_GSMAMR    0 
#endif


#if defined(GG_INCLUDE_ECUMA)    
  #define GGBITMAP_MIPS_AGENT   GGBIT_MIPS_AGENT 
#else
  #define GGBITMAP_MIPS_AGENT   0
#endif

#if defined(GG_INCLUDE_BELL)
  #define GGBITMAP_TDU_BELL     GGBIT_TDU_BELL 
#else
  #define GGBITMAP_TDU_BELL     0
#endif

#if defined(GG_INCLUDE_BELL)
  #define GGBITMAP_TDU_CALLDISC GGBIT_TDU_CALLDISC
#else
  #define GGBITMAP_TDU_CALLDISC 0
#endif

#if defined (GG_INCLUDE_CID_YDN_GENERATE)
  #define GGBITMAP_CID_YDNGEN GGBIT_CID_YDNGEN
#else
  #define GGBITMAP_CID_YDNGEN 0
#endif

#if defined (GG_INCLUDE_CID_YDN_DETECT)
  #define GGBITMAP_CID_YDNDET GGBIT_CID_YDNDET
#else
  #define GGBITMAP_CID_YDNDET 0
#endif

#if defined (GG_INCLUDE_CID_BT_GENERATE)
  #define GGBITMAP_CID_BTGEN GGBIT_CID_BTGEN
#else
  #define GGBITMAP_CID_BTGEN 0
#endif

#if defined (GG_INCLUDE_CID_BT_DETECT)
  #define GGBITMAP_CID_BTDET GGBIT_CID_BTDET
#else
  #define GGBITMAP_CID_BTDET 0
#endif

#define GGBITMAP_WORD20  (GGBITMAP_FAX_TFOP     | GGBITMAP_FAX_FRF11   | \
                          GGBITMAP_FAX_AAL2     | GGBITMAP_FAX_T38     | \
                          GGBITMAP_GSM_WBAMR    | GGBITMAP_GSMAMR      | \
                          GGBITMAP_TDU_BELL     | GGBITMAP_TDU_CALLDISC | \
                          GGBITMAP_CID_YDNGEN   | GGBITMAP_CID_YDNDET  | \
                          GGBITMAP_CID_BTGEN    | GGBITMAP_CID_BTDET | \
                          GGBITMAP_MIPS_AGENT )

/*******************************************************************************
 * WORD 21
 *******************************************************************************/
#define GGBIT_FRAME_SIZE_5_MS       GGBIT_15
#define GGBIT_FRAME_SIZE_5_5_MS     GGBIT_14
#define GGBIT_FRAME_SIZE_10_MS      GGBIT_13
#define GGBIT_SMV                   GGBIT_12
#define GGBIT_EVRC                  GGBIT_11
#define GGBIT_QCELP13               GGBIT_10
#define GGBIT_MP3                   GGBIT_9
#define GGBIT_AUDIO                 GGBIT_8

#if defined(AUDIO)
  #define GGBITMAP_AUDIO            GGBIT_AUDIO
#else
  #define GGBITMAP_AUDIO     0
#endif

#if defined(GG_INCLUDE_MP3)
  #define GGBITMAP_MP3              GGBIT_MP3
#else
  #define GGBITMAP_MP3              0
#endif


#if GG_SAMPLING_RATE == 8000
    #if GG_VOICE_FRAME_SIZE == 40
        #define GGBITMAP_FRAME_SIZE_5_MS     GGBIT_FRAME_SIZE_5_MS
    #else
        #define GGBITMAP_FRAME_SIZE_5_MS       0
    #endif

    #if GG_VOICE_FRAME_SIZE == 44
        #define GGBITMAP_FRAME_SIZE_5_5_MS   GGBIT_FRAME_SIZE_5_5_MS
    #else
        #define GGBITMAP_FRAME_SIZE_5_5_MS     0
    #endif

    #if GG_VOICE_FRAME_SIZE == 80
        #define GGBITMAP_FRAME_SIZE_10_MS    GGBIT_FRAME_SIZE_10_MS
    #else
        #define GGBITMAP_FRAME_SIZE_10_MS      0
    #endif
#endif

#if GG_SAMPLING_RATE == 16000 
    #if GG_VOICE_FRAME_SIZE == 80
        #define GGBITMAP_FRAME_SIZE_5_MS     GGBIT_FRAME_SIZE_5_MS
    #else
        #define GGBITMAP_FRAME_SIZE_5_MS       0
    #endif

    #if GG_VOICE_FRAME_SIZE == 88
        #define GGBITMAP_FRAME_SIZE_5_5_MS   GGBIT_FRAME_SIZE_5_5_MS
    #else
        #define GGBITMAP_FRAME_SIZE_5_5_MS     0
    #endif

    #if GG_VOICE_FRAME_SIZE == 160
        #define GGBITMAP_FRAME_SIZE_10_MS    GGBIT_FRAME_SIZE_10_MS
    #else
        #define GGBITMAP_FRAME_SIZE_10_MS      0
    #endif
#endif

#if defined(GG_INCLUDE_EVRC)               
  #define GGBITMAP_EVRC     GGBIT_EVRC
#else
  #define GGBITMAP_EVRC     0
#endif

#if defined(GG_INCLUDE_SMV)               
  #define GGBITMAP_SMV      GGBIT_SMV
#else
  #define GGBITMAP_SMV      0
#endif
#if defined(GG_INCLUDE_QCELP13)               
  #define GGBITMAP_QCELP13      GGBIT_QCELP13
#else
  #define GGBITMAP_QCELP13      0
#endif

#define GGBITMAP_WORD21  (GGBITMAP_AUDIO | GGBITMAP_MP3 | GGBITMAP_FRAME_SIZE_5_MS    | GGBITMAP_FRAME_SIZE_5_5_MS  | \
                          GGBITMAP_FRAME_SIZE_10_MS   | (GG_MAX_VOICE_PLAYOUT_FIFO_SIZE & 0x00FF) | \
                          GGBITMAP_EVRC               | GGBITMAP_SMV |  GGBITMAP_QCELP13 ) 
                          
                          /* AER Instances */
#if defined(GG_INCLUDE_AER)              
  #define GGBITMAP_AER_INST0_INFO       GG_AER_INST0_INFO
  #define GGBITMAP_AER_INST1_INFO       GG_AER_INST1_INFO
  #define GGBITMAP_AER_INST2_INFO       GG_AER_INST2_INFO
  #define GGBITMAP_AER_INST3_INFO       GG_AER_INST3_INFO
#else
  #define GGBITMAP_AER_INST0_INFO       0
  #define GGBITMAP_AER_INST1_INFO       0
  #define GGBITMAP_AER_INST2_INFO       0
  #define GGBITMAP_AER_INST3_INFO       0
#endif

#define GGBITMAP_WORD22 GGBITMAP_AER_INST0_INFO
#define GGBITMAP_WORD23 GGBITMAP_AER_INST1_INFO
#define GGBITMAP_WORD24 GGBITMAP_AER_INST2_INFO
#define GGBITMAP_WORD25 GGBITMAP_AER_INST3_INFO


/*******************************************************************************
 * WORD 32
 *******************************************************************************/

#define GGBIT_AIC23        GGBIT_0
#define GGBIT_48KHZ        GGBIT_1


#if defined(GG_AIC23)
  #define GGBITMAP_AIC23  GGBIT_AIC23
#else
  #define GGBITMAP_AIC23  0
#endif

#if (defined(GG_AUDIO_SAMPLING_RATE) && (GG_AUDIO_SAMPLING_RATE == 48000))
  #define GGBITMAP_48KHZ     GGBIT_48KHZ
#else
  #define GGBITMAP_48KHZ     0
#endif

#ifdef GG_INCLUDE_REDUNDANCY
  #define GGBITMAP_REDUNDANCY  ((GG_INCLUDE_REDUNDANCY_SLOTS << MGB_REDUNDANCY_SLOT_SHIFT))
#else
  #define GGBITMAP_REDUNDANCY  0
#endif 

#define GGBITMAP_WORD32  ( GGBITMAP_AIC23 | GGBITMAP_48KHZ | GGBITMAP_REDUNDANCY)


/*******************************************************************************
 * WORD 33
 *******************************************************************************/
#if defined(RELMSG_SUPPORT)
  #define GGBITMAP_RELMSG   GGBIT_6
#else
  #define GGBITMAP_RELMSG   0
#endif   

#define GGBITMAP_WORD33  (GGBITMAP_RELMSG) 

/*******************************************************************************
 * WORD 34
 *******************************************************************************/
#define GGBIT_H264BP_ENC   GGBIT_0
#define GGBIT_MPEG4_ENC    GGBIT_1
#define GGBIT_H263_ENC     GGBIT_2
#define GGBIT_MPEG2MP_DEC  GGBIT_3
#define GGBIT_MPEG4_DEC    GGBIT_4
#define GGBIT_H263_DEC     GGBIT_5
#define GGBIT_H264BP_DEC   GGBIT_6
#define GGBIT_H264MP_DEC   GGBIT_7
#define GGBIT_H264HP_DEC   GGBIT_8

#define GGBITMAP_WORD34  ( GGBIT_H264BP_ENC  | GGBIT_MPEG4_ENC  | GGBIT_H263_ENC  | \
                           GGBIT_MPEG2MP_DEC | GGBIT_MPEG4_DEC  | GGBIT_H263_DEC  | \
                           GGBIT_H264BP_DEC  | GGBIT_H264MP_DEC | GGBIT_H264HP_DEC)

#endif
/* Nothing past this point */

