/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _GGFEATURE2ECO_H
#define _GGFEATURE2ECO_H
/******************************************************************************
 * FILE PURPOSE: Golden Gateway Platform Mappings From Features to ECO Software 
 *               Configuration
 ******************************************************************************
 * FILE NAME: ggfeature2eco.h
 *
 * DESCRIPTION: Derives internally related #define(s) from the definitions
 *            of required features (in configuring Golden Gateway DSP software),
 *            i.e. maps features to ECOs.
 * These information are required by:
 * 1. ggcfg.c (to create call tables)     
 * 2. siutbl.c (to create table of valid messages from micro)          
 ******************************************************************************/
#include "mgdefs.h"
#ifdef GMP
#include "gmp.h"
#endif

#ifdef GG_INCLUDE_CID
#error GG_INCLUDE_CID in ggswcfg.h is obsolete.
#endif

#if (defined(GG_INCLUDE_CID_US_GENERATE) || defined(GG_INCLUDE_CID_US_DETECT) || \
     defined(GG_INCLUDE_CID_NTT_GENERATE) || defined(GG_INCLUDE_CID_NTT_DETECT) || \
     defined(GG_INCLUDE_CID_ETSI_GENERATE) || defined(GG_INCLUDE_CID_ETSI_DETECT) || \
     defined(GG_INCLUDE_CID_YDN_GENERATE) || defined(GG_INCLUDE_CID_YDN_DETECT) || \
     defined(GG_INCLUDE_CID_BT_GENERATE) || defined(GG_INCLUDE_CID_BT_DETECT) )
   #define GG_INCLUDE_CID
#endif

#if (defined(GG_INCLUDE_CID_US_GENERATE) || \
     defined(GG_INCLUDE_CID_NTT_GENERATE) || \
     defined(GG_INCLUDE_CID_ETSI_GENERATE) || \
     defined(GG_INCLUDE_CID_YDN_GENERATE) || \
     defined(GG_INCLUDE_CID_BT_GENERATE) )
   #define GG_INCLUDE_CID_GENERATE
#endif

#if (defined(GG_INCLUDE_CID_US_DETECT) || \
     defined(GG_INCLUDE_CID_NTT_DETECT) || \
     defined(GG_INCLUDE_CID_ETSI_DETECT) || \
     defined(GG_INCLUDE_CID_YDN_DETECT) || \
     defined(GG_INCLUDE_CID_BT_DETECT) )
   #define GG_INCLUDE_CID_DETECT
#endif

#ifdef GG_INCLUDE_TDU
#error GG_INCLUDE_TDU in ggswcfg.h is obsolete
#endif

#if (defined(GG_INCLUDE_V25) || defined(GG_INCLUDE_V21) || \
     defined(GG_INCLUDE_CPT) || defined(GG_INCLUDE_V18A) || \
     defined(GG_INCLUDE_SS7COT) || defined(GG_INCLUDE_SS7COR) || \
     defined(GG_INCLUDE_SF) || defined(GG_INCLUDE_CNG) || \
     defined(GG_INCLUDE_TT1KHZ) || defined(GG_INCLUDE_CIDCAS) || \
     defined(GG_INCLUDE_MF_DTMF) || defined(GG_INCLUDE_MF_R1) || \
     defined(GG_INCLUDE_MF_R2FW) || defined(GG_INCLUDE_MF_R2BW) || \
     defined(GG_INCLUDE_CALLDISC) || defined(GG_INCLUDE_V21_CM) || \
     defined(GG_INCLUDE_COIN_TONES) )
   #define GG_INCLUDE_TDU
#endif

#ifdef SIU_USE_LLMR_VOICE
#  ifndef GG_INCLUDE_V21
#    define GG_INCLUDE_V21
#  endif
#  ifndef GG_INCLUDE_V25
#    define GG_INCLUDE_V25
#  endif
#endif

#ifdef GG_INCLUDE_ECUMA
#error GG_INCLUDE_ECUMA in ggswcfg.h is obsolete.
#endif

#if (defined(GG_ECUMA_CORE_MIPS)   || defined(GG_ECUMA_GUARD_MIPS) || \
     defined(GG_ECUMA_STATIC_MIPS) || defined(GG_ECUMA_ALLOC_TICK) || \
     defined(GG_ECUMA_RRATE_MS)    || defined(GG_INCLUDE_ECUMA_CTRL))

#ifdef LEC_ECU
#define GG_INCLUDE_ECUMA 
#endif
#define GG_ECUMA_MAX_MIPS     (GG_ECUMA_CORE_MIPS - GG_ECUMA_GUARD_MIPS - GG_ECUMA_STATIC_MIPS) 
#define GG_ECUMA_UPDT_TICK    (GG_ECUMA_ALLOC_TICK + ((6 * 8)/GG_BSP_SEGMENT_LENGTH))  
#define GG_ECUMA_SRCH_TICK    (GG_ECUMA_UPDT_TICK  + ((6 * 8)/GG_BSP_SEGMENT_LENGTH))
#define GG_ECUMA_REFRESH_RATE ((GG_ECUMA_RRATE_MS * 8)/GG_BSP_SEGMENT_LENGTH)

#endif
/***********************************************************************************/

#if (defined(GG_INCLUDE_NET_ENCAP) || defined(GG_INCLUDE_NEU_UDPIP) || \
     defined(GG_INCLUDE_NEU_ETHERNET) || defined(GG_INCLUDE_NEU_PPP) || \
     defined(GG_INCLUDE_NEU_LLCSNAP) || defined(GG_INCLUDE_NEU_AAL5))

#define GG_INCLUDE_NEU

#endif

#ifndef GG_INCLUDE_PSU
#define GG_NUM_AES_CHANNELS 0
#endif

/* PCM, LBRC configurations */
/* todo: cleanup this define it shouldn't be required when neither of M + N or M/N is required */
#define GG_MAX_NUM_BITMAP_WORDS            ((GG_MAX_NUM_CHANNELS+15)/16)
#if (defined(GG_INCLUDE_DYN_CH_M_DIV_N_CONFIG) || defined(GG_INCLUDE_DYN_CH_M_ADD_N_CONFIG))
 #define GG_INCLUDE_DYN_BUF_ALLOCATION
 
 #ifndef GG_DYN_CH_MAX_LBRC_CHANNELS
  #error GG_DYN_CH_MAX_LBRC_CHANNELS not defined in ggswcfg.h
 #endif
 
 #define GG_NUM_PCM_ONLY_CH                 (GG_MAX_NUM_CHANNELS - GG_DYN_CH_MAX_LBRC_CHANNELS)

#else
 #define GG_NUM_PCM_ONLY_CH                 0
#endif

#if (defined(GG_G723_BUFF_SWAP_ENABLE) || defined(GG_G729AB_BUFF_SWAP_ENABLE))
 #define GG_SWAP_CODEC_BUFFERS_ENABLE
#endif

#if (defined(GG_INCLUDE_DMA_DATASWAPPING) || defined(GG_INCLUDE_REGULAR_DATASWAPPING))
 #define GG_INCLUDE_SWAP
#endif

#ifdef SIU_TDU_HD
#ifndef GG_INCLUDE_AUTOCOT
 # error GG_INCLUDE_AUTOCOT not defined inspite of TDU_HD command line option set to YES
#endif
#endif

#if (defined(GG_INCLUDE_GSMAMR)   || defined(GG_INCLUDE_G722_2) || defined(GG_INCLUDE_EVRC) || \
     defined(GG_INCLUDE_SMV) || defined(GG_INCLUDE_QCELP13))
 #define GG_INCLUDE_WIRELESS_CODECS
#endif

#if (defined(GG_INCLUDE_GSMAMR) || defined(GG_INCLUDE_G722_2) || defined(GG_INCLUDE_EVRC) || \
     defined(GG_INCLUDE_SMV)    || defined(GG_INCLUDE_QCELP13) || \
    defined(GG_INCLUDE_G723))
#define GG_INCLUDE_MULTIRATEMODE_CODECS
#endif

#if(defined(GG_INCLUDE_GSMAMR) && defined(GG_INCLUDE_GSMEFR))
 # error Obselete definition of GG_INCLUDE_GSMEFR when GG_INCLUDE_GSMAMR is already present.
#endif


/* GMC sizing to support wideband and narrowband builds along with 5 or 10 ms PC frame sizes.*/
#ifdef GMP
#if (((GG_MAX_VOICE_FRAME_LENGTH*1000)/GG_SAMPLING_RATE) == 10)
  #define GG_GMC_SIZE  GMC_MAX_PCM_10MS_SIZE
#else /* frame size is not 10 ms */
  #if defined(GG_INCLUDE_G722_2)
    #define GG_GMC_SIZE GMC_MAX_WBAMR_SIZE
  #else
    #define GG_GMC_SIZE GMC_MAX_PCM_5MS_SIZE
  #endif
#endif
#else
  #define GG_GMC_SIZE 0
#endif

 
#ifdef GG_INCLUDE_MP3
  #define GG_MAX_MP3_CODE_WORD_SEG_SIZE      (736*16) /*maximum size in bits before a decode can start*/
  #define GG_MAX_MP3_SAMPLES_PER_CODE_WORD   (1152)   /*Max samples generated by audio codec(mp3) per codeword*/
#else 
  #define GG_MAX_MP3_CODE_WORD_SEG_SIZE      (0)  
  #define GG_MAX_MP3_SAMPLES_PER_CODE_WORD   (0)  
#endif

#define GG_MAX_AUDIO_CODE_WORD_SEG_SIZE    (GG_MAX_MP3_CODE_WORD_SEG_SIZE)
#define GG_MAX_AUDIO_SAMPLES_PER_CODE_WORD (GG_MAX_MP3_SAMPLES_PER_CODE_WORD)

#endif /*  _GGFEATURE2ECO_H */

/* nothing past this point */

