#ifndef _GGMBUF_H
#define _GGMBUF_H
/******************************************************************************
 * FILE PURPOSE: Data structure for memory allocation
 ******************************************************************************
 * FILE NAME:  ggmbuf.h
 *
 * DESCRIPTION: Defines data structures required for allocation of ECO
 *              memory buffers and channel memory heap. Each channel has a
 *              FAX and VOICE memory heap whose location and size are by the
 *              "chHeapDesc" data strucure. The chBuffDesc structure is
 *              used to individually specify the location of each ECO memory
 *              buffer for each channel.  In general, if a value of NULL is
 *              specified as the buffer address -- then the buffer is to be
 *              allocated from the associated channel heap.
 *
 *    NOTE:  Presently, the number of buffers associated with each ECO is
 *           defined in this file.  This number MUST be the same as that
 *           defined in the ECO.  Therefore, when an ECO is created, SIU
 *           MUST check that the number of buffers defined in chBuffDesc
 *           for that ECO is correct.
 *
 *
 * TABS: NONE
 *
 * $Id: ggmbuf.h 1.34.2.2 1999/12/16 17:58:45 qsu Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: ggmbuf.h $
 * Revision 1.34.2.2  1999/12/16 17:58:45  qsu
 * - Added a buffer in RSU.
 * Revision 1.34.2.1  1999/12/14 21:32:39  qsu
 * - Added RSU.
 * Revision 1.34  1999/10/27 22:13:36  gsisli
 *   G729 buffers are 6.2.1 compatible.
 * Revision 1.33  1999/10/05 14:03:46  sakhtar
 * Merged changes for the fax only build
 * Revision 1.32.1.1  1999/09/30 14:17:22  sakhtar
 * Modified to add ACU to the FAX only build
 * Revision 1.32  1999/09/18 16:16:41  qsu
 * - G723 buffer number changed.
 * Revision 1.31  1999/09/15 15:27:47  qsu
 * - Merged 1.30 (g729e integration) and 1.29.1.1 (AAl2 integration).
 * Revision 1.29.1.1  1999/09/10 22:14:16  jdowdal
 *    AAL2 Integration
 * Revision 1.29  1999/08/09 22:41:31  qsu
 * - Redefined codec buffers.
 * Revision 1.28  1999/08/09 18:21:03  qsu
 * - Corrected a type error.
 * Revision 1.27  1999/08/09 16:00:39  jdowdal
 *    Memory allocation for 6.2 codec arch
 * Revision 1.26  1999/07/26 13:33:07  qsu
 * - Added definitions for G728.
 * Revision 1.25  1999/07/08 14:49:35  qsu
 * - Changed G729B buffer number from 6 to 7.
 * Revision 1.24  1999/06/28 21:27:44  qsu
 * - ggchMemSwapDesc structure changed.
 * Revision 1.23  1999/06/25 14:59:59  BONAL
 *    - ggchMemSwapDesc structure changed.
 * Revision 1.22  1999/06/24 19:56:08  BONAL
 *   - Swapping structures modified to have a task based swapping.
 * Revision 1.21  1999/06/17 20:08:25  BONAL
 *    - Long Tail ECU support added.
 * Revision 1.20  1999/05/20 18:02:41  BONAL
 *    - Conferencing added. 
 * Revision 1.19.1.1  1999/05/17 18:37:41  tgeorge
 * Revision 1.19  1999/05/14 20:14:08  qsu
 * - Added one buffer in TDU.
 * Revision 1.18  1999/05/14 18:36:22  qsu
 * - Removed FDU.
 * Revision 1.17  1999/03/18 16:46:12  ffruth
 * - Modified for FAX-only builds.
 * Revision 1.16  1999/03/18 18:56:21  qsu
 * - The number of PVP buffers is changed.
 * Revision 1.15  1999/03/15 14:51:10  ffruth
 * - Integrated new TDU V.21 detector.  Number of TDU memory
 *   buffers reduced fom 6 to 5.
 * Revision 1.14  1999/03/12 20:24:18  qsu
 * - Added CID buffers.
 * Revision 1.13  1998/12/05 15:20:04  BONAL
 *   - G729AB added.
 * Revision 1.12  1998/12/05 14:06:05  BONAL
 *     - TTU instance buffer added.
 * Revision 1.11  1998/11/20 14:43:10  ffruth
 * - Increased FIU number of buffers to 4. Fax Integration,,
 * Revision 1.10  1998/11/18 11:57:33  BONAL
 *   - Merge of 1.9 and 1.8.1.1 for ggmbuf swap changes.
 * Revision 1.8.1.1  1998/11/17 11:03:57  BONAL
 *    - Changes for ADPCM/PCM integration.
 * Revision 1.8  1998/11/12 17:45:16  BONAL
 *     - Merging of 1.7 and 1.6.4.2 
 * Revision 1.6.4.2  1998/11/07 11:27:25  BONAL
 *    - Removed one g723 buffer.
 * Revision 1.6  1998/07/09 08:53:53  ppatel
 *  Integration of new fiu in which the MD task has been eliminated
 * Revision 1.5  1998/04/21 15:50:56  ppatel
 *  Changes for integrating new eco "fdu" (Frame Delay Unit)
 * Revision 1.4  1998/04/19 15:46:51  ppatel
 *   Changes required for implementation of memory swapping
 * Revision 1.3  1998/04/07 16:14:35  ppatel
 *  Changes made for supporting G723; also ggConfig contains more 
 * parameters depending on voice frame size (because of different codec)
 * Revision 1.2  1998/04/02 09:25:45  ppatel
 *  Changes for integrating new TDU for Release 4.0 of gg
 * Revision 1.1  1998/02/06 22:17:26  ffruth
 * Initial revision
 *
 * (C) Copyright 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "ggport.h"

#ifndef max
#define max(x,y) ((x) > (y) ? (x):(y))
#endif

/* Definitions to be updated everytime a codec is added/removed from build  */
#define GG_MAX_NUM_VOICE_TABLES                   8 /* FAX + second G723 chan */
#define GG_MAX_NUM_FAX_TABLES                     5 /* Voice first chan       */
#define GG_MAX_NUM_TABLES         max(GG_MAX_NUM_VOICE_TABLES,GG_MAX_NUM_FAX_TABLES)
#define GG_TBLRELOC_MAX_NUM_CHANNELS             32 /* To be updated everytime*/
#define GG_TBLRELOC_MAX_EXTHEAP_BLOCKS_REQD       3 /*Need to be set accordingly,
                                                      when new codecs are added */
/**************** START Dynamic Heap Allocation Definitions *******************/
#define GG_DYNHEAP_CHUNKNUM_PER_HEAP 3 /* TODO! Why is this used in various modules?? Move it to make file */

/* Different modules from different groups can't go in one memory chunk */
#define GG_DYNHEAP_NO_OF_GRP 2  /* This also indicates the maximum number of memory chunks  
                                   that will be freed when one channel closes */ 
#define GG_DYNHEAP_GRP0_ID  0
#define GG_DYNHEAP_GRP1_ID  1
#define GG_DYNHEAP_GRP2_ID  2

/**************** END Dynamic Heap Allocation Definitions *******************/
/* There are  max numbers and can be here, as of now */
#define GG_DYNHEAP_MAX_BUFF_PER_MOD  2 /* # of buffers per modules per dyn.heap */
#define GG_DYNHEAP_RELOC_ARR_LEN     2 /* length of buffer relocation info array */

/* Relocation Definitions for Modules */
#define GG_RELOC_PLC_INFO     CPU_RELOC_PLC_INFO /* Can be overridden in ggmemmap.beh */
#define GG_RELOC_RX_NMU_INFO  CPU_RELOC_RX_NMU_INFO /* Can be overridden in ggmemmap.beh */
#define GG_RELOC_TX_NMU_INFO  CPU_RELOC_TX_NMU_INFO /* Can be overridden in ggmemmap.beh */

#define GG_RELOC_RTP_MSU_INFO  CPU_RELOC_RTP_MSU_INFO /* Can be overridden in ggmemmap.beh */
#define GG_RELOC_RTCP_MSU_INFO  CPU_RELOC_RTCP_MSU_INFO /* Can be overridden in ggmemmap.beh */
#define GG_RELOC_PLR_INFO  CPU_RELOC_PLR_INFO /* Can be overridden in ggmemmap.beh */
#define GG_RELOC_PLR_GMPSUPP_INFO  CPU_RELOC_PLR_GMPSUPP_INFO /* Can be overridden in ggmemmap.beh */


/**************** END   Dynamic Heap Allocation Definitions *******************/

#define GG_NUM_HDNDU_BUFF       8
#define GG_NUM_UTOPIA_NDU_BUFF  3
#define GG_NUM_MAC_NDU_BUFF     3
#define GG_NUM_NEU_BUFF        5  

/* assigning 2 as the buffer swap spaces 
 * CHECK 
 */
#define GG_NUM_AER_SWAP_SPACES 2



typedef struct {
  word *buffAddr;
  int   buffSize;
} ggbufDesc_t;

/* TODO: move to ggConfig */
extern  const tuint ggCodecTablesInfoEntries;

typedef struct {
  word  *heapAddr;
  uint  heapSize;
} ggconfHeapDesc_t;

/*-------------------------------------------------------------------------*/
/*  Data Structures for Heap memory allocation table                       */
/*-------------------------------------------------------------------------*/

typedef struct {
  word  *heapStartAddr;
  word  *heapEndAddr;
  uint  heapSize;
} gghalHeapDesc_t;

typedef struct {
  word  *tecEngHeapBase;
  uint   heapsize;
  word  *tecEngScratchStart;
  uint   tecEngScratchSize;
} ggtecEngHeapDesc_t;

typedef struct {
  word  *heapAddr;
  uint   heapSize;
} ggdsigtHeapDesc_t;

typedef struct {
  word  *heapAddr;
  uint   heapSize;
} ggdsigrHeapDesc_t;

typedef struct {
  int   numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_HDNDU_BUFF];
} ggnduHeapDesc_t;

typedef struct {
  int   numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_UTOPIA_NDU_BUFF];
} ggutNduDesc_t;

typedef struct {
  int   numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_UTOPIA_NDU_BUFF];
} gggmNduDesc_t;

typedef struct {
  word *heapAddr;
  uint heapSize;
} ggaerHeapDesc_t;

typedef struct {
  word  *heapAddr;
  uint  heapSize;
} ggsiouHeapDesc_t;

typedef struct {
  word  *heapAddr;
  uint  heapSize;
} ggclearDataHeapDesc_t;

typedef struct {
  word  *heapAddr;
  uint  heapSize;
} ggllmrHeapDesc_t;

typedef struct {
  word  *heapAddr;
  uint  heapSize;
} ggmodemHeapDesc_t;

/* Dynamic heap allocation usage info */
/* Contains info about each module that is allocated from dynamic heap */
typedef struct {
  tint   module_id; 
  tint   grp_id;
  tuint  relocInfo[GG_DYNHEAP_RELOC_ARR_LEN];
  tuint  offset;
  tint   buffNum;
  tuint *buffOffset;  
} ggDynHeapModInfo_t; 

typedef struct {
  tint                numOfModules;
  tint                maxBuffPerMod;
  ggDynHeapModInfo_t  *modInfo;
} ggDynHeapDesc_t;

typedef enum {
  GG_DYNHEAP_TYPE_NONCH,/* Non-voice heap.i.e dedicated area */
  GG_DYNHEAP_TYPE_CH	 /* Per channel area used for heap */
} ggDynHeapType_e;


/* Structure for program to data memory swapping */
typedef struct {
  tulong eramStartAddr; /* external program memory start address (far)  */
  tulong iramStartAddr; /* internal data    memory start address (near) */
  tuint  size;          /* size in words */
} ggchMemSwapDesc_t;

/* Structure for selective program to data memory swapping */
typedef struct {
  tint  moduleId;          /* Id of the module corresponding to the swap constants/data
                             * - Is the Voice coding type defined in the vctypes.h for voice 
                             *   codecs
                             */
  tint validSwappedInSize;   /* This is the uncorrupted swap size of the swaped data 
                              * that is present in internal memeory, because of an earlier swap
                              * The purpose is to minimize the actiual swaps required,
                              * by utilizing earlier swaps in the system */
  ggchMemSwapDesc_t ggModuleDataSwapDesc;
}ggModuleDataSwapDesc_t;

/* Structure for data to data memory swapping */
typedef struct {
  word   *eramStartAddr;  /* external data memory start address (far)  */
  word   *iramStartAddr;  /* internal data memory start address (near) */
  tuint  size;            /* size in words */
} ggchD2DSwapDesc_t;

/* Structure for table of swap lengths for different Vct*/
typedef struct {
  tuint  codecType; /* codec type */
  tuint  rxlength; /* size in words */
  tuint  txlength;          /* size in words */
} ggVctMemSwapDesc_t;

/* Structure for utopia NDU message aal5 instances */
typedef struct {
  word   *highAddr;
  word   *normalAddr;
  word   *lowAddr;
  tuint   size;
} ggutNduAal5Desc_t;

typedef struct {
  int  numBuffs;
  ggbufDesc_t bufDesc[GG_NUM_NEU_BUFF];
} ggbuffDescNEU_t;

/*-------------------------------------------------------------------------*/
/* Storage for non-voice modules which make use of eco modules             */
/* This is used for message between DSP and host while using NEU           */
/* to format check headers required for network transport                  */
/*-------------------------------------------------------------------------*/
typedef struct {
  ggbuffDescNEU_t    buffDescNEU;
} ggNvEcoBuffDesc_t;
  
/*-------------------------------------------------------------------------*/
/* Enumeration of the "channels" used for non-voice eco allocation         */
/*-------------------------------------------------------------------------*/
enum {
  GG_NVECO_MSG         = 0,       /* Message "channel"                 */
  GG_NVECO_BLK,       /* 1 */     /* Bulk memory request "channel"     */
  GG_NVECO_HLT,       /* 2 */     /* Keep Alive "channel"              */
  GG_NVECO_TRACE      /* 3 */     /* Tracing "channel"                 */
};


/*-------------------------------------------------------------------------*/
/* Structure for defining blocks to be used for checksum computation */
typedef struct {
  tulong blck_start;   /* Starting address of the block */
  tuint  blck_size;    /* Total size of page            */
} ggChkSum_Blck_t;
/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------*/
/*  Data Structures for Table Relocation                                   */
/*-------------------------------------------------------------------------*/
typedef struct {
void(*tableAddr)(void); /* Address of the table in shared memory */
tint  tableSize; /* Size of the table */
tint  blockno;   /* Specifies which block the table should go in to */
}ggTableDesc_t;

/* Structure for each codec */
typedef struct{
  tint enc_numTables;
  ggTableDesc_t *enc_TableDesc;
  tint dec_numTables;
  ggTableDesc_t *dec_TableDesc;
  tint common_numTables;
  ggTableDesc_t *common_TableDesc;
}ggTableDesc_xxx_t;

/* Type of relocation information */
typedef enum {
  GG_RELOC_CODEC,
  GG_RELOC_FAX
} ggRelocEco_t;

#ifdef SIU_MULTICORE_SHARED_DATA
/* Structure for supporting per-core data in shared memory.
 * Define the buffers for core 0; if address x falls in one of the ranges 
 * specified by this structure they will be remapped to 
 * ((core #) * (area_end - area_begin)) + x
 */
typedef struct {
  word *area_begin;  /* Beginning address of remappable shared region */
  word *area_end;    /* Ending address of remappable shared region */
} ggMultiCoreShared_t;

typedef struct {
  ggMultiCoreShared_t *build;
} vigdkMultiCoreSharedBuild_t;

#endif

/* External definition of memory buffer allocation table */
//extern relocQualifier ggchBuffDesc_t ggchBuffDesc[];
/* External definition of conferencing heap memory */
extern relocQualifier ggconfHeapDesc_t confHeapDesc;
extern relocQualifier ggconfHeapDesc_t cbuHeapDesc;
extern relocQualifier ggconfHeapDesc_t cbuScratchDesc;

/* External definition of channel memory heap allocation table */
/* extern const ggchHeapDesc_t ggchHeapDesc[]; */
extern word * relocQualifier ggchVoiceHeapStartAddr[];
extern word * relocQualifier ggchFaxHeapStartAddr[];
extern word * relocQualifier ggchClearDataHeapStartAddr[];
extern word * relocQualifier ggchLLMRheapStartAddr[];
extern word * relocQualifier ggchModemHeapStartAddr[];
extern word * relocQualifier ggchAudioHeapStartAddr[];
extern word * relocQualifier ggchnx64HeapStartAddr[];
 
extern relocQualifier tuint ggchVoiceHeapSize;
extern relocQualifier tuint ggchFaxHeapSize;
extern relocQualifier tuint ggchClearDataHeapSize;
extern relocQualifier tuint ggchLLMRheapSize;
extern relocQualifier tuint ggchModemHeapSize;
extern relocQualifier tuint ggchAudioHeapSize;
extern relocQualifier tuint ggchVoiceAuxHeapSize;
extern relocQualifier tuint ggchnx64HeapSize;

/* Tx network buffer */
extern word * relocQualifier ggTxNetwork_data[];

/* External definition of constants overlay information table */
extern relocQualifier ggchMemSwapDesc_t ggchMemConstOvlyDesc[];

/* External definition of constants overlay information table */
extern relocQualifier ggchMemSwapDesc_t ggchMemConstOvlyDesc[];

/* External definition of channel swap memory information table */
extern relocQualifier ggchMemSwapDesc_t ggchMemSwapDesc[];

/* External definition of AER channel swap memory information table */
extern relocQualifier ggchMemSwapDesc_t ggAerMemSwapDesc[][GG_NUM_AER_SWAP_SPACES];

/* External definition of MSU constant swap memory information table */
extern relocQualifier ggchMemSwapDesc_t ggMsuMemSwapDesc[];

/* External definition of NMUPLC constant swap memory information table */
extern relocQualifier ggchMemSwapDesc_t ggNmuPlcMemSwapDesc[];

/* External definition of channel constants data swap memory information table */
extern ggModuleDataSwapDesc_t ggModuleDataSwapDesc[];

/* External definition of ECU search buffer swap memory information table */
extern relocQualifier ggchD2DSwapDesc_t ggchECUSwapDesc[];

/* External definition of ggmbuf.c swap memory start address */
extern relocQualifier tulong ggmbufStartAddrInPRAM;

/* External definition of AER heap memory */
extern relocQualifier ggaerHeapDesc_t aerHeapDesc[];

/* Externel definition of hal heap memory */
extern relocQualifier gghalHeapDesc_t halHeapDesc;

/* External definition of dsigt memory */
extern relocQualifier ggdsigtHeapDesc_t dsigtHeapDesc;

/* External definition of TEC engine memory */
extern relocQualifier ggtecEngHeapDesc_t tecEngHeapDesc;

/* External definition of dsigr memory */
extern relocQualifier ggdsigrHeapDesc_t dsigrHeapDesc;

/* External definition of dynamic heap allocation */
extern ggDynHeapDesc_t dynHeapDesc;

/* External definition of high density NDU heap memory */
extern relocQualifier ggnduHeapDesc_t nduHeapDesc;

/* External definition of utopia NDU heap memory */
extern relocQualifier ggutNduDesc_t   utNduHeapDesc;

/* External definition of gmac NDU heap memory */
extern const gggmNduDesc_t   gmacNduHeapDesc;

/* External definition of SIOU (serial packet I/O) heap memory */
extern relocQualifier ggsiouHeapDesc_t siouHeapDesc;

/* External definition of utopia message aal5 instances */
extern const ggutNduAal5Desc_t ggutNduAal5Desc;

/* External definition for channel data page array. This array specifies
 * what data page each channel uses. (For DMPR) */
extern const tuint gg_channelDataPageInfo[];

/* External definition for secondary channel data page array. This array specifies
 * what secondary data page each channel uses. (For DMPR) */
extern const tuint gg_SPchannelDataPageInfo[];

/* # of program blocks available with location and size info - used by 
 * Checksum computation. */
extern relocQualifier ggChkSum_Blck_t ggChkSum_LocalBlck[];
extern relocQualifier ggChkSum_Blck_t ggChkSum_ProgBlck[];

/* Extern definition of non-voice channel eco buffers */
 extern const ggNvEcoBuffDesc_t ggNvBuffDesc[]; 

/* ------------------------ Added for Table Relocation --------------------- */
/* Data structure to hold shared memory addresses */
extern ggTableDesc_xxx_t * ggSharedMemTableAddresses[];

/* Space in words of an extra heap to be used for tables */
extern relocQualifier tuint ggtblExtraHeapSize;
/* Slot heap usage tracking array per slot */
extern tuint ggTblRelocHeapUsed[];

extern 
#ifndef SIU_MULTICORE_SHARED_DATA
relocQualifier 
#endif
word  *gg_siuheap_inst[];
extern const          tuint  gg_siuheap_inst_len;
#ifdef GG_DYNSUBSLOT
extern relocQualifier word  *gg_siu_inst[];
extern const          tuint  gg_siu_inst_len;
#endif

extern word  *gg_siuheap_inst_ext[];
extern const          tuint  gg_siuheap_inst_ext_len;

/* VIGDK */
typedef struct {
  word *area_begin;  /* Beginning address of remappable shared region */
  word *area_end;    /* Ending address of remappable shared region */
  tulong chunk_size;
} vigdkGmpDesc_t;

typedef struct {
  word *area_begin;  /* Beginning address of remappable shared region */
  word *area_end;    /* Ending address of remappable shared region */
} vigdkMemDesc_t;

typedef struct {
  const vigdkGmpDesc_t vppuHeadersDefault;   /* Default sized vppu Headers */
  const vigdkGmpDesc_t vppuPackets;
  const vigdkGmpDesc_t dec_resindep;
  const vigdkGmpDesc_t enc_resindep;
  const vigdkGmpDesc_t tsu;
  const vigdkGmpDesc_t dec_resdep;
  const vigdkGmpDesc_t enc_resdep;
  const vigdkGmpDesc_t dec_audio;
  const vigdkGmpDesc_t enc_audio;
  const vigdkGmpDesc_t yuv;
  const vigdkGmpDesc_t aud;
  const vigdkGmpDesc_t dec_resdep_sec;
  const vigdkMemDesc_t system_scratch;
  const vigdkMemDesc_t encdec_scratch;
  const vigdkMemDesc_t encdec_intshm;  /* Multi-core internal shared memory */
  const vigdkMemDesc_t encdec_extshm;  /* Multi-core external shared memory */
  const vigdkMemDesc_t mips_profiling;
  const tint           marRegs[33];
  const tint           marVals[33];
} vigdkMemoryContext_t;

#endif /* GGMBUF_H */

/* nothing past this point */
