/******************************************************************************
 * FILE PURPOSE: Reflection page configuration
 ******************************************************************************
 * FILE NAME:   ggref.h
 *
 * DESCRIPTION: Reflection page definitions.
 *
 * TABS: NONE
 *
 * $Id: $
 *
 * REVISION HISTORY:
 *
 * $Log: $
 *
 * (C) Copyright 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef __GGREF_H__
#define __GGREF_H__

/* These declarations follow Micro coding standards */
#define GG_DATA_DESC_ANCHOR_BLOCK_MAGIC_VALUE 0xfee1deadUL
#define GG_DATA_DESC_FORMAT_VER               0x00000100UL
#define GG_DATA_DESC_ARCH_TI_C55_STD          0x00000002UL

/* Some modules #define this */
#ifdef UINT32
#undef UINT32
#endif
typedef unsigned long UINT32;

typedef struct
{
    UINT32  magic;                  /* Magic ID: verifies data is valid
                                    ** and endian can be infered */
    UINT32  ver;                    /* version of this arch */
    UINT32  arch;                   /* cpu, compiler, and options */
    const void*   p_self;           /* pointer to this struct 
                                    ** in an offline copy of this data, it
                                    ** establishes the base address of this data
                                    ** (anchor block must be first item in offline
                                    ** copy of data */
    const void*   p_data_types;     /* pointer to the data types table */
    const void*   p_top_level_data; /* pointer to the data definition of the
                                    ** "global" structure */
} GG_DATA_DESC_ANCHOR_BLOCK_T;

typedef struct
{
    const char* name;
    const char* desc;
} GG_DATA_DESC_TYPE_TABLE_ENTRY_T;

/* in datadesc.c */

typedef struct
{
    UINT16*         p_log2phy_table;     /* pointer to logical to physical channel number table
                                         ** this must be on the DSP because it is dynamic */
    const siuInst_t * const *p_siu_inst_table;   /* pointer to table of siu_instance pointers 
                                               this may be in dsp or offline
                                               currently it may be offline as this instances are
                                               fixed locations in all builds 
                                               (but differ from build to build */

} GG_SYS_TOP_LEVEL_T;


#endif /* __GGREF_H__ */

/* Nothing past this point */
