#ifndef _GGHWCFG_H
#define _GGHWCFG_H
/******************************************************************************
 * FILE PURPOSE: HAL build time configuration file
 ******************************************************************************
 * FILE NAME: gghwcfg.h
 *
 * DESCRIPTION: Build time configuration for HAL which supports up to 2
 *              serial ports.
 *
 * TABS: None
 *
 * Copywrite (C) 2002, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

//#include "cslcpu.h"

#define HAL_C64X 64
#define HAL_C64X_EMCBSP 128

/* CPU CORE TYPE */
#define HAL_CONFIG_CPU_CORE             HAL_C64X

/* Device type. Used in ggcfg.c to setup secondary core boot */
#define GG_C6608

/* Peripheral type */
#define HAL_CONFIG_MCBSP_TYPE           HAL_TSIP

/* Timer Control Operation */
#define HAL_INCLUDE_CONFIG_TIMER_TICK_RATE      1
#define HAL_N_TIMERS                            3

/* Include DMA Support for Memory Move */
//#define GG_HAL_INCLUDE_DMAMEMMV

//#define HAL_QT_APP_BUILD  1
/***************************************************************************
 * DEFINITION: This following parameters are build specific used by ggConfig
 ***************************************************************************/

/* to include utopia code in the build */
//#define GG_HAL_INCLUDE_UTOPIA

/* Include timer support */
//#define GG_HAL_INCLUDE_TIMER

/* Include hal direct message support */
#define GG_HAL_INCLUDE_MSG

/* Include HAL HPI support */
//#define GG_HAL_INCLUDE_HPI

/* Enable Memory Protection Events */
//#define HAL_MEMORY_PROTECTION_ENABLE 1

/* Include HAL IPC support */
//#define GG_HAL_INCLUDE_IPC

/* Include HAL gmac support */
//#define GG_HAL_INCLUDE_GMAC

/* Do NOT include PLL support. The pll is setup by the boot loader */
/* #define GG_HAL_INCLUDE_PLL_CTL */

/* Include HAL tdm support. All TDM ports share top level resources */
/* When TSIP LLD is used the TDM #defines below should not be defined*/
//#define GG_HAL_INCLUDE_TDM
//#define GG_HAL_SHARED_TDM_RESOURCES
#define GG_TSIP_N_PORTS     2

/* Include cycle efficient DMA based memmove */
/* Note QDMA is not included at Tomahawk and Shannon */
//#define GG_HAL_INCLUDE_QDMA
                   /*****************************************
                    *      Peripheral Support Config        *
                    *****************************************/
/****************************************************************************
 * DEFINITION: Parameters for build time configuration of the TSIP ports
 ****************************************************************************/
/* Define which bits in a 16 bit timeslot field specify the port number, the
 * link number, and the timeslot number. For this build the mapping is:
 *
 *     15    14   13   12  11          9    8                   0
 *   /-------------------------------------------------------------\
 *   | unused  |  port   |     link      |     timeslot            |
 *   \-------------------------------------------------------------/
 *
 */
#define GG_TSIP_PORT_MSB    13
#define GG_TSIP_PORT_LSB    12

#define GG_TSIP_LINK_MSB    11
#define GG_TSIP_LINK_LSB     9

#define GG_TSIP_TS_MSB       8
#define GG_TSIP_TS_LSB       0


/* The number of bytes allocated for each timeslot in the dma buffer.
 * Set to 1 unless hardware companding is used */
#define GG_TSIP_ELEMENT_BUF_SIZE    1

/* The clock source register values. 0 = source A, 1 = source B */
#define GG_TSIP_TCSR_XMTSRC   0
#define GG_TSIP_RCSR_RCVSRC   0

/* The Clock rate mode. Single rate = 1, double rate = 0 */
#define GG_TSIP_TCR_XMTCLKM   1
#define GG_TSIP_RCR_RCVCLKM   1

/* Endian configuration  little endian = 0, big endian = 1 */
#define GG_TSIP_TDMUGCFGR_BEND  0

/* DMA transfer priorities. Lower values mean higher priorities */
#define GG_TSIP_DMATCUGCR_MAXPRI    0
#define GG_TSIP_DMATCUGCR_PRI       2

/* Transmit timeslot delay values */
/* 0 = interrupt on transfer ack, 1 = transfer on delay, 2 = interrupt on either
 *   ack or delay, 3 = interrupt on both ack and delay */
#define GG_TSIP_TTDC_XMTSFINT   0
#define GG_TSIP_TTDC_XMTFRINT   0
#define GG_TSIP_TTDC_XMTFDLY    0

/* Receive timeslot delay values - same values as transmit */
#define GG_TSIP_RTDC_RCVSFINT   0
#define GG_TSIP_RTDC_RCVFRINT   0
#define GG_TSIP_RTDC_RCVFDLY    0

/* Redundant clock setting - 0 for redundant, 1 for independent */
#define GG_TSIP_SIUGCR_CLKD     0

/* Default frame setting for ports that haven't been enabled yet
 * This needs to be changed to a software config parameter */
#define GG_TSIP_TEMP_UNOPENED_PORT_FRAME_ALLOC  16  /* 16 frames per dma buffer */

/* Two ISRs are required for each TDM */
/* changed based on new shared TSIP driver */
#define HAL_CONFIG_TDM_N_ISRS      (HAL_CONFIG_TDM_PORTS)

#define HAL_CONFIG_TDM_BUFFERING       2  /* Double buffering for TDM */

/* Define the Y2X delay measured for the processor */
#define HAL_Y2X_DELAY            0 /* Estimated to be 1 samples for c64x.Calculation was
                                      done by comparing hybrid positions with standard hybrid */
/***************************************************************************
 * DEFINITION: This following parameters configures the UTOPIA port
 ***************************************************************************/
#define GG_UTOPIA_HEAP_BUF_ALIGN       3   /* 32 bit heap buffer alignment */

/* Threshold of Cells the Utopia PDMA will wait for before Sending Cells */
#define GG_UTOPIA_PORT_XMT_THRESH 		1

/***************************************************************************
 *  parameters are for Processor Endian-ness selection
 ***************************************************************************/
#define HAL_UTOPIA_LITTLE_ENDIAN         0x00
#define HAL_UTOPIA_BIG_ENDIAN            0x01
#define HAL_UTOPIA_ENDIAN			HAL_UTOPIA_LITTLE_ENDIAN

/***************************************************************************
 * DEFINITION: This following parameters are for the USER DEFINED ROUTING
 ***************************************************************************/
#define HAL_UTOPIA_ENABLE_UDR			1
#define HAL_UTOPIA_VCI_UDR				54
#define HAL_UTOPIA_RTAG_UDR				58

/* For Routing based on VCI of the ATM Cell.  */
/* This will be used only in the Phase 3 HDP! */
#define HAL_UTOPIA_VCI_RBYT			0x0004
#define HAL_UTOPIA_VCI_RBIT			0x0004

#define HAL_UTOPIA_VCI_MASK         0x01C0
#define HAL_UTOPIA_VCI_MATCH        0x0000
#define HAL_UTOPIA_VCI_MATCH_SHFT   0x0006

/* For Routing based on Route TAG of the Cell */
/* This will be used only in the Phase 2 HDP! */
#define HAL_UTOPIA_RTAG_RBYT 		0x0004
#define HAL_UTOPIA_RTAG_RBIT 		0x0000

#define HAL_UTOPIA_RTAG_MASK        0x01C0
#define HAL_UTOPIA_RTAG_MATCH       0x0000
#define HAL_UTOPIA_RTAG_MATCH_SHFT  0x0006
/***********************************************/
/* This is a hook to be able to do VCI Routing *
 * in the 58 byte mode. This is there only for *
 * future expansion and not used right now.    */
/***********************************************/
#define HAL_UTOPIA_RTAG_OR_VCI_ROUTE 0

/* Declare global routing based on VCI or VPI*/
#if HAL_UTOPIA_RTAG_OR_VCI_ROUTE
  #define HAL_UTOPIA_GLOBAL_RBYT 		HAL_UTOPIA_RTAG_RBYT
  #define HAL_UTOPIA_GLOBAL_RBIT 		HAL_UTOPIA_RTAG_RBIT

  #define HAL_UTOPIA_GLOBAL_MASK        HAL_UTOPIA_RTAG_MASK
  #define HAL_UTOPIA_GLOBAL_MATCH       HAL_UTOPIA_RTAG_MATCH
#else
  #define HAL_UTOPIA_GLOBAL_RBYT 		HAL_UTOPIA_VCI_RBYT
  #define HAL_UTOPIA_GLOBAL_RBIT 		HAL_UTOPIA_VCI_RBIT

  #define HAL_UTOPIA_GLOBAL_MASK        HAL_UTOPIA_VCI_MASK
  #define HAL_UTOPIA_GLOBAL_MATCH       HAL_UTOPIA_VCI_MATCH
#endif
/*******************************************************************************
 * MAC configuration
 *******************************************************************************/
// #define GG_GMAC_LOOPBACK
// #define GG_GMAC_PROMISCOUS_MODE   /* all packets would go to core 0 */
#define GG_GMAC_HEAP_BUF_ALIGN   3   /* 32 bit alignments */
#define GG_GMAC_UNICAST_INDEX        /* Arrayed mac indexing used */
#define GG_GMAC_EMIC_WRAPPER         /* EMIC wrapper (Ethernet Multicore Interrupt Controller */
#define GG_GMAC_PULSE_INTERRUPTS     /* This part uses pulse interrupts */
#define GG_GMAC_SOFT_RESET           /* Soft reset enabled */
#define GG_GMAC_MAX_ISR_COUNT    0  /* Number of interrupts before processing forced */

/*******************************************************************************
 * MAC-EMIC Configuration.
 *******************************************************************************/
//Disable the transmit and receive pacing units
//#define GG_GMAC_EMIC_RPCFG_VAL     0x0000000f
//#define GG_GMAC_EMIC_TPCFG_VAL     0x0000000f
//#define GG_GMAC_EMIC_PRESCALE_VAL  0x00000000
//Enable the transmit and receive pacing units
#define GG_GMAC_EMIC_RPCFG_VAL     0 /* interrupt is generated for every rx packet */
#define GG_GMAC_EMIC_TPCFG_VAL     (7 | ( 0 << 8)) /* disabled */
#define GG_GMAC_EMIC_PRESCALE_VAL  0x00000000

/* KAILASH? */
//#define GG_GMAC_EMIC_RPCFG_VAL     (0x7 | (12 << 8))
//#define GG_GMAC_EMIC_TPCFG_VAL     (0x7 | (12 << 8))


/* MCBSP input from TDM */
#define HAL_CONFIG_TDM_A_LAW     0
#define HAL_CONFIG_TDM_MU_LAW    1
#define HAL_CONFIG_TDM_LINEAR    2
#define HAL_CONFIG_TDM_MCBSP_INPUT  HAL_CONFIG_TDM_MU_LAW
#define HAL_CONFIG_TDM_MCBSP_OUTPUT HAL_CONFIG_TDM_MU_LAW

#if 0
#define HAL_CONFIG_MAX_EDMA_CHANNELS   5

#define HAL_EDMA_CHANNEL_0             22
#define HAL_EDMA_CHANNEL_1             23
#define HAL_EDMA_CHANNEL_2             24
#define HAL_EDMA_CHANNEL_3             25
#define HAL_EDMA_CHANNEL_4             26
#define HAL_EDMA_CHANNEL_5             27

/* The element size is a property of whether external memory has a 64-bit/16-bit
   interface and hence varies from build to build depending on where the
   external code */
#define HAL_CONFIG_MEMMV_ELEMENT_SIZE_BITS   32
#define HAL_CONFIG_MEMMV_ELEMENT_SIZE_BYTES  (HAL_CONFIG_MEMMV_ELEMENT_SIZE_BITS/8)


/*****************************************************************************
 * EDMA channels for cycle efficient memory move: Paging from DDR2<->L1
 *****************************************************************************/
/* increasing this will require changes in halQdmaInit() function */
#define HAL_CFG_MAX_QDMA_CHS_PER_CORE   4

#define HAL_QDMA_CORE0_CHANNEL_0       38
#define HAL_QDMA_CORE1_CHANNEL_0       42
#define HAL_QDMA_CORE2_CHANNEL_0       46
#define HAL_QDMA_CORE3_CHANNEL_0       50
#define HAL_QDMA_CORE4_CHANNEL_0       54
#define HAL_QDMA_CORE5_CHANNEL_0       58

#endif

/*****************************************************************************
 * Cache build time configuration
 *****************************************************************************/
#define HAL_L1P_CACHE_0K               0x0
#define HAL_L1P_CACHE_4K               0x1
#define HAL_L1P_CACHE_8K               0x2
#define HAL_L1P_CACHE_16K              0x3
#define HAL_L1P_CACHE_32K              0x4

#define HAL_L1P_CACHE_MODE             HAL_L1P_CACHE_32K

#define HAL_L1D_CACHE_0K               0x0
#define HAL_L1D_CACHE_4K               0x1
#define HAL_L1D_CACHE_8K               0x2
#define HAL_L1D_CACHE_16K              0x3
#define HAL_L1D_CACHE_32K              0x4

#define HAL_L1D_CACHE_MODE             HAL_L1D_CACHE_32K

#define HAL_L2_CACHE_0K                0x0
#define HAL_L2_CACHE_32K               0x1
#define HAL_L2_CACHE_64K               0x2
#define HAL_L2_CACHE_128K              0x3
#define HAL_L2_CACHE_256K              0x4
#define HAL_L2_CACHE_512K              0x5
#define HAL_L2_CACHE_1024K             0x6   /* Not supported at Shannon */
#define HAL_L2_CACHE_ALL               0x7   /* The same as 512K at Shannon */

#define HAL_L2_CACHE_MODE              HAL_L2_CACHE_64K

/* External memory cache configuration */
/* There are  256 MAR registers. Instead of specifying them all, a -1 terminated list of
 * mar registers and a corresponding list of values is specified. Any values not
 * specified are left unchanged.
 *
 *  A value of 0xD indicates cacheable and pre-fetch, 0x4 non-cachable and no-pre-fetch
 *
 *  MAR: 0x80000000 - 0x8fffffff
 */
/* HAL_MAR_REGS and HAL_MAR_VALS: for Video Application are defined in ggmemdef.bec. These can be
   different based on active build among gg_supported_builds */

/* HAL_MAR_DYNAMIC_CFG is used in hal\csl\cache\c64x\tomahawk\icache.c */
#define HAL_MAR_DYNAMIC_CFG

/******************************************************************************
 *                     External Memory Setup                                  *
 ******************************************************************************
 * The structures used to configure the XMC, SES, and SMS MPAX units is defined
 * in hal/csl/mpax/c64x/shannon/mpaxutil.h.
 *
 * The memory's physical addresses are 36 bits where the DDR is at
 * 0x8 0000 0000.
 *
 * There are 3 relevant MPAX units:
 * XMC (Located in the XMC which is part of each corepac)
 *   - this maps 32-bit addresses generated by the CPU into 36 bit physical
 *     addresses used by the memory.
 * SES, SMS: (Located in the MSMC).
 *   - this maps 32-bit addresses generated by the other masters (peripherals,
 *     DMAs) into 36 bit physical memory addreses.
 *
 * On a Shannon device, there are a total of 10 MPAX units.  Each corepac has
 * one in its XMC, and the MSMC has one SMS and one SES.  All 3 must be
 * considered when interfacing with peripherals or other cores.
 *
 * The details of the XMC, SES, and SMS mpax units are in the device
 * documentation.
 *
 * IMPORTANT: the segments "execute" in reverse order (row 15 to row 0).
 * The first match wins.  The tables in HAL_MPAX_*_TBL execute in
 * forward order.  Consider both orders to make sure that the program
 * setting the MPAX table doesn't block its own access.  Low numbers
 * like 0/1 are "catch all" entries.
 *
 * The strategy below is to avoid the reset defaults and explicitly
 * program all the registers
 */


/* ======================= HAL_MPAX_XMC_TBL ===================================
 * The value of HAL_MPAX_XMC_TBL below creates the following configuration of
 * the memory for each CPU.
 *
 * Index 0: Force back to reset value to see low 2GB of address space
 * {0, 0, 0, 0x00000000, 0x0, 0x00000000, 31, 0x77}
 *
 * Index 1: Catch all for DDR.
 * Initial value to enable all of DDR
 * {1,  0, 0, 0x80000000,                0x8, 0x00000000, 31, 0x77}
 * Final value to block all DDR except those areas already configured
 * {1,  0, 0, 0x80000000,                0x0, 0x00000000, 31, 0x00}
 *
 * Index 2: Access to DDR MMRs
 * {2, 0, 0, CSL_DDR3_EMIF_CONFIG_REGS, 0x1, 0x00000000, 12, 0x77}
 *
 * Index 3:
 * {2, 0, 0, 0x80000000, 0x8, 0x00000000, 25, 0x77}
 * The logical addresses created are
 *   0x8000 0000 to
 *   0x81FF FFFF
 * shared by all cores (32MB = 2^25)
 *
 * The physical addresses consumed are:
 *   0x8 0000 0000 to
 *   0x8 01FF FFFF        (32MB total)
 *
 * Index 4:
 * {3, 1, 0, 0x90000000, 0x8, 0x02000000, 25, 0x77}
 * The logical addresses created are
 *   0x9000 0000 to
 *   0x91FF FFFF
 * private to each core (32MB = 2^25)
 *
 * The physical addresses consumed are:
 *   0x8 0200 0000 to
 *   0x8 11FF FFFF        (32MB * 8 = 256MB total)
 *
 * Index 5:
 * {4, 0, 0, 0x92000000, 0x8, 0x12000000, 25, 0x77}
 * The logical addresses created are
 *   0x9200 0000 to
 *   0x93FF FFFF
 * on shared by all cores
 *
 * The physical addresses consumed are:
 *   0x8 1200 0000 to
 *   0x8 13FF FFFF        (32MB total)
 *
 * The combined physical memory footprint is
 * [0x8 0000 0000 - 0x8 13FF FFFF] = 320MB.
 *
 * The cachebilty of the memory mapped by the MPAX is independently controlled
 * by the MAR registers which are defined in HAL_MAR_REGS and HAL_MAR_VALS.
 * Each MAR covers 16MB.
 *
 * ======================= HAL_MPAX_XMC_TBL ==================================*/
#define HAL_MPAX_XMC_TBL                                                    \
    {                                                                       \
      { 0,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 31, 0x77}, \
      { 1,  0, 0, 0, 0x80000000,                0x8, 0x00000000, 31, 0x77}, \
      { 2,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 3,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 4,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 5,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 6,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 7,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 8,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 9,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {10,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {11,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {12,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {13,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {14,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {15,  0, 0, 0, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      { 2,  0, 0, 0, CSL_DDR3_EMIF_CONFIG_REGS, 0x1, 0x00000000, 12, 0x77}, \
      { 3,  1, 0, 0, 0xF0000000,                0x8, 0x1FE00000, 18, 0x77}  \
    }

/* ======================= HAL_MPAX_SES_TBL ===================================
 * The value of HAL_MPAX_SES_TBL below creates the following configuration of
 * the memory for each CPU.
 *
 * Index 0: Default direct map to all DDR (allowed)
 * {0, 16, 0, 0x80000000,                0x8, 0x00000000, 31, 0x77}, \
 *
 * ======================= HAL_MPAX_SES_TBL ==================================*/
#define HAL_MPAX_SES_TBL                                                   \
    {                                                                      \
      {0, 0, 0, 15, 0x80000000,                0x8, 0x00000000, 31, 0x77}, \
      {1, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {2, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {3, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {4, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {5, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {6, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {7, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}  \
    }

/* ======================= HAL_MPAX_SMS_TBL ===================================
 * The value of HAL_MPAX_SMS_TBL below creates the following configuration of
 * the memory for each CPU.
 *
 * Index 0: Default direct map to all MSMC RAM (allowed)
 * {0, 16, 0, 0x0C000000,                0x0, 0x0C000000, 24, 0x77}
 * ======================= HAL_MPAX_SMS_TBL ==================================*/

#define HAL_MPAX_SMS_TBL                                                   \
    {                                                                      \
      {0, 0, 0, 15, 0x0C000000,                0x0, 0x0C000000, 24, 0x77}, \
      {1, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {2, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {3, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {4, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {5, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {6, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}, \
      {7, 0, 0, 15, 0x00000000,                0x0, 0x00000000, 0,  0x00}  \
    }

/*****************************************************************************
 * EMIF Configuration Values for SRAM
 *****************************************************************************/
#define EMIF_CTLCE_WRITE_SETUP          2
#define EMIF_CTLCE_WRITE_STROBE         2
#define EMIF_CTLCE_WRITE_HOLD           1

#define EMIF_CTLCE_READ_SETUP           2
#define EMIF_CTLCE_READ_STROBE          2
#define EMIF_CTLCE_READ_HOLD            1

/*****************************************************************************
 * EMIF Configuration Values for DDR2 SDRAM
 *****************************************************************************/
// Defines the CAS latency:
#define HAL_EMIF_DDR2_CAS_LATENCY         2  // 2 to 5
// No of banks of conected DDR SDRAM
#define HAL_EMIF_DDR2_I_BANK              3  // 0->1bank, 1->2, 2->4, 3->8                               PS_256,
// Chip select signal used
#define HAL_EMIF_DDR2_E_BANK              0  // 0->CS_1, 1->CS_2
// Page size of the connected DDR SDRAM
#define HAL_EMIF_DDR2_PAGE_SIZE           0  // 0->256, 1->512, 2->1024, 3->2048 words
// Data width of connected DDR SDRAM
#define HAL_EMIF_DDR2_DATA_WIDTH          0  // 0->32-bit, 1->16-bit

#define HAL_EMIF_DDR2_SDRAM_REFRESH_CTR   0x0000081B  // Setting refresh rate for 7.8us at 266 Mhz
#define HAL_EMIF_DDR2_SDRAM_TIMING_REG1   0x26DB5BC9
#define HAL_EMIF_DDR2_SDRAM_TIMING_REG2   0x0016C722

#define HAL_EMIF_DDR2_PHY_RST             0x50006420
#define HAL_EMIF_DDR2_PHY_RLAT            2   // 1 to 7
/*******************************************************************************
 * MAC configuration
 *******************************************************************************/
//#define GG_GMAC_EWCTL                /* EWCTL interrupt control used */

/*******************************************************************************
 * Interrupt Configuration
 *******************************************************************************/

/* Vector 4 unassigned */

/* Vector 5 unassigned */

/* Vector 6 unassigned */

/* Vector 7 unassigned */

/* Vector 8 unassigned */

/* Vector 9 unassigned */

/* Vector 10 unassigned */

/* Vector 11 unassigned */

/* Vector 12 unassigned */

/* Vector 13 unassigned */

/* Vector 14 assigned to edma local */
#define GG_HWCFG_VEC14_ENABLED
//#define CSL_DEVICE_ISR_DMA_XFER   CSL_DEVICE_ISR_TPCC_LOCAL
/* Temporay event mapping */
#define CSL_DEVICE_ISR_DMA_XFER   63
#define GG_HWCFG_VEC14_SOURCE     CSL_DEVICE_ISR_DMA_XFER

#define GG_HWCFG_VEC15_ENABLED
/* Temporay event mapping */
#define CSL_DEVICE_ISR_TIMER_8  66
#define GG_HWCFG_VEC15_SOURCE   CSL_DEVICE_ISR_TIMER_8


/**********************************************************************************
 * EDMA TPCC configuration
 *   For this build the edma is used only for memory to memory transfers
 *   DMA for memory moves are assigned channels 33-38, using queues
 *   1,2,3,1,2,3 respectively. All channels have priority 1. Each core
 *   uses only 1 channel for moves.
 **********************************************************************************/
//#define GG_HAL_INCLUDE_DMA_CONTEXT
//#define GG_DMA_MEM_TO_MEM_NO_INTERRUPTS

#define GG_DMA_INSTANCE_ARRAY       {1, 1, 1, 1, 1, 1, 1, 1}   /* Use EDMA instance 1 for all the cores   */
#define GG_DMA_QUEUE_ARRAY          {0, 0, 0, 0, 0, 0, 0, 0}   /* Use EDMA TC (Queue) 0 for all the cores */

#define GG_DMA_INSTANCE             1   /* Use EDMA instance 1 */

#define GG_DMA_CHANNELS_PER_CORE    4   /* 1 channel for memory moves: TBD: to be enhanced */
#define GG_DMA_CHANNELS_N           (8*GG_DMA_CHANNELS_PER_CORE)   /* 2 elements in the channels and reload sets array */
#define GG_DMA_CHANNELS         { 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51,52,53,54,55,56,57,58,59,60,61,62,63 }  /* Channels and event numbers are identical */
#define GG_DMA_RELOAD_SETS      { 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83,84,85,86,87,88,89,90,91,92,93,94,95 }  /* Parameter reloads on mem to mem transfer */
#define GG_DMA_QUEUTC_MAP       0x00003210  /* One queue to one transfer controller */
#define GG_DMA_QUEPRI           0x7777      /* all queues get priority 7 */
#define GG_DMA_QNUM0            0x00000000  /* Maps channels 7:0 to queues. */
#define GG_DMA_QNUM4            0x32323232  /* Maps channels 39:32 to queues. */
#define GG_DMA_QNUM5            0x32323232  /* Maps channels 40:47 to queues. */
#define GG_DMA_QNUM6            0x32323232  /* Maps channels 48:55 to queues. */
#define GG_DMA_QNUM7            0x32323232  /* Maps channels 56:63 to queues. */

/* There is only 1 edma interrupt source for this build per core. Core 0 uses
 * mar0, core 1 mar1, etc. */
#define GG_DMA_NUM_MAR_REGS     8
#define GG_DMA_NUM_ELEMS_MAR0       GG_DMA_CHANNELS_PER_CORE
#define GG_DMA_MAR_SOURCE_0         { 32, 33, 34, 35 }
#define GG_DMA_MAR_SOURCE_1         { 36, 37  38, 39 }
#define GG_DMA_MAR_SOURCE_2         { 40, 41, 42, 43 }
#define GG_DMA_MAR_SOURCE_3         { 44, 45, 46, 47 }
#define GG_DMA_MAR_SOURCE_4         { 48, 49, 50, 51 }
#define GG_DMA_MAR_SOURCE_5         { 52, 53, 54, 55 }
#define GG_DMA_MAR_SOURCE_6         { 56, 57, 58, 59 }
#define GG_DMA_MAR_SOURCE_7         { 60, 61, 62, 63 }

#define GG_DMA_SOURCE_ARRAY         { GG_DMA_MAR_SOURCE_0, \
                                      GG_DMA_MAR_SOURCE_1, \
                                      GG_DMA_MAR_SOURCE_2, \
                                      GG_DMA_MAR_SOURCE_3, \
                                      GG_DMA_MAR_SOURCE_4, \
                                      GG_DMA_MAR_SOURCE_5, \
                                      GG_DMA_MAR_SOURCE_6, \
                                      GG_DMA_MAR_SOURCE_7  \
                                     }

#define GG_DMA_NSOURCE_ARRAY        { GG_DMA_NUM_ELEMS_MAR0, GG_DMA_NUM_ELEMS_MAR0,   \
                                      GG_DMA_NUM_ELEMS_MAR0, GG_DMA_NUM_ELEMS_MAR0,   \
                                      GG_DMA_NUM_ELEMS_MAR0, GG_DMA_NUM_ELEMS_MAR0,   \
                                      GG_DMA_NUM_ELEMS_MAR0, GG_DMA_NUM_ELEMS_MAR0  }

/* INTC and Interrupt Mapping */
#define GG_DMA_INCLUDE_INTC
#define GG_INTC_INSTANCE_0        0
#define GG_INTC_INSTANCE_1        1

#define GG_DMA_INTC_MAP_NUM_ENTRIES   8          /* There is one system event to host event mapping for each DSP core */
#define GG_DMA_NUM_CORES_PER_INTC     4


#define GG_DMA_NTC_INSTANCE_ARRAY      { GG_INTC_INSTANCE_0, GG_INTC_INSTANCE_0, GG_INTC_INSTANCE_0, GG_INTC_INSTANCE_0,   \
                                         GG_INTC_INSTANCE_1, GG_INTC_INSTANCE_1, GG_INTC_INSTANCE_1, GG_INTC_INSTANCE_1 }

#define GG_DMA_SYS_INTR_ARRAY          { 8, 9, 10, 11, 12, 13, 14, 15 }
#define GG_DMA_HOST_INTR_ARRAY         { 3, 11, 19, 27, 3, 11, 19, 27 }
#if 0 /* The same event is used for all cores */
#define GG_DMA_HOST_EVENT_ARRAY        {CSL_DEVICE_ISR_DMA_XFER, CSL_DEVICE_ISR_DMA_XFER,  \
                                        CSL_DEVICE_ISR_DMA_XFER, CSL_DEVICE_ISR_DMA_XFER,  \
                                        CSL_DEVICE_ISR_DMA_XFER, CSL_DEVICE_ISR_DMA_XFER,  \
                                        CSL_DEVICE_ISR_DMA_XFER, CSL_DEVICE_ISR_DMA_XFER}
#endif

/*****************************************************************************************
 * EDMA TPTC configuration.
 *   The RDRATE field for the two transfer controllers used (2 and 3) must be set to 4 (32 cycles
 *   between reads). The value for tc 0 and tc 1 are also set, although they are not used. To
 *   not set a value (leave the current value in the register) use -1.
 *
 *   There are 4 TPTCs in Tomahawk
 *****************************************************************************************/
#define GG_DMA_TPTC_RDRATE     { 0, 0, 0, 0 }


#endif

