#ifndef _GGSWCFG_H
#define _GGSWCFG_H
/******************************************************************************
 * FILE PURPOSE: Golden Gateway Software Specific Configuration
 ******************************************************************************
 * FILE NAME: ggswcfg.h
 *
 * DESCRIPTION: Template file for defining options available for a specific
 *              build of Golden Gateway DSP software.
 *
 *              Simply comment out those options not required for your specific
 *              build, and change "##" to "#" for those options that are
 *              required in order to tailor this file.
 *
 *
 * (C) Copyright 2003, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"

#ifndef max
#define max(x,y) ((x) > (y) ? (x):(y))
#endif

/******************************************************************/
/* Miscellaneous parameters                                       */
/******************************************************************/
/* Number of channels per DSP core */
/* GG_MAX_NUM_CHANNELS is provided by the makegg via mkrel/makefile */
#define GG_MAX_AUX_PCM_CHANNELS 0
/* Sampling rate in Hz */
#define GG_SAMPLING_RATE                   8000

/* General voice parameters */
#ifdef SIU_TDM_10MS
#define GG_MAX_VOICE_FRAME_LENGTH          80     /* samples in 5.5 ms @ 8KHz */
#define GG_VOICE_FRAME_SIZE                80     /* 5 ms */
#else
#define GG_MAX_VOICE_FRAME_LENGTH          44     /* samples in 5.5 ms @ 8KHz */
#define GG_VOICE_FRAME_SIZE                40     /* 5 ms */
#endif
#define GG_MAX_VOICE_PLAYOUT_FIFO_SIZE     14     /* 148.5 ms approx */

#ifdef SIU_TDM_10MS
#define GG_PIU_RX_DELAY                    40     /* sec? msec? */
#else
#define GG_PIU_RX_DELAY                    20     /* sec? msec? */
#endif

/******************************************************************/
/* Voice codecs                                                   */
/******************************************************************/
//#define GG_INCLUDE_PCM
//#define GG_INCLUDE_G726
//#define GG_INCLUDE_G723
//#define GG_INCLUDE_G729AB
//#define GG_INCLUDE_G722
//#define GG_INCLUDE_G729
//#define GG_INCLUDE_G729E
//#define GG_INCLUDE_G728
//#define GG_INCLUDE_GSMFR
//#define GG_INCLUDE_GSMEFR
//#define GG_INCLUDE_GSMAMR
//#define GG_INCLUDE_EVRC
//#define GG_INCLUDE_SMV

#define GG_MAX_CODEC_FRAME_LENGTH          80     /* 10 ms */
#define GG_MAX_VCA_INST_SIZE               60
#define GG_STAGGER_RESOLUTION              2      /* 5ms staggering */
#ifdef SIU_TDM_10MS
#define GG_MAX_CWORD_SEG_SIZE              640    /* 64*10 bits in one frame */
#define GG_MAX_CWORD_SEG_SIZE_RTP          640
#else
#define GG_MAX_CWORD_SEG_SIZE              352    /* 64*10 bits in one frame */
#define GG_MAX_CWORD_SEG_SIZE_RTP          320
#endif
#define GG_MAX_SEGMENTS_PER_PACKET         3
#define GG_MAX_CORE_SEGMENTS_PER_PACKET    3

/* Voice Quality Monitor module VQM */
//#define GG_INCLUDE_VQM

/******************************************************************/
/* Network encap                                                  */
/******************************************************************/

#define GG_INCLUDE_NET_ENCAP
#define GG_INCLUDE_NEU_UDPIP       /* Only Required for RTP */
#define GG_INCLUDE_NEU_ETHERNET    /* Only Required for RTP */

/******************************************************************/
/* Low level network stacks                                       */
/******************************************************************/
//#define GG_INCLUDE_ATM_ENCAP

/******************************************************************/
/* Protocol modules                                               */
/******************************************************************/
//#define GG_INCLUDE_RTCP

/******************************************************************/
/* Network protocols                                              */
/******************************************************************/
#define GG_INCLUDE_RTP
//#define GG_INCLUDE_AAL2
//#define GG_INCLUDE_AAL1
//#define GG_INCLUDE_IUUP
/******************************************************************/
/* PAYLOAD FORMATS SUPPORTED */
/******************************************************************/
/* This needs updation whenever a new payload format is supported or whenever */
/* an existing support is removed !! Presently   two formats are supported -  */
/* linear format and rfc3267 format                                           */
#define GG_NUM_PAYLDFRMTS_SUPPORTED 0

//#define GG_INCLUDE_LINEAR_PAYLDFRMT  /*Default payload format for rtp,aal2,aal1*/
//#define GG_INCLUDE_RFC3267_PAYLDFRMT /* Payload format defined in rfc3267 */
//#define GG_INCLUDE_RFC2658_PAYLDFRMT /* Payload format defined in rfc2658 */



/******************************************************************/
/* FAX functionality - fax relay and fax termination applications */
/******************************************************************/
/* Fax relay application */
//#define GG_INCLUDE_FAX_RELAY

/* Fax modem application */
//#define GG_INCLUDE_FAX_MODEM

/* T.38 network protocol */
//#define GG_INCLUDE_FAX_RELAY_T38

/* AAL2 network protocol */
//#define GG_INCLUDE_FAX_RELAY_AAL2

/* S3/T2 Telogy network protocol */
//#define GG_INCLUDE_FAX_RELAY_S3

/* FRF.11 network protocol     */
//#define GG_INCLUDE_FAX_RELAY_FRF11

/* Turn on/off GMP external buffer support for FIU */
//#define GG_EMPLOY_GMP_FOR_FAX

/* Number of internal buffers allocated per channel for FIU. */
/* Legacy fax allocated itself 100 buffers internally.  Now */
/* that GMP is available, this number can be changed in order */
/* to use GMP's memory allocation functionality.  If GMP is */
/* NOT supported in this build (see #define GG_EMPLOY_GMP_FOR_FAX) */
/* then the number of internally allocated FIU buffers must be */
/* 100 in order to revert to 'hardened' legacy fax behavior. */
/* Any number less than this may affect fax relay performance. */
#define GG_NUM_INTERNAL_FIU_BUFS         53  /* NOTE when employing GMP,
                                                make sure GMP pool size
                                                is sufficient to account
                                                for the increased usage
                                                that may be required by
                                                FIU */


/* Other GG features available */
/*#define GG_INCLUDE_TTU*/
//#define GG_INCLUDE_CID


/* Number of samples in 10 ms @ 8KHz sampling rate */
#define GG_MAX_FAX_FRAME_LENGTH  80

/* 7.5 ms delay */
#define GG_FAX_PIU_RX_DELAY      60
#define GG_NV_ECO_BUFF_NUM       4  /* MSG + BULK + HLT + TRACE */
#define GG_NUM_FIU_OVERLAY_BUFS  0
/******************************************************************/
/* Caller ID (CID) functionality                                  */
/******************************************************************/
#define GG_CID_MAX_SUBSEGMENT   40   /* Fragment size, in bytes (has to be multiple of 4) */
/******************************************************************/
/* Caller ID (CID) functionality                                  */
/******************************************************************/
#define GG_CID_MAX_SUBSEGMENT   40   /* Fragment size, in bytes (has to be multiple of 4) */
//#define GG_INCLUDE_CID_US_GENERATE   /* Include CID generator, Bellcore compliant */
//#define GG_INCLUDE_CID_US_DETECT     /* Include CID detector, Bellcore compliant */
//#define GG_INCLUDE_CID_NTT_GENERATE  /* Include CID generator, NTT compliant */
//#define GG_INCLUDE_CID_NTT_DETECT    /* Include CID detector, NTT compliant */
//#define GG_INCLUDE_CID_ETSI_GENERATE /* Include CID detector, ETSI compliant */
//#define GG_INCLUDE_CID_ETSI_DETECT   /* Include CID detector, ETSI compliant */
//#define GG_INCLUDE_CID_YDN_GENERATE  /* Include CID detector, YDN compliant */
//#define GG_INCLUDE_CID_YDN_DETECT    /* Include CID detector, YDN compliant */
//#define GG_INCLUDE_CID_BT_GENERATE   /* Include CID detector, BT compliant */
//#define GG_INCLUDE_CID_BT_DETECT     /* Include CID detector, BT compliant */


/******************************************************************/
/* Tone detection (TSU) functionality                             */
/******************************************************************/
//#define GG_INCLUDE_MF_DTMF        /* DTMF detection */
//#define GG_INCLUDE_MF_R1          /* MF R1 support */
//#define GG_INCLUDE_MF_R2FW        /* MF R2 forward signaling support */
//#define GG_INCLUDE_MF_R2BW        /* MF R2 backward signaling support */
//#define GG_INCLUDE_V21            /* V.21 demodulation and flags detection support */
//#define GG_INCLUDE_V25            /* V.25 w/ or w/o phase reversals detection support */
//#define GG_INCLUDE_V25_RX         /* same, but on packet side */
//#define GG_INCLUDE_CPT            /* Call Progress Tone detection support */
//#define GG_INCLUDE_V18A           /* V.18A, 1400 Hz detection */
//#define GG_INCLUDE_SS7COT         /* SS7 Continuity Check tone, 2010 Hz */
//#define GG_INCLUDE_SF             /* Signaling Frequency at 2600 Hz */
//#define GG_INCLUDE_CNG            /* CNG, FAX tone, 1100 Hz */
//#define GG_INCLUDE_SS7COR         /* SS7 Continuity Check Return tone, 1780 Hz */
//#define GG_INCLUDE_TT1KHZ         /* Test tone, 1 kHz */
//#define GG_INCLUDE_CIDCAS         /* Call Waiting Alert Signal, used for CID Type II */
//#define GG_INCLUDE_BELL           /* 2225 and 2250 Hz bi-dir detectors */
//#define GG_INCLUDE_CALLDISC       /* Set of call discrimination features */
//#define GG_INCLUDE_V21_CM         /* V.21 fax CM detection */
//#define GG_INCLUDE_DIGIT_FILTER      /* Digit reporting events can suppressed */


/* Following option makes trade-off between some memory per channel and MIPS
 * resources (should go to ggfeature2eco.h). Also there is a stack size impact.
 * If you want to use this option, check if ##define TDU_DATA_MEMORY_SAVE
 * in file ..\dsp\tdu\tdustft.c is enabled.
 * The trade offer is 108 words per channel for 0.55 MIPS (also per channel).
 */
//#define GG_INCLUDE_TDU_SAVE_MEMORY_SPEND_MIPS

/* echo canceller */
//#define GG_INCLUDE_AER
//#define GG_INCLUDE_AEC
//#define GG_INCLUDE_AGC

/******************************************************************/
/* Conferencing (CONF) functionality                              */
/******************************************************************/
#define GG_INCLUDE_CONFERENCING
/******************************************************************/
/* Dual Tone Multi Frequency (DTMF) Relay functionality           */
/******************************************************************/
//#define GG_INCLUDE_DTMF_RELAY

/******************************************************************/
/* Clear Channel functionality                                    */
/******************************************************************/
//#define GG_INCLUDE_CLEAR_DATA

/******************************************************************/
/* Dynamic Codec Switchover                                       */
/******************************************************************/
//#define GG_INCLUDE_CSU

/******************************************************************/
/* Dual Tone Multi Frequency (DTMF) upspeed functionality         */
/******************************************************************/
//#define GG_INCLUDE_DIGIT_UPSPEED

#define GG_RTTRACE_BUFSIZE       127
#define GG_RTTRACE_INTERVAL       60     /* 30ms maximum delay */
#define GG_RTTRACE_PKT_FREE       14     /* Free Cells to allow trace */

/******************************************************************/
/* Security module (Encryption/Decryption)                        */
/******************************************************************/
/* MSU CONFIGURATION */
//#define GG_INCLUDE_MSU
//#define GG_INCLUDE_SECURITY_SRTP
//#define GG_INCLUDE_SECURITY_SRTCP
//#define GG_INCLUDE_SECURITY_PKTCBL_RTP
//#define GG_INCLUDE_SECURITY_PKTCBL_RTCP
//#define GG_INCLUDE_MSU_AES_CTR
//#define GG_INCLUDE_MSU_AES_f8
//#define GG_INCLUDE_MSU_AES_CBC_RBT
//#define GG_INCLUDE_MSU_MAC_HMAC_SHA1
//#define GG_HMAC_SHA1_TAG_MAX_SIZE (12) /* In Bytes */
//#define GG_INCLUDE_MSU_MAC_MMH
//#define GG_MMH_TAG_MAX_SIZE (4) /* In Bytes */
//#define GG_INCLUDE_MSU_MAC_HMAC_MD5
//#define GG_HMAC_MD5_TAG_MAX_SIZE (12) /* In Bytes */

//#define GG_NUM_MSU_CHANNELS (24)   /* this definition is required only if
//                                      GG_INCLUDE_MSU is defined */
/* TODO */
//#define GG_MSU_SRTP_BUFFER_SIZE       (0xB0)    /* 16bit words */
//#define GG_MSU_PKTCBL_RTP_BUFFER_SIZE (0x180) /* 16bit words */
//#define GG_MSU_MAX_BUFFER_SIZE        GG_MSU_SRTP_BUFFER_SIZE
//#define GG_MSU_MAX_BUFFER_SIZE  GG_MSU_PKTCBL_RTP_BUFFER_SIZE

/******************************************************************/
/* select McBSP2 */
/******************************************************************/
//##define GG_ASSIGNABLE_SERIAL_PORTS

/******************************************************************/
/* Echo Cancellation Unit (ECU) functionality                     */
/******************************************************************/
//#define GG_INCLUDE_ECU
#define GG_MAX_ECHO_TAIL_LENGTH            128    /* 128 ms */
#define GG_MAX_ECHO_FILTER_SEG_LENGTH      32     /* 32  ms */
#define GG_MAX_ECHO_FILTER_SEGS            3      /* 3 segments/hybrids */

/******************************************************************/
/* ECU MIPS agent initialization parameters                       */
/******************************************************************/
//#define GG_ECUMA_ALLOC_TICK           1  /* last tick before rollover */
//#define GG_ECUMA_RRATE_MS            20  /* Allocate MIPS every 20ms */
//#define GG_ECUMA_CORE_MIPS    0x096000L  /* 600 MIPS core (U22.10) */
//#define GG_ECUMA_GUARD_MIPS   0x001400L  /* Provide 5 MIPS "guard" (U22.10) */
//#define GG_ECUMA_STATIC_MIPS  0x002800L  /* 10.00 static MIPS (U22.10) */
//#define GG_INCLUDE_ECUMA_CTRL            /* Allow MAG debug control */


/******************************************************************/
/* Data modem                                                     */
/******************************************************************/
//#define GG_INCLUDE_DATA_MODEM
#define GG_MAX_MODEM_FRAME_LENGTH           48
#define GG_MAX_MODEM_CTRL_FRAME_LENGTH      80 /* samples in 10 ms @ 8KHz */
#define GG_MODEM_PIU_RX_DELAY               40


/******************************************************************/
/* Acoustic Echo Removal unit (AER)                               */
/******************************************************************/
#define GG_MAX_AER_TAIL_LENGTH             0

/******************************************************************/
/* mips measurement */
/******************************************************************/
//#define GG_INCLUDE_MIPS_CNT /* This is obsolete.Define it in makefile */

/******************************************************************/
/* messages */
/******************************************************************/
//#define GG_INCLUDE_ANNOUNCEMENTS
/* define message processing rate */
#define GG_MSG_PROCESS_PERIOD              8   /* quananteed message process period in 0.5ms ticks */

/******************************************************************/
/* memory management */
/******************************************************************/
//#define GG_CODEC_TABLE_RELOCATION
//#define GG_CODEC_FAX_TABLES        4
 /* Enabling/disabling of codec buffer swapping */
//#define GG_G729AB_BUFF_SWAP_ENABLE
//#define GG_G723_BUFF_SWAP_ENABLE
 /* dynamic buffer allocation parameters */
//#define GG_INCLUDE_DYN_BUF_ALLOCATION
// #define GG_MAX_NUM_BITMAP_WORDS         ((GG_MAX_NUM_CHANNELS+15)/16)

//#define GG_INCLUDE_DYN_CH_M_DIV_N_CONFIG
//#define GG_DYN_CH_MAX_LBRC_CHANNELS        2   /* N value here */
//#define GG_INCLUDE_BLOCK_ASYMETRIC

 /* heap pages */
//#define GG_DISGR_HEAP_PAGE    0
//#define GG_AIC_HEAP_PAGE      0
//#define GG_CONF_HEAP_PAGE     0
 /* dynamic buffer allocation */
//#define GG_INCLUDE_CHANNEL_ONCOMMONPAGE /*  */
//#define GG_MAX_NUM_DATA_PAGE              2
//#define GG_MAX_NUM_CHANNELS_PAGE1         5
//#define GG_MAX_NUM_CHANNELS_PAGE2         5
//#define GG_SWAP_ETOI
//#define GG_SWAP_ETOI_PROG
//#define GG_SWAP_ETOI_DATA
/******************************************************************/
/* PCM Interface Unit (PIU)                                       */
/******************************************************************/
#define GG_ISR_RATE_MICRO_SECS     1000   /* 0.5 ms/1ms ISR */
#define GG_BSP_ALLOCATED_SIZE      0x640
#define GG_BSP_SEGMENT_LENGTH        8
#define GG_SOFTWARE_SEGMENT_LENGTH   8
#define GG_INTR_PCM_SAMPLES          8
#define GG_TDM_SUB_FRAME_RATES        8, 8, 8
#define GG_TDM_WORD_SIZES             8, 8, 8
#define GG_TDM_BUF_INC                0, 0, 0
#define GG_TDM_MAX_PHASES             10,10,10
/* For two channels max phase * nchannels, not sure!*/
#define GG_TDM_MAX_MEGA_PHASES        10,10,10
#define GG_UTOPIA_PHYS_PORTS          0
#define GG_UTOPIA_ROUTE_MODE          HAL_UTOPIA_USER_ROUTE

#define GG_SHARED_MEM_DATA_ALLOCATION

/******************************************************************/
/* Buffered Serial Port (BSP) parameters                          */
/******************************************************************/
#define GG_TDM_PORTS_USED    2

#define GG_HAL_HEAP_START_ADDR       ((void*)0x824060)
#define GG_HAL_HEAP_END_ADDR         ((void*)0x825220)

#define GG_BSP_TX_ADDR       ((void*)0x8257C0)
#define GG_BSP_TX_END_ADDR   ((void*)0x825E00)
#define GG_BSP_RX_ADDR       ((void*)0x825E00)
#define GG_BSP_RX_END_ADDR   ((void*)0x826440)
 /* The max phase should be the least common integer multiple of the possible
  * PCM frame sizes. For example, if 5.5 and 5ms frames are used, the least
  * common multiple is 110. This parameter is used to stagger channel frames
  * around the sub-frame rate. */
#define GG_BSP_MAX_PHASE  10

//#define GG_MAX_REPLICATE_CHANNELS   1


/******************************************************************/
/* Host Port Interface (HPI) parameters                           */
/******************************************************************/
/* Size of HPI packet exchange FIFOs. */
#define GG_DSP_MIC_XCHG_LENGTH             896
//#define NDU_NEW_HPI
//#define NDU_UTOPIA
//#define NDU_GMAC
//#define NDU_HD_NDU
//#define NDU_LEGACY_HPI

/* The below lengths are in dprwords, and tells how many messages (dprwords
   can be allocated in a FIFO  */
#define GG_NDU_MSG_NORMAL_FIFO_L          8
#define GG_NDU_MSG_HIGH_FIFO_L            0
#define GG_NDU_MSG_LOW_FIFO_L             0

//#define GG_HD_HPI
//#define GG_HD_HPI_START               0x1000
//#define GG_HD_HPI_RX_START_ADDR       0x1020
//#define GG_HD_HPI_RX_LENGTH           896  /* must be divisible by 32 */
//#define GG_HD_HPI_TX_START_ADDR  (GG_HD_HPI_RX_START_ADDR + GG_HD_HPI_RX_LENGTH)
//#define GG_HD_HPI_TX_LENGTH           GG_HD_HPI_RX_LENGTH
//#define GG_HD_HPI_NORMAL_MSG_FIFO_L   400
//#define GG_HD_HPI_HIGH_MSG_FIFO_L     66
//#define GG_HD_HPI_LOW_MSG_FIFO_L      66
//#define GG_HD_HPI_MAGIC               0x2A
#define GG_UT_NDU
#define GG_UTNDU_MSG_NORMAL_FIFO_L       256
#define GG_UTNDU_MSG_LOW_FIFO_L           64
#define GG_UTNDU_MSG_HIGH_FIFO_L          64
#define GG_UTNDU_AAL5_RX_BUF_L           256
#define GG_UTNDU_AAL5_TX_BUF_L           256
#define GG_UTNDU_AAL5_ANU_TX_BUF_L        30   /* Single cell request */
#define GG_UTNDU_AAL5_ANU_RX_BUF_L       150   /* 5 cell response plus header */
 /* Physical utopia device requirements */
#define GG_UTOPIA_PORTS_USED              0
#define GG_GMAC_PORTS_USED                0

/* NEU configuration based on Utopia Mode */
//#define GG_INCLUDE_NEU_PPP         /* Only Required for RTP */
//#define GG_INCLUDE_NEU_LLCSNAP     /* Only Required for RTP */

//#define GG_GMAC_TX_A_START             0x83EB00
//#define GG_GMAC_TX_A_END               0x83EF40
//#define GG_GMAC_RX_A_START             0x83EF40
//#define GG_GMAC_RX_A_END               0x83F540

//#define GG_GMAC_TX_B_START             0x86C800
//#define GG_GMAC_TX_B_END               0x888000
//#define GG_GMAC_RX_B_START             0x83F540
//#define GG_GMAC_RX_B_END               0x84E2C0

//#define GG_GMAC_RX_A_ASSEMB_START      0x84EFC8
//#define GG_GMAC_RX_A_ASSEMB_END        0x84F608

//#define GG_GMAC_RX_B_ASSEMB_START      0x851D28
//#define GG_GMAC_RX_B_ASSEMB_END        0x852368

#define GG_GMAC_TX_BUFSIZE_WORDS         160,320   /* Comma seperated list of tx buffer sizes */
#define GG_GMAC_RX_BUFSIZE_WORDS         160,320   /* Comma seperated list of rx buffer sizes */
#define GG_GMAC_RX_ASSEMSIZE_WORDS       1600,1600  /* Comma seperated list of rx assembly sizes */

#define GG_NEU_UDP_CORE_ID_MASK   0x001C
#define GG_INCLUDE_NEU_PORTMAP  1
//#define GG_NEU_MAX_PORTMAP_CHANNELS (GG_MAX_NUM_CHANNELS+3)
#define GG_NEU_MAX_PORTMAP_CHANNELS (84+3)

//#define GG_INCLUDE_COMPOUND_MSG

/* #define GG_HPI_HD */
/*#define GG_HPI_GPV1             1
#define GG_HPI_DEFAULT_ADDRESS  0x10600

#define HPI_VERSIONID           DPR_HPI_VERSION_ID_HPI_VER_C
#define HPI_DESCRIPTOR_LENGTH   0X0040L
#define HPI_ACS_STARTADDRESS    0x0040L
#define HPI_ACS_LENGTH          0x0020L
#define HPI_TXFIFOS             3
#define HPI_RXFIFOS             3
#define HPI_FIFOS               (HPI_RXFIFOS + HPI_TXFIFOS)
#define HPI_DESC_SIZE           (6 * HPI_FIFOS + 7)
#define HPI_ALIGNMENT           (DPR_HPI_ALIGNMENT_32_BIT | DPR_HPI_ALIGNMENT_INDEX)
*/
/* fifo types */

/*#define HPIV1_MAILBOX        0
#define HPIV1_SFIFO          1
#define HPIV1_CFIFO          0
#define HPIV1_INDEX          0
#define HPIV1_FCNTL          0
*/
/*TX FIFOS */

/* Tx FIFO 1 */
/*
#define FIFO1_TYPE          DPR_FIFO_TYPE_GENERAL
#define FIFO1_START         0x060L
#define FIFO1_LENGTH        DPR_MSGBUF_SIZE * 3
#define FIFO1_PKTTYPES      DPR_PKT_CLASS_HIGH_PRIORITY_MSG
#define FIFO1_OPTION        DPR_FIFO_OPTIONS_NONE
*/

/* Tx FIFO 2 */
/*
#define FIFO2_TYPE          DPR_FIFO_TYPE_GENERAL
#define FIFO2_START         0x120L
#define FIFO2_LENGTH        DPR_MSGBUF_SIZE * 8
#define FIFO2_PKTTYPES      DPR_PKT_CLASS_LOW_PRIORITY_MSG | DPR_PKT_CLASS_NORMAL_PRIORITY_MSG
#define FIFO2_OPTION        DPR_FIFO_OPTIONS_NONE
*/
/* Tx FIFO 3 */
/*
#define FIFO3_TYPE          DPR_FIFO_TYPE_GENERAL
#define FIFO3_START         0x4A0
#define FIFO3_LENGTH        0xC00
#define FIFO3_PKTTYPES      DPR_PKT_CLASS_VFD
#define FIFO3_OPTION        DPR_FIFO_OPTIONS_NONE
*/
/* Rx FIFO 4 */
/*
#define FIFO4_TYPE          DPR_FIFO_TYPE_GENERAL
#define FIFO4_START         0x320L
#define FIFO4_LENGTH        DPR_MSGBUF_SIZE * 2
#define FIFO4_PKTTYPES      DPR_PKT_CLASS_HIGH_PRIORITY_MSG
#define FIFO4_OPTION        DPR_FIFO_OPTIONS_NONE
*/
/* Rx FIFO 5 */
/*
#define FIFO5_TYPE          DPR_FIFO_TYPE_GENERAL
#define FIFO5_START         0x3A0L
#define FIFO5_LENGTH        DPR_MSGBUF_SIZE * 4
#define FIFO5_PKTTYPES      DPR_PKT_CLASS_LOW_PRIORITY_MSG | DPR_PKT_CLASS_NORMAL_PRIORITY_MSG
#define FIFO5_OPTION        DPR_FIFO_OPTIONS_NONE
*/
/* Rx FIFO 6 */
/*
#define FIFO6_TYPE          DPR_FIFO_TYPE_GENERAL
#define FIFO6_START         0x10A0L
#define FIFO6_LENGTH        0x600
#define FIFO6_PKTTYPES      DPR_PKT_CLASS_VFD
#define FIFO6_OPTION        DPR_FIFO_OPTIONS_NONE
*/
/******************************************************************/
/* DSIG Signalling                                                */
/******************************************************************/
//#define GG_INCLUDE_DSIGT              /* Enables DSIGT inclusion */
#define GG_DSIGT_TX_ADDR      ((void *)0x87ABC0)  /* DMA Tx Buffer */
#define GG_DSIGT_TX_END_ADDR  ((void *)0x87B3C0)  /* DMA Tx End    */
#define GG_DSIGT_RX_ADDR      ((void *)0x87B3C0)  /* DMA Rx Buffer */
#define GG_DSIGT_RX_END_ADDR  ((void *)0x87BBC0)  /* DMA Rx End    */
#define GG_DSIGT_SIZE         512     /* Tx/Rx DMA buffer size  */
#define GG_DSIGT_FRAME_SIZE   8       /* DMA samples/channel per interrupt */
#define GG_DSIGT_NUM_CHANNELS GG_MAX_NUM_CHANNELS  /* Number of dsigt channels */
#define GG_DSIGT_HEAP_PAGE    0
#define GG_DSIGT_ISR_SKIP     1       /* ISR calls to skip before processing */ /*  */

//#define GG_INCLUDE_DSIGR_AAL2                         /* AAL2 dsig relay    */
//#define GG_INCLUDE_DSIGR_RTP                          /* RTP dsig relay     */
#define GG_DSIGR_NUM_CHANNELS GG_MAX_NUM_CHANNELS     /* Number of channels */

/******************************************************************/
/* Checksum of Local Memory and Shared Program memory             */
/******************************************************************/
// enable all later on
//#define GG_ENABLE_CHKSUM_CHECK
//#define GG_CHKSUM_LOCAL_BLCK_COUNT 4 /* # of local memory blocks */
//#define GG_CHKSUM_PROG_BLCK_COUNT  4 /* # of shared program memory blocks */

/******************************************************************/
/* Health check feature                                           */
/******************************************************************/
#define GG_INCLUDE_HEALTH_CHECK
#define GG_INCLUDE_KEEP_ALIVE_NDU    /* enable keep alive cells   */
/******************************************************************/
/* Digit Signalling                                               */
/******************************************************************/
//#define GG_INCLUDE_DIGITSIG

/******************************************************************/
/* Automatic SS7 speech path continuity check                     */
/******************************************************************/
//#define GG_INCLUDE_AUTOCOT


/******************************************************************/
/* Packet router functionality                                    */
/******************************************************************/
//#define GG_INCLUDE_PRU

/******************************************************************/
/* BERT Mode                                                      */
/******************************************************************/
//#define GG_INCLUDE_BERT

/******************************************************************/
/* number of samples reported in one message for PCM TRACE */
/******************************************************************/
#define GG_PCM_PKT_TRACE_INTERVAL       60     /* 30ms maximum delay */
#define GG_PCM_PKT_TRACE_PKT_FREE       14     /* Free Cells to allow trace */
#define GG_SIU_PCM_TRACE_SAMPLE_PER_MSG 240    /* 30 ms */
//#define GG_SIU_PCM_TRACE_SAMPLE_PER_MSG 160    /* 20 ms */
//#define GG_SIU_PCM_TRACE_SAMPLE_PER_MSG 196    /* 22 ms */
//#define GG_SIU_PCM_TRACE_SAMPLE_PER_MSG 80     /* 10 ms */
//#define GG_SIU_PCM_TRACE_SAMPLE_PER_MSG 88     /* 11 ms */

/******************************************************************
 * Announcement server
 ******************************************************************/
//#define GG_INCLUDE_ANUSVR

#define GG_ANUSVR_SCHEDULE_INTERVAL     5   /* 5 Tdm ISR = 5ms          */
#define GG_ANUSVR_MIN_SCHEDULE          5   /* min 5 requests per pass  */
#define GG_ANUSVR_MAX_SCHEDULE         20   /* max 20 requests per pass */
#define GG_ANUSVR_ALPHA_Q16        0x4000   /* 25% of requests per pass */
#define GG_ANUSVR_BETA_Q16         0xe000   /* Warn at 87.5% of queue   */
#define GG_ANUSVR_BLOCK_ADDR   0x80000000   /* Base offset address      */
#define GG_ANUSVR_ENABLE             TRUE   /* Enabled by default       */

#define GG_ANUSVR_REQ_FIFO_SIZE_BYTES 0x2000  /* 8Kbytes of queue       */
#define GG_ANUSVR_MAX_REQ_SIZE_BYTES     240  /* Max request size       */

#define GG_ANUSVR_UDP_PORT_DEFAULT     0x330  /* Default request port   */

/**************************************************************************
 * CPPI and QM buffers
 **************************************************************************/
/* The following is configuration for Tx/Rx control and packet linked buffers.
   The total number of buffers per core must be 2^x (e.g.,128, 256,...).
   The size needs to be multiple of Cache line size (CACHE_L2_LINESIZE=128)  */
/* Rx control */
#define GG_QM_RX_LINKED_BUF_1_SIZE            384
#define GG_QM_RX_LINKED_BUF_1_NBUFS_PER_CORE  10
#define GG_QM_RX_LINKED_BUF_1_NBUFS           (GG_QM_RX_LINKED_BUF_1_NBUFS_PER_CORE*CPU_NCORES)

/* Tx control */
#define GG_QM_TX_LINKED_BUF_1_SIZE            384
#define GG_QM_TX_LINKED_BUF_1_NBUFS_PER_CORE  10
#define GG_QM_TX_LINKED_BUF_1_NBUFS           (GG_QM_TX_LINKED_BUF_1_NBUFS_PER_CORE*CPU_NCORES)

/* Rx packet */
#define GG_QM_RX_LINKED_BUF_2_SIZE            (((1516+127)/128)*128)
#define GG_QM_RX_LINKED_BUF_2_NBUFS_PER_CORE  118
#define GG_QM_RX_LINKED_BUF_2_NBUFS           (GG_QM_RX_LINKED_BUF_2_NBUFS_PER_CORE*CPU_NCORES)

/* Tx packet */
#define GG_QM_TX_LINKED_BUF_2_SIZE            (((1516+127)/128)*128)
#define GG_QM_TX_LINKED_BUF_2_NBUFS_PER_CORE  118
#define GG_QM_TX_LINKED_BUF_2_NBUFS           (GG_QM_TX_LINKED_BUF_2_NBUFS_PER_CORE*CPU_NCORES)

/* QM */
#define GG_QM_NUM_DESCS_PER_CORE              (GG_QM_RX_LINKED_BUF_1_NBUFS_PER_CORE + \
                                               GG_QM_TX_LINKED_BUF_1_NBUFS_PER_CORE + \
                                               GG_QM_RX_LINKED_BUF_2_NBUFS_PER_CORE + \
                                               GG_QM_TX_LINKED_BUF_2_NBUFS_PER_CORE) /* Would need to be  2^x */
#define GG_QM_NUM_DESCS                       (GG_QM_NUM_DESCS_PER_CORE*CPU_NCORES)
#define GG_QM_DESC_SIZE                       128

/******************************************************************/
#define GG_MAX_YUV_INTERCORE_WIDTH  720
#define GG_MAX_YUV_INTERCORE_HEIGHT 576
/******************************************************************/
/* Internal ECO related #define(s) are created from features.     */
/* DO NOT MOVE THESE DIRECTIVES FROM THE END OF THIS FILE OR YOU  */
/* WILL DEFEAT THEIR PURPOSE!!                                    */
/******************************************************************/
#include "ggfeature2eco.h"
/* DSP Bitmap configuration */
#include "ggbitmap.h"

#endif  /* _GGSWCFG_H */

/* nothing past this point */

