;***********************************************************************************
;* FILE PURPOSE: Provide interrupt context save and C callout
;***********************************************************************************
;* FILE NAME: halicxt.s
;*
;* DESCRIPTION: Provides the functions that are called from the vector table.
;*				A context save is performed, then a callout to the application
;*				level (or hal muxed level) function. On return the context
;*				restore is performed.
;*
;* Copyright (C) 2005, Texas Instruments Incorporated.            *
; * 
; *  Redistribution and use in source and binary forms, with or without 
; *  modification, are permitted provided that the following conditions 
; *  are met:
; *
; *    Redistributions of source code must retain the above copyright 
; *    notice, this list of conditions and the following disclaimer.
; *
; *    Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the 
; *    documentation and/or other materials provided with the   
; *    distribution.
; *
; *    Neither the name of Texas Instruments Incorporated nor the names of
; *    its contributors may be used to endorse or promote products derived
; *    from this software without specific prior written permission.
; *
; *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; */

	.include "ggisrinc.inc"


	.asg	A15, FP
	.asg	B14, DP
	.asg	B15, SP

  .if $isdefed("GG_BIOS")
    .include  "hwi.h62"
  .endif


;************************************************************************************
;* BIOS - a buffer used for interrupt. Should automatically add 2 words for each
;*		  enabled interrupt.
;************************************************************************************

	.if $isdefed("GG_BIOS")
		.sect ".pristack"
cslPriStk	.int 0,0

	.if GG_HWCFG_VEC04_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC05_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC06_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC07_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC08_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC09_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC10_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC11_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC12_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC13_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC14_ENABLED
			.int 0, 0
	.endif

	.if GG_HWCFG_VEC15_ENABLED
			.int 0, 0
	.endif

cslPriStkPtr .int cslPriStk

	.endif  ; GG_BIOS


; ********************************************************************************
; * Context saves for bios and non-bios
; ********************************************************************************
	.if $isdefed("GG_BIOS")
		.ref	_CSL_ISR_CONTEXT_SAVE_BIOS
		.ref	_CSL_ISR_CONTEXT_REST_BIOS
		.asg	_CSL_ISR_CONTEXT_SAVE_BIOS, _CXT_SAVE
		.asg    _CSL_ISR_CONTEXT_REST_BIOS, _CXT_REST
	.else
		.ref    _CSL_ISR_CONTEXT_SAVE
		.ref    _CSL_ISR_CONTEXT_REST
		.asg    _CSL_ISR_CONTEXT_SAVE, _CXT_SAVE
		.asg    _CSL_ISR_CONTEXT_REST, _CXT_REST
	.endif


; ********************************************************************************
; * Vector 04
; ********************************************************************************
	.if GG_HWCFG_VEC04_ENABLED

    .def _halVec04
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec04_fcn_ptr, _halVec04_fcn_ptr
	.asg halVec04_cxt_ptr, _halVec04_cxt_ptr
    .endif

	.ref _halVec04_fcn_ptr
	.ref _halVec04_cxt_ptr


_halVec04:	
	STW    .D2T1   A31, *+SP(0)
	MVKL   .S1    _halVec04_callout, A31
	MVKH   .S1    _halVec04_callout, A31
	BNOP   .S2	  _CXT_SAVE, 5

_halVec04_callout:
	MVKL   .S1   _halVec04_fcn_ptr, A30
	MVKH   .S1   _halVec04_fcn_ptr, A30
	MVKL   .S1   _halVec04_cxt_ptr, A31
	MVKH   .S1   _halVec04_cxt_ptr, A31
	LDW    .D1   *A30, A3
	LDW    .D1   *A31, A4
	NOP			  4
	B      .S2    A3
	ADDKPC .S2   _halVec04_ret, B3, 4

_halVec04_ret:
	BNOP   .S2  _CXT_REST, 5

	.endif
	
; *******************************************************************************
; * Vector 05
; *******************************************************************************
	.if GG_HWCFG_VEC05_ENABLED

    .def _halVec05
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec05_fcn_ptr, _halVec05_fcn_ptr
	.asg halVec05_cxt_ptr, _halVec05_cxt_ptr
    .endif

	.ref _halVec05_fcn_ptr
	.ref _halVec05_cxt_ptr

_halVec05:
	STW     .D2T1   A31, *+SP(0)
	MVKL    .S1    _halVec05_callout, A31
	MVKH    .S1    _halVec05_callout, A31
	BNOP    .S2    _CXT_SAVE, 5

_halVec05_callout:
	MVKL    .S1    _halVec05_fcn_ptr, A30
	MVKH    .S1    _halVec05_fcn_ptr, A30
	MVKL    .S1    _halVec05_cxt_ptr, A31
	MVKH    .S1    _halVec05_cxt_ptr, A31
	LDW     .D1    *A30, A3
	LDW     .D1    *A31, A4
	NOP				4
	B		.S2     A3
	ADDKPC  .S2    _halVec05_ret, B3, 4

_halVec05_ret:
	BNOP    .S2    _CXT_REST, 5

	.endif

; ********************************************************************************
; * Vector 06
; ********************************************************************************
	.if GG_HWCFG_VEC06_ENABLED

    .def _halVec06
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec06_fcn_ptr, _halVec06_fcn_ptr
	.asg halVec06_cxt_ptr, _halVec06_cxt_ptr
    .endif

	.ref _halVec06_fcn_ptr
	.ref _halVec06_cxt_ptr

_halVec06:
	STW     .D2T1    A31, *+SP(0)
	MVKL    .S1     _halVec06_callout, A31
	MVKH    .S1     _halVec06_callout, A31
	BNOP    .S2     _CXT_SAVE, 5

_halVec06_callout:
	MVKL    .S1     _halVec06_fcn_ptr, A30
	MVKH    .S1     _halVec06_fcn_ptr, A30
	MVKL    .S1     _halVec06_cxt_ptr, A31
	MVKH    .S1     _halVec06_cxt_ptr, A31
	LDW     .D1     *A30, A3
	LDW     .D1     *A31, A4
	NOP				 4
	B       .S2      A3
	ADDKPC  .S2     _halVec06_ret, B3, 4

_halVec06_ret:
	BNOP    .S2     _CXT_REST,  5

	.endif

; ******************************************************************************
; * Vector 07
; ******************************************************************************
	.if GG_HWCFG_VEC07_ENABLED

	.def _halVec07
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec07_fcn_ptr, _halVec07_fcn_ptr
	.asg halVec07_cxt_ptr, _halVec07_cxt_ptr
    .endif

	.ref _halVec07_fcn_ptr
	.ref _halVec07_cxt_ptr

_halVec07:
	STW     .D2T1    A31, *+SP(0)
	MVKL    .S1     _halVec07_callout, A31
	MVKH    .S1     _halVec07_callout, A31
	BNOP    .S2     _CXT_SAVE, 5

_halVec07_callout:
	MVKL    .S1     _halVec07_fcn_ptr, A30
	MVKH    .S1     _halVec07_fcn_ptr, A30
	MVKL    .S1     _halVec07_cxt_ptr, A31
	MVKH    .S1     _halVec07_cxt_ptr, A31
	LDW     .D1     *A30, A3
	LDW     .D1     *A31, A4
	NOP				 4
	B       .S2      A3
	ADDKPC  .S2     _halVec07_ret, B3, 4

_halVec07_ret:
	BNOP    .S2     _CXT_REST, 5

	.endif


; *****************************************************************************
; * Vector 08
; *****************************************************************************
	.if GG_HWCFG_VEC08_ENABLED

	.def _halVec08
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec08_fcn_ptr, _halVec08_fcn_ptr
	.asg halVec08_cxt_ptr, _halVec08_cxt_ptr
    .endif

	.ref _halVec08_fcn_ptr
	.ref _halVec08_cxt_ptr

_halVec08:
	STW     .D2T1    A31, *+SP(0)
	MVKL    .S1     _halVec08_callout, A31
	MVKH    .S1     _halVec08_callout, A31
	BNOP    .S2     _CXT_SAVE, 5

_halVec08_callout:
    MVKL    .S1     _halVec08_fcn_ptr, A30
	MVKH    .S1     _halVec08_fcn_ptr, A30
	MVKL    .S1     _halVec08_cxt_ptr, A31
	MVKH    .S1     _halVec08_cxt_ptr, A31
	LDW     .D1     *A30, A3
	LDW     .D1     *A31, A4
	NOP				 4
	B		.S2      A3
	ADDKPC  .S2     _halVec08_ret, B3, 4

_halVec08_ret:
	BNOP    .S2     _CXT_REST, 5

	.endif


; *******************************************************************************
; * Vector 09
; *******************************************************************************
	.if GG_HWCFG_VEC09_ENABLED

	.def _halVec09
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec09_fcn_ptr, _halVec09_fcn_ptr
	.asg halVec09_cxt_ptr, _halVec09_cxt_ptr
    .endif

	.ref _halVec09_fcn_ptr
	.ref _halVec09_cxt_ptr

_halVec09:
	STW     .D2T1     A31, *+SP(0)
	MVKL    .S1      _halVec09_callout, A31
	MVKH    .S1      _halVec09_callout, A31
	BNOP    .S2      _CXT_SAVE, 5

_halVec09_callout:
	MVKL    .S1      _halVec09_fcn_ptr, A30
	MVKH    .S1      _halVec09_fcn_ptr, A30
	MVKL    .S1      _halVec09_cxt_ptr, A31
	MVKH    .S1      _halVec09_cxt_ptr, A31
	LDW     .D1      *A30, A3
	LDW     .D1      *A31, A4
	NOP				  4
	B       .S2       A3
	ADDKPC  .S2      _halVec09_ret, B3, 4

_halVec09_ret:
	BNOP    .S2      _CXT_REST, 5

	.endif


; *********************************************************************************
; * Vector 10
; *********************************************************************************
	.if GG_HWCFG_VEC10_ENABLED

	.def _halVec10
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec10_fcn_ptr, _halVec10_fcn_ptr
	.asg halVec10_cxt_ptr, _halVec10_cxt_ptr
    .endif

	.ref _halVec10_fcn_ptr
	.ref _halVec10_cxt_ptr

_halVec10:
	STW     .D2T1      A31, *+SP(0)
	MVKL    .S1       _halVec10_callout, A31
	MVKH    .S1       _halVec10_callout, A31
	BNOP    .S2       _CXT_SAVE, 5


_halVec10_callout:
	MVKL    .S1       _halVec10_fcn_ptr, A30
	MVKH    .S1       _halVec10_fcn_ptr, A30
	MVKL    .S1       _halVec10_cxt_ptr, A31
	MVKH    .S1       _halVec10_cxt_ptr, A31
	LDW	    .D1       *A30, A3
	LDW     .D1       *A31, A4
	NOP                4
	B       .S2        A3
	ADDKPC  .S2       _halVec10_ret, B3, 4

_halVec10_ret:
	BNOP    .S2       _CXT_REST, 5

	.endif


; **********************************************************************************
; * Vector 11
; **********************************************************************************
	.if GG_HWCFG_VEC11_ENABLED

	.def _halVec11
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec11_fcn_ptr, _halVec11_fcn_ptr
	.asg halVec11_cxt_ptr, _halVec11_cxt_ptr
    .endif

	.ref _halVec11_fcn_ptr
	.ref _halVec11_cxt_ptr

_halVec11:
	STW     .D2T1      A31, *+SP(0)
	MVKL    .S1       _halVec11_callout, A31
	MVKH    .S1       _halVec11_callout, A31
	BNOP    .S2       _CXT_SAVE, 5


_halVec11_callout:
	MVKL    .S1       _halVec11_fcn_ptr, A30
	MVKH    .S1       _halVec11_fcn_ptr, A30
	MVKL    .S1       _halVec11_cxt_ptr, A31
	MVKH    .S1       _halVec11_cxt_ptr, A31
	LDW     .D1       *A30, A3
	LDW     .D1       *A31, A4
	NOP                4
	B       .S2        A3
	ADDKPC  .S2       _halVec11_ret, B3, 4

_halVec11_ret:
	BNOP    .S2       _CXT_REST, 5

	.endif



; **********************************************************************************
; * Vector 12
; **********************************************************************************
	.if GG_HWCFG_VEC12_ENABLED

	.def _halVec12
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec12_fcn_ptr, _halVec12_fcn_ptr
	.asg halVec12_cxt_ptr, _halVec12_cxt_ptr
    .endif

	.ref _halVec12_fcn_ptr
	.ref _halVec12_cxt_ptr

_halVec12:
	STW     .D2T1      A31, *+SP(0)
	MVKL    .S1       _halVec12_callout, A31
	MVKH    .S1       _halVec12_callout, A31
	BNOP    .S2       _CXT_SAVE, 5


_halVec12_callout:
	MVKL    .S1       _halVec12_fcn_ptr, A30
	MVKH    .S1       _halVec12_fcn_ptr, A30
	MVKL    .S1       _halVec12_cxt_ptr, A31
	MVKH    .S1       _halVec12_cxt_ptr, A31
	LDW     .D1       *A30, A3
	LDW     .D1       *A31, A4
	NOP                4
	B       .S2        A3
	ADDKPC  .S2       _halVec12_ret, B3, 4

_halVec12_ret:
	BNOP    .S2       _CXT_REST, 5

	.endif


; **********************************************************************************
; * Vector 13
; **********************************************************************************
	.if GG_HWCFG_VEC13_ENABLED

	.def _halVec13
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec13_fcn_ptr, _halVec13_fcn_ptr
	.asg halVec13_cxt_ptr, _halVec13_cxt_ptr
    .endif

	.ref _halVec13_fcn_ptr
	.ref _halVec13_cxt_ptr

_halVec13:
	STW     .D2T1      A31, *+SP(0)
	MVKL    .S1       _halVec13_callout, A31
	MVKH    .S1       _halVec13_callout, A31
	BNOP    .S2       _CXT_SAVE, 5


_halVec13_callout:
	MVKL    .S1       _halVec13_fcn_ptr, A30
	MVKH    .S1       _halVec13_fcn_ptr, A30
	MVKL    .S1       _halVec13_cxt_ptr, A31
	MVKH    .S1       _halVec13_cxt_ptr, A31
	LDW     .D1       *A30, A3
	LDW     .D1       *A31, A4
	NOP                4
	B       .S2        A3
	ADDKPC  .S2       _halVec13_ret, B3, 4

_halVec13_ret:
	BNOP    .S2       _CXT_REST, 5

	.endif


; ***********************************************************************************
; * Vector 14
; ***********************************************************************************
	.if GG_HWCFG_VEC14_ENABLED

	.def _halVec14
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec14_fcn_ptr, _halVec14_fcn_ptr
	.asg halVec14_cxt_ptr, _halVec14_cxt_ptr
    .endif

	.ref _halVec14_fcn_ptr
	.ref _halVec14_cxt_ptr

_halVec14:
	STW     .D2T1      A31, *+SP(0)
	MVKL    .S1       _halVec14_callout, A31
	MVKH    .S1       _halVec14_callout, A31
	BNOP    .S2       _CXT_SAVE, 5


_halVec14_callout:
	MVKL    .S1       _halVec14_fcn_ptr, A30
	MVKH    .S1       _halVec14_fcn_ptr, A30
	MVKL    .S1       _halVec14_cxt_ptr, A31
	MVKH    .S1       _halVec14_cxt_ptr, A31
	LDW     .D1       *A30, A3
	LDW     .D1       *A31, A4
	NOP                4
	B       .S2        A3
	ADDKPC  .S2       _halVec14_ret, B3, 4

_halVec14_ret:
	BNOP    .S2       _CXT_REST, 5

	.endif

; ***********************************************************************************
; * Vector 15
; ***********************************************************************************
	.if GG_HWCFG_VEC15_ENABLED

	.def _halVec15
    .if __TI_EABI__
        ; Alias so COFF and EABI can share same code
	.asg halVec15_fcn_ptr, _halVec15_fcn_ptr
	.asg halVec15_cxt_ptr, _halVec15_cxt_ptr
    .endif

	.ref _halVec15_fcn_ptr
	.ref _halVec15_cxt_ptr

_halVec15:
	STW     .D2T1      A31, *+SP(0)
	MVKL    .S1		  _halVec15_callout, A31
	MVKH    .S1       _halVec15_callout, A31
	BNOP    .S2       _CXT_SAVE, 5


_halVec15_callout:
	MVKL    .S1       _halVec15_fcn_ptr, A30
	MVKH    .S1       _halVec15_fcn_ptr, A30
	MVKL    .S1       _halVec15_cxt_ptr, A31
	MVKH    .S1       _halVec15_cxt_ptr, A31
	LDW     .D1       *A30, A3
	LDW     .D1       *A31, A4
	NOP                4
	B       .S2        A3
	ADDKPC  .S2       _halVec15_ret, B3, 4

_halVec15_ret:
	BNOP    .S2       _CXT_REST, 5

	.endif




















	

