************************************************************************
*                                                                      *
*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
; * 
; *  Redistribution and use in source and binary forms, with or without 
; *  modification, are permitted provided that the following conditions 
; *  are met:
; *
; *    Redistributions of source code must retain the above copyright 
; *    notice, this list of conditions and the following disclaimer.
; *
; *    Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the 
; *    documentation and/or other materials provided with the   
; *    distribution.
; *
; *    Neither the name of Texas Instruments Incorporated nor the names of
; *    its contributors may be used to endorse or promote products derived
; *    from this software without specific prior written permission.
; *
; *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; */

; ************************************************************************************
; * FILE PURPOSE: Interrupt vector table
; ************************************************************************************
; * FILE NAME: vectors.s
; *
; * DESCRIPTION: The c64x interrupt vector table. Branches to context save, and
; *             directs context save to the application interrupt handler.
; *
; ************************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg VECTORS, _VECTORS
    .endif

    .ref    _c_int00
    .global _VECTORS
    .global cslPriStk
    .global cslPriStkPtr
    .asg  B15, SP

	.cdecls C,NOLIST, "gghwcfg.h"

; ********************************************************************************
; * Context saves for bios and non-bios
; ********************************************************************************

;************************************************************************************
;* BIOS - a buffer used for interrupt. Should automatically add 4 words for each
;*		  enabled interrupt. (2 for context pointer and 2 for function pointer)
;************************************************************************************

    .if $isdefed("GG_BIOS")
    	.sect ".pristack"
    	
cslPriStk    .int 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
cslPriStkPtr .int cslPriStk
    .endif
    
    .if $isdefed("GG_BIOS")
        .ref    _CSL_ISR_CONTEXT_SAVE_BIOS
        .ref    _CSL_ISR_CONTEXT_REST_BIOS
        .asg    _CSL_ISR_CONTEXT_SAVE_BIOS, _CXT_SAVE
        .asg    _CSL_ISR_CONTEXT_REST_BIOS, _CXT_REST
    .else
        .ref    _CSL_ISR_CONTEXT_SAVE
        .ref    _CSL_ISR_CONTEXT_REST
        .asg    _CSL_ISR_CONTEXT_SAVE, _CXT_SAVE
        .asg    _CSL_ISR_CONTEXT_REST, _CXT_REST
    .endif

	.ref  _CSL_ISR_NOCXT_SAVE

; **********************************************************************************
; * The full context save macro. On the branch to context save the following
; * registers have been saved on the stack and replaced with the following values:
; *
; *    B1:B0      B0 now contains the address of the application interrupt handler
; *    A31:A30    A30 now contains the context of the application interrupt handler
; *
; *    Note that there is one nop in the branch out. There isn't enough room in
; *    this fetch packet for any more instructions. The alternative was to save
; *    not the app context and callout, but an app index. The context save function
; *    could then load the values from the index in parallel with context save. 
; *    But it seems that this would require a copy from a table in memory, and that
; *    would eliminate the one cycle saved here.
; *
; **********************************************************************************    

fullsavebios    .macro    appCallout, appContext

	    BNOP   .S1   _CXT_SAVE, 1
   	||  ADDK   .S2   -24, SP                  ; Adjust stack frame to save registers that are used by 
   						  ; context save for BIOS
            STDW   .D2T1  A31:A30, *+SP(0)        ; Delay slot 2
    	    STDW   .D2T2  B1:B0, *+SP(8)          ; Delay Slot 3

	    MVKL   .S2  appCallout, B0            ; Delay slot 4
	||  MVKL   .S1  appContext, A30           ; This is were the interrupt handler will go
						  ; Additional context save is not done here
						  ; because there are only 32 bytes available
						  ; in the interrupt handler

	    MVKH   .S2  appCallout, B0            ; Delay slot 5
	||  MVKH   .S1  appContext, A30           ; The rest of the interrupt handler info

	    .endm
	    

fullsave    .macro    appCallout, appContext

    BNOP   .S1   _CXT_SAVE, 1             ; Branch to full context save, Delay slot 1
||  ADDK   .S2   -232, SP                 ; Adjust stack frame
    STDW   .D2T1  A31:A30, *+SP(216)      ; Delay slot 2
    STDW   .D2T2  B1:B0, *+SP(56)         ; Delay Slot 3

    MVKL   .S2  appCallout, B0            ; Delay slot 4
||  MVKL   .S1  appContext, A30           ; This is were the interrupt handler will go
                                          ; Additional context save is not done here
                                          ; because there are only 32 bytes available
                                          ; in the interrupt handler

    MVKH   .S2  appCallout, B0            ; Delay slot 5
||  MVKH   .S1  appContext, A30           ; The rest of the interrupt handler info


    .endm


; **************************************************************************************
; * The no context save macro. The code branches to the appCallout with no change
; * to the stack or registers, and no load of app Context
; **************************************************************************************

nosave		.macro	appCallout

	B		.S1	    _CSL_ISR_NOCXT_SAVE
||	ADDK	.S2		-8, SP				  ; Stack must be aligned on an 8 byte boundary

    STW		.D2		B0, *+SP(4)			  ; Temporarily save B0
	
	MVKL	.S2		appCallout, B0		  ; Load the pointer to the branch address
	MVKH	.S2		appCallout, B0		  ; into B0.

	LDW		.D2	   *B0, B0				  ; Load the branch address to B0

    NOP									  ; Final brnach delay slot

	.endm

        
    .sect ".vectors"
    .align 32
_VECTORS:
    mvkl .S2 _c_int00, B0
    mvkh .S2 _c_int00, B0
    B    .S2 B0
    NOP   4
    
    .align 32

NMI_NMI:    

    .ref  bogus_exc_hndlr
vec01Trap:  BNOP  .S2 bogus_exc_hndlr, 4
    NOP

    .align 32
RESV1:
    B     IRP
    NOP
    NOP
    NOP
    NOP
    NOP
    NOP
    .align 32
RESV2:
    B     IRP
    NOP
    NOP
    NOP
    NOP
    NOP
    NOP



; **********************************************************************************
; * Vector 04
; **********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec04_fcn_ptr, _halVec04_fcn_ptr
      .asg halVec04_cxt_ptr, _halVec04_cxt_ptr
    .endif

    .ref _halVec04_fcn_ptr   ; Declared in halisr.c, and by default pointing
    .ref _halVec04_cxt_ptr   ; to an interrupt trap
    .global INT4
    .align 32

INT4:
	.if $defined(GG_HWCFG_VEC04_NO_CXT_SAVE)
		nosave	_halVec04_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC04_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec04_fcn_ptr, _halVec04_cxt_ptr
	.else
	    fullsave _halVec04_fcn_ptr, _halVec04_cxt_ptr
	.endif
	.endif

; ***********************************************************************************
; * Vector 05
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec05_fcn_ptr, _halVec05_fcn_ptr
      .asg halVec05_cxt_ptr, _halVec05_cxt_ptr
    .endif

    .ref _halVec05_fcn_ptr
    .ref _halVec05_cxt_ptr
    .global INT5
    .align 32

INT5:
	.if $defined(GG_HWCFG_VEC05_NO_CXT_SAVE)
		nosave _halVec05_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC05_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec05_fcn_ptr, _halVec05_cxt_ptr
	.else
	    fullsave _halVec05_fcn_ptr, _halVec05_cxt_ptr
	.endif
	.endif

; ***********************************************************************************
; * Vector 06
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec06_fcn_ptr, _halVec06_fcn_ptr
      .asg halVec06_cxt_ptr, _halVec06_cxt_ptr
    .endif

    .ref _halVec06_fcn_ptr
    .ref _halVec06_cxt_ptr
    .global INT6
    .align 32
                   
INT6:
	.if $defined(GG_HWCFG_VEC06_NO_CXT_SAVE)
		nosave _halVec06_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC06_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec06_fcn_ptr, _halVec06_cxt_ptr
	.else
	    fullsave _halVec06_fcn_ptr, _halVec06_cxt_ptr
	.endif
	.endif


; ***********************************************************************************
; * Vector 07
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec07_fcn_ptr, _halVec07_fcn_ptr
      .asg halVec07_cxt_ptr, _halVec07_cxt_ptr
    .endif

    .ref _halVec07_fcn_ptr
    .ref _halVec07_cxt_ptr
    .global INT7
    .align 32
                   
INT7:
	.if $defined(GG_HWCFG_VEC07_NO_CXT_SAVE)
		nosave _halVec07_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC07_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec07_fcn_ptr, _halVec07_cxt_ptr
	.else
	    fullsave _halVec07_fcn_ptr, _halVec07_cxt_ptr
	.endif
	.endif


; ***********************************************************************************
; * Vector 08
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec08_fcn_ptr, _halVec08_fcn_ptr
      .asg halVec08_cxt_ptr, _halVec08_cxt_ptr
    .endif

    .ref _halVec08_fcn_ptr
    .ref _halVec08_cxt_ptr
    .global INT8
    .align 32
                   
INT8:
	.if $defined(GG_HWCFG_VEC08_NO_CXT_SAVE)
		nosave _halVec08_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC08_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec08_fcn_ptr, _halVec08_cxt_ptr
	.else
	    fullsave _halVec08_fcn_ptr, _halVec08_cxt_ptr
	.endif
	.endif


; ***********************************************************************************
; * Vector 09
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec09_fcn_ptr, _halVec09_fcn_ptr
      .asg halVec09_cxt_ptr, _halVec09_cxt_ptr
    .endif

    .ref _halVec09_fcn_ptr
    .ref _halVec09_cxt_ptr
    .global INT9
    .align 32
                   
INT9:
	.if $defined(GG_HWCFG_VEC09_NO_CXT_SAVE)
		nosave _halVec09_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC09_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec09_fcn_ptr, _halVec09_cxt_ptr
	.else
	    fullsave _halVec09_fcn_ptr, _halVec09_cxt_ptr
	.endif
	.endif


; ***********************************************************************************
; * Vector 10
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec10_fcn_ptr, _halVec10_fcn_ptr
      .asg halVec10_cxt_ptr, _halVec10_cxt_ptr
    .endif

    .ref _halVec10_fcn_ptr
    .ref _halVec10_cxt_ptr
    .global INT10
    .align 32
                   
INT10:
	.if $defined(GG_HWCFG_VEC10_NO_CXT_SAVE)
		nosave _halVec10_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC10_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec10_fcn_ptr, _halVec10_cxt_ptr
	.else
	    fullsave _halVec10_fcn_ptr, _halVec10_cxt_ptr
	.endif
	.endif



; ***********************************************************************************
; * Vector 11
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec11_fcn_ptr, _halVec11_fcn_ptr
      .asg halVec11_cxt_ptr, _halVec11_cxt_ptr
    .endif

    .ref _halVec11_fcn_ptr
    .ref _halVec11_cxt_ptr
    .global INT11
    .align 32
                   
INT11:
	.if $defined(GG_HWCFG_VEC11_NO_CXT_SAVE)
		nosave _halVec11_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC11_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec11_fcn_ptr, _halVec11_cxt_ptr
	.else
	    fullsave _halVec11_fcn_ptr, _halVec11_cxt_ptr
	.endif
	.endif



; ***********************************************************************************
; * Vector 12
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec12_fcn_ptr, _halVec12_fcn_ptr
      .asg halVec12_cxt_ptr, _halVec12_cxt_ptr
    .endif

    .ref _halVec12_fcn_ptr
    .ref _halVec12_cxt_ptr
    .global INT12
    .align 32
                   
INT12:
	.if $defined(GG_HWCFG_VEC12_NO_CXT_SAVE)
		nosave _halVec12_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC12_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec12_fcn_ptr, _halVec12_cxt_ptr
	.else
	    fullsave _halVec12_fcn_ptr, _halVec12_cxt_ptr
	.endif
	.endif


; ***********************************************************************************
; * Vector 13
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec13_fcn_ptr, _halVec13_fcn_ptr
      .asg halVec13_cxt_ptr, _halVec13_cxt_ptr
    .endif

    .ref _halVec13_fcn_ptr
    .ref _halVec13_cxt_ptr
    .global INT13
    .align 32

INT13:
	.if $defined(GG_HWCFG_VEC13_NO_CXT_SAVE)
		nosave _halVec13_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC13_SIMPLE_RETURN)
	    B      IRP
		NOP     5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec13_fcn_ptr, _halVec13_cxt_ptr
	.else
	    fullsave _halVec13_fcn_ptr, _halVec13_cxt_ptr
	.endif

	.endif


; ***********************************************************************************
; * Vector 14
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec14_fcn_ptr, _halVec14_fcn_ptr
      .asg halVec14_cxt_ptr, _halVec14_cxt_ptr
    .endif

    .ref _halVec14_fcn_ptr
    .ref _halVec14_cxt_ptr
    .global INT14
    .align 32

INT14:
	.if $defined(GG_HWCFG_VEC14_NO_CXT_SAVE)
		nosave _halVec14_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC14_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec14_fcn_ptr, _halVec14_cxt_ptr
	.else
	    fullsave _halVec14_fcn_ptr, _halVec14_cxt_ptr
	.endif

	.endif



; ***********************************************************************************
; * Vector 15
; ***********************************************************************************

    .if __TI_EABI__
      ; Alias so COFF and EABI can share same code
      .asg halVec15_fcn_ptr, _halVec15_fcn_ptr
      .asg halVec15_cxt_ptr, _halVec15_cxt_ptr
    .endif

    .ref _halVec15_fcn_ptr
    .ref _halVec15_cxt_ptr
    .global INT15
    .align 32

INT15:
	.if $defined(GG_HWCFG_VEC15_NO_CXT_SAVE)
		nosave _halVec15_fcn_ptr
	.elseif $defined(GG_HWCFG_VEC15_SIMPLE_RETURN)
	    B    IRP
		NOP  5
	.else
		.if $isdefed("GG_BIOS")
		    fullsavebios _halVec15_fcn_ptr, _halVec15_cxt_ptr
	.else
	    fullsave _halVec15_fcn_ptr, _halVec15_cxt_ptr
		.endif

	.endif



