/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/**************************************************************************************
 * FUNCTION PURPOSE: Provide re-assignable ISR call out ability
 **************************************************************************************
 * FILE NAME: halisr.c
 *
 * DESCRIPTION: For interrupts enabled through gghwcfg, this file creates space
 *				for a function pointer to call when an interrupt occurs, along
 *			    with a context. Provides a way for the upper level to set the
 *				values of the pointer and context.
 *
 **************************************************************************************/
#include "types.h"
#include "gghwcfg.h"
#include "hal.h"
#include "halloc.h"
#include "cslisr.h"
#include "cslisr_dev.h"
#include <stdio.h>

#ifdef GG_BIOS
#include <ti/sysbios/family/c64p/Hwi.h>
#include <ti/sysbios/interfaces/IHwi.h>
#endif

/**************************************************************************************
 * FUNCTION PURPOSE: The trap function
 **************************************************************************************
 * DESCRIPTION: At startup all function pointers are set to point here.
 **************************************************************************************/
void hal_isr_trap (void *cxt)
{
  for (;;);

} /* hal_isr_trap */


/*************************************************************************************
 * Global variable initialization
 *************************************************************************************/
void (*halVec04_fcn_ptr)(void *) = hal_isr_trap;
void *halVec04_cxt_ptr = NULL;

void (*halVec05_fcn_ptr)(void *) = hal_isr_trap;
void *halVec05_cxt_ptr = NULL;

void (*halVec06_fcn_ptr)(void *) = hal_isr_trap;
void *halVec06_cxt_ptr = NULL;

void (*halVec07_fcn_ptr)(void *) = hal_isr_trap;
void *halVec07_cxt_ptr = NULL;

void (*halVec08_fcn_ptr)(void *) = hal_isr_trap;
void *halVec08_cxt_ptr = NULL;

void (*halVec09_fcn_ptr)(void *) = hal_isr_trap;
void *halVec09_cxt_ptr = NULL;

void (*halVec10_fcn_ptr)(void *) = hal_isr_trap;
void *halVec10_cxt_ptr = NULL;

void (*halVec11_fcn_ptr)(void *) = hal_isr_trap;
void *halVec11_cxt_ptr = NULL;

void (*halVec12_fcn_ptr)(void *) = hal_isr_trap;
void *halVec12_cxt_ptr = NULL;

void (*halVec13_fcn_ptr)(void *) = hal_isr_trap;
void *halVec13_cxt_ptr = NULL;

void (*halVec14_fcn_ptr)(void *) = hal_isr_trap;
void *halVec14_cxt_ptr = NULL;

void (*halVec15_fcn_ptr)(void *) = hal_isr_trap;
void *halVec15_cxt_ptr = NULL;


/***************************************************************************************
 * FUNCTION PURPOSE: Route an interrupt callback
 ***************************************************************************************
 * DESCRIPTION: Routes the specified interrupt as specified. Returns
 *              a non-zero value if the interrupt could not be routed.
 ***************************************************************************************/
tint hal_route_interrupt (tint source, void (*fcn)(void *), void *cxt)
{
  tint vec;
#ifdef GG_BIOS
  Hwi_Params hwiParams;
  Hwi_Handle hwi;
#endif  
  /* Run the source through the csl to determine which vector is associated
   * with the interrupt. The return value is the integer number of the interrupt,
   * -1 if the source is not legal */
  vec = cslGetVecNum (source);

  switch (vec)  {

#ifdef GG_HWCFG_VEC04_ENABLED
    case 4:  halVec04_fcn_ptr = fcn;
             halVec04_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC05_ENABLED
    case 5:  halVec05_fcn_ptr = fcn;
             halVec05_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC06_ENABLED
    case 6:  halVec06_fcn_ptr = fcn;
             halVec06_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC07_ENABLED
    case 7:  halVec07_fcn_ptr = fcn;
             halVec07_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC08_ENABLED
    case 8:  halVec08_fcn_ptr = fcn;
             halVec08_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC09_ENABLED
    case 9:  halVec09_fcn_ptr = fcn;
             halVec09_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC10_ENABLED
    case 10: halVec10_fcn_ptr = fcn;
             halVec10_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC11_ENABLED
    case 11: halVec11_fcn_ptr = fcn;
             halVec11_cxt_ptr = cxt;
             break;
#endif


#ifdef GG_HWCFG_VEC12_ENABLED
    case 12: halVec12_fcn_ptr = fcn;
             halVec12_cxt_ptr = cxt;
             break;
#endif


#ifdef GG_HWCFG_VEC13_ENABLED
    case 13: halVec13_fcn_ptr = fcn;
             halVec13_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC14_ENABLED
    case 14: halVec14_fcn_ptr = fcn;
             halVec14_cxt_ptr = cxt;
             break;
#endif

#ifdef GG_HWCFG_VEC15_ENABLED
    case 15: halVec15_fcn_ptr = fcn;
             halVec15_cxt_ptr = cxt;
             break;
#endif


    default: return (-1);

  }

#ifdef GG_BIOS
  Hwi_Params_init(&hwiParams);

  /* Use MaskingOption_ALL to guarantee that interrupts cannot re-enter themselves, 
   * nor be interrupted by other interrupts */
  hwiParams.maskSetting = ti_sysbios_interfaces_IHwi_MaskingOption_ALL;
  hwiParams.enableInt = FALSE;
  hwiParams.arg = (xdc_UArg) cxt;

  /* Create an HWI instance object if it does not exist. For previously-created Hwi
   * instance objects, just do reconfiguration */
  hwi = Hwi_getHandle(vec);
  if (hwi == NULL) {
    Hwi_create(vec, (Hwi_FuncPtr)fcn, &hwiParams, NULL);
  } else {
    Hwi_reconfig(hwi, (Hwi_FuncPtr)fcn, &hwiParams);
  }
#endif

  return (0);

} /* hal_route_interrupt */
      

    
/********************************************************************************
 * FUNCTION PURPOSE: Change the interrupt level, return current interrupt level
 ********************************************************************************
 * DESCRIPTION: Reads and returns the current interrupt level. Modifies the
 *              current interrupt level.
 *********************************************************************************/
halIntrLevel_t halIntrLevel (halIntrLevel_t newLevel)
{

  cslIntContext_t cxt;
  tuint ilevel;

  /* Ask the csl to read the current interrupt status. It will return 
   * a context that is required to be returned after modifiing the
   * global enable bit */
  ilevel = cslIntChange (&cxt);

  /* Set the desired interrupt level */
  switch (newLevel)  {
    case HAL_INT_LEVEL_DISABLE:
      cslGlobDisableInt(); 
      break;
    case HAL_INT_LEVEL_ENABLE:
      cslGlobEnableInt(); 
      break;
    /* default case is to do nothing */
  }

  /* Restore the original interrupt mask registers */
  cslIntRestore (&cxt);

  /* Return the original global interrupt enable level */
  if (ilevel)
    return (HAL_INT_LEVEL_ENABLE);
  else
    return (HAL_INT_LEVEL_DISABLE);

} /* halIntrLevel */



