/*************************************************************************************
 * FILE PURPOSE: HAL level EDMA/ISR driver local definitions
 *************************************************************************************
 * FILE NAME: haledloc.h
 *
 * DESCRIPTION: Implements the HAL api memory move functions using the emda and isr.
 *
 * Copyright (C) 2006 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "hal.h"
#include "gghwcfg.h"
#include <ti/drv/edma3/drv/edma3_drv.h>

#ifdef GG_DMA_INCLUDE_INTC
#include <ti/csl/csl_cpIntcAux.h>
#endif

enum {
  HAL_EDMA_NO_LINK = -2,
  HAL_EDMA_END_LINK = -1
};
/*************************************************************************************
 * The EDMA context. The context consists of a variable number of edma channels.
 * multiple channels can be active at any time.
 *************************************************************************************/
typedef struct halEdmaChan_s  {

  tint channel;     /* The edma channel number */
  tint linkChannel; /* The next channel for a multi transfer request */
  tint status;      /* free (0) or busy (~0)   */
  
  halMemMoveComplete_t callBack;  /* Function to call when transfer is complete */
  void *cdata;                    /* Argument to function call back             */
  
} halEdmaChan_t;


typedef struct halEdmaSeq_s
{
  tint chainStatus;     /* Chain is free or in use */
  tint startChanIndex;  /* Index to the first channel in the chain */
  tint endChanIndex;    /* Index to the last channel in the chain  */
  
} halEdmaSeq_t;

typedef struct halEdmaContext_s  {

  EDMA3_DRV_Handle hEdma;
  
  #ifdef GG_DMA_INCLUDE_INTC
  CSL_CPINTC_Handle hIntc;
  CSL_CPINTCSystemInterrupt sysInt;
  #endif

  halEdmaChan_t halEdmaChan[GG_DMA_CHANNELS_PER_CORE];
  
  halEdmaSeq_t sequence[GG_DMA_CHANNELS_PER_CORE];
  
} halEdmaContext_t;

enum {
  HAL_EDMA_CHANNEL_FREE,
  HAL_EDMA_CHANNEL_BUSY,
  HAL_EDMA_CHANNEL_IN_CHAIN,
  HAL_EDMA_CHANNEL_END_CHAIN,
  HAL_EDMA_CHANNEL_WAITING_CALLBACK
};

enum {
  HAL_EDMA_SEQUENCE_FREE,
  HAL_EDMA_SEQUENCE_BUSY
};


/* Prototypes */
void halEdmaMemIsr (void *vContext);
void hal_edma_release_chain (halEdmaContext_t *context, halEdmaSeq_t *seq);
bool halEdmaGetIntStatus (tint channel);
