#ifndef _HALPORT_H
#define _HALPORT_H
/******************************************************************************
 * FILE PURPOSE: HAL Portability Definitions (C55x version)
 ******************************************************************************
 * FILE NAME:   halport.h
 *
 * DESCRIPTION: Contains HAL portability definitions.   
 *
 * TABS: NONE        
 *
 * $Id$
 *
 * REVISION HISTORY:
 *
 * $Log$
 * Initial revision
 *
 * (C) Copyright 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "const.h"
#include "types.h"	 
#include "hal.h"
#include "gghwcfg.h"

/* macro to even align internal carving of memory */
#define HAL_ALIGN_STRUCT(x)       ((x) = (x+7)&(~7))
 
/* Pragma's for code overlays */
#ifndef DEFAULT_CODE_SECTION
#pragma CODE_SECTION (halGetSizes,        ".progOverlay");
#pragma CODE_SECTION (halNew,             ".progOverlay");
#pragma CODE_SECTION (halPktGetHeapSize,  ".progOverlay");
#pragma CODE_SECTION (halPktGetTxBufInfo, ".progOverlay");
#pragma CODE_SECTION (halPktGetRxBufInfo, ".progOverlay");
#pragma CODE_SECTION (halPktGetCaBufInfo, ".progOverlay");
#pragma CODE_SECTION (halPktInitStart,    ".progOverlay");
#pragma CODE_SECTION (halStartupUl2,      ".progOverlay");
#pragma CODE_SECTION (set_ul2_params,     ".progOverlay");
#pragma CODE_SECTION (utopiaInit,         ".progOverlay");
#pragma CODE_SECTION (utopiaDevReset,     ".progOverlay");
#pragma CODE_SECTION (utopiaRcvCells,     ".progOverlay");

#pragma CODE_SECTION (halGmacGetHeapSize,     ".progOverlay");
#pragma CODE_SECTION (halGmacGetTxBufInfo,     ".progOverlay");
#pragma CODE_SECTION (halGmacGetRxBufInfo,     ".progOverlay");
#pragma CODE_SECTION (halGmacBufAlloc,     ".progOverlay");
#pragma CODE_SECTION (halGmacInit,     ".progOverlay");
#pragma CODE_SECTION (halGmacInitStart,     ".progOverlay");

#if (defined(GG_HAL_INCLUDE_TDM) || defined(TSIP_LLD))  
#pragma CODE_SECTION (halTdmResourceInit, ".progOverlay"); 
#pragma CODE_SECTION (halTdmGetHeapSize,  ".progOverlay");
#pragma CODE_SECTION (halTdmGetDmaBufInfo,".progOverlay");
#pragma CODE_SECTION (halTdmMcBspInit,    ".progOverlay");
#pragma CODE_SECTION (halTdmAllocAndInit, ".progOverlay");
#pragma CODE_SECTION (halStartupTdm,      ".progOverlay");
#pragma CODE_SECTION (halTdmInitStart,    ".progOverlay");
#pragma CODE_SECTION (halTsipGetSharedHeapSize, ".progOverlay"); 
#pragma CODE_SECTION (halTsipSharedInit, ".progOverlay"); 
#pragma CODE_SECTION (tsip_shared_buf_alloc, ".progOverlay"); 
#pragma CODE_SECTION (hal_tsip_config, ".progOverlay"); 
#pragma CODE_SECTION (halTsipGetHeapSize, ".progOverlay"); 
#pragma CODE_SECTION (hal_tsip_get_dma_mem_usage, ".progOverlay"); 
#pragma CODE_SECTION (halTsipGetDmaBufInfo, ".progOverlay"); 
#pragma CODE_SECTION (halTsipGetDmaBufInfo, ".progOverlay"); 
#pragma CODE_SECTION (halTsipResourceInit, ".progOverlay"); 
#pragma CODE_SECTION (halTsipInitStart, ".progOverlay"); 
#pragma CODE_SECTION (halTdmTsipInit, ".progOverlay"); 
#pragma CODE_SECTION (halTsipAllocAndInit, ".progOverlay"); 
#endif
  
#pragma CODE_SECTION (cslEdmaIsr,    ".progIsr");
#pragma CODE_SECTION (cslIntA,       ".progIsr");
#pragma CODE_SECTION (halTdmEdmaIsr, ".progIsr");
#pragma CODE_SECTION (halTdmDmaIsr,  ".progIsr");

#pragma CODE_SECTION (halCacheControl,".controlProg");
#pragma CODE_SECTION (cslIntA,       ".initProg");
#pragma CODE_SECTION (halTdmEdmaIsr, ".progIsr");
#pragma CODE_SECTION (halTdmDmaIsr,  ".progIsr");



#pragma CODE_SECTION (csl_gmac_intctl_set,".progExt")
#pragma CODE_SECTION (cslGmacEmicWrapperInit,".progExt")
#pragma CODE_SECTION (cslGmacEmicTxChEnable,".progExt")
#pragma CODE_SECTION (cslGmacEmicRxChEnable,".progExt")
#pragma CODE_SECTION (cslGmacEmicResetTx,".progExt")
#pragma CODE_SECTION (cslGmacEmicResetRx,".progExt")
#pragma CODE_SECTION (csl_enable_interrupt,".progExt")
#pragma CODE_SECTION (csl_disable_interrupt,".progExt")
#pragma CODE_SECTION (csl_clear_interrupt,".progExt")
#pragma CODE_SECTION (cslIsrInit,".progExt")
#pragma CODE_SECTION (cslInstallIsrTable,".progExt")
#pragma CODE_SECTION (cslGmacChannelOpenTx,".progExt")
#pragma CODE_SECTION (cslGmacChannelOpenRx,".progExt")
#pragma CODE_SECTION (cslCpmacReadPeriphId,".progExt")
#pragma CODE_SECTION (cslCpmacIsEnabled,".progExt")
#pragma CODE_SECTION (cslCpmacHwInit,".progExt")
#pragma CODE_SECTION (cslEdmaMemToMemSetup,".progExt")
#pragma CODE_SECTION (cslEdmaInit,".progExt")
#pragma CODE_SECTION (halUtopiaQuery,".progExt")
#pragma CODE_SECTION (halUtopiaQuery,".progExt")
#pragma CODE_SECTION (halMcBspQuery,".progExt")
#pragma CODE_SECTION (halTsipControl,".progExt")
#pragma CODE_SECTION (cslTsipSetBaseAndAlloc,".progExt")
#pragma CODE_SECTION (cslTsipNewTxContext,".progExt")
#pragma CODE_SECTION (cslTsipNewRxContext,".progExt")
#pragma CODE_SECTION (cslTsipEnableTxChannel,".progExt")
#pragma CODE_SECTION (cslTsipEnableTimeslot,".progExt")
#pragma CODE_SECTION (cslTsipEnableRxChannel,".progExt")
#pragma CODE_SECTION (cslTsipEnablePort,".progExt")
#pragma CODE_SECTION (cslTsipCreateOffsetMaps,".progExt")
#pragma CODE_SECTION (cslTsipConfigPort,".progExt")
#pragma CODE_SECTION (cslTsipClearChannel,".progExt")
#pragma CODE_SECTION (halWdTimerControl,".progExt")
#pragma CODE_SECTION (cslRtConfigureEmif,".progExt")
#pragma CODE_SECTION (cslConfigureEmif,".progExt")
#pragma CODE_SECTION (cslReset,".progExt")
#pragma CODE_SECTION (cslPllControl,".progExt")
#pragma CODE_SECTION (cslDeviceCoreReset,".progExt")
#pragma CODE_SECTION (cslDeviceCoreBoot,".progExt")
#pragma CODE_SECTION (cslChipRevision,".progExt")
#pragma CODE_SECTION (halGmacControl,".progExt")
#pragma CODE_SECTION (halEdIsrInit,".progExt")
#pragma CODE_SECTION (halEdIsrGetSizes,".progExt")
#pragma CODE_SECTION (hal_route_interrupt,".progExt")
#pragma CODE_SECTION (cslPscDelay,".progExt")
#pragma CODE_SECTION (cslPscWait,".progExt")
#pragma CODE_SECTION (cslPowerStateSet,".progExt")
#pragma CODE_SECTION (cslPowerStateGet,".progExt")
#pragma CODE_SECTION (hal_isr_trap,".progExt")
#pragma CODE_SECTION (halCacheControl,".progExt")
#pragma CODE_SECTION (halCacheConfig,".progExt")
#pragma CODE_SECTION (cslSetMacAddr,".progExt")
#pragma CODE_SECTION (cslGmacChannelOpenTx,".progExt")
#pragma CODE_SECTION (cslGmacChannelOpenRx,".progExt")
#pragma CODE_SECTION (cslCpmacReadPeriphId,".progExt")
#pragma CODE_SECTION (cslCpmacIsEnabled,".progExt")
#pragma CODE_SECTION (cslCpmacHwInit,".progExt")
//#pragma CODE_SECTION (halMemConfig,".progExt")

#pragma CODE_SECTION (halTsipQuery,".controlProg")
#pragma CODE_SECTION (halTsEncQuery,".controlProg")
#pragma CODE_SECTION (halTsDecQuery,".controlProg")
#pragma CODE_SECTION (halMacQuery,".controlProg")
#pragma CODE_SECTION (halCoreQuery,".controlProg")
#pragma CODE_SECTION (halPgQuery,".controlProg")
#pragma CODE_SECTION (halQuery,".controlProg")
#pragma CODE_SECTION (halTimerControl,".controlProg")
#endif

/* Prototypes */
halGlobAddr_t halMapLocal2Global (halLocAddr_t addr);
halGlobAddr_t halFormGlobAddr (halLocAddr_t addr, tint globalId);

tint halMemMoveChain (tint nXfers, halMemMove_t *xfers, halMemMoveComplete_t complete,
                          void *data, bool stallToCompletion);
halReturn_t halMemMove (halGlobAddr_t dest, halGlobAddr_t src, halSAE_t size,
                            halMemMoveComplete_t complete, void *data);
halReturn_t halMemSet (halGlobAddr_t addr, tulong sizeBytes, tulong v, tint tsizeBytes);

tint halMMStatus (tint handle);

#endif  /* _HALPORT_H */

/* nothing past this point */
