/******************************************************************************/
/*                                                                            */
/*     Copyright (C) 2001  Texas Instruments Incorporated.                    
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                         */
/******************************************************************************/
/*                                                                            */
/* File Name  : cpuutils.c                                                    */
/*                                                                            */
/* Description: Basic Utilities for Shannon                                   */
/*                                                                            */ 
/* GLOBAL FUNCTIONS:                                                          */
/*          whoAmI (void) ()                                                  */
/*                                                                            */
/* STATIC FUNCTIONS :   None                                                  */
/*                                                                            */
/*                                                                            */
/* Date : 17th August 2002                 OWNERSHIP : TARAKESH               */
/******************************************************************************/

/************************************************************************/
/************* Include Files ********************************************/
/************************************************************************/
#include "c64xint.h"
#include "types.h"
#include "cslintc.h"
#include "hal.h"
#include "cslpower_api.h"
#include "device.h"
#include <stdlib.h>
#include "halloc.h"
#include "cslisr.h"
#include "Timer64x.h"

/******************************************************************************/
/* LOCAL DEFINES                                                              */
/******************************************************************************/

/******************************************************************************/
/* FILE LOCAL (STATIC) VARIABLES                                              */
/******************************************************************************/

/******************************************************************************/
/* FILE LOCAL (STATIC) PROTOTYPES                                             */
/******************************************************************************/
/********************* Public functions **************************************/

/************************************************************************/
/* unsigned short int whoAmI(void)                                      */
/*                                                                      */
/* Description : This function returns CPU ID and revision ID           */
/*                                                                      */
/* Return      : SUBID value of Sub System.                             */
/*                                                                      */
/* This function returns the core ID of a DSP in multi-core device      */
/*                                                                      */
/************************************************************************/

int whoAmI ()
{
  return(DNUM);
//    return(0);
} /* End of whoAmI() */

/****************************************************************************
 * FUNCTION PURPOSE: Place all peripherals into a known state
 ****************************************************************************
 * DESCRIPTION: Called at startup to insure that all peripherals are in
 *              a known state on the 5510.
 ***************************************************************************/

void cslReset (void)
{

  /***************************************************************************
   *       INTERRUPT VECTOR Initialization                                   *
   ***************************************************************************/
  /* Add the Initilization of IVPD and IVPH Here                             */
  /* This function will set the interrupts for the SuperFrame Interrupts     */
  cslInstallIsrTable();

} /* cslReset */

/****************************************************************************
 * FUNCTION PURPOSE: Return the chip revision in q4 format. 
 ****************************************************************************
 * DESCRIPTION:
 *            
 ****************************************************************************/
tuint cslChipRevision (void)
{
  tuint revID  = CSL_FEXT(CSR, CHIP_CSR_REV_ID);
  tuint chipID = CSL_FEXT(CSR, CHIP_CSR_CPU_ID);
  return revID;
}


/*******************************************************************************
 * FUNCTION PURPOSE: Release a core from reset
 *******************************************************************************
 * DESCRIPTION: Sets the boot complete bit if the core has never been out of
 *              reset. Otherwise it releases a local reset.
 *******************************************************************************/
tint cslDeviceCoreBoot (tint core, tulong addr)
{
  /* Check for a valid core number */
  if ((core < 0) || (core >= DEVICE_NCORES))
    return (-1);

  /* Write the entry point address to the boot address location */
  CSL_BOOT_ADDR_REG32_WRITE(core, addr);


  /* Invoke IPC to wake up the core */
     CSL_IPC_WRITE(core, 1);

  return (0);

} /* cslDeviceCoreBoot */
  

/****************************************************************************
 * FUNCTION PURPOSE: Put a core into reset
 ****************************************************************************
 * DESCRIPTION: Uses the psc to assert LRSTz
 ****************************************************************************/
tint cslDeviceCoreReset (tint core)
{
  /* Check for a valid core number */
  if ((core < 0) || (core >= DEVICE_NCORES))
    return (-1);

  cslPowerStateSet (core, CSL_POWER_STATE_RESET);

  return (0);

} /* cslDeviceCoreReset */


/*****************************************************************************
 * FUNCTION PURPOSE: Set the CPU Pll
 *****************************************************************************
 * DESCRIPTION: The main pll is setup. The function call is to the
 *              pll function stored in the boot rom code.
 *****************************************************************************/
tint cslPllControl (tuint mult, tuint div)
{
  /* Call the romd function for pll 0, prediv = 1 */
//  hwPllSetPll (0, 1, mult, div);

  return 0;
} /* cslPllControl */


/*****************************************************************************
 * FUNCTION PURPOSE: Return the PG version
 *****************************************************************************
 * DESCRIPTION: Reads the boot rom signature
 *****************************************************************************/
tint readPgVer (void)
{
  /*stub for shannon*/
  return 0;
} /* readPgVer */

/******************************************************************************
 * FUNCTION PURPOSE: Perform an idle
 ******************************************************************************
 * DESCRIPTION: The core executes an idle. An interrupt wakes the core. The
 *              interrupt vector table should contain only a B IRP so no
 *              interrupt processing is done.
 ******************************************************************************/
void cslDeviceSleep (halCmdWakeOn_t wakeon)
{
  tulong oldier;
  halIntrLevel_t level;


  if (wakeon == HAL_COMMAND_SLEEP_WAKE_OCORE_INT)  {
    oldier = IER;
    IER    = 0;
  }

  /* Enable interrupts from other cores */
  csl_clear_interrupt (CSL_GEM_IPC_LOCAL);
  csl_enable_interrupt (CSL_GEM_IPC_LOCAL);

  level = halIntrLevel (HAL_INT_LEVEL_ENABLE);

  asm ("   idle");

  halIntrLevel (level);

  if (wakeon == HAL_COMMAND_SLEEP_WAKE_OCORE_INT)  {
    IER = oldier;
  }


} /* cslDeviceSleep */


/******************************************************************************
 * FUNCTION PURPOSE: Send an interrupt to another core
 ******************************************************************************
 * DESCRIPTION: The IPC is used to interrupt the requested cores.
 ******************************************************************************/
halReturn_t cslDeviceIntOcore (halCtlIntCore_t *icore)
{
  tuint i;

  for (i = 0; i < DEVICE_NCORES; i++)  {

    if ((icore->coreMask & (1 << i)))  {

      DEVICE_IPCGR(i) = 1;

    }

  }

  return (HAL_SUCCESS);

} /* cslDeviceIntOcore */


/********************************************************************************
 * FUNCTION PURPOSE: Send an interrupt to the host
 ********************************************************************************
 * DESCRIPTION: IPCGR15 is used to send an interrupt to the host
 ********************************************************************************/
void cslDeviceIntHost (void)
{
  DEVICE_IPCGR(15) = 1;

} /* cslDeviceIntHost */


/********************************************************************************
 * FUNCTION PURPOSE: Return the memory type based on address
 ********************************************************************************
 * DESCRIPTION: Memory is classified (L1P, L2D, etc) based on address
 ********************************************************************************/
tint cslDeviceGetMemType (tulong addr)
{

  if (addr >= 0x80000000)
    return (DEVICE_MEM_DDR);

  addr = addr & 0x00ffffff;

  if ((addr >= 800000) && (addr < 0x880000))
    return (DEVICE_MEM_L2);

  if ((addr >= 0xc000000) && (addr < 0xc040000))
    return (DEVICE_MEM_SL2);


  if ((addr >= 0x00e00000) && (addr < 0x00e08000))
    return (DEVICE_MEM_L1P);

  if ((addr >= 0x00f00000) && (addr < 0x00f08000))
    return (DEVICE_MEM_L1D);

  return (DEVICE_MEM_UNKNOWN);

} /* cslDeviceGetMemType */


