#ifndef _DEVICE_H
#define _DEVICE_H
/***********************************************************************************
 * FILE PURPOSE: Define the core and peripherals for Shannon
 ***********************************************************************************
 * FILE NAME: cslthwk.h
 *
 * DESCRIPTION: Defines the shannon memory map, cores, and peripherals
 *
 * Copyright (C) 2005, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
 
#include "ti/mas/types/types.h"
#include "ti/csl/cslr_device.h" /* TI device CSL */
#include "ti/csl/cslr_chip.h"
#include "ti/csl/cslr_cgem.h"

extern cregister volatile unsigned int DNUM;
extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int TSCH;

/***********************************************************************************
 * Shannon has TSIP for TDM
 ***********************************************************************************/
#define CSL_DEVICE_TDM_IS_TSIP
#define CSL_DEVICE_TSIP_MAX_FRAME_ALLOC_SIZE 1024  /* the max frame is always 1024 bytes */

/*********************************************************************************
 * The number of cores on the device
 *********************************************************************************/
#define DEVICE_NCORES   8

 /*********************************************************************************
  * Tomahawk does support core reset and boot
  *********************************************************************************/
#define DEVICE_CORE_BOOT
#define DEVICE_CORE_RESET
tint cslDeviceCoreBoot (tint core, tulong addr);
tint cslDeviceCoreReset (tint core);

enum
{
  DEVICE_MEM_DDR = 0x100,
  DEVICE_MEM_L2,
  DEVICE_MEM_SL2,
  DEVICE_MEM_L1P,
  DEVICE_MEM_L1D,
  DEVICE_MEM_UNKNOWN
};

/************************************************************************************
 * Inter processor interrupts
 ************************************************************************************/
#define DEVICE_IPCGR_BASE  0x2620240
#define DEVICE_IPCGR(x)    *((volatile unsigned int *)(DEVICE_IPCGR_BASE + ((x)*4)))

#define DEVICE_IPCAR_BASE  0x2620280
#define DEVICE_IPCAR(x)    *((volatile unsigned int *)(DEVICE_IPCAR_BASE + ((x)*4)))

#define CSL_IPC_READ()           DEVICE_IPCGR(DNUM)
#define CSL_IPC_WRITE(core, v)   DEVICE_IPCGR(core) = (v)
#define CSL_IPC_AR_WRITE(x)      DEVICE_IPCAR(DNUM) = (x)
#define CSL_IPC_CORE_READ(x)     DEVICE_IPCGR((x))
#define CSL_IPC_AR_CORE_WRITE(core, v)  DEVICE_IPCAR((core)) = (v)

/* Interrupt controller access are memory mapped */
#define CSL_INTCTL_ADDR(offset) (&((CSL_CgemRegs *)CSL_CGEM0_5_REG_BASE_ADDRESS_REGS)->INTMUX1 + (offset - CSL_GEM_INTCTL_REG_A_MUX1)/sizeof(Uint32))
#define CSL_INTCTL_REG32_READ(offset)  \
            *CSL_INTCTL_ADDR(offset)
            
#define CSL_INTCTL_REG32_WRITE(offset, value)  \
            *CSL_INTCTL_ADDR(offset) = (value)
            
/***************************************************************************************
 * Local/Global memory address conversion
 ***************************************************************************************/
#define CSL_LOCAL_TO_GLOBAL(core, localAddr)     ((1<<28)|(core<<24)|((localAddr)&0x00ffffff))
#define CSL_GLOBAL_TO_LOCAL(globAddr)            (globAddr & 0x00ffffff)
#define CSL_GLOBAL_EXTRACT_CORE(globAddr)        ((globAddr >> 24) & 0xf)

/************************************************************************************
 * Boot controller registers
 ************************************************************************************/
#define DEVICE_BOOT_ADDR_BASE   0x87FFFC
static inline void *restrict GLOBAL_ADDR_BOOT_ADDR(int core_id)
{  /* Convert single local to global addresses */
   return (void *)((1 << 28) | (core_id << 24) | DEVICE_BOOT_ADDR_BASE);
}

#define CSL_BOOT_ADDR_REG32_WRITE(x,v) \
               *((volatile unsigned int *)(GLOBAL_ADDR_BOOT_ADDR(x))) = (v)

/***************************************************************************************
 * Prototypes
 ***************************************************************************************/
tint cslPllControl (tuint mult, tuint div);
tint cslDeviceGetMemType (tulong addr);

#endif /* _DEVICE_H */
