/******************************************************************************/
/*                                                                            */
/*     Copyright (C) 2005  Texas Instruments Incorporated.          
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                         */
/******************************************************************************/
/*                                                                            */
/* Description: Driver code to initialise EMIF                                */
/*                                                                            */
/* GLOBAL FUNCTIONS:                                                          */
/*                                                                            */
/*   void cslConfigureEmif ();                                                */
/*                                                                            */
/* STATIC FUNCTIONS :   None                                                  */
/*                                                                            */
/* TABS: NONE                                                                 */
/*                                                                            */
/******************************************************************************/


/******************************************************************************
 * Include Files 
 ******************************************************************************/

#include "types.h"
#include "gghwcfg.h"
#include "device.h"
#include "hal.h"
#include "emifDdr3.h"
#include <ti/csl/csl_pllc.h>

/******************************************************************************
 * FUNCTION PURPOSE: To configure EMIF register
 ******************************************************************************
 * DESCRIPTION: This rountine configures DDR EMIF
 *
 * void cslConfigureEMIF()
 * 
 *****************************************************************************/
void cslConfigureEmif()
{
  /* Setup the EMIF of DDR3 for deafult mode operation */
  halDdr3Setup(WORD_SZ_32);

}/*cslConfigureEmif () */


/******************************************************************************
 * FUNCTION PURPOSE: add software delay
 ******************************************************************************
 * DESCRIPTION: This rountine adds software delay
 *
 * void swDelay(int nloop)
 * 
 *****************************************************************************/
void swDelay(int nloop)
{
    volatile int i;

    for( i = 0 ; i < nloop ; i++ )
    {
    }
}

/******************************************************************************
 * FUNCTION PURPOSE: Setup PLL 1 for DSP clock to external clock source with 
 *                   the specified
 ******************************************************************************
 * DESCRIPTION: This rountine configures PLL1 for DSP
 *
 * void cslSetDSPPLL(int pll_multiplier)
 * 
 *****************************************************************************/
#define PLL_REG_RSCFG_FIELD_POWER_ON_RESET       (1<<29)
#define PLL1_SECCTL         (0x02310000 + 0x108)
void cslSetDSPPLL(int pll_multiplier)
{
    CSL_PllcHandle hPllc;
    unsigned int* pll_secctl    = ( unsigned int* )PLL1_SECCTL; /*Not defined in CSL*/
    /* Default dividers */
    unsigned int div2=3, div5=5, div8=64;  

    hPllc = CSL_PLLC_open(0);

    /* Only core0 sets PLL*/
    if (DNUM == 0)
    {
        /* Verify if multiplier is ok...*/
        if (pll_multiplier>0 && pll_multiplier<=64)
        {
   
            /*Unlock Kicker mechanism in case it is locked */
            CSL_BootCfgUnlockKicker();	
            
            /* config reset control (isolation) */
            hPllc->RSTCTRL = CSL_PLLC_RSTCTRL_VALID_KEY;

            hPllc->RSTCFG |= PLL_REG_RSCFG_FIELD_POWER_ON_RESET;

            *pll_secctl = 0x00890000;
            
            /*  Step 1: Set PLL to BYPASS mode */
            hPllc->PLLCTL &= 0xFFFFFFDF; 
            hPllc->PLLCTL &= 0xFFFFFFFE;

            /* Wait Bypass mode switch, bypass sw time is 4 clkin cycles
               The following delay is much more than necessary...       */
            swDelay(150);            
            hPllc->PLLCTL |= 0x2;
            swDelay(150);
            hPllc->PLLCTL &= ~0x2;
            swDelay(150);

            /*  Step 2: Configure and stabilize PLL */
            hPllc->PLLCTL |= 0x8;                    /* Reset PLL */
    
            /* Verify if pll is in power down */
            if ((hPllc->PLLCTL & 0x00000002) !=0 )
            {
                hPllc->PLLCTL &= 0xFFFFFFFD;         /* Power up PLL */            
                /* Wait PLL Stabilization time that is 150 usec. The following delay
                   is much more than necessary and provide stable PLL... */
                swDelay(5000);
            }

            /* Step 3: Set PLL multiplier (minus 1 desired value) 
               (multiplier is splitted in 2 parts) */
            /* Set PLL multipler LSB */
        	hPllc->PLLM = pll_multiplier*2 - 1;
        	
            /* Set PLL multipler MSB */
        	hBootCfg->CORE_PLL_CTL0 |= (pll_multiplier << 23) & 0xFF000000;
        	hBootCfg->CORE_PLL_CTL0 &= (pll_multiplier << 23) | 0x00FFFFFF;
       	
            /* Wait for GOSTAT to be cleared so no go operation is in progress */
            while((hPllc->PLLSTAT & 0x01) !=0 ) swDelay(150);
    
            /* Step 4. Set PLL dividers if needed */
            hPllc->PLLDIV1_3[1] = (0x8000) | (div2 - 1);
            hPllc->PLLDIV4_16[1] = (0x8000) | (div5 - 1);
            hPllc->PLLDIV4_16[4] = (0x8000) | (div8 - 1);
            
            /* Adjust modified related sysclk align */
            hPllc->ALNCTL = hPllc->DCHANGE;
            
            /* Gives the GO cmd */
            hPllc->PLLCMD |= 0x00000001;
    
            /* Wait for phase alignment */
            while((hPllc->PLLSTAT & 0x01) !=0 )  swDelay(150);
    
            /* Step 5: Wait for PLL to lock */
            /* Wait for PLL to Reset. Reset time =128C */
            swDelay(1000);
            
            hPllc->PLLCTL &= 0xFFFFFFF7;             /* Release PLL from Reset */
    
            /* Wait for PLL to LOCK. Lock time =2000C */
            swDelay(4000);
            
            *pll_secctl = 0x00090000;
            hPllc->PLLCTL = 0x00000041;             /* Set PLL to PLL mode */

            /* Leave BootCfg area unlocked */
            //CSL_BootCfgLockKicker();
        }
    }
} /* cslSetDSPPLL () */

tulong cslDdr3DataGtlvlInitRatio[9] = 
{
    0x3C,
    0x3C,
    0x23,
    0x2D,
    0x13,
    0x11,
    0x9,
    0xC,
    0x21
};

tulong cslDdr3RdwrInitRatio[9] = 
{
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0
};



/**********************************************************************************
 * FUNCTION PURPOSE: Configure DDR based on Run time configuration
 **********************************************************************************
 * DESCRIPTION: The host has provided run time memory DDR3 configuration.
 *              Load the configuration, and test the memory if requested.
 *
 *              Will return a failure only if the memory test fails.
 **********************************************************************************/
halReturn_t cslRtConfigureEmif (struct MGB_MDS_EMIF_SDRAM *ecfg)
{

#warn need to enhance this function to use EMIF_SDRAM with ddr_type = DDR3
#warn need to add DDR3 memory test 
  volatile tulong v;
#ifdef GG_HWCFG_INCLUDE_DDR_CHECK
  bool setPll = FALSE;
#endif

  int i;
  volatile unsigned int loopCount;
  
  /* Update the system pll if required */
  if (ecfg->valid_params & MGB_SDRAM_CPU_PLL)  {

    cslSetDSPPLL (ecfg->pll);  /* TBD: should we use ROM function instead */
#ifdef GG_HWCFG_INCLUDE_DDR_CHECK
    setPll = TRUE;
#endif
  }

#if 0 /* This code uses host parameters which are incomplete; restore
       * if host parameters are defined */
  /*
   * Update the DDR PLL is required
   *  -  Unlock Boot Cfg mechanism
   *  -  Configure Boot Cfg register DDR3PLLCTL0 for DDR_PLL updates
   *  -  Lock Mechanism to prevent further Boot Cfg writes
   */
   
   
  /*Unlock Kicker mechanism in case it is locked */
  CSL_BootCfgUnlockKicker();	
   
  /*Configure DDR3-PLL*/
  if (ecfg->valid_params & MGB_SDRAM_DDR_PLL)  {
  
    hBootCfg->DDR3_PLL_CTL0 &= ~0x00800000;
    hBootCfg->DDR3_PLL_CTL0 = (ecfg->ddr_pll*2) << 24  |
                              (ecfg->ddr_pll*4 - 1) << 6 |
                              0x00080001;
  }                          
  swDelay(3000);
  /* It is assumed that the XMC MPAX is already configured to map
   * CSL_DDR3_EMIF_CONFIG_REGS to 0x1 0000 0000.
   */

  /*
   *  CSL API calls to configure EMIF4F registers with desired parameters
   */
   
  /* Timing 1 register */
  if (ecfg->valid_params & MGB_SDRAM_TIMING1)  {
    hEmif->SDRAM_TIM_1 = ((tulong)ecfg->timing1Msw << 16) | (tulong)ecfg->timing1Lsw;
  }
  
  /* Timing 2 register */
  if (ecfg->valid_params & MGB_SDRAM_TIMING2)  {
    hEmif->SDRAM_TIM_2 = ((tulong)ecfg->timing2Msw << 16) | (tulong)ecfg->timing2Lsw;
  }

  /* Timing 3 register */
  if (ecfg->valid_params & MGB_SDRAM_TIMING3)  {
    hEmif->SDRAM_TIM_3 = ((tulong)ecfg->timing3Msw << 16) | (tulong)ecfg->timing3Lsw;
  }

  /* Set up SR_TIM to Enter self-refresh after 4096 clocks */
  hEmif->PWR_MGMT_CTRL |= (0x9 << 4); 
  swDelay(1000);

  /* Configure DDR_PHY_CTRL1 Register using CSL */
  if (ecfg->valid_params & MGB_SDRAM_PHYC)  {
    hEmif->DDR_PHY_CTRL_1 = ((tulong)ecfg->phycMsw << 16) | (tulong)ecfg->phycLsw;
  }
  swDelay(1000);
  
  if (ecfg->valid_params & MGB_SDRAM_REFRESH)  {
  
    /* TBD:L Enable Initialization & Refresh using CSL*/
    //CSL_EMIF4F_EnableInitRefresh();
  
    /* Configure Refresh rate as 500us: 500 * 533 /16 = 0x4111 */
    hEmif->SDRAM_REF_CTRL = 0x4111; 
  }
  swDelay(1000);
       
  /* Configure SDCFG using CSL*/
  if (ecfg->valid_params & MGB_SDRAM_CONFIG)  {
    
    hEmif->SDRAM_CONFIG = ((tulong)ecfg->configMsw << 16) | (tulong)ecfg->configLsw;
  }
  swDelay(3000); 
  
  /* Configure the Data and RDWR leveling initial ratio */
  for (i = 0; i < DDR3_NUM_INIT_RATIOS; i++)
  {
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(i), cslDdr3DataGtlvlInitRatio[i]);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(i), cslDdr3RdwrInitRatio[i]);
  }  
	
  hBootCfg->DDR3_CONFIG_REG[DDR3_CONFIG_REG_0] |= 0xF;
  /* Set bit 9 = 1 to use forced ratio leveling for read DQS*/
  hBootCfg->DDR3_CONFIG_REG[DDR3_CONFIG_REG_23] |= DDR3_CONFIG_REG_23_FORCE_LEVELING; 
	
  /* TBD: DDR3_CONFIG_REG_23 = RD_DQS_SLAVE_RATIO_1066 | (WR_DQS_SLAVE_RATIO_1066 << 10) | (WR_DATA_SLAVE_RATIO_1066 << 20); */
  /* Set bit 9 = 1 to use forced ratio leveling for read DQS */
  hBootCfg->DDR3_CONFIG_REG[DDR3_CONFIG_REG_23] |= 0x00000200;
  swDelay(1000);

  /* enable full leveling */
  CSL_EMIF4F_SetLevelingRampControlInfo(TRUE, 0, 0, 0, 0);
  
  
  /* Trigger full leveling 
   * This ignores read DQS leveling result and uses ratio forced value (0x34) instead
   */
  CSL_EMIF4F_SetLevelingControlInfo(1, 0, 0, 0, 0); 
                                    
  /* Read MMR to ensure full leveling is complete */                               
  v = hEmif->SDRAM_TIM_1;
  swDelay(1000);
    
  /*Configure SDRFC with refresh rate */
  if (ecfg->valid_params & MGB_SDRAM_REFRESH)  {
  
    hEmif->SDRAM_REF_CTRL = ((tulong)ecfg->refreshMsw << 16) | (tulong)ecfg->refreshLsw; 
  }
  swDelay(3000);
  
#else 
  /* This is hardcoded DDR3 @ 1333 on C6678 EVM */
    CSL_BootCfgUnlockKicker();        

   /***************** 2.2 DDR3 PLL Configuration ************/
    hBootCfg->DDR3_PLL_CTL1      |= 0x00000040;      //Set ENSAT bit = 1
    hBootCfg->DDR3_PLL_CTL1      |= 0x00002000;      //Set RESET bit = 1
    hBootCfg->DDR3_PLL_CTL0       = 0x090804C0;       //Configure CLKR, CLKF, CLKOD, BWADJ;

    /* Wait for 5us min reset to complete  */
    loopCount = 10000;
    while (loopCount--) {
        asm("   NOP");
    }
    hBootCfg->DDR3_PLL_CTL1      &= ~(0x00002000);   //Clear RESET bit

   /* Wait for PLL to lock = min 500 ref clock cycles. 
      With refclk = 100MHz, = 5000 ns = 5us */
    loopCount = 10000;
    while (loopCount--) {
        asm("   NOP");
    }
    /**************** 3.0 Leveling Register Configuration ********************/
    /* Using partial automatic leveling due to errata */

    /**************** 3.2 Invert Clock Out ********************/
    hBootCfg->DDR3_CONFIG_REG[0] &= ~(0x007FE000);  // clear ctrl_slave_ratio field
    hBootCfg->DDR3_CONFIG_REG[0] |= 0x00200000;     // set ctrl_slave_ratio to 0x100
    hBootCfg->DDR3_CONFIG_REG[12] |= 0x08000000;    // Set invert_clkout = 1
    hBootCfg->DDR3_CONFIG_REG[0] |= 0xF;            // set dll_lock_diff to 15
    hBootCfg->DDR3_CONFIG_REG[23] |= 0x00000200;    //Set bit 9 = 1 to use forced ratio leveling for read DQS

    /*Values with invertclkout = 1 */
    /**************** 3.3+3.4 Partial Automatic Leveling ********************/
    
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(0), 0x20);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(1), 0x24);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(2), 0x3A);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(3), 0x38);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(4), 0x51);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(5), 0x5E);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(6), 0x5E);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(7), 0x5E);
    CSL_BootCfgSetDDRConfig(DDR3_RDWR_INIT_RATIO_CFG_REG(8), 0x44);

    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(0), 0xA1);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(1), 0x9E);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(2), 0xA7);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(3), 0xA9);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(4), 0xCA);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(5), 0xBE);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(6), 0xDD);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(7), 0xDD);
    CSL_BootCfgSetDDRConfig(DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(8), 0xBA);

    /*Do a PHY reset. Toggle DDR_PHY_CTRL_1 bit 15 0->1->0 */
    hEmif->DDR_PHY_CTRL_1  &= ~(0x00008000);
    hEmif->DDR_PHY_CTRL_1  |= (0x00008000);
    hEmif->DDR_PHY_CTRL_1  &= ~(0x00008000);

    /***************** 2.3 Basic Controller and DRAM configuration ************/
    hEmif->SDRAM_REF_CTRL    = 0x00005162;    // enable configuration 

    hEmif->SDRAM_TIM_1   = 0x1113783C;
    hEmif->SDRAM_TIM_2   = 0x304F7FE3;
    hEmif->SDRAM_TIM_3   = 0x559F849F;

    hEmif->DDR_PHY_CTRL_1   = 0x0010010F;
    hEmif->ZQ_CONFIG        = 0x70073214; 
    hEmif->PWR_MGMT_CTRL    = 0x0;
 
    /* New value with DYN_ODT disabled and SDRAM_DRIVE = RZQ/7 */ 
    hEmif->SDRAM_CONFIG     = 0x63062A32; 
    hEmif->SDRAM_REF_CTRL   = 0x00001450;       //Refresh rate = (7.8*666MHz]

    hEmif->RDWR_LVL_RMP_CTRL      =  0x80000000; //enable full leveling
    hEmif->RDWR_LVL_CTRL          =  0x80000000; //Trigger full leveling - This ignores read DQS leveling result and uses ratio forced value 

    /************************************************************
      Wait for min 1048576 DDR clock cycles for leveling to complete 
        = 1048576 * 1.5ns = 1572864ns = 1.57ms.
      Actual time = ~10-15 ms 
     **************************************************************/
    loopCount = 1600000;
    while (loopCount--) {
        asm("   NOP");
    }

#endif

  /* Leave BootCfg area unlocked */
  //CSL_BootCfgLockKicker();

  return HAL_SUCCESS;
} /* cslRtConfigureEmif */

/* cslemif.c Nothing Beyond this Point */

