/*****************************************************************************
 * FILE PURPOSE: Common Utility for DDR3/EMIF initialization
 *****************************************************************************
 * FILE NAME: emifDdr3.c
 *
 * DESCRIPTION: Common Utility for DDR3/EMIF initialization
 *               
 * TABS: None
 *
 * (C) Copyright 2009 Texas Instruments.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "emifDdr3.h"
#include <string.h>

/****************************************************************************************
*  Function Name    : halSetupSDTIM1
*  Description      : This function sets up SDTIM1
*
*  Parameters      : config structure
*  Return          : None
* 
****************************************************************************************/
void halSetupSDTIM1(EMIF4F_TIMING1_CONFIG  *tim1Config)
{
  /*Timing1 values for 800MHz operation*/
  tim1Config->t_rp 	= 	T_RP;
  tim1Config->t_rcd	= 	T_RCD;
  tim1Config->t_wr 	= 	T_WR;
  tim1Config->t_ras	= 	T_RAS;
  tim1Config->t_rc 	= 	T_RC;
  tim1Config->t_rrd	= 	T_RRD;
  tim1Config->t_wtr	= 	T_WTR;
}  

/****************************************************************************************
*  Function Name    : halSetupSDTIM2
*  Description      : This function sets up SDTIM2
*
*  Parameters      : config structure
*  Return          : None
* 
****************************************************************************************/
void halSetupSDTIM2(EMIF4F_TIMING2_CONFIG 	*tim2Config)
{
  /*Timing2 values for 800MHz operation*/
  tim2Config->t_xp 	= 	T_XP;
  tim2Config->t_odt = 	T_ODT;
  tim2Config->t_xsnr=	T_XSNR;
  tim2Config->t_xsrd=	T_XSRD;
  tim2Config->t_rtp	=	T_RTP;
  tim2Config->t_cke	=	T_CKE;
}

/****************************************************************************************
*  Function Name    : halSetupSDTIM3
*  Description      : This function sets up SDTIM3
*
*  Parameters      : config structure
*  Return          : None
* 
****************************************************************************************/
void halSetupSDTIM3(EMIF4F_TIMING3_CONFIG 	*tim3Config)
{
  /*Timing3 values for 800MHz operation*/
  tim3Config->t_ckesr 	= 	T_CKESR;
  tim3Config->zq_zqcs	=	ZQ_ZQCS;
  tim3Config->t_tdqsckmax	=	T_TDQSCKMAX;
  tim3Config->t_rfc	=	T_RFC;
  tim3Config->t_rasMax	=	T_RASMAX;
}

/****************************************************************************************
*  Function Name    : halSetupSDCFG
*  Description      : This function sets up SDCFG
*
*  Parameters      : config structure
*  Return          : None
* 
****************************************************************************************/
void halSetupSDCFG(tulong mode,EMIF4F_SDRAM_CONFIG  *sdramConfig)
{
  /*Setup DDR3 mode for 16/32/64-bit operation.*/
  switch(mode)
  {
    case DDR3_64BIT_MODE:
      sdramConfig->narrowMode = WORD_SZ_64;
    break;
    case DDR3_32BIT_MODE:
      sdramConfig->narrowMode = WORD_SZ_32;
    break;
    case DDR3_16BIT_MODE:
      sdramConfig->narrowMode = WORD_SZ_16;
    break;
    default:
      sdramConfig->narrowMode = WORD_SZ_64;
    break;
  }  
   
  sdramConfig->type		= DDR3_TYPE;
  sdramConfig->iBankPos	= I_BANK_POS_0;
  sdramConfig->ddrTerm		= DDR_TERM_RZQ_6;
  sdramConfig->ddrDDQS		= DDR_DDQS_DIFF;
  sdramConfig->dynODT		= DYN_ODT_RZQ_2;
  sdramConfig->disableDLL	= DIS_DLL_NORMAL;
  sdramConfig->SDRAMDrive	= SDRAM_DRIVE_RZQ_7;
  sdramConfig->CASWriteLat	= CAS_WRT_LATENCY;
  sdramConfig->CASLatency	= CAS_LATENCY;
  sdramConfig->rowSize		= ROW_SIZE;
  sdramConfig->iBank		= I_BANK_8;
  sdramConfig->eBank		= E_BANK_CS0;
  sdramConfig->pageSize	= PAGE_SIZE;
}


/****************************************************************************************
*  Function Name    : halDdr3Setup
*  Description      : This function sets up DDR3 
*
*  Parameters      : tulong mode, tlong xmc_index
*  Return          : None
* 
****************************************************************************************/
void halDdr3Setup(tulong mode)
{
  EMIF4F_TIMING1_CONFIG 	tim1Config;
  EMIF4F_TIMING2_CONFIG 	tim2Config;
  EMIF4F_TIMING3_CONFIG 	tim3Config;
  /* EMIF4F_OUTPUT_IMP_CONFIG  outputImpConfig; */
  EMIF4F_SDRAM_CONFIG           sdramConfig;
  /* EMIF4F_TEMP_ALERT_CONFIG  tempAlertConfig; */
	
  /**********************************************************************************
   STEP I: Unlock Boot Cfg mechanism
           Configure Boot Cfg register DDR3PLLCTL0 for 400MHz operation  (50 * 16 / 2)
           Lock Mechanism to prevent further Boot Cfg writes
  **********************************************************************************/
  /*Unlock Kicker mechanism in case it is locked */
  CSL_BootCfgUnlockKicker();	
   
  /*Configure DDR3-PLL*/
  *(tulong *)DDR3PLLCTL0 = *(tulong *)DDR3PLLCTL0 & 0xFF7FFFFF;
  *(tulong *)DDR3PLLCTL0 = DDR3PLL_400MHZ;
	
  /* Leave BootCfg area unlocked */
  //CSL_BootCfgLockKicker();
  	
  /* It is assumed that the XMC MPAX is already configured to map
   * CSL_DDR3_EMIF_CONFIG_REGS to 0x1 0000 0000.
   */

  /**********************************************************************************
  STEP II: Setup parameters only for EMIF4F registers for DDR3 configuration (800MHz)
  **********************************************************************************/
	
  /*Setup SDTIM1 Register parameters*/
  halSetupSDTIM1(&tim1Config);
	
  /*Setup SDTIM2 Register parameters*/
  halSetupSDTIM2(&tim2Config);
	
  /*Setup SDTIM3 Register parameters*/
  halSetupSDTIM3(&tim3Config);
	
  /*Setup SDCFG Register parameters*/
  halSetupSDCFG(mode,&sdramConfig);
  
  /*************************************************************************************
   STEP III: CSL API calls to configure EMIF4F registers with earlier setup parameters
   *************************************************************************************/
  
  /*Configure SDTIM1 using CSL*/
  CSL_EMIF4F_SetTiming1Config (&tim1Config);
      
  /*Configure SDTIM2 using CSL*/
  CSL_EMIF4F_SetTiming2Config (&tim2Config);
      
  /*Configure SDTIM3 using CSL*/
  CSL_EMIF4F_SetTiming3Config (&tim3Config);
  
  /*Configure ZQ_CONFIG using CSL*/
  /*CSL_EMIF4F_SetOutputImpedanceConfig(&outputImpConfig);*/
  
  /*Configure TEMP_ALERT_CONFIG using CSL*/
  /* CSL_EMIF4F_SetTempAlertConfig(&tempAlertConfig);*/
  
  /*Configure DDR_PHY_CTRL1 Register using CSL*/
  CSL_EMIF4F_SetPhyControl(DDRPHYCONTROL, DDRPHYREADLATENCY);
  
  /* Enable Initialization & Refresh using CSL*/
  CSL_EMIF4F_EnableInitRefresh();
      
  /*Configure SDCFG using CSL*/
  CSL_EMIF4F_SetSDRAMConfig (&sdramConfig);
  
  /*Configure SDRFC with refresh rate 7.8us using CSL*/
  CSL_EMIF4F_SetRefreshRate(REF_RATE_400M);
}



