/*****************************************************************************
 * FILE PURPOSE: Common Utility for DDR3/EMIF initialization
 *****************************************************************************
 * FILE NAME: emifDdr3.h
 *
 * DESCRIPTION: Common Utility for DDR3/EMIF initialization
 *               
 * TABS: None
 *
 * (C) Copyright 2009 Texas Instruments. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _EMIFDDR3_H_
#define _EMIFDDR3_H_

#include "ti/csl/csl_bootcfgAux.h"
#include "ti/csl/csl_emif4fAux.h"
#include "mpax_util.h"
#include "types.h"


/*DDR3 modes*/
#define DDR3_64BIT_MODE   0
#define DDR3_32BIT_MODE   1
#define DDR3_16BIT_MODE   2

/*DDR3 PLL Setup*/
#define DDR3PLLCTL0			(CSL_BOOT_CFG_REGS + 0x330)
#define DDR3PLL_400MHZ		0x080803C0    /* PLLM = 0xF, Assume 50MHz input clock */

/*XMC MPAX fields*/
/*The 32-bit logical address for DDR3 config space starts at 0x21000000. 
  The 36-bit physical address for the same begins at 0x1:00000000.*/

#define DDR3_CONFIG_SYS_ADDR_UPPER		0x00000001
#define DDR3_CONFIG_SYS_ADDR_LOWER		0x00000000
#define DDR3_CONFIG_LOGICAL_ADDR    	0x21000000

/*SDTIM1 Fields*/
#define T_RP				0xA 
#define T_RCD				0xA 
#define T_WR				0xB 
#define T_RAS	 			0x1B
#define T_RC				0x26
#define T_RRD				0x5 
#define T_WTR				0x5 

/*SDTIM2 Fields*/
#define T_XP				0x4 
#define T_ODT				0x6 
#define T_XSNR				0x5F 
#define T_XSRD				0x1FF
#define T_RTP				0x5 
#define T_CKE				0x3 

/*SDTIM3 Fields*/
#define	T_CKESR				0x4 
#define ZQ_ZQCS				0x3F 
#define T_TDQSCKMAX			0x0 
#define	T_RFC				0x57 
#define	T_RASMAX			0xF 


/*ZQ_CONFIG Fields*/
#define ZQCS1EN				0x0
#define ZQCS0EN				0x0
#define ZQDUALCSEN			0x0
#define ZQSFEXITEN			0x0
#define ZQZQCLINTERVAL		0x3
#define ZQZQCLMULT			0x3
#define ZQREFINTERVAL		0x0

/*TEMP_ALERT_CONFIG Fields*/
#define TACS1EN				0x0
#define TACS0EN				0x0
#define TADEVCNT			0x0
#define TADEVWDTH			0x0
#define TAREFINTERVAL		0x0
#define TASFEXITEN			0x0


/*DDR_PHY_CTRL1 Fields*/
#define DDRPHYCONTROL		0x0008008 
#define DDRPHYREADLATENCY	0xD

/*SDCFG Fields*/
#define DDR3_TYPE			0x3
#define	I_BANK_POS_0		0x0
#define DDR_TERM_DIS		0x0
#define DDR_TERM_RZQ_6		0x3
#define DDR_DDQS_DIFF		0x1
#define	DYN_ODT_OFF			0x0
#define	DYN_ODT_RZQ_2		0x2
#define	DIS_DLL_NORMAL		0x0
#define SDRAM_DRIVE_RZQ_7	0x1	


#define CAS_WRT_LATENCY		0x3 
#define CAS_LATENCY			0xE 


#define ROW_SIZE			0x6 
#define PAGE_SIZE			0x2 

#define I_BANK_8			0x3
#define E_BANK_CS0			0x0

/*DDR3 Narrow Modes - Set DDR3 wordsize 16/32/64-bit*/
#define WORD_SZ_64	0
#define WORD_SZ_32  1
#define WORD_SZ_16	2


#define REF_RATE_400M	0x0C30	  /* 7.8us at 400Mhz */


#define DDR3_NUM_INIT_RATIOS        9
#define DDR3_DATA_GTLVL_INIT_RATIO_START_INDEX      14
#define DDR3_RDWR_INIT_RATIO_START_INDEX             2
#define DDR3_DATA_GTLVL_INIT_RATIO_CFG_REG(i)       ((i) + DDR3_DATA_GTLVL_INIT_RATIO_START_INDEX)
#define DDR3_RDWR_INIT_RATIO_CFG_REG(i)             ((i) + DDR3_RDWR_INIT_RATIO_START_INDEX)
#define DDR3_CONFIG_REG_0                           0
#define DDR3_CONFIG_REG_12                          12
#define DDR3_CONFIG_REG_13                          13
#define DDR3_CONFIG_REG_23                          23
#define DDR3_CONFIG_REG_24                          24

#define DDR3_CONFIG_REG_23_FORCE_LEVELING           0x200;


#define DDR3_RD_DQS_SLAVE_RATIO_1333 0x34
#define DDR3_WR_DQS_SLAVE_RATIO_1333 0x45
#define DDR3_WR_DATA_SLAVE_RATIO_1333 0x85
#define DDR3_FIFO_WE_SLAVE_RATIO_1333 0xBC

#define DDR3_RD_DQS_SLAVE_RATIO_1066 0x34
#define DDR3_WR_DQS_SLAVE_RATIO_1066 0x37
#define DDR3_WR_DATA_SLAVE_RATIO_1066 0x77
#define DDR3_FIFO_WE_SLAVE_RATIO_1066 0xA0

#define DDR3_MK_CONFIG_REG_23(x, y, z)              ((x) | ((y) << 10) | ((z) << 20))


void halDdr3Setup(tulong mode);


#endif

/* Nothing past this point */

