#ifndef _CSLISR_H
#define _CSLISR_H
/*****************************************************************************
 * FILE PURPOSE: Header file for HAL Interrupts
 *****************************************************************************
 * FILE NAME: cslisr.h
 *
 * DESCRIPTION: Prototypes for HAL interrupts
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* Include device specific inlines */
//#include "cslisr_dev.h"
#include "types.h"
#include "gghwcfg.h"

void cslIsrInit (void);
tint cslGetVecNum (tint source);
tint csl_enable_interrupt (tint source);
tint csl_disable_interrupt (tint source);
tint csl_clear_interrupt (tint source);


#if  1





#if (HAL_CONFIG_CPU_CORE == HAL_C55X)
#include "cslisr55xt.h"
#elif (HAL_CONFIG_CPU_CORE == HAL_C64X)
#include "cslisr64xt.h"
#else
#error Unknown processor type
#endif


void cslInstallISR (tuint int_num, void (*int_routine)(void));

void cslSetIsrAddressA (void (*fcn)(), void *context);
void cslIntA (void);

void cslSetIsrAddressB (void (*fcn)(), void *context);
void cslIntB (void);

void cslSetIsrAddressC (void (*fcn)(), void *context);
void cslIntC (void);

void cslSetIsrAddressD (void (*fcn)(), void *context);
void cslIntD (void);

void cslSetIsrAddressE (void (*fcn)(), void *context);
void cslIntE (void);

void cslSetIsrAddressF (void (*fcn)(), void *context);
void cslIntF (void);

void cslSetIsrAddressURx (void (*fcn)(), void *context);
interrupt void cslIntURx (void);

void cslSetIsrAddressUTx (void (*fcn)(), void *context);
interrupt void cslIntUTx (void);

void cslSetIsrAddressGmacRx (void (*fcn)(), void *context);
interrupt void cslIntGmacRx (void);

void cslSetIsrAddressGmacTx (void (*fcn)(), void *context);
interrupt void cslIntGmacTx (void);

void cslSetIsrAddressHpi (void (*fcn)(), void *context);
interrupt void cslIntHpi (void);


void cslSetEdmaIsrAddress ( void (*fcn ) (), int chNum, void *context,
                            void (*edmafcn) (void) );
void cslEdmaIsr (void);                          

void cslSetGmacIsrAddress (void (*fcn)(), void *context);
void cslIntGmac (void);

/***********************************
 * Interrupt routing configuration *
 ***********************************/
typedef enum {
  CSL_ISEL_DO_ROUTE,        /* Makes the specified route the only one */
  CSL_ISEL_ADD_ROUTE,       /* Or's in the selected route             */
  CSL_ISEL_QUERY            /* Query route and vector information     */
} cslIntoselAction_t;

typedef enum {

  /* McBSP standard interrupts */
  CSL_PERIPH_MCBSP0_TX,
  CSL_PERIPH_MCBSP0_RX,
  
  CSL_PERIPH_MCBSP1_TX,
  CSL_PERIPH_MCBSP1_RX,
  
  CSL_PERIPH_MCBSP2_TX,
  CSL_PERIPH_MCBSP2_RX,
  
  CSL_PERIPH_MCBSP3_TX,
  CSL_PERIPH_MCBSP3_RX,
  
  /* McBSP superframe interrupts */
  CSL_PERIPH_MCBSP0_SF_TX,
  CSL_PERIPH_MCBSP0_SF_RX,
  
  CSL_PERIPH_MCBSP1_SF_TX,
  CSL_PERIPH_MCBSP1_SF_RX,
  
  CSL_PERIPH_MCBSP2_SF_TX,
  CSL_PERIPH_MCBSP2_SF_RX,
  
  CSL_PERIPH_MCBSP3_SF_TX,
  CSL_PERIPH_MCBSP3_SF_RX,
  
  /* GMAC interrupt routing */
  CSL_PERIPH_GMAC_TXRX
  
} cslIntoselPeriph_t;
 
typedef struct {
  cslIntoselAction_t action;  /* Confiuguration type     */
  cslIntoselPeriph_t periph;  /* Peripheral              */
  void *vector;               /* Vector offset           */
  cslIerReg_t ierReg;         /* Interrupt enable reg ID */
  cslIfrReg_t ifrReg;         /* Interrupt flag reg ID   */
  tuint ierMask;              /* Bit used in IERx        */
  tuint ifrMask;              /* Bit used in IFRx        */
  tuint channel;              /* Peripheral channel      */
} cslIntoselCfg_t;


/**********************************************************************************
 * Support includes. Include files that are core (not device) specific. These must
 * come after the above definitions since these use the above typedefs. In many
 * cores these can simpy be defined to voids or constants because the functionality
 * is not supported.
 **********************************************************************************/
#if (HAL_CONFIG_CPU_CORE == HAL_C55X)
#include "cslisr55x.h"
#elif (HAL_CONFIG_CPU_CORE == HAL_C64X)
/* empty */
#else
#error Unknown processor type
#endif


#endif  /* #if 0 */
 
 
#endif /* _CSLISR_H */
