/**********************************************************************************
 * FILE PURPOSE: Handle GEM specific interrupt functions
 **********************************************************************************
 * FILE NAME: cslgemisr.c
 *
 * DESCRIPTION: Provides gem specific interrupt managment
 *
 * Copyright (C) 2005, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "hal.h"
#include "halloc.h"
#include "device.h"
#include "gghwcfg.h" /* gghwcfg.h must come after device.h */
#include "cslgemisr_map.h"

/******************************************************************************/
/* EXTERNAL VARIABLE DECLARATIONS                                             */
/******************************************************************************/
#ifndef GG_BIOS
extern unsigned int VECTORS[] ;
#endif

/************************************************************************************
 * FUNCTION PURPOSE: Return the interrupt vector associated with an interrupt event
 ************************************************************************************
 * DESCRIPTION: The max values are configured in gghwcfg.h. This function returns
 *              the vector number that was associated with the specified device. 
 *              Returns the vector number, or -1 if the specified interrupt
 *              is not routed.
 ************************************************************************************/
tint cslGetVecNum (tint source)
{

#ifdef GG_HWCFG_VEC04_ENABLED
  if (source == GG_HWCFG_VEC04_SOURCE)
    return (4);
#endif

#ifdef GG_HWCFG_VEC05_ENABLED
  if (source == GG_HWCFG_VEC05_SOURCE)
    return (5);
#endif

#ifdef GG_HWCFG_VEC06_ENABLED
  if (source == GG_HWCFG_VEC06_SOURCE)
    return (6);
#endif

#ifdef GG_HWCFG_VEC07_ENABLED
  if (source == GG_HWCFG_VEC07_SOURCE)
    return (7);
#endif

#ifdef GG_HWCFG_VEC08_ENABLED
  if (source == GG_HWCFG_VEC08_SOURCE)
    return (8);
#endif

#ifdef GG_HWCFG_VEC09_ENABLED
  if (source == GG_HWCFG_VEC09_SOURCE)
    return (9);
#endif

#ifdef GG_HWCFG_VEC10_ENABLED
  if (source == GG_HWCFG_VEC10_SOURCE)
    return (10);
#endif

#ifdef GG_HWCFG_VEC11_ENABLED
  if (source == GG_HWCFG_VEC11_SOURCE)
    return (11);
#endif

#ifdef GG_HWCFG_VEC12_ENABLED
  if (source == GG_HWCFG_VEC12_SOURCE)
    return (12);
#endif

#ifdef GG_HWCFG_VEC13_ENABLED
  if (source == GG_HWCFG_VEC13_SOURCE)
    return (13);
#endif

#ifdef GG_HWCFG_VEC14_ENABLED
  if (source == GG_HWCFG_VEC14_SOURCE)
    return (14);
#endif

#ifdef GG_HWCFG_VEC15_ENABLED
  if (source == GG_HWCFG_VEC15_SOURCE)
    return (15);
#endif

  /* No source was found */
  return (-1);

} /* cslGetVecNum */


/*************************************************************************************
 * FUNCTION PURPOSE: Initialize the mux tables
 *************************************************************************************
 * DESCRIPTION: Loads the mux registers with the values defined through gghwcfg.h
 *************************************************************************************/
void cslIsrInit (void)
{
  UINT32 mux1, mux2, mux3;

  mux1 = CSL_INTCTL_REG32_READ (CSL_GEM_INTCTL_REG_A_MUX1);
  mux2 = CSL_INTCTL_REG32_READ (CSL_GEM_INTCTL_REG_A_MUX2);
  mux3 = CSL_INTCTL_REG32_READ (CSL_GEM_INTCTL_REG_A_MUX3);

#ifdef GG_HWCFG_VEC04_ENABLED
  mux1 = CSL_INTCTL_SET_INTSEL4 (mux1, GG_HWCFG_VEC04_SOURCE);
#endif

#ifdef GG_HWCFG_VEC05_ENABLED
  mux1 = CSL_INTCTL_SET_INTSEL5 (mux1, GG_HWCFG_VEC05_SOURCE);
#endif

#ifdef GG_HWCFG_VEC06_ENABLED
  mux1 = CSL_INTCTL_SET_INTSEL6 (mux1, GG_HWCFG_VEC06_SOURCE);
#endif

#ifdef GG_HWCFG_VEC07_ENABLED
  mux1 = CSL_INTCTL_SET_INTSEL7 (mux1, GG_HWCFG_VEC07_SOURCE);
#endif

#ifdef GG_HWCFG_VEC08_ENABLED
  mux2 = CSL_INTCTL_SET_INTSEL8 (mux2, GG_HWCFG_VEC08_SOURCE);
#endif

#ifdef GG_HWCFG_VEC09_ENABLED
  mux2 = CSL_INTCTL_SET_INTSEL9 (mux2, GG_HWCFG_VEC09_SOURCE);
#endif

#ifdef GG_HWCFG_VEC10_ENABLED
  mux2 = CSL_INTCTL_SET_INTSEL10 (mux2, GG_HWCFG_VEC10_SOURCE);
#endif

#ifdef GG_HWCFG_VEC11_ENABLED
  mux2 = CSL_INTCTL_SET_INTSEL11 (mux2, GG_HWCFG_VEC11_SOURCE);
#endif

#ifdef GG_HWCFG_VEC12_ENABLED
  mux3 = CSL_INTCTL_SET_INTSEL12 (mux3, GG_HWCFG_VEC12_SOURCE);
#endif

#ifdef GG_HWCFG_VEC13_ENABLED
  mux3 = CSL_INTCTL_SET_INTSEL13 (mux3, GG_HWCFG_VEC13_SOURCE);
#endif

#ifdef GG_HWCFG_VEC14_ENABLED
  mux3 = CSL_INTCTL_SET_INTSEL14 (mux3, GG_HWCFG_VEC14_SOURCE);
#endif

#ifdef GG_HWCFG_VEC15_ENABLED
  mux3 = CSL_INTCTL_SET_INTSEL15 (mux3, GG_HWCFG_VEC15_SOURCE);
#endif

  CSL_INTCTL_REG32_WRITE (CSL_GEM_INTCTL_REG_A_MUX1, mux1);
  CSL_INTCTL_REG32_WRITE (CSL_GEM_INTCTL_REG_A_MUX2, mux2);
  CSL_INTCTL_REG32_WRITE (CSL_GEM_INTCTL_REG_A_MUX3, mux3);

} /* cslIsrInit */


/*************************************************************************************
 * FUNCTION PURPOSE: Enable an individual interrupt
 *************************************************************************************
 * DESCRIPTION: Sets the specific bit, along with the non-maskable interrupt enable
 *              bit in the interrupt enable register. Returns -1 if unable to
 *              map the bit.
 *************************************************************************************/
extern cregister  volatile unsigned int IER;
tint csl_enable_interrupt (tint source)
{
  tint inum;
  UINT32 ormask; 

  inum = cslGetVecNum (source);
  if (inum < 0)
    return (-1);

  ormask = (1 << inum) | CSL_GEM_IER_REG_NMI_BIT;
  IER |= ormask;

  return (0);

} /* csl_enable_interrupt */


/*****************************************************************************
 * FUNCTION PURPOSE: Disable an individual interrupt
 *****************************************************************************
 * DESCRIPTION: Clears the specified bit in the interrupt enable register.
 *              Returns -1 if unable to map the bit
 *****************************************************************************/
tint csl_disable_interrupt (tint source)
{
  tint inum;
  UINT32 andmask;

  inum = cslGetVecNum (source);
  if (inum < 0)
    return (-1);

  andmask = ~(1 << inum);
  IER &= andmask;

  return (0);

} /* csl_disable_interrupt */

/******************************************************************************
 * FUNCTION PURPOSE: Force clear an interrupt
 ******************************************************************************
 * DESCRIPTION: Forces the interrupt to be cleared through the ICR
 ******************************************************************************/
extern cregister  volatile unsigned int ICR;
tint csl_clear_interrupt (tint source)
{
  tint inum;
  tuint ormask;

  inum = cslGetVecNum (source);
  if (inum < 0)
    return (-1);

  ormask = (1 << inum);
  ICR = ormask;   /* Do a write, not a read/modify/write */

  return (0);

} /* csl_clear_interrupt */

/******************************************************************************
 * FUNCTION PURPOSE: To install the interrupt vector table 
 ******************************************************************************
 * DESCRIPTION: IF vector table is different than the one at 0x0000, then this 
 *              function should be called. The function changes the IVT location
 *              by modifying  ISTP register.
 *
 * void cslInstallIsrTable ( void )
 *****************************************************************************/
extern cregister  volatile unsigned int ISTP;
void cslInstallIsrTable(void)
{
#ifndef GG_BIOS
  ISTP = ( unsigned int )( &VECTORS [0] ) ;
#endif

  /* Initialize the mux */
  cslIsrInit ();


} /*cslInstallIsrTable*/
  

#ifdef GG_HAL_EXCEPTION_STACK_SIZE_BYTES  

#pragma DATA_SECTION(exc_stack, ".exception_stack")
word exc_stack[GG_HAL_EXCEPTION_STACK_SIZE_BYTES];


/********************************************************************************
 * FUNCTION PURPOSE: Gem C exception handler
 ********************************************************************************
 * DESCRIPTION: The pre-exception context has been saved and the exception
 *              context stack has been setup. 
 ********************************************************************************/
void c_exception_handler (void)
{
#if (defined(GG_HAL_INCLUDE_WATCHDOG) && defined(GG_WATCHDOG_RETURN))
  halWdTimerControl_t tctl;
#endif
  unsigned int volatile *l1dwb=(unsigned int *)0x1845040;
  unsigned int volatile *l2wb=(unsigned int *)0x1845000;

  /* Let the host know an exception has occurred. Devices which don't have this
   * functionality should define the function to void */
  cslDeviceIntHost ();
  
  
  
#if (defined(GG_HAL_INCLUDE_WATCHDOG) && defined(GG_WATCHDOG_RETURN))

  /* If the watchdog was setup for return (re-fire) then the flag indicating a
   * fire will not be set. Instead check that the watch dog is enabled, and
   * that the timer has counted down to 0. */
  tctl.operation = HAL_WDTIMER_CHECK;
  halWdTimerControl (&tctl);
  if (tctl.status)
    return;

#endif

  /* Writing back L1D cache */
  *l1dwb=1;
  while((*l1dwb)!=0);
  
  /* Writing back L2 cache */
  *l2wb=1;
  while((*l2wb)!=0);
    


  /* Trap forever */
  for (;;);

} /* c_exception_handler */

#endif /* GG_HAL_EXCEPTION_HANDLER */
  
  
  





