#ifndef __CSLINTC_H
#define __CSLINTC_H
/************************************************************************/
/*                                                                      */
/*     Copyright (C) 2000  Texas Instruments Incorporated.             
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                   */
/************************************************************************/
/*                                                                      */
/* File Name  : cslintc.h                                               */
/*                                                                      */
/* Description: Header file describing APIs of cslint.c                 */
/*                                                                      */
/*                                                                      */
/* In line functions:                                                   */
/*    cslGlobDisableInt ()                                              */
/*    cslClearAllPendingInts ()                                         */
/*    cslGlobEnableInt ()                                               */
/*                                                                      */
/* Date : 21st Feb 2002                 OWNERSHIP : Tarakeshwar N.C     */
/*                                                                      */
/*                                                                      */
/************************************************************************/

/******************************************************************************
 * Include Files 
 ******************************************************************************/
#include "types.h"
#include "c64xint.h"


/******************************************************************************
* FUNCTION PURPOSE: Globally Disable All Interrupts
******************************************************************************
* DESCRIPTION: The function resets NMIE bit and GIE bits
*
* void cslGlobDisableInt ( void )
*****************************************************************************/

static inline void cslGlobDisableInt ( void )
{

  IER |= ( int )( ONE32 << NMIE ) ;  /*Resetting of GIE works only if NMIE=1*/
  CSR &= ( int )(~( ONE32 << GIE )) ;

} /* cslGlobDisableAllInt */

/******************************************************************************
* FUNCTION PURPOSE: CLears All Pending Interrupts
******************************************************************************
* DESCRIPTION: The function resets NMIE bit and GIE bits
*
* void cslClearAllPendingInts ( void )
*****************************************************************************/

static inline void cslClearAllPendingInts ( void )
{

  ICR = ( int ) 0xffff ;

} /* cslClearAllInts */

/******************************************************************************
 * FUNCTION PURPOSE: Globally Enable All Interrupts
 ******************************************************************************
 * DESCRIPTION: The function sets NMIE and GIE bits
 *****************************************************************************/

static inline void cslGlobEnableInt ( void )
{

  IER |= ( int )(( ONE32 << NMIE)) ;
  CSR |= ( int ) ( ( ONE32 << GIE ) ) ;
/******************************************************************************
EXCEP is the maskable external exception input to the CPU. It is enabled by
the XEN bit in TSR. For this exception to be recognized, the XEN bit must be
set to 1, the GEE bit must be set to 1, and the NMIE bit must be set to 1.
*****************************************************************************/
  /* 16 May 2006 : GEM Exceptions*/
  TSR |= ( int )(( ONE32 << XEN )) ;  
  TSR |= ( int )(( ONE32 << GEE )) ;  

} /* cslGlobEnableInt */

/* Prototypes */
void cslInstallIsrTable(void);

#endif /* __CSLINTC_H */

