;************************************************************************
;*                                                                      *
;*     Copyright (C) 2004 Texas Instruments Incorporated.            *
; * 
; *  Redistribution and use in source and binary forms, with or without 
; *  modification, are permitted provided that the following conditions 
; *  are met:
; *
; *    Redistributions of source code must retain the above copyright 
; *    notice, this list of conditions and the following disclaimer.
; *
; *    Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the 
; *    documentation and/or other materials provided with the   
; *    distribution.
; *
; *    Neither the name of Texas Instruments Incorporated nor the names of
; *    its contributors may be used to endorse or promote products derived
; *    from this software without specific prior written permission.
; *
; *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; */
;*                                     
;* File Name : isrcntxt.s
;*                                     
;* ABSTRACT:                           
;*
;* Function: CSL_ISR_CONTEXT_SAVE
;*           CSL_ISR_CONTEXT_REST
;*			 CSL_ISR_NOCXT_SAVE
;*
;* Description: These are functions for Saving and Restoring the ISR Context
;*
;* Arguments: None
;*
;* Inputs: None 
;*
;* Outputs: None
;*
;* Return value: None
;*
;* Prototype: 
;*
;*****************************************************************************
                      .asg  A15, FP
                      .asg  B14, DP
                      .asg  B15, SP

                      .global _CSL_ISR_CONTEXT_SAVE
                      .global _CSL_ISR_CONTEXT_REST
			          .global _CSL_ISR_NOCXT_SAVE

				      .cdecls C,NOLIST, "gghwcfg.h"

                      .text


; ****************************************************************************
; * On entry to _CSL_ISR_CONTEXT_SAVE the stack frame has been adjusted by
; * -216. The following registers already been saved and replaced with
; * the following values:
; *  B1:B0   B0 contains a pointer to the address of the application interrupt handler
; *  A5:A4   A4 contains a pointer to the context of the application interrupt handler
; *
; *  !!!!!!
; *   IRP is not saved as part of the context. Interrupts must not be re-enabled
; *   during interrupt processing or else the return information will be lost.
; *   If interrupt re-entry is required IRP must be added to the context
; *   save and restore
; *   ITSCR  would also have to be saved
; *  !!!!!!
; *
; *****************************************************************************


cslIsrContextSave	.macro	  retAddress

;            ADDK      .S2   -232,SP             <- Already done in vector table
;            STDW      .D2T2  B1:B0,*+SP(56)     <- Saved in vector table
;            STDW      .D2T1  A31:A30,*+SP(216)  <- Saved in vector table

            STDW      .D2T1  A1:A0, *+SP(16)
||          MV        .S1X   SP, A31

            LDW       .D2T2 *B0, B0              ; load the function address

            STDW      .D2T2  B31:B30,*+SP(208)
||          STDW      .D1T1  A5:A4, *+A31(32)

            STDW      .D2T2  B29:B28,*+SP(200)
||          STDW      .D1T1  A29:A28,*+A31(144)
||			MVC		  .S2    CSR, B31

            STDW      .D2T2  B27:B26,*+SP(192)
||          STDW      .D1T1  A27:A26,*+A31(136)
||			MVC		  .S2    SSR, B30

			STDW	  .D2T2  B31:B30, *+SP(224)   ; CSR:SSR
||          MVC		  .S2    AMR, B29
||			LDW		  .D1T1  *A30, A4			  ; Load the conext pointer

            STDW      .D2T2  B25:B24,*+SP(184) 
||          STDW      .D1T1  A25:A24,*+A31(128)
||			SUB		  .L2    B26, B26, B26

			STW       .D2    B29, *+SP(4)		 ; AMR
||		    MVC		  .S2    B26, AMR

            STDW      .D2T2  B23:B22,*+SP(176)
||          STDW      .D1T1  A23:A22,*+A31(120)
||          MVC       .S2    ILC, B31

            STDW      .D2T2  B21:B20,*+SP(168)
||          STDW      .D1T1  A21:A20,*+A31(112)
||          MVC       .S2    RILC, B30

            STDW      .D2T2  B3:B2,*+SP(64)
||          STDW      .D1T1  A19:A18,*+A31(104)
||          B         .S2    B0


            STDW      .D2T2  B17:B16,*+SP(152)                  ; Delay slot 1
||          STDW      .D1T1  A17:A16,*+A31(96)
||          ADDKPC    .S2    retAddress, B3

            STDW      .D2T2  B9:B8,*+SP(88)                  ; Delay slot 2
||          STDW      .D1T1  A9:A8,*+A31(48)
||          MV        .S1X   B31, A27

            STDW      .D2T2  B7:B6,*+SP(80)                  ; Delay slot 3
||          STDW      .D1T1  A7:A6,*+A31(40)
||          MV        .S1X   B30, A26

            STDW      .D2T2  B5:B4,*+SP(72)                  ; Delay slot 4
||          STDW      .D1T1  A3:A2,*+A31(24)    

            STDW      .D2T2  B19:B18,*+SP(160)                  ; Delay slot 5
||          STDW      .D1T1  A27:A26, *+A31(8)                ; ILC:RILC

			.endm



cslIsrContextRest	.macro

            LDDW     .D2T1  *+SP(8), A31:A30           ; ILC/RILC
||          MV       .S1X   SP, A0

            LDDW     .D2T2  *+SP(64), B3:B2            ; Delay slot 1 for ILC/RILC
||          LDDW     .D1T1  *+A0(24), A3:A2

            LDDW     .D2T2  *+SP(72), B5:B4            ; Delay slot 2 for ILC/RILC
||          LDDW     .D1T1  *+A0(32), A5:A4
||			ZERO     .L2    B0

            LDDW     .D2T2  *+SP(80), B7:B6            ; Delay slot 3 for ILC/RILC
||          LDDW     .D1T1  *+A0(40), A7:A6
||			SET      .S2    B0,9,9,B0

            LDDW     .D2T2  *+SP(88), B9:B8            ; Delay slot 4 for ILC/RILC
||          LDDW     .D1T1  *+A0(48), A9:A8

            LDDW     .D2T2  *+SP(152), B17:B16         ; Delay slot 5 for ILC/RILC
||          LDDW     .D1T1  *+A0(96),  A17:A16

			LDDW     .D2T2  *+SP(224), B31:B30		   ; CSR/SSR
||			LDW      .D1    *+A0(4),   A29			   ; AMR

            LDDW     .D2T2  *+SP(160), B19:B18		   ; Delay slot 1 for CSR/SSR/AMR
||          LDDW     .D1T1  *+A0(104), A19:A18
||          MVC      .S2    A31, ILC

            LDDW     .D2T2  *+SP(168), B21:B20		   ; Delay slot 2 for CSR/SSR/AMR
||          LDDW     .D1T1  *+A0(112), A21:A20
||          MVC      .S2    A30, RILC

            LDDW     .D2T2  *+SP(176), B23:B22		   ; Delay slot 3 for CSR/SSR/AMR
||          LDDW     .D1T1  *+A0(120), A23:A22
||			MVC      .S2    CSR, B1

            LDDW     .D2T2  *+SP(184), B25:B24         ; Delay slot 4 for CSR/SSR/AMR
||          LDDW     .D1T1  *+A0(128), A25:A24
||			CLR		 .S2    B1,9,9,B1

            LDDW     .D2T2  *+SP(192), B27:B26         ; Delay slot 5 for CSR/SSR/AMR
||          LDDW     .D1T1  *+A0(136), A27:A26
||			MVC      .S2    B1, CSR					   ; Clear the sat bit


            LDDW     .D2T2  *+SP(200), B29:B28      
||          LDDW     .D1T1  *+A0(144), A29:A28		   ; If B0 is non-zero then
||			AND		 .S2    B0, B31, B0				   ; the CSR sat bit must be set
||			MV       .L2X   A29, B1

			MVC	     .S2    B1, AMR			; restore AMR

            LDDW     .D2T2  *+SP(208), B31:B30      
||          LDDW     .D1T1  *+A0(216), A31:A30
||     [B0] MVKH	 .S2    0x7fff0000,B1

            LDDW     .D1T1  *+A0(16), A1:A0         
||          LDDW     .D2T2  *+SP(56), B1:B0
||	   [B0] SSHL	 .S2    B1, 1, B1				   ; Force saturation

	.endm

_CSL_ISR_CONTEXT_SAVE:   
; The following code is to enable Exception to be enbled during ISR.( The hardware disables it on entry)
; B1 is already preserved in vectors.s

                     MVC.S2        TSR,B1
                     OR.L2         8,B1,B1
                     MVC.S2        B1,TSR
			cslIsrContextSave	_CSL_ISR_CONTEXT_REST

; ***********************************************************************************
; * Context save restores the context, then branches to IRP to exit the
; * interrupt processing
; ***********************************************************************************

_CSL_ISR_CONTEXT_REST:

			cslIsrContextRest

            B        .S2    IRP             ; Do not put restores in the delay slots
			MVC      .S2    B30, SSR
            ADDK     .S2    232, SP         ; or the values will not be in place
            NOP             3               ; on return

	.if $defined(GG_HAL_EXCEPTION_STACK_SIZE_BYTES)

_CSL_EXCEPTION_CONTEXT_SAVE
			cslIsrContextSave   _CSL_EXCEPTION_CONTEXT_REST

_CSL_EXCEPTION_CONTEXT_REST:

			cslIsrContextRest
			bnop       telogy_exc_hndlr_ret 
			mvc	       b30, SSR
			addk       240, SP
			nop 3

	.endif   ; GG_HAL_EXCEPTION_STACK_SIZE_BYTES


; **************************************************************************************
; * Context nosave simply branches, and restores the B0 register
; * and the stack to the value before the interrupt occurred.
; *
; * On entry: SP has been decremented by 8
; * 		  Original B0 is at SP[4]
; *			  B0 now contains a pointer to the branch address, and
; *			  the instruction LDW *B0, B0 is already 2 cycles into
; *			  the execution phase.
; **************************************************************************************
_CSL_ISR_NOCXT_SAVE:

			NOP		     3					; Wait for the load to complete

			B		.S2  B0					; Branch to the load address
			LDW     .D2  *+SP(4), B0		; Restore pre-interrupt state
			ADDK	.S2  8, SP				; Note that B0 will be 1 cycle late
			NOP		3




; **************************************************************************************
; * Exception Handler
; *
; * Setup the temporary stack and call the C level handler
; *
; **************************************************************************************
	.def   telogy_exc_hndlr

	.if $defined(GG_HAL_EXCEPTION_STACK_SIZE_BYTES)

	.ref	exc_scratch			 ; defined in exchndlr.s
	.ref    _exc_stack
	.ref   _c_exception_handler  ; 

telogy_exc_hndlr:

	mvd		b8,           b8     ; Don't use a register which which can be in circular address mode
||	mvkl    exc_scratch,  b8
	mvkh    exc_scratch,  b8
	stdw    b1:b0,       *b8[0]
	stdw    a31:a30,     *b8[1]

 ; Change to the local stack
    mvkl    _exc_stack,          b1
	mvkh    _exc_stack,          b1
	mvkl    GG_HAL_EXCEPTION_STACK_SIZE_BYTES - 4, b0
	mvkh    GG_HAL_EXCEPTION_STACK_SIZE_BYTES - 4, b0
	add     b0,  b1, b1
	and     ~7,  b1, b1
	mv      b15, b0
	mv      b1,  b15


 ; The new stack is setup. Put the original value of the stack pointer into the new
 ; stack, then setup for a full context save exactly as an interrupt would do. Because
 ; The context save sets the return up to context restore, and context restore returns
 ; to IRP, the original IRP must be saved and replaced with the desired return in
 ; this function, for cases where the exception is allowed to return.

    addk    -16,    SP
	mvc     irp,    b1
	stdw    b1:b0, *+SP(8)              ; 
	mvkl    telogy_exc_hndlr_ret, b0
	mvkh    telogy_exc_hndlr_ret, b0
	mvc     b0, irp


;   Setup for standard context save
	addk  -240,  SP
	mvc  nrp, b1
	stw b1, *+SP(232)
	mvc ierr, b1
	stw b1, *+SP(236)
	mvkl   exc_scratch,  b0
	mvkh   exc_scratch,  b0
	lddw   *b0[1],       a31:a30     ; original a31:a30 back into a31:a30
	nop
	nop
	nop
	nop
	stdw   a31:a30, *+SP(216)         
	lddw   *b0[0],       a31:a30     ; original b1:b0 into a31:ad0
	mvkl   _c_exception_handler, b1
	mvkh   _c_exception_handler, b1  ; The pointer to the C function must be in b0
	stw    b1, *b0[0]
	bnop   _CSL_EXCEPTION_CONTEXT_SAVE,3   
	stdw   a31:a30, *+SP(56)        
	nop


;   For allowed returns from exceptions restore the original stack
;   The original context has been restored, but without sp and IRP
telogy_exc_hndlr_ret:

	mvd		b8,           b8     ; Don't use a register which which can be in circular address mode
||	mvkl    exc_scratch,  b8
	mvkh    exc_scratch,  b8
	stdw    b1:b0,       *b8[0]
	nop

    lddw    *+SP(8), b1:b0
	nop
	nop
	nop
	nop
	nop
	mvc     b1, irp
	mv      b0, b15
	mvkl    exc_scratch, b1
	mvkh    exc_scratch, b1
	lddw    *b1[0], b1:b0
	b       nrp
	nop
	nop
	nop
	nop
	nop

	.else

telogy_exc_hndlr:

	bnop   telogy_exc_hndlr, 5
	nop

	.endif   ; GG_HAL_EXCEPTION_STACK_SIZE_BYTES 
	
