#ifndef __MEMMV_HAL_H_
#define __MEMMV_HAL_H_
/*****************************************************************************/
/*                                                                           */
/*     Copyright (C) 2002  Texas Instruments Incorporated.                   
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                        */
/*****************************************************************************/
/*  FILE PURPOSE: Memory Move function for HAL.                              */
/*  File Name   : memmvhal.h                                                 */
/*                                                                           */
/*  Description: Header file for memmvhal.c                                  */
/*                                                                           */
/*  FUNCTIONS:                                                               */
/*                                                                           */
/*  halLocalMemMove ()         : Local Memory Moves, within the DSP.         */
/*  halGlobalMemMove ()        : Global Memory Moves, outside the DSP.       */
/*  halMapLocal2Global ()      : Maps Local Address to Global Address.       */
/*                                                                           */
/*  STATIC FUNCTIONS:  None                                                  */
/*                                                                           */
/*  TABS : None                                                              */
/*                                                                           */
/*                                                                           */
/*  Date : 15th May 2002                         OWNERSHIP : Hemant H        */
/*****************************************************************************/
/*********************Data Structures ****************************************/
#include "gghwcfg.h"
#include "hal.h"

/* Macros providing local core  */
/* Global address will have '1' in most significant Nibble and Core Id(0 to 5) in 2nd MSN 
   ex: for core 0: local addr = 0x800000 =>global Addr = 0x10800000
   ex: for core 2: local addr = 0x800000 =>global Addr = 0x12800000
   */
#define CSL_LOCAL_TO_GLOBAL(core, localAddr)     ((1<<28)|(core<<24)|((localAddr)&0x00ffffff))
#define CSL_GLOBAL_TO_LOCAL(globAddr)            (globAddr & 0x00ffffff)
#define CSL_GLOBAL_EXTRACT_CORE(globAddr)        ((globAddr >> 24) & 0xf)

#endif /* __MEMMV_HAL_H_ */
/* Nothing past this point */
