/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _CSLMAC_H
#define _CSLMAC_H
/*************************************************************************************
 * FILE PURPOSE: Define the external API provided for mac control
 *************************************************************************************
 * FILE NAME: cslmac.h
 *
 * DESCRIPTION: Provides the external API used to configure the mac
 *
 *************************************************************************************/
#include "types.h"
 
#define CSL_MAC_MAX_MAC_ADDR    32

typedef struct {
  tint   core;          /* core associated with the mac address, -1 for unused */
  tuint  mac_addr[3];   /* mac address        */
} cslMacAddr_t;
 
typedef enum {
  CSL_MAC_MODE_10_100,
  CSL_MAC_MODE_GIG
} cslMacSpeed_t;

typedef enum {
  CSL_MAC_DUPLEX_FULL,
  CSL_MAC_DUPLEX_HALF
} cslMacDuplex_t;

typedef enum {
  CSL_MAC_FLOW_CONTROL_OFF,
  CSL_MAC_FLOW_CONTROL_ON
} cslMacFlowCtrl_t;

typedef enum {
  CSL_MAC_RGMII_OFF,
  CSL_MAC_RGMII_ON
} cslMacRgmii_t;

typedef enum {
  CSL_MAC_SMII_OFF,
  CSL_MAC_SMII_ON
} cslMacSmii_t;

typedef enum {
  CSL_MAC_S3MII_OFF,
  CSL_MAC_S3MII_ON
} cslMacS3mii_t;

typedef enum {
  CSL_MAC_RMII_OFF,
  CSL_MAC_RMII_ON
} cslMacRmii_t;

typedef enum {
  CSL_MAC_RMII_SPEED_10MBS,
  CSL_MAC_RMII_SPEED_100MBS
} cslMacRmiiSpeed_t;
  
typedef enum {
  CSL_MAC_MULTICAST_OFF,
  CSL_MAC_MULTICAST_ON
} cslMacMulticastCtrl_t;
  

typedef struct {
  /* physical port number */
  tint  port;

  /* Physical port config */
  cslMacSpeed_t     speed;
  cslMacDuplex_t    duplex;
  cslMacFlowCtrl_t  flow;
  cslMacRgmii_t     rgmii;
  cslMacSmii_t      smii;
  cslMacS3mii_t     s3mii;
  cslMacRmii_t      rmii;
  cslMacRmiiSpeed_t rmiiSpeed;
  cslMacMulticastCtrl_t   multicastCtrl;
  
  /* mac addresses accepted */
  cslMacAddr_t      mac[CSL_MAC_MAX_MAC_ADDR];
  
  /* Linear buffering required for received packets */
  tint  maxRxPacketSizeBytes; /* The largest packet size accepted */
  
} cslMacConfig_t;


/******************************************************************************
 * Definition: Emac interrupt wrapper control
 ******************************************************************************/
typedef enum {
  CSL_MAC_EWINT_DISABLE,
  CSL_MAC_EWINT_ENABLE
} cslMacEwOp_t;

#endif  /* _CSLMAC_H */




