#ifndef _HALLOC_H
#define _HALLOC_H
/******************************************************************************
 * FILE PURPOSE: Local Hardware Abstraction Layer definitions and structures
 ******************************************************************************
 * FILE NAME: halloc.h
 *
 * DESCRIPTION: Contains definitions and structures restricted in scope to
 *              HAL.
 *
 * TABS: None
 * 
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "gghwcfg.h"
#include "hal.h"
#include "utldbg.h"

#define HAL_EXCEPTION_TSIP_FAIL1 600
#define HAL_EXCEPTION_TSIP_FAIL2 601
#define HAL_EXCEPTION_TSIP_FAIL3 602


/*****************************************************************************
 * Definition: The HAL context structure. There is a single instance of this
 *             structure defined globally.  It should maintain as small a 
 *             foot print as possible.
 *****************************************************************************/

#ifdef TSIP_LLD  
typedef struct {
  Tsip_DrvHandle  tsipLldDrv;
  Tsip_PortHandle tsipLldPort[GG_TSIP_N_PORTS]; 
} tsipLldContext_t;
#endif

typedef struct {

#if (defined(GG_HAL_INCLUDE_TDM) || defined(GG_HAL_INCLUDE_ITDM) || defined(TSIP_LLD))
  tint nTdmPorts;   /* The number of tdmPorts enabled                       */
#endif


#ifdef GG_HAL_INCLUDE_AIC
  tint nAicPorts;
#endif

#ifdef GG_HAL_INCLUDE_UTOPIA
  tint nPktPorts;   /* The number of pktPorts enabled                       */
#endif

#if ((defined(GG_HAL_INCLUDE_GMAC)) || (defined(GG_HAL_INCLUDE_EMAC)) || (defined(GG_HAL_INCLUDE_3PSW)))
  tint nGmacPorts;
#endif



#ifdef GG_HAL_INCLUDE_UTOPIA
  void **pktPorts;  /* Array of pointers (size nPktPorts) to port instances */
#endif

#if (defined(GG_HAL_INCLUDE_TDM) || defined(GG_HAL_INCLUDE_ITDM))
  void **tdmPorts;  /* Array of pointers (size nTdmPorts) to port instances */
#endif

#ifdef GG_HAL_SHARED_TDM_RESOURCES
  void *tdmShared;  /* Shared TDM resources context */
#endif

#ifdef TSIP_LLD  
  tsipLldContext_t tsipLldContext;
#endif

#ifdef GG_HAL_INCLUDE_AIC
  void **aicPorts;  /* Array of pointers (size nAicPorts) to AIC instances */
#endif

#if ((defined(GG_HAL_INCLUDE_GMAC)) || (defined(GG_HAL_INCLUDE_EMAC)) || (defined(GG_HAL_INCLUDE_3PSW)))
  void **gmacPorts; /* Array of pointers (size nGmacPorts) to gmac instances */
#endif

#ifdef GG_HAL_INCLUDE_DMA_CONTEXT
  void *dmaContext;
#endif

/* For c64x the timer tick rate when selecting internal clock is 
   8 (CPU Clock / 8) and cannot be configured.When period is set to 
   65535 and CPU is operating at 600 Mhz timer counter period is approx 0.875 ms
   only  and this gives wrong results in MIPS calculation.{If task duration
   is greater than 0.875 ms }.An additional divide factor is given to increase 
   timer period. */
#ifdef HAL_INCLUDE_CONFIG_TIMER_TICK_RATE
  tuint tickRate[HAL_N_TIMERS];
#endif
#ifdef GG_HAL_INCLUDE_TID_CONTROL
void *tidControlContext;
#endif /* GG_HAL_INCLUDE_TID_CONTROL*/
#ifndef C_ISR 
 debugInfo_t debugInfo;
#endif

 /* Adding a dummy field to take care of the case when the structure is empty
  * in the case of bootloader
  */
 tuint  dummy;

} halContext_t;
extern halContext_t *halContext;

/******************************************************************************
 * Definition: Internal call table used only for linking the hal initialization
 *             functions to the code.
 ******************************************************************************/
typedef struct {

  /* TDM initializations */
  tuint (*halTdmGetHeapSize)     (halTdmInit_t *cfg);  
  tint  (*halTdmGetTxDmaBufInfo) (halTdmInit_t *cfg, tuint *align, tuint *size);
  tint  (*halTdmGetRxDmaBufInfo) (halTdmInit_t *cfg, tuint *align, tuint *size);

#ifdef TSIP_LLD  
  void (*halTdmAllocAndInit)    (void *base,  tuint size, halTdmInit_t *newCfg,
                                 Tsip_PortHandle *portHandle, halReturn_t *ret);
#else
  void *(*halTdmAllocAndInit)    (void *base, tuint size, halTdmInit_t *newCfg, tuint *dmaTx,
                                  tuint dmaTxSize, tuint *dmaRx, tuint dmaRxSize,
                                  halReturn_t *ret);
#endif                                  


  halReturn_t (*halTdmInitStart) (void *tdmp);
  void (*halTdmResourceInit)     (void);
  tuint (*halTdmShGetHeapSize)   (void);
  tint  (*halTdmShInit)          (void *);
  
  /* Utopia initializations */
  tuint (*halPktGetHeapSize)  (halUtopiaSizes_t *cfg);
  tint  (*halPktGetTxBufInfo) (halUtopiaSizes_t *cfg, tuint *align, tuint *size);
  tint  (*halPktGetRxBufInfo) (halUtopiaSizes_t *cfg, tuint *align, tuint *size);
  tint  (*halPktGetCaBufInfo) (halUtopiaSizes_t *cfg, tuint *align, tuint *size);
  void *(*halPktAllocAndInit) (void *base, 
                               halUtopiaInit_t *cfg, tuint *dmaTx, tuint dmaTxSize, 
                               tuint *dmaRx, tuint dmaRxSize, 
                               tuint *dmaCa, tuint dmaCaSize, halReturn_t *ret);
  void (*halPktInitStart)     (void *pktp);
  
  /* Gmac initialization */
  tuint (*halGmacGetHeapSize)        (halGmacSizes_t *cfg);
  halReturn_t (*halGmacGetTxBufInfo) (halGmacSizes_t *cfg, tuint *align, tulong *size);
  halReturn_t (*halGmacGetRxBufInfo) (halGmacSizes_t *cfg, tuint *align, tulong *size);
  void *(*halGmacBufAlloc)           (void *vbase, halGmacSizes_t *sizes, tuint *dmaTx,
                                      tulong dmaTxSize, tuint *dmaRx, tulong dmaRxSize,
                                      tuint *assemBase, halReturn_t *ecode);
  halReturn_t (*halGmacInit)         (halGmacInit_t *cfg);                                
  
  /* DMA isr initialization */
  tint (*halDmaGetSizes) (void);
  halReturn_t (*halDmaInit)     (void *cxt, bool hwInit);
                                  
  tint (*halTidControlGetSizes) (void);
  halReturn_t (*halTidControlAllocInit)  (void *cxt, bool hwInit);
                              
} halInitCtable_t;                              
extern halInitCtable_t halInitCtable;
  
  

/*  Prototypes */
/* halinit.c */
halReturn_t halGetSizes (tint *nbufs, halMemBuffer_t **bufs,
                         halNewConfig_t *cfg);
halReturn_t halNew (tint nbufs, halMemBuffer_t *bufs, halNewConfig_t *cfg);

halReturn_t halControl (halControl_t *hal_ctrl);

halReturn_t halUtopiaGetSizes (tint *nbufs, halMemBuffer_t **bufs, halUtopiaSizes_t *sizeCfg);
halReturn_t halUtopiaInit (halUtopiaInit_t *cfg, tint nbufs, halMemBuffer_t *bufs);

halReturn_t halTdmGetSizes (tint *nbufs, halMemBuffer_t **bufs, halTdmInit_t *tdmInit);
halReturn_t halTdmInit (halTdmInit_t *cfg, tint nbufs, halMemBuffer_t *bufs);

halReturn_t halGmacInit (halGmacInit_t *cfg);
halReturn_t halGmacGetSizes (tint *nbufs, halMemBuffer_t **bufs, halGmacSizes_t *sizeCfg);
tint halGmacAlloc (tint nbufs, halMemBuffer_t *bufs, halGmacSizes_t *sizes);

/* timerhal.c */
#ifndef __TMS320C55X__
tulong halTimerControl(tint timerId,halTimerControl_t *ctl);
#else
tint halTimerControl(tint timerId,halTimerControl_t *ctl);
#endif

/* pllhal.c */
tint halPllControl(tuint multiplier, tuint divider, tuint pllNo);

/* memhal.c */
halReturn_t halMemConfig(halMemConfig_t *halMemCfg);

/* ehpihal.c  */
tint halHpiControl (halHpiControl_t *hpi_control);

tint halHpiRead (tuint offset, tuint size, word *data );

tint halHpiWrite (tuint offset, tuint size, word *data, tint intr_mode ); 
  
halReturn_t halSrioInit(halSrioInitCfg_t *ctl);

/* ipc.c */
tint halIpcControl (halIpcControl_t *ipc_control);
                                             
/* Interrupt */
halIntrLevel_t halIntrLevel (halIntrLevel_t newLevel);
halReturn_t halIntState (halIntStateControl_t *istate);
tint hal_route_interrupt (tint source, void (*fcn)(void *), void *cxt);

/* Message interface */
tint halMessage (tint msgId, void *vmsg, tint (*msgReply)(void *, tint, tint, tint),
                        tint header_size, tint trans_id);

/* cpuutils.c */
void halLowPower (void);
void cslDeviceSleep (halCmdWakeOn_t wakeon);
halReturn_t cslDeviceIntOcore (halCtlIntCore_t *icore);
void cslDeviceIntHost (void);

/* The following functions can be used for IPC call backs */
void cslUtopiaIpc (void);
void cslUtopiaIpcReady (void);


/************************************************************************************
 * Generic bit extraction macros
 ************************************************************************************/
#define HALBITMASK(x,y)      (   (   (  (1U << ((x)-(y)+1U) ) - 1U )   )   <<  (y)   )
#define HAL_READ_BITFIELD(z,x,y)   (((z) & HALBITMASK(x,y)) >> (y))
#define HAL_SET_BITFIELD(z,f,x,y)  ((z) & ~HALBITMASK(x,y)) | ( ((f) << (y)) & HALBITMASK(x,y) )

#define HAL_LOW_POWER_TDM_WAKEUP    1
#define HAL_LOW_POWER_RX_PKT_WAKEUP 2
#define HAL_LOW_POWER_TX_PKT_WAKEUP 3
     
#endif /* _HALLOC_H */

