/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/************************************************************************************
 * FILE PURPOSE: Create an assembly include file based on gghwcfg.h
 ************************************************************************************
 * FILE NAME: gghwcfg_to_inc.c
 * 
 * DESCRIPTION: Creates an include file for use in the vector define file
 *				based on interrupt definitions in gghwcfg.h
 *
 *				This file is compiled using gcc.
 *
 ************************************************************************************/
 #include <stdio.h>
 #include "gghwcfg.h"


int main (int argc, char *argv[])
{

  FILE *fout;

  /* Open the output file */
  fout = fopen ("ggisrinc.inc", "w");
  if (fout == NULL)  {
    fprintf (stderr, "%s: Could not open file ggisrinc.s\n", argv[0]);
    return (-1);
  }


  /* Create the file based on include values in gghwcfg.h */
#ifdef GG_HWCFG_VEC04_ENABLED
  fprintf (fout, "GG_HWCFG_VEC04_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC04_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC05_ENABLED
  fprintf (fout, "GG_HWCFG_VEC05_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC05_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC06_ENABLED
  fprintf (fout, "GG_HWCFG_VEC06_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC06_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC07_ENABLED
  fprintf (fout, "GG_HWCFG_VEC07_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC07_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC08_ENABLED
  fprintf (fout, "GG_HWCFG_VEC08_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC08_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC09_ENABLED
  fprintf (fout, "GG_HWCFG_VEC09_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC09_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC10_ENABLED
  fprintf (fout, "GG_HWCFG_VEC10_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC10_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC11_ENABLED
  fprintf (fout, "GG_HWCFG_VEC11_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC11_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC12_ENABLED
  fprintf (fout, "GG_HWCFG_VEC12_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC12_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC13_ENABLED
  fprintf (fout, "GG_HWCFG_VEC13_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC13_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC14_ENABLED
  fprintf (fout, "GG_HWCFG_VEC14_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC14_ENABLED  .set  0\n");
#endif

#ifdef GG_HWCFG_VEC15_ENABLED
  fprintf (fout, "GG_HWCFG_VEC15_ENABLED  .set  1\n");
#else
  fprintf (fout, "GG_HWCFG_VEC15_ENABLED  .set  0\n");
#endif

  fclose (fout);

  return (0);

}


