#ifndef _DPRUTIL_H
#define _DPRUTIL_H
/******************************************************************************
 * FILE PURPOSE: Utilities for accessing Dual-port RAM (DPR)
 ******************************************************************************
 * FILE NAME:  dprutil.h 
 *
 * DESCRIPTION: Defines useful macros for accessing DPR
 *
 * TABS: NONE
 *
 * $Id: dprutil.h 1.3 1998/01/13 14:56:28 BOGDANK Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: dprutil.h $
 * Revision 1.3  1998/01/13 14:56:28  BOGDANK
 *     - 16-bit interface used
 *    16-bit interface used.
 * Revision 1.2  1997/08/05 16:03:09  ffruth
 * - removed dprInterrupt macro.  SIU uses targethw.h routine.
 * Revision 1.1  1997/07/24 13:18:42  ffruth
 * Initial revision
 *
 * (C) Copyright 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "micdspdp.h"
#include "targethw.h"

/*--------------------------------*/
/*  Dual Port Ram Access Routines */
/*--------------------------------*/

/* condition input value, i.e. the value read from DPR.  Should be used prior
 * to assigning the value to a variable.
 *
 * Example:  x = dprInVal(VALUE)
 */
#define dprInVal(dprVal) ((dprword)(dprVal))

/* Condition the output value, i.e. the value that is to be written to DPR.
 * Should be used prior to writing a value to DPR. */
#define dprOutVal(x)     ((dprword)(x))


/******************************************************************************
 * FUNCTION PURPOSE: Get echo flag.
 ******************************************************************************
 * DESCRIPTION: Get echo flag from a buffer in DPR.  Returns the echo flag
 *              value.  The echo field is 16-bit, thus any extra bits may be
 *              masked if necessary.
 *
 *  static inline tint dprGetEcho (void *vecho) - pointer to echo byte.
 *
 *****************************************************************************/

static inline tint dprGetEcho (void *vecho)
{
  dprword echo1, echo2;
  volatile dprword *echop = (dprword *)vecho;

  /* Get the echo flag */
  echo1 = dprInVal(*echop);     /* no need for mask here */
  do {
     echo2 = echo1;
     echo1 = dprInVal(*echop);  /* no need for mask here */
  } while (echo1 ^ echo2);
  return((tint)echo1);
} /* dprGetEcho */
#endif /* _DPRUTIL_H */
/* nothing past this point */
