/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef EMUREG
#define EMUREG

extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int TSCH;

#define EMU_MIPS_DUMMY_WORDS 8

#define EMU_PROFILE_TYPE_ISR   0
#define EMU_PROFILE_TYPE_TASK  1
#define EMU_PROFILE_TYPE_MOD   2
#define EMU_PROFILE_TYPE_SYS   3
#define EMU_PROFILE_TYPE_IDLE  4
#define EMU_PROFILE_TYPE_BWAIT 5
#define EMU_PROFILE_TYPE_NET   6
#define EMU_PROFILE_TYPE_TDM   7
#define EMU_PROFILE_TYPE_END   8

#define EMU_TASK_ID_MARK_END  0 /* Mark end of block, no output */

#define EMU_TASK_ID_TXRX_TASK(x) (1+x),EMU_PROFILE_TYPE_TASK
#define EMU_TASK_ID_TDURX        4,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_VCURX        5,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_MMCU         6,EMU_PROFILE_TYPE_IDLE
#define EMU_TASK_ID_TDUTX        7,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_VCUTX        8,EMU_PROFILE_TYPE_MOD

#define EMU_TASK_ID_PVP_TX       9,EMU_PROFILE_TYPE_NET  
#define EMU_TASK_ID_RCU_TX      10,EMU_PROFILE_TYPE_NET
#define EMU_TASK_ID_NEU_TX      11,EMU_PROFILE_TYPE_NET
#define EMU_TASK_ID_NDU_TX      12,EMU_PROFILE_TYPE_NET
#define EMU_TASK_ID_PVP_RX      13,EMU_PROFILE_TYPE_NET
#define EMU_TASK_ID_RCU_RX      14,EMU_PROFILE_TYPE_NET
#define EMU_TASK_ID_NEU_RX      15,EMU_PROFILE_TYPE_NET
#define EMU_TASK_ID_NDU_RX      16,EMU_PROFILE_TYPE_NET

#define EMU_TASK_ID_PIU_TX      17,EMU_PROFILE_TYPE_TDM
#define EMU_TASK_ID_PIU_RX      18,EMU_PROFILE_TYPE_TDM

#define EMU_TASK_ID_VPPU_TX     19,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_VPPU_RX     20,EMU_PROFILE_TYPE_MOD

#define EMU_TASK_ID_VTK_DECODE       21,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_CUSTOM_ECO       22,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_VTK_ENCODE       23,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_SIU_CACHE_INV    24,EMU_PROFILE_TYPE_MOD 
#define EMU_TASK_ID_SIU_CACHE_WBINV  25,EMU_PROFILE_TYPE_MOD
#define EMU_TASK_ID_TSU_TX           26,EMU_PROFILE_TYPE_MOD

/* 27 - 28 reserved for ADDITIONAL channel DEPENDENT profile points */

#define EMU_TASK_ID_HAL_TX      29,EMU_PROFILE_TYPE_NET
#define EMU_TASK_ID_MCBSP(x)    (30+x),EMU_PROFILE_TYPE_ISR
#define EMU_TASK_ID_UTRX        33,EMU_PROFILE_TYPE_ISR
#define EMU_TASK_ID_UTRXPOLL    34,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_UTTX        35,EMU_PROFILE_TYPE_ISR
#define EMU_TASK_ID_DSIG        36,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_ECUMA       37,EMU_PROFILE_TYPE_SYS 
#define EMU_TASK_ID_PRD         38,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_TXRXSWI     39,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_MAIN        40,EMU_PROFILE_TYPE_SYS

#define EMU_TASK_ID_MIPSWASTER  41 ,EMU_PROFILE_TYPE_IDLE
#define EMU_TASK_ID_TDMEDMASORT 42 ,EMU_PROFILE_TYPE_SYS

#define EMU_TASK_ID_GMACCOMM    43 
#define EMU_TASK_ID_GMACTX      44,EMU_PROFILE_TYPE_ISR
#define EMU_TASK_ID_GMACRX      45,EMU_PROFILE_TYPE_ISR 
#define EMU_TASK_ID_GMACPOLL    46,EMU_PROFILE_TYPE_SYS

#define EMU_TASK_ID_DPBADPRED   47,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_DPBUSYDDR   48,EMU_PROFILE_TYPE_BWAIT
#define EMU_TASK_ID_DPBUSYCTL   49,EMU_PROFILE_TYPE_BWAIT
#define EMU_TASK_ID_DPPRED      50,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_DPRELOC     51,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_DPBUSYDDR_MP 52,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_ECUMACTL    53,EMU_PROFILE_TYPE_SYS

#define EMU_TASK_ID_IDLE        60,EMU_PROFILE_TYPE_IDLE

/* 54 - 59 reserved for ADDITIONAL channel INDEPENDENT profile points */
#define EMU_TASK_ID_PKTFIFO     54,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_TECENG      55,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_RXVOICE     56,EMU_PROFILE_TYPE_SYS
#define EMU_TASK_ID_BARWAIT     57,EMU_PROFILE_TYPE_SYS


#define EMU_PROF_NOPUSHPOP 0
#define EMU_PROF_PUSH      1
#define EMU_PROF_POP       2
#define EMU_PROFILE_MAX    128

#define EMU_PROFILE_BEGIN(sect,ch) {                                         \
                                     tuint emu_profile_ch = ch,              \
                                     emu_profile_type =                      \
                                     emu_profile_mark (sect,               \
                                                         /* sect has type */ \
                                                         emu_profile_ch,     \
                                                         EMU_PROF_PUSH);
#define EMU_PROFILE_END()            emu_profile_mark (EMU_TASK_ID_MARK_END, \
                                                       emu_profile_type,     \
                                                       emu_profile_ch,       \
                                                       EMU_PROF_POP);        \
                                   }

void emu_profile_change_mode(void);
void emu_profile_flush(void);
static inline void emu_profile_mark_i(tuint task_id, tuint ch, tuint pushpop)
{
  extern tuint *emu_profile_buf, emu_profile_run_mode;
  extern tulong emu_profile_idx; 
  extern tulong emu_profile_size;
  tuint val3, *loc_ptr;
  tulong loc_idx;
  tulong loc_timer;
  
  asm(" dint");
  loc_timer = TSCL;
  val3 = (task_id << 10) | ((pushpop & 3) << 8) | (ch & 0xff);
  loc_idx = emu_profile_idx;
  loc_ptr = emu_profile_buf;
  loc_ptr[loc_idx++] = (tuint)(loc_timer >> 16);
  loc_ptr[loc_idx++] = (tuint)(loc_timer);
  loc_ptr[loc_idx++] = val3;
  emu_profile_idx = loc_idx;
  asm(" rint");
}
void emu_profile_mark_f(tuint task_id, tuint ch, tuint pushpop);

static inline tint emu_profile_mark(tuint task_id, tuint type,tuint ch, tuint pushpop) {
  extern tuint emu_profile_run_mode;
  bool exec = FALSE;
#ifdef EMU_PROFILE_FULL
  exec = TRUE;  
#endif

#ifdef EMU_PROFILE_REG
  switch(type)
  {
    case EMU_PROFILE_TYPE_IDLE:
    case EMU_PROFILE_TYPE_ISR:
    case EMU_PROFILE_TYPE_BWAIT:
    case EMU_PROFILE_TYPE_TASK:
    case EMU_PROFILE_TYPE_MOD:
    case EMU_PROFILE_TYPE_SYS:
      exec = TRUE;
      break;
    default:
      break;
  }
#endif

#ifdef EMU_PROFILE_LITE
  switch(type)
  {
    case EMU_PROFILE_TYPE_IDLE:
    case EMU_PROFILE_TYPE_ISR:
    case EMU_PROFILE_TYPE_BWAIT:
      exec = TRUE;
      break;
    default:
      break;
  }
#endif
  if (exec)
    emu_profile_mark_f(task_id, ch, pushpop);
  return(type);
}
#endif //EMUREG
