/************************************************************************/
/*                                                                      */
/*     Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                                                      */
/* File Name  : startupb.c                                              */
/*                                                                      */
/* DESCRIPTION: Initialize BSS sections for BIOS 6 build                */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#include "types.h"
#include "targethw.h"
#include <string.h>
#include "hal.h"

extern char _bss_core_begin;
extern char _bss_core_end;
extern char _far1_core_begin;
extern char _far1_core_end;
extern char _far2_core_begin;
extern char _far2_core_end;
extern char _far_ext_core_begin;
extern char _far_ext_core_end;
extern char _far_siuinst_core_begin;
extern char _far_siuinst_core_end;

extern cregister volatile unsigned int DNUM;

extern tbool siu_is_ibl(void);
extern void siu_ibl_download_core(void);

extern tulong siuVigdkDiagAppMem[];
extern const tulong siuVigdkDiagAppMemIbl[];

extern halReturn_t halMemConfig(halMemConfig_t *halMemCfg);
/******************************************************************************
 * FUNCTION PURPOSE: Initialized BSS sections
 ******************************************************************************
 * DESCRIPTION: The reset function initializes all the BSS sections for the
 *              BIOS6 build. It will be called prior to cinit() in the XDCtools
 *              runtime startup sequence. The hook is provided in the 
 *              xdc.runtime.Startup module. The static configuration in 
 *              bios6.cfg is as follows: 
 *
 *              var xdc_runtime_Startup = xdc.useModule('xdc.runtime.Startup');
 *              xdc_runtime_Startup.resetFxn = "&startup_reset";
 *****************************************************************************/
void startup_reset ()
{
  halMemConfig_t halMemCfg;

  /* MPAX configuration */
  halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_XMC_MPAX_CONFIG;
  if (DNUM == 0) {
      halMemCfg.memCfgFunction    |= HAL_MEMCFGFUNC_SES_MPAX_CONFIG;
      halMemCfg.memCfgFunction    |= HAL_MEMCFGFUNC_SMS_MPAX_CONFIG;
  }
  halMemConfig(&halMemCfg);

  /* For IBL Demos, core 0 downloads and starts remaining cores */
  if( siu_is_ibl() && (DNUM==0) ) {
    /* 
     * IBL downloads core 0.
     * This will download the remaining cores. 
     * The cores will be started the first time 
     * the background task is executed.
     */    
    memcpy(siuVigdkDiagAppMem,siuVigdkDiagAppMemIbl,0x100);
    siu_ibl_download_core();
  }

/* The following memset is needed when using ROM autoinitialization module,
   but not for RAM autoinitialization module */  
/*
  memset (&_bss_core_begin,  0, &_bss_core_end - &_bss_core_begin);
  memset (&_far1_core_begin, 0, &_far1_core_end - &_far1_core_begin);
  memset (&_far2_core_begin, 0, &_far2_core_end - &_far2_core_begin);
  memset (&_far_ext_core_begin, 0, &_far_ext_core_end - &_far_ext_core_begin);
  memset (&_far_siuinst_core_begin, 0, &_far_siuinst_core_end - &_far_siuinst_core_begin);
*/
}

/* nothing after this point */
