/************************************************************************/
/*                                                                      */
/*     Copyright (C) 2000  Texas Instruments Incorporated.              
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                   */
/************************************************************************/
/* FILE PURPOSE: TARGET DSP Hardware Configuration                      */
/************************************************************************/
/*                                                                      */
/* File Name  : targethw.c                                              */
/*                                                                      */
/* DESCRIPTION: Defines memory map, and structures which describe the   */
/*              TARGET hardware.                                        */
/*                                                                      */
/* REQUIRES:    targethw.h  (target hardware Defs)                      */
/*              c55xint.h   (TMS320C55x interrupt definitions)          */
/*              c55xreg.h   (TMS320C55x register definitions)           */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/* Tools Version : 1.60                                                 */
/*                                                                      */
/* Date : 8th November 2000                 OWNERSHIP : Ebenezer V.     */
/************************************************************************/

#include "types.h"
#include "targethw.h"
cregister volatile unsigned int CSR ;
cregister volatile unsigned int IER ;
#define GIE     (1U<<0)

/*****************************************************************************
 * DESCRIPTION: This variable is a Debug option for critical sections        
 *              embeddement implementation.
 *****************************************************************************/
/* #define THW_BIT_STACK_DBG */
#ifdef THW_BIT_STACK_DBG
  volatile tuint thwBitStack = 0;    /* It is used for stack depth measurement */
  volatile tint thwMaxBitStack = 0;  /* Max. bit-stack level */
  volatile tint thwMinBitStack = 0;  /* Min. bit-stack level */
#endif

/*****************************************************************************
 * DESCRIPTION: This variable is specific bit-stack implementation. It is 
 *            manipulated by thwCriticalBegin() and thwCriticalEnd() functions
 *            in order to properly recover status of INTM bit after 
 *            completion of critical section. Up to 16 bits can be shifted in
 *            and out, meaning that 15-level nesting of critical sections
 *            is allowed (highly unlikely).
 *****************************************************************************/          
//volatile tuint thwCriticalStatusSaved = 0;
unsigned int thwCriticalStatusSaved[MAX_NESTING_LEVEL];
int thwCriticalStatusSavedIdx = 0;

/******************************************************************************
 * FUNCTION PURPOSE: Disable All Interrupts
 ******************************************************************************
 * DESCRIPTION: Disable all interrupts without checks.
 *
 ******************************************************************************/
void thwDisableAllInt (void)
{
    IER = 0;
    CSR &= ~(GIE);
} /* thwDisableAllInt */



/******************************************************************************
 * FUNCTION PURPOSE: Start of critical section (up to 15 sections can be nested)
 *                   
 ******************************************************************************
 * DESCRIPTION:  Save current interrupt enable/disable status, and disable 
 *               all interrupts.
 *****************************************************************************/

void thwCriticalBegin (void)
{
  unsigned int old_val = _disable_interrupts();
  thwCriticalStatusSaved[thwCriticalStatusSavedIdx++] = old_val;
}

/******************************************************************************
 * FUNCTION PURPOSE: End of critical section (up to 15 sections can be nested)
 *                   
 ******************************************************************************
 * DESCRIPTION: Restore interrupt enable/disable status encountered on entry to
 *              critical section.
 *****************************************************************************/

void thwCriticalEnd (void)
{
  unsigned int old_val = thwCriticalStatusSaved[--thwCriticalStatusSavedIdx];
  _restore_interrupts(old_val);
}
/******************************************************************************
 * FUNCTION PURPOSE: Begin critical section; returns old value
 *****************************************************************************/
tulong thwCriticalBeginFastFcn(void) {
  return (tulong)_disable_interrupts();
}
/******************************************************************************
 * FUNCTION PURPOSE: End critical section; uses old value
 *****************************************************************************/
void thwCriticalEndFastFcn(tulong old_val) {
  _restore_interrupts(old_val);
}


/* nothing after this point */
