#ifndef _TARGETHW_INCLUDED
#define _TARGETHW_INCLUDED
/************************************************************************/
/*                                                                      */
/*     Copyright (C) 2000  Texas Instruments Incorporated.              
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                   */
/************************************************************************/
/* FILE PURPOSE: Header file for TARGET DSP Hardware Configuration      */
/************************************************************************/
/*                                                                      */
/* File Name  : targethw.h                                              */
/*                                                                      */
/* DESCRIPTION: Defines memory map, and structures which describe the   */
/*              TARGET hardware.                                        */
/*                                                                      */
/* REQUIRES:    types.h     (generic types)                             */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/* Tools Version : 1.60                                                 */
/*                                                                      */
/* Date : 8th November 2000                 OWNERSHIP : Ebenezer V.     */
/************************************************************************/

#include "types.h"

cregister volatile unsigned int CSR ;
#ifndef GIE
#define GIE      (1U<<0)
#endif

#define MAX_NESTING_LEVEL 16

void thwDisableAllInt(void);

/******************************************************************************
 * FUNCTION PURPOSE: Start of critical section (up to 15 sections can be nested)
 *                   
 ******************************************************************************
 * DESCRIPTION:  Save current interrupt enable/disable status, and disable 
 *               all interrupts. Bit ST1_INTM is pushed in global variable.
 *               IER0 and IER1 are saved and restored
 *****************************************************************************/

//#define HD /* DEFINED ONLY FOR HIGH DENSITY, DO NOT MERGE BACK FROM HD */
/* Inline intrinsic, to save mips in data path */
#define thwCriticalBeginFast _disable_interrupts
tulong thwCriticalBeginFastFcn();

/* Type to save from _disable_interrupts, and to pass to _restore_interrupts */
typedef tulong thwCriticalState_t;


/* Function, to save memory in control code */
void thwCriticalBegin(void);

/******************************************************************************
 * FUNCTION PURPOSE: End of critical section (up to 15 sections can be nested)
 *                   
 ******************************************************************************
 * DESCRIPTION: Restore interrupt enable/disable status encountered on entry to
 *              critical section. Bit ST1_INTM is poped from the global variable.
 *****************************************************************************/
#define thwCriticalEndFast(x) _restore_interrupts(x)
void thwCriticalEndFastFcn(tulong);
void thwCriticalEnd(void);


/******************************************************************************
 * FUNCTION PURPOSE: critical section reset
 *                   
 ******************************************************************************
 * DESCRIPTION: Reset the nested critical section related variables and enable
 *              interrupt. 
 *              
 * Note: This function is called when we need to jump out the potential nested
 *       critical sections and re-enable interrupt
 *****************************************************************************/
static inline void thwCriticalReset (void) {
  extern int thwCriticalStatusSavedIdx;
  thwCriticalStatusSavedIdx = 0;
  _enable_interrupts();
}

/******************************************************************************
 * FUNCTION PURPOSE: Return the base address of the HPI Memory Block
 *                   
 ******************************************************************************
 * DESCRIPTION: Return the base address of the HPI Memory Block. 
 *              The function resides in dpriram.c
 *****************************************************************************/
 void *thwDprBaseAddress (void);

/******************************************************************************
 * FUNCTION PURPOSE: Return the size of the HPI Memory Block
 *                   
 ******************************************************************************
 * DESCRIPTION: Return the size of the HPI Memory Block. 
 *              The function resides in dpriram.c
 *****************************************************************************/
tuint thwDprSize(void);



/* PCM Silence Codes */
#define THW_ALAW_SILENCE    0x55  /* Corresponds to -8.0F */
#define THW_MULAW_SILENCE   0x7F  /* Corresponds to 0.0F  */
#define THW_LINEAR_SILENCE     0  /* Any linear A/D & D/A */

#define UNMASK_XCHANNEL     1     /* unmask the transmit channel in McBSP */
#define MASK_XCHANNEL       0     /* mask the transmit channel in McBSP   */
#define MAX_TIMESLOT_NUM    32    /* max. timeslot number                 */
#define DUMMY_LIST_EMPTY    (0xffff) /* illegal timeslot value            */

#define  THW_NOERR      0

/*****************************************************************************
 * DESCRIPTION: return codes from thwCreateCodec 
 *****************************************************************************/
typedef enum { 
  THW_AIC_SUCCESS = 0,
  THW_AIC_FAILURE
} thwAicCreateReturn_t; 




/******************************************************************************
 * FUNCTION PURPOSE: Dummy function that is not required for C55X
 ******************************************************************************/
static inline void thw_change_data_page (tuint data_page)
{
/* Not required to be implemented for c55x */
} /* thw_change_data_page */

static inline int thw_get_data_page ()
{
/* Not required to be implemented for c55x */
return(0);
} /* thw_change_data_page */

static inline tint thw_read_data_page (void)
{
/* Not required to be implemented for c55x */
return(0);
} /* thw_read_data_page */

/******************************************************************************
 * FUNCTION PURPOSE: Set external flag pin
 ******************************************************************************
 * DESCRIPTION: Sets external flag pin.
 *****************************************************************************/
static inline void thwSetXFPin (void)
{
} /* thwSetXFPin */

/******************************************************************************
 * FUNCTION PURPOSE: REset external flag pin
 ******************************************************************************
 * DESCRIPTION: Resets external flag pin.
 *
 *****************************************************************************/

static inline void thwResetXFPin (void)
{
} /* thwResetXFPin */

/******************************************************************************
 * FUNCTION PURPOSE: RESET CLKOFF bit 
 ******************************************************************************
 * DESCRIPTION: RESETS CLKOFF bit
 *****************************************************************************/
static inline void thwResetCLKOFFPin (void)
{
} /* thwResetCLKOFFPin */

/******************************************************************************
 * FUNCTION PURPOSE: SET CLKOFF bit 
 ******************************************************************************
 * DESCRIPTION: SETS CLKOFF bit
 *****************************************************************************/
static inline void thwSetCLKOFFPin (void)
{
} /* thwSetCLKOFFPin */

static inline void thwDspLowPower (void)
{
  /* TODO */
}

void * thwReadSP (void);
int thwReadAMR (void);

#endif  /* _TARGETHW_INCLUDED */
/* nothing after this point */
