/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/******************************************************************************
 * FILE PURPOSE: GG memory buffers information decompression source file.
 ******************************************************************************
 * FILE NAME: vmmcomp.c 
 *              
 *
 * DESCRIPTION: getProgBuffInfo () is a library function that retrieves
 *              buffer information for the module on specific channel.
 *              Buffer information are compressed and stored in program
 *              memory. Query is made with three arguments:
 *              Actual module ID, channel number and type of module.
 *              Distinction is made between three types:
 *              1. Non-codec ECOs  (like ECU, TDU, FIU, ...)
 *              2. Encoder ECOs (like encoder ADPCM, encoder G729, ...)
 *              3. Decoder ECOs (like decoder ADPCM, decoder G729, ...)
 *              In case of Encoder\Decoder ECOs, VCT is passed and direction
 *              argument is set to 1, for TX path, i.e. encoders.
 *              For decoders, direction argument is 2 (RX path).
 * REVISION HISTORY:
 *              2000/05/12 dsenicic
 *              - First cut. C code implementation seems to be extremely
 *                ineffective in code consumption.
 ************************************************************************************/
#include "types.h"
#include "vmmcomp.h"            /* External declarations for VMM decompression */
#include "vmmcomp_port.h"       /* Definition of macros and some inline functions */

#ifndef GG_INCLUDE_VMMCOMP
  #include "ggChBufDesc.h"
  #include "ggswcfg.h"
  #include <stdio.h>
  //#include "vmm_port.h"
  #include "common/siuloc.h"
  #include "vctypes.h"
  #include "ggmbuf.h"
#endif





#ifdef GG_INCLUDE_VMMCOMP

extern const tint PrGGMBufScratchSize;
vmmTable16bitOffsets_t vmmTable16bitOffsets;
typedef void (*GetAddrFunPtr)(tuint *scratch);
void vmmTableOffsetsInit(void *ggvmmbufAddr,vmmTable16bitOffsets_t *vmmTableOffsets){

  vmmZipTable_t *table=(vmmZipTable_t *)ggvmmbufAddr;
  tuint offset=0;
  
  vmmTableOffsets->vmmHeader=offset;
  offset+= sizeof(vmmZStruct_t)/sizeof(tuint);
  offset+=(VMM_IDT_MAX*sizeof(vmmZTable_t))/sizeof(tuint);
  
  
  vmmTableOffsets->vmmChannelBaseAddressesTable=offset;
  offset+= table->vmm_loc.totalChannels*2;	   //+8??


  vmmTableOffsets->vmmVctConversionTable=offset;
  offset+= table->vmm_loc.sizeOfVCT2VCT*1;
  
/*
ethod A Compression Area for Non-Codec ECOs - (hdr_loc->cntModules * 5)
table of indices for Scratch bufferscntModules-(hdr_loc->scSize*1)
table of indices for LR buffers(hdr_loc->lrSize*1)
Method D-1 Compression Area for Non-Codec ECO (hdr_loc->d1Size*1)
Different Addresses for Common ECOs(hdr_loc->cntDiffAddr*2) 
Different Sizes for Common ECOs(hdr_loc->cntDiffAddr*2)
LR lookup table for Common ECOs(hdr_loc->cntLR*1)
*/

  
  /*--------------------- Common table-------------------*/
  vmmTableOffsets->vmmCommonEcoA1Table=offset;
  offset+= table->vmm_zip[0].cntModules*5;/*A1 size */
  offset+=table->vmm_zip[0].scSize*1;/* Scratch size */
  offset+=table->vmm_zip[0].lrSize*1;/* Lr size */

  vmmTableOffsets->vmmCommonEcoD1Table=offset;
  offset+= table->vmm_zip[0].d1Size*1;

  vmmTableOffsets->vmmCommonEcoDifferentAddTable=offset;
  offset+= table->vmm_zip[0].cntDiffAddr*2;

  vmmTableOffsets->vmmCommonEcoDifferentSizesTable=offset;
  offset+=table->vmm_zip[0].cntDiffSize*1;
  offset+=table->vmm_zip[0].cntLR*1;
  
  /*----------------- Encoder table----------------------*/
  vmmTableOffsets->vmmEncoderA1Table=offset;
  offset+= table->vmm_zip[1].cntModules*5;/*A1 size */
  offset+=table->vmm_zip[1].scSize*1;/* Scratch size */
  offset+=table->vmm_zip[1].lrSize*1;/* Lr size */

  vmmTableOffsets->vmmEncoderD1Table=offset;
  offset+= table->vmm_zip[1].d1Size*1;

  vmmTableOffsets->vmmEncoderDifferentAddTable=offset;
  offset+= table->vmm_zip[1].cntDiffAddr*2;

  vmmTableOffsets->vmmEncoderDifferentSizesTable=offset;
  offset+=table->vmm_zip[1].cntDiffSize*1;
  offset+=table->vmm_zip[1].cntLR*1;

  /*----------------------- Decoder table ----------------*/
  vmmTableOffsets->vmmDecoderA1Table=offset;
  offset+= table->vmm_zip[2].cntModules*5;/*A1 size */
  offset+=table->vmm_zip[2].scSize*1;/* Scratch size */
  offset+=table->vmm_zip[2].lrSize*1;/* Lr size */

  vmmTableOffsets->vmmDecoderD1Table=offset;
  offset+= table->vmm_zip[2].d1Size*1;

  vmmTableOffsets->vmmDecoderDifferentAddTable=offset;
  offset+= table->vmm_zip[2].cntDiffAddr*2;

  vmmTableOffsets->vmmDecoderDifferentSizesTable=offset;
  offset+=table->vmm_zip[2].cntDiffSize*1;
  offset+=table->vmm_zip[2].cntLR*1;

}


/************************************************************************************
* FUNCTION PURPOSE: Retrieve buffer information from program memory.
*************************************************************************************
* DESCRIPTION: First, direction variable is tested to define which sub-table should be
*   searched. Than MID is searched in that sub-table using value ecoOrCodecId.
*   If searching for buffer information for encoders or decoders, conversion from actual
*   VCT to generic VCT is done. This is because several VCT share same memory allocation
*   information. There is a small vector (10 items, or so), which only contains mapping
*   from in-VCTs to out-VCTs.
*   Buffers that go from heap are located by query from sub-table A.
*   Buffers that do not go from heap are searched in sub-table D-1, using the offset 
*   calculated from the preceding modules in A table Return value
*   is 1 if all information are found. Negative values are returned if any error has
*   happened, like:
*   - Wrong direction argument
*   - Module identifier not found
*   - During search for buffers with fixed addresses,
*   - PR_GGMBUF_SCRATCH size insufficient,   
*************************************************************************************/
tuint *baseLR_global=(tuint *)0x0;
tuint addrIndex_global=0xffff;
tulong val_baselr_addr_index=0;
tint vmmGetBuffInfo (
    tulong startProgAddr_long,   /* Input:   Starting address
                             *          of buff info for this application */
    tint ecoOrCodecId,      /* Input:   ECO/VCT identifier */
    vmmIdt_t vmmIdt,        /* Input:   identifier type
                             *          0 - non-codec ECOs
                             *          1 - encoder ECOs
                             *          2 - decoder ECOs */
    tuint chan,             /* Input:   channel number */
    tuint *memBuffNum,      /* Output:  Number of buffers */
    tuint swapBuffInfo[]    /* Output:  Buffer with raw buffer information
                             *  at offset (2 * i + 0) : buffer i, base address
                             *  at offset (2 * i + 1) : buffer i, size
                             * Returns: Error code
                             ********************************************************/
)

{
    vmmZipTable_t loc;
    vmmZTable_t   *one_loc;
    tuint i, vmid;
    tuint on_lrbbf = 0;              /* Initialize count of buffers that are channel-based */
    tuint on_sbbf = 0;               /* Initialize count of buffers that are scratch-based */
    tuint on_d1   = 0;               /* Initialize count of buffers that are D1 encoded (all-the-rest) */
    tuint buffCnt = 0;               /* Initialize number of buffers to be reported */
    tuint value;
    tuint  addrIndex;                /* Index into the vector of addresses */
    tuint  sizeIndex;                /* Index into the vector of sizes */
	tuint tblOffset;
    tulong mid_hbbf, lrbbf, sbbf, specbbf;
    tuint *tmpProgPtr;
    tuint *tmpLRProgPtr;  
    tuint *tmpSProgPtr;
    tuint *tmpD1ProgPtr; 
    tuint *baseDiffAddr;             /* Base of vector of different addresses */                     
    tuint *baseDiffSize;             /* Base of vector of different sizes */                     
    tuint *baseLR;                   /* Pointer to base of LR lookup table (offsets stored as 1 16-bit word) */
    tuint *swap_ptr;
    tuint all_d1    = 0;
    tuint all_lrbbf = 0;
    tuint all_sbbf  = 0;
    tuint d1_per_chan = 0;
    tulong channel_base_address;
    tuint *channel_base_address_word_ptr;
    vmmZipTable_t *loc_ptr = &loc;
    tuint *startProgAddr=(tuint *)startProgAddr_long;

    vmmTableOffsetsInit((void *)ggvmmbuf,&vmmTable16bitOffsets);
    
    /* Transfer table information to data memory (stack)  */
    vmmHeader(loc_ptr);
    /* Translation from in-VCT to out-VCT (if necessary) - only for Encoder & Decoder */
    if (vmmIdt != VMM_IDT_NONCODEC_ECO) 
	{
   	  vmmVctConversionTable (startProgAddr);
      vmm_vct2vct (&ecoOrCodecId, (tulong)startProgAddr, loc_ptr->vmm_loc.sizeOfVCT2VCT);
	}
    /* Fetch channel base info */
    vmmChannelBaseAddressesTable (startProgAddr);
    /* Read channel base address of the particular channel: *
     * two words are used for base address of each channel  */
    tmpProgPtr = (tuint *)startProgAddr;
    channel_base_address = (tulong)tmpProgPtr[chan * 2];
    channel_base_address <<= 16;
    channel_base_address |= (tulong)tmpProgPtr[chan * 2 + 1];
    channel_base_address_word_ptr=(tuint *)channel_base_address;

    /* Point to selected group */
    one_loc = (vmmZTable_t *)&(loc_ptr->vmm_zip[vmmIdt]);  /* 0, 1, 2 */


    /* Set initial value of pointer */
    
    /* Bring in complete group table (all modules from same group) to scratch: */
    switch(vmmIdt)
    {
      case VMM_IDT_NONCODEC_ECO:
             vmmCommonEcoA1Table(startProgAddr);
             break;
      case VMM_IDT_ENCODER_ECO:
             vmmEncoderA1Table(startProgAddr);
             break;
      case VMM_IDT_DECODER_ECO:
             vmmDecoderA1Table(startProgAddr);
             break;
    }

	
    tblOffset  = one_loc->cntModules * VMM_ECO_BFIELD_SIZE + one_loc->lrSize + one_loc->scSize + one_loc->d1Size;    
    baseDiffAddr = (tuint *)(startProgAddr + (tulong)tblOffset);
	tblOffset += 2 * one_loc->cntDiffAddr;
    baseDiffSize = (tuint *)(startProgAddr + (tulong)tblOffset);
	tblOffset += one_loc->cntDiffSize;
    baseLR       = (tuint *)(startProgAddr + (tulong)tblOffset);
    tblOffset += one_loc->cntLR;
    
    baseLR_global=baseLR;
    /* Check if Size of Scratch is big enough to hold complete table */
    if(tblOffset > (tuint)PrGGMBufScratchSize) 
       return (VMM_ERR_SCRATCH_INSUFFICIENT);

    
    
    tmpProgPtr=startProgAddr; /* StartProgAddr now points to the beginning of A1 table */
    
    
    
    /* Search for identifier through sub-table A */
    for (vmid = 0; vmid < one_loc->cntModules; vmid ++) 
    {
      /* Read two words to make HBBF */
      mid_hbbf = (tulong)(*tmpProgPtr ++);
      mid_hbbf <<= 16;
      mid_hbbf |= (tulong)(*tmpProgPtr ++);
      /* 3 16-bit words are used for storage of 2 24 bit fields: LRBBF and SBBF */
      lrbbf  = (tulong)(*tmpProgPtr ++);
      lrbbf <<= 16;
      lrbbf |= (tulong)(*tmpProgPtr ++);
      sbbf = lrbbf;
      lrbbf >>= 8;
      sbbf &= 0x000000ff;
      sbbf <<= 16;
      sbbf |= (tulong)(*tmpProgPtr++);
      /* This bitfield indicates all buffers encoded as heap, LR or scratch buffers */
      /* Bits set to 0 indicate buffers encoded using rule D1                       */

      /* Extract module identifier from HBBF to test against one that is searched for. */
      if (ecoOrCodecId != (tuint)extractMid(mid_hbbf, MAX_BUFF))
      { /* Not this module */
        tuint bufferCount = vmm_determineCount (mid_hbbf); /* find number of ECO buffers */
        specbbf = mid_hbbf | lrbbf | sbbf;
        for (i = 0; i < bufferCount; i ++, specbbf >>= 1)
        {
          if(!(specbbf & 0x01L)) all_d1 ++;
        }
        specbbf = lrbbf;
        for (i = 0; i < bufferCount; i ++, specbbf >>= 1)
        {
          if(specbbf & 0x01L) all_lrbbf ++;
        }
        specbbf = sbbf;
        for (i = 0; i < bufferCount; i ++, specbbf >>= 1)
        {
          if(specbbf & 0x01L) all_sbbf ++;
        }
      }
      else {
        /* Module is found !
         *  Now, determine number of buffers. It is equal to bit 
         *   position of first bit equal to 1, starting from 
         *   position 24, and going to lower positions. */
        buffCnt = vmm_determineCount (mid_hbbf);
        specbbf = mid_hbbf | lrbbf | sbbf;
        for (i = 0; i < buffCnt; i ++, specbbf >>= 1)
        {
          if(!(specbbf & 0x01L)) d1_per_chan ++;
        }        
        break;
      }
    }
    *memBuffNum = buffCnt; /* Report number of buffers in the module */    
    
    if (buffCnt == 0) return (VMM_ERR_MODULE_NOT_FOUND);  /* Module not found ! */
    /* Skip LR buffers from modules that are stored before the requested one */
    tmpLRProgPtr  = (tuint *)(startProgAddr + (VMM_ECO_BFIELD_SIZE * one_loc->cntModules) + (tulong)all_lrbbf);
    /* Skip scratch buffers from modules that are stored before the requested one */
    tmpSProgPtr   = (tuint *)(startProgAddr + (VMM_ECO_BFIELD_SIZE * one_loc->cntModules) + one_loc->lrSize + (tulong)all_sbbf);
    /* Skip D1 buffers from modules that are stored before the requested one */
    tmpD1ProgPtr  = (tuint *)(startProgAddr + (tulong)(VMM_ECO_BFIELD_SIZE * one_loc->cntModules) + 
                              (tulong)(one_loc->lrSize + one_loc->scSize) + 
                              (tulong)(all_d1 * loc_ptr->vmm_loc.totalChannels + d1_per_chan * chan));
    /* Buffers that go from heap are encoded with 1 at the bit position equal to its number. */
    swap_ptr = (tuint *)swapBuffInfo; 
    for (i = 0; i < buffCnt; i ++)
    {
      if (testBit(mid_hbbf, i))
      { /* This buffer is on heap. */
        *swap_ptr++ = 0;       /* to take care of address higher byte*/
        *swap_ptr++ = 0;       /* This is indication that buffer is allocated from the heap. */
        *((int *)swap_ptr) = 0;       /* And the information on buffer size will be taken from ecoGetSize() */
      	swap_ptr += sizeof(int)/sizeof(tuint);
      }
      else {
        if (testBit(lrbbf, i))
        { /* Read LR buffer indices into stack array */
          value = tmpLRProgPtr[on_lrbbf ++];
          /* Upper byte contains index of actual address */
          addrIndex = MSBtenBits (value); /* index into vector of LR offsets */
          
          addrIndex_global=addrIndex;
          val_baselr_addr_index =(tulong)baseLR[addrIndex];
          sizeIndex = LSBsixBits (value); /* index into vector of different sizes */
          *((tulong  *)swap_ptr) = channel_base_address + baseLR[addrIndex];
		  swap_ptr ++;
		  swap_ptr ++;
        } else {
          if (testBit(sbbf, i))
          { /* Scratch based buffer */
            value = tmpSProgPtr[on_sbbf ++];
            addrIndex = MSBtenBits (value); /* index into vector of different addresses */
            sizeIndex = LSBsixBits (value); /* index into vector of different sizes */
          } else { /* The rest of buffers encoded using D1 table */
            value = tmpD1ProgPtr[on_d1 ++];
            addrIndex = MSBtenBits (value); /* index into vector of different addresses */
            sizeIndex = LSBsixBits (value); /* index into vector of different sizes */
          }
          *swap_ptr ++ = baseDiffAddr[2 * addrIndex];
          *swap_ptr ++ = baseDiffAddr[2 * addrIndex + 1];
        }
        *((int *)swap_ptr) = baseDiffSize[sizeIndex];
       	swap_ptr += sizeof(int)/sizeof(tuint);
      }
//      *swap_ptr ++ = 0;
    }

    return (VMM_ERR_OK); /* Data available ! */
}
#else
/* VMM Compression need not be present in external memory builds, as there is a lot of room to
  keep big tables. Also, it makes debugging easier by making buffer addresses easier to find */
typedef struct {
   int  numBuffs;
   ggbufDesc_t bufDesc[1];
} ggbuffDescXXX_t;

tint vmmGetBuffInfo_NonCodecEco(ggchBuffDesc_t ggchBuff, tint ecoOrCodecId,
                                tuint *memBuffNum, ggbufDesc_t swapBuffInfo[]);

tint vmmGetBuffInfo_CodecEco(ggchBuffDesc_t ggchBuff, tint ecoOrCodecId,
                               vmmIdt_t vmmIdt, tuint *memBuffNum,
                               ggbufDesc_t swapBuffInfo[]);

/************************************************************************************
 * FUNCTION PURPOSE: Retrieve buffer information from program memory.
 *************************************************************************************
 * DESCRIPTION: First, direction variable is tested to define which sub-table should be
 *   searched. Than MID is searched in that sub-table using value ecoOrCodecId.
 *   If searching for buffer information for encoders or decoders, conversion from actual
 *   VCT to generic VCT is done. This is because several VCT share same memory allocation
 *   information. There is a small vector (10 items, or so), which only contains mapping
 *   from in-VCTs to out-VCTs.
 *   Buffers that go from heap are located by query from sub-table A.
 *   Buffers that do not go from heap are searched in sub-table D-1, with key formed
 *   of MID/VCT and chan. When key is found all the information are available. Return value
 *   is 1 if all information are found. Negative values are returned if any error has
 *   happened, like:
 *   - Wrong direction argument
 *   - Module identifier not found
 *   - During search for buffers with fixed addresses,   
 *************************************************************************************/
 



tint vmmGetBuffInfo (
  tulong startProgAddr,   /* Input:   Starting address
                           *          of buff info for this application */
  tint ecoOrCodecId,      /* Input:   ECO/VCT identifier */
  vmmIdt_t vmmIdt,        /* Input:   identifier type
                           *          0 - non-codec ECOs
                           *          1 - encoder ECOs
                           *          2 - decoder ECOs */
  tuint chan,             /* Input:   channel number */
  tuint *memBuffNum,      /* Output:  Number of buffers */
  tuint *swapBuffInfo    /* Output:  Buffer with raw buffer information
                           *  at offset (2 * i + 0) : buffer i, base address
                           *  at offset (2 * i + 1) : buffer i, size
                           * Returns: Error code
                           ********************************************************/
  )
     
{
   tint retVal = 0;
   
   switch (vmmIdt)
   {
      case VMM_IDT_NONCODEC_ECO:     /* non-codec ECOs */
         retVal = vmmGetBuffInfo_NonCodecEco(ggchBuffDesc[chan], ecoOrCodecId,
                                             memBuffNum, (ggbufDesc_t *)swapBuffInfo);
         break;
         
#if defined(VOICE) || defined(VOICE_FAX)
      case VMM_IDT_ENCODER_ECO:     /* encoder ECOs -->  vmmIdt == 1 */
      case VMM_IDT_DECODER_ECO:     /* decoder ECOs -->  vmmIdt == 2 */
         retVal = vmmGetBuffInfo_CodecEco(ggchBuffDesc[chan], ecoOrCodecId, vmmIdt,
                                          memBuffNum, (ggbufDesc_t *)swapBuffInfo);
         break;
#endif
         
      default:
         /* ERROR! Should never arrive here. */
         retVal = VMM_ERR_MODULE_NOT_FOUND;
         break;
   }

   return retVal;
}

tint vmmGetBuffInfo_NonCodecEco(ggchBuffDesc_t ggchBuff, tint ecoOrCodecId,
                                tuint *memBuffNum, ggbufDesc_t swapBuffInfo[])
{
   tint i = 0;
   ggbuffDescXXX_t *ggchBuff_ptr = NULL;
   
   switch (ecoOrCodecId)
   {
#if defined(VOICE) || defined(VOICE_FAX)
      case SIU_MID_VCU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescVCU;
         break;
#endif

#if defined(VOICE) || defined(VOICE_FAX)
      case SIU_MID_PIU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescPIU;
         break;
#endif

#ifdef GG_INCLUDE_TDU
      case SIU_MID_TDU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTDU;
         break;
#endif

#ifdef GG_TDU_SAVEBUF_EXT
      case SIU_MID_TDUFAX:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTDUFax;
         break;
#endif 

#ifdef LEC_ECU
      case SIU_MID_ECU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescECU;
         break;
#endif

#ifdef LEC_TEC
	  case SIU_MID_TEC:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTEC;
         break;
#endif
#if defined(VOICE) || defined(VOICE_FAX)

#ifdef LEC_TECENG
	  case SIU_MID_TECENG_CHAN:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTECENG_CHAN;
         break;
#endif
#if defined(VOICE) || defined(VOICE_FAX)
      case SIU_MID_PVP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescPVP;
         break;

      case SIU_MID_VPU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescVPU;
         break;
#endif
#endif
         
#if defined(FAX) || defined(VOICE_FAX)
      case SIU_MID_FIU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescFIU;
         break;
#endif

#if defined(VOICE) || defined(VOICE_FAX)
      case SIU_MID_TGU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTGU;
         break;

      case SIU_MID_CID:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescCID;
         break;
#endif
#if (defined(GG_INCLUDE_AAL1) || defined(GG_INCLUDE_AAL2))
      case SIU_MID_ACU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescACU;
         break;
#endif

#ifdef GG_INCLUDE_RSU
      case SIU_MID_RSU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescRSU;
         break;
#endif

#ifdef GG_INCLUDE_AER
      case SIU_MID_AER:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescAER;
         break;
#endif
#if defined(VOICE) || defined(VOICE_FAX)
      case SIU_MID_VAU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescVAU;
         break;
#endif

#ifdef GG_INCLUDE_MSU
      case SIU_MID_MSU_RTP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescMSURTP;
	     break;
      case SIU_MID_MSU_RTCP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescMSURTCP;
         break;

#endif

#ifdef GG_INCLUDE_AER
      case SIU_MID_AGC:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescAGC;
         break;
#endif
#ifdef GG_INCLUDE_RTP
      case SIU_MID_RCU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescRCU;
         break;
#endif

#ifdef GG_INCLUDE_NET_ENCAP
      case SIU_MID_NEU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescNEU;
         break;
#endif

#ifdef GG_INCLUDE_ANNOUNCEMENTS
      case SIU_MID_ANU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescANU;
         break;
#endif

#ifdef GG_INCLUDE_PRU
      case SIU_MID_PRU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescPRU;
         break;
#endif

#if defined(GG_INCLUDE_DATA_MODEM) || defined(GG_INCLUDE_LINK_LAYER_MODEM_RELAY)
      case SIU_MID_DMP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescDMP;
         break;
#endif

#if (defined (GG_INCLUDE_CLEAR_DATA) || defined (GG_INCLUDE_FAX_MODEM) || defined (GG_INCLUDE_DATA_MODEM) || defined (AUDIO))
      case SIU_MID_MPP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescMPP;
         break;
#endif
#if defined (FAX) || defined (VOICE_FAX)
      case SIU_MID_FM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescFM;
         break;
#endif
#ifdef GG_INCLUDE_MODEM_RELAY
      case SIU_MID_MRU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescMRU;
         break;
#endif

#ifdef GG_INCLUDE_BERT
      case SIU_MID_BERT:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescBERT;
         break;
#endif

#ifdef GG_INCLUDE_LINK_LAYER_MODEM_RELAY
      case SIU_MID_LLMRU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescLLMRU;
         break;

      case SIU_MID_SPRTP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescSPRTP;
         break;
#endif

#ifdef GG_INCLUDE_VQM
      case SIU_MID_VQM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescVQM;
         break;
#endif

#ifdef NX64
      case SIU_MID_NPIU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescNPIU;
         break;
#endif

#ifdef GG_INCLUDE_MDMCP
      case SIU_MID_MDMCP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescMDMCP;
         break;
#endif

#if defined(VOICE) || defined(VOICE_FAX)
      case SIU_MID_PLC:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescPLC;
         break;

      case SIU_MID_TX_NMU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTxNMU;
         break;

      case SIU_MID_RX_NMU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescRxNMU;
         break;
#endif

#if defined (AUDIO)
      case SIU_MID_ADU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescADU;
         break;
#endif

#ifdef NX64
       case SIU_MID_NXU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescNXU;
         break;
#endif

#ifdef HDVIDEO
       case SIU_MID_VTK:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescVTK;
         break;
       case SIU_MID_ATK:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescATK;
         break;
       case SIU_MID_TSU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTSU;
         break;
       case SIU_MID_CUSTOM_ECO:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescCUSTOM_ECO;
         break;
       case SIU_MID_OVLY:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescOVLY;
         break;
       case SIU_MID_MMCU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescMMCU;
         break;
#endif
#if (defined(HDCT) || defined(HDVIDEO))
       case SIU_MID_VPPU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescVPPU;
         break;
       case SIU_MID_TFTP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTFTP;
         break;
#endif

#ifdef VEU
      case SIU_MID_VEU:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescVEU;
         break;
#endif
	  
#if (defined(PLRRED) || defined(PLRFEC))
   #ifdef PLR_VOICE_OVERLAY
     #if (MAX_RED_LEVEL_TEXT > 0)
       case SIU_MID_PLR:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescPLR; /* For TEXT */
		 break;
     #endif
     #if ((MAX_RED_LEVEL_VOICE > 0) || (MAX_FEC_LEVEL_2198 > 0))
       case SIU_MID_PLRVBD:
       ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescPLRVBD; /* For VBD */
	    break;
     #endif
  #else /* OVERLAY */
  
      case SIU_MID_PLR:  
        ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescPLR; /* For everything */
	    break;
   #endif
#endif /* PLR */

#ifdef TTU
	  case SIU_MID_TTU:
	    ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTTU;
		break;
#endif	    

#ifdef PLR_VOICE_OVERLAY
      case SIU_MID_TDULITE:
		ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescTDULITE;
		break;
#endif

#ifdef EQI
	  case SIU_MID_EQI:
		ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescEQI;
		break;
#endif

#ifdef GG_MULTSAMPVC
      case SIU_MID_MULTSAMP:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggchBuff.buffDescMultsampvc;
         break;
#endif

      default:
         ggchBuff_ptr = NULL;
         break;
   }

   if(ggchBuff_ptr == NULL)
      return (VMM_ERR_MODULE_NOT_FOUND);


   *memBuffNum = (tuint) ggchBuff_ptr->numBuffs;
   for(i = 0; i < *memBuffNum; i++)
   {
      swapBuffInfo[i].buffAddr = ggchBuff_ptr->bufDesc[i].buffAddr;
      swapBuffInfo[i].buffSize = ggchBuff_ptr->bufDesc[i].buffSize;
   }

   return (VMM_ERR_OK); /* Data available ! */
}

#if defined(VOICE) || defined(VOICE_FAX)
tint vmmGetBuffInfo_CodecEco(ggchBuffDesc_t ggchBuff, tint ecoOrCodecId,
                               vmmIdt_t vmmIdt, tuint *memBuffNum,
                               ggbufDesc_t swapBuffInfo[])
{
   tint i = 0;
   ggbuffDescXXX_t *ggchBuff_ptr = NULL;
   ggbuffDescCodec_t *ggbuffDescCodec_ptr = NULL;


   if(vmmIdt == 1)
      ggbuffDescCodec_ptr = (ggbuffDescCodec_t *) &ggchBuff.buffDescEncoder;
   else if(vmmIdt == 2)
      ggbuffDescCodec_ptr = (ggbuffDescCodec_t *) &ggchBuff.buffDescDecoder;
   else
      return VMM_ERR_MODULE_NOT_FOUND;


   switch (ecoOrCodecId)
   {
/*    case VCA_NULL:  */
/*
      case VCT_8BITS:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescVCA;
         break;
*/

#ifdef GG_INCLUDE_PCM
      case VCT_8APCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescPCM;
         break;

      case VCT_8MUPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescPCM;
         break;
#endif

#ifdef GG_INCLUDE_G726
      case VCT_2ADPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;

      case VCT_3ADPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;

      case VCT_4ADPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;

      case VCT_5ADPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;

      case VCT_4ADPCM_MSBF:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;

      case VCT_5ADPCM_MSBF:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;
#endif

#ifdef GG_INCLUDE_G728
      case EVCT_LDCELP_16K:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG728;
         break;
#endif

#ifdef GG_INCLUDE_G729
      case EVCT_G729:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG729;
         break;
#endif

#ifdef GG_INCLUDE_G729AB
	  case EVCT_G729AB:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG729AB;
         break;
#endif

#ifdef GG_INCLUDE_G729E
      case EVCT_G729E:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG729E;
         break;
#endif

#ifdef GG_INCLUDE_G726
      case VCT_2ADPCM_MSBF:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;

      case VCT_3ADPCM_MSBF:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescADPCM;
         break;
#endif

#ifdef GG_INCLUDE_GSMEFR
      case EVCT_GSMEFR:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescGSMEFR;
         break;
#endif

#ifdef GG_INCLUDE_GSMFR
      case EVCT_GSMFR:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescGSMFR;
         break;
#endif
         
#ifdef GG_INCLUDE_G723
      case EVCT_G723_53:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG723;
         break;
         
      case EVCT_G723_63:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG723;
         break;
#endif

#ifdef GG_INCLUDE_G722         
      case VCT_E8_6ADPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG722;
         break;
         
      case VCT_E7_5ADPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG722;
         break;
         
      case VCT_E6_4ADPCM:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescG722;
         break;
#endif

#ifdef GG_INCLUDE_PCM         
      case EVCT_DATA:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescPCM;
         break;
#endif

#ifdef GG_INCLUDE_ILBC
      case EVCT_ILBC_RATE_20MS:
	  case EVCT_ILBC_RATE_30MS:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescILBC;
         break;
#endif

#ifdef GG_INCLUDE_GSMAMR
      case EVCT_GSMAMR:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescGSMAMR;
         break;
#endif

#ifdef GG_INCLUDE_G722_2
      case EVCT_GSM_WBAMR:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescGSM_WBAMR;
         break;
#endif
#ifdef GG_INCLUDE_EVRC 
      case EVCT_EVRCFULL:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescEVRC;
         break;
#endif

#ifdef GG_INCLUDE_SMV                                       
      case EVCT_SMV:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescSMV;
         break;

#endif

#ifdef GG_INCLUDE_QCELP13
      case EVCT_QCELP13:
         ggchBuff_ptr = (ggbuffDescXXX_t *) &ggbuffDescCodec_ptr->buffDescQCELP13;
         break;

#endif

      default:
         ggchBuff_ptr = NULL;
         break;
   }
   
   if(ggchBuff_ptr == NULL)
      return (VMM_ERR_MODULE_NOT_FOUND);
   
   
   *memBuffNum = (tuint) ggchBuff_ptr->numBuffs;
   for(i = 0; i < *memBuffNum; i++)
   {
      swapBuffInfo[i].buffAddr      =    ggchBuff_ptr->bufDesc[i].buffAddr;
      swapBuffInfo[i].buffSize     =     ggchBuff_ptr->bufDesc[i].buffSize;
   }
   
   return (VMM_ERR_OK); /* Data available ! */
} 
#endif
#endif
/* nothing past this point */

