/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/******************************************************************************
 * FILE PURPOSE: External definitions for the GG memory buffer information
 *                   decompression.
 ******************************************************************************
 * FILE NAME:    vmmcomp.h 
 *
 ******************************************************************************/
#include "types.h"


#ifndef _VMMCOMP_H
#define _VMMCOMP_H

#define MAX_BUFF 24

/* Definition of report codes returned by getProgBuffInfo() */
#define VMM_ERR_ILLEGAL_DIRECTION    (-1)
#define VMM_ERR_MODULE_NOT_FOUND     (-2)
#define VMM_ERR_DIMENSION_EXCEEDED   (-3)
#define VMM_ERR_SCRATCH_INSUFFICIENT (-4)
#define VMM_ERR_OK                   (1)

#ifndef tsize
# define tsize tuint
#endif

/* Valid values for direction */
typedef enum {
  VMM_IDT_NONCODEC_ECO = 0,        /* Indication for common ECO */
  VMM_IDT_ENCODER_ECO  = 1,        /* Indication for encoder ECO */
  VMM_IDT_DECODER_ECO  = 2,        /* Indication for decoder ECO */
  VMM_IDT_MAX          = 3    
} vmmIdt_t;


/******************************************************************************
 * DATA DEFINITION: Definition of header of compressed table
 ******************************************************************************
 * DESCRIPTION: Contains necessary information to deal with fields of variable 
 *                  length.
 *****************************************************************************/
typedef struct {
    tuint cntModules;                
    tuint lrSize;
    tuint scSize;
    tuint d1Size;                

    tuint cntDiffAddr;      /* Count of different buffer base addresses */
    tuint cntDiffSize;      /* Count of different buffer sizes */
    tuint cntLR;            /* Count of LR offsets */
} vmmZTable_t;

typedef struct {
    tsize sizeOfVCT2VCT;    /* Size of in-VCT to out-VCT translation table */
    tuint totalChannels;
} vmmZStruct_t;

typedef struct {
    vmmZStruct_t vmm_loc;
    vmmZTable_t vmm_zip[VMM_IDT_MAX];
}  vmmZipTable_t;

/* This is a library function that returns all buffer information for a single module,
 *  on a particular channel. */
extern tint vmmGetBuffInfo (
    tulong startProgAddr,        /* Input:    Starting address
                                 *             of buff info for this application */
    tint ecoOrCodecId,           /* Input:     ECO identifier */
    vmmIdt_t vmmIdt,             /* Input:     ECO type:
                                 *          0 - non-codec ECOs
                                 *            1 - encoder ECOs
                                 *            2 - decoder ECOs */
    tuint chan,                  /* Input:     channel number */
    tuint *memBuffNum,           /* Output:  number of buffers */
    tuint memBuffInfo[]          /* Output:     raw buffer information */
);

#define VMM_ECO_BFIELD_SIZE 5
#define VMM_HEADER_SIZE  (sizeof(vmmZipTable_t) / sizeof(tuint))

#endif
/* nothing past this point */

