/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/******************************************************************************
 * FILE PURPOSE: Non-portable definitions for the GG memory buffer information
 *                 decompression.
 ******************************************************************************
 * FILE NAME:    vmmcomp_port.h - This instance is for C54x implementaion.
 *
 ******************************************************************************/
#ifndef _VMMCOMP_PORT_H
#define _VMMCOMP_PORT_H

#include "types.h"
#include "utlswap.h"

extern tuint ggvmmbuf[];
typedef struct vmmTable16bitOffsets_s {
  tuint vmmHeader ;
  tuint vmmChannelBaseAddressesTable;
  tuint vmmVctConversionTable;
  /* Common table */
  tuint vmmCommonEcoA1Table;
  tuint vmmCommonEcoD1Table;
  tuint vmmCommonEcoDifferentAddTable;
  tuint vmmCommonEcoDifferentSizesTable;
 
  /* Encoder table */
  tuint vmmEncoderDifferentAddTable;      
  tuint vmmEncoderDifferentSizesTable;      
  tuint vmmEncoderA1Table;      
  tuint vmmEncoderD1Table;      

  /* Decoder table */
  tuint vmmDecoderA1Table;
  tuint vmmDecoderD1Table;
  tuint vmmDecoderDifferentAddTable;
  tuint vmmDecoderDifferentSizesTable;
     
} vmmTable16bitOffsets_t;

extern vmmTable16bitOffsets_t vmmTable16bitOffsets;

#define vmmHeader(ptr)  (ptr =(void *)&ggvmmbuf[vmmTable16bitOffsets.vmmHeader])  
#define vmmChannelBaseAddressesTable(ptr)  (ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmChannelBaseAddressesTable])  
#define vmmVctConversionTable(ptr) (ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmVctConversionTable])  

#define vmmCommonEcoA1Table(ptr) (ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmCommonEcoA1Table] ) 
#define vmmCommonEcoD1Table(ptr) (ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmCommonEcoD1Table] ) 
#define vmmCommonEcoDifferentAddTable(ptr) (ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmCommonEcoDifferentAddTable] ) 
#define vmmCommonEcoDifferentSizesTable(ptr) ( ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmCommonEcoDifferentSizesTable] ) 

#define vmmEncoderDifferentAddTable(ptr) (ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmEncoderDifferentAddTable] ) 
#define vmmEncoderDifferentSizesTable(ptr) ( ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmEncoderDifferentSizesTable] ) 
#define vmmEncoderA1Table(ptr) (ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmEncoderA1Table]) 
#define vmmEncoderD1Table(ptr) ( ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmEncoderD1Table]) 

#define vmmDecoderA1Table(ptr) ( ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmDecoderA1Table]) 
#define vmmDecoderD1Table(ptr) ( ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmDecoderD1Table]) 
#define vmmDecoderDifferentAddTable(ptr) ( ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmDecoderDifferentAddTable]) 
#define vmmDecoderDifferentSizesTable(ptr) ( ptr = &ggvmmbuf[vmmTable16bitOffsets.vmmDecoderDifferentSizesTable]) 

#define CHAN_SIZE       5   /* Number of bits used to encode channel number
                             *  in sub-table D1 */
#define LEFT_CNT_SIZE   5   /* Number of bits used to encode number of buffers
                             *  in an item in sub-table D1 */
#define MAX_CHAN_NUM    (1<<CHAN_SIZE)

#define SIZE_OF_STRUCT(a)  (sizeof(a)/sizeof(tuint))  /* Size of header information
                                                       *  in tuints */
#define SIZE_OF_STRUCT_LONG(a)  (sizeof(a)/sizeof(tulong))  /* Size of header information in tulong */
#define VMM_CHUNK_SIZE  600
#define VMM_SCRATCH_FUDGE_FACTOR 100 /* FUDGE Factor needs to  be maximum of */
/* MACRO definitions */
#define upperByte(a)        ((a >> 8) & 0x00ff)       
#define lowerByte(a)        (a & 0x00ff)
#define testBit(a, b)       ((a >> b) & 0x00000001)
#define extractMid(a, b)    ((a >> b) & 0x000000ff)
#define getKey(a)           (a & 0xffe0)
#define getLeftCount(a)     (a & 0x001f)
#define MSBnineBits(a)      ((a >> 7 ) & 0x01ff ) /* Gets the MSB 9 bits' value */
#define LSBsevenBits(a)      (a & 0x007f )       /* Gets the LSB 7 bits' value */
#define MSBtenBits(a)      ((a >> 6 ) & 0x03ff ) /* Gets the MSB 10 bits' value */
#define LSBsixBits(a)      (a & 0x003f )       /* Gets the LSB 6 bits' value */
#define MSBelevenBits(a)      ((a >> 5 ) & 0x07ff ) /* Gets the MSB 10 bits' value */
#define MSBtwelveBits(a)      ((a >> 6 ) & 0x0fff ) /* Gets the MSB 10 bits' value */
#define LSBfiveBits(a)      (a & 0x001f )       /* Gets the LSB 6 bits' value */


#if _LITTLE_ENDIAN /* if operating in LITTLE ENDIAN mode swap tuint to create proper tulong */
#define SWAP_TUINT(input)  (input = ((input >> 16) & 0xFFFF) | input << 16) /* Needed for LITTLE ENDIAN MODE */
#else
#define SWAP_TUINT(input)  (input = input)
#endif
/*************************************************************************************
* FUNCTION PURPOSE: Determine total number of buffers from HBBF
**************************************************************************************
* DESCRIPTION: HBBF holds information on number of buffers in bit position of first
*               non-zero bit. E.g. Number of DPU buffers (21) is encoded in this way:
*               [xxxxxxxx001........................]
*                31      23
**************************************************************************************/
static inline tuint vmm_determineCount (tulong mid_hbbf)
{
    tuint i; 
    tlong temp_mid_hbbf = (tlong) mid_hbbf & 0x00FFFFFF; 
    i = (tuint) _norm(temp_mid_hbbf); /* use _norm instead of _lnorm since long in c6x is 40 bits */
    i = 32 - i - 1 - 1; /* for converting 32 to 24 bits + 1 sign bit + 1 bit offset */ 
    return (i);
}
/*************************************************************************************
* FUNCTION PURPOSE: Read a tuint from program memory
**************************************************************************************
* DESCRIPTION: One word is read from location prg_address from program memory.
*   tuint vmm_prgRead ( 
*       tulong prg_address,  - address in program memory
*       tuint  *ret_value    - value at location prg_address in program memory
*   )   
**************************************************************************************/
static inline void vmm_prgRead (tulong prg_address, tuint *ret_value)   
{
    utlMoveProg2Data(prg_address, (tulong)ret_value, 1);                                                       
}
/*************************************************************************************
* FUNCTION PURPOSE: Read a tulong from program memory
**************************************************************************************
* DESCRIPTION: One tulong is read from location prg_address from program memory.
*   void vmm_prgReadLong (  - 
*           tulong prg_address,  - address in program memory
*           tulong *ret_value    - value at location prg_address in program memory
*   )   
**************************************************************************************/
static inline void vmm_prgReadLong (tulong prg_address, tulong *ret_value)
{
#if _LITTLE_ENDIAN /* if operating in LITTLE ENDIAN mode swap tuint to create proper tulong */
    tulong temp;
#endif
    utlMoveProg2Data(prg_address, (tulong)ret_value, 2);
#if _LITTLE_ENDIAN /* if operating in LITTLE ENDIAN mode swap tuint to create proper tulong */
    temp = *ret_value;  
    SWAP_TUINT(temp);
    *ret_value = temp;
#endif 
}

/*************************************************************************************
* FUNCTION PURPOSE: Read a block from program memory
**************************************************************************************
* DESCRIPTION: One block is read from location prg_address from program memory.
*   void vmm_prgReadLong (  - 
*           tulong prg_address,  - address in program memory
*           tulong *ret_value,   - value at location prg_address in program memory
*           tsize  block_size
*   )   
**************************************************************************************/
static inline void vmm_prgReadBlock (tulong prg_address, void *ret_block, tuint block_size)
{
    utlMoveProg2Data(prg_address, (tulong)ret_block, block_size);
}

/**************************************************************************************
* FUNCTION PURPOSE: Convert actual VCT to generic VCT (in terms of memory allocation).
**************************************************************************************
* DESCRIPTION: Search for duplicate VCT (in terms of memory allocation) is undertaken.
*              If duplicate VCT is found, than identifier is replaced with the one that
*              represent that type of memory mapping. If it is not found, than translation
*              is not necessary.
**************************************************************************************/
static inline void vmm_vct2vct (tint *vctId, tulong startOfTranslation_long, tint sizeOfTranslation)
{
  tint i;
  tuint value, *ptr;
  tuint *startOfTranslation=(tuint *)startOfTranslation_long;
 /*Bring in the Values from Shared memory */
  vmmVctConversionTable(startOfTranslation);
  ptr = (tuint *)startOfTranslation;
  for (i = 0; i < sizeOfTranslation; i ++) {
    value = *ptr ++;
    if (upperByte (value) == *vctId) {		/* Upper byte contains actual VCT */
        *vctId = lowerByte (value);			/* and lower byte contains generic VCT */
        break;
    }
  }
}


#endif
/* nothing past this point */
