#ifndef _MEMMGM_H
#define _MEMMGM_H
/******************************************************************************
 * FILE PURPOSE: Memory Management for DSP ECO's
 ******************************************************************************
 * FILE NAME:   memmgm.h
 *
 * DESCRIPTION: This file contains definitions and API for memory management
 *              functions of the DSP ECO's.
 *
 * REQUIRES:    types.h (generic data types)
 *
 * TABS: NONE
 *
 * $Id: memmgm.h 1.3 1997/05/21 16:04:30 BOGDANK Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: memmgm.h $
 * Revision 1.3  1997/05/21 16:04:30  BOGDANK
 *     - new types used
 *    New types used.
 * Revision 1.2  1996/09/10 10:48:51  BOGDANK
 *    Removing alloc and free functions from memBuffer_t.
 * Revision 1.1  1996/09/09 17:45:18  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1996, 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"
#include "cgenAsst.h"

/* Memory classes (max. 32767) */
enum {
  MEM_CLASS_EXTERNAL = 0,   /* generic (bulk) external memory */
  MEM_CLASS_INTERNAL = 1    /* generic (bulk) internal memory */
};

enum {
  MEM_PERSISTANT = 0,
  MEM_VOLATILE   = 1,
  MEM_GMP_TYPE   = 2,
  MEM_BYPASS     = 3,
  MEM_EXT_SHARED_MEM = 4
};
/******************************************************************************
 * DATA DEFINITION: Memory Buffer Structure.
 ******************************************************************************
 * DESCRIPTION: This structure is used to request and/or supply the dynamic
 *              memory to the DSP ECO's.
 *
 *****************************************************************************/

typedef struct {

  /* REQUEST/RETURN parameters */

  tint  class;      /* Memory class. It is used to describe kind of memory
                     *  that is requested or returned. For example: external,
                     *  internal, or similar. One should use MEM_CLASS_xxx
                     *  constants. On RETURN, class may be changed if original
                     *  memory class is exhausted. */
  tuint log2align;  /* Alignment information (>= 0). If zero, no alignment is
                     *  is required nor provided. Otherwise, memory buffer is
                     *  aligned on appropriate power of 2 (e.g. if log2align=3,
                     *  buffer is aligned modulo-8). */
  uint  size;       /* Number of words that are requested or supplied. Word
                     *  length depends on the platform, and corresponds to
                     *  the shortest element that can be addressed by the CPU.
                     *  Word storage length in bits is defined in types.h as
                     *  TYP_WORD_SIZE. The maximum size depends on the
                     *  platform. */
  bool  volat;      /* TRUE: Memory buffer must be restored before and saved
                     *  after it is used.
                     * FALSE: Save/restore are not necessary.
                     *  On RETURN, volatile flag may be changed if original
                     *  request could not be satisfied. */
  void  *base;      /* Base address of the requested buffer. */
} memBuffer_t;



/* ECO memory management report codes */
#define ECO_NOERR              0 /* has to be 0, all errrors have to be non-zero */
#define ECO_ERR_INST_NON_NULL -1 /* ECO instance can not be NULL */
#define ECO_ERR_BAD_NBUFS     -2 /* Number of requested and provided buffers does not match */
#define ECO_ERR_BAD_BUF_ADDR  -3 /* Bad buffer address intended for assignment (NULL) */
#define ECO_ERR_BAD_BUF_SIZE  -4 /* Provided buffer size is smaller than requested */
#define ECO_ERR_BAD_ALIGN     -5 /* Alignment request is not met */

#define ECO_RELOC_NOERR        0 /* Succesful ECO relocation */
#define ECO_RELOC_ERROR       -1 /* Relocation failed */

/* Common ECO creation/relocation routines: used in xxxNew() and xxxReloc() functions */
tint ecoNewErrCheckUtil(void *ecoInst, tuint nBufs, tuint ecoNbufs, memBuffer_t *bufs, memBuffer_t *bufReq, word *bufAddrs[]);
void ecoAssignBufs(void **ecoInst, tuint nBufs, word* bufAdrs[CGEN_ASSIST_RESTRICT], 
                   tuint *CGEN_ASSIST_RESTRICT bufOffsets);
void ecoGetBufs(void **ecoInst, tuint nBufs, word *bufAdrs[CGEN_ASSIST_RESTRICT], 
                   tuint *CGEN_ASSIST_RESTRICT bufOffsets);
#endif /* _MEMMGM_H */
/* nothing past this point */
