#ifndef _MHMPORT_H
#define _MHMPORT_H
/******************************************************************************
 * FILE PURPOSE: Definitions that are used on a C54x platform.
 ******************************************************************************
 * FILE NAME:   MHMPORT.H
 *
 * DESCRIPTION: Contains definitions that are used on a C54x platform.
 *
 * TABS: NONE
 *
 * $Id: mhmport.h 1.1 1997/06/25 11:41:02 BOGDANK Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: mhmport.h $
 * Revision 1.1  1997/06/25 11:41:02  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"

/* Align mask is used to provide alignment of memory blocks to power of 2
 *  address. It is derived from the MHM_ALIGN_BITS which specifies how many
 *  low order bits must be zero for each block's base address. Base address
 *  on this platform is assumed to have up to 16 bits since the heap is used
 *  only within the data address space which is 16 bits long. */
#define MHM_ALIGN_BITS    3
#define MHM_ALIGN_MASK    (~((~0U)<<MHM_ALIGN_BITS))   /* 0x07 */

/* Consistency check for the align bits. Must align on a long boundary for
 *  this platform. */

#if (((1<<MHM_ALIGN_BITS)*TYP_WORD_SIZE) != TYP_LONG_SIZE)
//# error invalid number of alignment bits
#endif

/* Align gap is the maximum number of words that may be used to fill a gap
 *  to the next memory block due to alignment requirements. */
#define MHM_MAX_ALIGNGAP  MHM_ALIGN_MASK

/* The maximum value of a buffer size and the maximum buffer size. They are
 *  not the same due to alignment gap. */
#define MHM_MAX_SIZEVAL   0xFFFF
#define MHM_MAX_SIZE      (MHM_MAX_SIZEVAL - MHM_MAX_ALIGNGAP)

/* The address type is an unsigned integer type that has the same number of
 *  bits as the longest possible (data) address on this platform. */
typedef tulong  mhmAddress_t;

#ifndef DEFAULT_CODE_SECTION
#pragma CODE_SECTION (mhmFree,".initProg")
#pragma CODE_SECTION (mhmCreate,".initProg")
#pragma CODE_SECTION (mhmAlloc,".initProg")
#endif

#endif  /* _MHMPORT_H */
/* nothing past this point */
