#ifndef _CGENASST_H
#define _CGENASST_H
/*****************************************************************************
 * FILE PURPOSE: Code generation (compiler) assist macros
 *****************************************************************************
 * FILE NAME:   cgenAsst.h
 *
 * DESCRIPTION: Contains code generation assist macros that help to express
 *              assumptions to compiler to improve optimization. These 
 *              macros allow common C code to be used across processors. See
 *              coding guidelines for more information
 *
 *****************************************************************************
 * (C) Copyright 2007 Texas Instruments Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* restrict qualifier, see compiler user manual for more information */
#define CGEN_ASSIST_RESTRICT restrict

/* loop information, helps express truth values about the loop counter */
#define PRAGMA(x) _Pragma(#x)
#define CGEN_ASSIST_LOOP_COUNT_INFO(min, max, multiple, minOften, maxOften) \
        PRAGMA(PROB_ITERATE(minOften, maxOften)) \
        PRAGMA(MUST_ITERATE(min, max, multiple))

/* can be used to express alignment assumptions */
#define CGEN_ASSIST_PTR_IS_ALIGNED(ptr, pow2) \
  _nassert(((unsigned int) (ptr) & (pow2)-1) == 0);

#endif
/* Nothing past this point */
