/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "util_s.h"

#include "utlport.h"
#define fifo_copy(src,dst,size) memcpy((void *)(dst), (void *)(src), ((size)*sizeof(tint)))

void utilFifoWrite(tuint *start, tuint *input, tuint *in, tuint *out, 
                   tuint size, tuint len)
{
    tuint remain;
    
    if(size - *in >= len)
    {
        fifo_copy(input, start + *in, len);
        *in += len;
        if(*in == size)
            *in = 0;
    }
    else
    {
        fifo_copy(input, start + *in, (tuint)(size - *in));
        input += size - *in;
        remain = len - (size - *in);
        fifo_copy(input, start, remain);
        *in = remain;
    }
}

void utilFifoRead(tuint *start, tuint *output, tuint *in, tuint *out, 
                  tuint size, tuint len)
{
    tuint remain;

    if(size - *out >= len)
    {
        fifo_copy(start + *out, output, len);
        *out += len;
        if(*out == size)
            *out = 0;
    }
    else
    {
        fifo_copy(start + *out, output, (tuint)(size - *out));
        output += size - *out;
        remain = len - (size - *out);
        fifo_copy(start, output, remain);
        *out = remain;
    }
}


#ifdef C_SIM
void misc_utlCopy(tuint *in, tuint *out, tuint length)
{
	int i;
	for(i = 0; i < length; i++)
		*out++ = *in++;
}
#endif /* C_SIM */


