#ifndef _GLOBSTATS_H
#define _GLOBSTATS_H

#include "types.h"
#include "ggmbuf.h"
#include "util.h" /* UTL_BYTES_TO_WORD() */

#include <ti/mas/rcu/rcu.h>
#include <ti/mas/vqm/vqm.h>

/******************************************************************************
 * FILE PURPOSE: Interface to global(per channel) statistics.
 ******************************************************************************
 * FILE NAME:   globstats.h 
 *
 * DESCRIPTION: Channel-wide statistics data may be accessed through following 
 *              interface functions or data structures.
 *
 * (C) Copyright 2003 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#define STATS_ID2CHAN(a)   (((a) & 0x00ff)) 
/******************************************************************************
 * DATA DEFINITION: VPU statistics 
 ******************************************************************************
 * DESCRIPTION: This structure monitors vpu measured statistics
 *
 *****************************************************************************/

typedef struct vpuStats_s {
  LFract fifoAverageDelay;      /* Internally computed average FIFO Delay     *
                                 * in PCM samples. Kept as a fract to increase*
                                 * accuracy (Q18).                            */
                                /* (options=-v5)                              */
  tuint  underflowCounter;      /* number of FIFO underflows                  */
                                /* (options=-v5)                              */
  tuint  overflowCounter;       /* FIFO Buffer overflow counter               */
                                /* (options=-v5)                              */
  tulong lostPacketCounter;     /* number of lost packets                     */
                                /* (options=-v5)                              */
  tulong dropPacketCounter;     /* number of dropped segments due to FIFO     *
                                 * overflow/underflow/early/late              */
                                /* (options=-v5)                              */
  tulong idlePacketCounter;     /* number of idle (CN) segments sent to CODEC *
                                 * due to no voice activity or lextened PLC   */
                                /* (options=-v5)                              */
  tulong receivedPacketCounter; /* number of segments received from network   *
                                 * sucessfully                                */
                                /* (options=-v5)                              */
  LFract adaptAvgJitter;        /* An average of jitter over a period of time */
                                /* (options=-v5)                              */
  tint   fatalErrorCounter;     /* indication that something has to be fixed  */
                                /* (private)                                  */
  tuint  adaptIncreases;        /* # of times adaptive VPU increased delay    */
                                /* (options=-v5)                              */
  tuint  adaptDecreases;        /* # of times adaptive VPU decreased delay    */
                                /* (options=-v5)                              */
  tuint  cellStarvEventCounter; /* Count of Cell/Segment Starvation events    */
                                /* (options=-v5)                              */
} vpuStats_t;
/******************************************************************************
 * DATA DEFINITION: PVP statistics 
 ******************************************************************************
 * DESCRIPTION: This structure monitors pvp measured statistics
 *
 *****************************************************************************/
/* Error Statistics  */
typedef struct {
#if 0
  tulong  RxVPktDrop;        /* # from-pkt voice pkts dropped due to invalid  *
                              * header                                        */
                             /* (options=-v5)                                 */
  tulong  TxVPktDrop;        /* # to-pkt voice pkts dropped for busy target   */
                             /* (options=-v5)                                 */
  tulong  TxSIDPktDrop;      /* # to-pkt SID packets dropped for busy target  */
                             /* (options=-v5)                                 */
  tulong  TxDtmfPktDrop;     /* # to-pkt DTMF transitions dropped for busy    *
                              * target                                        */
                             /* (options=-v5)                                 */
  tulong  TxToipPktDrop;     /* # to-pkt TOIP packets dropped for busy        *
                              * target                                        */
                             /* (options=-v5)                                 */
  tulong  lostNotification;  /* # of from-pkt AAL2 Voice (not P2P) violations *
                              * dropped host busy                             */
                             /* (options=-v5)                                 */
  tulong  RxInvSSRCDiscard;  /* from-pkt discards due to invalid SSRC         */
                             /* (options=-v5)                                 */
#endif
  tulong  RxInvPayloadDiscard;/* from-pkt discards due to invalid PT          */
                             /* (options=-v5)                                 */
#if 0
  tulong  RxLostPackets;     /* # Voice Packets lost by Network               */
                             /* (options=-v5)                                 */
  tuint   TxVSegTruncated;   /* # to-pkt truncated voice segments (because    *
                              * codec produced frame bigger than expected)    */
                             /* (options=-v7)                                 */
  tuint   RxSIDPktInvSN;     /* # from-pkt SID packets with invalid seqn #    * 
                              * FRF11 - (obsolete)                            */
                             /* (options=-v9) */
  tuint   rxSeqNumDisc;      /* # from-pkt VOICE Sequence Number              *
                              * Discontinuity Events                          */
                             /* (options=-v5) */
#endif
} pvpChannelErrStat_t;

/* Transmit Statistics */
typedef struct {
  tulong  TxVPkts;              /* # of to-pkt voice packets                  */
                                /* (options=-v5)                              */
  tulong  SilenceVFrames;       /* # of to-pkt voice frames classified as     *
                                 * silent (SID packets excluded).             */
                                /* (options=-v5)                              */
  tulong  SIDPkts;              /* # of to-pkt SID packets                    */
                                /* (options=-v5)                              */
  tulong  dtmfPkts;             /* # of to-pkt DTMF transitions               */
                                /* (options=-v5)                              */
  tulong  toipPkts;             /* # of to-pkt Toip packets                   */
                                /* (options=-v5)                              */
  tulong  grantSyncDrop;        /* # of to-pkt frames dropped for grant sync  */
                                /* (options=-v5)                              */
  tulong  TxPackets;            /* Total number of to-pkt packets by AAL1     */
                                /* (options=-v5)                              */
  tulong  TxLastTimestamp;      /* last to-pkt voice time stamp               */
                                /* (options=-v5)                              */
  tulong  TxOctets;             /* To-pkt bytes -- only used by AAL1          */
                                /* (options=-v5)                              */
  tuint   TxLastSequenceNum;    /* last to-pkt voice sequence number          */
                                /* (options=-v5)                              */
  tuint   TxExtendedSequenceNum;/* last to-pkt voice extended sequence number */
                                /* (options=-v5)                              */
  tuint   TxLastPacketType;     /* last to-pkt packet type                    *
                                 * (0=voice;1=SID;2=DTMF)                     */
                                /* (options=-v5)                              */
} pvpChannelTxStat_t;

/* Receive Statistics  */
typedef struct {
#if 0
  tulong  RxVPkts;              /* # of from-pkt voice pkts sent for playout  */
                                /* (options=-v5)                              */
  tulong  minDelta;             /* Minimum from-pkt interarrival time (ms)    */
                                /* (options=-v5)                              */
  tulong  maxDelta;             /* Maximum from-pkt interarrival time (ms)    */
                                /* (options=-v5)                              */
  tulong  SIDPkts;              /* # of from-pkt SID pkts sent for play out   */
                                /* (options=-v5)                              */
  tulong  dtmfPkts;             /* # of from-pkt DTMF transitions to play out */
                                /* (options=-v5)                              */
  tulong  toipPkts;             /* # of from-pkt TOIP packets to play out     */
                                /* (options=-v5)                              */
  tulong  rtpJitter;            /* RTCP from-pkt interarrival jitter using    *
                                 * formula from RFC1889 for both RTP and AAL2 * 
                                 * in samples.                                */
                                /* (options=-v5)                              */
  tulong  vProfileChange;       /* # of from-pkt Voice Profile line changes   */
                                /* (options=-v5)                              */
  tulong  syncSrcChange;        /* # of from-pkt voice SSRC changes for RTP   */
                                /* (options=-v5)                              */
  tulong  RxLastTimestamp;      /* last from-pkt voice packet time stamp      */
                                /* (options=-v5)                              */
  tulong  RxLastSSRC;           /* SSRC of the last sending out packet        */
                                /* (options=-v5)                              */
  tulong  RxPackets;            /* Total number of from-pkt pkts in AAL1      */ 
                                /* (options=-v5)                              */
  tulong  RxOctets;             /* from-pkt bytes -- only used by AAL1        */
                                /* (options=-v5)                              */
#endif
  tuint   RxLastSequenceNum;    /* last from-pkt voice packet sequence number */
                                /* (options=-v5)                              */
#if 0
  tuint   RxExtendedSequenceNum;/* from-pkt voice extended sequence number    */
                                /* (options=-v5)                              */
  tuint   RxLastPacketType;     /* from-pkt last packet type (bitfield):      */
                                /* High 8 bits: last from-pkt profile idx;    *
                                 * low 8 bits: 0=voice; 1=SID; 2=DTMF.        */
                                /* (options=-v5-h)                            */
#endif
} pvpChannelRxStat_t;

/* All PVP statistics */
typedef struct pvpStats_s {
#if 0
  pvpChannelTxStat_t  TxStat;
#endif
  pvpChannelRxStat_t  RxStat;
  pvpChannelErrStat_t *ErrStat;
} pvpStats_t;
/************************* Network side statistics ***************************/
typedef  struct acuGlobStatsErr_s {
  tulong txBusy;            /* TX packets dropped for target busy, or no  *
                             * target defined                             */
                            /* (options=-v5)                              */
  tulong noRoute;           /* No route found or illegal state machine #  */
                            /* (options=-v5)                              */
  tulong RxPktDropChnState; /* Packets dropped due to channel state       */
                            /* (options=-v5)                              */
  tulong RxOctDrop;         /* # of octets dropped due to error           */
                            /* (options=-v5)                              */
  tulong rxInvalidPkts;     /* Number of invalid packets received         */
                            /* (options=-v5)                              */
  tuint  rxCrc10;           /* Number of packets with invalid CRC10       */
                            /* (options=-v5)                              */ 
} acuGlobStatsErr_t;

typedef struct acuGlobStatsTxRx_s {
  tulong txPkts;       /* Packets passed to network                           */
                       /* (options=-v5)                                       */
  tulong rxPkts;       /* Packets from network                                */
                       /* (options=-v5)                                       */
  tulong txOctets;     /* Bytes of payload transmitted                        */
                       /* (options=-v5)                                       */
  tulong rxOctets;     /* Bytes of payload received                           */
                       /* (options=-v5)                                       */
  tulong rxRedDrops;   /* Packets filtered because they were redundant        */
                       /* (options=-v5)                                       */  
} acuGlobStatsTxRx_t;
  

typedef struct acuGlobStats_s {
  acuGlobStatsTxRx_t txrx;
  acuGlobStatsErr_t  *err;
} acuGlobStats_t;


typedef struct pktStatsErr_s {
	union {
		rcuErrStats_t     rtp;
		acuGlobStatsErr_t aal2;
	} proto;
} pktStatsErr_t;

typedef struct pktStats_s {
	union {
		rcuStats_t     rtp;
		acuGlobStats_t aal2;
	} proto;
} pktStats_t;

/************************* Network side statistics ***************************/

#define VQM_GET_TDEGNET(a)   vqm_GET_TDEGNET(a)
#define VQM_SET_TDEGNET(a,b) vqm_SET_TDEGNET(a,b)
#define VQM_GET_TDEGJIT(a)   vqm_GET_TDEGJIT(a)
#define VQM_SET_TDEGJIT(a,b) vqm_SET_TDEGJIT(a,b)

#define VQM_GET_TDEGES(a)    vqm_GET_TDEGES(a)
#define VQM_SET_TDEGES(a,b)  vqm_SET_TDEGES(a,b)
#define VQM_GET_TDEGSES(a)   vqm_GET_TDEGSES(a)
#define VQM_SET_TDEGSES(a,b) vqm_SET_TDEGSES(a,b)

#define VQM_GET_INC_FLAG(a)       vqm_GET_INC_FLAG(a)
#define VQM_SET_INC_FLAG(a,b)     vqm_SET_INC_FLAG(a,b)
#define VQM_GET_UPDATE_FLAG(a)    vqm_GET_UPDATE_FLAG(a)
#define VQM_SET_UPDATE_FLAG(a,b)  vqm_SET_UPDATE_FLAG(a,b)

#define VQM_GET_ERRORS_SEC(a)     vqm_GET_ERRORS_SEC(a)
#define VQM_SET_ERRORS_SEC(a,b)   vqm_SET_ERRORS_SEC(a,b)
#define VQM_GET_ADPT_TIME_UP(a)   vqm_GET_ADPT_TIME_UP(a)
#define VQM_SET_ADPT_TIME_UP(a,b) vqm_SET_ADPT_TIME_UP(a,b)

#define VQM_GET_ADPT_TIME_DN(a)   vqm_GET_ADPT_TIME_DN(a)
#define VQM_SET_ADPT_TIME_DN(a,b) vqm_SET_ADPT_TIME_DN(a,b)
/* Increase during silence -- no degradation */
#define VQM_GET_ADPT_TIME_UP_NODEG(a)   vqm_GET_ADPT_TIME_UP_NODEG(a)
#define VQM_SET_ADPT_TIME_UP_NODEG(a,b) vqm_SET_ADPT_TIME_UP_NODEG(a,b)


/* must be >= sizeof(vqmInst_t) */
#define VQM_RFC3611_SIZE_B   vqm_RFC3611_SIZE_B
/* Allocate word 32 to guarantee 32 bit alignment */
#define VQM_RFC3611_SIZE_W32 vqm_RFC3611_SIZE_W32


#define VQM_GET_XR_TYPE(a)            vqm_GET_XR_TYPE(a)
#define VQM_SET_XR_TYPE(a,b)          vqm_SET_XR_TYPE(a,b)
#define VQM_TYPE_RFC3611              vqm_TYPE_RFC3611
#define VQM_TYPE_H248XNQ              vqm_TYPE_H248XNQ
#define VQM_GET_NEW_CYCLE(a)          vqm_GET_NEW_CYCLE(a)
#define VQM_SET_NEW_CYCLE(a,b)        vqm_SET_NEW_CYCLE(a,b)
/*
 * Store severe error seconds threshold in 1ms units.
 * The maximum threshold is obviously 1000ms can be stored in 10bits
 */
#define VQM_GET_XR_SEVERE_THRESH(a)   vqm_GET_XR_SEVERE_THRESH(a)
#define VQM_SET_XR_SEVERE_THRESH(a,b) vqm_SET_XR_SEVERE_THRESH(a,b)


/******************************************************************************
 * DATA DEFINITION: Channel related statistics
 ******************************************************************************
 * DESCRIPTION: Instance of this type is allocated once per channel. It exists
 *    all the time. We use storage area that is not overwritten in case of 
 *    application switchover (VOICE<->FAX). Content of this data structure is
 *    initialized on channel open, but it is available even after channel is
 *    closed.
 *****************************************************************************/
typedef struct siuErrStats_s {
  pvpChannelErrStat_t pvpChErrStats;
  pktStatsErr_t       pktErrStats;
}siuErrStats_t;

typedef struct siuInFreqStats_s {
/*****************************************************************
 * EVERYTHING that uses siuStats_t MUST have
 * these defines added to their makefiles!
 *****************************************************************/
#if (defined(VQM_RFC3611) || defined(VQM_H248XNQ))
  vqmStatsStorage_t vqmStats;
#else
  tuint dummy;
#endif
//   Infreq VPU Stats	 For future
// Infreq RTCP Stats	 For future
} siuInFreqStats_t;

typedef struct siuExtMemStats_s {
  siuErrStats_t err;
  siuInFreqStats_t inFreq; 
} siuExtMemStats_t;


typedef struct siuStats_s {
  pvpStats_t   pvpStats;  /* all PVP stats     */
#if 0
  vpuStats_t   vpuStats;  /* all VPU stats     */
#endif
  pktStats_t   pktStats;  /* all RCU-ACU stats */
#if (defined(VQM_RFC3611) || defined(VQM_H248XNQ))
  vqmStatsStorage_t *vqmStats;
#endif
} siuStats_t;

/******************************************************************************/
/* Return pointer to required global stats                                    */
/* These inline functions assume that siuHeapInst_t.stats is at offset 0.     */
/* This assumption is checked in sininit.c.                                   */
/******************************************************************************/
extern tint ggCh2Slot[];

#if 0
static inline void *globStatsVpuGetPtr (tuint ID) {
siuStats_t *all_stats = 
            (siuStats_t *)gg_siuheap_inst[ggCh2Slot[STATS_ID2CHAN(ID) - 1]];
  return (void *)(&(all_stats->vpuStats));
}
#endif

static inline void *globStatsPvpGetPtr (tuint ID) {
  siuStats_t *all_stats = 
            (siuStats_t *)gg_siuheap_inst[ggCh2Slot[STATS_ID2CHAN(ID) - 1]];
  return (void *)(&(all_stats->pvpStats));
}

static inline void *globStatsPktGetPtr (tuint ID) {
siuStats_t *all_stats = 
            (siuStats_t *)gg_siuheap_inst[ggCh2Slot[STATS_ID2CHAN(ID) - 1]];
  return (void *)(&(all_stats->pktStats));
}

static inline void *globStatsSiuGetPtr (tuint ID) {
siuStats_t *all_stats = 
            (siuStats_t *)gg_siuheap_inst[ggCh2Slot[STATS_ID2CHAN(ID) - 1]];
  return (void *)all_stats;
}

#if (defined(VQM_RFC3611) || defined(VQM_H248XNQ))
static inline void *globStatsVqmGetPtr (tuint ID) {
siuStats_t *all_stats = (siuStats_t *)gg_siuheap_inst[ggCh2Slot[STATS_ID2CHAN(ID) - 1]];
  return (void *)all_stats->vqmStats;
}
#endif
#endif
/* nothing past this point */
