#ifndef _GMP_H
#define _GMP_H
/******************************************************************************
 * FILE PURPOSE: Interface to global configuration parameters.
 ******************************************************************************
 * FILE NAME:   siucfg.h
 *
 * DESCRIPTION: System- and channel-wide configuration parameters may be 
 *              accessed through following interface functions or data 
 *              structures.
 *
 * (C) Copyright 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* Defined in: dsp/siu/siugmp.c */
#include "types.h"

#define GMC_MAX_PCM_5MS_SEG_BITS      352 /* 8000 samples/sec * 5.5e-3 sec  * 8 bits/sample */
#define GMC_MAX_PCM_5MS_SIZE          ( (GMC_MAX_PCM_5MS_SEG_BITS + TYP_WORD_SIZE - 1) \
                                                               /TYP_WORD_SIZE) 
                                                               /* GMC size in words */

#define GMC_MAX_PCM_10MS_SEG_BITS      640 /* 8000 samples/sec * 10e-3 sec  * 8 bits/sample */
#define GMC_MAX_PCM_10MS_SIZE          ( (GMC_MAX_PCM_10MS_SEG_BITS + TYP_WORD_SIZE - 1) \
                                                               /TYP_WORD_SIZE)
                                                               /* GMC size in words */

/* assume 16 KHz for L16 PCM, this should still work for 8 KHz in case 
   it is desired in future albeit with some wasteful memory but should be upper bounded
   by the 16 kHz case anyway */
#define GMC_MAX_L16PCM_5MS_SEG_BITS	  1280 /* 16000 samples/sec * 5 e-3 * 16 bits/sample */
#define GMC_MAX_L16PCM_5MS_SIZE        ( (GMC_MAX_L16PCM_5MS_SEG_BITS + TYP_WORD_SIZE - 1) \
                                                               /TYP_WORD_SIZE)

#define GMC_MAX_L16PCM_10MS_SEG_BITS  2560 /* 16000 samples/sec * 10 e-3 * 16 bits/sample */
#define GMC_MAX_L16PCM_10MS_SIZE      ( (GMC_MAX_L16PCM_10MS_SEG_BITS + TYP_WORD_SIZE - 1) \
                                                               /TYP_WORD_SIZE)

#define GMC_MIN_SIZE              GMC_MAX_PCM_5MS_SIZE /* minimum of GMC_MAX_PCM_5MS_SIZE                                                       
                                                          and GMC_MAX_WBAMR_SIZE */

#define GMC_MAX_WBAMR_SEG_BITS    501 /* Max size of codeword for all bit-rates 
                                         PLUS IF1 header */

#define GMC_MAX_WBAMR_SIZE        ( (GMC_MAX_WBAMR_SEG_BITS + TYP_WORD_SIZE - 1) \
                                                                 /TYP_WORD_SIZE)  
                                                                 /* GMC size in words */

#define GMC_MAX_ILBC_SEG_BITS    400 /* Max size of codeword for ILBC (30 ms) */

#define GMC_MAX_ILBC_SIZE        26 /* (GMC_MAX_ILBC_SEG_BITS ( + TYP_WORD_SIZE - 1) \
                                      /TYP_WORD_SIZE) + 1 */
                                                                 /* GMC size in words */

#define GMC_MAX_SIZE              GMC_MAX_PCM_10MS_SIZE /* max of GMC_MAX_PCM_10MS_SIZE 
                                                       and GMC_MAX_WBAMR_SIZE           
                                                       and any other component          
                                                       buffer's size, that goes         
                                                       into GMC whose size exceeds      
                                                       that of PCM */


/* Single GMC is allocated */
extern void *gmpAllocGmc (tuint);
/* Single GMC is de-allocated */
extern bool gmpFreeGmc   (tuint, void *);

/* alloc and free for codeword segments */
extern word *gmpAllocCwSeg (tuint ID, word *payload, tuint cwsegSize_bits);
extern void  gmpFreeCwSeg  (tuint ID, word *segPtr,  tuint cwsegSize_bits);
extern void  gmpGetCwSeg   (word  *segPtr, word *segFlatPtr, tuint cwsegSizeWords);

/* Single linked list of GMCs is allocated */
extern void *gmpLinkedRecordsAlloc(tuint, tuint, tuint);
/* Single linked list of GMCs is de-allocated */
extern bool gmpLinkedRecordsFree (tuint, void *);
/* Returns pointer to offset of buffer spanned over
 * single linked list of GMCs. Equivalent to seek with
 * SEEK_SET argument (in words) */
extern void *gmpLinkedRecordsIdx2Ptr (void *, tuint, tuint);
/* Returns GMC size */
extern tuint gmpGetGmcSize(void);
/* Returns area in GMC that may be used for multiple data
 * structures storage */
extern tuint gmpGetRecordSize(void);
/* Creation of FILE-like structure, mode is ignored at the moment */
#if 0
/* No need (at the moment) for general records */
extern void *gmpCreate (tuint input_MID, tuint mode, tuint recordSize);
#else
extern void *gmpCreate (tuint input_MID, tuint mode);
#endif
/* FILE-like structure is deleted */
extern void gmpDelete (tuint input_MID, void *headIn);
/* Write data into GMP based FILE-like structure. Append mode is assumed. No
 * limitation on amount of data to be written */
extern void gmpWrite (tuint input_MID, void *head, void *srcIn, tint records_to_copy);
/* Copy data from GMP into linear buffer. Internal read position marker is updated */
extern tint gmpRead (tuint input_MID, void *head, void *dstOut, tint records_to_copy);
/* Update read position marker. Implemented, but commented, as no module is
 * using it. */
extern void *gmpSeek (void *head, tint seek_pos, tint mode);

/* This is the maximum size for 5.5 ms frame of PCM when stored as packed in 16-bit
   words. Used only in tfo. */
#ifdef TDM_10MS
  #define GMP_MAX_PCM_SEG_SIZE GMC_MAX_PCM_10MS_SIZE
#else
  #define GMP_MAX_PCM_SEG_SIZE GMC_MAX_PCM_5MS_SIZE
#endif  

#endif
/* nothing past this point */


