#ifndef _RTCP_H
#define _RTCP_H
/******************************************************************************
 * FILE PURPOSE: RTCP related Definitions
 ******************************************************************************
 * FILE NAME:   rtcp.h
 *
 * DESCRIPTION: This file contains RTCP related definitions.
 *              The defines are used for interfacing between ECOs (NEU & RCU)
 *
 * REQUIRES:    types.h   (bool, and other generic types)
 *
 * TABS: NONE
 *
 * $Id$
 *
 * REVISION HISTORY:
 *
 * $Log$
 *
 * (C) Copyright 2005, Texas Instruments Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"

/* Defines to indicate if certain RTCP blocks from incoming Rx packet need to
   be transparently sent out in the outgoing packet, or if Tx side generates all
   RTCP packets                                                                  */
#define RTCP_RFC_3550_TRANSP 0x0001 /* Transparently route RFC 3550 packets (SR, RR, SDES, APP, BYE) */
#define RTCP_RFC_3611_TRANSP 0x0002 /* Transparently route RFC 3611 packets (XR)                     */
#define RTCP_RFC_3611_DEL_RX 0x0004 /* Transparent routing mode: Delete XR packet from Rx packet     */
#define RTCP_RFC_3611_INS_TX 0x0008 /* Transparent routing mode: Insert XR packet into Tx packet     */
#define RTCP_TX_ORIGINATE    0x0010 /* TDM->PKT mode: Tx side originates all RTCP packets            */

/******************************************************************************
 * DATA DEFINITION: Offsets of RTCP packets in a compound RTCP packet. In transparent routing mode,
 * a received RTCP packet is processed by rcuRtcpReceiveIn(), and offsets of RTCP packets are returned
 * to NEU in this structure.
 *    NEU then supplies this structure to rcuRtcp_send_packet() on destination channel, to form the
 * outgoing packet (with possible RTCP XR termination).
 *****************************************************************************/
typedef struct rtcpOffsets_s {
  tint   sr_offset;   /* Offset of SR packet in bytes */ 
  tint   sr_length;   /* Length of SR packet in bytes */
  tint   rr_offset;   /* Offset of RR packet in bytes */
  tint   rr_length;   /* Length of RR packet in bytes */
  tint   sdes_offset; /* Offset of SDES packet in bytes */
  tint   sdes_length; /* Length of SDES packet in bytes */
  tint   xr_offset;   /* Offset of XR packet in bytes */
  tint   xr_length;   /* Length of XR packet in bytes */
  tint   app_offset;  /* Offset of APP packet in bytes */
  tint   app_length;  /* Length of APP packet in bytes */
  tint   bye_offset;  /* Offset of BYE packet in bytes */
  tint   bye_length;  /* Length of BYE packet in bytes */
} rtcpOffsets_t;

#endif /* _RTCPXR_H */
/* nothing past this point */

