#ifndef _RTCPXR_H
#define _RTCPXR_H
/******************************************************************************
 * FILE PURPOSE: RTCP XR related Definitions
 ******************************************************************************
 * FILE NAME:   rtcpxr.h
 *
 * DESCRIPTION: This file contains RTCP XR related definitions and call-table. 
 *
 * REQUIRES:    types.h   (bool, and other generic types)
 *
 * TABS: NONE
 *
 * $Id$
 *
 * REVISION HISTORY:
 *
 * $Log$
 *
 * (C) Copyright 2005, Texas Instruments Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"

/******************************************************************************
 * DATA DEFINITION: XR VoIP Metrics Structure
 ******************************************************************************
 * DESCRIPTION: This structure contains all the parameters that are required 
 *              to prepare a RTCP XR VoIP metric report block with the exception
 *              of Markov model parameters.
 *              Check RFC 3611 for detailed descriptions
 *
 *****************************************************************************/
typedef struct rtcpXR_voipPrms_s {
  tuint remoteESDelay; /* Remote end system delay in ms. Includes sample 
                        * accumulation, encoding delay, jitter buffer delay,
                        * decoding and play-out delay */
  tuint localESDelay;  /* Local end system delay in ms. Includes sample 
                        * accumulation, encoding delay, jitter buffer delay,
                        * decoding and play-out delay */
  tuint oneWayDelay;   /* Average one-way delay in ms */
  tuint nomJBDelay;     /* Current nominal jitter buffer delay in ms */
  tuint maxJBDelay;     /* Maximum jitter buffer delay in ms recorded */
  tuint absMaxJBDelay;  /* Absolute maximum jitter buffer delay in ms */
  tuint intArrJitter;   /* Current interarrival jitter in ms */
  tuint intlCode;       /* Internation code used for call metrics */
  tuint pktDuration;    /* Length of a RTP packet in ms */
  tuint rxCodecType;    /* Current RX codec type (in Telogy format - vctypes.h) */
  tuint remoteInfo;     /* Bits 0-7:  Avg. remote residual echo return loss in dB.
                         *            set to 127 if not available 
                         * Bits 8-15: Avg. Speech energy signal level in dBm 
                         *            reported by the remote vocoder. set to 127
                         *            if not available */
  tuint levels;         /* Bits 0-7:  Avg. received silence period noise level 
                         *            in dBm. Set to 127 if not available 
                         * Bits 8-15: Avg. received speech energy signal level 
                         *            in dBm. Set to 127 if not available */
  tuint rerlRxCfg;      /* Bits 0-7:  A flag map describing the termination point
                         *            jitter buffer and packet loss concealment
                         *            capabilities. Check RFC 3611    
                         *            0 1 2 3 4 5 6 7
                         *            +-+-+-+-+-+-+-+-+
                         *            |PLC|JBA|JB rate|
                         *            +-+-+-+-+-+-+-+-+
                         * Bits 8-15: Local residual echo return loss level in dB.
                         *            set to 127 if not available. */
  tuint gminExtR;       /* Bits 0-7:  External R factor. Should be set to 127 
                         *            if not available.
                         * Bits 8-15: Gmin. The minimum Gap Threshold */
} rtcpXR_voipPrms_t;

#endif /* _RTCPXR_H */
/* nothing past this point */

 
