#ifndef __UTIL_H
#define __UTIL_H
/*****************************************************************************
 * FILE PURPOSE: Utility functions header file
 *****************************************************************************
 * FILE NAME: util.h   
 *
 * DESCRIPTION: This file defines inlined functions and prototypes for the
 *              util.c.
 *
 * TABS: NONE
 *
 * $Id: util.h 1.10 1998/04/20 16:59:26 BOGDANK Exp $
 * 
 * REVISION HISTORY:
 *
 * $Log: util.h $
 * Revision 1.10  1998/04/20 16:59:26  BOGDANK
 *     - utlLFractVecMax() and utlLsVecMax() moved to utilnum.c/.s
 *    utlLFractVecMax() and utlLsVecMax() moved to utilnum.s/.c/.h
 * Revision 1.9  1998/03/04 20:03:35  PPATEL
 *  API of utllnlg.c added.
 * Revision 1.8  1997/09/26 14:39:36  BOGDANK
 *     - some prototypes removed to utilnum.h
 *    Some function prototypes moved into utilnum.h.
 * Revision 1.7  1997/05/21 16:37:54  BOGDANK
 *     - new types added
 *    New types added.
 * Revision 1.6  1996/12/27 12:01:54  BOGDANK
 *     - bug fixes, improvements, additions prompted by ECU
 *    utlLog2Down() added. utlLog2Up(0) now returns 0 instead of 32!
 * Revision 1.5  1996/11/15 14:48:16  BOGDANK
 *     - a few functions added for different types
 *    utlLVecSum(), utlLFractVecMax() added.
 * Revision 1.4  1996/11/11 16:18:37  BOGDANK
 *     - utlFractMemSet(), utlLFractMemSet() added.
 *    Adding utlFractMemSet() and utlLFractMemSet().
 * Revision 1.3  1996/10/04 15:09:53  BOGDANK
 *     - utlLog2Up added
 *    utlLog2Up() added.
 * Revision 1.2  1996/09/24 14:46:43  BONAL
 * Moved utlMoveMemory to util.c from util.h
 *
 * (C) Copyright 1995-1998, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"

#include <ti/mas/types/types.h>
#include <ti/mas/util/utl.h>
/* Additional utility header files */
#include "utilnum.h"
#include "cgenAsst.h"

#define ABS_VALUE(value) ((value)>0?(value):(-(value))); 

#define UTL_CID_MASK  0xFF  /* Low 8 bits of module ID are the channel number */
/* Extracting channel number from a module's ID */
#define UTL_EXTRACT_CHNUM(id)   ((id) & UTL_CID_MASK)


/***      FUNCTION   PROTOTYPES              ***/
/* util.c */
void utlScaleUp6dB   (linSample *a, linSample *b, tint n);
void utlScaleDown6dB (linSample *a, linSample *b, tint n);
void utlFractMemSet  (Fract *a, Fract c, tint n);
void utlLFractMemSet (LFract *a, LFract c, tint n);
void utlLsMemSet     (linSample *a, linSample c, tint n);
void utlMoveMemory   (const linSample *in, linSample *out, tint number);
void utlCopyBits     (word *src,word *dest,tint src_offset,tint dest_offset,
                      tint num_bits);

/* ggdsp abstraction for RTSC misc_utlCopy function */
static inline void misc_utlCopy(tint * a, tint * b, tint c) {
  utlCopy((tword *) (a), (tword *) (b), (c) * sizeof(tint)); 
}

#define UTL_DB_Q  4
#define UTL_LIN_Q 0

/* utlbitcopy.c */
void utlEmbedLSBs (void *output_ptr, void *input_ptr, tint num_lsbs_per_sample,
                   tint bit_offset, tint frame_length);
void utlEmbed16thLSB (void *output_ptr, void *input_ptr, tint num_bits, 
                      tint offset);
void utlExtractAllLSBs (tuint *lsbs_ts, tint *tx_ptr, tint frame_size,
                        tint num_lsbs);
void utlUnpackLsbs (void *lsbs, void *lsbs_out, tint num_lsbs, tint length);
tint utlPrepare16thLSBs (void *input, tint input_bit_offset, void *output_ptr,
                         tint samp_offset, tint frame_length);
tint utlExtract16thLSB (void *input, tint input_frame_length, tuint input_samp_offset, 
                        void *output, tint outbitoff);
void utlEmbedAllLSBs (tuint *lsbs_ts, tint *tx_ptr, tint frame_size,
                      tint num_lsbs_packed, tint num_lsbs_to_embed);


/* Macros that convert bytes into MAU units and vice versa */
#define  UTL_BYTES_TO_WORD(numbytes)   (((numbytes) + ((TYP_WORD_SIZE/8) - 1)) / (TYP_WORD_SIZE/8))
#define  UTL_WORDS_TO_BYTE(numwords)   ((numwords) * (TYP_WORD_SIZE/8))
#define  UTL_WORDS_TO_WORD32(numwords) (((numwords) + ((32/TYP_WORD_SIZE) - 1)) / (32/TYP_WORD_SIZE))

#define  UTL_BYTES_TO_WORDS(numbytes)   (((numbytes) + ((TYP_WORD_SIZE/8) - 1)) / (TYP_WORD_SIZE/8))
#define  UTL_WORDS_TO_BYTES(numwords)   ((numwords) * (TYP_WORD_SIZE/8))
#define  UTL_WORDS_TO_WORD32S(numwords) (((numwords) + ((32/TYP_WORD_SIZE) - 1)) / (32/TYP_WORD_SIZE))
#define  UTL_WORD32S_TO_WORDS(num32s)   ((num32s) * (32/TYP_WORD_SIZE))

#define  UTL_BYTES_TO_TINT(numbytes) ((((numbytes) << 3) + TYP_TINT_SIZE - 1) / TYP_TINT_SIZE)
#define  UTL_TINTS_TO_BYTE(numtints) (((numtints) * TYP_TINT_SIZE) >> 3)

/******************************************************************************
 * FUNCTION PURPOSE: Returns the MAU Address at the specified byte offset from
 *                   the passed base ptr
 ******************************************************************************
 * DESCRIPTION: Function returns the word address within which the specified
 *              byte offset is present when passed the base address and byte 
 *              offset
 *  static inline word* utlGetMauAddr (
 *  word  *base_addr     - Base address
 *  tuint byte_offset    - Byte offset 
 *  )
 *****************************************************************************/
static inline word *  utlGetMauAddr(word *base_addr, tuint byte_offset)
{
  return (base_addr + (byte_offset/(TYP_WORD_SIZE/8)));
}

/* In lined version of utlMemCopy with assumption of minimum length one */
/* Note: Pragma below applies for both 55 and 64, other processors that 
         do not recognize the pragma will not complain */
static inline void utlMemCopyNonZeroLen (void *output, void *input, int length) {
  int i;
  word *d = (word *)output;
  word *s = (word *)input;

  /* Specify minimum loop count to be 1 */
  CGEN_ASSIST_LOOP_COUNT_INFO(1,,,,)
  for (i=0;i<length;i++)
    *d++ = *s++;
}

#endif  /* __UTIL_H */
/* nothing past this point */
