/******************************************************************************
 * FILE PURPOSE: DSP utility functions
 ******************************************************************************
 * FILE NAME: util.c   
 *
 * DESCRIPTION: Contains the utility functions.
 *
 * TABS: NONE   
 *
 * $Id: util.c 1.6 1998/04/20 16:59:26 BOGDANK Exp $
 *
 * REVISION HISTORY:   
 *
 * $Log: util.c $
 * Revision 1.6  1998/04/20 16:59:26  BOGDANK
 *     - utlLFractVecMax() and utlLsVecMax() moved to utilnum.c/.s
 * Revision 1.5  1997/05/21 16:37:53  BOGDANK
 *     - new types added
 *    New types added.
 * Revision 1.4  1996/11/15 14:48:16  BOGDANK
 *     - a few functions added for different types
 *    Adding utlLFractVecMax().
 * Revision 1.3  1996/11/11 16:18:37  BOGDANK
 *     - utlFractMemSet(), utlLFractMemSet() added.
 *    Adding utlFractMemSet() and utlLFractMemSet().
 * Revision 1.2  1996/09/24 14:46:30  BONAL
 * Moved utlMoveMemory to util.c from util.h
 *
 * (C) Copyright 1995-1998, Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header types */
#include "types.h"

/* Util.c specific header files */
#include "util.h"
#include "utlport.h"
/******************************************************************************
 * FUNCTION PURPOSE: Copying Bits from a non-aligned source to non-aligned 
 *                   destination
 ******************************************************************************
 * DESCRIPTION: Copies "no_of_bits" from a non-aligned bit boundary of source 
 *              to a non-aligned bit boundary of destination.
 * void utlCopyBits (
 *       word *src ,       -  pointer to source 
 *       word *dest        -  pointer to dest   
 *       tint src_offset   -  Offset in bits of source
 *       tint dest_offset  -  Offset in bits of destination
 *       tint no_of_bits ) -  Number of bits to be copied
 *****************************************************************************/
void utlCopyBits (word *src,word *dest,tint src_offset,tint dest_offset,
                  tint num_bits)
{
  word *src_begin_word, *dest_begin_word;
  tint num_words_to_copy;
  tint s_o,d_o,diff;
  /* Variables needed for restoring the start/end dest bits */
  tint num_save_bits_begin,num_save_bits_end;
  word dest_begin_saved_word,dest_end_saved_word;
  word dest_begin_save_mask,dest_end_save_mask; 
  /* Variables needed for the loop iteration */
  word *lag_ptr,*lead_ptr;
  word  lag_extract, lead_extract;
  tint  lag_shift, lead_shift, abs_lag_shift, abs_lead_shift;
  word *dest_ptr;
  tint i,incr;

  src_begin_word    = src + src_offset/TYP_WORD_SIZE;
  dest_begin_word   = dest + dest_offset/TYP_WORD_SIZE;
  
  s_o = src_offset  % TYP_WORD_SIZE;
  d_o = dest_offset % TYP_WORD_SIZE;
  num_words_to_copy = (d_o + num_bits + TYP_WORD_SIZE-1)/TYP_WORD_SIZE;

  diff = ABS_VALUE(s_o - d_o);

  num_save_bits_begin = d_o;
  num_save_bits_end =  num_words_to_copy*TYP_WORD_SIZE - (d_o + num_bits);

  dest_begin_save_mask = (word)(~((word)0x0)) >> num_save_bits_begin;
  dest_end_save_mask   = (word)(~((word)0x0)) << num_save_bits_end;
   
  /* Saving the words needed  for restoring the bits later */
  dest_begin_saved_word = dest_begin_word[0];
  dest_end_saved_word   = dest_begin_word[num_words_to_copy - 1];


  if(s_o > d_o ) /* CASE 1 */
  {
    lag_ptr = src_begin_word;
    lead_shift = diff - TYP_WORD_SIZE;/* its a right shift operation */
    lag_shift  = diff;
    lead_extract = (~((word)0x0)) << (TYP_WORD_SIZE - diff);
  }
  else
  {
    lag_ptr = src_begin_word-sizeof(word);
    lead_shift = -diff;/* lead is a right shift operation */
    lag_shift  = TYP_WORD_SIZE - diff;
    lead_extract = (~((word)0x0)) << diff;
  }  
   
  if(src_begin_word < dest_begin_word)
  { /* left to right propogation */
    incr = sizeof(word);
    dest_ptr = dest_begin_word;
  } 
  else
  {  /* right to left propogation */
    incr = -((tint)sizeof(word));
    dest_ptr = dest_begin_word + num_words_to_copy-1;
    lag_ptr = lag_ptr + num_words_to_copy-1;    
  }
  
  lag_extract = ~lead_extract;
  lead_ptr  = lag_ptr+sizeof(word);

   /* Do the copying operation */ 
  abs_lag_shift = ABS_VALUE(lag_shift);
  abs_lead_shift = ABS_VALUE(lead_shift);
  
  for(i=0;i<num_words_to_copy;i++)
  {
    if (lag_shift >= 0) {
      *dest_ptr = ((*lag_ptr & lag_extract) << lag_shift);
    }
    else {
      *dest_ptr = ((*lag_ptr & lag_extract) >> abs_lag_shift);
    }
    if (lead_shift >= 0) {
      *dest_ptr |= ((*lead_ptr & lead_extract) << lead_shift);
    }
    else {
      *dest_ptr |= ((*lead_ptr & lead_extract) >> abs_lead_shift);
    }
      
    dest_ptr += incr;         
    lag_ptr  +=incr;
    lead_ptr +=incr;
  }


  /* Restore the saved bits overwritten  first and last destination word */

  dest_begin_word[0] = ((dest_begin_word[0] & dest_begin_save_mask)
                                             |
                       (dest_begin_saved_word & ~dest_begin_save_mask));
                     
  dest_begin_word[num_words_to_copy-1] = 
             ((dest_begin_word[num_words_to_copy-1] & dest_end_save_mask)
                                        |
                   (dest_end_saved_word & ~dest_end_save_mask));
                      

  return;
} /* utlCopyBits */

/* nothing past this point */
