#ifndef _UTILNUM_H
#define _UTILNUM_H
/******************************************************************************
 * FILE PURPOSE: Numerical utility functions header file
 ******************************************************************************
 * FILE NAME:   utilnum.h
 *
 * DESCRIPTION: Contains numerical utility definitions and routines.
 *              (C54x Version)
 *
 * TABS: NONE
 *
 * $Id: utilnum.h 1.7 1999/03/04 21:54:43 BONAL Exp $
 * 
 * REVISION HISTORY:
 *
 * $Log: utilnum.h $
 * Revision 1.7  1999/03/04 21:54:43  BONAL
 *   - utlSetDirectAddressing() function prototype added.
 * Revision 1.6  1998/04/20 16:58:12  BOGDANK
 *     - utlLFractVecMax() and utlLsVecMax() moved to utilnum.c/.s
 * Revision 1.5  1998/01/28 16:29:11  BOGDANK
 *     - Sine polynomial table made external
 *    Sine polynomial table made external.
 * Revision 1.4  1998/01/27 13:26:03  BOGDANK
 *     - utlDummy() added
 *    utlDummy() function added. Implementation may need to change once
 *    RPTB bug in C54x compiler is fixed.
 * Revision 1.3  1997/12/11 10:17:29  BOGDANK
 *     - sine poly-table moved to .const, API uses const
 *    utlSin() API uses const.
 * Revision 1.2  1997/12/10 00:10:13  PPATEL
 *  API for sin computation through polynomial fit
 * Revision 1.1  1997/09/26 14:39:48  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1995-1998, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"
#include "fract.h"

/* External variables local to utilnum.c/.s */
/* Order of a sine polynomial */
#define UTL_SIN_POLYORDER   7
extern const tint utlSinCTable[];   /* sine polynomial table (size is
                                     *  UTL_SIN_POLYORDER+1) */
/* Function prototypes (utilnum.s) */
LFract    utlLFractVecMax (const LFract *x, tint x_len);
linSample utlLsVecMax     (const linSample *x, tint x_len);
LFract    utlVecAbsSum    (const Fract *x, tint Qx, tint n, tint Qres);
LFract    utlDot          (Fract *x, tint Qx, Fract *y, tint Qy, tint n,
                           tint Qs, tint Qres);
Fract     utlSin          (const Fract x);
void      utlDummy        (void);
void      utlSetDirectAddressing (void *base);

static inline LFract utlDotInline (Fract *x, tint Qx, Fract *y, tint Qy, tint n,
                             tint Qs, tint Qres)
{
  int i;
  LFract sum = 0;

  for (i = 0; i < n; i++)
    sum += frctMul (x[i], Qx, y[i], Qy, Qs);    /* Qp = Qs */

  return (frctAdjustQ(sum,Qs,Qres));
} /* utlDotInLine */

#endif  /* _UTILNUM_H */
/* nothing past this point */
