/******************************************************************************
 * FILE PURPOSE: Packing/unpacking header
 ******************************************************************************
 * FILE NAME:   utlrepck.h
 *
 * DESCRIPTION: Contains definitions and macros that deal with packing/unpacking
 *
 *    The functions are of the form:
 *        utlPack*Into*(
 *           word *src,     # Source words/bytes
 *           word *dst,     # Destination words/bytes
 *           tint nbytes,   # Number of bytes to operate on
 *           tint skip)     # Number of bytes in 'words' to skip over before 
 *                          # performing operation.  The bytes are numbered:
 *                          # (In 24 bit word) | 0 | 1 | 2 |
 *
 *    Functions are implemented in arch/pvppack.c or arch/pvppack.asm
 *    API allows for large amounts of generality for unusual architectures.
 * 
 *    Implementations may make assumptions reducing generality, but they
 *    should check such assumptions with exceptions.  Some reductions in
 *    generality may be to require offset to be 0 or nbytes to be even 
 *    for some/all functions.
 * 
 *    In the general case, 0 <= offset < PVP_WORDSIZE_BYTES
 *                              nbytes >= 0 
 *  
 *    The design intent is to allow PVP to be quickly ported to any
 *    new architecture only by providing arch/pvppack.* with sufficient
 *    generality for the new architecture.  It also allows for efficient
 *    packing/unpacking to be performed on architectures with 1 or 2 byte
 *    wordsize such as x86 or C54x under the assumption that all sizes
 *    are even.
 *
 * TABS: NONE
 *
 * $Id: pvppack.h 1.3 1999/03/04 23:47:11 jdowdal Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: pvppack.h $
 * Revision 1.3  1999/03/04 23:47:11  jdowdal
 *    Made packing routines more general.
 * Revision 1.2  1999/03/03 23:56:19  jdowdal
 *    S3 SendIn doesnt unpack cwsegs any more
 *        plus necessary changes so debugging works before all packing is fixed
 * Revision 1.1  1999/03/03 21:19:18  jdowdal
 * Initial revision
 *
 * (C) Copyright 1999 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef __UTLREPCK_H__
#define __UTLREPCK_H__

#include "utlrepck_m.h"

void utlPackBytesIntoWords(word *bytes, word *words, tint numbytes, tint skip);
void utlUnpackWordsIntoBytes(word *words, word *bytes, tint numbytes, 
                             tint skip);
void utlRepackWordsIntoWords(word *src, word *dst, tint sskip, tint numbytes, 
                             tint skip);
tuint utlBytesToWords (tuint bytes);
void utlXorWordsIntoWords (word *src, word *dst, 
                                         tint sskip, tint nbytes, tint skip);

void utlPackTuintToTulong (tuint *src, tulong *dest, tint NumTuints);
void utlPackWordToTuint (word *src, tuint *dest, tint NumBytes);
void  utlUnpackBuffer(word *src,word *dest,tint NumBytes);
word  utlReadnbits (word *base, tuint byte,tint nbits);
tuint utlRead8bits (word *base, tuint byte);
void utlWrite8bits (word *base, tuint byte, tuint val);
void utlWrite16bits (word *base, tuint byte, tuint val);
tuint utlRead16bits (word *base, tuint byte);
void utlWrite32bits (word *base, tuint byte, tulong val);
tulong utlRead32bits (word *base, tuint byte);
void utlOR16bits(word *src,tuint bytes,tuint val);
void utlAND16bits(word *src,tuint bytes, tuint val);
#if defined(LITTLE_ENDIAN) 
void utlBigEndianTransform (tuint *src,tuint count);
#endif
void utlWrite16bitsSwap (word *base, tuint byte, tuint val);
#endif
/* Nothing past this point */
