#*******************************************************************************
#* FILE PURPOSE: Define makefile variables for C64X
#*******************************************************************************
#* FILE NAME: makedefs.mk
#*
#* DESCRIPTION: Make include file which defines variables used by all DSP
#*    Makefiles.
#*              
#* (C) Copyright 1999 Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

# Coverity intercept
BASECC      = $(TOOLSC6XCGEN)/bin/cl6x
ifneq ($(findstring $(COVINT),yesYES),)
CC          = $(PERL) $(GGROOT)/clrcase/coverity/intercept.pl $(GGROOT)/mkrel/$(CPUDIR)/coverity C6X_C_DIR $(BASECC)
else
CC          = $(BASECC)
endif

ifneq ($(findstring $(CPUTYPEDIR),shannon SHANNON),)
ifeq ($(C66X_AS_C64X),YES)
# this can be removed once C66X is finalized
CPU64XPLUS = TRUE
else
CPU66X = TRUE
endif
endif

AS          = $(TOOLSC6XCGEN)/bin/cl6x --abi=eabi
LD          = $(TOOLSC6XCGEN)/bin/cl6x --abi=eabi -z -w --match_first_section_spec=on
# Put the path to other libraries (like CSL) here
LIBPATH     = -i=$(TOOLSC6XCGEN)/lib
ARIN        = $(TOOLSC6XCGEN)/bin/ar6x -rq
AROUT       = $(TOOLSC6XCGEN)/bin/ar6x -xq 

ifneq ($(findstring $(ENDIAN),big BIG),)
TCONFTOOL   = $(TOOLSBIOSTCF)/tconf -Dconfig.importPath="$(TOOLSBIOSDOS)" -Dconfig.compilerOpts="-me"
else
TCONFTOOL   = $(TOOLSBIOSTCF)/tconf -Dconfig.importPath="$(TOOLSBIOSDOS)"
endif
COBJEXT     = oc
LOBJEXT     = ol
AOBJEXT     = oa
CDEPEXT     = dc
LDEPEXT     = dl
ADEPEXT     = da
ASMEXT      = s
LASMEXT     = sa
LIBEXT      = lib
RM          = rm -f
CP          = cp -f
MV          = mv -f
CPUDIR      = c64x

#These defines allow the use of a RTSC package in a non-RTSC environment.
ifndef RTSC_RELATIVEPATH
RTSCPKGROOT = $(GGROOT)/../../../dsps_gtmas
else
RTSCPKGROOT = $(GGROOT)/$(RTSC_RELATIVEPATH)
endif

FXDCINC = $(GGROOT)/mkrel/rtsc;$(TOOLSXDCDOS)/packages;$(BIOS_MCSDK_PDK6678_DOS);$(NWAL_C66X_DOS);$(VIDEO_MPEG4_ENC_DOS);$(VIDEO_MPEG4_DEC_DOS);$(VIDEO_MPEG2_DEC_DOS);$(VIDEO_H264_ENC_DOS);$(VIDEO_H264_DEC_DOS);$(VIDEO_H264HP_DEC_DOS)
# compiler doesn't like quotes in its search path
XDCINC = $(subst ",,$(FXDCINC))

# List libraries required by build.  BIOS cares about order.  These need
# to be specified on command line to allow boot.s to be replaced with our own

# Common Compiler options
WARNOPT      = -pds1111 -pds827 -pds824 -pds837  -pds1037 -pds195 -pdsw225 -pdsw994 -pdsw262 -pds77 -pden
#C_STD_FLAGS  = -c -k -q --mem_model:data=far -al $(WARNOPT) --consultant
C_STD_FLAGS  = -c -k -q --mem_model:data=far -al $(WARNOPT) 
C_OPT_FLAGS  = -mt -mw -os -o3 --optimize_with_debug

ifeq ($(CPU64XPLUS), TRUE)
CPUFLAGS     = -mv64+
endif

ifeq ($(CPU64X), TRUE)
CPU         = c6416
CPUFLAGS    = -mv6400 
C_STD_FLAGS += -mi200 
endif

ifeq ($(CPU66X), TRUE)
CPUFLAGS     = -mv6600
endif

# incluse appropriate files based on Big or little endian build!!
ifneq ($(findstring $(ENDIAN),big BIG),)


# CPU Class flags
ifeq ($(CPU64X), TRUE)
XDCFLAGS = -Dti_targets_C64_big_endian -Dxdc_target_types__='ti/targets/std.h' -Dxdc__nolocalnames
XDCTARGET = c64_be
# Note that other libraries like CSL can be appended here
RTSLIB      = -lrts6400e.lib
ALLFARLIB       = $(TOOLSC6XCGENDOS)/lib/rts6400e.lib
endif

ifeq ($(CPU66X), TRUE)
XDCFLAGS = -Dti_targets_elf_C66_big_endian -Dxdc_target_types__='ti/targets/std.h'
#Note to change this once C66x target is available from XDC
XDCTARGET = c66be_elf 
C_STD_FLAGS += --abi=eabi
# Note that other libraries like CSL can be appended here
#RTSLIB       = -lrts64pluse_elf.lib
RTSLIB       = -lrts6600e_elf.lib
ALLFARLIB     = $(TOOLSC6XCGENDOS)/lib/rts6600e_elf.lib

# needed to help simulate TI environment for coverity
export COVONLYDEFS = -d_TMS320C6X -d_6x_ -d__WCHAR_T_TYPE__=short

endif

ifeq ($(CPU64XPLUS), TRUE)
XDCFLAGS = -Dti_targets_C64P_big_endian -Dxdc_target_types__='ti/targets/std.h'
XDCTARGET = c64Pbe
# Note that other libraries like CSL can be appended here
RTSLIB       = -lrts64pluse.lib
ALLFARLIB       = $(TOOLSC6XCGENDOS)/lib/rts64pluse.lib

# needed to help simulate TI environment for coverity
export COVONLYDEFS = -d_TMS320C6X -d_6x_ -d__WCHAR_T_TYPE__=short

endif

#-- else little endian
else
# CPU Class flags

ifeq ($(CPU64X), TRUE)
XDCFLAGS = -Dti_targets_C64 -Dxdc_target_types__='ti/targets/std.h'
XDCTARGET = c64
CPU         = c6416
C_OPT_FLAGS = -o3
CSLLIBDOS          = $(TOOLXDAISC6XDST)/bios/lib/csl6416.lib
# Note that other libraries like CSL can be appended here
RTSLIB      = -lrts6400.lib
ALLFARLIB       = $(TOOLSC6XCGENDOS)/lib/rts6400.lib
endif


ifeq ($(CPU66X), TRUE)
XDCFLAGS = -Dti_targets_elf_C66 -Dxdc_target_types__='ti/targets/std.h'
XDCTARGET = c66le_elf
C_STD_FLAGS += --abi=eabi
# Note that other libraries like CSL can be appended here
RTSLIB       = -lrts6600_elf.lib
ALLFARLIB    = $(TOOLSC6XCGENDOS)/lib/rts6600_elf.lib
endif

ifeq ($(CPU64XPLUS), TRUE)
XDCFLAGS = -Dti_targets_C64P -Dxdc_target_types__='ti/targets/std.h'
XDCTARGET = c64Ple
# Note that other libraries like CSL can be appended here
RTSLIB       = -lrts64plus.lib
ALLFARLIB    = $(TOOLSC6XCGENDOS)/lib/rts64plus.lib
endif
# following endif ends the endian dependent flags declaration
endif

RTLIBS      = $(ALLFARLIB)

# Common compilation flags
# -c     : disable linker
# -k     : keep .asm file
# -mi200 : set imterrupt threshold to max of 200 cycles.
# -ml3   : far memory model
# -pm    : program level optimization
# -pdr   : display remarks
# -pds1111: shut off #pragma symbol warnings
# -pds827: shut off #pragma symbol warnings (symbol undefined)
# -pds824: shut off #pragma symbol warnings (can be applied to file level symbol only)
# -pds837: shut off #pragma symbol warnings (can be applied to file level symbol only)
# -q     : Quiet Mode 
# -pdv   : Verbose diagnostics
# --mem_model:data=far   : Large Memory Model (far)
# -fs    : Generated Assembly file Store directory
# -fr    : Generated Object file Store directory
# -ft    : Generated Temp file Store directory
# -mv6400 or -mv6400+    : Processor Arch Identifier
#  -al   : Generate listing file


CFLAGS  = $(CPUFLAGS) $(C_STD_FLAGS) $(C_OPT_FLAGS) $(XDCFLAGS)
ifneq ($(findstring $(ENDIAN),big BIG),)
  CFLAGS += -me
endif

# Debug flags
# -as : keep local symbols
# -g  : enable debugging    
# -ss : interlist C source and optimizer with .asm source
DCFLAGS =  -as  -ss
CFLAGS += -g
#DCFLAGS = -gp


# ASM options
# -s  : place all symbols in object file
# -l  : produce listing
# -pw : warn pipelne conflicts 
# AFLAGS = -s -l -g -mk -ms $(CPUFLAGS)
AFLAGS   =   -ea.s  -c -k -mi200 -ml3 -pm $(CPUFLAGS) $(WARNOPT) $(XDCFLAGS)
#AFLAGS += -g

ifneq ($(findstring $(ENDIAN),big BIG),)
  AFLAGS += -me
endif
TECINCS += -I$(GGROOT)/tec/env/common -I$(GGROOT)/tec/task/include -I$(GGROOT)/tec/gg/c64x -I$(GGROOT)/mkrel/rtsc -I$(TOOLSXDCDOS)/packages

# Include files for standard (non siu) modules.  Also used by linker to find
# library files.
FSTDINC  =$(TOOLSC6XCGENDOS)/include;$(TOOLSC6XCGENDOS)/lib
FSTDINC +=;$(TOOLSBIOSDOS)/ti/rtdx/include/c6000;$(TOOLSBIOSDOS)/ti/rtdx/lib/c6000
FSTDINC +=;$(TOOLSBIOSDOS)/ti/bios/include;$(TOOLSBIOSDOS)/ti/bios/lib
FSTDINC +=;$(GGROOT)/../inc/$(CPUDIR);$(GGROOT)/../inc/ecoif
FSTDINC +=;$(GGROOT)/../inc;$(GGROOT)/misc/$(CPUDIR);$(GGROOT)/misc
FSTDINC +=;$(GGROOT)/memory/$(CPUDIR);$(GGROOT)/memory
FSTDINC +=;$(ECODIR)/$(CPUDIR);$(ECODIR)
FSTDINC +=;$(GGROOT)/gg;$(GGROOT)/gg/$(CPUDIR)
FSTDINC +=;$(XDAISPATHDOS)
FSTDINC +=;$(XDCPATH)

# compiler doesn't like quotes in its search path
STDINC  =$(subst ",,$(FSTDINC);$(XDCINC))

# Include files for ASM in standard (non siu) modules
# compiler doesn't like quotes in its search path
STDAINC =$(subst ",,$(TOOLSBIOSDOS)/bios/include;$(GGROOT)/misc/$(CPUDIR))

# Include files for this platform.  Change this for test code.
# compiler doesn't like quotes in its search path
PLATINC =$(subst ",,$(GGROOT)/hw/$(CPUDIR);$(ECODIR)/gg/$(CPUDIR);$(ECODIR)/$(CPUDIR))

# Include files for siu
FSIUINC  =$(GGROOT)/siu/$(CPUDIR);$(GGROOT)/siu
# Voice Modules
FSIUINC +=;$(GGROOT)/acu;$(GGROOT)/acu/$(CPUDIR);$(GGROOT)/tdu
FSIUINC +=;$(GGROOT)/pvp;$(GGROOT)/vpu/$(CPUDIR)
FSIUINC +=;$(GGROOT)/veu;$(GGROOT)/tec
FSIUINC +=;$(GGROOT)/vpu;$(GGROOT)/ecult;$(GGROOT)/ecult/agent
FSIUINC +=;$(GGROOT)/vau;$(GGROOT)/piu;$(GGROOT)/cid;$(GGROOT)/mhm
FSIUINC +=;$(GGROOT)/rsu;$(GGROOT)/aer/aes;$(GGROOT)/piu/$(CPUDIR)
FSIUINC +=;$(GGROOT)/aer;$(GGROOT)/agc;$(GGROOT)/hw/$(CPUDIR);$(GGROOT)/ecult/agent/$(CPUDIR)
FSIUINC +=;$(GGROOT)/rcu;$(GGROOT)/tgu;$(GGROOT)/neu;$(GGROOT)/vqm
FSIUINC +=;$(GGROOT)/ggcfg/isr/$(CPUDIR);$(GGROOT)/anu;$(GGROOT)/diag;$(GGROOT)/neu/aal5
FSIUINC +=;$(GGROOT)/ecult/agent/$(CPUDIR);$(GGROOT)/pru;$(GGROOT)/cbu;$(GGROOT)/cbu/$(CPUDIR)
FSIUINC +=;$(GGROOT)/tec/ec/c5510;$(GGROOT)/tec/env/c5510;$(GGROOT)/tec/env/common
FSIUINC +=;$(GGROOT)/tec/c55x/make
FSIUINC +=;$(GGROOT)/crclib
FSIUINC +=;$(GGROOT)/telc/$(CPUDIR)
FSIUINC +=;$(GGROOT)/plr
FSIUINC +=;$(GGROOT)/ttu
# AUDIO Modules
FSIUINC +=;$(GGROOT)/adu
FSIUINC +=;$(GGROOT)/nfe
# MSU Modules
FSIUINC +=;$(GGROOT)/msu;$(GGROOT)/msu/srtp;$(GGROOT)/msu/pktcbl

#ecodrv files
FSIUINC +=;$(GGROOT)/siu/ecodrv


FSIUINC +=;$(GGROOT)/vcu


# Link Layer Modem Relay includes
FSIUINC +=;$(GGROOT)/llmru/Main;$(GGROOT)/sprtp

# Data Modem includes
FSIUINC +=;$(GGROOT)/dmp

# MPP Modules
FSIUINC +=;$(GGROOT)/mpp

# Modem Controller includes
FSIUINC +=;$(GGROOT)/mdmcp/wrap;$(GGROOT)/mdmcp/api


# DSIG Modules
FSIUINC +=;$(GGROOT)/dsig;$(GGROOT)/dsig/dsigt;$(GGROOT)/dsig/dsigr

# Mru includes
FSIUINC +=;$(GGROOT)/mru/system

# Codec includes
FSIUINC +=;$(GGROOT)/pcm;$(GGROOT)/pcm/eco
FSIUINC +=;$(GGROOT)/g711;$(GGROOT)/g711/xdais;$(GGROOT)/g711/xdais/$(CPUDIR)
FSIUINC +=;$(GGROOT)/g723/xdais;$(GGROOT)/g723/source/$(CPUDIR)
FSIUINC +=;$(GGROOT)/g729ab/eco;$(GGROOT)/g729ab/xdais;$(GGROOT)/g729ab/xdais/$(CPUDIR)
FSIUINC +=;$(GGROOT)/g729b;$(GGROOT)/g729e/eco;$(GGROOT)/G728
FSIUINC +=;$(GGROOT)/gsmefr;$(GGROOT)/g722/eco
FSIUINC +=;$(GGROOT)/gsmfr;$(GGROOT)/evrc/eco;
FSIUINC +=;$(GGROOT)/gsm_amr/source/$(CPUDIR);$(GGROOT)/gsm_amr/xdais
FSIUINC +=;$(GGROOT)/gsm_fr/source/$(CPUDIR);$(GGROOT)/gsm_fr/xdais
FSIUINC +=;$(GGROOT)/smv/source/$(CPUDIR);$(GGROOT)/smv/xdais
FSIUINC +=;$(GGROOT)/evrc/source/$(CPUDIR);$(GGROOT)/evrc/xdais
FSIUINC +=;$(GGROOT)/g722_2/eco;$(GGROOT)/g722_2/source/$(CPUDIR)
FSIUINC +=;$(GGROOT)/ilbc/eco;$(GGROOT)/g726_7/xdais
FSIUINC +=;$(GGROOT)/ilbc/source/$(CPUDIR);$(GGROOT)/ilbc/xdais

# Audio Codec includes
FSIUINC +=;$(GGROOT)/mp3;$(GGROOT)/mp3/$(CPUDIR)


# PCM Appendix 1 and 2 includes
FSIUINC +=;$(GGROOT)/plc;$(GGROOT)/plc/$(CPUDIR)
FSIUINC +=;$(GGROOT)/nmu;$(GGROOT)/nmu/$(CPUDIR)

# NDU includes
FSIUINC +=;$(GGROOT)/ndu

# BERT includes
FSIUINC +=;$(GGROOT)/bert

# NXU includes
FSIUINC +=;$(GGROOT)/nxu

# EQI includes
FSIUINC +=;$(GGROOT)/pqlec
FSIUINC +=;$(GGROOT)/eqi

#Temporary Fix.Needs to be removed;Done to get siuhdndu to compile
FSIUINC +=;$(GGROOT)/siu/hdndu

#conferencing files
FSIUINC +=;$(GGROOT)/siu/cnf
FSIUINC +=;$(GGROOT)/siu/cnf/c64x

#TFO Files
FSIUINC +=;$(GGROOT)/tfo

#Hal include files path
FSIUINC +=;$(GGROOT)/hal
FSIUINC +=;$(GGROOT)/hal/cslif

#TFO include files path
FSIUINC +=;$(GGROOT)/tfo

#MAC router include path
FSIUINC +=;$(GGROOT)/macr

ifeq ($(BUILD_TYPE),HDMOVE)
FSIUINC +=;$(GGROOT)/mkrel/rtsc/ti/mas/mcl
FSIUINC +=;$(GGROOT)/mkrel/rtsc
endif

ifeq ($(BUILD_TYPE),HDVIDEO)
FSIUINC +=;$(GGROOT)/vppu
FSIUINC +=;$(GGROOT)/vppu/RFCs
FSIUINC +=;$(GGROOT)/vppu/RFCs/RFC3016
FSIUINC +=;$(GGROOT)/vppu/RFCs/RFC2429
FSIUINC +=;$(GGROOT)/vppu/RFCs/RFC3984
FSIUINC +=;$(GGROOT)/vtk
FSIUINC +=;$(GGROOT)/vtk/iface
FSIUINC +=;$(GGROOT)/vtk/iface/H264BP_DEC
FSIUINC +=;$(GGROOT)/vtk/iface/H264BP_ENC
FSIUINC +=;$(GGROOT)/vtk/iface/MPEG2MP_DEC
FSIUINC +=;$(GGROOT)/vtk/iface/M4H3_DEC
FSIUINC +=;$(GGROOT)/vtk/iface/M4H3_ENC
FSIUINC +=;$(GGROOT)/customEco
FSIUINC +=;$(GGROOT)/customEco/src
FSIUINC +=;$(GGROOT)/tsu
FSIUINC +=;$(GGROOT)/ovly
FSIUINC +=;$(GGROOT)/atk
FSIUINC +=;$(GGROOT)/atk/iface
endif

ifeq ($(PA_SS),YES)
# These are required to see qmss_types.h and cppi_types.h
# which are required by qmss_device.c and cppi_device.c
FSIUINC +=;$(BIOS_MCSDK_PDK6678_DOS)/ti/drv/qmss
FSIUINC +=;$(BIOS_MCSDK_PDK6678_DOS)/ti/drv/cppi
endif

# compiler doesn't like quotes in its search path
SIUINC =$(subst ",,$(FSIUINC))

# Determine special -d's needed only by SIU and 2nd stage
SIUDEFS = -d$(SAMP16K) -d$(BUILD_TYPE) -d$(BERT_DEF) -dMDU_DBG -d$(SCHEDULER)

ifneq ($(findstring $(AUDIO),yes YES),)
  SIUDEFS += -dAUDIO=1
endif

ifneq ($(findstring $(BIOSIPC),yes YES),)
  SIUDEFS += -dBIOSIPC=1
endif

SIUDEFS += -dVPUGMPMSWL=$(VPU_GMPPTR_MSW_L) -dVPUGMPLSWL=$(VPU_GMPPTR_LSW_L)
SIUDEFS += -dSIU_RDNDC_TOTAL_CHANNELS=$(NUMREDUNCH)

ifneq ($(findstring $(VEU),yes YES),)
  SIUDEFS += -dVEU=1
endif

ifneq ($(findstring $(LOOPBACK),yes YES),)
  SIUDEFS += -dNDU_DBG4=1
endif
ifneq ($(findstring $(LOOPBACK1CH),yes YES),)
  SIUDEFS += -dNDU_DBG5=1
endif
# This is to enable loopback for messages only.The packets will still be sent to 
# utopia peripheral. The packets will be loopbacked to the same channel.Messages
# can be given through gel files.
ifneq ($(findstring $(LOOPBACK_MSGONLY),yes YES),)
  SIUDEFS += -dNDU_DBG4 -dNDU_DBG6 
endif

ifneq ($(findstring $(LOOPBACK_SELF),yes YES),)
  SIUDEFS += -dNDU_LOOPBACK_SELF 
endif

ifneq ($(findstring $(BLOCK_SEND_MSGS),yes YES),)
  SIUDEFS += -dNDU_BLOCK_SEND_MSGS=1
endif
# define the special define for DPR 32 BIT ALIGN FOR 6416
ifneq ($(findstring $(CPUTYPEDIR),C6416 c6416),)
  SIUDEFS += -dDPR_FORCE_32_BIT_ALIGN=1
endif
# For Himalaya to loop-back Pkts
ifneq ($(findstring $(LOOPBACK_PKTS),yes YES),)
  SIUDEFS += -dNDU_DBG7=1
endif

ifneq ($(findstring $(EMUPROF),yes YES LITE FULL),)
  SIUDEFS += -dEMU_PROFILE=1
  HALDEFS += -dEMU_PROFILE=1
  TDUDEFS += -dEMU_PROFILE=1
  DIAGDEFS+= -dEMU_PROFILE=1
  HALDIAGDEFS += -dEMU_PROFILE=1
  NEUDEFS += -dEMU_PROFILE=1
  PIUDEFS += -dEMU_PROFILE=1
  PVPDEFS += -dEMU_PROFILE=1 
  RCUDEFS += -dEMU_PROFILE=1
  VPPUDEFS += -dEMU_PROFILE=1
  TSUDEFS += -dEMU_PROFILE=1
  VTKDEFS += -dEMU_PROFILE=1
  CUSTOMECODEFS += -dEMU_PROFILE=1
  OVLYDEFS += -dEMU_PROFILE=1
ifneq ($(findstring $(EMUPROF),LITE),)
  SIUDEFS += -dEMU_PROFILE_LITE=1
  HALDEFS += -dEMU_PROFILE_LITE=1
  TDUDEFS += -dEMU_PROFILE_LITE=1
  DIAGDEFS+= -dEMU_PROFILE_LITE=1
  HALDIAGDEFS += -dEMU_PROFILE_LITE=1
endif

ifneq ($(findstring $(EMUPROF),yes YES),)
  SIUDEFS += -dEMU_PROFILE_REG=1
  HALDEFS += -dEMU_PROFILE_REG=1
  TDUDEFS += -dEMU_PROFILE_REG=1
  DIAGDEFS+= -dEMU_PROFILE_REG=1
  HALDIAGDEFS += -dEMU_PROFILE_REG=1
endif

ifneq ($(findstring $(EMUPROF),FULL),)
  SIUDEFS += -dEMU_PROFILE_FULL=1
  HALDEFS += -dEMU_PROFILE_FULL=1
  TDUDEFS += -dEMU_PROFILE_FULL=1
  DIAGDEFS+= -dEMU_PROFILE_FULL=1
  HALDIAGDEFS += -dEMU_PROFILE_FULL=1
  NEUDEFS += -dEMU_PROFILE_FULL=1
  PIUDEFS += -dEMU_PROFILE_FULL=1
  PVPDEFS += -dEMU_PROFILE_FULL=1 
  RCUDEFS += -dEMU_PROFILE_FULL=1
  VPPUDEFS += -dEMU_PROFILE_FULL=1
  TSUDEFS  += -dEMU_PROFILE_FULL=1
  VTKDEFS += -dEMU_PROFILE_FULL=1
  OVLYDEFS += -dEMU_PROFILE_FULL=1
endif

endif

ifneq ($(findstring $(PROF_IDLE_MIPS),yesYES),)
  SIUDEFS += -dPROFILE_IDLE_MIPS=1
  HALDEFS += -dPROFILE_IDLE_MIPS=1
  DIAGDEFS+= -dPROFILE_IDLE_MIPS=1
  HALDIAGDEFS += -dPROFILE_IDLE_MIPS=1
  NEUDEFS += -dPROFILE_IDLE_MIPS=1
  RCUDEFS += -dPROFILE_IDLE_MIPS=1
  VPPUDEFS += -dPROFILE_IDLE_MIPS=1
  TSUDEFS  += -dPROFILE_IDLE_MIPS=1
  VTKDEFS += -dPROFILE_IDLE_MIPS=1
  OVLYDEFS += -dPROFILE_IDLE_MIPS=1
endif

ifneq ($(findstring $(ITDM),yesYES),)
  HALDIAGDEFS += -dITDM=1
  NEUDEFS += -dITDM=1 
endif

ifneq ($(findstring $(MACR),yesYES),)
  NEUDEFS += -dMACR
endif

# define the special define for DPR 32 BIT ALIGN FOR 6416
ifneq ($(findstring $(CPUTYPEDIR),TOMAHAWK tomahawk SHANNON shannon),)
  SIUDEFS += -dDPR_FORCE_32_BIT_ALIGN=1
endif

ifeq ($(BUILD_TYPE),HDVIDEO)
  # task queue depth is 8 times number of channels
  SIUDEFS += -dSIU_TASK_QUEUE_NUMCH='(8*($(NUMCH)+$(NUMPKTCH)))'
else
  SIUDEFS += -dSIU_TASK_QUEUE_NUMCH='($(NUMCH)+$(NUMPKTCH))'
endif

SIUDEFS += -dSIU_MAX_TDM_CHANNELS='($(NUMCH))'
#This addition of task queue length to Vcudefs is to get rid of 
# compilation error when siuloc is indirectly used by vcu.c. This inclusion of
# ggcfg.h by vcu needs to be cleanedup. TODO 
VCUDEFS += -dSIU_TASK_QUEUE_NUMCH='($(NUMCH)+$(NUMPKTCH))'


ifneq ($(findstring $(CSL),yesYES),)
  HALDIAGDEFS += -dCSL=1
endif

# Added for isolating ethernet specific code
ifneq ($(findstring $(GMAC),yes YES),)
  SIUDEFS += -dNDU_GMAC=1
  SIUDEFS += -dNEU_GMAC=1
  NEUDEFS += -dNEU_GMAC=1
  DIAGDEFS += -dNEU_GMAC=1
  DIAGDEFS += -dNDU_GMAC=1
endif

ifneq ($(findstring $(UTOPIA),yes YES),)
  SIUDEFS += -dNDU_UTOPIA=1
  NEUDEFS += -dNEU_UTOPIA=1
  DIAGDEFS += -dNDU_UTOPIA=1
  DIAGDEFS += -dNEU_UTOPIA=1
endif

# Defines for modules that need 16KHz conditional compilation
VAUDEFS = -d$(SAMP16K)
TGUDEFS = -d$(SAMP16K)
VPUDEFS = -d$(SAMP16K) 

ifneq ($(findstring $(UTOPIA_8BIT),yesYES),)
  SIUDEFS += -dUTOPIA_8BIT=1
else
  SIUDEFS += -dUTOPIA_16BIT=1
endif

ifneq ($(findstring $(RTP),yesYES),)
  SIUDEFS += -dRTP=1
endif

ifneq ($(findstring $(SIUTBLPRG),yesYES),)
  SIUDEFS += -dSIUTBLPRG=1
endif

ifneq ($(findstring $(SRTPCT),yesYES),)
  SIUDEFS += -dSIU_SECURITY=1
  SIUDEFS += -dSIU_SRTP=1
endif

ifneq ($(findstring $(PKTCBLCT),yesYES),)
  SIUDEFS += -dSIU_SECURITY=1
  SIUDEFS += -dSIU_PKTCBL=1
  MSUDEFS += -dGG_INCLUDE_SECURITY_PKTCBL_RTP=1
  MSUDEFS += -dGG_INCLUDE_SECURITY_PKTCBL_RCTP=1
endif

ifneq ($(findstring $(AAL2),yesYES),)
  SIUDEFS += -dAAL2=1
endif

ifneq ($(findstring $(AAL1),yesYES),)
  SIUDEFS += -dAAL1=1
endif

ifneq ($(findstring $(IU),yesYES),)
  SIUDEFS += -dIU=1
endif

ifneq ($(findstring $(NEU),yesYES),)
  SIUDEFS += -dNEU=1
endif

ifneq ($(findstring $(TDU_GMP_AVAIL),yesYES),)
  TDUDEFS += -dTDU_USE_GMP=1
endif

ifneq ($(findstring $(PVP_GMP_AVAIL),yesYES),)
  PVPDEFS += -dPVP_USE_GMP=1
endif

ifneq ($(findstring $(RFC3267_CRC_EXT),yesYES),)
  PVPDEFS += -dPVP_RFC3267_CRC_EXT=1
endif  

ifneq ($(findstring $(LLMR_VOICE),yesYES),)
  TDUDEFS += -dTDU_USE_LLMR=1
  TGUDEFS += -dTGU_USE_LLMR=1
  SIUDEFS += -dSIU_USE_LLMR_VOICE=1
endif

ifneq ($(findstring $(TDU_ANSAM),yesYES),)
  TDUDEFS += -dTDU_ENABLE_ANSAM=1
  GGDEFS  += -dTDU_ENABLE_ANSAM=1
endif

ifneq ($(findstring $(FAX_TONE_DET),yesYES),)
  SIUDEFS += -dSIU_FAX_TONE_DET=1
endif

ifneq ($(findstring $(TELCHEMYCT),yesYES),)
  SIUDEFS += -dSIU_TELCHEMY=1
endif

ifneq ($(findstring $(TTU_CTMCT),yesYES),)
  TTUDEFS += -dTTU_CTM_SUPPORT=1
  SIUDEFS += -dSIU_TTU_CTM=1
endif

ifneq ($(findstring $(MFRELAY),yesYES),)
  TGUDEFS += -dTGU_USE_MFRELAY=1
endif

ifneq ($(findstring $(TDU_HD),yesYES),)
  TDUDEFS += -dTDU_USE_ALL=1
  SIUDEFS += -dSIU_TDU_HD=1
endif

ifneq ($(findstring $(TDU_SAV_MIPS),yesYES),)
  TDUDEFS += -dV21_MIPS=1
  TDUDEFS += -dV25_RX_MIPS=1
endif

ifneq ($(findstring $(TDU_FFT_RADIX4),yesYES),)
  TDUDEFS += -dTDU_FFT_RADIX4=1
endif

ifneq ($(findstring $(TDU_FFT_RADIX4),yesYES),)
  TDUDEFS += -dTDU_FFT_RADIX4 
endif

ifneq ($(findstring $(TDU_FFT64PT_CVERSION),yesYES),)
  TDUDEFS += -dTDU_CVER_FFT64PT
endif

ifneq ($(findstring $(V92AWARE_V21FLAGS),yesYES),)
  TDUDEFS += -dTDU_V92AWARE_V21FLAGS=1
endif

ifneq ($(findstring $(V32EXT_DET),yesYES),)
  TDUDEFS += -dTDU_V32EXT_DETECT=1
endif

ifneq ($(findstring $(VQM_H248XNQ),yesYES),)
  SIUDEFS += -dVQM_H248XNQ=1
  VPUDEFS += -dVQM_H248XNQ=1
  RCUDEFS += -dVQM_H248XNQ=1
endif

ifneq ($(findstring $(VQM_3611),yesYES),)
  SIUDEFS += -dVQM_RFC3611=1
  VPUDEFS += -dVQM_RFC3611=1
  RCUDEFS += -dVQM_RFC3611=1
endif

PLR_DEFINED = NO
ifneq ($(findstring $(PLRRED),YES yes),)
  PLR_DEFINED = YES
  PLRDEFS += -dPLRRED=1
  RCUDEFS += -dPLRRED=1
  PVPDEFS += -dPLRRED=1
  GGDEFS  += -dPLRRED=1
endif

ifneq ($(findstring $(PLRFEC),YES yes),)
  PLR_DEFINED = YES
  PLRDEFS += -dPLRFEC=1
  RCUDEFS += -dPLRFEC=1
  PVPDEFS += -dPLRFEC=1
  GGDEFS  += -dPLRFEC=1
endif

ifeq ($(PLR_DEFINED),YES)
  PLRDEFS += -dMAX_RED_LEVEL_VOICE=$(PLRRED_VOICE_LEV)
  PLRDEFS += -dMAX_RED_LEVEL_FAX=$(PLRRED_FAX_LEV)
  PLRDEFS += -dMAX_RED_LEVEL_TEXT=$(PLRRED_TEXT_LEV)
  PLRDEFS += -dMAX_FEC_LEVEL_2198=$(PLRFEC_2198_LEV)
  GGDEFS  += -dGG_MAX_PLR_VOICE_RED_LEVEL=$(PLRRED_VOICE_LEV)
  GGDEFS  += -dGG_MAX_PLR_TEXT_RED_LEVEL=$(PLRRED_TEXT_LEV)
  GGDEFS  += -dGG_MAX_PLR_FEC_LEVEL_2198=$(PLRFEC_2198_LEV)
ifeq ($(PLRVBDOVL),YES)
  PLRDEFS += -dPLR_VOICE_OVERLAY=1
endif
ifeq ($(PLRSUPPGMP),YES)
  PLRDEFS += -dPLR_USE_GMP_SUPPLEMENT=1
endif
  SIUDEFS += $(PLRDEFS)
endif

ifneq ($(findstring $(SUBSLOT),YES yes),)
  SIUDEFS += -dSIU_DYNSUBSLOT=1
  GGDEFS  += -dGG_DYNSUBSLOT=1
endif

ifneq ($(findstring $(G711A1),yesYES),)
  PLCDEFS += -dG711A1=1 
  GGDEFS  += -dG711A1=1
  SIUDEFS += -dG711A1=1
endif

ifneq ($(findstring $(G711A1_USE_GMP),yesYES),)
  PLCDEFS += -dG711A1_USE_GMP=1 -dGMP=1
  VPUDEFS += -dG711A1_USE_GMP=1
  GGDEFS  += -dG711A1_USE_GMP=1
  SIUDEFS += -dG711A1_USE_GMP=1
endif

ifneq ($(findstring $(G711A2),yesYES),)
  PVPDEFS += -dG711A2=1
  NMUDEFS += -dG711A2=1
  GGDEFS  += -dG711A2=1
endif

ifneq ($(findstring $(LEC_5P5MS),yesYES),)
  SIUDEFS += -dSIU_5P5MS=1
endif

ifneq ($(findstring $(SLIM),yesYES),)
  SIUDEFS += -dSIU_SLIM=1
endif

ifneq ($(findstring $(PLRRED),YES yes),)
  SIUDEFS += -dPLRRED=1
  PLRDEFS += -dPLRRED=1
endif

ifneq ($(findstring $(XDAIS),YESyes),)
  SIUDEFS += -dXDAIS=1
  VCUDEFS += -dXDAIS=1
endif

ifneq ($(findstring $(SLIM),yesYES),)
  SIUDEFS += -dSIU_SLIM=1
endif

ifneq ($(findstring $(TONESUPPRESS),YESyes),)
  SIUDEFS += -dSIU_TONE_SUPPRESSION=1
endif

ifneq ($(findstring $(REPLI),YESyes),)
  PIUDEFS += -dPIU_REP=1
  GGDEFS  += -dGG_REP=1
  SIUDEFS += -dSIU_REP=1
endif

#GGDEFS used for makegg second stage compile files 
ifneq ($(findstring $(MIPSCNT),YESyes),)
  GGDEFS  += -dMIPS_CNT=1
  HALDEFS += -dMIPS_CNT=1
  SIUDEFS += -dMIPS_CNT=1
endif

ifneq ($(findstring $(TDU_SAVEBUF_EXT),YESyes),)
  GGDEFS  += -dGG_TDU_SAVEBUF_EXT=1
  SIUDEFS += -dSIU_TDU_SAVEBUF_EXT=1
endif

HALDEFS += -d$(SCHEDULER)

# SIMULATE 500 MHz for TH
ifneq ($(findstring $(SIMULATE_TH_500MHZ),YESyes),)
  HALDEFS += -dSIMULATE_TH_500MHZ
endif




ifneq ($(findstring $(AIC),AIC10),)
  AICDEFS = -dGG_AIC10=1
endif

ifneq ($(findstring $(SIMULATION_BUILD),YESyes),)
  SIUDEFS += -dSIMULATION_BUILD=1
  HALDEFS += -dSIMULATION_BUILD=1
endif

ifneq ($(findstring $(MSG_SPOOFING),YESyes),)
  SIUDEFS += -dMSG_SPOOFING=1
endif

ifneq ($(findstring $(TDM_SPOOFING),YESyes),)
  SIUDEFS += -dTDM_SPOOFING=1
endif

ifneq ($(findstring $(AIC),AIC22),)
  AICDEFS = -dGG_AIC22=1
endif

ifneq ($(findstring $(AIC),AIC10AIC22),)
  AICDEFS = -dGG_AIC10_AIC22=1
endif

ifneq ($(findstring $(AIC),AIC20),)
  AICDEFS = -dGG_AIC20=1
endif

ifneq ($(findstring $(AIC),AIC23),)
  AICDEFS = -dGG_AIC23=1
endif

ifneq ($(findstring $(TDM_10MS),yes YES),)
  SIUDEFS += -dSIU_TDM_10MS=1
  PCMDEFS += -dPCM_TDM_10MS=1
  ANUFLAGS += -dANU_TDM_10MS=1
  VCUDEFS += -dVCU_TDM_10MS=1
  TFODEFS += -dTFO_TDM_10MS=1
  PLCDEFS += -dPLC_TDM_10MS=1
  VEUDEFS += -dVEU_TDM_10MS=1
  PIUDEFS += -dPIU_TDM_10MS=1
endif

ifneq ($(findstring $(ETOI_SWAP),ETOI_SWAP_LEGACY),)
  GGDEFS +=  -dETOI_SWAP_LEGACY=1
  SIUDEFS += -dETOI_SWAP_LEGACY=1
endif

ifneq ($(findstring $(ETOI_SWAP),ETOI_SWAP_PROG),)
  GGDEFS +=  -dETOI_SWAP_PROG=1
  SIUDEFS += -dETOI_SWAP_PROG=1
endif

ifneq ($(findstring $(ETOI_SWAP),ETOI_SWAP_DATA),)
  GGDEFS +=  -dETOI_SWAP_DATA=1
  SIUDEFS += -dETOI_SWAP_DATA=1
endif

ifneq ($(findstring $(CH_SWAP),yes YES),)
  SIUDEFS += -dSIU_CH_SWAP=1
endif

ifneq ($(findstring $(EDMA_PAGING),yes YES),)
  SIUDEFS += -dSIU_PAGE_DATA=1
  GGDEFS += -dGG_EDMA_PAGING=1
endif

ifneq ($(findstring $(TFO_AE),yesYES),)
  SIUDEFS += -dSIU_TFO=1 -dPIU_TFO=1 -dVCU_TFO=1 -dVCU_TFO_AE=1 -dGG_TFO_AE=1
  VCUDEFS += -dVCU_TFO=1 -dVCU_TFO_AE=1
  PIUGLOBDEFS += -dPIU_TFO=1
  GGDEFS += -dGG_TFO_AE=1
  TFODEFS += -dTFO_OPT_AE=1
endif

ifneq ($(findstring $(TFO_IPE),yesYES),)
  SIUDEFS += -dSIU_TFO=1 -dPIU_TFO=1 -dVCU_TFO=1 -dVCU_TFO_IPE=1 -dGG_TFO_IPE=1
  VCUDEFS += -dVCU_TFO=1 -dVCU_TFO_IPE=1
  PIUGLOBDEFS += -dPIU_TFO=1
  GGDEFS += -dGG_TFO_IPE=1
  TFODEFS += -dTFO_OPT_IPE=1
  TFOSTDIPE = TFO_INPATH
endif

ifneq ($(findstring $(TFO_DEBUG),yesYES),)
  SIUDEFS += -dSIU_TFO_DEBUG=1
  VCUDEFS += -dVCU_TFO_DEBUG=1
  PIUGLOBDEFS += -dPIU_TFO_DEBUG=1
  TFODEFS += -dTFO_DEBUG=1
endif

ifneq ($(findstring $(TFO_TEST),yesYES),)
  TFODEFS += -dTFO_TEST=1
endif

ifneq ($(findstring $(GSMFR),"yes YES"),)
  TFODEFS += -dTFO_BLD_GSMUMTS=1
  TFOSTDGSM = GSM_UMTS
endif

ifneq ($(findstring $(GSMEFR),"yes YES"),)
  TFODEFS += -dTFO_BLD_GSMUMTS=1
  TFOSTDGSM = GSM_UMTS
endif

ifneq ($(findstring $(GSMAMR),"yes YES"),)
  TFODEFS += -dTFO_BLD_GSMUMTS=1
  TFOSTDGSM = GSM_UMTS
endif

ifneq ($(findstring $(SMV),"yes YES"),)
  TFODEFS += -dTFO_BLD_CDMA=1
  TFOSTDCDMA = CDMA
endif

ifneq ($(findstring $(EVRC),"yes YES"),)
  TFODEFS += -dTFO_BLD_CDMA=1
  TFOSTDCDMA = CDMA
endif

ifneq ($(findstring $(LEC),ecuECU),)
  SIUDEFS += -dLEC_ECU=1
else
ifneq ($(findstring $(LEC),tecTEC),)
  SIUDEFS += -dLEC_TEC=1
  PIUGLOBDEFS += -dLEC_TEC=1
  MISCDEFS += -dLEC_TEC=1
  HALDEFS += -dGG_TEC=1
  VEUDEFS += -dLEC_TEC=1
else
ifneq ($(findstring $(LEC),tecengTECENG),)
  SIUDEFS += -dLEC_TECENG=1
  PIUGLOBDEFS += -dLEC_TECENG=1
  MISCDEFS += -dLEC_TECENG=1
  HALDEFS += -dGG_TEC=1
  VEUDEFS += -dLEC_TECENG=1
endif
endif
endif

ifneq ($(findstring $(NX64),yesYES),)
  SIUDEFS += -dNX64=1
  PIUGLOBDEFS += -dNX64=1
endif

ifneq ($(findstring $(EQI),yesYES),)
  SIUDEFS += -dEQI=1
endif

ifneq ($(findstring $(PA_SS),yesYES),)
  SIUDEFS     += -dPA_SS=1
  NEUDEFS     += -dPA_SS=1
  NDUDEFS     += -dPA_SS=1
  DIAGDEFS    += -dPA_SS=1
  HALDIAGDEFS += -dPA_SS=1
  HALDEFS     += -dPA_SS=1
endif

ifneq ($(findstring $(TSIP_LLD),yesYES),)
  SIUDEFS     += -dTSIP_LLD=1
  DIAGDEFS    += -dTSIP_LLD=1
  HALDIAGDEFS += -dTSIP_LLD=1
  HALDEFS     += -dTSIP_LLD=1
endif

ifneq ($(findstring $(EDMA_LLD),yesYES),)
  HALDIAGDEFS += -dEDMA_LLD=1
  HALDEFS     += -dEDMA_LLD=1
endif

ifdef MAX_PROF_SIZE
  PROFDEFS += -dMAX_PROF=$(MAX_PROF_SIZE)
  SIUDEFS += $(PROFDEFS)
  VCUDEFS += $(PROFDEFS)
endif

ifneq ($(findstring $(CBUCT),yesYES),)
  SIUDEFS += -dSIU_CBU=1
endif

ifneq ($(findstring $(MRU),yesYES),)
  SIUDEFS += -dMRU=1
endif

ifneq ($(findstring $(DMP),yesYES),)
  SIUDEFS += -dDMP=1
endif

ifneq ($(findstring $(LLMRU),yesYES),)
  SIUDEFS += -dLLMRU=1
endif

ifneq ($(findstring $(SPRT),yesYES),)
  SIUDEFS += -dSPRT=1
endif

ifneq ($(findstring $(AGC),yesYES),)
  SIUDEFS += -dAGC=1
endif

ifneq ($(findstring $(RSU),yesYES),)
  SIUDEFS += -dRSU=1
endif

ifneq ($(findstring $(TTU),yesYES),)
  SIUDEFS += -dTTU=1
endif

ifneq ($(findstring $(VQMON),yesYES),)
  SIUDEFS += -dVQMON=1
  GGDEFS += -dVQMON=1
endif

ifneq ($(findstring $(LEC_RELOC),yesYES),)
  GGDEFS += -dECU_HAS_RELOC=1
endif

ifneq ($(findstring $(MCS_DATA),yesYES),)
  SIUDEFS += -dSIU_MULTICORE_SHARED_DATA=1
endif

ifneq ($(findstring $(RELMSG),yesYES),)
  SIUDEFS += -dRELMSG_SUPPORT=1
endif

ifneq ($(findstring $(GMP),yesYES),)
  RCUDEFS += -dGMP=1
endif


ifneq ($(findstring $(DSIG),yesYES),)
  SIUDEFS += -dSIU_RDNDC_INCLUDE_DSIG=1
endif

ifneq ($(findstring $(NWLITECH),yesYES),)
  SIUDEFS += -dSIU_NWLITECH=1
endif

ifneq ($(findstring $(VMMCOMP),yesYES),)
  GGDEFS += -dGG_INCLUDE_VMMCOMP=1
endif


ifneq ($(findstring $(TIMER_BASED_TDM),yesYES),)
  SIUDEFS += -dSIU_TIMER_BASED_TDM=1
  HALDEFS += -dHAL_TIMER_BASED_TDM=1
endif

ifneq ($(findstring $(TIMER_BASED_TDM_EXT),yesYES),)
  SIUDEFS += -dSIU_TIMER_BASED_TDM_EXT=1
  HALDEFS += -dHAL_TIMER_BASED_TDM_EXT=1
endif

ifneq ($(findstring $(CID),YESyes),)
GGDEFS  += -dGG_CID
endif

#Packetization related definitions
NEUDEFS += -dNUM_MAX_PKT_SEGMENTS=$(PKT_SEGMENTS)

#Dynamic Heap Allocation related definitions
GGDEFS += -dGG_DYNHEAP_NUM_MODULES=$(DH_NUM_MOD)
SIUDEFS+= -dGG_DYNHEAP_NUM_MODULES=$(DH_NUM_MOD)
SIUDEFS+= -dSIU_MAX_VOICE_CH_SLOTS=$(NUMCH)
GGDEFS += -dGG_NUM_DYNHEAP_TYPES=$(DH_NUM_HEAP_TYPES)
SIUDEFS+= -dGG_DYNHEAP_MAX_HEAP_TYPES=$(DH_NUM_HEAP_TYPES)
SIUDEFS+= -dGG_DYNHEAP_NUM_NONCH_HEAPS=$(DH_NUM_NONCH_HEAPS)

# # of cores
SIUDEFS+= -dCPU_NCORES=$(CPU_NCORES)
VCUDEFS+= -dCPU_NCORES=$(CPU_NCORES) # VCU should not include this!
DIAGDEFS+= -dCPU_NCORES=$(CPU_NCORES)
HALDIAGDEFS+= -dCPU_NCORES=$(CPU_NCORES)

#What should be the input value/argument to make it generic?To be reviewed and found out
MPPDEFS+= -dMPP_NO_FAX

#Redundancy related definitions
SIUDEFS+= -dSIU_RDNDC_SEC_CHANNELS=$(NUM_MAX_SECCH)
SIUDEFS+= -dSIU_RDNDC_TOTAL_CHANNELS=$(NUMCH)
# Get target independant definitions
include $(GGROOT)/mkrel/makedefs.mk

#VTK related definitions
ifneq ($(findstring $(VTK_REDUCE_MIPS),yesYES),)
  VTKDEFS += -dVTK_ACTIVE_MIPS_CTRL=1
endif

ifneq ($(findstring $(ENDIAN),big BIG),)
SIUDEFS += -BIGEND
endif

# Nothing past this point
