#*******************************************************************************
#* FILE PURPOSE: Perform standard variable computation and targets for ECOs
#*******************************************************************************
#* FILE NAME: makeeco.mk
#*
#* DESCRIPTION: Make include file which computes some variables and defines
#*    standard targets for DSP ECOs.
#*
#*    Input variables:
#*         CSRC   : List of C Source files
#*         ASMSRC : List of assembly source files
#*    Output variables:
#*         OBJS   : List of all object files
#*
#*              
#* (C) Copyright 1999-2001 Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

# get definitions
include $(GGROOT)/mkrel/c64x/makedefs.mk

# If ECODIR is not defined, derive it from ECONAME
ifndef ECODIR
   ECODIR = $(GGROOT)/$(ECONAME)
endif

# VPATH tells make how to find sources in other directories
vpath %.$(ASMEXT) $(ECODIR)/$(CPUDIR)
#The vpath below is needed to compile the dsp/memory folder properly
vpath %.c $(ECODIR)/$(CPUDIR)/$(CPUTYPEDIR)
vpath %.c $(ECODIR)/$(CPUDIR)
vpath %.sa $(ECODIR)/$(CPUDIR)
vpath %.s62 $(ECODIR)/$(CPUDIR)
vpath % $(ECODIR)

# Get filenames w/o paths

CFILES     = $(foreach file, $(CSRC), $(notdir $(file)))
AFILES     = $(foreach file, $(ASMSRC), $(notdir $(file)))
LINAFILES  = $(foreach file, $(LINASMSRC), $(notdir $(file)))
BIOSSFILES = $(foreach file, $(BIOSASMSRC), $(notdir $(file)))
EXCFILES   = $(foreach file, $(EXCSRC), $(notdir $(file)))
EXAFILES   = $(foreach file, $(EXASMSRC), $(notdir $(file)))
TESTCFILES = $(foreach file, $(TESTCSRC), $(notdir $(file)))
TESTAFILES = $(foreach file, $(TESTASMSRC), $(notdir $(file)))
TESTLINASMFILES = $(foreach file, $(TESTLINASMSRC), $(notdir $(file)))


DFULLPATH = $(patsubst %.s62, %.$(ADEPEXT),$(BIOSSFILES)  \
              $(patsubst %.c, %.$(CDEPEXT), $(CFILES) $(EXCFILES) $(TESTCFILES)) \
              $(patsubst %.$(ASMEXT), %.$(ADEPEXT), $(AFILES) $(EXAFILES) $(TESTAFILES) $(BIOSSFILES)) \
              $(patsubst %.sa, %.$(LDEPEXT), $(LINAFILES) $(TESTLINASMFILES)))
			

OBJS  = $(COBJS) $(AOBJS)
COBJS = $(patsubst %.c, %.$(COBJEXT), $(CFILES))
AOBJS = $(patsubst %.s62, %.$(AOBJEXT), $(BIOSSFILES) \
         $(patsubst %.$(ASMEXT), %.$(AOBJEXT), $(AFILES)) \
         $(patsubst %.sa, %.$(LOBJEXT), $(LINAFILES)))
REFTGT = 


# extra objects (not built by builtin rules)
EXOBJS = $(patsubst %.s62, %.$(AOBJEXT), $(BIOSSFILES) \
           $(patsubst %.c, %.$(COBJEXT), $(EXCFILES)) \
           $(patsubst %.$(ASMEXT), %.$(AOBJEXT), $(EXAFILES)))
 
# extra objects (used for test code)
TESTOBJS = $(patsubst %.s62, %.$(AOBJEXT), $(BIOSSFILES) \
             $(patsubst %.c, %.$(COBJEXT), $(TESTCFILES)) \
             $(patsubst %.$(ASMEXT), %.$(AOBJEXT), $(TESTAFILES)) \
             $(patsubst %.$(LASMEXT), %.$(LOBJEXT), $(TESTLINASMFILES)))
 
# intermediate files
CLEAN_INTERMEDIATES = $(patsubst %.s62, %.lst, $(BIOSSFILES)  \
                        $(patsubst %.c, %.asm, $(CFILES) $(EXCFILES) $(TESTCFILES)) \
                        $(patsubst %.c, %.lst, $(CFILES) $(EXCFILES) $(TESTCFILES)) \
                        $(patsubst %.c, %.cif, $(CFILES) $(EXCFILES) $(TESTCFILES)) \
                        $(patsubst %.$(ASMEXT), %.lst, $(AFILES) $(EXAFILES) $(TESTAFILES)) \
						$(patsubst %.sa, %.asm, $(LINAFILES) $(TESTLINASMFILES)) \
						$(patsubst %.sa, %.lst, $(LINAFILES) $(TESTLINASMFILES)) \
                        $(DFULLPATH))

# Compute FLATFILES, if necessary
ifndef FLATFILES
  FLATFILES  =$(patsubst %,$(ECODIR)/%,$(CSRC) $(ASMSRC) $(HDR) $(LINASMSRC) $(BIOSASMSRC) )
endif

# If FLATGGDIR isnt set, and we make flat, then we are making flat for test prg 
ifndef FLATGGDIR
  # This has two magical effects: 1) causes FLATGGDIR to be set now, and
  # causes recursive makefiles to set the correct FLAT directory even during
  # recursion.
  FLATFILES +=$(patsubst %,$(ECODIR)/%,$(TESTCSRC) $(TESTASMSRC) $(TESTHDR))
  ifdef ECONAME
    export FLATGGDIR = $(ECONAME)/c64x/make/flat
  else
    export FLATGGDIR = $(notdir $(ECODIR))/c64x/make/flat
  endif
endif

FLATDIR    =$(GGROOT)/$(FLATGGDIR)

ifneq ($(findstring $(FLAT),yes YES),)
  FLAT_TARGET =$(REFTGT) flat
else
  FLAT_TARGET =$(REFTGT)
endif

.PHONY: clean flat gen_flatdir 

# standard targets
clean:
	-$(RM) $(OBJS) $(EXOBJS) $(TESTOBJS) $(CLEAN_TARGETS) $(CLEAN_INTERMEDIATES) $(REFTGT)
	-$(RM) -r flat

ifneq ($(findstring $(FLAT),yes YES),)
flat: 
	-mkdir -p $(FLATDIR)
ifneq ($(FLATFILES),)
	-$(CP) -u $(FLATFILES) $(FLATDIR)
endif

# Define command to copy targets to flat
CPFLATCMD = $(CP) $@ $(FLATDIR)

else

CPFLATCMD =

endif

# Copy targets to delivery directory
# The variables STDDLV and BOOTLD should be set by calling makefile
ifneq ($(BOOTLD),none)
   CPBOOTLD=$(CP) $(BOOTPATH)/$(BOOTLD) $(DELIVERY)/boot_$(OUTFILE).out
   CPBOOTLD2=
else
   ifeq ($(GGDSP_CPU),12)
     CPBOOTLD=$(CP) $(BOOTPATH)/boot5560_pg10_20.out $(DELIVERY)/bootpg10_20_$(OUTFILE).out
     CPBOOTLD2=
   else
     CPBOOTLD=
     CPBOOTLD2=
   endif
endif

# how to build .c and .s files on c64x
FNAMEDEF   = -dTNFNAME=$(notdir $<)
GGCCOMPILE = $(CC) $(CFLAGS) $(DCFLAGS) $(CDEFS) $(FNAMEDEF) $(FUNCOPTS) $< -eo.$(COBJEXT)
GGLCOMPILE = $(CC) $(CFLAGS) $(DCFLAGS) $(CDEFS) $(FNAMEDEF) $< -eo.$(LOBJEXT)
GGACOMPILE = $(AS) $(AFLAGS) $(ADEFS) $< -eo.$(AOBJEXT)

# Use C6000 C compiler to generate compiler dependancies
#
# The SED sequences 
# 1) 's/\\/\//g' flip slashes  eg c:\test => c:/test
# 2) 's/ /\\ /g' escape spaces eg c:/program files => c:/program\ files
# 3) 's/:\\ /: /' remove escaped space after rule's : caused by expression 2
# 4) change *.obj to *.oc and add a dependance to *.dc 
# 5) 's/ \([a-zA-Z]\):/ \/\U\1/' convert c:/ to /C/
GGCMAKEDEP = $(CC) -ppd=$(patsubst %.$(CDEPEXT),%.$(CDEPEXT)_TMP,$@) $(CFLAGS) $(DCFLAGS) $(CDEFS) $(FNAMEDEF) $(FUNCOPTS) $< \
	&& sed -e 's/\\/\//g' \
              -e 's/ /\\ /g' \
	      -e 's/:\\ /: /' \
              -e 's/$(notdir $*).obj[ ]*:[ ]*/$(notdir $*).$(COBJEXT) $(subst /,\/,$@): /' \
	      -e 's/ \([a-zA-Z]\):/ \/\U\1/' \
	< $(patsubst %.$(CDEPEXT),%.$(CDEPEXT)_TMP,$@) > $@ \
	&& $(RM) $(patsubst %.$(CDEPEXT),%.$(CDEPEXT)_TMP,$@)

# Use C6000 C compiler to generate linear assembly dependancies
#
# The SED sequences 
# 1) 's/\\/\//g' flip slashes  eg c:\test => c:/test
# 2) 's/ /\\ /g' escape spaces eg c:/program files => c:/program\ files
# 3) 's/:\\ /: /' remove escaped space after rule's : caused by expression 2
# 4) change *.obj to *.oc and add a dependance to *.dc 
# 5) change file.asm to file.sa (because compiler first made a .asm then assembled it)
# 6) 's/ \([a-zA-Z]\):/ \/\U\1/' convert c:/ to /C/
GGLMAKEDEP = $(CC) -apd $(CFLAGS) $(DCFLAGS) $(CDEFS) $(FNAMEDEF) $< \
	&& sed -e 's/\\/\//g' \
              -e 's/ /\\ /g' \
	      -e 's/:\\ /: /' \
              -e 's/$(notdir $*).obj[ ]*:[ ]*/$(notdir $*).$(LOBJEXT) $(subst /,\/,$@): /' \
	      -e 's/$(notdir $*).asm/$(subst /,\/,$<)/' \
	      -e 's/ \([a-zA-Z]\):/ \/\U\1/' \
	< $(patsubst %.$(LDEPEXT),%.ppa,$@) > $@ \
	&& $(RM) $(patsubst %.$(LDEPEXT),%.$(LDEPEXT)_TMP,$@) $(patsubst %.$(LDEPEXT),%.ppa,$@)

# Use C6000 C compiler to generate scheduled assembly dependancies
#
# The SED sequences 
# The following GGAMAKEDEP can be used if the codegen tools can generate dependancies for scheduled assembly files
# The SED sequences 
# 1) 's/\\/\//g' flip slashes  eg c:\test => c:/test
# 2) 's/ /\\ /g' escape spaces eg c:/program files => c:/program\ files
# 3) 's/:\\ /: /' remove escaped space after rule's : caused by expression 2
# 4) change *.obj to *.oc and add a dependance to *.dc 
# 5) 's/ \([a-zA-Z]\):/ \/\U\1/' convert c:/ to /C/
GGAMAKEDEP = $(CC) -apd $(AFLAGS) $(ADEFS) $< \
	&& sed -e 's/\\/\//g' \
              -e 's/ /\\ /g' \
	      -e 's/:\\ /: /' \
              -e 's/$(notdir $*).obj[ ]*:[ ]*/$(notdir $*).$(AOBJEXT) $(subst /,\/,$@): /' \
	      -e 's/ \([a-zA-Z]\):/ \/\U\1/' \
	< $(patsubst %.$(ADEPEXT),%.ppa,$@) > $@ \
	&& $(RM) $(patsubst %.$(ADEPEXT),%.$(ADEPEXT)_TMP,$@) $(patsubst %.$(ADEPEXT),%.ppa,$@)

#Two options of invoking blockexpn:
# 1)Passing NUMCH for implicit expansion 
# 2)Explicitly giving the number of expansions in the .bex file
GGBLOCKEXPN = $(BLOCKEXPN) $< $@
GGBLOCKEXPNNUMCH = $(BLOCKEXPN) $< $@ $(NUMCH) $(NUMSL) lastregch=$(LASTREGCH) firstauxch=$(FIRSTAUXCH)

%.$(COBJEXT): %.c
	$(GGCCOMPILE)

# Compilation of Linear Assembly files
%.$(LOBJEXT): %.$(LASMEXT)
	$(GGLCOMPILE)

%.$(AOBJEXT): %.$(ASMEXT)
	$(GGACOMPILE)

CDEPINC = -I$(subst ;, -I,$(C6X_C_DIR))
ADEPINC = -I$(subst ;, -I,$(C6X_A_DIR))

%.$(CDEPEXT): %.c
	$(GGCMAKEDEP)

# dependency for .sa files
%.$(LDEPEXT): %.$(LASMEXT)
	$(GGLMAKEDEP)

%.$(ADEPEXT): %.$(ASMEXT)
	$(GGAMAKEDEP)

# Preprocessing via blockexpn.pl
# ggmbuf.bec and ggmheap.bec are not build specific but common.They need to use the
# NUMCH parameter
%.c: %.bec
	$(GGBLOCKEXPNNUMCH)

#Numch parameter not passed to linker command file / memmap as the number of expansion 
#required  may be different from num channels in the build
%.h: %.beh
	$(GGBLOCKEXPNNUMCH)

%.cmd: %.becmd
	$(GGBLOCKEXPNNUMCH)	

# Reflection page symbol tables
# %.oc is included so we inherit the dependancies from the .dc file
# Set GGREFDBG=-v# on command line to debug
%.ref: %.c %.oc $(REFPAGEDEP)
	$(REFPAGE) $(GGREFDBG) -iC6X_C_DIR $< -s$@ $(CDEFS) -dGG_REF_PAGE -d6416 -t$(REFSYM)
	
ifneq ($(DOMAKEDEP),no)
# include generated dependancies.
sinclude $(DFULLPATH)
endif

# Nothing past this point
