/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _NDUPORT_H
#define _NDUPORT_H
/******************************************************************************
 * FILE PURPOSE: NDU Portability Definitions (C64x version)
 ******************************************************************************
 * FILE NAME:   nduport.h
 *
 * DESCRIPTION: Contains NDU portability definitions.  Presently this file
 *              is used to support 3 different hardware platforms:c54x,c55x,c64x
 *              This file defines the NDU_HPIADDRESS_DIV_FACTOR which is needed
 *              for proper calculation of HPI Base addres by the micro.
 *              Micro always multiplies the provided HPI base address by two to
 *              get the byte address of HPI.
 *              This does not work on processors like c6x.Hence
 *              Need  
 ******************************************************************************
*/
#define HIGH_WORD(a)          ((a >> 16) & 0xffff)
#define LOW_WORD(a)           (a & 0xffff)

#define NDU_HPIADDRESS_DIV_FACTOR   2
#define NDU_HPI_32_BIT_ALIGNMASK  0x3 /* c6x is byte addressable and 32-bit aligned address must have its last to bits as 0.
                                         Hence  NDU_HPI_32_BIT_ALIGNMASK is defined to be 0x3 */

static inline void ndu_boot_status(volatile tuint *boot_status)
{   
while(*boot_status != 0);
}

#ifndef DEFAULT_CODE_SECTION
#pragma CODE_SECTION (ndu_gm_modify_vc_msg_header, ".controlProg")
#pragma CODE_SECTION (gmacndu_rcv_message,".controlProg")/* TODO! review it!*/
#pragma CODE_SECTION (gmacndu_msg_send, ".controlProg")
#pragma CODE_SECTION (nduGmacSendInCore, ".tdm2pktProg")
#pragma CODE_SECTION (gmacndu_rxcall_neu, ".initProg")
#pragma CODE_SECTION (gmacndu_fset,".initProg")
#pragma CODE_SECTION (gmacndu_get_udp,".initProg")
#pragma CODE_SECTION (gmacndu_message_port_map,".initProg")
#pragma CODE_SECTION (gmacndu_block_port_map,".controlProg")
#pragma CODE_SECTION (NDU_gmacSetIntMode,".controlProg")
#pragma CODE_SECTION (NDU_gmacSendMsg, ".controlProg")
#pragma CODE_SECTION (NDU_gmacSendData, ".tdm2pktProg")
#pragma CODE_SECTION (NDU_gmacReceive, ".pkt2tdmProg")
#pragma CODE_SECTION (NDU_gmacMsgBg, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (NDU_gmacGetMsg,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (NDU_gmacMainTask,".tdm2pktpkt2tdmProg")/* TODO! review it!*/ 
#pragma CODE_SECTION (NDU_gmacGetStats,".controlProg")
#pragma CODE_SECTION (NDU_gmacControl,".controlProg")
#pragma CODE_SECTION (NDU_SendAnuToNeu, ".pkt2tdmProg")
#pragma CODE_SECTION (NDU_FormatAnuFromNeu, ".pkt2tdmProg") /* TODO! review it!*/
#pragma CODE_SECTION (gmacndu_block_port_map, ".controlProg")
#pragma CODE_SECTION (gmacndu_fset, ".controlProg")
#pragma CODE_SECTION (gmacndu_get_udp, ".controlProg")
#pragma CODE_SECTION (gmacndu_message_port_map, ".controlProg")
#pragma CODE_SECTION (gmacndu_msg_send, ".controlProg")
#pragma CODE_SECTION (gmacndu_open_neu, ".initProg")
#pragma CODE_SECTION (gmacndu_rcv_message,".controlProg")
#pragma CODE_SECTION (gmacndu_rxcall_neu,".pkt2tdmProg")

#pragma CODE_SECTION (gmacndu_rt_open_neu,".initProg")
#pragma CODE_SECTION (NDU_gmacCOnfigAnuReq,".controlProg")

/* Prog Overlay */
#pragma CODE_SECTION (NDU_gmacConfig, ".progOverlay")
#pragma CODE_SECTION (gmacndu_open_neu,".progOverlay");

#pragma CODE_SECTION (NDU_utConfig, ".progOverlay");
#pragma CODE_SECTION (utndu_open_neu,        ".progOverlay");
#pragma CODE_SECTION (utndu_rxcall_neu,      ".progOverlay");
#pragma CODE_SECTION (utndu_create_msg_aal5, ".progOverlay");
#pragma CODE_SECTION (nduUtHealthInit,       ".progOverlay");
#endif
 
#pragma DATA_SECTION (nduNormMsgFifo , ".nduNormMsgFifo");
#pragma DATA_SECTION (nduHighMsgFifo , ".nduHighMsgFifo");
#pragma DATA_SECTION (nduLowMsgFifo  , ".nduLowMsgFifo");
#endif /* _NDUPORT_H */




