/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef  _NDU_H
#define  _NDU_H
#include "types.h"
#include "utlxferpkt.h"
#include "mgmicdsp.h"
#include "vbmicdsp.h"
#include "diagapp.h"
#include <ti/mas/neu/aal5.h>
#include "hal.h"
#include <ti/mas/neu/neu.h>
#include "targethw.h"
#ifdef PA_SS
#include <ti/drv/nwal/nwal.h>
#endif

/* Return values */
#define NDU_NOERR  0
#define NDU_ERROR -1

#define NULL 0
#define NDU_HPI_INT_VOICE        0x1  /* For reporting voice packets */
#define NDU_HPI_INT_SIGNALLING   0x2  /* For reporting signalling messages */
#define NDU_HPI_INT_REGULAR      0x4  /* For reporting regular messages */
#define NDU_HPI_INT_VOICE_ALWAYS 0x8  /* For reporting voice packets, regardless of FIFO state */

#define HDNDU_MSG_ROUTE_TAG_TYPE_SHIFT  11
#define HDNDU_ROUTE_PHY_SHIFT            6
#define HDNDU_MSG_ROUTE_TAG_FLAGS        4
#define HDNDU_ROUTE_TAG_HIGH_PRIORITY   28
#define HDNDU_ROUTE_TAG_NORMAL_PRIORITY 29
#define HDNDU_ROUTE_TAG_LOW_PRIORITY    30
#define HDNDU_ROUTE_TAG_ENABLE_MASK     0x0020
#define HDNDU_ROUTE_TAG_SET_MASK        0x001F
#define HDNDU_ROUTE_TAG_DPLD_MASK       0x07C0
#define HDNDU_ROUTE_TAG_DPLD_SHIFT      6
#define HDNDU_ROUTE_TAG_SIZE_WORDS      2  /* Size of route tag in words */

#define HDNDU_PCMPKT_TRACE_STREAM_ID_PKT_IN  0x0002
#define HDNDU_PCMPKT_TRACE_STREAM_ID_PKT_OUT 0x0003

typedef enum {
   NDU_DPR_SIGNAL_MSG,
   NDU_DPR_NORMAL_MSG,
   NDU_DPR_LOW_MSG,
   NDU_DPR_NONE_MSG
} NDU_DPRMsgType_t;

#define HDNDU_MSG_HIGH_PRIORITY     0     /* high priority message */
#define HDNDU_MSG_NORMAL_PRIORITY   1     /* normal priority message */
#define HDNDU_MSG_LOW_PRIORITY      2     /* low priority message */
#define HDNDU_MSG_DOWNLOAD          3     /* download block       */
#define HDNDU_PCKTS_TO_HOST         8     /* packets to the host */
#define HDNDU_CCS_DATA              9     /* CCS data */
#define HDNDU_ANNOUNCE             10     /* announcements */  
#define HDNDU_PCKTS_RTP            16     /* rtp packets from HW-A */
#define HDNDU_PCKTS_AAL1           17     /* AAL1 packets from HW-A */
#define HDNDU_PCKTS_AAL2           18     /* AAL2 packets from HW-A */
#define HDNDU_PCKTS_CLEARDATA      19     /* clear data packets from HW-A */
#define HDNDU_PCKTS_RTCP           20     /* rtcp packets from HW-A */


#define NDU_ROUTE_TAG_LEN_BYTES     4     /* 4 byte route tags              */
#define NDU_MAX_ANU_REQ_BLOCK       8     /* 8 word max requests, with tags */


#define NDU_DPRWORDSIZE_BYTES      (TYP_DPRWORD_SIZE / 8)
#define NDU_RXBUFSIZE              (VDB_PAYLOAD_MAXSIZE * NDU_DPRWORDSIZE_BYTES)

typedef struct {
  tint (*nduReceiveOut)(void *targetInst, xferPktInfo_t *pktInfo);
  void *targetInst;
} nduRxOut_t;

typedef enum { /* Do not change the value of these enumerators.Will break nduv1 stat update */
  NDU_INTERRUPT_MESSAGE = 0,
  NDU_NORMAL_MESSAGE    = 1,
  NDU_LOW_MESSAGE       = 2
} NDU_msgPriority_e;

typedef enum { 
  NDU_QUERY_FIFOFREE,
  NDU_QUERY_PRIORITY_SUPPORT,
  NDU_QUERY_MESSAGE_PENDING,
  NDU_ANUSVR_SET_UDP_PORT,
  NDU_ANUSVR_REMOVE_UDP_PORT,
  NDU_QUERY_MSG_FIFO
} NDU_ctrlcode_e;

#define GMAC_CTRL_NDU_LIN 		   0	/* HPI using Linear memory partition */
#define GMAC_CTRL_NDU_FIFO		   1	/* HPI using FIFO memory partition */
#define GMAC_CTRL_NDU_GETFREE    2  /* Return free space in ndu units */
#define GMAC_CTRL_NDU_GETMAC     3  /* Return assigned mac addresses */

typedef struct NDU_config_s {
    void     (*pUtMsgInit)(diagApp_t *);
    tint     (*pAnuRec)(void *, xferPktInfo_t *);
    tint     (*pAnuXmt)(void *inst, xferPktInfo_t *pkt);
    tint     (*pAal5SendCell)(tint, tuint, halUtopiaData_t *);
    tint     (*pAal5SendProcess)(aal5Inst_t *, tint *, void **, bool);
    tint     (*pAal5Receive)(aal5Inst_t *, tint *, void **, bool, tuint);
    tint     (*pProcessData)(tuint, xferPktInfo_t *);
    void     (*pProcessMsg)(NDU_msgPriority_e, mgb_t *);
    tint     (*pHalApiCtrl)(halHpiControl_t *);
    tint     (*phalUtopiaControl)  (tint , halUtopiaControl_t *);
    tuint    *start_address;
    void     (*pPcmPktTrace)(tuint ID, tuint dataSize, void *data, tuint streamId);
    void     *(*pNvNeuCreate)(tint nvEcoId, tint txHdrLenWords);
    tint     (*pNvneuOpen)(void *neuInst, neuCfg_t *cfg);
    tint      (*pNvneuControl)(void *neuInst, neuCtrl_t *cfg);
    tint     (*pNvneuReceiveIn)(void *neuInst, xferPktInfo_t *pktInfo);
    tint     (*pNvneuSendIn) (void *neuInst, xferPktInfo_t *pktInfo); 
    void      *(*pNeuGetInstance) (xferPktInfo_t *pktInfo);
    halReturn_t (*halGmacSendPacket) (tint portId, halGmacData_t *data);
    tuint    utopia_portid;
    tuint    coreId;
    tint     halId;
    bool     fUtopiaAnu;

}NDU_config_t;

typedef struct NDU_control_s {
    union {
        tuint fifo_free;          
        bool  priority_supported; 
        bool  message_pending;
        bool  msg_in_fifo;
    } out_params;
    union {
        tuint msg_priority;          
    } input_params;
    
    void (*anuSvrReceiveOut) (xferPktInfo_t *pkt);
    tuint anuSvrUdpPort;
	tuint halId;

} NDU_control_t;

typedef struct NDU_callTable_s                     
{
    tint  (*pSendMsg)(mgb_t *msg, NDU_DPRMsgType_t level, bool fDrop);
    tint  (*pSendData)(tuint, xferPktInfo_t *);
    tint  (*pGetMsg)(NDU_msgPriority_e level);
    void  (*pMsgBg)(void);
    void  (*pGetData)(void *, word *, tuint);   
    void  (*pGetDataSecond)(void *, word *, tuint, xferPktInfo_t *, void *);   
    tint  (*pConfig)(NDU_config_t *);
    void  (*pGetStats)(struct MGB_DMS_GET_NETWORK_STAT *, bool reset);
    void  (*pSetIntMode)(tuint);
    int   (*pControl)(NDU_ctrlcode_e ,NDU_control_t*);
    void  (*pSendHealth)(tint seqNum);
    void  (*pPushRcvData)(tuint portId, word *data, tuint sizeBytes);
} NDU_callTable_t;
#if 0
#ifndef GG_NDU_MSG_NORMAL_FIFO_L
#define GG_NDU_MSG_NORMAL_FIFO_L 2
#define GG_NDU_MSG_HIGH_FIFO_L   2
#define GG_NDU_MSG_LOW_FIFO_L    2
#endif
#endif

typedef struct {
  tuint *start;      /* fifo address */
  tuint inIndex;     /* location to place next message */
  tuint outIndex;    /* location of next message to send to DPR */
  tuint fifolen;     /* length of fifo */
  tuint free;        /* number of messages pending in the FIFO */ 
  tuint dropCounter; /* Number of messages dropped due to full FIFO */
} NDU_fifo_t;

typedef struct {
  void (*nduHealthInit) (void *,NDU_config_t *config);
  void (*nduHealthSend) (tint);
} NduHealthTable_t;

typedef struct NduNvInst_s {
  tint              vcIdRx;      /* Associated voice channel ID from host */
  tint              vcIdTx;      /* Associated voice channel ID to host   */
  void             *neuInst;     /* Standard NEU eco instance             */
  NDU_DPRMsgType_t   rxType;      /* Message priority from net             */
} NduNvInst_t;

/******************************************************************************
 * DATA DEFINITION: Network Driver Unit Structures 
 ******************************************************************************
 * DESCRIPTION: Defines structures used by the Network Driver Unit.
 *
 *****************************************************************************/

typedef struct MGB_DMS_GET_NETWORK_STAT NduStats_t;

//#if (defined(NDU_UTOPIA) || defined(NDU_HD_NDU))
typedef struct
{
	void        (*pProcessMsg)(mgb_t *);
	void        (*pProcessData)(tuint, xferPktInfo_t *);
    void        (*pHalHpiCtrl)(halHpiControl_t *);
	tint        (*pAnuRec)(void *, xferPktInfo_t *);
    NDU_fifo_t  msgFifo[3];
	tint        task_timer[3];
	NduStats_t     stats;
	tuint       dspId;
	tuint       coreId;
    tuint       dspCoreID;
	bool        doRoute;
	tuint       msgRouteTag;
	tuint       rxScratch[256];
} NDU_hdContext_t;
 
typedef struct
{
	void        (*pProcessMsg)(NDU_msgPriority_e,mgb_t *);
	tint        (*pProcessData)(tuint, xferPktInfo_t *);
	tint        (*pAal5SendProcess)(aal5Inst_t * inst, tint pktSize[],
		                void *pktIn[], bool fCrcGen);
	tint        (*pAal5Receive)(aal5Inst_t * inst, tint pktSize[], void* pktIn[],
						bool fCrcCheck, tuint instanceId);
    tint        (*pAal5SendCell)(tint, tuint, halUtopiaData_t *);
	tint        (*pAnuRec)(void *, xferPktInfo_t *);
    tint        (*phalUtopiaControl)(tint , halUtopiaControl_t *);
    int         (*pNvneuControl)(void *neuInst, neuCtrl_t *cfg);
    tint        (*pNvneuReceiveIn)(void *neuInst, xferPktInfo_t *pktInfo);
    tint        (*pNvneuSendIn) (void *neuInst, xferPktInfo_t *pktInfo); 
 
	NduStats_t     nduStats;
    NduHealthTable_t *NduHealthTable;   /* Ndu Health function pointers */

    NDU_fifo_t  msgFifo[3];
 
 /*	tint        task_timer[3];
    aal5Inst_t  aal5Inst; 
    bool        aal5CrcGen;     If true the aal5 crc is computed         
    bool        aal5CrcCheck;   If true the aal5 crc is checked          
    void        *aal5Header_p;   Pointer to aal5 header info              
    aal5Inst_t  aal5AnuInst;   
    void        *aal5AnuHeader_p; */
    
    tuint       cellSize;

    tint        ut_portid;
    tuint       dspId;
    tuint       kpAlive_gfc_vp_vcih;
    tuint       kpAlive_vcil_pt_clp;
    tuint       kpAliveAddr_hi;
    tuint       kpAliveAddr_lo;
    tuint       coreId;
    
  /* Message network encapsulation interface */
    NduNvInst_t nduMsg;    /* Non voice "channel" instance for messages       */
    NduNvInst_t nduBlk;    /* Non voice "channel" instance for block requests */
    NduNvInst_t nduHlt;    /* Non voice "channel" instance for health status  */
    
  /* Messages share an neu instance, but seperate aal5 instances are required
   * to allow messages with different priorities to arrived in interleaved cells */
  void *aal5High;
  void *aal5Normal;
  void *aal5Low;
    
} NDU_utContext_t;

typedef struct {
  void (*pProcessMsg)(NDU_msgPriority_e,mgb_t *);
  tint (*pProcessData)(tuint, xferPktInfo_t *);
  tint (*pAnuRec)(void *, xferPktInfo_t *);
  NDU_fifo_t  msgFifo[3];

  /* Parameters specific to high density HPI */
  tuint  msgRouteTag;  /* Route tag, 1st word for messages to host */
  bool   doRoute;      /* If true, message route tags are used     */
  tuint  cellSize;     /* UtopiaCellSize, bytes (54/58)   */
  tint   gmac_portid;
  tuint  dspId;
  tuint  coreId;

  tint (*pNvneuControl)(void *neuInst, neuCtrl_t *cfg);
  tint (*pNvneuReceiveIn)(void *neuInst, xferPktInfo_t *pktInfo);
  tint (*pNvneuSendIn) (void *neuInst, xferPktInfo_t *pktInfo); 
  void *(*pNeuGetInstance) (xferPktInfo_t *pktInfo);
  halReturn_t (*halGmacSendPacket) (tint portId, halGmacData_t *data);
  /* Message network encapsulation interface */
  NduNvInst_t nduMsg;    /* Non voice "channel" instance for messages       */
  NduNvInst_t nduBlk;    /* Non voice "channel" instance for block requests */
  NduNvInst_t nduHlt;    /* Non voice "channel" instance for health status  */
  NduNvInst_t nduTrace;  /* Non voice "channel" instance for Tracing */
  
#ifdef PA_SS
  nwal_Handle    udpHandle[3];
#endif

  /* Announcement server callout */
  void (*anuSvrReceiveOut) (xferPktInfo_t *pktInfo);

  NduStats_t     nduStats;

} NDU_gmacContext_t;

union NDU_uthdContext_t 
{
	NDU_utContext_t	ut;
	NDU_hdContext_t hd;
};

	extern union NDU_uthdContext_t	NDU_uthdContext;
//#endif

extern NDU_callTable_t *NDU_pCallTable;
extern tuint NDU_rxScratch[];
extern union NDU_uthdContext_t NDU_uthdContext;
extern NDU_utContext_t NDU_utContext;

extern NDU_callTable_t NDU_hdCallTable;
extern NDU_callTable_t NDU_utCallTable;
extern NDU_callTable_t NDU_gmacCallTable;
extern NDU_callTable_t NDU_nduv1CallTable;
extern NDU_callTable_t NDU_callTable;

static inline tint NDU_SendMsg(NDU_callTable_t *NDU_pCallTable,mgb_t *msg, NDU_DPRMsgType_t level, bool fDrop)
{
    return((*NDU_pCallTable->pSendMsg)(msg, level, fDrop));
}

static inline tint NDU_SendData(NDU_callTable_t *NDU_pCallTable,tuint extID, xferPktInfo_t *pPktInfo)
{
    return((*NDU_pCallTable->pSendData)(extID, pPktInfo));
}

static inline void NDU_SendHealth(NDU_callTable_t *NDU_pCallTable,tint seqNum)
{
   ((*NDU_pCallTable->pSendHealth)(seqNum));
}


static inline tint NDU_GetMsg(NDU_callTable_t *NDU_pCallTable,NDU_msgPriority_e level)
{
    return((*NDU_pCallTable->pGetMsg)(level));
} 

static inline void NDU_GetStats(NDU_callTable_t *NDU_pCallTable,struct MGB_DMS_GET_NETWORK_STAT *stat, bool reset)
{
    (*NDU_pCallTable->pGetStats)(stat, reset);
}

static inline void NDU_GetData(NDU_callTable_t *NDU_pCallTable,void *cxt, word *buf, tuint size)
{
    (*NDU_pCallTable->pGetData)(cxt, buf, size);
}

static inline void NDU_PushData(NDU_callTable_t *NDU_pCallTable,tuint portId, word *data, tuint sizeBytes)
{
    (*NDU_pCallTable->pPushRcvData)(portId, data, sizeBytes);
}

static inline void NDU_MsgBg(NDU_callTable_t *NDU_pCallTable)
{
    (*NDU_pCallTable->pMsgBg)();
}

static inline int NDU_Control(NDU_callTable_t *NDU_pCallTable,NDU_ctrlcode_e c_code,NDU_control_t *ctl)
{
    return((*NDU_pCallTable->pControl)(c_code, ctl));
}
void nduUtHealthInit (void *ai,NDU_config_t *config);
void ndu_UtHealthSend (tint seqnum);
void NDU_FormatAnuFromNeu(void *inst, xferPktInfo_t *pkt, tint chnum);
tint NDU_SendAnuToNeu(tuint instID, xferPktInfo_t *pktInfo);

#define NDU_STATE_BEGIN() (thwCriticalBegin())
#define NDU_STATE_END()   (thwCriticalEnd())

#endif /* _NDU_H */

