/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <string.h>

#include "micdspdp.h"
#include "ndu.h"
#include "nduloc.h"
#include "nduport.h"
#include "ndugmac.h"
#include "types.h"
#include <ti/mas/util/pkt.h>
#include "utilfifo.h"
#include "hal.h"
#include "ggcfg.h"  /* Needed since SIU_LAST_CH define references ggConfig. */
#include "ggmbuf.h"
#include <ti/mas/neu/neuproto.h>
#include "rttrace.h"

/* Different include files are used for the diagnostic code. The include
 * file typedefs the name nduInst_t to be either nduInst_t (for diag)
 * or siuInst_t (for application). The siu context is also typedefed */
#include <ti/mas/neu/aal5.h>
#include "diagapp.h"
#include "ggswcfg.h"
#include "siuneu.h"      /* Needed for MSG MIPS count */ 
#include "hal.h"

extern tint  dsiuPktSendOut (void *targetInst, xferPktInfo_t *pkt);

#ifdef NDU_GMAC

NDU_gmacContext_t  NDU_gmacContext;
 
/*****************************************************************************
 * For the diagnostic code, the call table instantiated 
 *****************************************************************************/
#ifdef DSIU
siuNduCallTable_t siuNduCallTable = {
  siuGmNduOpen,
  siuGmNduClose,
  siuGmNduSendIn,
  siuGmNduReceiveIn,
  siuGmIntReceiveIn,
  siuGmNduControl,
  siuGmGetNormalMessage,
  siuGmGetInterruptMessage,
  siuGmWriteMessageToDPR,
  siuGmMSG_test,
  siuGmHeartBeat,
  siuGmGetClockMul,
  siuGmMsgInit,
  siuGmPushFifo,
  siuGmFifo2DPR
};   
#endif /* DSIU */
/******************************************************************************
 * Function:    ones_complement_add
 ******************************************************************************
 * Description: Calculate an Internet style one's complement checksum
 *
 ******************************************************************************/
static inline tuint ones_complement_add (tuint value1, tuint value2)
{
  tulong result;

  result = (tulong)value1 + (tulong)value2;

  result = (result >> 16) + (result & 0xFFFF); /* add in carry   */
  result += (result >> 16);                    /* maybe one more */
  return (tuint)result;
} /* end of ones_complement_add() */

/******************************************************************************
 * Function:    ones_complement_chksum
 ******************************************************************************
 * Description: Calculate an Internet style one's complement checksum
 *
 ******************************************************************************/
static tuint ones_complement_chksum ( tuint *p_data, tuint len )
{
  tuint  chksum = 0;

  while (len > 0)
  {
    chksum = ones_complement_add(chksum, pktRead16bits_m ((word *)p_data,0));
    p_data++;
    len--;
  }
  return chksum;
} /* end of ones_complement_chksum() */

void NDU_gmacSetIntMode(tuint int_mode)
{
}

/******************************************************************************
 * FUNCTION PURPOSE: ndu statistics 
 ******************************************************************************
 * DESCRIPTION: This function retuns the ndu stats also it can reset
 *              the ndu stats.
 *****************************************************************************/

void NDU_gmacGetStats(NduStats_t *stats, bool reset)
{
  *stats = NDU_gmacContext.nduStats;
  /* reset if required */
  if (reset) {
    memset(&NDU_gmacContext.nduStats, 0, sizeof(NduStats_t));
  }
} /* NDU_gmacGetStats */

/******************************************************************************
 * FUNCTION PURPOSE: ndu message polling
 ******************************************************************************
 * DESCRIPTION: .This function reads the message in FIFOs and calls processing
 *               function.
 *****************************************************************************/

tint NDU_gmacGetMsg(NDU_msgPriority_e priority)
{
  NDU_fifo_t  *fifoInst;
  tuint       msgBuf[MGB_HD_COMPOUND_MSG_MAX_LENGTH+MGB_HEADER_SIZE];
  mgb_t       *msg;
  tuint       *msgbuf1;
  
  msg = (mgb_t *)msgBuf;
  msgbuf1 = (tuint *)msgBuf;

  fifoInst = &NDU_gmacContext.msgFifo[priority];
  if ((priority == NDU_NORMAL_MESSAGE) && (fifoInst->free == fifoInst->fifolen)){
    fifoInst = &NDU_gmacContext.msgFifo[NDU_LOW_MESSAGE];
  }
  
  if (fifoInst->free < fifoInst->fifolen) {
    utilFifoRead(fifoInst->start, msgbuf1, &fifoInst->inIndex, 
            &fifoInst->outIndex, fifoInst->fifolen, MGB_HEADER_SIZE);

    fifoInst->free += MGB_HEADER_SIZE;
    
    if (msg->header.length > MGB_HEADER_SIZE) {
      utilFifoRead(fifoInst->start, msgbuf1 + MGB_HEADER_SIZE, &fifoInst->inIndex,
              &fifoInst->outIndex, fifoInst->fifolen, msg->header.length - MGB_HEADER_SIZE);
      fifoInst->free += (msg->header.length - MGB_HEADER_SIZE);
    }
    (*NDU_gmacContext.pProcessMsg)(priority, msg);
  }
  return NDU_NOERR;
} /* NDU_gmacGetMsg */

/*********************************************************************************
 * FUNCTION PURPOSE: Locate the first non-zero bit
 *********************************************************************************
 * DESCRIPTION: Returns the first non-zero bit in an integer.
 *********************************************************************************/
tint gmacndu_fset (tuint val)
{
  tint shift;

  for (shift = 0; !(val & 0x1); shift++, val >>= 1);

  return (shift);

} /* gmacndu_fset */

/*********************************************************************************
 * FUNCTION PURPOSE: Return the source UDP port from a diagAppHeader with core ID
 *********************************************************************************
 * DESCRIPTION: Extracts the core ID from the header, returns the source UDP port
 *              with the core ID added. Returns 0xffff if there is no udp offset.
 *********************************************************************************/
tuint gmacndu_get_udp (diagAppHeader_t *hdr)
{
  tint  shift;
  tuint udp;

  /* Verify there is a valid UDP offset in the header */
  if (hdr->udpOffsetBytes < 0)
    return (~0);

  udp = pktRead16bits((word *)&hdr->hdrImage[UTL_BYTES_TO_TINT(hdr->udpOffsetBytes + neuproto_UDP_BYTE_OFFSET_SRC_PORT)],0);

  /* Get the 1st non-zero bit of the udp source mask */
  shift = gmacndu_fset (ggConfig.neuUdpSrcMask);

  /* Add the core Id into the port */
  udp = udp & ~ggConfig.neuUdpSrcMask;
  udp = udp | NDU_gmacContext.coreId << shift;

  return (udp);
} /* gmacndu_get_udp */

/********************************************************************************
 * FUNCTION PURPOSE: Add the message UDP ports to the NEU port map.
 ********************************************************************************
 * DESCRIPTION: Creates the UDP port values for messages. The core number and
 *              priority are factored into the UDP port.
 ********************************************************************************/
void gmacndu_message_port_map (diagAppHeader_t *msg, void *sInst)
{
  #ifndef PA_SS   /* TBD */

  neuCtrl_t  neuCtrl;
  tuint      udp;
  tint       i;

  /* Extract the current UDP source port from the message header */
  if (msg->udpOffsetBytes < 0)
    return;

  /* Get the UDP source port modified to include core ID */
  udp = gmacndu_get_udp (msg);

  /* Map all three udp ports to the message instance */
  neuCtrl.code = neu_CTRL_ADD_PORTMAP;

  /* Clear the 2 lsbs of the udp port. For the messages these last two
   * bits specifiy priority: 00 = high, 01 = normal, 10 = low, 11 = unused */
  udp = udp & (~0x0003);  

  for (i = 0; i <= 2; i++)  {
    neuCtrl.u.portMap.port = udp + i;
    neuCtrl.u.portMap.inst = sInst;
    (*NDU_gmacContext.pNvneuControl) (NDU_gmacContext.nduMsg.neuInst, &neuCtrl);
  }
  #endif
} /* siu_gmdnu_message_port_map */

/********************************************************************************
 * FUNCTION PURPOSE: Add the ANNOUNCE UDP ports to the NEU port map.
 ********************************************************************************
 * DESCRIPTION: Creates the UDP port values for ANNOUCEMENT. The core number and
 *              priority are factored into the UDP port.
 ********************************************************************************/
void gmacndu_block_port_map (diagAppHeader_t *msg, void *sInst)
{
  neuCtrl_t  neuCtrl;
  tuint      udp;

  /* Extract the current UDP source port from the message header */
  if (msg->udpOffsetBytes < 0)
    return;

  /* Get the UDP source port modified to include core ID */
  udp = gmacndu_get_udp (msg);

  /* Map all 2 udp ports to the message instance */
  neuCtrl.code = neu_CTRL_ADD_PORTMAP;

  neuCtrl.u.portMap.port = udp;
  neuCtrl.u.portMap.inst = sInst;
  (*NDU_gmacContext.pNvneuControl) (NDU_gmacContext.nduBlk.neuInst, &neuCtrl);

} /* siu_gmdnu_message_port_map */

/******************************************************************************
 * FUNCTION PURPOSE: Initializes the GMAC interface.
 ******************************************************************************
 * DESCRIPTION: This function initializes the application to handle the GMAC 
 *              cells. The input parameter is for compatibility with the
 *              legacy HPI.
 *****************************************************************************/
tint NDU_gmacConfig(NDU_config_t *config)
{
  diagApp_t         *appInit;
  NDU_gmacContext_t *cxt;
  tuint            lenWords;
  tint             i;
  tuint            dApp[sizeof(diagApp_t)+2];

  if ((tulong)&dApp[0] & 0x1)
    appInit = (diagApp_t *) &dApp[1];
  else
    appInit = (diagApp_t *) &dApp[0];

  /* For the application code the message fifos must be initialized. The
     diag/bootloader uses a different initialization process at system start */
#ifndef DSIU
   
  cxt = &(NDU_gmacContext);

  /*  Thse buffers are used for dsp to micro communication */
  NDU_gmacContext.msgFifo[0].start   = (tuint *)gmacNduHeapDesc.bufDesc[0].buffAddr;
  NDU_gmacContext.msgFifo[1].start   = (tuint *)gmacNduHeapDesc.bufDesc[1].buffAddr;
  NDU_gmacContext.msgFifo[2].start   = (tuint *)gmacNduHeapDesc.bufDesc[2].buffAddr;
  NDU_gmacContext.msgFifo[0].fifolen =
  NDU_gmacContext.msgFifo[0].free    = GG_NDU_MSG_HIGH_FIFO_L;
  NDU_gmacContext.msgFifo[1].fifolen = 
  NDU_gmacContext.msgFifo[1].free    = GG_NDU_MSG_NORMAL_FIFO_L;
  NDU_gmacContext.msgFifo[2].fifolen =
  NDU_gmacContext.msgFifo[2].free    = GG_NDU_MSG_LOW_FIFO_L;
 
  /*  There are 3 fifos for dsp to micro communication */
  for (i = 0; i < 3; i++) {
    NDU_gmacContext.msgFifo[i].inIndex  = 
    NDU_gmacContext.msgFifo[i].outIndex = 0;
  }

  /* Initialize the message aal5 interface. For the application the structure
   * defining cell size and route/tag atm headers is found in a pre-defined
   * memory location. For now use local data memory. 
   * Copy the stored memory structure to the local memory. This is required
   * since the global copy may be in program memory */
  (*config->pUtMsgInit)(appInit);
  
  /* coreId is initialized in halnew function */
  NDU_gmacContext.cellSize          = 0;
  NDU_gmacContext.dspId             = appInit->dspId;
  NDU_gmacContext.coreId            = config->coreId;
  NDU_gmacContext.pNvneuControl     = config->pNvneuControl;
  NDU_gmacContext.pNvneuReceiveIn   = config->pNvneuReceiveIn;
  NDU_gmacContext.pNvneuSendIn      = config->pNvneuSendIn;
  NDU_gmacContext.pNeuGetInstance   = config->pNeuGetInstance;

  /* -- for messages -- */
  lenWords = UTL_BYTES_TO_WORDS(appInit->u.mac.msg.totalHeaderLenBytes);
   
  /* create a non voice neu instance */
  NDU_gmacContext.nduMsg.neuInst = (*config->pNvNeuCreate)(GG_NVECO_MSG, lenWords);

  /* Open the non-voice NEU channels */
/*  gmacndu_open_neu (NDU_gmacContext.nduMsg.neuInst, appInit, &appInit->mac.msg,
                    nduGmacSendInCore, gmacndu_msg_send, config);*/
  gmacndu_open_neu (NDU_gmacContext.nduMsg.neuInst, appInit, &appInit->u.mac.msg,
                    &(NDU_gmacContext.nduMsg), gmacndu_msg_send, config);

  /* Enable the NEU channel receive callouts */
  gmacndu_rxcall_neu (NDU_gmacContext.nduMsg.neuInst, gmacndu_rcv_message, 
                      &(NDU_gmacContext.nduMsg));

  /* Configure the NEU port map to receive the UDP port for all three message types */
  gmacndu_message_port_map (&appInit->u.mac.msg, (void *)&(cxt->nduMsg));

#if 0
  /* Open NEU for announcements if present */
  if (config->pAnuRec) { 
      lenWords = UTL_BYTES_TO_WORDS(appInit->mac.block.totalHeaderLenBytes);
      /* create a non voice neu instance */
      NDU_gmacContext.nduBlk.neuInst = (*config->pNvNeuCreate)(GG_NVECO_BLK, lenWords);
      /* Open the non-voice NEU channels */
      gmacndu_open_neu (NDU_gmacContext.nduBlk.neuInst, appInit, &appInit->mac.block,
                          &(NDU_gmacContext.nduBlk), config->pAnuXmt, config);
      /* Enable the NEU channel receive callouts */
      gmacndu_rxcall_neu (NDU_gmacContext.nduBlk.neuInst, config->pAnuRec, 
                             &(NDU_gmacContext.nduBlk));
      /* Configure the NEU port map to receive the UDP port for Annoucement */
      gmacndu_block_port_map (&appInit->mac.block, (void *)&(cxt->nduBlk));
  }

#endif

  NDU_gmacContext.halGmacSendPacket = config->halGmacSendPacket;
  NDU_gmacContext.pProcessData      = config->pProcessData;
  NDU_gmacContext.pProcessMsg       = config->pProcessMsg;
  NDU_gmacContext.gmac_portid       = config->utopia_portid; 
  NDU_gmacContext.pAnuRec           = config->pAnuRec;
  
  /* Initialize NDU statistics */
  memset ((void *)&NDU_gmacContext.nduStats, 0, sizeof(NduStats_t));
  
#endif  /* DSIU */
  return (0);
} /* NDU_gmacConfig */

/****************************************************************************************
 * FUNCTION PURPOSE: Continue packet handling
 ****************************************************************************************
 * DESCRIPTION: Receive packet processing continues after the first pass through NEU.
 *              This is broken into two stages only if MAC router is compiled in.
 ****************************************************************************************/
#if !defined(GG_INCLUDE_MACR) && !defined(PA_SS) 
static inline
#endif
void NDU_gmacReceiveSecond (void *cxt, word *cell, tuint sizeBytes, xferPktInfo_t *pktOut, void *vInst)
{
  NduNvInst_t          *nInst;
  tint  channel, ret;
  tuint port;
  tuint                 origSizeBytes = sizeBytes;
  siuInst_t             *sInst;

  /* The original packet size must be restored */
  pktOut->pktSize = (tint *)&origSizeBytes;  

  /* If the instance returned was NULL then no match was found for the udp port */
  if (vInst == NULL) {
    NDU_gmacContext.nduStats.RxErrState += 1;
    return;
  }

  /* On return pktOut info field type will contain the UDP port. Use
  * this to update the instance with the priority of the received message */
  nInst = NULL;
  if (vInst == (void *)&(NDU_gmacContext.nduMsg))  {
    nInst = (NduNvInst_t *)vInst;
    port = pktOut->type & 0x3;
    
    switch (port)  {
      case 0: nInst->rxType = NDU_DPR_SIGNAL_MSG; 
        break;
      case 1: nInst->rxType = NDU_DPR_NORMAL_MSG; 
        break;
      case 2: nInst->rxType = NDU_DPR_LOW_MSG;    
        break;
      default:
        nInst->rxType = NDU_DPR_NORMAL_MSG; 
    }
    /* Reset the packet out type. The type field is not really too important,
    * it will return to the message processing function with type RTP */
    pktOut->type = XFER_PKT_TYPE_MSG_NORMAL;

  } else if (vInst == (void *)&(NDU_gmacContext.nduBlk))  {
    nInst = (NduNvInst_t  *)vInst;
    pktOut->type = XFER_PKT_TYPE_ANNOUNCE;

  } else if ( (vInst != NULL) && (vInst == (void *)NDU_gmacContext.anuSvrReceiveOut) )  {
    pktOut->type = XFER_PKT_TYPE_ANNOUNCE;
    pktOut->pktSize = (tint *)&origSizeBytes;  /* The original packet size must be restored */
    (*NDU_gmacContext.anuSvrReceiveOut) (pktOut);
    return;
  }



  /* If the packet was for messages or block announcements, pass it back to
  * neu */
  if (nInst)  {
    (*NDU_gmacContext.pNvneuReceiveIn) (nInst->neuInst, pktOut);
    return;
  }
    /* The packet was a voice packet. */
  sInst = (siuInst_t *)vInst;

  channel = sInst->ID & SIU_CID_MASK;  /* The channel is 1 based */
#ifdef NDU_DBG4
#ifndef NDU_LOOPBACK_SELF
    /* Debug mode loop back */
    if (channel & 0x1)
      channel += 1;
    else
      channel -= 1;
#endif
#endif
  /* Packet Trace */
  utlRtTrace (sInst->ID, origSizeBytes, pktOut->pktIn[0],
              MGB_RTTRACE_STREAM_ID_PKT_IN, UTL_RTTRACE_PACKED);

  ret =  (*NDU_gmacContext.pProcessData)(channel, pktOut);
  if (ret == NDU_ERROR) {
    NDU_gmacContext.nduStats.RxErrState += 1;
  } 
      
} /* NDU_gmacReceiveSecond */

/**************************************************************************************
 * FUNCTION PURPOSE: Interrupt handler for packets
 **************************************************************************************
 * DESCRIPTION: This function is called from the hal driver when packets are 
 *              available. The type of packet (message, voice data) is unkown.
 *
 *              The packet is sent to NEU with a NULL instance. NEU will 
 *              attempt to extract destination udp port number from the packet, and
 *              return with the associated instance. 
 *
 *              The instance returned can be either one of the non-voice instances
 *              or else a standard siu instance.
 *
 *              The received context is just the physical port number cast
 *              as a void pointer.
 ***************************************************************************************/
void NDU_gmacReceive (void *cxt, word *cell, tuint sizeBytes)
{
  void                 *vInst;
  xferPktInfo_t        pktOut;
  neuValidationInfo_t   vInfo;

  /* Format the cell for shipout */
  pktOut.pktSize = (tint *)&sizeBytes;
  pktOut.pktIn   = (void *)&cell;
  pktOut.type    = XFER_PKT_TYPE_AAL1;  /* Always use aal1 */
  pktOut.srcPort = (tuint)cxt;
  pktOut.npkts   = 1;

  memset(&vInfo,0,sizeof(neuValidationInfo_t));
  pktOut.suppInfo =  &vInfo;
  /* Send the packet to NEU. The NULL instance is used to tell NEU that
  * the channel/type information must be extracted from the NEU port map table. */
  vInst = (*NDU_gmacContext.pNeuGetInstance) (&pktOut);

  /* The receive processing is split to a second function so that the packet
   * can be intercepted by a different function for pre-processing. Specifically
   * mac router can seize the HAL callout and do any pre-processing. If the
   * packet is not a target for the router it calls this same function */
  NDU_gmacReceiveSecond (cxt, cell, sizeBytes, &pktOut, vInst);
  return;

} /* NDU_gmacReceive */

/*****************************************************************************
 * FUNCTION PURPOSE: Open an NEU instance 
 *****************************************************************************
 * DESCRIPTION: Opens the NEU instance with the information provided in the
 *              boot/application handoff.
 *****************************************************************************/
void gmacndu_open_neu (void *neuInst, diagApp_t *diagApp, diagAppHeader_t *hdr, 
                     void *tInst, tint (*neuOut)(void *, xferPktInfo_t *),NDU_config_t *config)
{
  neuCfg_t   cfg;
  neuCtrl_t  ctrl;
  tint       shift;
  tuint      val;
  tuint chksum, coreId;
  tuint tmp;

  /* Copy the cell size from the NDU context to the NEU context. */
  neuContext.cellSize = NDU_gmacContext.cellSize;

  /* Provide function pointer for outgoing data */
  cfg.neuSendOut.neuSendOut = neuOut;
  cfg.neuSendOut.targetInst = tInst;

  cfg.parm.chnl.enable = TRUE;  /* Enable NEU */

  /* Flags. Bit exactness of the desired flags was checked by the compiler
   * at the top of the function. */
  #warn disable NEU checksum for now: need to debug the potential host or linux issue
  cfg.parm.chnl.cfg_flag1 = hdr->flags1 & ~(0x0002);
  cfg.parm.chnl.cfg_flag2 = hdr->flags2;
  cfg.parm.chnl.sar_type  = neuproto_SAR_NONE;

  /* Alternate ports unused */
  cfg.parm.chnl.alt_local_port  = 0;
  cfg.parm.chnl.alt_remote_port = 0;
  memset(cfg.parm.chnl.alt_remote_addr, 0, MGB_IP_ADDR_MAX_NUM_WORDS * sizeof(UINT16));

  /* report control must be disabled */
  cfg.parm.chnl.report_ctrl          = NET_ENCAP_ENCAP_REPORT_DISCARD;
  cfg.parm.chnl.report_throttle_time = 0;
  cfg.parm.chnl.sysInst              = NULL;

  /* No second protocol allowed */
  cfg.parm.chnl.second_prototype = NET_ENCAP_NULL_PROTOCOL;

  /* This next parameter might need to be updated with the boot config cell changes */
  cfg.parm.chnl.rx_offset_bytes = 0;
  cfg.parm.chnl.rx_payload_type = hdr->rxPayloadType;

  /* Tx parameters */
  if (hdr->ipOffsetBytes > 0)
    cfg.parm.chnl.tx_ip_offset_bytes    = (hdr->ipOffsetBytes);
  else
    cfg.parm.chnl.tx_ip_offset_bytes    = -1;

  if (hdr->udpOffsetBytes > 0)
    cfg.parm.chnl.tx_udp_offset_bytes   = (hdr->udpOffsetBytes);
  else
    cfg.parm.chnl.tx_udp_offset_bytes   = -1;

  if (hdr->ethOffsetBytes > 0)
    cfg.parm.chnl.tx_802_3_offset_bytes = (hdr->ethOffsetBytes);
  else
    cfg.parm.chnl.tx_802_3_offset_bytes = -1;

  if (cfg.parm.chnl.rx_payload_type == DIAGAPP_PROTO_ETHERNET)
   cfg.parm.chnl.tx_min_cell_bytes   = DIAG_APP_MIN_ETHERNET_PACKET_SIZE_BYTES;
  else
    cfg.parm.chnl.tx_min_cell_bytes   = 0;

  /* For the utopia NDU with MII the core ID is placed into the UDP source address
   * as defined in the ggswcfg.h */
  if ( (cfg.parm.chnl.tx_udp_offset_bytes >= 0) && (ggConfig.neuUdpSrcMask) ) {
    val = ggConfig.neuUdpSrcMask;
    for (shift = 0; !(val & 0x1); shift++, val >>= 1);
    cfg.parm.chnl.tx_udp_src_mask = ggConfig.neuUdpSrcMask;
    cfg.parm.chnl.tx_udp_src_val  = NDU_gmacContext.coreId << shift;

  } else {
    cfg.parm.chnl.tx_udp_src_mask = 0;
    cfg.parm.chnl.tx_udp_src_val  = 0;
  }

  cfg.parm.chnl.halId = config->halId;

  /* Transmit header */
  cfg.parm.chnl.tx_total_hdr_len_bytes = hdr->totalHeaderLenBytes;
  memcpy (cfg.parm.chnl.header, hdr->hdrImage, UTL_BYTES_TO_WORDS(hdr->totalHeaderLenBytes));
  coreId = NDU_gmacContext.coreId;
  /* Change the src mac address  & src ip address according to the core id in the header*/
//  cfg.parm.chnl.header[3] = diagApp->macAddr[coreId].macAddr[0];
//  cfg.parm.chnl.header[4] = diagApp->macAddr[coreId].macAddr[1];
//  cfg.parm.chnl.header[5] = diagApp->macAddr[coreId].macAddr[2];
//  cfg.parm.chnl.header[13] = diagApp->ipAddr[coreId].srcIpHigh;
//  cfg.parm.chnl.header[14] = diagApp->ipAddr[coreId].srcIpLow;
  pktWrite16bits(&cfg.parm.chnl.header[3*(16/TYP_WORD_SIZE)],0,diagApp->u.mac.macAddr[coreId].macAddr[0]);
  pktWrite16bits(&cfg.parm.chnl.header[4*(16/TYP_WORD_SIZE)],0,diagApp->u.mac.macAddr[coreId].macAddr[1]);
  pktWrite16bits(&cfg.parm.chnl.header[5*(16/TYP_WORD_SIZE)],0,diagApp->u.mac.macAddr[coreId].macAddr[2]);
  pktWrite16bits(&cfg.parm.chnl.header[13*(16/TYP_WORD_SIZE)],0,diagApp->u.mac.ipAddr[coreId].srcIpHigh);
  pktWrite16bits(&cfg.parm.chnl.header[14*(16/TYP_WORD_SIZE)],0,diagApp->u.mac.ipAddr[coreId].srcIpLow);
  /* Recalculate IP checksum */
//  cfg.parm.chnl.header[12] = 0;
  pktWrite16bits(&cfg.parm.chnl.header[12*(16/TYP_WORD_SIZE)],0,0);

  chksum =  ones_complement_chksum((tuint *)&cfg.parm.chnl.header[hdr->ipOffsetBytes >>1],hdr->totalHeaderLenBytes/2);
    if (chksum != 0xFFFF)
      chksum = (tuint)(~chksum);

//  cfg.parm.chnl.header[12] = chksum;
  pktWrite16bits(&cfg.parm.chnl.header[12*(16/TYP_WORD_SIZE)],0,chksum);

  /* Update UDP pseudo checksum */
  //cfg.parm.chnl.header[20] = 0;
  pktWrite16bits(&cfg.parm.chnl.header[20*(16/TYP_WORD_SIZE)],0,0);

  /* This is the src & dst address */
    chksum =  ones_complement_chksum((tuint *)&cfg.parm.chnl.header[13*2], 4);
  /* This is the  protocol  type */
   	tmp = pktRead16bits(&cfg.parm.chnl.header[11*(16/TYP_WORD_SIZE)],0);
    chksum = ones_complement_add(chksum, tmp  & 0xff);
   // cfg.parm.chnl.header[20] = chksum;
    pktWrite16bits(&cfg.parm.chnl.header[20*(16/TYP_WORD_SIZE)],0,chksum);

   /* only in config as required once */
  (*config->pNvneuOpen) (neuInst, &cfg);

  /* The parameters must be re-passed to enable sending of cells */
  ctrl.code   = neu_CTRL_CHANNEL_CFG;
  ctrl.u.parm = cfg.parm;
  (*NDU_gmacContext.pNvneuControl) (neuInst, &ctrl);

  /* Modify source port of UDP if required */
  if(cfg.parm.chnl.tx_udp_src_mask) {
    pktAND16bits(&cfg.parm.chnl.header[0], 
                 cfg.parm.chnl.tx_udp_offset_bytes + neuproto_UDP_BYTE_OFFSET_SRC_PORT,
                 ~(cfg.parm.chnl.tx_udp_src_mask));
    pktOR16bits(&cfg.parm.chnl.header[0],
                cfg.parm.chnl.tx_udp_offset_bytes + neuproto_UDP_BYTE_OFFSET_SRC_PORT,
                cfg.parm.chnl.tx_udp_src_val);
  }

#ifdef PA_SS
      if(!(dsiuConfigL2L3L4(&(NDU_gmacContext.nduMsg),
                            &cfg.parm.chnl,
                            &NDU_gmacContext.udpHandle[0],
                            &NDU_gmacContext.udpHandle[1],
                            &NDU_gmacContext.udpHandle[2])))
      {
          #if 0
          return (DSIU_FAIL);
          #endif
      }
#endif
} /* gmacndu_open_neu */

/*****************************************************************************
 * FUNCTION PURPOSE: Pass formatted cells to the HAL Utopia driver
 *****************************************************************************
 * DESCRIPTION: Receives segmented and formatted cells. Each of the cells
 *              will have its top level header modified 
 *****************************************************************************/
tint gmacndu_msg_send (void *inst, xferPktInfo_t *pkt)
{
  halGmacData_t     data;

  switch (pkt->type)  {

    case XFER_PKT_TYPE_MSG_HIGH: NDU_gmacContext.nduStats.TxSigMsgCnt++;
      break;

    case XFER_PKT_TYPE_MSG_NORMAL: NDU_gmacContext.nduStats.TxCtrlMsgCnt++;
      break;

    case XFER_PKT_TYPE_MSG_LOW: NDU_gmacContext.nduStats.TxLowMsgCnt++;
      break;

    default: /* Invalid type */
      NDU_gmacContext.nduStats.TxInvldNvCh += 1;
      return (SIU_ERROR);

  }

  data.nSegs = pkt->npkts;
  data.type  = 0;
  data.sizes = (tuint *)pkt->pktSize;
  data.segs  = (tuint **)pkt->pktIn;

  #ifndef PA_SS
  if ((*NDU_gmacContext.halGmacSendPacket) (pkt->srcPort, &data) == HAL_PKT_TX_PKT_OVERFLOW)
  #else
  if(dsiuPktSendOut (NULL, pkt))
  #endif
  {
    NDU_gmacContext.nduStats.TxErrDrop += 1;
  }

  return (SIU_SUCCESS);

} /* gmacndu_msg_send */

/*****************************************************************************
 * FUNCTION PURPOSE: Setup the NEU rx callout
 *****************************************************************************
 * DESCRIPTION: Sets up callout for the non voice channel NEU for completed
 *              data.
 *****************************************************************************/
void gmacndu_rxcall_neu (void *inst, tint (*fcn)(void *, xferPktInfo_t *), void *rcvInst)
{
  neuCtrl_t cfg;

  cfg.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
  cfg.u.neuReceiveOutSpec.neuReceiveOut = fcn;
  cfg.u.neuReceiveOutSpec.targetInst    = rcvInst;

  (*NDU_gmacContext.pNvneuControl) (inst, &cfg);

} /* gmacndu_rxcall_neu */

/**********************************************************************************
 * FUNCTION PURPOSE: Format a message to be sent to the host
 **********************************************************************************
 * DESCRIPTION: A message is passed through the neu layer. The neu instance
 *              will then pass the packet with header to function
 *              siu_gmmdu_msg_send which passes it to hal.
 **********************************************************************************/
tint NDU_gmacSendMsg(mgb_t *msg, NDU_DPRMsgType_t mtype, bool fDrop)
{
 /* In Loopback do not send messages, or they will be looped back   */
#ifdef NDU_DBG4
  return (0);
#else
  xferPktInfo_t     pkt;
  void             *data;
  NduNvInst_t      *nvInst;
  tint              pktSizeBytes;
  tint              ret;

  /* Store the message ID for the return value */
  ret = msg->header.msg_id;

  /* Shorthand to the ndu Context */
  nvInst = &(NDU_gmacContext.nduMsg);

  /* If the message instance is NULL then messaging is not supported in this
   * interface. */
  if (!nvInst)
    return (0);  /* A return of 0 indicates failure to send the message */

  /* Modify Channel ID if Broadcast Message */
  if (msg->header.chnum == MGB_ALL_CHANNEL_BROADCAST_ID)
    msg->header.chnum = NDU_GMAC_MAX_CHANNEL_ID;

  /* Use the message "channel" corresponding to the priority */
  switch (mtype)  {

    case NDU_DPR_SIGNAL_MSG:
      pkt.type  =  XFER_PKT_TYPE_MSG_HIGH;
      break;

    case NDU_DPR_NORMAL_MSG:
      pkt.type =  XFER_PKT_TYPE_MSG_NORMAL;
      break;

    case NDU_DPR_LOW_MSG:
      pkt.type =  XFER_PKT_TYPE_MSG_LOW;
      break;

    /* No default required */
  } /* end switch */

  /* Compute the packet size. This must be done before the header translation, since
   * the size field is lost.
   *
   *        NOTE:
   *    The format of the message header is from the vcdemux cell format, this does
   *    not imply that the message will actually BE in vcdemux format. Just the
   *    message header is formatted to the vc demux msg header type which is
   *    also used by mac.
   */
  /* To be reviewed - Machu */
  pktSizeBytes = UTL_TINTS_TO_BYTE(msg->header.length - MGB_HEADER_SIZE +
                                    SIU_VC_DEMUX_MSG_HEADER_LENGTH);

  data = ndu_gm_modify_vc_msg_header (mtype, msg);

  /* Pass the formatted message to NEU. Once the message headers are
   * applied it will be sent to siu_gmndu_msg_send */
  pkt.type    = XFER_PKT_TYPE_MSG_NORMAL;
  pkt.npkts   = 1;
  pkt.pktSize = &pktSizeBytes;
  pkt.pktIn   = &data;

  (*gg_neu_ctable.neuSendIn) (nvInst->neuInst, &pkt);

  return (ret);
#endif
} /* siu_gm_send_message */

/*****************************************************************************
 * FUNCTION PURPOSE: Format a VC Demux header
 *****************************************************************************
 * DESCRIPTION: Re-formats a message header into vc demux format. The length
 *              field is lost.
 *****************************************************************************/
void *ndu_gm_modify_vc_msg_header (NDU_DPRMsgType_t mtype, mgb_t *msg)
{
  nduVcDemuxMsgPayload_t *vcd;
  tuint ttype, ch;

  /* Save the channel number before modifiing the memory in place */
  ch = msg->header.chnum;
  vcd = (nduVcDemuxMsgPayload_t *)&msg->header.length;

  /* The length field of the message type is replaced by the DSP ID */ 
  vcd->dsp_id = (NDU_gmacContext.dspId+NDU_gmacContext.coreId);

  /* Translate the proirity into route tag type field */
  switch (mtype)  {

    case NDU_DPR_SIGNAL_MSG:
      ttype = NDU_GMAC_VC_DEMUX_MSG_PRIORITY_HIGH;
      break;
    case NDU_DPR_NORMAL_MSG:
      ttype = NDU_GMAC_VC_DEMUX_MSG_PRIORITY_NORMAL;
      break;
    case NDU_DPR_LOW_MSG:
      ttype = NDU_GMAC_VC_DEMUX_MSG_PRIORITY_LOW;
      break;
  } /* end switch */
  
  /* The channel field of the message type is appended with the core ID
   * and the message priority */
  NDU_GMAC_VC_DEMUX_MSG_SET_PRIORITY(vcd, ttype);
  NDU_GMAC_VC_DEMUX_MSG_SET_COREID(vcd, NDU_gmacContext.coreId);
  NDU_GMAC_VC_DEMUX_MSG_SET_CHANNEL (vcd, ch);

  /* Return a pointer the vc demux message payload */
  return ((void *) vcd);

} /* ndu_gm_modify_vc_msg_header */

/************************************************************************************
 * FUNCTION PURPOSE: Send formatted packets to the network
 ************************************************************************************
 * DESCRIPTION: Fully formatted packets are sent to the hal layer
 ************************************************************************************/
tint NDU_gmacSendData (tuint extID, xferPktInfo_t *pktInfo)
{
  halGmacData_t halData;
  tint ret=0;

 /* Translate the data into hal format */
  halData.nSegs = (tuint)   pktInfo->npkts;
  halData.segs  = (tuint **)pktInfo->pktIn;
  halData.sizes = (tuint *) pktInfo->pktSize;
  halData.type  = 0;  /* types don't match */


  SIU_STATE_BEGIN();
  
#if ((defined(NDU_DBG4) && !defined(NDU_DBG6)) || defined(NDU_DBG7))
  /* Send the packet back. Channel crossing is handled in receive in */
#ifdef NDU_DBG7 /* Special case for Announcements*/
  if(pktInfo->type ==XFER_PKT_TYPE_ANNOUNCE)
  {
      /* Send the Cell. Existance of the function call was verified in nduControl */
      #ifndef PA_SS
      if (((*NDU_gmacContext.halGmacSendPacket)(pktInfo->srcPort, &halData)) == HAL_PKT_TX_PKT_OVERFLOW)
      #else
      if(dsiuPktSendOut (NULL, pktInfo))
      #endif
      {
          NDU_gmacContext.nduStats.TxErrDrop += 1;
          ret = -1; /* Cell is not sent */
      }
  }
  else
#endif /* Special case for Announcements*/
    NDU_gmacReceive (0, (word *)*halData.segs, *halData.sizes); 
#else
  /*  NDU_gmacReceive (0, (word *)*halData.segs, *halData.sizes); */
  /* Send the Cell. Existance of the function call was verified in nduControl */
  #ifndef PA_SS /* Temporary fix for PASS/NWAL */
  if (((*NDU_gmacContext.halGmacSendPacket)(pktInfo->srcPort, &halData)) == HAL_PKT_TX_PKT_OVERFLOW)
  #else
  if(dsiuPktSendOut (NULL, pktInfo))
  #endif
  {
    NDU_gmacContext.nduStats.TxErrDrop += 1;
    ret = -1; /* Cell is not sent */
  } 
#endif

  SIU_STATE_END();
  return (ret);
} /* NDU_gmacSendData */

void NDU_gmacMsgBg(void)
{
#if (defined(NDU_DBG4) || defined(NDU_DBG5))
  if (msgTest_flag) {
   (*NDU_gmacContext.pProcessMsg)(NDU_NORMAL_MESSAGE,&msgTest);
   msgTest_flag = FALSE;
  }
  if (sigTest_flag) {
    (*NDU_gmacContext.pProcessMsg)(NDU_INTERRUPT_MESSAGE,&sigTest);
    sigTest_flag = FALSE;
  }
#endif
}

void NDU_gmacMainTask(tuint timeStamp)
{
#ifdef NDU_DBG4
//  NDU_loopReceiveIn();
#endif /* NDU_DBG4 */

  NDU_gmacGetMsg(NDU_INTERRUPT_MESSAGE);
  NDU_gmacGetMsg(NDU_NORMAL_MESSAGE);
  NDU_gmacMsgBg();
}
/*******************************************************************************
 * FUNCTION PURPOSE: ndu query
 *******************************************************************************
 * DESCRIPTION: Checks NDU if priority is supported or Fifo is free
 ******************************************************************************/
//tint halGmacControl(tint port, halGmacControl_t *ctl);
int NDU_gmacControl(tuint c_code,void *ctl)
{
  NDU_control_t *ndu_ctl = (NDU_control_t *)ctl;
  halGmacControl_t gmac_ctl;
  neuCtrl_t  neuCtrl;
  uint i;

  switch(c_code)
  {
    case NDU_QUERY_PRIORITY_SUPPORT:
          ndu_ctl->out_params.priority_supported = TRUE;
          break; 

    case NDU_QUERY_FIFOFREE:  
      gmac_ctl.code = HAL_GMAC_CTL_CODE_TX_CPPI_AVAIL_BUF;
	  gmac_ctl.u.portId.halId = ndu_ctl->halId;
      hal.gmac->halGmacControl(&gmac_ctl);
      ndu_ctl->out_params.fifo_free = gmac_ctl.u.txCppiAvail.tx_cppi_bytes_available;
      break;

    case NDU_ANUSVR_SET_UDP_PORT:
      neuCtrl.code           = neu_CTRL_ADD_PORTMAP;
      neuCtrl.u.portMap.port = ndu_ctl->anuSvrUdpPort;
      neuCtrl.u.portMap.inst = ndu_ctl->anuSvrReceiveOut;
      (*NDU_gmacContext.pNvneuControl) (NULL, &neuCtrl);
      NDU_gmacContext.anuSvrReceiveOut = (void (*)(xferPktInfo_t *))ndu_ctl->anuSvrReceiveOut;
      break;

    case NDU_ANUSVR_REMOVE_UDP_PORT:
      neuCtrl.code           = neu_CTRL_RMV_PORTMAP;
      neuCtrl.u.portMap.port = ndu_ctl->anuSvrUdpPort;
      (*NDU_gmacContext.pNvneuControl) (NDU_gmacContext.nduMsg.neuInst, &neuCtrl);
      NDU_gmacContext.anuSvrReceiveOut = NULL;

    case NDU_QUERY_MSG_FIFO:
      ndu_ctl->out_params.msg_in_fifo = FALSE;
      for (i = 0; i < 3; i++) {
        if(NDU_gmacContext.msgFifo[i].inIndex != NDU_gmacContext.msgFifo[i].outIndex) {
          ndu_ctl->out_params.msg_in_fifo = TRUE;
          break;
        }
      }
      break;

	default:
	  break;
  }

  return (0);
} /* NDU_gmacControl */

/*******************************************************************************
 * FUNCTION PURPOSE: ANU request is passed to NEU
 ******************************************************************************/
tint NDU_SendAnuToNeu(tuint instID, xferPktInfo_t *pktInfo)
{
  NDU_gmacContext_t *cxt;
  NduNvInst_t       *nvInst;

  /* Shorthand to the ndu context and the non-voice channel instance */
  cxt    = &(NDU_gmacContext);
  nvInst = &(cxt->nduBlk);

  /* Protect the global variables from interrupt re-entry */
  NDU_STATE_BEGIN();

  /* Store the associated voice channel, do not perform dynamic channel translation */
  nvInst->vcIdTx = instID & UTL_CID_MASK;

  /* Pass the announcement request through NEU. NEU will segment the request
   * into cells and call function siu_anu_rqst_send when done */
  (*NDU_gmacContext.pNvneuSendIn) (nvInst->neuInst, pktInfo);

  NDU_STATE_END();

  return (NDU_NOERR);

} /* NDU_SendAnuToNeu */

void NDU_FormatAnuFromNeu(void *inst, xferPktInfo_t *pkt, tint chnum)
{
  nduRouteTagHeader_t *rth;
  nduVcDemuxHeader_t  *vcd;
  NDU_gmacContext_t   *cxt;
  tuint  *p;
  tint    cellSizeWords;
  tint    pktSizeWords;
  tint    i;

  /* Shorthand to the ndu context */
  cxt = (NDU_gmacContext_t *)&NDU_gmacContext;

  /* Modify the header for each cell */
  cellSizeWords = UTL_BYTES_TO_WORDS(cxt->cellSize);
  pktSizeWords  = UTL_BYTES_TO_WORDS(pkt->pktSize[0]);

  /* p points to the base of the sequence of cells */
  p = (tuint *)(pkt->pktIn[0]);

  /* In route tag mode, the route tag for each cell must be modified
   * to include the channel number */
  if (cxt->cellSize == 58) {
    for (i = 0; i < pktSizeWords; i += cellSizeWords) {
      rth = (nduRouteTagHeader_t *)&p[i];
      NDU_GMAC_RT_SET_CHANNEL(rth, chnum);
      NDU_GMAC_RT_SET_CORE_ID(rth, NDU_gmacContext.coreId);
    }
  } 
  else if (cxt->cellSize == 54) {
    /* For VC demux cells, only change the core Id field */
    for (i = 0; i < pktSizeWords; i += cellSizeWords) {
      vcd = (nduVcDemuxHeader_t *)&p[i];
      NDU_GMAC_VC_SET_VCI_CORE(vcd, NDU_gmacContext.coreId);
    }
  }
 }  /* NDU_FormatAnuFromNeu */

/******************************************************************************
 * FUNCTION PURPOSE: Send formatted cells to the network.
 ******************************************************************************
 * DESCRIPTION: Formatted cells (with route tags and/or ATM headers are
 *              passed to the hal layer to be sent to the network
 *****************************************************************************/
/* to match the neu API this function created siuInst is not avialble for NV*/ 
tint nduGmacSendInCore (void *inst, xferPktInfo_t *pktInfo)
{
 tuint extID = 0;
  return(NDU_SendData(&NDU_gmacCallTable,extID, pktInfo));
} 
 
/****************************************************************************
 * FUNCTION PURPOSE: Format and queue a received message
 ****************************************************************************
 * DESCRIPTION: The function is called with a message that has been
 *              processed through NEU. The standard message header is
 *              created and the message is placed into the appropriate
 *              queue.
 ****************************************************************************/
tint gmacndu_rcv_message (void *inst, xferPktInfo_t *pkt)
{
  nduVcDemuxMsgPayload_t  *vcmsg;
  NduNvInst_t      *nvInst = (NduNvInst_t *)inst;
  NDU_fifo_t       *msgFifo;                       
  NDU_gmacContext_t  *cxt;
  NduStats_t       *stats;
  mgbHeader_t      header;
  tuint            *rxMsgDropPtr;
  tuint            *rxMsgCntPtr;
  tuint            *body;

  /* Shorthand to the NDU context and stats */
  cxt   = &NDU_gmacContext;
  stats = &NDU_gmacContext.nduStats;

  /* Setup FIFO and statistics pointers based on the message priority */
  switch (nvInst->rxType)  {
   
   case NDU_DPR_SIGNAL_MSG:
     msgFifo      = &(cxt->msgFifo[NDU_INTERRUPT_MESSAGE]);
     rxMsgDropPtr = &(stats->RxSigMsgDrop);
     rxMsgCntPtr  = &(stats->RxSigMsgCnt);
     break;

   case NDU_DPR_NORMAL_MSG:
     msgFifo      = &(cxt->msgFifo[NDU_NORMAL_MESSAGE]);
     rxMsgDropPtr = &(stats->RxCtrlMsgDrop);
     rxMsgCntPtr  = &(stats->RxCtrlMsgCnt);
     break;

   case NDU_DPR_LOW_MSG:
     msgFifo      = &(cxt->msgFifo[NDU_LOW_MESSAGE]);
     rxMsgDropPtr = &(stats->RxLowMsgDrop);
     rxMsgCntPtr  = &(stats->RxLowMsgCnt);
     break;

   default: /* We got a bad non voice channel ID. */
     stats->RxInvldNvCh += 1;
     return (NDU_ERROR);

  }

  /* The received payload has the message with the VC demux message header.
   * THIS DOES NOT IMPLY CELLS HAVE BEEN RECEIVED */
  vcmsg           = (nduVcDemuxMsgPayload_t *)(pkt->pktIn[0]);
  /* To be reviewed - Machu */
  header.length   = UTL_BYTES_TO_TINT(pkt->pktSize[0]) + MGB_HEADER_SIZE - NDU_VC_DEMUX_MSG_HEADER_LENGTH;
  header.chnum    = NDU_GMAC_VC_DEMUX_MSG_EXTRACT_CHANNEL(vcmsg);
  header.msg_id   = vcmsg->msg_id;
  header.trans_id = vcmsg->trans_id;
  body            = &(vcmsg->msg_first_word);

  /* Put the message into the appropriate fifo */
  if (msgFifo->free < header.length) {
    *rxMsgDropPtr += 1;
     return (NDU_ERROR);
  }

  /* Put the message header in the Fifo. */
  utilFifoWrite(msgFifo->start, (tuint *)&header, &msgFifo->inIndex, 
                          &msgFifo->outIndex,msgFifo->fifolen, MGB_HEADER_SIZE);

  if (header.length > MGB_HEADER_SIZE)
    utilFifoWrite(msgFifo->start, body, &msgFifo->inIndex, &msgFifo->outIndex,
                             msgFifo->fifolen, header.length - MGB_HEADER_SIZE);

  msgFifo->free -= header.length;

  /* The message is successfully in the fifo */
  *rxMsgCntPtr +=1 ;
  return (NDU_NOERR);
} /* gmndu_rcv_message */

#endif

/*****************************************************************************
 * FUNCTION PURPOSE: Open an run time configurable NEU instance
 *****************************************************************************
 * DESCRIPTION: Opens the NEU instance with the information provided at run
 *              time by a message from the host.
 *****************************************************************************/
void gmacndu_rt_open_neu (void *neuInst, struct MGB_MDS_NET_ENCAP_CHANNEL_SET *chcfg,
                          void *tInst, tint (*neuOut)(void *, xferPktInfo_t *),
                          NDU_config_t *config)
{
  neuCfg_t   cfg;
  neuCtrl_t  ctrl;
  tint       shift;
  tuint      val;

  /* Provide function pointer for outgoing data */
  cfg.neuSendOut.neuSendOut = neuOut;
  cfg.neuSendOut.targetInst = tInst;

  cfg.parm.chnl.enable = TRUE;  /* Enable NEU */

  /* Flags. Bit exactness of the desired flags was checked by the compiler
   * at the top of the function. */
  #warn disable NEU checksum for now: need to debug the potential host or linux issue, as in gmacndu_open_neu()
  cfg.parm.chnl.cfg_flag1 = chcfg->flags1 & ~(0x0002);
  cfg.parm.chnl.cfg_flag2 = chcfg->flags2;
  cfg.parm.chnl.sar_type  = neuproto_SAR_NONE;

  /* Alternate ports unused */
  cfg.parm.chnl.alt_local_port  = 0;
  cfg.parm.chnl.alt_remote_port = 0;
  memset(cfg.parm.chnl.alt_remote_addr, 0, MGB_IP_ADDR_MAX_NUM_WORDS * sizeof(UINT16));

  /* report control must be disabled */
  cfg.parm.chnl.report_ctrl          = NET_ENCAP_ENCAP_REPORT_DISCARD;
  cfg.parm.chnl.report_throttle_time = 0;
  cfg.parm.chnl.sysInst              = NULL;

  /* No second protocol allowed */
  cfg.parm.chnl.second_prototype = NET_ENCAP_NULL_PROTOCOL;

  /* This next parameter might need to be updated with the boot config cell changes */
  cfg.parm.chnl.rx_offset_bytes = 0;
  cfg.parm.chnl.rx_payload_type = chcfg->pkt2tel_first_protocol;

  /* Tx parameters */
  cfg.parm.chnl.tx_ip_offset_bytes    = chcfg->tel2pkt_ip_offset;
  cfg.parm.chnl.tx_udp_offset_bytes   = chcfg->tel2pkt_udp_offset;
  cfg.parm.chnl.tx_802_3_offset_bytes = chcfg->tel2pkt_802_3_offset;
  cfg.parm.chnl.tx_min_cell_bytes     = chcfg->tel2pkt_min_len;

  /* For the utopia NDU with MII the core ID is placed into the UDP source address
   * as defined in the ggswcfg.h */
  if ( (cfg.parm.chnl.tx_udp_offset_bytes >= 0) && (ggConfig.neuUdpSrcMask) ) {
    val = ggConfig.neuUdpSrcMask;
    for (shift = 0; !(val & 0x1); shift++, val >>= 1);
    cfg.parm.chnl.tx_udp_src_mask = ggConfig.neuUdpSrcMask;
    cfg.parm.chnl.tx_udp_src_val  = NDU_gmacContext.coreId << shift;

  } else {
    cfg.parm.chnl.tx_udp_src_mask = 0;
    cfg.parm.chnl.tx_udp_src_val  = 0;
  }

  cfg.parm.chnl.halId = config->halId;

  /* Transmit header */
  cfg.parm.chnl.tx_total_hdr_len_bytes = chcfg->tel2pkt_total_hdr_len;
  memcpy (cfg.parm.chnl.header, chcfg->hdr_image, UTL_BYTES_TO_WORDS(chcfg->tel2pkt_total_hdr_len));

  /* only in config as required once */
  (*config->pNvneuOpen) (neuInst, &cfg);

  /* The parameters must be re-passed to enable sending of cells */
  ctrl.code   = neu_CTRL_CHANNEL_CFG;
  ctrl.u.parm = cfg.parm;
  (*NDU_gmacContext.pNvneuControl) (neuInst, &ctrl);

} /* gmacndu_rt_open_neu */

/**********************************************************************************
 * FUNCTION PURPOSE: Configure announcement network header
 **********************************************************************************
 * DESCRIPTION: Creates and configures the network header instance for
 *              bulk memory read (announcement) requests.
 **********************************************************************************/
tint NDU_gmacConfigAnuReq (struct MGB_MDS_NET_ENCAP_CHANNEL_SET *chcfg, NDU_config_t *config, bool *localServer)
{
  neuCtrl_t neuCtl;

  /* Configure NEU for announcements if present */
  if (config->pAnuRec) { 

    if (NDU_gmacContext.nduBlk.neuInst == NULL)  {

      /* create a non voice neu instance */
      NDU_gmacContext.nduBlk.neuInst = (*config->pNvNeuCreate)(GG_NVECO_BLK, 
                                        UTL_BYTES_TO_WORDS(chcfg->tel2pkt_total_hdr_len));
      if (NDU_gmacContext.nduBlk.neuInst == NULL)
        return (-1);

    }  else  {

      /* There is already a request configuration. The unmap the old udp
       * port before configuring the new request. There is no error if the port
       * is not found since it wasn't in the map. */
      neuCtl.code = neu_CTRL_GET_SRC_UDP_PORT;
      neuCtl.u.portMap.inst = NDU_gmacContext.nduBlk.neuInst;
      if ((*NDU_gmacContext.pNvneuControl) (NDU_gmacContext.nduMsg.neuInst, &neuCtl) == neu_NOERR)  {
        neuCtl.code = neu_CTRL_RMV_PORTMAP;
        (*NDU_gmacContext.pNvneuControl) (NDU_gmacContext.nduMsg.neuInst, &neuCtl);
      }


    }

    /* Open the non-voice NEU channel */
    gmacndu_rt_open_neu (NDU_gmacContext.nduBlk.neuInst, chcfg, 
                          &(NDU_gmacContext.nduBlk), config->pAnuXmt, config);

    /* Enable the NEU channel receive callouts */
    gmacndu_rxcall_neu (NDU_gmacContext.nduBlk.neuInst, config->pAnuRec, 
                             &(NDU_gmacContext.nduBlk));


    /* Check for local server configuration. A local server is identified by the loopback IP
     * address 127.0.0.1 for the destination IP address */
    neuCtl.code            = neu_CTRL_CHECK_LOOPBACK;
    neuCtl.u.loopBack.inst = &(NDU_gmacContext.nduBlk);
    if ((*NDU_gmacContext.pNvneuControl) (NDU_gmacContext.nduBlk.neuInst, &neuCtl) != neu_NOERR)
      return (-1);

    if (neuCtl.u.loopBack.isLoopback == TRUE)   {
      *localServer = TRUE;
      return (0);
    }

    *localServer = FALSE;
    

    /* Configure the NEU port map to receive the UDP port for Annoucement */
    /* Verify there is a valid UDP offset in the header. */
    if (chcfg->tel2pkt_udp_offset < 0)
      return (-1);

    neuCtl.code           = neu_CTRL_ADD_PORTMAP;
    neuCtl.u.portMap.inst = &(NDU_gmacContext.nduBlk);
    neuCtl.u.portMap.port = pktRead16bits((word *)&chcfg->hdr_image[UTL_BYTES_TO_TINT(chcfg->tel2pkt_udp_offset + neuproto_UDP_BYTE_OFFSET_SRC_PORT)],0);

    /* Return a failure if the port could not be added to the port map, since
     * announcements won't work */
    if ((*NDU_gmacContext.pNvneuControl) (NDU_gmacContext.nduMsg.neuInst, &neuCtl) != neu_NOERR)
      return (-1);


  } else  {

    /* Announcements not enabled */
    return (-1);

  }

 
 return (0);

} /* NDU_gmacConfigAnuReq */

/**********************************************************************************
 * FUNCTION PURPOSE: Configure streaming server network header
 **********************************************************************************
 * DESCRIPTION: Creates and configures the network header instance for
 *              streaming NEU.
 **********************************************************************************/
tint NDU_gmacConfigNeuStreamer(struct MGB_MDS_NET_ENCAP_CHANNEL_SET *chcfg, NDU_config_t *config)
{
    tint retVal = 0;

    if (NDU_gmacContext.nduTrace.neuInst == NULL)  {
      /* create a non voice neu instance */
      NDU_gmacContext.nduTrace.neuInst = (*config->pNvNeuCreate)(GG_NVECO_TRACE, 
                                        UTL_BYTES_TO_WORDS((MGB_NET_ENCAP_MAX_HDR_LEN_UINT16 << 1)));
      if (NDU_gmacContext.nduTrace.neuInst == NULL)
        return (-1);

      /* Open the non-voice NEU channel */
      gmacndu_rt_open_neu (NDU_gmacContext.nduTrace.neuInst, chcfg, 
                            &(NDU_gmacContext.nduTrace), config->pAnuXmt, config);
    } else {
      retVal = -1;
    }

    return(retVal);
}

/* Nothing past this point */




