#ifndef _NDUGMAC_H
#define _NDUGMAC_H
/******************************************************************************
 * FILE PURPOSE: System level GAMC definitions and prototypes.
 ******************************************************************************
 * DESCRIPTION: GAMC function prototypes and definitions
 *
 * TABS: None
 *
 * Copyright (C) 2005, Texas Instruments 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*****************************************************************************
 * DEFINITION: The maximum size of an atm cell
 ****************************************************************************/
#define NDU_GAMC_MAX_CELL_SIZE  58

/* Maximum Channel ID supported */
#define NDU_GMAC_MAX_CHANNEL_ID  0x00FF

/*****************************************************************************
 * DEFINITION: Route Tag mode structure
 *****************************************************************************/
typedef struct {
  tuint msg_id;              /* Message ID                    */
  tuint trans_id;            /* Transaction ID                */
  tuint msg_first_word;      /* First word of message body    */
} nduRouteTagMsgPayload_t;

#define NDU_ROUTE_TAG_MSG_HEADER_LENGTH   2

typedef struct {
  tuint type_dest_src;       /* Type, dest phy, source phy    */
  tuint janus_core_channel;  /* Janus ID, Core ID, Channel ID */
  tuint gfc_vpi_vci;         /* GFC, VPI, VCI                 */
  tuint vci_pt;              /* VCI, PT                       */
  tuint hec_dummy;           /* HEC, padding                  */
  tuint payload_first_word;  /* 1st word of payload           */
} nduRouteTagHeader_t;

#define NDU_GAMC_RT_HEADER_SIZE_WORDS   5  /* Size less 1st byte */ 
/* Macros for extraction of the type field */
#define NDU_GAMC_RT_TYPE_MASK   0xF800
#define NDU_GAMC_RT_TYPE_SHIFT      11
#define NDU_GAMC_RT_EXTRACT_TYPE(x) \
   ((((x)->type_dest_src)&NDU_GAMC_RT_TYPE_MASK)>>NDU_GAMC_RT_TYPE_SHIFT)
#define NDU_GAMC_RT_SET_TYPE(x,y) \
   ((x)->type_dest_src)=(((x)->type_dest_src)&~NDU_GAMC_RT_TYPE_MASK)|\
                        ((y)<<NDU_GAMC_RT_TYPE_SHIFT)
#define NDU_GAMC_RT_DEST_PHY_MASK  0x07C0
#define NDU_GAMC_RT_DEST_PHY_SHIFT 6
#define NDU_GAMC_RT_SET_DEST_PHY(x,y) \
    ((x)->type_dest_src)= \
      (((x)->type_dest_src)&~NDU_GMAC_RT_DEST_PHY_MASK)|\
      ((y)<<NDU_GMAC_RT_DEST_PHY_SHIFT)
/* Change to reflect 0-7 Bits for Channel and 8-10 Bits for Core*/
#define NDU_GMAC_RT_CORE_ID_MASK    0x700
#define NDU_GMAC_RT_CORE_ID_SHIFT       8
#define NDU_GMAC_RT_SET_CORE_ID(x,y) \
   ((x)->janus_core_channel)= \
     (((x)->janus_core_channel)&~NDU_GMAC_RT_CORE_ID_MASK)|\
     ((y)<<NDU_GMAC_RT_CORE_ID_SHIFT)
#define NDU_GMAC_RT_CHANNEL_MASK    0x00FF
#define NDU_GMAC_RT_CHANNEL_SHIFT        0
#define NDU_GMAC_RT_SET_CHANNEL(x,y) \
   ((x)->janus_core_channel)= \
     (((x)->janus_core_channel)&~NDU_GMAC_RT_CHANNEL_MASK)|\
     ((y)<<NDU_GMAC_RT_CHANNEL_SHIFT)
/* Values for the type field, names must match application code */
#define NDU_GMAC_RT_TYPE_NORMAL_MESSAGE   29
#define NDU_GMAC_RT_TYPE_HIGH_MSG     28
#define NDU_GMAC_RT_TYPE_LOW_MSG      30
#define NDU_GMAC_RT_TYPE_AAL1          6
#define NDU_GMAC_RT_TYPE_ANNOUNCE      4
#define NDU_GMAC_RT_TYPE_HEALTH        9

/*****************************************************************************
 * DEFINITION: VC demux mode structure 
 *****************************************************************************/
typedef struct {
  tuint dsp_id;              /* DSP ID                        */
  tuint prior_core_ch;       /* Priority, core id, channel id */
  tuint msg_id;              /* Message ID                    */
  tuint trans_id;            /* Transaction ID                */
  tuint msg_first_word;      /* First word of message         */
} nduVcDemuxMsgPayload_t;

#define NDU_VC_DEMUX_MSG_HEADER_LENGTH   4
#define NDU_GMAC_VC_DEMUX_MSG_PRIORITY_MASK  0xF800
#define NDU_GMAC_VC_DEMUX_MSG_PRIORITY_SHIFT 11
#define NDU_GMAC_VC_DEMUX_MSG_EXTRACT_PRIORITY(x) \
  ((((x)->prior_core_ch) & NDU_GMAC_VC_DEMUX_MSG_PRIORITY_MASK) >> \
  NDU_GMAC_VC_DEMUX_MSG_PRIORITY_SHIFT)
#define NDU_GMAC_VC_DEMUX_MSG_SET_PRIORITY(x,y) \
  ((x)->prior_core_ch) = \
  ((((x)->prior_core_ch) & ~NDU_GMAC_VC_DEMUX_MSG_PRIORITY_MASK) | \
  ((y) << NDU_GMAC_VC_DEMUX_MSG_PRIORITY_SHIFT))

/* Values for the type field, names must match application code */
#define NDU_GMAC_VC_DEMUX_MSG_PRIORITY_NORMAL NDU_GMAC_RT_TYPE_NORMAL_MESSAGE
#define NDU_GMAC_VC_DEMUX_MSG_PRIORITY_HIGH   NDU_GMAC_RT_TYPE_HIGH_MSG
#define NDU_GMAC_VC_DEMUX_MSG_PRIORITY_LOW    NDU_GMAC_RT_TYPE_LOW_MSG
/* Change to reflect 0-7 Bits for Channel and 8-10 Bits for Core*/
#define NDU_GMAC_VC_DEMUX_MSG_COREID_MASK  0x0700
#define NDU_GMAC_VC_DEMUX_MSG_COREID_SHIFT 8
#define NDU_GMAC_VC_DEMUX_MSG_SET_COREID(x,y) \
  ((x)->prior_core_ch) = \
  ((((x)->prior_core_ch) & ~NDU_GMAC_VC_DEMUX_MSG_COREID_MASK) | \
  ((y) << NDU_GMAC_VC_DEMUX_MSG_COREID_SHIFT))

#define NDU_GMAC_VC_DEMUX_MSG_CHANNEL_MASK 0x00FF
#define NDU_GMAC_VC_DEMUX_MSG_CHANNEL_SHIFT 0
#define NDU_GMAC_VC_DEMUX_MSG_EXTRACT_CHANNEL(x) \
  ((((x)->prior_core_ch) & NDU_GMAC_VC_DEMUX_MSG_CHANNEL_MASK) >> \
  NDU_GMAC_VC_DEMUX_MSG_CHANNEL_SHIFT)
#define NDU_GMAC_VC_DEMUX_MSG_SET_CHANNEL(x,y) \
  ((x)->prior_core_ch) = \
  ((x)->prior_core_ch) & (~NDU_GMAC_VC_DEMUX_MSG_CHANNEL_MASK) | \
  ((y) << NDU_GMAC_VC_DEMUX_MSG_CHANNEL_SHIFT)
  
/*                                                                         */
/* #define NDU_GMAC_VC_MSG_CHANNEL_MASK  0x3F                              */
/* #define NDU_GMAC_VC_MSG_CHANNEL_SHIFT 0                                 */
/* #define NDU_GMAC_VC_EXTRACT_MSG_CHANNEL(x) \                            */
/*   ((((x)->prior_core_chprior_core_ch)& NDU_GMAC_VC_MSG_CHANNEL_MASK)>>\ */
/*                                NDU_GMAC_VC_MSG_CHANNEL_SHIFT)           */

typedef struct {
  tuint gfc_vpi_vci;          /* GFC, VPI and VCI fields                   */
  tuint vci_core_channel_pt;  /* VCI, VCI core, VCI channel and PT fields  */
  tuint hec_dummy;            /* HEC and dummy fields                      */
  tuint payload_first_word;   /* 1st word of payload                       */
} nduVcDemuxHeader_t;

//#define NDU_GMAC_VC_HEADER_SIZE_WORDS   3 /* Size less first word  */
/* Macros for extraction of the vci channel field */
#define NDU_GMAC_VC_VCI_CHANNEL_MASK    0x0FF0 
#define NDU_GMAC_VC_VCI_CHANNEL_SHIFT        4 
/* NOTE THAT THE VCI MACROS CAN ONLY HANDLE 12 OF THE 16 BITS */
#define NDU_GMAC_VC_EXTRACT_VCI_CHANNEL(x) \
  ((((x)->vci_core_channel_pt)&NDU_GMAC_VC_VCI_CHANNEL_MASK)>>\
                               NDU_GMAC_VC_VCI_CHANNEL_SHIFT)
#define NDU_GMAC_VC_SET_VCI_CHANNEL(x,y) \
   ((x)->vci_core_channel_pt)= \
      (((x)->vci_core_channel_pt)&~NDU_GMAC_VC_VCI_CHANNEL_MASK)|\
      ((y)<<NDU_GMAC_VC_VCI_CHANNEL_SHIFT)
#define NDU_GMAC_VC_VCI_CORE_MASK      0x7000
#define NDU_GMAC_VC_VCI_CORE_SHIFT     12
#define NDU_GMAC_VC_SET_VCI_CORE(x,y) \
   ((x)->vci_core_channel_pt)= \
      (((x)->vci_core_channel_pt)&~NDU_GMAC_VC_VCI_CORE_MASK)|\
      ((y)<<NDU_GMAC_VC_VCI_CORE_SHIFT)

/* #define NDU_GMAC_VC_VCI_GFC_MASK       0xF000                */
/* #define NDU_GMAC_VC_VCI_GFC_SHIFT      12                    */
/* #define NDU_GMAC_VC_EXTRACT_GFC(x)     \                     */
/*         ((x)->gfc_vpi_vci & NDU_GMAC_VC_VCI_GFC_MASK) >> \   */
/*         NDU_GMAC_VC_VCI_GFC_SHIFT                            */
/*                                                              */
/* #define NDU_GMAC_VC_SET_GFC(x,y)       \                     */
/*         (x)->gfc_vpi_vci = \                                 */
/*         (((x)->gfc_vpi_vci) & ~NDU_GMAC_VC_VCI_GFC_MASK) | \ */
/*         ((y) << NDU_GMAC_VC_VCI_GFC_SHIFT)                   */

/* Valid values for the vci channel field */
//#define NDU_GMAC_VC_MSG_HIGH_CHANNEL   2   /* High priority msg channel   */
//#define NDU_GMAC_VC_MSG_CHANNEL        3   /* Normal priority msg channel */
//#define NDU_GMAC_VC_MSG_LOW_CHANNEL    4   /* Low priority msg channel    */
//#define NDU_GMAC_VC_ANU_CHANNEL        5   /* Announcement channel */


/* Definition: Structure definition defining the payload used for keep alive data
 *************************************************************************************/
//typedef struct {
//  tuint seqNum;              /* Sequence number - currently unused */
//  tuint chip_core;           /* DSP and core ID values             */
//} ka_payload_rt_t;

/* Macros to manipulate ka_payload */
/* #define NDU_KEEP_ALIVE_CHIP_MASK       0xF800                            */
/* #define NDU_KEEP_ALIVE_CHIP_SHIFT      11                                */
/* #define NDU_KEEP_ALIVE_SET_CHIP_ID(x,y) \                                */
/*   ((x)->chip_core) = (((x)->chip_core) & ~NDU_KEEP_ALIVE_CHIP_MASK)  | \ */
/*                      ((y) << NDU_KEEP_ALIVE_CHIP_SHIFT)                  */
/*                                                                          */
/* #define NDU_KEEP_ALIVE_CORE_MASK       0x01C0                            */
/* #define NDU_KEEP_ALIVE_CORE_SHIFT      6                                 */
/* #define NDU_KEEP_ALIVE_SET_CORE_ID(x,y)  \                               */
/*   ((x)->chip_core) = (((x)->chip_core) & ~NDU_KEEP_ALIVE_CORE_MASK) | \  */
/*                      ((y) << NDU_KEEP_ALIVE_CORE_SHIFT)                  */
/*                                                                          */

/*************************************************************************************
 * Definition: Definition of keep alive payload for mii cells
 *************************************************************************************/
//typedef struct {
//  tuint transId;          /* Transaction ID                    */
//  tuint opCode_count;     /* Opcode and count field            */
//  tuint addrMsw;          /* Most significant word of DSP ID   */
//  tuint addrLsw;          /* Least significant word of DSP ID  */
//  tuint valMsw;           /* Most significant word of core ID  */
//  tuint valLsw;           /* Least significant word of core ID */
//} ka_payload_vc_t;

/* There is only one valid value for the opCode_count field */
/* #define NDU_KEEP_ALIVE_VC_OP_COUNT  0x5001 */
/*                                            */
/* #define NDU_KEEP_ALIVE_SET_ADDR(x,y) \     */
/*         (x)->addrMsw = 0; (x)->addrLsw = y */
/*                                            */
/* #define NDU_KEEP_ALIVE_SET_VAL(x,y) \      */
/*         (x)->valMsw = 0; (x)->valLsw = y   */

/* GFC value used to identify keep alive cells */
/* #define NDU_GMAC_VC_GFC_KEEP_ALIVE    0x2                                              */
/*                                                                                        */
/* #define NDU_KEEP_ALIVE_PAYLOAD_SIZE_BYTES  UTL_WORDS_TO_BYTES(sizeof(ka_payload_rt_t)) */

/*****************************************************************************
 * DEFINITION: The payload size for cells is always 48 bytes
 *****************************************************************************/
#define NDU_GMAC_PAYLOAD_SIZE_WORDS    48

tint gmacndu_msg_send (void *inst, xferPktInfo_t *pkt);
tint gmacndu_rcv_message (void *inst, xferPktInfo_t *pkt);
void *ndu_gm_modify_vc_msg_header (NDU_DPRMsgType_t mtype, mgb_t *msg);
void gmacndu_open_neu (void *neuInst, diagApp_t *diagApp, diagAppHeader_t *hdr, 
                       void *tInst, tint (*neuOut)(void *, xferPktInfo_t *),
                       NDU_config_t *config);
void gmacndu_rxcall_neu (void *inst, tint (*fcn)(void *, xferPktInfo_t *), 
                       void *rcvInst);
tint NDU_gmacConfigAnuReq (struct MGB_MDS_NET_ENCAP_CHANNEL_SET *chcfg, NDU_config_t *config, bool *localServer);
tint NDU_gmacConfigNeuStreamer(struct MGB_MDS_NET_ENCAP_CHANNEL_SET *chcfg, NDU_config_t *config);

#endif /* _NDUGMAC_H */
