/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "micdspdp.h"
#include "ndu.h"
#include "nduloc.h"
#include "nduport.h"
#include "nduutop.h"
#include "types.h"
#include "util.h"
#include "utilfifo.h"
#include "hal.h"
#include "ggcfg.h"  /* Needed since SIU_LAST_CH define references ggConfig. */
#include "ggmbuf.h"
#include <ti/mas/neu/neuproto.h>
#include "utlrepck_m.h"


/* Different include files are used for the diagnostic code. The include
 * file typedefs the name nduInst_t to be either nduInst_t (for diag)
 * or siuInst_t (for application). The siu context is also typedefed */
#include <ti/mas/neu/aal5.h>
#include "diagapp.h"
#include "ggswcfg.h"

NDU_utContext_t  NDU_utContext;

#ifdef NDU_UTOPIA
 
/*****************************************************************************
 * For the diagnostic code, the call table instantiated 
 *****************************************************************************/
#ifdef DSIU
siuNduCallTable_t siuNduCallTable = {
  siuUtNduOpen,
  siuUtNduClose,
  siuUtNduSendIn,
  siuUtNduReceiveIn,
  siuUtIntReceiveIn,
  siuUtNduControl,
  siuUtGetNormalMessage,
  siuUtGetInterruptMessage,
  siuUtWriteMessageToDPR,
  siuUtMSG_test,
  siuUtHeartBeat,
  siuUtGetClockMul,
  siuUtMsgInit,
  siuUtPushFifo,
  siuUtFifo2DPR
};
#endif /* DSIU */

//static tint NDU_utInit = 0;
extern NduHealthTable_t gg_utndu_health_ctable;


void NDU_utSetIntMode(tuint int_mode)
{
}

/******************************************************************************
 * FUNCTION PURPOSE: ndu statistics 
 ******************************************************************************
 * DESCRIPTION: This function retuns the ndu stats also it can reset
 *              the ndu stats.
 *****************************************************************************/

void NDU_utGetStats(NduStats_t *stats, bool reset)
{
  //NDU_utContext.nduStats.RxBadAAL5CRCDrop = NDU_utContext.aal5Inst.errStats.rxInvdPkt;
 *stats = NDU_utContext.nduStats;
  /* reset if required */
  if (reset) {
   memset(&NDU_utContext.nduStats, 0, sizeof(NduStats_t));
  //  NDU_utContext.aal5Inst.errStats.rxInvdPkt = 0;
 }
} /* NDU_utGetStats */

/******************************************************************************
 * FUNCTION PURPOSE: ndu message polling
 ******************************************************************************
 * DESCRIPTION: .This function reads the message in FIFOs and calls processing
 *               function.
 *****************************************************************************/

tint NDU_utGetMsg(NDU_msgPriority_e priority)
{
    NDU_fifo_t   *fifoInst;
    tuint       msgBuf[MGB_HD_COMPOUND_MSG_MAX_LENGTH+MGB_HEADER_SIZE];
    mgb_t       *msg;
    tuint       *msgbuf1;

    msg = (mgb_t *)msgBuf;
    msgbuf1 = (tuint *)msgBuf;

    fifoInst = &NDU_utContext.msgFifo[priority];
  if ((priority == NDU_NORMAL_MESSAGE) && (fifoInst->free == fifoInst->fifolen)){
        fifoInst = &NDU_utContext.msgFifo[NDU_LOW_MESSAGE];
  }
  
  if (fifoInst->free < fifoInst->fifolen) {
    NDU_STATE_BEGIN();
        utilFifoRead(fifoInst->start, msgbuf1, &fifoInst->inIndex, 
            &fifoInst->outIndex, fifoInst->fifolen, MGB_HEADER_SIZE);
        
        fifoInst->free += MGB_HEADER_SIZE;
                
    if (msg->header.length > MGB_HEADER_SIZE) {
            utilFifoRead(fifoInst->start, msgbuf1 + MGB_HEADER_SIZE, &fifoInst->inIndex,
                &fifoInst->outIndex, fifoInst->fifolen, msg->header.length - MGB_HEADER_SIZE);
            fifoInst->free += (msg->header.length - MGB_HEADER_SIZE);
        }
    NDU_STATE_END();

        (*NDU_utContext.pProcessMsg)(priority, msg);
        
  }
  return NDU_NOERR;
} /* NDU_utGetMsg */

/******************************************************************************
 * FUNCTION PURPOSE: Initializes the Utopia interface.
 ******************************************************************************
 * DESCRIPTION: This function initializes the application to handle the utopia 
 *              cells. The input parameter is for compatibility with the
 *              legacy HPI.
 *              In the case of application code, function will return the 
 *              updated cell size back.
 *****************************************************************************/
tint NDU_utConfig(NDU_config_t *config)
{
#ifndef DSIU
  diagApp_t    appInit;
  tint i;
#endif
  tint lenWords;

  /* For the application code the message fifos must be initialized. The
     diag/bootloader uses a different initialization process at system start */
#ifndef DSIU
  /* Since this function is called twice, once at startup, and  once after
   * hardware config, prevent re-entry, which could cause the loss of a 
   * message when the fifos are reset */
 // if (NDU_utInit)
 //   return (0);
 // NDU_utInit = 1;
  
  /*  Thse buffers are used for dsp to micro communication */
  NDU_utContext.msgFifo[0].start   = (tuint *)utNduHeapDesc.bufDesc[0].buffAddr;
  NDU_utContext.msgFifo[1].start   = (tuint *)utNduHeapDesc.bufDesc[1].buffAddr;
  NDU_utContext.msgFifo[2].start   = (tuint *)utNduHeapDesc.bufDesc[2].buffAddr;
 NDU_utContext.msgFifo[0].fifolen =
 NDU_utContext.msgFifo[0].free    = GG_NDU_MSG_HIGH_FIFO_L;
 NDU_utContext.msgFifo[1].fifolen = 
 NDU_utContext.msgFifo[1].free    = GG_NDU_MSG_NORMAL_FIFO_L;
 NDU_utContext.msgFifo[2].fifolen =
 NDU_utContext.msgFifo[2].free    = GG_NDU_MSG_LOW_FIFO_L;
#ifdef  GG_INCLUDE_KEEP_ALIVE_NDU
 NDU_utContext.NduHealthTable  = &gg_utndu_health_ctable;
#else
 NDU_utContext.NduHealthTable  = NULL;
#endif
 
  /*  There are 3 fifos for dsp to micro communication */
  for (i = 0; i < 3; i++) {
   NDU_utContext.msgFifo[i].inIndex = 
   NDU_utContext.msgFifo[i].outIndex = 0;
 }

  /* Initialize the message aal5 interface. For the application the structure
   * defining cell size and route/tag atm headers is found in a pre-defined
   * memory location. For now use local data memory. 
   * Copy the stored memory structure to the local memory. This is required
   * since the global copy may be in program memory */
   (*config->pUtMsgInit)(&appInit);
  
  /* coreId is initialized in halnew function */
   siuContext.cellSize =  NDU_utContext.cellSize = appInit.utopia.cellSize;
  NDU_utContext.dspId             = appInit.dspId;

  NDU_utContext.kpAlive_gfc_vp_vcih    = appInit.kpAlive_gfc_vp_vcih;
  NDU_utContext.kpAlive_vcil_pt_clp    = appInit.kpAlive_vcil_pt_clp;

  NDU_utContext.kpAliveAddr_hi    = appInit.kpAliveAddr_hi;
  NDU_utContext.kpAliveAddr_lo    = appInit.kpAliveAddr_lo;
  NDU_utContext.coreId            = config->coreId;
    NDU_utContext.pNvneuControl     = config->pNvneuControl;  
    NDU_utContext.pNvneuReceiveIn   = config->pNvneuReceiveIn;
    NDU_utContext.pNvneuSendIn      = config->pNvneuSendIn;

  /* -- for messages -- */
    lenWords = UTL_BYTES_TO_WORDS(appInit.utopia.msg.totalHeaderLenBytes);
   
  /* create a non voice neu instance */
    NDU_utContext.nduMsg.neuInst = (*config->pNvNeuCreate)(GG_NVECO_MSG, lenWords);
 
   /* Open the non-voice NEU channels */
  utndu_open_neu (NDU_utContext.nduMsg.neuInst, &appInit, &appInit.utopia.msg,
                                 &(NDU_utContext.nduMsg), utndu_msg_send,config);

   /* Enable the NEU channel receive callouts */
   utndu_rxcall_neu (NDU_utContext.nduMsg.neuInst, utndu_rcv_message, &(NDU_utContext.nduMsg));

  /* Only one NEU instance is used for messages, but each priority has its own aal5
   * instance. Allocate the space for these and initialize them. */
   utndu_create_msg_aal5();
  
   /* Open NEU for announcements if present */
  if (config->pAnuRec) { 
       /* -- for block memory requests -- */
    lenWords = UTL_BYTES_TO_WORDS(appInit.utopia.block.totalHeaderLenBytes);
    NDU_utContext.nduBlk.neuInst = (*config->pNvNeuCreate)(GG_NVECO_BLK, lenWords);
   utndu_open_neu (NDU_utContext.nduBlk.neuInst, &appInit, &appInit.utopia.block,
                                 &(NDU_utContext.nduBlk), config->pAnuXmt,config);
    utndu_rxcall_neu (NDU_utContext.nduBlk.neuInst, config->pAnuRec, &(NDU_utContext.nduBlk));
    } 
    else {
    NDU_utContext.nduBlk.neuInst = NULL;
    }

  /* Create an NEU for keep alives, if present */
    if (NDU_utContext.NduHealthTable)  {
     (*NDU_utContext.NduHealthTable->nduHealthInit)((void *)&appInit,config);
     } 
  else {
      NDU_utContext.nduHlt.neuInst = NULL;
     }

    /* Final control initialization */
  //NDU_utContext.aal5CrcGen        = 0;     //appInit.crcGen;
  //NDU_utContext.aal5CrcCheck      = 0;     //appInit.crcCheck;
    NDU_utContext.pAal5SendCell     = config->pAal5SendCell;
    NDU_utContext.pAal5SendProcess  = config->pAal5SendProcess;
    NDU_utContext.pAal5Receive      = config->pAal5Receive;
    NDU_utContext.pProcessData      = config->pProcessData;
    NDU_utContext.pProcessMsg       = config->pProcessMsg;
    NDU_utContext.pAnuRec           = config->pAnuRec;
    NDU_utContext.phalUtopiaControl = config->phalUtopiaControl;
    NDU_utContext.ut_portid         = config->utopia_portid; 
  
  /* Initialize NDU statistics */
  memset ((void *)&NDU_utContext.nduStats, 0, sizeof(NduStats_t));
  
  return (NDU_utContext.cellSize);
#else  /* DSIU */
    return (0);
#endif
} /* NDU_utConfig */
/*****************************************************************************
 * FUNCTION PURPOSE: poll for cells
 *****************************************************************************
 * DESCRIPTION: This could be used to call halPoll()
 *****************************************************************************/
tint NDU_utGetData ()
{
  return 0;
} /* NDU_utGetData */

/*****************************************************************************
 * FUNCTION PURPOSE: Interrupt handler for cells
 *****************************************************************************
 * DESCRIPTION: This function is called from the hal driver when cells are
 *              available. This function determines the cell type. Messages
 *              are routed directly to the aal5 library. All other types are
 *              sent to the ndu receive out function pointer.
 *****************************************************************************/
void NDU_utPushRcvData (tuint utopiaChannelId, word *cell, tuint sizeBytes)
{
  nduRouteTagHeader_t  *rth;
  nduVcDemuxHeader_t   *vcd;

  tuint                *pkt_p;
  NDU_utContext_t      *cxt;
  NduNvInst_t          *nvp;
  void                 *aal5;
  xferPktInfo_t         pktOut;
  neuAal5Ctrl_t         neuAal5Ctrl; /* Light NEU control to save stack */
  tuint type;
  tuint cellSizeWords;
  tuint cellSizeBytes;
  tint channel,ret;
  tint p = 0;

  /* Reference to the system ndu context */
  cxt = &(NDU_utContext);

  /* Convert the individual cell size to words */
  cellSizeWords = UTL_BYTES_TO_WORDS(cxt->cellSize);
  cellSizeBytes = cxt->cellSize;

  /* Process all cells received */
  while (sizeBytes >= cxt->cellSize)  {
    nvp  = NULL; /* Set to non-null for message/announcement types     */
    aal5 = NULL; /* If set an aal5 instance change is required         */

    switch (cxt->cellSize) {
      case 58:
        /* Route tags used - routing information is in the type field */
        rth     = (nduRouteTagHeader_t *)&cell[p];
        channel = NDU_UTOPIA_RT_EXTRACT_CHANNEL(rth);
        type    = NDU_UTOPIA_RT_EXTRACT_TYPE(rth);

        if (type == NDU_UTOPIA_RT_TYPE_NORMAL_MESSAGE)  {
          nvp         =  &(cxt->nduMsg);
          nvp->rxType =  NDU_DPR_NORMAL_MSG;
          aal5        =  cxt->aal5Normal;
        }
        else if (type == NDU_UTOPIA_RT_TYPE_LOW_MSG)  {
          nvp         =  &(cxt->nduMsg);
          nvp->rxType =  NDU_DPR_LOW_MSG;
          aal5        =  cxt->aal5Low;
        }
        else if (type == NDU_UTOPIA_RT_TYPE_HIGH_MSG)  {
          nvp         =  &(cxt->nduMsg);
          nvp->rxType =  NDU_DPR_SIGNAL_MSG;
          aal5        =  cxt->aal5High;
        } 
        else if (type == NDU_UTOPIA_RT_TYPE_ANNOUNCE)  {
          nvp         =  &(cxt->nduBlk);
        }
        break;

      case 54: 
        /* VC Demux used - routing information is in the atm header */
        vcd = (nduVcDemuxHeader_t *)&cell[p];
        channel = NDU_UTOPIA_VC_EXTRACT_VCI_CHANNEL(vcd);

        if (channel == NDU_UTOPIA_VC_MSG_CHANNEL) {
          nvp         =  &(cxt->nduMsg);
          nvp->rxType =  NDU_DPR_NORMAL_MSG;
          aal5        =  cxt->aal5Normal;
        }
        else if (channel == NDU_UTOPIA_VC_MSG_HIGH_CHANNEL)  {
          nvp         =  &(cxt->nduMsg);
          nvp->rxType =  NDU_DPR_SIGNAL_MSG;
          aal5        =  cxt->aal5High;
        }
        else if (channel == NDU_UTOPIA_VC_MSG_LOW_CHANNEL)  {
          nvp         =  &(cxt->nduMsg);
          nvp->rxType =  NDU_DPR_LOW_MSG;
          aal5        =  cxt->aal5Low;
        }
        else if (channel == NDU_UTOPIA_VC_ANU_CHANNEL)  {
          nvp         = &(cxt->nduBlk);
        }
        else 
          channel -= 5;  /* Index channels from 1 */

        break;

      /* No default needed - cell size must be verified before utopia starts */
    } /* end switch (cellSize) */

    /* Format the cell for shipout */
    pkt_p          = (tuint *) &cell[p];
    pktOut.pktSize = (tint *)  &cellSizeBytes;
    pktOut.pktIn   = (void **) &pkt_p;
    pktOut.npkts   = 1;
   
    /* Route message/packets/announcements */     
    if (nvp)  { /* If non-NULL then the cell is a message cell */
      /* Pass the entire cell to the message handler */
      nvp->vcIdRx = channel;

      /* Setup the correct aal5 instance */
      if (aal5)  {
        neuAal5Ctrl.code       = neu_CTRL_SET_AAL5;
        neuAal5Ctrl.u.aal5Inst = aal5;
        (*NDU_utContext.pNvneuControl)(cxt->nduMsg.neuInst, (neuCtrl_t *)&neuAal5Ctrl);
      }
      
      /* Currently callouts are always to neu. The call table was already
       * verified when ndu was initialized */
      (*NDU_utContext.pNvneuReceiveIn) (nvp->neuInst, &pktOut);

      /* Save the aal5 instance */
      if (aal5)  {
        neuAal5Ctrl.code  = neu_CTRL_GET_AAL5;
        neuAal5Ctrl.u.aal5Inst = aal5;
        (*NDU_utContext.pNvneuControl)(cxt->nduMsg.neuInst, (neuCtrl_t *)&neuAal5Ctrl);
      }

    }
  else { 
     /* The cell is not a message or an announcement */  
      /* Extract siu instance from channel number. Channel # indexed from 1*/
     if ((channel <= 0) || 
         (channel > (GG_MAX_NUM_CHANNELS + GG_MAX_AUX_PCM_CHANNELS + GG_MAX_PKT2PKT_CHANNELS))) {
        NDU_utContext.nduStats.RxErrChnum += 1;
        break;
      }  

/*****************************************************************************
* Perform even/odd channel translation for vpbx external utopia loopback 
******************************************************************************/
#ifdef NDU_DBG4
#ifndef NDU_DBG6
        if (channel & 0x0001)
          channel += 1;
        else
          channel -= 1;
#endif          
#endif

      /* Route the cell */
     ret =  (*NDU_utContext.pProcessData)(channel, &pktOut);
     if (ret == NDU_ERROR) {
       NDU_utContext.nduStats.RxErrState += 1;
       break;
     } 
       
    } /* end else - non-message handler */         

    /* Advance to the next cell */
    sizeBytes -= NDU_utContext.cellSize;
    p         += cellSizeWords;
  } /* end while (size >=nduContext.cellSize) */
        
} /* NDU_utopiaReceive */

/*****************************************************************************
 * FUNCTION PURPOSE: Open an NEU instance 
 *****************************************************************************
 * DESCRIPTION: Opens the NEU instance with the information provided in the
 *              boot/application handoff.
 *****************************************************************************/
void utndu_open_neu (void *neuInst, diagApp_t *diagApp, diagAppHeader_t *hdr, 
                     void *tInst, tint (*neuOut)(void *, xferPktInfo_t *),NDU_config_t *config)
{
  neuCfg_t   cfg;
  neuCtrl_t  ctrl;
  tint       shift;
  tuint      val,count;
  tint       tagOffset;
  tint       flen;

  /* Copy the cell size from the NDU context to the NEU context. */
  neuContext.cellSize = NDU_utContext.cellSize;

  /* Provide function pointer for outgoing data */
  cfg.neuSendOut.neuSendOut = neuOut;
  cfg.neuSendOut.targetInst = tInst;

  cfg.parm.chnl.enable = TRUE;  /* Enable NEU */

  /* Flags. Bit exactness of the desired flags was checked by the compiler
   * at the top of the function. */
  cfg.parm.chnl.cfg_flag1 = hdr->flags1;
  cfg.parm.chnl.cfg_flag2 = hdr->flags2;
  cfg.parm.chnl.sar_type  = neuproto_SAR_AAL5;

  /* Alternate ports unused */
  cfg.parm.chnl.alt_local_port  = 0;
  cfg.parm.chnl.alt_remote_port = 0;

  count = 0;
  while(count < MGB_IP_ADDR_MAX_NUM_WORDS)
  {
      cfg.parm.chnl.alt_remote_addr[count] = 0;
      count++;
  }

  /* report control must be disabled */
  cfg.parm.chnl.report_ctrl          = NET_ENCAP_ENCAP_REPORT_DISCARD;
  cfg.parm.chnl.report_throttle_time = 0;
  cfg.parm.chnl.sysInst              = NULL;

  /* No second protocol allowed */
  cfg.parm.chnl.second_prototype = NET_ENCAP_NULL_PROTOCOL;

  /* This next parameter might need to be updated with the boot config cell changes */
  cfg.parm.chnl.rx_offset_bytes    = hdr->rxOffsetBytes;
  cfg.parm.chnl.rx_payload_type = hdr->rxPayloadType;
  cfg.parm.chnl.tx_min_cell_bytes  = hdr->txMinCellBytes;

  /* Tx parameters */
  if (hdr->ipOffsetBytes > 0)
    cfg.parm.chnl.tx_ip_offset_bytes    = (hdr->ipOffsetBytes);
  else
    cfg.parm.chnl.tx_ip_offset_bytes    = -1;

  if (hdr->udpOffsetBytes > 0)
    cfg.parm.chnl.tx_udp_offset_bytes   = (hdr->udpOffsetBytes);
  else
    cfg.parm.chnl.tx_udp_offset_bytes   = -1;

  if (hdr->ethOffsetBytes > 0)
    cfg.parm.chnl.tx_802_3_offset_bytes = (hdr->ethOffsetBytes);
  else
    cfg.parm.chnl.tx_802_3_offset_bytes = -1;

  if (cfg.parm.chnl.rx_payload_type == DIAGAPP_PROTO_ETHERNET)
   cfg.parm.chnl.tx_min_cell_bytes   = DIAG_APP_MIN_ETHERNET_PACKET_SIZE_BYTES;
  else
    cfg.parm.chnl.tx_min_cell_bytes   = 0;

  /* For the utopia NDU with MII the core ID is placed into the UDP source address
   * as defined in the ggswcfg.h */
  if ( (cfg.parm.chnl.tx_udp_offset_bytes >= 0) && (ggConfig.neuUdpSrcMask) )  {
    val = ggConfig.neuUdpSrcMask;
    for (shift = 0; !(val & 0x1); shift++, val >>= 1);
    cfg.parm.chnl.tx_udp_src_mask = ggConfig.neuUdpSrcMask;
    cfg.parm.chnl.tx_udp_src_val  = NDU_utContext.coreId << shift;

  } else {
    cfg.parm.chnl.tx_udp_src_mask = 0;
    cfg.parm.chnl.tx_udp_src_val  = 0;
  }

  /* Configure the priority tag. The values in the priority tag are 
   * overlayed with the alt_remote_addr_lsw field */
  if (hdr->flags1 & (neuproto_FLAGS1_PDT_RX_CHK | neuproto_FLAGS1_PDT_TX_CHK)) {
    memset(&cfg.parm.chnl.alt_remote_addr[0], 0, MGB_IP_ADDR_MAX_NUM_WORDS * sizeof(tuint));
    cfg.parm.chnl.alt_remote_addr[1] = hdr->priDiffTag;
  }

  /* Transmit header */
  cfg.parm.chnl.tx_total_hdr_len_bytes = hdr->totalHeaderLenBytes;
  memcpy (cfg.parm.chnl.header, hdr->hdrImage, UTL_BYTES_TO_WORDS(hdr->totalHeaderLenBytes));

  /* The core ID field is added into the header. The size of the dsp field is hardcoded to 3 */
  if (cfg.parm.chnl.cfg_flag1 & neuproto_FLAGS1_PDT_TX_CHK)    {
    tagOffset = (hdr->totalHeaderLenBytes - (tint)neu_PDT_FIELD_TAG_LENGTH(cfg.parm.chnl.alt_remote_addr));
    flen = (tint) neu_PDT_FIELD_LEN(cfg.parm.chnl.alt_remote_addr);
    utlCopyBits ((word *)&siuContext.coreId, (word *)cfg.parm.chnl.header, TYP_WORD_SIZE - 3, 
                  (tint) (neu_PDT_FIELD_BYTE_OFFSET(cfg.parm.chnl.alt_remote_addr) * 8) +
                          (8 - neu_PDT_FIELD_BIT_OFFSET(cfg.parm.chnl.alt_remote_addr)) +
                          (tagOffset * 8) - flen - 3,
                   3);
  } 

   /* only in config as required once */
   (*config->pNvneuOpen) (neuInst, &cfg);

  /* The parameters must be re-passed to enable sending of cells */
  ctrl.code   = neu_CTRL_CHANNEL_CFG;
  ctrl.u.parm = cfg.parm;
  (*NDU_utContext.pNvneuControl) (neuInst, &ctrl);
} /* utndu_open_neu */

/*****************************************************************************
 * FUNCTION PURPOSE: Pass formatted cells to the HAL Utopia driver
 *****************************************************************************
 * DESCRIPTION: Receives segmented and formatted cells. Each of the cells
 *              will have its top level header modified 
 *****************************************************************************/
tint utndu_msg_send (void *inst, xferPktInfo_t *pkt)
{
  NduNvInst_t           *nvInst = (NduNvInst_t *)inst;
  nduRouteTagHeader_t  *rth;
  nduVcDemuxHeader_t   *vcd;
  NDU_utContext_t       *cxt;
  halUtopiaData_t        data;
  tint        cellSizeWords;
  tint        pktSizeBytes;
  tint        pktSizeWords;
  tint        rtype;
  tint        vtype;
  word      *p;
  tuint       i;


  /* Shorthand to the ndu context */
  cxt = (NDU_utContext_t *)&NDU_utContext;

  /* Both route tag and vc demux mode (determined by cell size) have 
   * pre-defined fields which determine the type of cells being sent.  */
  switch (pkt->type)  {

    case XFER_PKT_TYPE_MSG_HIGH:
      rtype = NDU_UTOPIA_RT_TYPE_HIGH_MSG;
      vtype = NDU_UTOPIA_VC_MSG_HIGH_CHANNEL;
      NDU_utContext.nduStats.TxSigMsgCnt++;
      break;

    case XFER_PKT_TYPE_MSG_NORMAL:
      rtype = NDU_UTOPIA_RT_TYPE_NORMAL_MESSAGE;
      vtype = NDU_UTOPIA_VC_MSG_CHANNEL;
      NDU_utContext.nduStats.TxCtrlMsgCnt++;
      break;

    case XFER_PKT_TYPE_MSG_LOW:
      rtype = NDU_UTOPIA_RT_TYPE_LOW_MSG;
      vtype = NDU_UTOPIA_VC_MSG_LOW_CHANNEL;
      NDU_utContext.nduStats.TxLowMsgCnt++;
      break;

    default:   /* Invalid non-voice channel */
      NDU_utContext.nduStats.TxInvldNvCh += 1;
      return (NDU_ERROR);

  }
      
   /* The cells are grouped into a single "block" */
   p = (word *)(pkt->pktIn[0]);
   
   /* Modify the header for each cell */
   cellSizeWords = UTL_BYTES_TO_WORDS(cxt->cellSize);
   pktSizeWords  = UTL_BYTES_TO_WORDS(pkt->pktSize[0]);
   for (i = 0; i < pktSizeWords; i += cellSizeWords)  {

     /* The cell headers are modified based on the cell mode (route tag or vc demux)
      * which is determined by the cell size */
     switch (cxt->cellSize)  {

       case 58:
         rth = (nduRouteTagHeader_t *)&p[i];
         /* Put the route tag type into the route tag field */
         NDU_UTOPIA_RT_SET_TYPE(rth, rtype);
         /* Set the channel number */
         NDU_UTOPIA_RT_SET_CHANNEL(rth, nvInst->vcIdTx);
         /* Add DSP ID and core */
         NDU_UTOPIA_RT_SET_CORE_ID(rth, NDU_utContext.coreId);
         break;

       case 54:
         vcd = (nduVcDemuxHeader_t *)&p[i];
         /* The channel field of the message type is appended with the core ID
          * and the message priority */
         NDU_UTOPIA_VC_SET_VCI_CHANNEL(vcd, vtype);
         /* Add the core Id */
         NDU_UTOPIA_VC_SET_VCI_CORE(vcd, NDU_utContext.coreId);
         break;

      /* No default required. Cell size was verified at ndu init */

    } /* end switch */
  } /* end for */

  /* Send the group of cells the the HAL utopia driver */
  pktSizeBytes = pkt->pktSize[0];
  data.nSegs   = 1;
  data.type    = 0; /* Check for HAL type define here */
  data.sizes   = (tuint *)&pktSizeBytes;
  data.segs    = (tuint **)pkt->pktIn;
  if (((*NDU_utContext.pAal5SendCell) (pkt->srcPort, 0, &data)) == HAL_PKT_TX_PKT_OVERFLOW)
    NDU_utContext.nduStats.TxErrDrop += 1;

  return (NDU_NOERR);

} /* utndu_msg_send */

/*****************************************************************************
 * FUNCTION PURPOSE: Setup the NEU rx callout
 *****************************************************************************
 * DESCRIPTION: Sets up callout for the non voice channel NEU for completed
 *              data.
 *****************************************************************************/
void utndu_rxcall_neu (void *inst, tint (*fcn)(void *, xferPktInfo_t *), void *rcvInst)
{
  neuCtrl_t cfg;

  cfg.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
  cfg.u.neuReceiveOutSpec.neuReceiveOut = fcn;
  cfg.u.neuReceiveOutSpec.targetInst    = rcvInst;

  (*NDU_utContext.pNvneuControl) (inst, &cfg);

} /* utndu_rxcall_neu */

/****************************************************************************
 * FUNCTION PURPOSE: Format and queue a received message
 ****************************************************************************
 * DESCRIPTION: The function is called with a message that has been
 *              processed through NEU. The standard message header is
 *              created and the message is placed into the appropriate
 *              queue.
 ****************************************************************************/
tint utndu_rcv_message (void *inst, xferPktInfo_t *pkt)
{
  nduRouteTagMsgPayload_t *rtmsg;
  nduVcDemuxMsgPayload_t  *vcmsg;
  NduNvInst_t      *nvInst = (NduNvInst_t *)inst;
  NDU_fifo_t       *msgFifo;                       
  NDU_utContext_t  *cxt;
  NduStats_t    *stats;
  mgbHeader_t       header;
  globData_t        data;
  tuint            *rxMsgDropPtr;
  tuint            *rxMsgCntPtr;
  tuint            *body;

  /* Shorthand to the NDU context and stats */
  cxt   = &NDU_utContext;
  stats = &NDU_utContext.nduStats;

  /* Setup FIFO and statistics pointers based on the message priority */
  switch (nvInst->rxType)  {
   
   case NDU_DPR_SIGNAL_MSG:
     msgFifo      = &(cxt->msgFifo[NDU_INTERRUPT_MESSAGE]);
     rxMsgDropPtr = &(stats->RxSigMsgDrop);
     rxMsgCntPtr  = &(stats->RxSigMsgCnt);
     break;

   case NDU_DPR_NORMAL_MSG:
     msgFifo      = &(cxt->msgFifo[NDU_NORMAL_MESSAGE]);
     rxMsgDropPtr = &(stats->RxCtrlMsgDrop);
     rxMsgCntPtr  = &(stats->RxCtrlMsgCnt);
     break;

   case NDU_DPR_LOW_MSG:
     msgFifo      = &(cxt->msgFifo[NDU_LOW_MESSAGE]);
     rxMsgDropPtr = &(stats->RxLowMsgDrop);
     rxMsgCntPtr  = &(stats->RxLowMsgCnt);
     break;

   default: /* We got a bad non voice channel ID. */
     stats->RxInvldNvCh += 1;
     return (NDU_ERROR);

  }

  /* The format of arriving messages varies by cell size, which defines
   * the transfer mode (route tag vs vc demux) */
  switch (cxt->cellSize)  {

    case 58:   /* Route Tag mode */
      rtmsg           = (nduRouteTagMsgPayload_t *)(pkt->pktIn[0]);
      header.length   = UTL_BYTES_TO_TINT(pkt->pktSize[0]) + MGB_HEADER_SIZE - NDU_ROUTE_TAG_MSG_HEADER_LENGTH;
      header.chnum    = nvInst->vcIdRx;
      header.msg_id   = rtmsg->msg_id;
      header.trans_id = rtmsg->trans_id;
      body            = &(rtmsg->msg_first_word);
      break;

    case 54:   /* VC demux */
      vcmsg = (nduVcDemuxMsgPayload_t *)(pkt->pktIn[0]);
      header.length   = UTL_BYTES_TO_TINT(pkt->pktSize[0]) + MGB_HEADER_SIZE - NDU_VC_DEMUX_MSG_HEADER_LENGTH;
      header.chnum    = NDU_UTOPIA_VC_DEMUX_MSG_EXTRACT_CHANNEL(vcmsg);
      header.msg_id   = vcmsg->msg_id;
      header.trans_id = vcmsg->trans_id;
      body            = &(vcmsg->msg_first_word);
      break;

    /* No default required - cell size has been verified in ndu initialization */

  }

  /* Change the channel number if all Channel Broadcast Message */
  if (header.chnum == NDU_UTOPIA_MAX_CHANNEL_ID) 
    header.chnum = MGB_ALL_CHANNEL_BROADCAST_ID;
  
        /* Put the message into the appropriate fifo */
  if (msgFifo->free < header.length) {
    *rxMsgDropPtr += 1;
    return (NDU_ERROR);
  }

        /* Put the message header in the Fifo. */
  utilFifoWrite(msgFifo->start, (tuint *)&header, &msgFifo->inIndex, 
                          &msgFifo->outIndex,msgFifo->fifolen, MGB_HEADER_SIZE);

  if (header.length > MGB_HEADER_SIZE)
     utilFifoWrite(msgFifo->start, body, &msgFifo->inIndex, &msgFifo->outIndex,
           msgFifo->fifolen, header.length - MGB_HEADER_SIZE);

        msgFifo->free -= header.length;

        /* The message is successfully in the fifo */
        *rxMsgCntPtr +=1 ;

  return (NDU_NOERR);

} /* utndu_rcv_message */

/****************************************************************************
 * FUNCTION PURPOSE: Allocate and initialize message aal5 instances
 ****************************************************************************
 * DESCRIPTION: All message priorities share a common NEU instance, but they
 *              each maintain a seperate AAL5 so that cells from different
 *              priority messages can be received interleaved.
 ****************************************************************************/
void utndu_create_msg_aal5(void)
{
  NDU_utContext_t     *cxt;
  neuCtrl_t            neuCtrl;
  
  /* Reference to the system ndu context */
  cxt =&(NDU_utContext);

  /* Get the size required for an aal5 instance */
  neuCtrl.code = neu_CTRL_GET_AAL5_SIZE;
  (*NDU_utContext.pNvneuControl) (cxt->nduMsg.neuInst, &neuCtrl);

  /* Verify size requirements */
  /* can not send any message till ndu is setup */
 /*  siu_exc_assert (neuCtrl.u.aal5SizeReq > ggutNduAal5Desc.size, SIUNDU_AAL5_SIZE, &siuContext); */

  /* Assign the buffers */
  cxt->aal5High   = (void *)ggutNduAal5Desc.highAddr;
  cxt->aal5Normal = (void *)ggutNduAal5Desc.normalAddr;
  cxt->aal5Low    = (void *)ggutNduAal5Desc.lowAddr;

  /* Initialize the buffers by copying the current aal5 instance to all three */
  neuCtrl.code = neu_CTRL_GET_AAL5;

  neuCtrl.u.aal5Inst = cxt->aal5High;
  (*NDU_utContext.pNvneuControl)(cxt->nduMsg.neuInst, &neuCtrl);

  neuCtrl.u.aal5Inst = cxt->aal5Normal;
  (*NDU_utContext.pNvneuControl)(cxt->nduMsg.neuInst, &neuCtrl);

  neuCtrl.u.aal5Inst = cxt->aal5Low;
  (*NDU_utContext.pNvneuControl)(cxt->nduMsg.neuInst, &neuCtrl);
  
} /* utndu_create_msg_aal5 */

/******************************************************************************
 * FUNCTION PURPOSE: Send cells to the network.
 ******************************************************************************
 * DESCRIPTION: Formatted cells (with route tags and/or ATM headers are
 *              passed to the hal layer to be sent to the network
 *****************************************************************************/
tint NDU_utSendMsg(mgb_t *pMsg, NDU_DPRMsgType_t mtype, bool fDrop)
{

 /* In Loopback do not send messages, or they will be looped back   */
#if(defined(NDU_DBG4) || defined(NDU_BLOCK_SEND_MSGS))
  return (0);
#else

  mgb_t *msg = (mgb_t *)pMsg;
  xferPktInfo_t     pkt;
  void             *data;
  NDU_utContext_t  *cxt;
  NduNvInst_t      *nvInst;
  tint              pktSizeBytes;

  /* Shorthand to the ndu Context */
  cxt = &(NDU_utContext);
  nvInst = &(cxt->nduMsg);

  /* Modify Channel ID if Broadcast Message */
  if (msg->header.chnum == MGB_ALL_CHANNEL_BROADCAST_ID)
    msg->header.chnum = NDU_UTOPIA_MAX_CHANNEL_ID;

  /* Use the message "channel" corresponding to the priority */
  switch (mtype)  {

    case NDU_DPR_SIGNAL_MSG:
      pkt.type  =  XFER_PKT_TYPE_MSG_HIGH;
      break;

    case NDU_DPR_NORMAL_MSG:
      pkt.type =  XFER_PKT_TYPE_MSG_NORMAL;
      break;

    case NDU_DPR_LOW_MSG:
      pkt.type =  XFER_PKT_TYPE_MSG_LOW;
      break;

    /* No default required */

  } /* end switch */

  /* Interrupts must be disabled prior to modifying the non-voice instance.
   * This is required since messages can be generated from both a foreground
   * and interrupt context. Interrupt protection is required for both the
   * non-voice instances and the neu tx scratch buffer */
  NDU_STATE_BEGIN();

  /* Store the channel number. In route tag mode this information is lost
   * in the segmentation process and must be post-fed into the header after
   * segmentation */
  nvInst->vcIdTx = msg->header.chnum;

  /* The format of the message header varies by mode (route tag vs vc demux)
   * which is determined by the cell size */

  switch (cxt->cellSize) {

    case 58:
      data = (void *)&msg->header.msg_id; 
      pktSizeBytes = 
        UTL_TINTS_TO_BYTE(msg->header.length - MGB_HEADER_SIZE + 
                           NDU_ROUTE_TAG_MSG_HEADER_LENGTH);
      break;

    /* In VC Demux mode, the message header must be changed to the
     * vc demux message header. The VCI channel may have to be modfied as well*/
    case 54:
      /* The packet size must be computed before modifiing the header */
      pktSizeBytes = UTL_TINTS_TO_BYTE (msg->header.length - MGB_HEADER_SIZE +
                                                NDU_VC_DEMUX_MSG_HEADER_LENGTH);
      data = ndu_ut_modify_vc_msg_header (mtype, msg);
      break;

    /* No default case is required. The cell size was verified valid in the 
     * ndu initialization */

  } /* end switch */


  /* Pass the formatted message to NEU. Once segmented it will be returned
   * to utndu_msg_send. The call table was already verified when NDU was
   * initialized */
  pkt.srcPort = 0;
  pkt.npkts   = 1;
  pkt.pktSize = &pktSizeBytes;
  pkt.pktIn   = &data;
  
     (*NDU_utContext.pNvneuSendIn)(nvInst->neuInst, &pkt);
  
  NDU_STATE_END();

  /* Nonzero return value is success, and by legacy is msg number */
  return (msg->header.msg_id ? msg->header.msg_id : 0xffff);
#endif
  
} /* Ndu_utsend_message */
/*****************************************************************************
 * FUNCTION PURPOSE: Format a VC Demux header
 *****************************************************************************
 * DESCRIPTION: Re-formats a message header into vc demux format. The length
 *              field is lost.
 *****************************************************************************/
void *ndu_ut_modify_vc_msg_header (NDU_DPRMsgType_t mtype, mgb_t *msg)
{
  nduVcDemuxMsgPayload_t *vcd;
  tuint ttype, ch;

  /* Save the channel number before modifiing the memory in place */
  ch = msg->header.chnum;
  vcd = (nduVcDemuxMsgPayload_t *)&msg->header.length;

  /* The length field of the message type is replaced by the DSP ID */ 
  vcd->dsp_id = (NDU_utContext.dspId+NDU_utContext.coreId) ;

  /* Translate the proirity into route tag type field */
  switch (mtype)  {

    case NDU_DPR_SIGNAL_MSG:
      ttype = NDU_UTOPIA_VC_DEMUX_MSG_PRIORITY_HIGH;
      break;
    case NDU_DPR_NORMAL_MSG:
      ttype = NDU_UTOPIA_VC_DEMUX_MSG_PRIORITY_NORMAL;
      break;
    case NDU_DPR_LOW_MSG:
      ttype = NDU_UTOPIA_VC_DEMUX_MSG_PRIORITY_LOW;
      break;
  } /* end switch */
  
  /* The channel field of the message type is appended with the core ID
   * and the message priority */
  NDU_UTOPIA_VC_DEMUX_MSG_SET_PRIORITY(vcd, ttype);
  NDU_UTOPIA_VC_DEMUX_MSG_SET_COREID(vcd, NDU_utContext.coreId);
  NDU_UTOPIA_VC_DEMUX_MSG_SET_CHANNEL (vcd, ch);

  /* Return a pointer the vc demux message payload */
  return ((void *) vcd);

} /* ndu_ut_modify_vc_msg_header */

/*****************************************************************************
 * FUNCTION PURPOSE: send data outside of NDU ********************************
 *****************************************************************************/
tint NDU_utSendData(tuint extID, xferPktInfo_t *pPktInfo)
{
    halUtopiaData_t haldata;
    tint ret = 0; /* Return value */
    
   /* Translate the data into hal format */
   haldata.nSegs = (tuint)   pPktInfo->npkts;
   haldata.segs  = (tuint **)pPktInfo->pktIn;
   haldata.sizes = (tuint *) pPktInfo->pktSize;
   haldata.type  = 0; /* types don't match */

#if (defined(NDU_DBG4) && defined(NDU_DBG6))
   /* Send the cell back. Channel crossing handled in receive in */
   NDU_utopiaReceive (0, (word *)*haldata.segs, *haldata.sizes); 
#else
   /* Send the Cell. Existance of the function call was verified in nduControl */
   if (((*NDU_utContext.pAal5SendCell)(pPktInfo->srcPort, 0, &haldata)) == HAL_PKT_TX_PKT_OVERFLOW)
    {
    NDU_utContext.nduStats.TxErrDrop += 1;
    ret = -1; /* Cell is not sent */
   }
#endif
   return (ret);
} /* NDU_utSendData */

/*****************************************************************************
 * FUNCTION PURPOSE: Poll for packets.
 *****************************************************************************
 * DESCRIPTION: Function called from the application to poll for packets.
 *              For the utopia interface this is unused, since arriving cells
 *              are serviced through an interrupt. A dummy function is
 *              required to populate the call table
 *****************************************************************************/
#ifdef NDU_DBG4
tint anuTestCell = 0;
void NDU_loopReceiveIn (void)
{
  /* Mechanism for testing announcemt data passthrough. Only valid for 
   * channel 1 in route tag mode */
  tuint cellData[29] = { 0x2140, 0x0001, 0x0000, 0x0000, 0x0000,
                         0x1234, 0x1234, 0x1234, 0x1234, 0x1234,
                         0x1234, 0x1234, 0x1234, 0x1234, 0x1234,
                         0x1234, 0x1234, 0x1234, 0x1234, 0x1234,
                         0x1234, 0x1234, 0x1234, 0x1234, 0x1234,
                         0x1234, 0x1234, 0x1234, 0x1234  };
          
  if (!anuTestCell)
    return;


  if (anuTestCell == 1)  {
    /* Announcement data header */
    cellData[5] = 0x0041;
    cellData[6] = 0x203C;
    cellData[7] = 0xAAAA;
    cellData[8] = 0xAAAA;
  }
  else if (anuTestCell == 6)  {
    cellData[3] = 0x0002;  /* ATM aal5 termination indication */
    cellData[25] = 0;
    cellData[26] = 0x00F8; /* Data length = 240 bytes data + 8 bytes header */
  }

  /* Send the cell to the system */
  NDU_utopiaReceive(0, (word *)cellData, 58);

  anuTestCell += 1;
  if (anuTestCell >= 8)
    anuTestCell = 0;
  
  /* empty */
} /* NDU_loopReceiveIn */
#endif

void NDU_utMsgBg(void)
{
    #if (defined(NDU_DBG4) || defined(NDU_DBG5))
  if (msgTest_flag) {
   (*NDU_utContext.pProcessMsg)(NDU_NORMAL_MESSAGE,&msgTest);
   msgTest_flag = FALSE;
 }
  if (sigTest_flag) {
   (*NDU_utContext.pProcessMsg)(NDU_INTERRUPT_MESSAGE,&sigTest);
   sigTest_flag = FALSE;
 }
 #endif
}

void NDU_utMainTask(tuint timeStamp)
{
    #ifdef NDU_DBG4
 NDU_loopReceiveIn();
    #endif /* NDU_DBG4 */

 NDU_utGetMsg(NDU_INTERRUPT_MESSAGE);
 NDU_utGetMsg(NDU_NORMAL_MESSAGE);
 NDU_utMsgBg();
}
/*******************************************************************************
 * FUNCTION PURPOSE: ndu query
 *******************************************************************************
 * DESCRIPTION: Checks NDU if priority is supported or Fifo is free
 ******************************************************************************/

int NDU_utControl(NDU_ctrlcode_e c_code,NDU_control_t *ctl)
{
    switch (c_code) {
  
        case NDU_QUERY_PRIORITY_SUPPORT:
            ctl->out_params.priority_supported = TRUE;
            break; 
            
        case NDU_QUERY_FIFOFREE:  
        {
            halUtopiaControl_t utctrl;
            utctrl.mode = HAL_UTOPIA_GET_FREE_CELLS;
            NDU_utContext.phalUtopiaControl (NDU_utContext.ut_portid, &utctrl);
            ctl->out_params.fifo_free = utctrl.u.freeCells;
      }
      break;

        default:
            return NDU_ERROR; /* Unknown control code */
    }   
    return NDU_NOERR;
}

/*******************************************************************************
 * FUNCTION PURPOSE: Initialize the keep alive neu instance
 *******************************************************************************
 * DESCRIPTION: A non-voice NEU instance is created. Keep alive 
 ******************************************************************************/
void nduUtHealthInit (void *ai,NDU_config_t *config)
{
  diagApp_t            *diagApp = (diagApp_t *)ai;
  nduRouteTagHeader_t *rth;
  nduVcDemuxHeader_t  *vcd;
  NDU_utContext_t      *cxt;
  diagAppHeader_t       kHeader;
  tint                  hLenWords;

  /* Reference to the system ndu context */
  cxt = &(NDU_utContext);

  /* Initialize control values to setup NEU */
  kHeader.flags1 = 0;  /* Flags 1 handled only udp and ip which are unused for keep alive */
  kHeader.flags2 = diagApp->utopia.msg.flags1 & DIAGAPP_F2_AAL5_GEN_CRC_BIT;
   
  kHeader.rxPayloadType  = NET_ENCAP_PROTOCOL_NULL;
  kHeader.ipOffsetBytes  = -1;
  kHeader.udpOffsetBytes = -1;
  kHeader.ethOffsetBytes = -1;
  kHeader.priDiffTag     =  0;
  kHeader.rxOffsetBytes  =  0;
  kHeader.txMinCellBytes =  0;

  /* Keep alive cells are aal5 encoded. The format of the cell header is
   * determined from the cell length */
  switch (diagApp->utopia.cellSize)  {

    case 58:

      /* Copy the input message header to pickup default values */
      hLenWords = UTL_BYTES_TO_WORDS(neuproto_ROUTE_TAG_SIZE_BYTES+ neuproto_ATM_HDR_SIZE_BYTES);
      memcpy (kHeader.hdrImage, diagApp->utopia.msg.hdrImage, hLenWords);
      /* Set the route tag type, destination phy, and channel */
      rth = (nduRouteTagHeader_t *)&kHeader.hdrImage;
      NDU_UTOPIA_RT_SET_TYPE     (rth, NDU_UTOPIA_RT_TYPE_HEALTH);
      NDU_UTOPIA_RT_SET_DEST_PHY (rth, NDU_UTOPIA_RT_DEST_PHY_MISC);
      NDU_UTOPIA_RT_SET_CHANNEL  (rth, 0);
      break;

    case 54:

      /* Copy the input message header to pick up default header values */
      hLenWords = UTL_BYTES_TO_WORDS(neuproto_ATM_HDR_SIZE_BYTES);
      memcpy (kHeader.hdrImage, diagApp->utopia.msg.hdrImage, hLenWords);
      /* Set the GFC to indicate this is a keep alive cell */
      vcd = (nduVcDemuxHeader_t *)&kHeader.hdrImage;
      vcd->gfc_vpi_vci = diagApp->kpAlive_gfc_vp_vcih;
      vcd->vci_core_channel_pt = diagApp->kpAlive_vcil_pt_clp;
      break;

  }

  /* Create the keep alive NEU instance */
  kHeader.totalHeaderLenBytes = UTL_WORDS_TO_BYTES(hLenWords);
  cxt->nduHlt.neuInst = (*config->pNvNeuCreate) (GG_NVECO_HLT, hLenWords);
  utndu_open_neu (cxt->nduHlt.neuInst, diagApp, &kHeader, &(cxt->nduHlt), nduUtSendInCore,config);

} /* nduUtHealthInit */

/*******************************************************************************
 * FUNCTION PURPOSE: Send a keep alive cell
 *******************************************************************************
 * DESCRIPTION: Creates and sends a keep alive cell to the utopia.
 ******************************************************************************/
void ndu_UtHealthSend (tint seqnum)
{
  NduNvInst_t      *nvInst = (NduNvInst_t *)(&(NDU_utContext.nduHlt));
  NDU_utContext_t  *cxt;
  void             *data;
  xferPktInfo_t     pkt;
  tint              sizeBytes;

  union {
    ka_payload_rt_t rt;
    ka_payload_vc_t vc;
  } payload;

  
  /* Reference to the system ndu context */
  cxt =&(NDU_utContext);

  /* The format of the keep alive body depends on the cell size */

  switch (cxt->cellSize)  {

    case 58:
      payload.rt.seqNum    = seqnum;
      payload.rt.chip_core = 0;
      sizeBytes            = UTL_WORDS_TO_BYTES(sizeof(ka_payload_rt_t));
      data                 = (void *)&(payload.rt);
      NDU_KEEP_ALIVE_SET_CHIP_ID (&payload.rt, NDU_utContext.dspId);
      NDU_KEEP_ALIVE_SET_CORE_ID (&payload.rt, NDU_utContext.coreId);
      break;

    case 54:
      payload.vc.transId      = seqnum;
      payload.vc.opCode_count = NDU_KEEP_ALIVE_VC_OP_COUNT;
      sizeBytes               = UTL_WORDS_TO_BYTES(sizeof(ka_payload_vc_t));
      data                    = (void *)&(payload.vc);
      NDU_KEEP_ALIVE_SET_ADDR (&payload.vc, NDU_utContext.kpAliveAddr_hi,NDU_utContext.kpAliveAddr_lo);
      NDU_KEEP_ALIVE_SET_VAL  (&payload.vc, (1 << NDU_utContext.coreId));
      break;

  }

  /* Send the cell to neu for aal5 encapsulation. Neu will pass it to the
   * utopia send cell function in siuutndu.c */
  pkt.srcPort = 0;
  pkt.npkts   = 1;
  pkt.pktSize = &sizeBytes;
  pkt.pktIn   = &data;
  pkt.type    = XFER_PKT_TYPE_NULL;

  NDU_STATE_BEGIN();
  (*NDU_utContext.pNvneuSendIn) (nvInst->neuInst, &pkt);
  NDU_STATE_END();

} /* ndu_UtHealthSend */

/*******************************************************************************
 * FUNCTION PURPOSE: ANU request is passed to NEU
 ******************************************************************************/
tint NDU_SendAnuToNeu(tuint instID, xferPktInfo_t *pktInfo)
{
  NDU_utContext_t   *cxt;
  NduNvInst_t     *nvInst;

  /* Shorthand to the ndu context and the non-voice channel instance */
  cxt    = &(NDU_utContext);
  nvInst = &(cxt->nduBlk);

  /* Protect the global variables from interrupt re-entry */
  NDU_STATE_BEGIN();

  /* Store the associated voice channel, do not perform dynamic channel translation */
  nvInst->vcIdTx = instID & UTL_CID_MASK;

  /* Pass the announcement request through NEU. NEU will segment the request
   * into cells and call function siu_anu_rqst_send when done */
  (*NDU_utContext.pNvneuSendIn) (nvInst->neuInst, pktInfo);

  NDU_STATE_END();

  return (NDU_NOERR);

} /* NDU_SendAnuToNeu */

void NDU_FormatAnuFromNeu(void *inst, xferPktInfo_t *pkt, tint chnum)
{
  nduRouteTagHeader_t *rth;
  nduVcDemuxHeader_t  *vcd;
  NDU_utContext_t      *cxt;
  tuint  *p;
  tint    cellSizeWords;
  tint    pktSizeWords;
  tint    i;

  /* Shorthand to the ndu context */
  cxt = (NDU_utContext_t *)&NDU_utContext;

  /* Modify the header for each cell */
  cellSizeWords = UTL_BYTES_TO_WORDS(cxt->cellSize);
  pktSizeWords  = UTL_BYTES_TO_WORDS(pkt->pktSize[0]);

  /* p points to the base of the sequence of cells */
  p = (tuint *)(pkt->pktIn[0]);

  /* In route tag mode, the route tag for each cell must be modified
   * to include the channel number */
  if (cxt->cellSize == 58)  {
    for (i = 0; i < pktSizeWords; i += cellSizeWords) {
      rth = (nduRouteTagHeader_t *)&p[i];
      NDU_UTOPIA_RT_SET_CHANNEL(rth, chnum);
      NDU_UTOPIA_RT_SET_CORE_ID(rth, NDU_utContext.coreId);
    }
  } 
  else if (cxt->cellSize == 54) {
    /* For VC demux cells, only change the core Id field */
    for (i = 0; i < pktSizeWords; i += cellSizeWords)  {
      vcd = (nduVcDemuxHeader_t *)&p[i];
      NDU_UTOPIA_VC_SET_VCI_CORE(vcd, NDU_utContext.coreId);
    }
  }
 }  /* NDU_FormatAnuFromNeu */

/******************************************************************************
 * FUNCTION PURPOSE: Send formatted cells to the network.
 ******************************************************************************
 * DESCRIPTION: Formatted cells (with route tags and/or ATM headers are
 *              passed to the hal layer to be sent to the network
 *****************************************************************************/
/* to match the neu API this function created siuInst is not avialble for NV*/ 
tint nduUtSendInCore (&NDU_utCallTable,void *inst, xferPktInfo_t *pktInfo)
{
 tuint extID = 0;
  return(NDU_SendData(extID, pktInfo));
} 
#endif
 
/* Nothing past this point */




