/******************************************************************************
 * FILE PURPOSE: NDU header
 ******************************************************************************
 * FILE NAME:   ndu.h
 *
 * DESCRIPTION: defined nduInst, nduContext structure.
 *
 * TABS: NONE
 *
 * (C) Copyright 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "ndu.h"

typedef struct nduFifoDrv_s {
	tint (*space)(void *, tuint);
	tint (*check)(void *, tuint);
	tint (*read)(void *, tuint *, tuint *, tuint*, tuint *);
	tint (*send)(void *, tint, tint, tuint *);
} NDU_nduv1FifoDrv_t;

/* fifo info structure */
typedef struct fifo_info_s {
	tuint          *start_address;   /* fifo start address          */
	tuint          fifo_length;      /* fifo length                 */
	tuint          fifo_type;        /* type of fifo                */
	tuint          pkt_types;        /* allowed pkt types           */
	tuint          config;           /* config field                */
	tuint          num_ch;           /* number of channels in fifo  */
	tuint          *acs_in;          /* pointer points to acs in_idx*/
	tuint          *acs_out;         /* pointer points to acs in_idx*/
	NDU_nduv1FifoDrv_t *driver;      /* fifo function call table    */
} fifo_info_t;  

typedef struct NDU_nduv1Context_s
{
  NDU_fifo_t  msgFifo[3];
  tuint       nduPktLoopBack[4];	//SIU_LASTCH - SIU_FIRSTCH];

  tuint       dsp2mic_pkt_fifo_tbl[16]; /* look up table to tie pkt class with FIFO */
  tuint       mic2dsp_pkt_fifo_tbl[16]; /* look up table to tie pkt class with FIFO */
  tint        task_timer[3];
  void        *acs;
  tuint       hpi_alignment;
  tuint       *hpiMem;
  tuint       hpi_intr_control;
  fifo_info_t *fifo_info;

  struct MGB_DMS_GET_NETWORK_STAT     stats;
  void (*pProcessMsg)(NDU_msgPriority_e, mgb_t *);
  void (*pProcessData)(tuint, xferPktInfo_t *);
  tint (*pHalApiCtrl)(halHpiControl_t *);
  void (*pPcmPktTrace)(tuint ID, tuint dataSize, void *data, tuint streamId);
}NDU_nduv1Context_t;

tint NDU_nduv1MailBoxSpace(void *, tuint);
tint NDU_nduv1MailBoxCheck(void *, tuint);
//tint NDU_nduv1MailBoxRead(void *, tuint *, tint *, tint*, tint *);
tint NDU_nduv1MailBoxSend(void *, tint, tint, tuint *);

tint NDU_nduv1SFIFOSpace(void *, tuint);
tint NDU_nduv1SFIFOCheck(void *, tuint);
tint NDU_nduv1SFIFORead(void *, tuint *, tuint *, tuint*, tuint *);
tint NDU_nduv1SFIFOSend(void *, tint, tint, tuint *);

tint NDU_nduv1CFIFOSpace(void *, tuint);
tint NDU_nduv1CFIFOCheck(void *, tuint);
tint NDU_nduv1CFIFORead(void *, tuint *, tint *, tint*, tint *);
tint NDU_nduv1CFIFOSend(void *, tint, tint, tuint *);

tint NDU_nduv1INDEXSpace(void *, tuint);
tint NDU_nduv1INDEXCheck(void *, tuint);
tint NDU_nduv1INDEXRead(void *, tuint *, tint *, tint*, tint *);
tint NDU_nduv1INDEXSend(void *, tint, tint, tuint *);

tint NDU_nduv1MAILBOXSpace(void *, tuint);
tint NDU_nduv1MAILBOXCheck(void *, tuint);
tint NDU_nduv1MAILBOXSend(void *,tint,tint,tuint *);

/* nothing after this point */
