#ifndef _SIUPORT_H
#define _SIUPORT_H
/******************************************************************************
 * FILE PURPOSE: SIU Portability Definitions (C54x version)
 ******************************************************************************
 * FILE NAME:   siuport.h
 *
 * DESCRIPTION: Contains SIU portability definitions.  Presently this file
 *              is used to support four different hardware platforms:
 *              TIGER, MARS, ATHENSHW1 (Athens hardware configured for
 *              receiving and transmiting one channel of PCM data), and 
 *              ATHENSHW2 (Athens hardware configured for receiving and 
 *              transmiting two channels of PCM samples). ONE of the above
 *              platforms must be specified as a compiler option.
 *
 * TABS: NONE
 *
 * $Id: siuport.h 1.25 1999/12/14 21:41:59 qsu Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: siuport.h $
 * Revision 1.25  1999/12/14 21:41:59  qsu
 * - Added RSU.
 * Revision 1.24  1999/09/10 22:11:36  jdowdal
 *    AAL2 Integration
 * Revision 1.23  1999/08/15 14:56:41  BONAL
 *    - SIU_COPY_BEGIN() and SIU_COPY_END() added to disable/enable
 *      interrupts in program copying.
 * Revision 1.22  1999/05/14 18:34:27  qsu
 * - Removed FDU.
 * Revision 1.21  1999/03/12 16:04:43  qsu
 * - Added module ID for CID.
 * Revision 1.20  1998/12/05 14:27:46  BONAL
 *    - Added module ID for TTU.
 * Revision 1.19  1998/11/25 16:02:33  BONAL
 *    - Added siudbg related interrupt enable/disable.
 *      Removed MXP related function calls.
 * Revision 1.18  1998/11/15 14:16:52  BONAL
 *    - Merging of 1.17.4.1 and changes for New HPI.
 * Revision 1.17.4.1  1998/11/04 17:48:39  BONAL
 *    - RELEASE 5.0 Changes..... First Cut.....
 * Revision 1.17  1998/04/21 15:27:45  ppatel
 *  Changes for integrating new eco "fdu" (Frame Delay Unit)
 * Revision 1.16  1998/04/19 14:57:40  ppatel
 *  Changes for implementing memory swapping
 * Revision 1.15  1998/04/07 15:42:09  ppatel
 * Parameters which depend on frame size of voice are now through ggConfig
 * structure rather than through #defines in siuport.h. siustmvf.c has a patch
 * form dtmf relay which needs to be corrected later.
 * Revision 1.14  1998/02/20 19:26:28  abura
 * - Changed maximum number of channels to 4 for voice.
 * Revision 1.13  1998/02/17 11:26:00  abura
 * - Max core segments per packet added.
 * Revision 1.12  1998/02/12 15:40:25  abura
 * - VIF size increased to 20 ms of PCM.
 * Revision 1.11  1998/02/06 22:03:52  ffruth
 * - Changed maximum number of channels to 4 for voice/fax and fax.
 * Revision 1.10  1998/01/15 13:41:09  BOGDANK
 *     - NDU Rx Buffer moved from stack to siumain.c (.bss)
 *    More NDU parameters defined.
 * Revision 1.9  1998/01/13 15:14:14  BOGDANK
 *     - CW segment constants redefined, some names changed
 * Revision 1.8  1998/01/10 03:02:46  ffruth
 * - Added SIU_MAX_NBUFS.
 * Revision 1.7  1997/12/11 00:07:54  ffruth
 * -  Added defintions of minimum codecs and max channels for Voice/Fax
 *    builds.  Added critical section defintions for FM task.
 * Revision 1.6  1997/12/05 18:30:16  BOGDANK
 *     - TSU integrated
 *    SIU_MID_TSU added.
 * Revision 1.5  1997/11/12 20:33:55  ffruth
 * -  Multi-channel FAX inetgartion changes:  Increased max channels for 
 *    FAX build from 1 to 2.
 * Revision 1.4  1997/09/12 10:06:02  BOGDANK
 *     - SIU_BSP_xx_ADDR removed to ggcfg.c
 *    SIU_BSP_xx_ADDR moved to ggcfg.c.
 * Revision 1.3  1997/08/07 15:20:17  ffruth
 * - Modified critical sections for SIU_STATE_BEGIN() and SIU_STATE_END()
 *   macros.
 * Revision 1.2  1997/08/06 14:09:02  ffruth
 * - Added SIU_MAX_FAX_PACKET_SIZE, and removed SIU_FAX_CWSEGSIZE
 *   and SIU_FAX_CWLENGTH.
 * Revision 1.1  1997/08/05 21:02:47  ffruth
 * Initial revision
 *
 * (C) Copyright 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"

/* configuration & target hardware files */
/* #include "ggcfg.h" */
#include "targethw.h"

/* Dual-port ram data definitions */
#include "vbmicdsp.h"
#include "micdspdp.h"

#include "siuregsave.h"

#define SIU_BYTE_ADDRESS_SHIFT 0

#define SIU_ALIGN_BITS    3
#define SIU_ALIGN_MASK    (~((~0U)<<SIU_ALIGN_BITS))   /* 0x07 */

/* Q-format for cosine table. On a 16-bit fixed point architecture it is 14. */
#define SIU_COSINE_Q          (TYP_COSINE_SIZE-2)

/* Maximum # configurable channels and minimum # configurable codecs */
#if (defined(FAX) || defined(HDMOVE) || defined(HDSKEL) || defined(HDVIDEO))
# define  SIU_MIN_VCAS        0
#else
# define  SIU_MIN_VCAS        1
#endif
#define SIU_FIRSTCH           1         /* channel numbers start from 1 */
#define SIU_LASTCH            (SIU_FIRSTCH+ggConfig.max_channels-1)

/* Maximum number of configurable codecs */
#define SIU_MAX_VCAS    9
#if SIU_MAX_VCAS < SIU_MIN_VCAS
# error invalid range of configurable codecs
#endif


#ifdef GG_MULTSAMPVC
/* Scale sampling rate dependent elements based on rate of operation */
/* Implements division by 2 or 1 by right shifting by 1 or 0         */
#define  SIU_SCALE_SAMPRATE_PARAMS(params, scale)  (params>>scale)


/* Macros for conversion between samples to milli seconds at 8/16 KHz */
#define  SIU_CONV_SAMPLESTOMS_AT16KHZ(numsamples) ((uint)(numsamples) >> 4)
#define  SIU_CONV_SAMPLESTOMS_AT8KHZ(numsamples)  ((uint)(numsamples) >> 3)
#endif /* GG_MULTSAMPVC */

/* Macros for conversion between milli seconds to samples at 8/16 KHz */
#define  SIU_CONV_MSTOSAMPLES_AT16KHZ(timeinms)   (timeinms * 16)
#define  SIU_CONV_MSTOSAMPLES_AT8KHZ(timeinms)    (timeinms * 8)

/* NDU parameters */
/* Maximum TX Busy Buffer Counter. */
#define SIU_NDU_TXBUFFER_TIMEOUT   20
/* Rx unpack buffer must equal the number of _bytes_ in a DPR payload.
 *  Used only when packed = FALSE. */
#define SIU_NDU_DPRWORDSIZE_BYTES   (TYP_DPRWORD_SIZE / 8)
#define SIU_NDU_RXBUFSIZE   (VDB_PAYLOAD_MAXSIZE*SIU_NDU_DPRWORDSIZE_BYTES)
#define HALF_SIU_NDU_RXBUFSIZE   VDB_PAYLOAD_MAXSIZE
/* Maximum number of memory buffers for a module by ecoGetSizes() */
#define SIU_MAX_NBUFS    21   /* DPU */
#define SIU_MAX_VCA_NBUFS 16  /* Worst case is SMV
                                 Tie up with ggmbuf.h to get max vca size */
#define SIU_MAX_ADA_NBUFS 11  /* Tie up with ggmbuf.h to get max ada size */

#define SIU_BITS_PER_BYTE       8

#define SIU_DBGMSG_SIZE   256     /* Size of debug message buffer */


/* Critical sections for the SIU */
/* For control path */
#define SIU_STATE_BEGIN() (thwCriticalBegin())
#define SIU_STATE_END()   (thwCriticalEnd())

/* For data path */
#define SIU_STATE_DATA_BEGIN() (thwCriticalBeginFast())
#define SIU_STATE_DATA_END(x)  (thwCriticalEndFast(x))

#define SIU_FIFO_DATA_BEGIN()   (thwCriticalBeginFast())
#define SIU_FIFO_DATA_END(x)     (thwCriticalEndFast(x))

/* Control path */
#define SIU_TX_BEGIN()   (thwCriticalBegin())
#define SIU_TX_END()     (thwCriticalEnd())
/* Data path */
#define SIU_TX_DATA_BEGIN()   (thwCriticalBeginFast())
#define SIU_TX_DATA_END(x)    (thwCriticalEndFast(x))

#define SIU_TX_WAKEUP_BEGIN()   ((void) 0)    /* Not required, executed in ISR */
#define SIU_TX_WAKEUP_END()     ((void) 0)
#define SIU_RX_BEGIN()   (thwCriticalBegin())
#define SIU_RX_END()     (thwCriticalEnd())
#define SIU_RX_WAKEUP_BEGIN()   ((void) 0)    /* Not required, executed in ISR */
#define SIU_RX_WAKEUP_END()     ((void) 0)
#define SIU_FM_BEGIN()   (thwCriticalBegin())
#define SIU_FM_END()     (thwCriticalEnd())
#define SIU_FM_WAKEUP_BEGIN()   ((void) 0)    /* Not required, executed in ISR */
#define SIU_FM_WAKEUP_END()     ((void) 0)
#define SIU_SWAP_BEGIN()   (thwCriticalBeginFast())
#define SIU_SWAP_END(x)    (thwCriticalEndFast(x))
#define SIU_COPY_BEGIN()   (thwCriticalBeginFast())
#define SIU_COPY_END(x)    (thwCriticalEndFast(x))
#define SIU_DBGCNT_BEGIN() (thwCriticalBeginFast())
#define SIU_DBGCNT_END(x)  (thwCriticalEndFast(x))
#define SIU_PROFILE_BEGIN() (thwCriticalBegin())
#define SIU_PROFILE_END()   (thwCriticalEnd())
#define SIU_RTTRACE_BEGIN() (thwCriticalBegin())
#define SIU_RTTRACE_END()   (thwCriticalEnd())
#define SIU_RXINT_BEGIN() (thwCriticalBegin()) /* Only required if rxndu is interrupt */
#define SIU_RXINT_END()   (thwCriticalEnd())

#define SIU_PAGE_DECLARE          
#define SIU_PAGE_SAVE_DATA(dst,src,size) ((void*)(src))
#define SIU_PAGE_SAVE()    
#define SIU_PAGE_CHANGE(dp)                 
#define SIU_PAGE_RESTORE()          
#define SIU_PAGE_RESTORE_DATA(dst,src,size) 

/*****************************************************************************
 * DEFINITION: Local SIU return values
 *****************************************************************************/
typedef enum {
  SIU_SUCCESS = 0,
  SIU_FAIL
} siuReturn_t;

#define siu_get_umsw(ulw) ((tuint)((ulw)>>16))
#define siu_get_ulsw(ulw) ((tuint)((ulw) & 0xFFFFU))

/******************************/
/* Pragma's for code overlays */
/******************************/
#ifndef DEFAULT_CODE_SECTION
#pragma CODE_SECTION (siu_get_bufslot_distr,".initProg") 
#pragma CODE_SECTION (siutblAllocStatic,".initProg")
#pragma CODE_SECTION (siuStoreRegSets,".controlProg")
#pragma CODE_SECTION (siu_mpp_channel_state,".controlProg")
#pragma CODE_SECTION (siu_mpp_mtp2_reset,".controlProg")
#pragma CODE_SECTION (siuAnuSvrStats,".controlProg")
#pragma CODE_SECTION (siuAnuSvrCreate,".initProg")
#pragma CODE_SECTION (siuAnuSvrCtl,".controlProg")
#pragma CODE_SECTION (siu_hal_mac_buffer_allocate,".initProg")
#pragma CODE_SECTION (siu_pktio_macport_config,".controlProg")
#pragma CODE_SECTION (siuReduceMIPS,".controlProg")

#pragma CODE_SECTION (siu_ecu_reset,".controlProg")
//#pragma CODE_SECTION (siu_ecu_report_rerl,"controlProg")
//#pragma CODE_SECTION (siu_ecu_report_delay,"controlProg")
#pragma CODE_SECTION (siu_ecu_ecpathCoeff,".controlProg")
#pragma CODE_SECTION (siu_ecu_default_config,".controlProg")
#pragma CODE_SECTION (siu_ecu_bypass,".controlProg")
#pragma CODE_SECTION (siuEQISendStats,".controlProg")
#pragma CODE_SECTION (siuEQIReportStats,".controlProg")




#pragma CODE_SECTION (siu_conn_req_neu,".controlProg")

#pragma CODE_SECTION (siutblAllocStatic,".initProg")
#pragma CODE_SECTION (siuTtuTduCtrl,".controlProg")
#pragma CODE_SECTION (siuOpenCbu,".initProg")
#pragma CODE_SECTION (siuCloseCbu,".initProg")
#pragma CODE_SECTION (siuCbuInit,".initProg")
#pragma CODE_SECTION (siuCbuControl,".controlProg")
#pragma CODE_SECTION (siuCbuConnReq,".controlProg")
#pragma CODE_SECTION (siuCbuAnuDisConnection,".controlProg")
#pragma CODE_SECTION (siuCbuAnuConnection,".controlProg")




#pragma CODE_SECTION (siu_etoi_init,".initProg")
#pragma CODE_SECTION (siu_edma_close_ecan,".initProg")
#pragma CODE_SECTION (siu_edma_fixup,".tdm2pktProg")
#pragma CODE_SECTION (siuSlimInit,".initProg")
#pragma CODE_SECTION (siuSlimConfig,".controlProg")
#pragma CODE_SECTION (siutblAllocStatic,".initProg")




#pragma CODE_SECTION (siuEcReset,".controlProg")
#pragma CODE_SECTION (siuC6486CoreBoot,".initProg")
#pragma CODE_SECTION (siuC6608CoreBoot,".initProg")
#pragma CODE_SECTION (siuInitCoreMhz6608,".initProg")
#pragma CODE_SECTION (siutbl_copy_from_sharedmem,".initProg")
#pragma CODE_SECTION (siuGetDspStateInfo,".controlProg")
#pragma CODE_SECTION (siuPktioIfCfg,".controlProg")
#pragma CODE_SECTION (siuAlarm,".controlProg")
#pragma CODE_SECTION (siuAnuReqSeg,".controlProg")
#pragma CODE_SECTION (siuAnuSysAvail,".controlProg")
#pragma CODE_SECTION (siuAutoCotClose,".initProg")
#pragma CODE_SECTION (siuAutoCotRqst,".controlProg")
/*#pragma CODE_SECTION (siuAutoCotStm      TODO..where should it belong to? */
/*#pragma CODE_SECTION (siuAutoCotTduTranslate TODO..where should it belong to? */
#pragma CODE_SECTION (siuBertGetStats,".controlProg")
#pragma CODE_SECTION (siuBertModeSwitchIn,".controlProg")
#pragma CODE_SECTION (siuBertModeSwitchOut,".controlProg")
#pragma CODE_SECTION (siuBertStatsReport,".controlProg")
#pragma CODE_SECTION (siuBlockAsymBldSpec,".controlProg")
#pragma CODE_SECTION (siuBufferAlloc,".initProg")
/*#pragma CODE_SECTION (siuCIDReport TODO..where should it belong to?*/
#pragma CODE_SECTION (siuCellStarvationNotify,".controlProg")
#pragma CODE_SECTION (siuCfgCodecCopyInfoBasic,".initProg")
#pragma CODE_SECTION (siuCfgTranscode,".controlProg")
#pragma CODE_SECTION (siuChannelResetRqst,".controlProg")
#pragma CODE_SECTION (siuChannelState,".controlProg")
/*#pragma CODE_SECTION (siuCheckGmpStatus..TODO, where should it belong to?  */
#pragma CODE_SECTION (siuClearDataConfig,".controlProg")
#pragma CODE_SECTION (siuClearDataEvent,".controlProg")
#pragma CODE_SECTION (siuClearDataMode,".initProg")
#pragma CODE_SECTION (siuClearDataModeSwitchIn,".initProg")
#pragma CODE_SECTION (siuClearDataModeSwitchOut,".initProg")
#pragma CODE_SECTION (siuCloseAllFax,".initProg")
#pragma CODE_SECTION (siuCloseAllVox,".initProg")
#pragma CODE_SECTION (siuCodecConfig,".initProg")
#pragma CODE_SECTION (siuCommitState,".initProg")
#pragma CODE_SECTION (siuCommitStateBert,".initProg")
#pragma CODE_SECTION (siuCommitStateClearDataMode,".initProg")
#pragma CODE_SECTION (siuCommitStateDtmf,".initProg")
#pragma CODE_SECTION (siuCommitStateFaxMode,".initProg")
#pragma CODE_SECTION (siuCommitStateOpen,".initProg")
#pragma CODE_SECTION (siuCommitStateVoice,".initProg")
#pragma CODE_SECTION (siuConfigTask,".initProg")
#pragma CODE_SECTION (siuConnectCmd,".controlProg")
#pragma CODE_SECTION (siuCreateAndConfigTdmPort,".initProg")
#pragma CODE_SECTION (siuCreateClearDataCh,".initProg")
#pragma CODE_SECTION (siuCreateFaxCh,".initProg")
#pragma CODE_SECTION (siuCreateVoiceCh,".initProg")
#pragma CODE_SECTION (siuDTMFMode,".initProg")
#pragma CODE_SECTION (siuDebugInfo,".initProg")
#pragma CODE_SECTION (siuDebugInternal,".initProg")
#pragma CODE_SECTION (siuDebugRepor,".initProg")
/*#pragma CODE_SECTION (siuDebugTask,".initProg") TODO where to put this?*/
#pragma CODE_SECTION (siuDeleteFaxCh,".initProg")
#pragma CODE_SECTION (siuDeleteVoiceCh,".initProg")
#pragma CODE_SECTION (siuDigitSigClose,".initProg")
#pragma CODE_SECTION (siuDigitSigInit,".initProg")
#pragma CODE_SECTION (siuDigitSigReport,".controlProg")
#pragma CODE_SECTION (siuDigitSigRqst,".controlProg")
/*#pragma CODE_SECTION (siuDigitSigStm,*/
/*#pragma CODE_SECTION (siuDigitSigTduTranslate
#pragma CODE_SECTION (siuDigitUpspeed TODO, Where to put these? */
#pragma CODE_SECTION (siuDisconnectCmd,".controlProg")
/*#pragma CODE_SECTION (siuDoNothing TODO Where to put these? */ 
#pragma CODE_SECTION (siuDynMemAlloc,".initProg")
#pragma CODE_SECTION (siuDynSlotChannelAlloc,".initProg")
#pragma CODE_SECTION (siuDynSlotLBRCBufAlloc,".initProg")
#pragma CODE_SECTION (siuDynSlotReleaseBufBits,".initProg")
#pragma CODE_SECTION (siuDynSlot_clr_bit_array,".initProg")
#pragma CODE_SECTION (siuDynSlot_free_memslot,".initProg")
#pragma CODE_SECTION (siuDynSlot_freememslot,".initProg")
#pragma CODE_SECTION (siuDynSlot_free_memslot,".initProg")
#pragma CODE_SECTION (siuDynSlot_restore_channel,".initProg")
#pragma CODE_SECTION (siuDynSlot_set_bit_array,".initProg")
#pragma CODE_SECTION (siuDynSlot_tst_bit_array,".initProg")

#pragma CODE_SECTION (siuSearchFreeBit,".initProg")
#pragma CODE_SECTION (siuSearchMemSlot,".initProg")
#pragma CODE_SECTION (siuDynSubSlot_alloc_subSlot,".initProg")
#pragma CODE_SECTION (siuDynSubSlot_free_subSlot,".initProg")
#pragma CODE_SECTION (siuDynSubSlotAlloc,".initProg")
#pragma CODE_SECTION (siuDynSubSlotRelease,".initProg")
#pragma CODE_SECTION (siuDynAuxSlotAlloc,".initProg")
#pragma CODE_SECTION (siuDynAuxSlotFree,".initProg")

/* iuup files */
#pragma CODE_SECTION (siu_pvpConfig_iuup,".initProg")
#pragma CODE_SECTION (siu_iuup_encap_config,".controlProg")
#pragma CODE_SECTION (siu_iuup_store_profile_recs,".controlProg")
#pragma CODE_SECTION (siuIuupSetNewRate,".controlProg")
#pragma CODE_SECTION (siu_get_iuup_stats,".controlProg")
#pragma CODE_SECTION (siu_iuup_channel_state,".controlProg")

/* nx64 files */

#pragma CODE_SECTION (siu_new_npiu,".initProg")
#pragma CODE_SECTION (siu_new_nxu,".initProg")
#pragma CODE_SECTION (siu_open_nxu,".initProg")
#pragma CODE_SECTION (siu_open_npiu,".initProg")
#pragma CODE_SECTION (siu_new_vpu_nx64,".initProg")
#pragma CODE_SECTION (siu_new_pvp_nx64,".initProg")
#pragma CODE_SECTION (siu_tdm_config_nx64,".controlProg")
#pragma CODE_SECTION (siuNx64ModeSwitchOut,".controlProg")
#pragma CODE_SECTION (siuNx64ModeSwitchIn,".controlProg")
#pragma CODE_SECTION (siuNx64ReportStats,".controlProg")
#pragma CODE_SECTION (siuNx64SetAlarms,".controlProg")
#pragma CODE_SECTION (siu_nx64_to_close,".controlProg")
#pragma CODE_SECTION (siuCommitStateNx64,".controlProg")
#pragma CODE_SECTION (siuNx64TxNetworkProcessing,".tdm2pktProg")
#pragma CODE_SECTION (siuNx64RxProcessing,".pkt2tdmProg")
#pragma CODE_SECTION (siuNx64ReportStatus,".controlProg")

/* veu files */

#pragma CODE_SECTION (siu_new_veu,".initProg")
#pragma CODE_SECTION (siu_open_veu,".initProg")
#pragma CODE_SECTION (siu_veu_create,".initProg")
#pragma CODE_SECTION (siu_veu_control,".controlProg")
#pragma CODE_SECTION (siu_veu_stats,".controlProg")

/* Tfo files */

#pragma CODE_SECTION (siuTfoConfig,".controlProg")
#pragma CODE_SECTION (siuTfoPostStats,".controlProg")
#pragma CODE_SECTION (siu_get_tfo_stats,".controlProg")

#pragma CODE_SECTION (siuEcCoeffs,".controlProg")
#pragma CODE_SECTION (siuEcConfig,".controlProg")
#pragma CODE_SECTION (siuEcDbgStats,".controlProg")
#pragma CODE_SECTION (siuEcRese,".controlProg")t
#pragma CODE_SECTION (siuEncapConfig,".controlProg")
#pragma CODE_SECTION (siuEncodeConfig,".controlProg")
#pragma CODE_SECTION (siuErrorIndication,".initProg")
#pragma CODE_SECTION (siuException,".initProg")
#pragma CODE_SECTION (siuFMToneDetCtrl,".controlProg")
#pragma CODE_SECTION (siuFMToneDetReport,".controlProg")
#pragma CODE_SECTION (siuFaxClear,".controlProg")
#pragma CODE_SECTION (siuFaxMode,".controlProg")
#pragma CODE_SECTION (siuFaxModemConfig,".controlProg")
#pragma CODE_SECTION (siuFaxRelayProtoConfig,".controlProg")
#pragma CODE_SECTION (siuFaxSwitchIn,".controlProg")
#pragma CODE_SECTION (siuFindExtStore,".controlProg")
#pragma CODE_SECTION (siuGeneralAck,".controlProg")
#pragma CODE_SECTION (siuGetCodecProfile,".initProg")
/*#pragma CODE_SECTION (siuGetConcealment*/
#pragma CODE_SECTION (siuGetDecRates,".initProg")
#pragma CODE_SECTION (siuGetErrorStats,".controlProg")
#pragma CODE_SECTION (siuGetFaxCallStats,".controlProg")
#pragma CODE_SECTION (siuGetFaxModemStats,".controlProg")
#pragma CODE_SECTION (siuGetFaxRelayStats,".controlProg")
#pragma CODE_SECTION (siuGetGlobalPktStats,".controlProg")
#pragma CODE_SECTION (siuGetMPPStats,".controlProg")
#pragma CODE_SECTION (siuGetModuleHandle,".tdm2pktProg")
#pragma CODE_SECTION (siuGetNetEncapStats,".controlProg")
#pragma CODE_SECTION (siuGetNetworkStats,".controlProg")
#pragma CODE_SECTION (siuGetPiuStats,".controlProg")
#pragma CODE_SECTION (siuGetRtTrace,".controlProg")
#pragma CODE_SECTION (siuGetRxNoiseType,".pkt2tdmProg")
/*#pragma CODE_SECTION (siuGetSilencePattern TODO where to?*/
#pragma CODE_SECTION (siuGetTxRxStats,".controlProg")
#pragma CODE_SECTION (siuGetVPDelay,".controlProg")
#pragma CODE_SECTION (siuGetVadStats,".controlProg")
#pragma CODE_SECTION (siuGetY2Xdelay,".initProg")
/*#pragma CODE_SECTION (siuGuaranteedMessageTask, ".tdm2pktpkt2tdmProg") */
#pragma CODE_SECTION (siuHalErrCheck,".initProg")
#pragma CODE_SECTION (siuHaltCheck,".isrProg")
#pragma CODE_SECTION (siuHaltDsp,".controlProg")
/*#pragma CODE_SECTION (siuHealthProcStat, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuHealthStackCheck, ".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siuHealthStackInit ,".initProg")
#pragma CODE_SECTION (siuInitiateCmr,".controlProg")
#pragma CODE_SECTION (siuInterruptMsgAck,".controlProg")
#pragma CODE_SECTION (siuIsNmuRxExternal,".initProg")
#pragma CODE_SECTION (siuIsNmuTxExternal,".initProg")
#pragma CODE_SECTION (siuIsPLCexternal,".controlProg")
#pragma CODE_SECTION (siuIsPLCreplayRequired,".pkt2tdmProg")
#pragma CODE_SECTION (siuMPPReport,".controlProg")
#pragma CODE_SECTION (siuMain,".initProg")
#pragma CODE_SECTION (siuMemoryRead,".controlProg")
#pragma CODE_SECTION (siuMemoryWrite,".controlProg")
/*#pragma CODE_SECTION (siuMessageLevel, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuMessageTask, ".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siuMppControl,".controlProg")
#pragma CODE_SECTION (siuMppFlowControl,".controlProg")
/*#pragma CODE_SECTION (siuMsgDispatch, ".tdm2pktpkt2tdmProg")*/
#pragma CODE_SECTION (siuMsgTdmPortConfig,".controlProg")
#pragma CODE_SECTION (siuMultipleToneDetectCfg,".controlProg")
/*#pragma CODE_SECTION (siuNduGetFree, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuNduHealthSend, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuNduSendMsg, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuNduSendMsgPriority, ".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siuNduSendPkt, ".tdm2pktProg")
#pragma CODE_SECTION (siuNduSendPktCore, ".tdm2pktProg")
#pragma CODE_SECTION (siuNduSendSimpleMsgPriority, ".tdm2pktProg")
#pragma CODE_SECTION (siuNeuViolReport,".controlProg")
#pragma CODE_SECTION (siuNewCodec, ".initProg")
#pragma CODE_SECTION (siuNewEco, ".initProg")
/*#pragma CODE_SECTION (siuNormalMsgAck, ".tdm2pktpkt2tdmProg")*/
#pragma CODE_SECTION (siuOpenChannel, ".initProg")
#pragma CODE_SECTION (siuOpenGain, ".initProg")
#pragma CODE_SECTION (siuPRDTask,".isrProg")
#pragma CODE_SECTION (siuPiuReport, ".initProg")
#pragma CODE_SECTION (siuPlcConfig,".controlProg")
#pragma CODE_SECTION (siuPostCmr,".controlProg")
/*#pragma CODE_SECTION (siuProcessInterruptMessage, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuProcessMessage, ".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuProcessNormalMessage, ".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siuProfileDefine,".controlProg")
#pragma CODE_SECTION (siuPruAcuRcuSendIn,".tdm2pktProg")
#pragma CODE_SECTION (siuPruTranscode,".tdm2pktProg")
#pragma CODE_SECTION (siuPvpReport,".controlProg")
#pragma CODE_SECTION (siuReadUnlinkInstExt,".initProg")
/*#pragma CODE_SECTION (siuReduceMIPS,".tdm2pktpkt2tdmProg")*/
#pragma CODE_SECTION (siuResetRx,".controlProg")
#pragma CODE_SECTION (siuResetVpu,".controlProg")
/*#pragma CODE_SECTION (siuRouteFiuReceiveIn,".pkt2tdmProg")*/
#pragma CODE_SECTION (siuRtTraceFcn,".controlProg")
#pragma CODE_SECTION (siuRtTraceFlush,".controlProg")
#pragma CODE_SECTION (siuRtcpGetStatsFromPkt,".controlProg")
#pragma CODE_SECTION (siuRtcpGetStatsToPkt,".controlProg")
#pragma CODE_SECTION (siuRtcpGetXRVoipLoc,".controlProg")
#pragma CODE_SECTION (siuRtcpGetXRVoipRem,".controlProg")
#pragma CODE_SECTION (siuRtcpReportXRVoip,".controlProg")
#pragma CODE_SECTION (siuRtcpRstTimeout,".controlProg")
#pragma CODE_SECTION (siuRxClose,".initProg")
#pragma CODE_SECTION (siuRxNmuConfig,".controlProg")
#pragma CODE_SECTION (siuRxOpen,".initProg")
#pragma CODE_SECTION (siuRxPartOfTxRxProcessing,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuRxReceiveIn,".pkt2tdmProg")
#pragma CODE_SECTION (siuRxReset,".pkt2tdmProg")/* needs to be verified */
#pragma CODE_SECTION (siuRxSwapDecoder,".controlProg")
#pragma CODE_SECTION (siuRxTguCidGate,".pkt2tdmProg")
#pragma CODE_SECTION (siuSIDOptions,".controlProg")
#pragma CODE_SECTION (siuSchedFcns,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuSearchFreeBit,".initProg")
#pragma CODE_SECTION (siuSearchMemSlot,".initProg")
#pragma CODE_SECTION (siuSendMicroABBitsChanged,".pkt2tdmProg")
#pragma CODE_SECTION (siuSetGain,".controlProg")
#pragma CODE_SECTION (siuSetGrantSync,".controlProg")
#pragma CODE_SECTION (siuSetRateMode,".controlProg")
#pragma CODE_SECTION (siuSetVadState,".controlProg")
#pragma CODE_SECTION (siuShutdownNduAcu,".initProg")
#pragma CODE_SECTION (siuSigLevEstGetLevels,".controlProg")
#pragma CODE_SECTION (siuSigLevEstProcess,".pkt2tdmProg")
#pragma CODE_SECTION (siuSigLevEstReset,".initProg")
#pragma CODE_SECTION (siuStarvationReset,".controlProg")
#pragma CODE_SECTION (siuStatsReqChstMask2Lsw,".controlProg")
#pragma CODE_SECTION (siuStatsReqChstMask2Msw,".controlProg")
#pragma CODE_SECTION (siuStatsReqChstMaskLsw,".controlProg")
#pragma CODE_SECTION (siuStatsReqChstMaskMsw,".controlProg")
#pragma CODE_SECTION (siuStatsRequest,".controlProg")
#pragma CODE_SECTION (siuStatusResponse,".controlProg")
#pragma CODE_SECTION (siuStoreEncode,".controlProg")
#pragma CODE_SECTION (siuSubRateFcn,".isrProg")
#pragma CODE_SECTION (siuSwitchOutFax,".controlProg")
/*#pragma CODE_SECTION (siuTaskScheduler,".tdm2pktpkt2tdmProg")*/
#pragma CODE_SECTION (siuTdmCntrl1,".initProg")
#pragma CODE_SECTION (siuTdmCntrl2,".initProg")
#pragma CODE_SECTION (siuTdmCntrl3,".initProg")
#pragma CODE_SECTION (siuTdmConfig,".controlProg")
#pragma CODE_SECTION (siuTduMfConfig,".controlProg")
/*#pragma CODE_SECTION (siuTmrCalcDTime,".tdm2pktpkt2tdmProg")*/
#pragma CODE_SECTION (siuTmrGetTime,".initProg")
/*#pragma CODE_SECTION (siuTmrReset TODO */
#pragma CODE_SECTION (siuToneDetect,".tdm2pktProg")
#pragma CODE_SECTION (siuToneDetectDisable,".controlProg")
#pragma CODE_SECTION (siuToneDetectEnable,".controlProg")
#pragma CODE_SECTION (siuToneGenerationReport,".controlProg")
#pragma CODE_SECTION (siuToneOff,".controlProg")/* TODO verify this */
#pragma CODE_SECTION (siuToneOn,".controlProg")/* TODO verify this */
#pragma CODE_SECTION (siuTranslateVcuToCwSeg,".tdm2pktProg")
//#pragma CODE_SECTION (siuTrcLastPlayoutTime
#pragma CODE_SECTION (siuTxClose,".initProg")
#pragma CODE_SECTION (siuTxNetwork,".tdm2pktProg")
#pragma CODE_SECTION (siuTxNetworkFramed,".tdm2pktProg")
#pragma CODE_SECTION (siuTxNetwork_frame_length,".initProg")
#pragma CODE_SECTION (siuTxNmuConfig,".initProg")
#pragma CODE_SECTION (siuTxRxSwi,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuTxSendIn,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siuTxSwapEncoder,".controlProg")
#pragma CODE_SECTION (siuTxToneGeneration,".tdm2pktProg")
#pragma CODE_SECTION (siuUnpackProfile,".initProg")
#pragma CODE_SECTION (siuUnpackProfilePtr,".initProg")
#pragma CODE_SECTION (siuUpdateRuntimeConfigRtp,".controlProg")
#pragma CODE_SECTION (siuUpdateVPUFrames,".controlProg")
#pragma CODE_SECTION (siuVPConfig,".controlProg")
#pragma CODE_SECTION (siuVauConfig,".controlProg")
#pragma CODE_SECTION (siuVoiceMode,".controlProg")
#pragma CODE_SECTION (siuWriteDebugTrace,".controlProg")
#pragma CODE_SECTION (siu_aal1_channel_state,".controlProg")
#pragma CODE_SECTION (siu_aal1_encap_config,".controlProg")
#pragma CODE_SECTION (siu_aal2_commit_encode_state,".controlProg")
#pragma CODE_SECTION (siu_aal2_encap_config,".controlProg")
#pragma CODE_SECTION (siu_aal2_encode_config,".controlProg")
/*#pragma CODE_SECTION (siu_aal2_find_sid_dtls*/
#pragma CODE_SECTION (siu_aal2_newRxCodecInfo,".controlProg")
#pragma CODE_SECTION (siu_aal2_read_encode_state,".controlProg")
#pragma CODE_SECTION (siu_aal2_read_encode_state_by_index,".controlProg")
#pragma CODE_SECTION (siu_aal2_search_profile,".controlProg")
#pragma CODE_SECTION (siu_aal2_store_profile_recs,".controlProg")
#pragma CODE_SECTION (siu_aal2_update_pktiface,".controlProg")
#pragma CODE_SECTION (siu_all_set_encap,".controlProg")
#pragma CODE_SECTION (siu_announce_begin,".controlProg")
#pragma CODE_SECTION (siu_announce_complete,".controlProg")
#pragma CODE_SECTION (siu_announce_restore,".controlProg")
#pragma CODE_SECTION (siu_announce_seg,".controlProg")
#pragma CODE_SECTION (siu_announce_stop,".controlProg")
#pragma CODE_SECTION (siu_anu_format_gmac,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_format_hd,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_format_ut,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_receive_gmac,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_receive_local,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_pass,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_receive_hd,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_receive_ut,".pkt2tdmProg")
#pragma CODE_SECTION (siu_anu_rqst_send,".controlProg")
#pragma CODE_SECTION (siu_bert_to_close,".controlProg")
#pragma CODE_SECTION (siu_bits_to_bytes,".controlProg")
#pragma CODE_SECTION (siu_buffer_align,".initProg")
/*#pragma CODE_SECTION (siu_cadence_done_dsigt,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siu_cadence_gen_dsigt,".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siu_cid_control,".controlProg")
#pragma CODE_SECTION (siu_clear_data_to_close,".controlProg")
#pragma CODE_SECTION (siu_close_acu,".initProg")
#pragma CODE_SECTION (siu_close_anu,".initProg")
#pragma CODE_SECTION (siu_close_dsigt,".initProg")
#pragma CODE_SECTION (siu_close_rcu,".initProg")
#pragma CODE_SECTION (siu_cnf_init,".initProg")
/*#pragma CODE_SECTION (siu_compute_sub_phase*/
#pragma CODE_SECTION (siu_config_voice_frame,".initProg")
#pragma CODE_SECTION (siu_configure_pvpConfig,".initProg")
#pragma CODE_SECTION (siu_configure_rate_and_config,".initProg")
#pragma CODE_SECTION (siu_configure_vcu_voice_profile,".initProg")
#pragma CODE_SECTION (siu_conn_req_cmd,".controlProg")
#pragma CODE_SECTION (siu_conn_req_cnf,".controlProg")
#pragma CODE_SECTION (siu_conn_req_pru,".controlProg")
#pragma CODE_SECTION (siu_copy_done,".initProg")
#pragma CODE_SECTION (siu_create_tdm_port,".initProg")
#pragma CODE_SECTION (siu_delete_acu,".initProg")
#pragma CODE_SECTION (siu_delete_anu,".initProg")
#pragma CODE_SECTION (siu_delete_rcu,".initProg")
#pragma CODE_SECTION (siu_dialPulse_cfg_dsigt,".initProg")
#pragma CODE_SECTION (siu_dialPulse_eventRprt_dsigt,".controlProg")
#pragma CODE_SECTION (siu_dialPulse_ops_dsigt,".controlProg")
#pragma CODE_SECTION (siu_digit_actions_config,".controlProg")
#pragma CODE_SECTION (siu_do_rx_process_last,".pkt2tdmProg")
#pragma CODE_SECTION (siu_dsig_report_stats,".controlProg")
/*#pragma CODE_SECTION (siu_dsigr_bit_queue TODO */
#pragma CODE_SECTION (siu_dsigr_close_encap,".controlProg")
#pragma CODE_SECTION (siu_dsigr_encap_aal2,".controlProg")
#pragma CODE_SECTION (siu_dsigr_encap_rtp,".controlProg")
/*#pragma CODE_SECTION (siu_dsigt_process_task,".pkt2tdmProg")*/
/*#pragma CODE_SECTION (siu_dsp_halt TODO */
#pragma CODE_SECTION (siu_ec_dbgStat,".controlProg")
#pragma CODE_SECTION (siu_ec_ecpathCoeff,".controlProg")
/*#pragma CODE_SECTION (siu_ecu_bypass TODO */
#pragma CODE_SECTION (siu_ecu_close,".initProg")
#pragma CODE_SECTION (siu_ecu_control,".controlProg")
#pragma CODE_SECTION (siu_ecu_dbgstat,".controlProg")
//#pragma CODE_SECTION (siu_ecu_default_config
//#pragma CODE_SECTION (siu_ecu_ecpathCoeff
#pragma CODE_SECTION (siu_ecu_new,".initProg")
#pragma CODE_SECTION (siu_ecu_open,".initProg")
//#pragma CODE_SECTION (siu_ecu_report_delay
//#pragma CODE_SECTION (siu_ecu_report_rerl
//#pragma CODE_SECTION (siu_ecu_reset
//#pragma CODE_SECTION (siu_ecuma_setDP
#pragma CODE_SECTION (siu_encap_dsigr,".controlProg")
#pragma CODE_SECTION (siu_exception,".controlProg")
#pragma CODE_SECTION (siu_execencmod_pru,".controlProg")
//#pragma CODE_SECTION (siu_find_tone_instance
//#pragma CODE_SECTION (siu_free_rxinst_gmc
//#pragma CODE_SECTION (siu_gain_adjust
//#pragma CODE_SECTION (siu_gain_ctrl
//#pragma CODE_SECTION (siu_gain_db
//#pragma CODE_SECTION (siu_gaintbl
//#pragma CODE_SECTION (siu_getPcmSyncIntr
//#pragma CODE_SECTION (siu_get_gains
//#pragma CODE_SECTION (siu_get_stats_unavail
//#pragma CODE_SECTION (siu_gm_modify_vc_msg_header
//#pragma CODE_SECTION (siu_hal_msg_reply
//#pragma CODE_SECTION (siu_inband_relay_notification
#pragma CODE_SECTION (siu_init_prg_copy,".initProg")
#pragma CODE_SECTION (siu_init_vca,".initProg")
#pragma CODE_SECTION (siu_intch_to_extch,".controlProg")
#pragma CODE_SECTION (siu_invalidate_cache,".initProg")
//#pragma CODE_SECTION (siu_isr_clock
#pragma CODE_SECTION (siu_locate_rtp_cprofile,".initProg")
#pragma CODE_SECTION (siu_macport_config,".controlProg")
#pragma CODE_SECTION (siu_mag_control,".controlProg")
//#pragma CODE_SECTION (siu_mag_dbgstat
#pragma CODE_SECTION (siu_map_codtype_to_xdmspeechtype,".initProg")
#pragma CODE_SECTION (siu_map_dsig,".controlProg")
#pragma CODE_SECTION (siu_map_pkngtype_to_xdmpkngtype,".initProg")
//#pragma CODE_SECTION (siu_mega_restore
//#pragma CODE_SECTION (siu_mega_toss_rx
//#pragma CODE_SECTION (siu_mega_toss_tx
#pragma CODE_SECTION (siu_memory_read_done,".controlProg")
#pragma CODE_SECTION (siu_memory_read_exec,".controlProg")
#pragma CODE_SECTION (siu_memory_write_done,".controlProg")
//#pragma CODE_SECTION (siu_mpp_channel_state
//#pragma CODE_SECTION (siu_mpp_mtp2_reset
#pragma CODE_SECTION (siu_msm_bidir_silence_ctrl ,".controlProg")
#pragma CODE_SECTION (siu_msm_bidir_silence_reset,".controlProg")
#pragma CODE_SECTION (siu_msm_dc_offset_ctrl,".controlProg")
#pragma CODE_SECTION (siu_msm_dtmf_to_voice,".controlProg")
#pragma CODE_SECTION (siu_msm_fax_relay_to_closed,".controlProg")
#pragma CODE_SECTION (siu_msm_fax_relay_to_idle,".controlProg")
#pragma CODE_SECTION (siu_msm_fax_relay_to_voice,".controlProg")
#pragma CODE_SECTION (siu_msm_idle_to_closed,".controlProg")
#pragma CODE_SECTION (siu_msm_idle_to_dtmf,".controlProg")
#pragma CODE_SECTION (siu_msm_idle_to_fax_relay,".controlProg")
#pragma CODE_SECTION (siu_msm_pattern_ctrl,".controlProg")
#pragma CODE_SECTION (siu_msm_voice_to_closed,".controlProg")
#pragma CODE_SECTION (siu_msm_voice_to_dtmf,".controlProg")
#pragma CODE_SECTION (siu_msm_voice_to_fax_relay,".controlProg")
#pragma CODE_SECTION (siu_msm_voice_to_idle,".controlProg")
#pragma CODE_SECTION (siu_native_frame_length,".initProg")
#pragma CODE_SECTION (siu_net_encap_control	,".controlProg")
#pragma CODE_SECTION (siu_net_encap_glob_cfg,".controlProg")
#pragma CODE_SECTION (siu_net_loopback,".controlProg")
#pragma CODE_SECTION (siu_new_acu,".initProg")
#pragma CODE_SECTION (siu_new_anu,".initProg")
#pragma CODE_SECTION (siu_new_bert,".initProg")
#pragma CODE_SECTION (siu_new_cid,".initProg")
#pragma CODE_SECTION (siu_new_fiu,".initProg")
#pragma CODE_SECTION (siu_new_fm,".initProg")
#pragma CODE_SECTION (siu_new_mpp,".initProg")
#pragma CODE_SECTION (siu_new_neu,".initProg")
#pragma CODE_SECTION (siu_new_piu,".initProg")
#pragma CODE_SECTION (siu_new_plc,".initProg")
#pragma CODE_SECTION (siu_new_pru,".initProg")
#pragma CODE_SECTION (siu_new_pvp,".initProg")
#pragma CODE_SECTION (siu_new_rcu,".initProg")
#pragma CODE_SECTION (siu_new_rx_nmu,".initProg")
#pragma CODE_SECTION (siu_new_tdu,".initProg")
#pragma CODE_SECTION (siu_new_tgu,".initProg")
#pragma CODE_SECTION (siu_new_tx_nmu,".initProg")
#pragma CODE_SECTION (siu_new_vau,".initProg")
#pragma CODE_SECTION (siu_new_vca,".initProg")
#pragma CODE_SECTION (siu_new_vcu,".initProg")
#pragma CODE_SECTION (siu_new_vpu,".initProg")
#pragma CODE_SECTION (siu_obtain_vcaInfo,".initProg")
#pragma CODE_SECTION (siu_open_acu,".initProg")
#pragma CODE_SECTION (siu_open_bert,".initProg")
#pragma CODE_SECTION (siu_open_channel_block_codecs,".initProg")
#pragma CODE_SECTION (siu_open_cid,".initProg")
#pragma CODE_SECTION (siu_open_dsigt,".initProg")
#pragma CODE_SECTION (siu_open_fiu,".initProg")
#pragma CODE_SECTION (siu_open_fm,".initProg")
#pragma CODE_SECTION (siu_open_mpp,".initProg")
#pragma CODE_SECTION (siu_open_network,".initProg")
#pragma CODE_SECTION (siu_open_neu,".initProg")
#pragma CODE_SECTION (siu_open_piu,".initProg")
#pragma CODE_SECTION (siu_open_plc,".initProg")
#pragma CODE_SECTION (siu_open_pru,".initProg")
#pragma CODE_SECTION (siu_open_pvp,".initProg")
#pragma CODE_SECTION (siu_open_rcu,".initProg")
#pragma CODE_SECTION (siu_open_rx_nmu,".initProg")
#pragma CODE_SECTION (siu_open_tdu,".initProg")
#pragma CODE_SECTION (siu_open_tgu,".initProg")
#pragma CODE_SECTION (siu_open_tx_nmu,".initProg")
#pragma CODE_SECTION (siu_open_vau,".initProg")
#pragma CODE_SECTION (siu_open_vcu,".initProg")
#pragma CODE_SECTION (siu_open_vpu,".initProg")
#pragma CODE_SECTION (siu_p2p_insertTxCfg,".controlProg")
#pragma CODE_SECTION (siu_p2p_to_pkt_config,".controlProg")
#pragma CODE_SECTION (siu_p2prxin_pru,".pkt2tdmProg")
#pragma CODE_SECTION (siu_pattern_generation_control,".controlProg")
#pragma CODE_SECTION (siu_pcm_loopback,".controlProg")
#pragma CODE_SECTION (siu_pcmn_set,".initProg")
#pragma CODE_SECTION (siu_periph_setup,".initProg")
//#pragma CODE_SECTION (siu_piu_drop_subsegment
#pragma CODE_SECTION (siu_piu_reset,".controlProg")
/*#pragma CODE_SECTION (siu_poll_changes_dsigr,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siu_poll_changes_dsigt,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siu_priority_queue_valid,".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siu_pvpConfig_aal1,".controlProg")
#pragma CODE_SECTION (siu_pvpConfig_aal2,".controlProg")
#pragma CODE_SECTION (siu_pvpConfig_rtp,".controlProg")
#pragma CODE_SECTION (siu_pvp_control,".controlProg")
/*#pragma CODE_SECTION (siu_queue_purge_all,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siu_queue_purge_channel,".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siu_queue_write,".tdm2pktpkt2tdmProg")

//#pragma CODE_SECTION (siu_receive_pkt
#pragma CODE_SECTION (siu_reconfigure_nmu_plc,".controlProg")
#pragma CODE_SECTION (siu_report_dsigr,".controlProg")
#pragma CODE_SECTION (siu_report_dsigt,".controlProg")
#pragma CODE_SECTION (siu_req_dsig,".controlProg")
#pragma CODE_SECTION (siu_req_voice_notification,".controlProg")
#pragma CODE_SECTION (siu_reset_channel_unavail,".initProg")
#pragma CODE_SECTION (siu_reset_configuration,".initProg")
#pragma CODE_SECTION (siu_rtcp_control,".controlProg")
#pragma CODE_SECTION (siu_rtcp_qa_config,".controlProg")
//#pragma CODE_SECTION (siu_rtcp_qa_reset
//#pragma CODE_SECTION (siu_rtcp_qa_rpt
#pragma CODE_SECTION (siu_rtcp_req_stats,".controlProg")
#pragma CODE_SECTION (siu_rtcp_sdes_string,".controlProg")
#pragma CODE_SECTION (siu_rtcp_send_packet,".controlProg")
#pragma CODE_SECTION (siu_rtcp_send_pkt_ack,".controlProg")
#pragma CODE_SECTION (siu_rtcp_status_rpt,".controlProg")
#pragma CODE_SECTION (siu_rtcp_to_host,".controlProg")
//#pragma CODE_SECTION (siu_rtcp_xr_sys_delay
//#pragma CODE_SECTION (siu_rtcpxr_ready
#pragma CODE_SECTION (siu_rtp_calc_vifSize,".controlProg")
#pragma CODE_SECTION (siu_rtp_commit_encode_state,".controlProg")
#pragma CODE_SECTION (siu_rtp_encap_config,".controlProg")
#pragma CODE_SECTION (siu_rtp_encode_config,".controlProg")
#pragma CODE_SECTION (siu_rtp_newRxCodecInfo,".controlProg")
#pragma CODE_SECTION (siu_rtp_read_encode_state,".controlProg")
#pragma CODE_SECTION (siu_rtp_read_encode_state_by_index,".controlProg")
#pragma CODE_SECTION (siu_rtp_search_profile,".controlProg")
#pragma CODE_SECTION (siu_rtp_store_profile_recs,".controlProg")
#pragma CODE_SECTION (siu_rtp_unpack_cprof_codec_options,".initProg")
#pragma CODE_SECTION (siu_rtp_unpack_cprof_payld_options,".initProg")
#pragma CODE_SECTION (siu_rtp_unpack_cprof_ratemode_info,".initProg")
#pragma CODE_SECTION (siu_rtp_validate_ratemode_change,".controlProg")
//#pragma CODE_SECTION (siu_rtpts_handoff
//#pragma CODE_SECTION (siu_rtpts_to_rcu
#pragma CODE_SECTION (siu_rx_process ,".pkt2tdmProg")
#pragma CODE_SECTION (siu_rx_stm_config,".controlProg")
#pragma CODE_SECTION (siu_search_vcaInfo,".initProg")
/*#pragma CODE_SECTION (siu_send_error_message,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siu_send_keep_alive,".tdm2pktpkt2tdmProg")
#pragma CODE_SECTION (siu_send_packet_ack,".tdm2pktpkt2tdmProg")
*/
#pragma CODE_SECTION (siu_send_pkt,".controlProg")
#pragma CODE_SECTION (siu_set_companding_piu,".initProg")
#pragma CODE_SECTION (siu_set_dsig,".controlProg")
#pragma CODE_SECTION (siu_setup_mega_stagger,".initProg")
/*#pragma CODE_SECTION (siu_sigbit_change_dsigc,".tdm2pktpkt2tdmProg") */
#pragma CODE_SECTION (siu_signalling_transfer,".controlProg")
#pragma CODE_SECTION (siu_stm_search,".tdm2pktProg")
/*#pragma CODE_SECTION (siu_sub_dummy,".tdm2pktpkt2tdmProg")*/
/*#pragma CODE_SECTION (siu_susspend_unsolicited*/
#pragma CODE_SECTION (siu_tdu_send_in_dummy,".controlProg")
#pragma CODE_SECTION (siu_tgu_tone_begin,".controlProg")
#pragma CODE_SECTION (siu_tgu_tone_complete,".controlProg")
#pragma CODE_SECTION (siu_to_closed,".controlProg")
#pragma CODE_SECTION (siu_tone_off,".controlProg")
#pragma CODE_SECTION (siu_tone_on,".controlProg")
#pragma CODE_SECTION (siu_translate_dynamic_chnum,".initProg")
#pragma CODE_SECTION (siu_trunk_dsigt,".tdm2pktProg")
#pragma CODE_SECTION (siu_tx_process,".tdm2pktProg")
#pragma CODE_SECTION (siu_unpack_profile_aal2,".controlProg")
#pragma CODE_SECTION (siu_unpack_profile_rtp,".controlProg")
#pragma CODE_SECTION (siu_unpack_rtp_cprofile,".controlProg")
#pragma CODE_SECTION (siu_update_apx_ecuma,".controlProg")
#pragma CODE_SECTION (siu_update_offset_ecuma,".controlProg")
#pragma CODE_SECTION (siu_utport_config,".controlProg")
#pragma CODE_SECTION (siu_validate_payload_options,".controlProg")
#pragma CODE_SECTION (siu_voice_notification,".controlProg")
#pragma CODE_SECTION (siugmpStats,".controlProg")
#pragma CODE_SECTION (globCfgPcmFrameLength,".pkt2tdmProg")
#pragma CODE_SECTION (gmpGetCwSeg,".pkt2tdmProg")
#pragma CODE_SECTION (gmpFreeCwSeg,".pkt2tdmProg")
#pragma CODE_SECTION (gmpFreeGmc,".pkt2tdmProg")


#pragma CODE_SECTION (cnfRouterIn,".pkt2tdmProg")
#pragma CODE_SECTION (cnfMixerIn,".pkt2tdmProg")
#pragma CODE_SECTION (cnfMixerOutput,".pkt2tdmProg")
#pragma CODE_SECTION (siuSlimIn,".pkt2tdmProg")

#pragma CODE_SECTION (siu_remap_mcs_data,".initProg");
#pragma CODE_SECTION (siu_determine_alloc_type,".initProg");
#pragma CODE_SECTION (siu_check_sslot,".initProg");
#pragma CODE_SECTION (siu_queue_purge_all,".controlProg");
#pragma CODE_SECTION (siu_exception,".controlProg");
#pragma CODE_SECTION (siuttuGetStats,".initProg");
#pragma CODE_SECTION (siu_open_ttu,".initProg");
#pragma CODE_SECTION (siu_new_ttu,".initProg");
#pragma CODE_SECTION (siu_delete_ttu,".initProg");
#pragma CODE_SECTION (siu_cmd_ttu_config,".controlProg");
#pragma CODE_SECTION (siu_close_ttu,".initProg");
#pragma CODE_SECTION (siu_memory_read_exec,".controlProg");
#pragma CODE_SECTION (siu_memory_read_done,".controlProg");
#pragma CODE_SECTION (siuDebugReport,".controlProg");
#pragma CODE_SECTION (siuChannelException,".initProg");
#pragma CODE_SECTION (siu_open_eqi,".initProg");
#pragma CODE_SECTION (siu_new_eqi,".initProg");
#pragma CODE_SECTION (siu_close_eqi,".initProg");
#pragma CODE_SECTION (siu_control_eqi,".initProg");
#pragma CODE_SECTION (siu_vqm_rpt_stats,".controlProg");
#pragma CODE_SECTION (siu_vqm_control,".controlProg");
#pragma CODE_SECTION (siu_open_vqm,".initProg");
#pragma CODE_SECTION (siu_new_vqm,".initProg");
#pragma CODE_SECTION (siu_get_vqm_stats,".controlProg");
#pragma CODE_SECTION (siumsu_alloc_buff,".initProg");
#pragma CODE_SECTION (siumsu_debug_info,".initProg");
#pragma CODE_SECTION (siu_new_msu_rtp,".initProg");
#pragma CODE_SECTION (siu_new_msu_rtcp,".initProg");
#pragma CODE_SECTION (siu_delete_msu_rtp,".initProg");
#pragma CODE_SECTION (siu_delete_msu_rtcp,".initProg");
#pragma CODE_SECTION (siu_msu_prof_config,".initProg");
#pragma CODE_SECTION (siu_msu_key_ctrl,".initProg");
#pragma CODE_SECTION (siu_msu_get_stats_rtp,".initProg");
#pragma CODE_SECTION (siu_msu_get_stats_rtcp,".initProg");
#pragma CODE_SECTION (siu_msu_control,".initProg");
#pragma CODE_SECTION (siu_global_config_anu,".controlProg");
#pragma CODE_SECTION (siu_dsp_halt,".controlProg");


#pragma CODE_SECTION (cnfHeapAlloc,".initProg");
#pragma CODE_SECTION (cnfRouterInit,".initProg");
#pragma CODE_SECTION (cnfDeleteLinker,".initProg");
#pragma CODE_SECTION (cnfDeleteConnector,".initProg");
#pragma CODE_SECTION (cnfDeleteBuffer,".initProg");
#pragma CODE_SECTION (cnfDisconnect,".initProg");
#pragma CODE_SECTION (cnfRemoveAllConnections,".initProg");
#pragma CODE_SECTION (cnfNewConnector,".initProg");
#pragma CODE_SECTION (cnfNewLinker,".initProg");
#pragma CODE_SECTION (cnfNewBuffer,".initProg");
#pragma CODE_SECTION (cnfConnect,".controlProg");
#pragma CODE_SECTION (cnfOpen,".initProg");
#pragma CODE_SECTION (cnfMixInit,".initProg");
#pragma CODE_SECTION (cnfInit,".initProg");
#pragma CODE_SECTION (cnfHeapInit,".initProg");
#pragma CODE_SECTION (cnfClose,".initProg");
#pragma CODE_SECTION (cnfSetFrameSize,".initProg");
#pragma CODE_SECTION (cnfHeapAlloc,".initProg");
#pragma CODE_SECTION (cnfRouterInit,".initProg");
#pragma CODE_SECTION (cnfDeleteLinker,".initProg");
#pragma CODE_SECTION (cnfDeleteConnector,".initProg");
#pragma CODE_SECTION (cnfDeleteBuffer,".initProg");
#pragma CODE_SECTION (cnfDisconnect,".controlProg");
#pragma CODE_SECTION (cnfRemoveAllConnections,".initProg");
#pragma CODE_SECTION (cnfNewConnector,".initProg");
#pragma CODE_SECTION (cnfNewLinker,".initProg");
#pragma CODE_SECTION (cnfNewBuffer,".initProg");
#pragma CODE_SECTION (cnfConnect,".controlProg");
#pragma CODE_SECTION (cnfOpen,".initProg");
#pragma CODE_SECTION (cnfMixInit,".initProg");
#pragma CODE_SECTION (cnfInit,".initProg");
#pragma CODE_SECTION (cnfHeapInit,".initProg");
#pragma CODE_SECTION (cnfClose,".initProg");

#pragma CODE_SECTION (siuUnpackProfilePtr , ".controlProg");
#pragma CODE_SECTION (siuUnpackProfile ,    ".controlProg");
#pragma CODE_SECTION (siu_open_plr,".initProg")
#pragma CODE_SECTION (siu_new_plr,".initProg")
#pragma CODE_SECTION (siuRedIsEnabled,".pkt2tdmProg")
#pragma CODE_SECTION (siuFecIsEnabled,".pkt2tdmProg")
#pragma CODE_SECTION (siu_get_plr_stats,".controlProg")
#pragma CODE_SECTION (siu_delete_plr,".initProg")
#pragma CODE_SECTION (siu_close_plr,".initProg")
#pragma CODE_SECTION (siu_pktio_macport_confi,".controlProg")
#pragma CODE_SECTION (siu_macport_config,".controlProg")

#pragma CODE_SECTION (siumsu_key_request,".controlProg");
#pragma CODE_SECTION (siumsu_free_buff,".initProg");
#pragma CODE_SECTION (siumsu_alloc_buff,".initProg");
#pragma CODE_SECTION (siumsu_debug_info,".initProg");
#pragma CODE_SECTION (siumsu_sync_txseq_num,".controlProg");
#pragma CODE_SECTION (siu_msu_mips_cost,".controlProg");
#pragma CODE_SECTION (siu_new_msu_rtp,".initProg");
#pragma CODE_SECTION (siu_new_msu_rtcp,".initProg");
#pragma CODE_SECTION (siu_delete_msu_rtp,".initProg");
#pragma CODE_SECTION (siu_delete_msu_rtcp,".initProg");
#pragma CODE_SECTION (siu_msu_prof_config,".controlProg");
#pragma CODE_SECTION (siu_msu_key_ctrl,".controlProg");
#pragma CODE_SECTION (siu_msu_get_stats_rtp,".controlProg");
#pragma CODE_SECTION (siu_msu_get_stats_rtcp,".controlProg");
#pragma CODE_SECTION (siu_msu_control,".controlProg");
#pragma CODE_SECTION (siuMsuIsEnabled,".controlProg");
#pragma CODE_SECTION (siuMsuGetDspStateResp,".controlProg");
#pragma CODE_SECTION (gmpCreate,".initProg");
#pragma CODE_SECTION (gmpDelete,".initProg");
#pragma CODE_SECTION (siuPlrConfig,".controlProg");
#pragma CODE_SECTION (siu_preserve_ecuma_request,".tdm2pktProg");
#pragma CODE_SECTION (siu_execute_ecuma_request,".tdm2pktProg");

#pragma CODE_SECTION (siu_osal_start,       ".controlProg");
#pragma CODE_SECTION (siu_osal_post_signal,         ".isrProg");

#pragma CODE_SECTION (siu_ipc_osal_start,              ".controlProg");
#pragma CODE_SECTION (siu_ipc_osal_event,              ".controlProg");
#pragma CODE_SECTION (siu_ipc_osal_remove_pending_msg, ".controlProg");
#pragma CODE_SECTION (siu_ipc_osal_intercore_isr,      ".isrProg");
#pragma CODE_SECTION (siu_ipc_osal_init,               ".initProg");
#pragma CODE_SECTION (siu_ipc_osal_set_msg_consumed,       ".tdm2pktpkt2tdmProg");

#pragma CODE_SECTION (siuIpcBiosInit,             ".initProg");
#pragma CODE_SECTION (siuIpcBiosAttachOwner,      ".controlProg");
#pragma CODE_SECTION (siuIpcBiosStart,            ".controlProg");
#pragma CODE_SECTION (siuIpcBiosEvent,            ".controlProg");
#pragma CODE_SECTION (siuIpcBiosMessageQCreate,   ".controlProg");
#pragma CODE_SECTION (siuIpcBiosMessageQOpenAlloc,".controlProg");
#pragma CODE_SECTION (siuIpcBiosMessageQClose,    ".controlProg");
#pragma CODE_SECTION (siuIpcBiosMessageQControl,  ".controlProg");
#pragma CODE_SECTION (siuIpcBiosObtainMsg,        ".tdm2pktpkt2tdmProg");
#pragma CODE_SECTION (siuIpcBiosMsgSend,          ".tdm2pktpkt2tdmProg");
#pragma CODE_SECTION (siuIpcBiosMessageQGet,      ".tdm2pktpkt2tdmProg");
#pragma CODE_SECTION (siuIpcBiosSetMsgConsumed,   ".tdm2pktpkt2tdmProg");

#pragma CODE_SECTION (siuIpcMcapiMsgSend,        ".tdm2pktpkt2tdmProg");
#pragma CODE_SECTION (siuIpcMcapiIntercoreIsr,   ".isrProg");

/************************** Overlay programs ************************/
#pragma CODE_SECTION (siuInit,            ".progOverlay");
#pragma CODE_SECTION (siuDPRMsgBufferInit,".progOverlay");
#pragma CODE_SECTION (siuInitTask,         ".progOverlay");
#pragma CODE_SECTION (siuHwConfig,         ".progOverlay");
#pragma CODE_SECTION (siuRestartIndication,".progOverlay");
#pragma CODE_SECTION (siuSendDummyTrace,   ".progOverlay");
#pragma CODE_SECTION (siuReadyIndication,  ".progOverlay");
#pragma CODE_SECTION (siuStart,            ".progOverlay");
#pragma CODE_SECTION (siuCreateChannels,".progOverlay");
#pragma CODE_SECTION (gmpInit,   ".progOverlay");
#pragma CODE_SECTION (siugmpInit,".progOverlay");
#pragma CODE_SECTION (siu_init_hal,".progOverlay");
#pragma CODE_SECTION (siu_new_hal, ".progOverlay");
#pragma CODE_SECTION (siuDsigtInit, ".progOverlay");
#pragma CODE_SECTION (siuDsigrInit, ".progOverlay");
#pragma CODE_SECTION (siuHealthInit, ".progOverlay");
#pragma CODE_SECTION (siuDebugBufferInit,".progOverlay");
#pragma CODE_SECTION (siuNduStart,         ".progOverlay");
#pragma CODE_SECTION (siuNduv1Init,        ".progOverlay");
#pragma CODE_SECTION (siuGmacnduInit,      ".progOverlay");
#pragma CODE_SECTION (siuUtnduInit,        ".progOverlay");
#pragma CODE_SECTION (siuCopyMsgInit,      ".progOverlay");
//#pragma CODE_SECTION (siu_new_nv_neu,      ".progOverlay");
//#pragma CODE_SECTION (siuNewNvEco,         ".progOverlay");
//#pragma CODE_SECTION (siuNvEcoGetBuffInfo, ".progOverlay");

/* Overlayed siuRdndc that is not needed after Activate starts */
#pragma CODE_SECTION (siu_rdndc_encap_config_valid_params,  ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_encode_config_valid_params, ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_tfo_config_valid_params,    ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_ec_config_valid_params,     ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_vqm_control_valid_params,   ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_tdm_config_valid_params,    ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_vad_config_valid_params,    ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_vp_config_valid_params,     ".progOverlay");
#pragma CODE_SECTION (siu_rdndc_mfdet_config_valid_params,  ".progOverlay");
#pragma CODE_SECTION (siuRdndcCheckConfigChan,              ".progOverlay");
#pragma CODE_SECTION (siuRdndcCheckConfigCore,              ".progOverlay");

#pragma CODE_SECTION (siuIpcBiosSetLocalId,              ".progOverlay");
#endif

/*****************************************************************************/
/* The function is invoked before DSP halt - to make register values visible */
/*****************************************************************************/
#define SIU_CPU_REGISTERS_HALT_MSG   1
#define SIU_CPU_REGISTERS_EXCEPTION  2
#define SIU_CPU_REGISTERS_HALT_CHECK 3
extern char gg_scratch[];

void siuStoreRegSets (siuC64xPlusReg_t *reg);

#define SIU_GET_TS ((TSCH << 16) | (TSCL >> 16))
/* Scaling with (core_MHz * 1000) / 65536 = 7.629 */
#define SIU_TS_SCALE(a,b) ((((((b) * 25) >> 10) * 5) * (a)) >> 3)

static inline void siu_preserve_registers (tuint id_code, void* scratch)
{
  extern cregister volatile unsigned int ISTP;
  extern cregister volatile unsigned int IFR;
  extern cregister volatile unsigned int IER;
  extern cregister volatile unsigned int IRP;
  extern cregister volatile unsigned int NRP;
  extern cregister volatile unsigned int AMR;
  extern cregister volatile unsigned int CSR;
  extern cregister volatile unsigned int GFPGFR;
  extern cregister volatile unsigned int DIER;
  extern cregister volatile unsigned int TSR;
  extern cregister volatile unsigned int ITSR;
  extern cregister volatile unsigned int NTSR;
  extern cregister volatile unsigned int EFR;
  extern cregister volatile unsigned int IERR;
  extern cregister volatile unsigned int SSR;
  extern cregister volatile unsigned int ILC;
  extern cregister volatile unsigned int RILC;
  extern cregister volatile unsigned int GPLYA;
  extern cregister volatile unsigned int GPLYB;
  extern cregister volatile unsigned int TSCL;
  extern cregister volatile unsigned int TSCH;
  extern cregister volatile unsigned int DNUM;
  extern cregister volatile unsigned int REP;
  siuC64xPlusReg_t *reg = (siuC64xPlusReg_t *)scratch;
  
  siuStoreRegSets(reg);
  
  reg->footprint = 0xabcdee00;
  reg->istp   = ISTP;
  reg->ifr    = IFR;
  reg->ier    = IER;
  reg->irp    = IRP;
  reg->nrp    = NRP;
  reg->amr    = AMR;
  reg->csr    = CSR;
  reg->gfpgfr = GFPGFR;
  reg->dier   = DIER;
  reg->tsr    = TSR;
  reg->itsr   = ITSR;
  reg->ntsr   = NTSR;
  reg->efr    = EFR;
  reg->ierr   = IERR;
  reg->ssr    = SSR;
  reg->ilc    = ILC;
  reg->rilc   = RILC;
  reg->gplya  = GPLYA;
  reg->gplyb  = GPLYB;
  reg->tscl   = TSCL;
  reg->tsch   = TSCH;
  reg->dnum   = DNUM;
  reg->rep    = REP;
  
  
}

#endif  /* _SIUPORT_H */
/* nothing past this point */


