/******************************************************************************
 * FILE PURPOSE: SIU Data Memory Allocation.
 ******************************************************************************
 * FILE NAME:   siuram.c
 *
 * DESCRIPTION: Used to allocate memory for SIU that depends on channel
 *              number, such as siu instance.
 *
 *
 * TABS: NONE
 *
 * $Id: siuram.c 1.1 1998/11/25 15:46:39 BONAL Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: siuram.c $
 * Revision 1.1  1998/11/25 15:46:39  BONAL
 * Initial revision
 *
 * (C) Copyright 1997, 1998 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"

/* GG header files */
#include "ggswcfg.h"

/* SIU header files */
#include "common/siuloc.h"
#ifdef GG_SWAP_ETOI
#include "siuetoi.h"
#endif

/* Array of pointers to SIU instances for all channels: it may point to statically allocated
 * SIU instance or to channel slot/sub-slot based buffer */

#pragma DATA_SECTION (siuInst, ".siuinst");
#ifdef SIU_DYNSUBSLOT
/* No statically allocated SIU instance */
siuInst_t   *ptrSiuInst[GG_MAX_NUM_CHANNELS + GG_MAX_AUX_PCM_CHANNELS + GG_MAX_PKT2PKT_CHANNELS];
/* Only full channels use statically allocated SIU instance */
siuInst_t   siuInst[GG_MAX_NUM_CHANNELS]; 
#else
/* SIU instance */
/* Put siuInst in its own section because it does not use .cinit
 * so therefore it can be overlayed with cinit                  
 */                                                                                                     
siuInst_t   siuInst[GG_MAX_NUM_CHANNELS + GG_MAX_AUX_PCM_CHANNELS + GG_MAX_PKT2PKT_CHANNELS];
#endif

#if (!defined(HDVIDEO) && !defined(HDCT))
#ifdef GG_SWAP_ETOI
#pragma DATA_SECTION (siuStorageCodeRelocInfo,".siuEtoiData");
static siuChannelReloc_t siuStorageCodeRelocInfo[GG_MAX_NUM_CHANNELS];
siuChannelReloc_t *siuAllCodeRelocInfo = &siuStorageCodeRelocInfo[0];
#else
/* It would be nice if compiler would allow us to have array size of 0 */
siuChannelReloc_t *siuAllCodeRelocInfo = (siuChannelReloc_t *)NULL;
#endif
#endif

/* nothing past this point */
