/*****************************************************************************
 * FILE PURPOSE: DSIU wrapper for NWAL
 *****************************************************************************
 * FILE NAME: dsiunwal.c
 *
 * DESCRIPTION: Provides the wrapper function for NWAL.
 *
 * TABS: None
 *
 * Copyright (C) 2010 Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef PA_SS


#include "dsiunwal.h"

#ifdef DSIU
#include "diagcfg.h" /* for diagCfg */
#include "dsiuloc.h" /* for dsiuContext */
#else
#include "ndu.h"
#endif
#include "dsiunwcfg.h"
#include "util.h"

#ifdef EMU_PROFILE
#include "emureg.h"
#endif

#include "targethw.h" /* for thwCritical */

#ifdef DSIU
#define MSG_TEST
#endif
#ifdef MSG_TEST
void dsiu_send_msg (dmsg_t *msg);
extern bool dsiuBootMacPacketReceived;
#endif

void dsiuBootRcvPacket (void *cxt, word *packet, tuint sizeBytes);
void Osal_writeBackCache (void *blockPtr, UInt32 size);

#define BRINGUP_WORKAROUND

#pragma DATA_SECTION (dsiuNwGlobContext, ".dsiuNwGlobContext");
dsiuNwGlobContext_t   dsiuNwGlobContext;

dsiuNwLocContext_t   dsiuNwLocContext;

/* This must be initialized to DSIU_NW_GLOB_INIT_STATE_MAGIC1 externally! It may be placed in
 * cachable or noncachable memory */
/* Allow this to be placed in any cachable memory with line size up to 128 */
#if 0 /* below code is correct, but fill= isn't working in elf mode */
#pragma DATA_SECTION (dsiuNwGlobInit, ".dsiuNwGlobInit");
#pragma DATA_ALIGN (dsiuNwGlobInit, DSIU_NW_GLOB_INIT_STATE_CACHELINE);
volatile dsiuNwGlobInit_t dsiuNwGlobInit;
#else
#pragma DATA_SECTION (dsiuNwGlobInitConsts, ".dsiuNwGlobInit");
#pragma DATA_ALIGN (dsiuNwGlobInitConsts, DSIU_NW_GLOB_INIT_STATE_CACHELINE);
const tulong dsiuNwGlobInitConsts[32] = {
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1,
  DSIU_NW_GLOB_INIT_STATE_MAGIC1
};
#endif

#define IP_PROTOCOL_NUM_UDP 17

UInt32    dsiu_nwal_mac_id = 0xffffffff;
UInt32    dsiu_nwal_ip_id  = 0xfffffffe;

extern cregister  volatile unsigned int DNUM;

#ifdef MSG_TEST
/**************************************************************************************************
 * FUNCTION PURPOSE: Initialize Diag App structure
 **************************************************************************************************
 * DESCRIPTION:  Initialize Diag App structure
 *
 **************************************************************************************************/
void dsiuInitDiagAppMem2(void)
{
  tint              count=0;
  tuint             offset_b;

  diagApp_t           *diagApp;

  diagApp = &dsiuContext.diagApp;

  diagApp->dspId = 1;

  /* Message route for mac - doesn't really matter for emulator start */
  diagApp->route   = 2; /* 0:UTOPIA, 2:EMAC, 1:HPI */
  diagApp->portNum = 1;

  /* Mac Config - make up the mac info - gig interface */
  diagApp->u.mac.configFlags        = 0x400;
  diagApp->u.mac.macBufferAssign    = 0;
  diagApp->u.mac.msg.flags1         = 0x0983;
  diagApp->u.mac.msg.flags2         = 1;
  diagApp->u.mac.msg.rxPayloadType  = 3;
  diagApp->u.mac.msg.ipOffsetBytes  = 14;
  diagApp->u.mac.msg.udpOffsetBytes = 34;
  diagApp->u.mac.msg.ethOffsetBytes = -1;
  diagApp->u.mac.msg.txMinCellBytes = 64;
  diagApp->u.mac.msg.totalHeaderLenBytes = 42;
  diagApp->u.mac.msg.rxOffsetBytes  = 0;
  diagApp->u.mac.msg.priDiffTag     = 0;

  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,neuproto_ETH_BYTE_OFFSET_DEST_MAC,0x2A2B);/* dest mac msw */
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_DEST_MAC+2),0x2C2D); /* dest mac mw  */
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_DEST_MAC+4),0x2E2F);/* dest mac lsw */

  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,neuproto_ETH_BYTE_OFFSET_SRC_MAC,0x2A2B); /* source mac msw */
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_SRC_MAC+2),0x2C2D); /* source mac mw  */
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_SRC_MAC+4),0x2E2F); /* source mac lsw */

  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_TYPE_LEN),neuproto_ETH_TYPE_IP);/* ethernet type */

  offset_b = diagApp->u.mac.msg.ipOffsetBytes;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage,(offset_b + neuproto_IPV6_BYTE_OFFSET_VER),0x45f0); /* ip version    */
  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x001c);/* ip length     */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x0048);/* ip ident      */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x0000); /* flags and fragment offset */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x4011); /* ip time to live protocol */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x36de); /* ip header checksum */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x6465); /* ip source addr msw  */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x6667); /* ip source addr lsw  */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x6465); /* ip dest addr msw */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x6667); /* ip dest addr lsw */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x0200); /* udp source port  */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x0200); /* udp dest port    */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0x0008); /* udp length       */

  offset_b += 2;
  utlWrite16bits_m((word*)diagApp->u.mac.msg.hdrImage, offset_b,0); /* udp checksum   */

  for(count=0;count < CPU_NCORES;count++)
  {
      diagApp->u.mac.macAddr[count].macAddr[0] = 0x2a2b;
      diagApp->u.mac.macAddr[count].macAddr[1] = 0x2c2d;
      diagApp->u.mac.macAddr[count].macAddr[2] = (0x2e2f + count);

      diagApp->u.mac.ipAddr[count].srcIpHigh = 0x6465;
      diagApp->u.mac.ipAddr[count].srcIpLow  = (0x6667 + count);
  }
}

#endif


/*****************************************************************************
 * FUNCTION PURPOSE: NWAL callback for L4 packets
 *****************************************************************************
 * DESCRIPTION: The function will create NWAL instance which is prerequiite for
 *              any call to NWAL
 *****************************************************************************/
void dsiuNWALRxPktCallback  (nwal_AppId         appHandle,
                             nwalPktInfo_t*     pPktInfo,
                             Cppi_HostDesc*     pHd,
                             bool*              pFreeHd)
{
    UInt8*          pBuf = (UInt8 *)pHd->buffPtr;
    UInt16          bufLen = pPktInfo->ploadLen;

    *pFreeHd = TRUE;

    if((UInt32)(appHandle) == dsiu_nwal_mac_id)
    {
#ifdef DSIU
        UInt16  dummy;
        UInt8   offset_b = 0; /* Ethernet:14,IP:20,UDP:8 */
        tuint           protocol_type;
#endif
        dsiuNwLocContext.numL2PktsRecvd++;
#ifdef DSIU
        /* Only packet expected is the Boot config packet
         * Parse the IP/UDP header and process payload
         */
        /* Skip the offset to the UDP payload */
        /* Check the IP Protocol Type */
        offset_b = neuproto_ETH_BYTE_OFFSET_TYPE_LEN; /* it is 6(minimum) */

        if (utlRead16bits_m (pBuf,neuproto_ETH_BYTE_VLAN_TAG) == neuproto_ETH_TYPE_VLAN) {
            offset_b += 4; /* 4 bytes of VLAN tag */
        }

        protocol_type = utlRead16bits_m (pBuf,offset_b);  /* pointer at type or frame length */
        if (protocol_type <= neuproto_ETH_MAX_LEN_FIELD) { /* offset to length */
            offset_b += 8;   /* offset 3 bytes LLC and 3 bytes SNAP org code */
            protocol_type = utlRead16bits_m (pBuf,offset_b); /* pointer at type */
        }

        if(protocol_type == neuproto_ETH_TYPE_IP)
        {
            offset_b += neuproto_IP_BASIC_HDR_SIZE_BYTES;

        }
        else if(protocol_type == neuproto_ETH_TYPE_IPV6)
        {
            offset_b += neuproto_IPV6_BASIC_HDR_SIZE_BYTES;
        }
        else
        {
            /* Not handling packet other than allowed protocols for Bootconfig
             * Trap here
             */
            while(1);
        }

#ifdef BRINGUP_WORKAROUND
        /* Debug code */
        offset_b = 42;
#endif

        pBuf = pBuf + offset_b;
        bufLen = bufLen - offset_b;
        dsiuBootRcvPacket(&dummy,(word *)(pBuf),bufLen);
#endif
    }
    else if((UInt32)(appHandle) == dsiu_nwal_ip_id)
    {
        dsiuNwLocContext.numL3PktsRecvd++;
#ifdef BRINGUP_WORKAROUND
#ifdef DSIU
    /* Debug code to resend boot config ACK message */
    if (diagCfg.bootAck)
     (*diagCfg.bootAck)(0);
#endif
#endif
    }
    else
    {
        xferPktInfo_t         pktOut;
        neuValidationInfo_t   vInfo;
        word                  *pktInp;
        tuint                 ip_offset_b;
        tuint                 protocol_type;
        tuint                 origSize = bufLen;

          /* Format the cell for shipout */
        pktOut.pktSize = (tint *)&bufLen;
        pktOut.pktIn   = (void *)&pBuf;
        pktOut.type    = XFER_PKT_TYPE_AAL1;  /* Always use aal1 */
        pktOut.srcPort = 0;
        pktOut.npkts   = 1;

        memset(&vInfo,0,sizeof(neuValidationInfo_t));
        pktOut.suppInfo =  &vInfo;

        pktInp = (word*)pBuf;

        ip_offset_b = neuproto_ETH_BYTE_OFFSET_TYPE_LEN; /* it is 6(minimum)
                                                                           * as in DIX Ethernet */
        if (utlRead16bits_m (pktInp,neuproto_ETH_BYTE_VLAN_TAG) == neuproto_ETH_TYPE_VLAN) {
            ip_offset_b += 4; /* 4 bytes of VLAN tag */
        }

        protocol_type = utlRead16bits_m (pktInp,ip_offset_b);  /* pointer at type or frame length */

        /* Get the 8023 offset from the packet */
        vInfo.offset8023 = -1;
        if (protocol_type <= neuproto_ETH_MAX_LEN_FIELD) { /* offset to length */
            vInfo.offset8023 = 0;
            ip_offset_b += 8;   /* offset 3 bytes LLC and 3 bytes SNAP org code */
            protocol_type = utlRead16bits_m (pktInp,ip_offset_b); /* pointer at type */
        }

        vInfo.ip_protocol_type = protocol_type;
        vInfo.ip_offset_b = pPktInfo->l3OffBytes;
        vInfo.l4_protocol_type = pPktInfo->l4ProtoType;
        vInfo.udp_offset_b = pPktInfo->l4OffBytes;
        vInfo.payload_offset_b =   pPktInfo->ploadOffBytes;


        /* Get the destination UDP port for the packet */
        pktInp = &pktInp[UTL_BYTES_TO_WORDS(vInfo.udp_offset_b)];

        vInfo.inst_key = utlRead16bits_m (pktInp,
                                          neuproto_UDP_BYTE_OFFSET_DEST_PORT);
    #ifdef EMU_PROFILE
      EMU_PROFILE_BEGIN (EMU_TASK_ID_RXVOICE,0)
    #endif

    #ifdef DSIU
        (*diagCfg.ctable.neu->neuReceiveInGmac)((void *)appHandle, (xferpktInfo_t *)&pktOut);
    #else
        NDU_pCallTable->pGetDataSecond(0, pBuf, origSize, &pktOut, (void *)appHandle);
    #endif

    #ifdef EMU_PROFILE
      EMU_PROFILE_END()
    #endif

        /* Invalidate cache since the cache line correponding to the packet payload
           has become dirty. Leaving it without invalidate can cause writing
           the current packet payload back to the buffer some time later while the
           same buffer has been used for storing payload for a later packet.
           Because of this, the packet payload of the later packet can be overwritten
           by the old data. */
        Osal_invalidateCache((void *)pBuf, origSize);
        dsiuNwLocContext.numL4PktsRecvd++;
    }
}

/*****************************************************************************
 * FUNCTION PURPOSE: Callback function from NWAL for config request
 *****************************************************************************
 * DESCRIPTION: The function will be called by the NWAL for the configuration
 *              request
 *****************************************************************************/
void dsiuNWALCmdCallBack (nwal_AppId    appHandle,
                          UInt16        trans_id,
                          nwal_RetValue ret)
{
    if(ret != nwal_OK)
    {
        dsiuNwLog ("ERROR: dsiuNWALCmdCallBack returned Error Code %d\n", ret);
        dsiuNwLocContext.numCmdFail++;
    }
    else
    {
        dsiuNwLocContext.numCmdPass++;
        switch(dsiuNwLocContext.transInfos[trans_id].transType)
        {
            case DSIU_NWAL_HANDLE_TRANS_MAC:
            {
                if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_OPEN_PENDING)
                {
                    dsiuNwLog ("MAC Open ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_OPEN;
                }
                else if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_CLOSE_PENDING)
                {
                    dsiuNwLog ("MAC Close ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_IDLE;
                }
                break;
            }
            case DSIU_NWAL_HANDLE_TRANS_IP:
            {
                if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_OPEN_PENDING)
                {
                    dsiuNwLog ("IP Open ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_OPEN;
                }
                else if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_CLOSE_PENDING)
                {
                    dsiuNwLog ("IP Close ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_IDLE;
                }
                break;
            }
            case DSIU_NWAL_HANDLE_TRANS_PORT:
            {
                if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_OPEN_PENDING)
                {
                    dsiuNwLog ("Connection Open ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_OPEN;
                }
                else if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_CLOSE_PENDING)
                {
                    dsiuNwLog ("Connection Close ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_IDLE;
                }
                break;
            }
            case DSIU_NWAL_HANDLE_TRANS_SEC_ASSOC:
            {
                if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_OPEN_PENDING)
                {
                    dsiuNwLog ("Set Security Assoc ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_OPEN;
                }
                else if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_CLOSE_PENDING)
                {
                    dsiuNwLog ("Set Security Assoc  Close ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_IDLE;
                }
                break;
            }
            case DSIU_NWAL_HANDLE_TRANS_SEC_POLICY:
            {
                if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_OPEN_PENDING)
                {
                    dsiuNwLog ("Set Security Policy ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_OPEN;
                }
                else if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_CLOSE_PENDING)
                {
                    dsiuNwLog ("Set Security Policy  Close ACK received for trans_id: %d\n",
                                dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                    dsiuNwLocContext.transInfos[trans_id].state =DSIU_NWAL_HANDLE_STATE_IDLE;
                }
                break;
            }
            default:
            {
                dsiuNwLog ("ERROR: Invalid transaction type %d for trans_id: %d\n",
                    dsiuNwLocContext.transInfos[trans_id].transType,trans_id);
                break;
            }
        }
    }

    if(trans_id != NWAL_TRANSID_SPIN_WAIT)
        dsiuNwLocContext.numPendingCfg--;

    if(dsiuNwLocContext.transInfos[trans_id].state == DSIU_NWAL_HANDLE_STATE_IDLE)
        dsiuNwLocContext.transInfos[trans_id].inUse = FALSE;
}


/*****************************************************************************
 * FUNCTION PURPOSE: Display statistics from PA
 *****************************************************************************
 * DESCRIPTION: The function will be called to collect statistics from PA
 *****************************************************************************/

void dsiuNWALCmdPaStatsReply (  nwal_AppId      appHandle,
                                nwal_TransID_t  trans_id,
                                paSysStats_t    *stats)
{
    memcpy(&dsiuNwGlobContext.paStats,stats,sizeof(paSysStats_t));
    dsiuNwLocContext.numPendingCfg--;
}


/*****************************************************************************
 * FUNCTION PURPOSE: Get Free Transaction Info for configuration request
 *****************************************************************************
 * DESCRIPTION: The function will be called for receiving free transaction info
 *****************************************************************************/
dsiuNwalTransInfo_t* dsiuGetFreeTransInfo (nwal_TransID_t   *pTransId)
{

    tuint           count=0;

    while(1)
    {
        count=0;
        while(count < DSIU_MAX_NUM_TRANS)
        {
            if((dsiuNwLocContext.transInfos[count].inUse) != TRUE)
            {
                dsiuNwLocContext.transInfos[count].inUse = TRUE;
                *pTransId = count;
                return(&dsiuNwLocContext.transInfos[count]);
            }
            count++;
        }

        /* Poll NWAL for the command response */
        if(dsiuNwLocContext.numPendingCfg)
        {
            /*dsiuNwLog ("Polling to free pending transactions \n");*/
            nwal_poll(dsiuNwGlobContext.nwalInstHandle,nwal_POLL_CTL_Q);
        }
    }
}

/*****************************************************************************
 * FUNCTION PURPOSE: Configure MAC at NWAL
 *****************************************************************************
 * DESCRIPTION: The function will be called to configure MAC instance
 *****************************************************************************/
nwal_Handle  dsiuAddMac    (nwalMacParam_t     *pMacInfo)
{
    nwal_RetValue       retValue;
    dsiuNwalTransInfo_t *pTransInfo;
    nwal_TransID_t      trans_id;


    pTransInfo = dsiuGetFreeTransInfo(&trans_id);
    pTransInfo->transType = DSIU_NWAL_HANDLE_TRANS_MAC;
    retValue = nwal_setMacIface(  dsiuNwGlobContext.nwalInstHandle,
                                  trans_id,
                                  (nwal_AppId)dsiu_nwal_mac_id,
                                  pMacInfo,
                                  &pTransInfo->handle);
    if(retValue !=  nwal_OK)
    {
        dsiuNwLog ("ERROR: nwal_setMacIface returned Error Code %d\n",
                    retValue);
        return NULL;
    }
    pTransInfo->inUse = FALSE;
    dsiuNwLog ("MAC Open sent with  trans_id: %d\n",trans_id);

    if(trans_id != NWAL_TRANSID_SPIN_WAIT)
    {
        dsiuNwLocContext.numPendingCfg++;
        while(dsiuNwLocContext.numPendingCfg)
        {
            nwal_poll(dsiuNwGlobContext.nwalInstHandle,nwal_POLL_CTL_Q);
        }
    }

#ifndef DSIU
    /* Update Switch ALE for the MAC Address so that packet will be routed to PA */
    if(dsiuNwSwUpdateMacAddr(pMacInfo->macAddr))
    {
        dsiuNwLog ("CPSW ALE table updated for the destination MAC address \n");
    }
#endif

    return(pTransInfo->handle);
}


/*****************************************************************************
 * FUNCTION PURPOSE: Configure IP at NWAL
 *****************************************************************************
 * DESCRIPTION: The function will be called to configure MAC instance
 *****************************************************************************/
nwal_Handle  dsiuAddIp  (nwalIpParam_t* pIpParam,
                         nwal_Handle    macHandle)
{
    nwal_RetValue           retValue;
    dsiuNwalTransInfo_t*    pTransInfo;
    nwal_TransID_t                trans_id;

    pTransInfo = dsiuGetFreeTransInfo(&trans_id);
    pTransInfo->transType = DSIU_NWAL_HANDLE_TRANS_IP;

    retValue = nwal_setIPAddr(    dsiuNwGlobContext.nwalInstHandle,
                                  trans_id,
                                  (nwal_AppId)dsiu_nwal_ip_id,
                                  macHandle,
                                  pIpParam,
                                  &pTransInfo->handle);
    if(retValue !=  nwal_OK)
    {
        dsiuNwLog ("ERROR: nwal_assignIPAddr returned Error Code %d\n",
                    retValue);
        return NULL;
    }
    pTransInfo->inUse = FALSE;

    dsiuNwLog ("IP Open sent with  trans_id: %d\n",trans_id);
    if(trans_id != NWAL_TRANSID_SPIN_WAIT)
    {
        dsiuNwLocContext.numPendingCfg++;
        while(dsiuNwLocContext.numPendingCfg)
        {
            nwal_poll(dsiuNwGlobContext.nwalInstHandle,nwal_POLL_CTL_Q);
        }
    }
    return(pTransInfo->handle);
}

/*****************************************************************************
 * FUNCTION PURPOSE: Configure UDP at NWAL
 *****************************************************************************
 * DESCRIPTION: The function will be called to configure MAC instance
 *****************************************************************************/
nwal_Handle dsiuConfigUdp   (nwal_Handle      inHandle,
                             uint16_t         tx_src_port,
                             void             *pNeuInst)
{
    nwal_RetValue       retValue;
    dsiuNwalTransInfo_t *pTransInfo;
    nwal_TransID_t      trans_id;
    nwalLocConnCfg_t    nwalLocConn;

	pTransInfo = dsiuGetFreeTransInfo(&trans_id);

    if ((tx_src_port < 1024) || ((tx_src_port % 32) == 0)) {
      dsiuNwLog ("Adding UDP port: %d \n",tx_src_port);
    }

    memset(&nwalLocConn, 0, sizeof(nwalLocConnCfg_t));
    nwalLocConn.inHandle = inHandle;
    nwalLocConn.appProto.udpPort = tx_src_port;
    nwalLocConn.rxCoreId =  DNUM;

    retValue = nwal_addConn(dsiuNwGlobContext.nwalInstHandle,
                            trans_id,
                            (nwal_AppId)(pNeuInst),
                            NWAL_APP_PLOAD_PROTO_UDP,
                            &nwalLocConn,
                            NULL,
                            &pTransInfo->handle);

    if(retValue !=  nwal_OK)
    {
        dsiuNwLog ("ERROR: nwal_addConn returned Error Code %d \n",
                    retValue);
        pTransInfo->inUse = FALSE;
        return FALSE;
    }

    pTransInfo->transType = DSIU_NWAL_HANDLE_TRANS_PORT;
    dsiuNwLog ("Port Connection Open sent with  trans_id: %d\n",trans_id);
    if(trans_id != NWAL_TRANSID_SPIN_WAIT)
    {
        dsiuNwLocContext.numPendingCfg++;
        while(dsiuNwLocContext.numPendingCfg)
        {
            nwal_poll(dsiuNwGlobContext.nwalInstHandle,nwal_POLL_CTL_Q);
        }
    }
    return(pTransInfo->handle);

}


/*****************************************************************************
 * FUNCTION PURPOSE: Configure MAC/IP/UDP based on the NET_ENCAP
 *****************************************************************************
 * DESCRIPTION: The function will be called along with the NET_ENCAP configuration
 *              at NEU for adding L2/L3/L4 classification at NWAL
 *****************************************************************************/
bool dsiuConfigL2L3L4 (void             *pNeuInst,
                       neuChlParms_t    *pChnl,
                       nwal_Handle      *udpHandle,
                       nwal_Handle      *udpHandle2,
                       nwal_Handle      *udpHandle3)
{
    word            *pktInp;
    tuint           protocol_type;
    tuint           offset_b;
    word            *pIpdata;
    word            *pUdpdata;
    uint16_t        tx_src_port;

    nwalMacParam_t nwalMacParam;
    nwalIpParam_t  nwalIpParam;

    memset(&nwalMacParam, 0, sizeof(nwalMacParam_t));
    memcpy((void*)&nwalMacParam.macAddr, &(pChnl->header[UTL_BYTES_TO_WORDS(neuproto_ETH_BYTE_OFFSET_SRC_MAC)]), sizeof(nwalMacAddr_t));
    nwalMacParam.nextRtfail = NWAL_NEXT_ROUTE_FAIL_ACTION_DISCARD;

    /* Check if NWAL MAC instance is already created */
    if(!nwal_getMacIface(dsiuNwGlobContext.nwalInstHandle,
                         &nwalMacParam,
                         &(dsiuNwLocContext.nwalMacHandle)))
    {
        /* Not found */
        dsiuNwLog ("Core %d: Adding MAC entry \n",DNUM);
        (dsiuNwLocContext.nwalMacHandle) = dsiuAddMac(&nwalMacParam);
        if((dsiuNwLocContext.nwalMacHandle) == NULL)
        {
            return FALSE;
        }
    }
    else
    {
        dsiuNwLog ("Using Existing MAC handle for Core: %d\n",DNUM);
    }


    pktInp = pChnl->header;
    pIpdata = &pChnl->header[UTL_BYTES_TO_WORDS(pChnl->tx_ip_offset_bytes)];

    /* Check the IP Protocol Type */
    offset_b = neuproto_ETH_BYTE_OFFSET_TYPE_LEN; /* it is 6(minimum)
                                                                       * as in DIX Ethernet */
    if (utlRead16bits_m (pktInp,neuproto_ETH_BYTE_VLAN_TAG) == neuproto_ETH_TYPE_VLAN) {
        offset_b += 4; /* 4 bytes of VLAN tag */
    }

    protocol_type = utlRead16bits_m (pktInp,offset_b);  /* pointer at type or frame length */
    if (protocol_type <= neuproto_ETH_MAX_LEN_FIELD) { /* offset to length */
        offset_b += 8;   /* offset 3 bytes LLC and 3 bytes SNAP org code */
        protocol_type = utlRead16bits_m (pktInp,offset_b); /* pointer at type */
    }

    memset(&nwalIpParam, 0, sizeof(nwalIpParam_t));
    if(protocol_type == neuproto_ETH_TYPE_IP)
    {
        nwalIpParam.ipType = pa_IPV4;
        memcpy((void*)&(nwalIpParam.locIpAddr),
               (UChar *)(&pIpdata[UTL_BYTES_TO_WORDS(neuproto_IP_BYTE_OFFSET_SRC_ADDR)]),
               NWAL_IPV4_ADDR_SIZE);
    }
    else if(protocol_type == neuproto_ETH_TYPE_IPV6)
    {
        nwalIpParam.ipType = pa_IPV6;
        memcpy((void*)&(nwalIpParam.locIpAddr),
               (UChar *)(&pIpdata[UTL_BYTES_TO_WORDS(neuproto_IP_BYTE_OFFSET_SRC_ADDR)]),
               NWAL_IPV6_ADDR_SIZE);
    }
    else
    {
        dsiuNwLog ("Invalid IP type: 0x%x, IPV4 Type: 0x%x, IPV6:0x%x \n",
                    protocol_type,
                    NWAL_IPV4_ADDR_SIZE,
                    NWAL_IPV6_ADDR_SIZE);

        /* Error */
        return FALSE;
    }

    nwalIpParam.ipOpt.validParams = NWAL_IP_OPT_VALID_PARAMS_L4_PROTO;
    nwalIpParam.ipOpt.proto = IP_PROTOCOL_NUM_UDP;
    nwalIpParam.ipOpt.tos = 0;
    nwalIpParam.ipOpt.flowLabel = 0;
    nwalIpParam.nextRtfail = NWAL_NEXT_ROUTE_FAIL_ACTION_DISCARD;

    /* Check if IP address is already configured */
    if(!nwal_getIPAddr(dsiuNwGlobContext.nwalInstHandle,
                       &nwalIpParam,
                       (dsiuNwLocContext.nwalMacHandle),
                       &(dsiuNwLocContext.nwalIpHandle)))
    {
        /* Not found */
    	dsiuNwLog ("Core %d: Adding IP entry - SRC=[%d, %d, %d, %d] \n",
                    DNUM,
                    nwalIpParam.locIpAddr.ipv4[0],
                    nwalIpParam.locIpAddr.ipv4[1],
                    nwalIpParam.locIpAddr.ipv4[2],
                    nwalIpParam.locIpAddr.ipv4[3]);


        (dsiuNwLocContext.nwalIpHandle) = dsiuAddIp(&nwalIpParam, (dsiuNwLocContext.nwalMacHandle));
        if((dsiuNwLocContext.nwalIpHandle) == NULL)
        {
            return FALSE;
        }
    }
    else
    {
        dsiuNwLog ("Using Existing IP handle for Core: %d\n",DNUM);
    }
    /* Set the global state to indicate that global common resource allocation is complete */
    dsiuNwGlobContext.state = DSIU_NW_CXT_GLOB_RES_ALLOC_COMPLETE;
    Osal_writeBackCache(&dsiuNwGlobContext,sizeof(dsiuNwGlobContext));


    pUdpdata = &pChnl->header[UTL_BYTES_TO_WORDS(pChnl->tx_udp_offset_bytes)];
    tx_src_port =
        utlRead16bits_m(pUdpdata,
                        neuproto_UDP_BYTE_OFFSET_SRC_PORT);


    /* Add UDP entry */
    dsiuNwLog ("Core %d: Adding UDP entry \n", DNUM);
    *udpHandle = dsiuConfigUdp((dsiuNwLocContext.nwalIpHandle), tx_src_port, pNeuInst);
    if(*udpHandle == NULL)
    {
        dsiuNwLog ("Core %d: Error adding UDP entry: 0x%x \n", DNUM,tx_src_port);
        return FALSE;
    }

    if(udpHandle2 != NULL)
    {
        tx_src_port++;
        *udpHandle2 = dsiuConfigUdp((dsiuNwLocContext.nwalIpHandle), tx_src_port, pNeuInst);
        if(*udpHandle2 == NULL)
        {
            dsiuNwLog ("Core %d: Error adding UDP entry: 0x%x \n", DNUM,tx_src_port);
            return FALSE;
        }
    }

    if(udpHandle3 != NULL)
    {
        tx_src_port++;
        *udpHandle3 = dsiuConfigUdp((dsiuNwLocContext.nwalIpHandle), tx_src_port, pNeuInst);
        if(*udpHandle3 == NULL)
        {
            dsiuNwLog ("Core %d: Error adding UDP entry: 0x%x \n", DNUM,tx_src_port);
            return FALSE;
        }
    }
    return TRUE;
}

#ifdef MSG_TEST
/******************************************************************************
 * FUNCTION PURPOSE: Send a Test Loopback message for Status Request
 ******************************************************************************
 * DESCRIPTION: Simulate a status request from host
 ******************************************************************************/
void dsiuInfiniteLoop ()
{
    while(1);
}


static tuint first =0;
volatile UInt32 clearPAStats = 1;
volatile UInt32 getPAStats = 1;

/******************************************************************************
 * FUNCTION PURPOSE: Send a Test Loopback message for Status Request
 ******************************************************************************
 * DESCRIPTION: Simulate a status request from host
 ******************************************************************************/
void dsiuMsgloopBkTest ()
{

  tuint *q;

  if(clearPAStats || getPAStats)
  {
      memset(&dsiuNwGlobContext.paStats,0,sizeof(paSysStats_t));
      nwal_getPAStats(&dsiuNwGlobContext.nwalInstHandle,
                      0,
                      &dsiuNwGlobContext.paStats,
                      clearPAStats);
      clearPAStats = 0;
      getPAStats = 0;
  }

  if(first ==0)
  {
      return;
  }

  if(first == 1)
  {
    /* Generate a READ MEM Request */
      dsiuContext.msgTxBuf->header.length   = 12;
      dsiuContext.msgTxBuf->header.chnum    = 0;
      dsiuContext.msgTxBuf->header.msg_id   = 55;
      dsiuContext.msgTxBuf->header.trans_id = 0;

      /* Echo back any parameters in the message body */
      q = (tuint *)&(dsiuContext.msgTxBuf->msg_body);
      *q = 0;
      q++;

      *q = 0;
      q++;

      *q = 0;
      q++;

      *q = 0;
      q++;

      *q = 4;

      *q = 0;
      q++;

      *q = 0;


      dsiuContext.msgTxBuf->msg_body.req_mem_write.addr_msw = 0x83;
      dsiuContext.msgTxBuf->msg_body.req_mem_write.addr_lsw = 0xFCF0;
      dsiuContext.msgTxBuf->msg_body.req_mem_write.mem_type = DMSG_MEM_TYPE_PROG;
      dsiuContext.msgTxBuf->msg_body.req_mem_write.length_words = 4;
      q = (tuint *)&(dsiuContext.msgTxBuf->msg_body.req_mem_write.first_word);
      *q = 0x004;
      q++;

      *q = 0xA120;

      /* Send the message */
      dsiu_send_msg (dsiuContext.msgTxBuf);
      first = 0;
  }


  if(first == 2)
  {
    /* Generate a BOOT_CORE_CONTROL Request */
      dsiuContext.msgTxBuf->header.length   = 9;
      dsiuContext.msgTxBuf->header.chnum    = 0;
      dsiuContext.msgTxBuf->header.msg_id   = 1045;
      dsiuContext.msgTxBuf->header.trans_id = 0;

      dsiuContext.msgTxBuf->msg_body.boot_core_control.enable_bitmask = 0x1;
      dsiuContext.msgTxBuf->msg_body.boot_core_control.reset_bitmask = 0x0;
      dsiuContext.msgTxBuf->msg_body.boot_core_control.addr_msw = 0x83;
      dsiuContext.msgTxBuf->msg_body.boot_core_control.addr_lsw = 0xFCF0;

       /* Send the message */
      dsiu_send_msg (dsiuContext.msgTxBuf);
      first = 0;
  }

   if(first == 3)
  {
    /* Test Infinite loop */
      dsiuInfiniteLoop();
      first = 0;
   }

   if(first == 4)
   {
       dsiuInitDiagAppMem2();
       dsiuBootMacPacketReceived = TRUE;
       dsiuContext.dregMsgCfg->route = DIAGAPP_ROUTE_GMAC;
       first = 0;

   }

   if(first == 5)
   {
       tuint                msg_buf[150];
       tuint*               pmsg_buf;
       diagApp_t           *diagApp;
       nwal_RetValue        retValue;

       diagApp = &dsiuContext.diagApp;

       dsiuInitDiagAppMem2();
       memcpy(msg_buf,diagApp->u.mac.msg.hdrImage,diagApp->u.mac.msg.totalHeaderLenBytes);
       pmsg_buf = &msg_buf[0];

       pmsg_buf = pmsg_buf + (diagApp->u.mac.msg.udpOffsetBytes/2);
       pmsg_buf += 2;
       *pmsg_buf = 274;
       retValue = nwal_sendRaw(dsiuNwGlobContext.nwalInstHandle,
                               300,
                               (UInt8*)msg_buf);
       if(retValue != nwal_OK)
       {
          dsiuNwLog ("ERROR: nwal_sendRaw returned Error Code %d \n",
                    retValue);
          while(1);
       }

       first = 0;
   }



}
#endif

/*****************************************************************************
 * FUNCTION PURPOSE: Polls different RX queues for the packets or command responses
 *****************************************************************************
 * DESCRIPTION: The function will be called for polling messages/packets
 *              TBD: The function will be replaced with the interrupt accumulator
 *              as a next phase
 *****************************************************************************/
void dsiuNwalPoll (void )
{
#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN (EMU_TASK_ID_GMACPOLL,0)
#endif
    thwCriticalBegin();

    if((dsiuNwLocContext.state & DSIU_NW_CXT_LOC_ACTIVE) != DSIU_NW_CXT_LOC_ACTIVE)
    {
        /* Resourcse not yet initialized.
         * Return
         */
        goto end;
    }

    if(!DNUM)
    {
        /* Poll for common L2/L3 packets */
        nwal_poll(dsiuNwGlobContext.nwalInstHandle,nwal_POLL_DEFAULT_GLOB_PKT_Q);
    }

    nwal_poll(dsiuNwGlobContext.nwalInstHandle,(nwal_POLL_CTL_Q|nwal_POLL_DEFAULT_LOC_PKT_Q));

#ifdef EMU_PROFILE
  EMU_PROFILE_END()
#endif


#ifdef MSG_TEST
    dsiuMsgloopBkTest ();
#endif

end:
  thwCriticalEnd();
  return;

}

/*****************************************************************************
 * FUNCTION PURPOSE: Send out packet to NWAL
 *****************************************************************************
 * DESCRIPTION: The function will be called to send packets out
 *****************************************************************************/
tint  dsiuPktSendOut (void *targetInst, xferPktInfo_t *pkt)
{
    tint              i;
    nwal_RetValue   retValue;

    for (i = 0; i < pkt->npkts; i++)  {
        retValue = nwal_sendRaw(dsiuNwGlobContext.nwalInstHandle,
                       (UInt16)(*pkt->pktSize),
                       (UInt8*)(pkt->pktIn[i]));

      if(retValue != nwal_OK)
      {
          dsiuNwLog ("ERROR: nwal_sendRaw returned Error Code %d \n",
                    retValue);
          dsiuNwLocContext.numL4PktsSendFails += 1;
      }
      else
      {
          dsiuNwLocContext.numL4PktsSent++;
      }
    }

    return (0);
}
#else
int dummy; /* no warning */
#endif
