/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/**************************************************************************************************
 * FILE PURPOSE: Header file for using NWAL
 **************************************************************************************************
 * FILE NAME: dsiunwal.h
 *
 * DESCRIPTION: 
 *
 **************************************************************************************************/

#ifndef _DSIUNWAL_H
#define _DSIUNWAL_H


#include <xdc/std.h>
#include <stdio.h>
#ifdef DSIU
#include "diagbld.h"
#endif
#include "utlxferpkt.h"

#include "types.h"
#include <ti/mas/neu/neu.h>
#include "utlrepck_m.h"
#include "utlrepck.h"
#include <string.h>

#include <ti/drv/nwal/nwal.h>

/* Define to enable descriptors and buffer memory being located in global MSMC/DDR memory.Linker command file need 
 * to reflect the memory placement
 */
#ifdef DSIU
#undef NWAL_DSIU_DESC_GLOB_MEM
#undef NWAL_DSIU_DESC_BUF_GLOB_MEM
#else
#define NWAL_DSIU_DESC_GLOB_MEM
#define NWAL_DSIU_DESC_BUF_GLOB_MEM
#endif

#ifdef DSIU
#define DSIU_MAX_NUM_MAC        1
#define DSIU_MAX_NUM_IP         1
#define DSIU_MAX_NUM_PORTS_PER_CORE      (4) /* 4 UDP ports for boot loader */
#define DSIU_MAX_NUM_PORTS              (DSIU_MAX_NUM_PORTS_PER_CORE) /* Master core only for boot loader */
#define DSIU_MAX_NUM_L2_L3_HDRS   1 
#else
#define DSIU_MAX_NUM_MAC        16
#define DSIU_MAX_NUM_IP         16
#define DSIU_MAX_NUM_PORTS_PER_CORE      (SIU_MAX_TDM_CHANNELS*2+3) /* (max number of channels*2+msg ports)*/
#define DSIU_MAX_NUM_PORTS              (DSIU_MAX_NUM_PORTS_PER_CORE * CPU_NCORES) 
#define DSIU_MAX_NUM_L2_L3_HDRS   16 
#endif

#define DSIU_MAX_NUM_IPSEC_CHANNELS 0

#define DSIU_MAX_NUM_TRANS  (DSIU_MAX_NUM_MAC + DSIU_MAX_NUM_IP + DSIU_MAX_NUM_PORTS + DSIU_MAX_NUM_IPSEC_CHANNELS)

#define DSIU_NWAL_DEBUG 1

/*****************************************************************************
 * Global Common Reources per Chip
 *****************************************************************************/ 

#define DSIU_CONFIG_MAX_DESC_NUM            16384   /* Maximum Number of descriptors configured */
#define DSIU_CONFIG_MAX_PA_TO_SA_DESC       0
#define DSIU_CONFIG_MAX_SA_TO_PA_DESC       0
#define DSIU_CONFIG_MAX_GLOB_DESC           (DSIU_CONFIG_MAX_PA_TO_SA_DESC + DSIU_CONFIG_MAX_SA_TO_PA_DESC)

#define DSIU_CONFIG_BASE_QM_GLOBAL_MSMC_MEMORY_REGION   Qmss_MemRegion_MEMORY_REGION0
#define DSIU_CONFIG_BASE_QM_MEMORY_REGION               Qmss_MemRegion_MEMORY_REGION1
#define DSIU_CONFIG_BASE_QM_GLOBAL_DDR_MEMORY_REGION    Qmss_MemRegion_MEMORY_REGION17

/******************************************************************************
 * DATA DEFINITION: NWAL Transaction Info structure being maintained in App
 *****************************************************************************/
#define DSIU_NWAL_HANDLE_TRANS_NONE             0
#define DSIU_NWAL_HANDLE_TRANS_MAC              1
#define DSIU_NWAL_HANDLE_TRANS_IP               2
#define DSIU_NWAL_HANDLE_TRANS_PORT             3
#define DSIU_NWAL_HANDLE_TRANS_SEC_ASSOC        4
#define DSIU_NWAL_HANDLE_TRANS_SEC_POLICY       5


#define DSIU_NWAL_HANDLE_STATE_IDLE             0
#define DSIU_NWAL_HANDLE_STATE_OPEN_PENDING     1
#define DSIU_NWAL_HANDLE_STATE_OPEN             2
#define DSIU_NWAL_HANDLE_STATE_CLOSE_PENDING    3
typedef struct {
    bool                    inUse;
    uint16_t                transType;
    uint16_t                state;
    nwal_Handle             handle;
    uint64_t                transId;

}dsiuNwalTransInfo_t;

/******************************************************************************
 * DATA DEFINITION: dsiuNwLocContext structure. Resource used per core
 *****************************************************************************/
typedef struct { 
    uint16_t            state;
#define DSIU_NW_CXT_LOC_INACTIVE     0x0
#define DSIU_NW_CXT_LOC_ACTIVE       0x2

    Qmss_QueueHnd       descQ;    /* Queue holding unassigned descriptors */
    tuint               numPendingCfg;
    tuint               numCmdFail;
    tulong              numCmdPass;
    tulong              numL2PktsRecvd; 
    tulong              numL3PktsRecvd; 
    tulong              numL4PktsRecvd; 
    tulong              numL4PktsSent;      
    tulong              numL4PktsSendFails;
    nwalLocCfg_t        nwalLocCfg;
    dsiuNwalTransInfo_t transInfos[DSIU_MAX_NUM_TRANS];
    nwal_Handle    nwalMacHandle;
    nwal_Handle    nwalIpHandle;
} dsiuNwLocContext_t;
extern dsiuNwLocContext_t   dsiuNwLocContext;

/******************************************************************************
 * DATA DEFINITION: dsiuNwGlobContext structure
 *****************************************************************************/
typedef struct {   
    uint16_t               state;
#define DSIU_NW_CXT_GLOB_INACTIVE               0x0
#define DSIU_NW_CXT_GLOB_ACTIVE                 0x1
#define DSIU_NW_CXT_GLOB_RES_ALLOC_COMPLETE     0x3

    nwal_Handle         nwalInstHandle;    
    Qmss_QueueHnd       descQ;    /* Queue holding unassigned descriptors */
    Qmss_QueueHnd       descExtMemQ;
    paSysStats_t        paStats;

}dsiuNwGlobContext_t;

extern dsiuNwGlobContext_t   dsiuNwGlobContext;

#define DSIU_NW_GLOB_INIT_STATE_MAGIC1 0x167eb0e7
#define DSIU_NW_GLOB_INIT_STATE_MAGIC2 0x41c67ea6
/* Maximum cache line size (for align and pad) */
#define DSIU_NW_GLOB_INIT_STATE_CACHELINE 128
typedef struct {
  tulong state;
  /* pad out the cache line */
  char pad[DSIU_NW_GLOB_INIT_STATE_CACHELINE-sizeof(tulong)];
} dsiuNwGlobInit_t;
extern volatile dsiuNwGlobInit_t dsiuNwGlobInit;

bool dsiuNwInit (void);
bool dsiuNwSwUpdateMacAddr(UInt8 macAddress[6]);

bool dsiuConfigL2L3L4 (void             *pNeuInst,
                       neuChlParms_t    *pChnl,
                       nwal_Handle      *udpHandle,
                       nwal_Handle      *udpHandle2,
                       nwal_Handle      *udpHandle3);
void dsiuNwalPoll (void );

tint  dsiuPktSendOut (void *targetInst, xferPktInfo_t *pkt);
void dsiuNWALCmdCallBack (nwal_AppId   appHandle, 
                          UInt16        trans_id,
                          nwal_RetValue ret);

void dsiuNWALRxPktCallback  (nwal_AppId         appHandle,
                             nwalPktInfo_t*     pPktInfo,
                             Cppi_HostDesc*     pHd,
                             bool*              pFreeHd);

void dsiuNWALCmdPaStatsReply (nwal_AppId   appHandle, 
                              nwal_TransID_t   trans_id,
                              paSysStats_t    *stats);

#if defined(DSIU_NWAL_DEBUG) && defined(DSIU)
void dsiuNwLog ( const char* format, ... );
#else
/* Inline stub to make it all go away including the const strings */
static inline void dsiuNwLog ( const char* format, ... )
{
}
#endif
#endif

/* Nothing past this point */

