/*****************************************************************************
 * FILE PURPOSE: DSIU file containing initialization of Network resources
 *****************************************************************************
 * FILE NAME: dsiunwinit.c
 *
 * DESCRIPTION: Functions to initialize framework resources for running NWAL
 *
 * TABS: None
 *
 * Copyright (C) 2010 Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef PA_SS

#include "dsiunwal.h"
#include "dsiunwcfg.h"

#ifdef DSIU
/* For dsiuSimConfig */
#include "dsiuotbrun.h"
#endif

/* CSL RL includes */
#include <ti/csl/cslr_device.h>
#include <ti/csl/cslr_qm_config.h>
#include <ti/csl/cslr_qm_descriptor_region_config.h>
#include <ti/csl/cslr_qm_queue_management.h>
#include <ti/csl/cslr_qm_queue_status_config.h>
#include <ti/csl/cslr_qm_intd.h>
#include <ti/csl/cslr_pdsp.h>
#include <ti/csl/csl_qm_queue.h>
#include <ti/csl/cslr_cppidma_global_config.h>
#include <ti/csl/cslr_cppidma_rx_channel_config.h>
#include <ti/csl/cslr_cppidma_rx_flow_config.h>
#include <ti/csl/cslr_cppidma_tx_channel_config.h>
#include <ti/csl/cslr_cppidma_tx_scheduler_config.h>
#include <ti/csl/csl_cppi.h>
#include <ti/csl/csl_pscAux.h>
#include <ti/csl/csl_chip.h>
#include <ti/csl/csl_semAux.h>
#include <ti/csl/csl_cacheAux.h>
#include <ti/csl/csl_xmcAux.h>
#include <ti/csl/csl_cpsw_3gfAux.h>
#include <ti/csl/csl_cpsw.h>
#include <ti/csl/csl_cpsgmiiAux.h>

#include <c6x.h>
#include <stdlib.h>
#include <stdio.h>

/* TBD: Need to be moved to shannon specific file.
 * CSL Semaphore module includes
 */
#include <ti/csl/csl_semAux.h>
#include <ti/drv/qmss/qmss_qm.h>
#include <ti/drv/qmss/device/qmss_device.c>
#include <ti/drv/cppi/device/cppi_device.c>

Void Osal_MtCsEnter(UInt32 *key);
Void Osal_MtCsExit(UInt32 *key);
void Osal_writeBackCache (void *blockPtr, UInt32 size);
void Osal_invalidateCache (void *blockPtr, UInt32 size);
unsigned int Osal_nwalLocToGlobAddr(unsigned int x);


/*** NWAL Memory Buffer Configuration ***/
#define DSIU_CONFIG_BUFSIZE_NWAL_HANDLE     3400
#pragma DATA_SECTION (nwalInstMem, ".nwalInstMem")
#pragma DATA_ALIGN(nwalInstMem, CACHE_L2_LINESIZE)
UInt8 nwalInstMem[DSIU_CONFIG_BUFSIZE_NWAL_HANDLE];

#define DSIU_CONFIG_BUFSIZE_NWAL_PER_MAC                    128
#define DSIU_CONFIG_BUFSIZE_NWAL_IPSEC_HANDLE_PER_CHAN      0
#define DSIU_CONFIG_BUFSIZE_NWAL_PER_IP                     128
#define DSIU_CONFIG_BUFSIZE_NWAL_PER_PORT                   128
#define DSIU_CONFIG_BUFSIZE_NWAL_PER_L2L3_HDR               128
#define TEST_CONFIG_BUFSIZE_NWAL_PER_LOC_CONTEXT            384
#define NWAL_CHAN_HANDLE_SIZE    ((DSIU_CONFIG_BUFSIZE_NWAL_PER_MAC * DSIU_MAX_NUM_MAC) + \
                                  (DSIU_CONFIG_BUFSIZE_NWAL_IPSEC_HANDLE_PER_CHAN * DSIU_MAX_NUM_IPSEC_CHANNELS) + \
                                  (DSIU_CONFIG_BUFSIZE_NWAL_PER_IP * DSIU_MAX_NUM_IP) + \
                                  (DSIU_CONFIG_BUFSIZE_NWAL_PER_PORT * DSIU_MAX_NUM_PORTS) + \
                                  (TEST_CONFIG_BUFSIZE_NWAL_PER_LOC_CONTEXT * CPU_NCORES) + \
                                  (DSIU_CONFIG_BUFSIZE_NWAL_PER_L2L3_HDR * DSIU_MAX_NUM_L2_L3_HDRS))

#pragma DATA_SECTION (nwalHandleMem, ".nwalHandleMem")
#pragma DATA_ALIGN(nwalHandleMem, CACHE_L2_LINESIZE)
UInt8 nwalHandleMem[NWAL_CHAN_HANDLE_SIZE];

/* TBD: Check if below size information can be made available from pa interface file */
#define DSIU_CONFIG_BUFSIZE_PA_BUF0      128
/* Memory used for PA */
#pragma DATA_SECTION (paBuf0, ".paBuf0")
#pragma DATA_ALIGN(paBuf0, CACHE_L2_LINESIZE)
UInt8 paBuf0[DSIU_CONFIG_BUFSIZE_PA_BUF0];

#ifdef DSIU
#define DSIU_CONFIG_BUFSIZE_PA_BUF1    128
#else
#define DSIU_CONFIG_BUFSIZE_PA_BUF1    (32*DSIU_MAX_NUM_MAC)
#endif
#pragma DATA_SECTION (paBuf1, ".paBuf1")
#pragma DATA_ALIGN(paBuf1, CACHE_L2_LINESIZE)
UInt8 paBuf1[DSIU_CONFIG_BUFSIZE_PA_BUF1];

#ifdef DSIU
#define DSIU_CONFIG_BUFSIZE_PA_BUF2    128
#else
#define DSIU_CONFIG_BUFSIZE_PA_BUF2    (64*DSIU_MAX_NUM_IP)
#endif
#pragma DATA_SECTION (paBuf2, ".paBuf2")
#pragma DATA_ALIGN(paBuf2, CACHE_L2_LINESIZE)
UInt8 paBuf2[DSIU_CONFIG_BUFSIZE_PA_BUF2];

/*****************************************************************************
 * FUNCTION PURPOSE: Log Routine
 *****************************************************************************
 * DESCRIPTION: The function will be used to log as a wrapper for dsiuNwLogs
 *****************************************************************************/
#if defined(DSIU_NWAL_DEBUG) && defined(DSIU)
void dsiuNwLog ( const char* format, ... ) {
#ifdef DSIU_SIMULATE_APPLICATION
   if(dsiuSimConfig.opt_diag_otb_BERT)
   {
         va_list args;
         fprintf(stdout, "LOG: ");
         va_start( args, format );
         vfprintf( stdout, format, args );
         va_end( args );
   }
#endif
}
#endif


/***************************************************************************************
 * FUNCTION PURPOSE: Do switch related initializations
 ***************************************************************************************
 * DESCRIPTION: Add switch related initializations
 ***************************************************************************************/
void dsiuNwCPSWInit(void)
{
    CSL_CPSW_3GF_ALE_PORTCONTROL        alePortControlCfg;

    CSL_CPSW_3GF_clearAleTable();

    alePortControlCfg.dropUntaggedEnable    =   0;
    alePortControlCfg.vidIngressCheckEnable =   0;

    alePortControlCfg.mcastLimit            =   0;
    alePortControlCfg.bcastLimit            =   0;

    /* Disable learning mode for Port 0 */
    alePortControlCfg.noLearnModeEnable     =   1;
    alePortControlCfg.portState     =   ALE_PORTSTATE_FORWARD;
    CSL_CPSW_3GF_setAlePortControlReg (0, &alePortControlCfg);

    /* Enable learning mode for Port 1 */
    alePortControlCfg.noLearnModeEnable     =   0;
    alePortControlCfg.portState     =   ALE_PORTSTATE_FORWARD;
    CSL_CPSW_3GF_setAlePortControlReg (1, &alePortControlCfg);

    /* Enable learning mode for Port 2 */
    alePortControlCfg.noLearnModeEnable     =   0;
    alePortControlCfg.portState     =   ALE_PORTSTATE_FORWARD;
    CSL_CPSW_3GF_setAlePortControlReg (2, &alePortControlCfg);
}


/***************************************************************************************
 * FUNCTION PURPOSE: Add MAC Address to the ALE for the packet being routed from network
 ***************************************************************************************
 * DESCRIPTION: Add MAC Address to the ALE for the packet being routed from network
 ***************************************************************************************/
bool dsiuNwSwUpdateMacAddr(UInt8 macAddress[6])
{
    Uint32                              i;
    CSL_CPSW_3GF_ALE_UNICASTADDR_ENTRY  ucastAddrCfg;
    /* Program the ALE with the MAC address.
    *
    * The ALE entries determine the switch port to which any
    * matching received packet must be forwarded to.
    */
    /* Get the next free ALE entry to program */
    for (i = 0; i < CSL_CPSW_3GF_NUMALE_ENTRIES; i++)
    {
        if (CSL_CPSW_3GF_getALEEntryType (i) == ALE_ENTRYTYPE_FREE)
        {
            /* Found a free entry */
            break;
        }
    }
    if (i == CSL_CPSW_3GF_NUMALE_ENTRIES)
    {
        /* No free ALE entry found. return error. */
        dsiuNwLog ("ERROR: No Free Entry in CPSW ALE table \n");
        return 0;
    }
    else
    {
        /* Found a free ALE entry to program our MAC address */
        memcpy (ucastAddrCfg.macAddress, macAddress, 6);    /* Set the MAC address */
        ucastAddrCfg.ucastType      =      ALE_UCASTTYPE_UCAST_NOAGE;  /* Add a permanent unicast address entry */
        ucastAddrCfg.secureEnable   =      FALSE;
        ucastAddrCfg.blockEnable    =      FALSE;
        ucastAddrCfg.portNumber     =      0;   /* DSP CPPI Port */

        /* Setup the ALE entry for this port's MAC address */
        CSL_CPSW_3GF_setAleUnicastAddrEntry (i, &ucastAddrCfg);
    }


    /* Done with upading address */
    return 1;
}

/*****************************************************************************
 * FUNCTION PURPOSE: Global Initialization of CPPI. Once Per System
 *****************************************************************************
 * DESCRIPTION: The function will initialize the CPPI
 *****************************************************************************/
static inline bool dsiuNwGlobCppiInit (void)
{
  Int32          result;

  result = Cppi_init (cppiGblCfgParams);
  if (result != CPPI_SOK)  {
    dsiuNwLog ("function dsiuNwGlobCppiInit: Cppi_init failed with error code %d\n", result);
    return (FALSE);
  }
  return (TRUE);
}

/*****************************************************************************
 * FUNCTION PURPOSE: Global Initialization of Queue Manager. Once Per System
 *****************************************************************************
 * DESCRIPTION: The function will initialize the Queue Manager
 *****************************************************************************/
bool dsiuNwGlobQmInit (void)
{
  Qmss_InitCfg     qmssInitConfig;
  Int32          result;


  memset (&qmssInitConfig, 0, sizeof (Qmss_InitCfg));

  /* Use Internal Linking RAM for optimal performance */
  qmssInitConfig.linkingRAM0Base = 0;
  qmssInitConfig.linkingRAM0Size = 0;
  qmssInitConfig.linkingRAM1Base = 0;
  qmssInitConfig.maxDescNum      = DSIU_CONFIG_MAX_DESC_NUM;

  result = Qmss_init (&qmssInitConfig, qmssGblCfgParams);
  if (result != QMSS_SOK)  {
    dsiuNwLog ("function dsiuNwGlobQmInit: qmss_Init failed with error code %d\n", result);
    return (FALSE);
  }
  return (TRUE);
}

/*****************************************************************************
 * FUNCTION PURPOSE: Global Initialization of Queue Manager. Once Per System
 *****************************************************************************
 * DESCRIPTION: The function will initialize the Queue Manager
 *****************************************************************************/
static bool dsiuNwLocQmInit (void)
{
    Int32          result;
     result = Qmss_start();
     if (result != QMSS_SOK)
     {
         dsiuNwLog ("function setupQmMem: Qmss_start failed with error code %d\n", result);
         return (FALSE);
     }
     return (TRUE);
}

/*****************************************************************************
 * FUNCTION PURPOSE: Initialize the Queue Manager
 *****************************************************************************
 * DESCRIPTION: The function will initialize the Queue Manager
 *****************************************************************************/
bool dsiuNwSetupQmMem(UInt32          numDesc,
                      UInt32          descSize,
                      UInt32*         pDescMemBase,
                      Qmss_MemRegion    memRegion,
                      Qmss_QueueHnd*    pdescQ)
{
  Qmss_MemRegInfo   memInfo;
  Cppi_DescCfg      descCfg;
  Int32             result;
  Int               n;

  memset(&memInfo,0,sizeof(Qmss_MemRegInfo));
  memset(&descCfg,0,sizeof(Cppi_DescCfg));
#ifdef NWAL_DSIU_DESC_GLOB_MEM
  memset(dsiuNwalCfg.descMemBase, 0, dsiuNwalCfg.numDescsPerCore*CPU_NCORES*dsiuNwalCfg.descSize);
#else
  memset(dsiuNwalCfg.descMemBase, 0, dsiuNwalCfg.numDescsPerCore*dsiuNwalCfg.descSize);
#endif
  memInfo.descBase       = pDescMemBase;
  memInfo.descSize       = descSize;
  memInfo.descNum        = numDesc;
  memInfo.manageDescFlag = Qmss_ManageDesc_MANAGE_DESCRIPTOR;
  memInfo.memRegion      = memRegion;

  if(memRegion == DSIU_CONFIG_BASE_QM_GLOBAL_MSMC_MEMORY_REGION)
  {
      memInfo.startIndex = 0;
  }else if(memRegion ==DSIU_CONFIG_BASE_QM_GLOBAL_DDR_MEMORY_REGION)
  {
      /* Global shared memory for all descriptors to all cores */
      memInfo.startIndex     = DSIU_CONFIG_MAX_GLOB_DESC;
  }
  else
  {
      memInfo.startIndex     = DSIU_CONFIG_MAX_GLOB_DESC + (dsiuNwalCfg.numDescsPerCore * DNUM);
  }

  result = Qmss_insertMemoryRegion (&memInfo);
  if (result < QMSS_SOK)  {
    dsiuNwLog ("function dsiuNwSetupQmMem: Qmss_insertMemoryRegion returned error code %d\n", result);
    return (FALSE);
  }

  /* Initialize the descriptors. This function opens a general
   * purpose queue and intializes the memory from region 0, placing
   * the initialized descriptors onto that queue */
  descCfg.memRegion         = memRegion;
  descCfg.descNum           = numDesc;
  descCfg.destQueueNum      = QMSS_PARAM_NOT_SPECIFIED;
  descCfg.queueType         = Qmss_QueueType_GENERAL_PURPOSE_QUEUE;
  descCfg.initDesc          = Cppi_InitDesc_INIT_DESCRIPTOR;
  descCfg.descType          = Cppi_DescType_HOST;
  descCfg.returnQueue.qNum  = QMSS_PARAM_NOT_SPECIFIED;
  descCfg.returnQueue.qMgr  = 0;
  descCfg.epibPresent       = Cppi_EPIB_EPIB_PRESENT;


  descCfg.cfg.host.returnPolicy     = Cppi_ReturnPolicy_RETURN_ENTIRE_PACKET;
  descCfg.cfg.host.psLocation       = Cppi_PSLoc_PS_IN_DESC;

  *pdescQ = Cppi_initDescriptor (&descCfg, (Uint32 *)&n);

  if (n != descCfg.descNum)  {
    dsiuNwLog ("function dsiuNwSetupQmMem: expected %d descriptors to be initialized, only %d are initialized\n",
                descCfg.descNum, n);
    return (FALSE);
  }

  return (TRUE);

}


/*****************************************************************************
 * FUNCTION PURPOSE: Setup the queues
 *****************************************************************************
 * DESCRIPTION: The function will setup all the queues used by the chip
 *****************************************************************************/
bool dsiuNwCrMemPool (Qmss_QueueHnd     descQ,
                     nwalMbufPool_t*    pMbufPool,
                     UInt32*          pNumBuf,    /* Array with number of buffers in the Pool */
                     UInt8 **         ppBufBase   /* Base Address Global */
                     )
{
    int               i;
    UInt8           isAlloc;
    UInt32          myswinfo[] = { 0x11112222, 0x33334444 };
    Cppi_HostDesc*    pHd;
    UInt8           count=0;
    nwalBufPool_t*    pBufPool = pMbufPool->bufPool;
    UInt8 *         pBufBase;
    Qmss_Queue        retQ;

    for(count=0;count < pMbufPool->numBufPools;count++,pBufPool++)
    {
        /* Open the Queue which will contain descriptors with free buffers */
        pBufPool->qHandle = Qmss_queueOpen (Qmss_QueueType_GENERAL_PURPOSE_QUEUE,
                                          QMSS_PARAM_NOT_SPECIFIED,
                                          &isAlloc);
        if (pBufPool->qHandle <= 0)
        {
          dsiuNwLog ("dsiuNwCrMemPool: Qmss_queueOpen failed for Buffer Pool %d \n",count);
          return (FALSE);
        }

        /* Pop descriptors off the free descriptor queue, attach buffers and drop them into the
        * Buffer Pool */
        pBufBase = ppBufBase[count];
        for (i = 0; i < pNumBuf[count]; i++)
        {
            pHd = (Cppi_HostDesc *)(QMSS_DESC_PTR((UInt32)Qmss_queuePop (descQ)));
            if (pHd == NULL)
            {
              dsiuNwLog ("dsiuNwCrMemPool: Qmss_queuePop returned NULL on mBufPool: %d pop number %d Descriptor Q %d\n",
                          count,
                          i,
                          descQ);
              return (FALSE);
            }
            Cppi_setOriginalBufInfo (Cppi_DescType_HOST,
                                    (Cppi_Desc *)pHd,
                                    pBufBase,
                                    pBufPool->bufSize);

            Cppi_setData (Cppi_DescType_HOST,
                          (Cppi_Desc *)pHd,
                          pBufBase,
                          pBufPool->bufSize);

            Cppi_setPacketLen(Cppi_DescType_HOST,
                              (Cppi_Desc *)pHd,
                              pBufPool->bufSize);

            Cppi_setSoftwareInfo (Cppi_DescType_HOST,
                                  (Cppi_Desc *)pHd,
                                  (Uint8 *)myswinfo);

            /* Setup the return for the descriptor */
            memset(&retQ,0,sizeof(Qmss_Queue));
            retQ.qMgr = 0;
            retQ.qNum = pBufPool->qHandle;
            Cppi_setReturnQueue (Cppi_DescType_HOST, (Cppi_Desc *)pHd, retQ);

            pHd->nextBDPtr = NULL;

            /* Write back the cache */
            Osal_writeBackCache((void *)pHd,pBufPool->descSize);

            Qmss_queuePushDescSize (pBufPool->qHandle,
                                    (void *)pHd,
                                    pBufPool->descSize);

            /* Move Buffer to next */
            pBufBase = pBufBase + pBufPool->bufSize;


        }
    }

    return (TRUE);
}

/*****************************************************************************
 * FUNCTION PURPOSE: Setup the queues
 *****************************************************************************
 * DESCRIPTION: The function will setup all the queues used by the chip
 *****************************************************************************/
bool dsiuNwTestMemPool ( nwalMbufPool_t*    pMbufPool)
{
    UInt8           isAlloc;
    UInt32          myswinfo[] = { 0x11112222, 0x33334444 };
    volatile Cppi_HostDesc*    pHd;
    UInt8           count=0;
    nwalBufPool_t*    pBufPool = pMbufPool->bufPool;
    Qmss_QueueHnd     qHandle;

    /* Open the Queue which will contain descriptors with free buffers */
    qHandle = Qmss_queueOpen  (Qmss_QueueType_GENERAL_PURPOSE_QUEUE,
                               QMSS_PARAM_NOT_SPECIFIED,
                               &isAlloc);
    if (qHandle <= 0)
    {
      dsiuNwLog ("dsiuNwCrMemPool: Qmss_queueOpen failed for Buffer Pool %d \n",count);
      return (FALSE);
    }

    for(count=0;count < pMbufPool->numBufPools;count++,pBufPool++)
    {


        while (Qmss_getQueueEntryCount (pBufPool->qHandle) > 0)
        {
            pHd = (Cppi_HostDesc *)QMSS_DESC_PTR(Qmss_queuePop (pBufPool->qHandle));
            if (Cppi_getSoftwareInfo (Cppi_DescType_HOST,
                                     (Cppi_Desc *)pHd,
                                     (UInt8 **) &myswinfo) == CPPI_EPIB_NOT_PRESENT)
            {
                while(1);
            }

            Qmss_queuePushDescSize (qHandle,
                                    (void *)pHd,
                                    pBufPool->descSize);
        }


        while (Qmss_getQueueEntryCount (qHandle) > 0)
        {
            pHd = (Cppi_HostDesc *)QMSS_DESC_PTR(Qmss_queuePop (qHandle));
            if (Cppi_getSoftwareInfo (Cppi_DescType_HOST,
                                     (Cppi_Desc *)pHd,
                                     (UInt8 **) &myswinfo) == CPPI_EPIB_NOT_PRESENT)
            {
                while(1);
            }

            Qmss_queuePushDescSize (pBufPool->qHandle,
                                    (void *)pHd,
                                    pBufPool->descSize);
        }
    }
    if (Qmss_queueClose (qHandle) < 0 )
    {
        while(1);
    }
    return(TRUE);
}

/***************************************************************************************
 * FUNCTION PURPOSE: NWAL start per core
 ***************************************************************************************
 * DESCRIPTION: function to start NWAL in a core
 ***************************************************************************************/
bool dsiuNwalStart(void)
{
    nwalLocCfg_t    nwalLocCfg;
    nwal_RetValue   nwalRetVal;
    UInt32        numBuf[NWAL_MAX_BUF_POOLS];
    UInt8*        bufBase[NWAL_MAX_BUF_POOLS];
    const dsiuNwalLinkedBufsDesc_t *linkedBufDesc;
    UInt32 linkBufSize;
    UInt32 linkBufNPerCore;

    memset(&nwalLocCfg,0,sizeof(nwalLocCfg));
    memset(&dsiuNwLocContext,0,sizeof(dsiuNwLocContext_t));

#ifndef NWAL_DSIU_DESC_GLOB_MEM
    /* Initialize Descriptors to be used for all local core related resources */
    if(dsiuNwSetupQmMem(dsiuNwalCfg.numDescsPerCore,
                        dsiuNwalCfg.descSize,
                        (UInt32 *)Osal_nwalLocToGlobAddr((UInt32)dsiuNwalCfg.descMemBase),
                        (Qmss_MemRegion)(DSIU_CONFIG_BASE_QM_MEMORY_REGION + DNUM ),
                        &dsiuNwLocContext.descQ)!= TRUE)
    {
        dsiuNwLog ("dsiuNwalStart:QM Memory region/Descriptor Initialization for Local failed\n");
        return FALSE;
    }
#endif

    /* Call back registration for the core */
    nwalLocCfg.pRxPktCallBack = dsiuNWALRxPktCallback;
    nwalLocCfg.pCmdCallBack = dsiuNWALCmdCallBack;
    nwalLocCfg.pPaStatsCallBack = dsiuNWALCmdPaStatsReply;

    for (linkedBufDesc = dsiuNwalCfg.linkedBufDescs; linkedBufDesc->bufsBase; linkedBufDesc++) {
        linkBufSize = linkedBufDesc->bufSize;
        linkBufNPerCore  = linkedBufDesc->nBufsPerCore;
        #ifdef NWAL_DSIU_DESC_GLOB_MEM
            memset (linkedBufDesc->bufsBase, 0, linkBufSize * linkBufNPerCore * CPU_NCORES);
        #else
            memset (linkedBufDesc->bufsBase, 0, linkBufSize * linkBufNPerCore);
        #endif

        switch (linkedBufDesc->type) {
            case DSIU_NWAL_RX_CTL_BUFS:
                /* Initialize Buffer Pool for Control packets from NetCP to Host */
                nwalLocCfg.rxCtlPool.numBufPools = 1;
                nwalLocCfg.rxCtlPool.bufPool[0].descSize = dsiuNwalCfg.descSize;
                nwalLocCfg.rxCtlPool.bufPool[0].bufSize = linkBufSize;
                numBuf[0] = linkBufNPerCore;
            #ifdef NWAL_DSIU_DESC_GLOB_MEM
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr(((UInt32)linkedBufDesc->bufsBase + DNUM*linkBufSize*linkBufNPerCore));
                if(dsiuNwCrMemPool(dsiuNwGlobContext.descExtMemQ,
                                   &nwalLocCfg.rxCtlPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #else
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr((UInt32)linkedBufDesc->bufsBase);
                if(dsiuNwCrMemPool(dsiuNwLocContext.descQ,
                                   &nwalLocCfg.rxCtlPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #endif
                {
                    dsiuNwLog ("dsiuNwalStart:Memory Pool Creation Failed for RX Ctl From NetCP\n");
                    return FALSE;
                }

                if(dsiuNwTestMemPool(&nwalLocCfg.rxCtlPool) != TRUE)
                {
                    dsiuNwLog ("dsiuNwTestMemPool Failed for RX packets from NetCP\n");
                    return FALSE;
                }
                break;

            case DSIU_NWAL_TX_CTL_BUFS:
                /* Initialize Buffer Pool for Control packets from Host to NetCP */
                nwalLocCfg.txCtlPool.numBufPools = 1;
                nwalLocCfg.txCtlPool.bufPool[0].descSize = dsiuNwalCfg.descSize;
                nwalLocCfg.txCtlPool.bufPool[0].bufSize = linkBufSize;
                numBuf[0] = linkBufNPerCore;
            #ifdef NWAL_DSIU_DESC_GLOB_MEM
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr(((UInt32)linkedBufDesc->bufsBase + DNUM*linkBufSize*linkBufNPerCore));
                if(dsiuNwCrMemPool(dsiuNwGlobContext.descExtMemQ,
                                   &nwalLocCfg.txCtlPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #else
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr((UInt32)linkedBufDesc->bufsBase);
                if(dsiuNwCrMemPool(dsiuNwLocContext.descQ,
                                   &nwalLocCfg.txCtlPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #endif
                {
                    dsiuNwLog ("dsiuNwalStart:Memory Pool Creation Failed for TX Control to NetCP\n");
                    return FALSE;
                }
                break;

            case DSIU_NWAL_RX_PKT_BUFS:
                /* Initialize Buffer Pool for Packets from NetCP to Host */
                nwalLocCfg.rxPktPool.numBufPools = 1;
                nwalLocCfg.rxPktPool.bufPool[0].descSize = dsiuNwalCfg.descSize;
                nwalLocCfg.rxPktPool.bufPool[0].bufSize = linkBufSize;
                numBuf[0] = linkBufNPerCore;
            #ifdef NWAL_DSIU_DESC_GLOB_MEM
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr(((UInt32)linkedBufDesc->bufsBase + DNUM*linkBufSize*linkBufNPerCore));
                if(dsiuNwCrMemPool(dsiuNwGlobContext.descExtMemQ,
                                   &nwalLocCfg.rxPktPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #else
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr((UInt32)linkedBufDesc->bufsBase);
                if(dsiuNwCrMemPool(dsiuNwLocContext.descQ,
                                   &nwalLocCfg.rxPktPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #endif
                {
                    dsiuNwLog ("dsiuNwalStart:Memory Pool Creation Failed for RX packets from NetCP\n");
                    return FALSE;
                }
                if(dsiuNwTestMemPool(&nwalLocCfg.rxPktPool) != TRUE)
                {
                    dsiuNwLog ("dsiuNwTestMemPool Failed for RX packets from NetCP\n");
                    return FALSE;
                }
                break;

            case DSIU_NWAL_TX_PKT_BUFS:
                /* Initialize Buffer Pool for Packets from Host to NetCP */
                nwalLocCfg.txPktPool.numBufPools = 1;
                nwalLocCfg.txPktPool.bufPool[0].descSize = dsiuNwalCfg.descSize;
                nwalLocCfg.txPktPool.bufPool[0].bufSize = linkBufSize;
                numBuf[0] = linkBufNPerCore;
            #ifdef NWAL_DSIU_DESC_GLOB_MEM
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr(((UInt32)linkedBufDesc->bufsBase + DNUM*linkBufSize*linkBufNPerCore));
                if(dsiuNwCrMemPool(dsiuNwGlobContext.descExtMemQ,
                                   &nwalLocCfg.txPktPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #else
                bufBase[0] = (UInt8 *)Osal_nwalLocToGlobAddr((UInt32)linkedBufDesc->bufsBase);
                if(dsiuNwCrMemPool(dsiuNwLocContext.descQ,
                                   &nwalLocCfg.txPktPool,
                                   numBuf,
                                   bufBase) != TRUE)
            #endif
                {
                    dsiuNwLog ("dsiuNwalStart:Memory Pool Creation Failed for TX packets to NetCP\n");
                    return FALSE;
                }
                break;
          default:
            dsiuNwLog ("function setupQueues: Invalid type\n");
            return FALSE;
        }
    }

    memcpy(&dsiuNwLocContext.nwalLocCfg,&nwalLocCfg,sizeof(nwalLocCfg_t));

    while(1)
    {
        nwalRetVal = nwal_start(dsiuNwGlobContext.nwalInstHandle,&nwalLocCfg);
        if(nwalRetVal == nwal_ERR_INVALID_STATE)
        {
            continue;
        }
        break;
    }

    if(nwalRetVal != nwal_OK)
    {
        dsiuNwLog ("nwal_start:Failed !!!\n");
        return FALSE;
    }

    return TRUE;
}


/*****************************************************************************
 * FUNCTION PURPOSE: Global Initializion of the Network  related resources
 *****************************************************************************
 * DESCRIPTION: Global Initializion of the Network  related resources
 *****************************************************************************/
bool dsiuNwGlobalInit (void)
{
    nwalGlobCfg_t   nwalGlobCfg;
    nwalSizeInfo_t  nwalSizeInfo;
    nwalMemBuf_t    nwalMemBuf[nwal_N_BUFS];
    nwal_RetValue   nwalRetVal;
    UInt8         count;
    int             sizes[nwal_N_BUFS];
    int             aligns[nwal_N_BUFS];
    void*           bases[nwal_N_BUFS];



    /* Global Network resource QM/CPPI/PA related initialization to
     * be done by Core 0
     */
    memset(&dsiuNwGlobContext,0,sizeof(dsiuNwGlobContext_t));
    memset(&nwalGlobCfg,0,sizeof(nwalGlobCfg_t));
    memset(sizes,0,sizeof(sizes));
    memset(aligns,0,sizeof(aligns));
    memset(bases,0,sizeof(bases));

    /* QM Global Init */
    if(!dsiuNwGlobQmInit())
    {
        dsiuNwLog ("QM Global Initialization failed\n");
        return FALSE;
    }

    /* QM Local Init */
    if(!dsiuNwLocQmInit())
    {
        dsiuNwLog ("QM Local Initialization failed\n");
        return FALSE;
    }

    if(!dsiuNwGlobCppiInit())
    {
        dsiuNwLog ("CPPI Global Initialization failed\n");
        return FALSE;
    }

#ifdef NWAL_DSIU_DESC_GLOB_MEM
    /* Initialize descriptor for packet and control processing at each core from global shared external memory */
    /* Initialize Global Descriptors */
    if(dsiuNwSetupQmMem(dsiuNwalCfg.numDescsPerCore*CPU_NCORES,
                        dsiuNwalCfg.descSize,
                        (UInt32 *)Osal_nwalLocToGlobAddr((UInt32)dsiuNwalCfg.descMemBase),
                        DSIU_CONFIG_BASE_QM_GLOBAL_DDR_MEMORY_REGION,
                        &dsiuNwGlobContext.descExtMemQ)!= TRUE)
    {
        dsiuNwLog ("dsiuNwGlobalInit:QM Memory region/Descriptor Initialization for per core descriptors failed\n");
        return FALSE;
    }

#endif

    /* Get the Buffer Requirement from NWAL */
    memset(&nwalMemBuf,0,sizeof(nwalMemBuf));
    memset(&nwalSizeInfo,0,sizeof(nwalSizeInfo));
    nwalSizeInfo.nMaxMacAddress = DSIU_MAX_NUM_MAC;
    nwalSizeInfo.nMaxIpAddress = DSIU_MAX_NUM_IP;
    nwalSizeInfo.nMaxL4Ports = DSIU_MAX_NUM_PORTS;
    nwalSizeInfo.nMaxIpSecChannels = DSIU_MAX_NUM_IPSEC_CHANNELS;
    nwalSizeInfo.nMaxL2L3Hdr = DSIU_MAX_NUM_L2_L3_HDRS;
    nwalSizeInfo.nProc = CPU_NCORES;
    for(count=0;count < nwal_N_BUFS;count++)
    {
        nwalMemBuf[count].cacheLineSize = CACHE_L2_LINESIZE;

    }
    nwalRetVal = nwal_getBufferReq(&nwalSizeInfo,
                                   sizes,
                                   aligns);
    if(nwalRetVal != nwal_OK)
    {
        dsiuNwLog ("dsiuNwGlobalInit:nwal_getBufferReq Failed \n");
        return FALSE;
    }

    /* Check for memory size requirement and update the base */
    count = 0;
    bases[nwal_BUF_INDEX_INST] = (UInt32 *)Osal_nwalLocToGlobAddr((UInt32)nwalInstMem);
    if(DSIU_CONFIG_BUFSIZE_NWAL_HANDLE < sizes[nwal_BUF_INDEX_INST])
    {
        /* Resize Memory */
        while(1);
    }
    count++;

    bases[nwal_BUF_INDEX_INT_HANDLES] = (UInt32 *)Osal_nwalLocToGlobAddr((UInt32)nwalHandleMem);
    if(NWAL_CHAN_HANDLE_SIZE  < sizes[nwal_BUF_INDEX_INT_HANDLES])
    {
        /* Resize Memory */
        while(1);
    }
    count++;

    bases[nwal_BUF_INDEX_PA_LLD_BUF0] = (UInt32 *)Osal_nwalLocToGlobAddr((UInt32)paBuf0);
    if((DSIU_CONFIG_BUFSIZE_PA_BUF0) < sizes[nwal_BUF_INDEX_PA_LLD_BUF0])
    {
        /* Resize Memory */
        while(1);
    }
    count++;

    bases[nwal_BUF_INDEX_PA_LLD_BUF1] = (UInt32 *)Osal_nwalLocToGlobAddr((UInt32)paBuf1);
    if((DSIU_CONFIG_BUFSIZE_PA_BUF1) < sizes[nwal_BUF_INDEX_PA_LLD_BUF1])
    {
        /* Resize Memory */
        while(1);
    }
    count++;

    bases[nwal_BUF_INDEX_PA_LLD_BUF2] = (UInt32 *)Osal_nwalLocToGlobAddr((UInt32)paBuf2);
    if((DSIU_CONFIG_BUFSIZE_PA_BUF2) < sizes[nwal_BUF_INDEX_PA_LLD_BUF2])
    {
        /* Resize Memory */
        while(1);
    }
    count++;

    bases[nwal_BUF_INDEX_SA_LLD_HANDLE] = 0;
    bases[nwal_BUF_INDEX_SA_CONTEXT] = 0;
    bases[nwal_BUF_INDEX_SA_LLD_CHAN_HANDLE] = 0;
    count = count+3;

    if(count != nwal_N_BUFS)
    {
        while(1);
    }

    /* Configurations: hopLimit, lpbackPass being set to 0 default */
    nwalGlobCfg.maxPktsPerPoll = 10;
    nwalGlobCfg.rxDefPktQ = QMSS_PARAM_NOT_SPECIFIED;
    /* Initialize NWAL module */
    nwalRetVal = nwal_create(&nwalGlobCfg,
                             &nwalSizeInfo,
                             sizes,
                             bases,
                             &dsiuNwGlobContext.nwalInstHandle);
    if(nwalRetVal != nwal_OK)
    {
        dsiuNwLog ("dsiuNwGlobalInit:nwal_create Failed \n");
    }

    dsiuNwLog("Global and Local Network initialization Successful \n");

    dsiuNwGlobContext.state = DSIU_NW_CXT_GLOB_ACTIVE;
    Osal_writeBackCache(&dsiuNwGlobContext,sizeof(dsiuNwGlobContext));
    return(TRUE);
}

/*****************************************************************************
 * FUNCTION PURPOSE: Initialization per local core
 *****************************************************************************
 * DESCRIPTION: Initialization per local core
 *****************************************************************************/
bool dsiuNwLocaInit (void)
{
  Int32            result;

  while(1)
  {
      Osal_invalidateCache(&dsiuNwGlobContext,sizeof(dsiuNwGlobContext));
      /* Invalidate the Global context */
      if(dsiuNwGlobContext.state == DSIU_NW_CXT_GLOB_INACTIVE)
      {
          continue;
      }

      /* Block until Qmss_init() has completed by core 0 */
      result = Qmss_start();
      if(result == QMSS_NOT_INITIALIZED)
      {
          dsiuNwLog ("QMSS Not yet Initialized Waiting for Core %d\n", DNUM);
          continue;
      }
      else if (result != QMSS_SOK)  {
        dsiuNwLog ("function setupQmMem: Qmss_start failed with error code %d\n", result);
        return (FALSE);
      }

      if (result == QMSS_SOK)
      {
          break;
      }
  }

  return TRUE;
}

/*****************************************************************************
 * FUNCTION PURPOSE: Initialize the Network  related resources and NWAL
 *****************************************************************************
 * DESCRIPTION: Initialize the Network PA related resources and NWAL
 *****************************************************************************/
bool dsiuNwInit (void)
{
    tuint           count=0;

    /* Invalidate the cache so we read "Real" value of dsiuNwGlobInit.state */
    CACHE_invL1d ((void *)&dsiuNwGlobInit.state, sizeof(dsiuNwGlobInit.state), CACHE_WAIT);
    CACHE_invL2 ((void *)&dsiuNwGlobInit.state, sizeof(dsiuNwGlobInit.state), CACHE_WAIT);

    if ((dsiuNwGlobInit.state != DSIU_NW_GLOB_INIT_STATE_MAGIC1) &&
        (dsiuNwGlobInit.state != DSIU_NW_GLOB_INIT_STATE_MAGIC2))
    {
      while(1);  /* Failed to correctly initialize dsiuNwGlobContext in .cmd! */
    }

    while (DNUM && (dsiuNwGlobInit.state == DSIU_NW_GLOB_INIT_STATE_MAGIC1))
    {
      /* Busy wait for core 0 to start */
      /* Invalidate the cache so we read "Real" value of dsiuNwGlobInit.state */
      CACHE_invL1d ((void *)&dsiuNwGlobInit.state, sizeof(dsiuNwGlobInit.state), CACHE_WAIT);
      CACHE_invL2 ((void *)&dsiuNwGlobInit.state, sizeof(dsiuNwGlobInit.state), CACHE_WAIT);
    }

    /* Common Initialization for all cores */
    while(count < DSIU_MAX_NUM_TRANS)
    {
        dsiuNwLocContext.transInfos[count].transId = count;
        count++;
    }

    if(!DNUM)
    {
        extern char _lcf_cppi_drv_start;
        extern char _lcf_cppi_drv_end;
        extern char _lcf_qmss_drv_start;
        extern char _lcf_qmss_drv_end;

        if((dsiuNwGlobInit.state == DSIU_NW_GLOB_INIT_STATE_MAGIC2) &&
            (dsiuNwGlobContext.state & DSIU_NW_CXT_GLOB_ACTIVE) &&
            (dsiuNwLocContext.state & DSIU_NW_CXT_LOC_ACTIVE))
        {
            /* Initialization complete */
             return TRUE;
        }

        /* clear nwal instance and handle buffers */
        memset (nwalInstMem,   0, DSIU_CONFIG_BUFSIZE_NWAL_HANDLE);
        memset (nwalHandleMem, 0, NWAL_CHAN_HANDLE_SIZE);

        /* clear all buffers used by dependancies to nwal */
        memset(paBuf0,             0, DSIU_CONFIG_BUFSIZE_PA_BUF0);
        memset(paBuf1,             0, DSIU_CONFIG_BUFSIZE_PA_BUF1);
        memset(paBuf2,             0, DSIU_CONFIG_BUFSIZE_PA_BUF2);

        /* clear buffers declared inside LLDs (with cmd file help) */
        /* to do after getting the sections in ggvf0.becmd */
        memset(&_lcf_cppi_drv_start, 0, &_lcf_cppi_drv_end - &_lcf_cppi_drv_start);
        memset(&_lcf_qmss_drv_start, 0, &_lcf_qmss_drv_end - &_lcf_qmss_drv_start);

        /* Switch Initialization */
#ifndef DSIU
        dsiuNwCPSWInit();
#endif

        /* Global Network resource QM/CPPI/PA related initialization to
         * be done by Core 0
         */
        if(!dsiuNwGlobalInit())
        {
            dsiuNwLog("ERROR: Global Network initialization Failed \n");
            return FALSE;
        }
        dsiuNwGlobInit.state = DSIU_NW_GLOB_INIT_STATE_MAGIC2;
        /* Write back MAGIC2 */
        CACHE_wbL1d ((void *)&dsiuNwGlobInit.state, sizeof(dsiuNwGlobInit.state), CACHE_WAIT);
        CACHE_wbL2 ((void *)&dsiuNwGlobInit.state, sizeof(dsiuNwGlobInit.state), CACHE_WAIT);
    }
    else
    {
        if(!dsiuNwLocaInit())
        {
            dsiuNwLog("ERROR: Local Network initialization Failed \n");
            return FALSE;
        }
        dsiuNwLog("Local Network initialization Successful \n");
    }

    /* Start NWAL on the core doing global initialization */
    if(dsiuNwalStart() != TRUE)
    {
        dsiuNwLog ("dsiuNwGlobalInit:dsiuNwalStart Failed \n");
    }

    dsiuNwLocContext.state = DSIU_NW_CXT_LOC_ACTIVE;

    return TRUE;
}

/*****************************************************************************
 * FUNCTION PURPOSE: Delete the queues
 *****************************************************************************
 * DESCRIPTION: The function will pop all descriptors and delet the queue
 *****************************************************************************/

Int dsiuNwDeleteQueue (Qmss_QueueHnd queue)
{

    Cppi_HostDesc *hd=NULL;
    Qmss_Result     result;

    /* Pops off descriptors from TX linked buffer Q */
    hd = (Cppi_HostDesc *)(((UInt32)Qmss_queuePop (queue)) & ~0xf);
    while(hd != NULL)
    {
        hd = (Cppi_HostDesc *)(((UInt32)Qmss_queuePop (queue)) & ~0xf);

        /* TBD: Cleanup of descriptors */

    }

    /* Close Tx Buf queue */
    if ((result = Qmss_queueClose (queue)) != CPPI_SOK)
    {
        dsiuNwLog ("Error Closing tx queue error code : %d\n", result);
        return (-1);
    }

    return (0);
}

/*****************************************************************************
 * FUNCTION PURPOSE: Cleanup for Network  related resources and NWAL
 *****************************************************************************
 * DESCRIPTION: Initialize the Network PA related resources and NWAL
 *****************************************************************************/
Int dsiuNwGlobClean (void)
{
    nwal_RetValue   retValue;
    Cppi_Result     result;

    /* Delete NWAL instance */
    retValue = nwal_delete(dsiuNwGlobContext.nwalInstHandle);
    if(retValue != nwal_OK)
    {
        dsiuNwLog ("ERROR: nwal_delete returned Error Code %d \n",
                    retValue);
    }

    /* Delete Tx/Rx linker buffer Queues */
    dsiuNwDeleteQueue(dsiuNwLocContext.nwalLocCfg.rxCtlPool.bufPool[0].qHandle);
    dsiuNwDeleteQueue(dsiuNwLocContext.nwalLocCfg.txCtlPool.bufPool[0].qHandle);
    dsiuNwDeleteQueue(dsiuNwLocContext.nwalLocCfg.rxPktPool.bufPool[0].qHandle);
    dsiuNwDeleteQueue(dsiuNwLocContext.nwalLocCfg.txPktPool.bufPool[0].qHandle);

    /* Do final cleanup of CPPI driver */
    if ((result = Cppi_exit ()) != CPPI_SOK)
    {
        dsiuNwLog ("Error Exiting CPPI : %d\n", result);
    }
    return (0);

}
#else
int dummy; /* no warning */
#endif

