/******************************************************************************
 * FILE PURPOSE: main - starts the DSP functionality.
 ******************************************************************************
 * FILE NAME:  main.c  
 *
 * DESCRIPTION:  This files contains the main function necessary to start the
 *               siu to the right after download state. 
 * TABS: NONE
 *
 * Initial revision Shiju Sivasankaran
 * REVISION HISTORY:
 *
 * (C) Copyright 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "siu.h"

#ifdef GG_BIOS
#include "osal/siuOsal.h"
#endif
#include "siuloc.h"

/* 
 * Include IBL Demo API 
 *  See these files for details on enabling/disabling building as IBL demo. 
 */
#ifdef HDCT
/* The following two arrays are dummy arrays now. They are added here
   for resolving linking error for hw/c64x/startupb.c. Need to review. */
tulong siuVigdkDiagAppMem[1];
const tulong siuVigdkDiagAppMemIbl[1];
#include "vct/siuVctIbl.h"
#else
#include "video/siuVigdkIbl.h"
#endif

/* Nice way for doing endless loop */
#define loop      for(;;)

/******************************************************************************
 * Function used to halt the DSP for any siuException events.  This is moved 
 * from siumain.c to incorporate the MIPS's exception handling (which different
 * from usal DSP implementation
******************************************************************************/
volatile tuint siu_debug_halt_dsp = 1;
void siu_dsp_halt(uint id, char *s)
{
   //while(1) but can change volatile tuint to step out of exception to debug in CCS
   while(siu_debug_halt_dsp);
}

/*****************************************************************************
 * First DSP function, which initiate the SIU and further DSP processing. This
 * is moved from siuinit.c to support the Core 10.2, whose main is different 
 * from the convensional DSP implementation.  The Infinite loop in the siuMain
 * in sinmain.c is moved in to this one to minimize the common code change for
 * Core 10.2 
*****************************************************************************/
extern cregister volatile unsigned int TSCL;
//volatile tuint emuwait_1=1;
volatile tuint siuNumCoreInit = 0;
#pragma DATA_SECTION(siuNumCoreInit,".siuNumCoreInit");
void main(void)
{
  /* Initialize cycle counting register */
  TSCL = 0;
   //while(emuwait_1);

   /* Call the siu initilization functions */
   siuMain();

   /* Used to make sure the cores are sequentially initialized */
   siuNumCoreInit++;
   siu_osal_wbinv_cache((void *)&siuNumCoreInit, sizeof(siuNumCoreInit), TRUE);

#ifdef HDCT
   /* codec stand-alone test framework */
   siuVctRun();
#else

#ifdef GG_BIOS
    siu_osal_start();
#else
  /* Main loop when BIOS is not present */
   loop {
     /* Call the siu task processing function */
     siuTaskScheduler();
   } /* Non-BIOS main loop */
#endif
#endif
}
/*Nothing past this point*/

