/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "setjmp.h"
#include "string.h"

/* SIU header files */
#include "siu.h"
#include "siuloc.h"
#include "siuport.h"

#ifdef GG_BIOS
/* Operating system related header files */
#include "osal/siuOsal.h"
#endif
#include "osal/siuIpcOsal.h"
#include "osal/siuIpcMsgOsal.h"

/* DPR definitions */
#include "micdspdp.h"
#ifndef RESIZABLEHPI
#include "dpriram.h"
#endif

/* low-level hardware files */
#include "targethw.h"

/* Utility func. headers */
#include "utilnum.h"
#include "utlport.h"
/* GG header files */
#include "ggcfg.h"

#include "common/siuCommonMain.h"
#include "common/dsiunwal.h"
#include "rttrace.h"
#ifndef HDCT
#include "common/siuctbl.h"
#endif
#include <ti/mas/util/pkt.h>

#include "video/siuVigdkIbl.h"

#define loop    for(;;)     /* endless loop */
/* SIU Global variables */
siuContext_t  siuContext;                 /* SIU Global Context */
utlRtTraceContext_t utlRtTraceContext;    /* Realtime trace context */

#ifdef GG_BIOS
void siuBiosBkgdTask(void);
#endif

/* Debug stuff */
tuint siu_dbg_id;         /* debug ID used in debug and exception functions */
tuint siu_dbg_message[MGB_MSG_MAXNPAR*MGB_MSGPAR_SIZE];    /* debug message */

/*
 * The SIU context environment used by setjmp() and longjmp()
 * Note: the critical section in siuMain() will prevent any task 
 *       from ever pre-empting any messages processing within BIOS.  
 *       This allows us to share one setjmp buffer.
 * Note: message processing should store siu_env on the stack, 
 *       and restore it when message processing is done.
 */
jmp_buf  siu_env;
 
tuint emu_profile_ready=0;
#ifdef EMU_PROFILE
#ifdef _TMS320C6X
#include "emureg.h"

tuint dummy_mips[EMU_MIPS_DUMMY_WORDS/sizeof(tuint)];

#pragma DATA_SECTION (emu_profile_buf, ".emu_prof_var");
#pragma DATA_SECTION (emu_profile_new_mode, ".emu_prof_var");
#pragma DATA_SECTION (emu_profile_run_mode, ".emu_prof_var");
#pragma DATA_SECTION (emu_profile_idx, ".emu_prof_var");
#pragma DATA_SECTION (emu_profile_idx_save, ".emu_prof_var");
#pragma DATA_SECTION (emu_profile_size, ".emu_prof_var");
#pragma DATA_SECTION (emu_profile_base, ".emu_prof_var");

tuint *emu_profile_buf = dummy_mips;
tuint  emu_profile_new_mode = 0xffff,    /* new mode to program */
       emu_profile_run_mode = 0xffff;    /* running mode */
tulong emu_profile_idx = 0;              /* Circular buffer index */
tulong emu_profile_idx_save = 0;
tulong emu_profile_size = 0;
word  *emu_profile_base;

typedef struct mipsProfilingPktHdr_s {
  tulong seqNum;
  tulong timeStamp;
} mipsProfilingPktHdr_t;

mipsProfilingPktHdr_t mipsProfilingHdr;

/* See AU1_CNT_ESEL_ESEL0_* */

void emu_profile_change_mode(void) {
  extern const word gg_mips_profiling_core1_buf[];
  extern const tulong gg_mips_profiling_core1_buf_size;
  extern const vigdkMemoryContext_t gg_supported_builds[];
  extern const tuint gg_mips_profiling_pkt_size;
  const vigdkMemoryContext_t *activeBuild;
  tulong mipsBufSize;

  activeBuild = &gg_supported_builds[siuContext.buildID];

  thwCriticalBegin();
  TSCL = 0; /* Activate timer */
  if (emu_profile_new_mode == 0xffff) {
    emu_profile_size = 0;
    emu_profile_buf  = dummy_mips;
  } else {
    mipsBufSize      = activeBuild->mips_profiling.area_end - activeBuild->mips_profiling.area_begin;
    if (mipsBufSize > gg_mips_profiling_pkt_size) {
       mipsBufSize = gg_mips_profiling_pkt_size;
    }
    emu_profile_size = (mipsBufSize - sizeof(mipsProfilingPktHdr_t))/sizeof(tuint);
    emu_profile_base = (word *)siu_remap_mcs_data((void *)activeBuild->mips_profiling.area_begin);
    emu_profile_buf  = (tuint *)(emu_profile_base + sizeof(mipsProfilingPktHdr_t));
    mipsProfilingHdr.seqNum = 0;
  }
  emu_profile_idx = 0;
  emu_profile_run_mode = emu_profile_new_mode;
  thwCriticalEnd();
}

void emu_profile_mark_f(tuint task_id, tuint ch, tuint pushpop) 
{
    if (emu_profile_run_mode != 0xffff) {
      emu_profile_mark_i(task_id,ch,pushpop);
      if ( (emu_profile_idx + 3) > emu_profile_size) {
          /* ship the payload using nduTrace */
          xferPktInfo_t pkt;
          word      *payload;
          tint      pktLength;
          extern    NDU_gmacContext_t NDU_gmacContext;

          SIU_STATE_BEGIN();
          /* turn off emu_profile_run_mode so that NEU in the send out direction doesn't update profiling */
          emu_profile_run_mode = 0xffff;
          if ((NDU_gmacContext.nduTrace).neuInst != NULL) {
              pkt.srcPort = 0;
              pkt.npkts   = 1;
              payload = (word *)(emu_profile_base);
              mipsProfilingHdr.seqNum++;
              mipsProfilingHdr.timeStamp = siu_isr_clock;
              memcpy(payload,&mipsProfilingHdr,sizeof(mipsProfilingPktHdr_t));
              pktLength = emu_profile_idx*sizeof(tuint) + sizeof(mipsProfilingPktHdr_t);
              pkt.pktSize = &pktLength;
              pkt.pktIn   = (void *)&payload;
              pkt.type    = XFER_PKT_TYPE_NULL;
              (*gg_neu_ctable.neuSendIn) ((NDU_gmacContext.nduTrace).neuInst, &pkt);
          }
          /* turn back ON emu_profile_run_mode */
          emu_profile_run_mode = 0;
          emu_profile_idx = 0;
          SIU_STATE_END();
      }
    }
}
#else /* C55X */
#include "emureg.h"

extern tuint siuDbgBuf;

tuint *emu_profile_buf = &siuDbgBuf;
tuint  emu_profile_new_mode = 0xffff,    /* new mode to program */
       emu_profile_run_mode = 0xffff,    /* running mode */
       emu_profile_seqn = 0;             /* Output sequence number */
tint   emu_profile_idx = 0;              /* Circular buffer index */
/* See AU1_CNT_ESEL_ESEL0_* */

void emu_profile_change_mode(void) {
  tuint *loc_ptr;
  tint loc_idx;

  if (emu_profile_new_mode == 0xffff) {
    *AU1_CNT_CNTL_ADDR = (*AU1_CNT_CNTL_ADDR & AU1_CNT_CNTL_ACNT_STATE_MASK) | 
                         AU1_CNT_CNTL_RELEASE;
    /* Release access to non-CPU MMR */
    asm("  bit(ST2, #11) = #0  ");
  } else {
    // Set EALLOW to get access to non-CPU MMR
    asm("  bit(ST2, #11) = #1  ");

    if ((*AU1_CNT_ID_ADDR & AU1_CNT_ID_OWN_MASK) != AU1_CNT_ID_DEBUGGER_OWNS) {
      /* claim counter logic */
      *AU1_CNT_CNTL_ADDR = (*AU1_CNT_CNTL_ADDR & AU1_CNT_CNTL_ACNT_STATE_MASK) | 
                            AU1_CNT_CNTL_CLAIM;
                        
      /* was the claim given? */
      if ((*AU1_CNT_ID_ADDR & AU1_CNT_ID_OWN_MASK) == AU1_CNT_ID_APP_OWNS) {
   
        /* set counter 0 to count cache misses and counter 1 to count instruction NULL */
        *AU1_CNT_CNTL_ADDR = AU1_CNT_CNTL_CEN0_CNT_ON_EXT_CNT0   |
                             AU1_CNT_CNTL_FREE0_STOP_ON_DBG      |
                             AU1_CNT_CNTL_STOP0_NO_HALT_ON_MATCH |
                             AU1_CNT_CNTL_CM_ONE_32BIT_CNTR      |
                             0x2;
                        
        *AU1_CNT_ESEL_ADDR = 
                         AU1_CNT_ESEL_EQUAL0_DISABLE |
                         emu_profile_new_mode;
      } else {
        emu_profile_new_mode = 0xffff; /* Failed */
      }
    } else {
      emu_profile_new_mode = 0xffff; /* Failed */
    }
  }

  loc_idx = 0;
  loc_ptr = emu_profile_buf;
  loc_ptr[loc_idx++] = emu_profile_seqn++;
  loc_ptr[loc_idx++] = siu_isr_clock & 0xffff;
  loc_ptr[loc_idx++] = *AU1_CNT1_ADDR;
  loc_ptr[loc_idx++] = *AU1_CNT0_ADDR;
  emu_profile_idx = loc_idx;
  emu_profile_run_mode = emu_profile_new_mode;
}

void emu_profile_flush(void) {
  tuint *loc_ptr;
  tint loc_idx;
  struct {
    mgbHeader_t hdr;
    tuint data[EMU_PROFILE_MAX];
  } big_msg;

  memcpy (big_msg.data, emu_profile_buf, EMU_PROFILE_MAX);
  big_msg.hdr.chnum = 0;  /* Channel independent message */
  big_msg.hdr.msg_id = MGB_DM_DEBUG_TRACE;
  big_msg.hdr.length = MGB_HEADER_SIZE + emu_profile_idx;
  big_msg.hdr.trans_id = 0; /* Not a response to a message */

  /* Send message to MPU */
  siuNduSendMsg((void *)&big_msg);

  loc_idx = 0;
  loc_ptr = emu_profile_buf;
  loc_ptr[loc_idx++] = emu_profile_seqn++;
  loc_ptr[loc_idx++] = siu_isr_clock & 0xffff;
  loc_ptr[loc_idx++] = *AU1_CNT1_ADDR;
  loc_ptr[loc_idx++] = *AU1_CNT0_ADDR;
  emu_profile_idx = loc_idx;
}

void emu_profile_mark_f(tuint task_id, tuint ch, tuint pushpop) 
{
  emu_profile_mark_i(task_id,ch,pushpop);
}
#endif /* C64x/C55x */
#endif /* EMUPROF */
void cslDisableDmacInt(tint);
/******************************************************************************
 * FUNCTION PURPOSE: Exception function
 ******************************************************************************
 * DESCRIPTION: Relays a fatal exception through the global exception
 *              pointer.
 *****************************************************************************/
void siu_exception(tuint ID, tuint code)
{
   siuDebugInfo(ID, 
                SIU_IS_CHAN_EXCEP(code)? SIU_DEBUG_CHAN_EXCEP:SIU_DEBUG_FATAL, 
                SIU_GET_EXCEPTION_CODE(code), 
                0, 
                NULL);
}

#ifdef TRACE_DBG
/******************************************************************************
 * FUNCTION PURPOSE: Debug Trace function
 ******************************************************************************
 * DESCRIPTION: Implements a trace capability which logs specified data
 *              (i.e., value1 and value2) along with a time stamp in a
 *              circular debug trace buffer.  Each trace buffer entry consists
 *              of 4 word.  Used for integration debugging.
 *
 *****************************************************************************/

tuint  siu_trace_buffer[4*SIU_TRACE_BUF_SIZE];
tuint  *siu_trace_ptr  = &siu_trace_buffer[0];
tint   siu_trace_cnt   = 0;

void siu_trace_write( tuint value1, tuint value2  )
{
 tulong *p1 = (tulong *) siu_trace_ptr;
 
 *p1 = isr_clock;                         /* Log time stamp */
 siu_trace_ptr = siu_trace_ptr + 2;
 *siu_trace_ptr++ = value1;               /* Log value1 */
 *siu_trace_ptr++ = value2;               /* Log value2 */
 siu_trace_cnt++;
 if (siu_trace_cnt >= SIU_TRACE_BUF_SIZE)
 {
   siu_trace_cnt = 0;
   siu_trace_ptr = &siu_trace_buffer[0]; 
 }
}  /* siu_trace_write */

#endif /* TRACE_DBG  */

/******************************************************************************
 * FUNCTION PURPOSE: SIU Debug & exception functions.
 ******************************************************************************
 * DESCRIPTION: These functions put the id and debug information in global
 *              memory (i.e., can be read by an emulator) and send appropriate
 *              messages to MIC. They may also pack character strings into
 *              dprword's if necessary.
 *
 *****************************************************************************/


void siuDebugInternal (uint id, char *str, bool queue)
{
  int i;
  char *src;
  unsigned char c1, c2;
  tuint *dst;

  /* Copy id and message into a global buffer */
  siu_dbg_id = id;
  src = str; dst = (tuint *)&siu_dbg_message[0];
  for (i = 0; i < MGB_MSG_MAXNPAR*MGB_MSGPAR_SIZE; i++) {
#if (defined(BIGEND))
  /* BIG endian exception report */
    c1 = (unsigned char)(*src++);         /* no need to mask high bits */
    c2 = (unsigned char)(*src++ & 0xFFu); /*   since we shift them up */
#else
  /* LITTLE endian exception report */
    c2 = (unsigned char)(*src++);         /* no need to mask high bits */
    c1 = (unsigned char)(*src++ & 0xFFu); /*   since we shift them up */
#endif    
    *dst++ = (tuint)(c1 << 8 | c2);        /*   when packing into words */
  }

  /* send debug message to MIC */
  siuDebugReport (id, &siu_dbg_message[0], queue);
} /* siuDebugInternal */


void siuException (uint id, char *s)
{
  halControl_t halControl;
  thwDisableAllInt ();  /* disable all interrupts */
  siu_preserve_registers(SIU_CPU_REGISTERS_EXCEPTION, siuContext.siu_common_scratch);
  /* send debug message to DPR directly, bypassing the message fifo */
  siuDebugInternal (id, s, FALSE);

  /* Send alarm indication message to DPR */
  siuAlarm (id); 

  /* Flush the message through HAL */
  halControl.cmd = HAL_COMMAND_HALT;
  halControl.u.periphReset.portId =  NULL;
  
  (*siuContext.halApi->halControl) (&halControl);

  siu_dsp_halt(id, s);     /* loop forever */

} /* siuException */

/******************************************************************************
 * FUNCTION PURPOSE: SIU Channel exception function.
 ******************************************************************************
 * DESCRIPTION: The function performs the following actions:
 *              - Set Channel State to EXCEPTION
 *              - Prepare and Send the "MGB_DM_CHANNEL_EXCEPTION" message to the host
 *              - Invoke longjmp() to resume the execution 
 *
 *****************************************************************************/
void siuProcChanExcep (tuint moduleID, tuint error_code)
{

  siuInst_t *inst = (siuInst_t *)SIUINSTp((moduleID & SIU_CID_MASK) - SIU_FIRSTCH);
  tint      chnum = (tint)(inst->ID & SIU_CID_MASK);

  /* Restore interrupt */
  thwCriticalReset();

  SIU_STATE_BEGIN();
  
  /*************  Warning ************
   * Note: Use pending_state instead for HDGW 11.0 and above
   */
    inst->state = SIU_CHST_EXCEPTION;    /* declare this channel as in EXCEPTION mode */
    
  /* Disable all tasks for this channel */
  /* siuTxClose(inst); */
  /* siuRxClose(inst); */
#ifndef HDCT
  siu_queue_purge_channel (chnum, SIU_READ_TASK_PRIORITY_ID(inst));
#endif
  SIU_STATE_END();

  /* Send channel exception message to DPR */
  siuChannelException(moduleID, error_code);

  /* Resume task or message execution */
  longjmp(siu_env, 1);

} /* siuProcChanExcep */

/*-------------------------------------------------------------------------*/

/******************************************************************************
 * FUNCTION PURPOSE: Call siuGuaranteedMessageTaskProc directly (non-BIOS) or 
 *                   post event to trigger siuGuaranteedMessageTaskProc (BIOS)
 ******************************************************************************
 * DESCRIPTION:
 *
 *****************************************************************************/
void siuGuaranteedMessageTask ( void )
{
  jmp_buf  siu_env_tmp;   /* store the original siu_env */
#ifdef GG_BIOS
  NDU_control_t ctl;
#ifdef SIU_NDU_HETERO_PKT_IFACE 
  tuint    nduCallTblIdx;
#endif /*SIU_NDU_HETERO_PKT_IFACE*/
#endif /*GG_BIOS*/

    /* Run the HAL poll task */
#ifdef PA_SS
  dsiuNwalPoll ();
#else    
  if(*siuContext.halApi->halPoll !=NULL)
  {
    (*siuContext.halApi->halPoll)((tuint)siu_timer_tick, siuContext.numActiveChannels);
  }
#endif
  /* 
   * store the original siu_env and call setjmp()
   * If channel-based exception occur during the message processing, the
   * program execution will resume here 
   */
  memcpy(&siu_env_tmp, &siu_env, sizeof(jmp_buf)); 
  
  setjmp(siu_env);

#ifdef GG_BIOS
  /* Check if there is message in FIFO */
  SIU_STATE_BEGIN();
#ifdef SIU_NDU_HETERO_PKT_IFACE
  nduCallTblIdx = 
     SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                           SIU_CTL_PKT_IDX_MASK,
                           SIU_CTL_PKT_IDX_SHIFT);

  NDU_Control(siuContext.ifaceNduCallTable[nduCallTblIdx], NDU_QUERY_MSG_FIFO, &ctl);
#else  /*SIU_NDU_HETERO_PKT_IFACE*/
  NDU_Control(NDU_pCallTable, NDU_QUERY_MSG_FIFO, &ctl);
#endif /*SIU_NDU_HETERO_PKT_IFACE*/
  SIU_STATE_END();

  /* Post event for triggering message task if there is message in FIFO */
  if(ctl.out_params.msg_in_fifo) {
    siu_osal_post_signal(SIU_OSAL_SIGNAL_HANDLE_MESSAGE);
  }
#else  /*GG_BIOS*/
  siuGuaranteedMessageTaskProc();
#endif /*GG_BIOS*/

  /* message processing is complete successfully, restore the siu_env */
  memcpy(&siu_env, &siu_env_tmp, sizeof(jmp_buf));

}  /* siuGuaranteedMessageTask() */

/******************************************************************************
 * FUNCTION PURPOSE: Messages from MICRO are guaranteed to be processed 
 *                   at least at a given rate.
 ******************************************************************************
 * DESCRIPTION:
 *
 *****************************************************************************/
void siuGuaranteedMessageTaskProc ( void )
{
#ifdef SIU_NDU_HETERO_PKT_IFACE
 tuint           nduCallTblIdx;
#endif

  siuContext.msgTaskActiveFlag = 1;

#ifdef SIU_NDU_HETERO_PKT_IFACE
 nduCallTblIdx = 
     SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                           SIU_CTL_PKT_IDX_MASK,
                           SIU_CTL_PKT_IDX_SHIFT);

  /* Check for new messages from MICRO */
  NDU_GetMsg(siuContext.ifaceNduCallTable[nduCallTblIdx],NDU_INTERRUPT_MESSAGE);

  /* TODO: To make interrupt messages work on non-utopia builds, another 
   * nduFifo2DPR may be needed below
   */
  NDU_GetMsg(siuContext.ifaceNduCallTable[nduCallTblIdx],NDU_NORMAL_MESSAGE);

  /* Send normal priority messages to micro. If there are none, send
   * any low priority messages  */
  NDU_MsgBg(siuContext.ifaceNduCallTable[nduCallTblIdx]);

#else
  /* Check for new messages from MICRO */
  NDU_GetMsg(NDU_pCallTable,NDU_INTERRUPT_MESSAGE);

  /* TODO: To make interrupt messages work on non-utopia builds, another 
   * nduFifo2DPR may be needed below
   */
  NDU_GetMsg(NDU_pCallTable,NDU_NORMAL_MESSAGE);

    /* Send normal priority messages to micro. If there are none, send
   * any low priority messages  */
  NDU_MsgBg(NDU_pCallTable);

#endif 
    
  siuContext.msgTaskActiveFlag = 0;
} /* siuGuaranteedMessageTaskProc() */

/******************************************************************************
 * FUNCTION PURPOSE: Provide Restart indication to MIC.
 ******************************************************************************
 * DESCRIPTION: Provides a restart indication message to MIC.
 *
 *****************************************************************************/
void siuRestartIndication (tuint revId)
{
  extern tuint dspImageMap[];
  mgb_t      send_message;
  tuint  *dstImageptr = (tuint *)&send_message.msg_body;
  int i;
  /* Prepare message */
  send_message.header.length   = MGB_HEADER_SIZE + 48;
  send_message.header.chnum    = 0;  /* Use channel number 0 */
  send_message.header.trans_id = 0;  /* Not a response to a MIC message */
  send_message.header.msg_id   = MGB_DM_RESTART_INDICATION;

  /* copy the entire DSP image data.  This is hardcoded to 32, just as length
   * is hardcoded to 37 */
  /* Copy is done using utlWrite16bits_m instead of memcpy.This is done for
     c64x to work in little endian confguration.Compiler generates proper code 
     for c54x and c55x since utlWrite16bits macro is used with constant byte offset */
  for (i = 0;i < 48;i++) {
    pktWrite16bits_m ((word *)dstImageptr,0,dspImageMap[i]);
    dstImageptr++;
  }

  /* Put the chip revision ID. This is not known at compile time */
  send_message.msg_body.restart_indication.chip_rev_id = revId;
  
  /* send message to FIFO */
  siuNduSendMsg(&send_message); 
} /* siuRestartIndication */

/******************************************************************************
 * FUNCTION PURPOSE:  Provides READY indication to MIC.
 ******************************************************************************
 * DESCRIPTION:  Sends a READY indication to MIC.  Sent acknowleging a valid
 *               hardware config message. 
 *
 *****************************************************************************/
void siuReadyIndication(mgb_t *message)
{
  mgb_t send_message;

  send_message.header.length = MGB_HEADER_SIZE; /* no msg. body */
  send_message.header.chnum  = 0;  /* use channel number 0 - not channel specific */
  send_message.header.msg_id = MGB_DM_READY_INDICATION;
  send_message.header.trans_id = message->header.trans_id;
  
  /* send message to FIFO */  
  siuNduSendMsg(&send_message);  
} /* siuReadyIndication() */ 

/******************************************************************************
 * FUNCTION PURPOSE: Low Priority CP background processing task
 ******************************************************************************
 * DESCRIPTION:
 *
 *****************************************************************************/
#ifdef GG_BIOS
void siuBiosBkgdTask(void)
{
#ifdef GG_INCLUDE_MDMCP
  int   i;
  tint  state;
#endif
  /* Static is OK; there is only one context of background task in system */
  static volatile tulong old_siu_isr_clock = 0;

#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_IDLE,0)
#endif  

#ifdef GG_INCLUDE_MDMCP
  /* Loop through all channels */
  for (i = 0; i < siuContext.maxNumberChannels; i++)
  {
    state = siuInst[i].state;

    /* Run MDMCP background task if the channel is in any _CP_ state */
    if ((state == SIU_CHST_CP_IDLE) ||
        (state == SIU_CHST_CP_DTMF) ||
        (state == SIU_CHST_CP_CLEAR_DATA) ||
        (state == SIU_CHST_CP_MODEM_TERM) ||
        (state == SIU_CHST_CP_FAX_TERM))
    {
      gg_siu_ctable.Mdmcp.siuMdmcpBGTask(&siuInst[i]);
    }
  }  
#endif

   /* 
    * This function looks for built-in DIM messages in external memory. If the 
    * next message is for this core, it will get placed in the message FIFOs 
    * to be picked up during the message task. 
    */
   siuVigdkIblCfg();

  if (siuContext.state != SIU_ST_WORKING) 
    /* post the siuMain task */
    siu_osal_post_signal(SIU_OSAL_SIGNAL_HANDLE_TASKSCHEDULER);

  /* Feed the watchdog / keepalive */
#ifdef GG_INCLUDE_HEALTH_CHECK
  /* Don't waste MIPS busy-waiting inside the health check */
  if (old_siu_isr_clock != siu_isr_clock) {
    /* Critical section needed since this is also called from siuTxRxSwi */
    thwCriticalBegin();
    siuHealthMain(siu_timer_tick, siuContext.state);
    thwCriticalEnd();
    old_siu_isr_clock = siu_isr_clock;
  }
#endif

#ifdef EMU_PROFILE
  if( siuContext.state == SIU_ST_WORKING ) {
    tulong siu_isr_clock_old;
    volatile tulong *siu_isr_clock_p = &siu_isr_clock;
    siu_isr_clock_old = *siu_isr_clock_p;
    while (siu_isr_clock_old == *siu_isr_clock_p);
  }
#endif /*EMU_PROFILE*/

#ifdef EMU_PROFILE
    EMU_PROFILE_END()
#endif /*EMU_PROFILE*/

} /* siuBiosBkgdTask() */
#endif /* GG_BIOS */

/* nothing after this point */
