#ifndef _SIUCMPND_H
#define _SIUCMPND_H
/******************************************************************************
 * FILE PURPOSE: SIU Call table definitions for compound message support.
 ******************************************************************************
 * FILE NAME:   siuneu.h
 *
 * DESCRIPTION: Contains local SIU definitions for compound message support.
 *
 * TABS: NONE
 *
 * (C) Copyright 2001, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "siuloc.h"
#include "ndu.h"

extern void siuCmpndMsgFromMicro(NDU_msgPriority_e priority, void *buff);
extern void siuCmpndMsgToMicroBegin (tuint mtype, tuint channel, tuint trans_id);
extern bool siuCmpndMsgToMicroAdd (NDU_DPRMsgType_t mtype, void *vmsg);
extern void siuCmpndMsgToMicroEnd (tuint mtype);

typedef struct {
  void (*msgFromMicro)     (NDU_msgPriority_e priority, void *buff);
  void (*msgToMicroBegin)  (tuint mtype, tuint channel, tuint trans_id);
  bool (*msgToMicroAdd)    (NDU_DPRMsgType_t mtype, void *vmsg);
  void (*msgToMicroEnd)    (tuint mtype);
} siuCmpndCallTable_t;

extern siuCmpndCallTable_t gg_cmpnd_ctable;
/* pointer to background context scratch area, used for
 * DSP->Micro compound message support. */
extern const UINT16 *gg_compound_scratch;

#if NDU_HDSIM
/* vpbx has smaller HPI */
#define HD_COMPOUND_MSG_MAX_LENGTH 64
#else
/* vs. HD platform */
#define HD_COMPOUND_MSG_MAX_LENGTH MGB_HD_COMPOUND_MSG_MAX_LENGTH
#endif

#endif /* _SIUCMPND_H */

/* Nothing past this point */
