#ifndef _SIUNDU_H
#define _SIUNDU_H
/******************************************************************************
 * FILE PURPOSE: SIU NDU (Network Driver Unit) interface description
 ******************************************************************************
 * FILE NAME: siundu.h
 *
 * DESCRIPTION: Defines the call table used for the ndu
 *
 * TABS: NONE
 *
 * (C) Copyright 2001, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "common/siuloc.h"
#include "mgmicdsp.h"

tuint sinNduSendMsg(mgb_t *msg);
tint  siuNduSendPkt(void *inst, xferPktInfo_t *pktIn);

/* The diag code creates the ndu call table in ndu */
#ifdef DSIU
extern siuNduCallTable_t siuNduCallTable;
#endif


#ifdef NDU_UTOPIA
#define SIU_NDU_GET_NEU_RECEIVE_IN  (gg_neu_ctable.neuReceiveInUt)
#else
#define SIU_NDU_GET_NEU_RECEIVE_IN  (gg_neu_ctable.neuReceiveInGmac)
#endif

#define SIU_NDU_CALL_TABLE_DEF_IDX      0   /* Index in the array of 
                                             * of call table for 
                                             * default interface
                                             */
#if defined(NDU_UTOPIA) && defined(NDU_GMAC)
#define SIU_NDU_HETERO_PKT_IFACE            /* Indication for presence of support
                                             * heterogeneous PKT interface
                                             */
#define SIU_NDU_CALL_TABLE_GMAC_IDX     1   /* Second index */
#define SIU_NDU_MAX_CALL_TABLE          2   /* One for each interfaces */
#else
#define SIU_NDU_CALL_TABLE_GMAC_IDX     SIU_NDU_CALL_TABLE_DEF_IDX   /* Default index */
                                            
#define SIU_NDU_MAX_CALL_TABLE          1   /* One for each interfaces */
#endif

#define SIU_NDU_PKT_IFACE_DEFAULT       0
#define SIU_NDU_PKT_IFACE_UTOPIA        1
#define SIU_NDU_PKT_IFACE_GMAC          2

#endif /* _SIUNDU_H */

