/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "ggcfg.h"
#include "ggswcfg.h" 
#include "siuloc.h"
#ifndef HDCT
#include "common/siucmpnd.h"
#include "common/siuctbl.h"
#endif
#include "ndu.h"
#include "common/dsiunwal.h"
#include <ti/mas/iface/ifpkt/xferpkt.h>
#include "rttrace.h"

#ifndef DSIU
#include "diagapp.h"
void siuCopyMsgInit (diagApp_t *appInit);
extern diagApp_t diagAppMem;

#endif

extern siuContext_t siuContext;

tint AM_ProcessData(tuint extID, xferPktInfo_t *pktOut)
{
 tint       intID;
 siuInst_t *inst;
 intID = extID;

/****************************************************************************
 * Diagnostic code does not support dynamic buffer channel maps
 ****************************************************************************/

   inst = SIUINSTp(intID - SIU_FIRSTCH);
   /*
    * For HD, all the packets should be delivered and examized at Idle state
    * For non-HD branch, please be cautious and evaluate whether you want to
    * pick up the change
    */

    if((inst->state == SIU_CHST_CLOSED) || (inst->state == SIU_CHST_EXCEPTION))
     return NDU_ERROR;
      

    thw_change_data_page(SIU_READ_PRIMARY_DATA_PAGE(inst));

    pktOut->type    =  XFER_PKT_TYPE_AAL1; /* Always use aal1 */
    
    switch (inst->amInst.pktLoopback)
    {
      /* snap the connection to receiveIn to other units 
      in case of send packet loopback */
    case NDU_CTRL_SND_LOOPBACK_ON:
        /* don't call the receiveOut() */
        break;
    case NDU_CTRL_RCV_LOOPBACK_ON:
        /* call separate receive packet loopback function to route the packet */
        siuNduSendPktCore(inst, pktOut);
        break;
    default:
        if(inst->amInst.RxOutSpec.nduReceiveOut != NULL)
        {
         (*inst->amInst.RxOutSpec.nduReceiveOut)
                (inst->amInst.RxOutSpec.targetInst, pktOut);
        }
        break;
    }
    return NDU_NOERR;
}

void AM_ProcessMsg(NDU_msgPriority_e level, mgb_t *msgBuf)
{

#if defined(RELMSG_SUPPORT)
  /* Message Ack Processing, TRUE if the message should be dropped */
  if(siuProcMsgAck(level, msgBuf))
    return;
#endif

#ifndef HDCT
  if(gg_cmpnd_ctable.msgFromMicro) 
  {  
    gg_cmpnd_ctable.msgFromMicro (level, (void *)msgBuf);
  } 
  else 
  {
    siuProcessMessage(level, msgBuf); 
  }
#endif

}

#ifdef NDU_UTOPIA
void siuUtnduInit()
{
 NDU_config_t     config;

 NDU_pCallTable = &NDU_utCallTable;
 
 /* Verify that the HAL interface supports utopia send out  */
   if (!siuContext.halApi->utopia->halUtopiaSendCell)
   return;  

 config.pProcessMsg      = AM_ProcessMsg;
 config.pProcessData     = AM_ProcessData;
 config.pAal5SendCell    = siuContext.halApi->utopia->halUtopiaSendCell;
 config.pAal5SendProcess = aal5_send_process;
 config.pAal5Receive     = aal5_receive_process;
 config.pUtMsgInit       = siuCopyMsgInit;
 config.pAnuRec          = gg_siu_ctable.Anu.rec_ut;
 config.pAnuXmt          = gg_siu_ctable.Anu.xmt_ut; /* Neu for ANU */
 config.pNvNeuCreate     = gg_siu_ctable.Neu.new_nv; /* For creating NEUs */
 config.pNvneuOpen       = gg_neu_ctable.neuOpen;
 config.pNvneuControl    = gg_neu_ctable.neuControl;
 config.pNvneuReceiveIn  = gg_neu_ctable.neuReceiveInUt;
 config.pNvneuSendIn     = gg_neu_ctable.neuSendIn;
 config.fUtopiaAnu       = (gg_siu_ctable.Anu.new != NULL);
 config.phalUtopiaControl = siuContext.halApi->utopia->halUtopiaControl;
 config.utopia_portid    = 0; /* TODO: Hardcoded port ID.This needs to be
                                     configurable */
 config.coreId           = siuContext.coreId; /* coreId is required in NDU, but initialized in siuHal.c */
                                      
                                     
 siuContext.cellSize = (NDU_pCallTable->pConfig)(&config); 
}
#endif /* NDU_UTOPIA */

#ifdef NDU_GMAC
void siuGmacnduInit()
{
 NDU_config_t     config;

 NDU_pCallTable = &NDU_gmacCallTable;

 #ifndef PA_SS
 /* Verify that the HAL interface supports utopia send out  */
   if (!siuContext.halApi->gmac->halGmacSendPacket)
   return;  
   
 config.halGmacSendPacket = siuContext.halApi->gmac->halGmacSendPacket;
 #else
 config.halGmacSendPacket = dsiuPktSendOut;
 #endif

 config.pProcessMsg      = AM_ProcessMsg;
 config.pProcessData     = AM_ProcessData;
#if 0
 config.pAal5SendProcess = aal5_send_process;
 config.pAal5Receive     = aal5_receive_process;
#else
 config.pAal5SendProcess = NULL;
 config.pAal5Receive     = NULL;
#endif
 config.pUtMsgInit       = siuCopyMsgInit;
#ifdef GG_INCLUDE_ANNOUNCEMENTS
 config.pAnuRec          = gg_siu_ctable.Anu.rec_ut;
 config.pAnuXmt          = gg_siu_ctable.Anu.xmt_ut; /* Neu for ANU */
#endif
 config.pNvNeuCreate     = siu_new_nv_neu; /* For creating NEUs */
 config.pNvneuOpen       = gg_neu_ctable.neuOpen;
 config.pNvneuControl    = gg_neu_ctable.neuControl;
 config.pNvneuReceiveIn  = gg_neu_ctable.neuReceiveInGmac;
 config.pNvneuSendIn     = gg_neu_ctable.neuSendIn;
 config.pNeuGetInstance  = gg_neu_ctable.neuGetInstance;
#ifdef GG_INCLUDE_ANNOUNCEMENTS
 config.fUtopiaAnu       = (gg_siu_ctable.Anu.new != NULL);
#endif
 config.phalUtopiaControl = NULL;
 config.utopia_portid    = 0; /* TODO: Hardcoded port ID.This needs to be
                                     configurable */
 config.coreId           = siuContext.coreId; /* coreId is required in NDU, but initialized in siuHal.c */
 config.halId            = siuContext.halIdMsg;
                                      
                                     
 (NDU_pCallTable->pConfig)(&config); 
}
#endif /* NDU_GMAC */

#ifdef NDU_HD_NDU
void siuHdnduInit()
{
    NDU_config_t config;

 NDU_pCallTable = &NDU_hdndu_calltable;

 config.pProcessMsg      = AM_ProcessMsg;
 config.pProcessData     = AM_ProcessData;
 config.phalApiCtrl      = siuContext.halApi->halHpiControl;
 config.pAnuRec          = gg_siu_ctable.anu.rec_hd;
 (*NDU_pCalltable->pConfig)(&config);
}
#endif /* NDU_HD_NDU */

#ifdef NDU_LEGACY_HPI
void siuNduLegInit()
{  
 NDU_config_t     config;

 NDU_pCallTable = &NDU_callTable;

 config.start_address = thwDprBaseAddress();
 config.pHalApiCtrl   = siuContext.halApi->halHpiControl;
 config.pProcessMsg   = AM_ProcessMsg;
 config.pProcessData  = AM_ProcessData;
 siuContext.hpiMem    = config.start_address;
 (*NDU_pCallTable->pConfig)(&config);
}
#endif /* NDU_LEGACY_HPI */


#ifdef NDU_NEW_HPI
extern tuint hpi_descriptor[];

void siuNduv1Init()
{
 NDU_config_t     config;

 NDU_pCallTable = &NDU_nduv1CallTable;

  config.pHalApiCtrl   = siuContext.halApi->halHpiControl;
  config.start_address = hpi_descriptor;
  config.pProcessMsg   = AM_ProcessMsg;
  config.pProcessData  = AM_ProcessData;
  config.pPcmPktTrace  = siuPcmPktTrace;
  (*NDU_pCallTable->pConfig)(&config);
}
#endif /* NDU_NEW_HPI */

void siuNduStart()
{
  NDU_control_t ctl;
    /* utndu hdndu can be used in same build, which is selected in run time */
#ifdef NDU_UTOPIA
  NDU_pCallTable = &NDU_utCallTable;
 siuUtnduInit();
#endif /* NDU_UTOPIA */

#ifdef NDU_GMAC
 NDU_pCallTable = &NDU_gmacCallTable;
 siuGmacnduInit();
#endif /* NDU_GMAC */

#ifdef NDU_HD_HPI
 NDU_pCallTable = &NDU_callTable;
 siuHdnduInit();
#endif /* NDU_HD_HPI */

#ifdef NDU_LEGACY_HPI 
 NDU_pCallTable = &NDU_callTable;
 siuNduLegInit();
#endif /* NDU_LEGACY_HPI */

#ifdef NDU_NEW_HPI
 NDU_pCallTable = &NDU_callTable;
 siuNduv1Init();
#endif /* NDU_NEW_HPI */
    /* Determine if NDU supports configurable message priority 
       and store it in siuContext.This flag is used in function
       siuNduSendMsgPriority. */ 
    NDU_Control(NDU_pCallTable,NDU_QUERY_PRIORITY_SUPPORT,&ctl);
    siuContext.msg_priority_config_support = ctl.out_params.priority_supported;
}

tuint siuNduSendMsg(mgb_t *msg)
{
 bool            fDrop;
 NDU_DPRMsgType_t  level;
#ifdef SIU_NDU_HETERO_PKT_IFACE
 tuint           nduCallTblIdx;
#endif

 fDrop = TRUE;

 level = siuMessageLevel(msg->header.msg_id);

    msg->header.echo = 0x1; /* non zero value */

    if(msg->header.msg_id != MGB_DM_COMPOUND_MSG)
 {
   /* for dynamic channel allocation, convert channel id */
   if (! ((msg->header.msg_id == MGB_DM_ERROR_INDICATION) &&
        (MGB_ERR_GET_MODULE_ID_CODE(msg->msg_body.error_indication.error_code) == 
         MGB_ERR_INVD_MSG_CHST_CLOSED))) {
     msg->header.chnum = siu_translate_dynamic_chnum(msg->header.msg_id, 
                                                   msg->header.chnum);
   }
#ifndef HDCT
   if(gg_cmpnd_ctable.msgToMicroAdd) 
   {
     if(gg_cmpnd_ctable.msgToMicroAdd (level, (void *)msg)) 
     {
       /* Compound message collection intercepted this simple message: */
       return(0);
     }
   }
#endif
 }

 if((msg->header.msg_id == MGB_DM_DEBUG_INFO) ||
    (msg->header.msg_id == MGB_DM_ALARM_INDICATION) ||
    (msg->header.msg_id == MGB_DM_COMPOUND_MSG))
 {
   fDrop = FALSE; /* message can not be dropped */
 }
 
#ifdef SIU_NDU_HETERO_PKT_IFACE
 nduCallTblIdx = 
     SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                           SIU_CTL_PKT_IDX_MASK,
                           SIU_CTL_PKT_IDX_SHIFT);
 NDU_SendMsg(siuContext.ifaceNduCallTable[nduCallTblIdx],msg, level, fDrop);
#else
 NDU_SendMsg(NDU_pCallTable,msg, level, fDrop);
#endif
 
 return(msg->header.msg_id);
}

void siuNduSendSimpleMsgPriority(mgb_t *msg, NDU_DPRMsgType_t priority)
{
	bool	        	fDrop;
#ifdef SIU_NDU_HETERO_PKT_IFACE
    tuint           nduCallTblIdx;
#endif

	fDrop = TRUE;
    if (siuContext.msg_priority_config_support == FALSE) {
    /* If NDU does not support configurable priority discard the passed 
       priority and get the message priority from the msg_id by static mapping
       funtion */  
      priority = siuMessageLevel(msg->header.msg_id);
    }

       msg->header.echo = 0x1; /* non zero value */
    	if((msg->header.msg_id == MGB_DM_DEBUG_INFO) ||
	   (msg->header.msg_id == MGB_DM_ALARM_INDICATION) ||
	   (msg->header.msg_id == MGB_DM_COMPOUND_MSG))
	{
		fDrop = FALSE; /* message can not be dropped */
	}
#ifdef SIU_NDU_HETERO_PKT_IFACE
    nduCallTblIdx = 
        SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                              SIU_CTL_PKT_IDX_MASK,
                              SIU_CTL_PKT_IDX_SHIFT);
    NDU_SendMsg(siuContext.ifaceNduCallTable[nduCallTblIdx],msg, priority, fDrop);
#else
    NDU_SendMsg(NDU_pCallTable,msg, priority, fDrop);
#endif

}

/******************************************************************************
 * FUNCTION PURPOSE: Send Message with configurable priority to HOST
 ******************************************************************************
 * DESCRIPTION: Function receives the priority for the message to be sent to 
 *              host instead of calculating the priority from msgid.The only 
 *              difference between siuNduSendMsg and this function is that 
 *              here there is no call to siuMessageLevel function which maps
 *              each msg ID to a priority.
 *
 *  tuint siuNduSendMsgPriority (
 *    mgb_t *msg,                 - message to be sent 
 *    NDU_DPRMsgType_t priority)  - priority of the message
 *
 *  RETURN VALUE: Returns msg id of message to be sent
 *
 *****************************************************************************/
tuint siuNduSendMsgPriority (mgb_t *msg,NDU_DPRMsgType_t priority)
{
 bool            fDrop;
#ifdef SIU_NDU_HETERO_PKT_IFACE
 tuint           nduCallTblIdx;
#endif

 fDrop = TRUE;

    if (siuContext.msg_priority_config_support == FALSE) {
    /* If NDU does not support configurable priority discard the passed 
       priority and get the message priority from the msg_id by static mapping
       funtion */  
      priority = siuMessageLevel(msg->header.msg_id);
    }

    msg->header.echo = 0x1; /* non zero value */

    if(msg->header.msg_id != MGB_DM_COMPOUND_MSG) {
      /* for dynamic channel allocation, convert channel id */
      if (! ((msg->header.msg_id == MGB_DM_ERROR_INDICATION) &&
        (MGB_ERR_GET_MODULE_ID_CODE(msg->msg_body.error_indication.error_code) == 
         MGB_ERR_INVD_MSG_CHST_CLOSED))) {
        msg->header.chnum = siu_translate_dynamic_chnum(msg->header.msg_id, 
                                                   msg->header.chnum);
      }
#ifndef HDCT
      if(gg_cmpnd_ctable.msgToMicroAdd) {
        if(gg_cmpnd_ctable.msgToMicroAdd (priority, (void *)msg)) {
          /* Compound message collection intercepted this simple message: */
          return(0);
        }
      }
#endif
    }

 if((msg->header.msg_id == MGB_DM_DEBUG_INFO) ||
    (msg->header.msg_id == MGB_DM_ALARM_INDICATION) ||
    (msg->header.msg_id == MGB_DM_COMPOUND_MSG)) {
   fDrop = FALSE; /* message can not be dropped */
 }
#ifdef SIU_NDU_HETERO_PKT_IFACE
    nduCallTblIdx = 
        SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                              SIU_CTL_PKT_IDX_MASK,
                              SIU_CTL_PKT_IDX_SHIFT);
    NDU_SendMsg(siuContext.ifaceNduCallTable[nduCallTblIdx],msg, priority, fDrop);
#else
    NDU_SendMsg(NDU_pCallTable,msg, priority, fDrop);
#endif
 return(msg->header.msg_id);
}
/***************************************************************************
 * FUNCTION PURPOSE: Core function which does packet SendIn.
 ****************************************************************************
 *
 ***************************************************************************/
tint siuNduSendPktCore(siuInst_t *inst, xferPktInfo_t *pktIn)
{
 tuint     extID;
#ifdef SIU_NDU_HETERO_PKT_IFACE
 tuint           nduCallTblIdx;
#endif

 extID = (inst->ID & SIU_CID_MASK); /* extract channel number */
#if (defined(NDU_HD_NDU))
  /* The input tag value can be overwritten if the packet is to be routed to
   * the host, independent of the type */
  if (inst->amInst.hd_pkt_routing_type == SIU_HDNDU_PCKTS_TO_HOST)
   pktIn->type = SIU_HDNDU_PCKTS_TO_HOST;
#endif

#ifdef SIU_NDU_HETERO_PKT_IFACE
 nduCallTblIdx = 
     SIU_PKT_GET_IDX_IFACE(inst->pkt_iface_idx,
                           SIU_CHAN_PKT_IDX_MASK,
                           SIU_CHAN_PKT_IDX_SHIFT);
  /* Send Operation */
  return(NDU_SendData(siuContext.ifaceNduCallTable[nduCallTblIdx],extID, pktIn));
#else
  return(NDU_SendData(NDU_pCallTable,extID, pktIn));
#endif
}

 /***************************************************************************
 * FUNCTION PURPOSE: Send data packets toward the network.
 ****************************************************************************
 * DESCRIPTION:  Bridge function, calls  siuNduSendPktCore
 *               - Has the logic to do packet loopbacks
 *
 ***************************************************************************/
tint siuNduSendPkt(void *siuInst, xferPktInfo_t *pktIn)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
#ifdef NDU_GMAC  
  int i;

 /* This is the proper place for trace */
 for (i=0;i<pktIn->npkts;i++) {
    /* Chop up into cells */
      utlRtTrace (inst->ID,pktIn->pktSize[i], 
                  ((word *)pktIn->pktIn[i]),
                  MGB_RTTRACE_STREAM_ID_PKT_OUT, UTL_RTTRACE_PACKED);
  }
#endif

#ifdef NDU_UTOPIA  
{
  int i;
  tint cellOffset, cellSizeWords, cellSizeBytes, pktSizeWords;

 
  /* Do packet trace regardless of loopbacks */
  cellSizeBytes = siuContext.cellSize;
  cellSizeWords = UTL_BYTES_TO_WORDS(siuContext.cellSize);

  for (i=0;i<pktIn->npkts;i++) {
    pktSizeWords = UTL_BYTES_TO_WORDS(pktIn->pktSize[i]);
    /* Chop up into cells */
    for(cellOffset = 0; 
      cellOffset < pktSizeWords; 
      cellOffset += cellSizeWords) {
      utlRtTrace (inst->ID, cellSizeBytes, 
                  ((word *)pktIn->pktIn[i]) + cellOffset,
                  MGB_RTTRACE_STREAM_ID_PKT_OUT, UTL_RTTRACE_PACKED);
    }
  }

}  
#endif
  
  
  
  if(inst->amInst.pktLoopback != NDU_CTRL_RCV_LOOPBACK_ON)
  {
    if(inst->amInst.pktLoopback != NDU_CTRL_SND_LOOPBACK_ON)
    {
      return (siuNduSendPktCore(inst, pktIn));
    }
    else
    {
       #ifdef NDU_GMAC 
       /*
        * For GMAC operation, the NEU validation information, which is required by the second pass, 
        * will be extracted from the first NEU pass when the packet is received from the network.
        * For Network TX loopback, we need to call the NEU first pass processing function to extract
        * the validation information.
        *
        * Please note that the NEU validation information may not be complete if the first pass 
        * return NULL. We simply drop the packet in this case.
        */
       void                 *vInst; 
       neuValidationInfo_t  vInfo;

       memset(&vInfo,0,sizeof(neuValidationInfo_t));
       pktIn->suppInfo =  &vInfo;
       #endif
    
      thwCriticalBegin();
      
      #ifdef NDU_GMAC 
      /* first pass processing */
      if ((vInst = (*gg_neu_ctable.neuGetInstance) (pktIn)) == NULL){
       thwCriticalEnd();
       return (0);
      }
      #endif
      
      (*inst->amInst.RxOutSpec.nduReceiveOut)
          (inst->amInst.RxOutSpec.targetInst,pktIn);
      thwCriticalEnd();
    }
  }
 return(0);
}
/****************************************************************************
 * FUNCTION PURPOSE: Keep the health independent
 *****************************************************************************/
void siuNduHealthSend(tint seqNum)
{
#ifdef SIU_NDU_HETERO_PKT_IFACE
  tuint           nduCallTblIdx;
#endif

#ifdef SIU_NDU_HETERO_PKT_IFACE
  nduCallTblIdx = 
     SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                           SIU_CTL_PKT_IDX_MASK,
                           SIU_CTL_PKT_IDX_SHIFT);
  NDU_SendHealth(siuContext.ifaceNduCallTable[nduCallTblIdx],seqNum);
#else
  NDU_SendHealth(NDU_pCallTable,seqNum);
#endif
}

/****************************************************************************
 * FUNCTION PURPOSE: Complete announcement request
 ****************************************************************************
 * DESCRIPTION: The announcement request has passed through NEU and sent
 *              here. Final cell formatting is done before being sent
 *              to HAL.
 ****************************************************************************/
tint siu_anu_rqst_send (void *inst, xferPktInfo_t *pkt)
{
  NduNvInst_t          *nvInst = (NduNvInst_t *)inst;
  siuInst_t            *sInst;
  tint    chnum;

  sInst = SIUINSTp(nvInst->vcIdTx - SIU_FIRSTCH);
  
    /* vcIdTx contains the non-mapped channel number, 1 indexed. Convert this
     * to the mapped value for the route tag header. */
    chnum = nvInst->vcIdTx;

    NDU_FormatAnuFromNeu(inst, pkt,chnum);
  /* Get the channel system instance for passing to ndu */
  /* Send out the cells */
  siuNduSendPktCore(sInst, pkt);

  return (0);

} /* siu_anu_rqst_send  */


/****************************************************************************
 * FUNCTION PURPOSE: Returns the current free space available in NDU FIFO
 *****************************************************************************
 * DESCRIPTION: The function returns the free space available in the fifo
 *              identified by the queue_id (Message Priority).This function
 *              is used to check if enough space is available in the fifo 
 *              before writing to the FIFO.The returned value is in NDU units.
 *              NOTE:This function returns 0 incase there is an error in NDU
 *              when it returns the free space.
 ****************************************************************************/
tuint siuNduGetFree (NDU_DPRMsgType_t queue_id)
{
#ifdef SIU_NDU_HETERO_PKT_IFACE
 tuint           nduCallTblIdx;
#endif

  NDU_control_t ctl;
  tint          retStatus;
  ctl.input_params.msg_priority = queue_id;

#ifdef SIU_NDU_HETERO_PKT_IFACE
 nduCallTblIdx = 
     SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                           SIU_CTL_PKT_IDX_MASK,
                           SIU_CTL_PKT_IDX_SHIFT);
 retStatus = NDU_Control(siuContext.ifaceNduCallTable[nduCallTblIdx],NDU_QUERY_FIFOFREE,&ctl);
#else
 retStatus = NDU_Control(NDU_pCallTable,NDU_QUERY_FIFOFREE,&ctl);
#endif


  if (retStatus == NDU_NOERR)
      return (ctl.out_params.fifo_free);
  else
      return 0; /* Incase of error in NDU Control return 0 */
}

#ifdef NDU_GMAC
#ifdef GG_INCLUDE_ANNOUNCEMENTS
/****************************************************************************
 * FUNCTION PURPOSE: Format an announcement for GMAC NDU
 ****************************************************************************
 * DESCRIPTION: The raw announcement request is provided. The request passed
 *              through NEU and the resulting group of cells is sent to 
 *              siu_anu_rqst_send.
 *****************************************************************************/

tint siu_anu_format_gmac (void *siu_inst, xferPktInfo_t *pktInfo)
{
  siuInst_t       *inst = (siuInst_t *)siu_inst;
  tuint           instID = inst->ID;
  

#ifdef LITTLE_ENDIAN
  /* The announcement requests are not byte swapped by the micro FIX-ME!!! TEMP*/
  utlBigEndianTransform((tuint *)pktInfo->pktIn[0],4);
#endif
 return(NDU_SendAnuToNeu(instID, pktInfo));
 
}
#endif
#endif

#if defined(NDU_UTOPIA) || defined (NDU_GMAC)

/****************************************************************************
 * FUNCTION PURPOSE: Format an announcement for utopia NDU
 ****************************************************************************
 * DESCRIPTION: The raw announcement request is provided. The request passed
 *              through NEU and the resulting group of cells is sent to 
 *              siu_anu_rqst_send.
 *****************************************************************************/
#ifdef GG_INCLUDE_ANNOUNCEMENTS
tint siu_anu_format_ut (void *siu_inst, xferPktInfo_t *pktInfo)
{
  siuInst_t       *inst = (siuInst_t *)siu_inst;
  tuint           instID = inst->ID;
  
#ifdef LITTLE_ENDIAN 
/* In the MCG Platform, which is a Phase 3 platform, the announcement requests
  are not serviced by the micro (the xdphw,where byte swap usually is done for little
  endian messages) but are serviced by the winpath, which currently do not 
  do the byte-swapping as desired. So, for phase 3 platform working with
  little endian DSP (eg,MCG platform) this byte swapping needs to be done,
  scaffold atleast */

  utlBigEndianTransform((tuint *)pktInfo->pktIn[0],4);
#endif
 return(NDU_SendAnuToNeu(instID, pktInfo));
 
}
#endif

/****************************************************************************
 * FUNCTION PURPOSE: HAL DMA move complete callback
 *****************************************************************************
 * DESCRIPTION: Serves as a function which is provided to hal as a callback.
 *              When this function is called it simply sets the global variable
 *              siu_hal_copy_done to 1 and exits.
 ****************************************************************************/
#ifndef DSIU
static volatile tint siu_copy_done_flag;
tint siu_copy_done (void *v)
{
  siu_copy_done_flag = 1;
  return (0);
} /* siu_copy_done */
#endif


/*****************************************************************************
 * FUNCTION PURPOSE: Copy initial utopia configuration information
 *****************************************************************************
 * DESCRIPTION: Copies the configuration for the utopia that was left by the
 *              bootloader.
 *****************************************************************************/
#ifndef DSIU
void siuCopyMsgInit (diagApp_t *appInit)
{
 siu_copy_done_flag = 0;
#ifdef __TMS320C55X__
 if(siuContext.halApi->halGlobMemMove!=NULL)
 {
  halGlobAddr_t dst, src;

  /* The source structure is in program memory at a location defined in
  * diagapp.h */
  src = (halGlobAddr_t)&diagAppMem;
  HAL_MEM_ADDRESS_PROG(src);
  HAL_MEM_SHARED_ADDR(src);
  
  /* The destination is in local data memory    *
  * get the Global Address for the Destination */
  dst = (*siuContext.halApi->halMapLocalToGlobal) ((halLocAddr_t)appInit) ;
  HAL_MEM_ADDRESS_DATA(dst);

 /* Invoke the HAL to copy the data. Have the hal call the function 
       siu_copy_done when the transfer is complete */
  (*siuContext.halApi->halGlobMemMove) (dst, src, sizeof(diagApp_t), 
                                         siu_copy_done, NULL);
 }
 else

#else

 {
   tuint *src = (tuint *)&diagAppMem ; 
   tuint *dst = (tuint *)appInit;
   memcpy (dst,src,sizeof(diagApp_t));
   siu_copy_done_flag=1;
 }                                        

#endif

 /* siu_copy_done sets siu_copy_done_flag to 1 when the copy is complete. */
 while (!siu_copy_done_flag);

 siuContext.dspId  = appInit->dspId;  /* Core ID setup in siuhal.c */
} /* siuCopyMsgInit */
#endif
#endif /* NDU_UTOPIA */  

