/******************************************************************************
 * FILE PURPOSE: SIU Timer Routines.
 ******************************************************************************
 * FILE NAME:   siutmr.c
 *
 * DESCRIPTION: Contains SIU timer routines.
 *
 * TABS: NONE
 *
 * $Id: siutmr.c 1.1 1997/09/15 17:20:18 BOGDANK Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: siutmr.c $
 * Revision 1.1  1997/09/15 17:20:18  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"      /* DSP types */
#include "siuloc.h"     /* siu_timer_tick reference */
#include "siutmr.h"     /* prototypes */

/******************************************************************************
 * FUNCTION PURPOSE: Calculate Delta Time.
 ******************************************************************************
 * DESCRIPTION: Calculates delta time given two time measurements. Takes into
 *              account possible wrap around of a timer counter.
 *
 *  tulong siuTmrCalcDTime (
 *    const tulong new_time,        - recent time measurement
 *    const tulong old_time)        - old time measurement
 *
 *****************************************************************************/

tulong siuTmrCalcDTime (const tulong new_time, const tulong old_time)
{
  return (new_time - old_time);   /* this should always work! */
} /* siuTmrCalcDTime */

/******************************************************************************
 * FUNCTION PURPOSE: Get time.
 ******************************************************************************
 * DESCRIPTION: Returns time in ms. One must call siuTmrTick every ms in order
 *              to keep time units correct. Critical section is NOT used.
 *
 *  tulong siuTmrGetTime (void)
 *
 *              RETURN VALUE: time in ms.
 *
 *****************************************************************************/

tulong siuTmrGetTime (void)
{
  return (siu_timer_tick);
} /* siuTmrGetTime */

/******************************************************************************
 * FUNCTION PURPOSE: Reset timer.
 ******************************************************************************
 * DESCRIPTION: Resets timer tick to zero. Critical section is NOT used.
 *
 *  void siuTmrReset (void)
 *
 *****************************************************************************/

void siuTmrReset (void)
{
  siu_timer_tick = 0;
} /* siuTmrReset */
/* nothing past this point */
