#ifndef _SIUTMR_H
#define _SIUTMR_H
/******************************************************************************
 * FILE PURPOSE: SIU Timer Definitions and Inlined Routines.
 ******************************************************************************
 * FILE NAME:   siutmr.h
 *
 * DESCRIPTION: Contains SIU timer definitions and inlined routines.
 *
 * TABS: NONE
 *
 * $Id: siutmr.h 1.1 1997/09/15 17:20:19 BOGDANK Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: siutmr.h $
 * Revision 1.1  1997/09/15 17:20:19  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"      /* DSP types */
#include "siuloc.h"     /* siu_timer_tick reference, etc. */

/* External functions */

tulong  siuTmrCalcDTime (const tulong new_time, const tulong old_time);
tulong  siuTmrGetTime   (void);
void    siuTmrReset     (void);

/* Inlined functions */

/******************************************************************************
 * FUNCTION PURPOSE: Increment timer tick.
 ******************************************************************************
 * DESCRIPTION: Increments timer tick. Resolution is assumed to be 1ms.
 *              Hence, if the ISR occurs every 3ms, one should call this
 *              function three times within the ISR.
 *
 *  void siuTmrTick (void)
 *
 *****************************************************************************/

static inline void siuTmrTick (void)
{
  siu_timer_tick++;
} /* siuTmrTick */

#endif  /* _SIUTMR_H */
/* nothing past this point */
