/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "siuCustomEco.h"
#include "common/siuloc.h"

/******************************************************************************
 * FUNCTION PURPOSE: Executes Video Overlay Configuration
 ******************************************************************************/
void siu_new_custom_eco (tint chnum) {
  tint         stat;
  memBuffer_t  customEcoBufs[SIU_MAX_NBUFS];
  customEcoNewCfg_t customEcoCfgNew;
  customEcoSizeCfg_t customEcoCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  customEcoCfgNew.ID             = siuMakeID (SIU_MID_CUSTOM_ECO, chnum);
  customEcoCfgNew.sizeCfg        = customEcoCfgSize;

  stat = siuNewEco (inst, (siuEcoGetSizes_t)customEcoGetSizes, &customEcoCfgSize, 
                    (siuEcoNew_t)customEcoNew, &customEcoCfgNew, 
                    &(inst->stateInst.video.customEcoInst),  customEcoBufs, SIU_MID_CUSTOM_ECO);
  
  siu_exc_assert (stat == SIU_NOERR, SIUCUSTOMECO_EXC_NEW_CUSTOM_ECO, inst);
}

/******************************************************************************
 * FUNCTION PURPOSE: Executes Video Overlay Configuration
 ******************************************************************************/
void siu_open_custom_eco (tint chnum, tuint codingType, tuint width, tuint height) {
  customEcoCfg_t customEcoCfg;
  siuInst_t *inst;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  customEcoCfg.coreId       = siuContext.coreId;
  customEcoCfg.codingType   = codingType;
  customEcoCfg.img_width    = width;
  customEcoCfg.img_height   = height;
  customEcoCfg.action_type  = 0;

  /* open custom ECO */
  if (gg_custom_eco_ctable.customEcoOpen) {
    if ((gg_custom_eco_ctable.customEcoOpen)(inst->stateInst.video.customEcoInst,&customEcoCfg) != CUSTOM_ECO_NOERR) {
       siu_exc_assert (0, SIUCUSTOMECO_EXC_OPEN_CUSTOM_ECO, inst);
    }
  }
}
/******************************************************************************
 * FUNCTION PURPOSE: Executes Video Overlay Configuration
 ******************************************************************************
 * DESCRIPTION: Run-time configuration for the Custom Eco
 *
 *  void siuCustomEcoCfg (
 *    siuInst_t *inst         - pointer to channel information
 *    mgb_t     *message)     - pointer to micro message
 *
 *****************************************************************************/
void siuCustomEcoCfg (void *siuInst, mgb_t *message) {
  siuInst_t *inst = (siuInst_t *)siuInst;
  customEcoCtrl_t ctrl;
  customEcoCfg_t cfg;
  
  ctrl.control    = CUSTOM_ECO_SET;
  cfg.action_type = message->msg_body.custom_eco_cfg.actionType;
  gg_custom_eco_ctable.customEcoControl(inst->stateInst.video.customEcoInst, &cfg, &ctrl);
  return;
}

void siu_custom_msg_processing (void *siuInst, mgb_t *message){
  siuInst_t *inst = (siuInst_t *)siuInst;

  /* Custom message processing code */


  return;
}

/* nothing past this point */

