#ifndef _SIUMMCU_H
#define _SIUMMCU_H
/******************************************************************************
 * FILE PURPOSE: SIU Call table definitions for MMCU.
 ******************************************************************************
 * FILE NAME:   siummcu.h
 *
 * DESCRIPTION: Contains local SIU definitions for MMCU.
 *
 * TABS: NONE
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siuloc.h"
#include <ti/mas/mmcu/mmcu.h>

#define SIU_PMT_PID     256
#define SIU_PCR_PID     512
#define SIU_VIDEO_PID   512
#define SIU_AUDIO_PID   513

#define SIU_VIDEO_STREAM_TYPE   ts_STREAM_TYPE_H264_VIDEO
#define SIU_AUDIO_STREAM_TYPE   ts_STREAM_TYPE_AAC_AUDIO

typedef struct mmcMapping_s {
  tint            mmcType;      /**< Media Container numer as defined in vimgmicdsp.h         */
  const char     *mmcName;      /**< Media container string name */
} mmcMapping_t;

void siu_new_mmcu(tint chnum);
void siu_open_mmcu(siuInst_t *inst);
void siu_mmcu_parse_header(siuInst_t *inst);
void siu_mmcu_send_header(siuInst_t *inst);
void siuMmcuCfg(siuInst_t *inst, mgb_t *message);

#endif /* _SIUMMCU_H */
