/*
 *******************************************************************************
 *
 * C64x+ SMP M/C Interface definition
 * 
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/**
  ******************************************************************************
  *  @file     ividmc.h
  *
  *  @brief    Video Codec Multicore Interface Header
  *
  *  @author   
  *
  *  @version  0.1 - Feb, 3,2010 : Initial Version
  *
  *****************************************************************************
*/

/**
 *  @defgroup   C64x+ Multicore IVIDMC
 *  @ingroup    C64P
 *
 */

#ifndef _IVIDMC_H_  //--{
#define _IVIDMC_H_

#include <ti/xdais/ialg.h>
#include <ti/xdais/xdas.h>

#define IVIDMC_SHMEM_ATTRS_DDR_CACHED   (0)
#define IVIDMC_SHMEM_ATTRS_DDR_UNCACHED (1)
#define IVIDMC_SHMEM_ATTRS_SL2          (2)



typedef XDAS_Int32 IVIDMC_SPINLOCK;
typedef enum {
  IVIDMC_TASK_MASTER  = 0,
  /* Nonspecific slave task, useful for Symetrical MP */
  IVIDMC_TASK_SLAVE   = 1,
  /* Specific task partitioning is possible for Asymetrical MP */
  IVIDMC_TASK_SLAVE_1 = 2,
  IVIDMC_TASK_SLAVE_2 = 3,
  IVIDMC_TASK_SLAVE_3 = 4,
  IVIDMC_TASK_SLAVE_4 = 5,
  IVIDMC_TASK_SLAVE_5 = 6,
  IVIDMC_TASK_SLAVE_6 = 7,
  IVIDMC_TASK_SLAVE_7 = 8,
  IVIDMC_TASK_SLAVE_8 = 9
} IVIDMC_TASK_e;

typedef XDAS_Int32 IVIDMC_SWBARR;
typedef enum {
  IVIDMC_SWBARR_FIRST = 0,
  IVIDMC_SWBARR0  = 0,
  IVIDMC_SWBARR1  = 1,
  IVIDMC_SWBARR2  = 2,
  IVIDMC_SWBARR3  = 3,
  IVIDMC_SWBARR4  = 4,
  IVIDMC_SWBARR5  = 5,
  IVIDMC_SWBARR6  = 6,
  IVIDMC_SWBARR7  = 7,
  IVIDMC_SWBARR8  = 8,
  IVIDMC_SWBARR9  = 9,
  IVIDMC_SWBARR10 = 10,
  IVIDMC_SWBARR11 = 11,
  IVIDMC_SWBARR12 = 12,
  IVIDMC_SWBARR13 = 13,
  IVIDMC_SWBARR14 = 14,
  IVIDMC_SWBARR15 = 15,
  IVIDMC_SWBARR_LAST  = 15,
  IVIDMC_NUM_BARRIERS = 16
} IVIDMC_SWBARR_e;

typedef XDAS_Int32  IVIDMC_SHMEMKEY;
typedef XDAS_UInt32 IVIDMC_SHMEM_ATTRS;
typedef enum {
  IVIDMC_SHMEMKEY_FIRST = 0,
  IVIDMC_SHMEM_KEY_0  = 0,
  IVIDMC_SHMEM_KEY_1  = 1,
  IVIDMC_SHMEM_KEY_2  = 2,
  IVIDMC_SHMEM_KEY_3  = 3,
  IVIDMC_SHMEM_KEY_4  = 4,
  IVIDMC_SHMEM_KEY_5  = 5,
  IVIDMC_SHMEM_KEY_6  = 6,
  IVIDMC_SHMEM_KEY_7  = 7,
  IVIDMC_SHMEM_KEY_8  = 8,
  IVIDMC_SHMEM_KEY_9  = 9,
  IVIDMC_SHMEM_KEY_10 = 10,
  IVIDMC_SHMEM_KEY_11 = 11,
  IVIDMC_SHMEM_KEY_12 = 12,
  IVIDMC_SHMEM_KEY_13 = 13,
  IVIDMC_SHMEM_KEY_14 = 14,
  IVIDMC_SHMEM_KEY_15 = 15,
  IVIDMC_SHMEMKEY_LAST  = 15,
  IVIDMC_SHMEM_NUM_KEYS = 16,
  IVIDMC_SHMEM_KEY_CLEAN = 0xFFFF
} IVIDMC_SHMEM_KEY_e;

/* Indication that this is unused core. DNUM can not be equal to this value */
#define IVIDMC_CORE_NOT_USED 0xF

/* Following data structure is included in static params, provided by the framework in Video Codec creation phase */
typedef struct {
  /* Function pointers, to be set by the framework */
  /* Software barrier - codec generates call-out and sends number (swbarr_cnt) of cores from the team (<ncores)   */
  /* that need to reach the software barrier, before execution proceeds. It is possible to have multiple software */
  /* barriers in the system present simultaneosly. This is why codec needs to identify barrier with swbarr_id parameter */
  /* Core needs to provide correct value for swbarr_cnt, or 0 */
  XDAS_Void (*swbarr) (XDAS_Int32 coreID, IVIDMC_SWBARR swbarr_id, XDAS_UInt32 swbarr_cnt);
  
  /* Dynamically allocate shared memory region identified with global key "shmem_key". */
  /* If region with same global key "shmem_key" is already allocated, its base address is provided */
  /* Attribute provided here indicates shared memory region placement: SL2 or DDR.    */
  /* Meaning of alignment is identical to ialg/xdais definition. Having alignment in the API allows
     the Codec to request a shared buffer with required alignment */
  XDAS_Int32 *(*shmmap) (XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 shmem_size, IVIDMC_SHMEM_ATTRS attr, XDAS_Int32 alignment);
  /* Deallocate shared memory region. It is completely recycled after all clients do unmap operation */
  XDAS_Int32 (*shmmunmap)  (XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 *shmem_base);
  /* Synchronize/flush shared memory region - explicit cache coherency operation */
  XDAS_Int32 (*shmmap_sync)(XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 *shmem_base);

  /* Critical region lock acquire - in M/C environment */
  /* If spinlock has been succesfully acquired, this function returns 1 */
  /* If same spinlock has been already acquired by another participant, this function returns 0 */
  XDAS_Int32  (*Lock)  (XDAS_Int32 coreID, IVIDMC_SPINLOCK spinlock);
  /* Critical region lock release */
  XDAS_Void  (*UnLock)(XDAS_Int32 coreID, IVIDMC_SPINLOCK spinlock);

  /* Identification parameters, to be set by the framework */
  XDAS_Int32 ncores;           /* Number of cores in the team - i.e. cores running together processing same stream */
  IVIDMC_TASK_e core_task_ID;  /* Core task Identification: Reserved MASTER_CORE */
  XDAS_Int32 coreID;
} IVIDMC_t;

#endif
