/******************************************************************************
 * FILE PURPOSE: Multi-core video codecs header file
 ******************************************************************************
 * FILE NAME: siuVidMc.h
 *
 * DESCRIPTION:
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#ifndef _SIUVID_MC_H
#define _SIUVID_MC_H

#include <xdc/std.h>
#include <ti/xdais/dm/xdm.h>
#include "ividmc.h"
#include "types.h"

/* Maximum number of cores running concurrently */
#define SIUVID_MC_MAXCORES (CPU_NCORES)

typedef struct SIUVID_MC_Shared_MemTab{
    XDAS_Int32 *baseptr[SIUVID_MC_MAXCORES][IVIDMC_SHMEM_NUM_KEYS];
    XDAS_Int32 attribute[SIUVID_MC_MAXCORES][IVIDMC_SHMEM_NUM_KEYS];
    XDAS_Int32 size[SIUVID_MC_MAXCORES][IVIDMC_SHMEM_NUM_KEYS];
    XDAS_UInt8 *shmem_DDR2;
    XDAS_UInt8 *shmem_SL2;
    XDAS_Int32 shmemsize_DDR2; 
    XDAS_Int32 shmemsize_SL2; 
    XDAS_UInt8 *outputBitStreamPtr[SIUVID_MC_MAXCORES];
    XDAS_Int32 outputBitStreamSize[SIUVID_MC_MAXCORES];
} SIUVID_MC_SHMEM_MemTab;

/*********************************************************************************
 * STRUCTURE PURPOSE: Store the Handles for codec shared internal/external memory
 *********************************************************************************
  DESCRIPTION: siuVidShmHandle_t is part of siuContext and updated in Init()
 *********************************************************************************/
typedef struct siuVidShmHandle_s {
  /* handle to the codec internal-share memory */
  word *intShm;
  /* size of the codec internal-share memory */
  tulong intShmSize;
  /* handle to the codec external-share memory */
  word *extShm;
  /* size of the codec external-share memory */
  tulong extShmSize;
} siuVidShmHandle_t;

XDAS_Int32 siuVidMc_Init_ShmemTab(XDAS_Int32 Core_ID);
XDAS_Void siuVidMc_Swbarr(XDAS_Int32 core_id, IVIDMC_SWBARR sync_point, XDAS_UInt32 swbarr_cnt);
XDAS_Int32 *siuVidMc_Map_Shmem(XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 shmem_size, IVIDMC_SHMEM_ATTRS attr,
                               XDAS_Int32 alignment);
XDAS_Int32 siuVidMc_Unmap_Shmem(XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 *shmem_base);
XDAS_Int32 siuVidMc_Shmem_Sync(XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 *shmem_base);
XDAS_Void siuVidMc_Set_outputBitStreamInfo(XDAS_Int32 Core_ID, XDAS_UInt8 *bitStreamPtr, XDAS_Int32 bitStreamSize);
XDAS_Void siuVidMc_Get_outputBitStreamInfo(XDAS_Int32 Core_ID, XDAS_UInt8 **bitStreamPtr, XDAS_Int32 *bitStreamSize);
#endif
/* nothing past this point */
