#ifndef _GATEMPBARRIER_H
#define _GATEMPBARRIER_H
/******************************************************************************
 * FILE PURPOSE: Declarations and definitions used by software barrier
 ******************************************************************************
 * FILE NAME:   GateMPBarrier.h
 *
 * DESCRIPTION: Contains local definitions and API for implementing software
 *              barrier using BIOS IPC
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <stdio.h>
#include <xdc/std.h>

/* Cache line size when implementing softare barrier on SL2 */
#define CACHELINESIZE   64
/* The maximal number of sync points */
#define MAXSYNCPOINT    16

#define BARRIER_AVAILABLE    0
#define BARRIER_NOTAVAILABLE 1


/* Padding to match cache line size */
typedef union UInt32_padding {
  Int32 nCoresInBar;
  Int32 barAvailFlag;
  Char  padding[CACHELINESIZE];
} UInt32_padding;

/* Software barrier supporting multiple sync points */
typedef struct Bar {
    UInt32_padding max;                   /* The total number of cores to be in software barrier */
    UInt32_padding count[MAXSYNCPOINT];   /* Real time number of cores which are currently in software barrier */
    UInt32_padding release[MAXSYNCPOINT]; /* Flag to indicate if all the cores enter/leave the barrier */
} Bar;


Bar *Bar_create(String name, Int max, void* barrierTable);

Bar *Bar_open(String name, void* barrierTable);

Bool Bar_wait(Bar *bar, UInt32 syncPoint, GateMP_Handle gateBarrier);

#endif
/* nothing past this point */
