#ifndef _SIUIPCBIOS6LOC_H
#define _SIUIPCBIOS6LOC_H
/******************************************************************************
 * FILE PURPOSE: BIOS6 IPC local definitions
 ******************************************************************************
 * FILE NAME:   siuIpcBios6Loc.h
 *
 * DESCRIPTION: Contains local definitions used by BIOS6 IPC related functions
 *              
 * TABS: NONE
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h" 

/* XDC runtime module headers */
#include <xdc/runtime/System.h>
#include <xdc/runtime/IHeap.h>
#include <xdc/runtime/Memory.h>

/* BIOS6 module headers */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/family/c66/Cache.h>
#include <ti/sysbios/interfaces/ICache.h>

/* IPC module headers */
#include <ti/ipc/Ipc.h>
#include <ti/ipc/SharedRegion.h>
#include <ti/ipc/GateMP.h>
#include <ti/ipc/MultiProc.h>

#include <ti/sdo/utils/_NameServer.h>

/* IPC barrier */
#include "GateMPBarrier.h"

/* Get globals from cfg header */
#include <xdc/cfg/global.h>

#include "common/siuloc.h"

typedef struct biosIpcContext_s {
  Bar* ipcBarrier;                            /* IPC barrier */
  GateMP_Handle gateBarrier;                  /* Gate for barrier */ 
  GateMP_Handle ipcGate;                      /* IPC gate */
  ipcContext_t *ipcContext;                   /* IPC context passed from application */
} biosIpcContext_t; 

extern cregister volatile unsigned int DNUM;

static inline void simTaskSleep(void)
{
    /* The following code is used to emulate Task_sleep(1) so that the slave cores will not
    trap the master core with continuous interrupts */
    tulong siu_isr_clock_old;
    volatile tulong *siu_isr_clock_p = &siu_isr_clock;
    siu_isr_clock_old = *siu_isr_clock_p;
    while (siu_isr_clock_old == *siu_isr_clock_p);
}

#endif
