#ifndef _SIUIPCMSGBIOS6LOC_H
#define _SIUIPCMSGBIOS6LOC_H
/******************************************************************************
 * FILE PURPOSE: 
 ******************************************************************************
 * FILE NAME:   siuIpcMsBios6Loc.h
 *
 * DESCRIPTION: Contains local definitions used by functions in siuIpcMsgBios6.c 
 *              
 * TABS: NONE
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h" 
#include "osal/siuIpcMsgOsal.h"
#include "siuIpcBios6Loc.h"

/* IPC module headers */
#include <ti/ipc/MessageQ.h>
#include <ti/ipc/HeapBufMP.h>

/* BIOS6 module headers */
#include <ti/sysbios/knl/Task.h>

#define HEAP_ID     0
#define HEAP_NAME   "siuIpcMsgHeap"
/* Number of IPC message blocks which will be allocated. This is 
   seen by all the cores. The value depends on size of the shared
   region used for message allocation, the message size, and the
   cache line size. The above information is in platform.xs and 
   bios6ipc.cfg under ggcfg\build\hdg\<tv01>\bios\ directory.
   In Video GDK, sizeof(msgIpcBios_t) + appMsgSize < 64. 
   Cache line size is 64/128 for SL2/DDR2; Shared region size is
   0x1000/0x2000 for SL2/DDR2.*/
#define MAX_IPC_MSG_SEND_PER_CORE   (16)  /* Number of IPC messages allocated to a core */
#define HEAP_NUM_BLOCK      (CPU_NCORES*MAX_IPC_MSG_SEND_PER_CORE)

typedef struct msgIpcBios_s {
   MessageQ_MsgHeader header;
   bool consumed;
   void* msgApp;
} msgIpcBios_t;

typedef struct biosIpcMsgContext_s {
  msgIpcBios_t *ipcMsg[MAX_IPC_MSG_SEND_PER_CORE];  /* BIOS IPC messages allcoated for the core */
  tuint ipcMsgIndex;                                /* Index of BIOS IPC message 
                                                       which will be used next */
  MessageQ_QueueId remoteQueueId[CPU_NCORES]; /* Array to store remote MessageQ ID. 
                                                 A MessageQ_QueueId is 32-bit value 
                                                 whose top 16-bits are the coreId and
                                                 bottom bits are the queue id */
  MessageQ_Handle  messageQ;                  /* Local MessageQ */
  tuint ipc_message_shared_region_id;         /* ID of shared region used for message 
                                                 allocation.Shared regions are defined
                                                 in the BIOSIPC configuration file
                                                 ggcfg\build\hdg\tv01\bios\bios6ipc.cfg:
                                                 Region 1: Shared L2; Region 2: DDR2 */
  bool ipc_message_open_heap;                 /* Flag to indicate if HeapBufMP has 
                                                 been opened for the core */
  ipcMsgContext_t *ipcMsgContext;             /* IPC context passed from application */

} biosIpcMsgContext_t;

void siuIpcBiosMessageQCreate(void);
void siuIpcBiosMessageQOpen(tuint remote_core);
void siuIpcBiosMessageQControl(xdc_UArg arg0, xdc_UArg arg1);
#endif
