#ifndef _SIUIPCMSGOSAL_H
#define _SIUIPCMSGOSAL_H
/******************************************************************************
 * FILE PURPOSE: Declarations and definitions used by inter-core messaging
 ******************************************************************************
 * FILE NAME:   siuIpcMsgOsal.h
 *
 * DESCRIPTION: Contains declaration of OSAL interprocess messaging functions
 *              
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h" 

/* Intercore comunication configuration */
typedef struct siuInterCoreConn_s {
  tuint dec_intercore;	 /* Bitmap indicating decoder core (upper byte) and channel (lower byte, 1-based) */
  tuint enc_intercore;   /* Bitmap indicating encoder core (upper byte) and channel (lower byte, 1-based) */
} siuInterCoreConn_t;

#define MAX_SIU_INTERCORE_CONN (CPU_NCORES)

extern siuInterCoreConn_t siuInterCoreConn[];

#define SIU_GET_INTER_CORE_CONN_ENC_COREID(i)          (siuInterCoreConn[i].enc_intercore >> 8)
#define SIU_GET_INTER_CORE_CONN_ENC_CHANID(i)          (siuInterCoreConn[i].enc_intercore & 0x00FF)
#define SIU_GET_INTER_CORE_CONN_DEC_COREID(i)          (siuInterCoreConn[i].dec_intercore >> 8)
#define SIU_GET_INTER_CORE_CONN_DEC_CHANID(i)          (siuInterCoreConn[i].dec_intercore & 0x00FF)

#define SIU_INTER_CORE_CONN_ENC_COREMATCH(i, coreId)      (SIU_GET_INTER_CORE_CONN_ENC_COREID(i) == (tuint) coreId)
#define SIU_INTER_CORE_CONN_ENC_CHANMATCH(i, chanId)      (SIU_GET_INTER_CORE_CONN_ENC_CHANID(i) == (chanId + 1))
#define SIU_INTER_CORE_CONN_DEC_COREMATCH(i, coreId)      (SIU_GET_INTER_CORE_CONN_DEC_COREID(i) == (tuint) coreId)
#define SIU_INTER_CORE_CONN_DEC_CHANMATCH(i, chanId)      (SIU_GET_INTER_CORE_CONN_DEC_CHANID(i) == (chanId + 1))

#define SIU_INTER_CORE_CONN_ENC_MATCH(i, coreId, chanId)      (SIU_INTER_CORE_CONN_ENC_COREMATCH(i, coreId) && SIU_INTER_CORE_CONN_ENC_CHANMATCH(i, chanId))
#define SIU_INTER_CORE_CONN_DEC_MATCH(i, coreId, chanId)      (SIU_INTER_CORE_CONN_DEC_COREMATCH(i, coreId) && SIU_INTER_CORE_CONN_DEC_CHANMATCH(i, chanId))

#define SIU_INTER_CORE_CONN_ENC_DEC_COREMATCH(i, encCoreId, decCoreId) ( SIU_INTER_CORE_CONN_ENC_COREMATCH(i, encCoreId) && SIU_INTER_CORE_CONN_DEC_COREMATCH(i, decCoreId))
#define SIU_INTER_CORE_CONN_ENC_DEC_MATCH(i, encCoreId, encChanId, decCoreId, decChanId) (SIU_INTER_CORE_CONN_ENC_MATCH(i, encCoreId, encChanId) && SIU_INTER_CORE_CONN_DEC_MATCH(i, decCoreId, decChanId))

#define SIU_INTER_CORE_CONN_ENC_IS_RESET(i)          (siuInterCoreConn[i].enc_intercore == 0)
#define SIU_INTER_CORE_CONN_DEC_IS_RESET(i)          (siuInterCoreConn[i].dec_intercore == 0)
#define SIU_INTER_CORE_CONN_IS_RESET(i)              (SIU_INTER_CORE_CONN_ENC_IS_RESET(i) && SIU_INTER_CORE_CONN_DEC_IS_RESET(i))

#define SIU_RESET_INTER_CORE_CONN_ENC(i)          siuInterCoreConn[i].enc_intercore = 0
#define SIU_RESET_INTER_CORE_CONN_DEC(i)          siuInterCoreConn[i].dec_intercore = 0

#define SIU_SET_INTER_CORE_CONN_ENC(i, coreId, chanId)  siuInterCoreConn[i].enc_intercore = (coreId << 8) | (chanId + 1)
#define SIU_SET_INTER_CORE_CONN_DEC(i, coreId, chanId)  siuInterCoreConn[i].dec_intercore = (coreId << 8) | (chanId + 1)


/* BIOS Events used in siuIpcMessageQControl() */
#define  EVENT_ID_MESSAGEQ_CLOSE        0 /* Point to BIOS event (Event_Id_01) for closing remote MessageQ */


/* Intercore comunication configuration */
typedef struct ipcChannel_s {
  tuint remote_core; /* Reomte core to receive a message */
  tuint remote_chan; /* Chan on the remote core to receive a message */
  tuint local_core;  /* Local core to send a message */
  tuint local_chan;  /* Chan on the lcoal core to send a message */
} ipcChannel_t;

/* information needs to be communicated between application and BIOS */
typedef struct ipcMsgContext_s {
  tuint sharedRegionType; /* Type of shared memory used for IPC messaging: shared L2 or DDR */
#define SHARED_REGION_TYPE_SL2         1
#define SHARED_REGION_TYPE_DDR         2
  tuint appMsgSize;       /* Size of application message */
  void  (*msgRevCallout) (void*); /* Callout after receiving an IPC message */
} ipcMsgContext_t;

void siu_ipc_osal_create_msgq_task(void);
void siu_ipc_osal_msg_init(void);
void siu_ipc_osal_msg_start(void *ipcMsgContext);
void siu_ipc_osal_event(tuint event);
void siu_ipc_osal_msgq_open_alloc(void);
void siu_ipc_osal_msgq_close(void);
void siu_ipc_osal_msg_send(void *msg, tuint appMsgSize, ipcChannel_t *ipcChan);
void siu_ipc_osal_set_msg_consumed(void *msg);
void siu_ipc_osal_remove_pending_msg(tuint chnum);
#endif
