#ifndef _SIUIPCOSAL_H
#define _SIUIPCOSAL_H
/******************************************************************************
 * FILE PURPOSE: Declarations and definitions used by IPC 
  *****************************************************************************
 * FILE NAME:   siuIpcOsal.h
 *
 * DESCRIPTION: Contains declaration of OSAL interprocess communication functions
 *              
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h" 

#define NUM_CORE_FROM_CFG 0            

/* IPC information between application and BIOS IPC */
typedef struct ipcContext_s {
  bool  ipcStarted;       /* Flag to indicate if IPC has been started */
  tuint nCoresInBarrier;  /* Number of cores that will be using the IPC barrier */
  tuint masterCoreId;     /* Core ID of master core */
  tuint shmSize;          /* Size of shared global */
  void* codecShm;         /* Base of shared global */
} ipcContext_t;

/* IPC init and startup */
void siu_ipc_osal_init(void);
void siu_ipc_osal_start(void *ipcContext);
void siu_ipc_osal_attach_owner(tuint nCores);
void siu_ipc_osal_attach_remote(tuint remote_core);

/* Software barrier */
void siu_ipc_osal_barrier_create(tuint nCoresInBarrier);
void siu_ipc_osal_barrier_open(tuint masterCoreId);
void siu_ipc_osal_barrier_wait(tuint syncPoint);

/* Shared global */
void siu_ipc_osal_create_share(tuint shmSize, void**shareBase);
void siu_ipc_osal_open_share(void**shareBase);
tuint siu_ipc_osal_enter_share(void* shareBase, tuint shareSize);
void siu_ipc_osal_leave_share(void* shareBase, tuint shareSize, tuint gateKey);

/* Gate */
void siu_ipc_osal_create_gate(void);
void siu_ipc_osal_open_gate(void);
void siu_ipc_osal_close_gate(void);
#endif
