/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctDV25DecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <xdc/std.h>
#include <ti/mas/types/types.h>
#include "idv25vdec.h"
#include "dv25vdec_ti.h"
#include "ividmc.h"

typedef struct dv25DecInst_s {
    IDV25VDEC_Params         extn_params;
    IDV25VDEC_DynamicParams  extn_dynamicParams;
    IDV25VDEC_InArgs         inArgs;
    IDV25VDEC_OutArgs        outArgs;
    IDV25VDEC_Status         status;
    IDV25VDEC_Fxns           IALG_Fxns;
    IVIDMC_t                 mcParams;
} dv25DecInst_t;

void dv25DecInitInstance(void *dv25DecInst);
void dv25DecSetStaticParams(void *dv25DecInst, void *codecParams);
void dv25DecSetDynamicParams(void *dv25DecInst, IVIDDEC2_Params *params);
void *dv25DecGetStaticParams(void *dv25DecInst);
void *dv25DecGetDynamicParams(void *dv25DecInst);
void *dv25DecGetInArgs(void *dv25DecInst);
void *dv25DecGetOutArgs(void *dv25DecInst);
void *dv25DecGetStatus(void *dv25DecInst);
void *dv25DecGetFxns(void *dv25DecInst);
void dv25DecSetMcParams(void *dv25DecInst, IVIDMC_t *params);
void *dv25DecGetIresFxns(void);

#define dv25DecAPI {             \
    (const char *)"DV25DEC",     \
    sizeof(dv25DecInst_t),       \
    dv25DecInitInstance,         \
    dv25DecSetStaticParams,      \
    dv25DecSetDynamicParams,     \
    dv25DecGetStaticParams,      \
    dv25DecGetDynamicParams,     \
    dv25DecGetInArgs,            \
    dv25DecGetOutArgs,           \
    dv25DecGetStatus,            \
    dv25DecGetFxns,              \
    dv25DecSetMcParams,          \
    dv25DecGetIresFxns           \
}



/* nothing past this point */
