/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctH264HpDecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_DV100DEC_CLIENT_H
#define _VCT_DV100DEC_CLIENT_H

#include <xdc/std.h>
#include <ti/mas/types/types.h>
#include "dv100vdec_ti.h"
#include "idv100vdec.h"

typedef struct dv100DecInst_s {
    IDV100VDEC_Params         extn_params;
    IDV100VDEC_DynamicParams  extn_dynamicParams;
    IDV100VDEC_InArgs         inArgs;
    IDV100VDEC_OutArgs        outArgs;
    IDV100VDEC_Status         status;
    IDV100VDEC_Fxns           IALG_Fxns;
    IVIDMC_t                 mcParams;
} dv100DecInst_t;

void dv100DecInitInstance(void *dv100DecInst);
void dv100DecSetStaticParams(void *dv100DecInst, void *codecParams);
void dv100DecSetDynamicParams(void *dv100DecInst, IVIDDEC2_Params *params);
void *dv100DecGetStaticParams(void *dv100DecInst);
void *dv100DecGetDynamicParams(void *dv100DecInst);
void *dv100DecGetInArgs(void *dv100DecInst);
void *dv100DecGetOutArgs(void *dv100DecInst);
void *dv100DecGetStatus(void *dv100DecInst);
void *dv100DecGetFxns(void *dv100DecInst);
void dv100DecSetMcParams(void *dv100DecInst, IVIDMC_t *params);
void *dv100DecGetIresFxns(void);

#define dv100DecAPI {             \
    (const char *)"DV100DEC",     \
    sizeof(dv100DecInst_t),       \
    dv100DecInitInstance,         \
    dv100DecSetStaticParams,      \
    dv100DecSetDynamicParams,     \
    dv100DecGetStaticParams,      \
    dv100DecGetDynamicParams,     \
    dv100DecGetInArgs,            \
    dv100DecGetOutArgs,           \
    dv100DecGetStatus,            \
    dv100DecGetFxns,              \
    dv100DecSetMcParams,          \
    dv100DecGetIresFxns           \
}

#endif

/* nothing past this point */
