/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctDv50DecClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctDv50DecClient.h"
#include <string.h>

void *dv50DecGetStaticParams(void *dv50DecInst) {
   dv50DecInst_t *inst = (dv50DecInst_t *)dv50DecInst;
   return((void *)&inst->extn_params);
}

void *dv50DecGetDynamicParams(void *dv50DecInst){
   dv50DecInst_t *inst = (dv50DecInst_t *)dv50DecInst;
   return((void *)&inst->extn_dynamicParams);
}

void *dv50DecGetInArgs(void *dv50DecInst) {
   dv50DecInst_t *inst = (dv50DecInst_t *)dv50DecInst;
   return((void *)&inst->inArgs);
}

void *dv50DecGetOutArgs(void *dv50DecInst) {
   dv50DecInst_t *inst = (dv50DecInst_t *)dv50DecInst;
   return((void *)&inst->outArgs);
}

void *dv50DecGetStatus(void *dv50DecInst) {
   dv50DecInst_t *inst = (dv50DecInst_t *)dv50DecInst;
   return((void *)&inst->status);
}

void *dv50DecGetFxns(void *dv50DecInst) {
   dv50DecInst_t *inst = (dv50DecInst_t *)dv50DecInst;
   return((void *)&inst->IALG_Fxns);
}

void dv50DecInitInstance(void *dv50DecInst)
{
    dv50DecInst_t *inst = (dv50DecInst_t *)dv50DecInst;
    memset(inst, 0, sizeof(dv50DecInst_t));

    inst->IALG_Fxns                    = DV50VDEC_TI_IDV50VDEC;
    inst->status.viddecStatus.size     = sizeof(IDV50VDEC_Status);
    inst->inArgs.viddecInArgs.size     = sizeof(IDV50VDEC_InArgs);
    inst->outArgs.viddecOutArgs.size   = sizeof(IDV50VDEC_OutArgs);
    return;
}

void dv50DecSetStaticParams(void *dv50DecInst, void *codecParams)
{
    dv50DecInst_t *inst          = (dv50DecInst_t *)dv50DecInst;
    IDV50VDEC_Params *ext_params   = &(inst->extn_params);
    IVIDDEC2_Params *params        = &ext_params->viddecParams;

    params->size                   = sizeof(IDV50VDEC_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(IDV50VDEC_Params));
    } else {
    params->maxHeight              = 576;
    params->maxWidth               = 720;
    params->maxFrameRate           = 0;
    params->maxBitRate             = 0;
    params->dataEndianness         = XDM_BYTE;
    params->forceChromaFormat      = 2;
	ext_params->MetadataEnable     = 1;
    }

    return;
}

void dv50DecSetDynamicParams(void *dv50DecInst, IVIDDEC2_Params *params)
{
    dv50DecInst_t *inst                 = (dv50DecInst_t *)dv50DecInst;
    IDV50VDEC_DynamicParams *extn_dynpar  = &inst->extn_dynamicParams;
    IVIDDEC2_DynamicParams *dynamicParams = &extn_dynpar->viddecDynamicParams;

    dynamicParams->size             = sizeof(IDV50VDEC_DynamicParams);
    dynamicParams->decodeHeader     = XDM_DECODE_AU;
    dynamicParams->displayWidth     = 0;
    dynamicParams->frameSkipMode    = IVIDEO_NO_SKIP;
    dynamicParams->frameOrder       = 0;
    dynamicParams->newFrameFlag     = 0;
    dynamicParams->mbDataFlag       = 0;
    extn_dynpar->chromaformat        = 2;

    return;
}

void dv50DecSetMcParams(void *dv50DecInst, IVIDMC_t *params)
{
    dv50DecInst_t *inst                 = (dv50DecInst_t *)dv50DecInst;
    memcpy(&(inst->mcParams), params, sizeof(IVIDMC_t));
    return;
}

void *dv50DecGetIresFxns(void) {
   return((void *)&DV50VDEC_TI_IRES);
}

/* nothing past this point */
