/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctDv50DecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_DV50DEC_CLIENT_H
#define _VCT_DV50DEC_CLIENT_H

#include <xdc/std.h>
#include <ti/mas/types/types.h>
#include "dv50vdec_ti.h"
#include "idv50vdec.h"
#include "ividmc.h"

typedef struct dv50DecInst_s {
    IDV50VDEC_Params         extn_params;
    IDV50VDEC_DynamicParams  extn_dynamicParams;
    IDV50VDEC_InArgs         inArgs;
    IDV50VDEC_OutArgs        outArgs;
    IDV50VDEC_Status         status;
    IDV50VDEC_Fxns           IALG_Fxns;
    IVIDMC_t                   mcParams;
} dv50DecInst_t;

void dv50DecInitInstance(void *dv50DecInst);
void dv50DecSetStaticParams(void *dv50DecInst, void *codecParams);
void dv50DecSetDynamicParams(void *dv50DecInst, IVIDDEC2_Params *params);
void *dv50DecGetStaticParams(void *dv50DecInst);
void *dv50DecGetDynamicParams(void *dv50DecInst);
void *dv50DecGetInArgs(void *dv50DecInst);
void *dv50DecGetOutArgs(void *dv50DecInst);
void *dv50DecGetStatus(void *dv50DecInst);
void *dv50DecGetFxns(void *dv50DecInst);
void dv50DecSetMcParams(void *dv50DecInst, IVIDMC_t *params);
void *dv50DecGetIresFxns(void);

#define dv50DecAPI {             \
    (const char *)"DV50DEC",     \
    sizeof(dv50DecInst_t),       \
    dv50DecInitInstance,         \
    dv50DecSetStaticParams,      \
    dv50DecSetDynamicParams,     \
    dv50DecGetStaticParams,      \
    dv50DecGetDynamicParams,     \
    dv50DecGetInArgs,            \
    dv50DecGetOutArgs,           \
    dv50DecGetStatus,            \
    dv50DecGetFxns,              \
    dv50DecSetMcParams,          \
    dv50DecGetIresFxns           \
}

#endif

/* nothing past this point */
