/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctH264hpDecClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctH264HpDecClient.h"
#include <string.h>

void *h264hpDecGetStaticParams(void *h264hpDecInst) {
   h264hpDecInst_t *inst = (h264hpDecInst_t *)h264hpDecInst;
   return((void *)&inst->extn_params);
}

void *h264hpDecGetDynamicParams(void *h264hpDecInst){
   h264hpDecInst_t *inst = (h264hpDecInst_t *)h264hpDecInst;
   return((void *)&inst->extn_dynamicParams);
}

void *h264hpDecGetInArgs(void *h264hpDecInst) {
   h264hpDecInst_t *inst = (h264hpDecInst_t *)h264hpDecInst;
   return((void *)&inst->inArgs);
}

void *h264hpDecGetOutArgs(void *h264hpDecInst) {
   h264hpDecInst_t *inst = (h264hpDecInst_t *)h264hpDecInst;
   return((void *)&inst->outArgs);
}

void *h264hpDecGetStatus(void *h264hpDecInst) {
   h264hpDecInst_t *inst = (h264hpDecInst_t *)h264hpDecInst;
   return((void *)&inst->status);
}

void *h264hpDecGetFxns(void *h264hpDecInst) {
   h264hpDecInst_t *inst = (h264hpDecInst_t *)h264hpDecInst;
   return((void *)&inst->IALG_Fxns);
}

void h264hpDecInitInstance(void *h264hpDecInst)
{
    h264hpDecInst_t *inst = (h264hpDecInst_t *)h264hpDecInst;
    memset(inst, 0, sizeof(h264hpDecInst_t));

    inst->IALG_Fxns                    = H264HPVDEC_TI_IH264HPVDEC;
    inst->status.viddecStatus.size     = sizeof(IH264HPVDEC_Status);
    inst->inArgs.viddecInArgs.size     = sizeof(IH264HPVDEC_InArgs);
    inst->outArgs.viddecOutArgs.size   = sizeof(IH264HPVDEC_OutArgs);
    return;
}

void h264hpDecSetStaticParams(void *h264hpDecInst, void *codecParams)
{
    h264hpDecInst_t *inst          = (h264hpDecInst_t *)h264hpDecInst;
    IH264HPVDEC_Params *ext_params   = &(inst->extn_params);
    IVIDDEC2_Params *params        = &ext_params->viddecParams;

    params->size                   = sizeof(IH264HPVDEC_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(IH264HPVDEC_Params));
    } else {
      params->maxHeight              = 1088;
      params->maxWidth               = 1920;
      params->maxFrameRate           = 30;
      params->maxBitRate             = 10000000;
      params->dataEndianness         = 1;
      params->forceChromaFormat      = 1;

      ext_params->displayDelay       = 1;
      ext_params->mcViddecParams     = &(inst->mcParams);
    }

    return;
}

void h264hpDecSetDynamicParams(void *h264hpDecInst, IVIDDEC2_Params *params)
{
    h264hpDecInst_t *inst                 = (h264hpDecInst_t *)h264hpDecInst;
    IH264HPVDEC_DynamicParams *extn_dynpar  = &inst->extn_dynamicParams;
    IVIDDEC2_DynamicParams *dynamicParams = &extn_dynpar->viddecDynamicParams;

    dynamicParams->size             = sizeof(IH264HPVDEC_DynamicParams);
    dynamicParams->decodeHeader     = 0;
    dynamicParams->displayWidth     = 0;
    dynamicParams->frameSkipMode    = 0;
    dynamicParams->frameOrder       = 0;
    dynamicParams->newFrameFlag     = 0;
    dynamicParams->mbDataFlag       = 0;

    return;
}

void h264hpDecSetMcParams(void *h264hpDecInst, IVIDMC_t *params)
{
    h264hpDecInst_t *inst                 = (h264hpDecInst_t *)h264hpDecInst;
    memcpy(&(inst->mcParams), params, sizeof(IVIDMC_t));
    return;
}

void *h264hpDecGetIresFxns(void) {
   return((void *)&H264HPVDEC_TI_IRES);
}

/* nothing past this point */
