/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctH264HpDecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_H264HPDEC_CLIENT_H
#define _VCT_H264HPDEC_CLIENT_H

#include <ti/mas/types/types.h>
#include <ti/sdo/codecs/h264hpvdec/h264hpvdec_ti.h>
#include <ti/sdo/codecs/h264hpvdec/ih264hpvdec.h>

typedef struct h264hpDecInst_s {
    IH264HPVDEC_Params         extn_params;
    IH264HPVDEC_DynamicParams  extn_dynamicParams;
    IH264HPVDEC_InArgs         inArgs;
    IH264HPVDEC_OutArgs        outArgs;
    IH264HPVDEC_Status         status;
    IH264HPVDEC_Fxns           IALG_Fxns;
    IVIDMC_t                 mcParams;
} h264hpDecInst_t;

void h264hpDecInitInstance(void *h264hpDecInst);
void h264hpDecSetStaticParams(void *h264hpDecInst, void *codecParams);
void h264hpDecSetDynamicParams(void *h264hpDecInst, IVIDDEC2_Params *params);
void *h264hpDecGetStaticParams(void *h264hpDecInst);
void *h264hpDecGetDynamicParams(void *h264hpDecInst);
void *h264hpDecGetInArgs(void *h264hpDecInst);
void *h264hpDecGetOutArgs(void *h264hpDecInst);
void *h264hpDecGetStatus(void *h264hpDecInst);
void *h264hpDecGetFxns(void *h264hpDecInst);
void h264hpDecSetMcParams(void *h264hpDecInst, IVIDMC_t *params);
void *h264hpDecGetIresFxns(void);

#define h264hpDecAPI {             \
    (const char *)"H264HPDEC",     \
    sizeof(h264hpDecInst_t),       \
    h264hpDecInitInstance,         \
    h264hpDecSetStaticParams,      \
    h264hpDecSetDynamicParams,     \
    h264hpDecGetStaticParams,      \
    h264hpDecGetDynamicParams,     \
    h264hpDecGetInArgs,            \
    h264hpDecGetOutArgs,           \
    h264hpDecGetStatus,            \
    h264hpDecGetFxns,              \
    h264hpDecSetMcParams,          \
    h264hpDecGetIresFxns           \
}

#endif

/* nothing past this point */
