/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctMpeg4DecClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctJpegDecClient.h"
#include <string.h>

void *jpegDecGetStaticParams(void *jpegDecInst) {
   jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
   return((void *)&inst->extn_params);
}

void *jpegDecGetDynamicParams(void *jpegDecInst){
   jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
   return((void *)&inst->extn_dynamicParams);
}

void *jpegDecGetInArgs(void *jpegDecInst) {
   jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
   return((void *)&inst->inArgs);
}

void *jpegDecGetOutArgs(void *jpegDecInst) {
   jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
   return((void *)&inst->outArgs);
}

void *jpegDecGetStatus(void *jpegDecInst) {
   jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
   return((void *)&inst->status);
}

void *jpegDecGetFxns(void *jpegDecInst) {
   jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
   return((void *)&inst->IALG_Fxns);
}

void jpegDecInitInstance(void *jpegDecInst)
{
    jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
    memset(inst, 0, sizeof(jpegDecInst_t));

    inst->IALG_Fxns                    = JPEGDEC_TI_IJPEGDEC;
    inst->status.imgdecStatus.size     = sizeof(IJPEGDEC_Status);
    inst->inArgs.imgdecInArgs.size     = sizeof(IJPEGDEC_InArgs);
    inst->outArgs.imgdecOutArgs.size   = sizeof(IJPEGDEC_OutArgs);
    return;
}

void jpegDecSetStaticParams(void *jpegDecInst)
{
    jpegDecInst_t *inst             = (jpegDecInst_t *)jpegDecInst;
    IJPEGDEC_Params *ext_params     = &(inst->extn_params);
    IIMGDEC1_Params *params         = &ext_params->imgdecParams;

    params->size                    = sizeof(IJPEGDEC_Params);
    params->maxHeight               = 1536;
    params->maxWidth                = 2048;
    params->maxScans                = 15;
    params->dataEndianness          = 1;
    params->forceChromaFormat       = 0;
    ext_params->progressiveDecFlag  = 0;
    ext_params->outImgRes           = 0;

    return;
}

void jpegDecSetDynamicParams(void *jpegDecInst, IIMGDEC1_Params *params)
{
    jpegDecInst_t *inst                  = (jpegDecInst_t *)jpegDecInst;
    IJPEGDEC_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    IIMGDEC1_DynamicParams *dynamicParams = &extn_dynpar->imgdecDynamicParams;

    dynamicParams->size             = sizeof(IJPEGDEC_DynamicParams);
    dynamicParams->numAU            = 0;
    dynamicParams->decodeHeader     = 0;
    dynamicParams->displayWidth     = 0;
    extn_dynpar->frame_numbytes     = 0;
    extn_dynpar->progDisplay        = 0;
    extn_dynpar->resizeOption       = 0;
    extn_dynpar->RGB_Format         = 0;
    extn_dynpar->numMCU_row         = 0;
    extn_dynpar->x_org              = 0;
    extn_dynpar->y_org              = 0;
    extn_dynpar->x_length           = 0;
    extn_dynpar->y_length           = 0;
    extn_dynpar->alpha_rgb          = 0;

    return;
}

void jpegDecSetDynamicParamsFrameNumBytes(void *jpegDecInst, XDAS_Int32 frame_numbytes)
{
    jpegDecInst_t *inst                  = (jpegDecInst_t *)jpegDecInst;
    IJPEGDEC_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    extn_dynpar->frame_numbytes     = frame_numbytes;
    return;
}

XDAS_Int32 jpegDecGetDynamicParamsProgDisplay(void *jpegDecInst)
{
    jpegDecInst_t *inst                  = (jpegDecInst_t *)jpegDecInst;
    IJPEGDEC_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    return (extn_dynpar->progDisplay);
}

XDAS_Int32 jpegDecGetStatusEndOfSeq(void *jpegDecInst)
{
   jpegDecInst_t *inst = (jpegDecInst_t *)jpegDecInst;
   return(inst->status.end_of_seq);
}

/* nothing past this point */
