/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to VCT
******************************************************************************
* FILE NAME:   vctJpegDecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to VCT
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_JPEGDEC_CLIENT_H
#define _VCT_JPEGDEC_CLIENT_H

#include <ti/mas/types/types.h>
#include "jpegdec_ti.h"
#include "ijpegdec.h"

typedef struct jpegDecInst_s {
    IJPEGDEC_Params          extn_params;
    IJPEGDEC_DynamicParams   extn_dynamicParams;
    IJPEGDEC_InArgs          inArgs;
    IJPEGDEC_OutArgs         outArgs;
    IJPEGDEC_Status          status;
    IJPEGDEC_Fxns            IALG_Fxns;
} jpegDecInst_t;

void jpegDecInitInstance(void *jpegDecInst);
void jpegDecSetStaticParams(void *jpegDecInst);
void jpegDecSetDynamicParams(void *jpegDecInst, IIMGDEC1_Params *params);
void *jpegDecGetStaticParams(void *jpegDecInst);
void *jpegDecGetDynamicParams(void *jpegDecInst);
void *jpegDecGetInArgs(void *jpegDecInst);
void *jpegDecGetOutArgs(void *jpegDecInst);
void *jpegDecGetStatus(void *jpegDecInst);
void *jpegDecGetFxns(void *jpegDecInst);
void jpegDecSetDynamicParamsFrameNumBytes(void *jpegDecInst, XDAS_Int32 frame_numbytes);
XDAS_Int32 jpegDecGetDynamicParamsProgDisplay(void *jpegDecInst);
XDAS_Int32 jpegDecGetStatusEndOfSeq(void *jpegDecInst);

#define jpegDecAPI {          \
    (const char *)"JPEGDEC",  \
    sizeof(jpegDecInst_t),    \
    jpegDecInitInstance,         \
    jpegDecSetStaticParams,      \
    jpegDecSetDynamicParams,     \
    jpegDecGetStaticParams,      \
    jpegDecGetDynamicParams,     \
    jpegDecGetInArgs,            \
    jpegDecGetOutArgs,           \
    jpegDecGetStatus,            \
    jpegDecGetFxns,              \
    jpegDecSetDynamicParamsFrameNumBytes, \
    jpegDecGetDynamicParamsProgDisplay, \
    jpegDecGetStatusEndOfSeq, \
    }

#endif

/* nothing past this point */
