/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctMp3DecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_MP3DEC_CLIENT_H
#define _VCT_MP3DEC_CLIENT_H

#include <ti/mas/types/types.h>
#include <ti/bios/include/std.h>
#include <ti/xdais/xdas.h>
#include <ti/xdais/ialg.h>

#include "imp3dec.h"
#include "mp3dec_tii.h"

typedef struct mp3DecInst_s {
    IMP3DEC_Params           extn_params;
    IMP3DEC_DynamicParams    extn_dynamicParams;
    IMP3DEC_InArgs           inArgs;
    IMP3DEC_OutArgs          outArgs;
    IMP3DEC_Status           status;
    IMP3DEC_Fxns             IALG_Fxns;
} mp3DecInst_t;

void mp3DecInitInstance(void *mp3DecInst);
void mp3DecSetStaticParams(void *mp3DecInst, void *codecParams);
void mp3DecSetDynamicParams(void *mp3DecInst, IAUDDEC1_Params *params);
void *mp3DecGetStaticParams(void *mp3DecInst);
void *mp3DecGetDynamicParams(void *mp3DecInst);
void *mp3DecGetInArgs(void *mp3DecInst);
void *mp3DecGetOutArgs(void *mp3DecInst);
void *mp3DecGetStatus(void *mp3DecInst);
void *mp3DecGetFxns(void *mp3DecInst);

#define mp3DecAPI {          \
    (const char *)"MP3DEC",  \
    sizeof(mp3DecInst_t),    \
    mp3DecInitInstance,         \
    mp3DecSetStaticParams,      \
    mp3DecSetDynamicParams,     \
    mp3DecGetStaticParams,      \
    mp3DecGetDynamicParams,     \
    mp3DecGetInArgs,            \
    mp3DecGetOutArgs,           \
    mp3DecGetStatus,            \
    mp3DecGetFxns               \
}

#endif

/* nothing past this point */
