/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctMpeg4DecClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctMpeg4DecClient.h"
#include <string.h>

void *mpeg4DecGetStaticParams(void *mpeg4DecInst) {
   mpeg4DecInst_t *inst = (mpeg4DecInst_t *)mpeg4DecInst;
   return((void *)&inst->extn_params);
}

void *mpeg4DecGetDynamicParams(void *mpeg4DecInst){
   mpeg4DecInst_t *inst = (mpeg4DecInst_t *)mpeg4DecInst;
   return((void *)&inst->extn_dynamicParams);
}

void *mpeg4DecGetInArgs(void *mpeg4DecInst) {
   mpeg4DecInst_t *inst = (mpeg4DecInst_t *)mpeg4DecInst;
   return((void *)&inst->inArgs);
}

void *mpeg4DecGetOutArgs(void *mpeg4DecInst) {
   mpeg4DecInst_t *inst = (mpeg4DecInst_t *)mpeg4DecInst;
   return((void *)&inst->outArgs);
}

void *mpeg4DecGetStatus(void *mpeg4DecInst) {
   mpeg4DecInst_t *inst = (mpeg4DecInst_t *)mpeg4DecInst;
   return((void *)&inst->status);
}

void *mpeg4DecGetFxns(void *mpeg4DecInst) {
   mpeg4DecInst_t *inst = (mpeg4DecInst_t *)mpeg4DecInst;
   return((void *)&inst->IALG_Fxns);
}

void mpeg4DecInitInstance(void *mpeg4DecInst)
{
    mpeg4DecInst_t *inst = (mpeg4DecInst_t *)mpeg4DecInst;
    memset(inst, 0, sizeof(mpeg4DecInst_t));

    inst->IALG_Fxns                    = M4H3DEC_TI_IM4H3DEC;
    inst->status.viddecStatus.size     = sizeof(IM4H3DEC_Status);
    inst->inArgs.viddecInArgs.size     = sizeof(IM4H3DEC_InArgs);
    inst->outArgs.viddecOutArgs.size   = sizeof(IM4H3DEC_OutArgs);
    return;
}

void mpeg4DecSetStaticParams(void *mpeg4DecInst, void *codecParams)
{
    mpeg4DecInst_t *inst           = (mpeg4DecInst_t *)mpeg4DecInst;
    IM4H3DEC_Params *ext_params    = &(inst->extn_params);
    IVIDDEC2_Params *params        = &ext_params->viddecParams;

    params->size                   = sizeof(IM4H3DEC_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(IM4H3DEC_Params));
    } else {
      params->maxHeight              = 144;
      params->maxWidth               = 176;
      params->maxFrameRate           = 30;
      params->maxBitRate             = 10000000;
      params->dataEndianness         = 1;
      params->forceChromaFormat      = 1;
      /* No Codec specific Static Param */
    }

    return;
}

void mpeg4DecSetDynamicParams(void *mpeg4DecInst, IVIDDEC2_Params *params)
{
    mpeg4DecInst_t *inst                  = (mpeg4DecInst_t *)mpeg4DecInst;
    IM4H3DEC_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    IVIDDEC2_DynamicParams *dynamicParams = &extn_dynpar->viddecDynamicParams;

    dynamicParams->size             = sizeof(IM4H3DEC_DynamicParams);
    dynamicParams->decodeHeader     = 0;
    dynamicParams->displayWidth     = 0;
    dynamicParams->frameSkipMode    = 0;
    dynamicParams->frameOrder       = 0;
    dynamicParams->newFrameFlag     = 0;
    dynamicParams->mbDataFlag       = 0;
    extn_dynpar->postDeblock        = 0;
    extn_dynpar->postDering         = 0;
    extn_dynpar->errorConceal       = 0;
    extn_dynpar->FrameLevelByteSwap = 1;

    return;
}

void mpeg4DecSetMcParams(void *mpeg4DecInst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine)
{
    mpeg4DecInst_t *inst  = (mpeg4DecInst_t *)mpeg4DecInst;
    memcpy(&(inst->mcParams), params, sizeof(IVIDMC_t));
    return;
}

void *mpeg4DecGetIresFxns(void) {
   //return((void *)&M4H3DEC_TI_IRES); /* When FC-based codec library */
    return((void *)NULL);
}
/* nothing past this point */
