/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctMpeg4DecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_MPEG4DEC_CLIENT_H
#define _VCT_MPEG4DEC_CLIENT_H

#include <ti/mas/types/types.h>
#include <ti/sdo/codecs/m4h3dec/m4h3dec_ti.h>
#include <ti/sdo/codecs/m4h3dec/im4h3dec.h>
#include "ividmc.h"

typedef struct mpeg4DecInst_s {
    IM4H3DEC_Params          extn_params;
    IM4H3DEC_DynamicParams   extn_dynamicParams;
    IM4H3DEC_InArgs          inArgs;
    IM4H3DEC_OutArgs         outArgs;
    IM4H3DEC_Status          status;
    IM4H3DEC_Fxns            IALG_Fxns;
    IVIDMC_t                 mcParams;
} mpeg4DecInst_t;

void mpeg4DecInitInstance(void *mpeg4DecInst);
void mpeg4DecSetStaticParams(void *mpeg4DecInst, void *codecParams);
void mpeg4DecSetDynamicParams(void *mpeg4DecInst, IVIDDEC2_Params *params);
void *mpeg4DecGetStaticParams(void *mpeg4DecInst);
void *mpeg4DecGetDynamicParams(void *mpeg4DecInst);
void *mpeg4DecGetInArgs(void *mpeg4DecInst);
void *mpeg4DecGetOutArgs(void *mpeg4DecInst);
void *mpeg4DecGetStatus(void *mpeg4DecInst);
void *mpeg4DecGetFxns(void *mpeg4DecInst);
void mpeg4DecSetMcParams(void *mpeg4DecInst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine);
void *mpeg4DecGetIresFxns(void);

#define mpeg4DecAPI {          \
    (const char *)"MPEG4DEC",  \
    sizeof(mpeg4DecInst_t),    \
    mpeg4DecInitInstance,         \
    mpeg4DecSetStaticParams,      \
    mpeg4DecSetDynamicParams,     \
    mpeg4DecGetStaticParams,      \
    mpeg4DecGetDynamicParams,     \
    mpeg4DecGetInArgs,            \
    mpeg4DecGetOutArgs,           \
    mpeg4DecGetStatus,            \
    mpeg4DecGetFxns,              \
    mpeg4DecSetMcParams,          \
    mpeg4DecGetIresFxns           \
}

#endif

/* nothing past this point */
