/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder
******************************************************************************
* FILE NAME:   vctSorsparkClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctSorsparkDecClient.h"
#include <string.h>

void *sorsparkGetStaticParams(void *sorsparkInst) {
   sorsparkInst_t *inst = (sorsparkInst_t *)sorsparkInst;
   return((void *)&inst->extn_params);
}

void *sorsparkGetDynamicParams(void *sorsparkInst){
   sorsparkInst_t *inst = (sorsparkInst_t *)sorsparkInst;
   return((void *)&inst->extn_dynamicParams);
}

void *sorsparkGetInArgs(void *sorsparkInst) {
   sorsparkInst_t *inst = (sorsparkInst_t *)sorsparkInst;
   return((void *)&inst->inArgs);
}

void *sorsparkGetOutArgs(void *sorsparkInst) {
   sorsparkInst_t *inst = (sorsparkInst_t *)sorsparkInst;
   return((void *)&inst->outArgs);
}

void *sorsparkGetStatus(void *sorsparkInst) {
   sorsparkInst_t *inst = (sorsparkInst_t *)sorsparkInst;
   return((void *)&inst->status);
}

void *sorsparkGetFxns(void *sorsparkInst) {
   sorsparkInst_t *inst = (sorsparkInst_t *)sorsparkInst;
   return((void *)&inst->IALG_Fxns);
}

void sorsparkInitInstance(void *sorsparkInst)
{
    sorsparkInst_t *inst = (sorsparkInst_t *)sorsparkInst;
    memset(inst, 0, sizeof(sorsparkInst_t));

    inst->IALG_Fxns                    = SSPARKDEC_TI_ISSPARKDEC;
    inst->status.viddecStatus.size     = sizeof(ISSPARKDEC_Status);
    inst->inArgs.viddecInArgs.size     = sizeof(ISSPARKDEC_InArgs);
    inst->outArgs.viddecOutArgs.size   = sizeof(ISSPARKDEC_OutArgs);
    return;
}

void sorsparkSetStaticParams(void *sorsparkInst, void *codecParams)
{
    sorsparkInst_t *inst           = (sorsparkInst_t *)sorsparkInst;
    ISSPARKDEC_Params *ext_params  = &(inst->extn_params);
    IVIDDEC_Params *params         = &ext_params->viddecParams;

    params->size                   = sizeof(ISSPARKDEC_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(ISSPARKDEC_Params));
    } else {
      params->size                   = sizeof(ISSPARKDEC_Params);
      params->maxHeight              = 480;
      params->maxWidth               = 864;
      params->maxFrameRate           = 30;
      params->maxBitRate             = 10000000;
      params->dataEndianness         = 1;
      params->forceChromaFormat      = 1;
      /* No Codec specific Static Param */
    }

    return;
}

void sorsparkSetDynamicParams(void *sorsparkInst, IVIDDEC_Params *params)
{
    sorsparkInst_t *inst                   = (sorsparkInst_t *)sorsparkInst;
    ISSPARKDEC_DynamicParams *extn_dynpar  = &inst->extn_dynamicParams;
    IVIDDEC_DynamicParams *dynamicParams   = &extn_dynpar->viddecDynamicParams;

    dynamicParams->size             = sizeof(ISSPARKDEC_DynamicParams);
    dynamicParams->decodeHeader     = 0;
    dynamicParams->displayWidth     = 0;
    dynamicParams->frameSkipMode    = 0;
    extn_dynpar->postDeblock        = 0;
    extn_dynpar->postDering         = 0;
    extn_dynpar->errorConceal       = 0;

    return;
}

void sorsparkSetMcParams(void *sorsparkInst, IVIDMC_t *params)
{
    sorsparkInst_t *inst  = (sorsparkInst_t *)sorsparkInst;
    memcpy(&(inst->mcParams), params, sizeof(IVIDMC_t));
    return;
}

void *sorsparkGetIresFxns(void) {
    return((void *)&SSPARKDEC_TI_IRES);
}
/* nothing past this point */
