/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctSorsparkClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_SORSPARK_CLIENT_H
#define _VCT_SORSPARK_CLIENT_H

#include <ti/mas/types/types.h>
#include "ssparkdec_ti.h"
#include "issparkdec.h"
#include "ividmc.h"

typedef struct sorsparkInst_s {
    ISSPARKDEC_Params          extn_params;
    ISSPARKDEC_DynamicParams   extn_dynamicParams;
    ISSPARKDEC_InArgs          inArgs;
    ISSPARKDEC_OutArgs         outArgs;
    ISSPARKDEC_Status          status;
    ISSPARKDEC_Fxns            IALG_Fxns;
    IVIDMC_t                   mcParams;
} sorsparkInst_t;

void sorsparkInitInstance(void *sorsparkInst);
void sorsparkSetStaticParams(void *sorsparkInst, void *codecParams);
void sorsparkSetDynamicParams(void *sorsparkInst, IVIDDEC_Params *params);
void *sorsparkGetStaticParams(void *sorsparkInst);
void *sorsparkGetDynamicParams(void *sorsparkInst);
void *sorsparkGetInArgs(void *sorsparkInst);
void *sorsparkGetOutArgs(void *sorsparkInst);
void *sorsparkGetStatus(void *sorsparkInst);
void *sorsparkGetFxns(void *sorsparkInst);
void sorsparkSetMcParams(void *sorsparkInst, IVIDMC_t *params);
void *sorsparkGetIresFxns(void);

#define sorsparkDecAPI {          \
    (const char *)"SORSPARKDEC",  \
    sizeof(sorsparkInst_t),    \
    sorsparkInitInstance,         \
    sorsparkSetStaticParams,      \
    sorsparkSetDynamicParams,     \
    sorsparkGetStaticParams,      \
    sorsparkGetDynamicParams,     \
    sorsparkGetInArgs,            \
    sorsparkGetOutArgs,           \
    sorsparkGetStatus,            \
    sorsparkGetFxns,              \
    sorsparkSetMcParams,          \
    sorsparkGetIresFxns           \
}

#endif

/* nothing past this point */
