/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctVc1DecClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctVc1DecClient.h"
#include <string.h>

void *vc1DecGetStaticParams(void *vc1DecInst) {
   vc1DecInst_t *inst = (vc1DecInst_t *)vc1DecInst;
   return((void *)&inst->extn_params);
}

void *vc1DecGetDynamicParams(void *vc1DecInst){
   vc1DecInst_t *inst = (vc1DecInst_t *)vc1DecInst;
   return((void *)&inst->extn_dynamicParams);
}

void *vc1DecGetInArgs(void *vc1DecInst) {
   vc1DecInst_t *inst = (vc1DecInst_t *)vc1DecInst;
   return((void *)&inst->inArgs);
}

void *vc1DecGetOutArgs(void *vc1DecInst) {
   vc1DecInst_t *inst = (vc1DecInst_t *)vc1DecInst;
   return((void *)&inst->outArgs);
}

void *vc1DecGetStatus(void *vc1DecInst) {
   vc1DecInst_t *inst = (vc1DecInst_t *)vc1DecInst;
   return((void *)&inst->status);
}

void *vc1DecGetFxns(void *vc1DecInst) {
   vc1DecInst_t *inst = (vc1DecInst_t *)vc1DecInst;
   return((void *)&inst->IALG_Fxns);
}

void vc1DecInitInstance(void *vc1DecInst)
{
    vc1DecInst_t *inst = (vc1DecInst_t *)vc1DecInst;
    memset(inst, 0, sizeof(vc1DecInst_t));

    inst->IALG_Fxns                    = VC1VDEC_TI_IVC1VDEC;
    inst->status.viddecStatus.size     = sizeof(IVC1DEC_Status);
    inst->inArgs.viddecInArgs.size     = sizeof(IVC1DEC_InArgs);
    inst->outArgs.viddecOutArgs.size   = sizeof(IVC1DEC_OutArgs);
    return;
}

void vc1DecSetStaticParams(void *vc1DecInst, void *codecParams)
{
    vc1DecInst_t *inst           = (vc1DecInst_t *)vc1DecInst;
    IVC1DEC_Params *ext_params    = &(inst->extn_params);
	IVIDDEC2_Params *params        = &ext_params->viddecParams;

    params->size                   = sizeof(IVC1DEC_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(IVC1DEC_Params));
    } else {
      params->maxHeight              = 1088;
      params->maxWidth               = 1920;
      params->maxFrameRate           = 30;
      params->maxBitRate             = 10000000;
      params->dataEndianness         = 1;
      params->forceChromaFormat      = 1;
	  ext_params->display_order      = 1;
	  //ext_params->ividmc     = inst->mcParams;
    }

    return;
}

void vc1DecSetDynamicParams(void *vc1DecInst, IVIDDEC2_Params *params)
{
    vc1DecInst_t *inst                  = (vc1DecInst_t *)vc1DecInst;
    IVC1DEC_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    IVIDDEC2_DynamicParams *dynamicParams = &extn_dynpar->viddecDynamicParams;

    dynamicParams->size             = sizeof(IVC1DEC_DynamicParams);
    dynamicParams->decodeHeader     = 0;
    dynamicParams->displayWidth     = 0;
    dynamicParams->frameSkipMode    = 0;
    dynamicParams->frameOrder       = 0;
    dynamicParams->newFrameFlag     = 0;
    dynamicParams->mbDataFlag       = 0;
	//extn_dynpar->bIsElementaryStream = 2;

    return;
}

void vc1DecSetMcParams(void *vc1DecInst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine)
{
    vc1DecInst_t *inst  = (vc1DecInst_t *)vc1DecInst;
	//memcpy(&(inst->mcParams), params, sizeof(IVIDMC_t));
	memcpy(&(inst->extn_params.ividmc), params, sizeof(IVIDMC_t));
    return;
}

void *vc1DecGetIresFxns(void) {
   return((void *)&VC1DEC_TI_IRES); /* When FC-based codec library */
}
/* nothing past this point */
