/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
******************************************************************************
* FILE NAME:   vctVc1DecClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Decoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_VC1DEC_CLIENT_H
#define _VCT_VC1DEC_CLIENT_H

#include <ti/mas/types/types.h>
#include "ivc1vdec.h"
#include "ivc1vdec_tii.h"
#include "ividmc.h"

typedef struct vc1DecInst_s {
    IVC1DEC_Params          extn_params;
    IVC1DEC_DynamicParams   extn_dynamicParams;
    IVC1DEC_InArgs          inArgs;
    IVC1DEC_OutArgs         outArgs;
    IVC1DEC_Status          status;
    IVC1DEC_Fxns            IALG_Fxns;
    IVIDMC_t                 mcParams;
} vc1DecInst_t;

void vc1DecInitInstance(void *vc1DecInst);
void vc1DecSetStaticParams(void *vc1DecInst, void *codecParams);
void vc1DecSetDynamicParams(void *vc1DecInst, IVIDDEC2_Params *params);
void *vc1DecGetStaticParams(void *vc1DecInst);
void *vc1DecGetDynamicParams(void *vc1DecInst);
void *vc1DecGetInArgs(void *vc1DecInst);
void *vc1DecGetOutArgs(void *vc1DecInst);
void *vc1DecGetStatus(void *vc1DecInst);
void *vc1DecGetFxns(void *vc1DecInst);
void vc1DecSetMcParams(void *vc1DecInst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine);
void *vc1DecGetIresFxns(void);

#define vc1DecAPI {          \
    (const char *)"VC1DEC",  \
    sizeof(vc1DecInst_t),    \
    vc1DecInitInstance,         \
    vc1DecSetStaticParams,      \
    vc1DecSetDynamicParams,     \
    vc1DecGetStaticParams,      \
    vc1DecGetDynamicParams,     \
    vc1DecGetInArgs,            \
    vc1DecGetOutArgs,           \
    vc1DecGetStatus,            \
    vc1DecGetFxns,              \
    vc1DecSetMcParams,          \
    vc1DecGetIresFxns           \
}

#endif

/* nothing past this point */
