/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM0.9 Compliant Encoder to TVCT
******************************************************************************
* FILE NAME:   vctH264EncClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_H264ENC_CLIENT_H
#define _VCT_H264ENC_CLIENT_H

#include <ti/mas/types/types.h>
#include <ti/sdo/codecs/h264venc/ih264venc.h>
#include <ti/sdo/codecs/h264venc/h264venc_ti.h>
#include "ividmc.h" 

typedef struct h264EncInst_s {
    IH264VENC_Params          extn_params;
    IH264VENC_DynamicParams   extn_dynamicParams;
    IH264VENC_InArgs          inArgs;
    IH264VENC_OutArgs         outArgs;
    IH264VENC_Status          status;
    IH264VENC_Fxns            IALG_Fxns;
    IVIDMC_t                  mcParams;
    XDAS_UInt32               topSliceLine;
    XDAS_UInt32               bottomSliceLine;
} h264EncInst_t;

void h264InitInstance(void *h264EncInst);
void h264SetStaticParams(void *h264EncInst, void *codecParams);
void h264SetDynamicParams(void *h264EncInst, void *dynParams);
void *h264GetStaticParams(void *h264EncInst);
void *h264GetDynamicParams(void *h264EncInst);
void *h264GetInArgs(void *h264EncInst);
void *h264GetOutArgs(void *h264EncInst);
void *h264GetStatus(void *h264EncInst);
void *h264GetFxns(void *h264EncInst);
void h264SetMcParams(void *h264EncInst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine);
void *h264GetIresFxns(void);

#define h264EncAPI {          \
    (const char *)"H264ENC",  \
    sizeof(h264EncInst_t),    \
    h264InitInstance,         \
    h264SetStaticParams,      \
    h264SetDynamicParams,     \
    h264GetStaticParams,      \
    h264GetDynamicParams,     \
    h264GetInArgs,            \
    h264GetOutArgs,           \
    h264GetStatus,            \
    h264GetFxns,              \
    h264SetMcParams,          \
    h264GetIresFxns           \
}

#endif

/* nothing past this point */
