/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Encoder to CTFW
******************************************************************************
* FILE NAME:   vctJpegEncClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder to CTFW
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctJpegEncClient.h"
#include <string.h>

int jpegEnc_Initialize_original_buffer(IDMJPGE_TIGEM_DynamicParams *params);

void *jpegGetStaticParams(void *jpegEncInst) {
   jpegEncInst_t *inst = (jpegEncInst_t *)jpegEncInst;
   return((void *)&inst->extn_params);
}

void *jpegGetDynamicParams(void *jpegEncInst){
   jpegEncInst_t *inst = (jpegEncInst_t *)jpegEncInst;
   return((void *)&inst->extn_dynamicParams);
}

void *jpegGetInArgs(void *jpegEncInst) {
   jpegEncInst_t *inst = (jpegEncInst_t *)jpegEncInst;
   return((void *)&inst->inArgs);
}

void *jpegGetOutArgs(void *jpegEncInst) {
   jpegEncInst_t *inst = (jpegEncInst_t *)jpegEncInst;
   return((void *)&inst->outArgs);
}

void *jpegGetStatus(void *jpegEncInst) {
   jpegEncInst_t *inst = (jpegEncInst_t *)jpegEncInst;
   return((void *)&inst->status);
}

void *jpegGetFxns(void *jpegEncInst) {
   jpegEncInst_t *inst = (jpegEncInst_t *)jpegEncInst;
   return((void *)&inst->IALG_Fxns);
}

void jpegInitInstance(void *jpegEncInst)
{
    jpegEncInst_t *inst = (jpegEncInst_t *)jpegEncInst;
    memset(inst, 0, sizeof(jpegEncInst_t));

    inst->IALG_Fxns                  = DMJPGE_TIGEM_IDMJPGE;
    inst->status.status.size         = sizeof(IDMJPGE_TIGEM_Status);
    inst->inArgs.inArgs.size         = sizeof(IDMJPGE_TIGEM_InArgs);
    inst->outArgs.outArgs.size       = sizeof(IDMJPGE_TIGEM_OutArgs);
    return;
}

void jpegSetStaticParams(void *jpegEncInst, void *codecParams)
{
    jpegEncInst_t *inst               = (jpegEncInst_t *)jpegEncInst;
    IDMJPGE_TIGEM_Params *ext_params  = &(inst->extn_params);
    IIMGENC1_Params *params           = &ext_params->params;

    params->size                   = sizeof(IIMGENC1_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(IDMJPGE_TIGEM_Params));
    } else {
      params->maxHeight              = 576;
      params->maxWidth               = 720;
      params->maxScans               = 0;
      params->dataEndianness         = XDM_BYTE;
      params->forceChromaFormat      = 2;

      ext_params->MaxThumbnail_H_size_App0     = 0;
      ext_params->MaxThumbnail_V_size_App0     = 0;
      ext_params->MaxThumbnail_H_size_App1     = 0;
      ext_params->MaxThumbnail_V_size_App1     = 0;
      ext_params->MaxThumbnail_H_size_App13    = 0;
      ext_params->MaxThumbnail_V_size_App13    = 0;
    }

    return;
}


void jpegSetDynamicParams(void *jpegEncInst, void *params)
{
    jpegEncInst_t *inst                        = (jpegEncInst_t *)jpegEncInst;
    IDMJPGE_TIGEM_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    IIMGENC1_DynamicParams *dynamicParams      = &extn_dynpar->params;
    IIMGENC1_Params  *staticParams             = (IIMGENC1_Params *)params;

    dynamicParams->size            = sizeof(IIMGENC1_DynamicParams);
    dynamicParams->numAU               = 0;       
    dynamicParams->inputChromaFormat   = XDM_YUV_422P; //XDM_YUV_422ILE;        
    dynamicParams->inputHeight         = staticParams->maxHeight;      
    dynamicParams->inputWidth          = staticParams->maxWidth;   
    dynamicParams->captureWidth        = staticParams->maxWidth; 
    dynamicParams->generateHeader      = XDM_ENCODE_AU;
    dynamicParams->qValue              = 5;    

    extn_dynpar->captureHeight          = staticParams->maxHeight;
    extn_dynpar->DRI_Interval           = 9;
    extn_dynpar->quantTable             = NULL;
    extn_dynpar->Thumbnail_Index_App0   = 0;
    extn_dynpar->Thumbnail_Index_App1   = 0;
    extn_dynpar->Thumbnail_Index_App13  = 0;
    extn_dynpar->APPN0_numBufs          = 0;
    extn_dynpar->APPN1_numBufs          = 0;
    extn_dynpar->APPN13_numBufs         = 0;
    extn_dynpar->APPN0_startBuf         = 0;
    extn_dynpar->APPN1_startBuf         = 0;
    extn_dynpar->APPN13_startBuf        = 0;
    extn_dynpar->COMMENT_startBuf       = 0;
    extn_dynpar->COMMENT_insert         = 1;
    extn_dynpar->Thumbnail_H_size_App1  = 0;
    extn_dynpar->Thumbnail_V_size_App1  = 0;
    extn_dynpar->Thumbnail_H_size_App0  = 0;
    extn_dynpar->Thumbnail_V_size_App0  = 0;
    extn_dynpar->Thumbnail_H_size_App13 = 0;
    extn_dynpar->Thumbnail_V_size_App13 = 0;

    jpegEnc_Initialize_original_buffer(extn_dynpar);

    return;
}

/******************************************************************************
* FUNCTION PURPOSE: Initialize original buffer
******************************************************************************
* DESCRIPTION: Initialize original buffer
*
* )
*****************************************************************************/
int jpegEnc_Initialize_original_buffer(IDMJPGE_TIGEM_DynamicParams *params)
{
    int num_buf, temp, Capture_Height;
    int InputBuffSize[4];

    if(params->params.size == sizeof(IIMGENC1_DynamicParams))
        Capture_Height = 0;
    else
        Capture_Height = params->captureHeight;

    if (params->params.captureWidth == 0 && Capture_Height==0)
        temp = params->params.inputHeight * params->params.inputWidth;

    else if (params->params.captureWidth != 0 && Capture_Height==0)
        temp = params->params.inputHeight * params->params.captureWidth;

    else if (params->params.captureWidth == 0 && Capture_Height!=0)
        temp = Capture_Height * params->params.inputWidth;

    else
        temp = Capture_Height * params->params.captureWidth;  //REM

    switch(params->params.inputChromaFormat)
    {
    case XDM_YUV_422ILE:
        {
            num_buf  = 1;
            InputBuffSize[0] = temp<<1 ;
        }; break;

    case XDM_GRAY:
        {
            num_buf  = 1;
            InputBuffSize[0] = temp ;
        }; break;

    case XDM_YUV_444P:
        {
            num_buf  = 3;
            InputBuffSize[0] = temp ;
            InputBuffSize[1] = temp ;
            InputBuffSize[2] = temp ;
        }; break;
    case XDM_YUV_420P:
        {
            num_buf  = 3;
            InputBuffSize[0] = temp ;
            InputBuffSize[1] = temp >> 2 ;
            InputBuffSize[2] = temp >> 2;
        }; break;

    case XDM_YUV_422P:
        {
            num_buf = 3;
            InputBuffSize[0] = temp ;
            InputBuffSize[1] = temp >> 1;
            InputBuffSize[2] = temp >> 1;
        }; break;

    case XDM_YUV_411P:
        {
            num_buf = 3;
            InputBuffSize[0] = temp ;
            InputBuffSize[1] = temp >> 2;
            InputBuffSize[2] = temp >> 2;
        }; break;

    case XDM_DEFAULT:
        {
            num_buf = 3;
            InputBuffSize[0] = temp ;
            InputBuffSize[1] = temp ;
            InputBuffSize[2] = temp ;
        }; break;
    }

    return NULL;
}


/* nothing past this point */
