/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Encoder to CTFW
******************************************************************************
* FILE NAME:   vctJpegEncClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder to CTFW
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_JPEGENC_CLIENT_H
#define _VCT_JPEGENC_CLIENT_H

#include <ti/mas/types/types.h>
#include "idmjpge.h"

typedef struct jpegEncInst_s {
    IDMJPGE_TIGEM_Params          extn_params;
    IDMJPGE_TIGEM_DynamicParams   extn_dynamicParams;
    IDMJPGE_TIGEM_InArgs          inArgs;
    IDMJPGE_TIGEM_OutArgs         outArgs;
    IDMJPGE_TIGEM_Status          status;
    IDMJPGE_TIGEM_Fxns            IALG_Fxns;
} jpegEncInst_t;

void jpegInitInstance(void *jpegEncInst);
void jpegSetStaticParams(void *jpegEncInst, void *codecParams);
void jpegSetDynamicParams(void *jpegEncInst, void *params);
void *jpegGetStaticParams(void *jpegEncInst);
void *jpegGetDynamicParams(void *jpegEncInst);
void *jpegGetInArgs(void *jpegEncInst);
void *jpegGetOutArgs(void *jpegEncInst);
void *jpegGetStatus(void *jpegEncInst);
void *jpegGetFxns(void *jpegEncInst);

#define jpegEncAPI {          \
    (const char *)"JPEGENC",  \
    sizeof(jpegEncInst_t),    \
    jpegInitInstance,         \
    jpegSetStaticParams,      \
    jpegSetDynamicParams,     \
    jpegGetStaticParams,      \
    jpegGetDynamicParams,     \
    jpegGetInArgs,            \
    jpegGetOutArgs,           \
    jpegGetStatus,            \
    jpegGetFxns               \
}

#endif

/* nothing past this point */
