/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Encoder to TVCT
******************************************************************************
* FILE NAME:   vctMpeg4EncClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vctMpeg4EncClient.h"
#include <string.h>

void *mpeg4GetStaticParams(void *mpeg4EncInst) {
   mpeg4EncInst_t *inst = (mpeg4EncInst_t *)mpeg4EncInst;
   return((void *)&inst->extn_params);
}

void *mpeg4GetDynamicParams(void *mpeg4EncInst){
   mpeg4EncInst_t *inst = (mpeg4EncInst_t *)mpeg4EncInst;
   return((void *)&inst->extn_dynamicParams);
}

void *mpeg4GetInArgs(void *mpeg4EncInst) {
   mpeg4EncInst_t *inst = (mpeg4EncInst_t *)mpeg4EncInst;
   return((void *)&inst->inArgs);
}

void *mpeg4GetOutArgs(void *mpeg4EncInst) {
   mpeg4EncInst_t *inst = (mpeg4EncInst_t *)mpeg4EncInst;
   return((void *)&inst->outArgs);
}

void *mpeg4GetStatus(void *mpeg4EncInst) {
   mpeg4EncInst_t *inst = (mpeg4EncInst_t *)mpeg4EncInst;
   return((void *)&inst->status);
}

void *mpeg4GetFxns(void *mpeg4EncInst) {
   mpeg4EncInst_t *inst = (mpeg4EncInst_t *)mpeg4EncInst;
   return((void *)&inst->IALG_Fxns);
}

void mpeg4InitInstance(void *mpeg4EncInst)
{
    mpeg4EncInst_t *inst = (mpeg4EncInst_t *)mpeg4EncInst;
    memset(inst, 0, sizeof(mpeg4EncInst_t));

    inst->IALG_Fxns                    = MP4VENC_TI_IMP4VENC;
    inst->status.videncStatus.size     = sizeof(IMP4VENC_Status);
    inst->inArgs.videncInArgs.size     = sizeof(IMP4VENC_InArgs);
    inst->outArgs.videncOutArgs.size   = sizeof(IMP4VENC_OutArgs);
    return;
}

void mpeg4SetStaticParams(void *mpeg4EncInst, void *codecParams)
{
    mpeg4EncInst_t *inst           = (mpeg4EncInst_t *)mpeg4EncInst;
    IMP4VENC_Params *ext_params    = &(inst->extn_params);
    IVIDENC1_Params *params        = &ext_params->videncParams;

    params->size                   = sizeof(IMP4VENC_Params);

    if(codecParams != NULL) {
      memcpy(ext_params,codecParams,sizeof(IMP4VENC_Params));
    } else {
      params->dataEndianness         = XDM_BYTE;
      params->maxInterFrameInterval  = 1;
      params->inputContentType       = IVIDEO_CONTENTTYPE_DEFAULT ;
      params->reconChromaFormat      = XDM_CHROMA_NA;
      params->encodingPreset         = 2;
      params->rateControlPreset      = 1;
      params->maxHeight              = 288;
      params->maxWidth               = 352;
      params->maxFrameRate           = 30000;
      params->maxBitRate             = 512000;
      params->dataEndianness         = 1;
      params->maxInterFrameInterval  = 0;
      params->inputChromaFormat      = 1;
      params->inputContentType       = 0;
      ext_params->encodeMode         = 1;
      ext_params->levelIdc           = 5;
      ext_params->numFrames          = 240;
      ext_params->rcAlgo             = 9;
      ext_params->vbvBufferSize      = 112;
      ext_params->useVOS             = 1;
      ext_params->useGOV             = 0;
      ext_params->useDataPartition   = 0;
      ext_params->useRVLC            = 0;
      ext_params->maxDelay           = 1000;
      ext_params->enableSCD          = 0;
    }

    return;
}


void mpeg4SetDynamicParams(void *mpeg4EncInst, void *dynParams)
{
    mpeg4EncInst_t *inst                  = (mpeg4EncInst_t *)mpeg4EncInst;
    IMP4VENC_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    IVIDENC1_DynamicParams *dynamicParams = &extn_dynpar->videncDynamicParams;
    IVIDENC1_Params *params = (IVIDENC1_Params *)dynParams;

    dynamicParams->size            = sizeof(IMP4VENC_DynamicParams);
    dynamicParams->inputHeight     = params->maxHeight    ;       
    dynamicParams->inputWidth      = params->maxWidth     ;        
    dynamicParams->refFrameRate    = params->maxFrameRate ;      
    dynamicParams->targetFrameRate = params->maxFrameRate ;   
    dynamicParams->targetBitRate   = params->maxBitRate   ; 
    dynamicParams->intraFrameInterval = 30;
    dynamicParams->generateHeader  = 0;    
    dynamicParams->captureWidth    = 0;      
    dynamicParams->forceFrame      = IVIDEO_NA_FRAME;       
    dynamicParams->mbDataFlag      = 0;       
    dynamicParams->interFrameInterval = 1;
    extn_dynpar->resyncInterval  = 2000;
    extn_dynpar->hecInterval     = 0;
    extn_dynpar->airRate         = 0;
    extn_dynpar->mirRate         = 0;
    extn_dynpar->qpIntra         = 8;
    extn_dynpar->qpInter         = 8;
    extn_dynpar->fCode           = 5;
    extn_dynpar->useHpi          = 1;
    extn_dynpar->useAcPred       = 0;
    extn_dynpar->lastFrame       = 0;
    extn_dynpar->MVDataEnable    = 0;
    extn_dynpar->useUMV          = 1;
    extn_dynpar->qpMax           = 31;
    extn_dynpar->qpMin           = 2;
    extn_dynpar->MBIndexEnable   = 0;
    extn_dynpar->GOBIndexEnable  = 0;
    extn_dynpar->GOBHeaderEnable = 0;
    return;
}

void mpeg4SetMcParams(void *mpeg4EncInst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine)
{
    mpeg4EncInst_t *inst    = (mpeg4EncInst_t *)mpeg4EncInst;
    memcpy(&(inst->mcParams), params, sizeof(IVIDMC_t));
    return;
}

void *mpeg4GetIresFxns(void) {
   //return((void *)&M4H3ENC_TI_IRES); /* When integrating with FC-based codec library*/
    return((void *)NULL);
}
/* nothing past this point */
