/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM1.0 Compliant Encoder to TVCT
******************************************************************************
* FILE NAME:   vctMpeg4EncClient.h
*
* DESCRIPTION: Client Wrapper to integrate XDM1.0 Compliant Encoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _VCT_MPEG4ENC_CLIENT_H
#define _VCT_MPEG4ENC_CLIENT_H

#include <ti/mas/types/types.h>
#include "imp4venc.h"  
#include "mp4venc_ti.h"
#include "ividmc.h" 

typedef struct mpeg4EncInst_s {
    IMP4VENC_Params          extn_params;
    IMP4VENC_DynamicParams   extn_dynamicParams;
    IMP4VENC_InArgs          inArgs;
    IMP4VENC_OutArgs         outArgs;
    IMP4VENC_Status          status;
    IMP4VENC_Fxns            IALG_Fxns;
    IVIDMC_t                 mcParams;
} mpeg4EncInst_t;

void mpeg4InitInstance(void *mpeg4EncInst);
void mpeg4SetStaticParams(void *mpeg4EncInst, void *codecParams);
void mpeg4SetDynamicParams(void *mpeg4EncInst, void *dynParams);
void *mpeg4GetStaticParams(void *mpeg4EncInst);
void *mpeg4GetDynamicParams(void *mpeg4EncInst);
void *mpeg4GetInArgs(void *mpeg4EncInst);
void *mpeg4GetOutArgs(void *mpeg4EncInst);
void *mpeg4GetStatus(void *mpeg4EncInst);
void *mpeg4GetFxns(void *mpeg4EncInst);
void mpeg4SetMcParams(void *mpeg4EncInst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine);
void *mpeg4GetIresFxns(void);

#define mpeg4EncAPI {          \
    (const char *)"MPEG4ENC",  \
    sizeof(mpeg4EncInst_t),    \
    mpeg4InitInstance,         \
    mpeg4SetStaticParams,      \
    mpeg4SetDynamicParams,     \
    mpeg4GetStaticParams,      \
    mpeg4GetDynamicParams,     \
    mpeg4GetInArgs,            \
    mpeg4GetOutArgs,           \
    mpeg4GetStatus,            \
    mpeg4GetFxns,              \
    mpeg4SetMcParams,          \
    mpeg4GetIresFxns           \
}

#endif

/* nothing past this point */
