/******************************************************************************
* FILE PURPOSE: Install XDM compliant Codecs
******************************************************************************
* FILE NAME:   siuVctCodecAPI.h
*
* DESCRIPTION: Install XDM compliant Codecs
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#ifndef _SIUVCT_CODEC_API_H
#define _SIUVCT_CODEC_API_H
#include <ti/mas/types/types.h>
#include <string.h>
#include <stdio.h>
#include "xdc/std.h"
#include <ti/xdais/dm/xdm.h>
#include <ti/bios/include/std.h>
#include <ti/xdais/ialg.h>
#include <ti/xdais/dm/ivideo.h>
#include <ti/xdais/dm/ividenc.h>
#include <ti/xdais/dm/ividenc1.h>
#include <ti/xdais/dm/ividdec2.h>
#include <ti/xdais/dm/iimgdec1.h>
#include <ti/xdais/dm/iaudenc.h>
#include <ti/xdais/dm/iauddec1.h>
#include <ti/xdais/dm/iimgenc1.h>
#include <ti/xdais/dm/ividdec.h>
#include <ividmc.h>

typedef struct vidEncoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  tuint        encInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*encInitInstance)(void *inst);
  void         (*encSetStaticParams)(void *inst, void *codecParams);
  void         (*encSetDynamicParams)(void *inst, void *dynParams);
  void*        (*encGetStaticParams)(void *inst);
  void*        (*encGetDynamicParams)(void *inst);
  void*        (*encGetInArgs)(void *inst);
  void*        (*encGetOutArgs)(void *inst);
  void*        (*encGetStatus)(void *inst);
  void*        (*encGetFxns)(void *inst);
  void         (*encSetMcParams)(void *inst, IVIDMC_t *params, XDAS_UInt32 topSliceLine, XDAS_UInt32 bottomSliceLine);
  void*        (*encGetIresFxns)(void);
} vidEncoderAPI_t;

typedef struct vidDecoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  tuint        decInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*decInitInstance)(void *inst);
  void         (*decSetStaticParams)(void *inst, void *codecParams);
  void         (*decSetDynamicParams)(void *inst, void *params);
  void*        (*decGetStaticParams)(void *inst);
  void*        (*decGetDynamicParams)(void *inst);
  void*        (*decGetInArgs)(void *inst);
  void*        (*decGetOutArgs)(void *inst);
  void*        (*decGetStatus)(void *inst);
  void*        (*decGetFxns)(void *inst);
  void         (*decSetMcParams)(void *inst, IVIDMC_t *params);
  void*        (*decGetIresFxns)(void);
} vidDecoderAPI_t;

typedef struct imgEncoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  tuint        encInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*encInitInstance)(void *inst);
  void         (*encSetStaticParams)(void *inst, void *codecParams);
  void         (*encSetDynamicParams)(void *inst, void *params);
  void*        (*encGetStaticParams)(void *inst);
  void*        (*encGetDynamicParams)(void *inst);
  void*        (*encGetInArgs)(void *inst);
  void*        (*encGetOutArgs)(void *inst);
  void*        (*encGetStatus)(void *inst);
  void*        (*encGetFxns)(void *inst);
} imgEncoderAPI_t;

typedef struct imgDecoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  tuint        decInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*decInitInstance)(void *inst);
  void         (*decSetStaticParams)(void *inst, void *codecParams);
  void         (*decSetDynamicParams)(void *inst, IIMGDEC1_Params *params);
  void*        (*decGetStaticParams)(void *inst);
  void*        (*decGetDynamicParams)(void *inst);
  void*        (*decGetInArgs)(void *inst);
  void*        (*decGetOutArgs)(void *inst);
  void*        (*decGetStatus)(void *inst);
  void*        (*decGetFxns)(void *inst);
  void         (*decSetDynamicParamsFrameNumBytes)(void *inst, XDAS_Int32 frame_numbytes);
  XDAS_Int32   (*decGetDynamicParamsProgDisplay)(void *inst);
  XDAS_Int32   (*decGetStatusEndOfSeq)(void *inst);
} imgDecoderAPI_t;

typedef struct audEncoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  tuint        encInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*encInitInstance)(void *inst);
  void         (*encSetStaticParams)(void *inst, void *codecParams);
  void         (*encSetDynamicParams)(void *inst, void *dynParams);
  void*        (*encGetStaticParams)(void *inst);
  void*        (*encGetDynamicParams)(void *inst);
  void*        (*encGetInArgs)(void *inst);
  void*        (*encGetOutArgs)(void *inst);
  void*        (*encGetStatus)(void *inst);
  void*        (*encGetFxns)(void *inst);
} audEncoderAPI_t;

typedef struct audDecoderAPI_s {
  const char   *name;                         /**< Codec Name   */
  tuint        decInstanceSize;               /**< size of Codec instance, for memory allocation           */
  void         (*decInitInstance)(void *inst);
  void         (*decSetStaticParams)(void *inst, void *codecParams);
  void         (*decSetDynamicParams)(void *inst, IAUDDEC1_Params *params);
  void*        (*decGetStaticParams)(void *inst);
  void*        (*decGetDynamicParams)(void *inst);
  void*        (*decGetInArgs)(void *inst);
  void*        (*decGetOutArgs)(void *inst);
  void*        (*decGetStatus)(void *inst);
  void*        (*decGetFxns)(void *inst);
} audDecoderAPI_t;

#define vidEncNullAPI {       \
  (const char *)"NULL",    \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

#define vidDecNullAPI {       \
  (const char *)"NULL",    \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

#define imgEncNullAPI {       \
  (const char *)"NULL",    \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

#define imgDecNullAPI {       \
  (const char *)"NULL",    \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

#define audEncNullAPI {       \
  (const char *)"NULL",    \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

#define audDecNullAPI {       \
  (const char *)"NULL",    \
  0,                       \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL,                    \
  NULL                     \
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* vidEncoderAPI_t getVidEncoderAPI(
*   const char *encName        - Codec Name string
* )
*****************************************************************************/
static inline vidEncoderAPI_t *getVidEncoderAPI(const char *encName)
{
  extern vidEncoderAPI_t supportedVidEncoders[];
  vidEncoderAPI_t *suppEnc = supportedVidEncoders;

  while(strcmp(suppEnc->name, encName) != 0) {
     if(strcmp(suppEnc->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppEnc++;
     }
  }
  return(suppEnc);
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* vidDecoderAPI_t getVidDecoderAPI(
*   const char *decName        - Codec Name string
* )
*****************************************************************************/
static inline vidDecoderAPI_t *getVidDecoderAPI(const char *decName)
{
  extern vidDecoderAPI_t supportedVidDecoders[];
  vidDecoderAPI_t *suppDec = supportedVidDecoders;

  while(strcmp(suppDec->name, decName) != 0) {
     if(strcmp(suppDec->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppDec++;
     }
  }
  return(suppDec);
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* vidEncoderAPI_t getImgEncoderAPI(
*   const char *encName        - Codec Name string
* )
*****************************************************************************/
static inline imgEncoderAPI_t *getImgEncoderAPI(const char *encName)
{
  extern imgEncoderAPI_t supportedImgEncoders[];
  imgEncoderAPI_t *suppEnc = supportedImgEncoders;

  while(strcmp(suppEnc->name, encName) != 0) {
     if(strcmp(suppEnc->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppEnc++;
     }
  }
  return(suppEnc);
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* imgDecoderAPI_t getImgDecoderAPI(
*   const char *decName        - Codec Name string
* )
*****************************************************************************/
static inline imgDecoderAPI_t *getImgDecoderAPI(const char *decName)
{
  extern imgDecoderAPI_t supportedImgDecoders[];
  imgDecoderAPI_t *suppDec = supportedImgDecoders;

  while(strcmp(suppDec->name, decName) != 0) {
     if(strcmp(suppDec->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppDec++;
     }
  }
  return(suppDec);
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* audEncoderAPI_t getAudEncoderAPI(
*   const char *encName        - Codec Name string
* )
*****************************************************************************/
static inline audEncoderAPI_t *getAudEncoderAPI(const char *encName)
{
  extern audEncoderAPI_t supportedAudEncoders[];
  audEncoderAPI_t *suppEnc = supportedAudEncoders;

  while(strcmp(suppEnc->name, encName) != 0) {
     if(strcmp(suppEnc->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppEnc++;
     }
  }
  return(suppEnc);
}

/******************************************************************************
* FUNCTION PURPOSE: Search Installed Codecs by string name and return API
******************************************************************************
* DESCRIPTION: Search Installed Codecs by string name and return API
*
* audDecoderAPI_t getAudDecoderAPI(
*   const char *decName        - Codec Name string
* )
*****************************************************************************/
static inline audDecoderAPI_t *getAudDecoderAPI(const char *decName)
{
  extern audDecoderAPI_t supportedAudDecoders[];
  audDecoderAPI_t *suppDec = supportedAudDecoders;

  while(strcmp(suppDec->name, decName) != 0) {
     if(strcmp(suppDec->name, "NULL") == 0) {
        return(NULL);
     } else {
        suppDec++;
     }
  }
  return(suppDec);
}

#endif

/* nothing past this point */
