/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   siuVctEncode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "siuVctCodecAPI.h"
#include "siuVctParse.h"
#include "siuVctTftp.h"
#include "common/siuloc.h"
#include "osal/siuFcOsal.h"

#include "siuVctFileIO.h"
#include "targethw.h"
#include <ti/mas/tftp/tftp.h>

#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Log.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Memory.h>

extern IALG_Handle ALG_create(IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params);
extern Void ALG_delete(IALG_Handle alg);

#pragma DATA_SECTION( inpBuf_aud_dec, ".testCodecExtMem");
XDAS_Int8 inpBuf_aud_dec[2880];  

#pragma DATA_SECTION( outBuf_aud_dec, ".testCodecExtMem");
#define MAX_OUT_BUF_SIZE 4608
XDAS_UInt8 outBuf_aud_dec[MAX_OUT_BUF_SIZE];
XDAS_UInt8 outBuf_aud_temp[MAX_OUT_BUF_SIZE];

#define STRING_SIZE            256
#define NUM_ALGS                 1      /* Number of Algorithms              */

void WriteToFileAudioInterleaved(XDAS_UInt16 *out, XDAS_Int16 nch,XDAS_Int32 nSamples, XDAS_Int32 outputBitsPerSample)
{
    XDAS_Int16 i;
    XDAS_UInt16 c,c1,c2;
    XDAS_UInt8 *outbufPtr = &outBuf_aud_temp[0];
    XDAS_Int32 outbufSize = 0;

    if(outputBitsPerSample == 16)
    {
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i=i+1)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;
                memcpy(&outbufPtr[outbufSize++],&c2, 1);
                memcpy(&outbufPtr[outbufSize++],&c1, 1);
            }
        }
        else
        {
            for(i=0;i<nSamples*2;i=i+2)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outbufPtr[outbufSize++],&c2, 1);
                memcpy(&outbufPtr[outbufSize++],&c1, 1);
                /* Right Channel  */
                c    = out[i+1];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outbufPtr[outbufSize++],&c2, 1);
                memcpy(&outbufPtr[outbufSize++],&c1, 1);
            }
        }
    } else {/*24-bit samples */
        XDAS_Int32* out24 = (XDAS_Int32 *)out;
        XDAS_Int32 c24;
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i=i+1)
            {
                /* Left Channel */
                c24 = out24[i];
                memcpy(&outbufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
        else
        {
            for(i=0;i<nSamples*2;i=i+2)
            {

                /* Left Channel */
                c24 = out24[i];
                memcpy(&outbufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
                /* Left Channel */
                c24 = out24[i + 1];
                memcpy(&outbufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
    }
    if (outbufSize > MAX_OUT_BUF_SIZE) {
       printf("Error.. Max out buf size exceeded \n");
       fflush(stdout);
    }
    printf("frame size = %d ", outbufSize);
    siuVct_WriteOutputData(outbufPtr, outbufSize);
} // end function WriteToFileAudioInterleaved


void WriteToFileBlock(XDAS_UInt16 *out, XDAS_Int16 nch,XDAS_Int32 nSamples, XDAS_Int32 outputBitsPerSample)
{
    XDAS_Int16 i;
    XDAS_UInt16 c,c1,c2;
    XDAS_UInt8 *outbufPtr = &outBuf_aud_temp[0];
    XDAS_Int32 outbufSize = 0;

    if(outputBitsPerSample == 16)
    {
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i++)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outbufPtr[outbufSize++],&c2, 1);
                memcpy(&outbufPtr[outbufSize++],&c1, 1);
            }
        }
        else
        {
            for(i=0;i<nSamples;i++)
            {
                /* Left Channel */
                c    = out[i];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outbufPtr[outbufSize++],&c2, 1);
                memcpy(&outbufPtr[outbufSize++],&c1, 1);
                /* Right Channel  */
                c    = out[i+nSamples];
                c1 = (c >> 8) & 0xff;
                c2 = c & 0xff;

                memcpy(&outbufPtr[outbufSize++],&c2, 1);
                memcpy(&outbufPtr[outbufSize++],&c1, 1);
            }
        }
    }
    else /* 24-bit */
    {
        XDAS_Int32* out24 = (XDAS_Int32 *)out;
        XDAS_Int32 c24;
        if(nch == IAUDIO_1_0)
        {
            for(i=0;i<nSamples;i=i+1)
            {
                /* Left Channel */
                c24 = out24[i];
                memcpy(&outbufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
        else
        {
            for(i=0;i<nSamples;i=i+1)
            {

                /* Left Channel */
                c24 = out24[i];
                memcpy(&outbufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
                /* Left Channel */
                c24 = out24[i + nSamples];
                memcpy(&outbufPtr[outbufSize],&c24, 3);
                outbufSize += 3;
            }
        }
    }

    if (outbufSize > MAX_OUT_BUF_SIZE) {
       printf("Error.. Max out buf size exceeded \n");
       fflush(stdout);
    }
    siuVct_WriteOutputData(outbufPtr, outbufSize);
}


/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Audio Decode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Audio Decode Call
*
* XDAS_Int32 tvct_test_entry_decode_xdm1p0(
*    const char *codecname        - name of the codec 
*    XDAS_UInt32 framesToDecode   - # of frames to decode
*    void *codecParams            - Codec Params
* )
*****************************************************************************/
#pragma DATA_SECTION( audDecInst, ".testCodecExtMem");
#define MAX_AUD_DEC_INST_SIZE 5000
XDAS_Int8 audDecInst[MAX_AUD_DEC_INST_SIZE];
extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int DNUM;

XDAS_Int32 tvct_test_entry_aud_decode_xdm1p0 (const char *codecname, XDAS_UInt32 framesToDecode, void *codecParams)
{
    /* Input/Output  buffers  and their sizes                                  */
    IAUDDEC1_Fxns              *iaudDecFxns;
    IAUDDEC1_Status            *status;
    IAUDDEC1_InArgs            *inArgs;
    IAUDDEC1_OutArgs           *outArgs;
    IAUDDEC1_Params            *staticParams;
    IAUDDEC1_DynamicParams     *dynamicParams;

    /* Algorithm specific handle                                               */
    IALG_Handle handle;
    /* Input/Output Buffer Descriptors                                         */
    XDM1_BufDesc inputBufDesc;
    XDM1_BufDesc outBufDesc;

    /* Input Buffer Management specific variables                              */
    XDAS_UInt32 ii, bytesConsumed;
    XDAS_UInt32 BytesRead=0;
    /* Other variables                                                         */
    XDAS_Int32  frameCount, retVal, nDecodedSamples = 0;
    XDAS_Int16  nch, outputBitsPerSample;
    audDecoderAPI_t *decAPI;
    void *decFxns;

    siuVct_DDR_Init();

    siuVct_FileIO_Start();

    decAPI = getAudDecoderAPI(codecname);

    if (decAPI == NULL){
       printf("Decoder is not installed .. exiting \n");
       return -1;
    }

    if(decAPI->decInstanceSize >= MAX_AUD_DEC_INST_SIZE) {
       printf("Decoder Instance Size Exceeds allocated value.. exiting \n");
       return -1;
    }
    decAPI->decInitInstance((void *)audDecInst);

    /* Initialization of parameters needed for Algorithm Instance create */
    decAPI->decSetStaticParams((void *)audDecInst, codecParams);

    /* Get XDM1.0 pointers from the codec */
    decFxns               =  decAPI->decGetFxns((void *)audDecInst);
    inArgs                = (IAUDDEC1_InArgs *)decAPI->decGetInArgs((void *)audDecInst);
    outArgs               = (IAUDDEC1_OutArgs *)decAPI->decGetOutArgs((void *)audDecInst);
    staticParams          = (IAUDDEC1_Params *)decAPI->decGetStaticParams((void *)audDecInst);
    dynamicParams         = (IAUDDEC1_DynamicParams *)decAPI->decGetDynamicParams((void *)audDecInst);
    status                = (IAUDDEC1_Status *)decAPI->decGetStatus((void *)audDecInst);

    /* Initializing Variables                                                */
    frameCount                   = 0; 

    /* Create the Algorithm object (instance)                                */
    printf("\nCreating Algorithm Instance...");

    if ((handle =  (IALG_Handle)ALG_create (
        (IALG_Fxns *) decFxns, 
        (IALG_Handle) NULL, 
        (IALG_Params *) staticParams)) == NULL)
    {
        printf( "\nFailed to Create Instance... Exiting for this configuration..");
        return(-1);
    }
    printf("\nAlgorithm Instance Creation Done...\n");

    /* Assigning Algorithm handle fxns field to iaudDecfxns                  */
    iaudDecFxns = (IAUDDEC1_Fxns *)handle->fxns;

    /* Set run time dynamic parameters */
    decAPI->decSetDynamicParams((void *)audDecInst, (IAUDDEC1_Params *)staticParams);

    /*-----------------------------------------------------------------------*/
    /*  The XDM_RESET call gives resets all the internal data structures and */
    /*  context for the new stream that is to be decoded.                    */
    /*-----------------------------------------------------------------------*/
    //iaudDecFxns->control((IAUDDEC1_Handle)handle, XDM_RESET,     dynamicParams, status);
    iaudDecFxns->control((IAUDDEC1_Handle)handle, XDM_SETPARAMS, dynamicParams, status);

    /*-----------------------------------------------------------------------*/
    /*  The GETBUFINFO call gives information for number and size of input   */
    /*  and output buffers. For this first call, the actual width and height */
    /*  of the picture is not known hence, this control call returns the out */
    /*  buffer sizes based on the maxHeight and maxWidth provided as part    */
    /*  of the input params from testparams file.                            */
    /*-----------------------------------------------------------------------*/
    iaudDecFxns->control((IAUDDEC1_Handle)handle, XDM_GETBUFINFO, dynamicParams, status);

    /*-----------------------------------------------------------------------*/
    /*  Initialize the input buffer properties as required by algorithm      */
    /*  based on info received by preceding GETBUFINFO call. First init the  */
    /*  number of input bufs.                                                */
    /*-----------------------------------------------------------------------*/

    inputBufDesc.numBufs          = status->bufInfo.minNumInBufs ;
    inputBufDesc.descs[0].bufSize = status->bufInfo.minInBufSize[0];

    for(ii=0; ii<(status->bufInfo.minNumInBufs-1);ii++ )
    {
        inputBufDesc.descs[ii+1].buf = inputBufDesc.descs[ii].buf +
            status->bufInfo.minInBufSize[ii];
        inputBufDesc.descs[ii+1].bufSize =
            status->bufInfo.minInBufSize[ii +1];
    }

    /*-----------------------------------------------------------------------*/
    /*  Initialize the output buffer properties as required by algorithm     */
    /*  based on info received by preceding GETBUFINFO call.                 */
    /*-----------------------------------------------------------------------*/
    outBufDesc.numBufs       = status->bufInfo.minNumOutBufs;
    outBufDesc.descs[0].bufSize = status->bufInfo.minOutBufSize[0];
    outBufDesc.descs[0].buf     = (XDAS_Int8*)&outBuf_aud_dec;

    /* set space for buffer descriptors                                      */

    inputBufDesc.descs[0].buf     = inpBuf_aud_dec;

    /* Resetting bytesGenerated variable                                     */
    bytesConsumed              = 0;
    outArgs->bytesConsumed     = 0;
    inArgs->numBytes           = 0;
    inArgs->lfeFlag            = 0;
    inArgs->desiredChannelMode = IAUDIO_2_0;
    
    /* Do-While Loop for Decode Call                                         */
    do
    {
        /* Assign the number of bytes available                                */
        inputBufDesc.descs[0].buf   = (XDAS_Int8 *)(inpBuf_aud_dec);

        BytesRead = siuVct_ReadInputData(inputBufDesc.descs[0].buf,inputBufDesc.descs[0].bufSize, outArgs->bytesConsumed);

        siu_osal_wbinv_cache_all();

        /* Check for Break off condition                                       */
        if (BytesRead <= 8)
        {
            printf("\n Bitstream Ended...  ");
            break; /* Bitstream ended: Break off the while loop                  */
        }

        inArgs->numBytes =  BytesRead;

        /* Optional: Set Run time parameters in the Algorithm via control()    */
        //iaudDecFxns->control((IAUDDEC1_Handle)handle, XDM_SETPARAMS, dynamicParams, status);

        /* Basic Algorithm process() call                                      */
        retVal = iaudDecFxns->process((IAUDDEC1_Handle)handle, 
            (XDM1_BufDesc *)&inputBufDesc,
            (XDM1_BufDesc *)&outBufDesc,
            inArgs,
            outArgs);

        if((retVal == XDM_EFAIL) || (outArgs->extendedError))
        {
          printf("\n Process function returned an Error for stream...\n");
          siuVct_FileIO_Stop();
          return(-1);
        }

        /* Optional: Read status via control()                                 */
        iaudDecFxns->control((IAUDDEC1_Handle)handle, XDM_GETSTATUS, dynamicParams, status);

        nDecodedSamples = status->numSamples;
        nch = (short)(status->channelMode);
        outputBitsPerSample = status->outputBitsPerSample;

        siu_osal_wbinv_cache_all();

        /* Check for frame ready                    */
        if (outArgs->extendedError == XDM_EOK)
        {
            printf("\n Decoded Frame # %d  ",frameCount);
            /* Write the output frames                     */
            if (status->pcmFormat == IAUDIO_INTERLEAVED) {
                printf("nDecodedSamples = %d ",nDecodedSamples);
                WriteToFileAudioInterleaved((XDAS_UInt16 *)outBufDesc.descs[0].buf, nch,nDecodedSamples, outputBitsPerSample);
            } else if (status->pcmFormat == IAUDIO_BLOCK) {
                WriteToFileBlock((XDAS_UInt16 *)outBufDesc.descs[0].buf, nch,nDecodedSamples, outputBitsPerSample);
            } else {
                printf("Output format not supported \n");
            }
        }

        frameCount++;

        if (frameCount >= framesToDecode)
        {
            printf("\n Specified number of Frames Decoded...  ");
            break;
        }
        bytesConsumed          += outArgs->bytesConsumed;
    }  while(1);  /* end of Do-While loop                                    */
    
    /* Delete the Algorithm instance object specified by handle */
    ALG_delete (handle);
    siuVct_FileIO_Stop();

    printf("\n --------------  SUMMARY --------------------\n");
    printf("\t Total number of Frames              = %d\n",
        (XDAS_UInt32)frameCount);
    printf (" --------------    END   --------------------\n");
    printf("\n End of execution\n");

    return(0);

} /* tvct_test_entry_decode_xdm1p0 */

/* nothing past this point */
