/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   siuVctEncode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "siuVctCodecAPI.h"
#include "siuVctBufferManager.h"
#include "siuVctParse.h"
#include "siuVctTftp.h"
#include "ividmc/siuVidMc.h"
#include "common/siuloc.h"
#include "osal/siuFcOsal.h"

#include "siuVctFileIO.h"
#include "targethw.h"
#include <ti/mas/tftp/tftp.h>

#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Log.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Memory.h>

extern IALG_Handle ALG_create(IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params);
extern Void ALG_delete(IALG_Handle alg);

#pragma DATA_SECTION( inputDataAud, ".testCodecExtMemAud");
XDAS_Int8 inputDataAud[4096];

#pragma DATA_SECTION( outputDataAud, ".testCodecExtMemAud");
XDAS_Int8 outputDataAud[4096];

/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Image Decode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Image Decode Call
*
* XDAS_Int32 tvct_test_entry_img_decode_xdm1p0(
*    const char *codecname        - name of the codec 
*    XDAS_UInt32 codecParams      - # of cores employed in 1 Decode
* )
*****************************************************************************/
#pragma DATA_SECTION( encInst, ".testCodecExtMemAud");
#define MAX_ENC_INST_SIZE 5000
XDAS_Int8 audEncInst[MAX_ENC_INST_SIZE];
extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int DNUM;

XDAS_Int32 tvct_test_entry_aud_encode_xdm0p9 (const char *codecname, void *codecParams)
{
    /* Input/Output  buffers  and their sizes                                  */
    /* Input/Output  buffers  and their sizes */
    XDAS_Int8* inputbuf[XDM_MAX_IO_BUFFERS];
    XDAS_Int8* pOutputBuf  [XDM_MAX_IO_BUFFERS];
    XDAS_Int32 inputbufsize[XDM_MAX_IO_BUFFERS];
    XDAS_Int32 outBufSize  [XDM_MAX_IO_BUFFERS];

    XDAS_UInt32  ii, tscl_begin, tscl_end;
    IAUDENC_Fxns               *iaudENCFxns;
    IAUDENC_Status             *status;
    IAUDENC_InArgs             *inArgs;
    IAUDENC_OutArgs            *outArgs;
    IAUDENC_Params             *staticParams;
    IAUDENC_DynamicParams      *dynamicParams;
    void *encFxns;

    /* Algorithm specific handle                                               */
    IALG_Handle handle;
    /* Input/Output Buffer Descriptors                                         */
    XDM_BufDesc inputBufDesc;
    XDM_BufDesc outputBufDesc;

    /* Input Buffer Management specific variables                              */
    XDAS_UInt32 bytesConsumed = 0;
    XDAS_UInt32 BytesRead=0;
    /* Other variables                                                         */
    XDAS_Int32  retVal, frameCount = 0;
    audEncoderAPI_t *encAPI;

    siuVct_DDR_Init();

    siuVct_FileIO_Start();

    encAPI = getAudEncoderAPI(codecname);

    if (encAPI == NULL){
       printf("Decoder is not installed .. exiting \n");
       return -1;
    }

    if(encAPI->encInstanceSize >= MAX_ENC_INST_SIZE) {
       printf("Encoder Instance Size Exceeds allocated value.. exiting \n");
       return -1;
    }
    encAPI->encInitInstance((void *)audEncInst);

    /* Initialization of parameters needed for Algorithm Instance create */
    encAPI->encSetStaticParams((void *)audEncInst, codecParams);
 
    /* Get XDM0.9 pointers from the codec */
    encFxns               = encAPI->encGetFxns((void *)audEncInst);
    inArgs                = (IAUDENC_InArgs *)encAPI->encGetInArgs((void *)audEncInst);
    outArgs               = (IAUDENC_OutArgs *)encAPI->encGetOutArgs((void *)audEncInst);
    staticParams          = (IAUDENC_Params *)encAPI->encGetStaticParams((void *)audEncInst);
    dynamicParams         = (IAUDENC_DynamicParams *)encAPI->encGetDynamicParams((void *)audEncInst);
    status                = (IAUDENC_Status *)encAPI->encGetStatus((void *)audEncInst);

    if ((handle =  (IALG_Handle)ALG_create (
        (IALG_Fxns *) encFxns,
        (IALG_Handle) NULL,
        (IALG_Params *) staticParams)) == NULL)
    {
        printf("\nFailed to Create Instance... Exiting for this configuration..");
        return -1;
    }

    /* Assigning Algorithm handle fxns field to iaudEncFxns */
    iaudENCFxns = (IAUDENC_Fxns *)handle->fxns ;

    printf("\nAlgorithm Instance Creation Done...\n");

    /* Set run time dynamic parameters */
    encAPI->encSetDynamicParams((void *)audEncInst, (void *)staticParams);

    /* set space for buffer descriptors                                       */
    outputBufDesc.bufs = (XDAS_Int8 **)pOutputBuf;
    outputBufDesc.bufSizes = (XDAS_Int32 *)outBufSize ;

    siu_osal_wbinv_cache_all();

    /* read wav header */
    bytesConsumed   = siuVct_ReadInputData(inputDataAud, 44, bytesConsumed);

    /* Do-While Loop for Encode Call                                          */
    do
    {
        /* Optional: Set Run time parameters in the Algorithm via control()     */
        retVal = iaudENCFxns->control((IAUDENC_Handle)handle, XDM_SETPARAMS,
            (IAUDENC_DynamicParams *)dynamicParams, (IAUDENC_Status *)status);

        if(retVal == XDM_EFAIL)
        {
            printf("\n Control function returned an Error...  ");
            break; /* Error Condition: Application may want to break off          */
        }

        retVal= iaudENCFxns->control((IAUDENC_Handle)handle,
            XDM_GETBUFINFO,
            (IAUDENC_DynamicParams *)dynamicParams,
            (IAUDENC_Status *)status);

        if(retVal == XDM_EFAIL)
        {
            printf("\n Control function returned an Error...  ");
            break; /* Error Condition: Application may want to break off          */
        }

        /*Fill up the buffers as required by algorithm                            */
        inputBufDesc.numBufs = status->bufInfo.minNumInBufs ;
        inputBufDesc.bufs = inputbuf;
        inputBufDesc.bufs[0]  = inputDataAud;
        inputBufDesc.bufSizes = inputbufsize;
        inputBufDesc.bufSizes[0] = status->bufInfo.minInBufSize[0];

        for(ii=0; ii< (status->bufInfo.minNumInBufs - 1);ii++ )
        {
            inputBufDesc.bufs[ii+1] = inputBufDesc.bufs[ii] +
                status->bufInfo.minInBufSize[ii];
            inputBufDesc.bufSizes[ii+1]  =
                status->bufInfo.minInBufSize[ii +1];
        }

        outputBufDesc.numBufs     = status->bufInfo.minNumOutBufs ;
        outputBufDesc.bufs[0] = (XDAS_Int8 *)&outputDataAud[0];
        outputBufDesc.bufSizes[0] = status->bufInfo.minOutBufSize[0];

        for(ii=0; ii< (status->bufInfo.minNumOutBufs-1); ii++ )
        {
            outputBufDesc.bufs[ii+1] = outputBufDesc.bufs[ii] +
                status->bufInfo.minOutBufSize[ii];
            outputBufDesc.bufSizes[ii+1] =
                status->bufInfo.minOutBufSize[ii+1];
        }

        BytesRead = 0;
        for(ii=0; ii < inputBufDesc.numBufs ; ii++)
        {
            bytesConsumed   = siuVct_ReadInputData(inputBufDesc.bufs[ii], inputBufDesc.bufSizes[ii], bytesConsumed);
            BytesRead      += bytesConsumed;
        }

        if( BytesRead == 0) {
            printf("\n Bitstream Ended...  ");
            break; /* Bitstream ended: Break off the while loop                  */
        }

        /* Basic Algorithm process() call */
        tscl_begin = TSCL;
        retVal = iaudENCFxns->process((IAUDENC_Handle)handle,
            (XDM_BufDesc *)&inputBufDesc,
            (XDM_BufDesc *)&outputBufDesc,
            (IAUDENC_InArgs *)inArgs,
            (IAUDENC_OutArgs *)outArgs);
        tscl_end = TSCL;
        printf("Delta TSCL = %ld \n",(tscl_end - tscl_begin));

        if ((retVal == XDM_EFAIL) || (outArgs->extendedError != XDM_EOK))
        {
            printf("\n Process function returned an Error...  ");
            break; /* Error Condition: Application may want to break off   */
        }

        /* Optional: Read status via control()                                  */
        iaudENCFxns->control((IAUDENC_Handle)handle,
            XDM_GETSTATUS,
            (IAUDENC_DynamicParams *)dynamicParams,
            (IAUDENC_Status *)status);

        printf("\n Encoded Frame # %d, ",frameCount);
        fflush(stdout);

        /* Write the output frame */
        siuVct_WriteOutputData((XDAS_Int8 *)outputDataAud, outArgs->bytesGenerated);
        printf("Frame %d - Generated %d bytes \n", frameCount, outArgs->bytesGenerated);
        frameCount++;

    }  while(1);  /* end of Do-While loop */

    /* Delete the Algorithm instance object specified by handle */
    ALG_delete (handle);

    if(frameCount == 0)
    {
        frameCount = 1; /* To avoid division with zero */
    }

    siuVct_FileIO_Stop();
    printf("\n End of execution\n");
    return(0);
} /* tvct_test_entry_encode_xdm0p9 */

/* nothing past this point */
