/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   siuVctEncode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "siuVctCodecAPI.h"
#include "siuVctBufferManager.h"
#include "siuVctParse.h"
#include "siuVctTftp.h"
#include "ividmc/siuVidMc.h"
#include "common/siuloc.h"
#include "osal/siuFcOsal.h"

#include "siuVctFileIO.h"
#include "targethw.h"
#include <ti/mas/tftp/tftp.h>

#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Log.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Memory.h>

extern IALG_Handle ALG_create(IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params);
extern Void ALG_delete(IALG_Handle alg);

XDAS_Void siuIctDecode_WriteYUVData(XDM1_BufDesc *outputBufDesc,IIMGDEC1_Handle handle)
{
  XDAS_Int32 i;
  XDAS_UInt8 *output;

  siu_invalidate_cache(); 
  
  for(i = 0; i < outputBufDesc->numBufs; i++)
  {
    output  = (XDAS_UInt8 *)outputBufDesc->descs[i].buf;
    siuVct_WriteOutputData(output, outputBufDesc->descs[i].bufSize);
    
  }  
  return;
}

#pragma DATA_SECTION( ImgInpBuf1, ".testCodecExtMemImg");
XDAS_Int8 ImgInpBuf1[2097152];

#pragma DATA_SECTION( ImgOutBuf1, ".testCodecExtMemImg");
XDAS_Int8 ImgOutBuf1[10091456];  // 2048*1536*2

/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Image Decode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Image Decode Call
*
* XDAS_Int32 tvct_test_entry_img_decode_xdm1p0(
*    const char *codecname        - name of the codec 
*    XDAS_UInt32 codecParams      - # of cores employed in 1 Decode
* )
*****************************************************************************/
#pragma DATA_SECTION( encInst, ".testCodecExtMemImg");
#define MAX_DEC_INST_SIZE 5000
XDAS_Int8 imgDecInst[MAX_DEC_INST_SIZE];
extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int DNUM;
XDAS_Int32 tvct_test_entry_img_decode_xdm1p0 (const char *codecname, void *codecParams)
{
    /* Input/Output  buffers  and their sizes                                  */
    XDAS_UInt32  ii;
    IIMGDEC1_Fxns              *IIMGDEC1Fxns;
    IIMGDEC1_Status            *status;
    IIMGDEC1_InArgs            *inArgs;
    IIMGDEC1_OutArgs           *outArgs;
    IIMGDEC1_Params            *staticParams;
    IIMGDEC1_DynamicParams     *dynamicParams;

    /* Algorithm specific handle                                               */
    IALG_Handle handle;
    /* Input/Output Buffer Descriptors                                         */
    XDM1_BufDesc inputBufDesc;
    XDM1_BufDesc outputBufDesc;

    /* Input Buffer Management specific variables                              */
    XDAS_UInt32 bytesConsumed,ScanCount;
    XDAS_UInt32 BytesRead=0;
    /* Other variables                                                         */
    XDAS_Int32  retVal;
    imgDecoderAPI_t *decAPI;
    void *decFxns;

    siuVct_DDR_Init();

    siuVct_FileIO_Start();

    decAPI = getImgDecoderAPI(codecname);

    if (decAPI == NULL){
       printf("Decoder is not installed .. exiting \n");
       return -1;
    }

    if(decAPI->decInstanceSize >= MAX_DEC_INST_SIZE) {
       printf("Decoder Instance Size Exceeds allocated value.. exiting \n");
       return -1;
    }
    decAPI->decInitInstance((void *)imgDecInst);

    /* Initialization of parameters needed for Algorithm Instance create */
    decAPI->decSetStaticParams((void *)imgDecInst, codecParams);
   
    /* Get XDM1.0 pointers from the codec */
    decFxns               =  decAPI->decGetFxns((void *)imgDecInst);
    inArgs                = (IIMGDEC1_InArgs *)decAPI->decGetInArgs((void *)imgDecInst);
    outArgs               = (IIMGDEC1_OutArgs *)decAPI->decGetOutArgs((void *)imgDecInst);
    staticParams          = (IIMGDEC1_Params *)decAPI->decGetStaticParams((void *)imgDecInst);
    dynamicParams         = (IIMGDEC1_DynamicParams *)decAPI->decGetDynamicParams((void *)imgDecInst);
    status                = (IIMGDEC1_Status *)decAPI->decGetStatus((void *)imgDecInst);

    /* Initializing Variables                                                */
    ScanCount                   = 1; 

    /* Create the Algorithm object (instance)                                */
    printf("\nCreating Algorithm Instance...");

    if ((handle =  (IALG_Handle)ALG_create (
        (IALG_Fxns *) decFxns, 
        (IALG_Handle) NULL, 
        (IALG_Params *) staticParams)) == NULL)
    {
        printf( "\nFailed to Create Instance... Exiting for this configuration..");
        return(-1);
    }
    printf("\nAlgorithm Instance Creation Done...\n");

    /* Assigning Algorithm handle fxns field to ividDecfxns                  */
    IIMGDEC1Fxns = (IIMGDEC1_Fxns *)handle->fxns;

    bytesConsumed = 0;
    outArgs->bytesConsumed = 0;
    inArgs->numBytes = 0;

    inputBufDesc.descs[0].buf   = ImgInpBuf1;
    outputBufDesc.descs[0].buf     = ImgOutBuf1;

    /* Set run time dynamic parameters */
    decAPI->decSetDynamicParams((void *)imgDecInst, (IIMGDEC1_Params *)staticParams);

    dynamicParams->decodeHeader = XDM_PARSE_HEADER;
    inputBufDesc.descs[0].buf   = (XDAS_Int8 *)((XDAS_Int32) ImgInpBuf1 + bytesConsumed);

    IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle, XDM_GETBUFINFO,(IIMGDEC1_DynamicParams *)dynamicParams,(IIMGDEC1_Status *)status);

    inputBufDesc.numBufs     = status->bufInfo.minNumInBufs ;
    inputBufDesc.descs[0].bufSize = status->bufInfo.minInBufSize[0];

    for(ii=0; ii<(status->bufInfo.minNumInBufs-1);ii++ )
    {
        inputBufDesc.descs[ii+1].buf = inputBufDesc.descs[ii].buf +
            status->bufInfo.minInBufSize[ii];
        inputBufDesc.descs[ii+1].bufSize=
            status->bufInfo.minInBufSize[ii +1];
    }

    BytesRead = siuVct_ReadInputData(inputBufDesc.descs[0].buf,inputBufDesc.descs[0].bufSize, outArgs->bytesConsumed);

    if (BytesRead <=0)
    {
        printf("\n Bitstream Ended...  ");
    }

    inArgs->numBytes               =  BytesRead;
    decAPI->decSetDynamicParamsFrameNumBytes((void *)imgDecInst, inArgs->numBytes);
    outputBufDesc.numBufs          = status->bufInfo.minNumOutBufs ;
    outputBufDesc.descs[0].bufSize = status->bufInfo.minOutBufSize[0];

    for(ii=0; ii<(status->bufInfo.minNumOutBufs-1);ii++ )
    {
        outputBufDesc.descs[ii+1].buf = outputBufDesc.descs[ii].buf +
            status->bufInfo.minOutBufSize[ii];
        outputBufDesc.descs[ii+1].bufSize =
            status->bufInfo.minOutBufSize[ii+1];
    }

    IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle, XDM_SETPARAMS,(IIMGDEC1_DynamicParams *)dynamicParams, (IIMGDEC1_Status *)status);

    retVal = IIMGDEC1Fxns->process((IIMGDEC1_Handle)handle, 
        (XDM1_BufDesc *)&inputBufDesc,
        (XDM1_BufDesc *)&outputBufDesc,
        (IIMGDEC1_InArgs *)inArgs,
        (IIMGDEC1_OutArgs *)outArgs);

    IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle, XDM_GETSTATUS, (IIMGDEC1_DynamicParams *)dynamicParams, (IIMGDEC1_Status *)status);
    dynamicParams->decodeHeader = XDM_DECODE_AU;
    IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle, XDM_RESET, (IIMGDEC1_DynamicParams *)dynamicParams, (IIMGDEC1_Status *)status);

    bytesConsumed                  = 0;
    inArgs->numBytes               = BytesRead;
    inputBufDesc.descs[0].buf      = (XDAS_Int8 *)((XDAS_Int32) ImgInpBuf1);
    IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle, XDM_GETBUFINFO, (IIMGDEC1_DynamicParams *)dynamicParams, (IIMGDEC1_Status *)status);

    /* Fill up the buffers as required by algorithm                        */
    inputBufDesc.numBufs     = status->bufInfo.minNumInBufs ;
    inputBufDesc.descs[0].bufSize = status->bufInfo.minInBufSize[0];

    for(ii=0; ii<(status->bufInfo.minNumInBufs-1);ii++)
    {
        inputBufDesc.descs[ii+1].buf = inputBufDesc.descs[ii].buf +
            status->bufInfo.minInBufSize[ii];
        inputBufDesc.descs[ii+1].bufSize=
            status->bufInfo.minInBufSize[ii +1];
    }

    outputBufDesc.numBufs     = status->bufInfo.minNumOutBufs ;
    outputBufDesc.descs[0].bufSize = status->bufInfo.minOutBufSize[0];
    for(ii=0; ii<(status->bufInfo.minNumOutBufs-1);ii++ )
    {
        outputBufDesc.descs[ii+1].buf = outputBufDesc.descs[ii].buf +
            status->bufInfo.minOutBufSize[ii];
        outputBufDesc.descs[ii+1].bufSize =
            status->bufInfo.minOutBufSize[ii+1];
    }

    /* Optional: Set Run time parameters in the Algorithm via control()    */
    IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle, XDM_SETPARAMS,
        (IIMGDEC1_DynamicParams *)dynamicParams,(IIMGDEC1_Status *)status); 

    /* Do-While Loop for Decode Call                                         */
    do
    {
#ifdef PROFILING
        t0=TSCL_Begin = TSCL;
#endif
        /* Basic Algorithm process() call                                      */
        retVal = IIMGDEC1Fxns->process((IIMGDEC1_Handle)handle,
            (XDM1_BufDesc *)&inputBufDesc,
            (XDM1_BufDesc *)&outputBufDesc,
            (IIMGDEC1_InArgs *)inArgs,
            (IIMGDEC1_OutArgs *)outArgs);	
#ifdef PROFILING
        t1=TSCL_End   = TSCL;
        tslice +=(t1 - t0);
        decodetime = t1-t0;
        total_decodetime +=decodetime;
        Megapixel = (((float) (decodetime))/((float)(status->outputWidth*status->imgdecStatus.outputHeight)));
        printf("cycles taken                %d \n", decodetime);
        printf("cycles/pixel                : %f \n",
            (((float) (decodetime))/((float)(status->outputWidth*status->outputHeight))));
#endif
        bytesConsumed          += outArgs->bytesConsumed;

        if(retVal == XDM_EFAIL)
        {
            printf("\n Process function returned an Error...  ");
            break; /* Error Condition: Application may want to break off         */
        }

        /* Optional: Read status via control()                                 */
        IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle,XDM_GETSTATUS,
            (IIMGDEC1_DynamicParams *)dynamicParams,(IIMGDEC1_Status *)status);


        if(( outArgs->extendedError != IIMGDEC1_EOK))
        {
            printf("\n Decoder ERROR %0x \n ",outArgs->extendedError);
            break;
        }

        /* Check for frame ready via display buffer pointers                   */
        if(outputBufDesc.descs[0].buf != NULL)
        {
            printf("\n Decoded Frame # %d  ",ScanCount);
            ScanCount++;
        }

        if(decAPI->decGetStatusEndOfSeq((void *)imgDecInst) == 1)
        {
            break;
        }

    }  while(1);  /* end of Do-While loop */


#ifdef PROFILING
    if(dynamicParams->numMCU_row )
        total_decodetime = tslice;
    tslice=0;

    Megapixel = (((float) (total_decodetime))/((float)(status->outputWidth*status->outputHeight)));
    total_decodetime = Megapixel/(ScanCount-1);	
    printf("total cycles taken          %d \n", total_decodetime);
    printf("cycles/pixel                : %f \n\n",
        (((float) (total_decodetime))/((float)(status->outputWidth*status->outputHeight))));        
#endif

    IIMGDEC1Fxns->control((IIMGDEC1_Handle)handle,
        XDM_GETSTATUS,
        (IIMGDEC1_DynamicParams *)dynamicParams,
        (IIMGDEC1_Status *)status);

    for(ii=0; ii<(status->bufInfo.minNumOutBufs);ii++ )
    {

        outputBufDesc.descs[ii].bufSize =
            status->bufInfo.minOutBufSize[ii];
    }

    if(( outArgs->extendedError == IIMGDEC1_EOK))
    {
        XDAS_Int32 progDisplay = decAPI->decGetDynamicParamsProgDisplay((void *)imgDecInst);
        if(!progDisplay)
        {
            for(ii=0; ii < 1 ; ii++)
            {
                if(outArgs->bytesConsumed != 0 )
                {
                    /* Write the output frames                     */
                    siuIctDecode_WriteYUVData(&outputBufDesc,(IIMGDEC1_Handle)handle);
                }
            }
        }
    } 

    ALG_delete (handle);

    if(ScanCount == 0) 
    { 
        ScanCount = 1; /* To avoid division with zero */
    }

    siuVct_FileIO_Stop();

    printf("\n --------------  SUMMARY --------------------\n");
    printf("\t Total number of Scans              = %d\n",
        (XDAS_UInt32)ScanCount);

    printf("\t Width and Height                    = %d, %d \n",
        (XDAS_UInt32)status->outputWidth, (XDAS_UInt32)status->outputHeight);
#ifdef PROFILING
    printf(" \n Total cycles consumed = %d \n", total_decodetime);
#endif
    printf (" --------------    END   --------------------\n");
    printf("\n End of execution\n");
    return(0);

} /* tvct_test_entry_decode_xdm1p0 */

/* nothing past this point */
