/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   siuIctEncode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include "siuVctCodecAPI.h"
#include "siuVctFileIO.h"

extern IALG_Handle ALG_create(IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params);
extern Void ALG_delete(IALG_Handle alg);

#define IMAGE_WIDTH         2560      //!< Width of the Display Buffer
#define IMAGE_HEIGHT        2048      //!< Height of the Display Buffer
#define INPUT_BUFFER_SIZE   3 * IMAGE_WIDTH*IMAGE_HEIGHT //!< Input Buf Size
#define OUTPUT_BUFFER_SIZE  0x80000

#pragma DATA_SECTION( imgEncInpBuf, ".testCodecExtMemImg");
XDAS_Int8 imgEncInpBuf[INPUT_BUFFER_SIZE];

#pragma DATA_SECTION( imgEncOutBuf, ".testCodecExtMemImg");
XDAS_Int8 imgEncOutBuf[OUTPUT_BUFFER_SIZE];  /* 1 encoded frame */

#pragma DATA_SECTION( imgEncInst, ".testCodecExtMemImg");
#define MAX_ENC_INST_SIZE 5000
XDAS_Int8 imgEncInst[MAX_ENC_INST_SIZE];

/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Video Encode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Video Encode Call
*
* XDAS_Int32 tvct_test_entry_encode_xdm1p0(
*    const char *codecname        - name of the codec 
*    void       *codecParams      - # codec parameters
* )
*****************************************************************************/
extern cregister volatile unsigned int TSCL;
XDAS_Int32 tvct_test_entry_img_encode_xdm1p0 (const char *codecname, void *codecParams)
{
    IIMGENC1_Fxns            *IIMGENC1Fxns;
    IIMGENC1_Params          *staticParams;
    IIMGENC1_DynamicParams   *dynamicParams;
    IIMGENC1_Status          *status;
    IIMGENC1_InArgs          *inArgs;
    IIMGENC1_OutArgs         *outArgs;
    XDAS_UInt32				TotalBytes;
    Uint32	ii;
    /* Algorithm specific handle */
    IALG_Handle handle;
    /* Input/Output Buffer Descriptor variables */
    XDM1_BufDesc inputBufDesc; //xdm1.0
    XDM1_BufDesc outputBufDesc;
    /* Output Buffer Management specific variables                              */
    XDAS_UInt32 bytesGenerated;
    XDAS_UInt32 BytesRead;
    /* Other variables                                                          */
    XDAS_UInt32  ImageCount;
    XDAS_Int32   retVal;
    void *encFxns;
    imgEncoderAPI_t *encAPI;

    siuVct_DDR_Init();
    siuVct_FileIO_Start();

    encAPI = getImgEncoderAPI(codecname);

    if (encAPI == NULL)
    {
        printf("Encoder is not installed .. exiting \n");
        return -1;
    }

    if(encAPI->encInstanceSize >= MAX_ENC_INST_SIZE) 
    {
        printf("Encoder Instance Size Exceeds allocated value.. exiting \n");
        return -1;
    }
    encAPI->encInitInstance((void *)imgEncInst);

    /* Initialization of parameters needed for Algorithm Instance create */
    encAPI->encSetStaticParams((void *)imgEncInst, codecParams);

    /* Get XDM1.0 pointers from the codec */
    encFxns               =  encAPI->encGetFxns((void *)imgEncInst);
    inArgs                = (IIMGENC1_InArgs *)encAPI->encGetInArgs((void *)imgEncInst);
    outArgs               = (IIMGENC1_OutArgs *)encAPI->encGetOutArgs((void *)imgEncInst);
    staticParams          = (IIMGENC1_Params *)encAPI->encGetStaticParams((void *)imgEncInst);
    dynamicParams         = (IIMGENC1_DynamicParams *)encAPI->encGetDynamicParams((void *)imgEncInst);
    status                = (IIMGENC1_Status *)encAPI->encGetStatus((void *)imgEncInst);

    /* Initializing Variables */
    ImageCount                   = 0; /* Tracks the number of frames decoded */

    if ((handle =  (IALG_Handle)ALG_create (
        (IALG_Fxns *) encFxns,
        (IALG_Handle) NULL,
        (IALG_Params *) staticParams)) == NULL)
    {
        printf("\nFailed to Create Instance... Exiting for this configuration..");
        return -1;
    }

    /* Assigning Algorithm handle fxns field to ividdecfxns */
    IIMGENC1Fxns = (IIMGENC1_Fxns *)handle->fxns ;

    printf("\nAlgorithm Instance Creation Done...\n");

    /* Set run time dynamic parameters */
    encAPI->encSetDynamicParams((void *)imgEncInst, (IIMGENC1_Params *)staticParams);

    handle->fxns->algActivate(handle);

    /* Optional: Set Run time parameters in the Algorithm via control()     */
    IIMGENC1Fxns->control((IIMGENC1_Handle)handle, XDM_SETPARAMS,
        (IIMGENC1_DynamicParams *)dynamicParams, (IIMGENC1_Status *)status);

    /* Get Buffer information                                                 */
    IIMGENC1Fxns->control((IIMGENC1_Handle)handle,
        XDM_GETBUFINFO,
        (IIMGENC1_DynamicParams *)dynamicParams,
        (IIMGENC1_Status *)status);

     /* DeActivate the Algorithm                                               */
    handle->fxns->algDeactivate(handle);

    /*Fill up the buffers as required by algorithm                            */
    inputBufDesc.numBufs          = status->bufInfo.minNumInBufs ;
    inputBufDesc.descs[0].buf     = imgEncInpBuf;
    inputBufDesc.descs[0].bufSize = status->bufInfo.minInBufSize[0];

    for(ii=0; ii< (status->bufInfo.minNumInBufs - 1);ii++ )
    {
        inputBufDesc.descs[ii+1].buf = inputBufDesc.descs[ii].buf +
            status->bufInfo.minInBufSize[ii];
        inputBufDesc.descs[ii +1].bufSize =
            status->bufInfo.minInBufSize[ii +1];
    }

    outputBufDesc.numBufs          = status->bufInfo.minNumOutBufs ;
    outputBufDesc.descs[0].buf     = imgEncOutBuf;
    outputBufDesc.descs[0].bufSize = status->bufInfo.minOutBufSize[0];
    if(outputBufDesc.descs[0].bufSize > OUTPUT_BUFFER_SIZE)
    {
        outputBufDesc.descs[0].bufSize = OUTPUT_BUFFER_SIZE;
    }

    for(ii=0; ii< (status->bufInfo.minNumOutBufs-1); ii++ )
    {
        if(outputBufDesc.descs[0].bufSize > OUTPUT_BUFFER_SIZE)
        {
            exit(0);
        }

        outputBufDesc.descs[ii+1].buf = outputBufDesc.descs[ii].buf +
            status->bufInfo.minOutBufSize[ii];
        outputBufDesc.descs[ii+1].bufSize =
            status->bufInfo.minOutBufSize[ii+1];
    }

    /* Assigning Algorithm handle fxns field to IIMGENC1fxns                   */
    IIMGENC1Fxns = (IIMGENC1_Fxns *)handle->fxns;

    /* Resetting bytesGenerated variable                                      */
    bytesGenerated = 0;
    BytesRead = 0;

    for(ii=0;ii<inputBufDesc.numBufs;ii++)
    {
        BytesRead = siuVct_ReadInputData(inputBufDesc.descs[ii].buf,inputBufDesc.descs[ii].bufSize, BytesRead);
    }

    /* Activate the Algorithm                                               */
    handle->fxns->algActivate(handle);

    /* Optional: Set Run time parameters in the Algorithm via control()     */
    IIMGENC1Fxns->control((IIMGENC1_Handle)handle, XDM_SETPARAMS,
        (IIMGENC1_DynamicParams *)dynamicParams, (IIMGENC1_Status *)status);

    TotalBytes = 0;

    printf(" \n First Process inArgs->size=%d\n",inArgs->size);

    /* Basic Algorithm process() call */
    retVal = IIMGENC1Fxns->process((IIMGENC1_Handle)handle,
        (XDM1_BufDesc *)&inputBufDesc,
        (XDM1_BufDesc *)&outputBufDesc,
        (IIMGENC1_InArgs *)inArgs,
        (IIMGENC1_OutArgs *)outArgs);

    TotalBytes +=outArgs->bytesGenerated;

    if(retVal == XDM_EFAIL)
    {
        printf("\n Process function returned an Error...  ");
    }

    /* Optional: Set Run time parameters in the Algorithm via control()     */
    IIMGENC1Fxns->control((IIMGENC1_Handle)handle, XDM_GETSTATUS,
        (IIMGENC1_DynamicParams *)dynamicParams, (IIMGENC1_Status *)status);

    if(dynamicParams->numAU)
    {
        siuVct_WriteOutputData((XDAS_UInt8 *)outputBufDesc.descs[0].buf,outArgs->bytesGenerated);
        TotalBytes=0;
    }

    while (outArgs->currentAU < status->totalAU)
    {
        printf(" \n Second Process inArgs->size=%d\n",inArgs->size);
        /* Basic Algorithm process() call */
        retVal = IIMGENC1Fxns->process((IIMGENC1_Handle)handle,
            (XDM1_BufDesc *)&inputBufDesc,
            (XDM1_BufDesc *)&outputBufDesc,
            (IIMGENC1_InArgs *)inArgs,
            (IIMGENC1_OutArgs *)outArgs);

        TotalBytes +=outArgs->bytesGenerated;

        if(retVal == XDM_EFAIL)
        {
            printf("\n Process function returned an Error...  ");
            break; /* Error Condition: Application may want to break off    */
        }

        if(dynamicParams->numAU)
        {
            siuVct_WriteOutputData((XDAS_UInt8 *)outputBufDesc.descs[0].buf,outArgs->bytesGenerated);
            TotalBytes=0;
        }
    }

    /* Optional: Read status via control()     */
    IIMGENC1Fxns->control((IIMGENC1_Handle)handle,
        XDM_GETSTATUS,
        (IIMGENC1_DynamicParams *)dynamicParams,
        (IIMGENC1_Status *)status);


    /* DeActivate the Algorithm                                             */
    handle->fxns->algDeactivate(handle);

    for(ii=0; ii < 1 ; ii++)
    {
        if(outArgs->bytesGenerated != 0 )
        {
            siuVct_WriteOutputData((XDAS_UInt8 *)outputBufDesc.descs[ii].buf,outArgs->bytesGenerated);
            printf("Num bytes generated = %d\n",outArgs->bytesGenerated);
        }
    }

    ImageCount++;
    bytesGenerated += outArgs->bytesGenerated ;

    /* Delete the Algorithm instance object specified by handle */
    ALG_delete (handle);

    if(ImageCount == 0)
    {
        ImageCount = 1; /* To avoid division with zero */
    }

    siuVct_FileIO_Stop();

    printf("\n --------------  SUMMARY --------------------\n");
    printf("\t Total number of Scans              = %d\n",
        (XDAS_UInt32)ImageCount);

    printf (" --------------    END   --------------------\n");

    printf("\n End of execution\n");

    return(0);
} /* tvct_test_entry_img_encode_xdm1p0 */

/* nothing past this point */
