/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   siuVctEncode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "siuVctCodecAPI.h"
#include "siuVctBufferManager.h"
#include "siuVctParse.h"
#include "siuVctTftp.h"
#include "ividmc/siuVidMc.h"
#include "common/siuloc.h"
#include "osal/siuFcOsal.h"

#include "siuVctFileIO.h"
#include "targethw.h"
#include <ti/mas/tftp/tftp.h>

#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Log.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Memory.h>

#include <ti/xdais/ires.h>
extern IALG_Handle ALG_create(IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params);
extern Void ALG_delete(IALG_Handle alg);

XDAS_Void siuVctDecodeXDM0p9_WriteYUVData(XDM_BufDesc * outputBufDesc,IVIDDEC_OutArgs *outArgs)
{
    XDAS_UInt8 *s;
    XDAS_Int32 i;

    for(i = 0; i < outArgs->displayBufs.numBufs; i++)
    {
        s = (XDAS_UInt8 *)outArgs->displayBufs.bufs[i];
        siuVct_WriteOutputData(s, outArgs->displayBufs.bufSizes[i]);
    }

    return;
}


//#pragma DATA_SECTION( inpBuf1, ".codec_input");
extern XDAS_Int8 inpBuf1[1447936]; // Reuse buffers from XDM 1.0 definition

/* Output buffer allocation */
#pragma    DATA_SECTION(outputData, ".codec_output")
XDAS_Int8  outputData[3133440]; //1920*1088*1.5

extern int CORE_TEAM_MAPPING[];
#pragma    DATA_SECTION(multiCoreSync2, ".multiCoreSync")
/*---------------------------------------------------------------------------*/
/*  Buffer for multicore synchronozation in testapplication                  */
/*---------------------------------------------------------------------------*/
XDAS_Int8  multiCoreSync2[IVIDMC_NUM_BARRIERS];

#define STRING_SIZE            256
#define NUM_ALGS                 1      /* Number of Algorithms              */

/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Video Decode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Video Decode Call
*
* XDAS_Int32 tvct_test_entry_decode_xdm1p0(
*    const char *codecname        - name of the codec 
*    XDAS_Int8 *inputData         - input  data ptr in DDR
*    XDAS_Int8 *outputData        - output data ptr in DDR
*    XDAS_UInt32 framesToEncode   - # of frames to encode
*    XDAS_UInt32 nCores           - # of cores employed in 1 Decode
* )
*****************************************************************************/
#pragma DATA_SECTION( decXDM0p9Inst, ".testCodecExtMem");
#define MAX_DEC_INST_SIZE 5000
XDAS_Int8 decXDM0p9Inst[MAX_DEC_INST_SIZE];
extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int DNUM;
XDAS_Int32 tvct_test_entry_decode_xdm0p9 (const char *codecname, XDAS_UInt32 framesToDecode, XDAS_UInt32 nCores, void *codecParams)
{
    /* Input/Output  buffers  and their sizes                                  */
    XDAS_UInt32  ii;
    IVIDDEC_Fxns              *ividDecFxns;
    IVIDDEC_Status            *status;
    IVIDDEC_InArgs            *inArgs;
    IVIDDEC_OutArgs           *outArgs;
    IVIDDEC_Params            *staticParams;
    IVIDDEC_DynamicParams     *dynamicParams;
    IVIDMC_t                   mcViddecParams;
    /* Algorithm specific handle                                               */
    IALG_Handle handle;
    /* Input/Output Buffer Descriptors                                         */
    XDM_BufDesc inputBufDesc;
    XDM_BufDesc outputBufDesc;
    XDAS_Int8*  pInputBuf   [XDM_MAX_IO_BUFFERS];
    XDAS_Int8*  pOutputBuf  [XDM_MAX_IO_BUFFERS];
    XDAS_Int32  inBufSize   [XDM_MAX_IO_BUFFERS];
    XDAS_Int32  outBufSize  [XDM_MAX_IO_BUFFERS]; 

    /* Input Buffer Management specific variables                              */
    XDAS_UInt32 bytesConsumed;
    XDAS_UInt32 BytesRead=0;

    /* Other variables                                                         */
    XDAS_Int32  frameCount, retVal, displayFrame;
    vidDecoderAPI_t *decAPI;
    void *decFxns;
    int jj;
    IRES_Fxns * resFxns;

    siuVct_DDR_Init();

    if(DNUM==CORE_TEAM_MAPPING[0])
      siuVct_FileIO_Start();

    decAPI = getVidDecoderAPI(codecname);

    if (decAPI == NULL){
       printf("Decoder is not installed .. exiting \n");
       return -1;
    }

    if(decAPI->decInstanceSize >= MAX_DEC_INST_SIZE) {
       printf("Decoder Instance Size Exceeds allocated value.. exiting \n");
       return -1;
    }
    decAPI->decInitInstance((void *)decXDM0p9Inst);

    /* Initialization of multi-core params. Do this before decSetStaticParams
        so that it can be populated to static params */
    mcViddecParams.swbarr        = siuVidMc_Swbarr;
    mcViddecParams.shmmap        = siuVidMc_Map_Shmem;
    mcViddecParams.shmmunmap     = siuVidMc_Unmap_Shmem;
    mcViddecParams.shmmap_sync   = siuVidMc_Shmem_Sync;
    mcViddecParams.Lock          = NULL;
    mcViddecParams.UnLock        = NULL;
    mcViddecParams.ncores        = nCores;
    mcViddecParams.core_task_ID  = (DNUM==CORE_TEAM_MAPPING[0])?IVIDMC_TASK_MASTER:IVIDMC_TASK_SLAVE;
    mcViddecParams.coreID        = DNUM;
    siuVidMc_Init_ShmemTab(DNUM);
    decAPI->decSetMcParams((void *)decXDM0p9Inst, &mcViddecParams);

    /* Initialization of parameters needed for Algorithm Instance create */
    decAPI->decSetStaticParams((void *)decXDM0p9Inst, codecParams);
   
    /* Get XDM pointers from the codec */
    decFxns               =  decAPI->decGetFxns((void *)decXDM0p9Inst);
    inArgs                = (IVIDDEC_InArgs *)decAPI->decGetInArgs((void *)decXDM0p9Inst);
    outArgs               = (IVIDDEC_OutArgs *)decAPI->decGetOutArgs((void *)decXDM0p9Inst);
    staticParams          = (IVIDDEC_Params *)decAPI->decGetStaticParams((void *)decXDM0p9Inst);
    dynamicParams         = (IVIDDEC_DynamicParams *)decAPI->decGetDynamicParams((void *)decXDM0p9Inst);
    status                = (IVIDDEC_Status *)decAPI->decGetStatus((void *)decXDM0p9Inst);

    /* Initializing Variables                                                */
    frameCount                   = 0; 

    /*-----------------------------------------------------------------------*/
    /*  End of stream indicator to slave. if multiCoreSync2[0] is set         */
    /*  to 1 then slave exits the frame processing loop                      */
    /*-----------------------------------------------------------------------*/
    if(DNUM==CORE_TEAM_MAPPING[0])
        multiCoreSync2[0] = 0; 

    siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR12,nCores);

    /* Create the Algorithm object (instance)                                */
    printf("\nCreating Algorithm Instance...");

    if ((handle =  (IALG_Handle)ALG_create (
        (IALG_Fxns *) decFxns, 
        (IALG_Handle) NULL, 
        (IALG_Params *) staticParams)) == NULL)
    {
        printf( "\nFailed to Create Instance... Exiting for this configuration..");
        return(-1);
    }
    printf("\nAlgorithm Instance Creation Done...\n");
    handle->fxns->algActivate((IALG_Handle)handle);

    /* Assign resources to the algorithm */
    if(siuContext.rmanConfigFlag) {
      resFxns = decAPI->decGetIresFxns();
      if (siu_osal_fc_assign_resources((void *)handle, (void *)resFxns) == FALSE) {
          printf("Assign Resource Failed \n");
          return (-1);
      }
      siuContext.rmanConfigFlag = 0;
    }

    /* Assigning Algorithm handle fxns field to ividDecfxns                  */
    ividDecFxns = (IVIDDEC_Fxns *)handle->fxns;

    /* Set run time dynamic parameters */
    decAPI->decSetDynamicParams((void *)decXDM0p9Inst, (void *)staticParams);

    /*-----------------------------------------------------------------------*/
    /*  The XDM_RESET call gives resets all the internal data structures and */
    /*  context for the new stream that is to be decoded.                    */
    /*-----------------------------------------------------------------------*/
    ividDecFxns->control((IVIDDEC_Handle)handle, XDM_RESET,     dynamicParams, status);
    ividDecFxns->control((IVIDDEC_Handle)handle, XDM_SETPARAMS, dynamicParams, status);

    /*-----------------------------------------------------------------------*/
    /*  The GETBUFINFO call gives information for number and size of input   */
    /*  and output buffers. For this first call, the actual width and height */
    /*  of the picture is not known hence, this control call returns the out */
    /*  buffer sizes based on the maxHeight and maxWidth provided as part    */
    /*  of the input params from testparams file.                            */
    /*-----------------------------------------------------------------------*/
    ividDecFxns->control((IVIDDEC_Handle)handle, XDM_GETBUFINFO, dynamicParams, status);
    handle->fxns->algDeactivate((IALG_Handle)handle);

    /*-----------------------------------------------------------------------*/
    /*  Initialize the input buffer properties as required by algorithm      */
    /*  based on info received by preceding GETBUFINFO call. First init the  */
    /*  number of input bufs.                                                */
    /*-----------------------------------------------------------------------*/

    inputBufDesc.numBufs          = status->bufInfo.minNumInBufs ;
    /* set space for buffer descriptors */
    inputBufDesc.bufs        = (XDAS_Int8 **)pInputBuf;
    inputBufDesc.bufSizes    = (XDAS_Int32 *)inBufSize ;
    inputBufDesc.bufSizes[0] = status->bufInfo.minInBufSize[0];

    for(ii=0; ii<(status->bufInfo.minNumInBufs-1);ii++ )
    {
       inputBufDesc.bufs[ii+1]      = inputBufDesc.bufs[ii] + status->bufInfo.minInBufSize[ii];
       inputBufDesc.bufSizes[ii +1] = status->bufInfo.minInBufSize[ii +1];
    }

    /*-----------------------------------------------------------------------*/
    /*  Initialize the output buffer properties as required by algorithm     */
    /*  based on info received by preceding GETBUFINFO call.                 */
    /*-----------------------------------------------------------------------*/
    outputBufDesc.numBufs     = status->bufInfo.minNumOutBufs;
    outputBufDesc.bufs        = (XDAS_Int8 **)pOutputBuf;
    outputBufDesc.bufSizes    = (XDAS_Int32 *)outBufSize ;
    for(ii=0; ii<(status->bufInfo.minNumOutBufs-1);ii++)
    {
      outputBufDesc.bufs[ii+1]     = outputBufDesc.bufs[ii] + status->bufInfo.minOutBufSize[ii];
      outputBufDesc.bufSizes[ii+1] = status->bufInfo.minOutBufSize[ii+1];
    }

    /* Resetting bytesGenerated variable                                     */
    bytesConsumed = 0;
    outArgs->bytesConsumed = 0;
    inArgs->numBytes = 0;
    displayFrame = 0;
    
    /* Do-While Loop for Decode Call                                         */
    do
    {
        if(DNUM==CORE_TEAM_MAPPING[0])
        {
            /* Assign the number of bytes available                                */
            inputBufDesc.bufs[0]  = (XDAS_Int8 *)((XDAS_Int32) inpBuf1);
            outputBufDesc.bufs[0] = outputData;

            BytesRead = siuVct_ReadInputData(inputBufDesc.bufs[0],inputBufDesc.bufSizes[0], outArgs->bytesConsumed);
            
            /* Check for Break off condition                                       */
            if (BytesRead <=0)
            {
                printf("\n Bitstream Ended...  ");
                multiCoreSync2[0] = 1; 
                siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR13,nCores);
                break; /* Bitstream ended: Break off the while loop                  */
            }
            inArgs->numBytes =  BytesRead;
            inArgs->inputID  = 0;
        }
            
        siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR13,nCores);
           
        if(DNUM!=CORE_TEAM_MAPPING[0])
        {
            /*-------------------------------------------------------------------------*/
            /* Slave core - Check for end of frames                                    */
            /*-------------------------------------------------------------------------*/
            if(multiCoreSync2[0])
                break;
        }

        handle->fxns->algActivate((IALG_Handle)handle);
        
        /* Optional: Set Run time parameters in the Algorithm via control()    */
        ividDecFxns->control((IVIDDEC_Handle)handle, XDM_SETPARAMS, dynamicParams, status);     

        /* Basic Algorithm process() call                                      */
        retVal = ividDecFxns->process((IVIDDEC_Handle)handle, 
            (XDM_BufDesc *)&inputBufDesc,
            (XDM_BufDesc *)&outputBufDesc,
            inArgs,
            outArgs);

        handle->fxns->algDeactivate((IALG_Handle)handle);

        if(retVal == XDM_EFAIL)
        {
          printf("\n Process function returned an Error for stream...\n");
          if(DNUM==CORE_TEAM_MAPPING[0])
            siuVct_FileIO_Stop();
          return(-1);
        }

        if(DNUM==CORE_TEAM_MAPPING[0])
        {
            handle->fxns->algActivate((IALG_Handle)handle);

        
            /* Optional: Read status via control()                                 */
            ividDecFxns->control((IVIDDEC_Handle)handle, XDM_GETSTATUS, dynamicParams, status);

            handle->fxns->algDeactivate((IALG_Handle)handle);
            /* Check for frame ready via display buffer pointers                   */
            jj = 0;
            while((outArgs->displayBufs.bufs[0] != NULL) && (displayFrame < framesToDecode))
            {
                printf("\n Decoded Frame # %d  ",displayFrame);
                /* Write the output frames                     */
                siuVctDecodeXDM0p9_WriteYUVData(&outputBufDesc, outArgs);
                displayFrame++;
                jj++;
            }

            frameCount++;

            if (displayFrame >= framesToDecode)
            {
                printf("\n Specified number of Frames Decoded...  ");
                multiCoreSync2[0] = 1;

                //break; /* Bitstream ended: Break off the while loop                  */
            }

            bytesConsumed          += outArgs->bytesConsumed;
            
        }
        siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR14,nCores);
        if(multiCoreSync2[0])
        { 
            break;
        }

    }  while(1);  /* end of Do-While loop                                    */
    


    if(DNUM==CORE_TEAM_MAPPING[0])
    {
        if((displayFrame < framesToDecode) && (displayFrame < frameCount) )
        {
            //inArgs->numBytes        = 0; 
            handle->fxns->algActivate((IALG_Handle)handle);

            /* Add logic for flushing display buffers */
            ividDecFxns->control((IVIDDEC_Handle)handle,XDM_FLUSH, dynamicParams, status);

            handle->fxns->algDeactivate((IALG_Handle)handle);

            handle->fxns->algActivate((IALG_Handle)handle);

            /* Basic Algorithm process() call                                      */
            retVal = ividDecFxns->process((IVIDDEC_Handle)handle, 
                (XDM_BufDesc *)&inputBufDesc,
                (XDM_BufDesc *)&outputBufDesc,
                inArgs,
                outArgs);
            handle->fxns->algDeactivate((IALG_Handle)handle);

            //if(retVal)
              //break;

            /* Write the output frames in the display order                    */
            jj = 0;
            while((outArgs->displayBufs.bufs[0] != NULL) && (displayFrame < framesToDecode))
            {
                printf("\n Decoded Frame # %d  ",displayFrame);
                siuVctDecodeXDM0p9_WriteYUVData(&outputBufDesc, outArgs);
                displayFrame++;
                jj++;
            }
        }

    }

    siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR15,nCores);

    /* Delete the Algorithm instance object specified by handle */
    ALG_delete (handle);

    if(frameCount == 0) 
    { 
        frameCount = 1; /* To avoid division with zero */
    }


    if(DNUM==CORE_TEAM_MAPPING[0]) {
      siuVct_FileIO_Stop();

      printf("\n --------------  SUMMARY --------------------\n");
      printf("\t Total number of Frames              = %d\n",
          (XDAS_UInt32)frameCount);
      printf("\t Bit Rate at 30 frames/Sec           = %d Kbps\n",
          (XDAS_UInt32)(((bytesConsumed*8*30)/frameCount)/1000));
      printf("\t Width and Height                    = %d, %d \n",
          (XDAS_UInt32)status->outputWidth, (XDAS_UInt32)status->outputHeight);

      printf (" --------------    END   --------------------\n");
    } else {
      printf("\nSee master core output for summary.\n");
    }    

    printf("\n End of execution\n");

    return(0);

} /* tvct_test_entry_decode_xdm1p0 */

/* nothing past this point */
