/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   siuVctEncode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "siuVctCodecAPI.h"
#include "siuVctBufferManager.h"
#include "siuVctParse.h"
#include "siuVctTftp.h"
#include "ividmc/siuVidMc.h"
#include "common/siuloc.h"
#include "osal/siuFcOsal.h"

#include "siuVctFileIO.h"
#include "targethw.h"
#include <ti/mas/tftp/tftp.h>

#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Log.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Memory.h>

#include <ti/xdais/ires.h>
extern IALG_Handle ALG_create(IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params);
extern Void ALG_delete(IALG_Handle alg);


XDAS_Void siuVctDecode_WriteYUVData(IVIDEO1_BufDesc *displayBufs)
{
  XDAS_Int32 YWidth, YHeight;
  XDAS_Int32 row,oddwidth=0,oddheight=0;
  XDAS_Int8 *output;

  siu_invalidate_cache();
  
  output  = &displayBufs->bufDesc[0].buf[0];
  YWidth    = displayBufs->frameWidth;
  YHeight    = displayBufs->frameHeight;
    
  if(displayBufs->chromaFormat == 4)
  {
    for ( row=0; row < YHeight; row++ )
    {
      siuVct_WriteOutputData(output, YWidth*2);
      output += displayBufs->framePitch;
    }
  } 
  else if (displayBufs->chromaFormat == 1)
  {
    for ( row=0; row < YHeight; row++ )
    {
      siuVct_WriteOutputData(output, YWidth);
      output += displayBufs->framePitch;
    }

    output      = &displayBufs->bufDesc[1].buf[0];

    for ( row=0; row < YHeight>>1; row++ )
    {
      siuVct_WriteOutputData(output, YWidth>>1);

      if(oddwidth) {
        siuVct_WriteOutputData(output, (YWidth>>1)-1);
      }
      output +=    (displayBufs->framePitch>> 1);
    }

    if(oddheight)
    {
      output -=    (displayBufs->framePitch>> 1);

      siuVct_WriteOutputData(output, YWidth>>1);

      if(oddwidth) {
        siuVct_WriteOutputData(output, (YWidth>>1)-1);
      }
    }

    output     = &displayBufs->bufDesc[2].buf[0];

    for ( row=0; row < YHeight>>1; row++ )
    {
      siuVct_WriteOutputData(output, YWidth>>1);

      if(oddwidth) {
        siuVct_WriteOutputData(output, (YWidth>>1)-1);
      }
      output +=    (displayBufs->framePitch >> 1);
    }

    if(oddheight)
    {
      output -=    (displayBufs->framePitch>> 1);

      siuVct_WriteOutputData(output, YWidth>>1);

      if(oddwidth) {
        siuVct_WriteOutputData(output, (YWidth>>1)-1);
      }
    }
  }
  else if (displayBufs->chromaFormat == 2)
  {
    for ( row=0; row < YHeight; row++ )
    {
      siuVct_WriteOutputData(output, YWidth);
      output += displayBufs->framePitch;
    }

    output      = &displayBufs->bufDesc[1].buf[0];
    siuVct_WriteOutputData(output,displayBufs->bufDesc[1].bufSize);
    output     = &displayBufs->bufDesc[2].buf[0];	
    siuVct_WriteOutputData(output,displayBufs->bufDesc[2].bufSize);
  
  }
  else if (displayBufs->chromaFormat == 6)
  {
    for ( row=0; row < YHeight; row++ )
    {
      siuVct_WriteOutputData(output, YWidth);
      output += displayBufs->framePitch;
    }
    output      = &displayBufs->bufDesc[1].buf[0];
    siuVct_WriteOutputData(output,displayBufs->bufDesc[1].bufSize);
    output     = &displayBufs->bufDesc[2].buf[0];	
    siuVct_WriteOutputData(output,displayBufs->bufDesc[2].bufSize);
  
  }
  return;
}


#pragma DATA_SECTION( inpBuf1, ".codec_input");
XDAS_Int8 inpBuf1[3447936];

extern int CORE_TEAM_MAPPING[];
#pragma    DATA_SECTION(multiCoreSync, ".multiCoreSync")
/*---------------------------------------------------------------------------*/
/*  Buffer for multicore synchronozation in testapplication                  */
/*---------------------------------------------------------------------------*/
XDAS_Int8  multiCoreSync[IVIDMC_NUM_BARRIERS];

#define STRING_SIZE            256
#define NUM_ALGS                 1      /* Number of Algorithms              */

/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Video Decode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Video Decode Call
*
* XDAS_Int32 tvct_test_entry_decode_xdm1p0(
*    const char *codecname        - name of the codec 
*    XDAS_Int8 *inputData         - input  data ptr in DDR
*    XDAS_Int8 *outputData        - output data ptr in DDR
*    XDAS_UInt32 framesToEncode   - # of frames to encode
*    XDAS_UInt32 nCores           - # of cores employed in 1 Decode
* )
*****************************************************************************/
#pragma DATA_SECTION( encInst, ".testCodecExtMem");
#define MAX_DEC_INST_SIZE 5000
XDAS_Int8 decInst[MAX_DEC_INST_SIZE];
extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int DNUM;
XDAS_Int32 tvct_test_entry_decode_xdm1p0 (const char *codecname, XDAS_UInt32 framesToDecode, XDAS_UInt32 nCores, void *codecParams)
{
    /* Input/Output  buffers  and their sizes                                  */
    XDAS_UInt32  ii;
    IVIDDEC2_Fxns *ividDecFxns;
    IVIDDEC2_Status            *status;
    IVIDDEC2_InArgs            *inArgs;
    IVIDDEC2_OutArgs           *outArgs;
    IVIDDEC2_Params            *staticParams;
    IVIDDEC2_DynamicParams     *dynamicParams;
    IVIDMC_t                    mcViddecParams;
    /* Algorithm specific handle                                               */
    IALG_Handle handle;
    /* Input/Output Buffer Descriptors                                         */
    XDM1_BufDesc inputBufDesc;
    XDM_BufDesc outputBufDesc;

    /*-------------------------------------------------------------------------*/
    /*  Handle to a buffer element allocated by the buffer manager module      */
    /*-------------------------------------------------------------------------*/
    BUFFMGR_buffEleHandle buffEle;
    /* Input Buffer Management specific variables                              */
    XDAS_UInt32 bytesConsumed,ret_val,first_time;
    XDAS_UInt32 BytesRead=0;
    /* Other variables                                                         */
    XDAS_Int32  frameCount, retVal, displayFrame;
    vidDecoderAPI_t *decAPI;
    void *decFxns;
    int jj;
    IRES_Fxns * resFxns;

    siuVct_DDR_Init();

    if(DNUM==CORE_TEAM_MAPPING[0])
      siuVct_FileIO_Start();

    decAPI = getVidDecoderAPI(codecname);

    if (decAPI == NULL){
       printf("Decoder is not installed .. exiting \n");
       return -1;
    }

    if(decAPI->decInstanceSize >= MAX_DEC_INST_SIZE) {
       printf("Decoder Instance Size Exceeds allocated value.. exiting \n");
       return -1;
    }
    decAPI->decInitInstance((void *)decInst);

    /* Initialization of multi-core params. Do this before decSetStaticParams
        so that it can be populated to static params */
    mcViddecParams.swbarr        = siuVidMc_Swbarr;
    mcViddecParams.shmmap        = siuVidMc_Map_Shmem;
    mcViddecParams.shmmunmap     = siuVidMc_Unmap_Shmem;
    mcViddecParams.shmmap_sync   = siuVidMc_Shmem_Sync;
    mcViddecParams.Lock          = NULL;
    mcViddecParams.UnLock        = NULL;
    mcViddecParams.ncores        = nCores;
    if(!(strcmp(codecname, "VC1DEC")))
    {
      if(DNUM==CORE_TEAM_MAPPING[0])
      {
        mcViddecParams.core_task_ID = IVIDMC_TASK_MASTER;
      }
      if(DNUM==CORE_TEAM_MAPPING[1])
      {
        mcViddecParams.core_task_ID = IVIDMC_TASK_SLAVE;
      }
      if(DNUM==CORE_TEAM_MAPPING[2])
      {
        mcViddecParams.core_task_ID = IVIDMC_TASK_SLAVE_1;
      }
    }
    else
    {
      mcViddecParams.core_task_ID  = (DNUM==CORE_TEAM_MAPPING[0])?IVIDMC_TASK_MASTER:IVIDMC_TASK_SLAVE;
    }
    mcViddecParams.coreID        = DNUM;
    siuVidMc_Init_ShmemTab(DNUM);
    decAPI->decSetMcParams((void *)decInst, &mcViddecParams);

    /* Initialization of parameters needed for Algorithm Instance create */
    decAPI->decSetStaticParams((void *)decInst, codecParams);
   
    /* Get XDM1.0 pointers from the codec */
    decFxns               =  decAPI->decGetFxns((void *)decInst);
    inArgs                = (IVIDDEC2_InArgs *)decAPI->decGetInArgs((void *)decInst);
    outArgs               = (IVIDDEC2_OutArgs *)decAPI->decGetOutArgs((void *)decInst);
    staticParams          = (IVIDDEC2_Params *)decAPI->decGetStaticParams((void *)decInst);
    dynamicParams         = (IVIDDEC2_DynamicParams *)decAPI->decGetDynamicParams((void *)decInst);
    status                = (IVIDDEC2_Status *)decAPI->decGetStatus((void *)decInst);

    /* Initializing Variables                                                */
    frameCount                   = 0; 

    /*-----------------------------------------------------------------------*/
    /*  End of stream indicator to slave. if multiCoreSync[0] is set         */
    /*  to 1 then slave exits the frame processing loop                      */
    /*-----------------------------------------------------------------------*/
    if(DNUM==CORE_TEAM_MAPPING[0])
        multiCoreSync[0] = 0; 

    siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR12,nCores);

    /* Create the Algorithm object (instance)                                */
    printf("\nCreating Algorithm Instance...");

    if ((handle =  (IALG_Handle)ALG_create (
        (IALG_Fxns *) decFxns, 
        (IALG_Handle) NULL, 
        (IALG_Params *) staticParams)) == NULL)
    {
        printf( "\nFailed to Create Instance... Exiting for this configuration..");
        return(-1);
    }
    printf("\nAlgorithm Instance Creation Done...\n");
    handle->fxns->algActivate((IALG_Handle)handle);

    /* Assign resources to the algorithm */
    if(siuContext.rmanConfigFlag) {
      resFxns = decAPI->decGetIresFxns();
      if (siu_osal_fc_assign_resources((void *)handle, (void *)resFxns) == FALSE) {
          printf("Assign Resource Failed \n");
          return (-1);
      }
      siuContext.rmanConfigFlag = 0;
    }

    /*-----------------------------------------------------------------------*/
    /*  The outBufsInUseFlag tells us whether the previous input buffer given*/
    /*  by the application to the algorithm is still in use or not. Since    */
    /*  this is before the first decode call, assign this flag to 0. The     */
    /*  algorithm will take care to initialize this flag appropriately from  */
    /*  hereon for the current configuration.                                */
    /*-----------------------------------------------------------------------*/
    outArgs->outBufsInUseFlag     = 0;

    /* Assigning Algorithm handle fxns field to ividDecfxns                  */
    ividDecFxns = (IVIDDEC2_Fxns *)handle->fxns;

    /* Set run time dynamic parameters */
    decAPI->decSetDynamicParams((void *)decInst, (IVIDDEC2_Params *)staticParams);

    /*-----------------------------------------------------------------------*/
    /*  The XDM_RESET call gives resets all the internal data structures and */
    /*  context for the new stream that is to be decoded.                    */
    /*-----------------------------------------------------------------------*/
    ividDecFxns->control((IVIDDEC2_Handle)handle, XDM_RESET,     dynamicParams, status);
    ividDecFxns->control((IVIDDEC2_Handle)handle, XDM_SETPARAMS, dynamicParams, status);

    /*-----------------------------------------------------------------------*/
    /*  The GETBUFINFO call gives information for number and size of input   */
    /*  and output buffers. For this first call, the actual width and height */
    /*  of the picture is not known hence, this control call returns the out */
    /*  buffer sizes based on the maxHeight and maxWidth provided as part    */
    /*  of the input params from testparams file.                            */
    /*-----------------------------------------------------------------------*/
    ividDecFxns->control((IVIDDEC2_Handle)handle, XDM_GETBUFINFO, dynamicParams, status);
    handle->fxns->algDeactivate((IALG_Handle)handle);
    /*-----------------------------------------------------------------------*/
    /*  The buffermanager init function call allocates the memory for the    */
    /*  number of buffers of the corresponding sizes as passed in arguments  */
    /*  to it. This function initializes the output/reference bufs based on  */
    /*  the outBuf parameters returned from the GETBUFINFO call above. This  */
    /*  call will allocate each of the numOutbufs memory equal to (specified */
    /*  bufsize). The initialization and allocation of buff ptr will take    */
    /*  place only for the first ref buffer. For the remaining ref buffers,  */
    /*  allocation will happen after the first decode call, when actual      */
    /*  height and width of picture are known.                               */
    /*  Note that the allocation of output/ref buffers will happen each time */
    /*  for the entire bunch of configuration sets provided in the testvecs  */
    /*  config file.                                                         */
    /*-----------------------------------------------------------------------*/
    if(DNUM==CORE_TEAM_MAPPING[0])
    {

    ret_val = BUFFMGR_Init(status->bufInfo.minNumOutBufs,
                           status->bufInfo.minOutBufSize);

    if (ret_val)
    {
        printf ("\nMemory could not get allocated for output buffers\n");
            multiCoreSync[0] = 1; 
            return -1;
        }
    }

    /*-----------------------------------------------------------------------*/
    /*  Initialize the input buffer properties as required by algorithm      */
    /*  based on info received by preceding GETBUFINFO call. First init the  */
    /*  number of input bufs.                                                */
    /*-----------------------------------------------------------------------*/

    inputBufDesc.numBufs          = status->bufInfo.minNumInBufs ;
    inputBufDesc.descs[0].bufSize = status->bufInfo.minInBufSize[0];

    for(ii=0; ii<(status->bufInfo.minNumInBufs-1);ii++ )
    {
        inputBufDesc.descs[ii+1].buf = inputBufDesc.descs[ii].buf +
            status->bufInfo.minInBufSize[ii];
        inputBufDesc.descs[ii+1].bufSize =
            status->bufInfo.minInBufSize[ii +1];
    }

    /*-----------------------------------------------------------------------*/
    /*  Initialize the output buffer properties as required by algorithm     */
    /*  based on info received by preceding GETBUFINFO call.                 */
    /*-----------------------------------------------------------------------*/
    outputBufDesc.numBufs = status->bufInfo.minNumOutBufs;

    /*-----------------------------------------------------------------------*/
    /*  Initialize the first time flag to 1 which will get used for the      */
    /*  condition check for reinitializing the buffers after the first       */
    /*  decode call.                                                         */
    /*-----------------------------------------------------------------------*/
    first_time = 1;

    /* set space for buffer descriptors                                      */

    inputBufDesc.descs[0].buf     = inpBuf1;

    /* Resetting bytesGenerated variable                                     */
    bytesConsumed = 0;
    outArgs->bytesConsumed = 0;
    inArgs->numBytes = 0;
    displayFrame = 0;
    
    /* Do-While Loop for Decode Call                                         */
    do
    {
        if(DNUM==CORE_TEAM_MAPPING[0])
        {
            /* Assign the number of bytes available                                */
            inputBufDesc.descs[0].buf   = (XDAS_Int8 *)((XDAS_Int32) inpBuf1);
            
            BytesRead = siuVct_ReadInputData(inputBufDesc.descs[0].buf,inputBufDesc.descs[0].bufSize, outArgs->bytesConsumed);
            
            /* Check for Break off condition                                       */
            if (BytesRead <=0)
            {
                printf("\n Bitstream Ended...  ");
                multiCoreSync[0] = 1; 
                siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR13,nCores);
                break; /* Bitstream ended: Break off the while loop                  */
            }
            inArgs->numBytes =  BytesRead;

            /*---------------------------------------------------------------------*/
            /*  Check for the value of the outBufsInUseFlag. If the previous       */
            /*  decode call was for the first field of an interlaced frame, the    */
            /*  previous outbuf supplied by algorithm may still be required for    */
            /*  decoding second field. In that case, the algorithm will not need   */
            /*  a new buffer from the application. If above is not the case, call  */
            /*  the getfreebuffer() function of buffermanager to get the buffer    */
            /*  element to be supplied to algorithm for the next decode call.      */
            /*---------------------------------------------------------------------*/
            if (!outArgs->outBufsInUseFlag)
            {
                buffEle = BUFFMGR_GetFreeBuffer();
            }

            /*---------------------------------------------------------------------*/
            /*  Copy the output buffer size values and pointers from the buffer    */
            /*  element returned by getfreebuffer() function. Initialize the       */
            /*  numBufs to 2 as numoutbufs is always 2 for luma and chroma buffers.*/
            /*---------------------------------------------------------------------*/
            outputBufDesc.bufSizes    = (XDAS_Int32 *) buffEle->bufSize;
            outputBufDesc.bufs        = (XDAS_Int8 **) buffEle->buf;

            /*---------------------------------------------------------------------*/
            /*  Initialize the input ID in input arguments to the bufferid of the  */
            /*  buffer element returned from getfreebuffer() function.             */
            /*---------------------------------------------------------------------*/
            inArgs->inputID           = buffEle->bufId;
        }
            
        siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR13,nCores);
           
        if(DNUM!=CORE_TEAM_MAPPING[0])
        {
            /*-------------------------------------------------------------------------*/
            /* Slave core - Check for end of frames                                    */
            /*-------------------------------------------------------------------------*/
            if(multiCoreSync[0])
                break;
        }

        handle->fxns->algActivate((IALG_Handle)handle);
        
        /* Optional: Set Run time parameters in the Algorithm via control()    */
        ividDecFxns->control((IVIDDEC2_Handle)handle, XDM_SETPARAMS, dynamicParams, status);     

        outArgs->displayBufs->frameWidth =0;
        outArgs->displayBufs->frameHeight =0;

        /* Basic Algorithm process() call                                      */
        retVal = ividDecFxns->process((IVIDDEC2_Handle)handle, 
            (XDM1_BufDesc *)&inputBufDesc,
            (XDM_BufDesc *)&outputBufDesc,
            inArgs,
            outArgs);

        handle->fxns->algDeactivate((IALG_Handle)handle);

        if(retVal == XDM_EFAIL)
        {
          printf("\n Process function returned an Error for stream...\n");
          if(DNUM==CORE_TEAM_MAPPING[0])
            siuVct_FileIO_Stop();
          return(-1);
        }

        if(DNUM==CORE_TEAM_MAPPING[0])
        {
            handle->fxns->algActivate((IALG_Handle)handle);

        
            /* Optional: Read status via control()                                 */
            ividDecFxns->control((IVIDDEC2_Handle)handle, XDM_GETSTATUS, dynamicParams, status);

            /*---------------------------------------------------------------------*/
            /*  If this is the first time in the stream, reinitialize all the      */
            /*  buffers based on the picture width and height.                     */
            /*---------------------------------------------------------------------*/
            if (first_time)
            {
                /*-----------------------------------------------------------------*/
                /*  The GETBUFINFO call will give information for number of        */
                /*  output buffers. This call is after the first decode call hence */
                /*  the outputbuffer sizes returned by this call is calculated     */
                /*  based on the actual pic width and height. Hence, there is need */
                /*  for buffer allocation and initializations which happens        */
                /*  immediately after this call.                                   */
                /*-----------------------------------------------------------------*/
                ividDecFxns->control((IVIDDEC2_Handle)handle,XDM_GETBUFINFO, dynamicParams, status);

                /*-----------------------------------------------------------------*/
                /*  Reinitialize the buffers using the actual buffer sizes for     */
                /*  luma and chroma. This call will allocate and initialize the    */
                /*  remaining buf ptrs for the max num of ref frames supplied by   */
                /*  the alg in maxNumDisplayBufs (max ref buffers required). The   */
                /*  first call to buffmgr_init() had allocated the first ref       */
                /*  buffers hence this reinit call will allocate and init the      */
                /*  remaining ref buffers' pointers. The 1st ref buf ptr will      */
                /*  continue to be the one which was allocated by the buffmgr_init */
                /*  call.                                                          */
                /*-----------------------------------------------------------------*/
                if(!(strcmp(codecname, "VC1DEC")))
                {
                  status->maxNumDisplayBufs=status->maxNumDisplayBufs+1;
                }
                BUFFMGR_ReInit (status->maxNumDisplayBufs,
                    status->bufInfo.minNumOutBufs, status->bufInfo.minOutBufSize);

                /*-----------------------------------------------------------------*/
                /*  Reset the flag first_time to 0 since the buffers are correctly */
                /*  reinitialized.                                                 */
                /*-----------------------------------------------------------------*/
                first_time = 0;

            }
            handle->fxns->algDeactivate((IALG_Handle)handle);
            /* Check for frame ready via display buffer pointers                   */
            jj = 0;
            while(outArgs->outputID[jj] && (displayFrame < framesToDecode))
            {
                printf("\n Decoded Frame # %d  ",displayFrame);
                /* Write the output frames                     */
                siuVctDecode_WriteYUVData(&outArgs->displayBufs[jj]);
                displayFrame++;
                jj++;
            }

            frameCount++;


            /*---------------------------------------------------------------------*/
            /*  Always release buffers - which are released from the algorithm     */
            /*  side   -back to the buffer manager. The freebufID array of outargs */
            /*  contains the sequence of bufferIds which need to be freed. This    */
            /*  gets populated by the algorithm. The following function will do    */
            /*  the job of freeing up the buffers.                                 */
            /*---------------------------------------------------------------------*/
            BUFFMGR_ReleaseBuffer((XDAS_UInt32 *)outArgs->freeBufID);

            if (displayFrame >= framesToDecode)
            {
                printf("\n Specified number of Frames Decoded...  ");
                multiCoreSync[0] = 1;

                //break; /* Bitstream ended: Break off the while loop                  */
            }

            bytesConsumed          += outArgs->bytesConsumed;
            
        }
        siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR14,nCores);
        if(multiCoreSync[0])
        { 
            break;
        }

    }  while(1);  /* end of Do-While loop                                    */
    


    if(DNUM==CORE_TEAM_MAPPING[0])
    {
        if((displayFrame < framesToDecode) && (displayFrame < frameCount) )
        {
            //inArgs->numBytes        = 0; 
            handle->fxns->algActivate((IALG_Handle)handle);

            /* Add logic for flushing display buffers */
            ividDecFxns->control((IVIDDEC2_Handle)handle,XDM_FLUSH, dynamicParams, status);

            handle->fxns->algDeactivate((IALG_Handle)handle);

            handle->fxns->algActivate((IALG_Handle)handle);

            /* Basic Algorithm process() call                                      */
            retVal = ividDecFxns->process((IVIDDEC2_Handle)handle, 
                (XDM1_BufDesc *)&inputBufDesc,
                (XDM_BufDesc *)&outputBufDesc,
                inArgs,
                outArgs);
            handle->fxns->algDeactivate((IALG_Handle)handle);

            BUFFMGR_ReleaseAllBuffers();

            //if(retVal)
              //break;

            /* Write the output frames in the display order                    */
            jj = 0;
            while(outArgs->outputID[jj] && (displayFrame < framesToDecode))
            {
                printf("\n Decoded Frame # %d  ",displayFrame);
                siuVctDecode_WriteYUVData(&outArgs->displayBufs[jj]);
                displayFrame++;
                jj++;
            }
        }
        /*-----------------------------------------------------------------------*/
        /*  The algorithm currently does not guarantee to set freebufid of       */
        /*  all buffers in use when the framecount exceeds framestoDecode.       */
        /*  This is managed for now by an application function. Note that        */
        /*  doing this has significance only when running multiple streams.      */
        /*-----------------------------------------------------------------------*/
        BUFFMGR_ReleaseAllBuffers();

        /*-----------------------------------------------------------------------*/
        /*  Free the buffers allocated for output/reference for all the streams  */
        /*  that have been executed.                                             */
        /*-----------------------------------------------------------------------*/
        BUFFMGR_DeInit(status->maxNumDisplayBufs, outputBufDesc.numBufs);
    }

    siuVidMc_Swbarr(DNUM,IVIDMC_SWBARR15,nCores);

    /* Delete the Algorithm instance object specified by handle */
    ALG_delete (handle);

    if(frameCount == 0) 
    { 
        frameCount = 1; /* To avoid division with zero */
    }


    if(DNUM==CORE_TEAM_MAPPING[0]) {
      siuVct_FileIO_Stop();

      printf("\n --------------  SUMMARY --------------------\n");
      printf("\t Total number of Frames              = %d\n",
          (XDAS_UInt32)frameCount);
      printf("\t Bit Rate at 30 frames/Sec           = %d Kbps\n",
          (XDAS_UInt32)(((bytesConsumed*8*30)/frameCount)/1000));
      printf("\t Width and Height                    = %d, %d \n",
          (XDAS_UInt32)status->outputWidth, (XDAS_UInt32)status->outputHeight);

      printf (" --------------    END   --------------------\n");
    } else {
      printf("\nSee master core output for summary.\n");
    }    

    printf("\n End of execution\n");

    return(0);

} /* tvct_test_entry_decode_xdm1p0 */

/* nothing past this point */
