/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   siuVctEncode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include "siuVctCodecAPI.h"
#include "siuVctFileIO.h"

extern IALG_Handle ALG_create(IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params);
extern Void ALG_delete(IALG_Handle alg);

/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for Video Encode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for Video Encode Call
*
* XDAS_Int32 tvct_test_entry_encode_xdm1p0(
*    const char *codecname        - name of the codec 
*    XDAS_Int8 *inputData         - input  data ptr in DDR
*    XDAS_Int8 *outputData        - output data ptr in DDR
*    XDAS_UInt32 framesToEncode   - # of frames to encode
* )
*****************************************************************************/
#pragma DATA_SECTION( inpBuf, ".testCodecExtMem");
XDAS_Int8 inpBuf[622080]; /*720*576*1.5*/

#pragma DATA_SECTION( outBuf, ".testCodecExtMem");
XDAS_Int8 outBuf[65536];  /* 1 encoded frame */

#pragma DATA_SECTION( encInst, ".testCodecExtMem");
#define MAX_ENC_INST_SIZE 5000
XDAS_Int8 encInst[MAX_ENC_INST_SIZE];
extern cregister volatile unsigned int TSCL;
XDAS_Int32 tvct_test_entry_encode_xdm1p0 (const char *codecname, XDAS_UInt32 framesToEncode, XDAS_UInt32 nCores, void *codecParams)
{
    /* Input/Output  buffers  and their sizes */
    XDAS_Int8*  pOutputBuf  [XDM_MAX_IO_BUFFERS];
    XDAS_Int32  outBufSize  [XDM_MAX_IO_BUFFERS];
    XDAS_Int32 ii;
    IVIDENC1_Fxns         *ividEncfxns;
    IVIDENC1_InArgs       *inArgs;
    IVIDENC1_OutArgs      *outArgs;
    IVIDENC1_Params       *staticParams;
    IVIDENC1_DynamicParams *dynamicParams;
    IVIDENC1_Status       *status;
    void *encFxns;
    XDAS_UInt32  BytesRead=0;
    XDAS_UInt32  TSCL_Begin, TSCL_End;
   
    /* Algorithm specific handle */
    IALG_Handle handle;

    /* Input/Output Buffer Descriptor variables */
    IVIDEO1_BufDescIn inputBufDesc; //xdm1.0
    XDM_BufDesc outputBufDesc;
    
    XDAS_UInt32 bytesConsumed;
    /* Output Buffer Management specific variables */
    XDAS_UInt32 bytesGenerated;

    /* Other variables */
    XDAS_UInt32  frameCount;
    XDAS_Int32   retVal;
    XDAS_Int32   RetVal=0;
    vidEncoderAPI_t *encAPI;

    siuVct_DDR_Init();

    siuVct_FileIO_Start();

    bytesConsumed = 0;

    encAPI = getVidEncoderAPI(codecname);

    if (encAPI == NULL){
       printf("Encoder is not installed .. exiting \n");
       return -1;
    }

    if(encAPI->encInstanceSize >= MAX_ENC_INST_SIZE) {
       printf("Encoder Instance Size Exceeds allocated value.. exiting \n");
       return -1;
    }
    encAPI->encInitInstance((void *)encInst);

    /* Initialization of parameters needed for Algorithm Instance create */
    encAPI->encSetStaticParams((void *)encInst, codecParams);

    /* Get XDM1.0 pointers from the codec */
    encFxns               =  encAPI->encGetFxns((void *)encInst);
    inArgs                = (IVIDENC1_InArgs *)encAPI->encGetInArgs((void *)encInst);
    outArgs               = (IVIDENC1_OutArgs *)encAPI->encGetOutArgs((void *)encInst);
    staticParams          = (IVIDENC1_Params *)encAPI->encGetStaticParams((void *)encInst);
    dynamicParams         = (IVIDENC1_DynamicParams *)encAPI->encGetDynamicParams((void *)encInst);
    status                = (IVIDENC1_Status *)encAPI->encGetStatus((void *)encInst);

    if (staticParams->inputChromaFormat == 1)
        inputBufDesc.framePitch = staticParams->maxWidth;
    else
        inputBufDesc.framePitch = staticParams->maxWidth << 1;

    /* Initializing Variables */
    frameCount                   = 0; /* Tracks the number of frames decoded */

    if ((handle =  (IALG_Handle)ALG_create (
        (IALG_Fxns *) encFxns, 
        (IALG_Handle) NULL, 
        (IALG_Params *) staticParams)) == NULL)
    {
        printf("\nFailed to Create Instance... Exiting for this configuration..");
        return -1;
    }

    /* Assigning Algorithm handle fxns field to ividdecfxns */
    ividEncfxns = (IVIDENC1_Fxns *)handle->fxns ;

    printf("\nAlgorithm Instance Creation Done...\n");

    /* Set run time dynamic parameters */
    encAPI->encSetDynamicParams((void *)encInst, staticParams);

    /* set space for buffer descriptors                                       */
    outputBufDesc.bufs = (XDAS_Int8 **)pOutputBuf;
    outputBufDesc.bufSizes = (XDAS_Int32 *)outBufSize ;

    /* Optional: Set Run time parameters in the Algorithm via control()     */
    RetVal = ividEncfxns->control((IVIDENC1_Handle)handle, XDM_GETVERSION, 
        (IVIDENC1_DynamicParams *)dynamicParams, (IVIDENC1_Status *)status);

    if(RetVal == XDM_EFAIL) {
        printf("\n Control function returned an Error...  ");
        return -1;
    }

    /* Assigning Algorithm handle fxns field to ividEncfxns                   */
    ividEncfxns = (IVIDENC1_Fxns *)handle->fxns;

    /* Resetting bytesGenerated variable                                      */
    bytesGenerated = 0;

    /* Do-While Loop for Encode Call                                          */
    do
    {
        /* Activate the Algorithm                                               */
        handle->fxns->algActivate(handle);

        /* Optional: Set Run time parameters in the Algorithm via control()     */

        inputBufDesc.frameWidth = dynamicParams->inputWidth;
        inputBufDesc.frameHeight= dynamicParams->inputHeight;

        RetVal = ividEncfxns->control((IVIDENC1_Handle)handle, XDM_SETPARAMS, 
            (IVIDENC1_DynamicParams *)dynamicParams, (IVIDENC1_Status *)status);

        if(RetVal == XDM_EFAIL)
        {
            printf("\n Control function returned an Error...  ");
            break; /* Error Condition: Application may want to break off          */
        }
  
        RetVal= ividEncfxns->control((IVIDENC1_Handle)handle, 
            XDM_GETBUFINFO, 
            (IVIDENC1_DynamicParams *)dynamicParams, 
            (IVIDENC1_Status *)status);

        if(RetVal == XDM_EFAIL)
        {
            printf("\n Control function returned an Error...  ");
            break; /* Error Condition: Application may want to break off          */
        }

        handle->fxns->algDeactivate(handle);

        /*Fill up the buffers as required by algorithm                            */

        inputBufDesc.numBufs  = status->bufInfo.minNumInBufs ;
        inputBufDesc.bufDesc[0].buf     = inpBuf;
        inputBufDesc.bufDesc[0].bufSize = status->bufInfo.minInBufSize[0];

        for(ii=0; ii< (status->bufInfo.minNumInBufs - 1);ii++ )
        {
            inputBufDesc.bufDesc[ii+1].buf = inputBufDesc.bufDesc[ii].buf + 
                status->bufInfo.minInBufSize[ii];
            inputBufDesc.bufDesc[ii +1].bufSize = 
                status->bufInfo.minInBufSize[ii +1];
        }

        outputBufDesc.numBufs     = status->bufInfo.minNumOutBufs ;
        outputBufDesc.bufs[0]     = outBuf;
        outputBufDesc.bufSizes[0] = status->bufInfo.minOutBufSize[0];

        for(ii=0; ii< (status->bufInfo.minNumOutBufs-1); ii++ )
        {
            outputBufDesc.bufs[ii+1] = outputBufDesc.bufs[ii] + 
                status->bufInfo.minOutBufSize[ii];
            outputBufDesc.bufSizes[ii+1] = 
                status->bufInfo.minOutBufSize[ii+1];
        }

        /* Read from Input File                                                 */
        BytesRead = 0;
        for(ii=0; ii < inputBufDesc.numBufs ; ii++)
        {
          bytesConsumed   = siuVct_ReadInputData(inputBufDesc.bufDesc[ii].buf, inputBufDesc.bufDesc[ii].bufSize, bytesConsumed);
          BytesRead      += bytesConsumed;
        }

        if( BytesRead == 0) {
          printf("\n Bitstream Ended...  ");
          break; /* Bitstream ended: Break off the while loop                  */
        }

        handle->fxns->algActivate(handle);

        TSCL_Begin = TSCL;

        /* Basic Algorithm process() call */
        retVal = ividEncfxns->process((IVIDENC1_Handle)handle, 
            (IVIDEO1_BufDescIn *)&inputBufDesc,
            (XDM_BufDesc *)&outputBufDesc,
            (IVIDENC1_InArgs *)inArgs,
            (IVIDENC1_OutArgs *)outArgs);

        TSCL_End   = TSCL;

        if(retVal == XDM_EFAIL)
        {
            printf("\n Process function returned an Error...  ");
            break; /* Error Condition: Application may want to break off          */
        }

        /* Optional: Read status via control()                                  */
        ividEncfxns->control((IVIDENC1_Handle)handle, 
            XDM_GETSTATUS, 
            (IVIDENC1_DynamicParams *)dynamicParams, 
            (IVIDENC1_Status *)status);

        /* DeActivate the Algorithm                                             */
        handle->fxns->algDeactivate(handle);

        /* Check for frame ready via recon buffer information                   */
        if(outArgs->reconBufs.numBufs)
        {
            printf("\n Encoded Frame # %d, Cycles consumed = %d  ",frameCount, (TSCL_End - TSCL_Begin));
            fflush(stdout);
            /* Write the output frames in the display order */
            for(ii=0; ii < 1 ; ii++)
            {
              if(outArgs->bytesGenerated != 0 )
              {
                siuVct_WriteOutputData((XDAS_Int8 *)outputBufDesc.bufs[ii],outArgs->bytesGenerated);
                printf("Num bytes generated = %d\n",outArgs->bytesGenerated);
              }
            }
            frameCount++;
        }
        bytesGenerated += outArgs->bytesGenerated ;

        /* Check for Break off condition */
        if (frameCount >= framesToEncode)
        {
            printf("\n Specified number of Frames Encoded...  ");
            break; 
        }
    }  while(1);  /* end of Do-While loop */

    /* Delete the Algorithm instance object specified by handle */
    ALG_delete (handle);

    if(frameCount == 0) 
    {
        frameCount = 1; /* To avoid division with zero */
    }

    siuVct_FileIO_Stop();

    printf("\n --------------  SUMMARY --------------------\n");
    printf("  Total number of Frames          = %d\n",
        (XDAS_UInt32)frameCount);
    printf("  Bit Rate at 30 frames/Sec       = %d Kbps\n",(
        XDAS_UInt32)(((bytesGenerated*8*30)/frameCount)/1000));
    printf("  Width and Height                = %d, %d \n",
        dynamicParams->inputWidth, 
        dynamicParams->inputHeight);
    printf (" --------------    END   --------------------\n");

    printf("\n End of execution\n");

    /* return the number of bytes written to DDR, i.e codec output */
    return(0);
} /* tvct_test_entry_encode_xdm1p0 */

/* nothing past this point */
