#ifndef _SIUVCTPARSE_H_
#define _SIUVCTPARSE_H_
/******************************************************************************
 * FILE PURPOSE: Parse the configuration file for video.
 ******************************************************************************
 * FILE NAME:   siuVctParse.c
 *
 * DESCRIPTION: Contains routines that parse configuration file
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <ti/mas/types/types.h>

#define SIUVCT_PARSE_FILE_NAME_SIZE  160
#define SIUVCT_PARSE_MAX_NUM_ITEMS   200
#define SIUVCT_PARSE_ADDRESS_SIZE     24
#define SIUVCT_PARSE_NCORES            8


typedef enum {
  SIU_VCT_PARAM_TYPE_INT_DEC  = 0,  /* Decimal integer */
  SIU_VCT_PARAM_TYPE_FILENAME = 1,  /* string of max length SIUVCT_PARSE_FILE_NAME_SIZE */
  SIU_VCT_PARAM_TYPE_DOUBLE   = 2,  /* Double integer */
  SIU_VCT_PARAM_TYPE_INT_HEX  = 3,  /* Hexadecimal integer */
  SIU_VCT_PARAM_TYPE_ADDRESS  = 4   /* String of max length SIUVCT_PARSE_ADDRESS_SIZE */
} siuVctParamType_t;

typedef struct siuVctParseMapping_s {
    char                *TokenName;
    void                *Place;
    siuVctParamType_t   Type;
} siuVctParseMapping_t;

typedef struct siuVctParams_s {
    tword  serverIpAddress[SIUVCT_PARSE_ADDRESS_SIZE];
    tword  serverMacAddress[SIUVCT_PARSE_ADDRESS_SIZE];
    tword  localIpAddress[SIUVCT_PARSE_ADDRESS_SIZE];
    tword  localMacAddress[SIUVCT_PARSE_ADDRESS_SIZE];
    tword  tftpGetFilename[SIUVCT_PARSE_FILE_NAME_SIZE];
    tword  tftpPuFilename[SIUVCT_PARSE_FILE_NAME_SIZE];
    tword  encode_or_decode[SIUVCT_PARSE_ADDRESS_SIZE];
    tword  xdmVersion[SIUVCT_PARSE_ADDRESS_SIZE];
    tword  encResolution[SIUVCT_PARSE_ADDRESS_SIZE];
    tulong framesToEncode;
    tulong nCores;
    tulong coreTeamMapping[SIUVCT_PARSE_NCORES];
    tword  codecName[SIUVCT_PARSE_ADDRESS_SIZE];
    tword  decCodecName[SIUVCT_PARSE_ADDRESS_SIZE]; /* Applicable to video transcode only. */
    tword  encCodecName[SIUVCT_PARSE_ADDRESS_SIZE]; /* Applicable to video transcode only. */
    tword  codecType[SIUVCT_PARSE_ADDRESS_SIZE];
    void   *codecParams;  /* Pointer to codec paramters obtained from config 
                           * file. Void pointer is used because each codec uses 
                           * a different parameter structure.
                           */
    void   *decCodecParams; /* Applicable to video transcode only. 
                             * Pointer to decoder paramters obtained from config 
                             * file. Void pointer is used because each codec uses 
                             * a different parameter structure.
                             */
    void   *encCodecParams;  /* Applicable to video transcode only. 
                              * Pointer to encoder paramters obtained from config 
                              * file. Void pointer is used because each codec uses 
                              * a different parameter structure.
                              */
}siuVctParams_t;

tlong siuVctReadParamFile(const char *fileName);

#endif

/* nothing past this point */
