/******************************************************************************
* FILE PURPOSE: Run Codec Standalone test.
******************************************************************************
* FILE NAME:   siuVctRun.c
*
* DESCRIPTION: Contains routines that support TFTP and stand-alone codec test
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "siuVctParse.h"
#include "siuVctNeu.h"
#include "ividmc/siuVidMc.h"
#include "osal/siuOsal.h"
#include "osal/siuIpcOsal.h"
#include "common/siuloc.h"
#include <stdio.h>
#include <string.h>


/******************************************************************************
* FUNCTION PURPOSE: siuVctRun
******************************************************************************
* DESCRIPTION: siuVctRun.
*
* void siuVctRun(
*         void
* )
*****************************************************************************/
extern tlong tvct_test_entry_encode_xdm1p0 (const char* codecname, 
                                            tulong framesToEncode,
                                            tulong nCores,
                                            void *codecParams);
extern tlong tvct_test_entry_encode_xdm0p9 (const char* codecname, 
                                            tulong framesToEncode,
                                            tulong nCores,
                                            void *codecParams);
extern tlong tvct_test_entry_decode_xdm1p0 (const char* codecname, 
                                            tulong framesToDecode,
                                            tulong nCores,
                                            void *codecParams);
extern tlong tvct_test_entry_decode_xdm0p9 (const char* codecname, 
                                            tulong framesToDecode,
                                            tulong nCores,
                                            void *codecParams);
extern tlong tvct_test_entry_img_encode_xdm1p0 (const char *codecname,
                                            void *codecParams);
extern tlong tvct_test_entry_img_decode_xdm1p0 (const char *codecname, 
                                            void *codecParams);
extern XDAS_Int32 tvct_test_entry_aud_encode_xdm0p9 (const char *codecname, 
                                            void *codecParams);
extern XDAS_Int32 tvct_test_entry_aud_decode_xdm1p0 (const char *codecname, 
                                            XDAS_UInt32 framesToDecode, 
                                            void *codecParams);

extern siuVctParams_t siuVctParams;

extern cregister volatile unsigned int DNUM;
int CORE_TEAM_MAPPING[6] = 
{
  IVIDMC_CORE_NOT_USED, IVIDMC_CORE_NOT_USED, IVIDMC_CORE_NOT_USED,
  IVIDMC_CORE_NOT_USED, IVIDMC_CORE_NOT_USED, IVIDMC_CORE_NOT_USED 
};
volatile tulong hdResolution;

void siuVctRunTask(void)
{
    char *configFile = "..\\source\\tftp.cfg";
    char *testFile   = "..\\source\\multiclip.cfg";
    FILE *ftest;
    tuint i;
    int first_time = 1;
    siuInst_t *inst;

    ftest = fopen(testFile,"r");
    if (ftest == NULL) {
       printf("File cannot be opened \n");
    }

    while(!feof(ftest))
    {
        /* Parse Config file and fill structure siuVctParams */
        siuVctReadParamFile(configFile);
        fscanf(ftest,"%s",&siuVctParams.tftpGetFilename[0]);
        fscanf(ftest,"%s",&siuVctParams.tftpPuFilename[0]);
        fscanf(ftest,"%d",&siuVctParams.framesToEncode);

        if ((strlen(&siuVctParams.tftpGetFilename[0]) == 0) ||
            (strlen(&siuVctParams.tftpPuFilename[0]) == 0) ||
            (siuVctParams.framesToEncode <= 0)) {
          break;
        }
        
        if(first_time)
        {
            /* Do reset only once. Reseting multiple times will give rise to syncing issues
             * at the teststream boundaries */

            for(i=0;i<siuVctParams.nCores;i++) {
                CORE_TEAM_MAPPING[i] = siuVctParams.coreTeamMapping[i];
            }

            if(CORE_TEAM_MAPPING[0] == DNUM) {
              /* Configure NEU TX header and EMAC port. */
              siuVctConfigureNeu( (const char *)siuVctParams.serverMacAddress,
                                (const char *)siuVctParams.serverIpAddress,
                                (const char *)siuVctParams.localMacAddress,
                                (const char *)siuVctParams.localIpAddress);
            }

            siu_ipc_osal_attach_owner(siuVctParams.nCores);

            if(CORE_TEAM_MAPPING[0] == DNUM) {
                siu_ipc_osal_barrier_create(siuVctParams.nCores);
                siu_ipc_osal_create_gate();
                siu_ipc_osal_create_share(sizeof(SIUVID_MC_SHMEM_MemTab), (void**)&siuContext.ipcContext.codecShm);
            } else {
                siu_ipc_osal_barrier_open(CORE_TEAM_MAPPING[0]);
                siu_ipc_osal_open_share((void**)&siuContext.ipcContext.codecShm);
            }

            first_time = 0;
        }

        /* Open gate */
        siu_ipc_osal_open_gate();

        /* Record multi-core information in siuInst[0] */
        siuContext.current_working_chnum = 0;  /* Apply to SIU instance 0 */
        inst = SIUINSTp(siuContext.current_working_chnum);
        inst->mcInst.nCores = siuVctParams.nCores;
        for (i=0; i<inst->mcInst.nCores; i++)
        {
          inst->mcInst.coreTeamMapping[i] = siuVctParams.coreTeamMapping[i];
        }

        /* 
        Customer, 3rd Parties would plug in the Codec here
        */
        if (0==strcmp ("VIDEO", (char *)siuVctParams.codecType)) {
            if (0==strcmp ("ENCODE", (char *)siuVctParams.encode_or_decode)) {
                if (0==strcmp ("0.9", (char *)siuVctParams.xdmVersion)) {
                    /* HD resolution */
                    if (0==strcmp ("1080p", (char *)siuVctParams.encResolution)) {
                        hdResolution = 1080;
                    } else if (0==strcmp ("720p", (char *)siuVctParams.encResolution)) {
                        hdResolution = 720;
                    }
                    tvct_test_entry_encode_xdm0p9((const char *)siuVctParams.codecName, 
                                                   siuVctParams.framesToEncode,
                                                   siuVctParams.nCores,
                                                   siuVctParams.codecParams);
                } else if (0==strcmp ("1.0", (char *)siuVctParams.xdmVersion)) {
                    tvct_test_entry_encode_xdm1p0((const char *)siuVctParams.codecName, 
                                                   siuVctParams.framesToEncode,
                                                   siuVctParams.nCores,
                                                   siuVctParams.codecParams);
                }
            } else if (0==strcmp ("DECODE", (char *)siuVctParams.encode_or_decode)) {
                if (0==strcmp ("0.9", (char *)siuVctParams.xdmVersion)) {
                    tvct_test_entry_decode_xdm0p9((const char *)siuVctParams.codecName, 
                                               siuVctParams.framesToEncode,
                                               siuVctParams.nCores,
                                               siuVctParams.codecParams);
                } else if (0==strcmp ("1.0", (char *)siuVctParams.xdmVersion)) {
                    tvct_test_entry_decode_xdm1p0((const char *)siuVctParams.codecName, 
                                               siuVctParams.framesToEncode,
                                               siuVctParams.nCores,
                                               siuVctParams.codecParams);
                }
            } else {
                printf("Neither Encode or Decode specified \n");
            }
        } else if (0==strcmp ("IMAGE", (char *)siuVctParams.codecType)) {
            if (0==strcmp ("ENCODE", (char *)siuVctParams.encode_or_decode)) {
              tvct_test_entry_img_encode_xdm1p0((const char *)siuVctParams.codecName, 
                                                  siuVctParams.codecParams);
            } else if (0==strcmp ("DECODE", (char *)siuVctParams.encode_or_decode)) {
                tvct_test_entry_img_decode_xdm1p0((const char *)siuVctParams.codecName, 
                                                  siuVctParams.codecParams);
            } else {
                printf("Neither Encode or Decode specified \n");
            }
        } else if (0==strcmp ("AUDIO", (char *)siuVctParams.codecType)) {
            if (0==strcmp ("ENCODE", (char *)siuVctParams.encode_or_decode)) {
                tvct_test_entry_aud_encode_xdm0p9 ((const char *)siuVctParams.codecName, 
                                                  siuVctParams.codecParams);
            } else if (0==strcmp ("DECODE", (char *)siuVctParams.encode_or_decode)) {
                tvct_test_entry_aud_decode_xdm1p0((const char *)siuVctParams.codecName,
                                                  siuVctParams.framesToEncode,
                                                  siuVctParams.codecParams);
            } else {
                printf("Neither Encode or Decode specified \n");
            }
        }

        /* Close gate */
        siu_ipc_osal_close_gate();

    }
    fclose(ftest);
    printf("End of Program \n");
}

void siuVctRun(void)
{
  siu_ipc_osal_start(&(siuContext.ipcContext));
  siu_osal_start();
}
/* nothing after this point */
