/******************************************************************************
 * FILE PURPOSE: SIU Framework Routines for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkLoc.h
 *
 * DESCRIPTION: Includes the framework related definitions for Vigdk Application
 *
 * TABS: NONE
 *
 * (C) Copyright 2008, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <ti/mas/neu/neu.h>
#include "memmgm.h"
#include "ndu.h"
#include "utlxferpkt.h"
#include <string.h>
/* Needed for defining prototype of siuNewCodec */
#include "vmmcomp.h"
#include "hal.h"
#include "common/siundu.h"
#ifdef PA_SS
#include <ti/drv/nwal/nwal.h>
#endif
#include "ividmc/siuVidMc.h"
#include "osal/siuIpcOsal.h"

#define SIU_LASTCH            (SIU_FIRSTCH+ggConfig.max_channels-1)
#define SIU_ALL_LASTCH        (SIU_FIRSTCH+ggConfig.max_channels+ggConfig.aux_pcm_channels+ggConfig.pkt_channels - 1)
#define SIU_ALL_LASTCH_NOPKT  (SIU_FIRSTCH+ggConfig.max_channels+ggConfig.aux_pcm_channels-1)

/* Useful macros */
#define siuMakeID(mid,chnum)    ((tuint)(mid)<<8|(tuint)(chnum))
#define siuGetModID(mid_chnum)  ( ((tuint)(mid_chnum) >> 8) & 0xFF )
#define SIU_NV_ID_BASE          251
#define siuMakeNvID(mid,nvId)   ((tuint)(mid)<<8|(tuint)(nvId+SIU_NV_ID_BASE))
/* Get unsigned most/least significant word of an unsigned long */

/* combine unsigned least/most significant word to an unsigned long */ 
#define siu_conv_2_tulong(lsb, msb) (((tulong) msb << TYP_TUINT_SIZE) | (tulong) lsb)

/* NULL function pointer definition */
#define FP_NULL ((void (*)) 0)
/*----------------------------*/
/* Exception and Debug Macros */
/*----------------------------*/
/* Exception Messages */
enum {
  SIUINIT_EXC_INIT_CFG              = 90,            
  SIUINIT_EXC_HEAPINST_MEMORY       = 91,
  SIUINIT_EXC_INIT_NDUCTBL          = 92,            

  SIUMAIN_EXC_MAIN_STATE            = 100,
  SIUMGR_EXC_BUFALLOC               = 110,            
  SIUMGR_EXC_NEWECO                 = 111,
  SIUMGR_EXC_OPEN_CODEC             = 117, 
  SIUMGR_EXC_WRONG_BUFF_SIZE        = 121,
  SIUMGR_EXC_INT_SCRATCH_OVERFLOW   = 122,
  SIUMGR_EXC_EXT_SCRATCH_OVERFLOW   = 123,
  SIUMGR_EXC_EXT_RESDEP_OVERFLOW    = 124,
  SIUMGR_EXC_EXT_RESINDEP_OVERFLOW  = 125,
  SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL    = 126,
  SIUMGR_EXC_EXT_SHM_OVERFLOW       = 127,
  SIUMGR_EXC_INT_SHM_OVERFLOW       = 128,

  SIUNEU_EXC_NEW_NEU                = 240,            
  SIUNEU_EXC_OPEN_NEU               = 241,            
  SIURTP_EXC_NEW_RCU                = 270,            
  SIURTP_EXC_OPEN_RCU               = 271,            
  SIUSTMVF_EXC_OPEN_AER             = 290,            
  SIUSTMVF_EXC_DUMMYINST            = 291,
  SIUSTMVF_EXC_INVD_THWCFG          = 293,            

  SIUSTMVF_EXC_INVD_IDLE2CLOSE      = 302,            
 
  SIUSTMVF_EXC_INVD_CH_RECONFIG     = 306,
  SIUSTMVF_EXC_INVD_CH_CLOSE        = 307,
  
  SIUFM_EXC_INVD_PENDING_STATE      = 310,

  SIUSTRT_EXC_INVD_NDUCFG           = 320,           
  SIUSTRT_EXC_INVD_HALCFG           = 321,           

  SIUDYNHEAP_EXC_ALLOC              = 330,
  SIUDYNHEAP_EXC_RELOC              = 331,
  SIUHAL_EXC_HALNEW                 = 380,
  SIUHAL_EXC_HALGETSIZES            = 381,
  SIUHAL_EXC_HALINIT                = 382,
  SIUHAL_EXC_UTOPIA_MEMALLOC        = 383,
  SIUHAL_EXC_UTOPIA_INIT            = 384,
  SIUHAL_EXC_UTOPIA_ENABLE          = 385,
  SIUHAL_EXC_TDM_ALLOC              = 386,
  SIUHDNDU_EXC_NOT_SUPPORTED        = 412,
  SIUGG_EXC_GGPLATINIT              = 450, /* platFormInit (ggcfg.c) exception */
/* Security Unit (MSU) Exceptions */
  SIU_EXC_MSU_NEW_RTP                = 480,
  SIU_EXC_MSU_NEW_RTCP               = 481,
  SIU_EXC_MSU_OPEN_RTP               = 482,
  SIU_EXC_MSU_OPEN_RTCP              = 483,
  SIU_EXC_MSU_CTRL_RTP               = 484,
  SIU_EXC_MSU_CTRL_RTCP              = 485,
  SIU_EXC_MSU_KEYCTRL_RTP            = 486,
  SIU_EXC_MSU_KEYCTRL_RTCP           = 487,
  SIU_EXC_MSU_GETSTATS_RTP           = 488,
  SIU_EXC_MSU_GETSTATS_RTCP          = 489,
  SIU_EXC_MSU_ERR_MSGTYPE            = 490,
  SIU_EXC_MSU_ERR_NO_RTP_INST        = 491,
  SIU_EXC_MSU_ERR_NO_RTCP_INST       = 492,
  SIUTASK_EXC_INSUFFICIENT_QUEUE_LEN = 520, /* SIU_TASK_QUEUE_LENGTH is not sufficient */
  SIUTASK_EXC_MAX_NUM_CH_MISMATCH    = 521, /* siuContext.maxNumChannels != SIU_TASK_MAX_NUM_CHANNELS */
/* VPPU Exceptions */
  SIUVPPU_EXC_NEW_VPPU             = 600,
  SIUVPPU_EXC_OPEN_VPPU            = 601,
  SIUVPPU_EXC_BLOCKALLOC_FAILED    = 602,
  SIUVPPU_EXC_BLOCKFREE_FAILED     = 603,
  SIUVPPU_EXC_RXFRAME_TOO_BIG      = 604,
  SIUVPPU_EXC_TXFRAME_TOO_BIG      = 605,

/* TSU Exceptions */
  SIUTSU_EXC_NEW_TSU               = 607,
  SIUTSU_EXC_OPEN_TSU              = 608,
  SIUTSU_EXC_BLOCKALLOC_FAILED     = 609,

/* VTK Exceptions */
  SIUVTK_EXC_NEW_VTK               = 610,
  SIUVTK_EXC_OPEN_VTK              = 611,
  SIUVTK_EXC_COMMIT_VTK            = 612,
/* OVLY Exceptions */
  SIUOVLY_EXC_NEW_OVLY             = 613,
  SIUOVLY_EXC_OPEN_OVLY            = 614,

/* CUSTOM_ECO Exceptions */
  SIUCUSTOMECO_EXC_NEW_CUSTOM_ECO  = 615,
  SIUCUSTOMECO_EXC_OPEN_CUSTOM_ECO = 616,
/* VIDEO SCHEDULER Exceptions */
  SIUVIDEO_EXC_SCHED_FIFO_FULL     = 620,
  SIUYUV_EXC_BLOCKALLOC_FAILED     = 621,
/* COMMIT STATE VIDEO Exceptions */
  SIUVIDEO_EXC_COMMIT_STATE        = 622,

/* RX Thread Exceptions */
  SIUVIGDK_RX_DECODER_ERROR        = 625,
  SIUVIGDK_TX_GETYUV_ERROR         = 626,

/* TX Thread Exception */
  SIUVIGDK_TX_YUV_ALLOC_ERROR      = 632,

/* YUV Exception */
  SIUVIGDK_EXC_YUV_ALLOC_FAIL      = 640,
  SIUVIGDK_EXC_YUV_SETBUSY_FAIL    = 641,
  SIUVIGDK_EXC_YUV_FREE_FAIL1      = 642,
  SIUVIGDK_EXC_YUV_FREE_FAIL2      = 643,
  SIUVIGDK_EXC_NEW_YUV             = 644,

/* GMP Exceptions */
  SIUVIGDK_EXC_GMP_BEGIN           = 650,
  /* GMP Exceptions are defined in siuVigdkGmp.h */
  SIUVIGDK_EXC_GMP_END             = 660,
  SIUVIGDK_EXC_GMP_CREATE_FAIL     = 661,

/* TFTP Exceptions */
  SIUTFTP_EXC_NEW_TFTP             = 664,
  SIUTFTP_EXC_OPEN_TFTP            = 665,
  SIUTFTP_EXC_ALLOC_TFTP           = 666,
  SIUTFTP_EXC_FREE_TFTP            = 667,

/* IPC Barrier Exceptions*/
  SIUVIGDK_EXC_IPC_BARRIER_CREATE  = 680,
  SIUVIGDK_EXC_IPC_BARRIER_NOT_AVAILABLE = 681,
  SIUVIGDK_EXC_IPC_BARRIER_INVALID_COUNT = 682,

/* IPC Share Global Exceptions*/
  SIUVIGDK_EXC_IPC_SHARE_CREATE = 690,

/* FC-RMAN exceptions*/
  SIUVIGDK_EXC_RMAN_INIT_FAIL = 700,
  SIUVIGDK_EXC_FC_ALG_ARRAY_OVERFLOW = 701,

#define SIU_EXC_LAST                1023   /* 10 bit max */
  SIU_EXC_TODO                      = SIU_EXC_LAST  /* for TODOs */
  
};
/*
 * The exception code has been enhanced to support chanel-based exception as well.
 * The DSP will enter infinite loop when DSP-based exception occurs
 * Only the affected channel enters CHAN_EXCEP state when Channel-based exception occurs
 * The DSP is still alive to process the remaining channels.
 *
 * The most significant bit in the exception code is used to specify the exception type
 */
#define SIU_EXCEPTION_TYPE_CHANNEL 	 0x8000
#define SIU_EXCEPTION_TYPE_DSP	  	 0x0000   
#define SIU_EXCEPTION_CODE_MASK      0x03FF
#define SIU_IS_CHAN_EXCEP(code)      ((code) & SIU_EXCEPTION_TYPE_CHANNEL)
#define SIU_GET_EXCEPTION_CODE(code) ((code) & SIU_EXCEPTION_CODE_MASK)

void siu_exception(tuint ID, tuint code);

#define siu_exc_assert(expr,code,inst) { \
  (expr) ? ((void)0) : siu_exception((inst)->ID, code); }

/* Debug macro  */
#define siu_dbg_out(inst,s) {  (siuContext.debug)((inst)->ID,(s));  }

/* Some other macros */
#define siuMax(x,y) (((x)>(y))?(x):(y))

/******************************************************************************
 * DATA DEFINITION: Parameters that are used by SIU Module
 ******************************************************************************
 * DESCRIPTION:
 *
 *****************************************************************************/

/* SIU states */
#define SIU_ST_WAITDOWNLOAD    1    /* Wait for download state */
#define SIU_ST_READY           2    /* Ready State             */
#define SIU_ST_WORKING         3    /* Working state           */

/* SIU Channel States */
#ifdef  AUDIO 
  #define SIU_MAX_NUM_CHST             20   /* Total number of channel states */
#else
#define SIU_MAX_NUM_CHST               20   /* Total number of channel states */
#endif /* AUDIO */

#define SIU_CHST_CLOSED                 0   /* Channel is closed, modules closed */
#define SIU_CHST_IDLE                   1   /* Channel is open - PIU works */
#define SIU_CHST_VOICE                  2   /* Channel in voice mode - codec active */
#define SIU_CHST_DTMF                   3   /* Channel in DTMF mode */
#define SIU_CHST_FAX_RELAY              4   /* Channel in Fax Relay mode */
#define SIU_CHST_APPCLOSED              5   /* switchover, channel closed */
#define SIU_CHST_MODEM_RELAY            6   /* Channel in Modem Relay Mode */ 
#define SIU_CHST_DATA_MODEM             7   /* Channel is in data modem state */
#define SIU_CHST_CLEAR_DATA_MODE        8   /* Channel in clear data mode */
#define SIU_CHST_FAX_MODEM              9   /* Channel in fax modem mode */
#define SIU_CHST_LINK_LAYER_MODEM_RELAY 10  /* Channel in Link Layer Modem Relay mode */
#define SIU_CHST_CP_IDLE                11  /* Channel in CP - idle */
#define SIU_CHST_CP_DTMF                12  /* Channel in CP - DTMF mode */
#define SIU_CHST_CP_MODEM_TERM          13  /* Channel in CP - Modem Termination */
#define SIU_CHST_CP_CLEAR_DATA          14  /* Channel in CP - Clear Data */
#define SIU_CHST_CP_FAX_TERM            15  /* Channel in CP - Fax Termination */
#define SIU_CHST_BERT_MODE              16  /* channel in bert mode*/
#define SIU_CHST_NX64_MODE              17  /* channel in Nx64 mode*/
#define SIU_CHST_EXCEPTION              18  /* channel in exception mode */

#define SIU_CHST_AUDIO                 (19) /* channel in audio mode*/

#if (defined(HDVIDEO) || defined(HDCT))
#define SIU_CHST_VIDEO                  19  /* Channel is in VIDEO mode */
#endif

#define SIU_CHST_STATE_CHANGE_ERROR  (0xffff)  /* Indication for error in 
                                                * channel state change */

#define SIU_CID_MASK UTL_CID_MASK /* Low 8 bits of module ID are the channel number */
/* Module ID's */
#define SIU_MID_SIU     0x01      /* SIU module */
#define SIU_MID_RCU     0x15      /* RCU module */
#define SIU_MID_NEU     0x16      /* NEU module */
#define SIU_MID_VPPU    0x2C      /* Video Protocol Processing Unit */
#define SIU_MID_VTK     0x2D      /* Video Toolkit */
/*************************************************/
/* Module IDs 0x31-0x33, 0x35-0x3F are available */
/*************************************************/
#define SIU_MID_CUSTOM_ECO 0x2E
#define SIU_MID_TSU     0x2F
#define SIU_MID_OVLY    0x30
#define SIU_MID_TFTP    0x31

#define SIU_MID_MSU_RTCP   0x34    /* MSU RTCP module */
/* Extracting channel number from a module's ID */
#define SIU_EXTRACT_CHNUM(id)   UTL_EXTRACT_CHNUM(id)
#define SIU_EXTRACT_MID(id)     ((id) >> 8)
#define siuGetChnId(id)             ((id) & SIU_CID_MASK)
#define siuGetChnOffset(id)         (siuGetChnId(id) - SIU_FIRSTCH)

/* Debug report levels */
#define SIU_DEBUG_INFO         0    /* Passing debug information */
#define SIU_DEBUG_WARNING      1    /* Warning message */
#define SIU_DEBUG_FATAL        2    /* Fatal message. Report to MICRO and
                                     * go to infinite loop. */
#define SIU_DEBUG_CHAN_EXCEP   3    /* Channel-based exception */                                 

#ifdef GG_ISR_2p5ms
#define SIU_MAX_MSG_DELAY_CNT 2000 /* Number of 2.5ms frames message processing
                                   * can be delayed.
                                   * If message processing
                                   * is delayed more than this value, steps
                                   * are taken to reduce MIPS (such as disable
                                   * echo canceller update). */
#else
#define SIU_MAX_MSG_DELAY_CNT 10000 /* Number of 0.5ms frames message processing
                                   * can be delayed. 
                                   * If message processing
                                   * is delayed more than this value, steps
                                   * are taken to reduce MIPS (such as disable
                                   * echo canceller update). */
#endif /* GG_ISR_2p5ms */
                                   
#ifdef GG_ISR_2p5ms
#define SIU_MIN_FREE_MIPS_CNT 2000    /* Minimum time(120ms) to stay in free mips
                                       mode */
#else
#define SIU_MIN_FREE_MIPS_CNT 10000   /* Minimum time(120ms) to stay in free mips
                                       mode */
#endif /* GG_ISR_2p5ms */

#define SIU_CTRL_NDU_GETFREE       2    /* Return free space in ndu units */
#define SIU_CTRL_NDU_FIFO		   1	/* HPI using FIFO memory partition */
#define SIU_CTRL_NDU_LIN 		   0	/* HPI using Linear memory partition */

/* Messaging type (signalling or regular) selection */
/* CAUTION: When converting to #define, make sure start is at zero because this is used
   as an index into an array (see for example siuNduv1Msg.c) */
typedef enum {
   SIU_DPR_SIGNAL_MSG,
   SIU_DPR_NORMAL_MSG,
   SIU_DPR_LOW_MSG,
   SIU_DPR_IGNORE_MSG,
   SIU_DPR_NONE_MSG
} siuDPRMsgType_t;
/* translation for non-prioritized drivers */
#define SIU_NORMAL_MESSAGE    SIU_MSG_PRIORITY_NORMAL
#define SIU_INTERRUPT_MESSAGE SIU_MSG_PRIORITY_INTERRUPT
#define siuMsgPriority_e  tsiuMsgPriority_t
/* NDU packet loopback Control flags */
typedef enum {
  NDU_CTRL_SND_LOOPBACK_OFF = 0, /* default setting will be send_loopback off */
  NDU_CTRL_SND_LOOPBACK_ON,      /* == 1 will set it to send_loopback on */
  NDU_CTRL_RCV_LOOPBACK_ON,      /* == 2 will set it to receive_loopback on 
                                      when loopback is done in NDU (if NEU is not present)*/
  NDU_CTRL_RCV_LOOPBACK_OFF = 0     /* default setting will be receive_loopback off */
} NduLoopbackControlFlag_t;

/* Channel replication definition */
#define SIU_REPLICATE       -1 /* channel_rx_timeslot value for replication */
/* Return values */
#define SIU_NOERR 0
#define SIU_ERROR 1
/******************************************************************************
 * DATA DEFINITION:  Define the Message FIFO parameters and structures
 ******************************************************************************
 * DESCRIPTION:  The following definitions are for the message DSP-to-MICRO 
 *               FIFO.
 *
 *****************************************************************************/

typedef struct {
  tuint inIndex;     /* location to place next message */
  tuint outIndex;    /* location of next message to send to DPR */
  tuint freeSpace;   /* number of messages pending in the FIFO */ 
  tuint dropCounter; /* Number of messages dropped due to full FIFO */
} siuFIFO_t;
/* The maximum mega-phase is configured to be 12.  Each mega phase is 5 ms, so
 * 12 mega-phases comprises 60ms. This is required to be able to account for
 * 10, 20 and 30ms (60 is the least common multiple of these) with correct handling
 * of wrapping */
#define SIU_HAL_MAX_MEGA_PHASE  12 

typedef struct {
  tint (*nduReceiveOut)(void *targetInst, xferPktInfo_t *pktInfo);
  void *targetInst;
} siuNduRxOutSpec_t; 

/* NDU Configuration parameters */
typedef struct {
  nduRxOut_t  RxOutSpec;  /* Receive Out object specification */
} siuNduConfig_t;

#define tsiuNduConfig_t siuNduConfig_t
/* NDU Instance structure */
typedef struct {
  tuint hd_pkt_routing_type;    /* toward host or switch PLD */
  nduRxOut_t RxOutSpec;  /* Receive Out object specification. */
  tuint pktLoopback;            /* packet loopback (recieve on/off, send on/off */
} siuAmInst_t;

/*****************************************************************************
 * DEFINITION: Data passing structure. Required for compatibility with
 *             utopia fifo structure
 *****************************************************************************/
typedef struct {
  tuint dataSize;    /* Number of elements in data array */
  void *data;        /* The data                         */
  tuint ts;          /* Timestamp of 1st sample          */
} globData_t;

/*****************************************************************************
 * DATA DEFINITION: High Density NDU message FIFO structure
 *****************************************************************************
 * DESCRIPTION: Definition for the micro to DSP message FIFO. There is a
 *              FIFO for each message priority
 ****************************************************************************/
#define siuHdFifo_t dsiuFifo_t
/* Message Fifo Queue Structure */
typedef struct 
{
  tuint     fifoLen;    /* message FIFO length */
  dprword   *fifoBuf;   /* Pointer to message FIFO */
  siuFIFO_t *fifoCtrl;  /* pointer to FIFO control index and length  */
} siuMsgQueue_t;
/******************************************************************************
 * DATA DEFINITION: This is a data structure to represent the portion of 
 * heapInstance in external memory.
 *****************************************************************************/
#if 0 /* Removing Global Statistics to remove dependency on RCU */
typedef struct siuHeapInstExt_s {
 siuExtMemStats_t stats;
} siuHeapInstExt_t;
typedef struct siuHeapInst_s {
  siuStats_t stats;      /* Global statistics */
} siuHeapInst_t;
#endif

typedef struct {
  tuint  codingType;
  /* Static configuration */
  tuint  maxImgWidth;
  tuint  maxImgHeight;
  tuint  maxFrameRate;
  tuint  inputStreamFormat;
  tulong maxBitRate;
} siuCodecCfg_t;

/******************************************************************************
 * DATA DEFINITION: SIU Multicore Instance.
 ******************************************************************************
 * DESCRIPTION: Contains information for Multi-core codecs
 *
 *****************************************************************************/
typedef struct siuMcInst_s {
  tuint          nCores;                      /* Number of cores participating in encoding or decoding */
  tuint          coreTeamMapping[CPU_NCORES]; /* Specify the cores which form a team to do multi-core encoding or decoding */
  tuint          chanIdOnCores[CPU_NCORES];   /* Specify the channle IDs on the participating cores */
} siuMcInst_t;

/******************************************************************************
 * DATA DEFINITION: SIU Instance.
 ******************************************************************************
 * DESCRIPTION: Contains state information for one SIU channel.
 *
 *****************************************************************************/

typedef struct siuInst_s {
  /*------------------------------------------------*/
  /* Run-time operation information (needed by SIU) */
  /*------------------------------------------------*/
  tuint  ID;                  /* instance ID                                  */
                              /* (options=-h-v5)                              */
                              /* (initialized in siuInit()                    */
  tint  state;                /* Channel State: 0=closed; 1=idle; 2=voice;    * 
                               * 3=dtmf; 4=fax; 5=transparent; 6-10=reserved; *
                               * 11=BERT                                      */
                              /* (options=-v3)                                */
  tint  pending_state;        /* Channel is transitioning to this state. 
                               * Configuration parameters are received, like channel
                               * is already in this state. */
  tuint state_transition_msg_id;  /* Message ID of a message that initiated
                                   * state transition */
  siuAmInst_t    amInst;    /* NDU instance */
  void    *chHeapID;      /* Current channel memory heap ID */
#if (defined(HDVIDEO) || defined(HDCT))
#define SIU_GET_VIDEO_CHANNEL_TYPE(a)       UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField0, 0, 4)
#define SIU_SET_VIDEO_CHANNEL_TYPE(a,b)     UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField0, b, 0, 4)
#define SIU_GET_VIDEO_CHANNEL_EXT_TYPE(a)   UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField0, 4, 4)
#define SIU_SET_VIDEO_CHANNEL_EXT_TYPE(a,b) UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField0, b, 4, 4)
#define SIU_GET_VIDEO_CODING_TYPE(a)        UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField0, 8, 8)
#define SIU_SET_VIDEO_CODING_TYPE(a,b)      UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField0, b, 8, 8)
  tuint          videoBitField0;
/* Store Linear Domain(YUV) connection */
#define SIU_GET_VIDEO_ROUTE_DST_CH_ID_LD(a)    UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField1, 0, 8)
#define SIU_SET_VIDEO_ROUTE_DST_CH_ID_LD(a,b)  UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField1, b, 0, 8)
/* Store Packet Domain(VPPU) connection */
#define SIU_GET_VIDEO_ROUTE_DST_CH_ID_PD(a)    UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField1, 8, 8)
#define SIU_SET_VIDEO_ROUTE_DST_CH_ID_PD(a,b)  UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField1, b, 8, 8)
  tuint          videoBitField1;
#define SIU_GET_VIDEO_ROUTE_PD_DEFAULT_CONN(a)     UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField2, 0, 1)
#define SIU_SET_VIDEO_ROUTE_PD_DEFAULT_CONN(a,b)   UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField2, b, 0, 1)
#define SIU_GET_VIDEO_STATIC_CFG_DONE(a)           UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField2, 1, 1)
#define SIU_SET_VIDEO_STATIC_CFG_DONE(a,b)         UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField2, b, 1, 1)
#define SIU_GET_VIDEO_DYNAMIC_CFG_DONE(a)          UTL_GET_BITFIELD(((siuInst_t *)a)->videoBitField2, 2, 1)
#define SIU_SET_VIDEO_DYNAMIC_CFG_DONE(a,b)        UTL_SET_BITFIELD(((siuInst_t *)a)->videoBitField2, b, 2, 1)
  tuint          videoBitField2;
  word           *yuvBuffer;
  tuint          yuvWidth;
  tuint          yuvHeight;
#define SIU_DEFAULT_TX_MAX_PAYLOAD_SIZE 1460
  tuint          maxPayloadSize;    /* MTU size, without headers for TX direction    */
  siuMcInst_t    mcInst;
#endif

  tuint lastPacketType;     /* Last packet type to speed up siu_stm_search */
  tuint payloadType;

  /* Important Note: Make sure that all the bitfield elements in one bitfield are accessed  *
   * from the same task, otherwise if read/modify/write are not atomic then it may result   *
   * in a race condition                                                                    */

  tuint chan_bfield2;
#define SIU_READ_PRIMARY_DATA_PAGE(a)  UTL_GET_BITFIELD(((siuInst_t *)a)->chan_bfield2, 0, 1)
#define SIU_SET_PRIMARY_DATA_PAGE(a,b) UTL_SET_BITFIELD(((siuInst_t *)a)->chan_bfield2, b, 0, 1)
#define SIU_READ_TASK_PRIORITY_ID(a)   UTL_GET_BITFIELD(((siuInst_t *)a)->chan_bfield2, 12, 2)
#define SIU_SET_TASK_PRIORITY_ID(a,b)  UTL_SET_BITFIELD(((siuInst_t *)a)->chan_bfield2, b, 12, 2)
  tuint chan_bfield0;
#define SIU_READ_TX_ENABLED(a)      UTL_GET_BITFIELD(((siuInst_t *)a)->chan_bfield0, 8, 1)
#define SIU_SET_TX_ENABLED(a,b)     UTL_SET_BITFIELD(((siuInst_t *)a)->chan_bfield0, b, 8, 1)
  /*-----------------------------*/
  /* Network Protocol Parameters */
  /*-----------------------------*/
  tuint protocol_type;       /* PVP_PT_xxx */  
                             /* (options=-v1) */
  union {
    struct {
        void *vppuInst;
    } video;
  } stateInst;

  union {
    void  *rcuInst;             /* a pointer to RCU instance */
                                /* (vptr=t:rcuInst_t) */
    void *tftpInst;
  } net;
  void  *neuInst;               /* a pointer to NEU instance */
                                /* (vptr=t:neuInst_t) */
#ifdef PA_SS
  nwal_Handle    udpHandle;
#endif

#if 0 /* Removing Global Statistics to remove dependency on RCU */
  siuHeapInst_t *heapInst;      /* More SIU per channel data stored with
                                 * channel heap instead of bss */
                                /* This could be accessed through 
                                 * gg_siuheap_inst[], but reflection page can't
                                 * see there until its enhanced.
                                 */
#endif
} siuInst_t;

extern siuInst_t siuInst[];
/* Macro for access to particular SIU instance */
#define SIUINSTp(a) (&siuInst[a])
#define SIUINSTpCHINDEP siuInst
#define SIUINSTpNOTCLOSED(a) (siuInst[a].state != SIU_CHST_CLOSED)
static inline void SIUINSTpSETCLOSED(siuInst_t *inst)
{
  /* Statically allocated SIU instance */
}

#define tsiuInst_t siuInst_t    /* For utopia bootloader/application compat */
#define TSIUINSTp(a)  (&siuInst[a])

/* MHM macro (allocate memory from the current channel heap) */
#define siuChAlloc(chnum, size) \
  mhmAlloc (SIUINSTp(chnum-SIU_FIRSTCH)->chHeapID, size)

/* Sizeof mgb_t in tuint units                                       */
/* Changed from hardcoding of 127 as it will not work for CPE builds *
 * where max message size can only be 64 16-bit words(mgb_t size)    */                                                                   
#define SIU_PCMPKT_TRACE_BUF_SIZE      (sizeof(mgb_t)/sizeof(tuint))
/******************************************************************************
 * DATA DEFINITION: buffer dynamic allocation structure
 ******************************************************************************
 * DESCRIPTION: buffer dynamic allocation structure
 ****************************************************************************/
#define SIU_CHANNEL_ALLOC 0
#define SIU_LBRCBUF_ALLOC 1
typedef struct {
  bool  dynBufAllocEnable;
  tint  numBitmapWord;
  tuint *maxChannelsPerWord;
  tuint *bitmapBufIndexPerWord;
  tint  *siuCh2Slot;
  tint  *siuCh2SlotLBRC;
  tuint *maxLBRCBuffers;
  tuint *bitmapLBRCBufIndexPerWord;
} siuDynBufAlloc_t;

/******************************************************************************
 * DATA DEFINITION: Dynamic heap allocation structure
 ******************************************************************************
 * DESCRIPTION: Dynamic heap allocation structure 
 *****************************************************************************/
typedef struct {
  bool   enableFlag;    /* Dynamic heap allocation enable flag */
  tint   maxDynSlot;    /* # of maximum allowed heap slots available */
  tint  *memSlotID;     /* Array containing the slot ID of heap slots */
} siuDynHeapAlloc_t;

/******************************************************************************
 * DATA DEFINITION: Halt DSP control
 *****************************************************************************/
typedef union {
  struct {
    tulong halt_if_set;    /* Write here to cause halt */
    tulong halted_if_set;  /* Will write here to ack halt */
  } halt;
} siuHalt_t;


extern const tuint siuStatsReqChstMaskLsw[SIU_MAX_NUM_CHST];
extern const tuint siuStatsReqChstMaskMsw[SIU_MAX_NUM_CHST];
extern const tuint siuStatsReqChstMask2Lsw[SIU_MAX_NUM_CHST];
extern const tuint siuStatsReqChstMask2Msw[SIU_MAX_NUM_CHST];

/* Utopia bootloader/application compatibility definition */
#define tsiuContext_t siuContext_t
#define tsiuContext   siuContext

/* siuiram.c */
extern tulong       siu_isr_clock;        /* PCM ISR Clock */
extern tulong       siu_timer_tick;       /* Timer tick (1ms) */
extern tuint     siu_stagger_clock;  /* stagger clock */

/* Variable for decimating PCM ISRs. Required for operation with PCM ISRs
 * different from 2.5 ms (e.g. 0.5 ms)                                    */
extern tuint  siu_2_5msIsrDecimate;
/* Message acknowledgment control structure */
typedef struct {
  tuint msgId;     /* Store message ID for error indications */
  tuint ackType;   /* Store ack type if ack needed */
} siuMsgAck_t;

extern siuMsgAck_t siuNormalMsgAck, siuInterruptMsgAck;
/* siumgr.c  - common eco management routines */
void siuSetChannelInfo (mgb_t *message);
tint siuBufferAlloc (tint ecoNbufs, siuInst_t *inst, const memBuffer_t *reqBuffer, 
                     memBuffer_t *assignedBuffer, void *EcoBufTbl, tuint mid,
                     bool allocFromHeap);

typedef tint (*siuEcoGetSizes_t)(tint *modNbufs, const memBuffer_t **bufs,
                               void *modSizeCfg);
typedef tint (*siuEcoNew_t)(void **modInst, tint modNbufs, memBuffer_t *modBufs,
                            void *modNewCfg);
typedef tint (*siuEcoReloc_t)(void **modInst, tint modNbufs, memBuffer_t *modBufs,
                              tuint *modRelocBitMask);

#ifdef SIU_MULTICORE_SHARED_DATA
void *siu_remap_mcs_data (void *input_ptr);
#else
#define siu_remap_mcs_data(input_ptr) (input_ptr)
#endif

tint siuNewEco (siuInst_t *inst, siuEcoGetSizes_t getsizes_fcn, void *sizeCfg, 
                siuEcoNew_t new_fcn, void *modNewCfg, void **modInst, 
                memBuffer_t *modBufs, tuint moduleId);

void siu_open_network (tint chnum, 
                       tint (*netReceiveIn) (void *inst, xferPktInfo_t *),
                       void *targetInst,
                       bool rcuApi);
tint siuNewNvEco (siuEcoGetSizes_t getsizes_fcn, void *sizeCfg,
                  siuEcoNew_t new_fcn, void *ecoNewCfg, void **ecoInst,
                  memBuffer_t *ecoBufs, tint nvId);

/* siuneu.c */
void siu_new_neu (tint chnum);
void *siu_new_nv_neu (tint nvEcoId, tint txHdrLenWords);
void siu_open_neu (tint chnum, neuParms_t *cfg);
void siu_net_encap_glob_cfg (void *message);
void siu_net_encap_control (siuInst_t *inst, mgb_t *msg);
void siu_net_encap_bwc_config (siuInst_t *inst, mgb_t *msg);
void siuGetNetEncapStats (void *siuInst, bool reset, tuint trans_id, tuint priority);
void siuGetGlobalPktStats (void *siuInst, bool reset, tuint trans_id, tuint priority);
void* siuGetNetEncapInst (tuint ID);
void siuUpdateTxStats (tuint instance_ID, xferPktInfo_t *pktInfo, tint retval);
void siuProcessPktFifo(void);
tint siuConfigStreamingNeu(void *configmsg);
void siuEncapConfig (siuInst_t *inst, mgb_t *message); 
void siuAlarm (tuint module_id);
NDU_DPRMsgType_t siuMessageLevel (tuint msgId);
tuint siu_translate_dynamic_chnum (tuint msg_id, tuint chan);
void siuNeuViolReport ( tuint ID, void *vdata );
void siuHaltDsp (mgb_t *message);
void siuPktioIfCfg (mgb_t *message);
void siuDebugReport (tuint id, tuint *str, bool queue);

void siuStatusResponse          (mgb_t *message);
void siuGeneralAck              (tuint msgId, tuint chnum, tuint transId,
                                 tuint ackType, tuint gen_param, tuint priority);
void siuMemoryWrite             (mgb_t *message);
void siuMemoryRead              (mgb_t *message);
void siu_memory_read_exec       (void *mem_read);
void siuHwConfig (mgb_t *microMessage);
void siuReadyIndication (mgb_t *microMessage);
void siuErrorIndication (tuint moduleID, tuint trans_id, tuint error_code);
void siuChannelException (tuint moduleID, tuint error_code);
void siuGetRtTrace (siuInst_t *inst, mgb_t *message);
void siuRtTraceFlush ();
void siuGetNetworkStats (void *siuInst, bool reset, tuint trans_id, tuint priority);
/* siustmxx.c - siu message processing state machines */
void siuCreateChannels ( void );
void siuOpenChannel (siuInst_t *inst, mgb_t *message);
void siuCommitStateOpen (siuInst_t *inst, mgb_t *message);
void siuChannelState (siuInst_t *inst, mgb_t *message);
void siuProcessNormalMessage (mgb_t *microMessage, siuMsgAck_t *msgAck);
void siuProcessInterruptMessage (mgb_t *message, siuMsgAck_t *msgAck);
void siuProcessMessage (NDU_msgPriority_e priority, mgb_t *message);
bool siuProcMsgAck(NDU_msgPriority_e priority, mgb_t *message);

/* siustrt.c */
void siuFrameworkInit (void);
void siuVigdkInit(void);
tuint siuHwInit (void);
void siuSwInit(void);
void siuRestartIndication (tuint revId);
void siuHaltCheck (void);

/* siuhal.c*/
/* siuReturn_t siu_new_hal (tint cellSize, tuint *revId); */
siuReturn_t siu_new_hal (void);
siuReturn_t siu_init_hal (const tint *marRegs, const tint *marVals);
siuReturn_t siu_periph_setup (void);
void siu_invalidate_cache (void);
void siuMsgTdmPortConfig (mgb_t *msg);
#ifdef SIU_TIMER_BASED_TDM    
void siuTsipEmulConfig(mgb_t *msg);
#endif
/* siuprd.c */
void siuSubRateFcn (void *cxt, tuint ts);
void siuPRDTask (void);
void siuPRDTaskIdle (void);

#ifdef NDU_GMAC
bool siu_pktio_macport_config(struct MGB_MDS_PKTIO_IF_MAC_PARAMS*  pktio_params);
#endif
/******************************************************************************
 * FUNCTION PURPOSE: Opens a SIU Tx Task.
 ******************************************************************************
 * DESCRIPTION: Configures and enables SIU Tx task starting at ECU.
 *
 *****************************************************************************/
static inline void siuTxOpen (siuInst_t *siuInst)
{
  SIU_SET_TX_ENABLED(siuInst, TRUE);
}
void siuInitSiu (tuint i);

/* siumain.c */
void siuMessageTask( void );
void siuException (tuint id, char *s);
void siuProcChanExcep (tuint moduleID, tuint error_code);

/* siudbg.c */
void siuDebugBufferInit (void);
void siuWriteDebugTrace (tuint moduleID, tuint messageCode, tuint msgLength, 
                         tuint *messageData);
void siuDebugInfo (tuint moduleID, tuint msgType, tuint messageCode, 
                   tuint msgLength, tuint *messageData);
void siuRtscDebugInfo (void *handle, tuint msgType, tuint messageCode, 
                       tuint msgLength, tuint *messageData);
void siuDebugTask (void);
void siuShutdownNduAcu    (siuInst_t *inst);
void siuCommitState(siuInst_t *inst, mgb_t *message);
void siu_reset_configuration (siuInst_t *inst);
void *siu_buffer_align (void *bstart, void *bend, tint log2a, tuint *rsize);
void *siu_buffer_align_large (void *bstart, void *bend, tint log2a, tulong *rsize);
tint siuTdmCntrl3(siuInst_t *inst, tint timeSlot, bool enable, tuint trans_id,
                  halTdmControl_t *halTdmControl);

void siuNduStart(void);

/* siunduif.c    */
tint AM_ProcessData(tuint extID, xferPktInfo_t *pktOut);
void AM_ProcessMsg(NDU_msgPriority_e level, mgb_t *msgBuf);
void siuUtnduInit();
void siuHdnduInit();
void siuNduLegInit();
void siuNduv1Init();
void siuNduStart();
tuint siuNduSendMsg(mgb_t *msg);
tuint siuNduSendMsgPriority(mgb_t *msg,NDU_DPRMsgType_t priority);
void siuNduSendSimpleMsgPriority(mgb_t *msg, NDU_DPRMsgType_t priority);

tuint siuNduGetFree (NDU_DPRMsgType_t queue_id);
tint siuNduSendPkt(void *siuInst, xferPktInfo_t *pktIn);
tint siuNduSendPktCore(siuInst_t *inst, xferPktInfo_t *pktIn);
void siuCopyMsgInit (diagApp_t *appInit);
/* siu6608 */
tint siuC6608CoreBoot (tuint core, tulong entry, tint *ecode);
void siuInitCoreMhz6608 (tuint mhz);
typedef void (*fcnGetStats_t) (void *siuInst, bool reset, tuint trans_id, tuint priority);
void siu_open_channel_block_codecs (siuInst_t *inst, tint vctype);

#if (defined(HDVIDEO) || defined(HDCT))
tuint siu_video_stm_search (void *siuInst, tuint type, bool subtypeValid,
                            tuint subtype, tuint payloadlen);
tuint siu_rtp_search_video_profile (siuInst_t *inst, tuint pt);
void siuCreateVideoCh (tint chnum);
void siuVideoMode (siuInst_t *inst, mgb_t *message);
void siu_msm_video_to_idle (siuInst_t *inst, mgb_t *microMessage);
void siuCommitStateVideo (siuInst_t *inst,  mgb_t *message);
#endif
/* Two arrays of function pointers used for statistics request/response
 * operation */
/* The default cell size configuration. Can be overridden at run time */
#define SIU_DEFAULT_CELL_SIZE       58
/* Macros to compile out rtp, aal2 and aal1 related code */
#ifdef RTP
#define SIU_IS_PROTO_RTP(x)      (x->protocol_type == MGB_PT_RTP)
#define SIU_IS_FAX_PROTO_RTP(x)  0
#define SIU_IS_FAX_PROTO_UDPTL(x) 0
#else
#define SIU_IS_PROTO_RTP(x)      (0)
#define SIU_IS_FAX_PROTO_RTP(x)  (0)
#define SIU_IS_FAX_PROTO_UDPTL(x)  (0)
#endif

#ifdef AAL2
#define SIU_IS_PROTO_AAL2(x)     (x->protocol_type == MGB_PT_AAL2)
#define SIU_IS_FAX_PROTO_AAL2(x)  (x->faxProtocolType == MGB_PT_AAL2)
#else
#define SIU_IS_PROTO_AAL2(x)     (0)
#define SIU_IS_FAX_PROTO_AAL2(x) (0)
#endif

#ifdef AAL1
#define SIU_IS_PROTO_AAL1(x)     (x->protocol_type == MGB_PT_AAL1)
#define SIU_IS_FAX_PROTO_AAL1(x)  (x->faxProtocolType == MGB_PT_AAL1)
#else
#define SIU_IS_PROTO_AAL1(x)     (0)
#define SIU_IS_FAX_PROTO_AAL1(x) (0)
#endif

#ifdef IU
#define SIU_IS_PROTO_IU(x)      ((x)->protocol_type == MGB_PT_IU)
#define SIU_IS_FAX_PROTO_IU(x)  ((x)->faxProtocolType == MGB_PT_IU)
#define SIU_IS_IU_OVER_IP(x)    ((x)->proto.u.iu.p_rtp_params != NULL)
#else
#define SIU_IS_PROTO_IU(x)      (0)
#define SIU_IS_FAX_PROTO_IU(x)  (0)
#define SIU_IS_IU_OVER_IP(x)    (0)
#endif

#ifdef NEU
#define SIU_IS_NEU_PRESENT(x)    (x->neuInst != NULL)
#else
#define SIU_IS_NEU_PRESENT(x)    (0)
#endif
/******************************************************************************
 * DATA DEFINITION: Memory read pacing state machine
 ******************************************************************************
 * DESCRIPTION: Contains reformated READ_MEM message.  Any time msg_rate
 *              is nonzero, the isr-rate task will send msg_rate messages
 *              per isr period.
 *
 *****************************************************************************/

typedef struct siuReadMemPacing_s {
  tulong cur_addr;      /* address to read in mgmicdsp.h units */
  tulong rem_len;       /* Remaining length to read in mgmicdsp.h units */
  tint   type;          /* 0: data memory; 1: program memory; 2: DMA; 3:I/O
                         * (see MEM_TYPE_* in mgmicdsp.h) */
  tuint  msg_rate;      /* Max number of responses per ISR. */
  tuint  trans_id;      /* Response trans_id */
  tuint  isr_counter;   /* Counts isr's, for pacing */
} siuReadMemPacing_t;

#ifdef PROFILE_IDLE_MIPS
typedef struct siuIdleMipsReport_s {
  tulong idleMIPSCycles;
  tulong isr_clock;
  tulong seq_num;
  tulong numRxTasksProcessed;
  tulong numRxPkts;
  tulong numTxPkts;
  tulong numReportsMissed;
  bool   reportPending;
} siuIdleMipsReport_t;
#endif
/******************************************************************************
 * DATA DEFINITION: SIU Global Context, kept at end to see all definitions.
 ******************************************************************************
 * DESCRIPTION: Contains state information for SIU module.
 *
 *****************************************************************************/
/* Include function that require siuinst definition */

typedef struct {
  tuint ID;                                 /* SIU Module ID (initialized in
                                             *  siuInit() */

  tuint state;                              /* SIU State */

  tuint dspId;      /* Multi DSP Id  */
  tuint coreId;     /* Multi Core Id */

  bool hw_config_received;
  /* Master Debug Info function that merges exception and debug fuctionality.
   * (initialized in siuInit() */
  void  (*debugInfo)(tuint ModuleID, tuint msgType, tuint msgCode, 
                     tuint msgLength, tuint *msg);

  tuint   maxNumberChannels;      /* max. numbers of channels */
  tuint   maxFullChannels;        /* max. number of TDM-PKT full channels */    
  tuint   numActiveChannels;      /* # channels not closed at this time */

  tuint   timer_period;           /* Power management timer period */
  tuint   wake_up_interrupt_mask; /* Interrput mask for power management */
  
  tint    msgDelayCnt;            /* Number of 2.5ms segments message processing
                                   * has delayed. */

  bool    FreeMIPS;               /* TRUE: Free MIPS available mode. */

  /* Paced memory read */
  siuReadMemPacing_t readMemPacing;

  void    (*siuInitCoreMhz)(tuint mhz); /* Function to configure mips table */
  siuFIFO_t msgFifo;         /* Normal Message FIFO Control Structure */
  siuFIFO_t sigFifo;         /* Signaling Message FIFO Control Structure */
  siuFIFO_t lowFifo;         /* Low priority Message FIFO Control Structure */

  tuint   halHeapUsed;

  tuint             cellSize;
  siuDynBufAlloc_t dynBufAlloc; /* dynamic buffer allocation */
  siuDynHeapAlloc_t dynHeapAlloc; /* Dynamic heap allocation */

  tuint pcm5msCounter;          /* isr task rate counter */
  tuint pcmMsgTaskIsrCounter;   /* isr msg task rate counter */
  tuint prdLastPhase;           /* Last sub-rate phase processed */
  tuint phaseWrap;              /* Tracks missed tasks */
  tuint msgTaskCounter;         /* number of msg tasks */
  halApi_t  *halApi;            /*Handle to the HalAPI structure         */   
  tint       halIdMsg;          /* Hal port ID for outgoing messages     */

  bool       errorReport;       /* TRUE: All error reporting is done normally 
                                 * FALSE: No error reporting - applies to 
                                 * broadcast messages ONLY */

  siuHalt_t halt;               /* Halt DSP register */

  bool       msg_priority_config_support; /* Indicates whether NDU supports 
                                             configurable message priority.  
                                             At present only utopia has it */ 

#define SIU_GMPFORMAT_UNFORMATTED 0  /* GMP is not formatted */
#define SIU_GMPFORMAT_DOFORMAT    1  /* GMP will be formatted after this message is done */
#define SIU_GMPFORMAT_FORMATTED   2  /* GMP is formatted */
  bool       gmpFormatState;    /* State of formatting remaing GMP (happens
                                 * on first open channel or redundancy
                                 * activate
                                 */

  ipcContext_t    ipcContext;   /* IPC information between application and BIOS */

  tuint current_working_chnum;  /* The current working SIU instance. Always use 0 in VCT */

#if defined(RELMSG_SUPPORT)  
  tuint seqNum;   /* Bit 15:8 expected sequence number for high priority message 
                   * Bit  7:0 expected sequence number for normal and low priority_message */
#define SIU_GET_MSG_SEQ_NUM_HI()          ((siuContext.seqNum & 0xFF00) >> 8)
#define SIU_GET_MSG_SEQ_NUM()             (siuContext.seqNum & 0x00FF)
#define SIU_SET_MSG_SEQ_NUM_HI(newSeqNum) siuContext.seqNum = ((siuContext.seqNum & 0x00FF) | ((newSeqNum) << 8))
#define SIU_SET_MSG_SEQ_NUM(newSeqNum)    siuContext.seqNum = ((siuContext.seqNum & 0xFF00) | ((newSeqNum) & 0x00FF))
#define SIU_MAX_MSG_SEQ_NUM            255 
  UINT16 msg_burst_size;           /* The maxmium Micro-to-DSP burst size 
                                    * Note: It is used for the reliable message protocol
                                    *       0: Disable the reliable message protocol
                                    */
#endif

#if (defined(HDVIDEO) || defined(HDCT))
/* Encoder puts the encoded frame in tx_video_frame_scratch and used by VPPU/RFC to ship out */
  word *siu_tx_video_frame_scratch;
  tulong max_tx_video_frame_size;
/* RFC forms a video frame from multiple packets and uses this buffer to feed the decoder */
  word *siu_rx_video_frame_scratch;
  tulong max_rx_video_frame_size;
  word *siu_video_tsu_scratch;       /* output for TSU */
  tulong max_video_tsu_scratch_size; /* output for TSU */
  
  siuVidShmHandle_t   codecShmHandle; /* Codec shared memory handle */
  bool rmanConfigFlag;                /* Flag indicating if RMAN configuration is needed */

#define SIU_VIGDK_BUILD_ID0_QCIF_CIF 0
#define SIU_VIGDK_BUILD_ID1_D1       1
  tuint buildID;

#ifdef PROFILE_IDLE_MIPS
  tulong siuIdleMIPSCycles;
  tulong siuAccIsrCycles;
  bool siuMeasuringIdleTimeFlag;
  bool siuUpdateIdleReportFlag;
  /* report that goes out every 1 sec */
  siuIdleMipsReport_t idleReport;
#endif

  /* BEGIN - Stats accumulated between Idle reports */
  tulong numRxTasksProcessed;
  /* END - Stats accumulated between Idle reports */
#endif

#ifdef NDU_GMAC
#define SIU_MAC_BOOT_BUFFER_MSG     0
#define SIU_MAC_BOOT_BUFFER_VCH     1
#define SIU_MAC_BOOT_BUFFER_SINGLE  2
  tuint macBufferAssign;           /* Mac buffer assignment */
  tuint numLateMsgProcess;         /* The number of messages which are late
                                    * processed because of no or too few CPPI
                                    * buffers available */
#endif

/* Please leave these at the bottom.  It allows for easy task drop debug */
/* ===================================================================== */
  tuint      numLateTxSends;    /* Number of late transmits */
  tuint      numLatePlayouts;   /* Number of late playouts */
  tuint      core_mhz;          /* CPU core MHz */
} siuContext_t;

#ifdef SIU_NDU_HETERO_PKT_IFACE
#define SIU_PKT_GET_IDX_IFACE(idx_iface,mask,shift) ((idx_iface & mask)>>shift)
#define SIU_PKT_SET_IDX_IFACE(idx_iface1,idx_iface2,mask,shift) (idx_iface1 | (idx_iface2 << shift) & mask)
#endif
extern siuContext_t siuContext;           /* SIU Global Context (siumain.c) */

static inline void siuAmLinkNdu(siuInst_t *inst, tint (*receiveOut)(void*, xferPktInfo_t *), void *targetInst)
{
	inst->amInst.RxOutSpec.nduReceiveOut = receiveOut;
	inst->amInst.RxOutSpec.targetInst    = targetInst;
}

static inline void siuAmUnlinkNdu(siuInst_t *inst)
{
	inst->amInst.RxOutSpec.nduReceiveOut = NULL;
	inst->amInst.RxOutSpec.targetInst    = NULL;
    inst->amInst.pktLoopback = NDU_CTRL_SND_LOOPBACK_OFF | NDU_CTRL_RCV_LOOPBACK_OFF;

}
tulong  siuTmrCalcDTime (const tulong new_time, const tulong old_time);
tulong  siuTmrGetTime   (void);
void    siuTmrReset     (void);
void siuDebugInternal (uint id, char *str, bool queue);
void siu_dsp_halt(uint id, char *s);
void siuVctRun(void);

/* nothing after this point */
