/******************************************************************************
* FILE PURPOSE: Configure network addresses.
******************************************************************************
* FILE NAME:   siuVctNeu.c
*
* DESCRIPTION: Configure network addresses.
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <ti/mas/types/types.h>
#include "common/siuloc.h"
#include "ggcfg.h"
#include <stdio.h>
#include <string.h>

#include "siuVctNeu.h"

/******************************************************************************
* FUNCTION PURPOSE: Configure EMAC port with spciefied MAC address.
******************************************************************************
* DESCRIPTION: Configure EMAC port with spciefied MAC address.
*
* tint siuVctMacAddrCfg(
*   const char *    macAddr,    - MAC address of DSP core
* )
*****************************************************************************/
tint siuVctMacAddrCfg(const char *macAddr)
{
  halGmacControl_t  macCtrl;
  tint              halId;
  char              str[20];
  char              *pch;
  int               i = 0, ntok = 0, val;

  if( (macAddr == NULL) || (strlen(macAddr) == 0) )
    return 0;

  macCtrl.code = HAL_GMAC_CTL_CODE_ID_FROM_PORT;
  macCtrl.u.portId.physPort = 0;

  (*siuContext.halApi->gmac->halGmacControl)(&macCtrl);

  halId = macCtrl.u.portId.halId;

  macCtrl.code = HAL_GMAC_CTL_CODE_ADD_MAC_ADDR;
  macCtrl.u.addr.halId = halId;

  strcpy(str,macAddr);
  i=0; ntok = 0;
  pch = strtok (str,":-");
  ntok++;
  while (pch != NULL) {
    sscanf(pch, "%x", &val);

    if(i & 1)
      macCtrl.u.addr.macAddr[i>>1] |= (tuint) (val & 0xFF);
    else 
      macCtrl.u.addr.macAddr[i>>1] = (tuint) ((val & 0xFF) << 8);

    i++;
    pch = strtok (NULL, " :-");
    ntok++;
    if (ntok > 8) {
        printf("Invalid local MAC address... exiting\n");
        return -1;
    }
  }
  
  if ((*siuContext.halApi->gmac->halGmacControl)(&macCtrl) != HAL_SUCCESS) {
    printf("Error configuring EMAC for local MAC address... exiting\n");
    return -1;
  }

  return 0;
}


/******************************************************************************
* FUNCTION PURPOSE: Configure NEU TX for TFTP
******************************************************************************
* DESCRIPTION: Configure NEU TX for TFTP.
*
* void siuVctConfigureNeu(
*   const char *    serverMacAddress,   - MAC address of TFTP server
*   const char *    serverIpAddress,    - IP address of TFTP server
*   const char *    localMacAddress,    - MAC address of DSP core
*   const char *    localIpAddress,     - IP address of DSP core
* )
*****************************************************************************/
void siuVctConfigureNeu(const char *serverMacAddress, const char *serverIpAddress, 
                        const char  *localMacAddress, const char  *localIpAddress)
{ 
  void             *neuInst;
  neuCtrl_t         neuCtrl;
  neuTxHdrUpdate_t *newHdr = &(neuCtrl.u.newTxHdr);
  char str[20];
  char *pch;
  int i=0, j, ntok = 0, val;

#ifndef PA_SS
  siuVctMacAddrCfg(localMacAddress);
#endif

  for ( j=0; j<2; j++) {
    neuInst = siuInst[j].neuInst;

    neuCtrl.code = neu_CTRL_SET_TX_HDR;
    newHdr->validParams = 0;

    if( serverMacAddress  && (strlen(serverMacAddress) > 0) ) 
    {
      strcpy(str,serverMacAddress);
      i=0; ntok=0;
      pch = strtok (str,":-");
      ntok++;
      while (pch != NULL) {
          sscanf(pch, "%x", &val);
          newHdr->remMacAddress[i++] = val;
          pch = strtok (NULL, " :-");
          ntok++;
          if (ntok > 8) {
              printf("Invalid remote MAC address... exiting\n");
              return;
          }
      }
      newHdr->validParams   |= neu_TX_HDR_VALID_REM_MAC;
    }

    if( localMacAddress && (strlen(localMacAddress) > 0) ) 
    {
      strcpy(str,localMacAddress);
      i=0; ntok = 0;
      pch = strtok (str,":-");
      ntok++;
      while (pch != NULL) {
          sscanf(pch, "%x", &val);
          newHdr->locMacAddress[i++] = val;
          pch = strtok (NULL, " :-");
          ntok++;
          if (ntok > 8) {
              printf("Invalid local MAC address... exiting\n");
              return;
          }
      }
      newHdr->validParams   |= neu_TX_HDR_VALID_LOC_MAC;
    }

    if( serverIpAddress && (strlen(serverIpAddress) > 0) ) 
    {
      strcpy(str, serverIpAddress);
      i=0; ntok = 0;
      pch = strtok (str,".");
      ntok++;
      while (pch != NULL) {
          sscanf(pch, "%d", &val);
          newHdr->remIpAddress[i++] = val;
          pch = strtok (NULL, ".");
          ntok++;
          if (ntok > 6) {
              printf("Invalid remote IP address... exiting\n");
              return;
          }
      }
      newHdr->validParams   |= neu_TX_HDR_VALID_REM_IP;
    }

    if( localIpAddress && (strlen(localIpAddress) > 0) ) 
    {
      strcpy(str, localIpAddress);
      i=0; ntok = 0;
      pch = strtok (str,".");
      ntok++;
      while (pch != NULL) {
          sscanf(pch, "%d", &val);
          newHdr->locIpAddress[i++] = val;
          pch = strtok (NULL, ".");
          ntok++;
          if (ntok > 6) {
              printf("Invalid local IP address... exiting\n");
              return;
          }
      }
      newHdr->validParams   |= neu_TX_HDR_VALID_LOC_IP;
    }

    (*gg_neu_ctable.neuControl)(neuInst, &neuCtrl);
  }
}

/******************************************************************************
* FUNCTION PURPOSE: Configure NEU with remote port 69.
******************************************************************************
* DESCRIPTION: Configure NEU with remote port 69. This is the port required 
*               to initiate TFTP.
*
* tint siuVctMacAddrCfg(
*   const char *    macAddr,    - MAC address of DSP core
* )
*****************************************************************************/
void siuVctConfigureTftpUdp(void *neuInst)
{
  neuCtrl_t neuCtrl;

  neuCtrl.code                      = neu_CTRL_SET_TX_HDR;
  neuCtrl.u.newTxHdr.remUdpPortNum  = 69;
  neuCtrl.u.newTxHdr.validParams    = neu_TX_HDR_VALID_REM_UDP;

  (*gg_neu_ctable.neuControl)(neuInst, &neuCtrl);
}

/* nothing past this point */
