/******************************************************************************
 * FILE PURPOSE: API to move DIM messages from external memory to the 
 *                message FIFO.
 ******************************************************************************
 * FILE NAME: siuVigdkIbl.c
 *
 * DESCRIPTION: 
 *
 * (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <ti/mas/types/types.h>
#include <ti/mas/iface/ifmicdsp/mgmicdsp.h> /* mgb_t structure */

#include "utilfifo.h"
#include "ndu.h"

#include "siuVigdkIbl.h"
#include "common/siuloc.h"

extern cregister volatile unsigned int DNUM;

extern NDU_gmacContext_t  NDU_gmacContext;

extern volatile tuint *siuVigdkMsgsPtr;
extern volatile tuint *siuVigdkMsgsEnd;

/******************************************************************************
 * FUNCTION PURPOSE: Returns if the build is for IBL demo or not.
 ******************************************************************************
 * DESCRIPTION: Returns TRUE if the build is IBL.
 *              Returns FALSE if the build is not IBL.
 *****************************************************************************/
tbool siu_is_ibl(void)
{
  return (tbool) siuVigdkIblDemoSelectionPtr[0];
}

/******************************************************************************
 * FUNCTION PURPOSE: Replace hardcoded remote MAC address with new one.
 ******************************************************************************
 * DESCRIPTION: This function takes searches all of the configuration messages 
 *              to replace hardcoded remote MAC address with a new one.
 *
 *  siuVigdkIblMsgFifo(void)
 *
 *****************************************************************************/
void siuVigdkIblRemMacCfg(void)
{

  tword *msgBuf           = (tword *)siuVigdkMsgsPtr;
  tword *msgEnd           = (tword *)siuVigdkMsgsEnd;

  while (msgBuf < msgEnd-5) {
    if(!memcmp(msgBuf,siuVigdkIblDemos[siuVigdkIblDemoSelectionPtr[0]].mac_address,6))
      memcpy(msgBuf,siuVigdkIblDemoMacAddress,6);

    msgBuf++;
  }

//#endif /* SIU_VIGDK_NEW_REM_MAC_ADDR */
}

/******************************************************************************
 * FUNCTION PURPOSE: Place built-in DIM message into the message FIFO.
 ******************************************************************************
 * DESCRIPTION: This function takes a built-in DIM message and places it in 
 *              the message FIFO.
 *
 *  siuVigdkIblMsgFifo(
 *    mgb_t *msg            - pointer to DIM message.
 *  )
 *****************************************************************************/
tuint siuVigdkIblMsgFifo(mgb_t *msg)
{
  NDU_fifo_t *msgFifo = &NDU_gmacContext.msgFifo[NDU_NORMAL_MESSAGE];
  tuint len = 0;

  if(msgFifo->inIndex == msgFifo->outIndex) {
    
    /* Write message header to FIFO */
    utilFifoWrite(msgFifo->start, (tuint *)&msg->header, &msgFifo->inIndex, 
               &msgFifo->outIndex,msgFifo->fifolen, MGB_HEADER_SIZE);

    /* Write message body to FIFO */
    if (msg->header.length > MGB_HEADER_SIZE)
      utilFifoWrite(msgFifo->start, (tuint *)&msg->msg_body, &msgFifo->inIndex, &msgFifo->outIndex,
                       msgFifo->fifolen, msg->header.length - MGB_HEADER_SIZE);

    msgFifo->free -= msg->header.length;
    len += msg->header.length;
  }

  return(len);
}


/******************************************************************************
 * FUNCTION PURPOSE: Place built-in DIM message into the message FIFO.
 ******************************************************************************
 * DESCRIPTION: This function checks to see if the next built-in DIM message's
 *              core ID matches the core this function is running on. If so, 
 *              the message will be place in the FIFO to be picked up during 
 *              the message processing task. The global message pointer will 
 *              be updated so that the following message will get processed by
 *              the appropriate core.
 *
 *  siuVigdkIblCfg(void)
 *
 *****************************************************************************/
volatile tuint siuVigdkMsgsFirstTime = 1;
#pragma DATA_SECTION(siuVigdkMsgsFirstTime,".siuVigdkMsgsFirstTime");
extern cregister volatile unsigned int TSCL;
tuint my_turn = 0;
void siuVigdkIblCfg(void)
{
  tuint length;
  volatile tulong tscl_old;

  if( !siu_is_ibl() )
    return;

  if( siuVigdkMsgsPtr < siuVigdkMsgsEnd ) {
    siu_invalidate_cache();
  }

  if(siuVigdkMsgsFirstTime){
    if(DNUM==0) {
      /* Core 0 initializes the message pointers and remote MAC address */
      siuVigdkMsgsPtr = siuVigdkIblDemos[siuVigdkIblDemoSelectionPtr[0]].msgs_start_ptr;
      siuVigdkMsgsEnd = siuVigdkIblDemos[siuVigdkIblDemoSelectionPtr[0]].msgs_end_ptr;
      siuVigdkIblRemMacCfg();
      siuVigdkMsgsFirstTime = 0;
      siu_osal_wbinv_cache_all();
    }
    return;
  }

  if(siuVigdkMsgsPtr < siuVigdkMsgsEnd) 
  {
    /* Check if current message is for this core. */
    if(siuVigdkMsgsPtr[0] == DNUM) 
    {
      /* Ensure that there are no race conditions in message processing. */
      if(my_turn == 0) {
        tscl_old = TSCL;
        my_turn = 1;
      }      

      /* If time to process message, place into message FIFO to be picked up in message task. */
      if( ((tulong)(TSCL - tscl_old) > (tulong)500) && (length = siuVigdkIblMsgFifo((mgb_t *)&siuVigdkMsgsPtr[1]))) {
        /* Update message pointer. */
        my_turn = 0;
        siuVigdkMsgsPtr = &siuVigdkMsgsPtr[length + 1];

        siu_osal_wbinv_cache((void *)&siuVigdkMsgsPtr, sizeof(siuVigdkMsgsPtr), TRUE);
      }
    }
  }
}

