/******************************************************************************
 * FILE PURPOSE: API to move DIM messages from external memory to the 
 *                message FIFO.
 ******************************************************************************
 * FILE NAME: siuVigdkIblDemos.c
 *
 * DESCRIPTION: 
 *
 * (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <ti/mas/types/types.h>

#include "siuVigdkIbl.h"

/*
 *  This is were the IBL demo configuration gets defined. Each of the 
 *  following header files contains the configuration messages for a single
 *  demo. Only ONE header file should be included in an IBL demo build.
 *
 *  If no header file is included, the normal Video GDK image will be built.
 */
#include "video/ibl_demos/siuVigdkIblDemoNAB.h"
#include "video/ibl_demos/siuVigdkIblDemo720to1080p.h"
#include "video/ibl_demos/siuVigdkIblDemoQCIF_IBC.h"

/* 
 * If the configuration messages are defined, include the diagAppMem in build.
 */
#include "video/ibl_demos/siuVigdkDiagAppMem.h"


#pragma DATA_SECTION(siuVigdkIblDemoSelection,".siuVigdkIblDemoSelection");
const tulong  siuVigdkIblDemoSelection = 0;

volatile tulong *siuVigdkIblDemoSelectionPtr = (volatile tulong *)&siuVigdkIblDemoSelection;

#pragma DATA_SECTION(siuVigdkIblDemoMacAddress,".siuVigdkIblDemoMacAddress");
const tword   siuVigdkIblDemoMacAddress[6] = {0xDE,0xC0,0xDE,0xFA,0xCE,0xEE};


#ifdef SIU_VIGDK_DIAG_APP_MEM
#pragma DATA_SECTION(siuVigdkDiagAppMem, ".siuVigdkDiagAppMem");
#pragma DATA_SECTION(siuVigdkDiagAppMemIbl, ".siuVigdkDiagAppMemIbl");
tulong siuVigdkDiagAppMem[];
const tulong siuVigdkDiagAppMemIbl[] = {SIU_VIGDK_DIAG_APP_MEM};
#else
#pragma DATA_SECTION(siuVigdkDiagAppMem, ".siuVigdkDiagAppMem");
tulong siuVigdkDiagAppMem;
#endif

#pragma DATA_SECTION(siuVigdkIblDemo1,".siuVigdkIblDemos");
const tuint siuVigdkIblDemo1[] = {SIU_VIGDK_IBL_DEMO1};

#pragma DATA_SECTION(siuVigdkIblDemo2,".siuVigdkIblDemos");
const tuint siuVigdkIblDemo2[] = {SIU_VIGDK_IBL_DEMO2};

#pragma DATA_SECTION(siuVigdkIblDemo3,".siuVigdkIblDemos");
const tuint siuVigdkIblDemo3[] = {SIU_VIGDK_IBL_DEMO3};

#pragma DATA_SECTION(siuVigdkIblDemos,".siuVigdkIblDemos");

const siuVigdkIblDemoTable_t siuVigdkIblDemos[] = {
  {
    NULL,
    NULL
  },
  {
    (tuint *) (&siuVigdkIblDemo1[0]),
    (tuint *) (&siuVigdkIblDemo1[0] + sizeof(siuVigdkIblDemo1)/2),
    {SIU_VIGDK_IBL_DEMO1_OLD_REM_MAC_ADDR}
  },
  {
    (tuint *) (&siuVigdkIblDemo2[0]),
    (tuint *) (&siuVigdkIblDemo2[0] + sizeof(siuVigdkIblDemo2)/2),
    {SIU_VIGDK_IBL_DEMO2_OLD_REM_MAC_ADDR}
  },
  {
    (tuint *) (&siuVigdkIblDemo3[0]),
    (tuint *) (&siuVigdkIblDemo3[0] + sizeof(siuVigdkIblDemo3)/2),
    {SIU_VIGDK_IBL_DEMO3_OLD_REM_MAC_ADDR}
  }
};



#pragma DATA_SECTION(siuVigdkMsgsPtr, ".siuVigdkIblDemos");
volatile tuint *siuVigdkMsgsPtr = NULL;

#pragma DATA_SECTION(siuVigdkMsgsEnd, ".siuVigdkIblDemos");
const tuint *siuVigdkMsgsEnd = NULL;


