/******************************************************************************
 * FILE PURPOSE: MIPS tuning functions used by SIU.
 ******************************************************************************
 * FILE NAME:   siuVigdkMIPS.c
 *
 * DESCRIPTION: MIPS control functions.
 *  
 *
 * (C) Copyright 2009, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"

/* SIU header files */
#include "common/siu.h"
#include "common/siuloc.h"
#include "siuport.h"
#include "vtk.h"
/* GG Header Files */
#include "ggcfg.h"

/******************************************************************************
 * FUNCTION PURPOSE: Disable certain features to free up some MIPS.
 ******************************************************************************
 * DESCRIPTION:              
 *****************************************************************************/

void siuVigdkReduceMIPS (bool reduce_mips)
{
  int        i;
  siuInst_t  *inst;
  vtkCtrl_t  vtkCtrl;
  
  for (i=0; i < (tint)siuContext.maxNumberChannels; i++) 
  {	/* Send request to reduce MIPS to all the channels */
    if(SIUINSTpNOTCLOSED(i))
    {
      inst = SIUINSTp(i);							
      vtkCtrl.code = reduce_mips ? VTK_REDUCE_MIPS : VTK_RESTORE_MIPS;
      vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);
    }
  }
} /* siuVigdkReduceMIPS() */

/* Nothing after this point */
