/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "common/siutmr.h"
#include "common/siuloc.h"
#include "ggcfg.h"
#include "ggswcfg.h"
#include "siutimercnst.h"
#include "siuport.h"
#ifdef GG_BIOS
#include "osal/siuOsal.h"
#endif
#include "targethw.h"
#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif
#include "rttrace.h"
#ifdef EMU_PROFILE
#include "emureg.h"
#endif
#include "siummcu.h"

/******************************************************************************
 * FUNCTION PURPOSE: Dummy function
 ******************************************************************************
 * DESCRIPTION: Assigned to the dsigtst timer tick when dsigtst is not
 *              in the program. Saves a few cpu cycles by avoiding the 
 *              compare.
 ******************************************************************************/
void siu_sub_dummy (void)
{
  /* empty */
} /* siu_sub_dummy */


/******************************************************************************
 * FUNCTION PURPOSE: Periodic function called by tdmDmaIsr
 ******************************************************************************
 * DESCRIPTION: This function is called by tdmDmaIsr and invokes acu, rcu,
 *              vpu, anu, and maintains system clock
 *
 *  void siuSubRateFcn (
 *    void *fxt,  - Application supplied call back context
 *    tuint ts)   - Current PCM sample timestamp/number.
 *
 *****************************************************************************/
void siuSubRateFcn(void *cxt, tuint ts)
{
  tint i;
#if (GG_ISR_RATE_MICRO_SECS == 500)
  /* 0.5ms ISR, increment millisecond clock every other tick */
  if (siu_isr_clock & 0x1) {
    siuTmrTick ();
  }
  siuContext.pcmMsgTaskIsrCounter++;
#elif (GG_ISR_RATE_MICRO_SECS == 2500)
  /* 2.5ms ISR, increment millisecond clock with 2ms, 3ms pattern */
  siuTmrTick ();
  siuTmrTick ();
  if (siu_isr_clock & 0x1) {
    siuTmrTick ();
  }
  siuContext.pcmMsgTaskIsrCounter += 5;
#elif (GG_ISR_RATE_MICRO_SECS == 1000)
  /* 1ms ISR, increment millisecond clock every tick */
    siuTmrTick ();

#else
#error Clock increment code needed for this ISR rate
#endif
  /* Increment Message Delay Count */
  siuContext.msgDelayCnt++;
  /* increment ISR counter/clock */
  siu_isr_clock++;

  for(i=0; i<siuContext.numActiveChannels; i++) {
    siuInst_t *locSiuInst = SIUINSTp(i);

    if( (SIU_GET_VIDEO_CHANNEL_TYPE(locSiuInst) == SIU_CHTYPE_DECODER) && mmcuHasEnoughData(locSiuInst->stateInst.video.mmcuInst)) 
    {
      if(SIU_GET_MMCU_HDR_PARSED(locSiuInst)) {
        tlong time_elapsed = (tlong) siu_isr_clock - locSiuInst->prev_task_post;

        if( time_elapsed > ((1000/SIU_GET_MMCU_FPS(locSiuInst)) - 1) ) {
          siuRxVideoPost(locSiuInst, NULL);
          locSiuInst->prev_task_post = siu_isr_clock;
        }

      } else if(SIU_GET_MMCU_HDR_TASK_POST_PENDING(locSiuInst)) {
        siuRxVideoPost(locSiuInst, NULL);
        SIU_SET_MMCU_HDR_TASK_POST_DONE(locSiuInst);
      }

    } /* if(mmcuHasEnoughData(locSiuInst->stateInst.video.mmcuInst)) */
    else if ( (SIU_GET_VIDEO_CHANNEL_TYPE(locSiuInst) == SIU_CHTYPE_ENCODER) && SIU_GET_MMCU_HDR_PARSED(locSiuInst) )
    {
      /* Shipout TS packets */
      mmcuShipout(locSiuInst->stateInst.video.mmcuInst);
    }
  } /* for(i=0; i<siuContext.numActiveChannels; i++) */

  /* Tick stagger clock every 2.5 ms and also update MIPS tracking if enabled */
  if(siu_2_5msIsrDecimate) {
    siu_2_5msIsrDecimate --;
  }
  else {
    
  
#if (GG_ISR_RATE_MICRO_SECS == 500)  
    siu_2_5msIsrDecimate = 4;
#elif (GG_ISR_RATE_MICRO_SECS == 2500)
    siu_2_5msIsrDecimate = 0;    
#elif (GG_ISR_RATE_MICRO_SECS == 1000)
   if(siu_stagger_clock & 0x1) 
     siu_2_5msIsrDecimate = 2;
   else
     siu_2_5msIsrDecimate = 1;
#else
#error ISR decimate rate needed for this ISR rate
#endif
    siu_stagger_clock++;
  }

#ifdef PROFILE_IDLE_MIPS
  if (siu_isr_clock % 1000 == 0) {
   /* every 1 sec */
   if (siuContext.siuMeasuringIdleTimeFlag == FALSE) {
      /* We are not currently in idle loop, update Idle report */
       siuUpdateIdleReport();
   } else {
      /* We are currently in IDLE loop when this ISR is serviced. We update IDLE report once we 
         get out of IDLE loop so that the Idle Cycles are accurately measured. Set the updateIdleReportFlag 
       */
      siuContext.siuUpdateIdleReportFlag = TRUE;
   }
  }
#endif

  /* Increment 5ms isr task counter */
  siuContext.pcm5msCounter += 1;
  /* reset 5ms isr task counter */
  if (siuContext.pcm5msCounter >= ggConfig.pcmIntPer5ms) {
    siuContext.pcm5msCounter = 0;
  }

  /* Update message counter */
  while (siuContext.pcmMsgTaskIsrCounter >= GG_MSG_PROCESS_PERIOD) {
    siuContext.pcmMsgTaskIsrCounter -= GG_MSG_PROCESS_PERIOD;
      siuContext.msgTaskCounter++;
  }

  /* Update phase wrap */
  if (siuContext.pcm5msCounter == siuContext.prdLastPhase)
    siuContext.phaseWrap += 1;

  /* Flush rttrace */
  if (utlRtTraceContext.clock) {
    utlRtTraceContext.clock--;
    if (!utlRtTraceContext.clock) {
      /* Switch buffer halfs to flush stragglers in siuPcmPktFlush() below */
      utlRtTraceNextBuf();
    }
    /* Flush any pending traces */
    siuRtTraceFlush();
  }

  siuContext.readMemPacing.isr_counter++;
  siuContext.readMemPacing.isr_counter++;
#ifdef GG_BIOS
  /* Post the siuMain task */
  siu_osal_post_signal(SIU_OSAL_SIGNAL_HANDLE_TASKSCHEDULER);
#endif /* GG_BIOS */

} /* End of siuSubRateFcn() */

/******************************************************************************
 * FUNCTION PURPOSE: Periodic function called through main loop
 ******************************************************************************
 * DESCRIPTION: This function is called by polling the task ID posted by
 *              siuSubRateFcn(). The task ID is posted every 5ms.
 *              It invokes acu, rcu and anu.
 *
 *  void siuPRDTask (void)
 *
 *****************************************************************************/
void siuPRDTask (void)
{
  int i, cha, chb;
  bool wrap = FALSE;
  /* Every sub-frame the pcm5msCounter is incremented. In this foreground loop
   * the 5ms tasks are processed for the channels who are in the current sub-frame
   * phase. For example, at a 0.5ms interrupt rate there are 10 phases. In a 10
   * channel build 1 channel would have its 5ms task processed every 0.5ms. The
   * phase wrap is used to track when the interrupt counter passes the last
   * phase processed. This condition does indicate an out of mips condition,but
   * for now the missed tasks are simply queued up and processed when MIPS are
   * available */
  do  {
    while ((siuContext.prdLastPhase != siuContext.pcm5msCounter) || wrap)  {
      wrap = FALSE;
      cha = ggPrdCh[siuContext.prdLastPhase];
      chb = ggPrdCh[siuContext.prdLastPhase + 1];

      for (i = cha; i < chb; i++) 
      {
        if (SIUINSTpNOTCLOSED(i)) 
        {
          siuInst_t *inst = SIUINSTp(i);
          thw_change_data_page (SIU_READ_PRIMARY_DATA_PAGE(inst));
        
          
          /* RCU packet repeat processing for RTP protocol */
          if ( (SIU_IS_PROTO_RTP(inst) ||
                (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst)))
               && (gg_rcu_ctable.rcuIsr) )
            (*gg_rcu_ctable.rcuIsr) (inst->net.rcuInst);

          if((SIU_GET_VIDEO_CHANNEL_TYPE(inst) == SIU_CHTYPE_DECODER) && 
             ((tulong)(siu_timer_tick - inst->prevFrameDecTimerTick) > (tulong)inst->timeOutTicks)){
             /* temporarly flush disabled */
             //siuVideoRxFlushFrames(inst);
          }

        } /* End of !=SIU_CHST_CLOSED Loop */
      } /* End for loop */

      siuContext.prdLastPhase += 1;
      if (siuContext.prdLastPhase >= ggConfig.pcmIntPer5ms) {
        siuContext.prdLastPhase = 0;
      }



    } /* End while */

    /* At this point prdLastPhase is equal to the 5ms counter. The above loop 
     * will have made a full cycle through the sub-phases if the phasewrap was
     * set. Now the phase wrap must be decremented and the full cycle repeated
     * until the phase wrap has caught up. */
    if (siuContext.phaseWrap)  {
      siuContext.phaseWrap -= 1;
      wrap = TRUE;
    }

  } while (wrap);



} /* End of siuPRDTask */


/*************************************************************************************
 * FUNCTION PURPOSE: The periodic task processing in the idle state
 *************************************************************************************
 * DESCRIPTION: Processes tasks which must be handled when no channels are open.
 *************************************************************************************/
void siuPRDTaskIdle (void)
{

  while (siuContext.prdLastPhase != siuContext.pcm5msCounter)  {

    siuContext.prdLastPhase += 1;
    if (siuContext.prdLastPhase >= ggConfig.pcmIntPer5ms) {
      siuContext.prdLastPhase = 0;
    }

  }
} /* siuPRDTaskIdle */

/* Nothing past this point */

