/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include <string.h>

#include "types.h"
#include "vctypes.h"
#include "globstats.h"
#include "rttrace.h"

/* ICD message definitions */
#include "mgmicdsp.h"
/* SIU header files */
#include "common/siu.h"
#include "common/siuloc.h"
#include "siuport.h"
/* gg header file */
#include "ggcfg.h"
#include "ggmbuf.h"
#include "ggswcfg.h"
#include "targethw.h"
#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif


#include "ndu.h"
#include "util_s.h"

#include "siutblfn.h"

#include "common/siundu.h"
#include "common/siuCommonMpu.h"
#include "ecodrv/siuCustomEco.h"

#include "siutftp.h"

/* Interrupt messages cannot be interrupted.  Regular messages could
 * be interrupted by an interrupt message */

siuMsgAck_t siuNormalMsgAck   = { MGB_ERR_MSG_ID_UNDEF, 0 };
siuMsgAck_t siuInterruptMsgAck = { MGB_ERR_MSG_ID_UNDEF, 0 };

#ifdef SIUTBLPRG
siuDispatch_t *dispatchPtr;
siuDispatch_t *dispatchTbl;
#else
const siuDispatch_t *dispatchPtr;
const siuDispatch_t *dispatchTbl;
#endif

/******************************************************************************
 * FUNCTION PURPOSE: Create channels at DSP start-up                     
 ******************************************************************************
 * DESCRIPTION: This routine is performed at DSP start-up and initializes all
 *              SIU channels.  For all channels voice modules are created.
 *              Upon return all channels are in CLOSED state.
 *                         
 *****************************************************************************/
void siuCreateChannels ( void )
{
#ifndef SIU_DYNSUBSLOT  
  tint chnum,i;
  siuInst_t *inst;
  const tint n = siuContext.maxNumberChannels;
  
  for (i = 0, chnum = SIU_FIRSTCH; i < n ; i++, chnum++)  
  {  
    inst = SIUINSTp(i);
    /* set the data page register to point to channel's data page */
    thw_change_data_page( SIU_READ_PRIMARY_DATA_PAGE(inst) );
  }
#endif  

  #ifdef SIUTBLPRG
  siu_init_msg_dispatch();
  #endif
}  /* siuCreateChannels */

/******************************************************************************
 * FUNCTION PURPOSE: Open DSP Channel
 ******************************************************************************
 * DESCRIPTION: This routine copies the channel information, and opens PIU
 *              module in IDLE channel state configuration to get ready for 
 *              possible tone generation. In the IDLE state PIU is configured
 *              for Voice/DTMF operation. Channel state is set to SIU_CHST_IDLE
 *              before normal return.  It is assumed Channel is currently CLOSED.
 *
 *              NOTE: Will result in exception if invalid parameters are
 *                    found in message.
 * 
 *****************************************************************************/
void siuOpenChannel (siuInst_t *inst, mgb_t *message)
{
  tint slotnum,heapInstSlot;
  word *siu_heap_inst_ext_ptr;
  tint chanInt = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;

  /* Init heap instance */
  /* Get the slot for this channel */
  slotnum = chanInt;
  if (siuContext.dynBufAlloc.dynBufAllocEnable) {
    slotnum = siuContext.dynBufAlloc.siuCh2Slot[chanInt];
  }
  heapInstSlot = slotnum;

  inst->heapInst = (siuHeapInst_t *)gg_siuheap_inst[heapInstSlot];

  /*********************************/
  /* Initialize channel statistics */
  /*********************************/  
  memset ((void *)globStatsSiuGetPtr(inst->ID), 0, sizeof (siuStats_t));
  
  /* In some builds, a part of siu heap instance, comprising of profiles,
     error stats are kept in external memory. Pointers to these places
     are kept in gg_siuheap_inst_ext[]. Unlike its internal memory counterpart,
     (gg_siuheap_inst[]), this table of pointers is not accessed after
     opening a channel,so could be kept in external memory. Since this is a 
     common table across cores and the location of where the pointers point to,
     are in external shared memory, they need to be mapped through remap_mcs
     just like every other buffer, before the pointer is assigned to 
     the internal mem data structure */
  siu_heap_inst_ext_ptr=siu_remap_mcs_data(gg_siuheap_inst_ext[heapInstSlot]);
  /* Initialize the portion of stats in external memory */
  memset (siu_heap_inst_ext_ptr, 0, sizeof (siuHeapInstExt_t));

#ifdef RTP /* need some better way... */
  inst->heapInst->stats.pktStats.proto.rtp.err=
          &(((siuHeapInstExt_t *)siu_heap_inst_ext_ptr)->stats.err.pktErrStats.proto.rtp);
#endif

  siuCreateVideoCh(message->header.chnum);
  /* set the videoChannelType to NOT_CONFIGURED. VideoMode determines if channel is ENCODER or DECODER TYPE */
  SIU_SET_VIDEO_CHANNEL_TYPE(inst,SIU_CHTYPE_NOT_CONFIGURED);
  /* disable all conferencing connections */
  SIU_SET_VIDEO_ROUTE_DST_CH_ID_LD(inst,0);
  SIU_SET_VIDEO_ROUTE_DST_CH_ID_PD(inst,0);
  /* enable default PD connection */
  SIU_SET_VIDEO_ROUTE_PD_DEFAULT_CONN(inst,1);

  /* reset configuration done bitfields */
  SIU_SET_VIDEO_STATIC_CFG_DONE(inst, 0);
  SIU_SET_VIDEO_DYNAMIC_CFG_DONE(inst, 0);

  /* set coding type to NOT_CONFIGURED */
  SIU_SET_VIDEO_CODING_TYPE (inst, SIU_CHTYPE_NOT_CONFIGURED);

  /* Disable traces for this channel */
  utlRtTraceContext.enable[chanInt].enabled_bitfield = 0;

  /* Still not aware of protocol type */
  inst->protocol_type =  MGB_PT_NONE;

  siuConfigTask (inst, SIU_CHST_VIDEO);

  /* Default configuration parameters are set. */
  /* increment # of active channels (those that are NOT closed). */
  /* Note: We need do it here, not at siuCommitStateOpen() since channel-based exception
   *       may occur between "OPEN_CHANNEL" and "COMMIT_STATE" */  
  siuContext.numActiveChannels++;
  inst->pending_state = SIU_CHST_IDLE;
} /* siuOpenChannel */ 
/******************************************************************************
 * FUNCTION PURPOSE: Finish transition to idle state.
 ******************************************************************************
 * DESCRIPTION: Add necessary default parameters, according to second stage data,
 *  and already received configuration parameters from the HOST:
 * 1. If no encap config data are received between open_channel and commit_state,
 *    and build supports AAL1 mode, it defaults to it. But, if build does not 
 *   support AAL1, error indication is generated.
 * 2. According to configured protocol type, data flow on the network side is
 *    initialized.
 * 3. NEU is opened and initialized w/ default configuration parameters.
 * 4. NDU or serial packet interface is opened
 * 5. If TDM configuration has not been initialized, we provide default with
 *    timeslots set to channel number.
 ******************************************************************************/
void siuCommitStateOpen (siuInst_t *inst, mgb_t *message) {
  siuNduConfig_t   nduConfig;   /* configuration structure for NDU */
  neuCtrl_t        neuConfig;   /* configuration data for NEU */
  tuint            count;
    
  memset(&nduConfig, 0, sizeof(siuNduConfig_t));
  memset(&neuConfig, 0, sizeof(neuCtrl_t));

  /* Data flow definition on the network side */
  switch(inst->protocol_type) {
  
  #ifdef RTP
  case MGB_PT_RTP:
    /* RCU case */
    nduConfig.RxOutSpec.nduReceiveOut = gg_rcu_ctable.rcuReceiveIn;
    nduConfig.RxOutSpec.targetInst    = inst->net.rcuInst;

    break;
  #endif
  case MGB_PT_TFTP:
    /* Override RCU connection with TFTP */
    nduConfig.RxOutSpec.nduReceiveOut = tftpReceiveIn;
    nduConfig.RxOutSpec.targetInst    = inst->net.tftpInst;
    break;

  case MGB_PT_NONE:
    /* Override RCU connection with TFTP */
    nduConfig.RxOutSpec.nduReceiveOut = mmcuReceiveIn;
    nduConfig.RxOutSpec.targetInst    = inst->stateInst.video.mmcuInst;
    break;

  default:
    break;
  }

  
  if (SIU_IS_NEU_PRESENT(inst)) {
    /* If NEU is in the build, initialize configuration parameters. */
    neuConfig.u.parm.chnl.cfg_flag1              = 0;
    neuConfig.u.parm.chnl.cfg_flag2              = 0;
    neuConfig.u.parm.chnl.sar_type               = neuproto_SAR_NONE;
    neuConfig.u.parm.chnl.u.sar_param.cid           = 0;
    neuConfig.u.parm.chnl.u.sar_param.ra_time_out   = 0;
    neuConfig.u.parm.chnl.u.sar_param.max_cps_size  = 45;
    neuConfig.u.parm.chnl.alt_local_port         = 0;
    neuConfig.u.parm.chnl.alt_remote_port        = 0;
    
    count = 0;
    while(count < MGB_IP_ADDR_MAX_NUM_WORDS)
    {
        neuConfig.u.parm.chnl.alt_remote_addr[count] = 0;
        count++;
    }

    neuConfig.u.parm.chnl.report_ctrl            = 0;
    neuConfig.u.parm.chnl.report_throttle_time   = 0;
    neuConfig.u.parm.chnl.sysInst                = inst;
    neuConfig.u.parm.chnl.second_prototype       = 0;
    neuConfig.u.parm.chnl.rx_offset_bytes        = 0;
    neuConfig.u.parm.chnl.rx_payload_type        = 0;
    neuConfig.u.parm.chnl.tx_min_cell_bytes      = 0;
    neuConfig.u.parm.chnl.tx_ip_offset_bytes     = -1;
    neuConfig.u.parm.chnl.tx_udp_offset_bytes    = -1;
    neuConfig.u.parm.chnl.tx_802_3_offset_bytes  = -1;
    neuConfig.u.parm.chnl.tx_total_hdr_len_bytes = 0;
    neuConfig.u.parm.chnl.tx_udp_src_mask        = 0;
    neuConfig.u.parm.chnl.tx_udp_src_val         = 0;
    neuConfig.u.parm.chnl.enable                 = FALSE;
    siu_open_neu (message->header.chnum, &neuConfig.u.parm);

    /* Insert NEU in receive path */
    neuConfig.u.neuReceiveOutSpec.neuReceiveOut = nduConfig.RxOutSpec.nduReceiveOut;
    neuConfig.u.neuReceiveOutSpec.targetInst    = nduConfig.RxOutSpec.targetInst;
    nduConfig.RxOutSpec.nduReceiveOut           = SIU_NDU_GET_NEU_RECEIVE_IN;
    nduConfig.RxOutSpec.targetInst              = inst->neuInst; 
    neuConfig.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
    (*gg_neu_ctable.neuControl)(inst->neuInst, &neuConfig);
  }	 

  /* HPI packet transfer: Open NDU */
  siuAmLinkNdu(inst, SIU_NDU_GET_NEU_RECEIVE_IN, inst->neuInst);      

  /* Note: It is moved to siuOpenChannel()    
   * siuContext.numActiveChannels++; */
  inst->state = SIU_CHST_IDLE; 
}
/******************************************************************************
 * FUNCTION PURPOSE: General commit configuration state machine.
 ******************************************************************************
 * DESCRIPTION: Invoke function that does final transition to pending state.
 ******************************************************************************/
void siuCommitState (siuInst_t *inst,  mgb_t *message)
{

  /* Check for Wrong Commit Message. Commit Should not arrive twice */
  if(inst->state == inst->pending_state) {
    siuErrorIndication (inst->ID, message->header.trans_id, 
                      MGB_ERR_INVD_MSG_FUNC_NOT_SUPPORTED);
    return;
  }

  switch(inst->pending_state) {
  case SIU_CHST_IDLE:
    /* The only way to get to this case is by receiving a open message.
     * The idle mode message transitions directly to idle mode without 
     * waiting in pending_state IDLE
     */
    if (inst->state == SIU_CHST_CLOSED) {
       siuCommitStateOpen (inst, message);
    } else {
       inst->state = SIU_CHST_IDLE;
    }
    break;

#ifdef HDVIDEO
  case SIU_CHST_VIDEO:
    siuCommitStateVideo(inst,message);
    break;
#endif

  }/*switch(inst->pending_state)*/
}/*siuCommitState*/

/******************************************************************************
 * FUNCTION PURPOSE: Change encode or set protocol parameters on the fly 
 ******************************************************************************
 * DESCRIPTION: Change encode or set protocol parameters on the fly from 
 *
 *    void siuEncapConfig (
 *         siuInst_t *inst,    - pointer to siu's instance 
 *         mgb_t *msg)         - msg from MIC
 *
 *****************************************************************************/
void siuEncapConfig (siuInst_t *inst, mgb_t *message)
{

  if(message->msg_body.encap_config.valid_params & MGB_ENCAP_CONFIG_VALID_ENCAPSULATION)
  {
    if(inst->state == SIU_CHST_IDLE)
    {
      /* We cannot reconfigure protocol on-the-fly to a different type!. Suppress the 
       * error indication if the encapsulation supplied is same as the current type
       */
      if(inst->protocol_type != message->msg_body.encap_config.encapsulation)
      {
          siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_MSG_REJECTED_FROM_THIS_STATE);
          return;
      }
    } 
    else 
    {
      inst->protocol_type = message->msg_body.encap_config.encapsulation;

      if (SIU_IS_PROTO_RTP(inst)) {
        if(inst->net.rcuInst == NULL) {  /* NEW&OPEN for RCU go together */
          siu_new_rcu (message->header.chnum);
          siu_open_rcu (message->header.chnum);
        }
        siu_rtp_encap_config (inst, &message->msg_body.encap_config.encap_params.rtp);
      } else if(SIU_IS_PROTO_TFTP(inst)) {
        if(inst->net.tftpInst == NULL) {
          siu_new_tftp(message->header.chnum);
          siu_open_tftp(inst);
        }
        siuTftpCfg(inst,message);
      }
    }
  }
} /* siuEncapConfig */

/*****************************************************************************
 * FUNCTION PURPOSE: Handle hal messages to host
 *****************************************************************************
 * DESCRIPTION: This function address is passed to the hal message processing
 *              function. If a reply is required hal will call this function
 *              with the message body to be sent to the host. 
 *****************************************************************************/
tint siu_hal_msg_reply (void *vmsg, tint size, tint msgId, tint tId)
{
  mgb_t *msg = (mgb_t *)vmsg;

  /* Hal was instructed to leave empty space at the beginning of the the reply.
   * This is for the message header. Form the header and send the message */
  msg->header.length   = size;
  msg->header.msg_id   = msgId;
  msg->header.trans_id = tId;
  msg->header.chnum    = 0;

  siuNduSendMsg(vmsg);
  return (SIU_NOERR);
} /* siu_hal_msg_reply */
/*-----------------NORMAL (POLLLED) MESSAGE PROCESSING----------------------*/
/******************************************************************************
 * FUNCTION PURPOSE: Process messages when in SIU_ST_WORKING state.
 ******************************************************************************
 * DESCRIPTION: The SIU is in SIU_ST_WORKING state, we take new message and
 *              perform required action.  Messages with invalid channel 
 *              numbers are ignored. Exception are MGB_MD_STATUS_REQUEST and
 *              MGB_MD_MEM_TRACE message which are not channel specific.
 * 
 *  There were formerly two functions for the so-called READY state and WORKING
 *  state. THese have been collapsed to a since finction as the code was
 *  nearly identical
 *
 *****************************************************************************/
void siuProcessNormalMessage (mgb_t *microMessage, siuMsgAck_t *msgAck)
{
  siuInst_t *inst = SIUINSTp(microMessage->header.chnum - SIU_FIRSTCH);
  tint state, pending_state, ecode;
  tuint i, chProcessCnt = 1, cnt;
  tulong entry;
  bool   error;
  tuint  orig_chan;
  tint errCode;
  halControl_t halControl;
  
  orig_chan = microMessage->header.chnum;
  /* RTCP SDES string message handling uses untranslated channel */
  if ((microMessage->header.msg_id == MGB_MD_RTCP_SDES_STRING) && 
      (microMessage->header.chnum == 0)) {
#ifdef GG_INCLUDE_RTCP
    siu_rtcp_sdes_string(NULL, microMessage);
#endif
    return;
  }
  
  /* NDU stats are channel-independant.  If bitmap stats request arrives
   * with ndu stats requested, process the NDU portion as a channel
   * independant message.  No other stats are allowed if the channel is 0.
   * Global Packet Stats Request is also handled here.
   */
  if ((microMessage->header.chnum == 0) &&
      (microMessage->header.msg_id == MGB_MD_STATS_REQ) &&
      (microMessage->msg_body.stats_req.stat_req_mask_lsw == 0)) {
     bool reset = microMessage->msg_body.stats_req.stat_req_reset & 0x0001;
     tuint priority = (microMessage->msg_body.stats_req.stat_req_reset>>8)&0x00FF;
     /* Stats Request is either NETWORK stat or GLOBAL PACKET stat */
     if (microMessage->msg_body.stats_req.stat_req_mask_msw == 
         MGB_STATS_ID_NETWORK_STAT) {
       siuGetNetworkStats (NULL, reset, microMessage->header.trans_id, 
                           (siuDPRMsgType_t)priority);
       return;
     }
     else if ((microMessage->msg_body.stats_req.stat_req_mask_msw == 
              (MGB_STATS_GLOBAL_MASK | MGB_STATS_ID_GLBL_PKT_STAT)) 
              ) {
#if defined GG_INCLUDE_NET_ENCAP
       siuGetGlobalPktStats (NULL, reset, 
                                              microMessage->header.trans_id, 
                                              (siuDPRMsgType_t)priority);
#endif
       return;       
     }
  }

  if ((microMessage->header.msg_id >= MGB_MD_HAL_MSG_BEGIN) &&
      (microMessage->header.msg_id <= MGB_MD_HAL_MSG_END))  {
    (*siuContext.halApi->halMessage)(microMessage->header.msg_id, 
                                    (void *)&microMessage->msg_body,
                                     siu_hal_msg_reply,
                                     MGB_HEADER_SIZE,
                                     microMessage->header.trans_id);
    return;
  }

  if (microMessage->header.msg_id == MGB_MD_TRANS_COMMIT) {
    return;  /* TRANS_COMMIT does nothing other than provide acks */
  }

  if (microMessage->header.msg_id >= MGB_MD_CUSTOM_IFACE1_BEGIN && 
      microMessage->header.msg_id <= MGB_MD_CUSTOM_IFACE1_END) {
        /* call custom msg processing */
        siu_custom_msg_processing(inst, microMessage); 

        /*  Stub code for sending response */
        microMessage->header.trans_id = 0;
        siuNduSendMsg(microMessage);
        
        /* ACK for message */
        msgAck->ackType = MGB_ACK_TYPE_PROC | MGB_ACK_TYPE_DONE;
        return;
  }



  /* Tomahawk config*/
  if ((microMessage->header.chnum == 0) &&
      (microMessage->header.msg_id == MGB_MD_TDM_PORT_CONFIG))
  {
    #ifndef SIU_TIMER_BASED_TDM    
      siuMsgTdmPortConfig(microMessage);
    #else
      siuTsipEmulConfig(microMessage);
    #endif  
    return;
  }    

  /* Low Power Message - can change clock multiplier and/or go to low power mode */
  if (microMessage->header.msg_id == MGB_MD_DSP_LOW_POWER) {
    /* check first for change in clock multiplier */
    if (microMessage->msg_body.dsp_low_power.bit_map & MGB_NEW_CLOCK_MULT) {
      /* Program clock PLL */
      if (microMessage->msg_body.hw_config.clk_multplier != MGB_CLK_MULT_INVALID)
       if (*siuContext.halApi->halPllControl != NULL) {
          /* if VDSP Low Power HACK set clk_mult to 1/4, else process as usual */
          if(microMessage->msg_body.dsp_low_power.clk_multiplier == 0xDEAD)
             (*siuContext.halApi->halPllControl) (1,4);
          else
             (*siuContext.halApi->halPllControl) (microMessage->msg_body.dsp_low_power.clk_multiplier,1);
       }
	}
    if (microMessage->msg_body.dsp_low_power.bit_map & MGB_LOW_POWER_BIT) {
      /* DSP low power mode - only valid if no channels open */
      halControl.cmd = HAL_COMMAND_LOW_POWER;
      halControl.u.periphReset.portId = NULL;
       if (siuContext.numActiveChannels == 0 ) {
         if (*siuContext.halApi->halControl != NULL) {
            (*siuContext.halApi->halControl) (&halControl);
         }
       }
    }
    return;
  }

  if (SIU_ST_READY == siuContext.state) {
    if (microMessage->header.msg_id == MGB_MD_HW_CONFIG) {
      if (!siuContext.hw_config_received) {
        siuHwConfig (microMessage);
        siuContext.hw_config_received = TRUE;
#ifdef RELMSG_SUPPORT        
        siuContext.seqNum = 0;
#endif        
        return;
      }
    }

    if (microMessage->header.msg_id == MGB_MD_WARM_BOOT_START) {
      /* TODO resolve thwWarmBoot using HAL for C54x if necessary */
      /* thwWarmBoot() */
      siuErrorIndication (siuContext.ID, microMessage->header.trans_id, 
                        MGB_ERR_INVD_WARMBOOT);
      return;
    }
  }

  if ((SIU_ST_READY == siuContext.state) ||(SIU_ST_WORKING == siuContext.state)) {
    if (microMessage->header.msg_id == MGB_MD_VIDEO_SW_CFG) {
        siuSwConfig (microMessage);
        return;
    }
  }


  /* channel/state independent network encap. messaging */
  if ((microMessage->header.msg_id == MGB_MD_NET_ENCAP_CONTROL)
      &&(microMessage->msg_body.net_encap_control.mode == MGB_NET_ENCAP_GLOB_CFG)
      &&(gg_neu_ctable.neuNew != NULL)) {
#if defined GG_INCLUDE_NET_ENCAP
    siu_net_encap_glob_cfg((void *)microMessage);    
#endif
    return;
  }

  /* Processor specific boot config message handling */
  if ((microMessage->header.msg_id == MGB_MD_BOOT_CORE_CONTROL)  &&
       ggConfig.coreBootFcn)  {
    entry = (((tulong)microMessage->msg_body.boot_core_control.addr_msw) << 16) |
            ((tulong)microMessage->msg_body.boot_core_control.addr_lsw);
    for (i = 0; i < 16; i++)  {
      if (microMessage->msg_body.boot_core_control.enable_mask & (1 << i)) {
        if ((*ggConfig.coreBootFcn)(i, entry, &ecode)) 
          siuErrorIndication (siuContext.ID, microMessage->header.trans_id, 
                              ecode);
      } 
    } /* end for loop */
    return;
  } /* end boot core control message */

  if (microMessage->header.msg_id == MGB_MD_HALT_DSP) {
    siuHaltDsp(microMessage);
    /* Unreachable */
    return;
  }
  
  if (microMessage->header.chnum != MGB_ALL_CHANNEL_BROADCAST_ID) {
    if (microMessage->header.msg_id != MGB_MD_OPEN_CHANNEL)   
    {
      /* Catch all messages that are channel independant or require
       * untranslated channel numbers
       */
      siuGetDispatchTbl(dispatchTbl, siuLoadsiuMsgTableChIndep, 
                                     (siuDispatch_t *) siuMsgTableChIndep);
      if (!siuMsgDispatch ((long)dispatchTbl, MGB_ERR_MSG_ID_UNDEF, 
                           NULL, microMessage)) {
        return; /* Sucessfully processed signalling message */
      }
    }
  
    if (microMessage->header.msg_id == MGB_MD_DSP_NV_NET_ENCAP_CONTROL)  {
      if (microMessage->msg_body.nv_net_encap_control.valid_chan_type ==
            MGB_NV_NET_ENCAP_TYPE_STREAM) {
        tint retVal;
        retVal = siuConfigStreamingNeu((void *)&(microMessage->msg_body.nv_net_encap_control.channel));
        if(retVal != 0){
          siuErrorIndication (siuContext.ID, microMessage->header.trans_id, MGB_ERR_NEU_BAD_CONFIG);          
        }
      }

      return;
    }

    /* Check for channel creation in redundancy activation mode */
    if(microMessage->header.msg_id == MGB_MD_OPEN_CHANNEL)
    {
      /* Format the rest of GMP */
      if (siuContext.gmpFormatState == SIU_GMPFORMAT_UNFORMATTED) {
        siuContext.gmpFormatState = SIU_GMPFORMAT_DOFORMAT;
      }
    }

    /* Check channel number and ignore and send error indication if invalid */
    if ((microMessage->header.chnum < SIU_FIRSTCH) || 
        (microMessage->header.chnum > siuContext.maxNumberChannels))
    {
       if ((orig_chan >= SIU_FIRSTCH) && (orig_chan <= SIU_ALL_LASTCH)) {
         /* The channel is valid but the translation failed; the channel
          * must be closed */
         tuint ID = siuContext.ID | orig_chan;
         siuErrorIndication (ID, microMessage->header.trans_id, 
                             MGB_ERR_INVD_MSG_CHST_CLOSED);
       } else {
         siuErrorIndication (siuContext.ID, microMessage->header.trans_id, 
                             MGB_ERR_INVD_CH);
       }
       return;
    }

    chProcessCnt           = 1;    /* Number of channels to process */
    siuContext.errorReport = TRUE; /* For single channel messages error 
                                    * reporting is valid. */
  } /* NOT All TCID Broadcast Message */
  else {
    /* # of channels to process */ 
    chProcessCnt           = siuContext.maxNumberChannels;     
    siuContext.errorReport = FALSE; /* Do NOT report error for Broadcast messages */
  }

  /* Process channel specific message - it may be a single message or 
   * a broadcast message */
  for (cnt = 0; cnt < chProcessCnt; cnt++) { 
    if (chProcessCnt > 1) 
    { /* Broadcast message case - access all channels */
      inst = (siuInst_t *)SIUINSTp(cnt);      
      if(!SIUINSTpNOTCLOSED(cnt))
      { /* Skip processing of broadcast message by closed channels */
        continue;
      }
    }

    /* Set the data pager register (DMPR) to the data page of the channel before
     * running messages */
    thw_change_data_page( SIU_READ_PRIMARY_DATA_PAGE(inst) );

    /* Store the current channel state */
    state = inst->state;

    /* Store transition state */
    pending_state = inst->pending_state;

    /* Try signalling messages -- they are allowed as regular messages too */
    SIU_STATE_BEGIN();
   
    siuGetDispatchTbl(dispatchTbl, siuLoadInterruptMessageTable, InterruptMessageTable);
    if(!siuMsgDispatch((tlong)dispatchTbl,MGB_ERR_MSG_ID_UNDEF,inst,microMessage)){
      SIU_STATE_END();
      return; /* Sucessfully processed signalling message */
    }
    SIU_STATE_END();

    /* Assume no errors */
    error = FALSE;
    
    
    /*
     * Only the MGB_MD_CLOSE_CHANNEL message should be processing at the EXCEPTION state
     * while the pending_state stores the original state which is needed for message
     * dispatch.
     * Note: For HDGW 11.0 the role of pending state and state should exchange
     */
    if((microMessage->header.msg_id != MGB_MD_CLOSE_CHANNEL) &&
       (state == SIU_CHST_EXCEPTION)) {
       pending_state = state;
    }  

    /* Process message according to channel state */
    switch (pending_state)
    {
      case SIU_CHST_CLOSED:
          siuGetDispatchTbl(dispatchTbl, siuLoadChClosedStateTable, ChClosedStateTable);
          errCode = MGB_ERR_INVD_MSG_CHST_CLOSED;
          break;

      case SIU_CHST_IDLE:
          siuGetDispatchTbl(dispatchTbl, siuLoadChIdleStateTable, ChIdleStateTable);
          errCode = MGB_ERR_INVD_MSG_CHST_IDLE;
          break;
      
      case SIU_CHST_VOICE:
#if (!defined(VOICE) && !defined(VOICE_FAX))
        error = TRUE;
#else
        siuGetDispatchTbl(dispatchTbl, siuLoadChVoiceStateTable, ChVoiceStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_VOICE;
#endif
        break;

      case SIU_CHST_DTMF:
#if (!defined(VOICE) && !defined(VOICE_FAX))
        error = TRUE;
#else
        siuGetDispatchTbl(dispatchTbl, siuLoadChDtmfStateTable, ChDtmfStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_DTMF;
#endif
        break;

      case SIU_CHST_FAX_RELAY:
#if (!defined (VOICE_FAX) && !defined (FAX) )
        error = TRUE;
#else
        siuGetDispatchTbl(dispatchTbl, siuLoadChFaxRelayStateTable, ChFaxRelayStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_FAX;
#endif
        break;

#if ( defined(GG_INCLUDE_FAX_MODEM) && (!defined(GG_INCLUDE_MDMCP)) )
    case SIU_CHST_FAX_MODEM:
        siuGetDispatchTbl(dispatchTbl, siuLoadChFaxTermStateTable, ChFaxTermStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_FAX_MODEM;
      break;
#endif

#if ( defined(GG_INCLUDE_DATA_MODEM) && (!defined(GG_INCLUDE_MDMCP)) )
    case SIU_CHST_DATA_MODEM:
        siuGetDispatchTbl(dispatchTbl, siuLoadChDataModemStateTable, ChDataModemStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_DATA_MODEM;
      break;
#endif

#ifdef GG_INCLUDE_MODEM_RELAY
      case SIU_CHST_MODEM_RELAY:
        siuGetDispatchTbl(dispatchTbl, siuLoadChModemRelayStateTable, ChModemRelayStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_MODEM_RELAY;
        break;
#endif

#if ( defined(GG_INCLUDE_CLEAR_DATA) && (!defined(GG_INCLUDE_MDMCP)) )
      case SIU_CHST_CLEAR_DATA_MODE:
        siuGetDispatchTbl(dispatchTbl, siuLoadChClearDataStateTable, ChClearDataStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_CLEAR_DATA;
        break;
#endif

#ifdef BERT
      case SIU_CHST_BERT_MODE:
        siuGetDispatchTbl(dispatchTbl, siuLoadChBitErrorRateTestStateTable, ChBitErrorRateTestStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_BERT;
      break;
#endif      

#ifdef NX64
      case SIU_CHST_NX64_MODE:
        siuGetDispatchTbl(dispatchTbl, siuLoadChNx64StateTable, ChNx64StateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_NX64;
      break;
#endif  

#ifdef GG_INCLUDE_MDMCP
    case SIU_CHST_CP_IDLE:
        siuGetDispatchTbl(dispatchTbl, siuLoadChCpIdleStateTable, ChCpIdleStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_CP_IDLE;
        break;

    case SIU_CHST_CP_DTMF:
        siuGetDispatchTbl(dispatchTbl, siuLoadChCpDtmfStateTable, ChCpDtmfStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_CP_DTMF;
        break;

#ifdef GG_INCLUDE_DATA_MODEM
    case SIU_CHST_CP_MODEM_TERM:
        siuGetDispatchTbl(dispatchTbl, siuLoadChCpModemTermStateTable, ChCpModemTermStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_CP_MODEM_TERM;
        break;
#endif

#ifdef GG_INCLUDE_CLEAR_DATA
    case SIU_CHST_CP_CLEAR_DATA:
        siuGetDispatchTbl(dispatchTbl, siuLoadChCpClearDataStateTable, ChCpClearDataStateTable);
        errCode = MGB_ERR_INVD_MSG_CHST_CP_CLEAR_DATA;
        break;
#endif

#ifdef GG_INCLUDE_FAX_MODEM
  case SIU_CHST_CP_FAX_TERM:
       siuGetDispatchTbl(dispatchTbl, siuLoadChCpFaxTermStateTable, ChCpFaxTermStateTable);
       errCode = MGB_ERR_INVD_MSG_CHST_CP_FAX_TERM;
       break;
#endif
#endif /* GG_INCLUDE_MDMCP */
#if defined (AUDIO)
      case SIU_CHST_AUDIO:
       siuGetDispatchTbl(dispatchTbl, siuLoadChCpAudioStateTable, ChCpAudioStateTable);
       errCode = MGB_ERR_INVD_MSG_CHST_AUDIO;
        break;
#endif/*AUDIO*/
#if defined (HDVIDEO)
      case SIU_CHST_VIDEO:
       siuGetDispatchTbl(dispatchTbl, siuLoadChVideoStateTable, ChVideoStateTable);
       errCode = MGB_ERR_INVD_MSG_CHST_VIDEO;
        break;
#endif/*VIDEO*/
     
      default:
        error = TRUE;
        break;
    } /* Switch channel state */

    if (error) {
#ifdef HDMOVE
    NFE_ReceiveInMsg(microMessage);
#else
      siuErrorIndication (inst->ID, microMessage->header.trans_id, 
                          MGB_ERR_INVD_MESSAGE_CONFIG);
#endif
      return;
    }
    else {
      siuMsgDispatch((tlong)dispatchTbl,errCode,inst,microMessage);   
    }     
    
    /* Capture message ID of a message that initiates transition to a new state */
    /* Note:pending_state will be set to SIU_CHST_EXCEPTION for non "CLOSE_CHANNEL"
     *      message at the Channel Exception state
     */      
    if ((pending_state != SIU_CHST_EXCEPTION) && (pending_state != inst->pending_state))
       inst->state_transition_msg_id = microMessage->header.msg_id;

    /* If state transition occurs, notify the mips agent, and use message
     * ID of the message which caused state transition (not the commit)
     */
    if (state != inst->state) { 
      /* Notify the MIPS Agent if the channel state changes. */
#ifdef GG_INCLUDE_ECUMA
      if (gg_ecuma_ctable.ecumaEvtChanState != NULL) {
        gg_ecuma_ctable.ecumaEvtChanState(inst->ID, state, inst->state);
      }
#endif

      if (inst->state_transition_msg_id) {
        /* Use original msg_id that initiated transition */
        msgAck->msgId = inst->state_transition_msg_id;
        inst->state_transition_msg_id = 0;
      }

      /* Special case (ECU/TEC): CLOSE always gets ACK */
      if (inst->state == SIU_CHST_CLOSED) {
        /* First assume ECU:  full ACK */
        msgAck->ackType = MGB_ACK_TYPE_PROC | MGB_ACK_TYPE_DONE;
      }

      /* Special case (ECU/TEC): OPEN always get full ACK */
      if ((inst->state == SIU_CHST_IDLE) && (state == SIU_CHST_CLOSED))
        msgAck->ackType = MGB_ACK_TYPE_PROC | MGB_ACK_TYPE_DONE;
    }
  } /* for (cnt = 0; cnt < chProcessCnt; cnt++) - All Channel processing */
  
  
  siuContext.errorReport = TRUE; /* Re-enable error reporting */

} /* siuProcessNormalMessage */
/******************************************************************************
 * FUNCTION PURPOSE: Handle Interrupt Driven Messages 
 ******************************************************************************
 * DESCRIPTION:  Handles interrupt driven messages.  Ignores messages with
 *               invalid channel number.
 *
 *****************************************************************************/

void siuProcessInterruptMessage (mgb_t *message, siuMsgAck_t *msgAck)
{
  tint ch_id;
  siuInst_t *inst;
  tuint orig_chan = message->header.chnum;

  ch_id = message->header.chnum - SIU_FIRSTCH;

  if ( ch_id >= 0)
    inst = (siuInst_t *)SIUINSTp(ch_id);
  else
    inst = (siuInst_t *)SIUINSTp(message->header.chnum - SIU_FIRSTCH);

  /* If Invalid channel number -- send error indication and ignore */
  if (message->header.chnum < SIU_FIRSTCH || 
      message->header.chnum > siuContext.maxNumberChannels) {
     if ((orig_chan >= SIU_FIRSTCH) && (orig_chan <= SIU_ALL_LASTCH)) {
       /* The channel is valid but the translation failed; the channel
        * must be closed */
       tuint ID = siuContext.ID | orig_chan;
       siuErrorIndication (ID, message->header.trans_id, 
                           MGB_ERR_INVD_MSG_CHST_CLOSED);
     } else {
     siuErrorIndication (siuContext.ID, message->header.trans_id, 
                        MGB_ERR_INVD_CH);
     }
     return;
  }

  /* Set the data pager register (DMPR) to the data page of the channel before
   * running messages */
  thw_change_data_page( SIU_READ_PRIMARY_DATA_PAGE(inst) );
  siuGetDispatchTbl(dispatchTbl, siuLoadInterruptMessageTable, InterruptMessageTable);
  siuMsgDispatch((tlong)dispatchTbl,MGB_ERR_INVD_SIG_MSG,inst,message);
} /* siuProcessInterruptMessage */

/******************************************************************************
 * FUNCTION PURPOSE: Shut down NDU and close/delete ACU 
 *****************************************************************************/
void siuShutdownNduAcu (siuInst_t *inst)
{
  neuCtrl_t cfg;
  /* Point NDU RxOut to NULL */
  siuAmUnlinkNdu(inst);

#ifndef HDVIDEO  
  /* Close/Delete ACU */
  if (SIU_IS_PROTO_AAL2(inst)) {
    gg_siu_ctable.Aal2.close (inst);
    gg_siu_ctable.Aal2.delete (inst);
  }
#endif  
  /* Close/Delte RCU */
  if ((SIU_IS_PROTO_RTP(inst)) ||
      (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst))) {
#ifndef HDVIDEO  
    /* free the GMP buffer if IUUP over IP */
    if(!SIU_IS_PROTO_RTP(inst))
    {
        gmpFreeGmc(inst->ID, inst->proto.u.iu.p_rtp_params);    
    }

    gg_siu_ctable.Rtp.close (inst);
    gg_siu_ctable.Rtp.delete (inst);
#else
    siu_close_rcu(inst);
    siu_delete_rcu(inst);
#endif

    /* Close/Delete MSU */
#ifndef HDVIDEO
    if (gg_siu_ctable.MsuRtp.delete)
    {
      gg_siu_ctable.MsuRtp.delete (inst);
    }
    /* Close/Delete MSU */
    if (gg_siu_ctable.MsuRtcp.delete)
    {
      gg_siu_ctable.MsuRtcp.delete (inst);
    }
#endif
    
    #if (defined(PLRRED) || defined(PLRFEC))
     /* Close PLR module */
     siu_close_plr( inst );
     /* Delete PLR module */
     siu_delete_plr( inst );
    #endif /* PLRRED || PLRFEC */

	   /* Take care of dynamic heap allocation relocation */
   if (siuContext.dynHeapAlloc.enableFlag) {
#ifdef GG_INCLUDE_DYN_HEAP_ALLOCATION
    (*gg_siu_ctable.dynHeap.chkReloc)(inst);
#endif
   }
  } else if (SIU_IS_PROTO_TFTP(inst)) {
    tftpClose(inst->net.tftpInst);
    tftpDelete(&(inst->net.tftpInst));
    inst->protocol_type = MGB_PT_NONE;
  }


  /* Close NEU */
  if (SIU_IS_NEU_PRESENT(inst)) {
    /* un-register any udp ports mapped to channel */
    cfg.code = neu_CTRL_RMV_PORTMAP;
    cfg.u.portMap.inst = (void *)inst;
    (*gg_neu_ctable.neuControl) (inst->neuInst, &cfg);
    (*gg_neu_ctable.neuClose) (inst->neuInst);
  }
} /* siuShutdownNduAcu */
/*****************************************************************************
 * FUNCTION PURPOSE: process connection request
 *****************************************************************************
 * DESCRIPTION: process connection request
 *****************************************************************************/

static inline tuint siu_translate_chnum (tuint chnum)
{
    return chnum;
}

void siu_conn_req_cmd (siuInst_t *inst, mgb_t *msg)
{
  struct MGB_MDS_CONN_REQ *pCmd = &msg->msg_body.connect_request;
  int k;
  int retVal;
  int errorCode = MGB_ERR_INVD_CONN_REQ;

  for (k = 0; k < pCmd->num_elements; k++) 
  {

    /* If the src channel is 'dont care', then convert it into
     * something meaningful that will not generate errors. 
     * Then we won't have to handle special cases 
     */
    if (pCmd->elements[k].source_chan == MGB_NO_CHNUM)
      pCmd->elements[k].source_chan = pCmd->elements[k].dest_chan; 

    if (  (pCmd->elements[k].source_chan >= siuContext.maxNumberChannels)
        ||(pCmd->elements[k].dest_chan   >= siuContext.maxNumberChannels))
      goto ERROR;

    pCmd->elements[k].source_chan = siu_translate_chnum (pCmd->elements[k].source_chan);

    if (((pCmd->elements[k].req_type & MGB_CONN_DOMAIN_MASK) == MGB_CONN_YUV_DOMAIN) ||
        ((pCmd->elements[k].req_type & MGB_CONN_DOMAIN_MASK) == MGB_CONN_PACKET_DOMAIN)){
        retVal = siuVideoConnReq(pCmd->elements + k);
        if (retVal == 0) {
           /* Connect Request Message is Consumed by video Channel */
            continue;
        } else if (retVal != -1) {
            errorCode = retVal;
            goto ERROR;
        } else {
            /* -1 is returned - msg is not consumed by video channel */
            /* fall through */
        }
    }

    if (pCmd->elements[k].req_type & MGB_CONN_TRANSP_DOMAIN) {
      if (!siu_conn_req_neu (pCmd->elements + k))
        goto ERROR;
    }
  }

  return;

ERROR:
  siuErrorIndication (siuContext.ID, msg->header.trans_id, errorCode);
}

/******************************************************************************
 * FUNCTION PURPOSE: reset configuration parameters in siu instance 
 *                   at channel closing time
 ******************************************************************************
 * DESCRIPTION: 
 *****************************************************************************/
void siu_reset_configuration (siuInst_t *inst)
{

  /* reset packet loopback configuration */
  (inst->amInst).pktLoopback = NDU_CTRL_SND_LOOPBACK_OFF;
}

/* nothing past this point */
