/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _SIUVIGDKTASK_H
#define _SIUVIGDKTASK_H

/* Warning: siu_queue_purge_all() depends on priority ordering */
#define SIU_FIRST_PRIORITY_ID     0
#define SIU_NUM_PRIORITY_ID_MRATE 1    /* Number of priorities bios/multirate */
#define SIU_NUM_PRIORITY_ID_SRATE 1    /* Number of priorities one rate       */

#if (defined(GG_BIOS) || defined(GG_MULTITASKING))
  #define SIU_NUM_PRIORITY_ID SIU_NUM_PRIORITY_ID_MRATE
#else
  #define SIU_NUM_PRIORITY_ID SIU_NUM_PRIORITY_ID_SRATE
#endif

#define SIU_TASK_TYPE_RX           0
#define SIU_TASK_TYPE_TX           1
#define SIU_NUM_TASK_TYPES         2    /* Two task types: Rx and Tx           */
#define SIU_TASK_MAX_NUM_CHANNELS  SIU_MAX_TDM_CHANNELS
#define SIU_TASK_QUEUE_LENGTH (SIU_TASK_QUEUE_NUMCH + 1)

/* Task structure for RX/TX task for A PRIORITY (TXRX, 5ms, 10ms, 6ms) */
typedef struct {
  tint    in_index;          /* In index for receive and transmit tasks  */ 
  tint    out_index;         /* Out index for receive and transmit tasks */
  tint    cnt;               /* Number of tasks in queue                 */  
  tuint   queue[SIU_TASK_QUEUE_LENGTH]; /* Task queues for trasmit and receive */
  void    *pMsg[SIU_TASK_QUEUE_LENGTH]; /* IPC message from remote cores */
  tuint   task_type[SIU_TASK_QUEUE_LENGTH];
  tulong  queue_ts[SIU_TASK_QUEUE_LENGTH]; /* Timestamps of submitted tasks    */
  tuint   ch_exec_time[SIU_TASK_MAX_NUM_CHANNELS];  /* Channel execution time (exponentially averaged) */
} siuTask_t;

void siu_init_task (void);
tint siu_queue_write ( tuint max_number_channels, tuint channel, void *pMsg, tuint task_priority_ID, tuint task_type, tulong timestamp );
void siu_queue_purge_channel (tuint purgeChannel, tuint task_priority_ID);
tint siu_queue_purge_most_busy_channel (tuint max_number_channels, tuint task_priority_ID);
bool siu_queue_read( tuint *channel, void **pMsg, tuint *task_type, tuint task_priority_ID );
void siu_queue_update ( tuint max_number_channels, tuint numActiveChannels, tuint priority, tuint channel, tuint exec_time );
void siu_queue_purge_all(void);
#endif
/* nothing past this point */
