/******************************************************************************
 * FILE PURPOSE: Define message processing dispatch tables
 ******************************************************************************
 * FILE NAME:   siutbl.c
 *
 * DESCRIPTION: Contains dispatch tables for message processing in all the
 *  possible channel states. Each table also comes with a C-function for
 *  accessing that returns the tables stating address as a long pointer.
 *  this allows the tables to be placed in program memory.
 *
 * TABS: NONE
 *
 * $Id: $
 *
 * REVISION HISTORY: 
 *
 * $Log: $

 * Initial revision
 *
 * (C) Copyright 2000, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>

#include "types.h"
#include "vctypes.h"
/* ICD message definitions */
#include "mgmicdsp.h"
/* SIU header files */
#include "common/siu.h"
#include "common/siuloc.h"
#include "siuport.h"
#ifdef HDVIDEO
#include "siuvppu.h"
#include "siuvtk.h"
#include "siuovly.h"
#include "siummcu.h"
#include "siuCustomEco.h"
#endif

/* gg header file */
#include "ggcfg.h"
#include "ggswcfg.h"

#include "targethw.h"

#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif

/* Following section definitions are used by the state tables to avoid ugly 
   mkultiple occurrences of conditional defines. They can also be used to modify
   multiple entries that refer to the same message and processing function by 
   changing the connection in this section at one place or modify the table format */
/*-----------------------------------------------------------------------------
    DEFINES SECTION  BEGIN
 -----------------------------------------------------------------------------*/
#define SIU_MGB_MD_OPEN_CHANNEL    MGB_MD_OPEN_CHANNEL,   siuOpenChannel,
#define SIU_MGB_MD_ENCAP_CONFIG    MGB_MD_ENCAP_CONFIG,   siuEncapConfig,
#define SIU_MGB_MD_CHANNEL_STATE   MGB_MD_CHANNEL_STATE,  siuChannelState,
#define SIU_MGB_MD_RTTRACE         MGB_MD_RTTRACE,        siuGetRtTrace,
//#define SIU_MGB_MD_TDM_CONFIG      MGB_MD_TDM_CONFIG,     siuTdmConfig,
#define SIU_MGB_MD_COMMIT_STATE    MGB_MD_COMMIT_STATE,   siuCommitState,
#define SIU_MGB_MD_STATS_REQ         MGB_MD_STATS_REQ,       siuStatsRequest,
#if (defined(GG_INCLUDE_PRU) || defined(GG_INCLUDE_CONFERENCING) || defined(SIU_CBU))
#define SIU_MGB_MD_CONN_REQ          MGB_MD_CONN_REQ,        siu_conn_req_cmd,
#else
#define SIU_MGB_MD_CONN_REQ
#endif
#ifdef HDVIDEO
#define SIU_MGB_MD_VIDEO_MODE            MGB_MD_VIDEO_MODE,           siuVideoMode,
#define SIU_MGB_MD_CUSTOM_ECO_CFG        MGB_MD_CUSTOM_ECO_CFG,       siuCustomEcoCfg,
#define SIU_MGB_MD_VIDEO_OVLY_CFG        MGB_MD_VIDEO_OVLY_CFG,       siuOvlyCfg,
#define SIU_MGB_MD_VIDEO_FRC_CFG         MGB_MD_VIDEO_FRC_CFG,        siuFrcCfg,
#define SIU_MGB_MD_VIDEO_CODEC_GEN_CFG   MGB_MD_VIDEO_CODEC_GEN_CFG,  siuVideoCodecGenCfg,
#define SIU_MGB_MD_VIDEO_CTRL            MGB_MD_VIDEO_CTRL,           siuVideoCtrl,
#define SIU_VIDEO_MGB_MD_CLOSE_CHANNEL   MGB_MD_CLOSE_CHANNEL,        siu_msm_video_to_closed,
#define SIU_MGB_MD_VIDEO_MMCU_CFG        MGB_MD_VIDEO_MMCU_CFG,       siuMmcuCfg,
#define SIU_MGB_MD_AUDIO_CODEC_GEN_CFG   MGB_MD_AUDIO_CODEC_GEN_CFG   siuAudioCodecGenCfg,
#else
#define SIU_MGB_MD_VIDEO_MODE
#define SIU_MGB_MD_CUSTOM_ECO_CFG
#define SIU_MGB_MD_VIDEO_OVLY_CFG
#define SIU_MGB_MD_VIDEO_FRC_CFG
#define SIU_MGB_MD_VIDEO_CODEC_GEN_CFG
#define SIU_MGB_MD_VIDEO_CTRL
#define SIU_MGB_MD_VIDEO_MMCU_CFG
#endif
#ifdef GG_INCLUDE_NET_ENCAP
#define SIU_MGB_MD_NET_ENCAP_CONTROL       MGB_MD_NET_ENCAP_CONTROL, siu_net_encap_control,
#define SIU_MGB_MD_BWC_CONFIG              MGB_MD_BWC_CONFIG,siu_net_encap_bwc_config,

#else
#define SIU_MGB_MD_NET_ENCAP_CONTROL
#define SIU_MGB_MD_BWC_CONFIG
#endif
#ifdef GG_INCLUDE_RTCP
#define SIU_MGB_MD_RTCP_CONTROL       MGB_MD_RTCP_CONTROL,     siu_rtcp_control,
#define SIU_MGB_MD_RTCP_SDES_STRING   MGB_MD_RTCP_SDES_STRING, siu_rtcp_sdes_string,
#define SIU_MGB_MD_RTCP_SEND_PACKET   MGB_MD_RTCP_SEND_PACKET, siu_rtcp_send_packet,
#define SIU_MGB_MD_QA_CONFIG          MGB_MD_QA_CONFIG,        siu_rtcp_qa_config,
#else
#define SIU_MGB_MD_RTCP_CONTROL    
#define SIU_MGB_MD_RTCP_SDES_STRING
#define SIU_MGB_MD_RTCP_SEND_PACKET
#define SIU_MGB_MD_QA_CONFIG
#endif
#define SIU_DISPATCH_TERMINATE  0, NULL   /* this is the table termination entry */
/*-----------------------------------------------------------------------------
    DEFINES SECTION  END
 -----------------------------------------------------------------------------*/
/**************************************************************************************/
/* Message processing for non-voice-channel dependant messages                        */
/**************************************************************************************/
const siuIndepDispatch_t siuMsgTableChIndep[] = {

  MGB_MD_HALT_DSP,              siuHaltDsp,
  MGB_MD_STATUS_REQUEST,        siuStatusResponse,
  MGB_MD_MEM_READ,              siuMemoryRead,
  MGB_MD_MEM_WRITE,             siuMemoryWrite,
  MGB_MD_PKTIO_IF_CONFIG,          siuPktioIfCfg,

  SIU_DISPATCH_TERMINATE
};

/**************************************************************************************/


/**************************************************************************************/
/* Message processing when channel in "closed" state                                  */
/***********************************************************************************:)*/
const siuDispatch_t ChClosedStateTable[] = {
  /* Use appropriate macros for message entries without a separating "," */
  SIU_MGB_MD_OPEN_CHANNEL
  SIU_DISPATCH_TERMINATE
};

/**************************************************************************************/
/* Message processing when channel in "Idle" state or pending idle state             */
/***********************************************************************************:)*/
const siuDispatch_t ChIdleStateTable[] = {
  /* Use appropriate macros for message entries without a separating "," */
  SIU_MGB_MD_NET_ENCAP_CONTROL
  SIU_MGB_MD_STATS_REQ
  SIU_MGB_MD_COMMIT_STATE
#if defined (HDSKEL) || defined (HDVIDEO)
  SIU_MGB_MD_CONN_REQ
#endif
#ifdef HDVIDEO
  SIU_MGB_MD_VIDEO_MODE
  SIU_MGB_MD_VIDEO_CTRL
#endif
  SIU_MGB_MD_ENCAP_CONFIG              
  SIU_MGB_MD_CHANNEL_STATE
  SIU_MGB_MD_RTCP_CONTROL
  SIU_MGB_MD_RTCP_SDES_STRING
  SIU_DISPATCH_TERMINATE
};
#ifdef HDVIDEO
/**************************************************************************************/
/* Message processing when channel in "Video" state                                  */
/***********************************************************************************:)*/
const siuDispatch_t ChVideoStateTable[] = {
  /* Use appropriate macros for message entries without a separating "," */
  SIU_VIDEO_MGB_MD_CLOSE_CHANNEL
  SIU_MGB_MD_CHANNEL_STATE
  SIU_MGB_MD_RTCP_CONTROL   
  SIU_MGB_MD_RTCP_SDES_STRING
  SIU_MGB_MD_RTCP_SEND_PACKET
  SIU_MGB_MD_COMMIT_STATE
  SIU_MGB_MD_VIDEO_MODE
  SIU_MGB_MD_VIDEO_CTRL
  SIU_MGB_MD_STATS_REQ
  SIU_MGB_MD_CONN_REQ
  SIU_MGB_MD_CUSTOM_ECO_CFG
  SIU_MGB_MD_VIDEO_OVLY_CFG 
  SIU_MGB_MD_VIDEO_FRC_CFG
  SIU_MGB_MD_VIDEO_CODEC_GEN_CFG
  SIU_MGB_MD_NET_ENCAP_CONTROL
  SIU_MGB_MD_VIDEO_MMCU_CFG
  SIU_MGB_MD_ENCAP_CONFIG
  SIU_DISPATCH_TERMINATE
};
#endif
/**************************************************************************************/
/* Interrupt Message Processing                                                       */
/***********************************************************************************:)*/
const siuDispatch_t InterruptMessageTable[] = {
  SIU_DISPATCH_TERMINATE
};

/**************************************************************************************/
/* Multiple statistics request                                                        */
/***********************************************************************************:)*/
/* Possible stat responses defined by LSW (of 32-bit bit field) */
const fcnGetStats_t arrayGetStatsLSW[] = {
  siuGetErrorStats,             /* ID 0 */
#if defined(VOICE) || defined(VOICE_FAX)
  siuGetTxRxStats,              /* ID 1 */
  siuGetVPDelay,                /* ID 2 */
  siuGetPiuStats,               /* ID 3 */
#else
  siuRtpGetTxRxStats,           /* ID 1 */
  siu_get_stats_unavail,
  siu_get_stats_unavail,
#endif
#if (defined(LEC_ECU) || defined(GG_INCLUDE_AER) || defined(LEC_TEC) || defined(LEC_TECENG))
  siuEcDbgStats,                /* ID 4 */
#else
  siu_get_stats_unavail,
#endif

#if defined(FAX) || defined(VOICE_FAX)
  siuGetFaxRelayStats,         /* ID 5 */
  siuGetFaxModemStats,         /* ID 6 */
  siuGetFaxCallStats,          /* ID 7 */
#else
  siu_get_stats_unavail,       /* ID 5 */
  siu_get_stats_unavail,       /* ID 6 */
  siu_get_stats_unavail,       /* ID 7 */
#endif
#ifdef GG_INCLUDE_CLEAR_DATA
  siuGetMPPStats,              /* ID 8 */
#else
  siu_get_stats_unavail,       /* ID 8 */
#endif

#ifdef GG_INCLUDE_RTCP
  siuRtcpGetStatsToPkt,        /* ID 9 */
  siuRtcpGetStatsFromPkt,      /* ID 10 */
#else
  siu_get_stats_unavail,       /* ID 9 */
  siu_get_stats_unavail,       /* ID 10 */
#endif
#if (defined(GG_INCLUDE_DSIGT) || defined(GG_INCLUDE_DSIGR))
  siu_dsig_report_stats,       /* ID 11 */
#else
  siu_get_stats_unavail,       /* ID 11 */
#endif
#if defined(GG_INCLUDE_VQM)
  siu_get_vqm_stats,           /* ID 12 */
#else
  siu_get_stats_unavail,       /* ID 12 */
#endif
#ifdef AGC
  siuAgcDbgStat,
#else
  siu_get_stats_unavail,
#endif
#if (defined(GG_INCLUDE_MSU) && defined(GG_INCLUDE_RTP))
  siu_msu_get_stats_rtp,      /* ID 14 */
#else
  siu_get_stats_unavail,      /* ID 14 */
#endif
#if (defined(GG_INCLUDE_MSU) && defined(GG_INCLUDE_RTCP))
  siu_msu_get_stats_rtcp      /* ID 15 */
#else
  siu_get_stats_unavail       /* ID 15 */
#endif
};
/* Possible stat responses defined by MSW (of 32-bit bit field) */
const fcnGetStats_t arrayGetStatsMSW[] = {
#ifdef GG_INCLUDE_MRU
    siuGetModemRelayStats,  /* ID 16 */
#else
    siu_get_stats_unavail, 
#endif

#ifdef GG_INCLUDE_ECUMA
    siu_mag_dbgstat,	      /* ID 17 */
#else
    siu_get_stats_unavail,    
#endif
    siuGetNetworkStats,		  /* ID 18 */
#ifdef GG_INCLUDE_NET_ENCAP
    siuGetNetEncapStats,    /* ID 19 */
#else
    siu_get_stats_unavail,  /* ID 19 */
#endif
    siu_get_stats_unavail, 	/* ID 20 */
#ifdef SIU_TFO
    siu_get_tfo_stats, 	    /* ID 21 */
#else
    siu_get_stats_unavail, 	/* ID 21 */
#endif
#if (defined(PLRRED) || defined(PLRFEC))
    siu_get_plr_stats,      /* ID 22 */
#else
    siu_get_stats_unavail, 	/* ID 22 */
#endif
#ifdef VEU
  siu_veu_stats,            /* ID 23 */
#else
  siu_get_stats_unavail,    /* ID 23 */
#endif
#if defined(VOICE) || defined(VOICE_FAX)
    siuGetVadStats,             /* ID 24 */
#else
    siu_get_stats_unavail,
#endif
#ifdef AUDIO
    siuGetADUStats,			/* ID 25 */
#else
    siu_get_stats_unavail, 	/* ID 25 */
#endif 
#ifdef GG_INCLUDE_IUUP
    siu_get_iuup_stats, 	  /* ID 26 */
#else
    siu_get_stats_unavail, 	/* ID 26 */
#endif
#ifdef NX64
    (fcnGetStats_t)siuNx64ReportStats,		/* ID 27 */
#else
    siu_get_stats_unavail, 	/* ID 27 */
#endif    
#ifdef BERT
    (fcnGetStats_t)siuBertGetStats,		/* ID 28 */
#else
    siu_get_stats_unavail, 	/* ID 28 */
#endif    
#ifdef TTU
    siuttuGetStatsfromTdm,  /* ID 29 */
    siuttuGetStatsfromPkt,  /* ID 30 */
#else
    siu_get_stats_unavail, 	/* ID 29 */
    siu_get_stats_unavail, 	/* ID 30 */
#endif
    NULL
};

/* Possible stat responses defined by 2nd word LSW (of 32-bit bit field) */
const fcnGetStats_t arrayGetStats2LSW[] = {
#if (defined(GG_INCLUDE_RTCP) && defined(VQM_RFC3611))
    siuRtcpGetXRVoipLoc, /* ID 1 */
    siuRtcpGetXRVoipRem, /* ID 2 */
#else
    siu_get_stats_unavail,   /* ID 1 */
    siu_get_stats_unavail,   /* ID 2 */
#endif
    siu_get_stats_unavail, 	 /* ID 3 */
    siu_get_stats_unavail, 	 /* ID 4 */
#if (defined(GG_INCLUDE_RTCP) && defined(VQM_H248XNQ))
    siuRtcpGetXRVoip2Loc,    /* ID 5 */
    siuRtcpGetXRVoip2Rem,    /* ID 6 */
#else
    siu_get_stats_unavail,   /* ID 5 */
    siu_get_stats_unavail,   /* ID 6 */
#endif
#ifdef EQI    
    siuEQISendStats,     	 /* ID 7 */
#else
    siu_get_stats_unavail, 	 /* ID 7 */
#endif    
    siu_get_stats_unavail, 	 /* ID 8 */
    siu_get_stats_unavail, 	 /* ID 9 */
    siu_get_stats_unavail, 	 /* ID 10 */
#ifdef HDVIDEO    
    siuVtkGetStats,     	 /* ID 11 */
    siuFrcGetStats,     	 /* ID 12 */
    siuYuvGetStats,              /* ID 13 */
    siuVppuGetStats,     	 /* ID 14 */
#else
    siu_get_stats_unavail, 	 /* ID 11 */
    siu_get_stats_unavail, 	 /* ID 12 */
    siu_get_stats_unavail, 	 /* ID 13 */
    siu_get_stats_unavail, 	 /* ID 14 */
#endif 

    siu_get_stats_unavail, 	 /* ID 15 */
    NULL
};

const fcnGetStats_t arrayGetStats2MSW[] = {
    siu_get_stats_unavail, 	 /* ID 0 */
    siu_get_stats_unavail, 	 /* ID 1 */
    siu_get_stats_unavail, 	 /* ID 2 */
    siu_get_stats_unavail, 	 /* ID 3 */
    siu_get_stats_unavail, 	 /* ID 4 */
    siu_get_stats_unavail, 	 /* ID 5 */
    siu_get_stats_unavail, 	 /* ID 6 */
    siu_get_stats_unavail, 	 /* ID 7 */
    siu_get_stats_unavail, 	 /* ID 8 */
    siu_get_stats_unavail, 	 /* ID 9 */
    siu_get_stats_unavail, 	 /* ID 10 */
    siu_get_stats_unavail, 	 /* ID 11 */
    siu_get_stats_unavail, 	 /* ID 12 */
    siu_get_stats_unavail, 	 /* ID 13 */
    siu_get_stats_unavail, 	 /* ID 14 */
    siu_get_stats_unavail, 	 /* ID 15 */
    NULL
};

/**********************************************************************
 Stats request bitmap definded in mgmicdsp.h
 Lower 16-bits (bit masks)
#define MGB_STATS_ID_ERROR               (1 << 0)
#define MGB_STATS_ID_RX_TX               (1 << 1)
#define MGB_STATS_ID_VP                  (1 << 2)
#define MGB_STATS_ID_TELE_LEVELS         (1 << 3)
#define MGB_STATS_ID_EC_DBG              (1 << 4)
#define MGB_STATS_ID_FAX_RELAY           (1 << 5)
#define MGB_STATS_ID_FAX_MODEM           (1 << 6)
#define MGB_STATS_ID_FAX_RELAY_CALL      (1 << 7)
#define MGB_STATS_ID_DIGITAL_DATA        (1 << 8)
#define MGB_STATS_ID_RTCP_TO_PKT         (1 << 9)
#define MGB_STATS_ID_RTCP_FROM_PKT       (1 << 10)
#define MGB_STATS_ID_SIG                 (1 << 11)
#define MGB_STATS_ID_VQM                 (1 << 12)
#define MGB_STATS_ID_AGC_DBG             (1 << 13)

 Upper 16-bits
#define MGB_STATS_ID_MODEM_RELAY         (1 << 0)
#define MGB_STATS_ID_MAG_DBGSTAT         (1 << 1)
#define MGB_STATS_ID_NETWORK_STAT        (1 << 2)
#define MGB_STATS_ID_NETENCAP_STAT       (1 << 3)
#define MGB_STATS_ID_PLR_STAT            (1 << 6)
#define MGB_STATS_ID_GLBL_PKT_STAT       MGB_STATS_ID_NETENCAP_STAT

2nd word lower 16-bits 
#define MGB_STATS_ID_RTCP_LOC_XRVOIP     (1 << 0)
#define MGB_STATS_ID_RTCP_REM_XRVOIP     (1 << 1)

The following are defined in siuloc.h

#define SIU_MAX_NUM_CHST               17   // Total number of channel states 
#define SIU_CHST_CLOSED                 0   // Channel is closed, modules closed 
#define SIU_CHST_IDLE                   1   // Channel is open - PIU works 
#define SIU_CHST_VOICE                  2   // Channel in voice mode - codec active 
#define SIU_CHST_DTMF                   3   // Channel in DTMF mode 
#define SIU_CHST_FAX_RELAY              4   // Channel in Fax Relay mode 
#define SIU_CHST_APPCLOSED              5   // switchover, channel closed 
#define SIU_CHST_MODEM_RELAY            6   // Channel in Modem Relay Mode 
#define SIU_CHST_DATA_MODEM             7   // Channel is in data modem state 
#define SIU_CHST_CLEAR_DATA_MODE        8   // Channel in clear data mode 
#define SIU_CHST_FAX_MODEM              9   // Channel in fax modem mode 
#define SIU_CHST_LINK_LAYER_MODEM_RELAY 10  // Channel in Link Layer Modem Relay mode 
#define SIU_CHST_CP_IDLE                11  // Channel in CP - idle 
#define SIU_CHST_CP_DTMF                12  // Channel in CP - DTMF mode 
#define SIU_CHST_CP_MODEM_TERM          13  // Channel in CP - Modem Termination 
#define SIU_CHST_CP_CLEAR_DATA          14  // Channel in CP - Clear Data 
#define SIU_CHST_CP_FAX_TERM            15  // Channel in CP - Fax Termination 
#define SIU_CHST_BERT_MODE              16  // channel in bert mode
#define SIU_CHST_NX64_MODE              17  // channel in Nx64 mode

***********************************************************************/

#if (defined(GG_INCLUDE_DSIGT) || defined(GG_INCLUDE_DSIGR))
#define SIU_MGB_STATS_ID_SIG   MGB_STATS_ID_SIG
#else
#define SIU_MGB_STATS_ID_SIG   0
#endif

#if defined (GG_INCLUDE_RTCP)
#define SIU_MGB_STATS_ID_RTCP_TO_PKT    MGB_STATS_ID_RTCP_TO_PKT    
#define SIU_MGB_STATS_ID_RTCP_FROM_PKT  MGB_STATS_ID_RTCP_FROM_PKT  
#else
#define SIU_MGB_STATS_ID_RTCP_TO_PKT    0
#define SIU_MGB_STATS_ID_RTCP_FROM_PKT  0
#endif

#define SIU_MGB_STATS_ID_TELE_LEVELS    MGB_STATS_ID_TELE_LEVELS

#define SIU_MGB_STATS_ID_ERROR          MGB_STATS_ID_ERROR
#define SIU_MGB_STATS_ID_RX_TX          MGB_STATS_ID_RX_TX
#define SIU_MGB_STATS_ID_VP             MGB_STATS_ID_VP  
#define SIU_MGB_STATS_ID_EC_DBG         MGB_STATS_ID_EC_DBG

#ifdef AGC
#define SIU_MGB_STATS_ID_AGC_DBG        MGB_STATS_ID_AGC_DBG
#else
#define SIU_MGB_STATS_ID_AGC_DBG        0
#endif
#define SIU_MGB_STATS_ID_FAX_RELAY      MGB_STATS_ID_FAX_RELAY     
#define SIU_MGB_STATS_ID_FAX_MODEM      MGB_STATS_ID_FAX_MODEM     
#define SIU_MGB_STATS_ID_FAX_RELAY_CALL MGB_STATS_ID_FAX_RELAY_CALL
#define SIU_MGB_STATS_ID_DIGITAL_DATA   MGB_STATS_ID_DIGITAL_DATA
#ifdef GG_INCLUDE_VQM
#define SIU_MGB_STATS_ID_VQM            MGB_STATS_ID_VQM
#else
#define SIU_MGB_STATS_ID_VQM            0
#endif

#if (defined(GG_INCLUDE_MSU) && defined(GG_INCLUDE_RTP))
#define SIU_MGB_STATS_ID_SECURITY_RTP   MGB_STATS_ID_SECURITY_RTP
#else
#define SIU_MGB_STATS_ID_SECURITY_RTP   0
#endif
#if (defined(GG_INCLUDE_MSU) && defined(GG_INCLUDE_RTCP))
#define SIU_MGB_STATS_ID_SECURITY_RTCP  MGB_STATS_ID_SECURITY_RTCP
#else 
#define SIU_MGB_STATS_ID_SECURITY_RTCP  0
#endif

#ifdef VEU
#define SIU_MGB_STATS_ID_VEU         MGB_STATS_ID_VEU
#else
#define SIU_MGB_STATS_ID_VEU         0
#endif

const tuint siuStatsReqChstMaskLsw[SIU_MAX_NUM_CHST] = {

  /* SIU_CHST_CLOSED */
  SIU_MGB_STATS_ID_SIG,

  /* SIU_CHST_IDLE */
    SIU_MGB_STATS_ID_TELE_LEVELS 
  | SIU_MGB_STATS_ID_SIG
  | SIU_MGB_STATS_ID_ERROR
  | SIU_MGB_STATS_ID_RX_TX 
  | SIU_MGB_STATS_ID_VP
  | SIU_MGB_STATS_ID_SECURITY_RTP
  | SIU_MGB_STATS_ID_SECURITY_RTCP 
  | SIU_MGB_STATS_ID_RTCP_TO_PKT    
  | SIU_MGB_STATS_ID_RTCP_FROM_PKT  
  ,

  /* SIU_CHST_VOICE */
    SIU_MGB_STATS_ID_TELE_LEVELS    
  | SIU_MGB_STATS_ID_RTCP_TO_PKT    
  | SIU_MGB_STATS_ID_RTCP_FROM_PKT  
  | SIU_MGB_STATS_ID_SIG            
  | SIU_MGB_STATS_ID_ERROR
  | SIU_MGB_STATS_ID_RX_TX
  | SIU_MGB_STATS_ID_VP  
  | SIU_MGB_STATS_ID_EC_DBG
  | SIU_MGB_STATS_ID_AGC_DBG
  | SIU_MGB_STATS_ID_VQM
  | SIU_MGB_STATS_ID_SECURITY_RTP
  | SIU_MGB_STATS_ID_SECURITY_RTCP,

  /* SIU_CHST_DTMF */
    SIU_MGB_STATS_ID_TELE_LEVELS
  | SIU_MGB_STATS_ID_SIG
  | SIU_MGB_STATS_ID_ERROR
  | SIU_MGB_STATS_ID_RX_TX
  | SIU_MGB_STATS_ID_VP,

  /* SIU_CHST_FAX  */         
    SIU_MGB_STATS_ID_TELE_LEVELS
  | SIU_MGB_STATS_ID_FAX_RELAY      
  | SIU_MGB_STATS_ID_FAX_MODEM      
  | SIU_MGB_STATS_ID_FAX_RELAY_CALL
  | SIU_MGB_STATS_ID_SIG, 
  /* SIU_CHST_APPCLOSED */
    SIU_MGB_STATS_ID_SIG,

  /* SIU_CHST_MODEM_RELAY */
    SIU_MGB_STATS_ID_TELE_LEVELS  
  | SIU_MGB_STATS_ID_SIG,

  /* SIU_CHST_DATA_MODEM  */
    SIU_MGB_STATS_ID_TELE_LEVELS
  | SIU_MGB_STATS_ID_SIG,

  /* SIU_CHST_CLEAR_DATA_MODE */
    SIU_MGB_STATS_ID_TELE_LEVELS    
  | SIU_MGB_STATS_ID_DIGITAL_DATA,

  /* SIU_CHST_FAX_MODEM */    
    SIU_MGB_STATS_ID_TELE_LEVELS
  | SIU_MGB_STATS_ID_FAX_MODEM      
  | SIU_MGB_STATS_ID_SIG,


  /* SIU_CHST_LINK_LAYER_MODEM_RELAY */
   //TODO_STATS
  0,

  /* SIU_CHST_CP_IDLE  */
  //TODO_STATS
  0,
           
  /* SIU_CHST_CP_DTMF  */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_MODEM_TERM */ 
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_CLEAR_DATA */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_FAX_TERM   */       
  //TODO_STATS
  0,

  /* SIU_CHST_BERT_MODE */  
    SIU_MGB_STATS_ID_TELE_LEVELS
  | SIU_MGB_STATS_ID_ERROR
  | SIU_MGB_STATS_ID_RX_TX
  | SIU_MGB_STATS_ID_VP
  | SIU_MGB_STATS_ID_SIG
  ,
  /* SIU_CHST_NX64_MODE */  
    SIU_MGB_STATS_ID_RTCP_TO_PKT    
  | SIU_MGB_STATS_ID_RTCP_FROM_PKT  
  | SIU_MGB_STATS_ID_SIG            
  | SIU_MGB_STATS_ID_ERROR
  | SIU_MGB_STATS_ID_RX_TX
  | SIU_MGB_STATS_ID_VP
  ,
  /* SIU_CHST_EXCEPTION */
  0

#ifdef HDVIDEO
  /* SIU_CHST_VIDEO_MODE */  
  , SIU_MGB_STATS_ID_RX_TX
  | SIU_MGB_STATS_ID_ERROR
  | SIU_MGB_STATS_ID_RTCP_TO_PKT
  | SIU_MGB_STATS_ID_RTCP_FROM_PKT  
#endif
};


#define SIU_MGB_STATS_ID_MODEM_RELAY       MGB_STATS_ID_MODEM_RELAY   
#ifdef GG_INCLUDE_ECUMA
#define SIU_MGB_STATS_ID_MAG_DBGSTAT       MGB_STATS_ID_MAG_DBGSTAT   
#else
#define SIU_MGB_STATS_ID_MAG_DBGSTAT       0   
#endif
#define SIU_MGB_STATS_ID_NETWORK_STAT      MGB_STATS_ID_NETWORK_STAT  
#ifdef GG_INCLUDE_NET_ENCAP
#define SIU_MGB_STATS_ID_NETENCAP_STAT     MGB_STATS_ID_NETENCAP_STAT 
#else
#define SIU_MGB_STATS_ID_NETENCAP_STAT     0
#endif
#define SIU_MGB_STATS_ID_MODEM_RXTX        MGB_STATS_ID_MODEM_RXTX    
#ifdef BERT
#define SIU_MGB_STATS_ID_BERT_STAT         MGB_STATS_ID_BERT_STAT     
#else
#define SIU_MGB_STATS_ID_BERT_STAT         0     
#endif
#ifdef NX64
#define SIU_MGB_STATS_ID_NX64_STAT         MGB_STATS_ID_NX64_STAT     
#else
#define SIU_MGB_STATS_ID_NX64_STAT         0     
#endif
#ifdef AUDIO
#define SIU_MGB_STATS_ID_AUDIO_STAT         MGB_STATS_ID_AUDIO_STAT
#else
#define SIU_MGB_STATS_ID_AUDIO_STAT         0
#endif
#ifdef GG_INCLUDE_IUUP
#define SIU_MGB_STATS_ID_IUUP              MGB_STATS_ID_IUUP_STAT
#else
#define SIU_MGB_STATS_ID_IUUP              0
#endif
#define SIU_MGB_STATS_ID_VAD_STAT          MGB_STATS_ID_VAD
#define SIU_MGB_STATS_ID_PLR_STAT          MGB_STATS_ID_PLR
#ifdef SIU_TFO
#define SIU_MGB_STATS_ID_TFO_STAT          MGB_STATS_ID_TFO_STAT
#else
#define SIU_MGB_STATS_ID_TFO_STAT 0  
#endif

#define SIU_MGB_STATS_ID_TTU_TDM           MGB_STATS_ID_TTU_TDM
#define SIU_MGB_STATS_ID_TTU_NET           MGB_STATS_ID_TTU_NET


const tuint siuStatsReqChstMaskMsw[SIU_MAX_NUM_CHST] = {
  /* SIU_CHST_CLOSED */
  0,

  /* SIU_CHST_IDLE */
    SIU_MGB_STATS_ID_NETWORK_STAT,  

  /* SIU_CHST_VOICE */
    SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_MAG_DBGSTAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT
  | SIU_MGB_STATS_ID_TFO_STAT
#ifdef VEU
  | SIU_MGB_STATS_ID_VEU
#endif
  | SIU_MGB_STATS_ID_PLR_STAT
  | MGB_STATS_ID_IUUP_STAT
  | SIU_MGB_STATS_ID_VAD_STAT
  | SIU_MGB_STATS_ID_TTU_TDM
  | SIU_MGB_STATS_ID_TTU_NET,

  /* SIU_CHST_DTMF */
    SIU_MGB_STATS_ID_NETWORK_STAT,

  /* SIU_CHST_FAX  */         
    SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT,

  /* SIU_CHST_APPCLOSED */    
  0,

  /* SIU_CHST_MODEM_RELAY */  
    SIU_MGB_STATS_ID_MODEM_RELAY
  | SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT,

  /* SIU_CHST_DATA_MODEM  */  
    SIU_MGB_STATS_ID_MODEM_RELAY
  | SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT,

  /* SIU_CHST_CLEAR_DATA_MODE */
    SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT,

  /* SIU_CHST_FAX_MODEM */    
    SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT,

  /* SIU_CHST_LINK_LAYER_MODEM_RELAY */
   //TODO_STATS
  0,

  /* SIU_CHST_CP_IDLE  */
  //TODO_STATS
  0,
           
  /* SIU_CHST_CP_DTMF  */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_MODEM_TERM */ 
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_CLEAR_DATA */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_FAX_TERM   */       
  //TODO_STATS
  0,

  /* SIU_CHST_BERT_MODE */
    SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_BERT_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT,
  
  /* SIU_CHST_NX64_MODE */
    SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_NX64_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT
  /* SIU_CHST_EXCEPTION */
  , 0
#ifdef  AUDIO   
  , SIU_MGB_STATS_ID_AUDIO_STAT
#endif
#ifdef HDVIDEO
  /* SIU_CHST_VIDEO_MODE */  
  , SIU_MGB_STATS_ID_NETWORK_STAT
  | SIU_MGB_STATS_ID_NETENCAP_STAT
#endif
};

#if defined (GG_INCLUDE_RTCP)
#define SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP   MGB_STATS_ID_RTCP_LOC_XRVOIP    
#define SIU_MGB_STATS_ID_RTCP_REM_XRVOIP   MGB_STATS_ID_RTCP_REM_XRVOIP    
#define SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP2  MGB_STATS_ID_RTCP_LOC_XRVOIP2
#define SIU_MGB_STATS_ID_RTCP_REM_XRVOIP2  MGB_STATS_ID_RTCP_REM_XRVOIP2
#else
#define SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP   0
#define SIU_MGB_STATS_ID_RTCP_REM_XRVOIP   0
#define SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP2  0
#define SIU_MGB_STATS_ID_RTCP_REM_XRVOIP2  0
#endif

#ifdef EQI
#define SIU_MGB_STATS_ID_EQI            MGB_STATS_ID_EQI
#else
#define SIU_MGB_STATS_ID_EQI            0
#endif

#ifdef HDVIDEO
#define SIU_MGB_STATS_ID_VPPU           MGB_STATS_ID_VPPU
#define SIU_MGB_STATS_ID_VTK            MGB_STATS_ID_VTK
#define SIU_MGB_STATS_ID_FRC            MGB_STATS_ID_FRC
#define SIU_MGB_STATS_ID_YUV            MGB_STATS_ID_YUV
#else
#define SIU_MGB_STATS_ID_VPPU           0
#define SIU_MGB_STATS_ID_VTK            0
#define SIU_MGB_STATS_ID_FRC            0
#define SIU_MGB_STATS_ID_YUV            0
#endif

const tuint siuStatsReqChstMask2Lsw[SIU_MAX_NUM_CHST] = {
  /* SIU_CHST_CLOSED */
  0,

  /* SIU_CHST_IDLE */
    SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP  
  | SIU_MGB_STATS_ID_RTCP_REM_XRVOIP
  | SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP2 
  | SIU_MGB_STATS_ID_RTCP_REM_XRVOIP2,  

  /* SIU_CHST_VOICE */
    SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP  
  | SIU_MGB_STATS_ID_RTCP_REM_XRVOIP
  | SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP2 
  | SIU_MGB_STATS_ID_RTCP_REM_XRVOIP2
  | SIU_MGB_STATS_ID_EQI,  

  /* SIU_CHST_DTMF */
  0,

  /* SIU_CHST_FAX  */         
  0,

  /* SIU_CHST_APPCLOSED */    
  0,

  /* SIU_CHST_MODEM_RELAY */  
  0,

  /* SIU_CHST_DATA_MODEM  */  
  0,

  /* SIU_CHST_CLEAR_DATA_MODE */
  0, 

  /* SIU_CHST_FAX_MODEM */    
  0,

  /* SIU_CHST_LINK_LAYER_MODEM_RELAY */
  //TODO_STATS
  0,

  /* SIU_CHST_CP_IDLE  */
  //TODO_STATS
  0,
           
  /* SIU_CHST_CP_DTMF  */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_MODEM_TERM */ 
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_CLEAR_DATA */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_FAX_TERM   */       
  //TODO_STATS
  0,

  /* SIU_CHST_BERT_MODE */
  0,
  
  /* SIU_CHST_NX64_MODE */
    SIU_MGB_STATS_ID_RTCP_LOC_XRVOIP  
  | SIU_MGB_STATS_ID_RTCP_REM_XRVOIP 
  
  /* SIU_CHST_EXCEPTION */
  , 0

#ifdef HDVIDEO
  /* SIU_CHST_VIDEO_MODE */  
  , SIU_MGB_STATS_ID_VPPU
  | SIU_MGB_STATS_ID_VTK
  | SIU_MGB_STATS_ID_FRC
  | SIU_MGB_STATS_ID_YUV
#endif
};


const tuint siuStatsReqChstMask2Msw[SIU_MAX_NUM_CHST] = {
  /* SIU_CHST_CLOSED */
  0,

  /* SIU_CHST_IDLE */
  0,

  /* SIU_CHST_VOICE */
  0,

  /* SIU_CHST_DTMF */
  0,

  /* SIU_CHST_FAX  */         
  0,

  /* SIU_CHST_APPCLOSED */    
  0,

  /* SIU_CHST_MODEM_RELAY */  
  0,

  /* SIU_CHST_DATA_MODEM  */  
  0,

  /* SIU_CHST_CLEAR_DATA_MODE */
  0, 

  /* SIU_CHST_FAX_MODEM */    
  0,

  /* SIU_CHST_LINK_LAYER_MODEM_RELAY */
  //TODO_STATS
  0,

  /* SIU_CHST_CP_IDLE  */
  //TODO_STATS
  0,
           
  /* SIU_CHST_CP_DTMF  */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_MODEM_TERM */ 
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_CLEAR_DATA */
  //TODO_STATS
  0,
        
  /* SIU_CHST_CP_FAX_TERM   */       
  //TODO_STATS
  0,

  /* SIU_CHST_BERT_MODE */
  0,

  /* SIU_CHST_NX64_MODE */  
  0

  /* SIU_CHST_EXCEPTION */
  , 0
#ifdef HDVIDEO
  /* SIU_CHST_VIDEO_MODE */  
  ,0
#endif

};

/***************************************************************************:)*/
/* Multiple Reset State and Statistics Request                                */
/***************************************************************************:)*/
/* Possible state reset responses defined by LSW (of 32-bit bit field) */
const siuFcnResetCh_t siuArrayResetChStateLSW[] = {
#ifdef GG_INCLUDE_RTCP
  siuRtcpRstTimeout,
#else
  siu_reset_channel_unavail,
#endif
#if defined(VOICE) || defined(VOICE_FAX)
  siuStarvationReset,
  siuResetRx,
#else
  siu_reset_channel_unavail,
  siu_reset_channel_unavail,
#endif
/* TODO 
#if (defined(LEC_ECU) || defined(GG_INCLUDE_AER) || defined(LEC_TEC) || defined(LEC_TECENG))
  siuEcReset,
#else
  siu_reset_channel_unavail,  
#endif
*/
NULL, /* TODO(EC) */
NULL, /* TODO (BISIL) */
#ifdef GG_INCLUDE_CLEAR_DATA
  siu_mpp_mtp2_reset,
#else
  NULL,
#endif
#if (!defined(HDMOVE) && !defined(HDSKEL) && !defined(HDVIDEO)) /* will be added */
  siu_rtcp_qa_reset,  
#else
  siu_reset_channel_unavail,
#endif
  NULL
};
/* Possible state reset responses defined by MSW (of 32-bit bit field) */
const siuFcnResetCh_t siuArrayResetChStateMSW[] = {
  NULL
};
/* nothing past this point */

