/******************************************************************************
 * FILE PURPOSE: SIU TX Task Management Routines for AUDIO
 ******************************************************************************
 * FILE NAME:   siuVigdkTxAudio.c
 *
 * DESCRIPTION: SIU TX Task Management Routines for Audio
 *
 * TABS: NONE
 *
 * (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siuloc.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "atk.h"
#include "siummcu.h"
#include "siuVigdkInterTxRx.h"


/*********************************************************************************
 * FUNCTION PURPOSE: Encoding Audio call on a master core
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs: siuInst
                            encInput        : Encoder input params
                            encOutput       : Encoder output parmas
                    Output: None
 *********************************************************************************/
 tulong  vdebug_encoded_audio_timestamp = 0;

XDAS_Int32 siuTxAudioEncode(void *siuInst, ifvisys_audEncInput_t *audEncInput,
                                                 ifvisys_audEncOutput_t *audEncOutput)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  XDAS_Int32 xdmRetVal;
  ifvisys_audEncInput_t   locAudEncInput;
  ifvisys_audEncOutput_t  locAudEncOutput;
  tlong curr_idx = 0;

  siu_exc_assert( (inst->audBufferIndex + audEncInput->bitStream.size <= inst->audBufferSize), SIUATK_EXC_AUDIOBUFFER_OVERFLOW, inst);

  memcpy(&inst->audBuffer[inst->audBufferIndex], audEncInput->bitStream.ptr, audEncInput->bitStream.size);
  inst->audBufferIndex += audEncInput->bitStream.size;

  siu_invalidate_cache();

  /* Encode Process Call */
  while ((inst->audBufferIndex - curr_idx) >= 8192) {
    locAudEncInput.bitStream.ptr  = &inst->audBuffer[curr_idx];
    locAudEncInput.bitStream.size = 4096; 

    locAudEncOutput.bitStream.ptr = &audEncOutput->bitStream.ptr[audEncOutput->bitStream.size];

    xdmRetVal = atkEncode (inst->stateInst.video.atkInst, &locAudEncInput, &locAudEncOutput);
    curr_idx += 8192;

    /* ship-out raw audio */
    audEncOutput->bitStream.size += locAudEncOutput.bitStream.size;
    locAudEncOutput.bitStream.size = 0;

    siu_exc_assert( (audEncOutput->bitStream.size < siuContext.max_codec_frame_size), SIUMMCU_EXC_TXFRAME_TOO_BIG, inst);
  }

  if (curr_idx != 0) {
    memmove(&inst->audBuffer[0], &inst->audBuffer[curr_idx], inst->audBufferIndex - curr_idx);
    inst->audBufferIndex -= curr_idx;
  }

  siu_invalidate_cache();

  return(xdmRetVal);

} /* siuTxVideoMasterEncode() */


tint siuTxAudioEncoderProcess(siuInst_t *inst, word *audInputFrame, tulong audInputSize, tulong frameTimeStamp)
{
  ifvisys_audEncInput_t   audEncInput;
  ifvisys_audEncOutput_t  audEncOutput;
  tint  retval = SIU_NOERR;
  XDAS_Int32 xdmRetVal;

  siu_invalidate_cache();

  audEncInput.bitStream.ptr  = audInputFrame;
  audEncInput.bitStream.size = audInputSize;

  /* transcodedFrame is in L2 (no core competition in dual-core transcoding) */
  audEncOutput.bitStream.ptr  = siuContext.siu_codec_frame_scratch;
  audEncOutput.bitStream.size = 0;

  xdmRetVal = siuTxAudioEncode(inst, &audEncInput, &audEncOutput);
  
  if((inst->stateInst.video.mmcuInst != NULL) && (audEncOutput.bitStream.size > 0)) {
    ifmmcMuxInput_t muxInput;

    muxInput.bitStream.ptr = audEncOutput.bitStream.ptr; 
    muxInput.bitStream.size = audEncOutput.bitStream.size; 
    
    muxInput.ESID = SIU_AUDIO_PID;
    muxInput.mediaType = ifvisys_MEDIA_TYPE_AUDIO;
    muxInput.timestampIncrement = frameTimeStamp - vdebug_encoded_audio_timestamp;

    mmcuSendIn(inst->stateInst.video.mmcuInst,&muxInput);
    
    vdebug_encoded_audio_timestamp = frameTimeStamp;
  }

  if (xdmRetVal == XDM_EFAIL) 
        retval = SIU_ERROR;

  return retval;
} /* siuTxAudioEncoderProcess() */

/* nothing past this point */
