/***************************************************************************************
 * FILE PURPOSE: Generate DSP version section
 ***************************************************************************************
 * FILE NAME: cdspver.c
 *
 * DESCRIPTION: Generates the dsp header information
 *
 * TABS: None
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* Only two header files should be included, dspver.h and mgmicdsp.h.
 * dspver.h is generated through the application make files, and
 * can include other header files. */
#include "dspver.h"
#include "mgmicdsp.h"



#pragma DATA_SECTION (dspImageMap, "DSP_IMAGE")

const struct MGB_DMS_RESTART_INDICATION dspImageMap = {
  
  /* DSP_capabilities */
  GG_DSPVER_DSP_CODECS,

  /*  dsp_aer_tail_len */
  GGBITMAP_WORD2,

  /* dsp_rel */
  GG_DSPVER_DSP_REL,

  /* dsp_subrel */
  GG_DSPVER_DSP_SUBREL,

  /* dsp_make_rev */
  GG_DSPVER_MAKE_REV,

  /* dsp_make_subrev */
  GG_DSPVER_MAKE_SUBREV1,

  /* dsp_make_subrev_1 */
  GG_DSPVER_BUILD_ID_1,

  /* dsp_make_subrev_2 */
  GG_DSPVER_BUILD_ID_2,

  /* dsp_proc */
  GG_DSPVER_DSP_PROC,

  /* reserved */
#ifdef GGBITMAP_WORD10
  GGBITMAP_WORD10,
#else
  0,
#endif

  /* DSP_capabilities_2 */
  0 
#ifdef GG_DSPVER_FRF11_PROTOCOL
#  if (GG_DSPVER_FRF11_PROTOCOL)
  | FRF11_PRESENT 
#  endif
#endif

#ifdef GG_DSPVER_RTP_PROTOCOL
#  if (GG_DSPVER_RTP_PROTOCOL)
  |  RTP_PRESENT  
#  endif
#endif

#ifdef GG_DSPVER_AAL1_PROTOCOL
#  if (GG_DSPVER_AAL1_PROTOCOL)
  |  AAL1_PRESENT  
#  endif
#endif

#ifdef GG_DSPVER_AAL2_PROTOCOL
#  if (GG_DSPVER_AAL2_PROTOCOL)
  |  AAL2_PRESENT  
#  endif
#endif

#ifdef GG_DSPVER_FAX_NO_T38_ENABLED
#  if (GG_DSPVER_FAX_NO_T38_ENABLED)
  |  FAX_NO_T38U_PRESENT 
#  endif
#endif

#ifdef GG_DSPVER_FAX_T38_ENABLED
#  if (GG_DSPVER_FAX_T38_ENABLED)
  |  FAX_T38_PRESENT 
#  endif
#endif

#ifdef GG_DSPVER_CONF_ENABLED
#  if (GG_DSPVER_CONF_ENABLED)
  |  CONF_PRESENT 
#  endif
#endif

#ifdef GG_DSPVER_ECU_TAIL_ENABLED
#  if (GG_DSPVER_ECU_TAIL_ENABLED)
  |  LTEC_PRESENT 
#  endif
#endif

#ifdef GG_DSPVER_ADPCM_ENABLED
#  if (GG_DSPVER_ADPCM_ENABLED)
  | GG_DSPVER_ADPCM_ENABLED 
#endif
#endif

#ifdef GGBITMAP_WORD11
  |  GGBITMAP_WORD11 
#endif
  
  |   0,

  /* DSP_capabilities_3 */
#ifdef GGBITMAP_WORD12
  (GGBITMAP_WORD12  
#ifdef GG_DSPVER_WORD12CODECS
  & GG_DSPVER_WORD12CODECS 
#endif
  | 0),
#else
  0,
#endif 

  /* number of channels */
  (GG_MAX_NUM_CHANNELS + GG_MAX_AUX_PCM_CHANNELS + GG_MAX_PKT2PKT_CHANNELS),

  /* ECU_tail_length */
#ifdef GG_MAX_ECHO_TAIL_LENGTH
  GG_MAX_ECHO_TAIL_LENGTH,
#else
  0,
#endif

  /* number_of_PCM_only_channels */
#ifdef GG_NUM_PCM_ONLY_CH
  GG_NUM_PCM_ONLY_CH,
#else
  0,
#endif

  /* DSP_capabilites_4 */
  0 
#ifdef GG_DSPVER_DLCOMP
  | GG_DSPVER_DLCOMP  
#endif

#ifdef GG_DSPVER_MORE_CODECS
  | GG_DSPVER_MORE_CODECS 
#endif
  | 0 ,

  /* DSP_capabilites_5 */
#ifdef GGBITMAP_WORD17
  GGBITMAP_WORD17,
#else
  0,
#endif

  /* DSP_capabilities_6 */
  0 
#ifdef GGBITMAP_WORD18
  | GGBITMAP_WORD18 
#endif

#ifdef GG_DSPVER_CID_ENABLED_MASK
  & GG_DSPVER_CID_ENABLED_MASK 
#endif
  | 0,

  /* Security Capability Bits */
#ifdef GGBITMAP_WORD19
  GGBITMAP_WORD19,
#else
  0,
#endif

#ifdef GGBITMAP_WORD20
  (((GGBITMAP_WORD20 
#ifdef GG_DSPVER_FAXMASK
      & GG_DSPVER_FAXMASK
#endif 
    ) 
#ifdef GG_DSPVER_G711A1
    | GG_DSPVER_G711A1 
#endif 
#ifdef GG_DSPVER_G711A2
    | GG_DSPVER_G711A2 
#endif 
    | 0
   ) 
#ifdef GG_DSPVER_WORD20CODECS
   & GG_DSPVER_WORD20CODECS 
#endif
   | 0 
  ),
#else 
  0,
#endif

#ifdef GGBITMAP_WORD21
  (GGBITMAP_WORD21  
#ifdef GG_DSPVER_WORD21CODECS
  & GG_DSPVER_WORD21CODECS 
#endif
  | 0 ),
#else
  0,
#endif 

  /* AER INST0 INFO */
  GGBITMAP_WORD22,
 
  /* AER INST1 INFO */
  GGBITMAP_WORD23,
 
  /* AER INST2 INFO */
  GGBITMAP_WORD24,
 
  /* AER INST3 INFO */
  GGBITMAP_WORD25,
 
#ifdef GG_CID_MAX_SUBSEGMENT
  GG_CID_MAX_SUBSEGMENT, /* word 26 */
#else
  0,
#endif 

  /* buildid_msw */
  GG_DSPVER_IDMSW + 0, /* The +0 allows GG_DSPVER_IDMSW to be blank */
 
  /* buildid_lsw */
  GG_DSPVER_IDLSW + 0, /* The +0 allows GG_DSPVER_IDLSW to be blank */
 
  /* buildid_date */
  GG_DSPVER_DATE + 0, /* The +0 allows GG_DSPVER_DATE to be blank */
 
  /* buildid_time */
  GG_DSPVER_TIME + 0, /* The +0 allows GG_DSPVER_TIME to be blank */
 
  /* reserved 31 */
  0,
 
  /* reserved 32 */
#ifdef GGBITMAP_WORD32
  GGBITMAP_WORD32,
#else
#ifdef GG_INCLUDE_REDUNDANCY
  (GG_INCLUDE_REDUNDANCY_SLOTS << MGB_REDUNDANCY_SLOT_SHIFT),
#else
  0,
#endif  
#endif

  /* word 33 */
#ifdef GGBITMAP_WORD33
  GGBITMAP_WORD33,
#else
  0,
#endif
 
  /* word 34 */
#ifdef GGBITMAP_WORD34
  GGBITMAP_WORD34,
#else
  0,
#endif
 
  /* word 35 (note:It can be expanded to 48 words */
  0        

};
/* Nothing past this point */

