/******************************************************************************
 * FILE PURPOSE: Integrate H264 BP/MP Decoder in GDK
 ******************************************************************************
 * FILE NAME:   vtk_h264bp_mp_dec.c
 *
 * DESCRIPTION: This file contains the implementation for integrating H.264 BP 
 *              Decoder in VIDEO_GDK
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2011, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vtk.h"
#include "vtk_h264bp_mp_dec.h"

#pragma DATA_SECTION (H264BP_MP_DEC_BUFFER_BIT_MAP, ".codecBitMaps");
tulong H264BP_MP_DEC_BUFFER_BIT_MAP[2] = {0x0EFFFF80, 0x00000260};
//tulong H264BP_MP_DEC_BUFFER_BIT_MAP[2] = {0x02600E80};

XDAS_Int32 vtkCreateH264decoder (void *sysInst, IH264VDEC_Params *h264DecStaticParams, 
                                 IALG_Handle *handle) {

    if ((*handle =  (IALG_Handle)vtkContext.ALG_Create (sysInst,
      (IALG_Fxns *) &H264VDEC_TI_IH264VDEC, 
      (IALG_Handle) NULL, (IALG_Params *) h264DecStaticParams, &H264BP_MP_DEC_BUFFER_BIT_MAP[0])) == NULL) {
      return XDM_EFAIL ;
    }
    return XDM_EOK;
}

/* nothing past this point */
