/******************************************************************************
 * FILE PURPOSE: Integrate MPEG-4/H.263 Encoder in GDK
 ******************************************************************************
 * FILE NAME:   vtk_m4h3_enc.c
 *
 * DESCRIPTION: This file contains the implementation for integrating MPEG-4/H.263 
 *              Encoder in VIDEO_GDK
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2009, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "vtk.h"
#include "vtk_m4h3_enc.h"

#pragma DATA_SECTION (M4H3_ENC_BUFFER_BIT_MAP, ".codecBitMaps");
tulong M4H3_ENC_BUFFER_BIT_MAP = 0x00000304;

XDAS_Int32 vtkCreateM4H3encoder (void *sysInst, IMP4VENC_Params *ext_params, 
                                 IALG_Handle *handle) {
  
    if ((*handle =  (IALG_Handle)vtkContext.ALG_Create (sysInst,
      (IALG_Fxns *) &MP4VENC_TI_IMP4VENC, 
      (IALG_Handle) NULL, (IALG_Params *) ext_params,&M4H3_ENC_BUFFER_BIT_MAP)) == NULL) {
      return XDM_EFAIL ;
    }
    return XDM_EOK;
}

void vtkReduceMipsM4H3encoder (void *dyn_par, void *saved_pars)
{
  IMP4VENC_DynamicParams *mp4enc_dynpar = (IMP4VENC_DynamicParams *)dyn_par;            
  XDAS_Int32 *saved_parameters = (XDAS_Int32 *)saved_pars;
  
    *saved_parameters ++ = mp4enc_dynpar->useHpi;
    mp4enc_dynpar->useHpi = 0;
}

void vtkRestoreMipsM4H3encoder (void *dyn_par, void *saved_pars)
{
  IMP4VENC_DynamicParams *mp4enc_dynpar = (IMP4VENC_DynamicParams *)dyn_par;
  XDAS_Int32 *saved_parameters = (XDAS_Int32 *)saved_pars;  
     /* Restore original configuration */
     mp4enc_dynpar->useHpi = *saved_parameters ++;
}

/* nothing past this point */
