/******************************************************************************
 * FILE PURPOSE: Integrate MPEG-2 Decoder in GDK
 ******************************************************************************
 * FILE NAME:   vtk_mpeg2mp_dec.c
 *
 * DESCRIPTION: This file contains the implementation for integrating MPEG-2 
 *              Decoder in VIDEO_GDK
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2009, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vtk.h"
#include "vtk_mpeg2mp_dec.h"
#include <string.h> /* for memset */

#pragma DATA_SECTION (MPEG2MP_DEC_BUFFER_BIT_MAP, ".codecBitMaps");
tulong MPEG2MP_DEC_BUFFER_BIT_MAP = 0x00000020;

XDAS_Int32 vtkCreateMPEG2MPdecoder (void *sysInst, IMPEG2VDEC_Params *mpeg2viddecParams,
                                    IALG_Handle *handle) {

  if ((*handle =  (IALG_Handle)vtkContext.ALG_Create (sysInst,
      (IALG_Fxns *) &MPEG2VDEC_TI_IMPEG2VDEC, 
      (IALG_Handle) NULL, (IALG_Params *) mpeg2viddecParams,&MPEG2MP_DEC_BUFFER_BIT_MAP)) == NULL) {
      return XDM_EFAIL ;
  }
  return XDM_EOK;
}

void vtkResetMPEG2MPdecoder (IALG_Handle *handle, IVIDDEC2_Fxns *fxns, IVIDDEC2_DynamicParams *dynamicParams) 
{
    IMPEG2VDEC_Status status;

    memset (&status,0, sizeof(IMPEG2VDEC_Status));
    status.viddecStatus.size   = sizeof(IMPEG2VDEC_Status);

    /* Assigning Algorithm handle fxns field to ividDecfxns */
    fxns->control((IVIDDEC2_Handle)handle, XDM_RESET,
                   dynamicParams,
                  (IVIDDEC2_Status *)&status);

    return;
}

/* nothing past this point */
