/******************************************************************************/
/*            Copyright (c) 2006 Texas Instruments Incorporated.    
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/******************************************************************************/

/* Standard C header files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include "vtk.h"
#include "vtkloc.h"
#include "emureg.h"
#include "vctypes.h"
#include "ti/xdais/ialg.h"
#include "vtk_h264_enc.h"
#include "vtk_h264bp_mp_dec.h"
#include "vtk_h264hp_dec.h"
#include "vtk_m4h3_enc.h"
#include "vtk_m4h3_dec.h"
#include "vtk_mpeg2mp_dec.h"
#include "types.h"
/******************************************************************************/
#if 0
/* Compile only ONE codec library with _VIDEO_DMA_STATS macro set,
 * then enable this portion of code. */
#pragma DATA_SECTION (qSetQueueThreTC0TC1,".edmaStats") 
#pragma DATA_SECTION (qSetQueueThreTC2TC3,".edmaStats") 
#pragma DATA_SECTION (edma_wait_counter,".edmaStats") 
/* edma_total_wait_time is the total time waiting for transfers to finish */
#pragma DATA_SECTION (edma_total_wait_time,".edmaStats") 
#pragma DATA_SECTION (edma_min_wait,".edmaStats") 
#pragma DATA_SECTION (edma_max_wait,".edmaStats") 
#pragma DATA_SECTION (edma_tc_wait_counter,".edmaStats") 
#pragma DATA_SECTION (edma_tc_wait_counter0,".edmaStats") 
#pragma DATA_SECTION (edma_tc_wait_counter1,".edmaStats") 
#pragma DATA_SECTION (edma_tc_wait_counter2,".edmaStats")
/* edma_tc_total_wait_time is total time waiting to initiate the transfer. To initiate the transfer,
   we need to have one of TCs available. 
   edma_tc_total_wait_time = edma_tc_t0_wait + edma_tc_t1_wait + edma_tc_t2_wait
   case edma_tc_t0_wait: # of transfers in TC2/3 is <  threshold qSetQueueThreTC2TC3.
   case edma_tc_t1_wait: # of transfers in TC2/3 is >= threshold qSetQueueThreTC2TC3 AND 
                         # of transfers in TC0/1 is <  threshold qSetQueueThreTC0TC1
   case edma_tc_t2_wait: # of transfers in TC2/3 is >= threshold qSetQueueThreTC2TC3 AND 
                         # of transfers in TC0/1 is >= threshold qSetQueueThreTC0TC1,
                         we wait for # of transfers in TC2/3 to fall below threshold
 */
#pragma DATA_SECTION (edma_tc_total_wait_time,".edmaStats")
#pragma DATA_SECTION (edma_tc_t0_wait,".edmaStats") 
#pragma DATA_SECTION (edma_tc_t1_wait,".edmaStats") 
#pragma DATA_SECTION (edma_tc_t2_wait,".edmaStats")
#pragma DATA_SECTION (edma_tc_min_wait,".edmaStats") 
#pragma DATA_SECTION (edma_tc_max_wait,".edmaStats") 

/* edma_transfers_toddr2_bytes(MSW-LSW): # of bytes transferred to DDR2 from L2
   edma_transfers_toddr2_numlinks: # of linked transfers(not directly related to 
                                   edma_transfers_toddr2_bytes)
*/
#pragma DATA_SECTION (edma_transfers_toddr2_bytes_lsw,".edmaStats") 
#pragma DATA_SECTION (edma_transfers_toddr2_bytes_msw,".edmaStats") 
#pragma DATA_SECTION (edma_transfers_toddr2_numlinks,".edmaStats") 

/* edma_transfers_fromddr2_bytes(MSW-LSW): # of bytes transferred from DDR2 to L2
   edma_transfers_fromddr2_numlinks: # of linked transfers(not directly related to 
                                     edma_transfers_fromddr2_bytes)
*/
#pragma DATA_SECTION (edma_transfers_fromddr2_bytes_lsw,".edmaStats") 
#pragma DATA_SECTION (edma_transfers_fromddr2_bytes_msw,".edmaStats") 
#pragma DATA_SECTION (edma_transfers_fromddr2_numlinks,".edmaStats") 

/* edma_t0_wait,edma_t1_wait,edma_curr_wait_time are scratch variables */
#pragma DATA_SECTION (edma_t0_wait,".edmaStats") 
#pragma DATA_SECTION (edma_t1_wait,".edmaStats") 
#pragma DATA_SECTION (edma_curr_wait_time,".edmaStats") 
 
/* TC0TC1 throughput is low so want to minimize the transfers done by TC0TC1. In general, it is better
   to wait for TC2TC3 to drain and post to TC2TC3 instead of posting on TC0TC1 
 */
tulong qSetQueueThreTC0TC1 =  4;
/* max is 16 but set to 11 because there is no global critical section and cores may race*/
tulong qSetQueueThreTC2TC3 = 11; 
tulong edma_wait_counter = 0;
tulong edma_total_wait_time = 0;
tulong edma_min_wait = 1000000000;
tulong edma_max_wait = 0;
tulong edma_tc_wait_counter = 0;
tulong edma_tc_wait_counter0 = 0;
tulong edma_tc_wait_counter1 = 0;
tulong edma_tc_wait_counter2 = 0;
tulong edma_tc_total_wait_time = 0;
tulong edma_tc_min_wait = 1000000000;
tulong edma_tc_max_wait = 0;
/*************************************/
tulong edma_transfers_toddr2_bytes_lsw = 0;
tulong edma_transfers_toddr2_bytes_msw = 0;
tulong edma_transfers_toddr2_numlinks  = 0;
tulong edma_transfers_fromddr2_bytes_lsw = 0;
tulong edma_transfers_fromddr2_bytes_msw = 0;
tulong edma_transfers_fromddr2_numlinks  = 0;
/*************************************/
tulong edma_t0_wait = 0;
tulong edma_t1_wait = 0;
tulong edma_curr_wait_time = 0;
#endif
/******************************************************************************
 * FUNCTION PURPOSE: Exception function
 ******************************************************************************
 * DESCRIPTION: Relays a fatal exception through the global exception
 *              pointer.
 *****************************************************************************/
void vtk_exception(tuint ID, tuint code)
{
   vtkContext.debugInfo (ID, DEBUG_EXCEPTION_FATAL, code, 0, NULL);
}

/*************************************************************************************/
tint vtkOpen (void *vtkInst, vtkCfg_t *cfg)
{
  vtkInst_t *inst = (vtkInst_t *)vtkInst;

  vtk_exc_assert((inst->state == VTK_STATE_CLOSED), VTK_EXC_OPEN, inst);
  vtk_exc_assert((cfg->img_width  != 0),VTK_EXC_INVD_IMG_WIDTH, inst);
  vtk_exc_assert((cfg->img_height != 0),VTK_EXC_INVD_IMG_HEIGHT, inst);

  /* Store configuration data */
  inst->cfg     = *cfg;
  inst->sysInst = cfg->sysInst;
  
  /* reset tx and rx stats */
  memset(&inst->stats,0,sizeof(vtkStats_t));
  inst->state = VTK_STATE_OPENED_NOT_CONFIGURED;
  
  return(VTK_NOERR);
}
/******************************************************************************/
tint vtkCommitCodec (void *vtkInst) 
{
  vtkInst_t *inst = (vtkInst_t *)vtkInst;
  vtkEncInst_t     *vtkEncInst  = &(inst->codec.enc);
  vtkDecInst_t     *vtkDecInst  = &(inst->codec.dec);
  vtkCodecInfo_t info;
  tint retVal = VTK_NOERR;
  
  inst->statusCode = 0;
  inst->cfg.inputStreamFormat = 0;
  memset(&info,0,sizeof(vtkCodecInfo_t));
  vtkGetCodecInfo(inst, &info);
  
  switch (inst->cfg.codingType) {
  case EVCT_VIDEO_H264BP_ENC:
      {
          vtkEncInstH264_t *h264EncInst = &(vtkEncInst->u.h264);
          IH264VENC_Params *h264EncStaticParams     = &(h264EncInst->ext_params);
          h264EncStaticParams->mcVidEncParams = &(inst->mcParams);
          vtkCreateH264encoder(inst->sysInst, h264EncStaticParams, &(vtkEncInst->handle));
          if(vtkEncInst->handle == NULL){
              return(VTK_ERROR);
          }
          inst->cfg.maxImgWidth  = h264EncStaticParams->videncParams.maxWidth;
          inst->cfg.maxImgHeight = h264EncStaticParams->videncParams.maxHeight;
          inst->cfg.maxFrameRate = h264EncStaticParams->videncParams.maxFrameRate;
          inst->cfg.maxBitRate   = h264EncStaticParams->videncParams.maxBitRate;
          /* Get codec IRES functions */
          inst->iresFxns = (void*)&H264VENC_TI_IRES;
      }
       break;

  case EVCT_VIDEO_MPEG4SP_ENC:
  case EVCT_VIDEO_H263P_ENC:
      {
          vtkEncInstMPEG4_t *mpeg4EncInst = &(vtkEncInst->u.mpeg4);
          IMP4VENC_Params *mp4EncStaticParams  = &(mpeg4EncInst->ext_params);
          IMP4VENC_DynamicParams *mpeg4EncDynPar = &(mpeg4EncInst->ext_dynpar);
          vtkEncInst->headerFlag                 = 0; 
          if(vtkEncInst->headerFlag)
              mpeg4EncDynPar->videncDynamicParams.generateHeader = XDM_GENERATE_HEADER; 

          vtkCreateM4H3encoder (inst->sysInst, mp4EncStaticParams, &(vtkEncInst->handle));
          if(vtkEncInst->handle == NULL){
              return(VTK_ERROR);
          }
          inst->cfg.maxImgWidth  = mp4EncStaticParams->videncParams.maxWidth;
          inst->cfg.maxImgHeight = mp4EncStaticParams->videncParams.maxHeight;
          inst->cfg.maxFrameRate = mp4EncStaticParams->videncParams.maxFrameRate;
          inst->cfg.maxBitRate   = mp4EncStaticParams->videncParams.maxBitRate;
          /* Get codec IRES functions */
          inst->iresFxns = (void*)&MP4VENC_TI_IRES;
      }
      break;
  case EVCT_VIDEO_H264BP_MP_DEC:
      {
          vtkDecInstH264BP_MP_t *h264DecInst = &(vtkDecInst->u.h264bpmp);
          IH264VDEC_Params *h264DecStaticParams     = &(h264DecInst->ext_params);
          vtkDecInst->framesToDecode        = 1;  // Number of frames to be coded
          vtkDecInst->maxDisplayDelay       = inst->cfg.maxDisplayDelay;
          vtkDecInst->first_field_decoded   = FALSE;
          vtkCreateH264decoder (inst->sysInst, h264DecStaticParams, &(vtkDecInst->handle));
          if(vtkDecInst->handle == NULL){
              return(VTK_ERROR);
          }
          inst->cfg.maxImgWidth  = h264DecStaticParams->viddecParams.maxWidth;
          inst->cfg.maxImgHeight = h264DecStaticParams->viddecParams.maxHeight;
          inst->cfg.maxFrameRate = h264DecStaticParams->viddecParams.maxFrameRate;
          inst->cfg.maxBitRate   = h264DecStaticParams->viddecParams.maxBitRate;
          inst->cfg.inputStreamFormat = (tuint) h264DecStaticParams->inputStreamFormat;
          /* Get codec IRES functions */
          inst->iresFxns = (void*)&H264VDEC_TI_IRES;
      }
      break;

  case EVCT_VIDEO_H264HP_DEC:
      {
          vtkDecInstH264HP_t *h264DecInst = &(vtkDecInst->u.h264hp);
          IH264HPVDEC_Params *h264DecStaticParams     = &(h264DecInst->ext_params);
          vtkDecInst->framesToDecode        = 1;  // Number of frames to be coded
          vtkDecInst->maxDisplayDelay       = inst->cfg.maxDisplayDelay;
          vtkDecInst->first_field_decoded   = FALSE;
          h264DecStaticParams->mcViddecParams = &(inst->mcParams);
          vtkCreateH264HPdecoder (inst->sysInst, h264DecStaticParams, &(vtkDecInst->handle));
          if(vtkDecInst->handle == NULL){
              return(VTK_ERROR);
          }
          inst->cfg.maxImgWidth  = h264DecStaticParams->viddecParams.maxWidth;
          inst->cfg.maxImgHeight = h264DecStaticParams->viddecParams.maxHeight;
          inst->cfg.maxFrameRate = h264DecStaticParams->viddecParams.maxFrameRate;
          inst->cfg.maxBitRate   = h264DecStaticParams->viddecParams.maxBitRate;
          inst->cfg.inputStreamFormat = (tuint) h264DecStaticParams->inputStreamFormat;
          /* Get codec IRES functions */
          inst->iresFxns = (void*)&H264HPVDEC_TI_IRES;
      }
      break;

  case EVCT_VIDEO_MPEG4SP_DEC:
  case EVCT_VIDEO_H263P_DEC:
      {
          vtkDecInstMPEG4_t *mpeg4DecInst = &(vtkDecInst->u.mpeg4);
          IVIDDEC2_Params   *mpeg4DecStaticParams = &(mpeg4DecInst->params);
          IM4H3DEC_DynamicParams *dynamicParams   = &(mpeg4DecInst->ext_dynpar);
          vtkDecInst->framesToDecode        = 1;       // Number of frames to be coded
          vtkDecInst->maxDisplayDelay       = inst->cfg.maxDisplayDelay;
          vtkDecInst->first_field_decoded   = FALSE;
          vtkCreateM4H3decoder(inst->sysInst, mpeg4DecStaticParams,  &(vtkDecInst->handle));
          if(vtkDecInst->handle == NULL){
              return(VTK_ERROR);
          }
          vtkResetM4H3decoder (&(vtkDecInst->handle), (IVIDDEC2_Fxns *)vtkDecInst->handle->fxns,
                                (IVIDDEC2_DynamicParams *)&dynamicParams);
          inst->cfg.maxImgWidth  = mpeg4DecStaticParams->maxWidth;
          inst->cfg.maxImgHeight = mpeg4DecStaticParams->maxHeight;
          inst->cfg.maxFrameRate = mpeg4DecStaticParams->maxFrameRate;
          inst->cfg.maxBitRate   = mpeg4DecStaticParams->maxBitRate;
          /* Get codec IRES functions */
          inst->iresFxns = (void*)&M4H3DEC_TI_IRES;
      }
      break;
  case EVCT_VIDEO_BYPASS_ENC:
  case EVCT_VIDEO_BYPASS_DEC:
      break;

  case EVCT_VIDEO_MPEG2MP_DEC:
      {
          vtkDecInstMPEGMP_t *mpeg2DecInst = &(vtkDecInst->u.mpeg2);
          IMPEG2VDEC_Params  *mpeg2DecStaticParams = &(mpeg2DecInst->ext_params);
          IMPEG2VDEC_DynamicParams *dynamicParams  = &(mpeg2DecInst->ext_dynpar);
          vtkDecInst->framesToDecode        = 1;       // Number of frames to be coded
          vtkDecInst->maxDisplayDelay       = inst->cfg.maxDisplayDelay;
          vtkDecInst->first_field_decoded   = FALSE;
          vtkCreateMPEG2MPdecoder(inst->sysInst, mpeg2DecStaticParams,  &(vtkDecInst->handle));
          if(vtkDecInst->handle == NULL){
             return(VTK_ERROR);
          }
          vtkResetMPEG2MPdecoder (&(vtkDecInst->handle), (IVIDDEC2_Fxns *)vtkDecInst->handle->fxns,
                                   (IVIDDEC2_DynamicParams *)&dynamicParams);
          inst->cfg.maxImgWidth  = mpeg2DecStaticParams->viddecParams.maxWidth;
          inst->cfg.maxImgHeight = mpeg2DecStaticParams->viddecParams.maxHeight;
          inst->cfg.maxFrameRate = mpeg2DecStaticParams->viddecParams.maxFrameRate;
          inst->cfg.maxBitRate   = mpeg2DecStaticParams->viddecParams.maxBitRate;
          /* Get codec IRES functions */
          inst->iresFxns = (void*)&MPEG2VDEC_TI_IRES;
      }
      break;

  default:
      vtk_exception(inst->ID, VTK_EXC_INVD_CODEC);
      break;
  }
  /* set VTK state to OPEN as memory is allocated */
  inst->state = VTK_STATE_OPEN;

  switch (info.type) {
      case VTK_TYPE_ENCODER:
        {
          vtkEncInst->handle->fxns->algActivate(vtkEncInst->handle);
          /* assign EDMA resources */
          if(vtkContext.RES_assign((void*)vtkEncInst->handle, inst->iresFxns) == FALSE) {
            retVal = VTK_ERROR;   
          }
          if(info.xdmVer == VTK_XDM_1P0) {
            IVIDENC1_Status        *enc_status1 = info.encArgs.xdmver.xdm1p0.enc_status1;
            IVIDENC1_DynamicParams *enc_dynpar1 = info.encArgs.xdmver.xdm1p0.enc_dynpar1;
            IVIDENC1_Fxns         *ividEncFxns1 = (IVIDENC1_Fxns *)vtkEncInst->handle->fxns;
            if(ividEncFxns1->control((IVIDENC1_Handle)vtkEncInst->handle, XDM_SETPARAMS, enc_dynpar1, enc_status1) != XDM_EOK) {
              retVal = VTK_ERROR;   
            }
          } else if(info.xdmVer == VTK_XDM_0P9) {
            IVIDENC_Status        *enc_status = info.encArgs.xdmver.xdm0p9.enc_status;
            IVIDENC_DynamicParams *enc_dynpar = info.encArgs.xdmver.xdm0p9.enc_dynpar;
            IVIDENC_Fxns         *ividEncFxns = (IVIDENC_Fxns *)vtkEncInst->handle->fxns;
            if(ividEncFxns->control((IVIDENC_Handle)vtkEncInst->handle, XDM_SETPARAMS, enc_dynpar, enc_status) != XDM_EOK) {
              retVal = VTK_ERROR;   
            }
          }
          vtkEncInst->handle->fxns->algDeactivate(vtkEncInst->handle);
        }
        break;
      case VTK_TYPE_DECODER:
          {
          vtkDecInst->handle->fxns->algActivate(vtkDecInst->handle);
          /* assign EDMA resources */
          if(vtkContext.RES_assign((void*)vtkDecInst->handle, inst->iresFxns) == FALSE) {
            retVal = VTK_ERROR;   
          }
          if(info.xdmVer == VTK_XDM_1P0) {
            IVIDDEC2_Status  *dec_status2       = info.decArgs.xdmver.xdm1p0.dec_status2;
            IVIDDEC2_DynamicParams *dec_dynpar2 = info.decArgs.xdmver.xdm1p0.dec_dynpar2;
            IVIDDEC2_Fxns    *ividDecFxns2      = (IVIDDEC2_Fxns *)vtkDecInst->handle->fxns;
            if(ividDecFxns2->control((IVIDDEC2_Handle)vtkDecInst->handle, XDM_SETPARAMS, dec_dynpar2, dec_status2) != XDM_EOK) {
              retVal = VTK_ERROR;   
            }
          } else if(info.xdmVer == VTK_XDM_0P9) {
            IVIDDEC_Status  *dec_status       = info.decArgs.xdmver.xdm0p9.dec_status;
            IVIDDEC_DynamicParams *dec_dynpar = info.decArgs.xdmver.xdm0p9.dec_dynpar;
            IVIDDEC_Fxns        *ividDecFxns  = (IVIDDEC_Fxns *)vtkDecInst->handle->fxns;
            if(ividDecFxns->control((IVIDDEC_Handle)vtkDecInst->handle, XDM_SETPARAMS, dec_dynpar, dec_status) != XDM_EOK) {
              retVal = VTK_ERROR;   
            }
          }
          vtkDecInst->handle->fxns->algDeactivate(vtkDecInst->handle);
          }
          break;
  }
  return(retVal);
}


/******************************************************************************/
void vtkClose (void *vtkInst)
{
  vtkInst_t *inst = (vtkInst_t *)vtkInst;
  vtkCodecInfo_t info;
  IALG_Handle handle;

  memset(&info,0,sizeof(vtkCodecInfo_t));
  vtkGetCodecInfo(inst, &info);

  /* free the codec memory */
  switch(inst->cfg.codingType) {
  case EVCT_VIDEO_BYPASS_ENC:
  case EVCT_VIDEO_BYPASS_DEC:
       break;

  default:
       /* Free FC-RMAN resources */
       if(info.type == VTK_TYPE_ENCODER) {
          handle = inst->codec.enc.handle;
       } else if (info.type == VTK_TYPE_DECODER) {
          handle = inst->codec.dec.handle;
       }

       handle->fxns->algActivate(handle);
       vtkContext.RES_free((void *)handle, inst->iresFxns);
       handle->fxns->algDeactivate(handle);

       /* Free codec instance */
       if(inst->state == VTK_STATE_OPEN){
         vtkContext.ALG_Free(inst->sysInst);
       }
       break;
  }
  inst->state = VTK_STATE_CLOSED;  
  return;
}
/******************************************************************************/

/***********************************************************************************
 * FUNCTION PURPOSE: Perform Control actions on VTK
 ***********************************************************************************
 * DESCRIPTION: This function is called to perform Control actions on VTK including
 *              retrieval of codec static, dynamic configuration structures, commit
 *              of codec, retrieval of VTK configuration etc.
 *
 * void vtkControl (
 *    void *             vtkInst       - VTK instance pointer
 *    vtkCtrl_t *        ctrl          - VTK control structure
 *                 )
 *
 ************************************************************************************/
tint vtkControl (void *vtkInst, vtkCtrl_t *ctrl) {

    vtkInst_t *inst = (vtkInst_t *)vtkInst;
    vtkEncInst_t *vtkEncInst = &(inst->codec.enc);
    XDAS_UInt32 *saved_parameters = vtkEncInst->savedParameters;    
    tint retVal = VTK_NOERR;
  
    switch(ctrl->code) {
      case VTK_GET_STATIC_CFG_PTR:
          vtkGetStaticCfgPtr (inst, &(ctrl->u.codecCfgPtr.cfgPtr), &(ctrl->u.codecCfgPtr.size));
          break;

      case VTK_GET_DYNAMIC_CFG_PTR:
          vtkGetDynamicCfgPtr (inst, &(ctrl->u.codecCfgPtr.cfgPtr), &(ctrl->u.codecCfgPtr.size));
          break;

      case VTK_COMMIT_CODEC:
          retVal = vtkCommitCodec(inst);
          break;

      case VTK_GET_CFG:
          retVal = vtkGetConfig(inst, &(ctrl->u.vtkCfg));
          break;

      case VTK_GET_ENC_DYN_CFG:
          retVal = vtkGetEncDynConfig(inst, &(ctrl->u.vtkEncDynCfg));
          break;

      case VTK_SET_ENC_DYN_CFG:
          retVal = vtkSetEncDynConfig(inst, &(ctrl->u.vtkEncDynCfg));
          break;

      case VTK_IS_DYN_UPDATE_ALLOWED:
          retVal = vtkIsDynUpdateAllowed(inst, ctrl->u.codecCfgPtr.cfgPtr);
          break;

      case VTK_GET_STATS:
          break;
#ifdef VTK_ACTIVE_MIPS_CTRL          
      case VTK_REDUCE_MIPS:
          switch (inst->cfg.codingType) {
            case EVCT_VIDEO_H264BP_ENC:
              vtkReduceMipsH264encoder (&vtkEncInst->u.h264.ext_dynpar, saved_parameters);              
              vtkEncInst->savedParametersAvail = 1;  /* Indicate it has been saved now */
              break;

            case EVCT_VIDEO_MPEG4SP_ENC:
            case EVCT_VIDEO_H263P_ENC:      
              vtkReduceMipsM4H3encoder (&vtkEncInst->u.mpeg4.ext_dynpar, saved_parameters);
              vtkEncInst->savedParametersAvail = 1;              
            default:
              break;
          }            
          break;
      case VTK_RESTORE_MIPS:
          /* Return if there are no valid parameters saved */
          /* "Deserialize" paramters stored during MIPS shortage */          
          if(!vtkEncInst->savedParametersAvail) break;
          switch (inst->cfg.codingType) {
            case EVCT_VIDEO_H264BP_ENC:
              vtkRestoreMipsH264encoder (&vtkEncInst->u.h264.ext_dynpar, saved_parameters);
              vtkEncInst->savedParametersAvail = 0;      
              break;

            case EVCT_VIDEO_MPEG4SP_ENC:
            case EVCT_VIDEO_H263P_ENC:      
              vtkRestoreMipsM4H3encoder(&vtkEncInst->u.mpeg4.ext_dynpar, saved_parameters);
              vtkEncInst->savedParametersAvail = 0;                    
              break;              
            default:
              break;
          }                
		  break;
#endif

      case VTK_RESET_CODEC:
          retVal = vtkReset(inst);
          break;

      case VTK_FLUSH_DEC:
          inst->cfg.decFlush = TRUE;
          break;

      case VTK_SET_MC_PARAMS:
          vtkSetMcParams(inst, &(ctrl->u.vtkMcParams));
          break;

      default:
          break;
    }
    return (retVal);
}

/************************************************************************************/
void vtkDelete (void **vtkInst)
{
   vtkInst_t *inst = (vtkInst_t *) *vtkInst;
   /* Ensure it is already closed */
   vtk_exc_assert (inst->state == VTK_STATE_CLOSED, VTK_EXC_DELETE, inst);

   *vtkInst = NULL;   /* Mark the instance as free */
}
/************************************************************************************/

/******************************************************************************
 * FUNCTION PURPOSE: Update the frame statistics
 ******************************************************************************
 * DESCRIPTION: This function is called after both the encode and decode operation
 *              to update the frame statistics.
 *
 * void vtkStatsUpdateFrameDetails (
 *    vtkEncDecStats_t   *stats,
 *    vtkFrameType_t     type,
 *    tulong             frameSize)  
 *
 *****************************************************************************/
void vtkStatsUpdateFrameDetails (vtkEncDecStats_t *stats, vtkFrameType_t type, tulong frameSize) {

  switch(type) {
    case VTK_I_FRAME:
    case VTK_IDR_FRAME:
    case VTK_MBAFF_I_FRAME:
    case VTK_MBAFF_IDR_FRAME:
        stats->numIFrames++;
        if (frameSize > stats->maxIFrameSize) {
            stats->maxIFrameSize = frameSize;
        }
        stats->lastGOP = stats->lastGOPCtr;
        /* for back to back I-frames, GOP = 1 and if there are 29 P frames btwn I frames, GOP = 30 */
        /* GOP cannot be zero */
        stats->lastGOPCtr = 1; 
        break;
    case VTK_P_FRAME:
    case VTK_MBAFF_P_FRAME:
        stats->numPFrames++;
        /* increment GOP */
        stats->lastGOPCtr++;
        if (frameSize > stats->maxPFrameSize) {
            stats->maxPFrameSize = frameSize;
        }
        break;
    case VTK_B_FRAME:
    case VTK_MBAFF_B_FRAME:
        stats->numBFrames++;
        /* increment GOP */
        stats->lastGOPCtr++;
        if (frameSize > stats->maxBFrameSize) {
            stats->maxBFrameSize = frameSize;
        }
        break;
    default:
        break;
    }
  stats->totalNumBytes += frameSize;
  stats->bitRateByteCount += frameSize;
  stats->bitRateFrameCounter++;
  if (frameSize == 0) {
     stats->skippedFrames++;
  }
  /* TODO: This must not be hardcoded!!! */
  /* use ... videncDynamicParams.intraFrameInterval instead!!! */
  if (stats->bitRateFrameCounter >= 30) {
    tulong curTime = *(vtkContext.curTime);
    tulong deltaTime = curTime - stats->bitRateTime;
    tulong deltaTimeInSec = deltaTime/1000;
    /* we accumulated bitRateByteCount in deltaTime */
    /* bytes per sec */
    stats->bitrate = (tulong)((stats->bitRateByteCount * 8)/(tulong)deltaTimeInSec);

    /* Reset */
    stats->bitRateTime         = curTime;
    stats->bitRateFrameCounter = 0;
    stats->bitRateByteCount    = 0;
  }
  return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Statistics
 ******************************************************************************
 * DESCRIPTION: Get Stats from instance and report to application
 *
 * void vtkGetStats (
 *    void         *vtkInst,
 *    bool         reset
 *    vtkStats_t   *stats )  
 *
 *****************************************************************************/
void vtkGetStats (void *vtkInst, bool reset, vtkStats_t *stats) {
    vtkInst_t *inst = (vtkInst_t *)vtkInst;
    memcpy(stats,&inst->stats,sizeof(vtkStats_t ));
    if (reset) {
      memset(&inst->stats,0,sizeof(vtkStats_t));
    }
    return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Current VTK & Codec Configuration
 ******************************************************************************
 * DESCRIPTION: Get Current VTK Configuration from instance and report to application
 *
 * void vtkCodecConfig (
 *    void         *vtkInst,
 *    vtkCfg_t     *cfg)  
 *
 *****************************************************************************/
tint vtkGetConfig (void *vtkInst, vtkCfg_t *cfg) {
  vtkInst_t *inst = (vtkInst_t *)vtkInst;

  memcpy(cfg, &inst->cfg,sizeof(vtkCfg_t));
  return(VTK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Check if the dynamic config update is legal or not
 ******************************************************************************
 * DESCRIPTION: Check if the dynamic configuration values <= static configuration
 *
 * void vtkIsDynUpdateAllowed (
 *    vtkInst_t  *inst,
 *    tulong     *dynCfg)  
 *
 *****************************************************************************/
tint vtkIsDynUpdateAllowed(vtkInst_t *inst, tulong *dynCfg) {
  IVIDENC_DynamicParams  *dynamicParams09;
  IVIDENC1_DynamicParams *dynamicParams10;
  IVIDENC_Params         *staticParams09;
  IVIDENC1_Params        *staticParams10;
  vtkCodecInfo_t          info;
  vtkEncDynCfg_t          dynamicCfg;
  vtkCfg_t                staticCfg;
  vtkCtrl_t               ctrl;
  tulong *ptr;
  union {
    tulong dummy;
    tulong encDynCfg[sizeof(IVIDENC1_DynamicParams)]; /* making sure that encDynCfg is 32 bit aligned */
  }u;

  /* first variable size is not available in the structure, offset to account for it */
  ptr = u.encDynCfg;
  ptr++;

  utlPackTuintToTulong(dynCfg, ptr, sizeof(IVIDENC1_DynamicParams));
  vtkGetCodecInfo(inst, &info);

  ctrl.code = VTK_GET_STATIC_CFG_PTR;
  vtkControl(inst, &ctrl);

  switch(info.xdmVer){
    case VTK_XDM_0P9:
         dynamicParams09     = (IVIDENC_DynamicParams *)u.encDynCfg;
         staticParams09      = (IVIDENC_Params *)ctrl.u.codecCfgPtr.cfgPtr;
         dynamicCfg.targetFrameRate = dynamicParams09->targetFrameRate;
         dynamicCfg.refFrameRate    = dynamicParams09->refFrameRate;
         dynamicCfg.img_width       = dynamicParams09->inputWidth;
         dynamicCfg.img_height      = dynamicParams09->inputHeight;
         dynamicCfg.targetBitrate   = dynamicParams09->targetBitRate;
         staticCfg.maxImgWidth      = staticParams09->maxWidth;
         staticCfg.maxImgHeight     = staticParams09->maxHeight;
         staticCfg.maxFrameRate     = staticParams09->maxFrameRate;
         staticCfg.maxBitRate       = staticParams09->maxBitRate;
         break;

    case VTK_XDM_1P0:
         dynamicParams10     = (IVIDENC1_DynamicParams *)u.encDynCfg;
         staticParams10      = (IVIDENC1_Params *)ctrl.u.codecCfgPtr.cfgPtr;
         dynamicCfg.targetFrameRate = dynamicParams10->targetFrameRate;
         dynamicCfg.refFrameRate    = dynamicParams10->refFrameRate;
         dynamicCfg.img_width       = dynamicParams10->inputWidth;
         dynamicCfg.img_height      = dynamicParams10->inputHeight;
         dynamicCfg.targetBitrate   = dynamicParams10->targetBitRate;
         staticCfg.maxImgWidth      = staticParams10->maxWidth;
         staticCfg.maxImgHeight     = staticParams10->maxHeight;
         staticCfg.maxFrameRate     = staticParams10->maxFrameRate;
         staticCfg.maxBitRate       = staticParams10->maxBitRate;
         break;

    default:
         return(VTK_ERROR);
  }

  /* check if dynamic is <= static config: codec independent */
  if (dynamicCfg.img_width > staticCfg.maxImgWidth) {
     return(VTK_ERROR);
  }

  if (dynamicCfg.img_height > staticCfg.maxImgHeight) {
     return(VTK_ERROR);
  }

  if (dynamicCfg.targetFrameRate > staticCfg.maxFrameRate) {
     return(VTK_ERROR);
  }

  if(dynamicCfg.targetBitrate > staticCfg.maxBitRate) {
     return(VTK_ERROR);
  }

  /* check constraint from individual codecs */

  return(VTK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Current encoder Codec Configuration
 ******************************************************************************
 * DESCRIPTION: 
 *
 * void vtkGetEncDynConfig (
 *    void         *vtkInst,
 *    vtkEncDynCfg_t     *cfg)  
 *
 *****************************************************************************/
tint vtkGetEncDynConfig (void *vtkInst, vtkEncDynCfg_t *cfg) {
  vtkInst_t *inst = (vtkInst_t *)vtkInst;
  IVIDENC_DynamicParams  *dynamicParams09 = NULL;
  IVIDENC1_DynamicParams *dynamicParams10 = NULL;
  tulong                  size;
  vtkCodecInfo_t          info;

  vtkGetCodecInfo(inst, &info);

  switch(info.xdmVer){
    case VTK_XDM_0P9:
         vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams09, &size);
         cfg->targetFrameRate = dynamicParams09->targetFrameRate;
         cfg->refFrameRate    = dynamicParams09->refFrameRate;
         cfg->img_width       = dynamicParams09->inputWidth;
         cfg->img_height      = dynamicParams09->inputHeight;
         cfg->targetBitrate   = dynamicParams09->targetBitRate;
         break;
    case VTK_XDM_1P0:
         vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams10, &size);
         cfg->targetFrameRate = dynamicParams10->targetFrameRate;
         cfg->refFrameRate    = dynamicParams10->refFrameRate;
         cfg->img_width       = dynamicParams10->inputWidth;
         cfg->img_height      = dynamicParams10->inputHeight;
         cfg->targetBitrate   = dynamicParams10->targetBitRate;
         break;
     default:
         break;
  }
  return(VTK_NOERR);
}
/******************************************************************************
 * FUNCTION PURPOSE: Selectively Set Current encoder Codec Configuration
 ******************************************************************************
 * DESCRIPTION: 
 *
 * void vtkSetEncDynConfig (
 *    void         *vtkInst,
 *    vtkEncDynCfg_t     *cfg)  
 *
 *****************************************************************************/
tint vtkSetEncDynConfig (void *vtkInst, vtkEncDynCfg_t *cfg) {
  vtkInst_t *inst = (vtkInst_t *)vtkInst;
  IVIDENC_DynamicParams  *dynamicParams09 = NULL;
  IVIDENC1_DynamicParams *dynamicParams10 = NULL;
  tulong                  size;
  vtkCodecInfo_t          info;

  vtkGetCodecInfo(inst, &info);

  switch(info.xdmVer){
    case VTK_XDM_0P9:
         vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams09, &size);
         dynamicParams09->targetFrameRate = cfg->targetFrameRate;
         dynamicParams09->refFrameRate    = cfg->refFrameRate;
         break;
    case VTK_XDM_1P0:
         vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams10, &size);
         dynamicParams10->targetFrameRate = cfg->targetFrameRate;
         dynamicParams10->refFrameRate    = cfg->refFrameRate;
         break;
     default:
         break;
  }
  return(VTK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Static Config Pointer from VTK to configure
 ******************************************************************************
 * DESCRIPTION: Get Static Config Pointer from VTK to configure
 *
 * void vtkGetStaticCfgPtr (
 *    void         *vtkInst,     - Instance ptr
 *    tulong       **cfgPtr,     - Return the static Cfg ptr to get initialized
 *    tulong        *size        - Return the size of the static Cfg structure )  
 *
 *****************************************************************************/
tint vtkGetStaticCfgPtr (void *vtkInst,  tulong **cfgPtr, tulong *size) {
  vtkInst_t *inst = (vtkInst_t *)vtkInst;

  switch (inst->cfg.codingType) {
  case EVCT_VIDEO_H264BP_ENC:
      {
        vtkEncInst_t     *vtkEncInst  = &(inst->codec.enc);
        vtkEncInstH264_t *h264EncInst = &(vtkEncInst->u.h264);
        IH264VENC_Params *h264EncStaticParams     = &(h264EncInst->ext_params);
        *cfgPtr  = (tulong *)h264EncStaticParams;
        *size    = sizeof(IH264VENC_Params);
        h264EncStaticParams->videncParams.size = *size;
      }
      break;

  case EVCT_VIDEO_MPEG4SP_ENC:
  case EVCT_VIDEO_H263P_ENC:
      {
        vtkEncInst_t     *vtkEncInst  = &(inst->codec.enc);
        vtkEncInstMPEG4_t *mpeg4EncInst = &(vtkEncInst->u.mpeg4);
        IMP4VENC_Params *mpeg4EncStaticParams     = &(mpeg4EncInst->ext_params);
        *cfgPtr  = (tulong *)mpeg4EncStaticParams;
        *size    = sizeof(IMP4VENC_Params); 
        mpeg4EncStaticParams->videncParams.size = *size;
      }
      break;

  case EVCT_VIDEO_H264BP_MP_DEC: 
      {
        vtkDecInst_t     *vtkDecInst  = &(inst->codec.dec);
        vtkDecInstH264BP_MP_t *h264DecInst = &(vtkDecInst->u.h264bpmp);
        IH264VDEC_Params *h264DecStaticParams     = &(h264DecInst->ext_params);
        *cfgPtr  = (tulong *)h264DecStaticParams;
        *size    = sizeof(IH264VDEC_Params);
        h264DecStaticParams->viddecParams.size = *size;
      }
      break;

  case EVCT_VIDEO_H264HP_DEC: 
      {
        vtkDecInst_t     *vtkDecInst  = &(inst->codec.dec);
        vtkDecInstH264HP_t *h264DecInst = &(vtkDecInst->u.h264hp);
        IH264HPVDEC_Params *h264DecStaticParams     = &(h264DecInst->ext_params);
        *cfgPtr  = (tulong *)h264DecStaticParams;
        *size    = sizeof(IH264HPVDEC_Params);
        h264DecStaticParams->viddecParams.size = *size;
      }
      break;

  case EVCT_VIDEO_MPEG4SP_DEC:
  case EVCT_VIDEO_H263P_DEC:
      {
        vtkDecInst_t     *vtkDecInst  = &(inst->codec.dec);
        vtkDecInstMPEG4_t *mpeg4DecInst = &(vtkDecInst->u.mpeg4);
        IVIDDEC2_Params    *mpeg4DecStaticParams = &(mpeg4DecInst->params);
        *cfgPtr  = (tulong *)mpeg4DecStaticParams;
        *size    = sizeof(IVIDDEC2_Params);
        mpeg4DecStaticParams->size = *size;
      }
      break;

  case EVCT_VIDEO_MPEG2MP_DEC:
      {
        vtkDecInst_t       *vtkDecInst   = &(inst->codec.dec);
        vtkDecInstMPEGMP_t *mpeg2DecInst = &(vtkDecInst->u.mpeg2);
        IMPEG2VDEC_Params  *mpeg2StaticParams = &(mpeg2DecInst->ext_params);
        *cfgPtr = (tulong *)mpeg2StaticParams;
        *size   = sizeof(IMPEG2VDEC_Params);
        mpeg2StaticParams->viddecParams.size = *size;
      }
      break;

  default:
      *cfgPtr = NULL;
      *size   = 0;
      break;
  }
  return (VTK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Dynamic Config Pointer from VTK to configure
 ******************************************************************************
 * DESCRIPTION: Get Dynamic Config Pointer from VTK to configure
 *
 * void vtkGetDynamicCfgPtr (
 *    void         *vtkInst,     - Instance ptr
 *    tulong       **cfgPtr,     - Return the Dynamic Cfg ptr to get initialized
 *    tulong        *size        - Return the size of the Dynamic Cfg structure )  
 *
 *****************************************************************************/
tint vtkGetDynamicCfgPtr (void *vtkInst, tulong **cfgPtr, tulong *size)
{
  vtkInst_t *inst = (vtkInst_t *)vtkInst;

  switch (inst->cfg.codingType) {
  case EVCT_VIDEO_H264BP_ENC:
      {
        vtkEncInst_t     *vtkEncInst  = &(inst->codec.enc);
        vtkEncInstH264_t *h264EncInst = &(vtkEncInst->u.h264);
        IH264VENC_DynamicParams *h264EncDynParams     = &(h264EncInst->ext_dynpar);
        *cfgPtr  = (tulong *)h264EncDynParams;
        *size    = sizeof(IH264VENC_DynamicParams);
        h264EncDynParams->videncDynamicParams.size = *size;
      }
      break;

  case EVCT_VIDEO_MPEG4SP_ENC:
  case EVCT_VIDEO_H263P_ENC:
      {
        vtkEncInst_t     *vtkEncInst  = &(inst->codec.enc);
        vtkEncInstMPEG4_t *mpeg4EncInst = &(vtkEncInst->u.mpeg4);
        IMP4VENC_DynamicParams *mpeg4EncDynParams     = &(mpeg4EncInst->ext_dynpar);
        *cfgPtr  = (tulong *)mpeg4EncDynParams;
        *size    = sizeof(IMP4VENC_DynamicParams);
        mpeg4EncDynParams->videncDynamicParams.size = *size;
      }
      break;

  case EVCT_VIDEO_H264BP_MP_DEC: 
      {
        vtkDecInst_t     *vtkDecInst  = &(inst->codec.dec);
        vtkDecInstH264BP_MP_t *h264DecInst = &(vtkDecInst->u.h264bpmp);
        IH264VDEC_DynamicParams *h264DecDynParams = &(h264DecInst->ext_dynpar);
        *cfgPtr  = (tulong *)h264DecDynParams;
        *size    = sizeof(IH264VDEC_DynamicParams);
        h264DecDynParams->viddecDynamicParams.size = *size;
      }
      break;

  case EVCT_VIDEO_H264HP_DEC: 
      {
        vtkDecInst_t     *vtkDecInst  = &(inst->codec.dec);
        vtkDecInstH264HP_t *h264DecInst = &(vtkDecInst->u.h264hp);
        IH264HPVDEC_DynamicParams *h264DecDynParams = &(h264DecInst->ext_dynpar);
        *cfgPtr  = (tulong *)h264DecDynParams;
        *size    = sizeof(IH264HPVDEC_DynamicParams);
        h264DecDynParams->viddecDynamicParams.size = *size;
      }
      break;

  case EVCT_VIDEO_MPEG4SP_DEC:
  case EVCT_VIDEO_H263P_DEC:
      {
        vtkDecInst_t     *vtkDecInst  = &(inst->codec.dec);
        vtkDecInstMPEG4_t *mpeg4DecInst = &(vtkDecInst->u.mpeg4);
        IM4H3DEC_DynamicParams *mpeg4DecDynParams = &(mpeg4DecInst->ext_dynpar);
        *cfgPtr  = (tulong *)mpeg4DecDynParams;
        *size    = sizeof(IM4H3DEC_DynamicParams);
        mpeg4DecDynParams->viddecDynamicParams.size = *size;
      }
      break;

  case EVCT_VIDEO_MPEG2MP_DEC:
      {
        vtkDecInst_t       *vtkDecInst   = &(inst->codec.dec);
        vtkDecInstMPEGMP_t *mpeg2DecInst = &(vtkDecInst->u.mpeg2);
        IMPEG2VDEC_DynamicParams  *mpeg2DynamicParams = &(mpeg2DecInst->ext_dynpar);
        *cfgPtr = (tulong *)mpeg2DynamicParams;
        *size   = sizeof(IMPEG2VDEC_DynamicParams);
        mpeg2DynamicParams->viddecDynamicParams.size = *size;
      }
      break;

  default:
      *cfgPtr = NULL;
      *size   = 0;
      break;
  }
  return (VTK_NOERR);
}

/******************************************************************************
 * FUNCTION PURPOSE: Reset Codec on the fly
 ******************************************************************************
 * DESCRIPTION: Reset the Codec on the fly when there is an error
 *
 * tint vtkReset (
 *    vtkInst_t         *inst,     - Instance ptr )  
 *
 *****************************************************************************/
tint vtkReset (vtkInst_t *inst)
{
  tint                   retVal = VTK_ERROR;
  tulong                  size;
  vtkCodecInfo_t          info;

  inst->cfg.decFlush = FALSE;
  vtkGetCodecInfo(inst, &info);

  if(info.type == VTK_TYPE_DECODER) {
     vtkDecInst_t           *vtkDecInst  = &(inst->codec.dec);
     IVIDDEC_Status         status1;
     IVIDDEC2_Status        status2;
     IVIDDEC_DynamicParams  *dynamicParams1;
     IVIDDEC2_DynamicParams *dynamicParams2;
     IVIDDEC_Fxns           *fxns1;
     IVIDDEC2_Fxns          *fxns2;
     IVIDDEC_Handle         *handle1;
     IVIDDEC2_Handle        *handle2;

     switch(info.xdmVer){
       case VTK_XDM_1P0:
           memset (&status2, 0 , sizeof(IVIDDEC2_Status));
           status2.size   = sizeof(IVIDDEC2_Status);
           fxns2          = (IVIDDEC2_Fxns *)vtkDecInst->handle->fxns;
           handle2        = (IVIDDEC2_Handle *)(vtkDecInst->handle);

           vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams2, &size);

           /*Call Reset */
           retVal = fxns2->control((IVIDDEC2_Handle)handle2, XDM_RESET, dynamicParams2, (IVIDDEC2_Status *)&status2);
           break;

      case VTK_XDM_0P9:
           memset (&status1, 0 , sizeof(IVIDDEC_Status));
           status1.size   = sizeof(IVIDDEC_Status);
           fxns1          = (IVIDDEC_Fxns *)vtkDecInst->handle->fxns;
           handle1        = (IVIDDEC_Handle *)(vtkDecInst->handle);

           vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams1, &size);

           /*Call Reset */
           retVal = fxns1->control((IVIDDEC_Handle)handle1, XDM_RESET, dynamicParams1, (IVIDDEC_Status *)&status1);
           break;
    }
  } else if (info.type == VTK_TYPE_ENCODER) {
     vtkEncInst_t           *vtkEncInst  = &(inst->codec.enc);
     IVIDENC_Status         status1;
     IVIDENC1_Status        status2;
     IVIDENC_DynamicParams  *dynamicParams1;
     IVIDENC1_DynamicParams *dynamicParams2;
     IVIDENC_Fxns           *fxns1;
     IVIDENC1_Fxns          *fxns2;
     IVIDENC_Handle         *handle1;
     IVIDENC1_Handle        *handle2;

     switch(info.xdmVer){
       case VTK_XDM_1P0:
           memset (&status2, 0 , sizeof(IVIDENC1_Status));
           status2.size   = sizeof(IVIDENC1_Status);
           fxns2          = (IVIDENC1_Fxns *)vtkEncInst->handle->fxns;
           handle2        = (IVIDENC1_Handle *)(vtkEncInst->handle);

           vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams2, &size);

           /*Call Reset */
           retVal = fxns2->control((IVIDENC1_Handle)handle2, XDM_RESET, dynamicParams2, (IVIDENC1_Status *)&status2);
           break;

      case VTK_XDM_0P9:
           memset (&status1, 0 , sizeof(IVIDENC_Status));
           status1.size   = sizeof(IVIDENC_Status);
           fxns1          = (IVIDENC_Fxns *)vtkEncInst->handle->fxns;
           handle1        = (IVIDENC_Handle *)(vtkEncInst->handle);

           vtkGetDynamicCfgPtr(inst, (tulong **)&dynamicParams1, &size);

           /*Call Reset */
           retVal = fxns1->control((IVIDENC_Handle)handle1, XDM_RESET, dynamicParams1, (IVIDENC_Status *)&status1);
           break;
    }
  }

  return(retVal);
}

/******************************************************************************
 * FUNCTION PURPOSE: Get Codec Information
 ******************************************************************************
 * DESCRIPTION: Get Codec information
 *
 * tint vtkGetCodecInfo (
 *    vtkInst_t         *inst,     - Instance ptr 
 *    vtkCodecInfo_t    *info      - Return value )  
 *
 *****************************************************************************/
void vtkGetCodecInfo(vtkInst_t *inst, vtkCodecInfo_t *info) {
  vtkEncArgs_t    *encArgs = &(info->encArgs);
  vtkDecArgs_t    *decArgs = &(info->decArgs);
  vtkEncInst_t *vtkEncInst = &(inst->codec.enc);
  vtkDecInst_t *vtkDecInst = &(inst->codec.dec);
  
  switch (inst->cfg.codingType) {

    case EVCT_VIDEO_MPEG2MP_DEC:
        info->xdmVer = VTK_XDM_1P0;
        info->type   = VTK_TYPE_DECODER;
        decArgs->u.mpeg2.status.viddecStatus.size   = sizeof(IMPEG2VDEC_Status);
        decArgs->u.mpeg2.inArgs.viddecInArgs.size   = sizeof(IMPEG2VDEC_InArgs);
        decArgs->u.mpeg2.outArgs.viddecOutArgs.size = sizeof(IMPEG2VDEC_OutArgs); 
    
        decArgs->xdmver.xdm1p0.dec_status2  = (IVIDDEC2_Status *)&decArgs->u.mpeg2.status.viddecStatus;
        decArgs->xdmver.xdm1p0.dec_dynpar2  = (IVIDDEC2_DynamicParams *)&vtkDecInst->u.mpeg2.ext_dynpar.viddecDynamicParams;
        decArgs->xdmver.xdm1p0.dec_inargs2  = (IVIDDEC2_InArgs *)&decArgs->u.mpeg2.inArgs.viddecInArgs;
        decArgs->xdmver.xdm1p0.dec_outargs2 = (IVIDDEC2_OutArgs *)&decArgs->u.mpeg2.outArgs.viddecOutArgs;
        break;

    case EVCT_VIDEO_MPEG4SP_DEC:
    case EVCT_VIDEO_H263P_DEC:
        info->xdmVer = VTK_XDM_1P0;
        info->type   = VTK_TYPE_DECODER;
        decArgs->u.mpeg4.status.viddecStatus.size         = sizeof(IM4H3DEC_Status);
        decArgs->u.mpeg4.status.viddecStatus.data.buf     = NULL;
        decArgs->u.mpeg4.status.viddecStatus.data.bufSize = 0;
        decArgs->u.mpeg4.inArgs.viddecInArgs.size         = sizeof(IM4H3DEC_InArgs);
        decArgs->u.mpeg4.outArgs.viddecOutArgs.size       = sizeof(IM4H3DEC_OutArgs); 
    
        decArgs->xdmver.xdm1p0.dec_status2  = (IVIDDEC2_Status *)&decArgs->u.mpeg4.status;
        decArgs->xdmver.xdm1p0.dec_dynpar2  = (IVIDDEC2_DynamicParams *)&vtkDecInst->u.mpeg4.ext_dynpar.viddecDynamicParams;
        decArgs->xdmver.xdm1p0.dec_inargs2  = (IVIDDEC2_InArgs *)&decArgs->u.mpeg4.inArgs;
        decArgs->xdmver.xdm1p0.dec_outargs2 = (IVIDDEC2_OutArgs *)&decArgs->u.mpeg4.outArgs;
        break;

    case EVCT_VIDEO_H264BP_MP_DEC:
        info->xdmVer = VTK_XDM_1P0;
        info->type   = VTK_TYPE_DECODER;
        decArgs->u.h264bpmp.status.viddecStatus.size     = sizeof(IH264VDEC_Status);
        decArgs->u.h264bpmp.inArgs.viddecInArgs.size     = sizeof(IH264VDEC_InArgs);
        decArgs->u.h264bpmp.outArgs.viddecOutArgs.size   = sizeof(IH264VDEC_OutArgs);      

        decArgs->xdmver.xdm1p0.dec_status2  = (IVIDDEC2_Status *)&decArgs->u.h264bpmp.status.viddecStatus;
        decArgs->xdmver.xdm1p0.dec_dynpar2  = (IVIDDEC2_DynamicParams *)&vtkDecInst->u.h264bpmp.ext_dynpar.viddecDynamicParams;
        decArgs->xdmver.xdm1p0.dec_inargs2  = (IVIDDEC2_InArgs *)&decArgs->u.h264bpmp.inArgs.viddecInArgs;
        decArgs->xdmver.xdm1p0.dec_outargs2 = (IVIDDEC2_OutArgs *)&decArgs->u.h264bpmp.outArgs.viddecOutArgs;
        break;

    case EVCT_VIDEO_H264HP_DEC:
        info->xdmVer = VTK_XDM_1P0;
        info->type   = VTK_TYPE_DECODER;
        decArgs->u.h264hp.status.viddecStatus.size     = sizeof(IH264HPVDEC_Status);
        decArgs->u.h264hp.inArgs.viddecInArgs.size     = sizeof(IH264HPVDEC_InArgs);
        decArgs->u.h264hp.inArgs.numNALunits           = 1; // Needed for NAL unit format of inputStreamFormat only
        decArgs->u.h264hp.inArgs.viddecInArgs.inputID  = 0;
        decArgs->u.h264hp.outArgs.viddecOutArgs.size   = sizeof(IH264HPVDEC_OutArgs);      

        decArgs->xdmver.xdm1p0.dec_status2  = (IVIDDEC2_Status *)&decArgs->u.h264hp.status.viddecStatus;
        decArgs->xdmver.xdm1p0.dec_dynpar2  = (IVIDDEC2_DynamicParams *)&vtkDecInst->u.h264hp.ext_dynpar.viddecDynamicParams;
        decArgs->xdmver.xdm1p0.dec_inargs2  = (IVIDDEC2_InArgs *)&decArgs->u.h264hp.inArgs.viddecInArgs;
        decArgs->xdmver.xdm1p0.dec_outargs2 = (IVIDDEC2_OutArgs *)&decArgs->u.h264hp.outArgs.viddecOutArgs;
        break;

    case EVCT_VIDEO_MPEG4SP_ENC:
    case EVCT_VIDEO_H263P_ENC:
        info->xdmVer = VTK_XDM_1P0;
        info->type   = VTK_TYPE_ENCODER;
        encArgs->u.mpeg4.inArgs.videncInArgs.size   = sizeof(IMP4VENC_InArgs);
        encArgs->u.mpeg4.outArgs.videncOutArgs.size = sizeof(IMP4VENC_OutArgs);
        encArgs->u.mpeg4.status.videncStatus.size   = sizeof(IMP4VENC_Status);        
        encArgs->xdmver.xdm1p0.enc_status1     = (IVIDENC1_Status *)&encArgs->u.mpeg4.status;
        encArgs->xdmver.xdm1p0.enc_dynpar1     = (IVIDENC1_DynamicParams *)&vtkEncInst->u.mpeg4.ext_dynpar;
        encArgs->xdmver.xdm1p0.enc_inargs1     = (IVIDENC1_InArgs *)&encArgs->u.mpeg4.inArgs;
        encArgs->xdmver.xdm1p0.enc_outargs1    = (IVIDENC1_OutArgs *)&encArgs->u.mpeg4.outArgs; 
        encArgs->xdmver.xdm1p0.params_maxWidth = vtkEncInst->u.mpeg4.ext_params.videncParams.maxWidth;
        break;

    case EVCT_VIDEO_H264BP_ENC:
        info->xdmVer = VTK_XDM_0P9;
        info->type   = VTK_TYPE_ENCODER;
        encArgs->u.h264bp.status.videncStatus.size     = sizeof(IH264VENC_Status);
        encArgs->u.h264bp.inArgs.videncInArgs.size     = sizeof(IH264VENC_InArgs);
        encArgs->u.h264bp.outArgs.videncOutArgs.size   = sizeof(IH264VENC_OutArgs);                       
        encArgs->xdmver.xdm0p9.enc_status  = (IVIDENC_Status *)&encArgs->u.h264bp.status;
        encArgs->xdmver.xdm0p9.enc_dynpar  = (IVIDENC_DynamicParams *)&vtkEncInst->u.h264.ext_dynpar;
        encArgs->xdmver.xdm0p9.enc_inargs  = (IVIDENC_InArgs *)&encArgs->u.h264bp.inArgs;
        encArgs->xdmver.xdm0p9.enc_outargs = (IVIDENC_OutArgs *)&encArgs->u.h264bp.outArgs;
        break;

    default:
        info->xdmVer = VTK_XDM_UNKNOWN;
        info->type   = VTK_TYPE_UNKNOWN;
        break;
  }

  return;
}

void vtkSetMcParams(void *vtkInst, IVIDMC_t *params)
{
  vtkInst_t *inst = (vtkInst_t *)vtkInst;

  memcpy(&(inst->mcParams), params, sizeof(IVIDMC_t));
  return;
}


/*nothing past this point**/
