#ifndef _VTKLOC_H
#define _VTKLOC_H
/******************************************************************************
 * FILE PURPOSE: Macros and definitions private to VTK
 ******************************************************************************
 * FILE NAME:   vtkloc.h  
 *
 * DESCRIPTION: Defines macros and definitions seen only by the VTK unit.
 *
 * (C) Copyright 2008, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
/* Standard C header files */
#include <stdio.h>
#include <stdlib.h>
#include "xdc/std.h"
#include "vtk.h"
#include "vctypes.h"
#include <ti/mas/types/types.h>
#include "memmgm.h"

/* MPEG4 Encoder Interface header files */
#include <M4H3_ENC/vtk_m4h3_enc.h>

/* M4H3 Decoder Interface header files */
#include <M4H3_DEC/vtk_m4h3_dec.h>

/* H.264 encoder interface header files */
#include <H264BP_ENC/vtk_h264_enc.h>

/* MPEG2 Main Profile Decoder */
#include <MPEG2MP_DEC/vtk_mpeg2mp_dec.h>

/* H.264 baseprofile decoder interface header files */
#include <H264BP_MP_DEC/vtk_h264bp_mp_dec.h>

/* H.264 highprofile decoder interface header files */
#include <H264HP_DEC/vtk_h264hp_dec.h>

#include <ti/xdais/dm/ividdec.h>

/* Possible channel states */
typedef enum {
  VTK_STATE_OPEN,
  VTK_STATE_CLOSED,
  VTK_STATE_OPENED_NOT_CONFIGURED
} vtkState_t;

typedef struct vtkEncInstMPEG4_s {
  IMP4VENC_Params        ext_params;
  IMP4VENC_DynamicParams ext_dynpar;
} vtkEncInstMPEG4_t;

typedef struct vtkDecInstMPEG4_s {
  IVIDDEC2_Params         params;
  IM4H3DEC_DynamicParams ext_dynpar;
} vtkDecInstMPEG4_t;

typedef struct vtkEncInstH264_s {
  IH264VENC_Params        ext_params;
  IH264VENC_DynamicParams ext_dynpar;
} vtkEncInstH264_t;

typedef struct vtkDecInstH264BP_MP_s {
  IH264VDEC_Params        ext_params;
  IH264VDEC_DynamicParams ext_dynpar;
#if 0  
  sSeiVuiParams_t sei_vui_buffer;
#endif  
} vtkDecInstH264BP_MP_t;

typedef struct vtkDecInstH264HP_s {
  IH264HPVDEC_Params        ext_params;
  IH264HPVDEC_DynamicParams ext_dynpar;
#if 0  
  sSeiVuiParams_t sei_vui_buffer;
#endif  
} vtkDecInstH264HP_t;

typedef struct vtkDecInstMPEG2MP_s {
  IMPEG2VDEC_Params         ext_params;
  IMPEG2VDEC_DynamicParams  ext_dynpar;
} vtkDecInstMPEGMP_t;

typedef struct vtkEncInst_s {
/* Encoder algorithm specific handle */
  IALG_Handle           handle;
  XDAS_UInt32           framesToEncode;
  XDAS_UInt32           headerFlag;
  XDAS_UInt32           bytesGenerated;
  XDAS_UInt32           savedParametersAvail;
#define VTK_ENC_MAX_SAVED_PARAMETERS 4  
  XDAS_UInt32 savedParameters[VTK_ENC_MAX_SAVED_PARAMETERS];
  /**************************/
  /*Configuration parameters*/
  /**************************/
  union {
	vtkEncInstMPEG4_t mpeg4;
	vtkEncInstH264_t  h264;
  } u;
} vtkEncInst_t;

typedef struct vtkDecInst_s {
  /* Decoder algorithm specific handle */
  IALG_Handle handle;
  XDAS_UInt32 framesToDecode;
  /* Input Buffer Management specific variables  */
  XDAS_Int32  maxDisplayDelay;
  XDAS_Int32  first_field_decoded;
  /**************************/
  /*Configuration parameters*/
  /**************************/
  union {
    vtkDecInstMPEG4_t  mpeg4;
    vtkDecInstH264BP_MP_t h264bpmp;
    vtkDecInstH264HP_t h264hp;
    vtkDecInstMPEGMP_t mpeg2;
  } u;
} vtkDecInst_t;

/******************************************************************************/
/* Temporary decoder input, output and status arguments, allocated from stack.*/
/******************************************************************************/
typedef struct vtkXdmDec0p9_s {
      IVIDDEC_Status  *dec_status;
      IVIDDEC_InArgs  *dec_inargs;
      IVIDDEC_OutArgs *dec_outargs;
      IVIDDEC_DynamicParams *dec_dynpar;
}vtkXdmDec0p9_t;

typedef struct vtkXdmDec1p0_s {
      IVIDDEC2_Status  *dec_status2;
      IVIDDEC2_InArgs  *dec_inargs2;
      IVIDDEC2_OutArgs *dec_outargs2;
      IVIDDEC2_DynamicParams *dec_dynpar2;
}vtkXdmDec1p0_t;

typedef struct vtkDecArgs_s {
  union {
    struct {
      IMPEG2VDEC_InArgs  inArgs;
      IMPEG2VDEC_OutArgs outArgs;
      IMPEG2VDEC_Status  status;
    } mpeg2;
    struct {
      IM4H3DEC_InArgs    inArgs;
      IM4H3DEC_OutArgs   outArgs;
      IM4H3DEC_Status    status;
    } mpeg4;
    struct {
      IH264VDEC_InArgs   inArgs;
      IH264VDEC_OutArgs  outArgs;
      IH264VDEC_Status   status;
    } h264bpmp;
    struct {
      IH264HPVDEC_InArgs   inArgs;
      IH264HPVDEC_OutArgs  outArgs;
      IH264HPVDEC_Status   status;
    } h264hp;
  } u;
  union {
    vtkXdmDec0p9_t xdm0p9;
    vtkXdmDec1p0_t xdm1p0;
  } xdmver;
} vtkDecArgs_t;
/******************************************************************************/
/* Temporary encoder input, output and status arguments, allocated from stack.*/
/******************************************************************************/
typedef struct vtkXdmEnc0p9_s {
    IVIDENC_Status  *enc_status;
    IVIDENC_InArgs  *enc_inargs;
    IVIDENC_OutArgs *enc_outargs;
    IVIDENC_DynamicParams *enc_dynpar;
}vtkXdmEnc0p9_t;

typedef struct vtkXdmEnc1p0_s {
    IVIDENC1_Status  *enc_status1;
    IVIDENC1_InArgs  *enc_inargs1;
    IVIDENC1_OutArgs *enc_outargs1;
    IVIDENC1_DynamicParams *enc_dynpar1;
    XDAS_Int32 params_maxWidth;
}vtkXdmEnc1p0_t;

typedef struct vtkEncArgs_s {
  union {
    struct {
      IMP4VENC_InArgs     inArgs;
      IMP4VENC_OutArgs    outArgs;
      IMP4VENC_Status     status;
    } mpeg4;
    struct {
      IH264VENC_InArgs   inArgs;
      IH264VENC_OutArgs  outArgs;
      IH264VENC_Status   status;
    } h264bp;
  } u;
  union {
    vtkXdmEnc0p9_t xdm0p9;
    vtkXdmEnc1p0_t xdm1p0;
  } xdmver;
} vtkEncArgs_t;  

/************************************************************/
/* VTK common instance (for decoding and encoding channels) */
/************************************************************/
typedef struct vtkInst_s {
  tuint ID;
  tint state;
  vtkCfg_t cfg;
  XDAS_Int32  statusCode;
  void *sysInst;
  union {
    vtkEncInst_t enc;
    vtkDecInst_t dec;
  } codec;
  vtkStats_t   stats;
  IVIDMC_t     mcParams;
  void*        iresFxns;
} vtkInst_t;
/*****************************/
/* Codec XDM APIs enumerated */
/*****************************/
typedef enum {
  VTK_XDM_0P9 = 0,
  VTK_XDM_1P0 = 1,
  VTK_XDM_UNKNOWN
} vtkCodecXDMVersion_t;

typedef enum {
  VTK_TYPE_DECODER = 0,
  VTK_TYPE_ENCODER,
  VTK_TYPE_UNKNOWN
} vtkCodecType_t;

typedef struct vtkCodecInfo_s {
  vtkCodecXDMVersion_t xdmVer;
  vtkCodecType_t       type;
  vtkEncArgs_t         encArgs;
  vtkDecArgs_t         decArgs;
} vtkCodecInfo_t;
/***********************************/
/* Codec to XDM mapping resolution */
/***********************************/
void vtkGetCodecInfo(vtkInst_t *inst, vtkCodecInfo_t *info);
/*----------------------------*/
/* Exception and Debug Macros */
/*----------------------------*/
extern debugInfo_t vtk_debug_info;

/* Assert macro for the exceptions */
#define vtk_exc_assert(expr,code,inst) { \
  (expr) ? ((void)0) : vtk_exception((inst)->ID, code); }

void vtk_exception(tuint id, tuint code);
void vtkStatsUpdateFrameDetails (vtkEncDecStats_t *stats, vtkFrameType_t type, tulong frameSize);
tint vtkCommitCodec (void *vtkInst);
tint vtkGetConfig (void *vtkInst, vtkCfg_t *cfg);
tint vtkGetEncDynConfig (void *vtkInst, vtkEncDynCfg_t *cfg);
tint vtkSetEncDynConfig (void *vtkInst, vtkEncDynCfg_t *cfg);
tint vtkGetStaticCfgPtr (void *vtkInst,  tulong **cfgPtr, tulong *size);
tint vtkGetDynamicCfgPtr (void *vtkInst, tulong **cfgPtr, tulong *size);
/* VTK video codec memory management */
extern void vtkAllocHeap(tuint chan, tuint coreID);
extern void vtkFreeHeap(tuint chan);
extern void vtkResetScratch(void);
tint vtkIsDynUpdateAllowed(vtkInst_t *inst, tulong *dynCfg);
tint vtkReset (vtkInst_t *inst);
void vtkSetMcParams(void *vtkInst, IVIDMC_t *params);

//#define VTK_PRINT_CODEC_INTERNAL_MIPS
#ifdef VTK_PRINT_CODEC_INTERNAL_MIPS
  void ResetTimerVariables(void);
  void PrintTimerVariables(void);
#endif

#endif
/* Nothing past this point */

